//---------------------------------------------------------------------------
#ifndef GlobalDefinitionsH
#define GlobalDefinitionsH
//---------------------------------------------------------------------------

// ---- Configuration ----

/// define to create debug output during object streaming operations
#undef DEBUG_STREAMS

/// define to include point cloud / rendering PACKAGEs headers in precompiled headers of later projects
/// decreases build/make times for later projects but increases make times if headers in earlier PACKAGEs are changed
#define PRECOMP_POINTCLOUD_PACKAGES

/// define to perform range checking in linear algebra routines
#ifdef GEOXL_DEBUG
   #define RANGE_CHECK
   #define DVECT_RANGE_CHECK
#endif

/// define this token to use code that explicitly unrolls loops in often used routines in LinearAlgebra (i.e. 2-4 dim. vector/matrix operations)
#undef LINEAR_ALGEBRA_EXPLICIT_LOOP_UNROOLING

/// define this token in order to generate GUI code
#define GEOXL_GUI


// ---- Platform ----

#ifdef TARGET_PLATFORM_MSVC2005FF
   /// define compiler type ms visual c++
   #define COMPILER_MSVC

   /// enable qt gui support
   #define PLATFORM_QT
   /// use win32 specific low-level routines
   #define PLATFORM_WINDOWS

   // /// disable warning for "unknown pragma"
   // #pragma warning (disable: 4068)

	/// disable warning for " '...' was declared deprecated"
	#pragma warning (disable : 4996)

	/// disable warning for "forcing value to bool 'true' or 'false' (performance warning)"
	#pragma warning (disable : 4800)

	#ifndef VS2010_OR_LATER
		/// iteratore range checking: problems cause exceptions
		#define _SECURE_SCL_THROWS 1

		#if defined(GEOXL_DEBUG)
			/// enable checking in debug/fastdebug
			#define _SECURE_SCL 1
		#else
			///disable otherwise (release/fullopt)
			#define _SECURE_SCL 1
		#endif
   
	#endif

   #ifdef _WIN64
      #define GEOXL_64BIT
   #else
      #define GEOXL_32BIT
   #endif
   
#endif

/// default target for MS Visual Studio .NET 2003 / win32
#ifdef TARGET_PLATFORM_MSVC7_QT_WIN32

   /// define compiler type ms visual c++
   #define COMPILER_MSVC

   /// enable qt gui support
   #define PLATFORM_QT
   /// use win32 specific low-level routines
   #define PLATFORM_WINDOWS

   /// remove PACKAGE macros
   #define PACKAGE

   /// disable warning for "unknown pragma"
   #pragma warning (disable: 4068)
   
   #define GEOXL_32BIT
#endif

/// default target for GCC / QT / LINUX (32 Bit)
#ifdef TARGET_PLATFORM_LINUX_GCC

   /// define compiler type gcc
   #define COMPILER_GCC

   /// enable qt gui support
   #define PLATFORM_QT
   /// use win32 specific low-level routines
   #define PLATFORM_LINUX32

   /// remove PACKAGE macros
   #define PACKAGE

   /// change this for a linux-64bit port
   #define GEOXL_32BIT
#endif

//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
