/*
 * Copyright 1993-2007 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO USER:   
 *
 * This source code is subject to NVIDIA ownership rights under U.S. and 
 * international Copyright laws.  Users and possessors of this source code 
 * are hereby granted a nonexclusive, royalty-free license to use this code 
 * in individual and commercial software.
 *
 * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE 
 * CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR 
 * IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH 
 * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF 
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL, 
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS 
 * OF USE, DATA OR PROFITS,  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
 * OR OTHER TORTIOUS ACTION,  ARISING OUT OF OR IN CONNECTION WITH THE USE 
 * OR PERFORMANCE OF THIS SOURCE CODE.  
 *
 * U.S. Government End Users.   This source code is a "commercial item" as 
 * that term is defined at  48 C.F.R. 2.101 (OCT 1995), consisting  of 
 * "commercial computer  software"  and "commercial computer software 
 * documentation" as such terms are  used in 48 C.F.R. 12.212 (SEPT 1995) 
 * and is provided to the U.S. Government only as a commercial end item.  
 * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through 
 * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the 
 * source code with only those rights set forth herein. 
 *
 * Any use of this source code in individual and commercial software must 
 * include, in the user documentation and internal comments to the code,
 * the above Disclaimer and U.S. Government End Users Notice.
 */

#if !defined(__MATH_CONSTANTS_H__)
#define __MATH_CONSTANTS_H__

/* single precision constants */
#define CUDART_INF_F         __int_as_float(0x7f800000)
#define CUDART_NAN_F         __int_as_float(0x7fffffff)
#define CUDART_MIN_DENORM_F  __int_as_float(0x00000001)
#define CUDART_MAX_NORMAL_F  __int_as_float(0x7f7fffff)
#define CUDART_NEG_ZERO_F    __int_as_float(0x80000000)
#define CUDART_ZERO_F        0.0f
#define CUDART_ONE_F         1.0f
#define CUDART_SQRT_HALF_F   0.707106781f
#define CUDART_SQRT_TWO_F    1.414213562f
#define CUDART_THIRD_F       0.333333333f
#define CUDART_PIO4_F        0.785398163f
#define CUDART_PIO2_F        1.570796327f
#define CUDART_3PIO4_F       2.356194490f
#define CUDART_2_OVER_PI_F   0.636619772f
#define CUDART_PI_F          3.141592654f
#define CUDART_L2E_F         1.442695041f
#define CUDART_L2T_F         3.321928094f
#define CUDART_LG2_F         0.301029996f
#define CUDART_LGE_F         0.434294482f
#define CUDART_LN2_F         0.693147181f
#define CUDART_LNT_F         2.302585093f 
#define CUDART_LNPI_F        1.144729886f
#define CUDART_TWO_TO_M126_F 1.175494351e-38f
#define CUDART_TWO_TO_126_F  8.507059173e37f
#define CUDART_NORM_HUGE_F   3.402823466e38f
#define CUDART_TWO_TO_23_F   8388608.0f
#define CUDART_TWO_TO_24_F   16777216.0f
#define CUDART_TWO_TO_31_F   2147483648.0f
#define CUDART_TWO_TO_32_F   4294967296.0f
#define CUDART_REMQUO_BITS_F 3
#define CUDART_REMQUO_MASK_F (~((~0)<<CUDART_REMQUO_BITS_F))
#define CUDART_TRIG_PLOSS_F  48039.0f
#define CUDART_TRIG_TLOSS_F  1.0e7f

/* double precision constants */
#define CUDART_INF          __hiloint2double(0x7ff00000, 0x00000000)
#define CUDART_NAN          __hiloint2double(0xfff80000, 0x00000000)
#define CUDART_NEG_ZERO     __hiloint2double(0x80000000, 0x00000000)
#define CUDART_ZERO         0.0
#define CUDART_ONE          1.0
#define CUDART_SQRT_TWO     1.4142135623730951e+0
#define CUDART_SQRT_HALF    (0.5*1.4142135623730951e+0)
#define CUDART_THIRD        3.3333333333333333e-1
#define CUDART_TWOTHIRD     6.6666666666666667e-1
#define CUDART_PIO4         7.8539816339744828e-1         
#define CUDART_PIO2         1.5707963267948966e+0
#define CUDART_3PIO4        2.3561944901923448e+0
#define CUDART_2_OVER_PI    6.3661977236758138e-1
#define CUDART_PI           3.1415926535897931e+0 
#define CUDART_L2E          1.4426950408889634e+0
#define CUDART_L2T          3.3219280948873622e+0
#define CUDART_LG2          3.0102999566398120e-1
#define CUDART_LGE          4.3429448190325182e-1
#define CUDART_LN2          6.9314718055994529e-1
#define CUDART_LNPI         1.1447298858494002e+0
#define CUDART_TWO_TO_23    8388608.0
#define CUDART_TWO_TO_52    4503599627370496.0
#define CUDART_TWO_TO_54    18014398509481984.0
#define CUDART_TWO_TO_M54   5.5511151231257827e-17
#define CUDART_TWO_TO_M1022 2.22507385850720140e-308
#define CUDART_TRIG_PLOSS   2147483648.0

#endif /* !__MATH_CONSTANTS_H__ */
