//---------------------------------------------------------------------------
#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "MainWindow.h"
//---------------------------------------------------------------------------
#include "pointClouds.h"
#include "hierarchies.h"
#include "ObjectDictionary.h"
#include "MetaClass.h"
#include "ClassEditor.h"
#include "OnlineHelp.h"
#include "ProgressWindow.h"
#include "InfoDialogsWidget.h"
#include "DebugWindows.h"
#include "ClassCreator.h"
#include "ObjectDictionaryEditorWidget.h"
#include "UserSettings.h"
#include "ExceptionHandling.h"
#include "MultiObjectStream.h"
#include "HierarchicalOutOfCorePointCloud.h"
#define RENDERING_API __declspec(dllimport)
#include "SceneCreator.h"
#include "BinaryObjectStreams.h"
#include "Scene.h"
#include "CommandHistory.h"
#include "CommandObjMacro.h"
#include "CmdObj\CmdObjScriptProgram.h"
#include "DebugOutput.h"
#include "Timer.h"
//---------------------------------------------------------------------------
#include <qtGui/qboxlayout.h>
#include <qtGui/qevent.h>
#include <QtGui/QPushButton>
#include <QtGui/QMenuBar.h>
#include <QtGui/QMessageBox.h>
//---------------------------------------------------------------------------

using namespace geoxl;


MainWindow::MainWindow(QWidget * parent, Qt::WFlags f)
:QMainWindow(parent, f)
{
	ui.setupUi(this);

	setWindowTitle( "geoxl" );

	setAttribute(Qt::WA_DeleteOnClose);

	m_MainObject = new ObjectDictionary;
	m_MainEditor = (ClassEditor*)m_MainObject->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();


	QWidget * edWidget = m_MainEditor->makeEditor( ui.memoryFrame, m_MainObject );
	m_ObjectDictEditor = (ObjectDictionaryEditorWidget*)edWidget;

	QVBoxLayout *layout = new QVBoxLayout(ui.memoryFrame);
	layout->addWidget(edWidget);
	layout->setMargin(3);

	//QWidget::connect(ui.newButton,  SIGNAL(clicked()),			 this, SLOT(on_newButton_clicked()));
	//QWidget::connect(ui.openButton, SIGNAL(clicked()),			 this, SLOT(on_openButton_clicked()));
	//QWidget::connect(ui.saveButton, SIGNAL(clicked()),			 this, SLOT(on_saveButton_clicked()));
	//QWidget::connect(ui.exitButton, SIGNAL(clicked()),			 this, SLOT(on_exitButton_clicked()));
	//QWidget::connect(ui.menubar,    SIGNAL(triggered(QAction*)), this, SLOT(sltMenuHandler(QAction*)));
	
   card32 n = UnitTests::getNumTests();
   for (card32 i=0; i<n; i++) {
      UnitTest t = UnitTests::getTest(i);
      string name = UnitTests::getTestName(i);
      UnitTestEntry e;
      e.test = t;
      e.action = ui.menuUnit_Tests_and_Experiments->addAction(name.c_str()); 
      unitTestEntries.push_back(e);
   }
	move(0,0);

	ui.actionShow_Debug_Window->setShortcutContext( Qt::ApplicationShortcut );
	m_ConfigurationEditor = (ClassEditor *)Configuration->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
	m_ConfigurationWidget = m_ConfigurationEditor->makeEditor( this , Configuration );
	m_ConfigurationWidget->hide();
	m_ConfigurationWidget->setWindowFlags( Qt::Window );

	//on_actionCreate_New_Object_triggered();
}

MainWindow::~MainWindow(void)
{
	delete m_ConfigurationEditor;
}


void MainWindow::on_menubar_triggered(QAction *action)
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
   size_t n = unitTestEntries.size();
   for (card32 i=0; i<n; i++) {
      UnitTestEntry e = unitTestEntries[i];
      if (e.action == action) {
         if (e.test != NULL) {
            (e.test)();
         }
         return;      
      }
   }
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}


void MainWindow::on_newButton_clicked(){
	STD_GLOBAL_EXCEPTION_HANDLER_START
	on_actionCreate_New_Object_triggered();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_exitButton_clicked(){
	STD_GLOBAL_EXCEPTION_HANDLER_START
	on_actionE_xit_triggered();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_openButton_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	on_action_Open_Object_triggered();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_saveButton_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	on_actionSave_Object_As_triggered();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_actionCreate_New_Object_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	const MetaClass * mc = MetaClass::getClassByName( "Scene" );
	ClassCreator * cc = (ClassCreator*)mc->getGUIDescriptor()->getDefaultClassCreator()->newInstance();
	pAssert(cc);
	Object * newObj = cc->makeObject();
	delete cc;
	if (newObj != NULL) {
		m_MainObject->addObject(newObj);
	}
	m_MainEditor->update();

	// HACK
#ifdef HACKED_OOC_TEST
	SceneCreator sc;
	Object * obj = sc.makeObject();
	m_MainObject->addObject(obj);
	m_MainEditor->update();
#endif
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_action_Open_Object_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_ObjectDictEditor->load();		
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_actionSave_Object_As_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_ObjectDictEditor->save();		
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_actionConfiguration_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	m_ConfigurationWidget->hide();
	m_ConfigurationEditor->update();
	m_ConfigurationWidget->show();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::cleanupAllocs() {
	// force clean shutdown for all multi object stream objects

	#ifndef _DEBUG
	try {
	#endif
		
		vector<HierarchicalOutOfCorePointCloud*> remainingOOCPCs = HierarchicalOutOfCorePointCloud::getActiveOOCPCs();
		for (card32 i=0; i<remainingOOCPCs.size(); i++) {
			remainingOOCPCs[i]->shutdownAllocator();
		}

	#ifndef _DEBUG
	} catch (...) {}
	
	try {
	#endif
		bool goOn = false;
		do {
			vector<MultiObjectStream*> remainingStreams = MultiObjectStream::getActiveStreams();
			if (!remainingStreams.empty()) {
				remainingStreams[0]->sync();
				delete remainingStreams[0];
				goOn = true;
			} else {
				goOn = false;
			}
		} while (goOn);
	#ifndef _DEBUG
	} catch (...) {}
	#endif
}


void MainWindow::on_actionE_xit_triggered()
{
	close();
}

void MainWindow::closeEvent(QCloseEvent *event) {

	#if !defined(_DEBUG) && !defined(GEOXL_MAIN_FAST_DEBUG)

	UserSettings * u = dynamic_cast<UserSettings*>( Configuration->getData( "USER_SETTINGS" ) );
	bool askBeforeQuit = false;
	if( u )
		askBeforeQuit = u->getAskBeforeQuit();
	if( askBeforeQuit )
	{
		if (QMessageBox::question(this, "Confirm closing...", "Do you want to close GEOXL?", QMessageBox::Ok, QMessageBox::Cancel) == QMessageBox::Ok) {
			event->setAccepted(true);
			cleanupAllocs();
			// fix qt413 bug
			QApplication::exit(0);
		} else {
			event->setAccepted(false);
		}
	} else {
		cleanupAllocs();
		// fix qt413 bug
		QApplication::exit(0);
	}

	#else
		event->setAccepted(true);
		cleanupAllocs();
		// fix qt413 bug
		QApplication::exit(0);
	#endif
}

void MainWindow::on_actionShow_Debug_Window_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	if (debugWindow->isHidden()) {
	   static bool first = true;
	   if (first) {
	      first = false;
	      debugWindow->move(310,50);
	   }
	   debugWindow->show();
	} else {
	   debugWindow->hide();
	}
	ui.actionShow_Debug_Window->setChecked(!debugWindow->isHidden());
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_action_Online_Help_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	OnlineHelp::newTopic("index.html");
	OnlineHelp::show();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_action_Index_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	OnlineHelp::showIndex();
	OnlineHelp::show();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void MainWindow::on_action_Info_triggered()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	InfoDialogsWidget *info = new InfoDialogsWidget;
	info->show();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}


void geoxl::MainWindow::setBatchMode( string in, string out, string script )
{
	inputFile=in;
	outputFile=out;
	scriptFile=script;
	QTimer::singleShot(3000, this, SLOT(startBatch()));
	
}

void geoxl::MainWindow::startBatch()
{
	
	debugOutput <<"\n-runbatch\n=============\n"; 
	
	BinaryInputObjectStream in(inputFile.c_str());
	Persistent *instance = NULL;
	debugOutput << ("loading " + inputFile +"\n");
	debugOutput.flush();
	in.readObject(instance);
	Scene* inputScene= dynamic_cast<Scene*>(instance);

	BinaryInputObjectStream inScript(scriptFile.c_str());
	Persistent *obj = NULL;
	inScript.readObject(obj);

	debugOutput << ("processing " + scriptFile +" :\n");

	Timer wholeT; card32 vt;
	Timer T;


	CommandObject* cmdObj = dynamic_cast<CommandObject*>(obj);
	if (cmdObj == NULL) {
		CommandObjMacro* macro = dynamic_cast<CommandObjMacro*>(obj);
		if (macro == NULL) {
			QMessageBox::critical(this, "Wrong object type", "Only command objects and macros can be executed");
			return;
		} else {
			STD_GLOBAL_EXCEPTION_HANDLER_START
				wholeT.getDeltaValue();
				for (card32 i = 0; i < macro->numCommands(); ++i) {
					CommandObject* cmdObj = const_cast<CommandObject*>(macro->getCommand(i));
					debugOutput << "-> "<< cmdObj->getDescription()<<"\n";
					debugOutput.flush();
					T.getDeltaValue();
					inputScene->getCommandHistory()->executeCommand(inputScene, cmdObj);
					vt = T.getDeltaValue();
					if (vt > 5)
						debugOutput << "  operation time: "<< vt << " ms\n";
				}
				vt = wholeT.getDeltaValue();
				debugOutput << "Preprocessing time: "<< vt << " ms\n";
			STD_GLOBAL_EXCEPTION_HANDLER_END
		}
	} else {
		CmdObjScriptProgram* script = dynamic_cast<CmdObjScriptProgram*>(cmdObj);
		if (script != NULL) {
			try {
				debugOutput << "-> "<< cmdObj->getDescription()<<"\n";
				debugOutput.flush();
				inputScene->getCommandHistory()->executeCommand(inputScene, cmdObj);
			} catch(PException &e) {
				std::string msg = "Error in line " + intToStr(script->getScript()->getLine() + 1) + ": \n" + e.getMsg();
				QMessageBox::critical( this, "GEOXLscript error", QString(msg.c_str()));
			}
		} else {
			STD_GLOBAL_EXCEPTION_HANDLER_START
				debugOutput << "-> "<< cmdObj->getDescription()<<"\n";
				debugOutput.flush();
				inputScene->getCommandHistory()->executeCommand(inputScene, cmdObj);
			STD_GLOBAL_EXCEPTION_HANDLER_END
		}
	}

	debugOutput << ("writing to " + outputFile +"\n");
	debugOutput.flush();
	BinaryOutputObjectStream out(outputFile.c_str());
	out.writeObject((Persistent*)inputScene);


	delete inputScene;

	debugOutput << "done\n";
	debugOutput.flush();

	close();


}



#include "MainWindow_moc.cpp"

