//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include <qtGui/qapplication.h>
//---------------------------------------------------------------------------
#include "InitFoundation.h"
#include "InitPointClouds.h"
#include "InitGLTools.h"
#include "InitSceneGraph.h"
#include "InitRendering.h"
#include "InitHierarchies.h"
#include "InitEditor.h"
//#include "InitBSR.h"
//---------------------------------------------------------------------------
#include "InterfaceGlobals.h"
//---------------------------------------------------------------------------
#include <direct.h>
//---------------------------------------------------------------------------
#include "MainWindow.h"
#include "ClassSelectionDialog.h"
#include "Persistent.h"
#include "BinaryObjectStreams.h"
#include "Configuration.h"
#include "ObjectDictionary.h"
#include <qtGui/qboxlayout.h>
#include <qtGui/QPlastiqueStyle>
#include "DebugWindows.h"
#include "Shellapi.h"
#include "UserSettings.h"
#include "ExceptionHandling.h"
#include "ProgressWindow.h"
//---------------------------------------------------------------------------


using namespace std;
using namespace geoxl;

#define GEOXL_MAXPATH 1000

bool readConfig( string filename )
{
   FILE *file = fopen(filename.c_str(), "rb");
   if (file == NULL)
      return false;

   BinaryInputObjectStream inputStream( filename.c_str() );
   inputStream.readObject( (Persistent*&)Configuration );
   return true;
}

void writeConfig( string filename )
{
   BinaryOutputObjectStream outputStream( filename.c_str() );
   outputStream.writeObject( (Persistent*&)Configuration );
}

wstring removeFilenameFromPathWStr( const wstring &s )
{
  for (int i=(int)s.length()-1; i >= 0; i--)
    if ((s[i] == '/') || (s[i] == '\\'))
      return s.substr(0, i+1);

  return s;
}

std::wstring string2wString(const std::string& s)
{
    std::wstring temp(s.length(),L' ');
    std::copy(s.begin(), s.end(), temp.begin());
    return temp;
}

static bool tryLoadXGRTModule(LPCWSTR mod)
{
    if (NULL != LoadLibrary(mod))
    {
        std::wcout << mod << L" - ok" << std::endl;
        return true;
    } else {
        DWORD lastError = GetLastError();
        std::wcout << mod << L" - ERROR (" << lastError << ")" << std::endl;
        return false;
    }
}

static bool tryLoadXGRTModule(wstring mod)
{
    return tryLoadXGRTModule(mod.c_str());
}

static bool tryLoadXGRTModule(string f)
{
    wstring mod = string2wString(f);
    return tryLoadXGRTModule(mod.c_str());
}



#if defined(_DEBUG) || defined(GEOXL_MAIN_FAST_DEBUG)
int _tmain(int argc, _TCHAR* argv[])
{
#else
int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);
	UNREFERENCED_PARAMETER(nCmdShow);
#endif

   // set the current directory to the executable's location
   TCHAR lpFilenameBuf[1002];
   if (!GetModuleFileName(0, lpFilenameBuf, 1000)) {
      cout << "your path is too long (limit to 1000)";
      return 1;
   }
   wstring fileName = lpFilenameBuf;
   fileName = removeFilenameFromPathWStr(fileName);
   SetCurrentDirectory(fileName.c_str());



	// mw - activate heavy heap validation here ;-)
	#ifdef _DEBUG
	// int tmpFlag = _CrtSetDbgFlag( _CRTDBG_REPORT_FLAG );

	// tmpFlag |= _CRTDBG_ALLOC_MEM_DF;		  // turn on debugging
	// tmpFlag |= _CRTDBG_DELAY_FREE_MEM_DF; // never free any memory
	// tmpFlag |= _CRTDBG_CHECK_ALWAYS_DF;	  // heap check at every allocation operation
	// tmpFlag |= _CRTDBG_CHECK_CRT_DF;		  // include crt (runtime) memory
	// tmpFlag |= _CRTDBG_LEAK_CHECK_DF;     // leak check at programm exit

	// turn this into a comment this line if none of the above are uncommented
	// cout << "Currently using debug heap - may cause unwanted effects; uncomment in geoxl.cpp to turn off\n";

	// _CrtSetDbgFlag( tmpFlag );	
	#endif

	setStructureExceptionHandler();
   int n = 0;
   QApplication qapp(n, 0);

	InterfaceGlobals::setApplication(&qapp);
	// init projects
	initFoundation();
	initPointClouds();
	initGLTools();
	initSceneGraph();
	initRendering();
	initHierarchies();
	initEditor();
   tryLoadXGRTModule( "geoXLKinect.dll" );
   tryLoadXGRTModule( "animRecSGP07.dll" );
   tryLoadXGRTModule( "animRecTOG09.dll" );

	// Configuration
   Configuration = new AttachedDataContainer;
   // try to load configuration from config.object
   try {
      bool gotConfig = false;
      try {
         gotConfig = readConfig("config.object" );
      } catch (...) {
         readConfig("defaultConfig.object" );
         gotConfig = true;
      }
      if (!gotConfig) readConfig("defaultConfig.object" );
   } catch (...) {
      cout << "load config failed.";
   }

	// create user settings attachment, if it doesn't already exists in Configuration
	UserSettings * userSettings = dynamic_cast<UserSettings*>(Configuration->getData( "USER_SETTINGS" ));
	if( userSettings == NULL )
	{
		userSettings = new UserSettings;
		userSettings->setup( "USER_SETTINGS", AttachedData::ADF_PERSISTENT);
		Configuration->attachData( userSettings );
	}

	char buffer[GEOXL_MAXPATH];
   _getcwd(buffer, GEOXL_MAXPATH );
   string dir = buffer;

   // store bin-path in configuration (non-persistent)
   StringAttachment * binPath = new StringAttachment;
   binPath->setup( "BIN_PATH", 0 );
   binPath->setString( dir );
   Configuration->attachData( binPath );

	// store image-path
   StringAttachment * imagePath = new StringAttachment;
   imagePath->setup( "IMAGE_PATH", 0 );
   imagePath->setString( dir + "/../images" );
   Configuration->attachData( imagePath );

	// create main window	
	MainWindow * mw = new MainWindow;
	mw->setWindowTitle( "GeoXL" );
	mw->show();
	InterfaceGlobals::setMainWindow(mw);
	qapp.setActiveWindow( mw );
	
	// create debug window
	debugWindow = new DebugWindow(mw);
	debugWindow->setWindowFlags(Qt::Dialog);
	debugWindow->hide();

	// create progress window (child of main window)
	progressWindow = new ProgressWindow();
	
	//Close all windows with main window
	QApplication::connect(mw, SIGNAL(destroyed()), &qapp, SLOT(closeAllWindows()));

	// run it
	qapp.exec();

   writeConfig( "config.object" );

	//shutdown
	shutdownFoundation();
	shutdownPointClouds();
	shutdownGLTools();
	shutdownSceneGraph();
	shutdownRendering();
	shutdownHierarchies();
	shutdownEditor();

	return 0;
}

// include icons & other resources
#include "geoxl_resources_rcc.h"
