//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "InitAnimationReconstruction.h"
//---------------------------------------------------------------------------
#include "PointCloudAnimationSequence.h"
#include "TrajectorySet.h"
#include "PGDataPointIndices.h"
#include "PatchGraph.h"
#include "PatchGraphCreator3D.h"
#include "PatchGraphRenderer.h"
#include "Feature.h"
#include "FeatureDetector.h"
#include "ParametricFeatureDetector.h"
#include "LocalParameterization.h"
#include "RefFrameEst3D.h"
#include "PatchApprox3D.h"
#include "PCCResamplePointCloud.h"
#include "UnifyNormals.h"
#include "ComputeSampleSpacing.h"
#include "ImplicitPatchGraphMeshing.h"
#include "HFBFBivariateMonomials.h"
#include "HeighfieldParameterization.h"
#include "LocalExtremaDetector.h"
#include "PatchGraphContainer.h"
#include "CmdObjPointCloudDistortion.h"
#include "FeatureDescriptor.h"
#include "MLSMultiScaleFeatureDetector.h"
#include "FDAttribute.h"
#include "FDSIFT.h"
#include "FDHistogram.h"
#include "PatchGraphPruning.h"
#include "PatchGraphTriangleMesh.h"
#include "TopologyBreaker.h"
#include "DataPointAssignment.h"
#include "TriangleMeshTexture.h"
#include "PatchGraphComponentsHandler.h"
#include "Monomial2ndOrderDisplayer.h"

#include "PatchGraphExtract.h"
#include "PatchGraphMerge.h"
#include "PatchGraphResample.h"
#include "PatchGraphMergeCandidateSelect.h"
#include "AnimationBSRContext.h"
#include "AnimationBSRRenderer.h"
#include "AnimationBSRDebugger.h"
#include "AbstractMergeExtrapolator.h"
#include "FeatureRegistration.h"
#include "RegistrationContext.h"
//#include "SmallDeformationAlignment.h"
#include "DetectTopologicalErrorMerge.h"
#include "Correspondence.h"
#include "CorrespondenceList.h"
#include "FeatureCorrespondences.h"
#include "PatchGraphTopologicalResampling.h"
#include "PGDescriptorFactory.h"
#include "FDColorPCA.h"
#include "FDClusteredHistogram.h"
#include "FDCurvature.h"
#include "PatchGraphOpticalFlow.h"
#include "DescriptorClustering.h"
#include "FDAverage.h"
#include "FDCombined.h"
#include "FastFeature.h"
#include "FeatureAttachment.h"
#include "CentroidDescriptor.h"

// PCI
#include "PCIPatchGraphReconstruction3D.h"
#include "PCIAnimationBSR.h"
#include "PCIFeatureTest.h"
#include "PCIPatchGraphBSR.h"
#include "PCIGlobalRegistration.h"

// PCC
#include "PCCAnimationCreator.h"
#include "PCCPrepareAnimPointCloud.h"
#include "PCCConvertPointCloudTypeInList.h"
#include "PCCAddOutliers.h"

#ifdef GEOXL_GUI
// GUI
#include "PCIFeatureTestEditorWidget.h"
#include "PatchGraphMergeCandidateSelectEditorWidget.h"
#endif

// nonlinear optimization
#include "BSROptContext.h"
#include "BSROptPotential.h"
#include "PatchGraphBSROpt.h"
#include "PGTopologicalNeighborhood.h"
#include "UnitTests.h"
#include "Monomials2ndOrder.h"
#include "BSROptConfig.h"
#include "GeometricAlignment.h"

//Optimization 2
#include "OptimizationGraph.h"
#include "PatchGraphOpt2.h"
#include "PGOpt2Frame.h"
#include "PGOpt2FramePair.h"
#include "PGOpt2FramePairPair.h"
#include "PGOpt2Parameters.h"
#include "PGOpt2Potential.h"
#include "PGOpt2PotentialSelector.h"
#include "PGOpt2Processing.h"
#include "PGOpt2Config.h"

#include "GlobalOptimization2.h"
#include "HoleFilling2.h"
#include "PairwiseAlignment2.h"
#include "PatchGraphJoin2.h"
#include "PatchGraphMerge2.h"
#include "TopologyMerging2.h"
#include "PatchGraphResampler2.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

void geoxl::initAnimationReconstruction() 
{
	// patch graphs
	PointCloudAnimationSequence::init(Persistent::getClass());
	TrajectorySet::init(Persistent::getClass());
	PGDataPointIndices::init(Persistent::getClass());
	PatchGraph::init(Persistent::getClass());
	PatchGraphContainer::init(Persistent::getClass());

	PatchGraphExtract::init( Persistent::getClass() );
	PatchGraphMerge::init( Persistent::getClass() );
	PatchGraphResample::init( Persistent::getClass() );
	PatchGraphMergeCandidateSelect::init( Persistent::getClass() );
	AnimationBSRContext::init( Persistent::getClass() );
	AnimationBSRRenderer::init( Persistent::getClass() );
	AnimationBSRDebugger::init( Persistent::getClass() );
	AbstractMergeExtrapolator::init( Persistent::getClass() );
	//SmallDeformationAlignment::init( AbstractMergeExtrapolator::getClass() );
	GeometricAlignment::init( AbstractMergeExtrapolator::getClass() );
	FeatureRegistration::init( Persistent::getClass() );
	RegistrationContext::init( Persistent::getClass() );
	DetectTopologicalErrorMerge::init( Persistent::getClass() );
	PatchGraphPruning::init( Persistent::getClass() );
	PatchGraphTriangleMesh::init( Persistent::getClass() );
	TopologyBreaker::init( Persistent::getClass() );
	DataPointAssignment::init( Persistent::getClass() );
	TriangleMeshTexture::init( Persistent::getClass() );
	PatchGraphComponentsHandler::init( Persistent::getClass() );
	PatchGraphOpticalFlow::init( Persistent::getClass() );
	Monomial2ndOrderDisplayer::init( GLEventObject::getClass() );

	// operations
	PatchGraphCreator3D::init(Persistent::getClass());
	RefFrameEst3D::init(Persistent::getClass());
	PatchApprox3D::init(Persistent::getClass());
	UnifyNormals::init(Persistent::getClass());

	ComputeSampleSpacing::init(Persistent::getClass());
	ImplicitPatchGraphMeshing::init(Persistent::getClass());
	HFBFBivariateMonomials::init(Persistent::getClass());

	// rendering
	PatchGraphRenderer::init(Persistent::getClass());
	StandardPatchGraphRenderer::init(PatchGraphRenderer::getClass());


	FeatureDescriptor::init( Persistent::getClass() );
		FDAttribute::init(FeatureDescriptor::getClass());
		FDSIFT::init(FeatureDescriptor::getClass());
		FDHistogram::init(FeatureDescriptor::getClass());
		FDColorPCA::init( FeatureDescriptor::getClass() );
		FDClusteredHistogram::init( FeatureDescriptor::getClass() );
		FDCurvature::init( FeatureDescriptor::getClass() );
	Feature::init( Persistent::getClass() );
	FeatureList::init( Persistent::getClass() );
	FeatureDetector::init( Persistent::getClass() );
	LocalExtremaDetector::init( FeatureDetector::getClass() );
	ParametricFeatureDetector::init( FeatureDetector::getClass() );
	MLSMultiScaleFeatureDetector::init(FeatureDetector::getClass());
	LocalParameterization::init( Persistent::getClass() );
	HeighfieldParameterization::init( LocalParameterization::getClass() );
	PGDescriptorChannels::init( Persistent::getClass() );
	PGDescriptorFactory::init( PGDescriptorChannels::getClass() );
	PGDescriptorChannelsTest::init( PGDescriptorChannels::getClass() );
	
	DescriptorClustering::init( FeatureDetector::getClass() );
	FDAverage::init( FeatureDescriptor::getClass() );
	FDCombined::init( FeatureDescriptor::getClass() );
	FastFeature::init( Persistent::getClass() );
	FeatureAttachment::init(AttachedData::getClass() );
	CentroidDescriptor::init(Persistent::getClass() );

	Correspondence::init( Persistent::getClass() );
	CorrespondenceList::init( Persistent::getClass() );
	FeatureCorrespondences::init( Persistent::getClass() );
	PatchGraphTopologicalResampling::init( Persistent::getClass() );

	CmdObjPointCloudDistortion::init( ProcessPointsCmdObj::getClass() );


	// PCI
	PCIPatchGraphReconstruction3D::init( PCInteractionTool::getClass() );
	PCIAnimationBSR::init( PCInteractionTool::getClass() );
	PCIFeatureTest::init(PCInteractionTool::getClass());
	PCIPatchGraphBSR::init(PCInteractionTool::getClass());
	PCIGlobalRegistration::init( PCInteractionTool::getClass());

	// PCC
	PCCResamplePointCloud::init(PCCommand::getClass());
	PCCAnimationCreator::init( PCCommand::getClass() );
	PCCPrepareAnimPointCloud::init(PCCommand::getClass());
	PCCConvertPointCloudTypeInList::init(PCCommand::getClass());
	PCCAddOutliers::init(PCCommand::getClass());

	// GUI
#ifdef GEOXL_GUI
	PCIFeatureTestEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )PCIFeatureTest::getClass()->getGUIDescriptor())->addClassEditor(
		PCIFeatureTestEditor::getClass(), true);
	PatchGraphMergeCandidateSelectEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )PatchGraphMergeCandidateSelect::getClass()->getGUIDescriptor())->addClassEditor(
		PatchGraphMergeCandidateSelectEditor::getClass(), true);
#endif

	// non-linear opt
	BSROptContext::init(Persistent::getClass());
	BSROptPotential::init(Persistent::getClass());
	BOPGaussianPrjLikelihood::init(BSROptPotential::getClass());
	BOPSimpleICPLikelihood::init(BSROptPotential::getClass());
	BOPPrjSmoothness::init(BSROptPotential::getClass());
	BOPSpringBasedRigidity::init(BSROptPotential::getClass());
	BOPMultiResPotential::init(BSROptPotential::getClass());
	BOPDataLeastSquares::init(BSROptPotential::getClass());
	BOPSquaredDistancePotential::init(BSROptPotential::getClass());
	BOPMarkerErrorPotential::init(BSROptPotential::getClass());
	BOPPlanarSmoothnessPotential::init(BSROptPotential::getClass());
	BOPNormalSmoothnessPotential::init(BSROptPotential::getClass());
	BOPCircularSmoothnessPotential::init(BSROptPotential::getClass());
	BOPLaplacianSmoothnessPotential::init(BSROptPotential::getClass());
	BOPPositionRigidityPotential::init(BSROptPotential::getClass());
	BOPNormalRigidityPotential::init(BSROptPotential::getClass());
	BOPSTPositionCoherence::init(BSROptPotential::getClass());
	BOPSTNormalCoherence::init(BSROptPotential::getClass());
	BOPSTRotationCoherence::init(BSROptPotential::getClass());
	BSROptConfig::init(Persistent::getClass());


	//optimization2
	OptimizationGraph::init(Persistent::getClass());
	PatchGraphOpt2::init(Persistent::getClass());
	PGOpt2Frame::init(Persistent::getClass());
	PGOpt2FramePair::init(Persistent::getClass());
	PGOpt2FramePairPair::init(Persistent::getClass());
	PGOpt2Parameters::init(Persistent::getClass());
	PGOpt2Potential::init(Persistent::getClass());
	 PGOpt2SquaredDistancePotential_Global::init(PGOpt2Potential::getClass());
	 PGOpt2SquaredDistancePotential_Pairwise::init(PGOpt2Potential::getClass());
	 PGOpt2MarkerErrorPotential::init(PGOpt2Potential::getClass());
	 PGOpt2SmoothnessPotential::init(PGOpt2Potential::getClass());
	 PGOpt2RigidityPotential::init(PGOpt2Potential::getClass());
	 PGOpt2TemporalCoherence::init(PGOpt2Potential::getClass());
	 PGOpt2STColorSimilarity::init(PGOpt2Potential::getClass());
	PGOpt2PotentialSelector::init(Persistent::getClass());
	PGOpt2FramePotentialSelector::init(Persistent::getClass());
	PGOpt2FramePairPotentialSelector::init(Persistent::getClass());
	PGOpt2ThreeFramePotentialSelector::init(Persistent::getClass());
	PGOpt2Processing::init(Persistent::getClass());
	   PGOpt2Processing_varIndex::init(PGOpt2Processing::getClass());
	   PGOpt2Processing_Normal_Orientation::init(PGOpt2Processing::getClass());
	   PGOpt2Processing_Normal_Estimation::init(PGOpt2Processing::getClass());
	   PGOpt2Processing_Initial_Rotation_Matrices::init(PGOpt2Processing::getClass());
   PGOpt2Config::init(Persistent::getClass());

   GlobalOptimization2::init(Persistent::getClass());
   HoleFilling2::init(Persistent::getClass());
   PairwiseAlignment2::init(Persistent::getClass());
   PatchGraphJoin2::init(Persistent::getClass());
   PatchGraphMerge2::init(Persistent::getClass());
   TopologyMerging2::init(Persistent::getClass());
   PatchGraphResampler2::init(Persistent::getClass());

	PatchGraphBSROpt::init(Persistent::getClass());
	PGTopologicalNeighborhood::init(Persistent::getClass());
	PGTNKnn::init(PGTopologicalNeighborhood::getClass());
	PGTopologicalNeighborhoodDataPts::init(Persistent::getClass());
	TNDOneRing::init(PGTopologicalNeighborhoodDataPts::getClass());

}


void geoxl::shutdownAnimationReconstruction() {
	PointCloudAnimationSequence::shutdown();
	TrajectorySet::shutdown();
	PGDataPointIndices::shutdown();
	PatchGraph::shutdown();
	PatchGraphContainer::shutdown();

	// Patch graph
	PatchGraphCreator3D::shutdown();
	RefFrameEst3D::shutdown();
	PatchApprox3D::shutdown();
	UnifyNormals::shutdown();
	ComputeSampleSpacing::shutdown();
	ImplicitPatchGraphMeshing::shutdown();
	HFBFBivariateMonomials::shutdown();

	PatchGraphExtract::shutdown();
	PatchGraphMerge::shutdown();
	PatchGraphResample::shutdown();
	PatchGraphMergeCandidateSelect::shutdown();
	AnimationBSRContext::shutdown();
	AnimationBSRRenderer::shutdown();
	AnimationBSRDebugger::shutdown();
	AbstractMergeExtrapolator::shutdown();
	FeatureRegistration::shutdown();
	RegistrationContext::shutdown();
	//SmallDeformationAlignment::shutdown();
	GeometricAlignment::shutdown();
	DetectTopologicalErrorMerge::shutdown();
	PatchGraphPruning::shutdown();
	PatchGraphTriangleMesh::shutdown();
	TopologyBreaker::shutdown();
	DataPointAssignment::shutdown();
	TriangleMeshTexture::shutdown();
	PatchGraphComponentsHandler::shutdown();
	PatchGraphOpticalFlow::shutdown();
	Monomial2ndOrderDisplayer::shutdown();

	PatchGraphRenderer::shutdown();
	StandardPatchGraphRenderer::shutdown();

	Feature::shutdown();
	FeatureList::shutdown();
	FeatureDetector::shutdown();
	LocalExtremaDetector::shutdown();
	ParametricFeatureDetector::shutdown();
	MLSMultiScaleFeatureDetector::shutdown();
	LocalParameterization::shutdown();
	HeighfieldParameterization::shutdown();
	CmdObjPointCloudDistortion::shutdown();
	FeatureDescriptor::shutdown();
	FDAttribute::shutdown();
	FDSIFT::shutdown();
	FDHistogram::shutdown();
	Correspondence::shutdown();
	CorrespondenceList::shutdown();
	FeatureCorrespondences::shutdown();
	PatchGraphTopologicalResampling::shutdown();
	PGDescriptorFactory::shutdown();
	PGDescriptorChannels::shutdown();
	PGDescriptorChannelsTest::shutdown();
	FDColorPCA::shutdown();
	FDClusteredHistogram::shutdown();
	FDCurvature::shutdown();
	DescriptorClustering::shutdown();
	FDAverage::shutdown();
	FDCombined::shutdown();
	FastFeature::shutdown();
	FeatureAttachment::shutdown();
	CentroidDescriptor::shutdown();

	// PCID
	PCIPatchGraphReconstruction3D::shutdown();
	PCIAnimationBSR::shutdown();
	PCIFeatureTest::shutdown();
	PCIPatchGraphBSR::shutdown();
	PCIGlobalRegistration::shutdown();

	// PCC
	PCCResamplePointCloud::shutdown();
	PCCAnimationCreator::shutdown();
	PCCPrepareAnimPointCloud::shutdown();
	PCCConvertPointCloudTypeInList::shutdown();
	PCCAddOutliers::shutdown();

	// GUI
#ifdef GEOXL_GUI
	PCIFeatureTestEditor::shutdown();
	PatchGraphMergeCandidateSelectEditor::shutdown();
#endif

	// non-linear opt
	BSROptContext::shutdown();
	BSROptPotential::shutdown();
	BOPGaussianPrjLikelihood::shutdown();
	BOPSimpleICPLikelihood::shutdown();
	BOPPrjSmoothness::shutdown();
	BOPSpringBasedRigidity::shutdown();
	BOPMultiResPotential::shutdown();
	BOPSquaredDistancePotential::shutdown();
	BOPMarkerErrorPotential::shutdown();
	BOPPlanarSmoothnessPotential::shutdown();
	BOPNormalSmoothnessPotential::shutdown();
	BOPCircularSmoothnessPotential::shutdown();
	BOPLaplacianSmoothnessPotential::shutdown();
	BOPPositionRigidityPotential::shutdown();
	BOPNormalRigidityPotential::shutdown();
	BOPSTPositionCoherence::shutdown();
	BOPSTNormalCoherence::shutdown();
	BOPSTRotationCoherence::shutdown();
	BOPSimpleCenterLikelihood::shutdown();
	BOPSimpleSpaceTime::shutdown();
	BOPDataLeastSquares::shutdown();
	BSROptConfig::shutdown();
	PatchGraphBSROpt::shutdown();
	PGTopologicalNeighborhood::shutdown();
	PGTNKnn::shutdown();
	PGTopologicalNeighborhoodDataPts::shutdown();
	TNDOneRing::shutdown();


	//optimization2
	OptimizationGraph::shutdown();
	PatchGraphOpt2::shutdown();
	PGOpt2Frame::shutdown();
	PGOpt2FramePair::shutdown();
	PGOpt2FramePairPair::shutdown();
	PGOpt2Parameters::shutdown();
	PGOpt2Potential::shutdown();
   PGOpt2SquaredDistancePotential_Global::shutdown();
   PGOpt2SquaredDistancePotential_Pairwise::shutdown();
   PGOpt2MarkerErrorPotential::shutdown();
   PGOpt2SmoothnessPotential::shutdown();
   PGOpt2RigidityPotential::shutdown();
   PGOpt2TemporalCoherence::shutdown();
   PGOpt2STColorSimilarity::shutdown();
	PGOpt2PotentialSelector::shutdown();
	PGOpt2FramePotentialSelector::shutdown();
	PGOpt2FramePairPotentialSelector::shutdown();
	PGOpt2ThreeFramePotentialSelector::shutdown();
	PGOpt2Processing::shutdown();
	PGOpt2Processing_varIndex::shutdown();
	PGOpt2Processing_Normal_Orientation::shutdown();
	PGOpt2Processing_Normal_Estimation::shutdown();
	PGOpt2Processing_Initial_Rotation_Matrices::shutdown();
   PGOpt2Config::shutdown();

   GlobalOptimization2::shutdown();
   HoleFilling2::shutdown();
   PairwiseAlignment2::shutdown();
   PatchGraphJoin2::shutdown();
   PatchGraphMerge2::shutdown();
   TopologyMerging2::shutdown();
   PatchGraphResampler2::shutdown();
}







