#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIFeatureTestEditorWidget.h"
//---------------------------------------------------------------------------
#include "PCIFeatureTest.h"
#include "elementEditors/ObjectElementEditor.h"
#include "FeatureDetector.h"
#include "FeatureDescriptor.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( PCIFeatureTest )

PCIFeatureTestEditorWidget::PCIFeatureTestEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//Example: easy access to QtDesigner buttons, etc. by ui object
	//QWidget::connect(ui.loadButton,SIGNAL(clicked()),this,SLOT(sltLoadFile()));

}

PCIFeatureTestEditorWidget::~PCIFeatureTestEditorWidget(void)
{
}

void PCIFeatureTestEditorWidget::setup( PCIFeatureTest * obj )
{
	m_Object = obj;
	ObjectElementEditor * edDetector = new ObjectElementEditor(ui.frameDetector);
	edDetector->resize( ui.frameDetector->size() );
	edDetector->setup( (Object *&)m_Object->m_FeatureDetector, FeatureDetector::getClass() );

	ObjectElementEditor * edDescriptor = new ObjectElementEditor(ui.frameDescriptor);
	edDescriptor->resize( ui.frameDescriptor->size() );
	edDescriptor->setup( (Object *&)m_Object->m_FeatureDescriptor, FeatureDescriptor::getClass() );
}

void PCIFeatureTestEditorWidget::updateEditor()
{
	ui.cbDistortionMode->setCurrentIndex( m_Object->getDistortionMode()-1 );
	ui.dDistortionStrength->setValue( m_Object->getDistortionStrength() );
	ui.iNumGravityBalls->setValue( m_Object->getDistortionNumGravityBalls() );
	ui.iRandomSeed->setValue( m_Object->getDistortionRandomSeed() );
	ui.iNumGravityBalls->setEnabled( m_Object->getDistortionMode() == 3 );
	ui.bHideInitialPairs->setChecked( m_Object->getHideInitialPairs() );
	ui.dShowPairsPercentage->setValue( m_Object->getShowPairsPercentage() );
	ui.bDrawGradients->setChecked( m_Object->getDrawGradients() );
	ui.dGradientLength->setValue( m_Object->getGradientLength() );
	ui.dRelativeFeatureSize->setValue( m_Object->getRelativeFeatureSize() );
	ui.bDrawFeatureScale->setChecked( m_Object->getDrawFeatureScale() );
	ui.dSpectralSigma->setValue( m_Object->getSpectralSigma() );
	ui.dSpectralAddDiff->setValue( m_Object->getSpectralAddDiff() );
	ui.dSpectralScaleDiff->setValue( m_Object->getSpectralScaleDiff() );
	ui.dDescClusterRadius->setValue( m_Object->getDescClusterRadius() );
}

#include "PCIFeatureTestEditorWidget_moc.cpp"

void PCIFeatureTestEditorWidget::on_cbDistortionMode_currentIndexChanged(int i) 
{
	m_Object->setDistortionMode(i+1);
	ui.iNumGravityBalls->setEnabled( i == 2 );
}

void PCIFeatureTestEditorWidget::on_dDistortionStrength_valueChanged(double v) 
{
	m_Object->setDistortionStrength(v);
}

void PCIFeatureTestEditorWidget::on_pbDistort_clicked() 
{
	m_Object->distortPointClouds();
}

void PCIFeatureTestEditorWidget::on_iNumGravityBalls_valueChanged(int v) 
{
	m_Object->setDistortionNumGravityBalls(v);
}

void PCIFeatureTestEditorWidget::on_iRandomSeed_valueChanged(int v) 
{
	m_Object->setDistortionRandomSeed(v);
}

void PCIFeatureTestEditorWidget::on_pbTestDetector_clicked() 
{
	m_Object->testDetector( ui.iNumDetTests->value() );
}

void PCIFeatureTestEditorWidget::on_pbTestDescriptor_clicked() 
{
	m_Object->testDescriptor( ui.iNumDescTests->value(), ui.iNumSamples->value(), ui.dFeatureRadius->value());
}

void PCIFeatureTestEditorWidget::on_pbTestRegistration_clicked() 
{
	m_Object->testRegistration( ui.iMaxNumberOfPairs->value(), ui.dRegFeatureRadius->value(),
		ui.dPruningThreshold->value());
}

void PCIFeatureTestEditorWidget::on_bHideInitialPairs_clicked() 
{
	m_Object->setHideInitialPairs( ui.bHideInitialPairs->checkState() == Qt::Checked );
}

void PCIFeatureTestEditorWidget::on_dShowPairsPercentage_valueChanged(double d) 
{
	m_Object->setShowPairsPercentage(d);
}

void PCIFeatureTestEditorWidget::on_bDrawGradients_clicked() 
{
	m_Object->setDrawGradients( ui.bDrawGradients->isChecked() );
}

void PCIFeatureTestEditorWidget::on_dGradientLength_valueChanged(double d) 
{
	m_Object->setGradientLength(d);
}

void PCIFeatureTestEditorWidget::on_dRelativeFeatureSize_valueChanged(double d) 
{
	m_Object->setRelativeFeatureSize(d);
}

void PCIFeatureTestEditorWidget::on_bDrawFeatureScale_clicked() 
{
	m_Object->setDrawFeatureScale( ui.bDrawFeatureScale->isChecked() );
}

void PCIFeatureTestEditorWidget::on_pbCalcFeatures_clicked() 
{
	m_Object->calculateFeatures( ui.dRegFeatureRadius->value() );
}

void PCIFeatureTestEditorWidget::on_pbValidatePairs_clicked() 
{
	m_Object->validateFeaturePairs();
}

void PCIFeatureTestEditorWidget::on_pbPruning_clicked() 
{
	m_Object->buildFeaturePairs( ui.iMaxNumberOfPairs->value(), ui.dPruningThreshold->value() );
}

void PCIFeatureTestEditorWidget::on_pbTestGeodeticDistances_clicked() 
{
	m_Object->testGeodeticDistances();
}

void PCIFeatureTestEditorWidget::on_pbRecalcDesc_clicked() 
{
	m_Object->recalculateFeatureDescriptors(ui.dRegFeatureRadius->value());
}

void PCIFeatureTestEditorWidget::on_dSpectralSigma_valueChanged(double d) 
{
	m_Object->setSpectralSigma(d);
}

void PCIFeatureTestEditorWidget::on_dSpectralAddDiff_valueChanged(double d) 
{
	m_Object->setSpectralAddDiff(d);
}

void PCIFeatureTestEditorWidget::on_dSpectralScaleDiff_valueChanged(double d) 
{
	m_Object->setSpectralScaleDiff(d);
}

void PCIFeatureTestEditorWidget::on_pbDebugMultiScale_clicked() 
{
	m_Object->createMultiScalePointClouds();
}

void PCIFeatureTestEditorWidget::on_dDescClusterRadius_valueChanged(double d) 
{
	m_Object->setDescClusterRadius(d);
}

void PCIFeatureTestEditorWidget::on_pbCalculateCurvature_clicked() 
{
	m_Object->calculateCurvatureForAlleClouds( ui.dCurvatureSigma->value(), 2.0f, ui.bMedianFilter->isChecked());
}

void PCIFeatureTestEditorWidget::on_pbTestDetectorStability_clicked() 
{
	m_Object->testDetectorStability_Rigid();
}

void PCIFeatureTestEditorWidget::on_pbGlobalRegFeatures_clicked() 
{
	m_Object->calculateAllFeatures( ui.dRegFeatureRadius->value() );
}

void PCIFeatureTestEditorWidget::on_pbGlobalRegTransform_clicked() 
{
	m_Object->registerAllClouds( ui.dRegFeatureRadius->value(), ui.iMaxNumberOfPairs->value(), ui.dPruningThreshold->value() );
}

void PCIFeatureTestEditorWidget::on_pbTestDescriptorStability_clicked() 
{
	m_Object->testDescriptorStability();
}

void PCIFeatureTestEditorWidget::on_pbTestValidation_clicked() 
{
	m_Object->testValidation();
}

void PCIFeatureTestEditorWidget::on_pbCopyTrans_clicked() 
{
	m_Object->copyTransformationsToClipboard();
}

void PCIFeatureTestEditorWidget::on_pbPasteTrans_clicked() 
{
	m_Object->pasteTransformationsFromClipboard();
}
