//---------------------------------------------------------------------------
#include "StdAfx.h"
#include "PatchGraphMergeCandidateSelectEditorWidget.h"
#include "PatchGraphMergeCandidateSelect.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( PatchGraphMergeCandidateSelect )

PatchGraphMergeCandidateSelectEditorWidget::PatchGraphMergeCandidateSelectEditorWidget( QWidget * parent, Qt::WFlags f ) : QWidget( parent, f )
{
	obj = NULL;
	ui.setupUi( this );
}

PatchGraphMergeCandidateSelectEditorWidget::~PatchGraphMergeCandidateSelectEditorWidget()
{
}

void PatchGraphMergeCandidateSelectEditorWidget::setup( PatchGraphMergeCandidateSelect* obj )
{
	this->obj = obj;
	PatchGraphContainer* pgContainer = obj->getContext()->getPGContainer();
	Notifyer* notifyer = pgContainer->getNotifyer();
	connect( notifyer, SIGNAL( sigChanged() ), this, SLOT( sltPGContainerChanged() ) );
}

void PatchGraphMergeCandidateSelectEditorWidget::updateEditor()
{
	initFields();
}

/** 
 * Init manual selection fields based on current patch graph container
 */
void PatchGraphMergeCandidateSelectEditorWidget::initFields()
{
	ui.CB_mergeCrossing->clear();
	PatchGraphContainer* pgContainer = obj->getContext()->getPGContainer();
	for ( int32 frameIndex = pgContainer->getFrameStart(); frameIndex < pgContainer->getFrameEnd(); frameIndex++ ){
		ui.CB_mergeCrossing->addItem( QString::number( frameIndex ) + "<->" + QString::number( frameIndex + 1 ) );
	}
	initMergeCandidates();
}

/** 
 * Init manual selection merge candidate list based on current patch graph container and crossing selection
 */
void PatchGraphMergeCandidateSelectEditorWidget::initMergeCandidates()
{
	if ( obj == NULL ) return;
	QString text = ui.CB_mergeCrossing->currentText(); 
	int32 index = text.indexOf( "<->" );
	int32 frameIndex = text.left( index ).toInt();
	// Find possible merge candidates
	ui.CB_cand1->clear();
	ui.CB_cand2->clear();
	PatchGraphContainer* pgContainer = obj->getContext()->getPGContainer();
	for ( unsigned int pgIndex = 0; pgIndex < pgContainer->getNumPatchGraphs(); pgIndex++ ){
		PatchGraph* pg = pgContainer->getPatchGraph( pgIndex );
		if ( pg->getStartFrameIndex() + pg->getTrajectorySet()->getNumFrames() -1 == frameIndex ){
			ui.CB_cand1->addItem( QString::number( pgIndex ) );
		} else if ( pg->getStartFrameIndex() == frameIndex + 1 ){
			ui.CB_cand2->addItem( QString::number( pgIndex ) );
		}
	}
	obj->setCand1( ui.CB_cand1->currentText().toInt() );
	obj->setCand2( ui.CB_cand2->currentText().toInt() );
}

void PatchGraphMergeCandidateSelectEditorWidget::on_CB_mergeCrossing_currentIndexChanged( const QString& text )
{
	initMergeCandidates();
}

void PatchGraphMergeCandidateSelectEditorWidget::on_CB_cand1_currentIndexChanged( const QString& text )
{
	int32 i = text.toInt();
	if ( i > 0 ) obj->setCand1( i );
}

void PatchGraphMergeCandidateSelectEditorWidget::on_CB_cand2_currentIndexChanged( const QString& text )
{
	int32 i = text.toInt();
	if ( i > 0 ) obj->setCand2( i );
}

void PatchGraphMergeCandidateSelectEditorWidget::sltPGContainerChanged()
{
	initFields();
}

void PatchGraphMergeCandidateSelectEditorWidget::on_PB_selectAutomatical_clicked()
{
	if ( obj == NULL ) return;
	obj->select();
}

#include "PatchGraphMergeCandidateSelectEditorWidget_moc.cpp"