//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "AbstractMergeExtrapolator.h"
#include "PointSetKNNQuery.h"
#include "InCorePCTopologyGraph.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_ABSTRACT_CLASS( AbstractMergeExtrapolator, 0 )
{
	BEGIN_CLASS_INIT( AbstractMergeExtrapolator )
}

/***
* Find topologically close neighbors based on similar trajectories in past frames
*/
vector<card32> AbstractMergeExtrapolator::findTopologicalNeighbors( TrajectorySet* ts, 
															  const card32 patchIndex, 
															  const PointSet* ps, 
															  AttachedIndexedOctree* octree,
															  const card32 frameIndex,
															  const card32 lastFrameIndex,
															  const card32 intervStart, 
															  const card32 intervEnd, 
															  const card32 numNeighbors )
{
	vector<card32> neighbors; neighbors.clear();
	PointSetKNNQuery knn( ps, octree, NULL_VECTOR3F, ps->getAAT("position") );
	AAT POSITION = ts->getTrajectories()->getAAT( "position" );
	card32 nIndex;
	bool found;

	Vector3f pos = ts->getTrajectories()->get3f( frameIndex, patchIndex, POSITION );
	Vector3f posLast = ts->getTrajectories()->get3f( lastFrameIndex, patchIndex, POSITION );
	Vector3f traj = pos - posLast;

	knn.setTargetPoint( pos );
	// Skip self
	knn.getNextNearestPointIndex( nIndex, found );
	Vector3f nPos = ts->getTrajectories()->get3f( frameIndex, nIndex, POSITION );
	while( ( neighbors.size() < numNeighbors ) && found ){
		Vector3f nPos = ts->getTrajectories()->get3f( frameIndex, nIndex, POSITION );
		if ( ( nIndex >= intervStart ) && ( nIndex <= intervEnd ) ){
			Vector3f nPosLast = ts->getTrajectories()->get3f( lastFrameIndex, nIndex, POSITION );
			Vector3f nTraj = nPos - nPosLast;
			// TODO: Test, if trajectories match in previous frame
			if ( true ){
				neighbors.push_back( nIndex );
			}
		}
		knn.getNextNearestPointIndex( nIndex, found );
	}
	return neighbors;
}