//---------------------------------------------------------------------------
#ifndef AbstractMergeExtrapolatorH
#define AbstractMergeExtrapolatorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "PatchGraph.h"
#include "AnimationBSRContext.h"
//---------------------------------------------------------------------------

namespace geoxl {

class DebugRenderer;
class FeatureRegistration;

/// Merge patch graphs
class ANIMATIONRECONSTRUCTION_API AbstractMergeExtrapolator : public Persistent {
	GEOXL_ABSTRACT_CLASS( AbstractMergeExtrapolator )

	protected:
		/// Find topologically neighboring trajectories (not finished yet)
		virtual vector<card32> findTopologicalNeighbors( 
			TrajectorySet* ts, 
			const card32 patchIndex, 
			const PointSet* ps, 
			AttachedIndexedOctree* octree,
			const card32 frameIndex,
			const card32 lastFrameIndex, 
			const card32 intervStart, 
			const card32 intervEnd,
			const card32 numNeighbors );
		
	public:
		/// Apply extrapolation operation
		virtual void compute( 
			AnimationBSRContext* context,
			FeatureRegistration *featureMatcher,
			const PatchGraph* pg1, const PatchGraph* pg2, 
			PatchGraph* combinedPG,
			DebugRenderer *debugRenderer,  
			const bool verbose = false ) = 0;

		/// Apply extrapolation operation
		virtual void compute_Modified( 
			AnimationBSRContext* context,
			FeatureRegistration *featureMatcher,
			const PatchGraph* pg1, const PatchGraph* pg2, 
			PatchGraph* combinedPG,
			const map<card32, Vector3f> &featureCorrespondences12,
			const map<card32, Vector3f> &featureCorrespondences21,
			DebugRenderer *debugRenderer,  
			const bool verbose = false ) = 0;

		virtual void globalAssembly(AnimationBSRContext* context, 
			PatchGraph* combinedPG,
			DebugRenderer *debugRenderer,  
			const bool verbose = false ) = 0;
};
}

#endif