//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "AnimationBSRContext.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"
#include "ClassMethods.h"
#include "BinaryObjectStreams.h"
#include "RegistrationContext.h"
#include "SeparatorClassProperty.h"
#include "PGOpt2Config.h"
#include "PGOpt2Parameters.h"
#include "PGOpt2PotentialSelector.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------

 string AnimationBSRContext::getContextDataSetDescr( const card32 index )
{
	if ( index == TWO_CUBES ) return "Two cubes";
	else if ( index == BUNNY ) return "Bunny";
	else if ( index == BUST ) return "Bust";
	else if ( index == THEO_GYM ) return "Theo Gym";
	else if ( index == FACE ) return "Face";
	else if ( index == THEO_DANCE ) return "Theo Dance";
	else if ( index == FINGERS ) return "Fingers";
	else if ( index == DANCER_ONLY ) return "Dancer only";
	else if ( index == MAX_PLANCK ) return "Max Planck";
	else if ( index == SPOK ) return "Spok";
	else if ( index == PMD ) return "PMD";
	else if ( index == VENUS ) return "Venus";
	else if ( index == FACE_BENNI ) return "Face Benni";
	else if ( index == BEN ) return "Ben";
	else if ( index == ELEPHANT ) return "Elephant";
	else if ( index == TORSO ) return "Torso";
	else if ( index == ARM ) return "Arm";
	else if ( index == BAER ) return "Baer";
	else return "Undefined";
}


static string getICPRegistrationModeDescr( const card32 index ){
	if ( index == ICP_REGISTER_RIGID ) return "Rigid";
	else if ( index == ICP_REGISTER_SUBDIVIDED ) return "Subdivided";
	else return "Undefined";
}

static string getMergeCandSelectionModeDescr( const card32 index )
{
	if ( index == SELECTION_MODE_MANUAL ) return "Manual";
	else if ( index == SELECTION_MODE_AUTOMATIC ) return "Automatic";
	else return "Undefined";
}

static string descrMergeSel(card32 i) {
   if (i==0) return "MERGE_MIN_DEFORM_ENERGY";
   if (i==1) return "MERGE_MIN_TOTAL_ENERGY";
   return "MERGE_MIN_DEFORM_PLUS_DISTANCE_ENERGY";
}

IMPLEMENT_GEOXL_CLASS( AnimationBSRContext, 22 )
{
	BEGIN_CLASS_INIT( AnimationBSRContext )
	ADD_SEPARATOR("environment")
	ADD_CARD32_PROP_UPDATE_DESCR( dataSet, 0, NULL, getContextDataSetDescr, 18 )
	ADD_OBJECT_PROP( pgContainer, 0, PatchGraphContainer::getClass(), NULL )
	ADD_OBJECT_PROP( registrationContext, 3, RegistrationContext::getClass(),true);
	ADD_SEPARATOR("general parameters")
	ADD_BOOLEAN_PROP( saveAfterEachMerge, 21 )
	ADD_CARD32_PROP_UPDATE_DESCR( mergeCandSelectionMode, 0, NULL, getMergeCandSelectionModeDescr, 2 )
	ADD_FLOAT32_PROP( samplingSpacing, 0 )
	ADD_FLOAT32_PROP( dataPointsInfluence, 1 )
	ADD_FLOAT32_PROP( pgRegionGrowScale, 10 )
	ADD_FLOAT32_PROP( extractKNNFactor, 11 )
	ADD_CARD32_PROP( minPatchGraphSize, 6 )
	ADD_BOOLEAN_PROP( urshapeOpt, 7 )
	SET_USER_EDIT(false)
	ADD_BOOLEAN_PROP( urshapeMerge, 7 )
	SET_USER_EDIT(false)
	ADD_BOOLEAN_PROP( urshapeGlobalOpt, 14 )
	ADD_BOOLEAN_PROP( urshapeToScene, 14 )
	ADD_BOOLEAN_PROP( sceneToUrshape, 14 )
	ADD_BOOLEAN_PROP( cyclicTime, 14 )
	ADD_BOOLEAN_PROP( useOldPipeline, 13 )
	ADD_BOOLEAN_PROP( verbose, 8 )
   ADD_FLOAT32_PROP(maxRelEdgeLength, 12)
   ADD_FLOAT32_PROP(maxEdgeStretchFactor, 12)

	ADD_SEPARATOR("topology2")
   ADD_CARD32_PROP(topology2NumNearestNeighbors, 16)
   ADD_FLOAT32_PROP(topology2MaxDistanceRelSampleSpacing, 16)
   ADD_FLOAT32_PROP(topology2MaxAdditiveEdgeLengthDev, 16)
   ADD_FLOAT32_PROP(topology2MaxMultiplicativeEdgeLengthDev, 16)
   ADD_FLOAT32_PROP(topology2MinMultRejectLengthFraction, 16)
   ADD_CARD32_PROP_UPDATE_DESCR(topology2MergeSelCriterion, 17, NULL, descrMergeSel, 3)
   ADD_CARD32_PROP(topology2minimumVertexDegree, 20)

	//ADD_SEPARATOR("hole filling 2")
   //ADD_BOOLEAN_PROP(hfill2UseTemporalPrediction, 17)

	ADD_SEPARATOR("opt graph topology 2")
   ADD_BOOLEAN_PROP(optGraph2CyclicTime, 18)
	ADD_BOOLEAN_PROP(optGraph2twoWayTemporalEdges, 18)
	ADD_BOOLEAN_PROP(optGraph2useUrshape, 18)
	ADD_BOOLEAN_PROP(alignment2FeaturesFirstDataSecond, 22)

	ADD_SEPARATOR("tensor voting")
	ADD_FLOAT32_PROP( tvSigma, 0 )
	ADD_FLOAT32_PROP( tvCurvaturePenalty, 0 )
	ADD_FLOAT32_PROP( tvSurfacenessThreshold, 0 )
	ADD_SEPARATOR("rendering")
	ADD_FLOAT32_PROP( renderScale, 2 )
	ADD_SEPARATOR("topology merging (multiples of sampleSpacing)")
	ADD_FLOAT32_PROP( topMergePruneDistFactor, 4 )	
	ADD_FLOAT32_PROP( topEdgeResampleSearchRadiusFactor, 4 )	
	ADD_FLOAT32_PROP( topEdgeResamplePruneRadiusFactor, 4 )	
	ADD_SEPARATOR("icp merging")
	ADD_CARD32_PROP_UPDATE_DESCR( icpRegistrationMode, 0, NULL, getICPRegistrationModeDescr, 2 )
	ADD_INT32_PROP( numMergeSteps, 6 )
	ADD_SEPARATOR("optimization parameters")
	ADD_OBJECT_PROP(initialOptConfig, 19, PGOpt2Config::getClass(), true)
	ADD_OBJECT_PROP(pairwiseAlignmentConfig, 15, PGOpt2Config::getClass(), true)
	ADD_OBJECT_PROP(holeFillingConfig, 15, PGOpt2Config::getClass(), true)
	ADD_OBJECT_PROP(globalOptConfig, 15, PGOpt2Config::getClass(), true)
	
	ADD_NOARGS_METHOD( setParameters )
}

/**
 * Constructor
 */
AnimationBSRContext::AnimationBSRContext() 
{
	dataSet = BAER;
	tvSigma = -1.0f;
	tvCurvaturePenalty = -1.0f;
	tvSurfacenessThreshold = -1.0f;
	samplingSpacing = -1.0f;
	dataPointsInfluence = 1.5f;
	pgContainer =  new PatchGraphContainer();
	icpRegistrationMode = ICP_REGISTER_RIGID;
	renderScale = 0.3f;
	registrationContext = new RegistrationContext;
	mergeCandSelectionMode = SELECTION_MODE_AUTOMATIC;
	topMergePruneDistFactor = 3.0f;
	topEdgeResampleSearchRadiusFactor = 3.0f;
	topEdgeResamplePruneRadiusFactor = 2.0f;
	numMergeSteps = -1;
	urshapeOpt = false;
	urshapeMerge = false;
	minPatchGraphSize = 20;
	icpSubsampling = 1;
	verbose = true; 
	pgRegionGrowScale = 0.5f;
	extractKNNFactor = 2.0f;
   maxEdgeStretchFactor = 1.3f;
   maxRelEdgeLength = 4;
   useOldPipeline = false;
	urshapeGlobalOpt = false;
	urshapeToScene = false;
	sceneToUrshape = false;
	cyclicTime = false;
	topology2MergeSelCriterion = MERGE_MIN_DEFORM_PLUS_DISTANCE_ENERGY;
	saveAfterEachMerge = false;

   topology2NumNearestNeighbors = 8;
   topology2MaxDistanceRelSampleSpacing = 2.0f;
   topology2MaxAdditiveEdgeLengthDev = 1.0e30f;
   topology2MaxMultiplicativeEdgeLengthDev = 1.5f;
   topology2MinMultRejectLengthFraction = 0.3f;
   topology2minimumVertexDegree = 3;
   
   optGraph2CyclicTime = true;
	optGraph2twoWayTemporalEdges = false;
	optGraph2useUrshape = true;
	alignment2FeaturesFirstDataSecond = false;

   pairwiseAlignmentConfig = new PGOpt2Config();
	holeFillingConfig = new PGOpt2Config();
   globalOptConfig = new PGOpt2Config();
   initialOptConfig = new PGOpt2Config();

	initialOptConfig->potentials.preprocessingEstimateNormals = true;
	initialOptConfig->potentials.preprocessingUnifyNormalOrientation = true;
   initialOptConfig->potentials.preprocessingComputeRotations = false;
   initialOptConfig->potentials.preprocessingWriteVarIndices = true;

   initialOptConfig->potentials.framePotentials.distancePairwise = false;
   initialOptConfig->potentials.framePotentials.distanceGlobalOpt = true;
   initialOptConfig->potentials.framePotentials.distanceMarkers = false;
   initialOptConfig->potentials.framePotentials.colorSimilarity = false;
   initialOptConfig->potentials.framePotentials.smoothness = true;
   initialOptConfig->potentials.framePairPotentials.rigidity = false;
   initialOptConfig->potentials.threeFramePotentials.acceleration = false;

	initialOptConfig->params->numIterations             =   5;
	initialOptConfig->params->convergenceEps            =   0.001f;
	initialOptConfig->params->tradeoff_sdf              =   0.9f;
	initialOptConfig->params->weightSquaredDistance     =   2.0f;
	initialOptConfig->params->weightMarkerError         =   0.0f;
	initialOptConfig->params->weightPlanarSmoothness    =   0.001f;
	initialOptConfig->params->weightNormalSmoothness    =   0.001f;
	initialOptConfig->params->weightLaplacianSmoothness =   0.001f;
	initialOptConfig->params->weightPositionRigidity    =   0.01f; 
	initialOptConfig->params->weightNormalRigidity      =   0.01f; 
	initialOptConfig->params->weightPositionCoherence   =   0.0f;
	initialOptConfig->params->weightNormalCoherence     =   0.0f;
   initialOptConfig->params->weightRotationCoherence   =   0.0f;
	initialOptConfig->params->cgStopRes                 =   0.001f;
	initialOptConfig->params->cgMaxNumIterations        = 500;
	initialOptConfig->params->verbose                   = true;

	pairwiseAlignmentConfig->potentials.preprocessingEstimateNormals = true;
	pairwiseAlignmentConfig->potentials.preprocessingUnifyNormalOrientation = true;
   pairwiseAlignmentConfig->potentials.preprocessingComputeRotations = true;
   pairwiseAlignmentConfig->potentials.preprocessingWriteVarIndices = true;
   
   pairwiseAlignmentConfig->potentials.framePotentials.distanceMarkers = true;
   pairwiseAlignmentConfig->potentials.framePotentials.distancePairwise = true;
   pairwiseAlignmentConfig->potentials.framePotentials.distanceGlobalOpt = false;
   pairwiseAlignmentConfig->potentials.framePotentials.colorSimilarity = false;
   pairwiseAlignmentConfig->potentials.framePotentials.smoothness = true;
   pairwiseAlignmentConfig->potentials.framePairPotentials.rigidity = true;
   pairwiseAlignmentConfig->potentials.threeFramePotentials.acceleration = false;
	
   pairwiseAlignmentConfig->params->numIterations             =   20;
   pairwiseAlignmentConfig->params->convergenceEps            =    0.001f;
   pairwiseAlignmentConfig->params->tradeoff_sdf              =    0.5f;
   pairwiseAlignmentConfig->params->overlapping_rate          =    0.85f;
   pairwiseAlignmentConfig->params->weightSquaredDistance     =    2.0f;
   pairwiseAlignmentConfig->params->weightMarkerError         =   10.0f;
   pairwiseAlignmentConfig->params->weightPlanarSmoothness    =    0.0000001f;
   pairwiseAlignmentConfig->params->weightNormalSmoothness    =    0.0000001f;
   pairwiseAlignmentConfig->params->weightLaplacianSmoothness =    0.0000001f;
   pairwiseAlignmentConfig->params->weightPositionRigidity    =   25.0f; 
   pairwiseAlignmentConfig->params->weightNormalRigidity      =    0.0f;   
   pairwiseAlignmentConfig->params->weightPositionCoherence   =   0.0f;
   pairwiseAlignmentConfig->params->weightNormalCoherence     =   0.0f;
   pairwiseAlignmentConfig->params->weightRotationCoherence   =   0.0f;
   pairwiseAlignmentConfig->params->cgStopRes                 =   0.001f;
   pairwiseAlignmentConfig->params->cgMaxNumIterations        = 500;
   pairwiseAlignmentConfig->params->verbose                   = true;


	holeFillingConfig->potentials.preprocessingEstimateNormals = true;
	holeFillingConfig->potentials.preprocessingUnifyNormalOrientation = true;
   holeFillingConfig->potentials.preprocessingComputeRotations = true;
   holeFillingConfig->potentials.preprocessingWriteVarIndices = true;

   holeFillingConfig->potentials.framePotentials.distancePairwise = false;
   holeFillingConfig->potentials.framePotentials.distanceGlobalOpt = true;
   holeFillingConfig->potentials.framePotentials.distanceMarkers = false;
   holeFillingConfig->potentials.framePotentials.colorSimilarity = false;
   holeFillingConfig->potentials.framePotentials.smoothness = true;
   holeFillingConfig->potentials.framePairPotentials.rigidity = true;
   holeFillingConfig->potentials.threeFramePotentials.acceleration = false;

   holeFillingConfig->params->numIterations             =  20;
   holeFillingConfig->params->convergenceEps            =   0.0001f;
   holeFillingConfig->params->tradeoff_sdf              =   0.5f;
   holeFillingConfig->params->weightSquaredDistance     =   2.0f;
   holeFillingConfig->params->weightMarkerError         =   0.0f;
   holeFillingConfig->params->weightPlanarSmoothness    =   0.0001f;
   holeFillingConfig->params->weightNormalSmoothness    =   0.0001f;
   holeFillingConfig->params->weightLaplacianSmoothness =   0.0001f;
   holeFillingConfig->params->weightPositionRigidity    =  25.0f;
   holeFillingConfig->params->weightNormalRigidity      =   0.0f; 
   holeFillingConfig->params->weightPositionCoherence   =   0.0f;
   holeFillingConfig->params->weightNormalCoherence     =   0.0f;
   holeFillingConfig->params->weightRotationCoherence   =   0.0f;
   holeFillingConfig->params->cgStopRes                 =   0.001f;
   holeFillingConfig->params->cgMaxNumIterations        = 500;
   holeFillingConfig->params->verbose                   = true;


	globalOptConfig->potentials.preprocessingEstimateNormals = true;
	globalOptConfig->potentials.preprocessingUnifyNormalOrientation = true;
   globalOptConfig->potentials.preprocessingComputeRotations = true;
   globalOptConfig->potentials.preprocessingWriteVarIndices = true;

   globalOptConfig->potentials.framePotentials.distancePairwise = false;
   globalOptConfig->potentials.framePotentials.distanceGlobalOpt = true;
   globalOptConfig->potentials.framePotentials.distanceMarkers = false;
   globalOptConfig->potentials.framePotentials.colorSimilarity = false;
   globalOptConfig->potentials.framePotentials.smoothness = true;
   globalOptConfig->potentials.framePairPotentials.rigidity = true;
   globalOptConfig->potentials.threeFramePotentials.acceleration = false;

	globalOptConfig->params->numIterations             =   7;
	globalOptConfig->params->convergenceEps            =   0.001f;
	globalOptConfig->params->tradeoff_sdf              =   0.9f;
	globalOptConfig->params->weightSquaredDistance     =   2.0f;
	globalOptConfig->params->weightMarkerError         =   0.0f;
	globalOptConfig->params->weightPlanarSmoothness    =   0.0001f;
	globalOptConfig->params->weightNormalSmoothness    =   0.0001f;
	globalOptConfig->params->weightLaplacianSmoothness =   0.0001f;
	globalOptConfig->params->weightPositionRigidity    =  25.0f; 
	globalOptConfig->params->weightNormalRigidity      =   0.0f; 
	globalOptConfig->params->weightPositionCoherence   =   1.0f; // deactivated above (run in separate pass)
	globalOptConfig->params->weightNormalCoherence     =   1.0f; // deactivated above (run in separate pass)
   globalOptConfig->params->weightRotationCoherence   =   0.0f;
	globalOptConfig->params->cgStopRes                 =   0.001f;
	globalOptConfig->params->cgMaxNumIterations        = 500;
	globalOptConfig->params->verbose                   = true;
	
	setParameters();
}

/**
 * Destructor
 */
AnimationBSRContext::~AnimationBSRContext() 
{
	delete pgContainer;
	delete registrationContext;
	delete pairwiseAlignmentConfig;
	delete holeFillingConfig;
	delete globalOptConfig;
}

/** 
* Set the parameters required for each test dataset
*/
void AnimationBSRContext::setParameters()
{
	debugOutput << "Using template setting " << getContextDataSetDescr( dataSet ) << ".\n"; debugOutput.flush();
	if ( dataSet == TWO_CUBES ){
		samplingSpacing = 0.07f;
		tvSigma = 0.2f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.2f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 0.5f;
		extractKNNFactor = 2.0f;
	} else if ( dataSet == BUNNY ){
		samplingSpacing = 0.00075f;
		tvSigma = 0.004f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.1f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.05f;
		icpSubsampling = 10;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 0.5f;
		extractKNNFactor = 2.0f;
	} else if ( dataSet == BUST ){
		samplingSpacing = 0.025f;
		tvSigma = 0.02f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.15f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.2f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 0.5f;
		extractKNNFactor = 4.0f;
	} else if ( dataSet == THEO_GYM ){
		samplingSpacing = 0.02f;
		tvSigma = 0.05f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.155f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 0.5f;								  
		extractKNNFactor = 2.0f;
	} else if ( dataSet == FACE ){
		samplingSpacing = 0.015f;
		tvSigma = 0.02f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.1f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.15f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 0.5f;
		extractKNNFactor = 2.0f;
	} else if ( dataSet == THEO_DANCE ){
		samplingSpacing = 0.02f;
		tvSigma = 0.03f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.125f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 0.5f;
		extractKNNFactor = 2.0f;
	} else if ( dataSet == FINGERS ){
		samplingSpacing = 0.03f;
		tvSigma = 0.09f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.01f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 0.5f;
		extractKNNFactor = 2.0f;
	} else if ( dataSet == DANCER_ONLY ){
		samplingSpacing = 0.02f;
		tvSigma = 0.03f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.15f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 250;
		pgRegionGrowScale = 1.0f;
		extractKNNFactor = 2.0f;
	} else if ( dataSet == MAX_PLANCK ){
		samplingSpacing = 0.02f;
		tvSigma = 0.04f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.1f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 250;
		pgRegionGrowScale = 1.0f;
		extractKNNFactor = 4.0f;
	} else if ( dataSet == SPOK ){
		samplingSpacing = 0.04f;
		tvSigma = 0.03f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.3f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 250;
		pgRegionGrowScale = 1.0f;
		extractKNNFactor = 4.0f;
	} else if ( dataSet == PMD ){
		samplingSpacing = 0.005f;
		tvSigma = 0.0075f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.1f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 1.0f;
		extractKNNFactor = 4.0f;
	} else if ( dataSet == VENUS ){
		samplingSpacing = 0.05f;
		//tvSigma = 0.04f;						// No noise
		tvSigma = 0.07f;						// Noise = 0.025			
		//tvSigma = 0.09f;						// Noise = 0.050			
		tvCurvaturePenalty = 0.05f;
		//tvSurfacenessThreshold = 0.15f;			// No noise
		tvSurfacenessThreshold = 0.09f;		// Noise = 0.025
		//tvSurfacenessThreshold = 0.05f;		// Noise = 0.050
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 1.0f;
		extractKNNFactor = 4.0f;
	} else if ( dataSet == FACE_BENNI ){
		samplingSpacing = 0.03f;
		tvSigma = 0.05f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.2f;	
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 1.0f;
		extractKNNFactor = 4.0f;
	} else if ( dataSet == BEN ){
		samplingSpacing = 0.03f;
		tvSigma = 0.03f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.2f;	
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 1.0f;
		extractKNNFactor = 4.0f;
	} else if ( dataSet == ELEPHANT ){
		samplingSpacing = 0.045f;
		tvSigma = 0.04f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.09f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 1.0f;
		extractKNNFactor = 4.0f;
	} else if ( dataSet == TORSO ){
		samplingSpacing = 0.01f;
		tvSigma = 0.015f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.09f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 1.0f;
		extractKNNFactor = 4.0f;
	} else if ( dataSet == ARM ){
		samplingSpacing = 0.04f;
		tvSigma = 0.1f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.09f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 1.0f;
		extractKNNFactor = 4.0f;
	}  else if ( dataSet == BAER ){
		samplingSpacing = 0.02f;
		tvSigma = 0.02f;
		tvCurvaturePenalty = 0.05f;
		tvSurfacenessThreshold = 0.09f;
		dataPointsInfluence = 1.5f;
		renderScale = 0.3f;
		icpSubsampling = 1;
		minPatchGraphSize = 100;
		pgRegionGrowScale = 1.0f;
		extractKNNFactor = 4.0f;
	}
}

/** 
 * Load patch graph container from file
 */
void AnimationBSRContext::loadPGContainer( const QString& filename )
{
	QFileInfo fileInfo( filename );
	if ( fileInfo.absoluteDir().exists() && fileInfo.exists() ){
		BinaryInputObjectStream in_str( filename.toLatin1() );
		in_str.readObject( (Persistent*&)pgContainer );
		if (!pgContainer->getInstanceClass()->inheritsFrom(PatchGraphContainer::getClass())) {
		   delete (Object*)pgContainer;
		   pgContainer = NULL;
		   warning("AnimationBSRContext::loadPGContainer() - loaded object is of wrong type.");
		   return;
		}
	} else {
		debugOutput << "loadPGContainer: invalid patch graph container filename!\n"; debugOutput.flush();
		return;
	}
	debugOutput << "Loaded patch graph container with " << pgContainer->getNumPatchGraphs() << " patch graphs.\n";
	debugOutput.flush();
}


/** 
* Save patch graph container from file
*/
void AnimationBSRContext::savePGContainer( const QString& filename )
{
	QFileInfo fileInfo( filename );
	if ( fileInfo.absoluteDir().exists() ){
		BinaryOutputObjectStream out_str( filename.toLatin1() );
		out_str.writeObject( (Persistent*&)pgContainer );
	} else {
		debugOutput << "savePGContainer: directory does not exist!\n";
		debugOutput.flush();
		return;
	}
	debugOutput << "Saved patch graph container with " << pgContainer->getNumPatchGraphs() << " patch graphs.\n";
	debugOutput.flush();
}