//---------------------------------------------------------------------------
#ifndef AnimationBSRDebuggerH
#define AnimationBSRDebuggerH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "AnimationBSRContext.h"
#include "SGSimpleAnimationNode.h"
#include "RenderingTool.h"
#include "ClassMethods.h"
#include "Scene.h"
#include "PatchGraphExtract.h"
#include "BSROptContext.h"
//---------------------------------------------------------------------------

namespace geoxl{

#define COLORIZE_DATA_BLUE 0
#define COLORIZE_DATA_SURFACENESS 1

/// Rendering and visualization functionality associated to the AnimationBSR tool
class ANIMATIONRECONSTRUCTION_API AnimationBSRDebugger : public Persistent {
	GEOXL_CLASS( AnimationBSRDebugger )

	private:
		/// Context
		AnimationBSRContext* context;
		/// Colorize data mode
		card32 colorizeDataMode;
		/// Scene object
		Scene* scene;
		/// Extractor object
		PatchGraphExtract* extractor;
		/// Index of patch graph to be debugged
		card32 patchGraphIndex;
		/// 
		card32 numOutliers;

		/*** INPUT DATA ACCESS ***/
		/// Get number of frames
		card32 getNumFrames() const;
		/// Get the data animation node
		SGSimpleAnimationNode* getDataAnimationNode() const;
		/// Return index of current frame
		int32 getCurrentFrameIndex() const;
		/// Get data point cloud for frame index
		UnstructuredInCorePointCloud* getDataFrame(card32 index );
		/// Get name of data frame point cloud 
		string getDataFrameName( card32 index ){ char buffer[10]; return std::string( "root/data/frame" ) + itoa( index, buffer, 10 ); }

	public:
		/// Constructor
		AnimationBSRDebugger();
		/// Destructor
		~AnimationBSRDebugger();

		/// Colorize data
		void colorizeData();
		/// Check attributes
		void checkAttributes();
		/// Compute reference frames
		void computeReferenceFrames();
		/// Compute height field monomials
		void computeHeightFieldMonomials();
		/// Optimize all patch graphs
		void optimizeAllPatchGraphs();
		/// Add Outliers
		void addOutliers();

		/*** MEMBER ACCESS ***/
		void setScene( Scene* scene );
		inline void setExtractor( PatchGraphExtract* extractor ){ this->extractor = extractor; }
		inline void setContext( AnimationBSRContext* context ){ this->context = context; }
};
}

#endif