//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "CorrespondenceExtrapolation.h"
//---------------------------------------------------------------------------
#include "PointCloudAnimationSequence.h"
#include "PatchGraph.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "InCorePCTopologyGraph.h"
#include "PGDataPointIndices.h"
#include "PatchGraphOpt2.h"
#include "AnimationBSRContext.h"
#include "PGOperationsDebugging.h"
#include "PatchGraphRenderer.h"
#include "PatchGraphWCorr.h"
#include "BSROptPotential.h"
#include "PGOpt2GraphCreator.h"
#include "PGOpt2FramePairVars.h"
#include "OptimizationGraph.h"
#include "PGOpt2Config.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------


void CorrespondenceExtrapolation::buildCorrExtrapolationPatchGraph(
	const PatchGraph *geometry,
	const PointCloudAnimationSequence *dataPoints,
	const int32 frameIndex,
	const int32 globalTargetFrameIndex,
	const vector<PatchCorrespondence> knownCorrespondences,
	PatchGraph *&extrapolationOptPatchGraph,
	PointCloudAnimationSequence *&extrapolationOptData)
{
	pAssert(geometry->getTrajectorySet()->getNumFrames() > 0);

	const PointSet *geops					= geometry->getTrajectorySet()->getTrajectories();
	const card32 numPts						= geops->getHeight();
	const card32 absDataFrameIndex	= frameIndex	+	geometry->getStartFrameIndex();
	const card32 numTargetPoints		= dataPoints->getFrame(globalTargetFrameIndex)->getHeight();

	// --- setup data point indices
	PGDataPointIndices *dp					= new PGDataPointIndices();
	dp->setup(2, geometry->getTrajectorySet()->getNumPatches());
	// copy first frame dps
//	dp->setFullFrame(0, geometry->getTrajectorySet()->getDataPoints()->getFullFrame(frameIndex));
	// set correspondences for second frame
	vector< vector<card32> > corrPoints(numPts);
	for (unsigned i=0; i<knownCorrespondences.size(); i++) 
	{
		card32 index = knownCorrespondences[i].patchIndex;
		pAssert(corrPoints.size() > index);
		corrPoints[index].push_back(i+numTargetPoints);
	}
	dp->setFullFrame(1, corrPoints);
	
	// --- setup trajectories (initialize second frame with first)


	PointSet *ps = new PointSet();
	ps->clearAndSetup(2, numPts, geops->getDescr());
	for (card32 i=0; i<numPts; i++) 
	{
		ps->setVertex(0, i, geops->getDataPointer(frameIndex, i));
		ps->setVertex(1, i, geops->getDataPointer(frameIndex, i));
	}
	
	TrajectorySet *ts = new TrajectorySet();
	ts->clearAndSetup(ps, dp);

	// --- assemble resulting patch graph
	InCorePCTopologyGraph *tg = (InCorePCTopologyGraph*)(geometry->getTopology()->copy());
	extrapolationOptPatchGraph = new PatchGraph();
	extrapolationOptPatchGraph->clearAndSetup(ts, tg);	 

	// --- create data points
	extrapolationOptData = new PointCloudAnimationSequence();
	{
		// part I: copy first frame's data points (position only)
		PointSet *dataPS1 = new PointSet();
		VertexDescriptor vd;
		vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
		AAT destAAT = vd.getToken("position");
		AAT sourceAAT = dataPoints->getFrame(absDataFrameIndex)->getAAT("position");
		dataPS1->clearAndSetup(1, dataPoints->getFrame(absDataFrameIndex)->getHeight(), &vd);
		for (card32 i=0; i<dataPoints->getFrame(absDataFrameIndex)->getHeight(); i++) 
		{
			dataPS1->set3f(i, destAAT, dataPoints->getFrame(absDataFrameIndex)->get3f(i, sourceAAT));
		}
		extrapolationOptData->addFrame(dataPS1);
	} 
	{
		// part II: copy second frame's data points and add correspondences
		PointSet *dataPS2 = new PointSet();
		VertexDescriptor vd;
		vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
		AAT destAAT			= vd.getToken("position");
		AAT sourceAAT = dataPoints->getFrame(globalTargetFrameIndex)->getAAT("position");
		dataPS2->clearAndSetup(1, numTargetPoints, &vd);
		for (card32 i=0; i<numTargetPoints; i++) 
		{
			dataPS2->set3f(i, destAAT, dataPoints->getFrame(globalTargetFrameIndex)->get3f(i, sourceAAT));
		}
		extrapolationOptData->addFrame(dataPS2);
	}
}

void CorrespondenceExtrapolation::buildCorrExtrapolationPatchGraph_modified(
	const	PatchGraph	*source,						///this is used to produce the initial geometry of the target frame
	const	PatchGraph	*target,						/// this patch graph contains the data points associated with the target frame
	const PointCloudAnimationSequence *dataPoints,
	const int32 frameIndex,
	const int32 localDataIndex,
	const int32 globalTargetFrameIndex,
	const vector<PatchCorrespondence> knownCorrespondences,
	const map<card32, Vector3f> &featureCorrespondences,
	PatchGraph *&extrapolationOptPatchGraph,
	PointCloudAnimationSequence *&extrapolationOptData)
{
	pAssert(source->getTrajectorySet()->getNumFrames() > 0);

	const PointSet *geops					= source->getTrajectorySet()->getTrajectories();
	const card32	numPts						= geops->getHeight();
	const card32 absDataFrameIndex	= frameIndex	+	source->getStartFrameIndex();
	
	// --- setup data point indices
	PGDataPointIndices *dp					= new PGDataPointIndices();
	dp->setup(2, source->getTrajectorySet()->getNumPatches());
	// copy first frame dps
	//	dp->setFullFrame(0, geometry->getTrajectorySet()->getDataPoints()->getFullFrame(frameIndex));
	// set correspondences for second frame
	vector< vector<card32> > corrPoints(numPts);
	for (unsigned i=0; i<knownCorrespondences.size(); i++) 
	{
		card32 index = knownCorrespondences[i].patchIndex;
		pAssert(corrPoints.size() > index);
///		corrPoints[index].push_back(i+numTargetPoints);
	}
	dp->setFullFrame(1, corrPoints);

	// --- setup trajectories (initialize second frame with first)


	PointSet *ps = new PointSet();
	ps->clearAndSetup(2, numPts, geops->getDescr());
	for (card32 i=0; i<numPts; i++) 
	{
		ps->setVertex(0, i, geops->getDataPointer(frameIndex, i));
		ps->setVertex(1, i, geops->getDataPointer(frameIndex, i));
	}

	TrajectorySet *ts = new TrajectorySet();
	ts->clearAndSetup(ps, dp);

	// --- assemble resulting patch graph
	InCorePCTopologyGraph *tg = (InCorePCTopologyGraph*)(source->getTopology()->copy());

	
	extrapolationOptPatchGraph = new PatchGraphWCorr();
	((PatchGraphWCorr*)extrapolationOptPatchGraph)->correspondences = featureCorrespondences;
	extrapolationOptPatchGraph->clearAndSetup(ts, tg);	 

	// --- create data points
	extrapolationOptData = new PointCloudAnimationSequence();
	{
		// part II: copy second frame's data points and add correspondences
/*		PointSet *dataPS = new PointSet();
		VertexDescriptor vd;
		vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
		vd.pushAttrib(mVAD("flag_inlier", 1, VAD::DATA_FORMAT_INT32));
		AAT destAAT			= vd.getToken("position");
		AAT	FLAG_INLIER	= vd.getToken("flag_inlier"); 
		AAT sourceAAT = dataPoints->getFrame(globalTargetFrameIndex)->getAAT("position");
		PGDataPointIndices	*dataIndices	= target->getTrajectorySet()->getDataPoints();
		const card32	height_target = target->getTrajectorySet()->getTrajectories()->getHeight();
		vector<card32>		dataindices_buf;

//		FILE *file = fopen("D:\\hqx1.txt", "w");

		///this part is really a temporary code
		for(card32 surfelIndex = 0; surfelIndex < height_target; surfelIndex++)
		{
//			fprintf(file, "%d %d\n", frameIndex, surfelIndex);
//			fflush(file);
			const card32 numDataPoints = dataIndices->getNumDataPoints(localDataIndex, surfelIndex);
			for(card32 id = 0; id < numDataPoints; id++)
			{
				card32	dataIndexInData = dataIndices->getIndex( localDataIndex, surfelIndex, id);
				dataindices_buf.push_back(dataIndexInData);
			}
		}
//		fclose(file);
		dataPS->clearAndSetup(1, dataindices_buf.size(), &vd);
		for (card32 id = 0; id < dataindices_buf.size(); id++) 
		{
			Vector3f	pos = dataPoints->getFrame(globalTargetFrameIndex)->get3f(dataindices_buf[id], sourceAAT);
			dataPS->set3f(id, destAAT, pos);
			dataPS->set<int32>(id, FLAG_INLIER, 1);
		}
		dataindices_buf.clear();
		extrapolationOptData->addFrame(dataPS);*/
		PointSet *dataPS = new PointSet();
		extrapolationOptData->addFrame(dataPS);
		dataPS = new PointSet();
		VertexDescriptor vd;
		vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
		vd.pushAttrib(mVAD("flag_inlier", 1, VAD::DATA_FORMAT_INT32));
		AAT destAAT			= vd.getToken("position");
		AAT	FLAG_INLIER	= vd.getToken("flag_inlier"); 
		const PointSet	*targetPS			= target->getTrajectorySet()->getTrajectories();
		AAT			sourceAAT			= targetPS->getAAT("position");
		const card32	height_target	= targetPS->getHeight();
		dataPS->clearAndSetup(1, height_target, &vd);
		for (card32 id = 0; id < height_target; id++) 
		{
			Vector3f	pos = targetPS->get3f(localDataIndex, id, sourceAAT);
			dataPS->set3f(id, destAAT, pos);
			dataPS->set<int32>(id, FLAG_INLIER, 1);
		}
		extrapolationOptData->addFrame(dataPS);
	}

	extrapolationOptPatchGraph->createPatchAttribute(mVAD("topology_depth", 1,	VAD::DATA_FORMAT_INT32));
	ps = extrapolationOptPatchGraph->getTrajectorySet()->getTrajectories();
	AAT		TOP_DEPTH	= ps->getAAT("topology_depth");
	for (card32 i=0; i<numPts; i++) 
	{
		ps->set<int32>(0, i, TOP_DEPTH, 0);
		ps->set<int32>(1, i, TOP_DEPTH, 0);
	}
}
// ---

PriorsBasedCorrExtrapolation::PriorsBasedCorrExtrapolation(
	AnimationBSRContext *context,
	PatchGraph *optPG, 
	PointCloudAnimationSequence *optData, 
	DebugRenderer *debugRenderer,
	PatchGraphRenderer *debugPGRenderer)
{
	this->context = context;
	this->optPG = optPG;
	this->optData = optData;
	this->debugRenderer = debugRenderer;
	this->debugPGRenderer = debugPGRenderer;
}

void	PriorsBasedCorrExtrapolation::peterDoOptimization(PatchGraphOpt2 *optimizer, PGOpt2Config *config)
{
	const card32 numPatches = optPG->getTrajectorySet()->getNumPatches();
	//BSROptVariableIndexSet indices_pos(numPatches);
	//BSROptVariableIndexSet	indices_nor(numPatches);
	//BSROptVariableIndexSet	indices_rot(numPatches);
	//vector<card32>				indices_data;
	//indices_data.push_back(1);

	//for (card32 i=0; i<numPatches; i++) 
	//{
	//	BSROptPointIndex e1, e2, e3;
	//	e1.frame				=	1;		e1.point				=	i;		indices_pos[i]		= e1;
	//	e2.frame				=	1;		e2.point				=	i;		indices_nor[i]		= e2;
	//	e3.frame				=	0;		e3.point				=	i;		indices_rot[i]		= e3;
	//}
	
	
   vector< vector<card32> > frameVarIndices(2);
   vector< vector<card32> > edgeVarIndices(2);
   frameVarIndices[1].resize(numPatches);
   edgeVarIndices[0].resize(numPatches);
   edgeVarIndices[1].resize(numPatches);
   for (card32 i=0; i<numPatches; i++) {
      frameVarIndices[1][i] = i;
      edgeVarIndices[0][i] = i;
      edgeVarIndices[1][i] = i;
   }

   config->potentials.configure(optimizer);

   PGOpt2GraphCreator gc;
   PGOpt2FramePairVars *pairVars = gc.createPairVars(optPG, false, false, false, false, false);
   OptimizationGraph *optGraph = gc.createOptGraph(optPG, optData, pairVars, false, false, false, false, false, &frameVarIndices, &edgeVarIndices);
   
	optimizer->optimize(
	   config->params,
	   context,
      optGraph, optPG, optData,	
      debugRenderer
   );

   delete pairVars;
   delete optGraph;

	//optimizer->optimization(&indices_pos, &indices_nor, &indices_rot, &indices_data,
	//	optData, optPG, NULL, 0, debugRedraw, this);
}
