//---------------------------------------------------------------------------
#ifndef CorrespondenceExtrapolationH
#define CorrespondenceExtrapolationH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PGOperationsDebugging.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------


#include "SparseLinearAlgebra.h"
#include "SparseLinearAlgebra_templCode.h"

namespace geoxl {
	using namespace std;

class PointCloudAnimationSequence;
class PatchGraph;
class AnimationBSRContext;
class DebugRenderer;
class PatchGraphRenderer;
class PatchGraphOpt2;
class PGOpt2Config;


class CorrespondenceExtrapolation {
 public:
	struct PatchCorrespondence {
		card32 patchIndex;
		Vector3f targetPos;
	};

	static void buildCorrExtrapolationPatchGraph(
		const PatchGraph *geometry,	
		const PointCloudAnimationSequence *dataPoints,
		const int32 frameIndex,
		const int32 targetFrameIndex,
		const vector<PatchCorrespondence> knownCorrespondences,
		PatchGraph *&extrapolationOptPatchGraph,
		PointCloudAnimationSequence *&extrapolationOptData
	);

	static void buildCorrExtrapolationPatchGraph_modified(
		const PatchGraph *source,
		const PatchGraph *target,
		const PointCloudAnimationSequence *dataPoints,
		const int32 frameIndex,
		const int32 localDataIndex,
		const int32 targetFrameIndex,
		const vector<PatchCorrespondence> knownCorrespondences,
		const map<card32, Vector3f> &featureCorrespondences,
		PatchGraph *&extrapolationOptPatchGraph,
		PointCloudAnimationSequence *&extrapolationOptData
		);
};

class PriorsBasedCorrExtrapolation {
 private:
	AnimationBSRContext *context;
	PatchGraph *optPG;
	PointCloudAnimationSequence *optData;
	DebugRenderer *debugRenderer;
	PatchGraphRenderer *debugPGRenderer;
	
 public:
	PriorsBasedCorrExtrapolation(
		AnimationBSRContext *context,
		PatchGraph *optPG, 
		PointCloudAnimationSequence *optData, 
		DebugRenderer *debugRenderer,
		PatchGraphRenderer *debugPGRenderer	
	);
	
	/// peter's method to do optimization
	/// I use different configuration for optimization
	void		peterDoOptimization(PatchGraphOpt2 *optimizer, PGOpt2Config *config);
};
	
	


}

#endif