//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "DataPointAssignment.h"
#include "PGDataPointIndices.h"
#include "PointSetKNNQuery.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS( DataPointAssignment, 1 )
{
	BEGIN_CLASS_INIT( DataPointAssignment )
}

/** 
 * Assign data points to all patch graphs in container (max one patch per data point)
 */
void DataPointAssignment::assignDataPoints( PatchGraph* pg, const PointCloudAnimationSequence* animSeq, const float32 dataInfluenceRadius )
{
	AAT DATA_POSITION = animSeq->getFrame( 0 )->getAAT( "position" );
	card32 nIndex;
	bool found;
	PGDataPointIndices* newDataPointIndices = new PGDataPointIndices();
	newDataPointIndices->setup( pg->getTrajectorySet()->getNumFrames(), pg->getTrajectorySet()->getNumPatches() );
	AAT POSITION = pg->getTrajectorySet()->getTrajectories()->getAAT( "position" );
	for ( unsigned int frameIndex = 0; frameIndex < pg->getTrajectorySet()->getNumFrames(); frameIndex++ ){
		PointSet* ps = pg->getTrajectorySet()->copySingleFrame( frameIndex );
		AttachedIndexedOctree *octree = PointSetKNNQuery::createOctree( ps );
		PointSetKNNQuery knn( ps, octree, NULL_VECTOR3F, POSITION );
		card32 globalFrameIndex = frameIndex + pg->getStartFrameIndex();
		if ( globalFrameIndex >= animSeq->getNumFrames() ){
			debugOutput << "assignDataPoints: invalid animation sequence.\n"; debugOutput.flush();
			return;
		}
		for ( unsigned int dataPointIndex = 0; dataPointIndex < animSeq->getFrame( globalFrameIndex )->getNumEntries(); dataPointIndex++ ){
			Vector3f dataPos = animSeq->getFrame( globalFrameIndex )->get3f( dataPointIndex, DATA_POSITION );
			knn.setTargetPoint( dataPos );
			knn.getNextNearestPointIndex( nIndex, found );
			if ( found ){
				Vector3f patchPos = ps->get3f( nIndex, POSITION );
				if ( norm( patchPos - dataPos ) < dataInfluenceRadius ){
					newDataPointIndices->addIndex( frameIndex, nIndex, dataPointIndex );
				}
			}
		}
		delete octree;
		delete ps;
	}
	pg->getTrajectorySet()->setDataPoints( newDataPointIndices );
}