//---------------------------------------------------------------------------
#ifndef DetectTopologicalErrorMergeH
#define DetectTopologicalErrorMergeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
#include "Persistent.h"
#include "PatchGraph.h"
#include "AnimationBSRContext.h"
#include "PointSetKNNQuery.h"
//---------------------------------------------------------------------------

namespace geoxl {

/// Filter detects wrong additional topological connections at merge operation
class ANIMATIONRECONSTRUCTION_API DetectTopologicalErrorMerge : public Persistent {
	GEOXL_CLASS( DetectTopologicalErrorMerge )

	private:
		/// Context
		AnimationBSRContext* context;
		/// Minimum number of data points in for a patch to be valid
		int32 numDataPointsThreshold;
		/// Error detection threshold (invalid patch)
		float32 invalidTrajThreshold;
		/// Error detection threshold (invalid topology in patch graph)
		float32 invalidTopoThreshold;

		vector<AttachedIndexedOctree*> octrees;
		vector<PointSetKNNQuery> knns;
		vector<PointSet*> pss;

		/// Check if a trajectory is invalid
		bool isInvalidTrajectory ( 
			const PatchGraph* pg, 
			card32 patchIndex, 
			const card32 startFrame, 
			const card32 endFrame,
			const card32 patchOffset
		);
		/// Compute ratio of invalid patches in a part of a patch graph
		float32 computeInvalids( 
			const PatchGraph* pg, 
			const card32 startFrame, 
			const card32 endFrame,
			const card32 patchOffset );
		
	public:
		/// Constructor
		DetectTopologicalErrorMerge();
		/// Destructor
		~DetectTopologicalErrorMerge();

		/// Free mem
		void freeMemory();

		/// Detect topological error in patch graph in merge operation; return true when error was detected
		bool detectTopoError( 
			PatchGraph* combinedPG,						// Patch graph created in merge operation
			PatchGraph* pg[2],							// Patch graphs used in merge operation
			card32& replacePG,							// Index of input patch graphs to be replaced
			vector<PatchGraph*>& newPatchGraphs );		// Newly created patch graphs with valid topology; replace input patch graph

		/*** MEMBER ACCESS **/
		inline void setContext( AnimationBSRContext* context ){ this->context = context; }

};
}

#endif