//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "GeometricAlignment.h"
//---------------------------------------------------------------------------
#include "PGOpt2Parameters.h"
#include "PatchGraphOpt2.h"
#include "VariableArrayClassProperty.h"
#include "ClassTypeClassProperty.h"
#include "ObjectClassProperty.h"
#include "CorrespondenceExtrapolation.h"
#include "PatchGraphRenderer.h"
#include "NearestNeighborTopologyMerging.h"
#include "LocalTopologyResampling.h"
#include "PGOpt2Config.h"
#include "PatchTypes.h"
#include "PGNeighborhoodExtrapolator4D.h"
#include "FeatureRegistration.h"
#include "PGDataPointIndices.h"
#include "PatchGraphWCorr.h"
#include "PGOpt2GraphCreator.h"
#include "PGOpt2FramePairVars.h"
#include "OptimizationGraph.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS(GeometricAlignment, 0)
{
	BEGIN_CLASS_INIT(GeometricAlignment);
	ADD_OBJECT_PROP(pairwiseConfig,							0, PGOpt2Config::getClass(), true);
	ADD_OBJECT_PROP(correspondenceBasedPairwiseConfig,	0, PGOpt2Config::getClass(), true);
	ADD_OBJECT_PROP(partialOptimizationConfig,			0, PGOpt2Config::getClass(), true);
	ADD_OBJECT_PROP(globalAssemblyConfig,					0, PGOpt2Config::getClass(), true);
}


// ----


GeometricAlignment::GeometricAlignment() 
{
	optimizer = new PatchGraphOpt2();

	pairwiseConfig	= new PGOpt2Config();
	
	pairwiseConfig->potentials.preprocessingEstimateNormals = true;
	pairwiseConfig->potentials.preprocessingUnifyNormalOrientation = true;
   pairwiseConfig->potentials.preprocessingComputeRotations = true;
   pairwiseConfig->potentials.preprocessingWriteVarIndices = true;
   
   pairwiseConfig->potentials.framePotentials.distanceMarkers = true;
   pairwiseConfig->potentials.framePotentials.distancePairwise = true;
   pairwiseConfig->potentials.framePotentials.distanceGlobalOpt = false;
   pairwiseConfig->potentials.framePotentials.colorSimilarity = false;
   pairwiseConfig->potentials.framePotentials.smoothness = true;
   pairwiseConfig->potentials.framePairPotentials.rigidity = true;
   pairwiseConfig->potentials.threeFramePotentials.acceleration = false;
	
   pairwiseConfig->params->numIterations = 20;
   pairwiseConfig->params->convergenceEps = 0.001f;
   pairwiseConfig->params->tradeoff_sdf = 1.0f;
   pairwiseConfig->params->overlapping_rate = 0.85f;
   pairwiseConfig->params->weightSquaredDistance = 1.0f;
   pairwiseConfig->params->weightMarkerError = 0.0f;
   pairwiseConfig->params->weightPlanarSmoothness = 0.05f;
   pairwiseConfig->params->weightNormalSmoothness = 0.01f;
   pairwiseConfig->params->weightLaplacianSmoothness = 0.5f;
   pairwiseConfig->params->weightPositionRigidity = 200.0f;
   pairwiseConfig->params->weightNormalRigidity = 0.01f;
   pairwiseConfig->params->weightPositionCoherence = 0.0f;
   pairwiseConfig->params->weightNormalCoherence = 0.0f;
   pairwiseConfig->params->weightRotationCoherence = 0.1f;
   pairwiseConfig->params->cgStopRes = 0.001f;
   pairwiseConfig->params->cgMaxNumIterations = 100;
	
	correspondenceBasedPairwiseConfig = new PGOpt2Config();
	

	correspondenceBasedPairwiseConfig->potentials.preprocessingEstimateNormals = true;
	correspondenceBasedPairwiseConfig->potentials.preprocessingUnifyNormalOrientation = true;
   correspondenceBasedPairwiseConfig->potentials.preprocessingComputeRotations = true;
   correspondenceBasedPairwiseConfig->potentials.preprocessingWriteVarIndices = true;
   
   correspondenceBasedPairwiseConfig->potentials.framePotentials.distancePairwise = true;
   correspondenceBasedPairwiseConfig->potentials.framePotentials.distanceGlobalOpt = false;
   correspondenceBasedPairwiseConfig->potentials.framePotentials.distanceMarkers = true;
   correspondenceBasedPairwiseConfig->potentials.framePotentials.colorSimilarity = false;
   correspondenceBasedPairwiseConfig->potentials.framePotentials.smoothness = true;
   correspondenceBasedPairwiseConfig->potentials.framePairPotentials.rigidity = true;
   correspondenceBasedPairwiseConfig->potentials.threeFramePotentials.acceleration = false;
	
   correspondenceBasedPairwiseConfig->params->numIterations = 10;
   correspondenceBasedPairwiseConfig->params->convergenceEps = 0.0001f;
   correspondenceBasedPairwiseConfig->params->tradeoff_sdf = 0.5f;
   correspondenceBasedPairwiseConfig->params->weightSquaredDistance = 0.0f;
   correspondenceBasedPairwiseConfig->params->weightMarkerError = 4.0f;
   correspondenceBasedPairwiseConfig->params->weightPlanarSmoothness = 1.0f;
   correspondenceBasedPairwiseConfig->params->weightNormalSmoothness = 0.0f;
   correspondenceBasedPairwiseConfig->params->weightLaplacianSmoothness = 0.1f;
   correspondenceBasedPairwiseConfig->params->weightPositionRigidity = 5.0f;
   correspondenceBasedPairwiseConfig->params->weightNormalRigidity = 0.1f;
   correspondenceBasedPairwiseConfig->params->weightPositionCoherence = 0.0f;
   correspondenceBasedPairwiseConfig->params->weightNormalCoherence = 0.0f;
   correspondenceBasedPairwiseConfig->params->weightRotationCoherence = 0.1f;
   correspondenceBasedPairwiseConfig->params->cgStopRes = 0.001f;
   correspondenceBasedPairwiseConfig->params->cgMaxNumIterations = 100;

	partialOptimizationConfig = new PGOpt2Config();

	partialOptimizationConfig->potentials.preprocessingEstimateNormals = false;
	partialOptimizationConfig->potentials.preprocessingUnifyNormalOrientation = false;
   partialOptimizationConfig->potentials.preprocessingComputeRotations = true;
   partialOptimizationConfig->potentials.preprocessingWriteVarIndices = true;

   partialOptimizationConfig->potentials.framePotentials.distancePairwise = false;
   partialOptimizationConfig->potentials.framePotentials.distanceGlobalOpt = false;
   partialOptimizationConfig->potentials.framePotentials.distanceMarkers = false;
   partialOptimizationConfig->potentials.framePotentials.colorSimilarity = false;
   partialOptimizationConfig->potentials.framePotentials.smoothness = true;
   partialOptimizationConfig->potentials.framePairPotentials.rigidity = true;
   partialOptimizationConfig->potentials.threeFramePotentials.acceleration = false;

   partialOptimizationConfig->params->numIterations = 15;
   partialOptimizationConfig->params->convergenceEps = 0.0001f;
   partialOptimizationConfig->params->tradeoff_sdf = 0.5f;
   partialOptimizationConfig->params->weightSquaredDistance = 0.0f;
   partialOptimizationConfig->params->weightMarkerError = 0.0f;
   partialOptimizationConfig->params->weightPlanarSmoothness = 1.0f;
   partialOptimizationConfig->params->weightNormalSmoothness = 0.5f;
   partialOptimizationConfig->params->weightLaplacianSmoothness = 0.1f;
   partialOptimizationConfig->params->weightPositionRigidity = 10.0f;
   partialOptimizationConfig->params->weightNormalRigidity = 0.1f;
   partialOptimizationConfig->params->weightPositionCoherence = 0.0001f;
   partialOptimizationConfig->params->weightNormalCoherence = 0.0f;
   partialOptimizationConfig->params->weightRotationCoherence = 0.01f;
   partialOptimizationConfig->params->cgStopRes = 0.001f;
   partialOptimizationConfig->params->cgMaxNumIterations = 100;

	globalAssemblyConfig = new PGOpt2Config();

	globalAssemblyConfig->potentials.preprocessingEstimateNormals = true;
	globalAssemblyConfig->potentials.preprocessingUnifyNormalOrientation = true;
   globalAssemblyConfig->potentials.preprocessingComputeRotations = true;
   globalAssemblyConfig->potentials.preprocessingWriteVarIndices = true;

   globalAssemblyConfig->potentials.framePotentials.distancePairwise = false;
   globalAssemblyConfig->potentials.framePotentials.distanceGlobalOpt = true;
   globalAssemblyConfig->potentials.framePotentials.distanceMarkers = false;
   globalAssemblyConfig->potentials.framePotentials.colorSimilarity = false;
   globalAssemblyConfig->potentials.framePotentials.smoothness = true;
   globalAssemblyConfig->potentials.framePairPotentials.rigidity = true;
   globalAssemblyConfig->potentials.threeFramePotentials.acceleration = false;

	globalAssemblyConfig->params->numIterations = 7;
	globalAssemblyConfig->params->convergenceEps = 0.001f;
	globalAssemblyConfig->params->tradeoff_sdf = 0.9f;
	globalAssemblyConfig->params->weightSquaredDistance = 0.1f;
	globalAssemblyConfig->params->weightMarkerError = 0.0f;
	globalAssemblyConfig->params->weightPlanarSmoothness = 0.5f;
	globalAssemblyConfig->params->weightNormalSmoothness = 0.01f;
	globalAssemblyConfig->params->weightLaplacianSmoothness = 0.5f;
	globalAssemblyConfig->params->weightPositionRigidity = 5.0f;
	globalAssemblyConfig->params->weightNormalRigidity = 0.1f;
	globalAssemblyConfig->params->weightPositionCoherence = 0.0001f;
	globalAssemblyConfig->params->weightNormalCoherence = 0.0f;
	globalAssemblyConfig->params->cgStopRes = 0.001f;
	globalAssemblyConfig->params->cgMaxNumIterations = 200;

	visualization = new StandardPatchGraphRenderer();
	visualization->setUseDL(false);
	visualization->setShowTopology(true);
	visualization->setShowCoords(false);
	visualization->setShowDataPoints(true);
	visualization->setShowPoints(true);
	visualization->setShowLocalPatches(false);
	visualization->setShowAllFrames(true);
}

GeometricAlignment::~GeometricAlignment() 
{
	delete optimizer;
	delete pairwiseConfig;
	delete correspondenceBasedPairwiseConfig;
	delete partialOptimizationConfig;
	delete globalAssemblyConfig;
	delete visualization;
}

void GeometricAlignment::alignAndConnectParts_modified(
	AnimationBSRContext* context,
	const PatchGraph *part1, 
	const PatchGraph *part2,
	card32 part1FrameIndex, 
	card32 part2FrameIndex,
	card32 nextAfterPart1DataIndex,
	card32 destFrameIndex,
	card32 frameIndex_Modified,
	card32 extrapolatedPointsCopyIndex,
	card32 part1StartPatch,
	card32 part2StartPatch,
	PatchGraph *combinedPG,
	InCorePCTopologyGraph *topology,
	const vector<CorrespondenceExtrapolation::PatchCorrespondence> &knownCorrespondences,
	const map<card32, Vector3f> &featureCorrespondences,
	DebugRenderer *debugRenderer, 
	const bool verbose
	)
{
	PatchGraph *mergePG;
	PointCloudAnimationSequence *mergeDataPts;

	/// Build the patch graph for optimization
	CorrespondenceExtrapolation::buildCorrExtrapolationPatchGraph_modified(
		part1, part2, context->getPGContainer()->getAnimSeq(), 
		part1FrameIndex, part2FrameIndex, nextAfterPart1DataIndex, knownCorrespondences, 
		featureCorrespondences, mergePG, mergeDataPts 
		);

	pairwiseConfig->potentials.configure(optimizer);

	/// Do pairwise matching here
	mergeDataPts->createDataAttribute(mVAD("flag_inlier", 1, VAD::DATA_FORMAT_INT32));
	PriorsBasedCorrExtrapolation ce(context, mergePG, mergeDataPts, debugRenderer, visualization);
	ce.peterDoOptimization(optimizer, pairwiseConfig);

	{
		/// copy data to combinedPG
		const card32 numMergePGPoints	= mergePG->getTrajectorySet()->getNumPatches();
		PointSet	*ps_source			= mergePG->getTrajectorySet()->getTrajectories();
		PointSet	*combinedPS			= combinedPG->getTrajectorySet()->getTrajectories();
		const AAT AAT_DEPTH				= combinedPS->getAAT("topology_depth");
		for (card32 i=0; i<numMergePGPoints; i++) 
		{
			combinedPS->setVertex(destFrameIndex, i+extrapolatedPointsCopyIndex, ps_source->getDataPointer(1, i));
			combinedPS->set<int32>(destFrameIndex, i+extrapolatedPointsCopyIndex, AAT_DEPTH, 0);
		}
	}

	PointSet *interpolated			= mergePG->getTrajectorySet()->copySingleFrame(1);
	PointSet *target				= part2->getTrajectorySet()->copySingleFrame(part2FrameIndex);

	///The method for topology merging
	NearestNeighborTopologyMerging::connectNeighbors_Final(context,
		interpolated, target, part1, part2, topology,
		part1StartPatch,
		part2StartPatch,
		debugRenderer, verbose);

/*	NearestNeighborTopologyMerging::connectNeighbors_Modified(
		context, 
		interpolated, target, topology, 
		part1StartPatch,
		part2StartPatch,
		debugRenderer, verbose
		);

*/
	delete interpolated;
	delete target;

	delete mergePG;
	delete mergeDataPts;
}

void GeometricAlignment::alignAndConnectParts
  (AnimationBSRContext* context,
	const PatchGraph *part1,
	const PatchGraph *part2, 
	card32 part1FrameIndex, 
	card32 part2FrameIndex,
	card32 nextAfterPart1DataIndex,
	card32 destFrameIndex,
	card32 extrapolatedPointsCopyIndex,
	card32 part1StartPatch,
	card32 part2StartPatch,
	PatchGraph *combinedPG,
	const vector<CorrespondenceExtrapolation::PatchCorrespondence> &knownCorrespondences,
	DebugRenderer *debugRenderer, 
	const bool verbose)
{
	
	PatchGraph *mergePG;
	PointCloudAnimationSequence *mergeDataPts;

	map<card32, Vector3f> forgetMe;
	/// Build the patch graph for optimization
	CorrespondenceExtrapolation::buildCorrExtrapolationPatchGraph_modified(
		part1, part2, context->getPGContainer()->getAnimSeq(), 
		part1FrameIndex, part2FrameIndex, nextAfterPart1DataIndex, knownCorrespondences, 
		forgetMe, mergePG, mergeDataPts
	);

	pairwiseConfig->potentials.configure(optimizer);

	/// Do pairwise matching here
	PriorsBasedCorrExtrapolation ce(context, mergePG, mergeDataPts, debugRenderer, visualization);
	ce.peterDoOptimization(optimizer, pairwiseConfig);
	
	{
		/// copy data to combinedPG
		const card32 numMergePGPoints	= mergePG->getTrajectorySet()->getNumPatches();
		combinedPG->createPatchAttribute( mVAD("topology_depth",		1,		VAD::DATA_FORMAT_INT32));
		PointSet *ps_dest							= combinedPG->getTrajectorySet()->getTrajectories();
		PointSet *ps_source						= mergePG->getTrajectorySet()->getTrajectories();
		const AAT AAT_DEPTH					= ps_dest->getAAT("topology_depth");
		for (card32 i=0; i<numMergePGPoints; i++) 
		{
			ps_dest->setVertex(destFrameIndex, i+extrapolatedPointsCopyIndex, ps_source->getDataPointer(1, i));
			ps_dest->set<int32>(destFrameIndex, i+extrapolatedPointsCopyIndex, AAT_DEPTH, 0);
		}
	}

	PointSet *interpolated							= mergePG->getTrajectorySet()->copySingleFrame(1);
	PointSet *target									= part2->getTrajectorySet()->copySingleFrame(part2FrameIndex);
	
	///The method for topology merging
	NearestNeighborTopologyMerging::connectNeighbors(
		context, 
		interpolated, target, combinedPG, 
		part1StartPatch,
		part2StartPatch,
		debugRenderer, verbose
	);
	
	delete interpolated;
	delete target;
	  
	delete mergePG;
	delete mergeDataPts;
}

void GeometricAlignment::computeFeatureMatch
  (FeatureRegistration *featureMatcher,
   vector< pair<Vector3f, Vector3f> > &matchingPoints, 
	FeatureList *feat1,
	FeatureList *feat2)
{
	//vector<FeaturePair> pairs = featureMatcher->buildFeaturePairs(feat1, feat2, context->maxFeatureDistance);
	
}

void	GeometricAlignment::correspondenceBasedPairwiseMatching(
	AnimationBSRContext *context,
	const	card32	startSurfelIndex,
	const	card32	endSurfelIndex,
	PatchGraph *patchGraph,
	DebugRenderer *debugRenderer,  
	const bool verbose)
{
	PointSet	*ps = patchGraph->getTrajectorySet()->getTrajectories();
	
	card32		width = ps->getWidth(),
				height = ps->getHeight();

	pAssert(width == 2 && startSurfelIndex < endSurfelIndex && endSurfelIndex <= height && startSurfelIndex >= 0 );

	PointCloudAnimationSequence		*data = new PointCloudAnimationSequence();
	//Copy data sequence
	VertexDescriptor vd;
	vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(mVAD("flag_inlier", 1, VAD::DATA_FORMAT_INT32));
	AAT		destAAT		= vd.getToken("position");
	AAT		FLAG_INLIER	= vd.getToken("flag_inlier"); 
	AAT		sourceAAT	= ps->getAAT("position");
	PointSet	*dataPS = new PointSet();
	data->addFrame(dataPS);
	dataPS = new PointSet();
	dataPS->clearAndSetup(1, endSurfelIndex - startSurfelIndex, &vd);
	for (card32 id = startSurfelIndex; id < endSurfelIndex; id++) 
	{
		Vector3f	pos = ps->get3f(1, id, sourceAAT);
		dataPS->set3f(id - startSurfelIndex, destAAT, pos);
		dataPS->set<int32>(id - startSurfelIndex, FLAG_INLIER, 1);
	}
	data->addFrame(dataPS);

	//Copy geometry
	for (card32 i=0; i<height; i++) 
	{
		ps->setVertex(1, i, ps->getDataPointer(0, i));
	}
	//Configuration for optimization
	//BSROptVariableIndexSet		IndexSet_Pos;
	//BSROptVariableIndexSet		IndexSet_Nor;
	//BSROptVariableIndexSet		IndexSet_Rot;
	//vector<card32>				indices_data;	indices_data.push_back(1);
	//for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	//{
	//	BSROptPointIndex		e_pos_nor;	e_pos_nor.frame = 1; e_pos_nor.point = surfelIndex;
	//	IndexSet_Pos.push_back(e_pos_nor);
	//	IndexSet_Nor.push_back(e_pos_nor);
	//	BSROptPointIndex		e_rot;		e_rot.frame = 0;	e_rot.point = surfelIndex;
	//	IndexSet_Rot.push_back(e_rot);
	//}

	const card32 numPatches = ps->getHeight();
   vector< vector<card32> > frameVarIndices(2);
   vector< vector<card32> > edgeVarIndices(2);
   frameVarIndices[1].resize(numPatches);
   edgeVarIndices[0].resize(numPatches);
   edgeVarIndices[1].resize(numPatches);
   for (card32 i=0; i<numPatches; i++) {
      frameVarIndices[1][i] = i;
      edgeVarIndices[0][i] = i;
      edgeVarIndices[1][i] = i;
   }

	correspondenceBasedPairwiseConfig->potentials.configure(optimizer);
   PGOpt2GraphCreator gc;
   PGOpt2FramePairVars *pairVars = gc.createPairVars(patchGraph, false, false, false, false, false);
   OptimizationGraph *optGraph = gc.createOptGraph(patchGraph, data, pairVars, false, false, false, false, false, &frameVarIndices, &edgeVarIndices);
	optimizer->optimize(
	   correspondenceBasedPairwiseConfig->params,
	   context,
      optGraph, patchGraph,
		data,	
      debugRenderer
   );

   delete pairVars;
   delete optGraph;
	delete data;
}

void		Quaternion_To_Matrix3f_GA(const float	*q, Matrix3f &rotation)
{
	pAssert(q != NULL);
	rotation[0][0] = 2*q[1]*q[1] + (2*q[0]*q[0]-1);
	rotation[1][0] = 2*q[1]*q[2] - 2*q[0]*q[3];
	rotation[2][0] = 2*q[1]*q[3] + 2*q[0]*q[2];
	rotation[0][1] = 2*q[2]*q[1] + 2*q[0]*q[3];
	rotation[1][1] = 2*q[2]*q[2] + (2*q[0]*q[0]-1);
	rotation[2][1] = 2*q[2]*q[3] - 2*q[0]*q[1];
	rotation[0][2] = 2*q[3]*q[1] - 2*q[0]*q[2];
	rotation[1][2] = 2*q[3]*q[2] + 2*q[0]*q[1];
	rotation[2][2] = 2*q[3]*q[3] + (2*q[0]*q[0]-1);
}

void	GeometricAlignment::pairtialOptimization(
	AnimationBSRContext *context,
	const	card32	numOfPG1Frames,
	PatchGraph		*pg,
	PointCloudAnimationSequence	*data,
	DebugRenderer *debugRenderer)
{
	/// Do optimization on the left side
	PointSet								*ps		= pg->getTrajectorySet()->getTrajectories();
	InCorePCTopologyGraph	*tg		=	pg->getTopology();
	AAT							TOP_DEPTH	=	ps->getAAT("topology_depth");
	AAT							POSITION		= ps->getAAT("position");
	AAT							NORMAL			= ps->getAAT("normal");
	AAT							ROTATION		= ps->getAAT("rotation_matrix");
	AAT							REF_U				= ps->getAAT("tangent_u");
	card32							height		= ps->getHeight(),
		width		= ps->getWidth();

	if(numOfPG1Frames >= 2)
	{
		do 
		{
			/// Initialize the geometry, e.g position, normal and rotation matrix
			vector<card32>	m_finge;
			vector<card32>	m_finge_neighbor;
			for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
			{
				int32 ik_depth = ps->get<int32>(0, surfelIndex, TOP_DEPTH);
				if(ik_depth == 1)
				{
					m_finge.push_back(surfelIndex);
					card32 numOfEntries = tg->getNumAdjacentVertices(surfelIndex);
					for(card32 ni = 0; ni < numOfEntries; ni++)
					{
						card32 neighbor_id = tg->getAdjacentVertex(surfelIndex, ni);
						int32 jk_depth = ps->get<int32>(0, neighbor_id, TOP_DEPTH);
						if(jk_depth == 0)
						{
							m_finge_neighbor.push_back(neighbor_id);
							break;
						}
					}
				}
			}
			for(card32 ni = 0; ni < m_finge.size(); ni++)
			{
				card32	neighbor_id	= m_finge_neighbor[ni];
				card32	surfelIndex		= m_finge[ni];
				Vector3f	devp	=	ps->get3f(numOfPG1Frames-1, neighbor_id, POSITION) 
					- ps->get3f(numOfPG1Frames-1, surfelIndex, POSITION);
				float32	len_p = sqrt(devp.getSqrNorm());

				Vector3f	devn	=	ps->get3f(numOfPG1Frames-1, neighbor_id, NORMAL) 
					- ps->get3f(numOfPG1Frames-1, surfelIndex, NORMAL);
				float32	len_n = sqrt(devn.getSqrNorm());

				for(int32 frameIndex = numOfPG1Frames - 2; frameIndex >= 0; frameIndex--)
				{
/*					char			*dataS = ps->getDataPointer(frameIndex, neighbor_id);
					float32	*quatS = (float32*)(dataS+ROTATION.getOffset());

					char			*dataC = ps->getDataPointer(frameIndex, surfelIndex);
					float32	*quatC = (float32*)(dataC+ROTATION.getOffset());
					for(card32	k = 0; k < 4; k++)
					{
						quatC[k] = quatS[k];
					}
					Matrix3f	Aik;
					Quaternion_To_Matrix3f_GA(quatC, Aik);
					devp = (Aik.transpose())*devp;	devp.normalize();	devp	*=	len_p;
					devn = (Aik.transpose())*devn;	devn.normalize();	devn	*=	len_n;*/
					Vector3f	pik = ps->get3f(frameIndex, neighbor_id, POSITION) - devp;
					ps->set3f(frameIndex, surfelIndex, POSITION, pik);
					Vector3f	nik = ps->get3f(frameIndex, neighbor_id, NORMAL) - devn; nik.normalize();
					ps->set3f(frameIndex, surfelIndex, NORMAL, nik);
					Vector3f	Eik1;
					if(fabs(nik[0]) > fabs(nik[1]))
					{
						if(fabs(nik[0]) > fabs(nik[2])){	Eik1[0] = 1.0; Eik1[1] = 0.0; Eik1[2] = 0.0;}
						else{	Eik1[0] = 0.0; Eik1[1] = 0.0; Eik1[2] = 1.0;}
					}
					else
					{
						if(fabs(nik[1]) > fabs(nik[2])){	Eik1[0] = 0.0; Eik1[1] = 1.0; Eik1[2] = 0.0;}
						else{	Eik1[0] = 0.0; Eik1[1] = 0.0; Eik1[2] = 1.0;}
					}
					Eik1 -= nik*(Eik1*nik);	
					Eik1.normalize();
					ps->set3f(frameIndex, surfelIndex, REF_U, Eik1);
				}
			}
		
			vector< vector<card32> > optPatchIndices;
			card32 numSurfels = 0;

			for(card32 frameIndex = 0; frameIndex  < numOfPG1Frames-1; frameIndex++)
			{
				optPatchIndices.push_back(vector<card32>());
				for(card32 surfelIndex =  0; surfelIndex < height; surfelIndex++)
				{
					int depth = ps->get<int32>(frameIndex, surfelIndex, TOP_DEPTH);
					if(depth == 1)
					{
						optPatchIndices.back().push_back(surfelIndex);
						numSurfels++;
						//BSROptPointIndex		e1;		e1.frame		= frameIndex;	e1.point	= surfelIndex;	IndexSet_Pos.push_back(e1);
						//BSROptPointIndex		e2;		e2.frame		= frameIndex;	e2.point	=	surfelIndex;	IndexSet_Nor.push_back(e2);
						//BSROptPointIndex		e3;		e3.frame		= frameIndex;	e3.point	=	surfelIndex;	IndexSet_Rot.push_back(e3);
					}
				}
			}
			//if(IndexSet_Pos.size() + IndexSet_Nor.size() + IndexSet_Rot.size() == 0)
			if(numSurfels == 0)
			{
				break;
			}
			PointSet	*dataPoints = new PointSet();

			/// We don't use data points i.e squared distance field in this optimization

			partialOptimizationConfig->potentials.configure(optimizer);
			PGOpt2GraphCreator gc;
			PGOpt2FramePairVars *pairVars = gc.createPairVars(pg, false, false, false, false, false);
			OptimizationGraph *optGraph = gc.createOptGraph(pg, data, pairVars, false, false, false, false, false, &optPatchIndices, &optPatchIndices);
			optimizer->optimize(                             // ^--- which data?
				partialOptimizationConfig->params,
				context,
				optGraph, pg,
				data,	// <--- which data?
				debugRenderer
				);

			//optimizer->optimization(&IndexSet_Pos, &IndexSet_Nor, &IndexSet_Rot, &indices_data, 
			//	data, pg, NULL, 2, debugRedraw, this);

			delete pairVars;
			delete optGraph;
			delete dataPoints;
			for(card32 frameIndex = 0; frameIndex  < numOfPG1Frames-1; frameIndex++)
			{
				for(card32 surfelIndex =  0; surfelIndex < height; surfelIndex++)
				{
					int	depth = ps->get<int32>(frameIndex, surfelIndex, TOP_DEPTH);
					if(depth > 0)
						ps->set<int32>(frameIndex, surfelIndex, TOP_DEPTH, depth-1);
				}
			}
		} while(1);
	}
	if(width - numOfPG1Frames >= 2)
	{
		do 
		{
			/// Initialize the geometry, e.g position, normal and rotation matrix
			vector<card32>	m_finge;
			vector<card32>	m_finge_neighbor;
			for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
			{
				int32 ik_depth = ps->get<int32>(width-1, surfelIndex, TOP_DEPTH);
				if(ik_depth == 1)
				{
					m_finge.push_back(surfelIndex);
					card32 numOfEntries = tg->getNumAdjacentVertices(surfelIndex);
					for(card32 ni = 0; ni < numOfEntries; ni++)
					{
						card32 neighbor_id = tg->getAdjacentVertex(surfelIndex, ni);
						int32 jk_depth = ps->get<int32>(width-1, neighbor_id, TOP_DEPTH);
						if(jk_depth == 0)
						{
							m_finge_neighbor.push_back(neighbor_id);
							break;
						}
					}
				}
			}
			for(card32 ni = 0; ni < m_finge.size(); ni++)
			{
				card32	neighbor_id	= m_finge_neighbor[ni];
				card32	surfelIndex		= m_finge[ni];
				Vector3f	devp	=	ps->get3f(numOfPG1Frames, neighbor_id, POSITION) 
					- ps->get3f(numOfPG1Frames, surfelIndex, POSITION);
				float32	len_p = sqrt(devp.getSqrNorm());

				Vector3f	devn	=	ps->get3f(numOfPG1Frames, neighbor_id, NORMAL) 
					- ps->get3f(numOfPG1Frames, surfelIndex, NORMAL);
				float32	len_n = sqrt(devn.getSqrNorm());

				for(card32 frameIndex = numOfPG1Frames+1; frameIndex < width; frameIndex++)
				{
/*					char			*dataS = ps->getDataPointer(frameIndex-1, neighbor_id);
					float32	*quatS = (float32*)(dataS+ROTATION.getOffset());

					char			*dataC = ps->getDataPointer(frameIndex-1, surfelIndex);
					float32	*quatC = (float32*)(dataC+ROTATION.getOffset());
					for(card32	k = 0; k < 4; k++)
					{
						quatC[k] = quatS[k];
					}

					Matrix3f	Aik;
					Quaternion_To_Matrix3f_GA(quatC, Aik);
					devp = Aik*devp;		devp.normalize();	devp	*=	len_p;
					devn = Aik*devn;		devn.normalize();	devn	*=	len_n;*/
					Vector3f	pik = ps->get3f(frameIndex, neighbor_id, POSITION) - devp;
					ps->set3f(frameIndex, surfelIndex, POSITION, pik);
					Vector3f	nik = ps->get3f(frameIndex, neighbor_id, NORMAL) - devn; nik.normalize();
					ps->set3f(frameIndex, surfelIndex, NORMAL, nik);
					Vector3f	Eik1;
					if(fabs(nik[0]) > fabs(nik[1]))
					{
						if(fabs(nik[0]) > fabs(nik[2])){	Eik1[0] = 1.0; Eik1[1] = 0.0; Eik1[2] = 0.0;}
						else{	Eik1[0] = 0.0; Eik1[1] = 0.0; Eik1[2] = 1.0;}
					}
					else
					{
						if(fabs(nik[1]) > fabs(nik[2])){	Eik1[0] = 0.0; Eik1[1] = 1.0; Eik1[2] = 0.0;}
						else{	Eik1[0] = 0.0; Eik1[1] = 0.0; Eik1[2] = 1.0;}
					}
					Eik1 -= nik*(Eik1*nik);	
					Eik1.normalize();
					ps->set3f(frameIndex, surfelIndex, REF_U, Eik1);
				}
			}

		
			vector< vector<card32> > optPatchIndices;
			card32 numSurfels = 0;

			for(card32 frameIndex = numOfPG1Frames + 1; frameIndex  < width; frameIndex++)
			{
				optPatchIndices.push_back(vector<card32>());
				for(card32 surfelIndex =  0; surfelIndex < height; surfelIndex++)
				{
					int32 depth = ps->get<int32>(frameIndex, surfelIndex, TOP_DEPTH);
					if(depth == 1)
					{
						optPatchIndices.back().push_back(surfelIndex);
						numSurfels++;
					}
				}
			}
			//if(IndexSet_Pos.size() + IndexSet_Nor.size() + IndexSet_Rot.size() == 0)
			if(numSurfels == 0)
			{
				break;
			}
			PointSet	*dataPoints = new PointSet();

			partialOptimizationConfig->potentials.configure(optimizer);
			PGOpt2GraphCreator gc;
			PGOpt2FramePairVars *pairVars = gc.createPairVars(pg, false, false, false, false, false);
			OptimizationGraph *optGraph = gc.createOptGraph(pg, data, pairVars, false, false, false, false, false, &optPatchIndices, &optPatchIndices);
			optimizer->optimize(                             // ^--- which data?
				partialOptimizationConfig->params,
				context,
				optGraph, pg,
				data,	// <--- which data?
				debugRenderer
				);

			//			optimizer->optimization(&IndexSet_Pos, &IndexSet_Nor, &IndexSet_Rot, &indices_data, 
			//				data, pg, NULL, 2, debugRedraw, this);

			ps = pg->getTrajectorySet()->getTrajectories();

			delete pairVars;
			delete optGraph;
			delete dataPoints;
			for(card32 frameIndex = numOfPG1Frames + 1; frameIndex  < width; frameIndex++)
			{
				for(card32 surfelIndex =  0; surfelIndex < height; surfelIndex++)
				{
					int	depth = ps->get<int32>(frameIndex, surfelIndex, TOP_DEPTH);
					if(depth > 0)
						ps->set<int32>(frameIndex, surfelIndex, TOP_DEPTH, depth-1);
				}
			}
		} while(1);
	}
	/// Do optimization on the right side
}

void	GeometricAlignment::global_Assembly(
	AnimationBSRContext *context,
	PatchGraph* pg, PointCloudAnimationSequence *localDataPoints,
	DebugRenderer *debugRenderer,  
	const bool verbose)
{
	if ( optimizer == NULL ){
		debugOutput << "optimize: invalid optimizer object.\n"; debugOutput.flush();
		return;
	}

	// Extract indices (all)
	BSROptVariableIndexSet indices_pos;
	BSROptVariableIndexSet indices_nor;
	BSROptVariableIndexSet indices_rot;
	vector<card32>				indices_data;

	PointSet								*ps = pg->getTrajectorySet()->getTrajectories();

	card32								width	= ps->getWidth(),
												height	=	ps->getHeight();

	for(card32 frameIndex = 0; frameIndex < width; frameIndex++ )
	{
		indices_data.push_back(frameIndex);
		for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++ )
		{
			BSROptPointIndex i_pos, i_nor, i_rot;
			i_pos.frame	= frameIndex;		i_pos.point		= surfelIndex;		indices_pos.push_back(i_pos);
			i_nor.frame		= frameIndex;		i_nor.point		= surfelIndex;		indices_nor.push_back(i_nor);
//			if(frameIndex < width-1)
//			{
				i_rot.frame = frameIndex;		i_rot.point		= surfelIndex;		indices_rot.push_back(i_rot);
//			}
		}
	}

	globalAssemblyConfig->potentials.configure(optimizer);             
   PGOpt2GraphCreator gc;
   PGOpt2FramePairVars *pairVars = gc.createPairVars(
      pg, 
      context->getCyclicTime(), 
      context->getUrshapeGlobalOpt(), 
      context->getUrshapeToScene(), 
      context->getSceneToUrshape(),
      false
   );
   OptimizationGraph *optGraph = gc.createOptGraph(
      pg, 
      localDataPoints, 
      pairVars, 
      context->getCyclicTime(), 
      context->getUrshapeGlobalOpt(), 
      context->getUrshapeToScene(), 
      context->getSceneToUrshape(), 
      false,
      NULL, NULL
   );

	optimizer->optimize(
	   globalAssemblyConfig->params,
	   context,
      optGraph, pg,
		localDataPoints,	
      debugRenderer
   );
   
   delete pairVars;
   delete optGraph;

//	optimizer->optimization( &indices_pos, &indices_nor, &indices_rot, &indices_data,
//		localDataPoints, pg, NULL,3, debugRedraw, this);

};


void	GeometricAlignment::setDepthOfVertices(
	const	card32	numOfPG1Frames,
	const	card32	numOfPG1Surfels,
	PatchGraph	*combinedPG)
{
	/// Compute the depth of each vertex

	InCorePCTopologyGraph	*tg		= combinedPG->getTopology();
	PointSet				*ps		= combinedPG->getTrajectorySet()->getTrajectories();
	AAT					TOP_DEPTH	= ps->getAAT("topology_depth");
	AAT					POSITION	= ps->getAAT("position");
	AAT					NORMAL		= ps->getAAT("normal");
	AAT					REF_U		= ps->getAAT("tangent_u");
	AAT					ROTATION	= ps->getAAT("rotation_matrix");
	card32				height		= ps->getHeight(),id;

	vector<int32>	depthOfVertices;
	vector<int32>	fifo_Queue;
	depthOfVertices.resize(height);
	/// A FIFO queue used for breath-first search
	fifo_Queue.resize(height);
	/// Head and tail iterations of the FIFO queue
	card32					head, tail;

	/// vertex depth for patch graph 1
	/// initialize
	for(id = 0; id < height; id++)
	{
		depthOfVertices[id] = -1;
		fifo_Queue[id]			= -1;
	}
	/// depth of roots
	for(id = 0; id < numOfPG1Surfels; id++)
	{
		depthOfVertices[id] = 0;
		fifo_Queue[id]			= id;
	}
	head = 0;		tail	=	numOfPG1Surfels;
	/// breath first search now
	while(tail < height)
	{
		if(head == tail)
		{
/*			for(id = 0; id < height; id++)
			{
				if(depthOfVertices[id] == -1)
				{
					depthOfVertices[id] = 0;
					fifo_Queue[head] = id;
					tail++;
					break;
				}
			}*/
			break;
		}
		card32	surfelIndex = fifo_Queue[head];
		card32	numOfEntries = tg->getNumAdjacentVertices(surfelIndex);
		for(card32 ni = 0; ni < numOfEntries; ni++)
		{
			card32	neighbor_id = tg->getAdjacentVertex(surfelIndex, ni);
			if(neighbor_id == surfelIndex)
			{
				continue;
			}
			if(depthOfVertices[neighbor_id] < 0)
			{
				depthOfVertices[neighbor_id] = depthOfVertices[surfelIndex] + 1;
				fifo_Queue[tail] = neighbor_id;
				tail++;
			}
		}
		head++;
		
	}
	for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		if(depthOfVertices[surfelIndex] == -1)
		{
			depthOfVertices[surfelIndex] = 0;
			Vector3f	pos = ps->get3f(numOfPG1Frames-1, surfelIndex, POSITION);
			Vector3f	nor = ps->get3f(numOfPG1Frames-1, surfelIndex, NORMAL);
			Vector3f	E1	= ps->get3f(numOfPG1Frames-1, surfelIndex, REF_U);
/* We need to rewrite this part
			for(card32 frameIndex = 0; frameIndex < numOfPG1Frames -1; frameIndex++)
			{
				ps->set3f(frameIndex, surfelIndex, POSITION, pos);
				ps->set3f(frameIndex, surfelIndex, NORMAL, nor);
				ps->set3f(frameIndex, surfelIndex, REF_U, E1);
				char	*dataC	= ps->getDataPointer(frameIndex, surfelIndex);
				float32	*quatc	= (float*)(dataC+ROTATION.getOffset());
				quatc[0] = 1.0f;	quatc[1] = 0.0f;	quatc[2] = 0.0f;	quatc[3] = 0.0f;
			}*/
		}
	}
	for(card32 frameIndex = 0; frameIndex < numOfPG1Frames-1; frameIndex++)
	{
		for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
		{
			ps->set<int32>(frameIndex, surfelIndex, TOP_DEPTH, depthOfVertices[surfelIndex]);
		}
	}
	for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		ps->set<int32>(numOfPG1Frames-1, surfelIndex, TOP_DEPTH, 0);
	}
	/// 
	/// vertex depth for patch graph 2
	/// initialize
	for(id = 0;	 id < height; id++)
	{
		depthOfVertices[id] = -1;
		fifo_Queue[id]			=	-1;
	}
	/// depth of roots
	for(id = numOfPG1Surfels; id < height; id++)
	{
		depthOfVertices[id]								= 0;
		fifo_Queue[id - numOfPG1Surfels]	= id;
	}
	head = 0;		tail	=	height - numOfPG1Surfels;
	/// breath first search now
	while(tail < height)
	{
		if(head == tail)
		{
			break;
		}
		card32	surfelIndex = fifo_Queue[head];
		card32	numOfEntries = tg->getNumAdjacentVertices(surfelIndex);
		for(card32 ni = 0; ni < numOfEntries; ni++)
		{
			card32	neighbor_id = tg->getAdjacentVertex(surfelIndex, ni);
			if(neighbor_id == surfelIndex)
			{
				continue;
			}
			if(depthOfVertices[neighbor_id] < 0)
			{
				depthOfVertices[neighbor_id] = depthOfVertices[surfelIndex] + 1;
				fifo_Queue[tail] = neighbor_id;
				tail++;
			}
		}
		head++;
	}
	for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		ps->set<int32>(numOfPG1Frames, surfelIndex, TOP_DEPTH, 0);
	}
	for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		if(depthOfVertices[surfelIndex] == -1)
		{
			depthOfVertices[surfelIndex] = 0;
			Vector3f	pos = ps->get3f(numOfPG1Frames, surfelIndex, POSITION);
			Vector3f	nor = ps->get3f(numOfPG1Frames, surfelIndex, NORMAL);
			Vector3f	E1	= ps->get3f(numOfPG1Frames, surfelIndex, REF_U);
/* We need to rewrite this part
			for(card32 frameIndex = numOfPG1Frames+1; frameIndex < ps->getWidth(); frameIndex++)
			{
				ps->set3f(frameIndex, surfelIndex, POSITION, pos);
				ps->set3f(frameIndex, surfelIndex, NORMAL, nor);
				ps->set3f(frameIndex, surfelIndex, REF_U, E1);
				char	*dataC	= ps->getDataPointer(frameIndex, surfelIndex);
				float32	*quatc	= (float*)(dataC+ROTATION.getOffset());
				quatc[0] = 1.0f;	quatc[1] = 0.0f;	quatc[2] = 0.0f;	quatc[3] = 0.0f;
			}*/
		}
	}
	for(card32 frameIndex = numOfPG1Frames + 1; frameIndex < ps->getWidth(); frameIndex++)
	{
		for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
		{
			ps->set<int32>(frameIndex, surfelIndex, TOP_DEPTH, depthOfVertices[surfelIndex]);
		}
	}
}

void	GeometricAlignment::createLocalDataPoints(
	const PatchGraph *pg, 
	const PointCloudAnimationSequence *originalDataPoints,
	PointCloudAnimationSequence *localDataPoints )
{

	card32		width	= pg->getTrajectorySet()->getTrajectories()->getWidth(),
		height	= pg->getTrajectorySet()->getTrajectories()->getHeight();
	VertexDescriptor vd;
	vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(mVAD("flag_inlier", 1, VAD::DATA_FORMAT_INT32));
	AAT destAAT		= vd.getToken("position");
	AAT	FLAG_INLIER	= vd.getToken("flag_inlier"); 
	AAT sourceAAT	= originalDataPoints->getFrame(0)->getAAT("position");
	PGDataPointIndices	*dataIndices	= pg->getTrajectorySet()->getDataPoints();

	card32		startFrameIndex = pg->getStartFrameIndex();
	vector<card32>		dataindices_buf;
	for(card32 frameIndex = startFrameIndex; frameIndex < startFrameIndex + width; frameIndex++)
	{
		dataindices_buf.clear();
		PointSet *dataPS = new PointSet();
		for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
		{
			const card32 numDataPoints = dataIndices->getNumDataPoints(frameIndex - startFrameIndex, surfelIndex);
			for(card32 id = 0; id < numDataPoints; id++)
			{
				card32	dataIndexInData = dataIndices->getIndex( frameIndex - startFrameIndex, surfelIndex, id);
				dataindices_buf.push_back(dataIndexInData);
			}
		}
		dataPS->clearAndSetup(1, dataindices_buf.size(), &vd);
		for (card32 id = 0; id < dataindices_buf.size(); id++) 
		{
			Vector3f	pos = originalDataPoints->getFrame(frameIndex)->get3f(dataindices_buf[id], sourceAAT);
			dataPS->set3f(id, destAAT, pos);
			dataPS->set<int32>(id, FLAG_INLIER, 1);
		}
		dataindices_buf.clear();
		localDataPoints->addFrame(dataPS);
	}
}

void GeometricAlignment::compute( 
	AnimationBSRContext* context,
	FeatureRegistration *featureMatcher,
	const PatchGraph* pg1, const PatchGraph* pg2, 
	PatchGraph* combinedPG,
	DebugRenderer *debugRenderer, const bool verbose)
{
	int32 lastPG1Frame = pg1->getTrajectorySet()->getNumFrames()-1;
		
	vector<CorrespondenceExtrapolation::PatchCorrespondence> knownPg1Pg2Corr, knownPg2Pg1Corr;	
	
	alignAndConnectParts(
		context, pg1, pg2, 
		lastPG1Frame,															// part1FrameIndex,
		0,																					// part2FrameIndex
		lastPG1Frame+1+pg1->getStartFrameIndex(),	// nextAfterPart1DataIndex
		lastPG1Frame+1,														// destFrameIndex
		0,																					// extrapolatedPointsCopyIndex
		0,																					// part1StartPatch 
		pg1->getTrajectorySet()->getNumPatches(),	// part2StartPatch 
		combinedPG, 
		knownPg1Pg2Corr,
		debugRenderer, verbose
	);
	alignAndConnectParts(
		context, pg2, pg1, 
		0,																												// part1FrameIndex,
		lastPG1Frame,																						// part2FrameIndex
		pg2->getStartFrameIndex()-1,															// nextAfterPart1DataIndex
		lastPG1Frame,																						// destFrameIndex
		pg1->getTrajectorySet()->getTrajectories()->getHeight(),		// extrapolatedPointsCopyIndex
		pg1->getTrajectorySet()->getNumPatches(),								// part1StartPatch 
		0,																												// part1StartPatch 
		combinedPG, 
		knownPg2Pg1Corr,
		debugRenderer, verbose
	);
		
	LocalTopologyResampling::resampleTopology( 
		context,
		combinedPG,
		lastPG1Frame+1,
		debugRenderer, verbose
	);

	setDepthOfVertices(pg1->getTrajectorySet()->getTrajectories()->getWidth(),
		pg1->getTrajectorySet()->getTrajectories()->getHeight(),
		combinedPG);

	pairtialOptimization(context, pg1->getTrajectorySet()->getTrajectories()->getWidth(),
		                  combinedPG, context->getPGContainer()->getAnimSeq(), debugRenderer);
}

///The following function is used to determine which pair-wise match is good 
static	void	DetermineTopologySalientcy(
	const PatchGraph	*patchGraph,
	const card32		minNeighbors,
	card32	&numOfEdgesOfTheLargestConnectedComponent,
	const float32		
	)
{

}


static void	deformationEnergy(const float32	w_pos_r, const float32 w_nor_r,
							  PatchGraph *patchGraph, float32 &energy)
{
	pAssert(patchGraph != NULL);

	PointSet *ps	= patchGraph->getTrajectorySet()->getTrajectories();
	InCorePCTopologyGraph	*tg = patchGraph->getTopology();
	card32	 width	= ps->getWidth(),
			height	= ps->getHeight();

	AAT	POSITION	= ps->getAAT("position");
	AAT	NORMAL		= ps->getAAT("normal");
	AAT	REF_U		= ps->getAAT("tangent_u");
	AAT	ROTATION	= ps->getAAT("rotation_matrix");

	Matrix3f		Aik;

	if(width == 1)
	{
		energy = 0.0f;
		return;
	}
	for(card32	frameIndex = 0; frameIndex < width-1; frameIndex++)
	{
		for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
		{
			Vector3f	pik	 = ps->get3f(frameIndex, surfelIndex, POSITION);
			Vector3f	nik  = ps->get3f(frameIndex, surfelIndex, NORMAL);
			Vector3f	pikn = ps->get3f(frameIndex+1, surfelIndex, POSITION);
			Vector3f	nikn = ps->get3f(frameIndex+1, surfelIndex, NORMAL);

			char			*dataC1	= ps->getDataPointer(frameIndex, surfelIndex);
			float32	*quatc1	= (float*)(dataC1+ROTATION.getOffset());
			Quaternion_To_Matrix3f_GA(quatc1, Aik);

			card32	numOfEntries = tg->getNumAdjacentVertices(surfelIndex);
			for(card32 ni = 0; ni < numOfEntries; ni++)
			{
				card32	neighbor_id = tg->getAdjacentVertex(surfelIndex, ni);
				Vector3f	pjk	 = ps->get3f(frameIndex, neighbor_id, POSITION);
				Vector3f	njk  = ps->get3f(frameIndex, neighbor_id, NORMAL);
				Vector3f	pjkn = ps->get3f(frameIndex+1, neighbor_id, POSITION);
				Vector3f	njkn = ps->get3f(frameIndex+1, neighbor_id, NORMAL);
				Vector3f	rijk = pik - pjk;
				Vector3f	rijkn= pikn - pjkn;
				Vector3f	rijkc = Aik*rijk;
				Vector3f	devpijk = rijkc - rijkn;
				Vector3f	nijk = nik - njk;
				Vector3f	nijkn= nikn - njkn;
				Vector3f	nijkc = Aik*nijk;
				Vector3f	devnijk = nijkc - nijkn;
				energy += w_pos_r*devpijk.getSqrNorm() + w_nor_r*devnijk.getSqrNorm();
			}
		}
	}
}

void GeometricAlignment::compute_Modified(
	AnimationBSRContext *context,
	FeatureRegistration *featureMatcher,
	const PatchGraph* pg1, const PatchGraph* pg2, 
	PatchGraph* combinedPG,
	const map<card32, Vector3f> &featureCorrespondences12,
	const map<card32, Vector3f> &featureCorrespondences21,
	DebugRenderer *debugRenderer, 
	const bool verbose 
	)
{
	int32 lastPG1Frame = pg1->getTrajectorySet()->getNumFrames()-1;

	///Preparation for pair-wise matching

	//The following point set is used for refining pair-wise matching
	PointSet*			ps		= combinedPG->getTrajectorySet()->getTrajectories();
	InCorePCTopologyGraph *tg	= combinedPG->getTopology();
	card32				width	= ps->getWidth(),
						height	= ps->getHeight(),
						height1 = pg1->getTrajectorySet()->getTrajectories()->getHeight();

	/// allocate space
	TrajectorySet*	pair_ts1 = new TrajectorySet();
	PointSet*		pair_ps1 = new PointSet();
	pair_ps1->clearAndSetup(2, ps->getHeight() , ps->getDescr() );
	pair_ts1->clearAndSetup(pair_ps1);
	pair_ts1->createPatchAttribute(mVAD("dataIndex", 1, VAD::DATA_FORMAT_INT32));
	pair_ps1 = pair_ts1->getTrajectories();


	TrajectorySet*	pair_ts2 = new TrajectorySet();
	PointSet*		pair_ps2 = new PointSet();
	pair_ps2->clearAndSetup(2, ps->getHeight() , ps->getDescr() );
	pair_ts2->clearAndSetup(pair_ps2);
	pair_ts2->createPatchAttribute(mVAD("dataIndex", 1, VAD::DATA_FORMAT_INT32));
	pair_ps2 = pair_ts2->getTrajectories();

	/// set the geometry both on combinedPS

	//There are two topological graphs, only one of them will be used later
	InCorePCTopologyGraph	*potential_topology1 = new InCorePCTopologyGraph();
	InCorePCTopologyGraph	*potential_topology2 = new InCorePCTopologyGraph();

	potential_topology1->clearAndSetup(tg->getEdges()->getDescr());
	potential_topology2->clearAndSetup(tg->getEdges()->getDescr());
	// Copy topology from pg1
	for(card32 patchIndex = 0; patchIndex < height; patchIndex++ )
	{
		card32	NumAdjacentVertices = tg->getNumAdjacentVertices( patchIndex );
		for (card32 vertexIndex = 0; vertexIndex < NumAdjacentVertices; vertexIndex++ )
		{
			card32 neighbor_id = tg->getAdjacentVertex(patchIndex, vertexIndex); 
//			potential_topology1->addEdge(patchIndex, neighbor_id);
//			potential_topology2->addEdge(patchIndex, neighbor_id);
		}
	}
	

	/// Do pair-wise matching now
	vector<CorrespondenceExtrapolation::PatchCorrespondence> knownPg1Pg2Corr, knownPg2Pg1Corr;	

	alignAndConnectParts_modified(
		context, pg1, pg2, 
		lastPG1Frame,															// part1FrameIndex,
		0,																					// part2FrameIndex
		lastPG1Frame+1+pg1->getStartFrameIndex(),	// nextAfterPart1DataIndex
		lastPG1Frame+1,														// destFrameIndex
		1,
		0,																					// extrapolatedPointsCopyIndex
		0,																					// part1StartPatch 
		pg1->getTrajectorySet()->getNumPatches(),	// part2StartPatch 
		combinedPG,
		potential_topology2,
		knownPg1Pg2Corr,
		featureCorrespondences12, 
		debugRenderer, verbose
		);
	alignAndConnectParts_modified(
		context, pg2, pg1, 
		0,																												// part1FrameIndex,
		lastPG1Frame,																						// part2FrameIndex
		pg2->getStartFrameIndex()-1,															// nextAfterPart1DataIndex
		lastPG1Frame,																						// destFrameIndex
		0,
		pg1->getTrajectorySet()->getTrajectories()->getHeight(),		// extrapolatedPointsCopyIndex
		pg1->getTrajectorySet()->getNumPatches(),								// part1StartPatch 
		0,																												// part1StartPatch 
		combinedPG,
		potential_topology1,
		knownPg2Pg1Corr,
		featureCorrespondences21, 
		debugRenderer, verbose
		);


	////* This is the new part of the code
	//// Below is modified by peter
	////

	PatchGraph* pair_PG1 = new PatchGraphWCorr();
	for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		pair_ps1->setVertex(0, surfelIndex, ps->getDataPointer(lastPG1Frame, surfelIndex));
		pair_ps1->setVertex(1, surfelIndex, ps->getDataPointer(lastPG1Frame+1, surfelIndex));
	}
	AAT	DATA_INDEX = pair_ps1->getAAT("dataIndex");

	for(card32 surfelIndex = height1; surfelIndex < height; surfelIndex++)
	{
		pair_ps1->set<int32>(1, surfelIndex, DATA_INDEX, surfelIndex - height1);
	}

	pair_PG1->clearAndSetup(pair_ts1, potential_topology1, false );
	correspondenceBasedPairwiseMatching(context, height1, height, pair_PG1, debugRenderer, verbose);

	PatchGraph* pair_PG2 = new PatchGraphWCorr();
	//potential_topology1 is not ganna to be used
	for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		pair_ps2->setVertex(1, surfelIndex, ps->getDataPointer(lastPG1Frame, surfelIndex));
		pair_ps2->setVertex(0, surfelIndex, ps->getDataPointer(lastPG1Frame+1, surfelIndex));
	}
	DATA_INDEX = pair_ps2->getAAT("dataIndex");

	for(card32 surfelIndex = height1; surfelIndex < height; surfelIndex++)
	{
		pair_ps2->set<int32>(1, surfelIndex, DATA_INDEX, surfelIndex);
	}
	pair_PG2->clearAndSetup(pair_ts2, potential_topology2, false );

	correspondenceBasedPairwiseMatching(context, 0, height1, pair_PG2, debugRenderer, verbose);

	float32	w_pos_r = correspondenceBasedPairwiseConfig->params->weightPositionRigidity;
	float32	w_nor_r = correspondenceBasedPairwiseConfig->params->weightNormalRigidity;
	float32	energy1;
	deformationEnergy(w_pos_r, w_nor_r, pair_PG1, energy1);
	float32	energy2;
	deformationEnergy(w_pos_r, w_nor_r, pair_PG2, energy2);

	//FILE *file = fopen("J:\\pairwise.txt", "w");
	//fprintf(file, "energy1: %f, energy2: %f\n", energy1, energy2);
	//fflush(file);
	//fclose(file);

	if(energy1 < energy2)
	{
		/// Compute geometry back to combined-PG
		pair_ps1 = pair_PG1->getTrajectorySet()->getTrajectories();
		ps		= combinedPG->getTrajectorySet()->getTrajectories();
		for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
		{
			ps->setVertex(lastPG1Frame+1, surfelIndex, pair_ps1->getDataPointer(1, surfelIndex));
		}
		LocalTopologyResampling::resampleTopology_Modified( 
			context,
			combinedPG,
			lastPG1Frame+1,
			debugRenderer, verbose
			);

		pair_PG1->detachTopology();	delete pair_PG1;
		pair_PG2->detachTopology();	delete pair_PG2;
		delete potential_topology2;
		combinedPG->setTopology(potential_topology1);
	}	
	else
	{
		pair_ps2 = pair_PG2->getTrajectorySet()->getTrajectories();
		ps		= combinedPG->getTrajectorySet()->getTrajectories();
		for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
		{
			ps->setVertex(lastPG1Frame, surfelIndex, pair_ps2->getDataPointer(1, surfelIndex));
		}
		LocalTopologyResampling::resampleTopology_Modified( 
			context,
			combinedPG,
			lastPG1Frame,
			debugRenderer, verbose
			);
		pair_PG1->detachTopology();	delete pair_PG1;
		pair_PG2->detachTopology();	delete pair_PG2;
		delete potential_topology1;
		combinedPG->setTopology(potential_topology2);
	}
	
	//////////////////////////////////////////////////////////////////////////
	/// Above is modified by peter
	//////////////////////////////////////////////////////////////////////////

	setDepthOfVertices(pg1->getTrajectorySet()->getTrajectories()->getWidth(),
		pg1->getTrajectorySet()->getTrajectories()->getHeight(),
		combinedPG);

	///
	pairtialOptimization(context, pg1->getTrajectorySet()->getTrajectories()->getWidth(),
		combinedPG, context->getPGContainer()->getAnimSeq(), debugRenderer);
}

void GeometricAlignment::globalAssembly(AnimationBSRContext* context, 
					 PatchGraph* combinedPG,
					 DebugRenderer *debugRenderer,  
					 const bool verbose)
{
	//Do global assembly for this patch graph//
	PointCloudAnimationSequence *localDataPoints = new PointCloudAnimationSequence();
	createLocalDataPoints(combinedPG, context->getPGContainer()->getAnimSeq(), localDataPoints);
	global_Assembly(context, combinedPG, localDataPoints, debugRenderer, verbose);
	delete localDataPoints;
}
