//---------------------------------------------------------------------------
#ifndef GeometricAlignmentH
#define GeometricAlignmentH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "AbstractMergeExtrapolator.h"
#include "PGOperationsDebugging.h"
#include "CorrespondenceExtrapolation.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PGOpt2Parameters;
class PatchGraphBSROpt;
class StandardPatchGraphRenderer;
class PGOpt2Config;
class FeatureList;

///
///  Merging of patch graphs using feature correspondences and probabilistic optimization
///
class ANIMATIONRECONSTRUCTION_API GeometricAlignment : public AbstractMergeExtrapolator {
	GEOXL_CLASS(GeometricAlignment)

 private:
	PatchGraphOpt2  				 *optimizer;
	StandardPatchGraphRenderer  *visualization;

	PGOpt2Config                *pairwiseConfig;
	PGOpt2Config                *correspondenceBasedPairwiseConfig;
	PGOpt2Config                *partialOptimizationConfig;
	PGOpt2Config                *globalAssemblyConfig;
	
	void alignAndConnectParts(
		AnimationBSRContext* context,
		const PatchGraph *part1, 
		const PatchGraph *part2,
		card32 part1FrameIndex, 
		card32 part2FrameIndex,
		card32 nextAfterPart1DataIndex,
		card32 destFrameIndex,
		card32 extrapolatedPointsCopyIndex,
		card32 part1StartPatch,
		card32 part2StartPatch,
		PatchGraph *combinedPG,
		const vector<CorrespondenceExtrapolation::PatchCorrespondence> &knownCorrespondences,
		DebugRenderer *debugRenderer, 
		const bool verbose
	);


	/// The following procedure is used in modified geometric alignment

	void	initialTopologyForPairwiseMatching(
		const	PatchGraph	*part2,
		const	card32		frameIndex,
		float32 maxRadius,
		SparseMatrixF		*topologyOfTargetFrame
		);

	void alignAndConnectParts_modified(
		AnimationBSRContext* context,
		const PatchGraph *part1, 
		const PatchGraph *part2,
		card32 part1FrameIndex, 
		card32 part2FrameIndex,
		card32 nextAfterPart1DataIndex,
		card32 destFrameIndex,
		card32 frameIndex_Modifed,	
		card32 extrapolatedPointsCopyIndex,
		card32 part1StartPatch,
		card32 part2StartPatch,
		PatchGraph *combinedPG,
		InCorePCTopologyGraph *topology,
		const vector<CorrespondenceExtrapolation::PatchCorrespondence> &knownCorrespondences,
		const map<card32, Vector3f> &featureCorrespondences,
		DebugRenderer *debugRenderer, 
		const bool verbose
		);

	
	void computeFeatureMatch(
		FeatureRegistration *featureMatcher,
		vector< pair<Vector3f, Vector3f> > &matchingPoints, 
		FeatureList *feat1,
		FeatureList *feat2
	);

	void	setDepthOfVertices(
		const	card32	numOfPG1Frames,
		const	card32	numOfPG1Surfels,
		PatchGraph		*combinedPG);

	void createLocalDataPoints(
		const PatchGraph	*pg,
		const PointCloudAnimationSequence *originalDataPoints,
		PointCloudAnimationSequence *localDataPoints
		);

	void	pairtialOptimization(
		AnimationBSRContext *context,
		const	card32	numOfPG1Frames,	
		PatchGraph		*pg,
		PointCloudAnimationSequence	*data,
		DebugRenderer *debugRenderer);


	void	correspondenceBasedPairwiseMatching(
		AnimationBSRContext *context,
		const	card32	startSurfelIndex,
		const	card32	endSurfelIndex,
		PatchGraph		*patchGraph,
		DebugRenderer *debugRenderer,  
		const bool verbose = false
		);

	void	global_Assembly(
		AnimationBSRContext *context,
		PatchGraph *pg, PointCloudAnimationSequence *localDataPoints,
		DebugRenderer *debugRenderer,  
		const bool verbose = false);

 public:
	GeometricAlignment();

	virtual void compute( 
		AnimationBSRContext *context,
		FeatureRegistration *featureMatcher,
		const PatchGraph* pg1, const PatchGraph* pg2, 
		PatchGraph* combinedPG,
		DebugRenderer *debugRenderer, const bool verbose = false 
	);

	virtual void compute_Modified( 
		AnimationBSRContext *context,
		FeatureRegistration *featureMatcher,
		const PatchGraph* pg1, const PatchGraph* pg2, 
		PatchGraph* combinedPG,
		const map<card32, Vector3f> &featureCorrespondences12,
		const map<card32, Vector3f> &featureCorrespondences21,
		DebugRenderer *debugRenderer, 
		const bool verbose = false 
		);

	virtual void globalAssembly(AnimationBSRContext* context, 
		PatchGraph* combinedPG,
		DebugRenderer *debugRenderer,  
		const bool verbose = false );

	~GeometricAlignment();

};

}

#endif