//---------------------------------------------------------------------------
#ifndef ImplicitPatchGraphMeshingH
#define ImplicitPatchGraphMeshingH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "TGNeighborhoodRangeSearch.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PatchGraph;
class UnstructuredInCoreTriangleMesh;

/**
	create triangular surface for a single frame from a patch graph
*/
class ANIMATIONRECONSTRUCTION_API ImplicitPatchGraphMeshing : public Persistent {
	GEOXL_CLASS( ImplicitPatchGraphMeshing )

 public:
	static const card32 PATCH_INTERP_LIN = 0;
	static const card32 PATCH_INTERP_GAUSSIAN = 1;
	static const card32 PATCH_INTERP_COS = 2;
	
 private:
   class MyNeighborhoodSearch : public TGNeighborhoodRangeSearch {
    private:
      Vector3f pos;
      AAT samplingAAT;
      AAT posAAT;
      const ImplicitPatchGraphMeshing *owner;
      
    protected:
      virtual bool inRange(const PointSet *ps, card32 index);
    public:
      void setupMNS(const InCorePCTopologyGraph *tg, const PointSet *ps, card32 startIndex, 
                    const Vector3f &pos, const AAT &samplingAAT, const AAT &posAAT,
                    const ImplicitPatchGraphMeshing *owner);
   };
 
	float32 marchingCubesSideLength;
	bool flipNormals;
	card32 patchInterpolation;
	float32 gaussianFilterCutoffValue;
	float32 relativeFilterSize;
	float32 minWeightSum;
	bool localMeshing;
	float32 mcThickness;

	card32 startGrid;
	card32 numNewton;

   float32 computeWeight(const Vector3f &pos1, const Vector3f &pos2, float32 sigma) const;
   /// helper functions for marching cubes
   static float evalVol(void *context, float x, float y, float z);
   static float evalVolLocal(void *context, float x, float y, float z);
   static bool evalVolIsEmpty(void *context, const BoundingBox3f &bb);
   static bool evalVolIsEmptyLocal(void *context, const BoundingBox3f &bb);

 public:
	ImplicitPatchGraphMeshing();

	void setup(float32 marchingCubesSideLength, card32 patchInterpolation, 
	           float32 gaussianFilterCutoffValue, float32 relativeFilterSize,
	           card32 startGrid, card32 numNewton);

   virtual UnstructuredInCoreTriangleMesh *createMesh(PatchGraph *pg, const card32 frame);

	// --- properties
	float32 getMarchingCubesSideLength() const {return marchingCubesSideLength;}
	void setMarchingCubesSideLength(float32 v) {marchingCubesSideLength=v;}

	card32 getPatchInterpolation() const {return patchInterpolation;}
	void setPatchInterpolation(card32 v) {patchInterpolation=v;}

	float32 getGaussianFilterCutoffValue() const {return gaussianFilterCutoffValue;}
	void setGaussianFilterCutoffValue(float32 v) {gaussianFilterCutoffValue=v;}

	float32 getRelativeFilterSize() const {return relativeFilterSize;}
	void setRelativeFilterSize(float32 v) {relativeFilterSize=v;}

	card32 getStartGrid() const {return startGrid;}
	void setStartGrid(card32 v) {startGrid=v;}

	card32 getNumNewton() const {return numNewton;}
	void setNumNewton(card32 v) {numNewton=v;}

	float32 getMinWeightSum() const {return minWeightSum;}
	void setMinWeightSum(float32 v) {minWeightSum=v;}
	// ---

   void assign(const Object* obj, CopyContext *context = NULL);

	virtual ~ImplicitPatchGraphMeshing();
};

}

#endif