//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "LocalTopologyResampling.h"
//---------------------------------------------------------------------------
#include "InCorePCTopologyGraph.h"
#include "TGNeighborhoodRangeSearch.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "TGNeighborhoodKNNSearch.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

void LocalTopologyResampling::resampleTopology_Modified(
	AnimationBSRContext* context, 
	PatchGraph *pg, 
	const card32 stichFrame, 
	DebugRenderer *debugRenderer, 
	const bool verbose /* = false  */)
{
	PointSet *ps = pg->getTrajectorySet()->getTrajectories();
	InCorePCTopologyGraph *tg = pg->getTopology();
	const	AAT posAAT = ps->getAAT("position");
	const	AAT	norAAT = ps->getAAT("normal");

	const card32 numPatches = ps->getHeight();
	const float32 searchDist = context->getSamplingSpacing() * context->getTopEdgeResampleSearchRadiusFactor();
	const float32 pruneDist = context->getSamplingSpacing() * context->getTopEdgeResamplePruneRadiusFactor();
	PGNeighborhoodKNNSearch search(pg, stichFrame, 0);
	for (card32 i=0; i<numPatches; i++) 
	{
		Vector3f pos = ps->get3f(stichFrame, i, posAAT);
		Vector3f nor = ps->get3f(stichFrame, i, norAAT);
		search.reset(stichFrame, i);
		card32 numNeighbors = 0;
		while (!search.atEnd() && numNeighbors < 20) 
		{
			card32 nextNeighborIndex = search.getCurrentIndex();
			Vector3f npos = ps->get3f(stichFrame, nextNeighborIndex, posAAT);
			Vector3f nnor = ps->get3f(stichFrame, nextNeighborIndex, norAAT);
			Vector3f devpos = npos - pos; devpos.normalize();
			float32	 angle1 = fabs(devpos*nor), angle2 = fabs(devpos*nnor);
			if (norm(pos-npos) <= pruneDist && nextNeighborIndex != i && angle1 < 0.8 && angle2 < 0.8) 
			{
				tg->addEdge(i, nextNeighborIndex);
			}
			search.next();
			numNeighbors++;
		}
	}
}

void LocalTopologyResampling::resampleTopology
  (AnimationBSRContext* context,
	PatchGraph *pg,
	const card32 stichFrame,
	DebugRenderer *debugRenderer,  
	const bool verbose)
{
	PointSet *ps = pg->getTrajectorySet()->getTrajectories();
	InCorePCTopologyGraph *tg = pg->getTopology();
	const AAT posAAT = ps->getAAT("position");
	const card32 numPatches = ps->getHeight();
	const float32 searchDist = context->getSamplingSpacing() * context->getTopEdgeResampleSearchRadiusFactor();
	const float32 pruneDist = context->getSamplingSpacing() * context->getTopEdgeResamplePruneRadiusFactor();
   PGNeighborhoodKNNSearch search(pg, stichFrame, 0);
	for (card32 i=0; i<numPatches; i++) {
		Vector3f pos = ps->get3f(stichFrame, i, posAAT);
		search.reset(stichFrame, i);
		card32 numNeighbors = 0;
		while (!search.atEnd() && numNeighbors < 20) {
			card32 nextNeighborIndex = search.getCurrentIndex();
			Vector3f npos = ps->get3f(stichFrame, nextNeighborIndex, posAAT);
			if (norm(pos-npos) <= pruneDist && nextNeighborIndex != i) {
				tg->addEdge(i, nextNeighborIndex);
			}
			search.next();
			numNeighbors++;
		}
	}
}


	//PointSet *ps = pg->getTrajectorySet()->getTrajectories();
	//InCorePCTopologyGraph *tg = pg->getTopology();
	//const AAT posAAT = ps->getAAT("position");
	//const card32 numPatches = ps->getHeight();
	//const float32 searchDist = context->getSamplingSpacing() * context->getTopEdgeResampleSearchRadiusFactor();
	//const float32 pruneDist = context->getSamplingSpacing() * context->getTopEdgeResamplePruneRadiusFactor();
	//for (card32 i=0; i<numPatches; i++) {
	//	PGNeighborhoodSphereSearch search;
	//	Vector3f pos = ps->get3f(stichFrame, i, posAAT);
	//	search.setup(pg, stichFrame, i, pos, searchDist, posAAT);
	//	while (!search.atEnd()) {
	//		card32 nextNeighborIndex = search.getCurrentIndex();
	//		Vector3f npos = ps->get3f(stichFrame, nextNeighborIndex, posAAT);
	//		if (norm(pos-npos) <= pruneDist && nextNeighborIndex != i) {
	//			tg->addEdge(i, nextNeighborIndex);
	//		}
	//		search.next();
	//	}
	//}
