//---------------------------------------------------------------------------
#ifndef PGLaplacianDiffusionH
#define PGLaplacianDiffusionH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "BSROptPotential.h"
//---------------------------------------------------------------------------
#include "PatchGraph.h"
#include "TrajectorySet.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "InCorePCTopologyGraph.h"
//---------------------------------------------------------------------------


namespace geoxl {

/**
	Smooth interpolation on patch graphs using laplacian diffusion
*/
class ANIMATIONRECONSTRUCTION_API PGLaplacianDiffusion {
 public:
	template<typename ChannelType, int channelDim>
	static void iterativeInterpolationInit(
		PatchGraph *pg,
		const PatchGraph *lowres,
		const BSROptVariableIndexSet &unknowns,
		const vector< card32 > &clusterIndices,
		const AAT &channelAAT,
		const AAT &channelAAT_loweres
	);
	template<typename ChannelType, int channelDim>
	static void iterativeInterpolationOneIteration(
		PatchGraph *pg,
		const PatchGraph *lowres,
		const BSROptVariableIndexSet &unknowns,
		const AAT &channelAAT,
		const AAT &channelAAT_loweres
	);
};



// --- TEMPLATE IMPLEMENTATION


template<typename ChannelType, int channelDim>
void PGLaplacianDiffusion::iterativeInterpolationOneIteration
  (PatchGraph *pg,
	const PatchGraph *lowres,
	const BSROptVariableIndexSet &unknowns,
	const AAT &channelAAT,
	const AAT &channelAAT_loweres)
{
	PointSet *ps = pg->getTrajectorySet()->getTrajectories();
	PointSet *urshape = pg->getUrshape();
	const InCorePCTopologyGraph *tg = pg->getTopology();
	const PointSet *ps_lowres = lowres->getTrajectorySet()->getTrajectories();
	const PointSet *urshape_lowres = lowres->getUrshape();
	
	for (unsigned i=0; i<unknowns.size(); i++) {
		int32 frameIndex = unknowns[i].frame;
		card32 patchIndex = unknowns[i].point;
		StaticVector<ChannelType, channelDim> av;
		for (unsigned q=0; q<channelDim; q++) {av[q] = 0;}
		const card32 numEdges = tg->getNumAdjacentVertices(patchIndex);
		for (card32 e=0; e<numEdges; e++) {
			if (frameIndex != BSROPT_FRAME_URSHAPE) {
				av += ps->get<ChannelType, channelDim>(frameIndex, tg->getAdjacentVertex(patchIndex, e), channelAAT);
			} else {
				av += urshape->get<ChannelType, channelDim>(frameIndex, tg->getAdjacentVertex(patchIndex, e), channelAAT);
			}		
		}
		av /= numEdges;
		if (frameIndex != BSROPT_FRAME_URSHAPE) {	
			ps->set<ChannelType, channelDim>(frameIndex, patchIndex, channelAAT, av);
		} else {
			urshape->set<ChannelType, channelDim>(frameIndex, patchIndex, channelAAT, av);
		}
	}	
}


}

#endif