//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PGNeighborhoodExtrapolator4D.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointSet_templCode.h"
#include "PatchGraph.h"
#include "TrajectorySet.h"
#include "PatchTypes.h"
#include "InCorePCTopologyGraph.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



card32 PGNeighborhoodExtrapolator4D::extrapolateNeighbors
  (AnimationBSRContext *context, 
	PatchGraph *pg,
	int32 useInAvFlags,
	int32 useNotInAvFlags,
	int32 computeNotForFlags,
	int32 writeBackFlags)
{
	PointSet *ps = pg->getTrajectorySet()->getTrajectories();
	InCorePCTopologyGraph *tg = pg->getTopology();
	const card32 numPatches = ps->getHeight();
	const int32 numFrames = ps->getWidth();
	const AAT FLAGS_AAT = ps->getAAT("flags", 1, VAD::DATA_FORMAT_INT32);
	const AAT POS_AAT = ps->getAAT("position", 3, VAD::DATA_FORMAT_FLOAT32);
	card32 pointsFound = 0;
	for (int32 f=0; f<numFrames; f++) {
		for (card32 p=0; p<numPatches; p++) {
			const int32 flags = ps->get<int>(f, p, FLAGS_AAT);
			if ((flags & computeNotForFlags) == 0) {
				card32 numNFound = 0;
				Vector3f avPos = NULL_VECTOR3F;
				const int32 startFrame = f;//max(f-1,0);
				const int32 endFrame = f;//min(f+1,numFrames-1);
				for (int32 searchFrame=startFrame; searchFrame<=endFrame; searchFrame++) {
					const card32 numNeighbors = tg->getNumAdjacentVertices(p);
					for (card32 ni=0; ni<numNeighbors; ni++) {
						const card32 neighbor = tg->getAdjacentVertex(p, ni);
						const int32 nflags =  ps->get<int>(searchFrame, neighbor, FLAGS_AAT);
						if ((nflags & useInAvFlags) != 0 && (nflags & useNotInAvFlags) == 0) {
							numNFound++;
							avPos += ps->get3f(searchFrame, neighbor, POS_AAT);
						}
					}
					if (numNFound > 0) {
						avPos /= numNFound;
						ps->set3f(f,p, POS_AAT, avPos);
						ps->set<int>(f,p, FLAGS_AAT, ps->get<int>(f,p, FLAGS_AAT) | writeBackFlags);
						pointsFound++;
					}
				}
			}
		}
	}
	return pointsFound;
}

void PGNeighborhoodExtrapolator4D::partialOptimization
  (AnimationBSRContext *context,
	PatchGraph *pg,
	PointCloudAnimationSequence *data,
	PatchGraphBSROpt *optimizer,
   PatchGraphBSROpt::FeedbackCallback afterEachIteration,
   void *callbackContext,		
	int32 computeForFlags,
	int32 writeBackFlags,
	int32 writeBackClearFlags)
{
	BSROptVariableIndexSet indices;
	const AAT AAT_FLAGS = pg->getTrajectorySet()->getTrajectories()->getAAT("flags", 1, VAD::DATA_FORMAT_INT32);
	for ( unsigned int patchIndex = 0; patchIndex < pg->getTrajectorySet()->getNumPatches(); patchIndex++ ){
		for ( unsigned int frameIndex = 0; frameIndex < pg->getTrajectorySet()->getNumFrames(); frameIndex++ ){
			int32 flags = pg->getTrajectorySet()->getTrajectories()->get<int>(frameIndex, patchIndex, AAT_FLAGS);
			if ((flags & computeForFlags) != 0) {
				BSROptPointIndex i;
				i.frame = frameIndex;
				i.point = patchIndex;
				indices.push_back(i);
			}
		}
	}
	if (indices.size() > 0) {
		optimizer->compute( &indices, data, pg, afterEachIteration, callbackContext );
		
		for (unsigned int i=0; i<indices.size(); i++) {
			int32 frameIndex = indices[i].frame;
			card32 patchIndex = indices[i].point;
			int32 flags = pg->getTrajectorySet()->getTrajectories()->get<int>(frameIndex, patchIndex, AAT_FLAGS);;
			flags &= 0xFFFFFFFF ^ writeBackClearFlags;
			pg->getTrajectorySet()->getTrajectories()->set<int>(frameIndex, patchIndex, AAT_FLAGS, flags);
		}
	}
}
