//---------------------------------------------------------------------------
#ifndef PGNeighborhoodExtrapolator4DH
#define PGNeighborhoodExtrapolator4DH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "PatchTypes.h"
#include "PatchGraphBSROpt.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PatchGraph;
class AnimationBSRContext;

///
/// extrapolate points in patch graphs in "hole" areas
///
class ANIMATIONRECONSTRUCTION_API PGNeighborhoodExtrapolator4D {
 public:

	/// this method computes a initial guess for the position of a point from neighboring points
	/// it consideres the 4d neighborhood, i.e. performs spatial and temporal prediction (tmp: currently not implemented)
	/// points that can be used for prediction are expected to be marked with a flag in
	/// the "flags" channel (int32[1]), see PatchTypes.h
	///
	/// flags: useInAvFlags       - these flags must be present in neighboring points to be used estimate
	///        useNotInAvFlags    - these flags must not be present in neighboring points to be used estimate
	///        computeNotForFlags - estimates are only created for points that do not provide these flags
	///        writeBackFlags     - this flag is set for each estimated value
	/// return value: 
	///        number of points modified
   static card32 extrapolateNeighbors(
		AnimationBSRContext *context, 
		PatchGraph *pg,
		int32 useInAvFlags       = FLAG_PATCH_INITIALIZED,
		int32 useNotInAvFlags    = FLAG_PATCH_GUESS,
		int32 computeNotForFlags = FLAG_PATCH_INITIALIZED,
		int32 writeBackFlags     = FLAG_PATCH_GUESS | FLAG_PATCH_INITIALIZED
	);

	/// optimize part of a patch graph with certain flags set; clear flags afterwards
	/// optimizer must be fully initialized
	static void partialOptimization(
		AnimationBSRContext *context,
		PatchGraph *pg,
		PointCloudAnimationSequence *data,
		PatchGraphBSROpt *optimizer,
      PatchGraphBSROpt::FeedbackCallback afterEachIteration = NULL,
      void *callbackContext = NULL,		
		int32 computeForFlags     = FLAG_PATCH_GUESS,
		int32 writeBackFlags	     = 0,
		int32 writeBackClearFlags = FLAG_PATCH_GUESS
	);
};

}

#endif