//---------------------------------------------------------------------------
#ifndef PatchApprox3DH
#define PatchApprox3DH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PatchGraph;
class PointCloudAnimationSequence;
class HeightFieldBasisFunctionSet;

/**
	computes (regularized) smooth patches via local least squares fit
*/
class ANIMATIONRECONSTRUCTION_API PatchApprox3D : public Persistent {
	GEOXL_CLASS( PatchApprox3D )

 public:
	static const card32 WINDOW_NONE = 0;
	static const card32 WINDOW_GAUSSIAN = 1;
	static const card32 WINDOW_LIN = 2;

	static const card32 WINDOW_FIXED = 0;
	static const card32 WINDOW_CHANNEL = 1;

 private:
	card32 windowingFunction;
	/// if set to WINDOW_FIXED, constantWindowSize is used as window size
	/// if set to WINDOW_CHANNEL, constantWindowSize is multiplied with attribute in channel "sampleSpacing" of the patch graph point set
	card32 windowSizing;
	float32 constantWindowSize;

	HeightFieldBasisFunctionSet *bfs;

	static float32 winGaussian(const Vector2f &pos, float32 windowSize);
	static float32 winLin(const Vector2f &pos, float32 windowSize);
 public:
	PatchApprox3D();
	
	void setup(HeightFieldBasisFunctionSet *bfs, card32 windowingFunction = WINDOW_NONE, card32 windowSizing = WINDOW_FIXED, float32 constantWindowSize = 1);
   
   void assign(const Object* obj, CopyContext *context = NULL);
   
	virtual void computeCoeffsUnregularized(PatchGraph *pg, PointCloudAnimationSequence *data);
	
	virtual void computeCoeffsRegularized(PatchGraph *pg, PointCloudAnimationSequence *data);
   
	virtual ~PatchApprox3D();

};


/// first argument is pos in uv-space, second is window size
typedef float32(*WeightFunction)(const Vector2f &, float32);

/// setups a least squares height field approximation problem from scattered points and a coordinate system
/// output can be used as input to quadpack/LeastSquaresSystemSetup.h/DenseLeastSquaresSystemSetup
///
/// parameters:
///   basisFunctions - bf to be evaluated
///   weightFunction - specifying weights; if this parameter is NULL, targetValueWeights must also be NULL
///   points - scattered 3d points
///   origin, transf - local coordinate system (local coordinate axis must be ROWS of transf, row0==coord_u, row1==coord_v, row2==normal a.k.a. height)
/// output:
///   targetValues, basisFunctionValues, targetValueWeights - output for input in DenseLeastSquaresSystemSetup
void ANIMATIONRECONSTRUCTION_API setupDenseLS(HeightFieldBasisFunctionSet *basisFunctions,
			  WeightFunction weightFunction,
			  const vector<Vector3f> &points,
			  const Vector3f &origin,
			  const Matrix3f &transf,
			  DynamicVector<float32> &targetValues,
			  DynamicMatrix<float32> &basisFunctionValues,
			  DynamicVector<float32> *targetValuesWeights = NULL,
			  float32 windowSize = 1.0f);


}

#endif