//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PatchGraphComponentsHandler.h"
#include <stack>
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS( PatchGraphComponentsHandler, 0 )
{
	BEGIN_CLASS_INIT( PatchGraphComponentsHandler )
}

/**
 * Constructor
 */
PatchGraphComponentsHandler::PatchGraphComponentsHandler() 
{
}

/**
 * Destructor
 */
PatchGraphComponentsHandler::~PatchGraphComponentsHandler() 
{
}

/** 
 * Detect non-connected components in a patch graph
 */
void PatchGraphComponentsHandler::detectComponents( PatchGraph* pg )
{
	PointSet* traj = pg->getTrajectorySet()->getTrajectories();
	card32 numPatches = pg->getTrajectorySet()->getNumPatches();
	AAT FLAGS = traj->getAAT( "flags" );
	InCorePCTopologyGraph* tg = pg->getTopology();

	// Reset flags
	for ( unsigned int patchIndex = 0; patchIndex < numPatches; patchIndex++ ){
		traj->set1i( 0, patchIndex, FLAGS, traj->get1i( 0, patchIndex, FLAGS ) & !PF_FLAG_MODIFIED );
	}

	// Region grow on components
	card32 numComponents = 0;
	for ( unsigned int patchIndex = 0; patchIndex < numPatches; patchIndex++ ){
		if ( !( traj->get1i( 0, patchIndex, FLAGS ) & PF_FLAG_MODIFIED ) ){
			// Start region grow on 
			numComponents++;
			stack<card32> stack;
			stack.push( patchIndex );
			// Mark as visited
			traj->set1i( 0, patchIndex, FLAGS, traj->get1i( 0, patchIndex, FLAGS ) | PF_FLAG_MODIFIED );
			while( !stack.empty() ){
				card32 index = stack.top();
				stack.pop();
				for ( unsigned int n = 0; n < tg->getNumAdjacentVertices( index ); n++ ){
					card32 nIndex = tg->getAdjacentVertex( index, n );
					if ( !( traj->get1i( 0, nIndex, FLAGS ) & PF_FLAG_MODIFIED ) ){
						stack.push( nIndex );
						// Mark as visited
						traj->set1i( 0, nIndex, FLAGS, traj->get1i( 0, nIndex, FLAGS ) | PF_FLAG_MODIFIED );
					}
				}
			}
			
		}
	}
	debugOutput << numComponents << " independent components found.\n"; debugOutput.flush();
}