//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PatchGraphCreator3D.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "TrajectorySet.h"
#include "PatchGraph.h"
#include "InCorePCTopologyGraph.h"
#include "PGDataPointIndices.h"
#include "PatchTypes.h"
#include "PointCloudAnimationSequence.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


static string descrIncludeHeightField(card32 v) {
	switch (v) {
		case 0: return "PATCH_HEIGHTFIELD_NONE";
		case 1: return "PATCH_HEIGHTFIELD_CONST";
		case 2: return "PATCH_HEIGHTFIELD_LINEAR";
		case 3: return "PATCH_HEIGHTFIELD_QUAD";
		default: return "unknown";
	} 
}
static string descrIncludeCorrespondences(card32 v) {
	switch (v) {
		case 0: return "PATCH_CORR_NONE";
		case 1: return "PATCH_CORR_CONST";
		case 2: return "PATCH_CORR_LINEAR";
		case 3: return "PATCH_CORR_QUAD";
		default: return "unknown";
	} 
}

IMPLEMENT_GEOXL_CLASS( PatchGraphCreator3D ,0)
{
	BEGIN_CLASS_INIT( PatchGraphCreator3D )
	ADD_BOOLEAN_PROP(includeReferenceFrame, 0)
	ADD_CARD32_PROP_UPDATE_DESCR(includeHeightField, 0, NULL, descrIncludeHeightField, 4)
	ADD_CARD32_PROP_UPDATE_DESCR(includeCorrespondences, 0, NULL, descrIncludeCorrespondences, 4)
}


// ----


PatchGraphCreator3D::PatchGraphCreator3D() {
	includeReferenceFrame = true;
	includeHeightField = PATCH_HEIGHTFIELD_QUAD;
	includeCorrespondences = PATCH_CORR_NONE;
}

void PatchGraphCreator3D::setup(bool includeReferenceFrame, card32 includeHeightField, card32 includeCorrespondences) {
	this->includeReferenceFrame  = includeReferenceFrame;
	this->includeHeightField     = includeHeightField;
	this->includeCorrespondences = includeCorrespondences;
}

void PatchGraphCreator3D::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}


//TrajectorySet *PatchGraphCreator3D::createTrajectoryset(const PointSet *ps) {
//}

PatchGraph *PatchGraphCreator3D::createPatchGraph(const PointSet *ps, const InCorePCTopologyGraph *tg, bool in2d) 
{
	if (!ps->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) 
	{
		throw PException("PatchGraphCreator3D::createPatchGraph() - ps must provide position3f channel.");
	}
	if ( ( ps == NULL ) || ( tg == NULL ) ) 
	{
		throw PException( "PatchGraphCreator3D::createPatchGraph() - point set and topology graph cannot be NULL.");
	}

	VertexDescriptor newAttribs;
	if (!ps->providesAttribute("sampleSpacing")) 
	{
		newAttribs.pushAttrib(mVAD("sampleSpacing", 1, VAD::DATA_FORMAT_FLOAT32));
	}
	if (includeReferenceFrame) 
	{
		newAttribs.pushAttrib(NORMAL_PATCH_VAD);
		newAttribs.pushAttrib(TANGENT_U_PATCH_VAD);
	}

	switch (includeHeightField) 
	{
		case PATCH_HEIGHTFIELD_NONE:   
		break;
		case PATCH_HEIGHTFIELD_CONST:
			newAttribs.pushAttrib(createHeightfieldCoeffVAD(1));
		break;
		case PATCH_HEIGHTFIELD_LINEAR:
			newAttribs.pushAttrib(createHeightfieldCoeffVAD(3));
		break;
		case PATCH_HEIGHTFIELD_QUAD:
			newAttribs.pushAttrib(createHeightfieldCoeffVAD(6));
		break;
		default: break;
	}
	switch (includeCorrespondences) 
	{
		case PATCH_CORR_NONE:   
		break;
		case PATCH_CORR_CONST:
			newAttribs.pushAttrib(createCorrespondenceCoeffVAD(3));
		break;
		case PATCH_CORR_LINEAR:
			newAttribs.pushAttrib(createCorrespondenceCoeffVAD(9));
		break;
		case PATCH_CORR_QUAD:
			newAttribs.pushAttrib(createCorrespondenceCoeffVAD(18));
		break;
		default: break;
	}
	PointSet *psCopy = ps->appendFields(&newAttribs);

	PGDataPointIndices *dp = new PGDataPointIndices();
	dp->setup(1, ps->getNumEntries());
	const card32 n = ps->getNumEntries();
	for (card32 i=0; i<n; i++) 
	{
		const card32 nEdges = tg->getNumAdjacentVertices(i);
		vector<card32> neighbors(nEdges);
		for (card32 j=0; j<nEdges; j++) 
		{
			neighbors[j] = tg->getAdjacentVertex(i,j);
		}
		dp->setIndices(0, i, neighbors);
	}

	TrajectorySet *ts = new TrajectorySet();
	
	ts->clearAndSetup(psCopy, dp);
	
	PatchGraph *result = new PatchGraph();
	result->clearAndSetup(ts, (InCorePCTopologyGraph*)(tg->copy()), in2d);
	return result;
}

PointCloudAnimationSequence *PatchGraphCreator3D::createScene(const PointSet *ps) {
	PointCloudAnimationSequence *result = new PointCloudAnimationSequence;
	result->addFrame(ps);
	return result;
}
