//---------------------------------------------------------------------------
#ifndef PatchGraphCreator3DH
#define PatchGraphCreator3DH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PointSet;
class InCorePCTopologyGraph;
class TrajectorySet;
class PatchGraph;
class PointCloudAnimationSequence;



/**
	this class creates a non-time variant patch graph from a point set
	this is an "operation" class, it encapsulates a single operation (and its suboperations)
	member variables are parameters to this operation
*/
class ANIMATIONRECONSTRUCTION_API PatchGraphCreator3D : public Persistent {
	GEOXL_CLASS( PatchGraphCreator3D )

 public:
	static const card32 PATCH_HEIGHTFIELD_NONE   = 0;
	static const card32 PATCH_HEIGHTFIELD_CONST  = 1;
	static const card32 PATCH_HEIGHTFIELD_LINEAR = 2;
	static const card32 PATCH_HEIGHTFIELD_QUAD   = 3;

	static const card32 PATCH_CORR_NONE   = 0;
	static const card32 PATCH_CORR_CONST  = 1;
	static const card32 PATCH_CORR_LINEAR = 2;
	static const card32 PATCH_CORR_QUAD   = 3;

 private:
	bool includeReferenceFrame;
	card32 includeHeightField;
	card32 includeCorrespondences;

 public:
	PatchGraphCreator3D();

	void setup(bool includeReferenceFrame, card32 includeHeightField, card32 includeCorrespondences);

   virtual void assign(const Object* obj, CopyContext *context = NULL);
   
   /// create the patch graph (contains one frame)
   virtual PatchGraph *createPatchGraph(const PointSet *ps, const InCorePCTopologyGraph *tg, bool in2d);
   /// create the corresponding one-frame scene containing the original data points
   virtual PointCloudAnimationSequence *createScene(const PointSet *ps);
};

}

#endif