//---------------------------------------------------------------------------
#ifndef PatchGraphMergeH
#define PatchGraphMergeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "PatchGraphContainer.h"
#include "TensorVoting.h"
#include "UniformPointCloudResampling.h"
#include "InCorePCTopologyGraph.h"
#include "PatchGraphResample.h"
#include "AnimationBSRContext.h"
#include "AbstractMergeExtrapolator.h"
#include "PatchGraphBSROpt.h"
#include "UnifyNormals.h"
#include "DetectTopologicalErrorMerge.h"
#include "PatchGraphPruning.h"
#include "TopologyBreaker.h"
#include "PatchGraphOpticalFlow.h"
//---------------------------------------------------------------------------

namespace geoxl {

class FeatureRegistration;
class PatchGraphTopologicalResampling;

/// Merge patch graphs
class ANIMATIONRECONSTRUCTION_API PatchGraphMerge : public Persistent {
	GEOXL_CLASS( PatchGraphMerge )

	private:
		IMPLEMENT_VAR_ARRAY_METHODS_STL(alignMergeExtrapolate)
		/*** MEMBERS ***/		
		/// alternative resampler (can process urshapes, can preserve and/or reassign features)
		PatchGraphTopologicalResampling *altResampler;
		/// Align, merge & extrapolate: perform a list of operations in the given order
		vector<AbstractMergeExtrapolator*> alignMergeExtrapolate;
		/// Context
		AnimationBSRContext* context;

		///Peter's method to create combined trajectory set
		void createCombinedTrajectorySet_Modified( 
			AnimationBSRContext* context,
			TrajectorySet* combinedTS, 
			InCorePCTopologyGraph* tg,
			const PatchGraph* pg1,
			const PatchGraph* pg2 );
		/// reset parameters after external change (such as context)
		void extrapolatorChanged();
		
	public:
		/*** CLASS MANAGEMENT ***/
		/// Constructor
		PatchGraphMerge();   
		/// Destructor
		virtual ~PatchGraphMerge();

		/*** OPERATIONS ***/
		/// Peter's method to merge two patch graphs
		bool merge_optimized2(
			AnimationBSRContext *context, 
			FeatureRegistration *featureMatcher, 
			const int32 pgIndex1, 
			const int32 pgIndex2, 
			DebugRenderer *debugRenderer, 
			bool verbose  = false );

		/*** PERSISTENT ***/
		/// Object assignment
		void assign( const Object* obj, CopyContext *context = NULL );

		/*** MEMBER ACCESS ***/
		/// add align/merge/extrapolate step, ownership transfered to this
		void addAlignMergeExtrapolateStep(AbstractMergeExtrapolator *step);
		void clearAMESteps();
		AbstractMergeExtrapolator *getMerger(card32 i) {if (i>=alignMergeExtrapolate.size()) return NULL; else return alignMergeExtrapolate[i];}
		void setContext( AnimationBSRContext* context );
};
}

#endif


/*
/// Check if merge causes topological problems
bool checkTopology;
/// Apply frame internal merge
bool applyPruning;
/// Break up single connections in topology
bool applyTopologyBreaker;
/// distributed data points among neighbors
bool extrapolateDataPointsAfterMerge;
/// Optimizer
PatchGraphBSROpt* optimizer;
/// switch between resampling objects
bool useAltResampler;
/// Resampler
PatchGraphResample* resampler;
/// Normal unifier
UnifyNormals* normalUnifier;
/// Apply optimization
bool applyOptimization;
/// Optical flow
PatchGraphOpticalFlow* opticalFlow;
/// Frame internal merge operation
PatchGraphPruning* pruner;
/// Break up topology within patch graph if required
TopologyBreaker* topologyBreaker;
/// Copy all information existent in pg1 and pg2 to combined data structure
void createCombinedTrajectorySet( 
AnimationBSRContext* context,
TrajectorySet* combinedTS, 
InCorePCTopologyGraph* tg,
const PatchGraph* pg1,
const PatchGraph* pg2 );
/// Reassign data points to patch points
void extrapolateDataPoints(
AnimationBSRContext* context,
PatchGraph* combinedPG, 
const card32 frameLeft, 
const card32 frameRight,
const card32 patchOffset );
/// Optimize a patch graph
void optimize( PatchGraph* pg );
/// Peter's method to merge two patch graphs
bool merge_optimized(
AnimationBSRContext *context, 
FeatureRegistration *featureMatcher, 
const int32 pgIndex1, 
const int32 pgIndex2, 
DebugRenderer *debugRenderer, 
bool verbose  = false );
void setApplyOptimization( bool b ){ applyOptimization = b; }
inline void setApplyPruning( bool b ){ applyPruning = b; }
inline PatchGraphBSROpt* getOptimizer() const { return optimizer; }
inline PatchGraphOpticalFlow* getOpticalFlow() const { return opticalFlow; }
inline void setCheckTopology( bool b ){ checkTopology = b; }
/// Merge two patch graphs; return true on success
bool merge( 
AnimationBSRContext *context, 
FeatureRegistration *featureMatcher,
const int32 pgIndex1, 
const int32 pgIndex2, 
DebugRenderer *debugRenderer, 
bool verbose = false 
);
*/