//---------------------------------------------------------------------------
#ifndef PatchGraphOpticalFlowH
#define PatchGraphOpticalFlowH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
#include "Persistent.h"
#include "PatchGraph.h"
#include "AnimationBSRContext.h"
#include "SplatPullPushInterpolation.h"
#include "Monomial2ndOrderDisplayer.h"
//---------------------------------------------------------------------------

namespace geoxl {

struct PatchGraphOpticalFlowGradient
{
	Vector3f g;
	StaticVector<float32,6> coeffs;
};

/// Compute optical flow on a patch graph
class ANIMATIONRECONSTRUCTION_API PatchGraphOpticalFlow : public Persistent {
	GEOXL_CLASS( PatchGraphOpticalFlow )

	private:
		/// Interpolator
		SplatPullPushInterpolation* splatInterpolation;
		/// Factor of sampling spacing used as influence radius
		float32 influenceScale;
		/// Image size
		card32 imageSize;

		/*** VISUALIZATION ***/
		// Debug visualization
		bool visualize;

		/// Comparison window container
		TemplatedImage<Vector3f>* comparisonWindow;
		/// Error image container
		TemplatedImage<float32>* errorImage;
		
	public:
		/// Constructor
		PatchGraphOpticalFlow();
		/// Destructor
		~PatchGraphOpticalFlow();

		/// Compute the gradient of the optical flow weighted by the corresponding eigenValues
		/// localFrameIndex and localFrameIndex+1 at patch with number patchIndex, return true on success
		bool compute( 
			PatchGraphOpticalFlowGradient& gradient, 
			PatchGraph* pg, const card32 localFrameIndex, const card32 patchIndex, 
			PointCloudAnimationSequence* animSeq,
			const float32 sampling );

		/// Compute the gradient of the optical flow weighted by the corresponding eigenValues
		/// localFrameIndex and localFrameIndex+1 for all patches, return true on success
		bool computeAll( 
			vector<PatchGraphOpticalFlowGradient>& gradients, 
			PatchGraph* pg, const card32 localFrameIndex,
			PointCloudAnimationSequence* animSeq,
			const float32 sampling );

		/*** MEMBER ACCESS ***/
		inline void setVisualize( bool b ){ visualize = b; }
		void setImageSize( const card32 imageSize );
};
}

#endif