//---------------------------------------------------------------------------
#ifndef PatchGraphPruningH
#define PatchGraphPruningH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
#include "Persistent.h"
#include "PatchGraph.h"
#include "AnimationBSRContext.h"
//---------------------------------------------------------------------------

namespace geoxl {

/// Check if two patch graphs within the same frame(s) can be merged (delete one patch graph; reassign data points)
class ANIMATIONRECONSTRUCTION_API PatchGraphPruning : public Persistent {
	GEOXL_CLASS( PatchGraphPruning )

	private:
		/// Context
		AnimationBSRContext* context;
		/// Percentage of patches that need to match for valid merge
		float32 matchingThreshold;
		/// Minimum size factor for test to be applied
		float32 sizeThreshold;
		/// Remove threshold of patch graph is not supported by data points
		float32 prunePatchGraphThreshold;

		/// Check if first patch graph is contained in second patch graph
		bool isContainedFirstInSecond( const PatchGraph* pg1, const PatchGraph* pg2 ) const;

		/*
		/// DEPRECATED: NOT REQUIRED ANY LONGER
		/// Use all data points associated with pg1 to update pg2
		void reassignDataPointsFromFirstToSecond( const PatchGraph* pg1, PatchGraph* pg2 );
		*/
		
	public:
		/// Constructor
		PatchGraphPruning();
		/// Destructor
		~PatchGraphPruning();

		/// Prune patch graphs which are too small
		void pruneSmallPatchGraphs( bool verbose );
		/// Prune patch graphs which are not supported by data points
		void pruneNoDataPatchGraphs( bool verbose );
		/// Prune patch graphs which represent data contained in other patch graphs
		void pruneIncludedPatchGraphs( bool verbose );

		/*** MEMBER ACCESS ***/
		inline void setContext( AnimationBSRContext* context ){ this->context = context; }

};
}

#endif