//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PatchGraphRoutines.h"
//---------------------------------------------------------------------------
#include "PatchTypes.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


inline static float32 normalPlaneDist(const Vector3f &planeOrigin, const Vector3f &normalizedPlaneNormal, const Vector3f &point, Vector3f &minPoint) {
	
	float32 minDist = (point - planeOrigin) * normalizedPlaneNormal;
	minPoint = point - normalizedPlaneNormal*minDist;
	return minDist;
}

void geoxl::computeClosestPointQuadPatch(
	const Vector3f &patchOrigin, 
	const ReferenceFramePatch &ref, 
	const CoefficientSet &quadraticCoefficients, 
	const Vector3f &targetPoint, 
	const float32 searchDistance,
	Vector2f &minUV,
	float32 &minDist,
	const card32 startGrid,
	const card32 numNewton) 
{
	Matrix3f toLocal;
	toLocal[0] = ref.tangent_u;
	toLocal[1] = ref.computeTangentV();
	toLocal[2] = ref.normal;
	toLocal = toLocal.transpose();
	
	Vector3f localTarget = toLocal * (targetPoint - patchOrigin);
	
	float32 c00 = quadraticCoefficients.coeffs[0];
	float32 c01	= quadraticCoefficients.coeffs[1];
	float32 c10	= quadraticCoefficients.coeffs[2];
	float32 c02	= quadraticCoefficients.coeffs[3];
	float32 c11	= quadraticCoefficients.coeffs[4];
	float32 c20 = quadraticCoefficients.coeffs[5];
	minDist = 1E30f;
	for (card32 vi=0; vi<startGrid; vi++) {
		float32 vf = ((float32)vi/(float32)(startGrid-1))*2.0f*searchDistance - searchDistance;
		for (card32 ui=0; ui<startGrid; ui++) {
			float32 uf = ((float32)ui/(float32)(startGrid-1))*2.0f*searchDistance - searchDistance;
			float32 nf = c00 + c01*uf + c10*vf + c11*uf*vf + c02*uf*uf + c20*vf*vf;
			float32 dn_du = c01 + c11*vf + 2.0f*c02*uf;
			float32 dn_dv = c10 + c11*uf + 2.0f*c20*vf;
			Vector3f normal = normalize(makeVector3f(-dn_du, -dn_dv, 1));
			Vector3f localVector = makeVector3f(uf, vf, nf);
			
			Vector3f minPoint3f;
			float32 dist = normalPlaneDist(localVector, normal, localTarget, minPoint3f);
			
			if (minDist > dist) {
				minDist = dist;
				minUV = makeVector2f(minPoint3f[0], minPoint3f[1]);
			}
		}
	}
	for (card32 i=0; i<numNewton; i++) {
		float32 uf = minUV[0];
		float32 vf = minUV[1];
		float32 nf = c00 + c01*uf + c10*vf + c11*uf*vf + c02*uf*uf + c20*vf*vf;
		float32 dn_du = c01 + c11*vf + 2.0f*c02*uf;
		float32 dn_dv = c10 + c11*uf + 2.0f*c20*vf;
		Vector3f normal = normalize(makeVector3f(-dn_du, -dn_dv, 1));
		Vector3f localVector = makeVector3f(uf, vf, nf);
		
		Vector3f minPoint3f;
		float32 dist = normalPlaneDist(localVector, normal, localTarget, minPoint3f);
		
		if (minDist > dist) {
			minDist = dist;
			minUV = makeVector2f(minPoint3f[0], minPoint3f[1]);
		}
	}
}
