//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "RefFrameEst3D.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PatchGraph.h"
#include "InCorePCTopologyGraph.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "VertexDescriptor.h"
#include "TrajectorySet.h"
#include "PatchTypes.h"
#include "PointCloudAnimationSequence.h"
#include "PGDataPointIndices.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS(RefFrameEst3D, 0)
{
	BEGIN_CLASS_INIT(RefFrameEst3D)
}


// --


RefFrameEst3D::RefFrameEst3D() {
}

void RefFrameEst3D::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

RefFrameEst3D::~RefFrameEst3D() {
}

void RefFrameEst3D::estimateReferenceFrameFromNeighbors(PatchGraph *pg) {
	InCorePCTopologyGraph *tg = pg->getTopology();
	TrajectorySet *ts = pg->getTrajectorySet();
	const card32 numFrames = ts->getNumFrames();

	for (card32 frame=0; frame<numFrames; frame++) {
		PointSet *ps = ts->copySingleFrame(frame);
		if (   !ps->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)
		    || !ps->getDescr()->providesAttribute("normal", 3, VAD::DATA_FORMAT_FLOAT32)
		    || !ps->getDescr()->providesAttribute("tangent_u", 3, VAD::DATA_FORMAT_FLOAT32)) 
		{
			throw PException("RefFrameEst3D::estimateReferenceFrame() - did not find all required channels.");
		}
		AAT POS = ps->getAAT("position");
		AAT NORMAL = ps->getAAT("normal");
		AAT TANGENT_U = ps->getAAT("tangent_u");

		const card32 numPatches = ps->getNumEntries();
		for (card32 i=0; i<numPatches; i++) {
			RegressionPlane rp;
			rp.clear();
			rp.addPoint(ps->get3f(i, POS));
			const card32 numNeighbors = tg->getNumAdjacentVertices(i);

			for (card32 j=0; j<numNeighbors; j++) {
				rp.addPoint( ps->get3f( tg->getAdjacentVertex(i,j), POS ) );
			}
	  
			rp.computePlane();

			ReferenceFramePatch refPatch;
			refPatch.normal = normalize(rp.getNormal());
			refPatch.tangent_u = normalize(rp.getU());
			refPatch.set(ps, i, NORMAL, TANGENT_U);
		}
		ts->setSingleFrame(frame, ps);
		delete ps;	
	}
}

void RefFrameEst3D::estimateReferenceFrameFromDataPoints(PatchGraph *pg, PointCloudAnimationSequence *dataPoints) {
	TrajectorySet *ts = pg->getTrajectorySet();
	const card32 numFrames = ts->getNumFrames();

	for (card32 frame=0; frame<numFrames; frame++) {
		PointSet *patchPS = ts->copySingleFrame(frame);
		const PointSet *dataPS = dataPoints->getFrame(frame);
		if (   !patchPS->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)
		    || !patchPS->getDescr()->providesAttribute("normal", 3, VAD::DATA_FORMAT_FLOAT32)
		    || !patchPS->getDescr()->providesAttribute("tangent_u", 3, VAD::DATA_FORMAT_FLOAT32)
		    || !dataPS->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32) ) 
		{
			throw PException("RefFrameEst3D::estimateReferenceFrame() - did not find all required channels.");
		}
		AAT NORMAL = patchPS->getAAT("normal");
		AAT TANGENT_U = patchPS->getAAT("tangent_u");
		AAT DATA_POS = dataPS->getAAT("position");

		const card32 numPatches = patchPS->getNumEntries();
		for (card32 i=0; i<numPatches; i++) {
			RegressionPlane rp;
			rp.clear();
			const PGDataPointIndices *dataPoints = ts->getDataPoints();
			const card32 numNeighbors = dataPoints->getNumDataPoints(frame, i);

			for (card32 j=0; j<numNeighbors; j++) {
				rp.addPoint( dataPS->get3f( dataPoints->getIndex(frame, i, j), DATA_POS ) );
			}
	  
			rp.computePlane();

			ReferenceFramePatch refPatch;
			refPatch.normal = normalize(rp.getNormal());
			refPatch.tangent_u = normalize(rp.getU());
			refPatch.set(patchPS, i, NORMAL, TANGENT_U);
		}
		ts->setSingleFrame(frame, patchPS);
		delete patchPS;	
	}
}