//---------------------------------------------------------------------------
#ifndef TGNeighborhoodKNNSearchH
#define TGNeighborhoodKNNSearchH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "VertexDescriptor.h"
//---------------------------------------------------------------------------
#include <queue>
//---------------------------------------------------------------------------


namespace geoxl {
   using namespace std;
   
class InCorePCTopologyGraph;
class PointSet;
class PatchGraph;


/// nearest-neighbors search along topology graph edges
/// the parameter "frame" is used as x-coordinate in the point set
/// for 1d point sets, this parameter can be set to zero
/// for 2d point sets that do not store frames in columns, this class cannot be used
class ANIMATIONRECONSTRUCTION_API TGNeighborhoodKNNSearch {
 private:
   const InCorePCTopologyGraph *tg;
   const PointSet *ps;
   Vector3f pos;
   AAT channelToken;
   card32 frame;
   
   struct QueueEntry {
		float32 dist;
		card32 index;

		inline QueueEntry() {}
		inline QueueEntry(float32 d, card32 i) {dist=d; index=i;}		
		inline bool operator<(const QueueEntry &e) const {return dist > e.dist;}
   };
   
   priority_queue<QueueEntry> neighbors;
   set<card32> visited;

 public:
   TGNeighborhoodKNNSearch(const PointSet *ps, const InCorePCTopologyGraph *tg, card32 frame, card32 startIndex, const string channel3f = "position");
   bool atEnd(); 
   void next();
   void reset(card32 frame, card32 startIndex);
   card32 getCurrentIndex();
};


/// same as TGNeighborhoodKNNSearch, but for PatchGraphs
class ANIMATIONRECONSTRUCTION_API GeneralPGNeighborhoodKNNSearch {
 private:
   const PatchGraph *pg;
   card32 frame;
   card32 startIndex;
   AAT channelToken;
   
   struct QueueEntry {
		float32 dist;
		card32 index;

		inline QueueEntry() {}
		inline QueueEntry(float32 d, card32 i) {dist=d; index=i;}		
		inline bool operator<(const QueueEntry &e) const {return dist > e.dist;}
   };
   
   priority_queue<QueueEntry> neighbors;
   set<card32> visited;
   
 protected:
   virtual float32 calcDistance(const PatchGraph *pg, card32 frame, card32 patch1, card32 patch2, const AAT &channelToken) = 0;

 public:
   GeneralPGNeighborhoodKNNSearch(const PatchGraph *pg, card32 frame, card32 startIndex, const string channel3f = "position");
   bool atEnd(); 
   void next();
   void reset(card32 frame, card32 startIndex);
   card32 getCurrentIndex();
};


/// same as TGNeighborhoodKNNSearch, but for PatchGraphs
class ANIMATIONRECONSTRUCTION_API PGNeighborhoodKNNSearch : public GeneralPGNeighborhoodKNNSearch {
 private:
   bool averageDist;
   
 protected:
   virtual float32 calcDistance(const PatchGraph *pg, card32 frame, card32 patch1, card32 patch2, const AAT &channelToken);

 public:
   PGNeighborhoodKNNSearch(const PatchGraph *pg, card32 frame, card32 startIndex, const string channel3f = "position", bool averageDist = false);
};


}


#endif