//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "TGNeighborhoodRangeSearch.h"
//---------------------------------------------------------------------------
#include "InCorePCTopologyGraph.h"
#include "PatchGraph.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------


TGNeighborhoodRangeSearch::TGNeighborhoodRangeSearch() {
   tg = NULL;
   ps = NULL;
}
void TGNeighborhoodRangeSearch::setup(const InCorePCTopologyGraph *tg, const PointSet *ps, card32 startIndex) {
   this->tg = tg;
   this->ps = ps;
   if (inRange(ps, startIndex)) {
      indexStack.push_back(startIndex);
      visited.insert(startIndex);
   }
}

bool TGNeighborhoodRangeSearch::atEnd() {
   return indexStack.empty();
}

void TGNeighborhoodRangeSearch::next() {
   pAssert(!indexStack.empty());
   card32 index = indexStack.back();
   indexStack.pop_back();
   const card32 n = tg->getNumAdjacentVertices(index);
   for (card32 i=0; i<n; i++) {
      card32 j = tg->getAdjacentVertex(index, i);
      if (visited.find(j) == visited.end()) {
         if (inRange(ps, j)) {
            indexStack.push_back(j);
            visited.insert(j);
         }
      }
   }
}

card32 TGNeighborhoodRangeSearch::getCurrentIndex() {
   pAssert(!indexStack.empty());
   return indexStack.back();
}


// ----


PGNeighborhoodRangeSearch::PGNeighborhoodRangeSearch() {
   pg = NULL;
   frame = 0;
}
void PGNeighborhoodRangeSearch::setup(const PatchGraph *pg, card32 frame, card32 startIndex) {
   visited.clear();
   this->pg = pg;
   this->frame = frame;
   if (inRange(pg, frame, startIndex)) {
      indexStack.push_back(startIndex);
      visited.insert(startIndex);
   }
}

bool PGNeighborhoodRangeSearch::atEnd() {
   return indexStack.empty();
}

void PGNeighborhoodRangeSearch::next() {
   pAssert(!indexStack.empty());
   card32 index = indexStack.back();
   indexStack.pop_back();
   const card32 n = pg->getTopology()->getNumAdjacentVertices(index);
   for (card32 i=0; i<n; i++) {
      card32 j = pg->getTopology()->getAdjacentVertex(index, i);
      if (visited.find(j) == visited.end()) {
         if (inRange(pg, frame, j)) {
            indexStack.push_back(j);
            visited.insert(j);
         }
      }
   }
}

card32 PGNeighborhoodRangeSearch::getCurrentIndex() {
   pAssert(!indexStack.empty());
   return indexStack.back();
}


void PGNeighborhoodSphereSearch::setup(const PatchGraph *pg, card32 frame, const card32 startIndex,
                                       const Vector3f &center, const float32 dist, const AAT &posChannel)
{
	this->center = center;
	this->dist = dist;
	this->posChannel = posChannel;
	PGNeighborhoodRangeSearch::setup(pg, frame, startIndex);
}

bool PGNeighborhoodSphereSearch::inRange(const PatchGraph *pg, card32 frame, card32 index) {
	const PointSet *ps = pg->getTrajectorySet()->getTrajectories();
	Vector3f pos = ps->get<float32, 3>(frame, index, posChannel);
	return norm(pos-center) <= dist;
}

