//---------------------------------------------------------------------------
#ifndef TGNeighborhoodRangeSearchH
#define TGNeighborhoodRangeSearchH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------


namespace geoxl {
   using namespace std;
   
class InCorePCTopologyGraph;
class PointSet;   
class PatchGraph;

/// search a range that is topologically reachable 
class ANIMATIONRECONSTRUCTION_API TGNeighborhoodRangeSearch {
 private:
   const InCorePCTopologyGraph *tg;
   const PointSet *ps;
   vector<card32> indexStack;
   set<card32> visited;

 protected:
   virtual bool inRange(const PointSet *ps, card32 index) = 0;
   void setup(const InCorePCTopologyGraph *tg, const PointSet *ps, card32 startIndex);
   
 public:
   TGNeighborhoodRangeSearch();
   bool atEnd(); 
   void next();
   card32 getCurrentIndex();
};


/// same as TGNeighborhoodRangeSearch but for patch graphs
class ANIMATIONRECONSTRUCTION_API PGNeighborhoodRangeSearch {
 private:
   const PatchGraph *pg;
   card32 frame;
   vector<card32> indexStack;
   set<card32> visited;

 protected:
   virtual bool inRange(const PatchGraph *pg, card32 frame, card32 index) = 0;
   void setup(const PatchGraph *pg, card32 frame, card32 startIndex);
   
 public:
   PGNeighborhoodRangeSearch();
   bool atEnd(); 
   void next();
   card32 getCurrentIndex();
};


// --- example instances

/// search topologically reachable spherical region
class ANIMATIONRECONSTRUCTION_API PGNeighborhoodSphereSearch : public PGNeighborhoodRangeSearch {
 private:
	Vector3f center;
	float32 dist;
	AAT posChannel;

 protected:
   virtual bool inRange(const PatchGraph *pg, card32 frame, card32 index);
   
 public:
   void setup(const PatchGraph *pg, card32 frame, const card32 startIndex, 
              const Vector3f &center, const float32 dist, const AAT &posChannel);
};


}


#endif