//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "TopologyBreaker.h"
#include "InCorePCTopologyGraph.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS( TopologyBreaker, 0 )
{
	BEGIN_CLASS_INIT( TopologyBreaker )
}

/** 
 * Constructor
 */
TopologyBreaker::TopologyBreaker()
{
	context = NULL;
	lengthScaleThreshold = 2.0f;
}

/** 
 * Destructor
 */
TopologyBreaker::~TopologyBreaker()
{
}

/** 
 * Check for invalid connections; break ir required
 */
void TopologyBreaker::checkAndApply( PatchGraph* pg )
{
	AAT POSITION = pg->getTrajectorySet()->getTrajectories()->getAAT( "position" );
	card32 nIndex;
	float32 minLength;
	float32 maxLength;
	card32 numDeletedEdges = 0;
	for ( unsigned int patchIndex = 0; patchIndex < pg->getTrajectorySet()->getNumPatches(); patchIndex++ ){
		vector<card32> indices = pg->getTopology()->getEdgeIndices( patchIndex );
		for ( unsigned int n = 0; n < pg->getTopology()->getNumAdjacentVertices( patchIndex ); n++ ){
			nIndex = pg->getTopology()->getAdjacentVertex( patchIndex, n );
			minLength = 100000000;
			maxLength = 0;
			for ( unsigned int frameIndex = 0; frameIndex < pg->getTrajectorySet()->getNumFrames(); frameIndex++ ){
				Vector3f pos = pg->getTrajectorySet()->getTrajectories()->get3f( frameIndex, patchIndex, POSITION );
				Vector3f nPos = pg->getTrajectorySet()->getTrajectories()->get3f( frameIndex, nIndex, POSITION );
				float32 l = norm( pos - nPos );
				if ( l > maxLength ) maxLength = l;
				if ( l < minLength ) minLength = l;
				
			}
			// Check if edge is invalid
			if ( maxLength / minLength > lengthScaleThreshold ){
				// Remove edge
				pg->getTopology()->deleteEdge( patchIndex, n );
				n--;
				numDeletedEdges++;
			}
		}
	}
	debugOutput << "   Deleted " << numDeletedEdges << " edges due to large deformation.\n"; debugOutput.flush();
	return;
}