//---------------------------------------------------------------------------
#ifndef TriangleMeshTextureH
#define TriangleMeshTextureH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

#define REGION_GROW 0
#define MANUAL_SELECTION 1
#define DISTANCE 2

/// Check if two patch graphs within the same frame(s) can be merged (delete one patch graph; reassign data points)
class ANIMATIONRECONSTRUCTION_API TriangleMeshTexture : public Persistent {
	GEOXL_CLASS( TriangleMeshTexture )

	private:
		/// Filename of the texture
		string textureFilename;
		/// Index of seed point of texture
		card32 seedPointIndex;
		/// Node name
		string meshNode;
		/// Up vector in triangulation
		Vector3f up;
		/// Mode of selection of points used in texture mapping process
		card32 usePoints;

		/// Scene object
		Scene* scene;

		/// Load Texture to image; return true on success
		bool getTexture( QImage& image );
		/// Interpolate the color from the texture image
		Vector3f getColor( Vector2f p, const QImage& texture );
		/// Select points used in texturizing process; return true on success
		bool TriangleMeshTexture::selectPoints( PointSet* vertices, vector<card32>& points );
		
	public:
		/// Constructor
		TriangleMeshTexture();
		/// Destructor
		~TriangleMeshTexture();

		/// Pick a texture file
		void pickTexture();
		/// Apply texture
		void applyTexture();

		/*** MEMBER ACCESS ***/
		inline void setScene( Scene* scene ){ this->scene = scene; }
};
}

#endif