//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "BSRFitting.h"
//---------------------------------------------------------------------------
#include "PatchGraph.h"
#include "InCorePCTopologyGraph.h"
#include "TrajectorySet.h"
#include "AttachedIndexedOctree.h"
#include "PointSetKNNQuery.h"
#include "Monomials2ndOrder.h"
#include "PointSet_templCode.h"
#include "TGNeighborhoodKNNSearch.h"
#include "BSROptContext.h"
#include "PGTopologicalNeighborhood.h"
#include "PatchTypes.h"
#include "ProgressWindow.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------



void BSRFitting::fit2ndOrderHeightfToPSPoint
  (const AAT &POSITION,	const AAT &REF_U,
	const AAT &REF_N, const AAT &COEFF,
   BSROptContext *context, PointSet *ps, InCorePCTopologyGraph *tg, 
   const card32 frameIndex, const card32 patchIndex) 
{
	Matrix3f T;
	StaticVector<float32,6> coeff;
	vector<Vector3f> points;

   Vector3f pos = ps->get3f( frameIndex, patchIndex, POSITION );
   T[0] = ps->get3f( frameIndex, patchIndex, REF_U );
   T[2] = ps->get3f( frameIndex, patchIndex, REF_N );
   T[1] = T[2].crossProduct(T[0]);
   T = T.transpose();
   
   if (context->priorsNeighborhood->getPointSet() != ps) {
      context->priorsNeighborhood->setup(ps, tg);
   }
   vector<card32> neighbors;
   context->priorsNeighborhood->collectNeighbors(frameIndex, patchIndex, neighbors);

   const size_t numNeighbors = neighbors.size();
   
   points.clear();
   for (card32 ni=0; ni<numNeighbors; ni++) {
	   const card32 j = neighbors[ni];
	   Vector3f localPos = T*(ps->get<float32, 3>(frameIndex, j, POSITION) - pos);
	   points.push_back(localPos);
   }

   Monomials2ndOrder::computeCoeffs( points, coeff );
   ps->set<float32,6>( frameIndex, patchIndex, COEFF, coeff );
}

void BSRFitting::fit2ndOrderHeightf(BSROptContext *context, PatchGraph* pg, const BSROptVariableIndexSet &indices) 
{
   PointSet *ps = pg->getTrajectorySet()->getTrajectories();
   PointSet *urshape = pg->getUrshape();
   InCorePCTopologyGraph *tg = pg->getTopology();
	AAT POSITION = ps->getAAT( "position" );
	AAT REF_U = ps->getAAT( "tangent_u" );
	AAT REF_N = ps->getAAT( "normal" );
	AAT COEFF = ps->getAAT( "hfieldCoeffs" );
	
   const size_t numIndices = indices.size();
   for (unsigned index = 0; index<numIndices; index++) {
	   int32 frameIndex = indices[index].frame;
		card32 patchIndex = indices[index].point;
		if (frameIndex != BSROPT_FRAME_URSHAPE) {
	      fit2ndOrderHeightfToPSPoint(
	         POSITION, REF_U, REF_N, COEFF,
            context, ps, tg, frameIndex, patchIndex
         );
	   } else {
	      fit2ndOrderHeightfToPSPoint(
	         POSITION, REF_U, REF_N, COEFF,
            context, urshape, tg, 0, patchIndex
         );
		}
   }
}


/*void BSRFitting::fit1stOrderCorr( BSROptContext *context, PatchGraph* pg, const BSROptVariableIndexSet &indices ) {
	pAssert(pg);
	PointSet *ps = pg->getTrajectorySet()->getTrajectories();

	AAT POSITION = ps->getAAT( "position" );
	AAT REF_U = ps->getAAT( "tangent_u" );
	AAT REF_N = ps->getAAT( "normal" );
	AAT CORR_COEFF = ps->getAAT(CORRESPONDENCE_COEFFS_NAME);

	Matrix3f T;
	vector<Vector3f> corrX, corrY, corrZ;
   context->priorsNeighborhood->setup(pg);
   const card32 numFrames = ps->getWidth();
   const unsigned numIndices = indices.size();
   for (unsigned index = 0; index<numIndices; index++) {
	   int32 frameIndex = indices[index].frame;
		card32 patchIndex = indices[index].point;
		if (frameIndex+1 == numFrames) continue;
		if (frameIndex != BSROPT_FRAME_URSHAPE) {
		   Vector3f pos = ps->get3f( frameIndex, patchIndex, POSITION );
		   T[0] = ps->get3f( frameIndex, patchIndex, REF_U );
		   T[2] = ps->get3f( frameIndex, patchIndex, REF_N );
		   T[1] = T[2].crossProduct(T[0]);
		   T = T.transpose();
         
         vector<card32> neighbors;
         context->priorsNeighborhood->collectNeighbors(frameIndex, patchIndex, neighbors);
         const card32 numNeighbors = neighbors.size();
         
		   corrX.clear(); corrY.clear(); corrZ.clear();
         for (card32 ni=0; ni<numNeighbors; ni++) {
			   const card32 j = neighbors[ni];
			   Vector3f localPos = T*(ps->get3f(frameIndex, j, POSITION) - pos);
			   Vector3f corr = ps->get3f(frameIndex + 1, j, POSITION)  - ps->get3f(frameIndex, j, POSITION);
			   localPos[2] = corr[0];
			   corrX.push_back(localPos);
			   localPos[2] = corr[1];
			   corrY.push_back(localPos);
			   localPos[2] = corr[2];
			   corrZ.push_back(localPos);
		   }
         //Vector3f coeffX, coeffY, coeffZ;
         StaticVector<float32, 6> coeffX, coeffY, coeffZ;

		   Monomials2ndOrder::computeCoeffs( corrX, coeffX );     /// todo: 2nd order
		   Monomials2ndOrder::computeCoeffs( corrY, coeffY );
		   Monomials2ndOrder::computeCoeffs( corrZ, coeffZ );
		   StaticVector<float32, 9> corrCoeff;
		   corrCoeff[0] = coeffX[0];
		   corrCoeff[1] = coeffX[1];
		   corrCoeff[2] = coeffX[2];
		   corrCoeff[3] = coeffY[0];
		   corrCoeff[4] = coeffY[1];
		   corrCoeff[5] = coeffY[2];
		   corrCoeff[6] = coeffZ[0];
		   corrCoeff[7] = coeffZ[1];
		   corrCoeff[8] = coeffZ[2];
		   ps->set<float32,9>( frameIndex, patchIndex, CORR_COEFF, corrCoeff );
	   } else {
      	pAssert(false);
      	// TODO: set urshape
      }
   }
}  */


void BSRFitting::fitRefFrames( BSROptContext *context, PatchGraph* pg, const BSROptVariableIndexSet &indices ) 
{
	InCorePCTopologyGraph *tg = pg->getTopology();
	PointSet *ps = pg->getTrajectorySet()->getTrajectories();
	PointSet *urshape = pg->getUrshape();
	const card32 numFrames = ps->getWidth();
	if (   !ps->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)
	    || !ps->getDescr()->providesAttribute("normal", 3, VAD::DATA_FORMAT_FLOAT32)
	    || !ps->getDescr()->providesAttribute("tangent_u", 3, VAD::DATA_FORMAT_FLOAT32)) 
	{
		throw PException("BSRFitting::fitRefFrames() - did not find all required channels.");
	}
	if (urshape && !ps->getDescr()->equal(urshape->getDescr())) {
	   throw PException("BSRFitting::fitRefFrames() - urshape and ps descriptors do not match.");
	}
	AAT POS = ps->getAAT("position");
	AAT NORMAL = ps->getAAT("normal");
	AAT TANGENT_U = ps->getAAT("tangent_u");

   const size_t numIndices = indices.size();
   for (unsigned index = 0; index<numIndices; index++) {
	   int32 frameIndex = indices[index].frame;
		card32 patchIndex = indices[index].point;

      if (frameIndex != BSROPT_FRAME_URSHAPE) {
		   RegressionPlane rp;
		   rp.clear();
		   vector<card32> neighbors;
         if (context->priorsNeighborhood->getPointSet() != ps) {
            context->priorsNeighborhood->setup(ps, tg);
         }
         context->priorsNeighborhood->collectNeighbors(frameIndex, patchIndex, neighbors);
         size_t numNeighbors = neighbors.size();
		   for (card32 j=0; j<numNeighbors; j++) {
			   rp.addPoint( ps->get3f( frameIndex, neighbors[j], POS ) );
		   }
		   rp.computePlane();

		   ReferenceFramePatch refPatch;
		   refPatch.normal = normalize(rp.getNormal());
		   refPatch.tangent_u = normalize(rp.getU());
		   refPatch.set(ps, frameIndex, patchIndex, NORMAL, TANGENT_U);
      } else {
		   RegressionPlane rp;
		   rp.clear();
		   vector<card32> neighbors;
         if (context->priorsNeighborhood->getPointSet() != urshape) {
            context->priorsNeighborhood->setup(urshape, tg);
         }
         context->priorsNeighborhood->collectNeighbors(0, patchIndex, neighbors);
         size_t numNeighbors = neighbors.size();
		   for (card32 j=0; j<numNeighbors; j++) {
			   rp.addPoint( urshape->get3f( 0, neighbors[j], POS ) );
		   }
		   rp.computePlane();

		   ReferenceFramePatch refPatch;
		   refPatch.normal = normalize(rp.getNormal());
		   refPatch.tangent_u = normalize(rp.getU());
		   refPatch.set(urshape, 0, patchIndex, NORMAL, TANGENT_U);
      }
	}
}


void BSRFitting::estSampleSpacing( BSROptContext *context, PatchGraph *pg, const BSROptVariableIndexSet &indices ) 
{
	InCorePCTopologyGraph *tg = pg->getTopology();
	PointSet *ps = pg->getTrajectorySet()->getTrajectories();
	PointSet *urshape = pg->getUrshape();
   const size_t numIndices = indices.size();
   if ( !ps->providesAttribute( "position" ) || !ps->providesAttribute( "sampleSpacing" ) ){
	   debugOutput << "estSampleSpacing: attribute channel missing.\n"; debugOutput.flush();
	   return;
   }
   AAT POS = ps->getAAT("position");
   AAT SAMPLE_SPACING = ps->getAAT("sampleSpacing");
	for (card32 i=0; i<numIndices; i++) {
	   int32 frameIndex = indices[i].frame;
		card32 patchIndex = indices[i].point;
      if (frameIndex != BSROPT_FRAME_URSHAPE) {
		   vector<card32> neighbors;
         if (context->priorsNeighborhood->getPointSet() != ps) {
            context->priorsNeighborhood->setup(ps, tg);
         }
         context->priorsNeighborhood->collectNeighbors(frameIndex, patchIndex, neighbors);
         
         Vector3f centerPos = ps->get3f(frameIndex, patchIndex, POS);
         size_t numNeighbors = neighbors.size();
         float32 dist=0;
         for (unsigned ni=0; ni<numNeighbors; ni++) {
            Vector3f pos = ps->get3f(frameIndex, neighbors[ni], POS);
            float32 d = norm(pos-centerPos);
            if (d>dist) dist = d;
         }

		   float32 sampleSpacing;
		   if (dist > 0) {
			   float32 area = dist*dist*M_PI;
			   sampleSpacing = sqrt(fabs(1.21*area/(M_PI*numNeighbors)));
			   /// correct for border points (not exact)
			   area += (dist*M_PI*2 / sampleSpacing)*sampleSpacing*sampleSpacing*M_PI/2;
			   sampleSpacing = sqrt(fabs(1.21*area/(M_PI*numNeighbors)));
		   } else {
			   sampleSpacing = 1E36f;
		   }
		   ps->set<float32>(frameIndex, patchIndex, SAMPLE_SPACING, sampleSpacing);
	   } else {
		   vector<card32> neighbors;
         if (context->priorsNeighborhood->getPointSet() != urshape) {
            context->priorsNeighborhood->setup(urshape, tg);
         }
         context->priorsNeighborhood->collectNeighbors(0, patchIndex, neighbors);
         
         Vector3f centerPos = urshape->get3f(0, patchIndex, POS);
         size_t numNeighbors = neighbors.size();
         float32 dist=0;
         for (unsigned ni=0; ni<numNeighbors; ni++) {
            Vector3f pos = urshape->get3f(0, neighbors[ni], POS);
            float32 d = norm(pos-centerPos);
            if (d>dist) dist = d;
         }

		   float32 sampleSpacing;
		   if (dist > 0) {
			   float32 area = dist*dist*M_PI;
			   sampleSpacing = sqrt(fabs(1.21*area/(M_PI*numNeighbors)));
			   /// correct for border points (not exact)
			   area += (dist*M_PI*2 / sampleSpacing)*sampleSpacing*sampleSpacing*M_PI/2;
			   sampleSpacing = sqrt(fabs(1.21*area/(M_PI*numNeighbors)));
		   } else {
			   sampleSpacing = 1E36f;
		   }
		   urshape->set<float32>(0, patchIndex, SAMPLE_SPACING, sampleSpacing);
	   }
   }
}