//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "BSROptConfig.h"
//---------------------------------------------------------------------------
#include "BSROptContext.h"
#include "ObjectClassProperty.h"
#include "NumericalClassProperty.h"
#include "BooleanClassProperty.h"
#include "SeparatorClassProperty.h"
#include "CopyObjectProperties.h"
#include "PatchGraphBSROpt.h"
#include "BSROptPotential.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



static string descrOptMode(card32 i) {
	if (i==0) return "MODE_PETERS_OPT";
	if (i==1) return "MODE_OLD_OPT";
	return "undefined";
} 

IMPLEMENT_GEOXL_CLASS( BSROptConfig ,0)
{
	BEGIN_CLASS_INIT( BSROptConfig );
	ADD_SEPARATOR("parameters")
	ADD_OBJECT_PROP(parameters, 0, BSROptContext::getClass(), true)
	ADD_CARD32_PROP_UPDATE_DESCR(optMode, 0, NULL, descrOptMode, 3)
	ADD_SEPARATOR("potentials (Peter's optimizer)")
	ADD_BOOLEAN_PROP(ccSquaredDistancePotential, 0)
	ADD_BOOLEAN_PROP(ccMarkerErrorPotential, 0)
	ADD_BOOLEAN_PROP(ccNormalSmoothnessPotential, 0)
	ADD_BOOLEAN_PROP(ccPlanarSmoothnessPotential, 0)
	ADD_BOOLEAN_PROP(ccCircularSmoothnessPotential, 0)
	ADD_BOOLEAN_PROP(ccLaplacianSmoothnessPotential, 0)
	ADD_BOOLEAN_PROP(ccPositionRigidityPotential,	0)
	ADD_BOOLEAN_PROP(ccNormalRigidityPotential,		0)
	ADD_BOOLEAN_PROP(ccSTPositionCoherence,		0)
	ADD_BOOLEAN_PROP(ccSTNormalCoherence,			0)
	ADD_BOOLEAN_PROP(ccSTRotationCoherence,		0)
	ADD_SEPARATOR("potentials (old code)")
	ADD_BOOLEAN_PROP(ccGaussianPrjLikelihood, 0)
	ADD_BOOLEAN_PROP(ccSimpleICPLikelihood, 0)
	ADD_BOOLEAN_PROP(ccPrjSmoothness, 0)
	ADD_BOOLEAN_PROP(ccSpringBasedRigidity, 0)
}

BSROptConfig::BSROptConfig() {
	parameters = new BSROptContext();
	ccGaussianPrjLikelihood					= false;
	ccSimpleICPLikelihood						= false;
	ccPrjSmoothness								= false;
	ccSpringBasedRigidity						= false;

	ccSquaredDistancePotential			= true;
	ccMarkerErrorPotential						= false;
	ccNormalSmoothnessPotential		= true;
	ccPlanarSmoothnessPotential			= true;
	ccCircularSmoothnessPotential		= false;
	ccLaplacianSmoothnessPotential	= false;
	ccPositionRigidityPotential				= true;
	ccNormalRigidityPotential				= true;
	ccSTPositionCoherence					= true;
	ccSTNormalCoherence						= false;
	ccSTRotationCoherence					= false;
	optMode												= MODE_PETERS_OPT;
}

BSROptConfig::~BSROptConfig() 
{
	delete parameters;
}

void BSROptConfig::assign(const Object* obj, CopyContext *context) {
	Persistent::assign(obj,context);
	const BSROptConfig * o = dynamic_cast<const BSROptConfig*>(obj);
	pAssert( o != NULL );
	copyObjectProperties(o, this);
}

void BSROptConfig::configure(PatchGraphBSROpt *opt) 
{
	opt->clearPotentials();	
	opt->potential_flags.clear();

	if (ccGaussianPrjLikelihood) 
	{
		opt->addPotential(new BOPGaussianPrjLikelihood());
	}
	if (ccSimpleICPLikelihood) 
	{
		opt->addPotential(new BOPSimpleICPLikelihood());
	}
	if (ccPrjSmoothness) 
	{
		opt->addPotential(new BOPPrjSmoothness());
	}
	if (ccSpringBasedRigidity) 
	{
		opt->addPotential(new BOPSpringBasedRigidity());
	}
	opt->potential_flags.push_back(ccSquaredDistancePotential);
	if (ccSquaredDistancePotential) 
	{
		opt->addPotential(new BOPSquaredDistancePotential());
	}
	opt->potential_flags.push_back(ccMarkerErrorPotential);
	if (ccMarkerErrorPotential) 
	{
		opt->addPotential(new BOPMarkerErrorPotential());
	}
	opt->potential_flags.push_back(ccPlanarSmoothnessPotential);
	if (ccPlanarSmoothnessPotential) 
	{
		opt->addPotential(new BOPPlanarSmoothnessPotential());
	}
	opt->potential_flags.push_back(ccNormalSmoothnessPotential);
	if (ccNormalSmoothnessPotential) 
	{
		opt->addPotential(new BOPNormalSmoothnessPotential());
	}
	opt->potential_flags.push_back(ccCircularSmoothnessPotential);
	if (ccCircularSmoothnessPotential) 
	{
		opt->addPotential(new BOPCircularSmoothnessPotential());
	}
	opt->potential_flags.push_back(ccLaplacianSmoothnessPotential);
	if (ccLaplacianSmoothnessPotential) 
	{
		opt->addPotential(new BOPLaplacianSmoothnessPotential());
	}
	opt->potential_flags.push_back(ccPositionRigidityPotential);
	if (ccPositionRigidityPotential)
	{
		opt->addPotential(new BOPPositionRigidityPotential());
	}
	opt->potential_flags.push_back(ccNormalRigidityPotential);
	if (ccNormalRigidityPotential) 
	{
		opt->addPotential(new BOPNormalRigidityPotential());
	}
	opt->potential_flags.push_back(ccSTPositionCoherence);
	if (ccSTPositionCoherence)
	{
		opt->addPotential(new BOPSTPositionCoherence());
	}
	opt->potential_flags.push_back(ccSTNormalCoherence);
	if (ccSTNormalCoherence) 
	{
		opt->addPotential(new BOPSTNormalCoherence());
	}
	opt->potential_flags.push_back(ccSTRotationCoherence);
	if (ccSTRotationCoherence) 
	{
		opt->addPotential(new BOPSTRotationCoherence());
	}
	opt->getContext()->assign(parameters);
}
