//---------------------------------------------------------------------------
#ifndef BSROptConfigH
#define BSROptConfigH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {
	class BSROptContext;
	class PatchGraphBSROpt;

/**
	configuration of PatchGraphBSROpt
*/
class ANIMATIONRECONSTRUCTION_API BSROptConfig : public Persistent {
	GEOXL_CLASS( BSROptConfig )

 public:
	static const card32 MODE_PETERS_OPT = 0;
	static const card32 MODE_OLD_OPT = 1;
 
 private:
	BSROptContext *parameters;	
	
	card32 optMode;	
	
	bool		ccGaussianPrjLikelihood;
	bool		ccSimpleICPLikelihood;
	bool		ccPrjSmoothness;
	bool		ccSpringBasedRigidity;
	bool		ccSquaredDistancePotential;
	bool		ccMarkerErrorPotential;
	bool		ccNormalSmoothnessPotential;
	bool		ccPlanarSmoothnessPotential;
	bool		ccCircularSmoothnessPotential;
	bool		ccLaplacianSmoothnessPotential;
	bool		ccPositionRigidityPotential;
	bool		ccNormalRigidityPotential;
	bool		ccSTPositionCoherence;
	bool		ccSTNormalCoherence;
	bool		ccSTRotationCoherence;

 public:
	BSROptConfig();
	~BSROptConfig();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	virtual void configure(PatchGraphBSROpt *opt);

	// --- member access
	
	/// which code path to use (this flat is not used in configure)
	card32 getOptMode() const {return optMode;}
	void setOptMode(card32 v) {optMode = v;}

	BSROptContext *getParameters() const {return parameters;}
	void setParameters(BSROptContext *v) {parameters = v;}

	bool getCCGaussianPrjLikelihood() const {return ccGaussianPrjLikelihood;}
	void setCCGaussianPrjLikelihood(bool v) {ccGaussianPrjLikelihood = v;}

	bool getCCSimpleICPLikelihood() const {return ccSimpleICPLikelihood;}
	void setCCSimpleICPLikelihood(bool v) {ccSimpleICPLikelihood = v;}

	bool getCCPrjSmoothness() const {return ccPrjSmoothness;}
	void setCCPrjSmoothness(bool v) {ccPrjSmoothness = v;}

	bool getCCSpringBasedRigidity() const {return ccSpringBasedRigidity;}
	void setCCSpringBasedRigidity(bool v) {ccSpringBasedRigidity = v;}

	bool getCCSquaredDistancePotential() const {return ccSquaredDistancePotential;}
	void setCCSquaredDistancePotential(bool v) {ccSquaredDistancePotential = v;}

	bool getCCMarkerErrorPotential() const {return ccMarkerErrorPotential;}
	void setCCMarkerErrorPotential(bool v) {ccMarkerErrorPotential = v;}

	bool getCCNormalSmoothnessPotential() const {return ccNormalSmoothnessPotential;}
	void setCCNormalSmoothnessPotential(bool v) {ccNormalSmoothnessPotential = v;}

	bool getCCPlanarSmoothnessPotential() const {return ccPlanarSmoothnessPotential;}
	void setCCPlanarSmoothnessPotential(bool v) {ccPlanarSmoothnessPotential = v;}

	bool getCCCircularSmoothnessPotential() const {return ccCircularSmoothnessPotential;}
	void setCCCircularSmoothnessPotential(bool v) {ccCircularSmoothnessPotential = v;}

	bool getCCLaplacianSmoothnessPotential() const {return ccLaplacianSmoothnessPotential;}
	void setCCLaplacianSmoothnessPotential(bool v) {ccLaplacianSmoothnessPotential =v;}

	bool getCCPositionRigidityPotential() const {return ccPositionRigidityPotential;}
	void setCCPositionRigidityPotential(bool v) {ccPositionRigidityPotential = v;}

	bool getCCNormalRigidityPotential() const {return ccNormalRigidityPotential;}
	void setCCNormalRigidityPotential(bool v) {ccNormalRigidityPotential = v;}

	bool getCCPositionCoherencePotential() const {return ccSTPositionCoherence;}
	void setCCPositionCoherencePotential(bool v) {ccSTPositionCoherence = v;}

	bool getCCNormalCoherencePotential() const {return ccSTNormalCoherence;}
	void setCCNormalCoherehcePotential(bool v) {ccSTNormalCoherence = v;}

	bool getCCRotationCoherencePotential() const {return ccSTRotationCoherence;}
	void setCCRotationCoherehcePotential(bool v) {ccSTRotationCoherence = v;}
};

}

#endif