//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "BSROptContext.h"
#include "NumericalClassProperty.h"
#include "SeparatorClassProperty.h"
#include "PGTopologicalNeighborhood.h"
#include "ObjectClassProperty.h"
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------



static string descrMode(card32 i) {
	if (i == 0) return "move points";
	if (i == 1) return "fit coefficients";
	return "invalid";
}

static string descrStop(card32 i) {
	if (i == 0) return "STOP_NUM_ITERATIONS";
	if (i == 1) return "STOP_CONV_GRAD_INFNORM";
	return "invalid";
}

static string descrSolver(card32 i) {
	if (i == 0) return "SOLVER_GRADIENT_DESCENT";
	if (i == 1) return "SOLVER_NEWTON_CG";
	if (i == 2) return "SOLVER_GAUSS_NEWTON_CG";
	if (i == 3) return "SOLVER_QUASI_NEWTON_CG";
	if (i == 4) return "SOLVER_CONJUGATE_GRADIENT";
	return "invalid";
}

static string descrMultiGrid(card32 i) {
	if (i == 0) return "MG_NO_MULTIGRID";
	if (i == 1) return "MG_INTERPOLATION";
	if (i == 2) return "MG_LONG_EDGES";
	return "invalid";
}

IMPLEMENT_GEOXL_CLASS( BSROptContext, 1 )
{
	BEGIN_CLASS_INIT( BSROptContext )
	ADD_SEPARATOR("numerics")
	ADD_CARD32_PROP_UPDATE_DESCR( optMode, 0, NULL, descrMode, 2 )
	ADD_CARD32_PROP_UPDATE_DESCR( solver, 0, NULL, descrSolver, 5 )
	ADD_CARD32_PROP_UPDATE_DESCR( stoppingCriterion, 0, NULL, descrStop, 2 )
	ADD_FLOAT32_PROP( stepSize,													0 )
	ADD_CARD32_PROP( numIterations,											0 )
	ADD_FLOAT32_PROP( convergenceEps,									0 )
	ADD_CARD32_PROP( coeffsUpdateFreq,									0 )
	ADD_CARD32_PROP( coordsUpdateFreq,									0 )
	ADD_CARD32_PROP_UPDATE_DESCR( multiGrid, 0, NULL, descrMultiGrid, 3 )
	ADD_FLOAT32_PROP( lowresSampleSpacing, 0 )
	ADD_FLOAT32_PROP( newtonRegularization, 0 )
	ADD_SEPARATOR("weights")
	ADD_FLOAT32_PROP( weightLikelihood,									0 )
	ADD_FLOAT32_PROP( weightSmooth,										0 )
	ADD_FLOAT32_PROP( weightICPLikelihood,								0 )
	ADD_FLOAT32_PROP( weightSimpleSpaceTime,								1 )
	ADD_FLOAT32_PROP( weightSimpleCenterLikelihood, 1 )
	ADD_FLOAT32_PROP( weightSmoothCurvature,						0 )
	ADD_FLOAT32_PROP( weightSTSmoothness,							0 )
	ADD_FLOAT32_PROP( weightSTAngleSmoothness,				0 )
	ADD_FLOAT32_PROP( weightCorrSmooth,								0 )
	ADD_FLOAT32_PROP( weightDensity,										0 )
	
	ADD_SEPARATOR("Weights of various potentials")
	ADD_FLOAT32_PROP( lambda,										0 )
	ADD_FLOAT32_PROP( pruneDist_smoothness,			0 )
	ADD_FLOAT32_PROP( tradeoff_sdf,					0	)
	ADD_FLOAT32_PROP( overlapping_rate,						0 )
	ADD_FLOAT32_PROP( weightSquaredDistance,			0 )
	ADD_FLOAT32_PROP( weightMarkerError,               0 )
	ADD_FLOAT32_PROP( weightPlanarSmoothness,		0 )
	ADD_FLOAT32_PROP( weightNormalSmoothness,		0 )
	ADD_FLOAT32_PROP( weightCircularSmoothness,    0 )
	ADD_FLOAT32_PROP( weightLaplacianSmoothness,	0 )
	ADD_FLOAT32_PROP( weightPositionRigidity,				0 )
	ADD_FLOAT32_PROP( weightNormalRigidity,				0 )
	ADD_FLOAT32_PROP( weightPositionCoherence,		0 )
	ADD_FLOAT32_PROP( weightNormalCoherence,			0 )
	ADD_FLOAT32_PROP( weightRotationCoherence,		0 )

	ADD_SEPARATOR("Parameters of line search")
	ADD_FLOAT32_PROP( minimumEnergyDecay,				0 )
	ADD_FLOAT32_PROP( minimumStepsize,						0 )

	ADD_SEPARATOR("NCG with Newton-Raphson and Fletcher-Reeves")
	ADD_CARD32_PROP( maxIterations_CG_NCG_NRFR,				0)
	ADD_CARD32_PROP( maxIterations_NR_NCG_NRFR,				0)
	ADD_FLOAT32_PROP( errorTolerance_CG_NCG_NRFR,			0)
	ADD_FLOAT32_PROP( errorTolerance_NR_NCG_NRFR,			0)

	ADD_SEPARATOR("parameters of Gauss-Newton method")
	ADD_CARD32_PROP( maxIterations_Gauss_Newton,				0)
	ADD_FLOAT32_PROP( errorTolerance_Gauss_Newton,			0)

	ADD_SEPARATOR("parameters of Newton method")
	ADD_CARD32_PROP( maxIterations_Newton,							0)
	ADD_FLOAT32_PROP( errorTolerance_Newton,						0)

	ADD_FLOAT32_PROP( weightMultires, 0 )
	ADD_FLOAT32_PROP( edgeLengthWeightExponent, 0 )
	ADD_SEPARATOR("flags")

	ADD_BOOLEAN_PROP( simpleLikelihood,									0)
	ADD_BOOLEAN_PROP( simpleSmoothness,								0)
	ADD_BOOLEAN_PROP( hardLikelihoodConstraint,						0)
	ADD_BOOLEAN_PROP( stSmoothnessForward,						0)
	ADD_BOOLEAN_PROP( stSmoothnessBackward,						0)
	ADD_BOOLEAN_PROP( stSmoothnessUrshapeToFrames,		0)
	ADD_BOOLEAN_PROP( stSmoothnessFramesToUrshape,		0)
	ADD_BOOLEAN_PROP( stCorrLinPrjSmoothing,							0)
	ADD_BOOLEAN_PROP( stPreserveAngles,									0)
	ADD_BOOLEAN_PROP( smoothPatchGraph,								0)
	ADD_BOOLEAN_PROP( smoothUrshape,									0)
	ADD_BOOLEAN_PROP( unifyNormals,											0)

	ADD_SEPARATOR("neighborhoods")
	ADD_OBJECT_PROP(likelihoodNeighborhood, 0, PGTopologicalNeighborhoodDataPts::getClass(), true)
	ADD_OBJECT_PROP(priorsNeighborhood, 0, PGTopologicalNeighborhood::getClass(), true)
	ADD_SEPARATOR("linear system solver")
	ADD_FLOAT32_PROP(cgStopRes, 0)
   ADD_CARD32_PROP(cgMaxNumIterations, 0)
   ADD_BOOLEAN_PROP(cgDebugging, 0)
	ADD_SEPARATOR("misc")
	ADD_BOOLEAN_PROP( verbose, 0 )
}

/**
 * Constructor
 */
BSROptContext::BSROptContext()
{
	optMode															= MODE_MOVE_POINTS;
	solver																= SOLVER_GAUSS_NEWTON_CG;
	stoppingCriterion											= STOP_CONV_GRAD_INFNORM;

	numIterations												= 10;
	convergenceEps											= 1e-3f;	
	stepSize															= 0.5f;
	newtonRegularization = 0.0f;
	
	weightSmooth												= 1.0f;
	weightSmoothCurvature								= 1.0f;
	weightLikelihood											= 1.0f;
	weightICPLikelihood										= 1.0f;
	weightSimpleSpaceTime								= 1.0f;
	weightSimpleCenterLikelihood = 1.0f;
	weightSTSmoothness									= 1.0f;
   weightSTAngleSmoothness							= 1.0f;
	weightCorrSmooth										= 1.0f;
	weightDensity												= 1.0f;
	weightMultires = 0.01f;
	edgeLengthWeightExponent = -2.0f;

	maxIterations_CG_NCG_NRFR						= 10;
	maxIterations_NR_NCG_NRFR						= 10;
	errorTolerance_CG_NCG_NRFR						= 1e-3f;
	errorTolerance_NR_NCG_NRFR						= 1e-3f;

	maxIterations_Gauss_Newton						= 20;
	errorTolerance_Gauss_Newton						=	1e-3f;
	maxIterations_Newton							=	20;
	errorTolerance_Newton							=	1e-3f;

	lambda											= 0.0f;
	pruneDist_smoothness							= 0.01f;
	tradeoff_sdf									= 0.5f;
	overlapping_rate								= 0.75f;
	weightSquaredDistance							= 1.0f;
	weightMarkerError								= 10.0f;
	weightPlanarSmoothness							= 1.0f;
	weightNormalSmoothness							= 0.1f;
	weightCircularSmoothness						= 1.0f;
	weightLaplacianSmoothness						= 1.0f;
	weightPositionRigidity							= 4.0f;
	weightNormalRigidity							= 0.4f;
	weightPositionCoherence							= 0.0f;
	weightNormalCoherence							= 0.001f;
	weightRotationCoherence							= 0.1f;

	minimumEnergyDecay						=  0.02f;
	minimumStepsize								=  1e-3f;		

	simpleLikelihood									= true;
	coeffsUpdateFreq								= 2;
	coordsUpdateFreq							= 10;
//	useMultiGrid										= false;
	lowresSampleSpacing						= 0.2f;
	likelihoodNeighborhood						= new TNDOneRing();
	priorsNeighborhood							= new PGTNKnn();
   stSmoothnessForward						= true;
   stSmoothnessBackward					= false;
	stSmoothnessUrshapeToFrames	= false;
	stSmoothnessFramesToUrshape	= false;
	stPreserveAngles								= false;
	stLengthWeighting							= false;
   stCorrLinPrjSmoothing									= true;
	hardLikelihoodConstraint								= false; 
	simpleSmoothness										= false;
   smoothPatchGraph										= true;
   smoothUrshape												= true;  	
   unifyNormals													= true;
	verbose															= false;
	cgStopRes														= 1E-4f;
   cgMaxNumIterations										= 100;
   cgDebugging													= false;

	usingFeatureValues										= false;
	numOfFeatureValues									=	0;
};

void BSROptContext::assign(const Object* obj, CopyContext *context) {
   copyObjectProperties(obj, this);
}

BSROptContext::~BSROptContext() {
   delete likelihoodNeighborhood;
   delete priorsNeighborhood;

	featureWeights.clear();
}