//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "BSROptPotential.h"
#include "PointSet_templCode.h"
#include "Monomials2ndOrder.h"
#include "TGNeighborhoodKNNSearch.h"
#include "BSROptContext.h"
#include "PointCloudAnimationSequence.h"
#include "PGDataPointIndices.h"
#include "InCorePCTopologyGraph.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra_templCode.h"
#include "PatchTypes.h"
#include "DynamicLinearAlgebraTools.h"
#include "DynamicLinearAlgebraTools_templCode.h"
#include "SparseLinearAlgebra.h"
#include "SparseLinearAlgebra_templCode.h"
#include "HFBFBivariateMonomials.h"
#include "PatchApprox3d.h"
#include "LeastSquaresSystemSetup.h"
#include "PGTopologicalNeighborhood.h"
#include "PointSetKNNQuery.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_ABSTRACT_CLASS( BSROptPotential, 0 )
{
	BEGIN_CLASS_INIT( BSROptPotential )
}

IMPLEMENT_GEOXL_CLASS( BOPGaussianPrjLikelihood, 0 )
{
	BEGIN_CLASS_INIT( BOPGaussianPrjLikelihood )
}

IMPLEMENT_GEOXL_CLASS( BOPSimpleICPLikelihood, 0 )
{
	BEGIN_CLASS_INIT( BOPSimpleICPLikelihood )
}		

IMPLEMENT_GEOXL_CLASS( BOPSimpleSpaceTime, 0 )
{
	BEGIN_CLASS_INIT( BOPSimpleSpaceTime )
}		

IMPLEMENT_GEOXL_CLASS( BOPSimpleCenterLikelihood, 0 )
{
	BEGIN_CLASS_INIT( BOPSimpleCenterLikelihood )
}

IMPLEMENT_GEOXL_CLASS( BOPPrjSmoothness, 0 )
{
	BEGIN_CLASS_INIT( BOPPrjSmoothness )
}

IMPLEMENT_GEOXL_CLASS( BOPSpringBasedRigidity, 0 )
{
	BEGIN_CLASS_INIT( BOPSpringBasedRigidity )
}

IMPLEMENT_GEOXL_CLASS( BOPMultiResPotential, 0 )
{
	BEGIN_CLASS_INIT( BOPMultiResPotential )
}

IMPLEMENT_GEOXL_CLASS( BOPDataLeastSquares, 0 )
{
	BEGIN_CLASS_INIT( BOPDataLeastSquares )
}

IMPLEMENT_GEOXL_CLASS(BOPSquaredDistancePotential, 0)
{
	BEGIN_CLASS_INIT(BOPSquaredDistancePotential);
}

IMPLEMENT_GEOXL_CLASS(BOPMarkerErrorPotential, 0)
{
	BEGIN_CLASS_INIT(BOPMarkerErrorPotential);
}

IMPLEMENT_GEOXL_CLASS(BOPCircularSmoothnessPotential, 0)
{
	BEGIN_CLASS_INIT(BOPCircularSmoothnessPotential)
}

IMPLEMENT_GEOXL_CLASS(BOPNormalSmoothnessPotential, 0)
{
	BEGIN_CLASS_INIT(BOPNormalSmoothnessPotential);
}

IMPLEMENT_GEOXL_CLASS(BOPPlanarSmoothnessPotential, 0)
{
	BEGIN_CLASS_INIT(BOPPlanarSmoothnessPotential);
}

IMPLEMENT_GEOXL_CLASS(BOPLaplacianSmoothnessPotential, 0)
{
	BEGIN_CLASS_INIT(BOPLaplacianSmoothnessPotential);
}

IMPLEMENT_GEOXL_CLASS(BOPPositionRigidityPotential, 0)
{
	BEGIN_CLASS_INIT(BOPPositionRigidityPotential);
}

IMPLEMENT_GEOXL_CLASS(BOPNormalRigidityPotential, 0)
{
	BEGIN_CLASS_INIT(BOPNormalRigidityPotential);
}

IMPLEMENT_GEOXL_CLASS(BOPSTPositionCoherence, 0)
{
	BEGIN_CLASS_INIT(BOPSTPositionCoherence);
}

IMPLEMENT_GEOXL_CLASS(BOPSTNormalCoherence, 0)
{
	BEGIN_CLASS_INIT(BOPSTNormalCoherence);
}

IMPLEMENT_GEOXL_CLASS(BOPSTRotationCoherence, 0)
{
	BEGIN_CLASS_INIT(BOPSTRotationCoherence);
}
// -------------------------------------------------------------------------------

static void scatterLocalPosConstraint(const Matrix3f &localQuad, const Vector3f &localLin, SparseMatrixF &quad, DVectorF &lin, card32 varIndex) {
   for (card32 y=0; y<3; y++) {
      lin[3*varIndex+y] += localLin[y];
      for (card32 x=0; x<3; x++) {
         quad[3*varIndex+y].addEntry(3*varIndex+x, localQuad[x][y]);
      }
   }
}

static void scatterTwoPointConstraint(
   const StaticMatrix<float32, 6,6> &localQuad, const StaticVector<float32, 6> &localLin, 
   SparseMatrixF &quad, DVectorF &lin, card32 varIndex1, card32 varIndex2) 
{
   // ([11] [21])
   // ([12] [22])
   //
   // this is [11]
   for (card32 y=0; y<3; y++) {
      lin[3*varIndex1+y] += localLin[y];
      for (card32 x=0; x<3; x++) {
         quad[3*varIndex1+y].addEntry(3*varIndex1+x, localQuad[x][y]);
      }
   }

   // this is [21]
   for (card32 y=0; y<3; y++) {
      for (card32 x=0; x<3; x++) {
         quad[3*varIndex1+y].addEntry(3*varIndex2+x, localQuad[x+3][y]);
      }
   }

   // this is [12]
   for (card32 y=0; y<3; y++) {
      for (card32 x=0; x<3; x++) {
         quad[3*varIndex2+y].addEntry(3*varIndex1+x, localQuad[x][y+3]);
      }
   }

   // this is [22]
   for (card32 y=0; y<3; y++) {
      lin[3*varIndex2+y] += localLin[y+3];
      for (card32 x=0; x<3; x++) {
         quad[3*varIndex2+y].addEntry(3*varIndex2+x, localQuad[x+3][y+3]);
      }
   }
}

// -------------------------------------------------------------------------------


///
/// BOPGaussianPrjLikelihood
///
/// Likelihood potentials that projects data point on approximating surface
///
card32 BOPGaussianPrjLikelihood::needPrecomputedChannels(BSROptContext *context) {
   if (context->simpleLikelihood) {
      return 0;
   } else {
      return FLAG_PATCH_REF_FRAMES | FLAG_NEED_PATCH_2ND_ORDER_HFIELD;
   }
}

void BOPGaussianPrjLikelihood::computeGradient(
   const BSROptContext *context,
   const BSROptVariableIndexSet *indices,
   const PointCloudAnimationSequence *data,
   PatchGraph *patchGraph,
   DVectorF &gradient,
   float32 &potential,
   DVectorF *perPointPotentials,
   SparseMatrixF *quadApprox,
   DVectorF *linApprox)
{
	PointSet * ps = patchGraph->getTrajectorySet()->getTrajectories();
	PGDataPointIndices *dataPoints = patchGraph->getTrajectorySet()->getDataPoints();
	AAT COEFF;
	if (!context->simpleLikelihood) COEFF = ps->getAAT( "hfieldCoeffs", 6, VAD::DATA_FORMAT_FLOAT32 );
	AAT REF_U = ps->getAAT( "tangent_u", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT REF_N = ps->getAAT( "normal", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT POSITION = ps->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT VARINDEX = ps->getAAT( "varIndex", 1, VAD::DATA_FORMAT_INT32 );
	AAT DATA_POS_AAT = data->getFrame(0)->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );
	
	Matrix3f T;
	
	for ( unsigned int n = 0; n < indices->size(); n++ ) {
		int32 frameIndex = (*indices)[n].frame;
		card32 patchIndex = (*indices)[n].point;
		if ( frameIndex != BSROPT_FRAME_URSHAPE ) {
   			int32 varIndex = ps->get<int32>(frameIndex, patchIndex, VARINDEX);
   			pAssert(varIndex >=0 && varIndex < (int32)gradient.getDim());
			StaticVector<float32,6> coeff;
			T[0] = ps->get<float32,3>( frameIndex, patchIndex, REF_U );
			T[2] = ps->get<float32,3>( frameIndex, patchIndex, REF_N );
			Vector3f worldSpaceN = T[2];
			T[1] = T[2].crossProduct(T[0]); // tangent_v = normal x tangent_u
			T = T.transpose();
			Vector3f pos = patchGraph->getTrajectorySet()->getTrajectories()->get<float32,3>( frameIndex, patchIndex, POSITION );

			const card32 numDataPoints = dataPoints->getNumDataPoints( frameIndex, patchIndex );
			Vector3f gradientVector = NULL_VECTOR3F;
			float32 potentialValue = 0;

			if (context->simpleLikelihood) {
				if (numDataPoints > 0) {
					card32 dataIndexInData = dataPoints->getIndex( frameIndex, patchIndex, 0);
					Vector3f dataPos = data->getFrame( frameIndex )->get3f(dataIndexInData, DATA_POS_AAT);
					if (context->hardLikelihoodConstraint) {
						gradientVector = NULL_VECTOR3F;
						ps->set3f(frameIndex, patchIndex, POSITION, dataPos);
					} else {
						gradientVector = (pos-dataPos) * context->weightLikelihood;
						potentialValue = normQuad(gradientVector);
						gradientVector *= 2.0f;
						if (quadApprox && linApprox) {
							Matrix3f quadLocal = makeMatrix3f(
								context->weightLikelihood, 0, 0,
								0, context->weightLikelihood, 0,
								0, 0, context->weightLikelihood);
							Vector3f linLocal = dataPos*(-2*context->weightLikelihood);
							scatterLocalPosConstraint(quadLocal, linLocal, *quadApprox, *linApprox, varIndex);
						}
					}
				}
			} else {
   				coeff = ps->get<float32,6>( frameIndex, patchIndex, COEFF );
   				float32 gradientLength = 0;
				for ( card32 dataIndex=0; dataIndex<numDataPoints; dataIndex++ ) {
					card32 dataIndexInData = dataPoints->getIndex(frameIndex, patchIndex, dataIndex);
					Vector3f dataPos = data->getFrame( frameIndex )->get3f( dataIndexInData, DATA_POS_AAT );
					Vector3f localPos = T*(dataPos-pos);
					float32 n_surface = Monomials2ndOrder::evaluate( coeff, makeVector2f(localPos[0], localPos[1]) );
					float32 n_projection = localPos[2];
	      			
					gradientLength += n_projection - n_surface;
					potentialValue += sqr(gradientLength) * context->weightLikelihood/numDataPoints;
					gradientLength *= 2.0f;
				}
   				gradientLength /= numDataPoints;
   				gradientVector = -worldSpaceN * (gradientLength * context->weightLikelihood);
				if (quadApprox || linApprox) {
					notImplemented();
				}
			}
			gradient[3*varIndex  ] += gradientVector[0];
			gradient[3*varIndex+1] += gradientVector[1];
			gradient[3*varIndex+2] += gradientVector[2];
			potential += potentialValue;
			if (perPointPotentials) (*perPointPotentials)[varIndex] += potentialValue;
		} 
	/// do not consider urshape - urshape has no data points (purely latent frame)
	}
}



card32 BOPSimpleICPLikelihood::needPrecomputedChannels(BSROptContext *context) {
	//return FLAG_PATCH_REF_FRAMES;	
	return 0;
}

void BOPSimpleICPLikelihood::computeGradient(
   const BSROptContext *context,
   const BSROptVariableIndexSet *indices,
   const PointCloudAnimationSequence *data,
   PatchGraph *patchGraph,
   DVectorF &gradient,
   float32 &potential,
   DVectorF *perPointPotentials,
   SparseMatrixF *quadApprox,
   DVectorF *linApprox)
{
	PointSet * ps = patchGraph->getTrajectorySet()->getTrajectories();
	PGDataPointIndices *dataPoints = patchGraph->getTrajectorySet()->getDataPoints();
	AAT POSITION = ps->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT REF_U = ps->getAAT( "tangent_u", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT REF_N = ps->getAAT( "normal", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT VARINDEX = ps->getAAT( "varIndex", 1, VAD::DATA_FORMAT_INT32 );
	AAT DATA_POS_AAT = data->getFrame(0)->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );
	
	float32 minDataDistance = 0;
	for ( unsigned int n = 0; n < indices->size(); n++ ) {
		int32 frameIndex = (*indices)[n].frame;
		int32 dataFrameIndex = frameIndex + patchGraph->getStartFrameIndex();
		card32 patchIndex = (*indices)[n].point;
		if ( frameIndex != BSROPT_FRAME_URSHAPE ) {
			if (dataPoints->getNumDataPoints(frameIndex, patchIndex) > 0) {
				Vector3f pos = ps->get3f(frameIndex, patchIndex, POSITION);
				Vector3f target = data->getFrame(dataFrameIndex)->get3f( dataPoints->getIndex(frameIndex, patchIndex, 0) , DATA_POS_AAT);
				float32 distance = norm(pos-target);
				if (distance > minDataDistance) minDataDistance = distance;
			}
		}
	}
	float32 icpWeight;
	if (minDataDistance > 1E-5) {
		icpWeight = max(1.0f/minDataDistance, context->weightLikelihood);
	} else {
		icpWeight = context->weightLikelihood;
	}
	
	for ( unsigned int n = 0; n < indices->size(); n++ ) {
		int32 frameIndex = (*indices)[n].frame;
		int32 dataFrameIndex = frameIndex + patchGraph->getStartFrameIndex();
		card32 patchIndex = (*indices)[n].point;

		if ( frameIndex != BSROPT_FRAME_URSHAPE ) {

   		int32 varIndex = ps->get<int32>(frameIndex, patchIndex, VARINDEX);
   		pAssert(varIndex >=0 && varIndex < (int32)gradient.getDim());
			Vector3f normal = ps->get<float32,3>( frameIndex, patchIndex, REF_N );
			Vector3f pos = patchGraph->getTrajectorySet()->getTrajectories()->get<float32,3>( frameIndex, patchIndex, POSITION );
			
			const PointSet *dataPoints = data->getFrame(dataFrameIndex);
			PointSetKNNQuery knn(dataPoints, ((PointCloudAnimationSequence*)data)->fetchOctree(dataFrameIndex), pos, DATA_POS_AAT);

			Vector3f gradientVector = NULL_VECTOR3F;
			float32 potentialValue = 0;

			bool found;
			card32 dataIndexInData;
			knn.getNextNearestPointIndex(dataIndexInData, found);
			Vector3f dataPos = makeVector3f( 0,0,0 );
         
			if (found) {
				dataPos = data->getFrame(dataFrameIndex)->get3f(dataIndexInData, DATA_POS_AAT);

				gradientVector = (pos-dataPos) * icpWeight * context->weightICPLikelihood;
				// project onto normal - point to plane distance
				gradientVector -= normal*(gradientVector*normal);
				potentialValue = normQuad(gradientVector);
				gradientVector *= 2.0f;
				if (quadApprox && linApprox) {
					Matrix3f quadLocal = makeMatrix3f(
						context->weightLikelihood, 0, 0,
						0, context->weightLikelihood, 0,
						0, 0, context->weightLikelihood);
					Vector3f linLocal = dataPos*(-2*context->weightLikelihood);
					scatterLocalPosConstraint(quadLocal, linLocal, *quadApprox, *linApprox, varIndex);
				}

				gradient[3*varIndex  ] += gradientVector[0];
				gradient[3*varIndex+1] += gradientVector[1];
				gradient[3*varIndex+2] += gradientVector[2];
				potential += potentialValue;
				if (perPointPotentials) (*perPointPotentials)[varIndex] += potentialValue;
			}
		} 
		/// do not consider urshape - urshape has no data points (purely latent frame)
	}
}

card32 BOPSimpleCenterLikelihood::needPrecomputedChannels(BSROptContext *context) 
{
	return 0;	
}

void BOPSimpleCenterLikelihood::computeGradient(
	const BSROptContext *context,
	const BSROptVariableIndexSet *indices,
	const PointCloudAnimationSequence *data,
	PatchGraph *patchGraph,
	DVectorF &gradient,
	float32 &potential,
	DVectorF *perPointPotentials,
	SparseMatrixF *quadApprox,
	DVectorF *linApprox)
{
	PointSet* ps = patchGraph->getTrajectorySet()->getTrajectories();
	AAT POSITION = ps->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT VARINDEX = ps->getAAT( "varIndex", 1, VAD::DATA_FORMAT_INT32 );
	AAT DATA_POSITION = data->getFrame( 0 )->getAAT( "position" );

	for ( unsigned int n = 0; n < indices->size(); n++ ) {
		int32 frameIndex = (*indices)[n].frame;
		int32 globalFrameIndex = frameIndex + patchGraph->getStartFrameIndex();
		card32 patchIndex = (*indices)[n].point;
		int32 varIndex = ps->get<int32>(frameIndex, patchIndex, VARINDEX);
		pAssert( varIndex >=0 && varIndex < (int32)gradient.getDim() );
		Vector3f gradientVector = makeVector3f( 0,0,0 );
		float32 potentialValue = 0;
		Vector3f pos = ps->get3f( frameIndex, patchIndex, POSITION );

		const PointSet *dataPoints = data->getFrame( globalFrameIndex );
		PointSetKNNQuery knn(dataPoints, ((PointCloudAnimationSequence*)data)->fetchOctree(globalFrameIndex), pos, DATA_POSITION);
		bool found;
		card32 dataIndexInData;

		if ( frameIndex != BSROPT_FRAME_URSHAPE ) {
			Vector3f centroid = makeVector3f( 0,0,0 );
			for ( unsigned int i = 0; i < 5; i++ ){
				knn.getNextNearestPointIndex( dataIndexInData, found );
				centroid += data->getFrame( globalFrameIndex )->get3f( dataIndexInData, DATA_POSITION );
			}
			centroid /= 5.0f;
			gradientVector = ( pos - centroid ); 
			potential += normQuad( gradientVector ) * context->weightSimpleCenterLikelihood;
			gradientVector *= context->weightSimpleCenterLikelihood;
			gradientVector *= 2;
			if (perPointPotentials) (*perPointPotentials)[varIndex] += normQuad(gradientVector)*context->weightSimpleCenterLikelihood;
		}

		gradient[3*varIndex  ] += gradientVector[0];
		gradient[3*varIndex+1] += gradientVector[1];
		gradient[3*varIndex+2] += gradientVector[2];
		potential += potentialValue;
	}
}

card32 BOPSimpleSpaceTime::needPrecomputedChannels(BSROptContext *context) 
{
	return 0;	
}

void BOPSimpleSpaceTime::computeGradient(
	const BSROptContext *context,
	const BSROptVariableIndexSet *indices,
	const PointCloudAnimationSequence *data,
	PatchGraph *patchGraph,
	DVectorF &gradient,
	float32 &potential,
	DVectorF *perPointPotentials,
	SparseMatrixF *quadApprox,
	DVectorF *linApprox)
{
	PointSet* ps = patchGraph->getTrajectorySet()->getTrajectories();
	AAT POSITION = ps->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT VARINDEX = ps->getAAT( "varIndex", 1, VAD::DATA_FORMAT_INT32 );

	for ( unsigned int n = 0; n < indices->size(); n++ ) {
		int32 frameIndex = (*indices)[n].frame;
		int32 dataFrameIndex = frameIndex + patchGraph->getStartFrameIndex();
		card32 patchIndex = (*indices)[n].point;
		int32 varIndex = ps->get<int32>(frameIndex, patchIndex, VARINDEX);
		pAssert( varIndex >=0 && varIndex < (int32)gradient.getDim() );
		Vector3f gradientVector = makeVector3f( 0,0,0 );
		float32 potentialValue = 0;

		if ( frameIndex != BSROPT_FRAME_URSHAPE ) {
			if ( ( frameIndex > 0 ) && ( frameIndex < int32( patchGraph->getTrajectorySet()->getNumFrames() - 1 ) ) ){
				/// (1) Prev. and next frame available: optimal position: average between position in last and in next frame
				Vector3f posLast = ps->get<float32,3>( frameIndex-1, patchIndex, POSITION );
				Vector3f pos = ps->get<float32,3>( frameIndex, patchIndex, POSITION );
				Vector3f posNext = ps->get<float32,3>( frameIndex+1, patchIndex, POSITION );
				Vector3f pOpt = ( posLast + posNext ) / 2.0f;
				gradientVector = pos - pOpt;
				potential += normQuad( gradientVector ) * context->weightSimpleSpaceTime;
				gradientVector *= context->weightSimpleSpaceTime;
				gradientVector *= 2;
				if (perPointPotentials) (*perPointPotentials)[varIndex] += normQuad(gradientVector)*context->weightSimpleSpaceTime;
			} /*else if ( frameIndex > 0 ){
				// (2) Prev. frame available only: apply average velocity as neighbors
				Vector3f v = makeVector3f( 0,0,0 );
				for ( unsigned int i = 0; i < patchGraph->getTopology()->getNumAdjacentVertices( patchIndex ); i++ ){
					card32 nIndex = patchGraph->getTopology()->getAdjacentVertex( patchIndex, i );
					Vector3f posLast = ps->get<float32,3>( frameIndex-1, nIndex, POSITION );
					Vector3f pos = ps->get<float32,3>( frameIndex, nIndex, POSITION );
					v = ( v * float32( i ) + ( pos - posLast ) ) / float32( i+1 );
				}
				Vector3f pos = ps->get<float32,3>( frameIndex, patchIndex, POSITION );
				Vector3f posLast = ps->get<float32,3>( frameIndex-1, patchIndex, POSITION );
				Vector3f pOpt = ( posLast + v );
				gradientVector = pos - pOpt;
				potential += normQuad( gradientVector ) * context->weightSimpleSpaceTime;
				gradientVector *= context->weightSimpleSpaceTime;
				gradientVector *= 2;
				if (perPointPotentials) (*perPointPotentials)[varIndex] += normQuad(gradientVector)*context->weightSimpleSpaceTime;
			} else if ( frameIndex < int32( patchGraph->getTrajectorySet()->getNumFrames() - 1 ) ){
				// (3) Next. frame available only: apply average velocity as neighbors
				Vector3f v = makeVector3f( 0,0,0 );
				for ( unsigned int i = 0; i < patchGraph->getTopology()->getNumAdjacentVertices( patchIndex ); i++ ){
					card32 nIndex = patchGraph->getTopology()->getAdjacentVertex( patchIndex, i );
					Vector3f posNext = ps->get<float32,3>( frameIndex+1, nIndex, POSITION );
					Vector3f pos = ps->get<float32,3>( frameIndex, nIndex, POSITION );
					v = ( v * float32( i ) + ( posNext - pos ) ) / float32( i+1 );
				}
				Vector3f pos = ps->get<float32,3>( frameIndex, patchIndex, POSITION );
				Vector3f posNext = ps->get<float32,3>( frameIndex+1, patchIndex, POSITION );
				Vector3f pOpt = ( posNext - v );
				gradientVector = pos - pOpt;
				potential += normQuad( gradientVector ) * context->weightSimpleSpaceTime;
				gradientVector *= context->weightSimpleSpaceTime;
				gradientVector *= 2;
				if (perPointPotentials) (*perPointPotentials)[varIndex] += normQuad(gradientVector)*context->weightSimpleSpaceTime;
			}*/
		}

		gradient[3*varIndex  ] += gradientVector[0];
		gradient[3*varIndex+1] += gradientVector[1];
		gradient[3*varIndex+2] += gradientVector[2];
		potential += potentialValue;
	}
}



///
/// BOPPrjSmoothness
///
/// Smoothing by projecting on local polynomials
///
card32 BOPPrjSmoothness::needPrecomputedChannels(BSROptContext *context) {
   if (context->simpleSmoothness) {
	   return FLAG_PATCH_REF_FRAMES | FLAG_NEED_PATCH_2ND_ORDER_HFIELD;
	} else {
		return FLAG_PATCH_REF_FRAMES;
	}
}

inline void BOPPrjSmoothness::computeSinglePotential(
   const BSROptContext *context,
   PointSet *ps,
   InCorePCTopologyGraph *tg,
   const card32 frameIndex,
   const card32 patchIndex,
   const AAT &POSITION,
   const AAT &VARINDEX, const AAT &COEFF,
   const AAT &REF_U, const AAT &REF_N,
   DVectorF &gradient,
   float32 &potential,
   DVectorF *perPointPotentials,
   SparseMatrixF *quadApprox,
   DVectorF *linApprox)
{
	if (context->simpleSmoothness) {
		StaticVector<float32,6> coeff;
		Matrix3f T;
		Vector3f posT, pos;
		Vector3f gradientVector = makeVector3f( 0,0,0 );
		int32 varIndex = ps->get<int32>(frameIndex, patchIndex, VARINDEX);
		pAssert(varIndex >=0 && varIndex < (int32)gradient.getDim());
		coeff = ps->get<float32,6>( frameIndex, patchIndex, COEFF );
		T[0] = ps->get<float32,3>( frameIndex, patchIndex, REF_U );
		T[2] = ps->get<float32,3>( frameIndex, patchIndex, REF_N );
		T[1] = T[2].crossProduct(T[0]); // tangent_v = normal x tangent_u
		pos = ps->get<float32,3>( frameIndex, patchIndex, POSITION );
		// Transform position into local reference frame
		float32 n = Monomials2ndOrder::evaluate( coeff, makeVector2f( 0,0 ) );
		posT = T * makeVector3f( 0,0,n ) + pos;
		gradientVector = pos - posT;
		potential += normQuad(gradientVector)*context->weightSmooth;
		gradientVector *= context->weightSmooth;
		gradientVector *= 2;
		gradient[3*varIndex  ] += gradientVector[0];
		gradient[3*varIndex+1] += gradientVector[1];
		gradient[3*varIndex+2] += gradientVector[2];
		if (perPointPotentials) (*perPointPotentials)[varIndex] += normQuad(gradientVector)*context->weightSmooth;
		if (quadApprox && linApprox) {
			Matrix3f sigma;
			// n * n^t (outer product)
			sigma[0] = T[2] * T[2][0];
			sigma[1] = T[2] * T[2][1];
			sigma[2] = T[2] * T[2][2];
			sigma *= context->weightSmooth;
			Vector3f b = sigma * (posT*(-2.0f));
			scatterLocalPosConstraint(sigma, b, *quadApprox, *linApprox, varIndex);
		}
	} else {
		Matrix3f localFrame;
		localFrame[0] = ps->get<float32,3>( frameIndex, patchIndex, REF_U );
		localFrame[2] = ps->get<float32,3>( frameIndex, patchIndex, REF_N );
		localFrame[1] = localFrame[2].crossProduct(localFrame[0]);
		Matrix3f convToLocalFrame = localFrame.transpose();
		Vector3f origin = ps->get<float32,3>( frameIndex, patchIndex, POSITION );

		if (context->priorsNeighborhood->getPointSet() != ps) {
			context->priorsNeighborhood->setup(ps, tg);
		}
		vector<card32> neighbors;
		context->priorsNeighborhood->collectNeighbors(frameIndex, patchIndex, neighbors);
		const size_t numN = neighbors.size();
		vector<Vector3f> points(neighbors.size());
		for (card32 i=0; i<numN; i++) {
			points[i] = convToLocalFrame*(ps->get3f(frameIndex, neighbors[i], POSITION) - origin);
		}

		StaticVector<float32, 6> coeffs;
		vector< StaticVector<float32, 6> > coeffsDerivs;
		Monomials2ndOrder::computeCoeffs(points, coeffs, coeffsDerivs);
	
		for (card32 i=0; i<numN; i++) {
			float32 dSmooth_di = (2.0f/(float32)numN) * (Monomials2ndOrder::evaluate(coeffs, NULL_VECTOR2F) - points[i][2])
																	* (Monomials2ndOrder::evaluate(coeffsDerivs[i], NULL_VECTOR2F) - 1);
			float32 dCurv_di = 2*coeffs[4]*coeffsDerivs[i][4]
			                 + 4*coeffs[3]*coeffsDerivs[i][3]
			                 + 4*coeffs[5]*coeffsDerivs[i][5];
			Vector3f grad = localFrame[2]*(dSmooth_di * context->weightSmooth + dCurv_di*context->weightSmoothCurvature);
			
			int32 varIndex_i = ps->get<int32>(frameIndex, patchIndex, VARINDEX);
			gradient[3*varIndex_i]   += grad[0];
			gradient[3*varIndex_i+1] += grad[1];
			gradient[3*varIndex_i+2] += grad[2];
			//for (card32 j=0; j<numN; j++) {
			//
			//}		
		}

		int32 varIndex = ps->get<int32>(frameIndex, patchIndex, VARINDEX);
		pAssert(varIndex >=0 && varIndex < (int32)gradient.getDim());
	
	}
}


void BOPPrjSmoothness::computeGradient( 
   const BSROptContext *context,
	const BSROptVariableIndexSet *indices,
	const PointCloudAnimationSequence *data,
	PatchGraph *patchGraph,
   DVectorF &gradient,
   float32 &potential,
   DVectorF *perPointPotentials,
   SparseMatrixF *quadApprox,
   DVectorF *linApprox)
{
	PointSet *ps = patchGraph->getTrajectorySet()->getTrajectories();
	PointSet *urshape = patchGraph->getUrshape();
	InCorePCTopologyGraph *tg = patchGraph->getTopology();
	AAT COEFF;
	if (context->simpleSmoothness) COEFF = ps->getAAT( "hfieldCoeffs" );
	AAT REF_U = ps->getAAT( "tangent_u" );
	AAT REF_N = ps->getAAT( "normal" );
	AAT POSITION = ps->getAAT( "position" );
	AAT VARINDEX = ps->getAAT( "varIndex", 1, VAD::DATA_FORMAT_INT32 );
	for ( unsigned int n = 0; n < indices->size(); n++ ){
		int32 frameIndex = (*indices)[n].frame;
		card32 patchIndex = (*indices)[n].point;
		if ( frameIndex != BSROPT_FRAME_URSHAPE ){
         if (context->smoothPatchGraph) {
            computeSinglePotential(
               context,
               ps, tg, frameIndex, patchIndex,
               POSITION, VARINDEX, COEFF, REF_U, REF_N,
               gradient, potential, perPointPotentials,
               quadApprox, linApprox
            );
         }
		} else {
         if (context->smoothUrshape && urshape) {
            computeSinglePotential(
               context,
               urshape, tg, 0, patchIndex,
               POSITION, VARINDEX, COEFF, REF_U, REF_N,
               gradient, potential, perPointPotentials,
               quadApprox, linApprox
            );
   		}
      }
	}
}


///
/// BOPSpringBasedRigidity
///
/// spatial-temporal rigidity prior, based on simple spring model
///
card32 BOPSpringBasedRigidity::needPrecomputedChannels(BSROptContext *context) {
   return FLAG_PATCH_REF_FRAMES;
}

static inline float32 calcSpringPotential(const Vector3f &as, const Vector3f &bs, const Vector3f &at, const Vector3f &bt) {
   float32 t1 = at[0]*at[0];
   float32 t4 = bt[0]*bt[0];
   float32 t5 = at[1]*at[1];
   float32 t8 = bt[1]*bt[1];
   float32 t9 = at[2]*at[2];
   float32 t12 = bt[2]*bt[2];
   float32 t14 = sqrt(fabs(t1-2.0f*at[0]*bt[0]+t4+t5-2.0f*at[1]*bt[1]+t8+t9-2.0f*at[2]*bt[2]+t12));
   float32 t15 = as[0]*as[0];
   float32 t18 = bs[0]*bs[0];
   float32 t19 = as[1]*as[1];
   float32 t22 = bs[1]*bs[1];
   float32 t23 = as[2]*as[2];
   float32 t26 = bs[2]*bs[2];
   float32 t28 = sqrt(fabs(t15-2.0f*as[0]*bs[0]+t18+t19-2.0f*as[1]*bs[1]+t22+t23-2.0f*as[2]*bs[2]+t26));
   return sqr(t14-t28);
}

static inline Vector3f calcSpringGradDAT(const Vector3f &as, const Vector3f &bs, const Vector3f &at, const Vector3f &bt) {
   float32 t1 = at[0]*at[0];
   float32 t4 = bt[0]*bt[0];
   float32 t5 = at[1]*at[1];
   float32 t8 = bt[1]*bt[1];
   float32 t9 = at[2]*at[2];
   float32 t12 = bt[2]*bt[2];
   float32 t14 = sqrt(fabs(t1-2.0f*at[0]*bt[0]+t4+t5-2.0f*at[1]*bt[1]+t8+t9-2.0f*at[2]*bt[2]+t12));
   float32 t15 = as[0]*as[0];
   float32 t18 = bs[0]*bs[0];
   float32 t19 = as[1]*as[1];
   float32 t22 = bs[1]*bs[1];
   float32 t23 = as[2]*as[2];
   float32 t26 = bs[2]*bs[2];
   float32 t28 = sqrt(fabs(t15-2.0f*as[0]*bs[0]+t18+t19-2.0f*as[1]*bs[1]+t22+t23-2.0f*as[2]*bs[2]+t26));
   float32 t31 = (t14-t28)/t14;
   Vector3f result;
   result[0] = 2.0f*t31*(at[0]-bt[0]);
   result[1] = 2.0f*t31*(at[1]-bt[1]);
   result[2] = 2.0f*t31*(at[2]-bt[2]);
   return result;
}
static inline StaticVector<float32, 6> calcSpringGradDatDbt(const Vector3f &as, const Vector3f &bs, const Vector3f &at, const Vector3f &bt) {
   float32 t1 = at[0]*at[0];
   float32 t4 = bt[0]*bt[0];
   float32 t5 = at[1]*at[1];
   float32 t8 = bt[1]*bt[1];
   float32 t9 = at[2]*at[2];
   float32 t12 = bt[2]*bt[2];
   float32 t14 = sqrt(fabs(t1-2.0f*at[0]*bt[0]+t4+t5-2.0f*at[1]*bt[1]+t8+t9-2.0f*at[2]*bt[2]+t12));
   float32 t15 = as[0]*as[0];
   float32 t18 = bs[0]*bs[0];
   float32 t19 = as[1]*as[1];
   float32 t22 = bs[1]*bs[1];
   float32 t23 = as[2]*as[2];
   float32 t26 = bs[2]*bs[2];
   float32 t28 = sqrt(fabs(t15-2.0f*as[0]*bs[0]+t18+t19-2.0f*as[1]*bs[1]+t22+t23-2.0f*as[2]*bs[2]+t26));
   float32 t31 = (t14-t28)/t14;
   float32 t32 = at[0]-bt[0];
   float32 t34 = at[1]-bt[1];
   float32 t36 = at[2]-bt[2];
   StaticVector<float32, 6> lengthScoreGrad_at_bt;
	lengthScoreGrad_at_bt[0] = 2.0f*t31*t32;
	lengthScoreGrad_at_bt[1] = 2.0f*t31*t34;
	lengthScoreGrad_at_bt[2] = 2.0f*t31*t36;
	lengthScoreGrad_at_bt[3] = -2.0f*t31*t32;
	lengthScoreGrad_at_bt[4] = -2.0f*t31*t34;
	lengthScoreGrad_at_bt[5] = -2.0f*t31*t36;
	return lengthScoreGrad_at_bt;
}



static inline StaticMatrix<float32, 6,6> calcSpringHessianDatDbt(const Vector3f &as, const Vector3f &bs, const Vector3f &at, const Vector3f &bt) {
   float32 t1 = at[0]*at[0];
   float32 t4 = bt[0]*bt[0];
   float32 t5 = at[1]*at[1];
   float32 t8 = bt[1]*bt[1];
   float32 t9 = at[2]*at[2];
   float32 t12 = bt[2]*bt[2];
   float32 t13 = t1-2.0f*at[0]*bt[0]+t4+t5-2.0f*at[1]*bt[1]+t8+t9-2.0f*at[2]*bt[2]+t12;
   float32 t14 = 1/t13;
   float32 t15 = at[0]-bt[0];
   float32 t16 = 4.0f*t15*t15;
   float32 t19 = sqrt(t13);
   float32 t20 = as[0]*as[0];
   float32 t23 = bs[0]*bs[0];
   float32 t24 = as[1]*as[1];
   float32 t27 = bs[1]*bs[1];
   float32 t28 = as[2]*as[2];
   float32 t31 = bs[2]*bs[2];
   float32 t33 = sqrt(fabs(t20-2.0f*as[0]*bs[0]+t23+t24-2.0f*as[1]*bs[1]+t27+t28-2.0f*as[2]*bs[2]+t31));
   float32 t34 = t19-t33;
   float32 t37 = t34/t19/t13;
   float32 t42 = 2.0f*t34/t19;
   float32 t43 = t14*t16/2.0f-t37*t16/2.0f+t42;
   float32 t44 = 2.0f*t14*t15;
   float32 t45 = at[1]-bt[1];
   float32 t48 = 4.0f*t37*t45*t15;
   float32 t49 = 2.0f*t44*t45-t48;
   float32 t50 = at[2]-bt[2];
   float32 t53 = 4.0f*t37*t50*t15;
   float32 t54 = 2.0f*t44*t50-t53;
   float32 t60 = -t44*t15+2.0f*t37*t15*t15-t42;
   float32 t63 = -4.0f*t37*t45*t15;
   float32 t64 = -2.0f*t44*t45-t63;
   float32 t67 = -4.0f*t37*t50*t15;
   float32 t68 = -2.0f*t44*t50-t67;
   float32 t69 = 4.0f*t45*t45;
   float32 t74 = t14*t69/2.0f-t37*t69/2.0f+t42;
   float32 t75 = 2.0f*t14*t45;
   float32 t78 = 4.0f*t37*t50*t45;
   float32 t79 = 2.0f*t75*t50-t78;
   float32 t81 = -2.0f*t75*t15-t63;
   float32 t87 = -t75*t45+2.0f*t37*t45*t45-t42;
   float32 t90 = -4.0f*t37*t50*t45;
   float32 t91 = -2.0f*t75*t50-t90;
   float32 t92 = 4.0f*t50*t50;
   float32 t97 = t14*t92/2.0f-t37*t92/2.0f+t42;
   float32 t98 = 2.0f*t14*t50;
   float32 t100 = -2.0f*t98*t15-t67;
   float32 t102 = -2.0f*t98*t45-t90;
   float32 t108 = -t98*t50+2.0f*t37*t50*t50-t42;
   float32 t109 = -2.0f*t14*t15;
   float32 t111 = -2.0f*t109*t45-t48;
   float32 t113 = -2.0f*t109*t50-t53;
   float32 t116 = 4.0f*t14*t45*t50-t78;
   StaticMatrix<float32, 6,6> lengthScoreHessian_at_bt;
   lengthScoreHessian_at_bt[0][0] = t43;
   lengthScoreHessian_at_bt[0][1] = t49/2.0f;
   lengthScoreHessian_at_bt[0][2] = t54/2.0f;
   lengthScoreHessian_at_bt[0][3] = t60;
   lengthScoreHessian_at_bt[0][4] = t64/2.0f;
   lengthScoreHessian_at_bt[0][5] = t68/2.0f;
   lengthScoreHessian_at_bt[1][0] = t49/2.0f;
   lengthScoreHessian_at_bt[1][1] = t74;
   lengthScoreHessian_at_bt[1][2] = t79/2.0f;
   lengthScoreHessian_at_bt[1][3] = t81/2.0f;
   lengthScoreHessian_at_bt[1][4] = t87;
   lengthScoreHessian_at_bt[1][5] = t91/2.0f;
   lengthScoreHessian_at_bt[2][0] = t54/2.0f;
   lengthScoreHessian_at_bt[2][1] = t79/2.0f;
   lengthScoreHessian_at_bt[2][2] = t97;
   lengthScoreHessian_at_bt[2][3] = t100/2.0f;
   lengthScoreHessian_at_bt[2][4] = t102/2.0f;
   lengthScoreHessian_at_bt[2][5] = t108;
   lengthScoreHessian_at_bt[3][0] = t60;
   lengthScoreHessian_at_bt[3][1] = t81/2.0f;
   lengthScoreHessian_at_bt[3][2] = t100/2.0f;
   lengthScoreHessian_at_bt[3][3] = t43;
   lengthScoreHessian_at_bt[3][4] = t111/2.0f;
   lengthScoreHessian_at_bt[3][5] = t113/2.0f;
   lengthScoreHessian_at_bt[4][0] = t64/2.0f;
   lengthScoreHessian_at_bt[4][1] = t87;
   lengthScoreHessian_at_bt[4][2] = t102/2.0f;
   lengthScoreHessian_at_bt[4][3] = t111/2.0f;
   lengthScoreHessian_at_bt[4][4] = t74;
   lengthScoreHessian_at_bt[4][5] = t116/2.0f;
   lengthScoreHessian_at_bt[5][0] = t68/2.0f;
   lengthScoreHessian_at_bt[5][1] = t91/2.0f;
   lengthScoreHessian_at_bt[5][2] = t108;
   lengthScoreHessian_at_bt[5][3] = t113/2.0f;
   lengthScoreHessian_at_bt[5][4] = t116/2.0f;
   lengthScoreHessian_at_bt[5][5] = t97;
   return lengthScoreHessian_at_bt;
}       


/// --- spring potentials


static inline float32 calcAngleSpringPotential(
   const Vector3f &s1, const Vector3f &s2, const Vector3f &s3, 
   const Vector3f &t1, const Vector3f &t2, const Vector3f &t3)
{
   float32 t4 = s2[0]*s3[0];
   float32 t5 = s2[0]*s2[0];
   float32 t7 = s1[0]*s2[0];
   float32 t8 = s2[1]*s3[1];
   float32 t9 = s2[1]*s2[1];
   float32 t11 = s1[1]*s2[1];
   float32 t12 = s2[2]*s3[2];
   float32 t13 = s2[2]*s2[2];
   float32 t15 = s1[2]*s2[2];
   float32 t16 = t4-t5-s1[0]*s3[0]+t7+t8-t9-s1[1]*s3[1]+t11+t12-t13-s1[2]*s3[2]+t15;
   float32 t17 = t16*t16;
   float32 t19 = s1[0]*s1[0];
   float32 t21 = s1[1]*s1[1];
   float32 t23 = s1[2]*s1[2];
   float32 t27 = s3[0]*s3[0];
   float32 t29 = s3[1]*s3[1];
   float32 t31 = s3[2]*s3[2];
   float32 t37 = sqrt(1.0f-t17/(t5-2.0f*t7+t19+t9-2.0f*t11+t21+t13-2.0f*t15+t23)/(t27-2.0f*t4+t5+t29-2.0f*t8+t9+t31-2.0f*t12+t13));
   float32 t38 = t2[0]*t3[0];
   float32 t39 = t2[0]*t2[0];
   float32 t41 = t1[0]*t2[0];
   float32 t42 = t2[1]*t3[1];
   float32 t43 = t2[1]*t2[1];
   float32 t45 = t1[1]*t2[1];
   float32 t46 = t2[2]*t3[2];
   float32 t47 = t2[2]*t2[2];
   float32 t49 = t1[2]*t2[2];
   float32 t50 = t38-t39-t1[0]*t3[0]+t41+t42-t43-t1[1]*t3[1]+t45+t46-t47-t1[2]*t3[2]+t49;
   float32 t51 = t50*t50;
   float32 t53 = t1[0]*t1[0];
   float32 t55 = t1[1]*t1[1];
   float32 t57 = t1[2]*t1[2];
   float32 t61 = t3[0]*t3[0];
   float32 t63 = t3[1]*t3[1];
   float32 t65 = t3[2]*t3[2];
   float32 t71 = sqrt(1.0f-t51/(t39-2.0f*t41+t53+t43-2.0f*t45+t55+t47-2.0f*t49+t57)/(t61-2.0f*t38+t39+t63-2.0f*t42+t43+t65-2.0f*t46+t47));
   return sqr(t37-t71);
}

static inline Vector3f calcAngleSpringGradDT3(
   const Vector3f &s1, const Vector3f &s2, const Vector3f &s3, 
   const Vector3f &t1, const Vector3f &t2, const Vector3f &t3)
{
   float32 t4 = s2[0]*s3[0];
   float32 t5 = s2[0]*s2[0];
   float32 t7 = s1[0]*s2[0];
   float32 t8 = s2[1]*s3[1];
   float32 t9 = s2[1]*s2[1];
   float32 t11 = s1[1]*s2[1];
   float32 t12 = s2[2]*s3[2];
   float32 t13 = s2[2]*s2[2];
   float32 t15 = s1[2]*s2[2];
   float32 t16 = t4-t5-s1[0]*s3[0]+t7+t8-t9-s1[1]*s3[1]+t11+t12-t13-s1[2]*s3[2]+t15;
   float32 t17 = t16*t16;
   float32 t19 = s1[0]*s1[0];
   float32 t21 = s1[1]*s1[1];
   float32 t23 = s1[2]*s1[2];
   float32 t27 = s3[0]*s3[0];
   float32 t29 = s3[1]*s3[1];
   float32 t31 = s3[2]*s3[2];
   float32 t37 = sqrt(1.0f-t17/(t5-2.0f*t7+t19+t9-2.0f*t11+t21+t13-2.0f*t15+t23)/(t27-2.0f*t4+t5+t29-2.0f*t8+t9+t31-2.0f*t12+t13));
   float32 t38 = t2[0]*t3[0];
   float32 t39 = t2[0]*t2[0];
   float32 t41 = t1[0]*t2[0];
   float32 t42 = t2[1]*t3[1];
   float32 t43 = t2[1]*t2[1];
   float32 t45 = t1[1]*t2[1];
   float32 t46 = t2[2]*t3[2];
   float32 t47 = t2[2]*t2[2];
   float32 t49 = t1[2]*t2[2];
   float32 t50 = t38-t39-t1[0]*t3[0]+t41+t42-t43-t1[1]*t3[1]+t45+t46-t47-t1[2]*t3[2]+t49;
   float32 t51 = t50*t50;
   float32 t53 = t1[0]*t1[0];
   float32 t55 = t1[1]*t1[1];
   float32 t57 = t1[2]*t1[2];
   float32 t59 = 1/(t39-2.0f*t41+t53+t43-2.0f*t45+t55+t47-2.0f*t49+t57);
   float32 t60 = t51*t59;
   float32 t61 = t3[0]*t3[0];
   float32 t63 = t3[1]*t3[1];
   float32 t65 = t3[2]*t3[2];
   float32 t67 = t61-2.0f*t38+t39+t63-2.0f*t42+t43+t65-2.0f*t46+t47;
   float32 t68 = 1/t67;
   float32 t71 = sqrt(1.0f-t60*t68);
   float32 t74 = (t37-t71)/t71;
   float32 t75 = t50*t59;
   float32 t80 = t67*t67;
   float32 t81 = 1/t80;
   Vector3f result;
   result[0] = -t74*(-2.0f*t75*t68*(t2[0]-t1[0])+2.0f*t60*t81*(t3[0]-t2[0]));
   result[1] = -t74*(-2.0f*t75*t68*(t2[1]-t1[1])+2.0f*t60*t81*(t3[1]-t2[1]));
   result[2] = -t74*(-2.0f*t75*t68*(t2[2]-t1[2])+2.0f*t60*t81*(t3[2]-t2[2]));
   return result;
}

static inline Matrix3f calcAngleSpringHessianDT3(
   const Vector3f &s1, const Vector3f &s2, const Vector3f &s3, 
   const Vector3f &t1, const Vector3f &t2, const Vector3f &t3)
{
   float32 t4 = t2[0]*t3[0];
   float32 t5 = t2[0]*t2[0];
   float32 t7 = t1[0]*t2[0];
   float32 t8 = t2[1]*t3[1];
   float32 t9 = t2[1]*t2[1];
   float32 t11 = t1[1]*t2[1];
   float32 t12 = t2[2]*t3[2];
   float32 t13 = t2[2]*t2[2];
   float32 t15 = t1[2]*t2[2];
   float32 t16 = t4-t5-t1[0]*t3[0]+t7+t8-t9-t1[1]*t3[1]+t11+t12-t13-t1[2]*t3[2]+t15;
   float32 t17 = t16*t16;
   float32 t19 = t1[0]*t1[0];
   float32 t21 = t1[1]*t1[1];
   float32 t23 = t1[2]*t1[2];
   float32 t25 = 1/(t5-2.0*t7+t19+t9-2.0*t11+t21+t13-2.0*t15+t23);
   float32 t26 = t17*t25;
   float32 t27 = t3[0]*t3[0];
   float32 t29 = t3[1]*t3[1];
   float32 t31 = t3[2]*t3[2];
   float32 t33 = t27-2.0*t4+t5+t29-2.0*t8+t9+t31-2.0*t12+t13;
   float32 t34 = 1/t33;
   float32 t36 = 1.0-t26*t34;
   float32 t37 = 1/t36;
   float32 t38 = t16*t25;
   float32 t39 = t2[0]-t1[0];
   float32 t43 = t33*t33;
   float32 t44 = 1/t43;
   float32 t45 = t3[0]-t2[0];
   float32 t48 = -2.0*t38*t34*t39+2.0*t26*t44*t45;
   float32 t49 = t48*t48;
   float32 t52 = s2[0]*s3[0];
   float32 t53 = s2[0]*s2[0];
   float32 t55 = s1[0]*s2[0];
   float32 t56 = s2[1]*s3[1];
   float32 t57 = s2[1]*s2[1];
   float32 t59 = s1[1]*s2[1];
   float32 t60 = s2[2]*s3[2];
   float32 t61 = s2[2]*s2[2];
   float32 t63 = s1[2]*s2[2];
   float32 t64 = t52-t53-s1[0]*s3[0]+t55+t56-t57-s1[1]*s3[1]+t59+t60-t61-s1[2]*s3[2]+t63;
   float32 t65 = t64*t64;
   float32 t67 = s1[0]*s1[0];
   float32 t69 = s1[1]*s1[1];
   float32 t71 = s1[2]*s1[2];
   float32 t75 = s3[0]*s3[0];
   float32 t77 = s3[1]*s3[1];
   float32 t79 = s3[2]*s3[2];
   float32 t85 = sqrt(1.0-t65/(t53-2.0*t55+t67+t57-2.0*t59+t69+t61-2.0*t63+t71)/(t75-2.0*t52+t53+t77-2.0*t56+t57+t79-2.0*t60+t61));
   float32 t86 = sqrt(t36);
   float32 t87 = t85-t86;
   float32 t90 = t87/t86/t36;
   float32 t94 = t87/t86;
   float32 t95 = t39*t39;
   float32 t104 = 1/t43/t33;
   float32 t110 = 2.0*t26*t44;
   float32 t114 = t37*t48;
   float32 t115 = t2[1]-t1[1];
   float32 t116 = t34*t115;
   float32 t119 = t3[1]-t2[1];
   float32 t120 = 2.0*t44*t119;
   float32 t122 = -2.0*t38*t116+t26*t120;
   float32 t128 = t39*t25;
   float32 t130 = t44*t115;
   float32 t140 = t114*t122/2.0+t90*t122*t48/2.0-2.0*t94*(-t128*t116+2.0*t38*t130*t45+t38*t120*t39-4.0*t26*t104*t119*t45);
   float32 t141 = t2[2]-t1[2];
   float32 t142 = t34*t141;
   float32 t145 = t3[2]-t2[2];
   float32 t146 = 2.0*t44*t145;
   float32 t148 = -2.0*t38*t142+t26*t146;
   float32 t155 = t44*t141;
   float32 t160 = 2.0*t104*t145;
   float32 t165 = t114*t148/2.0+t90*t148*t48/2.0-2.0*t94*(-t128*t142+2.0*t38*t155*t45+t38*t146*t39-2.0*t26*t160*t45);
   float32 t166 = t122*t122;
   float32 t171 = t115*t115;
   float32 t201 = t37*t122*t148/2.0+t90*t148*t122/2.0-2.0*t94*(-t115*t25*t142+2.0*t38*t155*t119+t38*t146*t115-2.0*t26*t160*t119);
   float32 t202 = t148*t148;
   float32 t207 = t141*t141;
   Matrix3f result;
   result[0][0] = t37*t49/2.0+t90*t49/2.0-t94*(-2.0*t95*t25*t34+8.0*t38*t44*t39*t45-8.0*t26*t104*t45*t45+t110);
   result[0][1] = t140;
   result[0][2] = t165;
   result[1][0] = t140;
   result[1][1] = t37*t166/2.0+t90*t166/2.0-t94*(-2.0*t171*t25*t34+8.0*t38*t130*t119-8.0*t26*t104*t119*t119+t110);
   result[1][2] = t201;
   result[2][0] = t165;
   result[2][1] = t201;
   result[2][2] = t37*t202/2.0+t90*t202/2.0-t94*(-2.0*t207*t25*t34+8.0*t38*t155*t145-8.0*t26*t104*t145*t145+t110);
   return result;
}

inline void BOPSpringBasedRigidity::computeSinglePotential
  (const BSROptContext *context,
   const AAT &REF_U,
	const AAT &REF_N,
	const AAT &POSITION,
	const AAT &VARINDEX,
	PointSet *sourcePS,
	PointSet *targetPS,
	InCorePCTopologyGraph *tg,
	const card32 sourceFrame,
	const card32 targetFrame,
	const card32 patchIndex,	
   DVectorF &gradient,
   float32 &potential,
   DVectorF *perPointPotentials,
   SparseMatrixF *quadApprox,
   DVectorF *linApprox,
   float32 generalLinkWeight)
{
   int32 varIndex = targetPS->get<int32>(targetFrame, patchIndex, VARINDEX);
   pAssert(varIndex >=0 && varIndex < (int32)gradient.getDim());

   vector<card32> neighbors;
   if (context->priorsNeighborhood->getPointSet() != sourcePS) 
	{
      context->priorsNeighborhood->setup(sourcePS, tg);
   }
   context->priorsNeighborhood->collectNeighbors(sourceFrame, patchIndex, neighbors);
   const size_t numN = neighbors.size();
   
   Vector3f gradientVector = NULL_VECTOR3F;

   // --- edge length preservation
   Vector3f centerSource = sourcePS->get3f(sourceFrame, patchIndex, POSITION);
   Vector3f centerTarget = targetPS->get3f(targetFrame, patchIndex, POSITION);
   Vector3f avDisplacement = NULL_VECTOR3F;
   float32 springPot = 0;
   
   for (card32 ni=0; ni<numN; ni++) 
	{
      Vector3f neighborSource = sourcePS->get3f(sourceFrame, neighbors[ni], POSITION);
      float32 lengthSource = norm(centerSource - neighborSource);
      Vector3f neighborTarget = targetPS->get3f(targetFrame, neighbors[ni], POSITION);
      float32 lengthTarget = norm(centerTarget - neighborTarget);
      if (lengthTarget > 0) 
		{
			float32 variableWeight = 1.0f;
			if (context->stLengthWeighting && lengthSource > 0) {
				variableWeight = pow(lengthSource, context->edgeLengthWeightExponent);
			}
         springPot += calcSpringPotential(centerSource, neighborSource, centerTarget, neighborTarget);
         gradientVector += calcSpringGradDAT(centerSource, neighborSource, centerTarget, neighborTarget)*(context->weightSTSmoothness*variableWeight);
         if (quadApprox && linApprox) 
			{
            StaticVector<float32, 6> x0;
            x0[0] = centerTarget[0]; x0[1] = centerTarget[1]; x0[2] = centerTarget[2];
            x0[3] = neighborTarget[0]; x0[4] = neighborTarget[1]; x0[5] = neighborTarget[2];
            StaticVector<float32, 6> jacobian = calcSpringGradDatDbt(centerSource, neighborSource, centerTarget, neighborTarget)*(context->weightSTSmoothness*generalLinkWeight*variableWeight);
            StaticMatrix<float32, 6,6> hessian = calcSpringHessianDatDbt(centerSource, neighborSource, centerTarget, neighborTarget)*(context->weightSTSmoothness*generalLinkWeight*variableWeight);
            StaticVector<float32, 6> localLin = jacobian - ((hessian*x0)*2.0f);
            int32 varIndexTargetCenter = targetPS->get<int32>(targetFrame, patchIndex, VARINDEX);
            int32 varIndexTargetNeighbor = targetPS->get<int32>(targetFrame, neighbors[ni], VARINDEX);
            scatterTwoPointConstraint(hessian, localLin, *quadApprox, *linApprox, varIndexTargetCenter, varIndexTargetNeighbor);
         }
      }
      
		avDisplacement += neighborTarget - neighborSource;
      potential += springPot;
      
		if (perPointPotentials) 
			(*perPointPotentials)[varIndex] += springPot;
      if (numN > 0) 
			gradientVector /= numN;
      
      // --- linear projection smoothing (evil hack)
      if (context->stCorrLinPrjSmoothing && !quadApprox) 
		{
		   vector<AffineMapping3dFit::MapPoint> inputCorr;
		   for (card32 ni=0; ni<numN; ni++) 
			{
			   const card32 j = neighbors[ni];
			   Vector3f posSource = sourcePS->get3f(sourceFrame, j, POSITION);
			   Vector3f posTarget = targetPS->get3f(targetFrame, j, POSITION);
			   AffineMapping3dFit::MapPoint mp;
			   mp.from = posSource;
			   mp.to = posTarget;
			   inputCorr.push_back(mp);						
		   }

         AffineMapping3dFit::AffineMap lsf;
		   AffineMapping3dFit::computeCoeffs(inputCorr, lsf, 1E-5f); /// todo: set adaptive threshold with sample spacing

		   Vector3f av = lsf.b + lsf.M*sourcePS->get3f(sourceFrame, patchIndex, POSITION);

         Vector3f linSmoothGrad = (targetPS->get3f(targetFrame, patchIndex, POSITION) - av)*context->weightCorrSmooth;
         potential += normQuad(linSmoothGrad);
         gradientVector += linSmoothGrad*2.0f;

         if (perPointPotentials) 
				(*perPointPotentials)[varIndex] += normQuad(linSmoothGrad);
      }
   }

   if (context->stPreserveAngles) 
	{
      for (card32 ni=0; ni<numN; ni++) 
		{
         for (card32 nj=ni; nj<numN; nj++) 
			{
            Vector3f s1 = sourcePS->get3f(sourceFrame, neighbors[ni], POSITION);
            Vector3f s2 = sourcePS->get3f(sourceFrame, patchIndex, POSITION);
            Vector3f s3 = sourcePS->get3f(sourceFrame, neighbors[nj], POSITION);

            Vector3f t1 = targetPS->get3f(targetFrame, neighbors[ni], POSITION);
            Vector3f t2 = targetPS->get3f(targetFrame, patchIndex, POSITION);
            Vector3f t3 = targetPS->get3f(targetFrame, neighbors[nj], POSITION);

            springPot += calcAngleSpringPotential(s1, s2, s3, t1, t2, t3);
            Vector3f gradientAngle = calcAngleSpringGradDT3(s1, s2, s3, t1, t2, t3)*(context->weightSTSmoothness/(numN*numN));
            int32 varIndexT3 = targetPS->get<int32>(targetFrame, neighbors[nj], VARINDEX);
            gradient[varIndexT3*3  ] += gradientAngle[0]*generalLinkWeight;
            gradient[varIndexT3*3+1] += gradientAngle[1]*generalLinkWeight;
            gradient[varIndexT3*3+2] += gradientAngle[2]*generalLinkWeight;
            
            if (quadApprox && linApprox) 
				{
               Vector3f jacobian = gradientAngle*generalLinkWeight;
               Matrix3f hessian = calcAngleSpringHessianDT3(s1,s2,s3, t1,t2,t3);
               Vector3f localLin = jacobian - ((hessian*t3)*2.0f);
               scatterLocalPosConstraint(hessian, localLin, *quadApprox, *linApprox, varIndexT3);
            }
         }
      }
   }
      
   // --- sum up gradients
   gradient[varIndex*3  ] += gradientVector[0]*generalLinkWeight;
   gradient[varIndex*3+1] += gradientVector[1]*generalLinkWeight;
   gradient[varIndex*3+2] += gradientVector[2]*generalLinkWeight;
}


void BOPSpringBasedRigidity::computeGradient(
	const BSROptContext *context,
	const BSROptVariableIndexSet *indices,
	const PointCloudAnimationSequence *data,
	PatchGraph *patchGraph,
   DVectorF &gradient,
   float32 &potential,
   DVectorF *perPointPotentials,
   SparseMatrixF *quadApprox,
   DVectorF *linApprox)
{
	PointSet *ps = patchGraph->getTrajectorySet()->getTrajectories();
	PointSet *urshape = patchGraph->getUrshape();
	InCorePCTopologyGraph *tg = patchGraph->getTopology();
	AAT REF_U = ps->getAAT( "tangent_u" );
	AAT REF_N = ps->getAAT( "normal" );
	AAT POSITION = ps->getAAT( "position" );
	AAT VARINDEX = ps->getAAT( "varIndex", 1, VAD::DATA_FORMAT_INT32 );
	const int32 numFrames = ps->getWidth();
	if (numFrames == 0) 
		return;
	
	for ( unsigned int n = 0; n < indices->size(); n++ ) 
	{
		card32 patchIndex = (*indices)[n].point;
	   int32 frameIndex = (*indices)[n].frame;
	   int32 sourceFrame;
	   int32 targetFrame;
      if ( frameIndex != BSROPT_FRAME_URSHAPE ) 
		{
         for (int direction=0; direction<2; direction++) 
			{
            if (direction==0) 
				{
               sourceFrame = frameIndex+1;
               targetFrame = frameIndex;
               if (!context->stSmoothnessBackward || sourceFrame >= numFrames) 
						continue;
            } 
				else 
				{
               sourceFrame = frameIndex-1;
               targetFrame = frameIndex;
               if (!context->stSmoothnessForward || sourceFrame<0) 
						continue;
            }
            computeSinglePotential(
               context, REF_U, REF_N, POSITION, VARINDEX,
	            ps, ps, tg, sourceFrame, targetFrame, patchIndex,
               gradient, potential, perPointPotentials, quadApprox, linApprox
            );
         }
         if (urshape && context->stSmoothnessUrshapeToFrames) 
			{
            computeSinglePotential(
               context, REF_U, REF_N, POSITION, VARINDEX,
               urshape, ps, tg, 0, frameIndex, patchIndex,
               gradient, potential, perPointPotentials, quadApprox, linApprox
            );
         }
		} 
		else 
		{
		   if (urshape && context->stSmoothnessFramesToUrshape) 
			{
            /// all frames affect urshape
            for (int32 frameToUS=0; frameToUS<numFrames; frameToUS++) 
				{ 
               computeSinglePotential(
                  context, REF_U, REF_N, POSITION, VARINDEX,
                  ps, urshape, tg, frameToUS, 0, patchIndex,
                  gradient, potential, perPointPotentials,
                  quadApprox, linApprox, 1.0f/(float32)numFrames
               );
            }
         }
		}
	}
}

card32 BOPMultiResPotential::needPrecomputedChannels(BSROptContext *context)
{
	return 0;
}
	
void BOPMultiResPotential::computeGradient
  (const BSROptContext *context,
	const BSROptVariableIndexSet *indices,
	const PointCloudAnimationSequence *data,
	PatchGraph *patchGraph,
	DVectorF &gradient,
	float32 &potential,
	DVectorF *perPointPotentials,
	SparseMatrixF *quadApprox,
	DVectorF *linApprox)
{
	PointSet * ps = patchGraph->getTrajectorySet()->getTrajectories();
	AAT POSITION  = ps->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT LOWRESPOS = ps->getAAT( "lowresInterpPos", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT VARINDEX  = ps->getAAT( "varIndex", 1, VAD::DATA_FORMAT_INT32 );
	
	for ( unsigned int n = 0; n < indices->size(); n++ ) {
		int32 frameIndex = (*indices)[n].frame;
		card32 patchIndex = (*indices)[n].point;
		PointSet * ps = patchGraph->getTrajectorySet()->getTrajectories();
		card32 actualFrameIndex = frameIndex;
		if ( frameIndex == BSROPT_FRAME_URSHAPE ) {
			ps = patchGraph->getUrshape();
			actualFrameIndex = 0;
		}
		int32 varIndex = ps->get<int32>(actualFrameIndex, patchIndex, VARINDEX);

		Vector3f lowresPos = ps->get<float32,3>(actualFrameIndex, patchIndex, LOWRESPOS);
		Vector3f pos = ps->get<float32,3>(actualFrameIndex, patchIndex, POSITION);

		Vector3f gradientVector = NULL_VECTOR3F;
		float32 potentialValue = 0;
		
		gradientVector = (pos-lowresPos) * context->weightMultires;
		potentialValue = normQuad(gradientVector);
		gradientVector *= 2.0f;
		if (quadApprox && linApprox) {
			Matrix3f quadLocal = makeMatrix3f(
				context->weightMultires, 0, 0,			
				0, context->weightMultires, 0,
				0, 0, context->weightMultires);
			Vector3f linLocal = lowresPos*(-2*context->weightMultires);
			scatterLocalPosConstraint(quadLocal, linLocal, *quadApprox, *linApprox, varIndex);
		}
		gradient[3*varIndex  ] += gradientVector[0];
		gradient[3*varIndex+1] += gradientVector[1];
		gradient[3*varIndex+2] += gradientVector[2];
		potential += potentialValue;
		if (perPointPotentials) (*perPointPotentials)[varIndex] += potentialValue;
	}	
}


///
/// BOPDataLeastSquares
///
/// Least-Squares Fitting of Data Points (Likelihood)
///

card32 BOPDataLeastSquares::needPrecomputedChannels(BSROptContext *context) {
   return FLAG_PATCH_REF_FRAMES | FLAG_PATCH_SAMPLE_SPACING;
}

void BOPDataLeastSquares::initialize() {
   cachedLSS.clear();
}
void BOPDataLeastSquares::coordsChanged() {
   cachedLSS.clear();
}

void BOPDataLeastSquares::finished() {
   cachedLSS.clear();
}

void BOPDataLeastSquares::computeLSS(
   const BSROptContext *context,
   const BSROptVariableIndexSet *indices,
   const PointCloudAnimationSequence *data,
   const PatchGraph *patchGraph)
{
   cachedLSS.clear();

	const TrajectorySet *ts = patchGraph->getTrajectorySet();
	const InCorePCTopologyGraph *tg = patchGraph->getTopology();
	const PGDataPointIndices *dataIndices = ts->getDataPoints();
	const PointSet *ps = patchGraph->getTrajectorySet()->getTrajectories();

	AAT DATA_POS = data->getFrame(0)->getAAT("position");
	AAT PATCH_POS = ps->getAAT("position");
	AAT NORMAL = ps->getAAT("normal", 3, VAD::DATA_FORMAT_FLOAT32);
	AAT TANGENT_U = ps->getAAT("tangent_u", 3, VAD::DATA_FORMAT_FLOAT32);
	AAT COEFFS_AAT = ps->getAAT(HEIGHTFIELD_COEFFS_NAME);
	AAT SAMPLE_SPACING = ps->getAAT("sampleSpacing");

   // allocate dynamic linear algebra variables only once to improve performance
	DVectorF targetValues;
	DMatrixF basisFunctionValues;
	DVectorF targetValuesWeights;
	DMatrixF mat, invMat;
	DVectorF rhs;
	CoefficientSet coeffs(6);
	HFBFBivariateMonomials bfs;
	bfs.setup(2);
	
	size_t numIndices = indices->size();
	for (card32 i=0; i<numIndices; i++) {
		int32 frame = (*indices)[i].frame;
		card32 patch = (*indices)[i].point;
		const PointSet *dataPS = data->getFrame(frame);
		
		pAssert(frame != BSROPT_FRAME_URSHAPE);

		ReferenceFramePatch refFrame;
		refFrame.get(ps, frame, patch, NORMAL, TANGENT_U);
		Matrix3f transf;
		transf[0] = refFrame.tangent_u;
		transf[1] = refFrame.computeTangentV();
		transf[2] = refFrame.normal;
		transf = transf.transpose();
		Vector3f origin = ps->get<float32, 3>(frame, patch, PATCH_POS);
	
		//float32 windowSize = constantWindowSize;
		//if (windowSizing == WINDOW_CHANNEL) {
		//	windowSizing *= patchPS->get<float32, 1>(patch, SAMPLE_SPACING)[0];
		//}
		vector<card32> dataPointIndices;
		context->likelihoodNeighborhood->setup(patchGraph, data);
		context->likelihoodNeighborhood->collectNeighbors(frame, patch, dataPointIndices);

		vector<Vector3f> points;
		const size_t numDataPoints = dataPointIndices.size();
		for (card32 dp=0; dp<numDataPoints; dp++) {
			points.push_back(dataPS->get3f(dataPointIndices[dp], DATA_POS));
		}
		
		setupDenseLS(&bfs, NULL, points, origin, transf,
					    targetValues, basisFunctionValues, &targetValuesWeights, 0);

		DenseLeastSquaresSystemSetup<float32> dlss;
		dlss.systemSetup(targetValues, basisFunctionValues, mat, rhs, NULL);
      LocalLSSystem e;
      e.mat = convertMatrixType< DMatrixF, StaticMatrix<float32,6,6> >(mat, 6, 6);
      e.rhs = convertVectorType< DVectorF, StaticVector<float32, 6> >(rhs, 6);
      cachedLSS.push_back(e);
	}
}


void BOPDataLeastSquares::computeGradient(
   const BSROptContext *context,
   const BSROptVariableIndexSet *indices,
   const PointCloudAnimationSequence *data,
   PatchGraph *patchGraph,
   DVectorF &gradient,
   float32 &potential,
   DVectorF *perPointPotentials,
   SparseMatrixF *quadApprox,
   DVectorF *linApprox)
{
	PointSet * ps = patchGraph->getTrajectorySet()->getTrajectories();
	PGDataPointIndices *dataPoints = patchGraph->getTrajectorySet()->getDataPoints();
	AAT COEFF = ps->getAAT( "hfieldCoeffs" );
	AAT DATA_POS_AAT = data->getFrame(0)->getAAT( "position" );
	AAT VARINDEX = ps->getAAT("varIndex");
	
	const size_t numIndices = indices->size();
	
	if (numIndices != cachedLSS.size()) {
	   computeLSS(context, indices, data, patchGraph);
	}
	
	for ( unsigned int i = 0; i < numIndices; i++ ) {
		int32 frame = (*indices)[i].frame;
		card32 patch = (*indices)[i].point;
		int32 varIndex = ps->get<int32>(frame, patch, VARINDEX);
		pAssert(varIndex >=0 && varIndex < (int32)gradient.getDim());
		pAssert(frame != BSROPT_FRAME_URSHAPE);

		//StaticVector<float32, 6> coeff = invertMatrix(cachedLSS[i].mat) * cachedLSS[i].rhs;
		//ps->set<float32, 6>(frame, patch, COEFF, coeff);

		StaticVector<float32, 6> coeff = ps->get<float32, 6>(frame, patch, COEFF);
		StaticVector<float32, 6> gradientVector = cachedLSS[i].mat * coeff - cachedLSS[i].rhs;
      gradient[varIndex  ] += gradientVector[0];
      gradient[varIndex+1] += gradientVector[1];
      gradient[varIndex+2] += gradientVector[2];
      gradient[varIndex+3] += gradientVector[3];
      gradient[varIndex+4] += gradientVector[4];
      gradient[varIndex+5] += gradientVector[5];
	} 
	/// todo: compute potentials
}


// -------------------------------------------------------------------------------

void BOPIndexingChannel::writeIndexingChannel(
	const BSROptVariableIndexSet *indices,
	PatchGraph *patchGraph)
{
   PointSet *ps = patchGraph->getTrajectorySet()->getTrajectories();
   PointSet *urshape = patchGraph->getUrshape();
   pAssert(urshape == NULL || ps->getDescr()->equal(urshape->getDescr()));
	AAT VARINDEX = ps->getAAT("varIndex", 1, VAD::DATA_FORMAT_INT32);

	const size_t n = indices->size();
	for (unsigned i=0; i<n; i++) {
		const int32 frame = (*indices)[i].frame;
		const card32 point = (*indices)[i].point;
		if ( frame != BSROPT_FRAME_URSHAPE ) {
         ps->set<int32>(frame, point, VARINDEX, (int32)i);
		} else {
         urshape->set<int32>(point, VARINDEX, (int32)i);
		}
	}	
}

void BOPIndexingChannel::readChannel(
	const BSROptVariableIndexSet *indices,
	const PatchGraph *patchGraph,
	const string channel,
	const card32 channelDim,
	DVectorF &result)
{
   const PointSet *ps = patchGraph->getTrajectorySet()->getTrajectories();
   const PointSet *urshape = patchGraph->getUrshape();
   pAssert(urshape == NULL || ps->getDescr()->equal(urshape->getDescr()));
	AAT channelAAT = ps->getAAT(channel, channelDim, VAD::DATA_FORMAT_FLOAT32);

	const card32 n = (card32)indices->size();
   result.setDim(channelDim*n);
	for (unsigned i=0; i<n; i++) {
		const int32 frame = (*indices)[i].frame;
		const card32 point = (*indices)[i].point;
		const char *data;
		if ( frame != BSROPT_FRAME_URSHAPE ) {
		   data = ps->getDataPointer(frame, point);
		} else {
		   data = urshape->getDataPointer(point);
		}
		float* dataF = (float*)(data+channelAAT.getOffset());
		for (card32 c=0; c<channelDim; c++) {
		   result[i*channelDim+c] = dataF[c];
		}
	}	
}

void BOPIndexingChannel::writeChannel(
	const BSROptVariableIndexSet *indices,
	PatchGraph *patchGraph,
	const string channel,
	const card32 channelDim,
	const DVectorF &data)
{
   PointSet *ps = patchGraph->getTrajectorySet()->getTrajectories();
   PointSet *urshape = patchGraph->getUrshape();
   pAssert(urshape == NULL || ps->getDescr()->equal(urshape->getDescr()));
	AAT channelAAT = ps->getAAT(channel, channelDim, VAD::DATA_FORMAT_FLOAT32);

	const size_t n = indices->size();
	for (unsigned i=0; i<n; i++) {
		const int32 frame = (*indices)[i].frame;
		const card32 point = (*indices)[i].point;
		char *dataC;
		if ( frame != BSROPT_FRAME_URSHAPE ) {
		   dataC = ps->getDataPointer(frame, point);
		} else {
		   dataC = urshape->getDataPointer(point);
		}
		float* dataF = (float*)(dataC+channelAAT.getOffset());
		for (card32 c=0; c<channelDim; c++) {
		   dataF[c] = data[i*channelDim+c];
		}
	}	
}