//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "BSROptPotential.h"
#include "PointSet_templCode.h"
#include "Monomials2ndOrder.h"
#include "TGNeighborhoodKNNSearch.h"
#include "BSROptContext.h"
#include "PointCloudAnimationSequence.h"
#include "PGDataPointIndices.h"
#include "InCorePCTopologyGraph.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra_templCode.h"
#include "PatchTypes.h"
#include "DynamicLinearAlgebraTools.h"
#include "DynamicLinearAlgebraTools_templCode.h"
#include "SparseLinearAlgebra.h"
#include "SparseLinearAlgebra_templCode.h"
#include "HFBFBivariateMonomials.h"
#include "PatchApprox3d.h"
#include "LeastSquaresSystemSetup.h"
#include "PGTopologicalNeighborhood.h"
#include "PointSetKNNQuery.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------

// -------------------------------------------------------------------------------

/// helpers
 

static	void	UpdateMatrix_by_a_matrix(const float32		&weight,			/// The weight to be weighted
														const DMatrixF			&	mat,				/// The matrix to be added
														const DVectorI			&	m_indices,		/// The indices of variables to be optimized
														SparseMatrixF				&quad)			/// The Hessian of the energy function	
{
	pAssert(m_indices.getDim() == mat.getRows());

	for(card32 row_i = 0; row_i < m_indices.getDim(); row_i++)
	{
		int row = m_indices[row_i];
		if(row >= 0 && row < int(quad.getRows()))
		{
			for(card32 colume_i = 0; colume_i < m_indices.getDim(); colume_i++)
			{
				int	colume = m_indices[colume_i];
				if(colume >= 0 && colume < int(quad.getRows()))
				{
					quad[row].addEntry(colume, weight*mat[row_i][colume_i]);
				}
			}
		}
	}
}

static	void	UpdateVector_by_a_vector(const float32	&weight, 
														const DVectorF			&	vec,
														DVectorI					&		m_indices, 
														DVectorF						&lin)
{
	pAssert(m_indices.getDim() == vec.getDim());
	
	for(card32 row_i = 0; row_i < m_indices.getDim(); row_i++)
	{
		int row = m_indices[row_i];
		if(row >= 0 && row < int(lin.getDim()))
		{
			lin[row] += weight*vec[row_i];
		}
	}
}

/// helpers


static	void	UpdateMatrix_by_a_matrix(const DMatrixF	&	mat,				
													 const DVectorI	& m_indices,
													 const	 card32		& numOfEntries,
													 SparseMatrixF		&quad)				
{
	pAssert(m_indices.getDim() == mat.getRows()&& numOfEntries <= m_indices.getDim());

	for(card32 row_i = 0; row_i < numOfEntries; row_i++)
	{
		int row = m_indices[row_i];
		pAssert(row>=0);
		quad[row].addEntries_Binary(&m_indices, &mat[row_i], numOfEntries);
	}
}

static	void	UpdateVector_by_a_vector(const	DVectorF	& vec,
													 const	DVectorI		& m_indices, 
													 const	card32	&	numOfEntries,
													 DVectorF	&lin)
{
	pAssert(m_indices.getDim() == vec.getDim()
		&& numOfEntries <= m_indices.getDim());

	for(card32 row_i = 0; row_i < numOfEntries; row_i++)
	{
		int row = m_indices[row_i];
		pAssert(row>=0);
		lin[row] += vec[row_i];
	}
}

//Convert quaternion to rotation matrix

void		Quaternion_To_RotationMatrix(const float	*q, Matrix3f &rotation)
{
	pAssert(q != NULL);
	rotation[0][0] = 2*q[1]*q[1] + (2*q[0]*q[0]-1);
	rotation[0][1] = 2*q[1]*q[2] - 2*q[0]*q[3];
	rotation[0][2] = 2*q[1]*q[3] + 2*q[0]*q[2];
	rotation[1][0] = 2*q[2]*q[1] + 2*q[0]*q[3];
	rotation[1][1] = 2*q[2]*q[2] + (2*q[0]*q[0]-1);
	rotation[1][2] = 2*q[2]*q[3] - 2*q[0]*q[1];
	rotation[2][0] = 2*q[3]*q[1] - 2*q[0]*q[2];
	rotation[2][1] = 2*q[3]*q[2] + 2*q[0]*q[1];
	rotation[2][2] = 2*q[3]*q[3] + (2*q[0]*q[0]-1);
}
/************************************************************************/
/* NeedPrecomputedChannels                                                                     */
/************************************************************************/

card32 BOPSquaredDistancePotential::needPrecomputedChannels(BSROptContext *context)
{
//	return FLAG_NEED_PATCH_2ND_ORDER_HFIELD|FLAG_PATCH_REF_FRAMES;
	return FLAG_PATCH_REF_FRAMES;
}

card32 BOPMarkerErrorPotential::needPrecomputedChannels(BSROptContext *context)
{
	return FLAG_PATCH_REF_FRAMES;
}

card32 BOPPlanarSmoothnessPotential::needPrecomputedChannels(BSROptContext *context)
{
	return FLAG_PATCH_REF_FRAMES;
}

card32 BOPNormalSmoothnessPotential::needPrecomputedChannels(BSROptContext *context)
{
	return FLAG_PATCH_REF_FRAMES;
}

card32 BOPCircularSmoothnessPotential::needPrecomputedChannels(BSROptContext *context)
{
	return FLAG_PATCH_REF_FRAMES;
}

card32 BOPLaplacianSmoothnessPotential::needPrecomputedChannels(BSROptContext *context)
{
	return FLAG_PATCH_REF_FRAMES;
}

card32 BOPPositionRigidityPotential::needPrecomputedChannels(BSROptContext *context)
{
	return FLAG_NEED_ROTATION_MATRIX|FLAG_PATCH_REF_FRAMES;
}

card32 BOPNormalRigidityPotential::needPrecomputedChannels(BSROptContext *context)
{
	return FLAG_NEED_ROTATION_MATRIX|FLAG_PATCH_REF_FRAMES;
}

card32 BOPSTPositionCoherence::needPrecomputedChannels(BSROptContext *context)
{
	return FLAG_NEED_ROTATION_MATRIX|FLAG_PATCH_REF_FRAMES;
}

card32 BOPSTNormalCoherence::needPrecomputedChannels(BSROptContext *context)
{
	return FLAG_NEED_ROTATION_MATRIX|FLAG_PATCH_REF_FRAMES;
}

card32 BOPSTRotationCoherence::needPrecomputedChannels(BSROptContext *context)
{
	return FLAG_NEED_ROTATION_MATRIX;
}
/************************************************************************/
/* UpdateGradientandHessian                                                                     */
/************************************************************************/

void		BOPSquaredDistancePotential::peter_UpdateGradientandHessian(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const	card32	numOfNeighbors,
	const	int	*neighbor_buffer,
	SparseMatrixF *quadApprox, 
	DVectorF *linApprox)
{

}

void		BOPMarkerErrorPotential::peter_UpdateGradientandHessian(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const	card32	numOfNeighbors,
	const	int	*neighbor_buffer,
	SparseMatrixF *quadApprox, 
	DVectorF *linApprox)
{
	PointSet*									ps	= patchGraph->getTrajectorySet()->getTrajectories();
	PGDataPointIndices *dataPoints	 = patchGraph->getTrajectorySet()->getDataPoints();
	AAT COEFF;

	AAT POSITION				= ps->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT VARINDEX_POS		= ps->getAAT( "varIndex_pos", 1, VAD::DATA_FORMAT_INT32 );
	AAT DATA_POS_AAT	= data->getFrame(0)->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );

	float	weightmarker		= context->weightMarkerError;
	card32	width				= patchGraph->getTrajectorySet()->getNumFrames(),
					height				= patchGraph->getTrajectorySet()->getNumPatches();

	DVectorF	TermGradient;	TermGradient.setDim(3);
	DMatrixF		TermHessian;		TermHessian.setDimension(3,3, false);
	DVectorI		m_indices;			m_indices.setDim(3);

	for(card32 frame = 0; frame < width; frame++)
	{
		card32	dataPointsPerframe = 0, point;
		for(point = 0; point < height; point++)
		{
			const card32 numDataPoints = dataPoints->getNumDataPoints( frame, point );
			if (numDataPoints > 0) 
				dataPointsPerframe++;
		}

		if(dataPointsPerframe == 0)
			continue;

		//		float		weight = weightmarker*height/dataPointsPerframe;
		float		weight	 = weightmarker;
		for(point = 0; point < height; point++)
		{
			int32 ik_posid = ps->get<int32>(frame, point, VARINDEX_POS);

			card32	k;
			for(k = 0; k < 3; k++)
			{
				m_indices[k] = ik_posid + k;
			}

			Vector3f pik = ps->get3f( frame, point, POSITION );
			const card32 numDataPoints = dataPoints->getNumDataPoints( frame, point );

			if (numDataPoints > 0) 
			{
				card32 dataIndexInData = dataPoints->getIndex( frame, point, 0);
				Vector3f devik = pik - data->getFrame(frame)->get3f(dataIndexInData, DATA_POS_AAT);
				/// Update Gradient
				for(card32 i = 0; i < 3; i++)
				{
					TermGradient[i] = -devik[i]*weight;
					for(card32 j = 0; j < 3; j++)
					{
						TermHessian[i][j] = 0.0f;
					}
					TermHessian[i][i] += weight;
				}
				UpdateVector_by_a_vector(TermGradient, m_indices, 3, *linApprox);
				/// Update Hessian
				UpdateMatrix_by_a_matrix(TermHessian, m_indices, 3, *quadApprox);
			}
		}
	}
}

void		BOPPlanarSmoothnessPotential::peter_UpdateGradientandHessian(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32	numOfNeighbors,
	const int	*neighbor_buffer,
	SparseMatrixF *quadApprox, 
	DVectorF *linApprox)
{
	pAssert(neighbor_buffer != NULL);
	float32	weight_planar	= context->weightPlanarSmoothness,
		lambda				=	context->lambda;

	card32	width              = patchGraph->getTrajectorySet()->getNumFrames(),
		height             = patchGraph->getTrajectorySet()->getNumPatches();

	PointSet	*ps		= patchGraph->getTrajectorySet()->getTrajectories();
	AAT	POSITION	= ps->getAAT("position");
	AAT	NORMAL		= ps->getAAT("normal");
	AAT	REF_U			= ps->getAAT("tangent_u");
	AAT	POS_INDEX= ps->getAAT("varIndex_pos");
	AAT	NOR_INDEX= ps->getAAT("varIndex_nor");

	/// The term gradient, termHessian and temporary matrix 
	DVectorF			termGradient;		termGradient.setDim(8);
	DMatrixF				termHessian;			termHessian.setDimension(8, 8, false);
	DVectorF			tempMatrix;			tempMatrix.setDim(8);

	/// The variable index transform
	DVectorI				m_indices;				m_indices.setDim(8);

	for(card32 frame = 0; frame < width; frame++)
	{
		for(card32 point = 0; point < height; point++)
		{
			Vector3f		pik		= ps->get3f(frame, point, POSITION);
			Vector3f		nik		= ps->get3f(frame, point, NORMAL);
			Vector3f		Eik1		= ps->get3f(frame, point, REF_U);
			Vector3f		Eik2		= nik.crossProduct(Eik1);
			card32		offset	= (frame*height + point)*numOfNeighbors;

			int				ik_posid= ps->get<int32>(frame, point, POS_INDEX);
			int				ik_norid	= ps->get<int32>(frame, point, NOR_INDEX);
			card32		k;


			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				int		jk_posid	= ps->get<int32>(frame,  neighbor_buffer[ni], POS_INDEX);

				for(k = 0; k < 3; k++)
				{
					if(ik_posid < jk_posid)
					{
						m_indices[k	 ] = ik_posid + k;
						m_indices[3+k] = jk_posid + k;
					}
					else
					{
						m_indices[k+3] = ik_posid + k;
						m_indices[k] = jk_posid + k;
					}
				}
				for(k = 0; k < 2; k++)
					m_indices[6+k] = ik_norid + k;
				Vector3f	pjk		=	ps->get3f(frame, neighbor_buffer[ni], POSITION);
				Vector3f	rijk		=	pik - pjk;
				float32	dis		= nik*rijk;
				float32	weight= weight_planar*exp(lambda*rijk.getSqrNorm())/numOfNeighbors;

				/// Compute tempMatirx
				card32	i, j;
				for(i = 0; i < 3; i++)
				{
					if(ik_posid < jk_posid)
					{
						tempMatrix[i	  ]	= nik[i]+lambda*dis*rijk[i];
						tempMatrix[i+3]	= - tempMatrix[i];
					}	
					else
					{
						tempMatrix[i+3]	= nik[i]+lambda*dis*rijk[i];
						tempMatrix[i]	= - tempMatrix[i+3];
					}
				}
				tempMatrix[6]			=	rijk*Eik1;
				tempMatrix[7]			=	rijk*Eik2;
				for(i = 0; i < 8; i++)
				{
					termGradient[i] = -tempMatrix[i]*dis*weight;
					for(j = i; j < 8; j++)
					{
						termHessian[i][j] = tempMatrix[i]*tempMatrix[j]*weight;
					}
				}
				for(i = 0; i < 8; i++)
				{
					for(j = i+1; j < 8; j++)
					{
						termHessian[j][i] = termHessian[i][j];
					}
				}
				///	Update Gradient
				UpdateVector_by_a_vector(termGradient, m_indices, 8, *linApprox);
				///	Update Hessian
				UpdateMatrix_by_a_matrix(termHessian, m_indices, 8, *quadApprox);
			}
		}
	}
}

void		BOPNormalSmoothnessPotential::peter_UpdateGradientandHessian(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32	numOfNeighbors,
	const int	*neighbor_buffer,
	SparseMatrixF *quadApprox, 
	DVectorF *linApprox)
{
	pAssert(neighbor_buffer != NULL);
	float32	weight_normal	= context->weightNormalSmoothness,
					lambda				=	context->lambda;

	card32	width              = patchGraph->getTrajectorySet()->getNumFrames(),
		height             = patchGraph->getTrajectorySet()->getNumPatches();

	PointSet	*ps		= patchGraph->getTrajectorySet()->getTrajectories();
	AAT	POSITION	= ps->getAAT("position");
	AAT	NORMAL		= ps->getAAT("normal");
	AAT	REF_U			= ps->getAAT("tangent_u");
	AAT	POS_INDEX= ps->getAAT("varIndex_pos");
	AAT	NOR_INDEX= ps->getAAT("varIndex_nor");

	/// The term gradient, termHessian and temporary matrix 
	DVectorF			termGradient;		termGradient.setDim(10);
	DMatrixF				termHessian;			termHessian.setDimension(10, 10, false);
	DMatrixF				tempMatrix;			tempMatrix.setDimension(10, 3, false);

	/// The variable index transform
	DVectorI				m_indices;				m_indices.setDim(10);

	for(card32 frame = 0; frame < ps->getWidth(); frame++)
	{
		for(card32 point = 0; point < ps->getHeight(); point++)
		{
			Vector3f		pik			= ps->get3f(frame, point, POSITION);
			Vector3f		nik			= ps->get3f(frame, point, NORMAL);
			Vector3f		Eik1			= ps->get3f(frame, point, REF_U);
			Vector3f		Eik2			= nik.crossProduct(Eik1);
			card32		offset		= (frame*height + point)*numOfNeighbors;
			int				ik_posid	= ps->get<int32>(frame, point, POS_INDEX);
			int				ik_norid		= ps->get<int32>(frame, point, NOR_INDEX);

			card32		k;


			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				int		jk_posid	= ps->get<int32>(frame,  neighbor_buffer[ni], POS_INDEX);
				int		jk_norid		= ps->get<int32>(frame,  neighbor_buffer[ni], NOR_INDEX);

				for(k = 0; k < 3; k++)
				{
					if(ik_posid < jk_posid)
					{
						m_indices[k	   ]	= ik_posid + k;
						m_indices[3+k]		= jk_posid + k;
					}
					else
					{
						m_indices[3+k]	= ik_posid + k;
						m_indices[k]		= jk_posid + k;
					}
				}
				for(k = 0; k < 2; k++)
				{
					if(ik_norid < jk_norid)
					{
						m_indices[6+k]		= ik_norid + k;
						m_indices[8+k]		= jk_norid + k;
					}
					else
					{
						m_indices[8+k]		= ik_norid + k;
						m_indices[6+k]		= jk_norid + k;
					}
				}
				Vector3f	njk		= ps->get3f(frame, neighbor_buffer[ni], NORMAL);
				Vector3f	rijk		=	pik - ps->get3f(frame, neighbor_buffer[ni], POSITION);
				Vector3f nijk		= nik - njk;
				Vector3f Ejk1   = ps->get3f(frame, neighbor_buffer[ni], REF_U);
				Vector3f Ejk2   = njk.crossProduct(Ejk1);
				float32	weight= weight_normal*exp(lambda*rijk.getSqrNorm())/numOfNeighbors;

				//Compute tempMatirx
				card32	i, j;
				for(j = 0; j < 3; j++)
				{
					for(i = 0; i < 3; i++)
					{
						if(ik_posid < jk_posid)
						{
							tempMatrix[i	 ][j]		= lambda*rijk[i]*nijk[j];
							tempMatrix[i+3][j]	= - tempMatrix[i][j];
						}
						else
						{
							tempMatrix[i+3][j]		= lambda*rijk[i]*nijk[j];
							tempMatrix[i][j]	= - tempMatrix[i+3][j];
						}
					}
					if(ik_norid < jk_norid)
					{
						tempMatrix[6][j]			= Eik1[j];
						tempMatrix[7][j]			= Eik2[j];
						tempMatrix[8][j]			= -Ejk1[j];
						tempMatrix[9][j]			= -Ejk2[j];
					}
					else
					{
						tempMatrix[8][j]			= Eik1[j];
						tempMatrix[9][j]			= Eik2[j];
						tempMatrix[6][j]			= -Ejk1[j];
						tempMatrix[7][j]			= -Ejk2[j];
					}
				}
				for(i = 0; i < 10; i++)
				{
					termGradient[i] = -(tempMatrix[i][0]*nijk[0]+tempMatrix[i][1]*nijk[1]+tempMatrix[i][2]*nijk[2])*weight;
					for(j = i; j < 10; j++)
					{
						termHessian[i][j] = (tempMatrix[i][0]*tempMatrix[j][0]+tempMatrix[i][1]*tempMatrix[j][1]
						+tempMatrix[i][2]*tempMatrix[j][2])*weight;
					}
				}
				for(i = 0; i < 10; i++)
				{
					for(j = i+1; j < 10; j++)
					{
						termHessian[j][i] = termHessian[i][j];
					}
				}
				///	Update Gradient
				UpdateVector_by_a_vector(termGradient, m_indices, 10, *linApprox);
				///	Update Hessian
				UpdateMatrix_by_a_matrix(termHessian, m_indices, 10, *quadApprox);
			}
		}
	}
}

void		BOPCircularSmoothnessPotential::peter_UpdateGradientandHessian(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32	numOfNeighbors,
	const int	*neighbor_buffer,
	SparseMatrixF *quadApprox, 
	DVectorF *linApprox)
{
	pAssert(neighbor_buffer != NULL);
	float32	weight_circular	= context->weightCircularSmoothness,
					lambda				=	context->lambda;

	card32	width              = patchGraph->getTrajectorySet()->getNumFrames(),
					height             = patchGraph->getTrajectorySet()->getNumPatches();

	PointSet		*ps					= patchGraph->getTrajectorySet()->getTrajectories();
	AAT			POSITION			= ps->getAAT("position");
	AAT			NORMAL				= ps->getAAT("normal");
	AAT			REF_U					= ps->getAAT("tangent_u");
	AAT			POS_INDEX		= ps->getAAT("varIndex_pos");
	AAT			NOR_INDEX		= ps->getAAT("varIndex_nor");

	/// The term gradient, termHessian and temporary matrix 
	DVectorF			termGradient;		termGradient.setDim(10);
	DMatrixF				termHessian;			termHessian.setDimension(10, 10, false);
	DVectorF			tempMatrix;			tempMatrix.setDim(10);

	/// The variable index transform
	DVectorI				m_indices;				m_indices.setDim(10);

	for(card32 frame = 0; frame < width; frame++)
	{
		for(card32 point = 0; point < height; point++)
		{
			int			i_posid	= ps->get<int32>(frame, point, POS_INDEX);
			int			i_norid		= ps->get<int32>(frame, point, NOR_INDEX);

			card32		k;
			for(k = 0; k < 3; k++)
			{
				m_indices[k]		= i_posid + k;
			}
			for(k = 0; k < 2; k++)
			{
				m_indices[5+k] = i_norid + k;
			}

			Vector3f		pik		= ps->get3f(frame, point, POSITION);
			Vector3f		nik		= ps->get3f(frame, point, NORMAL);
			Vector3f		Eik1		= ps->get3f(frame, point, REF_U);
			Vector3f		Eik2		= nik.crossProduct(Eik1);

			card32		offset	= (frame*height + point)*numOfNeighbors;
			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				if(neighbor_buffer[ni] >= 0)
				{
					int		j_posid	= ps->get<int32>(frame,  neighbor_buffer[ni], POS_INDEX);
					int		j_norid		= ps->get<int32>(frame,  neighbor_buffer[ni], NOR_INDEX);

					if(i_posid < 0 && j_posid && i_norid < 0 && j_norid < 0)
						continue;

					for(k = 0; k < 3; k++)
					{
						m_indices[3+k] = j_posid + k;
					}
					for(k = 0; k < 2; k++)
					{
						m_indices[8+k] = j_norid + k;
					}
					Vector3f		njk		= ps->get3f(frame, neighbor_buffer[ni], NORMAL);
					Vector3f		rijk		=	pik - ps->get3f(frame, neighbor_buffer[ni], POSITION);
					Vector3f		nijk		= nik + njk;
					Vector3f		Ejk1   = ps->get3f(frame, neighbor_buffer[ni], REF_U);
					Vector3f		Ejk2   = njk.crossProduct(Ejk1);
					float32		weight= weight_circular*exp(lambda*rijk.getSqrNorm())/numOfNeighbors;
					float32		dis		= rijk*nijk;

					/// Compute tempMatirx
					card32	i, j;
					for(i = 0; i < 3; i++)
					{
						tempMatrix[i	  ]= nijk[i]+lambda*dis*rijk[i];
						tempMatrix[i+3]= - tempMatrix[i];	
					}
					tempMatrix[6]		=	rijk*Eik1;
					tempMatrix[7]		=	rijk*Eik2;
					tempMatrix[8]		=	rijk*Ejk1;
					tempMatrix[9]		=	rijk*Ejk2;
					for(i = 0; i < 10; i++)
					{
						termGradient[i] = -tempMatrix[i]*dis;
						for(j = 0; j < 10; j++)
						{
							termHessian[i][j] = tempMatrix[i]*tempMatrix[j];
						}
					}
					///	Update Gradient
					UpdateVector_by_a_vector(weight, termGradient, m_indices, *linApprox);
					///	Update Hessian
					UpdateMatrix_by_a_matrix(weight, termHessian, m_indices, *quadApprox);
				}
			}
		}
	}
}

void		BOPLaplacianSmoothnessPotential::peter_UpdateGradientandHessian(const BSROptContext *context, 
																								 const PointCloudAnimationSequence *data, 
																								 PatchGraph *patchGraph, 
																								 const card32	numOfNeighbors,
																								 const int	*neighbor_buffer,
																								 SparseMatrixF *quadApprox, 
																								 DVectorF *linApprox)
{
	pAssert(neighbor_buffer != NULL);
	float32	weight_laplacian	= context->weightLaplacianSmoothness,
					lambda					=	context->lambda;

	card32	width						= patchGraph->getTrajectorySet()->getNumFrames(),
		height				      = patchGraph->getTrajectorySet()->getNumPatches();

	PointSet	*ps		= patchGraph->getTrajectorySet()->getTrajectories();
	AAT	POSITION	= ps->getAAT("position");
	AAT	POS_INDEX= ps->getAAT("varIndex_pos");

	/// The term gradient, termHessian and temporary matrix 
	DVectorF			termGradient;		termGradient.setDim(3*numOfNeighbors+3);
	DMatrixF				termHessian;			termHessian.setDimension(3*numOfNeighbors+3, 3*numOfNeighbors+3, false);
	DMatrixF				tempMatrix;			tempMatrix.setDimension(3*numOfNeighbors+3, 3,  false);

	/// The variable index transform
	DVectorI				m_indices;				m_indices.setDim(3*numOfNeighbors+3);


	for(card32 frame = 0; frame < width; frame++)
	{
		for(card32 point = 0; point < height; point++)
		{
			Vector3f		devik;	devik[0] = devik[1] = devik[2] = 0.0f;

			Vector3f		pik		= ps->get3f(frame, point, POSITION);
			card32		offset	= (frame*height + point)*numOfNeighbors;
			int				i_posid= ps->get<int32>(frame, point, POS_INDEX);

			float			asqrLen = 0.0;
			card32		count = 0, i, j, k;
			for(k = 0; k < 3; k++)
			{
				m_indices[k] = i_posid + k;
			}
			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				if(neighbor_buffer[ni] >= 0)
				{
					count++;
					int			j_posid	= ps->get<int32>(frame,  neighbor_buffer[ni], POS_INDEX);
					for(k = 0; k < 3; k++)
					{
						m_indices[ni - offset + 3 +k] = j_posid+k;
					}
					Vector3f	rijk			=	pik - ps->get3f(frame, neighbor_buffer[ni], POSITION);
					for(j = 0; j < 3; j++)
					{
						for(i = 0; i < 3; i++)
						{
							tempMatrix[ni - offset + 3 + i][j] = rijk[i];
						}
					}
					devik					+= rijk;
					asqrLen				+= rijk.getSqrNorm();
				}
				else
				{
					for(k = 0; k < 3; k++)
					{
						m_indices[ni - offset + 3 +k] = -1;
					}
					for(j = 0; j < 3; j++)
					{
						for(i = 0; i < 3; i++)
						{
							tempMatrix[ni - offset + 3 + i][j] = 0.0f;
						}
					}
				}
			}
			devik		/= count;			asqrLen	/= count;
			float	 weight = weight_laplacian*exp(lambda*asqrLen);

			for(j = 0; j < 3; j++)
			{
				for(i = 0; i < 3; i++)
				{
					tempMatrix[i][j] = lambda*devik[i]*devik[j];
				}
				tempMatrix[i][i]  += 1.0;
			}

			for(i = 3; i < 3*numOfNeighbors+3; i++)
			{
				for(j = 0; j < 3; j++)
				{
					tempMatrix[i][j]		= -tempMatrix[i][j]*lambda*devik[j]/count;
				}
				if((i - j)%3 == 0)
					tempMatrix[i][j] -= 1.0f/count;
			}
			for(i = 0; i < 3*numOfNeighbors+3; i++)
			{
				termGradient[i] = -(tempMatrix[i][0]*devik[0]+tempMatrix[i][1]*devik[1]+tempMatrix[i][2]*devik[2]);
				for(j = 0; j < 3*numOfNeighbors+3; j++)
				{
					termHessian[i][j] = tempMatrix[i][0]*tempMatrix[j][0]+tempMatrix[i][1]*tempMatrix[j][1]
					+tempMatrix[i][2]*tempMatrix[j][2];
				}
			}
			///	Update Gradient
			UpdateVector_by_a_vector(weight, termGradient, m_indices, *linApprox);
			///	Update Hessian
			UpdateMatrix_by_a_matrix(weight, termHessian, m_indices, *quadApprox);
		}
	}
}

void		BOPPositionRigidityPotential::peter_UpdateGradientandHessian(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32	numOfNeighbors,
	const int	*neighbor_buffer,
	SparseMatrixF *quadApprox, 
	DVectorF *linApprox)
{
	pAssert(neighbor_buffer != NULL);
	float32	weight_position	= context->weightPositionRigidity,
					lambda				=	context->lambda;

	card32	width              = patchGraph->getTrajectorySet()->getNumFrames(),
					height             = patchGraph->getTrajectorySet()->getNumPatches();

	PointSet	*ps						= patchGraph->getTrajectorySet()->getTrajectories();
	AAT	POSITION					= ps->getAAT("position");
	AAT	ROTATION					= ps->getAAT("rotation_matrix");
	AAT	POS_INDEX				= ps->getAAT("varIndex_pos");
	AAT	ROT_INDEX				= ps->getAAT("varIndex_rot");

	/// The term gradient, termHessian and temporary matrix 
	DVectorF			termGradient;		termGradient.setDim(15);
	DMatrixF				termHessian;			termHessian.setDimension(15, 15, false);
	DMatrixF				tempMatrix;			tempMatrix.setDimension(15, 3, false);

	/// The variable index transform
	DVectorI				m_indices;				m_indices.setDim(15);

	/// Current term rotation matrix
	Matrix3f				Aik;

	/// We don't have rotation matrix for the last frame
	for(card32 frame = 0; frame < width-1; frame++)
	{
		for(card32 point = 0; point < height; point++)
		{
			int			ik_posid		= ps->get<int32>(frame,	 point, POS_INDEX);
			int			ikn_posid	= ps->get<int32>(frame+1, point, POS_INDEX);
			int			ik_rotid		= ps->get<int32>(frame,	 point, ROT_INDEX);

			card32		k;

			Vector3f		pik			= ps->get3f(frame   , point, POSITION);
			Vector3f		pikn			= ps->get3f(frame+1, point, POSITION);

			//get current rotation matrix
			char			*dataC		= ps->getDataPointer(frame, point);
			float32	*quatc		= (float*)(dataC+ROTATION.getOffset());

			Quaternion_To_RotationMatrix(quatc, Aik);
			card32		offset		= (frame*height + point)*numOfNeighbors;
			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				int	jk_posid		= ps->get<int32>(frame,		neighbor_buffer[ni], POS_INDEX);
				int	jkn_posid	= ps->get<int32>(frame+1,   neighbor_buffer[ni], POS_INDEX);


				for(k = 0; k <3 ; k++)
				{
					if(ik_posid < jk_posid)
					{
						m_indices[k   ]		= ik_posid		+ k;
						m_indices[3+k ]	= jk_posid		+ k;
						m_indices[6+k]		= ikn_posid		+	k;
						m_indices[9+k ]	= jkn_posid	+	k;
					}
					else
					{
						m_indices[3+k]		= ik_posid		+ k;
						m_indices[	k]			= jk_posid		+ k;
						m_indices[9+k]		= ikn_posid		+	k;
						m_indices[6+k]		= jkn_posid	+	k;
					}
					m_indices[12+k]	= ik_rotid		+ k;
				}

				Vector3f		pjk			= ps->get3f(frame,		neighbor_buffer[ni], POSITION);
				Vector3f		pjkn			=	ps->get3f(frame+1, neighbor_buffer[ni], POSITION);

				Vector3f		rijk			= pik - pjk;					Vector3f	rijkn				= pikn - pjkn;
				Vector3f		rijkc			= (Aik.transpose())*rijk;
				Vector3f		dev			= rijkc - rijkn;

				float32		sqrDev= dev.getSqrNorm();
				float32		weight= weight_position*exp(lambda*(rijk.getSqrNorm()+rijkn.getSqrNorm())/2)/numOfNeighbors;

				//Compute tempMatirx
				card32	i, j;
				for(j = 0; j < 3; j++)
				{
					if(ik_posid < jk_posid)
					{
						for(i = 0; i < 3; i++)
						{
							tempMatrix[i	  ][j]	= rijk[i]*dev[j]*lambda/2 + Aik[j][i];
							tempMatrix[i+3][j]	= - tempMatrix[i][j];
							tempMatrix[i+6][j]	= rijkn[i]*dev[j]*lambda/2;
							tempMatrix[i+9][j]	= - tempMatrix[i+6][j];
						}
						tempMatrix[j+6][j]    -= 1.0f;
						tempMatrix[j+9][j]    += 1.0f;
					}
					else
					{
						for(i = 0; i < 3; i++)
						{
							tempMatrix[i+3][j]	= rijk[i]*dev[j]*lambda/2 + Aik[j][i];
							tempMatrix[i][j]	= - tempMatrix[i+3][j];
							tempMatrix[i+9][j]	= rijkn[i]*dev[j]*lambda/2;
							tempMatrix[i+6][j]	= - tempMatrix[i+9][j];
						}
						tempMatrix[j+9][j]    -= 1.0f;
						tempMatrix[j+6][j]    += 1.0f;
					}
				}
				tempMatrix[12][0]	=	  0.0f;			tempMatrix[12][1]	= - rijkc[2];	tempMatrix[12][2]	=   rijkc[1];
				tempMatrix[13][0]	=	  rijkc[2];	tempMatrix[13][1]	=   0.0f;			tempMatrix[13][2]	= - rijkc[0];
				tempMatrix[14][0]	= - rijkc[1];	tempMatrix[14][1]	=  rijkc[0];		tempMatrix[14][2]	=   0.0f;
				for(i = 0; i < 15; i++)
				{
					termGradient[i] = -(tempMatrix[i][0]*dev[0]+tempMatrix[i][1]*dev[1]+tempMatrix[i][2]*dev[2])*weight;
					for(j = i; j < 15; j++)
					{
						termHessian[i][j] = (tempMatrix[i][0]*tempMatrix[j][0]+tempMatrix[i][1]*tempMatrix[j][1]
						+tempMatrix[i][2]*tempMatrix[j][2])*weight;
					}
				}
				for(i = 0; i < 15; i++)
				{
					for(j = i+1; j < 15; j++)
					{
						termHessian[j][i] = termHessian[i][j];
					}
				}
				///	Update Gradient
				UpdateVector_by_a_vector(termGradient, m_indices, 15, *linApprox);
				///	Update Hessian
				UpdateMatrix_by_a_matrix(termHessian, m_indices, 15, *quadApprox);
			}
		}
	}
}

void		BOPNormalRigidityPotential::peter_UpdateGradientandHessian(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32	numOfNeighbors,
	const int	*neighbor_buffer,
	SparseMatrixF *quadApprox, 
	DVectorF *linApprox
	)
{
	pAssert(neighbor_buffer != NULL);

	float32	weight_normal	= context->weightNormalRigidity,
					lambda				=	context->lambda;

	card32	width              = patchGraph->getTrajectorySet()->getNumFrames(),
					height             = patchGraph->getTrajectorySet()->getNumPatches();

	PointSet		*ps					= patchGraph->getTrajectorySet()->getTrajectories();
	AAT		POSITION				= ps->getAAT("position");
	AAT		NORMAL					= ps->getAAT("normal");
	AAT		ROTATION				= ps->getAAT("rotation_matrix");
	AAT		REF_U						= ps->getAAT("tangent_u");
	AAT		POS_INDEX			= ps->getAAT("varIndex_pos");
	AAT		NOR_INDEX			= ps->getAAT("varIndex_nor");
	AAT		ROT_INDEX			= ps->getAAT("varIndex_rot");

	/// The term gradient, termHessian and temporary matrix 
	DVectorF			termGradient;		termGradient.setDim(23);
	DMatrixF				termHessian;			termHessian.setDimension(23, 23, false);
	DMatrixF				tempMatrix;			tempMatrix.setDimension(23, 3, false);

	/// The variable index transform
	DVectorI				m_indices;				m_indices.setDim(23);

	/// Current term rotation matrix
	Matrix3f				Ajk;

	/// We don't have rotation matrix for the last frame
	for(card32 frame = 0; frame < width-1; frame++)
	{
		for(card32 patchIndex = 0; patchIndex <height; patchIndex++)
		{
			int			ik_posid		= ps->get<int32>(frame,	 patchIndex, POS_INDEX);
			int			ikn_posid	= ps->get<int32>(frame+1, patchIndex, POS_INDEX);
			int			ik_norid		= ps->get<int32>(frame,	 patchIndex, NOR_INDEX);
			int			ikn_norid		= ps->get<int32>(frame+1, patchIndex, NOR_INDEX);

			card32		k;
			for(k = 0; k <3 ; k++)
			{
				m_indices[k   ]		= ik_posid		+ k;
				m_indices[6+k]		= ikn_posid		+	k;
			}
			for(k = 0; k <2 ; k++)
			{
				m_indices[12+k]	= ik_norid		+	k;
				m_indices[16+k]	= ikn_norid		+	k;
			}

			Vector3f		pik				= ps->get3f(frame   , patchIndex, POSITION);
			Vector3f		pikn				= ps->get3f(frame+1, patchIndex, POSITION);
			Vector3f		nik				= ps->get3f(frame   , patchIndex, NORMAL);
			Vector3f		nikn				= ps->get3f(frame+1, patchIndex, NORMAL);

			Vector3f		Eik1				= ps->get3f(frame   , patchIndex, REF_U);
			Vector3f		Eik2				= nik.crossProduct(Eik1);
			Vector3f		Eikn1			= ps->get3f(frame+1, patchIndex, REF_U);
			Vector3f		Eikn2			= nikn.crossProduct(Eikn1);

			card32		offset			= (frame*height + patchIndex)*numOfNeighbors;
			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				if(neighbor_buffer[ni] >= 0)
				{
					int		jk_posid		= ps->get<int32>(frame,		neighbor_buffer[ni], POS_INDEX);
					int		jkn_posid	= ps->get<int32>(frame+1,   neighbor_buffer[ni], POS_INDEX);
					int		jk_norid		= ps->get<int32>(frame,		neighbor_buffer[ni], NOR_INDEX);
					int		jkn_norid		= ps->get<int32>(frame+1,   neighbor_buffer[ni], NOR_INDEX);
					int		jk_rotid		= ps->get<int32>(frame,		neighbor_buffer[ni], ROT_INDEX);

					///Check whether we need further operations
					if(ik_posid < 0 && jk_posid < 0 && ikn_posid < 0 && jkn_posid < 0 &&
						ik_norid < 0 && jk_norid < 0  && ikn_norid < 0 && jkn_norid < 0  && jk_rotid < 0)
						continue;

					for(k = 0; k <3 ; k++)
					{
						m_indices[3+k ]	= jk_posid		+ k;
						m_indices[9+k ]	= jkn_posid	+	k;
						m_indices[20+k]	= jk_rotid		+ k;
					}
					for(k = 0; k <2 ; k++)
					{
						m_indices[14+k]	= jk_norid		+	k;
						m_indices[18+k]	= jkn_norid		+	k;
					}
					//get current rotation matrix
					char			*dataC			= ps->getDataPointer(frame, neighbor_buffer[ni]);
					float32	*quatc				= (float*)(dataC+ROTATION.getOffset());

					Quaternion_To_RotationMatrix(quatc, Ajk);


					Vector3f		pjk		= ps->get3f(frame,		neighbor_buffer[ni], POSITION);
					Vector3f		pjkn		=	ps->get3f(frame+1, neighbor_buffer[ni], POSITION);
					Vector3f		njk		= ps->get3f(frame,		neighbor_buffer[ni], NORMAL);
					Vector3f		njkn		=	ps->get3f(frame+1, neighbor_buffer[ni], NORMAL);

					Vector3f		rijk		= pik - pjk;					Vector3f	rijkn				= pikn - pjkn;
					Vector3f		nijk		= nik - njk;					Vector3f	nijkn			= nikn - njkn;

					Vector3f		Ejk1		= ps->get3f(frame,		neighbor_buffer[ni], REF_U);
					Vector3f		Ejk2		= njk.crossProduct(Ejk1);
					Vector3f		Ejkn1	= ps->get3f(frame+1, neighbor_buffer[ni], REF_U);
					Vector3f		Ejkn2	=	njkn.crossProduct(Ejkn1);

					Vector3f	AjkEik1	= Ajk*Eik1;				Vector3f	AjkEik2		=	Ajk*Eik2;
					Vector3f	AjkEjk1	= Ajk*Ejk1;				Vector3f	AjkEjk2		=	Ajk*Ejk2;
					Vector3f		nijkc   = Ajk*nijk;
					Vector3f		dev		= nijkc - nijkn;

					float32		sqrDev= dev.getSqrNorm();
					float32		weight= weight_normal*exp(lambda*(rijk.getSqrNorm()+rijkn.getSqrNorm())/2);

					//Compute tempMatirx
					card32	i, j;
					for(j = 0; j < 3; j++)
					{
						for(i = 0; i < 3; i++)
						{
							tempMatrix[i		][j]	= rijk[i]*dev[j]*lambda/2;
							tempMatrix[i+3][j]	= - tempMatrix[i][j];
							tempMatrix[i+6][j]	= rijkn[i]*dev[j]*lambda/2;
							tempMatrix[i+9][j]	= - tempMatrix[i+6][j];
						}
						tempMatrix[12][j]			= AjkEik1[j];
						tempMatrix[13][j]			= AjkEik2[j];
						tempMatrix[14][j]			= -AjkEjk1[j];
						tempMatrix[15][j]			= -AjkEjk2[j];
						tempMatrix[16][j]			= -Eikn1[j];
						tempMatrix[17][j]			= -Eikn2[j];
						tempMatrix[18][j]			= Ejkn1[j];
						tempMatrix[19][j]			= Ejkn2[j];
					}
					tempMatrix[20][0]	=	  0.0f;			tempMatrix[20][1]	= - nijkc[2];	tempMatrix[20][2]	=   nijkc[1];
					tempMatrix[21][0]	=	  nijkc[2];	tempMatrix[21][1]	=   0.0f;			tempMatrix[21][2]	= - nijkc[0];
					tempMatrix[22][0]	= - nijkc[1];	tempMatrix[22][1]	=  nijkc[0];	tempMatrix[22][2]	=   0.0f;

					for(i = 0; i < 23; i++)
					{
						termGradient[i] = -(tempMatrix[i][0]*dev[0]+tempMatrix[i][1]*dev[1]+tempMatrix[i][2]*dev[2]);
						for(j = 0; j < 23; j++)
						{
							termHessian[i][j] = tempMatrix[i][0]*tempMatrix[j][0]+tempMatrix[i][1]*tempMatrix[j][1]
							+tempMatrix[i][2]*tempMatrix[j][2];
						}
					}
					///	Update Gradient
					UpdateVector_by_a_vector(weight, termGradient, m_indices, *linApprox);
					///	Update Hessian
					UpdateMatrix_by_a_matrix(weight, termHessian, m_indices, *quadApprox);
				}
			}
		}
	}
}
/************************************************************************/
/* peter_ComputeEnergy                                                                     */
/************************************************************************/
void		BOPSquaredDistancePotential::peter_ComputeEnergy(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32 numOfNeighbors, 
	const int *neighbor_buffer, 
	const DVectorF deviatioon, 
	float &energy, 
	bool m_flag)
{

}

void		BOPMarkerErrorPotential::peter_ComputeEnergy(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32 numOfNeighbors, 
	const int *neighbor_buffer, 
	const DVectorF perturb, 
	float &energy, 
	bool m_flag)
{
	PointSet*					ps	= patchGraph->getTrajectorySet()->getTrajectories();
	PGDataPointIndices *dataPoints	= patchGraph->getTrajectorySet()->getDataPoints();
	AAT COEFF;

	AAT POSITION					= ps->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT VARINDEX_POS			= ps->getAAT( "varIndex_pos", 1, VAD::DATA_FORMAT_INT32 );
	AAT DATA_POS_AAT		= data->getFrame(0)->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );

	float	weightmarker			= context->weightMarkerError;
	card32	width					= patchGraph->getTrajectorySet()->getNumFrames(),
					height					= patchGraph->getTrajectorySet()->getNumPatches();

	for(card32 frame = 0; frame < width; frame++)
	{
		card32	dataPointsPerframe = 0, point;
		for(point = 0; point < height; point++)
		{
			const card32 numDataPoints = dataPoints->getNumDataPoints( frame, point );
			if (numDataPoints > 0) 
				dataPointsPerframe++;
		}

		if(dataPointsPerframe == 0)
			continue;

		float		weight = weightmarker*height/dataPointsPerframe;
		for(card32 point = 0; point < height; point++)
		{
			int32 ik_posid = ps->get<int32>(frame, point, VARINDEX_POS);
			
			card32	k;
			
			Vector3f pik = ps->get3f( frame, point, POSITION );
			if(m_flag)
			{
				for(k = 0; k < 3; k++)
				{
					pik[k] += perturb[ik_posid + k];
				}
			}
			const card32 numDataPoints	= dataPoints->getNumDataPoints( frame, point);

			if (numDataPoints > 0) 
			{
				card32 dataIndexInData	= dataPoints->getIndex( frame, point, 0);
				Vector3f devik			= pik - data->getFrame( frame )->get3f(dataIndexInData, DATA_POS_AAT);
				energy						+= weight*devik.getSqrNorm(); 
			}
		}
	}
}

void		BOPPlanarSmoothnessPotential::peter_ComputeEnergy(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32 numOfNeighbors, 
	const int *neighbor_buffer, 
	const DVectorF perturb, 
	float &energy, 
	bool m_flag)
{
	pAssert(neighbor_buffer != NULL);
	float32	weight_planar	= context->weightPlanarSmoothness,
					lambda				=	context->lambda;

	card32	width					= patchGraph->getTrajectorySet()->getNumFrames(),
					height					= patchGraph->getTrajectorySet()->getNumPatches();

	PointSet	*ps						= patchGraph->getTrajectorySet()->getTrajectories();
	AAT	POSITION					= ps->getAAT("position");
	AAT	NORMAL						= ps->getAAT("normal");
	AAT	REF_U							= ps->getAAT("tangent_u");
	AAT	POS_INDEX				= ps->getAAT("varIndex_pos");
	AAT	NOR_INDEX				= ps->getAAT("varIndex_nor");

	for(card32 frame = 0; frame < width; frame++)
	{
		for(card32 point = 0; point < height; point++)
		{
			int				ik_posid	= ps->get<int32>(frame, point, POS_INDEX);
			int				ik_norid	= ps->get<int32>(frame, point, NOR_INDEX);
			card32		k;

			Vector3f		pik			= ps->get3f(frame, point, POSITION);
			if(m_flag)
			{
				for(k = 0; k < 3; k++)
				{
					pik[k]	+= perturb[ik_posid + k];
				}
			}
			Vector3f		nik			= ps->get3f(frame, point, NORMAL);
			if(m_flag)
			{
				Vector3f		Eik1	= ps->get3f(frame, point, REF_U);
				Vector3f		Eik2	= nik.crossProduct(Eik1);
				nik						+=	Eik1*perturb[ik_norid]	+ Eik2*perturb[ik_norid+1];
				nik.normalize();
			}
			card32	offset		= (frame*height + point)*numOfNeighbors;

			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				int	jk_posid	= ps->get<int32>(frame,  neighbor_buffer[ni], POS_INDEX);

				Vector3f	pjk		=	ps->get3f(frame, neighbor_buffer[ni], POSITION);
				if(m_flag)
				{
					for(k = 0; k < 3; k++)
					{
						pjk[k]			+= perturb[jk_posid + k];
					}
				}
				Vector3f		rijk	=	pik - pjk;
				float32		dis	=	nik*rijk;
				float32	weight	=	weight_planar*exp(lambda*rijk.getSqrNorm())/numOfNeighbors;
				energy				+=	weight*dis*dis;
			}
		}
	}
}

void		BOPNormalSmoothnessPotential::peter_ComputeEnergy(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32 numOfNeighbors, 
	const int *neighbor_buffer, 
	const DVectorF perturb, 
	float &energy, 
	bool m_flag)
{
	pAssert(neighbor_buffer != NULL);
	float32	weight_normal	= context->weightNormalSmoothness,
					lambda				= context->lambda;

	card32		width           = patchGraph->getTrajectorySet()->getNumFrames(),
						height				= patchGraph->getTrajectorySet()->getNumPatches();

	PointSet	*ps						= patchGraph->getTrajectorySet()->getTrajectories();
	AAT	POSITION					= ps->getAAT("position");
	AAT	NORMAL						= ps->getAAT("normal");
	AAT	REF_U							= ps->getAAT("tangent_u");
	AAT	POS_INDEX				= ps->getAAT("varIndex_pos");
	AAT	NOR_INDEX				= ps->getAAT("varIndex_nor");

	for(card32 frame = 0; frame < ps->getWidth(); frame++)
	{
		for(card32 point = 0; point < ps->getHeight(); point++)
		{
			int				ik_posid	= ps->get<int32>(frame, point, POS_INDEX);
			int				ik_norid	= ps->get<int32>(frame, point, NOR_INDEX);

			Vector3f		pik			= ps->get3f(frame, point, POSITION);
			card32		k;
			if(m_flag)
			{
				for(k = 0; k < 3; k++)
				{
					pik[k]	+= perturb[ik_posid + k];
				}
			}
			Vector3f		nik			= ps->get3f(frame, point, NORMAL);
			if(m_flag)
			{
				Vector3f		Eik1		= ps->get3f(frame, point, REF_U);
				Vector3f		Eik2		= nik.crossProduct(Eik1);
				nik							+=	Eik1*perturb[ik_norid]	+ Eik2*perturb[ik_norid + 1];
				nik.normalize();
			}
			card32			offset	= (frame*height + point)*numOfNeighbors;

			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				int		jk_posid	= ps->get<int32>(frame,  neighbor_buffer[ni], POS_INDEX);
				int		jk_norid	= ps->get<int32>(frame,  neighbor_buffer[ni], NOR_INDEX);

				Vector3f	pjk		= ps->get3f(frame, neighbor_buffer[ni], POSITION);
				if(m_flag)
				{
					for(k = 0; k < 3; k++)
					{
						pjk[k]			+= perturb[jk_posid + k];
					}
				}
				Vector3f	rijk		= pik - pjk;
				Vector3f	njk		= ps->get3f(frame, neighbor_buffer[ni], NORMAL);
				if(m_flag)
				{
					Vector3f Ejk1	= ps->get3f(frame, neighbor_buffer[ni], REF_U);
					Vector3f Ejk2	= njk.crossProduct(Ejk1);
					njk					+=	Ejk1*perturb[jk_norid] + Ejk2*perturb[jk_norid+1];
					njk.normalize();
				}
				Vector3f	 nijk		= nik - njk;
				float32	weight	= weight_normal*exp(lambda*rijk.getSqrNorm())/numOfNeighbors;
				energy				+= weight*nijk.getSqrNorm();
			}
		}
	}
}

void		BOPCircularSmoothnessPotential::peter_ComputeEnergy(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32 numOfNeighbors, 
	const int *neighbor_buffer, 
	const DVectorF perturb, 
	float &energy, 
	bool m_flag)
{
	pAssert(neighbor_buffer != NULL);
	float32	weight_circular	= context->weightCircularSmoothness,
						lambda			=	context->lambda;

	card32		width           = patchGraph->getTrajectorySet()->getNumFrames(),
						height          = patchGraph->getTrajectorySet()->getNumPatches();

	PointSet			*ps				= patchGraph->getTrajectorySet()->getTrajectories();
	AAT			POSITION			= ps->getAAT("position");
	AAT			NORMAL				= ps->getAAT("normal");
	AAT			REF_U					= ps->getAAT("tangent_u");
	AAT			POS_INDEX		= ps->getAAT("varIndex_pos");
	AAT			NOR_INDEX		= ps->getAAT("varIndex_nor");

	for(card32 frame = 0; frame < width; frame++)
	{
		for(card32 point = 0; point < height; point++)
		{
			int				ik_posid	= ps->get<int32>(frame, point, POS_INDEX);
			int				ik_norid	= ps->get<int32>(frame, point, NOR_INDEX);

			card32			k;

			Vector3f		pik			= ps->get3f(frame, point, POSITION);
			if(ik_posid >= 0&&m_flag)
			{
				for(k = 0; k < 3; k++)
				{
					pik[k] +=	perturb[ik_posid + k];
				}
			}
			Vector3f		nik			= ps->get3f(frame, point, NORMAL);
			if(ik_norid >= 0&&m_flag)
			{
				Vector3f		Eik1	= ps->get3f(frame, point, REF_U);
				Vector3f		Eik2	= nik.crossProduct(Eik1);
				nik		+=	Eik1*perturb[ik_norid]	+	Eik2*perturb[ik_norid + 1];
				nik.normalize();
			}

			card32		offset			= (frame*height + point)*numOfNeighbors;
			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				if(neighbor_buffer[ni] >= 0)
				{
					int		jk_posid		= ps->get<int32>(frame,  neighbor_buffer[ni], POS_INDEX);
					int		jk_norid		= ps->get<int32>(frame,  neighbor_buffer[ni], NOR_INDEX);

					if(ik_posid < 0 && jk_posid && ik_norid < 0 && jk_norid < 0)
						continue;

					Vector3f	pjk			= ps->get3f(frame, neighbor_buffer[ni], POSITION);
					if(jk_posid	>= 0&&m_flag)
					{
						for(k = 0; k < 3; k++)
						{
							pjk[k] += perturb[jk_posid + k];
						}
					}
					Vector3f	njk			= ps->get3f(frame, neighbor_buffer[ni], NORMAL);
					if(jk_norid >= 0&&m_flag)
					{
						Vector3f		Ejk1	= ps->get3f(frame, neighbor_buffer[ni], REF_U);
						Vector3f		Ejk2	= njk.crossProduct(Ejk1);
						njk						+=	Ejk1*perturb[jk_norid]	+ Ejk2*perturb[jk_norid + 1];
						njk.normalize();
					}
					
					Vector3f		rijk		= pik - pjk;
					Vector3f		nijk		= nik + njk;
					float32		dis		= rijk*nijk;
					float32		weight	= weight_circular*exp(lambda*rijk.getSqrNorm())/numOfNeighbors;
					energy					+=	weight*dis*dis;
				}
			}
		}
	}
}

void		BOPLaplacianSmoothnessPotential::peter_ComputeEnergy(const BSROptContext *context, 
																			  const PointCloudAnimationSequence *data, 
																			  PatchGraph *patchGraph, 
																			  const card32 numOfNeighbors, 
																			  const int *neighbor_buffer, 
																			  const DVectorF perturb, 
																			  float &energy, 
																			  bool flag)
{
	pAssert(neighbor_buffer != NULL);
	float32	weight_laplacian	= context->weightLaplacianSmoothness,
						lambda				=	context->lambda;

	card32			width				= patchGraph->getTrajectorySet()->getNumFrames(),
							height				= patchGraph->getTrajectorySet()->getNumPatches();

	PointSet	*ps							= patchGraph->getTrajectorySet()->getTrajectories();
	AAT	POSITION						= ps->getAAT("position");
	AAT	POS_INDEX					= ps->getAAT("varIndex_pos");


	for(card32 frame = 0; frame < width; frame++)
	{
		for(card32 point = 0; point < height; point++)
		{
			int			ik_posid			= ps->get<int32>(frame, point, POS_INDEX);

			card32		k;
			Vector3f			pik			= ps->get3f(frame, point, POSITION);
			bool					m_flag		= false;
			if(ik_posid >= 0&&m_flag)
			{
				m_flag							= true;
				for(k = 0; k < 3; k++)
				{
					pik[k]						+= perturb[ik_posid + k];
				}
			}

			Vector3f		devik;	devik[0] = devik[1] = devik[2] = 0.0f;
			card32			count		= 0;
			float			asqrLen		= 0.0;
			card32			offset		= (frame*height + point)*numOfNeighbors;
			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				if(neighbor_buffer[ni] >= 0)
				{
					count++;
					int			jk_posid	= ps->get<int32>(frame,  neighbor_buffer[ni], POS_INDEX);
					Vector3f	pjk			= ps->get3f(frame, neighbor_buffer[ni], POSITION);
					if(jk_posid >= 0&&m_flag)
					{
						m_flag					= true;
						for(k = 0; k < 3; k++)
						{
							pjk[k]				+= perturb[jk_posid + k];
						}
					}
					Vector3f		rijk		= pik - pjk;
					devik						+= rijk;
					asqrLen					+= rijk.getSqrNorm();
				}
			}
			if(!m_flag)
				continue;

			devik			/= count;			
			asqrLen		/= count;
			float			weight = weight_laplacian*exp(lambda*asqrLen);
			energy		+= weight*devik.getSqrNorm();	
		}
	}
}

void		BOPPositionRigidityPotential::peter_ComputeEnergy(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32 numOfNeighbors, 
	const int *neighbor_buffer, 
	const DVectorF perturb, 
	float &energy, 
	bool m_flag)
{
	pAssert(neighbor_buffer != NULL);
	float32	weight_position	= context->weightPositionRigidity,
						lambda			=	context->lambda;

	card32		width           = patchGraph->getTrajectorySet()->getNumFrames(),
						height          = patchGraph->getTrajectorySet()->getNumPatches();

	PointSet		*ps					= patchGraph->getTrajectorySet()->getTrajectories();
	AAT			POSITION			= ps->getAAT("position");
	AAT			ROTATION			= ps->getAAT("rotation_matrix");
	AAT			POS_INDEX		= ps->getAAT("varIndex_pos");
	AAT			ROT_INDEX		= ps->getAAT("varIndex_rot");

	/// Current term rotation matrix
	Matrix3f								Aik;

	/// We don't have rotation matrix for the last frame
	for(card32 frame = 0; frame < width-1; frame++)
	{
		for(card32 point = 0; point < height; point++)
		{
			int				ik_posid		= ps->get<int32>(frame,	 point, POS_INDEX);
			int				ikn_posid	= ps->get<int32>(frame+1, point, POS_INDEX);
			int				ik_rotid		= ps->get<int32>(frame,	 point, ROT_INDEX);

			card32		k;
			//get current rotation matrix
			char			*dataC	= ps->getDataPointer(frame, point);
			float32	*quatc	= (float*)(dataC+ROTATION.getOffset());

			if(m_flag)
			{
				float32	theta	= sqrt(perturb[ik_rotid]*perturb[ik_rotid] 
				+ perturb[ik_rotid + 1]*perturb[ik_rotid + 1] 
				+ perturb[ik_rotid + 2]*perturb[ik_rotid + 2]);

				float32			quat[4], quatn[4], sina;
				sina						= sin(theta/2);
				quat[0]				= cos(theta/2);
				if(theta > 1e-6)
				{
					for(k = 0; k < 3; k++)
					{
						quat[k + 1]	= sina*perturb[ik_rotid + k]/theta;
					}
				}
				else
				{
					quat[1] = sina;			quat[2] = 0.0f;			quat[3] = 0.0f;
				}
				quatn[0] = quat[0]*quatc[0] - quat[1]*quatc[1] - quat[2]*quatc[2] - quat[3]*quatc[3];
				quatn[1] = quat[0]*quatc[1] + quat[1]*quatc[0] + quat[2]*quatc[3] - quat[3]*quatc[2];
				quatn[2] = quat[0]*quatc[2] + quat[2]*quatc[0] + quat[3]*quatc[1] - quat[1]*quatc[2];
				quatn[3] = quat[0]*quatc[3] + quat[3]*quatc[0] + quat[1]*quatc[2] - quat[2]*quatc[1];
				theta = sqrt(quatn[0]*quatn[0]+quatn[1]*quatn[1]+quatn[2]*quatn[2]+quatn[3]*quatn[3]);
				quatn[0] /= theta;	quatn[1] /= theta;	quatn[2] /= theta;	quatn[3] /= theta;
				Quaternion_To_RotationMatrix(quatn, Aik);
			}
			else
			{
				Quaternion_To_RotationMatrix(quatc, Aik);
			}
			Vector3f		pik				= ps->get3f(frame   , point, POSITION);
			if(m_flag)
			{
				for(k = 0; k < 3; k++)
				{
					pik[k]						+= perturb[ik_posid + k];
				}
			}
			Vector3f			pikn			= ps->get3f(frame+1, point, POSITION);
			if(m_flag)
			{
				for(k = 0; k < 3; k++)
				{
					pikn[k]					+= perturb[ikn_posid + k];
				}
			}
			card32		offset			= (frame*height + point)*numOfNeighbors;
			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				int		jk_posid		= ps->get<int32>(frame,		neighbor_buffer[ni], POS_INDEX);
				int		jkn_posid	= ps->get<int32>(frame+1,   neighbor_buffer[ni], POS_INDEX);

				Vector3f		pjk		= ps->get3f(frame,		neighbor_buffer[ni], POSITION);
				if(jk_posid >= 0&&m_flag)
				{
					for(k = 0; k < 3; k++)
					{
						pjk[k]				+= perturb[jk_posid + k];
					}
				}
				Vector3f		pjkn		= ps->get3f(frame+1, neighbor_buffer[ni], POSITION);
				if(jkn_posid >= 0&&m_flag)
				{
					for(k = 0; k < 3; k++)
					{
						pjkn[k]			+= perturb[jkn_posid + k];
					}
				}
				Vector3f		rijk		= pik - pjk;					
				Vector3f		rijkn		= pikn - pjkn;

				Vector3f		rijkc			= (Aik.transpose())*rijk;
				Vector3f		dev			= rijkc - rijkn;

				float32		sqrDev	= dev.getSqrNorm();
				float32		weight		= weight_position*exp(lambda*(rijk.getSqrNorm()+rijkn.getSqrNorm())/2)/numOfNeighbors;
				energy						+= weight*sqrDev;
			}
		}
	}
}

void		BOPNormalRigidityPotential::peter_ComputeEnergy(const BSROptContext *context, 
																			const PointCloudAnimationSequence *data, 
																			PatchGraph *patchGraph, 
																			const card32 numOfNeighbors, 
																			const int *neighbor_buffer, 
																			const DVectorF perturb, 
																			float &energy, 
																			bool m_flag)
{
	pAssert(neighbor_buffer != NULL);

	float32	weight_normal	= context->weightNormalRigidity,
					lambda				=	context->lambda;

	card32	width              = patchGraph->getTrajectorySet()->getNumFrames(),
					height             = patchGraph->getTrajectorySet()->getNumPatches();

	PointSet	*ps		= patchGraph->getTrajectorySet()->getTrajectories();
	AAT	POSITION	= ps->getAAT("position");
	AAT	NORMAL		= ps->getAAT("normal");
	AAT	ROTATION	= ps->getAAT("rotation_matrix");
	AAT	REF_U			= ps->getAAT("tangent_u");
	AAT	POS_INDEX= ps->getAAT("varIndex_pos");
	AAT	NOR_INDEX= ps->getAAT("varIndex_nor");
	AAT	ROT_INDEX= ps->getAAT("varIndex_rot");

	/// Current term rotation matrix
	Matrix3f				Ajk;

	/// We don't have rotation matrix for the last frame
	for(card32 frame = 0; frame < ps->getWidth()-1; frame++)
	{
		for(card32 point = 0; point < ps->getHeight(); point++)
		{
			card32		k;
			int			ik_posid		= ps->get<int32>(frame,	 point, POS_INDEX);
			int			ikn_posid	= ps->get<int32>(frame+1, point, POS_INDEX);
			int			ik_norid		= ps->get<int32>(frame,	 point, NOR_INDEX);
			int			ikn_norid		= ps->get<int32>(frame+1, point, NOR_INDEX);
			
			Vector3f		pik			= ps->get3f(frame   , point, POSITION);
			if(ik_posid >= 0&&m_flag)
			{
				for(k = 0; k < 3; k++)
					pik[k]					+= perturb[ik_posid + k];
			}
			Vector3f		pikn			= ps->get3f(frame+1, point, POSITION);
			if(ikn_posid >= 0&&m_flag)
			{
				for(k = 0; k < 3; k++)
					pikn[k]				+= perturb[ikn_posid + k];
			}
			Vector3f		nik			= ps->get3f(frame   , point, NORMAL);
			if(ik_norid >= 0&&m_flag)
			{
				Vector3f		Eik1		= ps->get3f(frame   , point, REF_U);
				Vector3f		Eik2		= nik.crossProduct(Eik1);
				nik							+= Eik1*perturb[ik_norid] + Eik2*perturb[ik_norid+1];
				nik.normalize();
			}
			Vector3f		nikn			= ps->get3f(frame+1, point, NORMAL);
			if(ikn_norid >= 0&&m_flag)
			{
				Vector3f		Eikn1	= ps->get3f(frame+1, point, REF_U);
				Vector3f		Eikn2	= nikn.crossProduct(Eikn1);
				nikn							+= Eikn1*perturb[ikn_norid] + Eikn2*perturb[ikn_norid+1];
				nikn.normalize();
			}
			
			card32		offset		= (frame*height + point)*numOfNeighbors;
			for(card32 ni = offset; ni < offset + numOfNeighbors; ni++)
			{
				if(neighbor_buffer[ni] >= 0)
				{
					int		jk_posid		= ps->get<int32>(frame,		neighbor_buffer[ni], POS_INDEX);
					int		jkn_posid	= ps->get<int32>(frame+1,   neighbor_buffer[ni], POS_INDEX);
					int		jk_norid		= ps->get<int32>(frame,		neighbor_buffer[ni], NOR_INDEX);
					int		jkn_norid		= ps->get<int32>(frame+1,   neighbor_buffer[ni], NOR_INDEX);
					int		jk_rotid		= ps->get<int32>(frame,		neighbor_buffer[ni], ROT_INDEX);

					///Check whether we need further operations
					if(ik_posid < 0 && jk_posid < 0 && ikn_posid < 0 && jkn_posid < 0 &&
						ik_norid < 0 && jk_norid < 0  && ikn_norid < 0 && jkn_norid < 0  && jk_rotid < 0)
						continue;

					Vector3f	pjk			= ps->get3f(frame,		neighbor_buffer[ni], POSITION);
					if(jk_posid >= 0&&m_flag)
					{
						for(k = 0; k < 3; k++)
							pjk[k]				+= perturb[jk_posid + k];
					}
					Vector3f		pjkn		= ps->get3f(frame+1, neighbor_buffer[ni], POSITION);
					if(jkn_posid >= 0&&m_flag)
					{
						for(k = 0; k < 3; k++)
							pjkn[k]			+= perturb[jkn_posid + k];
					}
					Vector3f		rijk		= pik - pjk;					
					Vector3f		rijkn		= pikn - pjkn;

					Vector3f		njk		= ps->get3f(frame,		neighbor_buffer[ni], NORMAL);
					if(jk_norid >= 0&&m_flag)
					{
						Vector3f		Ejk1	= ps->get3f(frame,		neighbor_buffer[ni], REF_U);
						Vector3f		Ejk2	= njk.crossProduct(Ejk1);
						njk						+= Ejk1*perturb[jk_norid] + Ejk2*perturb[jk_norid+1];
						njk.normalize();
					}
					Vector3f	njkn			= ps->get3f(frame+1, neighbor_buffer[ni], NORMAL);
					if(jkn_posid >= 0&&m_flag)
					{
						Vector3f	Ejkn1	= ps->get3f(frame+1, neighbor_buffer[ni], REF_U);
						Vector3f	Ejkn2	= njkn.crossProduct(Ejkn1);
						njkn						+= Ejkn1*perturb[jkn_norid] + Ejkn2*perturb[jkn_norid+1];
						njkn.normalize();
					}
					Vector3f	nijk			= nik - njk;
					Vector3f	nijkn		= nikn - njkn;

					char			*dataC	= ps->getDataPointer(frame, neighbor_buffer[ni]);
					float32	*quatc	= (float*)(dataC+ROTATION.getOffset());

					if(jk_rotid >= 0&&m_flag)
					{
						float32	theta	= sqrt(perturb[jk_rotid]*perturb[jk_rotid] 
						+ perturb[jk_rotid + 1]*perturb[jk_rotid + 1] 
						+ perturb[jk_rotid + 2]*perturb[jk_rotid + 2]);

						float32			quat[4], quatn[4], sina;
						sina						= sin(theta/2);
						quat[0]				= cos(theta/2);
						if(theta > 1e-6)
						{
							for(k = 0; k < 3; k++)
							{
								quat[k + 1]= sina*perturb[jk_rotid + k]/theta;
							}
						}
						else
						{
							quat[1] = sina;			quat[2] = 0.0f;			quat[3] = 0.0f;
						}
						quatn[0] = quat[0]*quatc[0] - quat[1]*quatc[1] - quat[2]*quatc[2] - quat[3]*quatc[3];
						quatn[1] = quat[0]*quatc[1] + quat[1]*quatc[0] + quat[2]*quatc[3] - quat[3]*quatc[2];
						quatn[2] = quat[0]*quatc[2] + quat[2]*quatc[0] + quat[3]*quatc[1] - quat[1]*quatc[2];
						quatn[3] = quat[0]*quatc[3] + quat[3]*quatc[0] + quat[1]*quatc[2] - quat[2]*quatc[1];
						theta = sqrt(quatn[0]*quatn[0]+quatn[1]*quatn[1]+quatn[2]*quatn[2]+quatn[3]*quatn[3]);
						quatn[0] /= theta;	quatn[1] /= theta;	quatn[2] /= theta;	quatn[3] /= theta;
						Quaternion_To_RotationMatrix(quatn, Ajk);
					}
					else
					{
						Quaternion_To_RotationMatrix(quatc, Ajk);
					}
					
					Vector3f		nijkc   = Ajk*nijk;
					Vector3f		dev		= nijkc - nijkn;
					float			weight	= weight_normal*exp(lambda*(rijk.getSqrNorm()+rijkn.getSqrNorm())/2)/numOfNeighbors;
					energy					+= weight*dev.getSqrNorm();
				}
			}
		}
	}
}

/************************************************************************/
/* peter_writeIndexingChannel                                                                     */
/************************************************************************/

void BOPIndexingChannel::peter_writeIndexingChannel(const card32 offset, 
																	const std::string name, 
																	const BSROptVariableIndexSet *indices,
																	PatchGraph *patchGraph) 

{
	PointSet		*ps = patchGraph->getTrajectorySet()->getTrajectories();
	PointSet		*urshape = patchGraph->getUrshape();
	pAssert(urshape == NULL || ps->getDescr()->equal(urshape->getDescr()));

	card32		numOfVairsPerpoint;
	if(name == "varIndex_pos")
	{
		numOfVairsPerpoint = 3;
	}
	else if(name == "varIndex_nor")
	{
		numOfVairsPerpoint = 2;
	}
	else if(name == "varIndex_rot")
	{
		numOfVairsPerpoint = 3;
	}
	else
	{
		return;
	}

	AAT VARINDEX = ps->getAAT(name);
	
	//Initially, all the indices are set to zero
	for ( card32 frame = 0; frame < ps->getWidth(); frame++) 
	{
		for ( unsigned int patchIndex = 0; patchIndex < ps->getHeight(); patchIndex++ )
		{
			ps->set<int32>(frame, patchIndex, VARINDEX, -3);
		}
	}

	const size_t n = indices->size();
	for (unsigned i=0; i<n; i++) 
	{
		const int32 frame = (*indices)[i].frame;
		const card32 point = (*indices)[i].point;
		if ( frame != BSROPT_FRAME_URSHAPE ) 
		{
			ps->set<int32>(frame, point, VARINDEX, int32(offset + i*numOfVairsPerpoint));
		} 
		else 
		{
			urshape->set<int32>(point, VARINDEX, int32(offset + i*numOfVairsPerpoint));
		}
	}	
}

void	BOPIndexingChannel::peter_readChannel(const geoxl::BSROptVariableIndexSet *indices, 
														  const geoxl::PatchGraph *patchGraph, 
														  const geoxl::card32 offset, 
														  const std::string channel, 
														  const geoxl::card32 channelDim, 
														  geoxl::DVectorF &result)
{
	const PointSet *ps = patchGraph->getTrajectorySet()->getTrajectories();
	const PointSet *urshape = patchGraph->getUrshape();
	pAssert(urshape == NULL || ps->getDescr()->equal(urshape->getDescr()));
	AAT channelAAT = ps->getAAT(channel, channelDim, VAD::DATA_FORMAT_FLOAT32);

	const size_t n = indices->size();

	for (unsigned i=0; i<n; i++) 
	{
		const int32	frame = (*indices)[i].frame;
		const card32 point = (*indices)[i].point;
		const char		*data;
		if ( frame != BSROPT_FRAME_URSHAPE ) 
		{
			data = ps->getDataPointer(frame, point);
		} 
		else 
		{
			data = urshape->getDataPointer(point);
		}
		float* dataF = (float*)(data+channelAAT.getOffset());
		for (card32 c=0; c<channelDim; c++) {
			result[offset + i*channelDim+c] = dataF[c];
		}
	}	
}

void	BOPIndexingChannel::peter_writeChannel(const BSROptVariableIndexSet *indices, 
															 PatchGraph *patchGraph, 
															 const card32 offset, 
															 const string channel, 
															 const card32 channelDim, 
															 const DVectorF &data )
{
	PointSet *ps = patchGraph->getTrajectorySet()->getTrajectories();
	PointSet *urshape = patchGraph->getUrshape();
	pAssert(urshape == NULL || ps->getDescr()->equal(urshape->getDescr()));
	AAT channelAAT = ps->getAAT(channel, channelDim, VAD::DATA_FORMAT_FLOAT32);

	const size_t n = indices->size();
	for (unsigned i=0; i<n; i++) 
	{
		const int32 frame = (*indices)[i].frame;
		const card32 point = (*indices)[i].point;
		char *dataC;
		if ( frame != BSROPT_FRAME_URSHAPE ) 
		{
			dataC = ps->getDataPointer(frame, point);
		} 
		else 
		{
			dataC = urshape->getDataPointer(point);
		}
		float* dataF = (float*)(dataC+channelAAT.getOffset());
		for (card32 c=0; c<channelDim; c++) 
		{
			dataF[c] = data[offset + i*channelDim + c];
		}
	}	
}