//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PatchGraphBSROpt.h"
//---------------------------------------------------------------------------
#include "BSRFitting.h"
#include "BSROptContext.h"
#include "BSROptPotential.h"
#include "DynamicLinearAlgebra_templCode.h"
#include "InCorePCTopologyGraph.h"
#include "IterativeSolvers.h"
#include "IterativeSolvers_templCode.h"
#include "Monomials2ndOrder.h"
#include "PatchGraph.h"
#include "MgcLinearSystem.h"
#include "PatchTypes.h"
#include "PGTopologicalNeighborhood.h"
#include "PointCloudAnimationSequence.h"
#include "PGDataPointIndices.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "PointSetKNNQuery.h"
#include "SeparatorClassProperty.h"
#include "SparseLinearAlgebra.h"
#include "SparseLinearAlgebra_templCode.h"
#include "Timer.h"
#include "PointSetKNNQuery.h"
#include "PatchGraphWCorr.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------

/************************************************************************/
/*PETER: The following codes are just for testing                                                                     */
/************************************************************************/

void		Quaternion_To_Matrix3f_GNM(const float	*q, Matrix3f &rotation)
{
	pAssert(q != NULL);
	rotation[0][0] = 2*q[1]*q[1] + (2*q[0]*q[0]-1);
	rotation[1][0] = 2*q[1]*q[2] - 2*q[0]*q[3];
	rotation[2][0] = 2*q[1]*q[3] + 2*q[0]*q[2];
	rotation[0][1] = 2*q[2]*q[1] + 2*q[0]*q[3];
	rotation[1][1] = 2*q[2]*q[2] + (2*q[0]*q[0]-1);
	rotation[2][1] = 2*q[2]*q[3] - 2*q[0]*q[1];
	rotation[0][2] = 2*q[3]*q[1] - 2*q[0]*q[2];
	rotation[1][2] = 2*q[3]*q[2] + 2*q[0]*q[1];
	rotation[2][2] = 2*q[3]*q[3] + (2*q[0]*q[0]-1);
}

void		BinaryInsertion_Sort(
	vector<int>	&m_array, //the sorted array
	int newElement, // the new element to be inserted
	card32 &index)	// the position of this element after insertion
{
	if(m_array.size() > 0)
	{
		vector<int>::iterator	begin = m_array.begin(), pos;
		card32 left = 0, right = (card32)m_array.size()-1;
		if(newElement <= (*begin))
		{
			m_array.insert(begin, newElement);
			index = 0;
		}
		else if(newElement >= m_array[right])
		{
			index = (card32)m_array.size();
			m_array.push_back(newElement);
		}
		else
		{
			while (right - left > 1)
			{
				card32 mid = (right + left)/2;
				if(newElement < m_array[mid])
				{
					right = mid;
				}
				else
				{
					left = mid;
				}
			}
			index = right;
			pos = begin + right;
			m_array.insert(pos, newElement);
		}
	}
	else
	{
		m_array.push_back(newElement);
		index = 0;
	}
}

static	void	UpdateMatrix_by_a_matrix(
													 const DMatrixF		*mat,				
													 const DVectorI	*m_indices,
													 const	DVectorI		*m_mul,
													 const card32		numOfElements,
													 SparseMatrixF		*quad)			/// The Hessian of the energy function	
{
	card32 id = 0;
	for(card32 i = 0; i < numOfElements; i++)
	{
		int row = (*m_indices)[i];
		if(row >= 0)
		{
			for(card32 j = 0; j < card32((*m_mul)[i]); j++)
			{
				(*quad)[row+j].updateEntries_Binary(&((*mat)[id]), m_indices, m_mul, numOfElements);
				id++;
			}
		}
		else
		{
			id+= (*m_mul)[i];
		}
	}
}

static	void	UpdateVector_by_a_vector(const	DVectorF	*vec,
													 const	DVectorI		*m_indices, 
													 const	DVectorI		*m_mul,
													 const	card32		numOfElements,
													 DVectorF	*lin)
{
	card32 id = 0;
	for(card32 i = 0; i < numOfElements; i++)
	{
		int row = (*m_indices)[i];
		if(row >= 0)
		{
			for(card32 j = 0; j < card32((*m_mul)[i]); j++)
			{
				(*lin)[row+j] += (*vec)[id];
				id++;
			}
		}
		else
		{
			id += (*m_mul)[i];
		}
	}
}

static	bool	matrixInverse(int iSize, float32** aafA, float32** aafAInv)
{
	float32* afV = new float32[iSize];
	pAssert( afV );

	int i0, i1;
	for (i0 = 0; i0 < iSize; i0++)
	{
		for (i1 = 0; i1 < iSize; i1++)
			aafAInv[i0][i1] = ( i0 != i1 ? 0.0f : 1.0f );
	}

	for (i1 = 0; i1 < iSize; i1++)
	{
		for (i0 = 0; i0 < i1; i0++)
			afV[i0] = aafA[i1][i0]*aafA[i0][i0];

		afV[i1] = aafA[i1][i1];
		for (i0 = 0; i0 < i1; i0++)
			afV[i1] -= aafA[i1][i0]*afV[i0];

		aafA[i1][i1] = afV[i1];
		for (i0 = i1+1; i0 < iSize; i0++)
		{
			for (int i2 = 0; i2 < i1; i2++)
				aafA[i0][i1] -= aafA[i0][i2]*afV[i2];
			aafA[i0][i1] /= afV[i1];
		}
	}
	delete[] afV;

	for (int iCol = 0; iCol < iSize; iCol++)
	{
		// forward substitution
		for (i0 = 0; i0 < iSize; i0++)
		{
			for (i1 = 0; i1 < i0; i1++)
				aafAInv[i0][iCol] -= aafA[i0][i1]*aafAInv[i1][iCol];
		}

		// diagonal division
		const float32 fTolerance = 1e-06f;
		for (i0 = 0; i0 < iSize; i0++)
		{
			if ( fabs(aafA[i0][i0]) <= fTolerance )
				return false;
			aafAInv[i0][iCol] /= aafA[i0][i0];
		}

		// back substitution
		for (i0 = iSize-2; i0 >= 0; i0--)
		{
			for (i1 = i0+1; i1 < iSize; i1++)
				aafAInv[i0][iCol] -= aafA[i1][i0]*aafAInv[i1][iCol];
		}
	}
	return true;
}


static	void		compute_Preconditioning_matrix(
	PatchGraph		*pg, 
	SparseMatrixF	*quadApprox,
	SparseMatrixF	*precond)
{
	float32	**Mat1, **InvMat1, **Mat2, **InvMat2;
	card32	m_indices[8], i, j, k;
	Mat1 = new float32*[8];
	InvMat1 = new float32*[8];
	for(i = 0; i < 8; i++)
	{
		Mat1[i]		= new float32[8];
		InvMat1[i] = new float32[8];
	}
	Mat2 = new float32*[5];
	InvMat2 = new float32*[5];
	for(i = 0; i < 5; i++)
	{
		Mat2[i] = new float32[5];
		InvMat2[i] = new float32[5];
	}
	PointSet								*ps					= pg->getTrajectorySet()->getTrajectories();
	PointSet								*urshape		= pg->getUrshape();
	InCorePCTopologyGraph *tg					= pg->getTopology();
	PGDataPointIndices			*dataPoints	= pg->getTrajectorySet()->getDataPoints();

	/// The dimension of the patchgraph
	card32								width				=	ps->getWidth(),
												height				= ps->getHeight();

	float32								value;

	int										ik_posid, ik_norid, ik_rotid;

	AAT										POS_INDEX	= ps->getAAT("varIndex_pos");
	AAT										NOR_INDEX	= ps->getAAT("varIndex_nor");
	AAT										ROT_INDEX	= ps->getAAT("varIndex_rot");

	
	for(card32 frameIndex = 0; frameIndex < width; frameIndex++)
	{
		for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
		{
			ik_posid	= ps->get<int32>(frameIndex, surfelIndex, POS_INDEX);
			ik_norid	= ps->get<int32>(frameIndex, surfelIndex, NOR_INDEX);
			ik_rotid	= ps->get<int32>(frameIndex, surfelIndex, ROT_INDEX);
			if(ik_rotid >= 0)
			{
				m_indices[0] = ik_posid;		m_indices[1] = ik_posid+1;		m_indices[2] = ik_posid+2;
				m_indices[3] = ik_norid;		m_indices[4] = ik_norid+1;		
				m_indices[5] = ik_rotid;		m_indices[6] = ik_rotid+1;		m_indices[7] = ik_rotid+2;
				for(j = 0; j < 8; j++)
				{
					for(k = 0; k < 8; k++)
					{
						(*quadApprox)[m_indices[j]].get_Value(m_indices[k], value);
						Mat1[j][k] = value;
					}
				}
				pAssert(matrixInverse(8, Mat1, InvMat1));
				for(j = 0; j < 8; j++)
				{
					for(k = 0; k < 8; k++)
					{
						(*precond)[m_indices[j]].addEntry(m_indices[k], InvMat1[j][k]);
					}
				}
			}
			else
			{
				m_indices[0] = ik_posid;		m_indices[1] = ik_posid+1;		m_indices[2] = ik_posid+2;
				m_indices[3] = ik_norid;		m_indices[4] = ik_norid+1;	
				for(j = 0; j < 5; j++)
				{
					for(k = 0; k < 5; k++)
					{
						(*quadApprox)[m_indices[j]].get_Value(m_indices[k], value);
						Mat2[j][k] = value;
					}
				}
				pAssert(matrixInverse(5, Mat2, InvMat2));
				for(j = 0; j < 5; j++)
				{
					for(k = 0; k < 5; k++)
					{
						(*precond)[m_indices[j]].addEntry(m_indices[k], InvMat2[j][k]);
					}
				}
			}
		}
	}
	for(i = 0; i < 8; i++)
	{
		delete []Mat1[i];
		delete []InvMat1[i];
	}
	delete []Mat1;
	delete []InvMat1;
	for(i = 0; i < 5; i++)
	{
		delete []Mat2[i];
		delete []InvMat2[i];
	}
	delete []Mat2;
	delete []InvMat2;
}


static	void		helper_For_Collecting_Variables(
	const	PointSet *ps,  
	const	int frameIndex, 
	const	AAT	&DESCRIPTOR,
	const	card32	numOfVariables,
	const	vector<int32> *m_array,
	vector<int32>	*buffer)
{
	card32	width = ps->getWidth();

	if(frameIndex < 0 || frameIndex >= int32(width))
		return;

	for(card32	surfelIndex = 0; surfelIndex < m_array->size(); surfelIndex++)
	{
		int	varIndex = ps->get<int32>(frameIndex, (*m_array)[surfelIndex], DESCRIPTOR);
		if(varIndex >= 0)
		{
			for(card32 k = 0; k < numOfVariables; k++)
			{
				buffer->push_back(varIndex+k);
			}
		}
	}
}


static	void		helper_For_Collecting_Variable(
	const	PointSet *ps,  
	const	int frameIndex, 
	const  int surfelIndex,
	const	AAT	&DESCRIPTOR,
	const	card32	numOfVariables,
	vector<int32>	*buffer)
{
	card32	width = ps->getWidth();

	
	if(frameIndex < 0 || frameIndex >= int32(width))
		return;

	int	varIndex = ps->get<int32>(frameIndex, surfelIndex, DESCRIPTOR);
	if(varIndex >= 0)
	{
		for(card32 k = 0; k < numOfVariables; k++)
		{
			buffer->push_back(varIndex+k);
		}
	}
}

void	PatchGraphBSROpt::initialize_Sparse_Matrix
(const BSROptVariableIndexSet *indices_pos, 
 const BSROptVariableIndexSet *indices_nor, 
 const BSROptVariableIndexSet *indices_rot, 
 PatchGraph *patchGraph, 
 SparseMatrixF *quadApprox)
{
	size_t	m_dimension = 3*indices_pos->size() + 2*indices_nor->size() + 3*indices_rot->size();
	

	/// Get geometry and topology data structure from patch graph
	PointSet				*ps	= patchGraph->getTrajectorySet()->getTrajectories();
	InCorePCTopologyGraph	*tg	= patchGraph->getTopology();

	/// The dimension of the patchgraph
	card32	width =	ps->getWidth(),
			height = ps->getHeight();

	/// Indices
	int	ik_posid, ik_norid, ik_rotid;

	/// 
	AAT		POS_INDEX	= ps->getAAT("varIndex_pos");
	AAT		NOR_INDEX	= ps->getAAT("varIndex_nor");
	AAT		ROT_INDEX	= ps->getAAT("varIndex_rot");


	/// temporary code,
	set<int32>			*neighbor_buf_set		= new set<int32>[height];
	vector<int32>	*neighbor_buf_array	= new vector<int32>[height];
	for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		card32			numOfEntries = tg->getNumAdjacentVertices(surfelIndex);
		card32			neighbor_id, neighbor_id1, ni, ni1;
		neighbor_buf_set[surfelIndex].insert(surfelIndex);
		for(ni = 0; ni < numOfEntries; ni++)
		{
			neighbor_id = tg->getAdjacentVertex(surfelIndex, ni);
			if(neighbor_id == surfelIndex)
			{
				continue;
			}
			neighbor_buf_set[surfelIndex].insert(neighbor_id);
		}
		for(ni = 0; ni < numOfEntries; ni++)
		{
			neighbor_id = tg->getAdjacentVertex(surfelIndex, ni);
			if(neighbor_id == surfelIndex)
			{
				continue;
			}
			for(ni1 = ni+1; ni1 < numOfEntries; ni1++)
			{
				neighbor_id1 = tg->getAdjacentVertex(surfelIndex, ni1);
				if(neighbor_id1 == surfelIndex)
					continue;
				neighbor_buf_set[neighbor_id].insert(neighbor_id1);
				neighbor_buf_set[neighbor_id1].insert(neighbor_id);
			}
		}
	}
	for(card32 ni = 0; ni < height; ni++)
	{
		for(set<int32>::iterator iter_set = neighbor_buf_set[ni].begin();
			iter_set != neighbor_buf_set[ni].end();
			iter_set++)
		{
			neighbor_buf_array[ni].push_back(*iter_set);
		}
		neighbor_buf_set[ni].clear();
	}
	delete []neighbor_buf_set;
	for(card32	surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		///number of neighbors
		card32			numOfEntries = tg->getNumAdjacentVertices(surfelIndex);
		card32			ni, neighbor_id, i_index, k;
		vector<int32>	m_array;

		for(ni = 0; ni < numOfEntries; ni++)
		{
			neighbor_id = tg->getAdjacentVertex(surfelIndex, ni);
			if(neighbor_id == surfelIndex)
			{
				continue;
			}
			BinaryInsertion_Sort(m_array, neighbor_id, i_index);
		}
		BinaryInsertion_Sort(m_array, surfelIndex, i_index);
		size_t	m_size = m_array.size();

		vector<int32>	buffer;
		/// frameIndex = 0
		ik_posid	= ps->get<int32>(0, surfelIndex, POS_INDEX);
		if(ik_posid >= 0)
		{
			buffer.clear();
			helper_For_Collecting_Variables(ps, 0, POS_INDEX, 3, &neighbor_buf_array[surfelIndex], &buffer);
			helper_For_Collecting_Variables(ps, 1, POS_INDEX, 3, &m_array, &buffer);
			if(width == 3)
			{
				helper_For_Collecting_Variables(ps, width-1, POS_INDEX, 3, &m_array, &buffer);
			}
			else if(width >=4)
			{
				helper_For_Collecting_Variable(ps,  2, surfelIndex, POS_INDEX, 3, &buffer);
				helper_For_Collecting_Variables(ps, width-1, POS_INDEX, 3, &m_array, &buffer);
			}
			helper_For_Collecting_Variables(ps, 0, NOR_INDEX, 2, &m_array, &buffer);
			helper_For_Collecting_Variables(ps, 0, ROT_INDEX, 3, &m_array, &buffer);
			if(width == 2)
			{
				helper_For_Collecting_Variables(ps, width-1, ROT_INDEX, 3, &m_array, &buffer);
			}
			else if(width >= 3)
			{
				helper_For_Collecting_Variable(ps, 1, surfelIndex, ROT_INDEX, 3, &buffer);
				helper_For_Collecting_Variables(ps, width-1, ROT_INDEX, 3, &m_array, &buffer);
			}
			for(k = 0; k < 3; k++)
			{
				(*quadApprox)[ik_posid+k].initialize(&buffer);
			}
		}

		ik_norid	= ps->get<int32>(0, surfelIndex, NOR_INDEX);
		if(ik_norid >= 0)
		{
			buffer.clear();
			helper_For_Collecting_Variables(ps, 0, POS_INDEX, 3, &m_array, &buffer);
			helper_For_Collecting_Variables(ps, 0, NOR_INDEX, 2, &m_array, &buffer);
			helper_For_Collecting_Variables(ps, 1, NOR_INDEX, 2, &m_array, &buffer);
			if(width == 3)
			{
				helper_For_Collecting_Variables(ps, width-1, NOR_INDEX, 2, &m_array, &buffer);
			}
			else if(width >=4)
			{
				helper_For_Collecting_Variable(ps,  2, surfelIndex, NOR_INDEX, 2, &buffer);
				helper_For_Collecting_Variables(ps, width-1, NOR_INDEX, 2, &m_array, &buffer);
			}
			helper_For_Collecting_Variables(ps, 0, ROT_INDEX, 3, &m_array, &buffer);
			if(width == 2)
			{
				helper_For_Collecting_Variables(ps, width-1, ROT_INDEX, 3, &m_array, &buffer);
			}
			else if(width >= 3)
			{
				helper_For_Collecting_Variable(ps, 1, surfelIndex, ROT_INDEX, 3, &buffer);
				helper_For_Collecting_Variables(ps, width-1, ROT_INDEX, 3, &m_array, &buffer);
			}
			for(k = 0; k < 2; k++)
			{
				(*quadApprox)[ik_norid+k].initialize(&buffer);
			}
		}
		
		ik_rotid	=	ps->get<int32>(0, surfelIndex, ROT_INDEX);
		if(ik_rotid >= 0)
		{
			buffer.clear();
			helper_For_Collecting_Variables(ps, 0, POS_INDEX, 3, &m_array, &buffer);
			helper_For_Collecting_Variables(ps, 1, POS_INDEX, 3, &m_array, &buffer);
			helper_For_Collecting_Variables(ps, 0, NOR_INDEX, 2, &m_array, &buffer);
			helper_For_Collecting_Variables(ps, 1, NOR_INDEX, 2, &m_array, &buffer);
			for(k = 0; k < 3; k++)
			{
				buffer.push_back(ik_rotid + k);
			}
			if(width > 2)
			{
				helper_For_Collecting_Variable(ps, 1, surfelIndex, ROT_INDEX, 3, &buffer);
			}
			for(k = 0; k < 3; k++)
			{
				(*quadApprox)[ik_rotid+k].initialize(&buffer);
			}
		}
	
		for(card32 frameIndex = 1; frameIndex < width-1; frameIndex++)
		{	
			ik_posid	= ps->get<int32>(frameIndex, surfelIndex,  POS_INDEX);
			if(ik_posid >= 0)
			{
				buffer.clear();
				helper_For_Collecting_Variable(ps, frameIndex-2, surfelIndex, POS_INDEX, 3, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex-1,	POS_INDEX, 3, &m_array, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex,		POS_INDEX, 3, &neighbor_buf_array[surfelIndex], &buffer);
				helper_For_Collecting_Variables(ps, frameIndex+1,	POS_INDEX, 3, &m_array, &buffer);
				helper_For_Collecting_Variable(ps,  frameIndex+2, surfelIndex, POS_INDEX, 3, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex,		NOR_INDEX, 2, &m_array, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex-1,	ROT_INDEX, 3, &m_array, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex,		ROT_INDEX, 3, &m_array, &buffer);
				helper_For_Collecting_Variable(ps, frameIndex+1, surfelIndex, ROT_INDEX, 3, &buffer);
				for(k = 0; k < 3; k++)
				{
					(*quadApprox)[ik_posid+k].initialize(&buffer);
				}
			}
			
			ik_norid	= ps->get<int32>(frameIndex, surfelIndex, NOR_INDEX);
			if(ik_norid >= 0)
			{
				buffer.clear();
				helper_For_Collecting_Variables(ps, frameIndex,		POS_INDEX, 3, &m_array, &buffer);
				helper_For_Collecting_Variable(ps, frameIndex-2, surfelIndex, NOR_INDEX, 2, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex-1,	NOR_INDEX, 2, &m_array, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex,		NOR_INDEX, 2, &m_array, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex+1,	NOR_INDEX, 2, &m_array, &buffer);
				helper_For_Collecting_Variable(ps, frameIndex+2, surfelIndex, NOR_INDEX, 2, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex-1,	ROT_INDEX, 3, &m_array, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex,		ROT_INDEX, 3, &m_array, &buffer);
				helper_For_Collecting_Variable(ps, frameIndex+1, surfelIndex, ROT_INDEX, 3, &buffer);
				for(k = 0; k < 2; k++)
				{
					(*quadApprox)[ik_norid+k].initialize(&buffer);
				}
			}

			ik_rotid	=	ps->get<int32>(frameIndex, surfelIndex, ROT_INDEX);
			if(ik_rotid >= 0)
			{
				buffer.clear();
				helper_For_Collecting_Variable(ps, frameIndex-1, surfelIndex, POS_INDEX, 3, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex,		POS_INDEX, 3, &m_array, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex+1,	POS_INDEX, 3, &m_array, &buffer);
				helper_For_Collecting_Variable(ps, frameIndex-1, surfelIndex, NOR_INDEX, 2, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex,		NOR_INDEX, 2, &m_array, &buffer);
				helper_For_Collecting_Variables(ps, frameIndex+1,	NOR_INDEX, 2, &m_array, &buffer);
				helper_For_Collecting_Variable(ps, frameIndex-1, surfelIndex, ROT_INDEX, 3, &buffer);
				for(k = 0; k < 3; k++)
				{
					buffer.push_back(ik_rotid + k);
				}
				if(frameIndex < width-2)
				{
					helper_For_Collecting_Variable(ps, frameIndex+1, surfelIndex, ROT_INDEX, 3, &buffer);
				}
				for(k = 0; k < 3; k++)
				{
					(*quadApprox)[ik_rotid+k].initialize(&buffer);
				}
			}
		}
		/// frameIndex = (width-1)
		ik_posid	= ps->get<int32>(width - 1, surfelIndex, POS_INDEX);
		if(ik_posid >= 0)
		{
			buffer.clear();
			if(width == 3)
			{
				helper_For_Collecting_Variables(ps, 0,	POS_INDEX, 3,	&m_array, &buffer);
			}
			else if(width >= 4)
			{
				helper_For_Collecting_Variables(ps, 0,	POS_INDEX, 3,	&m_array, &buffer);
				helper_For_Collecting_Variable(ps, width - 3, surfelIndex, POS_INDEX, 3, &buffer);
			}
			helper_For_Collecting_Variables(ps, width - 2,	POS_INDEX, 3,	&m_array, &buffer);
			helper_For_Collecting_Variables(ps, width - 1,	POS_INDEX, 3,	&neighbor_buf_array[surfelIndex], &buffer);
			helper_For_Collecting_Variables(ps, width - 1,	NOR_INDEX, 2,	&m_array, &buffer);
			helper_For_Collecting_Variables(ps, width - 2,	ROT_INDEX, 3,	&m_array, &buffer);
			helper_For_Collecting_Variables(ps, width - 1,	ROT_INDEX, 3,	&m_array, &buffer);
			for(k = 0; k < 3; k++)
			{
				(*quadApprox)[ik_posid+k].initialize(&buffer);
			}
		}
		
		ik_norid	= ps->get<int32>(width - 1, surfelIndex, NOR_INDEX);
		if(ik_norid >= 0)
		{
			buffer.clear();
			helper_For_Collecting_Variables(ps, width - 1,	POS_INDEX, 3,	&m_array, &buffer);
			if(width == 3)
			{
				helper_For_Collecting_Variables(ps, 0,	NOR_INDEX, 2,	&m_array, &buffer);
			}
			else if(width >= 4)
			{
				helper_For_Collecting_Variables(ps, 0,	NOR_INDEX, 2,	&m_array, &buffer);
				helper_For_Collecting_Variable( ps, width - 3, surfelIndex, NOR_INDEX, 2, &buffer);
			}
			helper_For_Collecting_Variables(ps, width - 2,	NOR_INDEX, 2,	&m_array, &buffer);
			helper_For_Collecting_Variables(ps, width - 1,	NOR_INDEX, 2,	&m_array, &buffer);
			helper_For_Collecting_Variables(ps, width - 2,	ROT_INDEX, 3,	&m_array, &buffer);
			helper_For_Collecting_Variables(ps, width - 1,	ROT_INDEX, 3,	&m_array, &buffer);
			for(k = 0; k < 2; k++)
			{
				(*quadApprox)[ik_norid+k].initialize(&buffer);
			}
		}
		ik_rotid	=	ps->get<int32>(width-1, surfelIndex, ROT_INDEX);
		if(ik_rotid >= 0)
		{
			buffer.clear();
			helper_For_Collecting_Variables(ps, 0,		 POS_INDEX, 3, &m_array, &buffer);
			helper_For_Collecting_Variables(ps, width-1, POS_INDEX, 3, &m_array, &buffer);
			helper_For_Collecting_Variables(ps, 0,		 NOR_INDEX, 2, &m_array, &buffer);
			helper_For_Collecting_Variables(ps, width-1, NOR_INDEX, 2, &m_array, &buffer);
			for(k = 0; k < 3; k++)
			{
				buffer.push_back(ik_rotid + k);
			}
			for(k = 0; k < 3; k++)
			{
				(*quadApprox)[ik_rotid+k].initialize(&buffer);
			}
		}
	}
	for(card32 ni = 0; ni < height; ni++)
	{
		neighbor_buf_array[ni].clear();
	}
	delete []neighbor_buf_array;
}

void	PatchGraphBSROpt::compute_Related_Variable_Indices( 
BSROptVariableIndexSet *indices_related, 
 PatchGraph *patchGraph )
{
	/// Get geometry and topology data structure from patch graph
	PointSet								*ps					= patchGraph->getTrajectorySet()->getTrajectories();
	InCorePCTopologyGraph *tg					= patchGraph->getTopology();

	/// The dimension of the patchgraph
	card32								width				=	ps->getWidth(),
												height				= ps->getHeight();

	/// Indices
	int										ik_posid, ik_norid, 
												ikn_posid, ikn_norid, 
												ikp_posid, ikp_norid, 
												jk_posid, jk_norid, 
												jkp_posid, jkp_norid,
												ikp_rotid, ik_rotid;

	AAT										POS_INDEX	= ps->getAAT("varIndex_pos");
	AAT										NOR_INDEX	= ps->getAAT("varIndex_nor");
	AAT										ROT_INDEX	= ps->getAAT("varIndex_rot");

	for(card32 frameIndex = 0; frameIndex < width; frameIndex++)
	{
		for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
		{
			card32			numOfEntries = tg->getNumAdjacentVertices(surfelIndex);
			card32			ni, neighbor_id;
			ik_posid	= ik_norid = ikp_posid = ikp_norid = ikn_posid = ikn_norid = ikp_rotid = ik_rotid = -1;
			ik_posid	= ps->get<int32>(frameIndex, surfelIndex, POS_INDEX);
			ik_norid	= ps->get<int32>(frameIndex, surfelIndex, NOR_INDEX);
			if(frameIndex < width -1)
			{
				ikn_posid	= ps->get<int32>(frameIndex+1, surfelIndex, POS_INDEX);
				ikn_norid		= ps->get<int32>(frameIndex+1, surfelIndex, NOR_INDEX);
				ik_rotid		=	ps->get<int32>(frameIndex,    surfelIndex, ROT_INDEX);
			}
			if(frameIndex > 0)
			{
				ikp_posid	= ps->get<int32>(frameIndex-1, surfelIndex, POS_INDEX);
				ikp_norid		= ps->get<int32>(frameIndex-1, surfelIndex, NOR_INDEX);
				ikp_rotid		=	ps->get<int32>(frameIndex-1,   surfelIndex, ROT_INDEX);
			}
			if(ik_posid >=0 || ik_norid >= 0 || ikp_rotid >= 0 || ikp_posid >= 0 || ikp_norid >= 0
				||ikn_posid >=0 ||ikn_norid>=0||ik_rotid>=0)
			{
				BSROptPointIndex		index;
				index.frame	= frameIndex;
				index.point		=	surfelIndex;
				indices_related->push_back(index);
				continue;
			}
			for(ni = 0; ni < numOfEntries; ni++)
			{
				neighbor_id = tg->getAdjacentVertex(surfelIndex, ni);
				if(neighbor_id == surfelIndex)
				{
					continue;
				}
				jk_posid	= ps->get<int32>(frameIndex, neighbor_id, POS_INDEX);
				jk_norid	= ps->get<int32>(frameIndex, neighbor_id, NOR_INDEX);
				jkp_posid = jkp_norid = -1;
				if(frameIndex > 0)
				{
					jkp_posid	= ps->get<int32>(frameIndex-1, neighbor_id, POS_INDEX);
					jkp_norid		= ps->get<int32>(frameIndex-1, neighbor_id, NOR_INDEX);
				}
				if(jk_posid >=0 || jk_norid >= 0 || jkp_posid >= 0 || jkp_norid >= 0)
				{
					BSROptPointIndex		index;
					index.frame	= frameIndex;
					index.point		=	surfelIndex;
					indices_related->push_back(index);
					break;
				}
			}
		}
	}
}


void	PatchGraphBSROpt::setSmoothnessFlag(
	const	card32		 TypeOfOptimization,
	PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph)
{
	PointSet			*ps		= patchGraph->getTrajectorySet()->getTrajectories();

	card32				width	= ps->getWidth(),
						height	= ps->getHeight();

	AAT		POSITION	= ps->getAAT("position");
	AAT	DATA_POS		= data->getFrame(0)->getAAT("position");
	AAT AAT_FLAGS		= ps->getAAT("flags"); // this is an int32[1] channel

	card32				footIndex_Data;
	PointSet			*pc;

	for(card32	frameIndex =  0; frameIndex < width; frameIndex++)
	{
		if(TypeOfOptimization == 3)
		{
			if(index_transfer[frameIndex] == -1)
			{
				for(card32 surfelIndex= 0; surfelIndex < height; surfelIndex++)
				{
					ps->set<int32>(frameIndex, surfelIndex, AAT_FLAGS, ps->get<int32>(frameIndex, surfelIndex, AAT_FLAGS) | FLAG_PATCH_NOSMOOTH);
				}
				continue;
			}
			pc	= data->getFrame(index_transfer[frameIndex]);
			AttachedIndexedOctree *dataOctree = data->fetchOctree(index_transfer[frameIndex]);
			PointSetKNNQuery data_knn(pc, dataOctree, NULL_VECTOR3F, pc->getAAT("position"));

			for(card32 surfelIndex= 0; surfelIndex < height; surfelIndex++)
			{
				Vector3f		pik = ps->get3f(frameIndex, surfelIndex, POSITION);
				data_knn.setTargetPoint(pik);
				{
					bool found;
					data_knn.getNextNearestPointIndex(footIndex_Data, found);

					Vector3f	rik = pc->get3f(0, footIndex_Data, DATA_POS) - pik;
					float32	sqr2 = rik.getSqrNorm();
					if(sqr2 > context->pruneDist_smoothness)
					{
						ps->set<int32>(frameIndex, surfelIndex, AAT_FLAGS, ps->get<int32>(frameIndex, surfelIndex, AAT_FLAGS) | FLAG_PATCH_NOSMOOTH);
					}
					else
					{
						ps->set<int32>(frameIndex, surfelIndex, AAT_FLAGS, ps->get<int32>(frameIndex, surfelIndex, AAT_FLAGS) & (0xFFFFFFFF ^ FLAG_PATCH_NOSMOOTH));
					}
				}
			}
		}
		else
		{
			for(card32 surfelIndex= 0; surfelIndex < height; surfelIndex++)
			{
				ps->set<int32>(frameIndex, surfelIndex, AAT_FLAGS, ps->get<int32>(frameIndex, surfelIndex, AAT_FLAGS) & (0xFFFFFFFF ^ FLAG_PATCH_NOSMOOTH));
			}
		}
	}
}

void	PatchGraphBSROpt::compute_GNM_Gradient_and_Hessian( 
	const	BSROptVariableIndexSet				*indices_related,	/// only these indices are related to this optimization
	const	PointCloudAnimationSequence	*data,						/// the raw data
	PatchGraph		*patchGraph,												/// the patch graph to be optimized
	DVectorF		*gradient,														/// the first order derivative
	SparseMatrixF	*quadApprox												/// the second order derivative
	)
{
/*	bool		flag_SDF		= potential_flags[0], flag_Mar	 = potential_flags[1], flag_PlaS	=	potential_flags[2], flag_NorS	= potential_flags[3],
				flag_CirS		= potential_flags[4], flag_LapS	 = potential_flags[5], flag_PosR	= potential_flags[6], flag_NorR	= potential_flags[7],
				flag_STPC	=	potential_flags[8], flag_STNC = potential_flags[9], flag_STRC = potential_flags[10];

	float32	w_sdf		= context->weightSquaredDistance,		w_marker	=	context->weightMarkerError,
				w_pla_s		= context->weightPlanarSmoothness,		w_nor_s		= context->weightNormalSmoothness,
				w_cir_s		= context->weightCircularSmoothness,	w_lap_s		= context->weightLaplacianSmoothness,
				w_pos_r		= context->weightPositionRigidity,			w_nor_r		= context->weightNormalRigidity,
				w_stpc		= context->weightPositionCoherence,		w_stnc		= context->weightNormalCoherence,
				w_strc		= context->weightRotationCoherence;

	/// Controls the smoothness
	float32	lambda	=	context->lambda;

	/// Get geometry and topology data structure from patch graph

	PointSet								*ps					= patchGraph->getTrajectorySet()->getTrajectories();
	PointSet								*urshape			= patchGraph->getUrshape();
	InCorePCTopologyGraph					*tg					= patchGraph->getTopology();
	PGDataPointIndices						*dataPoints		= patchGraph->getTrajectorySet()->getDataPoints();

	/// The dimension of the patchgraph
	card32								width				=	ps->getWidth(),
												height				= ps->getHeight();

	/// Buffers for updating sparse matrix 
	DMatrixF								mat;			mat.setDimension(15, 15, false);		
	DVectorF							vec;			vec.setDim(15);									
	DVectorI								m_indices;	m_indices.setDim(5);							
	DVectorI								m_mul;		m_mul.setDim(5);

	/// Attributes
	AAT					POSITION		= ps->getAAT("position"),			NORMAL		= ps->getAAT("normal"),
						REF_U			= ps->getAAT("tangent_u"),			ROTATION	= ps->getAAT("rotation_matrix"),
						POS_INDEX		= ps->getAAT("varIndex_pos"),		NOR_INDEX	= ps->getAAT("varIndex_nor"),
						TOP_DEPTH		= ps->getAAT("topology_depth"),		ROT_INDEX	= ps->getAAT("varIndex_rot"),
						AAT_FLAGS	= ps->getAAT("flags"),				// this is an int32[1] channel
						DATA_POS_AAT	= data->getFrame(0)->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );

	/// Variables to be used
	Vector3f								pikp,nikp,Eikp1,Eikp2,AikpEikp1,AikpEikp2, 
												pjkp,njkp,Ejkp1,Ejkp2,AikpEjkp1,AikpEjkp2,
												pik,nik,Eik1,Eik2,pjk,njk,Ejk1,Ejk2,
												pikn, nikn,Eikn1,Eikn2,
												AikEikp1,AikEikp2, AikEik1,AikEik2,
												rijk,rijkp,rijkpc,nijk,nijkp,nijkpc,
												rikpk, nikpk, rikpkc, nikpkc, rikkn, nikkn, 
												dev_pijk, dev_nijk, cen_pik, dev_pikkpkn, dev_nikkpkn;

	/// Distances
	float32									disijk, cirijk;

	/// Indies
	int32									ik_posid, ik_norid, ikp_posid, ikp_norid, ikp_rotid,
												jk_posid, jk_norid, jkp_posid, jkp_norid, 
												ik_depth, jk_depth, ikp_depth, jkp_depth,
												ikn_posid, ikn_norid, ik_rotid, ikn_depth;

	bool									ik_need_smoothness_flag,
											jk_need_smoothness_flag;

	/// Current term rotation matrix
	Matrix3f								Aikp, Aik;

	card32									offset = 0,					/// Access edge_weight for smoothness and rigidity
											numOfEdges = 0;		/// Number of edges in the topology graph

	for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		numOfEdges += tg->getNumAdjacentVertices(surfelIndex);
	}

	/// Compute Marker-error potential,
	/// Planar, Normal, Circular and Laplacian smoothness potentials,
	/// Position and rigidity potentials in turn.

	for(card32	index = 0; index < indices_related->size(); index++)
	{
		///number of neighbors
		card32			frameIndex		= (*indices_related)[index].frame;
		card32			surfelIndex		= (*indices_related)[index].point;
		ik_depth		=	ps->get<int32>(frameIndex, surfelIndex, TOP_DEPTH);
		if(ik_depth >= 2)
			continue;
		card32			numOfEntries = tg->getNumAdjacentVertices(surfelIndex);
		card32			ni, neighbor_id, i_index;
		card32			j, k;

		
		ik_posid			= ps->get<int32>(frameIndex, surfelIndex, POS_INDEX);
		ik_norid			= ps->get<int32>(frameIndex, surfelIndex, NOR_INDEX);
		ik_need_smoothness_flag = (ps->get<int32>(frameIndex, surfelIndex, AAT_FLAGS) & FLAG_PATCH_NOSMOOTH) == 0;

		/// get geometry
		pik					= ps->get3f(frameIndex, surfelIndex,	POSITION);
		nik					= ps->get3f(frameIndex, surfelIndex,  NORMAL); 
		Eik1				= ps->get3f(frameIndex, surfelIndex, REF_U);
		Eik2				= nik.crossProduct(Eik1);
		if(frameIndex > 0)
		{
			/// We have position and normal rigidity potentials
			/// Get the geometry
			pikp				=	ps->get3f(frameIndex-1, surfelIndex,	POSITION);
			nikp				= ps->get3f(frameIndex-1, surfelIndex,  NORMAL); 
			Eikp1			= ps->get3f(frameIndex-1, surfelIndex, REF_U);
			Eikp2			= nik.crossProduct(Eikp1);

			ikp_posid	= ps->get<int32>(frameIndex -1, surfelIndex, POS_INDEX);
			ikp_norid		= ps->get<int32>(frameIndex -1, surfelIndex, NOR_INDEX);
			ikp_rotid		= ps->get<int32>(frameIndex -1, surfelIndex, ROT_INDEX);
			ikp_depth	=	ps->get<int32>(frameIndex -1, surfelIndex, TOP_DEPTH);

			/// Get the current rotation matrix
			char			*dataC	= ps->getDataPointer(frameIndex-1, surfelIndex);
			float32	*quatc	= (float*)(dataC+ROTATION.getOffset());
			Quaternion_To_Matrix3f_GNM(quatc, Aikp);

			/// These variables will be used in normal rigidity potential
			AikpEikp1	= Aikp*Eikp1;
			AikpEikp2	= Aikp*Eikp2;
			if(frameIndex < width -1)
			{
				pikn				=	ps->get3f(frameIndex+1, surfelIndex,	POSITION);
				nikn				= ps->get3f(frameIndex+1, surfelIndex,  NORMAL); 
				Eikn1			= ps->get3f(frameIndex+1, surfelIndex, REF_U);
				Eikn2			= nik.crossProduct(Eikp1);

				ikn_posid	= ps->get<int32>(frameIndex +1, surfelIndex, POS_INDEX);
				ikn_norid		= ps->get<int32>(frameIndex +1, surfelIndex, NOR_INDEX);
				ik_rotid		= ps->get<int32>(frameIndex , surfelIndex, ROT_INDEX);
				ikn_depth	=	ps->get<int32>(frameIndex +1, surfelIndex, TOP_DEPTH);

				char			*dataC1	= ps->getDataPointer(frameIndex, surfelIndex);
				float32	*quatc1	= (float*)(dataC1+ROTATION.getOffset());
				Quaternion_To_Matrix3f_GNM(quatc1, Aik);

				AikEik1	=	Aik*Eik1;				AikEik2	= Aik*Eik2;
				AikEikp1	= Aik*Eikp1;				AikEikp2	= Aik*Eikp2;
				rikpk		= pikp - pik;				nikpk		= nikp - nik;
				rikkn		= pik- pikn;				nikkn		= nik - nikn;
				rikpkc		= Aik*rikpk;				nikpkc		= Aik*nikpk;
				dev_pikkpkn	=	rikpkc - rikkn;
				dev_nikkpkn	=	nikpkc - nikkn;
				if(flag_STPC&&ikp_depth < 2 && ikn_depth < 2)
				{
					float32	weight = w_stpc;
					Vector3f	curvik = rikpk - rikkn;
					float32	buf_v[9][3];
					m_mul[0] = m_mul[1] = m_mul[2]  = 3;
					for(k = 0; k < 3; k++)
					{
						for(j = 0; j < 3; j++)
						{
							buf_v[k][j] = 0.0f;
							buf_v[3+k][j] = 0.0f;
							buf_v[6+k][j] = 0.0f;
						}
						buf_v[k][k] = 1.0f;			buf_v[3+k][k] = -2.0f;			buf_v[6+k][k] = 1.0f;
					}
					m_indices[0] = ikp_posid;			m_indices[1] = ik_posid;		m_indices[2] = ikn_posid;

					for(k = 0; k < 9; k++)
					{
						vec[k] = -(curvik[0]*buf_v[k][0]+curvik[1]*buf_v[k][1]+curvik[2]*buf_v[k][2])*weight;
						for(j = 0; j < 9; j++)
						{
							mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
						}
					}
					UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 4, quadApprox);
					UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 4, gradient);
				}
				if(flag_STNC&&ikp_depth < 2 && ikn_depth < 2)
				{
					float32 weight	 = w_stnc;
					float32	buf_v[9][3];
					m_mul[0] = m_mul[1] = m_mul[2] = 2;	m_mul[3] = 3;
					for(j = 0; j < 3; j++)
					{
						buf_v[0][j] = AikEikp1[j];
						buf_v[1][j] = AikEikp2[j];
						buf_v[2][j] = -AikEik1[j] - Eik1[j];
						buf_v[3][j] = -AikEik2[j] - Eik2[j];
						buf_v[4][j] = Eikn1[j];
						buf_v[5][j] = Eikn2[j];
					}
					m_indices[0] = ikp_norid;			m_indices[1] = ik_norid;			m_indices[3] = ikn_norid;

					buf_v[6][0] = 0.0f;				buf_v[6][1] = -nikpkc[2];	buf_v[6][2] = nikpkc[1];
					buf_v[7][0] = nikpkc[2];		buf_v[7][1] = 0.0f;				buf_v[7][2] = -nikpkc[0];
					buf_v[8][0] = -nikpkc[1];	buf_v[8][1] = nikpkc[0];	buf_v[8][2] = 0.0f;
					m_indices[3] = ik_rotid;
					for(k = 0; k < 9; k++)
					{
						vec[k] = -(dev_nikkpkn[0]*buf_v[k][0]+dev_nikkpkn[1]*buf_v[k][1]+dev_nikkpkn[2]*buf_v[k][2])*weight;	
						for(j = 0; j < 9; j++)
						{
							mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
						}
					}
					UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 4, quadApprox);
					UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 4, gradient);
				}
				if(flag_STRC&&ikp_depth < 2)
				{
					float32	 weight	 = w_strc;
					float32	buf_v[6][9];
					float32	buf_colume[9];
					m_mul[0] = m_mul[1] = 3;
				
					for(k = 0; k < 3; k++)
					{
						for(j = 0; j < 3; j++)
						{
							buf_colume[3*k + j] = Aikp[k][j] - Aik[k][j];
						}
						buf_v[0][3*k+0] = 0.0f;				buf_v[0][3*k+1] = -Aikp[k][0];		buf_v[0][3*k+2] = Aikp[k][1];
						buf_v[1][3*k+0] = Aikp[k][0];		buf_v[1][3*k+1] = 0.0f;				buf_v[1][3*k+2] = -Aikp[k][2];
						buf_v[2][3*k+0] = -Aikp[k][1];		buf_v[2][3*k+1] = Aikp[k][2];		buf_v[2][3*k+2] = 0.0f;

						buf_v[3][3*k+0] = 0.0f;				buf_v[3][3*k+1] = Aik[k][0];		buf_v[3][3*k+2] = -Aik[k][1];
						buf_v[4][3*k+0] = -Aik[k][0];		buf_v[4][3*k+1] = 0.0f;				buf_v[4][3*k+2] = Aik[k][2];
						buf_v[5][3*k+0] = Aik[k][1];		buf_v[5][3*k+1] = -Aik[k][2];		buf_v[5][3*k+2] = 0.0f;
					}
					m_indices[0] = ikp_rotid;			m_indices[1] = ik_rotid;
					for(k = 0; k < 6; k++)
					{
						vec[k] = -(buf_colume[0]*buf_v[k][0]+buf_colume[1]*buf_v[k][1]+buf_colume[2]*buf_v[k][2]
										+ buf_colume[3]*buf_v[k][3]+buf_colume[4]*buf_v[k][4]+buf_colume[5]*buf_v[k][5]
										+ buf_colume[6]*buf_v[k][6]+buf_colume[7]*buf_v[k][7]+buf_colume[8]*buf_v[k][8])*weight;	
						for(j = 0; j < 6; j++)
						{
							mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2]
												+buf_v[k][3]*buf_v[j][3]+buf_v[k][4]*buf_v[j][4]+buf_v[k][5]*buf_v[j][5]
												+buf_v[k][6]*buf_v[j][6]+buf_v[k][7]*buf_v[j][7]+buf_v[k][8]*buf_v[j][8])*weight;
						}
					}
					UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 2, quadApprox);
					UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 2, gradient);
				}
			}
		}

		/// initialize for Laplaciansmoothness potential
		cen_pik[0] = cen_pik[1] = cen_pik[2] = 0.0;
		vector<int32>	m_array;
		
		/// Marker-error potential
		if(flag_Mar)
		{
			PatchGraphWCorr *corrPG = dynamic_cast<PatchGraphWCorr*>(patchGraph);
			if (corrPG == NULL) {
				throw PException("GaussNewtonMethod.cpp - corrPG is NULL (this should not happen).");
			}
			const map<card32, Vector3f> &correspondences = corrPG->correspondences;
			
			float32	weight = w_marker;
			
			map<card32, Vector3f>::const_iterator corrIt = correspondences.find(surfelIndex);
			if (corrIt != correspondences.end()) {
				Vector3f		devik	= pik - corrIt->second;
				m_mul[0]				= 3;
				m_indices[0]			= ik_posid;
				/// Update Gradient
				for(j = 0; j < 3; j++)
				{
					vec[j]								= -devik[j]*weight;
					for(k = 0; k < 3; k++)
					{
						mat[j][k]					= 0.0f;
					}
					mat[j][j]							+= weight;
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 1, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 1, gradient);			
			}

			const card32 numDataPoints = dataPoints->getNumDataPoints( frameIndex, surfelIndex);
//			float32	weight = w_marker;
			if (numDataPoints > 0 && index_transfer[frameIndex] >= 0) 
			{
				card32 dataIndexInData	= dataPoints->getIndex( frameIndex, surfelIndex, 0);
				Vector3f		devik	= pik - data->getFrame(index_transfer[frameIndex])->get3f(dataIndexInData, DATA_POS_AAT);
				m_mul[0]				= 3;
				m_indices[0]			= ik_posid;
				/// Update Gradient
				for(j = 0; j < 3; j++)
				{
					vec[j]								= -devik[j]*weight;
					for(k = 0; k < 3; k++)
					{
						mat[j][k]					= 0.0f;
					}
					mat[j][j]							+= weight;
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 1, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 1, gradient);
			}
		}
		bool flag = true;
		for(ni = 0; ni < numOfEntries; ni++)
		{
			/// the index of this neighbor
			neighbor_id	= tg->getAdjacentVertex(surfelIndex, ni);
			if(neighbor_id == surfelIndex)
			{
				continue;
			}

			BinaryInsertion_Sort(m_array, neighbor_id, i_index);
			jk_depth			=	ps->get<int32>(frameIndex, neighbor_id, TOP_DEPTH);
			if(jk_depth >= 2)
			{
				flag = false;
				continue;
			}
			/// the geometry of this neighbor
			pjk					=	ps->get3f(frameIndex, neighbor_id,	POSITION);
			njk					=	ps->get3f(frameIndex, neighbor_id,  NORMAL); 
			Ejk1				=	ps->get3f(frameIndex, neighbor_id, REF_U);
			Ejk2				=	njk.crossProduct(Ejk1);

			jk_posid			=	ps->get<int32>(frameIndex, neighbor_id, POS_INDEX);
			jk_norid			=	ps->get<int32>(frameIndex, neighbor_id, NOR_INDEX);

			jk_need_smoothness_flag = (ps->get<int32>(frameIndex, surfelIndex, AAT_FLAGS) & FLAG_PATCH_NOSMOOTH) == 0;
			
			/// precomputed geometric quantities 
			rijk					= pik	-	pjk;
			nijk					= nik	-	njk;
			disijk				= nik*rijk;
			cirijk				= (nik+njk)*rijk;

			if(frameIndex > 0)
			{
				pjkp				=	ps->get3f(frameIndex-1, neighbor_id,	POSITION);
				njkp				= ps->get3f(frameIndex-1, neighbor_id,  NORMAL); 
				Ejkp1			= ps->get3f(frameIndex-1, neighbor_id, REF_U);
				Ejkp2			= nik.crossProduct(Ejkp1);
				AikpEjkp1	= Aikp*Ejkp1;
				AikpEjkp2	= Aikp*Ejkp2;

				jkp_posid	= ps->get<int32>(frameIndex-1, neighbor_id, POS_INDEX);
				jkp_norid		=	ps->get<int32>(frameIndex-1, neighbor_id, NOR_INDEX);
				jkp_depth	=	ps->get<int32>(frameIndex-1, neighbor_id, TOP_DEPTH);

				rijkp				= pikp		-	pjkp;
				rijkpc			= Aikp*rijkp;
				nijkp			= nikp		-	njkp;
				nijkpc			= Aikp*nijkp;
				dev_pijk		= rijkpc	-	rijk;
				dev_nijk		= nijkpc	-	nijk;
			}

			/// precomputed weights
			float32	sweightijk = exp(lambda*rijk.getSqrNorm());
			float32	sedgeweight, redgeweight;
			sedgeweight	= 1.0f;
			if(frameIndex > 0)
				redgeweight	= 1.0f;

			float32	weight;

			///Planar smoothness potential
			if(flag_PlaS)
			{
				weight = w_pla_s*sweightijk*sedgeweight/numOfEntries;
//				if(!ik_need_smoothness_flag||!jk_need_smoothness_flag)
//					weight /= 100.0f;
				float32	buf_v[8];
				m_mul[0] = m_mul[1] = 3;		m_mul[2] = 2;
				if(surfelIndex < neighbor_id)
				{
					for(k = 0; k < 3; k++)
					{
						buf_v[k]		= nik[k] + lambda*disijk*rijk[k];
						buf_v[3+k]	= -buf_v[k];
					}
					m_indices[0] = ik_posid;			m_indices[1] = jk_posid;
				}
				else
				{
					for(k = 0; k < 3; k++)
					{
						buf_v[3+k]		= nik[k] + lambda*disijk*rijk[k];
						buf_v[k]	= -buf_v[3+k];
					}
					m_indices[1] = ik_posid;			m_indices[0] = jk_posid;
				}
				buf_v[6]			= rijk*Eik1;				buf_v[7]			= rijk*Eik2;			m_indices[2] = ik_norid;
				for(k = 0; k < 8; k++)
				{
					vec[k] = -disijk*buf_v[k]*weight;
					for(j = 0; j < 8; j++)
					{
						mat[k][j] = buf_v[k]*buf_v[j]*weight;
					}
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 3, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 3, gradient);
			}
			/// Normal smoothness potential
			if(flag_NorS)
			{
				weight = w_nor_s*sweightijk*sedgeweight/numOfEntries;
//				if(!ik_need_smoothness_flag||!jk_need_smoothness_flag)
//					weight /= 100.0f;

				float32	buf_v[10][3];
				m_mul[0] = m_mul[1] = 3;		m_mul[2] = m_mul[3] = 2;
				if(surfelIndex < neighbor_id)
				{
					for(k = 0; k < 3; k++)
					{
						for(j = 0; j < 3; j++)
						{
							buf_v[k][j]		= lambda*rijk[k]*nijk[j];
							buf_v[3+k][j]	= -buf_v[k][j];
						}
						buf_v[6][k]		= Eik1[k];				buf_v[7][k]		= Eik2[k];
						buf_v[8][k]		= -Ejk1[k];				buf_v[9][k]		= -Ejk2[k];
					}
					m_indices[0] = ik_posid;						m_indices[1] = jk_posid;
					m_indices[2] = ik_norid;						m_indices[3] = jk_norid;
				}
				else
				{
					for(k = 0; k < 3; k++)
					{
						for(j = 0; j < 3; j++)
						{
							buf_v[3+k][j]		= lambda*rijk[k]*nijk[j];
							buf_v[k][j]	= -buf_v[3+k][j];
						}
						buf_v[8][k]		= Eik1[k];				buf_v[9][k]		= Eik2[k];
						buf_v[6][k]		= -Ejk1[k];				buf_v[7][k]		= -Ejk2[k];
					}
					m_indices[1] = ik_posid;						m_indices[0] = jk_posid;
					m_indices[3] = ik_norid;						m_indices[2] = jk_norid;
				}
				for(k = 0; k < 10; k++)
				{
					vec[k] = -(nijk[0]*buf_v[k][0]+nijk[1]*buf_v[k][1]+nijk[2]*buf_v[k][2])*weight;
					for(j = 0; j < 10; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]
						+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
					}
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 4, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 4, gradient);
			}

			/// Circular smoothness potential
			if(flag_CirS)
			{
				weight = w_cir_s*sweightijk*sedgeweight/numOfEntries;
//				if(!ik_need_smoothness_flag||!jk_need_smoothness_flag)
//					weight /= 100.0f;

				float32	buf_v[10];
				m_mul[0] = m_mul[1] = 3;			m_mul[2] = m_mul[3] = 2;
				if(surfelIndex < neighbor_id)
				{
					for(k = 0; k < 3; k++)
					{
						buf_v[k]		= lambda*cirijk*rijk[k] + nik[k]+njk[k];
						buf_v[3+k]	= -buf_v[k];	
					}
					buf_v[6]		= Eik1*rijk;						buf_v[7]		= Eik2*rijk;
					buf_v[8]		= Ejk1*rijk;					buf_v[9]		= Ejk2*rijk;
					m_indices[0] = ik_posid;					m_indices[1] = jk_posid;
					m_indices[2] = ik_norid;					m_indices[3] = jk_norid;
				}
				else
				{
					for(k = 0; k < 3; k++)
					{
						buf_v[3+k]		= lambda*cirijk*rijk[k] + nik[k]+njk[k];
						buf_v[k]			= -buf_v[3+k];	
					}
					buf_v[8]		= Eik1*rijk;						buf_v[9]		= Eik2*rijk;
					buf_v[6]		= Ejk1*rijk;					buf_v[7]		= Ejk2*rijk;
					m_indices[1] = ik_posid;					m_indices[0] = jk_posid;
					m_indices[3] = ik_norid;					m_indices[2] = jk_norid;
				}
				for(k = 0; k < 10; k++)
				{
					vec[k] = -cirijk*buf_v[k]*weight;
					for(j = 0; j < 10; j++)
					{
						mat[k][j] += buf_v[k]*buf_v[j]*weight;
					}
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 4, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 4, gradient);
			}

			/// Laplacian smoothness potential
			if(flag_LapS)
			{
				cen_pik -= pjk;
			}

			/// Position rigidity potential
			if(frameIndex > 0&&flag_PosR && ikp_depth <= 1 && jkp_depth <= 1)
			{
				weight = w_pos_r*redgeweight/numOfEntries;
				float32	buf_v[15][3];
				m_mul[0] = m_mul[1] = m_mul[2] = m_mul[3] = m_mul[4] = 3;
				if(surfelIndex < neighbor_id)
				{
					for(k = 0; k < 3; k++)
					{
						for(j = 0; j < 3; j++)
						{
							buf_v[k][j] = Aikp[k][j];
							buf_v[3+k][j] = -buf_v[k][j];
							buf_v[6+k][j] = 0.0f;			buf_v[9+k][j] = 0.0f;
						}
						buf_v[6+k][k] = -1.0f;			buf_v[9+k][k] = 1.0f;
					}
					m_indices[0] = ikp_posid;			m_indices[1] = jkp_posid;
					m_indices[2] = ik_posid;				m_indices[3] = jk_posid;
				}
				else
				{
					for(k = 0; k < 3; k++)
					{
						for(j = 0; j < 3; j++)
						{
							buf_v[3+k][j] = Aikp[k][j];
							buf_v[k][j] = -buf_v[3+k][j];
							buf_v[9+k][j] = 0.0f;			buf_v[6+k][j] = 0.0f;
						}
						buf_v[9+k][k] = -1.0f;			buf_v[6+k][k] = 1.0f;
					}
					m_indices[1] = ikp_posid;			m_indices[0] = jkp_posid;
					m_indices[3] = ik_posid;				m_indices[2] = jk_posid;
				}
				buf_v[12][0] = 0.0f;			buf_v[12][1] = -rijkpc[2];	buf_v[12][2] = rijkpc[1];
				buf_v[13][0] = rijkpc[2];	buf_v[13][1] = 0.0f;			buf_v[13][2] = -rijkpc[0];
				buf_v[14][0] = -rijkpc[1];	buf_v[14][1] = rijkpc[0];	buf_v[14][2] = 0.0f;
				m_indices[4] = ikp_rotid;
				for(k = 0; k < 15; k++)
				{
					vec[k] = -(dev_pijk[0]*buf_v[k][0]+dev_pijk[1]*buf_v[k][1]+dev_pijk[2]*buf_v[k][2])*weight;
					for(j = 0; j < 15; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
					}
				}

				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 5, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 5, gradient);
			}

			/// Normal rigidity potential
			if(frameIndex > 0 && flag_NorR && ikp_depth <= 1 && jkp_depth <= 1)
			{
				weight	 = w_nor_r*redgeweight/numOfEntries;
				float32	buf_v[11][3];
				m_mul[0] = m_mul[1] = m_mul[2] = m_mul[3] = 2;	m_mul[4] = 3;
				if(surfelIndex < neighbor_id)
				{
					for(j = 0; j < 3; j++)
					{
						buf_v[0][j] = AikpEikp1[j];
						buf_v[1][j] = AikpEikp2[j];
						buf_v[2][j] = -AikpEjkp1[j];
						buf_v[3][j] = -AikpEjkp2[j];
						buf_v[4][j] = -Eik1[j];
						buf_v[5][j] = -Eik2[j];
						buf_v[6][j] = Ejk1[j];
						buf_v[7][j] = Ejk2[j];
					}
					m_indices[0] = ikp_norid;				m_indices[1] = jkp_norid;
					m_indices[2] = ik_norid;					m_indices[3] = jk_norid;
				}
				else
				{
					for(j = 0; j < 3; j++)
					{
						buf_v[2][j] = AikpEikp1[j];
						buf_v[3][j] = AikpEikp2[j];
						buf_v[0][j] = -AikpEjkp1[j];
						buf_v[1][j] = -AikpEjkp2[j];
						buf_v[6][j] = -Eik1[j];
						buf_v[7][j] = -Eik2[j];
						buf_v[4][j] = Ejk1[j];
						buf_v[5][j] = Ejk2[j];
					}
					m_indices[1] = ikp_norid;				m_indices[0] = jkp_norid;
					m_indices[3] = ik_norid;					m_indices[2] = jk_norid;
				}
				buf_v[8][0] = 0.0f;				buf_v[8][1] = -nijkpc[2];	buf_v[8][2] = nijkpc[1];
				buf_v[9][0] = nijkpc[2];		buf_v[9][1] = 0.0f;				buf_v[9][2] = -nijkpc[0];
				buf_v[10][0] = -nijkpc[1];	buf_v[10][1] = nijkpc[0];	buf_v[10][2] = 0.0f;
				m_indices[4] = ikp_rotid;
				for(k = 0; k < 11; k++)
				{
					vec[k] = -(dev_nijk[0]*buf_v[k][0]+dev_nijk[1]*buf_v[k][1]+dev_nijk[2]*buf_v[k][2])*weight;	
					for(j = 0; j < 11; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
					}
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 5, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 5, gradient);
			}
		}
		if(flag_LapS && numOfEntries >= 1 && flag)
		{
			cen_pik /= m_array.size();
			cen_pik += pik;
			BinaryInsertion_Sort(m_array, surfelIndex, i_index);
			card32	m_dim = 3*m_array.size(), id, m, n;

			float32	**buf_v = new float32*[m_dim];
			for(id = 0; id < m_dim; id++)
			{
				buf_v[id] = new float32[3];
			}
			DMatrixF								mat1;			mat1.setDimension(m_dim, m_dim, false);		
			DVectorF							vec1;			vec1.setDim(m_dim);									
			DVectorI								m_indices1;	m_indices1.setDim(m_array.size());							
			DVectorI								m_mul1;		m_mul1.setDim(m_array.size());
			float32								sumSqrNorm = 0.0;

			for(id = 0; id < m_array.size(); id++)
			{
				if(m_array[id] == surfelIndex)
				{
					for(m = 0; m < 3; m++)
					{
						for(n = 0; n < 3; n++)
						{
							buf_v[3*id + m][n] = lambda*cen_pik[m]*cen_pik[n];
						}
						buf_v[3*id+m][m] += 1.0f;
					}
				}
				else
				{
					pjk = ps->get3f(frameIndex, m_array[id], POSITION);
					rijk = pik - pjk;
					sumSqrNorm += rijk.getSqrNorm();
					for(m = 0; m < 3; m++)
					{
						for(n = 0; n < 3; n++)
						{
							buf_v[3*id + m][n] = -lambda*rijk[m]*cen_pik[n]/(m_array.size()-1);
						}
						buf_v[3*id+m][m] -= 1.0f/(m_array.size()-1);
					}
				}
				ik_posid = ps->get<int32>(frameIndex, m_array[id], POS_INDEX);
				m_indices1[id] = ik_posid;
				m_mul1[id] = 3;
			}
			sumSqrNorm /= m_array.size() - 1;
			float32	weight = exp(lambda*sumSqrNorm)*w_lap_s;
//			if(!ik_need_smoothness_flag||!jk_need_smoothness_flag)
//				weight /= 100.0f;

			for(k = 0; k < m_dim; k++)
			{
				vec1[k] = -(cen_pik[0]*buf_v[k][0]+cen_pik[1]*buf_v[k][1]+cen_pik[2]*buf_v[k][2])*weight;	
				for(j = 0; j < m_dim; j++)
				{
					mat1[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
				}
			}
			UpdateMatrix_by_a_matrix(&mat1, &m_indices1, & m_mul1, m_array.size(), quadApprox);
			UpdateVector_by_a_vector(&vec1, &m_indices1, & m_mul1, m_array.size(), gradient);


			for(id = 0; id < m_dim; id++)
			{
				delete []buf_v[id];
			}
			delete []buf_v;
		}
	}*/

	bool		flag_SDF		= potential_flags[0], flag_Mar	 = potential_flags[1], flag_PlaS	=	potential_flags[2], flag_NorS	= potential_flags[3],
				flag_CirS		= potential_flags[4], flag_LapS	 = potential_flags[5], flag_PosR	= potential_flags[6], flag_NorR	= potential_flags[7],
				flag_STPC	=	potential_flags[8], flag_STNC = potential_flags[9], flag_STRC = potential_flags[10];

	float32	w_sdf		= context->weightSquaredDistance,		w_marker	=	context->weightMarkerError,
				w_pla_s		= context->weightPlanarSmoothness,		w_nor_s		= context->weightNormalSmoothness,
				w_cir_s		= context->weightCircularSmoothness,	w_lap_s		= context->weightLaplacianSmoothness,
				w_pos_r		= context->weightPositionRigidity,			w_nor_r		= context->weightNormalRigidity,
				w_stpc		= context->weightPositionCoherence,		w_stnc		= context->weightNormalCoherence,
				w_strc		= context->weightRotationCoherence;

	/// Controls the smoothness
	float32	lambda	=	context->lambda;

	/// Get geometry and topology data structure from patch graph

	PointSet								*ps					= patchGraph->getTrajectorySet()->getTrajectories();
	PointSet								*urshape			= patchGraph->getUrshape();
	InCorePCTopologyGraph					*tg					= patchGraph->getTopology();
	PGDataPointIndices						*dataPoints		= patchGraph->getTrajectorySet()->getDataPoints();

	/// The dimension of the patchgraph
	card32								width				=	ps->getWidth(),
												height				= ps->getHeight();

	/// Buffers for updating sparse matrix 
	DMatrixF								mat;			mat.setDimension(15, 15, false);		
	DVectorF							vec;			vec.setDim(15);									
	DVectorI								m_indices;	m_indices.setDim(5);							
	DVectorI								m_mul;		m_mul.setDim(5);

	/// Attributes
	AAT					POSITION		= ps->getAAT("position"),			NORMAL		= ps->getAAT("normal"),
						REF_U			= ps->getAAT("tangent_u"),			ROTATION	= ps->getAAT("rotation_matrix"),
						POS_INDEX		= ps->getAAT("varIndex_pos"),		NOR_INDEX	= ps->getAAT("varIndex_nor"),
						TOP_DEPTH		= ps->getAAT("topology_depth"),		ROT_INDEX	= ps->getAAT("varIndex_rot"),
						AAT_FLAGS	= ps->getAAT("flags"),				// this is an int32[1] channel
						DATA_POS_AAT	= data->getFrame(0)->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );

	/// Variables to be used
	Vector3f								pikp,nikp,Eikp1,Eikp2,AikpEikp1,AikpEikp2, 
												pjkp,njkp,Ejkp1,Ejkp2,AikpEjkp1,AikpEjkp2,
												pik,nik,Eik1,Eik2,pjk,njk,Ejk1,Ejk2,
												pikn, nikn,Eikn1,Eikn2,
												AikEikp1,AikEikp2, AikEik1,AikEik2,
												rijk,rijkp,rijkpc,nijk,nijkp,nijkpc,
												rikpk, nikpk, rikpkc, nikpkc, rikkn, nikkn, 
												dev_pijk, dev_nijk, cen_pik, dev_pikkpkn, dev_nikkpkn;

	/// Distances
	float32									disijk, cirijk;

	/// Indies
	int32									ik_posid, ik_norid, ikp_posid, ikp_norid, ikp_rotid,
												jk_posid, jk_norid, jkp_posid, jkp_norid, 
												ik_depth, jk_depth, ikp_depth, jkp_depth,
												ikn_posid, ikn_norid, ik_rotid, ikn_depth;

	bool									ik_need_smoothness_flag,
											jk_need_smoothness_flag;

	/// Current term rotation matrix
	Matrix3f								Aikp, Aik;

	card32									offset = 0,					/// Access edge_weight for smoothness and rigidity
											numOfEdges = 0;		/// Number of edges in the topology graph

	for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		numOfEdges += tg->getNumAdjacentVertices(surfelIndex);
	}

	/// Compute Marker-error potential,
	/// Planar, Normal, Circular and Laplacian smoothness potentials,
	/// Position and rigidity potentials in turn.

	for(card32	index = 0; index < indices_related->size(); index++)
	{
		///number of neighbors
		card32			frameIndex		= (*indices_related)[index].frame;
		card32			surfelIndex		= (*indices_related)[index].point;
		ik_depth		=	ps->get<int32>(frameIndex, surfelIndex, TOP_DEPTH);
		if(ik_depth >= 2)
			continue;
		card32			numOfEntries = tg->getNumAdjacentVertices(surfelIndex);
		card32			ni, neighbor_id, i_index;
		card32			j, k;


		ik_posid			= ps->get<int32>(frameIndex, surfelIndex, POS_INDEX);
		ik_norid			= ps->get<int32>(frameIndex, surfelIndex, NOR_INDEX);
		ik_need_smoothness_flag = (ps->get<int32>(frameIndex, surfelIndex, AAT_FLAGS) & FLAG_PATCH_NOSMOOTH) == 0;

		/// get geometry
		pik					= ps->get3f(frameIndex, surfelIndex,	POSITION);
		nik					= ps->get3f(frameIndex, surfelIndex,  NORMAL); 
		Eik1				= ps->get3f(frameIndex, surfelIndex, REF_U);
		Eik2				= nik.crossProduct(Eik1);

		pikp				= ps->get3f((frameIndex+width-1)%width, surfelIndex,	POSITION);
		nikp				= ps->get3f((frameIndex+width-1)%width, surfelIndex,  NORMAL); 
		Eikp1				= ps->get3f((frameIndex+width-1)%width, surfelIndex, REF_U);
		Eikp2				= nik.crossProduct(Eikp1);

		ikp_posid			= ps->get<int32>((frameIndex+width-1)%width, surfelIndex, POS_INDEX);
		ikp_norid			= ps->get<int32>((frameIndex+width-1)%width, surfelIndex, NOR_INDEX);
		ikp_rotid			= ps->get<int32>((frameIndex+width-1)%width, surfelIndex, ROT_INDEX);
		
		ikp_depth			= ps->get<int32>((frameIndex+width-1)%width, surfelIndex, TOP_DEPTH);

		/// Get the current rotation matrix
		char			*dataC	= ps->getDataPointer(frameIndex-1, surfelIndex);
		float32	*quatc	= (float*)(dataC+ROTATION.getOffset());
		Quaternion_To_Matrix3f_GNM(quatc, Aikp);

		/// These variables will be used in normal rigidity potential
		AikpEikp1	= Aikp*Eikp1;
		AikpEikp2	= Aikp*Eikp2;
		if(frameIndex < width -1 && frameIndex > 0)
		{
			pikn			= ps->get3f(frameIndex+1, surfelIndex,	POSITION);
			nikn			= ps->get3f(frameIndex+1, surfelIndex,  NORMAL); 
			Eikn1			= ps->get3f(frameIndex+1, surfelIndex, REF_U);
			Eikn2			= nik.crossProduct(Eikp1);

			ikn_posid		= ps->get<int32>(frameIndex +1, surfelIndex, POS_INDEX);
			ikn_norid		= ps->get<int32>(frameIndex +1, surfelIndex, NOR_INDEX);
			ik_rotid		= ps->get<int32>(frameIndex , surfelIndex, ROT_INDEX);
			ikn_depth		= ps->get<int32>(frameIndex +1, surfelIndex, TOP_DEPTH);

			char	*dataC1	= ps->getDataPointer(frameIndex, surfelIndex);
			float32	*quatc1	= (float*)(dataC1+ROTATION.getOffset());
			Quaternion_To_Matrix3f_GNM(quatc1, Aik);

			AikEik1		=	Aik*Eik1;				AikEik2		= Aik*Eik2;
			AikEikp1	=	Aik*Eikp1;				AikEikp2	= Aik*Eikp2;
			rikpk		=	pikp - pik;				nikpk		= nikp - nik;
			rikkn		=	pik- pikn;				nikkn		= nik - nikn;
			rikpkc		=	Aik*rikpk;				nikpkc		= Aik*nikpk;
			dev_pikkpkn	=	rikpkc - rikkn;
			dev_nikkpkn	=	nikpkc - nikkn;
			if(flag_STPC&&ikp_depth < 2 && ikn_depth < 2)
			{
				float32	weight = w_stpc;
				Vector3f	curvik = rikpk - rikkn;
				float32	buf_v[9][3];
				m_mul[0] = m_mul[1] = m_mul[2]  = 3;
				for(k = 0; k < 3; k++)
				{
					for(j = 0; j < 3; j++)
					{
						buf_v[k][j] = 0.0f;
						buf_v[3+k][j] = 0.0f;
						buf_v[6+k][j] = 0.0f;
					}
					buf_v[k][k] = 1.0f;			buf_v[3+k][k] = -2.0f;			buf_v[6+k][k] = 1.0f;
				}
				m_indices[0] = ikp_posid;			m_indices[1] = ik_posid;		m_indices[2] = ikn_posid;

				for(k = 0; k < 9; k++)
				{
					vec[k] = -(curvik[0]*buf_v[k][0]+curvik[1]*buf_v[k][1]+curvik[2]*buf_v[k][2])*weight;
					for(j = 0; j < 9; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
					}
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 4, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 4, gradient);
			}
			if(flag_STNC&&ikp_depth < 2 && ikn_depth < 2)
			{
				float32 weight	 = w_stnc;
				float32	buf_v[9][3];
				m_mul[0] = m_mul[1] = m_mul[2] = 2;	m_mul[3] = 3;
				for(j = 0; j < 3; j++)
				{
					buf_v[0][j] = AikEikp1[j];
					buf_v[1][j] = AikEikp2[j];
					buf_v[2][j] = -AikEik1[j] - Eik1[j];
					buf_v[3][j] = -AikEik2[j] - Eik2[j];
					buf_v[4][j] = Eikn1[j];
					buf_v[5][j] = Eikn2[j];
				}
				m_indices[0] = ikp_norid;			m_indices[1] = ik_norid;			m_indices[3] = ikn_norid;

				buf_v[6][0] = 0.0f;				buf_v[6][1] = -nikpkc[2];	buf_v[6][2] = nikpkc[1];
				buf_v[7][0] = nikpkc[2];		buf_v[7][1] = 0.0f;				buf_v[7][2] = -nikpkc[0];
				buf_v[8][0] = -nikpkc[1];	buf_v[8][1] = nikpkc[0];	buf_v[8][2] = 0.0f;
				m_indices[3] = ik_rotid;
				for(k = 0; k < 9; k++)
				{
					vec[k] = -(dev_nikkpkn[0]*buf_v[k][0]+dev_nikkpkn[1]*buf_v[k][1]+dev_nikkpkn[2]*buf_v[k][2])*weight;	
					for(j = 0; j < 9; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
					}
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 4, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 4, gradient);
			}
			if(flag_STRC&&ikp_depth < 2)
			{
				float32	 weight	 = w_strc;
				float32	buf_v[6][9];
				float32	buf_colume[9];
				m_mul[0] = m_mul[1] = 3;

				for(k = 0; k < 3; k++)
				{
					for(j = 0; j < 3; j++)
					{
						buf_colume[3*k + j] = Aikp[k][j] - Aik[k][j];
					}
					buf_v[0][3*k+0] = 0.0f;				buf_v[0][3*k+1] = -Aikp[k][0];		buf_v[0][3*k+2] = Aikp[k][1];
					buf_v[1][3*k+0] = Aikp[k][0];		buf_v[1][3*k+1] = 0.0f;				buf_v[1][3*k+2] = -Aikp[k][2];
					buf_v[2][3*k+0] = -Aikp[k][1];		buf_v[2][3*k+1] = Aikp[k][2];		buf_v[2][3*k+2] = 0.0f;

					buf_v[3][3*k+0] = 0.0f;				buf_v[3][3*k+1] = Aik[k][0];		buf_v[3][3*k+2] = -Aik[k][1];
					buf_v[4][3*k+0] = -Aik[k][0];		buf_v[4][3*k+1] = 0.0f;				buf_v[4][3*k+2] = Aik[k][2];
					buf_v[5][3*k+0] = Aik[k][1];		buf_v[5][3*k+1] = -Aik[k][2];		buf_v[5][3*k+2] = 0.0f;
				}
				m_indices[0] = ikp_rotid;			m_indices[1] = ik_rotid;
				for(k = 0; k < 6; k++)
				{
					vec[k] = -(buf_colume[0]*buf_v[k][0]+buf_colume[1]*buf_v[k][1]+buf_colume[2]*buf_v[k][2]
					+ buf_colume[3]*buf_v[k][3]+buf_colume[4]*buf_v[k][4]+buf_colume[5]*buf_v[k][5]
					+ buf_colume[6]*buf_v[k][6]+buf_colume[7]*buf_v[k][7]+buf_colume[8]*buf_v[k][8])*weight;	
					for(j = 0; j < 6; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2]
						+buf_v[k][3]*buf_v[j][3]+buf_v[k][4]*buf_v[j][4]+buf_v[k][5]*buf_v[j][5]
						+buf_v[k][6]*buf_v[j][6]+buf_v[k][7]*buf_v[j][7]+buf_v[k][8]*buf_v[j][8])*weight;
					}
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 2, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 2, gradient);
			}
		}


		/// initialize for Laplaciansmoothness potential
		cen_pik[0] = cen_pik[1] = cen_pik[2] = 0.0;
		vector<int32>	m_array;
		
		/// Marker-error potential
		if(flag_Mar)
		{
			PatchGraphWCorr *corrPG = dynamic_cast<PatchGraphWCorr*>(patchGraph);
			if (corrPG == NULL) {
				throw PException("GaussNewtonMethod.cpp - corrPG is NULL (this should not happen).");
			}
			const map<card32, Vector3f> &correspondences = corrPG->correspondences;
			
			float32	weight = w_marker;
			
			map<card32, Vector3f>::const_iterator corrIt = correspondences.find(surfelIndex);
			if (corrIt != correspondences.end()) {
				Vector3f		devik	= pik - corrIt->second;
				m_mul[0]				= 3;
				m_indices[0]			= ik_posid;
				/// Update Gradient
				for(j = 0; j < 3; j++)
				{
					vec[j]								= -devik[j]*weight;
					for(k = 0; k < 3; k++)
					{
						mat[j][k]					= 0.0f;
					}
					mat[j][j]							+= weight;
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 1, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 1, gradient);			
			}

			const card32 numDataPoints = dataPoints->getNumDataPoints( frameIndex, surfelIndex);
//			float32	weight = w_marker;
			if (numDataPoints > 0 && index_transfer[frameIndex] >= 0) 
			{
				card32 dataIndexInData	= dataPoints->getIndex( frameIndex, surfelIndex, 0);
				Vector3f		devik	= pik - data->getFrame(index_transfer[frameIndex])->get3f(dataIndexInData, DATA_POS_AAT);
				m_mul[0]				= 3;
				m_indices[0]			= ik_posid;
				/// Update Gradient
				for(j = 0; j < 3; j++)
				{
					vec[j]								= -devik[j]*weight;
					for(k = 0; k < 3; k++)
					{
						mat[j][k]					= 0.0f;
					}
					mat[j][j]							+= weight;
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 1, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 1, gradient);
			}
		}
		bool flag = true;
		for(ni = 0; ni < numOfEntries; ni++)
		{
			/// the index of this neighbor
			neighbor_id	= tg->getAdjacentVertex(surfelIndex, ni);
			if(neighbor_id == surfelIndex)
			{
				continue;
			}

			BinaryInsertion_Sort(m_array, neighbor_id, i_index);
			jk_depth			=	ps->get<int32>(frameIndex, neighbor_id, TOP_DEPTH);
			if(jk_depth >= 2)
			{
				flag = false;
				continue;
			}
			/// the geometry of this neighbor
			pjk					=	ps->get3f(frameIndex, neighbor_id,	POSITION);
			njk					=	ps->get3f(frameIndex, neighbor_id,  NORMAL); 
			Ejk1				=	ps->get3f(frameIndex, neighbor_id, REF_U);
			Ejk2				=	njk.crossProduct(Ejk1);

			jk_posid			=	ps->get<int32>(frameIndex, neighbor_id, POS_INDEX);
			jk_norid			=	ps->get<int32>(frameIndex, neighbor_id, NOR_INDEX);

			jk_need_smoothness_flag = (ps->get<int32>(frameIndex, surfelIndex, AAT_FLAGS) & FLAG_PATCH_NOSMOOTH) == 0;
			
			/// precomputed geometric quantities 
			rijk			=	pik	-	pjk;
			nijk			=	nik	-	njk;
			disijk			=	nik*rijk;
			cirijk			=	(nik+njk)*rijk;

			pjkp			=	ps->get3f((frameIndex+width-1)%width, neighbor_id,	POSITION);
			njkp			=	ps->get3f((frameIndex+width-1)%width, neighbor_id,  NORMAL); 
			Ejkp1			=	ps->get3f((frameIndex+width-1)%width, neighbor_id, REF_U);
			Ejkp2			=	nik.crossProduct(Ejkp1);
			AikpEjkp1		=	Aikp*Ejkp1;
			AikpEjkp2		=	Aikp*Ejkp2;

			jkp_posid		=   ps->get<int32>((frameIndex+width-1)%width, neighbor_id, POS_INDEX);
			jkp_norid		=	ps->get<int32>((frameIndex+width-1)%width, neighbor_id, NOR_INDEX);
			jkp_depth		=	ps->get<int32>((frameIndex+width-1)%width, neighbor_id, TOP_DEPTH);

			rijkp			=	pikp		-	pjkp;
			rijkpc			=	Aikp*rijkp;
			nijkp			=	nikp		-	njkp;
			nijkpc			=	Aikp*nijkp;
			dev_pijk		=	rijkpc	-	rijk;
			dev_nijk		=	nijkpc	-	nijk;

			/// precomputed weights
			float32	sweightijk = exp(lambda*rijk.getSqrNorm());
			float32	sedgeweight, redgeweight;
			sedgeweight	= 1.0f;
			redgeweight	= 1.0f;

			float32	weight;

			///Planar smoothness potential
			if(flag_PlaS)
			{
				weight = w_pla_s*sweightijk*sedgeweight/numOfEntries;
//				if(!ik_need_smoothness_flag||!jk_need_smoothness_flag)
//					weight /= 100.0f;
				float32	buf_v[8];
				m_mul[0] = m_mul[1] = 3;		m_mul[2] = 2;
				if(surfelIndex < neighbor_id)
				{
					for(k = 0; k < 3; k++)
					{
						buf_v[k]		= nik[k] + lambda*disijk*rijk[k];
						buf_v[3+k]	= -buf_v[k];
					}
					m_indices[0] = ik_posid;			m_indices[1] = jk_posid;
				}
				else
				{
					for(k = 0; k < 3; k++)
					{
						buf_v[3+k]		= nik[k] + lambda*disijk*rijk[k];
						buf_v[k]	= -buf_v[3+k];
					}
					m_indices[1] = ik_posid;			m_indices[0] = jk_posid;
				}
				buf_v[6]			= rijk*Eik1;				buf_v[7]			= rijk*Eik2;			m_indices[2] = ik_norid;
				for(k = 0; k < 8; k++)
				{
					vec[k] = -disijk*buf_v[k]*weight;
					for(j = 0; j < 8; j++)
					{
						mat[k][j] = buf_v[k]*buf_v[j]*weight;
					}
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 3, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 3, gradient);
			}
			/// Normal smoothness potential
			if(flag_NorS)
			{
				weight = w_nor_s*sweightijk*sedgeweight/numOfEntries;
//				if(!ik_need_smoothness_flag||!jk_need_smoothness_flag)
//					weight /= 100.0f;

				float32	buf_v[10][3];
				m_mul[0] = m_mul[1] = 3;		m_mul[2] = m_mul[3] = 2;
				if(surfelIndex < neighbor_id)
				{
					for(k = 0; k < 3; k++)
					{
						for(j = 0; j < 3; j++)
						{
							buf_v[k][j]		= lambda*rijk[k]*nijk[j];
							buf_v[3+k][j]	= -buf_v[k][j];
						}
						buf_v[6][k]		= Eik1[k];				buf_v[7][k]		= Eik2[k];
						buf_v[8][k]		= -Ejk1[k];				buf_v[9][k]		= -Ejk2[k];
					}
					m_indices[0] = ik_posid;						m_indices[1] = jk_posid;
					m_indices[2] = ik_norid;						m_indices[3] = jk_norid;
				}
				else
				{
					for(k = 0; k < 3; k++)
					{
						for(j = 0; j < 3; j++)
						{
							buf_v[3+k][j]		= lambda*rijk[k]*nijk[j];
							buf_v[k][j]	= -buf_v[3+k][j];
						}
						buf_v[8][k]		= Eik1[k];				buf_v[9][k]		= Eik2[k];
						buf_v[6][k]		= -Ejk1[k];				buf_v[7][k]		= -Ejk2[k];
					}
					m_indices[1] = ik_posid;						m_indices[0] = jk_posid;
					m_indices[3] = ik_norid;						m_indices[2] = jk_norid;
				}
				for(k = 0; k < 10; k++)
				{
					vec[k] = -(nijk[0]*buf_v[k][0]+nijk[1]*buf_v[k][1]+nijk[2]*buf_v[k][2])*weight;
					for(j = 0; j < 10; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]
						+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
					}
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 4, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 4, gradient);
			}

			/// Circular smoothness potential
			if(flag_CirS)
			{
				weight = w_cir_s*sweightijk*sedgeweight/numOfEntries;
//				if(!ik_need_smoothness_flag||!jk_need_smoothness_flag)
//					weight /= 100.0f;

				float32	buf_v[10];
				m_mul[0] = m_mul[1] = 3;			m_mul[2] = m_mul[3] = 2;
				if(surfelIndex < neighbor_id)
				{
					for(k = 0; k < 3; k++)
					{
						buf_v[k]		= lambda*cirijk*rijk[k] + nik[k]+njk[k];
						buf_v[3+k]	= -buf_v[k];	
					}
					buf_v[6]		= Eik1*rijk;						buf_v[7]		= Eik2*rijk;
					buf_v[8]		= Ejk1*rijk;					buf_v[9]		= Ejk2*rijk;
					m_indices[0] = ik_posid;					m_indices[1] = jk_posid;
					m_indices[2] = ik_norid;					m_indices[3] = jk_norid;
				}
				else
				{
					for(k = 0; k < 3; k++)
					{
						buf_v[3+k]		= lambda*cirijk*rijk[k] + nik[k]+njk[k];
						buf_v[k]			= -buf_v[3+k];	
					}
					buf_v[8]		= Eik1*rijk;						buf_v[9]		= Eik2*rijk;
					buf_v[6]		= Ejk1*rijk;					buf_v[7]		= Ejk2*rijk;
					m_indices[1] = ik_posid;					m_indices[0] = jk_posid;
					m_indices[3] = ik_norid;					m_indices[2] = jk_norid;
				}
				for(k = 0; k < 10; k++)
				{
					vec[k] = -cirijk*buf_v[k]*weight;
					for(j = 0; j < 10; j++)
					{
						mat[k][j] += buf_v[k]*buf_v[j]*weight;
					}
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 4, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 4, gradient);
			}

			/// Laplacian smoothness potential
			if(flag_LapS)
			{
				cen_pik -= pjk;
			}

			/// Position rigidity potential
			if(ikp_rotid>=0&&flag_PosR && ikp_depth <= 1 && jkp_depth <= 1)
			{
				weight = w_pos_r*redgeweight/numOfEntries;
				float32	buf_v[15][3];
				m_mul[0] = m_mul[1] = m_mul[2] = m_mul[3] = m_mul[4] = 3;
				if(surfelIndex < neighbor_id)
				{
					if(ikp_posid < ik_posid)
					{
						for(k = 0; k < 3; k++)
						{
							for(j = 0; j < 3; j++)
							{
								buf_v[k][j] = Aikp[k][j];
								buf_v[3+k][j] = -buf_v[k][j];
								buf_v[6+k][j] = 0.0f;			buf_v[9+k][j] = 0.0f;
							}
							buf_v[6+k][k] = -1.0f;			buf_v[9+k][k] = 1.0f;
						}
						m_indices[0] = ikp_posid;			m_indices[1] = jkp_posid;
						m_indices[2] = ik_posid;				m_indices[3] = jk_posid;
					}
					else
					{
						for(k = 0; k < 3; k++)
						{
							for(j = 0; j < 3; j++)
							{
								buf_v[6+k][j] = Aikp[k][j];
								buf_v[9+k][j] = -buf_v[k][j];
								buf_v[k][j] = 0.0f;			buf_v[3+k][j] = 0.0f;
							}
							buf_v[k][k] = -1.0f;			buf_v[3+k][k] = 1.0f;
						}
						m_indices[2] = ikp_posid;			m_indices[3] = jkp_posid;
						m_indices[0] = ik_posid;			m_indices[1] = jk_posid;
					}
				}
				else
				{
					if(ikp_posid < ik_posid)
					{
						for(k = 0; k < 3; k++)
						{
							for(j = 0; j < 3; j++)
							{
								buf_v[3+k][j] = Aikp[k][j];
								buf_v[k][j] = -buf_v[3+k][j];
								buf_v[9+k][j] = 0.0f;			buf_v[6+k][j] = 0.0f;
							}
							buf_v[9+k][k] = -1.0f;			buf_v[6+k][k] = 1.0f;
						}
						m_indices[1] = ikp_posid;			m_indices[0] = jkp_posid;
						m_indices[3] = ik_posid;				m_indices[2] = jk_posid;
					}
					else
					{
						for(k = 0; k < 3; k++)
						{
							for(j = 0; j < 3; j++)
							{
								buf_v[9+k][j] = Aikp[k][j];
								buf_v[6+k][j] = -buf_v[3+k][j];
								buf_v[3+k][j] = 0.0f;			buf_v[k][j] = 0.0f;
							}
							buf_v[3+k][k] = -1.0f;			buf_v[k][k] = 1.0f;
						}
						m_indices[3] = ikp_posid;			m_indices[2] = jkp_posid;
						m_indices[1] = ik_posid;			m_indices[0] = jk_posid;
					}
				}
				buf_v[12][0] = 0.0f;			buf_v[12][1] = -rijkpc[2];	buf_v[12][2] = rijkpc[1];
				buf_v[13][0] = rijkpc[2];	buf_v[13][1] = 0.0f;			buf_v[13][2] = -rijkpc[0];
				buf_v[14][0] = -rijkpc[1];	buf_v[14][1] = rijkpc[0];	buf_v[14][2] = 0.0f;
				m_indices[4] = ikp_rotid;

				for(k = 0; k < 15; k++)
				{
					vec[k] = -(dev_pijk[0]*buf_v[k][0]+dev_pijk[1]*buf_v[k][1]+dev_pijk[2]*buf_v[k][2])*weight;
					for(j = 0; j < 15; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
					}
				}


				//if(ikp_rotid >= int32(gradient->getDim())-3)
				//{
				//	FILE *file = fopen("J:\\fc.txt","w");
				//	for(k = 0; k < 15; k++)
				//	{
				//		for(j = 0; j < 3; j++)
				//		{
				//			fprintf(file, "%f ", buf_v[k][j]);
				//		}
				//		fprintf(file, "\n");
				//	}
				//	fprintf(file,"%d %d %d %d %d %d\n", gradient->getDim(), m_indices[0],m_indices[1],m_indices[2],m_indices[3],m_indices[4]);
				//	fprintf(file,"%d fc", ikp_rotid);
				//	fclose(file);
				//}


				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 5, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 5, gradient);
			}

			/// Normal rigidity potential
			if(ikp_rotid>=0&&flag_NorR && ikp_depth <= 1 && jkp_depth <= 1)
			{
				weight	 = w_nor_r*redgeweight/numOfEntries;
				float32	buf_v[11][3];
				m_mul[0] = m_mul[1] = m_mul[2] = m_mul[3] = 2;	m_mul[4] = 3;
				if(surfelIndex < neighbor_id)
				{
					if(ikp_norid < ik_norid)
					{
						for(j = 0; j < 3; j++)
						{
							buf_v[0][j] = AikpEikp1[j];
							buf_v[1][j] = AikpEikp2[j];
							buf_v[2][j] = -AikpEjkp1[j];
							buf_v[3][j] = -AikpEjkp2[j];
							buf_v[4][j] = -Eik1[j];
							buf_v[5][j] = -Eik2[j];
							buf_v[6][j] = Ejk1[j];
							buf_v[7][j] = Ejk2[j];
						}
						m_indices[0] = ikp_norid;				m_indices[1] = jkp_norid;
						m_indices[2] = ik_norid;					m_indices[3] = jk_norid;
					}
					else
					{
						for(j = 0; j < 3; j++)
						{
							buf_v[4][j] = AikpEikp1[j];
							buf_v[5][j] = AikpEikp2[j];
							buf_v[6][j] = -AikpEjkp1[j];
							buf_v[7][j] = -AikpEjkp2[j];
							buf_v[0][j] = -Eik1[j];
							buf_v[1][j] = -Eik2[j];
							buf_v[2][j] = Ejk1[j];
							buf_v[3][j] = Ejk2[j];
						}
						m_indices[2] = ikp_norid;				m_indices[3] = jkp_norid;
						m_indices[0] = ik_norid;				m_indices[1] = jk_norid;
					}
				}
				else
				{
					if(ikp_norid < ik_norid)
					{
						for(j = 0; j < 3; j++)
						{
							buf_v[2][j] = AikpEikp1[j];
							buf_v[3][j] = AikpEikp2[j];
							buf_v[0][j] = -AikpEjkp1[j];
							buf_v[1][j] = -AikpEjkp2[j];
							buf_v[6][j] = -Eik1[j];
							buf_v[7][j] = -Eik2[j];
							buf_v[4][j] = Ejk1[j];
							buf_v[5][j] = Ejk2[j];
						}
						m_indices[1] = ikp_norid;				m_indices[0] = jkp_norid;
						m_indices[3] = ik_norid;				m_indices[2] = jk_norid;
					}
					else
					{
						for(j = 0; j < 3; j++)
						{
							buf_v[6][j] = AikpEikp1[j];
							buf_v[7][j] = AikpEikp2[j];
							buf_v[4][j] = -AikpEjkp1[j];
							buf_v[5][j] = -AikpEjkp2[j];
							buf_v[2][j] = -Eik1[j];
							buf_v[3][j] = -Eik2[j];
							buf_v[0][j] = Ejk1[j];
							buf_v[1][j] = Ejk2[j];
						}
						m_indices[3] = ikp_norid;				m_indices[2] = jkp_norid;
						m_indices[1] = ik_norid;				m_indices[0] = jk_norid;
					}
				}
				buf_v[8][0] = 0.0f;				buf_v[8][1] = -nijkpc[2];	buf_v[8][2] = nijkpc[1];
				buf_v[9][0] = nijkpc[2];		buf_v[9][1] = 0.0f;				buf_v[9][2] = -nijkpc[0];
				buf_v[10][0] = -nijkpc[1];	buf_v[10][1] = nijkpc[0];	buf_v[10][2] = 0.0f;
				m_indices[4] = ikp_rotid;
				for(k = 0; k < 11; k++)
				{
					vec[k] = -(dev_nijk[0]*buf_v[k][0]+dev_nijk[1]*buf_v[k][1]+dev_nijk[2]*buf_v[k][2])*weight;	
					for(j = 0; j < 11; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
					}
				}
				UpdateMatrix_by_a_matrix(&mat, &m_indices, & m_mul, 5, quadApprox);
				UpdateVector_by_a_vector(&vec, &m_indices, & m_mul, 5, gradient);
			}
		}
		if(flag_LapS && numOfEntries >= 1 && flag)
		{
			cen_pik /= m_array.size();
			cen_pik += pik;
			BinaryInsertion_Sort(m_array, surfelIndex, i_index);
			card32	m_dim = 3*(card32)m_array.size(), id, m, n;

			float32	**buf_v = new float32*[m_dim];
			for(id = 0; id < m_dim; id++)
			{
				buf_v[id] = new float32[3];
			}
			DMatrixF							mat1;			mat1.setDimension(m_dim, m_dim, false);		
			DVectorF							vec1;			vec1.setDim(m_dim);									
			DVectorI							m_indices1;		m_indices1.setDim((card32)m_array.size());							
			DVectorI							m_mul1;			m_mul1.setDim((card32)m_array.size());
			float32								sumSqrNorm = 0.0;

			for(id = 0; id < m_array.size(); id++)
			{
				if(m_array[id] == surfelIndex)
				{
					for(m = 0; m < 3; m++)
					{
						for(n = 0; n < 3; n++)
						{
							buf_v[3*id + m][n] = lambda*cen_pik[m]*cen_pik[n];
						}
						buf_v[3*id+m][m] += 1.0f;
					}
				}
				else
				{
					pjk = ps->get3f(frameIndex, m_array[id], POSITION);
					rijk = pik - pjk;
					sumSqrNorm += rijk.getSqrNorm();
					for(m = 0; m < 3; m++)
					{
						for(n = 0; n < 3; n++)
						{
							buf_v[3*id + m][n] = -lambda*rijk[m]*cen_pik[n]/(m_array.size()-1);
						}
						buf_v[3*id+m][m] -= 1.0f/(m_array.size()-1);
					}
				}
				ik_posid = ps->get<int32>(frameIndex, m_array[id], POS_INDEX);
				m_indices1[id] = ik_posid;
				m_mul1[id] = 3;
			}
			sumSqrNorm /= m_array.size() - 1;
			float32	weight = exp(lambda*sumSqrNorm)*w_lap_s;
//			if(!ik_need_smoothness_flag||!jk_need_smoothness_flag)
//				weight /= 100.0f;

			for(k = 0; k < m_dim; k++)
			{
				vec1[k] = -(cen_pik[0]*buf_v[k][0]+cen_pik[1]*buf_v[k][1]+cen_pik[2]*buf_v[k][2])*weight;	
				for(j = 0; j < m_dim; j++)
				{
					mat1[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
				}
			}
			UpdateMatrix_by_a_matrix(&mat1, &m_indices1, & m_mul1, (card32)m_array.size(), quadApprox);
			UpdateVector_by_a_vector(&vec1, &m_indices1, & m_mul1, (card32)m_array.size(), gradient);


			for(id = 0; id < m_dim; id++)
			{
				delete []buf_v[id];
			}
			delete []buf_v;
		}
	}
}

void	PatchGraphBSROpt::compute_GNM_Gradient_and_Hessian_SDF_Global(
	const vector<card32>					*indices_data,
	PointCloudAnimationSequence	*data, 
	PatchGraph		*patchGraph, 
	DVectorF			*gradient, 
	SparseMatrixF	*quadApprox,
	float32				tradeoff)
{
	if(tradeoff < 0.0)		tradeoff = 0.0f;					if(tradeoff > 1.0)		tradeoff = 1.0f;

	float32								w_sdf				= context->weightSquaredDistance;
	/// Get geometry and topology data structure from patch graph

	PointSet								*ps					= patchGraph->getTrajectorySet()->getTrajectories();
	PointSet								*urshape		= patchGraph->getUrshape();

	/// The dimension of the patchgraph
	card32								width				=	ps->getWidth(),
												height				= ps->getHeight();

	/// 
	DMatrixF								mat;			mat.setDimension(5, 5, false);		
	DVectorF							vec;			vec.setDim(5);									
	DVectorI								m_indices;	m_indices.setDim(2);							
	DVectorI								m_mul;		m_mul.setDim(2);

	/// Attributes
	AAT										POSITION		= ps->getAAT("position");
	AAT										NORMAL			= ps->getAAT("normal");
	AAT										REF_U				= ps->getAAT("tangent_u");
	AAT										POS_INDEX	= ps->getAAT("varIndex_pos");
	AAT										NOR_INDEX	= ps->getAAT("varIndex_nor");
	AAT										FLAG_INLIER	= data->getFrame(0)->getAAT("flag_inlier");
	AAT										DATA_POS		= data->getFrame(0)->getAAT("position");

	patchGraph->getTrajectorySet()->invalidateAllAttachments();
	PointSetKNNQuery **pg_knn = new PointSetKNNQuery*[patchGraph->getTrajectorySet()->getNumFrames()];
	PointSet **pg_knn_ps = new PointSet*[patchGraph->getTrajectorySet()->getNumFrames()];
	for (card32 frameNumInit=0; frameNumInit < patchGraph->getTrajectorySet()->getNumFrames(); frameNumInit++) {
		pg_knn[frameNumInit] = NULL;
		pg_knn_ps[frameNumInit] = NULL;
	}

	/// Variables to be used

	const									PointSet	*pc;
	card32								footIndex;

	for(card32 index = 0; index < indices_data->size(); index++)
	{
		int	frameIndex = (*indices_data)[index];
		pc	= data->getFrame(index);
		AttachedIndexedOctree *dataOctree = data->fetchOctree(index);
		PointSetKNNQuery data_knn(pc, dataOctree, NULL_VECTOR3F, pc->getAAT("position"));

		card32	numOfDataPoints = pc->getHeight();
		card32	pointIndex;
		for(pointIndex = 0; pointIndex	< numOfDataPoints; pointIndex++)
		{
			int32			flag_inlier	= pc->get<int32>(0, pointIndex, FLAG_INLIER);
			if(flag_inlier < 0)
				continue;
			Vector3f		pik				= pc->get3f(0, pointIndex, DATA_POS);

			if (pg_knn[frameIndex] == NULL) 
			{
				AttachedIndexedOctree *octree = patchGraph->getTrajectorySet()->fetchOctree(frameIndex);
				pg_knn_ps[frameIndex] = patchGraph->getTrajectorySet()->copySingleFrame(frameIndex);
				pg_knn[frameIndex] = new PointSetKNNQuery(pg_knn_ps[frameIndex], octree, NULL_VECTOR3F, ps->getAAT("position"));
			}
			pg_knn[frameIndex]->setTargetPoint(pik);
			{
				bool found;
				pg_knn[frameIndex]->getNextNearestPointIndex(footIndex, found);
				pAssert(found);
			}
			Vector3f		devik_foot	= pik - ps->get3f(frameIndex, footIndex, POSITION);
			Vector3f		nik_foot		= ps->get3f(frameIndex, footIndex, NORMAL);
			Vector3f		Eik1_foot	= ps->get3f(frameIndex, footIndex, REF_U);
			Vector3f		Eik2_foot	= nik_foot.crossProduct(Eik1_foot);
			int32			posid_foot	= ps->get<int32>(frameIndex, footIndex, POS_INDEX);
			int32			norid_foot	= ps->get<int32>(frameIndex, footIndex, NOR_INDEX);
			float32		weight			= w_sdf;
			
			float32		disik				= devik_foot*nik_foot;
			float			buf_v[5];
			m_mul[0]		= 3;							m_mul[1] = 2;
			m_indices[0] = posid_foot;		m_indices[1] = norid_foot;
			for(card32 k = 0; k < 3; k++)
			{
				buf_v[k] = - nik_foot[k];
			}
			buf_v[3]		= devik_foot*Eik1_foot;
			buf_v[4]		= devik_foot*Eik2_foot;
			for(card32 i = 0; i < 5; i++)
			{
				vec[i] = -disik*buf_v[i]*weight*(1-tradeoff);
				for(card32 j = 0; j < 5; j++)
				{
					mat[i][j] = buf_v[i]*buf_v[j]*weight*(1-tradeoff);
				}
			}
			for(card32 i = 0; i < 3; i++)
			{
				vec[i] += devik_foot[i]*weight*tradeoff;
				mat[i][i] += weight*tradeoff;
			}
			UpdateMatrix_by_a_matrix(&mat, &m_indices, &m_mul, 2, quadApprox);
			UpdateVector_by_a_vector(&vec, &m_indices, &m_mul, 2, gradient);
		}
	}
	for (card32 frameNumInit=0; frameNumInit < patchGraph->getTrajectorySet()->getNumFrames(); frameNumInit++) {
		delete pg_knn[frameNumInit];
		delete pg_knn_ps[frameNumInit];
	}
	delete[] pg_knn;
	delete[] pg_knn_ps;
}

float32	PatchGraphBSROpt::compute_GNM_Gradient_and_Hessian_SDF_Pairwise(
	const vector<card32> *indices_data, 
	PointCloudAnimationSequence *data, 
	PatchGraph		*patchGraph,
	SparseMatrixF	*topologyOfTargetFrame, 
	DVectorF		*gradient,
	SparseMatrixF	*quadApprox,
	float32			&previous_median_d2,
	const float32	tradeoff_sdf)
{
	float32	w_sdf		= context->weightSquaredDistance;
	/// Get geometry and topology data structure from patch graph

	PointSet								*ps					= patchGraph->getTrajectorySet()->getTrajectories();
	PointSet								*urshape		= patchGraph->getUrshape();

	/// The dimension of the patchgraph
	card32								width				=	ps->getWidth(),
		height				= ps->getHeight();

	/// 
	DMatrixF								mat;			mat.setDimension(5, 5, false);		
	DVectorF							vec;			vec.setDim(5);									
	DVectorI								m_indices;	m_indices.setDim(2);							
	DVectorI								m_mul;		m_mul.setDim(2);

	/// Attributes
	AAT										POSITION		= ps->getAAT("position");
	AAT										NORMAL			= ps->getAAT("normal");
	AAT										REF_U			= ps->getAAT("tangent_u");
	AAT										POS_INDEX		= ps->getAAT("varIndex_pos");
	AAT										NOR_INDEX		= ps->getAAT("varIndex_nor");
	AAT										DATA_POS_AAT	= data->getFrame(0)->getAAT("position", 3, VAD::DATA_FORMAT_FLOAT32 );
	AAT										FLAG_INLIER		= data->getFrame(0)->getAAT("flag_inlier");

	/// Variables to be used

	patchGraph->getTrajectorySet()->invalidateAllAttachments();
	PointSetKNNQuery **pg_knn = new PointSetKNNQuery*[patchGraph->getTrajectorySet()->getNumFrames()];
	PointSet **pg_knn_ps				= new PointSet*[patchGraph->getTrajectorySet()->getNumFrames()];
	for (card32 frameNumInit		=0; frameNumInit < patchGraph->getTrajectorySet()->getNumFrames(); frameNumInit++) 
	{
		pg_knn[frameNumInit] = NULL;
		pg_knn_ps[frameNumInit] = NULL;
	}

	float32 mean_d2;

	for(card32 index = 0; index < indices_data->size(); index++)
	{
		int		frameIndex		= (*indices_data)[index];
		PointSet*			pc	= data->getFrame(index);

		vector<Vector3f>		buf_dev_df_ik;						vector<Vector3f>		buf_dev_fd_ik;
		vector<Vector3f>		buf_frame_foot_nik;		vector<Vector3f>		buf_frame_foot_Eik1;
		vector<int32>			buf_varId_foot_pos;	vector<int32>			buf_varId_foot_nor;
		set<float32>				buf_d2;

		AttachedIndexedOctree *dataOctree = data->fetchOctree(index);
		PointSetKNNQuery data_knn(pc, dataOctree, NULL_VECTOR3F, pc->getAAT("position"));

		card32	numOfDataPoints = pc->getHeight();
		for(card32 pointIndex = 0; pointIndex	< numOfDataPoints; pointIndex++)
		{
			Vector3f		data_source_pik		= pc->get3f(0, pointIndex, DATA_POS_AAT);
			card32		footIndex_Frame, footIndex_Data;

			if (pg_knn[frameIndex] == NULL) {
				AttachedIndexedOctree *octree = patchGraph->getTrajectorySet()->fetchOctree(frameIndex);
				pg_knn_ps[frameIndex] = patchGraph->getTrajectorySet()->copySingleFrame(frameIndex);
				pg_knn[frameIndex] = new PointSetKNNQuery(pg_knn_ps[frameIndex], octree, NULL_VECTOR3F, ps->getAAT("position"));
			}
			pg_knn[frameIndex]->setTargetPoint(data_source_pik);
			{
				bool found;
				pg_knn[frameIndex]->getNextNearestPointIndex(footIndex_Frame, found);
				pAssert(found);
			}

			int32			varId_foot_pos	= ps->get<int32>(frameIndex, footIndex_Frame, POS_INDEX);
			int32			varId_foot_nor	= ps->get<int32>(frameIndex, footIndex_Frame, NOR_INDEX);
			/// The foot point can be moved
			if(varId_foot_pos >= 0 || varId_foot_nor >= 0)
			{
				Vector3f		frame_foot_pik		= ps->get3f(frameIndex, footIndex_Frame, POSITION);
				data_knn.setTargetPoint(frame_foot_pik);
				{
					bool found;
					data_knn.getNextNearestPointIndex(footIndex_Data, found);
					pAssert(found);
				}
				bool	good_correspondence = true;
				if(topologyOfTargetFrame != NULL)
				{
					good_correspondence = (*topologyOfTargetFrame)[pointIndex].hasEntry(footIndex_Data);
				}
				if(good_correspondence)
				{
					Vector3f		data_foot_pik	= pc->get3f(0, footIndex_Data, DATA_POS_AAT);
					Vector3f		frame_foot_nik	= ps->get3f(frameIndex, footIndex_Frame, NORMAL);
					Vector3f		frame_foot_Eik1= ps->get3f(frameIndex, footIndex_Frame, REF_U);
					Vector3f		dev_df_ik			= data_source_pik - frame_foot_pik;
					Vector3f		dev_fd_ik			= frame_foot_pik - data_foot_pik;

					buf_dev_df_ik.push_back(dev_df_ik);
					buf_dev_fd_ik.push_back(dev_fd_ik);
					buf_frame_foot_nik.push_back(frame_foot_nik);
					buf_frame_foot_Eik1.push_back(frame_foot_Eik1);
					buf_varId_foot_pos.push_back(varId_foot_pos);
					buf_varId_foot_nor.push_back(varId_foot_nor);
					buf_d2.insert(dev_df_ik.getSqrNorm());
				}	
			}
		}
		set<float32>::iterator		iter_float = buf_d2.begin();
		for(card32 pointIndex = 0; pointIndex < card32(buf_d2.size()*context->overlapping_rate); pointIndex++)
		{
			pointIndex++;		iter_float++;
		}
		mean_d2 = *iter_float;

		card32 count = 0;
		for(card32 pointIndex = 0; pointIndex < buf_d2.size(); pointIndex++)
		{
			Vector3f		dev_df_ik	= buf_dev_df_ik[pointIndex];
			Vector3f		dev_fd_ik	= buf_dev_fd_ik[pointIndex];

			float32	sqrDevdfik			= dev_df_ik.getSqrNorm(),
				sqrDevfdik				= dev_fd_ik.getSqrNorm();

			dev_df_ik.normalize();		dev_fd_ik.normalize();
			float32	angleik				= fabs(dev_df_ik*dev_fd_ik);

			// Check whether data_pik reside in the overlapping region
			if((angleik < 0.7 || sqrDevdfik > 4*sqrDevfdik) &&sqrDevdfik > 4*mean_d2)
			{
				pc->set<int32>(0, pointIndex, FLAG_INLIER, -1);
			}
			else
			{
				pc->set<int32>(0, pointIndex, FLAG_INLIER, 1);
			}
			if((angleik < 0.7 || sqrDevdfik > 4*sqrDevfdik) &&sqrDevdfik > 2*mean_d2)
			{
				continue;
			}
			
			float32	weight						= w_sdf;
			int32		varId_foot_pos		=	buf_varId_foot_pos[pointIndex];
			int32		varId_foot_nor		=	buf_varId_foot_nor[pointIndex];
			Vector3f	Eik2_foot				= buf_frame_foot_nik[pointIndex].crossProduct(buf_frame_foot_Eik1[pointIndex]);
			///In this case we only use point-plane distances because we assume that 
			///we have a very good initial position
			m_mul[0]		= 3;						m_indices[0]	= varId_foot_pos;	
			m_mul[1]		= 2;						m_indices[1] = varId_foot_nor;
			float			buf_v[5],  disik	= buf_dev_df_ik[pointIndex]*buf_frame_foot_nik[pointIndex];

			for(card32 k = 0; k < 3; k++)
			{
				buf_v[k] = - buf_frame_foot_nik[pointIndex][k];
			}
			buf_v[3]		= buf_dev_df_ik[pointIndex]*buf_frame_foot_Eik1[pointIndex];
			buf_v[4]		= buf_dev_df_ik[pointIndex]*Eik2_foot;
			for(card32 i = 0; i < 5; i++)
			{
				vec[i] = -disik*buf_v[i]*weight*(1.0 - tradeoff_sdf);
				for(card32 j = 0; j < 5; j++)
				{
					mat[i][j] = buf_v[i]*buf_v[j]*weight*(1.0 - tradeoff_sdf);
				}
			}
			for(card32 i = 0; i < 3; i++)
			{
				vec[i] += buf_dev_df_ik[pointIndex][i]*weight*tradeoff_sdf;
				mat[i][i] += weight*tradeoff_sdf;
			}
			UpdateMatrix_by_a_matrix(&mat, &m_indices, &m_mul, 2, quadApprox);
			UpdateVector_by_a_vector(&vec, &m_indices, &m_mul, 2, gradient);
		}
		//
/*		FILE *file = fopen("J:\\overlapping.txt", "a");
		fprintf(file, "%d %d\n", count, numOfDataPoints);
		fflush(file);
		fclose(file);*/
	}
	for (card32 frameNumInit=0; frameNumInit < patchGraph->getTrajectorySet()->getNumFrames(); frameNumInit++) {
		delete pg_knn[frameNumInit];
		delete pg_knn_ps[frameNumInit];
	}
	delete[] pg_knn;
	delete[] pg_knn_ps;

	return mean_d2;
}

void	PatchGraphBSROpt::gauss_Newton_Method(
	const BSROptVariableIndexSet *indices_pos, 
	const BSROptVariableIndexSet *indices_nor, 
	const BSROptVariableIndexSet *indices_rot, 
	const vector<card32> *indices_data,
	PointCloudAnimationSequence *data,
	PatchGraph	*patchGraph, 
	SparseMatrixF	*topologyOfTargetFrame,
	card32			typeOfOptimization, 
	FeedbackCallback afterEachIteration,
	void *callbackContext)
{
	DVectorF					direction,			// specify the direction for line search
		linApprox,				// the gradient direction
		diag;
	

	SparseMatrixF			quadApprox;		//	the quadratic term which is  matrix
	SparseMatrixF			precond;				// the preconditioning matrix used in conjugate gradient 

	card32	m_dimension	= (card32)(3*indices_pos->size() + 2*indices_nor->size() + 3*indices_rot->size());

	bool	descent_flag = true;
	float32	previous_median_d2 = 1e10;

	direction.setDim(m_dimension);				linApprox.setDim(m_dimension);		
	quadApprox.setRows(m_dimension);		precond.setRows(m_dimension);				
	diag.setDim(m_dimension);

	initialize_Sparse_Matrix(indices_pos, indices_nor, indices_rot, patchGraph, &quadApprox);

	BSROptVariableIndexSet	indices_related;
	compute_Related_Variable_Indices(&indices_related, patchGraph);



	for ( unsigned int step = 0; step < context->numIterations; step++ ) 
	{
		Timer timer;

		direction.setZero();			/// Direction for updating
		linApprox.setZero();			/// Gradient of the energy function
		diag.setZero();					/// The diagonal matrix for preconditioning
		precond.setZero();

		timer.getDeltaValue();

		compute_GNM_Gradient_and_Hessian(&indices_related, data, patchGraph, 
			&linApprox, &quadApprox);

		float32 tradeoff = 1.0 - 0.1*step/context->numIterations;
		///SDF Potential is only used for global_assembly and pair-wise matching
		if(potential_flags[0]&&typeOfOptimization == 3)
		{
			///Global_Assembly
			compute_GNM_Gradient_and_Hessian_SDF_Global(indices_data, data, patchGraph, 
				&linApprox, &quadApprox, context->tradeoff_sdf);
		}
		if(potential_flags[0]&&typeOfOptimization == 0)
		{
			/// Pair-wise matching
			if(step < context->numIterations/2)
			{
				float32 current_median_d2 = compute_GNM_Gradient_and_Hessian_SDF_Pairwise(indices_data, data, patchGraph, topologyOfTargetFrame,
					&linApprox, &quadApprox, previous_median_d2, context->tradeoff_sdf);

				previous_median_d2 = current_median_d2;
			}
			else
			{
				compute_GNM_Gradient_and_Hessian_SDF_Global(indices_data, data, patchGraph, 
					&linApprox, &quadApprox, context->tradeoff_sdf);
			}
		}
		for(card32 id = 0; id < m_dimension; id++)
		{
			diag[id] = 1.0/sqrt(quadApprox[id]*quadApprox[id]);
		}
		float32 t1 = timer.getDeltaValue()/1000.0;
		

		card32 numIterations = context->cgMaxNumIterations;
		conjugateGradientsSolve_Diag_Precond<float32>(quadApprox, direction, linApprox, diag,
			numIterations, context->cgStopRes, context->cgDebugging);

		
		//		preconditioned_ConjugateGradientsSolve<float32>(quadApprox, direction, linApprox, precond,
		//			context->cgStopRes, context->cgMaxNumIterations, context->cgDebugging);
		float32	t2 = timer.getDeltaValue()/1000.0;

		float32		cEnergy = 0.0f, nEnergy = 0.0f, stepsize = 1.0f;

		projectionOntoManifold(&indices_related, patchGraph, &direction);

		//Detect convergence
		float32	maxDeviation = -1e10f;
		for(card32 ni = 0; ni < indices_pos->size(); ni++)
		{
			if(maxDeviation < fabs(direction[ni]))
			{
				maxDeviation = fabs(direction[ni]);
			}
			if(maxDeviation < context->convergenceEps)
				break;
		}

		float32	t3 = timer.getDeltaValue()/1000.0;

		if (afterEachIteration) 
			afterEachIteration(callbackContext);
		if (context->verbose) 
		{
			debugOutput<<"Iteration "<<step<<"\n";

			debugOutput<<"Spent "<<t1<<"s on computing Hessian and Gradient.\n";
			debugOutput<<"Spent "<<t2<<"s on solving the linear system.\n";
			debugOutput<<"Spent "<<t3<<"s on line search and projection.\n";
			debugOutput<<"Energy at previous iteration: "<<cEnergy<<"\t"<<"Energy at current iteration: "<<nEnergy<<"\n";
			debugOutput<<"\n";
		}
		quadApprox.refresh();
	}
	for ( unsigned int potentialIndex = 0; potentialIndex < potentials.size(); potentialIndex++ )
	{
		potentials[potentialIndex]->finished();
	}
}
