//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "LinearTransformPriors.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "LinearAlgebra_templCode.h"
#include "DynamicLinearAlgebra_templCode.h"
#include "NumericalClassProperty.h"
#include "NumericalClassProperty_templCode.h"
#include "MathTools.h"
#include "MathTools_templCode.h"
//#include "VectorCanvas.h"
#include "Random.h"
#include "Timer.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------


const bool USE_WEIGHT = false;
/// unit test
const int NUM_POINTS = 100000;


// ---- GLTMonomials2ndOrder


void GLTMonomials2ndOrder::computeCoeffs(
   const vector<Vector3f> &points,
   StaticVector<float32, 6> &coeffs,
   vector< StaticVector<float32, 6> > &coeffsDerivs, float sigma)
{
   const size_t NUM_NEIGHBORS = points.size();
   coeffs[0] = 0.0f; coeffs[1] = 0.0f; coeffs[2] = 0.0f; coeffs[3] = 0.0f; coeffs[4] = 0.0f; coeffs[5] = 0.0f;
   coeffsDerivs.resize(NUM_NEIGHBORS);
   StaticVector<float32, 6> cd;
   StaticMatrix<float32, 6,6> dm;
   for (card32 i=0; i<6;i++) {
      for (card32 j=0; j<6;j++) {
         dm[i][j] = 0.0f;
      }
   }

	float kernelScale = 1.0f;
	float normalizeKernel = 1.0f;
	if( sigma > 0.0f )
	{
		kernelScale = 1.0f / pow( sigma,2.0f);
		float summed_weight = 0.0f;
		for( card32 i=0;i<NUM_NEIGHBORS;i++ )
		{
			const float32 u = points[i][0];
			const float32 v = points[i][1];
			summed_weight += exp( -(u*u+v*v) * kernelScale);
		}
		normalizeKernel = 1.0f / summed_weight; 
	}

   for (card32 pIndex=0; pIndex<NUM_NEIGHBORS; pIndex++){
      const float32 u = points[pIndex][0];
      const float32 v = points[pIndex][1];
      const float32 n = points[pIndex][2];
      //const float32 w = (USE_WEIGHT ? sqr(exp(-4*(u*u+v*v))) : 1) / (float32)NUM_NEIGHBORS;
		const float32 w = sigma > 0.0f ? exp( -(u*u+v*v) * kernelScale)*normalizeKernel : 1.0f / (float32)NUM_NEIGHBORS;

      // c[j] += b[j](uv(i)) * n(i) * w(uv(i));
      cd[0] =       w;
      cd[1] = u   * w;
      cd[2] = v   * w;
      cd[3] = u*v * w;
      cd[4] = u*u * w;
      cd[5] = v*v * w;
      coeffsDerivs[pIndex] = cd;

      coeffs[0] += cd[0] * n;
      coeffs[1] += cd[1] * n;
      coeffs[2] += cd[2] * n;
      coeffs[3] += cd[3] * n;
      coeffs[4] += cd[4] * n;
      coeffs[5] += cd[5] * n;

      // 1 u v uv uu vv

      dm[0][0] += w;     dm[0][1] += u*w;     dm[0][2] += v*w;     dm[0][3] += u*v*w;     dm[0][4] += u*u*w;     dm[0][5] += v*v*w;
      dm[1][0] += w*u;   dm[1][1] += u*w*u;   dm[1][2] += v*w*u;   dm[1][3] += u*v*w*u;   dm[1][4] += u*u*w*u;   dm[1][5] += v*v*w*u;
      dm[2][0] += w*v;   dm[2][1] += u*w*v;   dm[2][2] += v*w*v;   dm[2][3] += u*v*w*v;   dm[2][4] += u*u*w*v;   dm[2][5] += v*v*w*v;
      dm[3][0] += w*u*v; dm[3][1] += u*w*u*v; dm[3][2] += v*w*u*v; dm[3][3] += u*v*w*u*v; dm[3][4] += u*u*w*u*v; dm[3][5] += v*v*w*u*v;
      dm[4][0] += w*u*u; dm[4][1] += u*w*u*u; dm[4][2] += v*w*u*u; dm[4][3] += u*v*w*u*u; dm[4][4] += u*u*w*u*u; dm[4][5] += v*v*w*u*u;
      dm[5][0] += w*v*v; dm[5][1] += u*w*v*v; dm[5][2] += v*w*v*v; dm[5][3] += u*v*w*v*v; dm[5][4] += u*u*w*v*v; dm[5][5] += v*v*w*v*v;
   } /*
   for (card32 pIndex=0; pIndex<NUM_NEIGHBORS; pIndex++){
      const float32 u = points[pIndex][0];
      const float32 v = points[pIndex][1];
      const float32 n = points[pIndex][2];
      const float32 w = (USE_WEIGHT ? sqr(exp(-4*(u*u+v*v))) : 1) / (float32)NUM_NEIGHBORS;
      const float32 uv = u*v;
      const float32 uu = u*u;
      const float32 vv = v*v;
      const float32 wu = w*u;
      const float32 wv = w*v;
      const float32 wuv = w*uv;
      const float32 wuu = w*uu;
      const float32 wvv = w*vv;

      // c[j] += b[j](uv(i)) * n(i) * w(uv(i));
      cd[0] = w;
      cd[1] = wu;
      cd[2] = wv;
      cd[3] = wuv;
      cd[4] = wuu;
      cd[5] = wvv;
      coeffsDerivs[pIndex] = cd;

      coeffs[0] += cd[0] * n;
      coeffs[1] += cd[1] * n;
      coeffs[2] += cd[2] * n;
      coeffs[3] += cd[3] * n;
      coeffs[4] += cd[4] * n;
      coeffs[5] += cd[5] * n;

      // 1 u v uv uu vv

      dm[0][0] += w;        dm[0][1] += wu;       dm[0][2] += wv;       dm[0][3] += wuv;      dm[0][4] += wuu;      dm[0][5] += wvv;
      dm[1][0] += dm[0][1]; dm[1][1] += u*wu;     dm[1][2] += v*wu;     dm[1][3] += uv*wu;    dm[1][4] += uu*wu;    dm[1][5] += vv*wu;
      dm[2][0] += dm[0][2]; dm[2][1] += dm[1][2]; dm[2][2] += v*wv;     dm[2][3] += uv*wv;    dm[2][4] += uu*wv;    dm[2][5] += vv*wv;
      dm[3][0] += dm[0][3]; dm[3][1] += dm[1][3]; dm[3][2] += dm[2][3]; dm[3][3] += uv*wuv;   dm[3][4] += uu*wuv;   dm[3][5] += vv*wuv;
      dm[4][0] += dm[0][4]; dm[4][1] += dm[1][4]; dm[4][2] += dm[2][4]; dm[4][3] += dm[3][4]; dm[4][4] += uu*wuu;   dm[4][5] += vv*wuu;
      dm[5][0] += dm[0][5]; dm[5][1] += dm[1][5]; dm[5][2] += dm[2][5]; dm[5][3] += dm[3][5]; dm[5][4] += dm[4][5]; dm[5][5] += vv*wvv;
   }      */

   StaticMatrix<float32, 6,6> invDm;
   try {
      invDm = invertMatrix(dm);
   } catch (...) {
      for (card32 i=0; i<6; i++) {
         for (card32 j=0; j<6; j++) {
            invDm[i][j] = 0.0f;
         }
      }
   }
   coeffs = invDm * coeffs;
   for (card32 pIndex=0; pIndex<NUM_NEIGHBORS; pIndex++){
      coeffsDerivs[pIndex] = invDm*coeffsDerivs[pIndex];
   }

}

float32 GLTMonomials2ndOrder::evaluate(
   const StaticVector<float32, 6> &coeffs,
   const Vector2f &position)
{
   const float32 u = position[0];
   const float32 v = position[1];
   return   coeffs[0]
          + coeffs[1] * u
          + coeffs[2] * v
          + coeffs[3] * u*v
          + coeffs[4] * u*u
          + coeffs[5] * v*v;
}


// -- GTLMon2OSmoothnessScore


GLTMon2OSmoothnessScore::GLTMon2OSmoothnessScore(
   float32 onCurveWeight, float32 curvatureWeight,
   bool useCurvature, bool useMeanCurvature,
   float32 onCurveExponent, float32 curvatureExponent)
{
   this->onCurveWeight = onCurveWeight;
   this->curvatureWeight = curvatureWeight;
   this->useCurvature = useCurvature;
   this->useMeanCurvature = useMeanCurvature;
   this->onCurveExponent = onCurveExponent;
   this->curvatureExponent = curvatureExponent;
}

static inline float32 calc2ndOrderCurvature(const StaticVector<float32, 6> &coeffs, bool mean)
{
   // return  ( 2*(1+c01^2)*c20 - 2*c10*c01*c11 + 2*(1+c10^2)*c02 )
   //  //     ------------------------------------------------------
   //       /                 2*(1+c10^2+c01^2)^3/2

   // c0  c1  c2  c3  c4  c5
   //  1   u   v  uv  uu  vv
   //c00 c10 c01 c11 c20 c02

   float32 c10 = coeffs[1];
   float32 c01 = coeffs[2];
   float32 c11 = coeffs[3];
   float32 c20 = coeffs[4];
   float32 c02 = coeffs[5];

   if (mean) {
      return  (  (1+sqr(c01))*c20 - 2*c10*c01*c11 +   (1+sqr(c10))*c02 )
      //      -----------------------------------------------------------
            /         2.0f*pow( 1 + sqr(c10)+sqr(c01), 3.0f/2.0f);
   } else {

      return  ( c20*c02-c11*c11 )
      //      -------------------
            / (1+c10*c10+c01*c01);
   }
}


static inline float32 calc2ndOrderCurvatureDerivative(
   const StaticVector<float32, 6> &coeffs,
   const StaticVector<float32, 6> &coeffDerivatives, bool mean)
{
//   return (   calc2ndOrderMeanCurvature(coeffs + coeffDerivatives*0.01)
//            - calc2ndOrderMeanCurvature(coeffs - coeffDerivatives*0.01) ) * (coeffDerivatives*0.01) / 0.02;
   StaticVector<float32, 6> dOuter;
   for (card32 i=0; i<6; i++) {
      StaticVector<float32, 6> cdP, cdM;
      for (card32 j=0; j<6; j++) {
         cdP[j] = coeffs[i];
         cdM[j] = coeffs[i];
      }
      cdP[i] += coeffDerivatives[i]*0.01f;
      cdM[i] -= coeffDerivatives[i]*0.01f;
      dOuter[i] = ( calc2ndOrderCurvature(cdP, mean) - calc2ndOrderCurvature(cdM, mean) ) / 0.02f;
   }
   return dOuter * coeffDerivatives;
}

static inline float32 calc2ndOrderDerivNorm(const StaticVector<float32, 6> &c) {
   // c0  c1  c2  c3  c4  c5
   //  1   u   v  uv  uu  vv
   return c[3]*c[3] + 2.0f*c[4]*c[4] + 2.0f*c[5]*c[5];
}

static inline float32 calc2ndOrderDerivNormDerivative(const StaticVector<float32, 6> &c, const StaticVector<float32, 6> &cd) {
   // c0  c1  c2  c3  c4  c5
   //  1   u   v  uv  uu  vv
   return 2.0f*c[3]*cd[3] + 4.0f*c[4]*cd[4] + 4.0f*c[5]*cd[5];
}

void GLTMon2OSmoothnessScore::evaluate(
   const vector<Vector3f> &points,
   const StaticVector<float32, 6> &coeffs,
   const vector< StaticVector<float32, 6> > &coeffsDerivs,
   float32 &score,
   DynamicVector<float32> &normalDirDerivatives) const
{
   const card32 N = (card32)points.size();

   score = 0;
   normalDirDerivatives.setDim(N);
   for (card32 i=0; i<N; i++) normalDirDerivatives[i] = 0.0f;

   // const float32 NORMALIZE = 1.0f / (float32)N;

   if (onCurveWeight != 0.0f) {
      for (card32 i=0; i<N; i++) {
         const float32 u = points[i][0];
         const float32 v = points[i][1];
         const float32 n = points[i][2];
         const float32 w = USE_WEIGHT ? exp(-4*(u*u+v*v)) : 1;

         const Vector2f p2d      = makeVector2f(u, v);
         const float32  eval     = GLTMonomials2ndOrder::evaluate(coeffs,          p2d);
         const float32  evalDiff = GLTMonomials2ndOrder::evaluate(coeffsDerivs[i], p2d);

         if (onCurveExponent == 1.0f) {
            score += sqr((n - eval) * w) * onCurveWeight;
            //normalDirDerivatives[i] = onCurveWeight * ( 1.0f - 2.0f*eval - 2.0f*n*evalDiff + 2.0f*eval*evalDiff ) * w*w;
            normalDirDerivatives[i] = 2*(n - eval) * (1 - evalDiff) * w*w * onCurveWeight;
         } else {
            score += pow(fabs(sqr((n - eval) * w) * onCurveWeight), onCurveExponent);
            normalDirDerivatives[i] =
               onCurveExponent * pow(fabs(sqr((n - eval) * w) * onCurveWeight), onCurveExponent-1)
                               * ( 2*(n - eval) * (1 - evalDiff) * w*w * onCurveWeight );
         }
      }
   }

   if (curvatureWeight != 0.0f) {
      if (useCurvature) {
         score += curvatureWeight * calc2ndOrderCurvature(coeffs, useMeanCurvature);
         for (card32 i=0; i<N; i++) {
            normalDirDerivatives[i] += curvatureWeight * calc2ndOrderCurvatureDerivative(coeffs, coeffsDerivs[i], useMeanCurvature);
         }
      } else {
         if (curvatureExponent == 1.0f) {
            score += curvatureWeight * calc2ndOrderDerivNorm(coeffs);
            for (card32 i=0; i<N; i++) {
               normalDirDerivatives[i] += curvatureWeight * calc2ndOrderDerivNormDerivative(coeffs, coeffsDerivs[i]);
            }
         } else {
            score += pow(fabs(curvatureWeight * calc2ndOrderDerivNorm(coeffs)), curvatureExponent);
            for (card32 i=0; i<N; i++) {
               normalDirDerivatives[i] +=
                  curvatureExponent * pow(fabs(curvatureWeight * calc2ndOrderDerivNorm(coeffs)), curvatureExponent-1)
                                    * curvatureWeight * calc2ndOrderDerivNormDerivative(coeffs, coeffsDerivs[i]);
            }
         }
      }
   }
}


void geoxl::evaluateSmoothnessPrior(
   const vector< Vector3f > &data,
   float32 &score,
   vector< Vector3f > &gradient,
   card32 technique,
   float32 onCurveWeight,
   float32 curvatureWeight,
   float32 onCurveExponent,
   float32 curvatureExponent)
{
   bool useCurv = (technique > SMOOTHNESS_CURV_2ND_ORDER_NORM);
   bool useMean = useCurv && (technique == SMOOTHNESS_CURV_MEAN);
   GLTMon2OSmoothnessScore smoother(onCurveWeight, curvatureWeight, useCurv, useMean, onCurveExponent, curvatureExponent);

   StaticVector<float32, 6> coeffs;
   vector< StaticVector<float32, 6> > coeffsDerivs;
   GLTMonomials2ndOrder::computeCoeffs(data, coeffs, coeffsDerivs);

   DVectorF normalDirDerivatives;
   smoother.evaluate(data, coeffs, coeffsDerivs, score, normalDirDerivatives);
   const mpcard n = normalDirDerivatives.size();
   gradient.resize(n);
   for (mpcard i=0; i<n; i++) {
      gradient[i][0] = 0;
      gradient[i][1] = 0;
      gradient[i][2] = -normalDirDerivatives[i];
   }
}



// ----



static inline Vector2f proj(const Vector3f &pos) {
   return makeVector2f(pos[0]+pos[1]*0.7f, pos[2]+pos[1]*0.7f);
}

const float32 F_RANGE = 1;
const float32 GRAD_SCALE = 0.1f;

/*#pragma option -w-8080
void MAPRECONSTRUCTION_API geoxl::linearTransformPriors_unitTest() {
	message("geoxl::linearTransformPriors_unitTest()\n\n");
   TForm *win = new TForm(Application);
   win->Top = 200;
   win->Left = 500;
   win->Width = 500;
   win->Height = 500;
   win->Caption = "linearTransformPriors_unitTest";
   TVectorCanvas2d *frame = new TVectorCanvas2d(win);
   frame->Parent = win;
   frame->Align = alClient;
   win->Show();

   vector<Vector3f> points(NUM_POINTS);
   for (card32 i=0; i<NUM_POINTS; i++) {
      float32 u = (2.0f*rnd01()-1.0f)*F_RANGE;
      float32 v = (2.0f*rnd01()-1.0f)*F_RANGE;
      //float32 n = 1+rnd01();
      float32 n = u*u+v*v + rnd01();
      //float32 n = 0.25*(-2 + u + v +u*v + u*u + v*v + u*u*v + u*v*v + u*u*v*v) + rnd01();
      points[i] = makeVector3f(u,v,n);
   }

   win->BringToFront();

   GLTMonomials2ndOrder bfs;
   GLTMon2OSmoothnessScore score(1,100,false);

   const card32 NUM_FRAMES = 2000;

   for (card32 anim=0; anim<NUM_FRAMES; anim++) {

      Timer t;
      StaticVector<float32, 6> coeffs;
      vector< StaticVector<float32, 6> > coeffsDerivs;
      bfs.computeCoeffs(points, coeffs, coeffsDerivs);
      card32 delta = t.getDeltaValue();
      if (anim < 5) debugOutput << "coeffs and coeffsDerivs time [ms]: " << delta << "\n";

      DVectorF normalDirDerivatives;
      float32 curvatureScore;
      t.getDeltaValue();
      score.evaluate(points, coeffs, coeffsDerivs, curvatureScore, normalDirDerivatives);
      delta = t.getDeltaValue();
      if (anim < 5) debugOutput << "scoring time [ms]: " << delta << "\n";

      frame->clear();

      for (card32 i=0; i<NUM_POINTS; i++) {
         VCPoint *p = new VCPoint;
         p->pos = proj(points[i]);
         p->size = 1;
         int r = (points[i][0]+1)*127;
         int b = (points[i][0]+1)*127;
         p->color = TColor(RGB(r,0,b));
         frame->addElement(p);

         VCLine *l = new VCLine;
         l->start = proj(points[i]);
         l->end = proj(points[i] + makeVector3f(0,0, normalDirDerivatives[i])*GRAD_SCALE);
         l->color = clRed;
         frame->addElement(l);
      }

      Vector3f displ[21][21];

      for (card32 v=0; v<21; v++) {
         for (card32 u=0; u<21; u++) {
            float32 uf = (u/10.0f-1.0f)*F_RANGE;
            float32 vf = (v/10.0f-1.0f)*F_RANGE;
            float32 nf = bfs.evaluate(coeffs, makeVector2f(uf, vf));
            displ[v][u] = makeVector3f(uf, vf, nf);
         }
      }

      for (card32 v=0; v<20; v++) {
         for (card32 u=0; u<21; u++) {
            VCLine *l = new VCLine;
            l->start = proj(displ[v][u]);
            l->end = proj(displ[v+1][u]);
            l->color = clBlack;
            frame->addElement(l);
         }
      }
      for (card32 v=0; v<21; v++) {
         for (card32 u=0; u<20; u++) {
            VCLine *l = new VCLine;
            l->start = proj(displ[v][u]);
            l->end = proj(displ[v][u+1]);
            l->color = clBlack;
            frame->addElement(l);
         }
      }

      if (anim==0) {
         frame->autoSize();
         Application->MessageBox("first fit", "unit test", MB_OK);
      } else {
         frame->repaint();
      }

      float32 step = 0.01;
      if (anim > 10) step = 0.03;
      for (card32 i=0; i<NUM_POINTS; i++) {
         points[i][2] -= normalDirDerivatives[i] * step;
      }
   }
	
}*/

