//---------------------------------------------------------------------------
#ifndef LinearTransformPriorsH
#define LinearTransformPriorsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "DynamicLinearAlgebra.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;


class ANIMATIONRECONSTRUCTION_API GLTMonomials2ndOrder {
 private:
   static MetaClass *GLTOrthoPoly2ndOrderMetaClass;
   static Object* newInstance();

 public:
   static const card32 NUM_BASIS_FUNS = 6;

   static void computeCoeffs(const vector<Vector3f> &points,
                             StaticVector<float32, 6> &coeffs,
                             vector< StaticVector<float32, 6> > &coeffsDerivs,
									  float sigma = -1.0f);
   static float32 evaluate(const StaticVector<float32, 6> &coeffs,
                            const Vector2f &position);
};


class GLTMon2OSmoothnessScore {
 private:
   /// weighting of attraction to 2nd order least-squares curve  (set to zero to deactivate computation)
   float32 onCurveWeight;
   /// weighting of small curvature assumption                   (set to zero to deactivate computation)
   float32 curvatureWeight;
   /// use curvature measures instea of 2nd order derivative norm (not recommended)
   bool useCurvature;
   /// use mean cuvature or Gauss curvature?
   bool useMeanCurvature;
   /// exponent within the Gaussian distribution, 1 refers to a quadratic model
   float32 onCurveExponent;
   /// exponent within the Gaussian distribution, 1 refers to a quadratic model
   float32 curvatureExponent;


 public:
   GLTMon2OSmoothnessScore(float32 onCurveWeight = 1.0f, float32 curvatureWeight = 100.0f,
                           bool useCurvature = false, bool useMeanCurvature = false,
                           float32 onCurveExponent = 1.0f, float32 curvatureExponent = 1.0f);

   void evaluate(const vector<Vector3f> &points,
                 const StaticVector<float32, 6> &coeffs,
                 const vector< StaticVector<float32, 6> > &coeffsDerivs,
                 float32 &score,
                 DynamicVector<float32> &normalDirDerivatives) const;
};


const card32 SMOOTHNESS_CURV_2ND_ORDER_NORM = 0;
const card32 SMOOTHNESS_CURV_MEAN = 1;
const card32 SMOOTHNESS_CURV_GAUSS = 2;

void ANIMATIONRECONSTRUCTION_API evaluateSmoothnessPrior(
   const vector< Vector3f > &data,
   float32 &score,
   vector< Vector3f > &gradient,
   card32 technique = SMOOTHNESS_CURV_2ND_ORDER_NORM,
   float32 onCurveWeight = 1.0f,
   float32 curvatureWeight = 100.0f,
   float32 onCurveExponent = 1.0f,
   float32 curvatureExponent = 1.0f
);





extern void ANIMATIONRECONSTRUCTION_API linearTransformPriors_unitTest();


}

#endif
