//---------------------------------------------------------------------------
#ifndef MultiResolutionPGOptH
#define MultiResolutionPGOptH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "PatchGraphBSROpt.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------

namespace geoxl {
	using namespace std;

class PatchGraphTopologicalResampling;
class PointCloudAnimationSequence;

/**
	Multi-Grid solver for PatchGraphBSROpt
*/
class ANIMATIONRECONSTRUCTION_API MultiResolutionPGOpt : public Persistent {
	GEOXL_CLASS( MultiResolutionPGOpt )

 private:
	PatchGraphTopologicalResampling *resampler;
 
	PatchGraph *lowres;
	PatchGraph *pg;
	PointCloudAnimationSequence *data;
	vector< vector<card32> > clusters;
	vector<int32> inverseClusterMapping;
	vector< vector< pair<card32, float32> > > interpolationWeights;
	bool debugVis;
	PatchGraphBSROpt::FeedbackCallback afterEachIteration;
	void *afterEachIterationContext;
	const BSROptVariableIndexSet *indices;

	static void afterEachIterationCallback(void *context);
	void extrapolateHighres(bool directWrite);
																					
	void calcLowresIndices(const BSROptVariableIndexSet *indices, BSROptVariableIndexSet &lowresIndices);

 public:	
	MultiResolutionPGOpt();

	void setupAndBuildLowresTopology(
      PointCloudAnimationSequence *data,
      PatchGraph *patchGraph,
      float32 lowresDist,
      vector<bool> *lowResRemovablePatches = NULL,
      bool debugVis = false
	);
	
	void twoResolutionOpt(
		PatchGraphBSROpt *optimizer, 
      const BSROptVariableIndexSet *indices,
      PatchGraphBSROpt::FeedbackCallback afterEachIteration = NULL,
      void *callbackContext = NULL
   );

	//void longEdgesOpt(
	//	PatchGraphBSROpt *optimizer, 
 //     const BSROptVariableIndexSet *indices,
 //     PatchGraphBSROpt::FeedbackCallback afterEachIteration = NULL,
 //     void *callbackContext = NULL
 //  );
   
	~MultiResolutionPGOpt();
};

}

#endif