//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PatchGraphBSROpt.h"
//---------------------------------------------------------------------------

#include "BSRFitting.h"
#include "BSROptContext.h"
#include "BSROptPotential.h"
#include "DynamicLinearAlgebra_templCode.h"
#include "InCorePCTopologyGraph.h"
#include "IterativeSolvers.h"
#include "IterativeSolvers_templCode.h"
//#include "LineSearchConjGradDescent.h"
#include "MgcEigen.h"
#include "Monomials2ndOrder.h"
#include "PatchGraph.h"
#include "PatchTypes.h"
#include "PGTopologicalNeighborhood.h"
#include "PointCloudAnimationSequence.h"
#include "PGDataPointIndices.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "PointSetKNNQuery.h"
#include "SeparatorClassProperty.h"
#include "SparseLinearAlgebra.h"
#include "SparseLinearAlgebra_templCode.h"
#include "Timer.h"
#include "UnifyNormals.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------

/************************************************************************/
/*PETER: The following codes are just for testing                                                                     */
/************************************************************************/

void		Quaternion_To_Matrix3f_NCGNRFR(const float	*q, Matrix3f &rotation)
{
	pAssert(q != NULL);
	rotation[0][0] = 2*q[1]*q[1] + (2*q[0]*q[0]-1);
	rotation[1][0] = 2*q[1]*q[2] - 2*q[0]*q[3];
	rotation[2][0] = 2*q[1]*q[3] + 2*q[0]*q[2];
	rotation[0][1] = 2*q[2]*q[1] + 2*q[0]*q[3];
	rotation[1][1] = 2*q[2]*q[2] + (2*q[0]*q[0]-1);
	rotation[2][2] = 2*q[2]*q[3] - 2*q[0]*q[1];
	rotation[0][2] = 2*q[3]*q[1] - 2*q[0]*q[2];
	rotation[1][2] = 2*q[3]*q[2] + 2*q[0]*q[1];
	rotation[2][2] = 2*q[3]*q[3] + (2*q[0]*q[0]-1);
}

void	PatchGraphBSROpt::nonlinear_Conjugate_Gradients_NRFR(
	const	BSROptVariableIndexSet				*indices_related,
	PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const card32 m_dimension, 
	FeedbackCallback afterEachIteration, 
	void *callbackContext)
{
	DVectorF		linApprox,			/// the first-order approximation: f'(x);
		direction,			/// direction for line search
		displacement;	/// 

	linApprox.setDim(m_dimension);				direction.setDim(m_dimension);			displacement.setDim(m_dimension);

	float32			term_1st,			/// first order term in the Taylor series
		term_2st;			/// second order term in the Taylor series

	//variables 
	card32			i = 0, k = 0;
	compute_NCG_NRFR_Gradient(data, patchGraph, &linApprox);
	direction			= -linApprox;
	float32			delta_new	=	linApprox*linApprox, delta_old;
	float32			delta_zero	=	delta_new;

	Timer				timer;
	while (i < context->maxIterations_CG_NCG_NRFR 
		&& delta_new > delta_zero*context->errorTolerance_CG_NCG_NRFR*context->errorTolerance_CG_NCG_NRFR)
	{
		timer.getDeltaValue();
		if(context->verbose)
		{
			debugOutput<<"Outer Loop Iteration "<<i<<"\n";
		}

		card32	j =0;
		float32	delta_d = direction*direction;
		float32	alpha;
		do 
		{
			compute_NCG_NRFR_1st_and_2st_Terms(data, patchGraph, term_1st, term_2st);
			alpha = - term_1st/term_2st;
			displacement = direction*alpha;
			projectionOntoManifold(indices_related, patchGraph, &displacement);
			j = j+1;
		} while(j < context->maxIterations_NR_NCG_NRFR
			&& delta_d*alpha*alpha > context->errorTolerance_NR_NCG_NRFR*context->errorTolerance_NR_NCG_NRFR);
		timer.getDeltaValue();

		if(context->verbose)
		{
			debugOutput<<"Line search is completed in "<<j<<"  iterations with alpha = "<<alpha<<".\n";
		}

		compute_NCG_NRFR_Gradient(data, patchGraph, &linApprox);
		delta_old	= delta_new;
		delta_new	= linApprox*linApprox;
		float32	beta = delta_new/delta_old;
		direction = direction*beta - linApprox;
		k = k+1;
		if(k == m_dimension || linApprox*direction > 0)
		{
			direction = - linApprox;
			k = 0;
		}
		if(context->verbose)
		{
			debugOutput<<"One iteration of NCG is completed with delta_old = "<<delta_old<<" and delta_new = "<<delta_new<<".\n";
			debugOutput<<"\n";
		}
		i = i + 1;
	}
	for (unsigned int potentialIndex = 0; potentialIndex < potentials.size(); potentialIndex++ )
	{
		potentials[potentialIndex]->finished();
	}
}

void	PatchGraphBSROpt::compute_NCG_NRFR_1st_and_2st_Terms(
	const PointCloudAnimationSequence	*data, 
	PatchGraph *patchGraph, 
	float32 &term_1st, 
	float32 &term_2st)
{

}

void	PatchGraphBSROpt::compute_NCG_NRFR_Gradient(
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	DVectorF *gradient)
{

}


/*
void	PatchGraphBSROpt::UpdateFor_NCG_Method(
	const BSROptContext *context, 
	const PointCloudAnimationSequence *data, 
	PatchGraph *patchGraph, 
	const DVectorF *direction,
	DVectorF *gradient, 
	float32 &dt_Hessian_d, 
	bool m_flag)
{
	bool		flag_SDFPotential		= potential_flags[0],
		flag_MarPotential		= potential_flags[1],
		flag_PlaSPotential		=	potential_flags[2],
		flag_NorSPotential	= potential_flags[3],
		flag_CirSPotential		= potential_flags[4],
		flag_LapSPotential	= potential_flags[5],
		flag_PosRPotential	= potential_flags[6],
		flag_NorRPotential	= potential_flags[7];

	float32	w_sdf			= context->peter_weightSquareddistance,
		w_marker	=	context->peter_weightMarkerError,
		w_pla_s		= context->peter_weightPlanarsmoothness,
		w_nor_s		= context->peter_weightCircularsmoothness,
		w_cir_s		= context->peter_weightCircularsmoothness,
		w_lap_s		= context->peter_weightLaplaciansmoothness,
		w_pos_r		= context->peter_weightPositionrigidity,
		w_nor_r		= context->peter_weightNormalrigidity;


	PointSet								*ps					= patchGraph->getTrajectorySet()->getTrajectories();
	PointSet								*urshape		= patchGraph->getUrshape();
	InCorePCTopologyGraph *tg					= patchGraph->getTopology();
	PGDataPointIndices			*dataPoints	= patchGraph->getTrajectorySet()->getDataPoints();

	card32								width				=	ps->getWidth(),
		height				= ps->getHeight();

	AAT										POSITION		= ps->getAAT("position");
	AAT										NORMAL			= ps->getAAT("normal");
	AAT										REF_U				= ps->getAAT("tangent_u");
	AAT										ROTATION		= ps->getAAT("rotation_matrix");
	AAT										POS_INDEX	= ps->getAAT("varIndex_pos");
	AAT										NOR_INDEX	= ps->getAAT("varIndex_nor");
	AAT										ROT_INDEX	= ps->getAAT("varIndex_rot");
	AAT										DATA_POS_AAT	= data->getFrame(0)->getAAT( "position", 3, VAD::DATA_FORMAT_FLOAT32 );

	/// Current term rotation matrix
	Matrix3f								Aik;

	card32								offset = 0, numOfEdges = 0, k, j;
	for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		numOfEdges += tg->getNumAdjacentVertices(surfelIndex);
	}
	for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
	{
		card32		numOfEntries = tg->getNumAdjacentVertices(surfelIndex);
		for(card32 frameIndex = 0; frameIndex < width-1; frameIndex++)
		{
			Vector3f	pik	=	ps->get3f(frameIndex, surfelIndex,	POSITION);
			Vector3f	nik	= ps->get3f(frameIndex, surfelIndex,  NORMAL); 
			Vector3f	pikn	=	ps->get3f(frameIndex+1, surfelIndex,	POSITION);
			Vector3f	nikn	= ps->get3f(frameIndex+1, surfelIndex,  NORMAL); 
			Vector3f	Eik1 = ps->get3f(frameIndex, surfelIndex, REF_U);
			Vector3f	Eik2 = nik.crossProduct(Eik1);
			Vector3f	Eikn1 = ps->get3f(frameIndex+1, surfelIndex, REF_U);
			Vector3f	Eikn2 = nik.crossProduct(Eikn1);

			int	ik_posid	= ps->get<int32>(frameIndex, surfelIndex, POS_INDEX);
			int	ik_norid	= ps->get<int32>(frameIndex, surfelIndex, NOR_INDEX);
			int	ikn_posid= ps->get<int32>(frameIndex+1, surfelIndex, POS_INDEX);
			int	ikn_norid	= ps->get<int32>(frameIndex+1, surfelIndex, NOR_INDEX);
			int	ik_rotid	= ps->get<int32>(frameIndex, surfelIndex, ROT_INDEX);

			//get current rotation matrix
			char			*dataC	= ps->getDataPointer(frameIndex, surfelIndex);
			float32	*quatc	= (float*)(dataC+ROTATION.getOffset());
			Quaternion_To_Matrix3f(quatc, Aik);

			if(flag_MarPotential)
			{
				card32 numDataPoints = dataPoints->getNumDataPoints(frameIndex, surfelIndex);
				if (numDataPoints > 0) 
				{
					card32 dataIndexInData = dataPoints->getIndex(frameIndex, surfelIndex, 0);
					Vector3f devik = pik - data->getFrame(frameIndex)->get3f(dataIndexInData, DATA_POS_AAT);
					for(k = 0; k < 3; k++)
					{
						gradient[ik_posid + k] -= w_marker*devik[k];
						dt_Hessian_d += w_marker*(*direction)[ik_posid+k]*(*direction)[ik_posid+k];
					}
				}
				if(frameIndex == width -2)
				{
					numDataPoints = dataPoints->getNumDataPoints(frameIndex+1, surfelIndex);
					if (numDataPoints > 0) 
					{
						card32	dataIndexInData = dataPoints->getIndex(frameIndex+1, surfelIndex, 0);
						Vector3f devikn = pikn - data->getFrame(frameIndex+1)->get3f(dataIndexInData, DATA_POS_AAT);
						for(k = 0; k < 3; k++)
						{
							gradient[ikn_posid + k] -= w_marker*devikn[k];
							dt_Hessian_d += w_marker*(*direction)[ikn_posid+k]*(*direction)[ikn_posid+k];
						}
					}
				}
			}
			Vector3f	lap_ijk;
			Vector3f	center_ik;
			if(flag_LapSPotential)
			{
				center_ik = pik;
				for (card32 ni = 0; ni< numOfEntries; ni++) 
				{
					card32		neighbor_id =		tg->getAdjacentVertex(surfelIndex, ni);
					Vector3f		pjk		=	ps->get3f(frameIndex,		neighbor_id, POSITION);
					center_ik	-=	pjk/numOfEntries;	
				}
				for(k = 0; k < 3; k++)
				{
					lap_ijk[k] = (*direction)[ik_posid+k];
					(*gradient)[ik_posid+k] -= center_ik[k];
				}
			}
			for (card32 ni = 0; ni< numOfEntries; ni++) 
			{
				card32		neighbor_id =		tg->getAdjacentVertex(surfelIndex, ni);
				Vector3f		pjk		=	ps->get3f(frameIndex,		neighbor_id, POSITION);
				Vector3f		njk		= ps->get3f(frameIndex,			neighbor_id,  NORMAL);
				Vector3f		pjkn		=	ps->get3f(frameIndex+1,	neighbor_id, POSITION);
				Vector3f		njkn		= ps->get3f(frameIndex+1,	neighbor_id,  NORMAL);
				Vector3f		Ejk1		= ps->get3f(frameIndex,			neighbor_id, REF_U);
				Vector3f		Ejk2		= njk.crossProduct(Ejk1);
				Vector3f		Ejkn1	= ps->get3f(frameIndex+1,	neighbor_id, REF_U);
				Vector3f		Ejkn2	= njk.crossProduct(Ejkn1);
				Vector3f		rijk		= pik	- pjk;
				Vector3f		rijkn		=	pikn	- pjkn;
				Vector3f		nijk		=	nik	- njk;
				Vector3f		nijkn	= nikn	- njkn;
				Vector3f		rijkc		= Aik*rijk;
				Vector3f		nijkc	=	Aik*nijk;
				float32		rweightijk	= edge_weight_rigidity[offset + ni + frameIndex*numOfEdges],
					sweightijk	= edge_weight_smoothness[offset + ni + frameIndex*numOfEdges],
					weightijk		= exp(context->peter_lambda*rijk.getSqrNorm());

				int		jk_posid		= ps->get<int32>(frameIndex, neighbor_id, POS_INDEX);
				int		jk_norid		= ps->get<int32>(frameIndex, neighbor_id, NOR_INDEX);
				int		jkn_posid	= ps->get<int32>(frameIndex+1, neighbor_id, POS_INDEX);
				int		jkn_norid		= ps->get<int32>(frameIndex+1, neighbor_id, NOR_INDEX);

				float32		weight;
				if(flag_NorRPotential)
				{
					weight = w_nor_r*rweightijk/numOfEntries;
					Vector3f		dev_nijk = nijkc - nijkn;
					Vector3f		AikEik1 = Aik*Eik1;
					Vector3f		AikEik2 = Aik*Eik2;
					Vector3f		AikEjk1 = Aik*Ejk1;
					Vector3f		AikEjk2 = Aik*Ejk2;
					(*gradient)[ik_norid]		-= (dev_nijk*AikEik1)*weight;
					(*gradient)[ik_norid+1]	-= (dev_nijk*AikEik2)*weight;
					(*gradient)[jk_norid]		+= (dev_nijk*AikEjk1)*weight;
					(*gradient)[jk_norid+1]	+= (dev_nijk*AikEjk2)*weight;
					(*gradient)[ik_norid]		+= (dev_nijk*Eikn1)*weight;
					(*gradient)[ik_norid+1]	+= (dev_nijk*Eikn2)*weight;
					(*gradient)[jk_norid]		-= (dev_nijk*Ejkn1)*weight;
					(*gradient)[jk_norid+1]	-= (dev_nijk*Ejkn2)*weight;
					(*gradient)[ik_rotid]		-= weight*(nijkc[1]*dev_nijk[2] - nijkc[2]*dev_nijk[1]);
					(*gradient)[ik_rotid+1]	-= weight*(nijkc[2]*dev_nijk[0] - nijkc[0]*dev_nijk[2]);
					(*gradient)[ik_rotid+2]	-= weight*(nijkc[0]*dev_nijk[1] - nijkc[1]*dev_nijk[0]);
					Vector3f	vtp;
					vtp[0] = -weight*(nijkc[1]*(*direction)[ik_rotid+2] - nijkc[2]*(*direction)[ik_rotid+1]);
					vtp[1] = -weight*(nijkc[2]*(*direction)[ik_rotid+0] - nijkc[0]*(*direction)[ik_rotid+2]);
					vtp[2] = -weight*(nijkc[0]*(*direction)[ik_rotid+1] - nijkc[1]*(*direction)[ik_rotid+0]);
					vtp += AikEik1*(*direction)[ik_norid] + AikEik2*(*direction)[ik_norid+1] 
					- AikEjk1*(*direction)[jk_norid] - AikEjk2*(*direction)[jk_norid+1]
					+ Eikn1*(*direction)[ikn_norid] + Eikn2*(*direction)[ikn_norid+1]
					- Ejkn1*(*direction)[jkn_norid] - Ejkn2*(*direction)[jkn_norid+1];
					dt_Hessian_d	+= weight*vtp.getSqrNorm();
				}
				if(flag_PosRPotential)
				{
					weight		= w_pos_r*rweightijk/numOfEntries;
					Vector3f	dev_pijk = rijkc - rijkn;
					float32	tp;	
					Vector3f	vtp;
					vtp[0]		= -weight*(rijkc[1]*(*direction)[ik_rotid+2] - rijkc[2]*(*direction)[ik_rotid+1]);
					vtp[1]		= -weight*(rijkc[2]*(*direction)[ik_rotid+0] - rijkc[0]*(*direction)[ik_rotid+2]);
					vtp[2]		= -weight*(rijkc[0]*(*direction)[ik_rotid+1] - rijkc[1]*(*direction)[ik_rotid+0]);
					for(k = 0; k < 3; k++)
					{
						tp = weight*(Aik[k][0]*dev_pijk[0] + Aik[k][1]*dev_pijk[1] + Aik[k][2]*dev_pijk[2]);
						(*gradient)[ik_posid + k]	-= tp;
						(*gradient)[jk_posid + k]	+= tp;
						tp = weight*dev_pijk[k];
						(*gradient)[ikn_posid+k]		+= tp;
						(*gradient)[jkn_posid+k]		-= tp;
						for(j = 0; j < 3; j++)
						{
							vtp[j] += Aik[k][j]*((*direction)[ik_posid+k]-(*direction)[jk_posid+k]);
						}
						vtp[k] -= (*direction)[ikn_posid+k]-(*direction)[jkn_posid+k];
					}
					gradient[ik_rotid]		+= weight*(dev_pijk[1]*rijkc[2] - dev_pijk[2]*rijkc[1]);
					gradient[ik_rotid+1] += weight*(dev_pijk[2]*rijkc[0] - dev_pijk[0]*rijkc[2]);
					gradient[ik_rotid+2] += weight*(dev_pijk[0]*rijkc[1] - dev_pijk[1]*rijkc[0]);
					dt_Hessian_d	+= weight*vtp.getSqrNorm();
				}
				if(flag_LapSPotential)
				{
					for(k = 0; k < 3; k++)
					{
						lap_ijk[k] -= (*direction)[jk_posid+k]/numOfEntries;
						(*gradient)[ik_posid+k] += center_ik[k]/numOfEntries;
					}
				}
				if(flag_CirSPotential)
				{
					weight = w_cir_s*sweightijk*weightijk/numOfEntries;
					float32	dis = (nik + njk)*rijk;
					///					gradient[ik]
				}
				if(flag_NorSPotential)
				{
					weight = w_nor_r*sweightijk*weightijk/numOfEntries;
				}
				if(flag_PlaSPotential)
				{
					weight = w_pla_s*sweightijk*weightijk/numOfEntries;
					float32	dis = rijk*nik;
				}
			}
			dt_Hessian_d += w_lap_s*lap_ijk.getSqrNorm();
		}
		offset += numOfEntries;
	}
}*/