//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PGTopologicalNeighborhood.h"
#include "PatchGraph.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "InCorePCTopologyGraph.h"
#include "PGDataPointIndices.h"
#include "TGNeighborhoodKNNSearch.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------


IMPLEMENT_GEOXL_ABSTRACT_CLASS( PGTopologicalNeighborhood, 0 )
{
	BEGIN_CLASS_INIT( PGTopologicalNeighborhood )
}

IMPLEMENT_GEOXL_ABSTRACT_CLASS( PGTopologicalNeighborhoodDataPts, 0 )
{
	BEGIN_CLASS_INIT( PGTopologicalNeighborhoodDataPts )
}

IMPLEMENT_GEOXL_CLASS( TNDOneRing, 0 )
{
	BEGIN_CLASS_INIT( TNDOneRing )
}

IMPLEMENT_GEOXL_CLASS( PGTNKnn, 0 )
{
	BEGIN_CLASS_INIT( PGTNKnn )
	ADD_CARD32_PROP(numNeighbors, 0)
}


// ----


PGTopologicalNeighborhood::PGTopologicalNeighborhood() {
   ps = NULL;
   tg = NULL;
}

void PGTopologicalNeighborhood::setup(const PointSet *ps, const InCorePCTopologyGraph *tg) {
   this->ps = ps;
   this->tg = tg;
}


// --


PGTopologicalNeighborhoodDataPts::PGTopologicalNeighborhoodDataPts() {
   pg = NULL;
   data = NULL;
}

void PGTopologicalNeighborhoodDataPts::setup(const PatchGraph *pg, const PointCloudAnimationSequence *data) {
   this->pg = pg;
   this->data = data;
}


// --

void TNDOneRing::collectNeighbors(card32 frame, card32 point, vector<card32> &neighbors) const {
   neighbors.clear();
   pAssert(pg != NULL);
   pAssert(data != NULL);
   if (pg == NULL || data == NULL) return;
   const InCorePCTopologyGraph *tg = pg->getTopology();
   const PGDataPointIndices *dp = pg->getTrajectorySet()->getDataPoints();
   const card32 n = tg->getNumAdjacentVertices(point);
   for (card32 i=0; i<=n; i++) {
      card32 j;
      if (i < n) {
         j = tg->getAdjacentVertex(point, i);
      } else {
         j = point;
      }
      const card32 numDataPoints = dp->getNumDataPoints(frame, j);
      for (card32 k=0; k<numDataPoints; k++) {
         neighbors.push_back(dp->getIndex(frame, j, k));
      }
   }
}


// --


PGTNKnn::PGTNKnn() {
	/// Modified by peter: for symmetric consideration
	numNeighbors = 21;
	knn = NULL;
}

void PGTNKnn::setup(const PointSet *ps, const InCorePCTopologyGraph *tg) {
	PGTopologicalNeighborhood::setup(ps, tg);
	delete knn;
	knn = new TGNeighborhoodKNNSearch(ps, tg, 0, 0);
}

void PGTNKnn::setNumNeighbors(card32 numNeighbors) {
	this->numNeighbors = numNeighbors;
}

void PGTNKnn::collectNeighbors(card32 frame, card32 point, vector<card32> &neighbors) const {
	knn->reset(frame, point);
	card32 count = 0;
	while (!knn->atEnd() && count < numNeighbors) {
		neighbors.push_back(knn->getCurrentIndex());
		knn->next();
		count++;
	}
}

PGTNKnn::~PGTNKnn() {
	delete knn;
}
