//---------------------------------------------------------------------------
#ifndef PGTopologicalNeighborhoodH
#define PGTopologicalNeighborhoodH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;

class PatchGraph;
class PointCloudAnimationSequence;
class TGNeighborhoodKNNSearch;
class InCorePCTopologyGraph;

/// neighborhood of patches in patch graph
class PGTopologicalNeighborhood : public Persistent {
   GEOXL_ABSTRACT_CLASS(PGTopologicalNeighborhood)
 protected:
   const PointSet *ps;
   const InCorePCTopologyGraph *tg;
   
 public:
   PGTopologicalNeighborhood();
   virtual void setup(const PointSet *ps, const InCorePCTopologyGraph *tg);
   const PointSet *getPointSet() {return ps;}
   virtual void collectNeighbors(card32 frame, card32 point, vector<card32> &neighbors) const = 0;
};


/// neighborhood of data points in patch graph
class PGTopologicalNeighborhoodDataPts : public Persistent {
   GEOXL_ABSTRACT_CLASS(PGTopologicalNeighborhoodDataPts)
 protected:
   const PatchGraph *pg;
   const PointCloudAnimationSequence *data;
   
 public:
   PGTopologicalNeighborhoodDataPts();
   virtual void setup(const PatchGraph *pg, const PointCloudAnimationSequence *data);
   virtual void collectNeighbors(card32 frame, card32 point, vector<card32> &neighbors) const = 0;
};



/// one ring neighborhood on the data points
class TNDOneRing : public PGTopologicalNeighborhoodDataPts {
   GEOXL_CLASS(TNDOneRing)

 public:
   virtual void collectNeighbors(card32 frame, card32 point, vector<card32> &neighbors) const;
};


/// k-nearest-neighbors on the data points
class PGTNKnn : public PGTopologicalNeighborhood {
   GEOXL_CLASS(PGTNKnn)

 private:
   card32 numNeighbors;
   TGNeighborhoodKNNSearch *knn;

 public:
   PGTNKnn();
   virtual void setup(const PointSet *ps, const InCorePCTopologyGraph *tg);
   void setNumNeighbors(card32 numNeighbors = 20);
   virtual void collectNeighbors(card32 frame, card32 point, vector<card32> &neighbors) const;
   virtual ~PGTNKnn();
};
   
}

#endif