//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "HFBFBivariateMonomials.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS(HFBFBivariateMonomials, 0)
{
	BEGIN_CLASS_INIT(HFBFBivariateMonomials)
	ADD_CARD32_PROP_UPDATE(degree, 0, &setupDegrees)
}


// --


HFBFBivariateMonomials::HFBFBivariateMonomials() {
	degree = 0;
	setupDegrees();
}

void HFBFBivariateMonomials::setupDegrees() {
	// basis function layout:
	//
	//  [i]  uv	  [i]   uv  [i] uv	 [i]  uv	 [i]  uv
	//	 ------------------------------------------------
	//  [0] c00, 
	//  [1] c01,  [2] c10, 
   //  [3] c02,  [4] c11,  [5] c20, 
   //  [6] c03,  [7] c12,  [8] c21,  [9] c30,
   // [10] c04, [11] c13, [12] c22, [13] c31, [14] c40,
   // ... 
   
	degrees.clear();
	card32 n = getNumBasisFunctions();
	degrees.resize(n);
	card32 k=0;

	// ij ij ij ij		uv uv uv uv
	// 00					00
	// 10 11				01 10
	// 20 21 22			02 11 20
	// 30 31 32 33		03 12 21 30
	for (card32 i=0; i<=degree; i++) {
		for (card32 j=0; j<=i; j++) {
			degrees[k] = UVDeg(j,i-j);
			k++;
		}
	}
}

inline void HFBFBivariateMonomials::computeDegrees(const card32 i, card32 &degU, card32 &degV) {
	pAssert(i<getNumBasisFunctions());
	degU = degrees[i].degU;
	degV = degrees[i].degV;
}

	
void HFBFBivariateMonomials::setup(card32 degree) {
	this->degree = degree;
	setupDegrees();
}
 
card32 HFBFBivariateMonomials::getNumBasisFunctions() {
	// = 1 + 2 + 3 + ... + (degree+1)
	return (degree+1)*(degree+2)/2;
}

card32 HFBFBivariateMonomials::getDegreeFromNumBF(card32 numBF) {
	if (numBF == 1) {
		return 0;
	} else if (numBF == 3) {
		return 1;
	} else if (numBF == 6) {
		return 2;
	} else {
		if (numBF <6) {
			throw ERangeCheck("HFBFBivariateMonomials::getDegreeFromNumBF() - impossible input.");
		}
		warning("HFBFBivariateMonomials::getDegreeFromNumBF() - higher degrees not implemented.");
		return 2;
	}
}


float32 HFBFBivariateMonomials::evaluate(card32 bfIndex, const Vector2f &uv) {
	pAssert(bfIndex<getNumBasisFunctions());
	//switch (bfIndex) {
	//	case 0: return 1;
	//	case 1: return uv[0];
	//	case 2: return uv[1];
	//	case 3: return uv[1]*uv[1];
	//	case 4: return uv[0]*uv[1];
	//	case 5: return uv[0]*uv[0];
	//	default: {
	card32 degU, degV;
	computeDegrees(bfIndex, degU, degV);
	return pow(uv[0], (float32)degU)*pow(uv[1], (float32)degV);
	//	}
	//}
}
