//---------------------------------------------------------------------------
#ifndef HFBFBivariateMonomialsH
#define HFBFBivariateMonomialsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "HeightFieldBasisFunctionSet.h"
//---------------------------------------------------------------------------

namespace geoxl {

/// bivariate monomial basis functions
/// layout:
///  [0] c00, 
///  [1] c01,  [2] c10, 
///  [3] c02,  [4] c11,  [5] c20, 
///  [6] c03,  [7] c12,  [8] c21,  [9] c30,
/// [10] c04, [11] c13, [12] c22, [13] c31, [14] c40,
///
/// i.e.: c00, c01, c10, c02, c11, c20, ...
class HFBFBivariateMonomials : public HeightFieldBasisFunctionSet {
	GEOXL_CLASS( HFBFBivariateMonomials )
	
 private:
	// persistent
	card32 degree;
	// temporary lookup table
	struct UVDeg {
		card32 degU;
		card32 degV;
		UVDeg() {}
		UVDeg(card32 u, card32 v) {degU=u; degV=v;}
	};
	vector<UVDeg> degrees;	

	inline void computeDegrees(const card32 i, card32 &degU, card32 &degV);
	void setupDegrees();

 public:
	HFBFBivariateMonomials();
	
	/// create all monomials u^(n) * v^(m) with n+m <= degree
	/// default value is zero
	void setup(card32 degree);
 
	virtual card32 getNumBasisFunctions();
	virtual float32 evaluate(card32 bfIndex, const Vector2f &uv);
	virtual card32 getDegreeFromNumBF(card32 numBF);
};


}

#endif