//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "HFBFFourier.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS(HFBFFourier, 0)
{
	BEGIN_CLASS_INIT(HFBFFourier)
	ADD_CARD32_PROP(degree, 0)
}


// --


HFBFFourier::HFBFFourier() {
	degree = 0;
}

void HFBFFourier::setup(card32 degree) {
	this->degree = degree;
}
 
card32 HFBFFourier::getNumBasisFunctions() {
	return (2*(degree+1)-1)*(2*(degree+1)-1);
}

inline float32 HFBFFourier::get1dFunc(card32 index, float32 val) {
   // degree	0	 1	  1	2	 2	  3	3
	// index    0	 1   2   3	 4	  5   6
	//	type    cos sin cos sin cos sin cos
	//	n			0	 1	  1	2	 2	  3   3
	
	card32 n = (index+1)/2;
	if (index%2==0) {
		return cos(n*val);
	} else {
		return sin(n*val);
	}
}


float32 HFBFFourier::evaluate(card32 bfIndex, const Vector2f &uv) {
	card32 oneDimNum = 2*(degree+1)-1; // 1 3 5 7 9 ...
	card32 uInd = bfIndex / oneDimNum;
	card32 vInd = bfIndex % oneDimNum;
	pAssert(uInd < oneDimNum);
	pAssert(bfIndex<getNumBasisFunctions());
	return get1dFunc(uInd, uv[0]) * get1dFunc(vInd, uv[1]);
}
