//---------------------------------------------------------------------------
#ifndef LeastSquaresSystemSetupH
#define LeastSquaresSystemSetupH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PTypes.h"
#include "SparseLinearAlgebra.h"
#include "DynamicLinearAlgebra.h"
//---------------------------------------------------------------------------
#include <string>
//---------------------------------------------------------------------------

namespace geoxl {
	using namespace std;

class VariablesNamingScheme;


/**
	sets up a linear system that solves a least squares problem	(dense matrix)
*/
template <typename FloatType, typename IntType = int32>
class DenseLeastSquaresSystemSetup {
 public:
	/// setup linear system for least squares quadratic optimization problem
	/// Sum[j] ((Sum[i] coeff[i] * basisFunctionValue[i][j]) - targetValue[j])^2*weight[j]  --> min
	/// targetValues are the n values that should be approximated
	/// basisFunctionValues are k basis functions evaluated at the n target positions
	/// the matrix must have k columns (no. basis func.) and n rows (no. of target values)
	/// optional n weights (per target) are given in targetValueWeights
	///
	/// standard value for weights is one (if pointer == NULL)
	/// the resulting dense linear system is retured in mat, rhs (right hand side)
	///
	/// exceptions are thrown if the dimensions do not match
	void systemSetup(const DynamicVector<FloatType> &targetValues,
						  const DynamicMatrix<FloatType> &basisFunctionValues,
						  DynamicMatrix<FloatType> &mat, DynamicVector<FloatType> &rhs,
						  const DynamicVector<FloatType> *targetValuesWeights = NULL);
};


/**
	sets up a linear system that solves a least squares problem (sparse matrix)
*/
template <typename FloatType, typename IntType = int32>
class LeastSquaresSystemSetup {
 private:
	
 public:
	/// setup linear system for least squares quadratic optimization problem
	/// Sum {(var_targetVariableIndex[i] - targetValue[i])^2 * targetValuesWeights[i] }  ---> min
	/// standard value for weights is one (if pointer == NULL)
	void systemSetup(const VariablesNamingScheme *naming,
						  const string targetVariableName, const IntType targetVariableIndex,
						  const DynamicVector<FloatType> &targetValues,
						  SparseMatrix<FloatType> &mat, DynamicVector<FloatType> &rhs,
						  const DynamicVector<FloatType> *targetValuesWeights = NULL);
};


}

#endif