//---------------------------------------------------------------------------
#include "LeastSquaresSystemSetup.h"
//---------------------------------------------------------------------------
#include "SparseLinearAlgebra_templCode.h"
#include "DynamicLinearAlgebra_templCode.h"
#include "Exceptions.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



template <typename FloatType, typename IntType>
void DenseLeastSquaresSystemSetup<FloatType, IntType>::systemSetup(
	const DynamicVector<FloatType> &targetValues,
	const DynamicMatrix<FloatType> &basisFunctionValues,
	DynamicMatrix<FloatType> &mat, DynamicVector<FloatType> &rhs,
	const DynamicVector<FloatType> *targetValuesWeights)
{
	if (targetValues.getDim() == 0 || basisFunctionValues.getRows() == 0 || basisFunctionValues.getColumns() == 0) {
		throw ERangeCheck("DenseLeastSquaresSystemSetup::systemSetup() - arguments must not be empty.");
	}
	if (targetValues.getDim() != basisFunctionValues.getRows()) {
		throw ERangeCheck("DenseLeastSquaresSystemSetup::systemSetup() - number of target values and basis "
								"function evaluations does not match "
								"(make sure that targetValues.getDim() == basisFunctionValues.getRows()).");
	}
	if (targetValuesWeights != NULL && targetValuesWeights->getDim() != targetValues.getDim()) {
		throw ERangeCheck("number of weights does not match number of target values. "
								"(make sure that targetValues.getDim() == targetValuesWeights.getRows()).");
	}
	
	const IntType numBF = basisFunctionValues.getColumns();
	const IntType numTargets = targetValues.getDim(); 
	
	mat.setDimension(numBF, numBF, false);
	rhs = nullDVector<FloatType>(numBF);

	for (IntType row=0; row<numBF; row++) {
		for (IntType target=0; target<numTargets; target++) {
		   float32 v = basisFunctionValues[row][target] * targetValues[target];
		   if (targetValuesWeights) {
		      v *= (*targetValuesWeights)[target];
		   }
			rhs[row] += v;
		}
		for (IntType column=0; column<numBF; column++) {
			for (IntType target=0; target<numTargets; target++) {
			   float32 v = basisFunctionValues[row][target] * basisFunctionValues[column][target];
		      if (targetValuesWeights) {
		         v *= (*targetValuesWeights)[target];
		      }
				mat[column][row] += v;
			}
		}
	}
}



template <typename FloatType, typename IntType>
void LeastSquaresSystemSetup<FloatType, IntType>::systemSetup(
	const VariablesNamingScheme *naming,
	const string targetVariableName, const IntType targetVariableIndex,
	const DynamicVector<FloatType> &targetValues,
	SparseMatrix<FloatType> &mat, DynamicVector<FloatType> &rhs,
	const DynamicVector<FloatType> *targetValuesWeights = NULL) 
{
	notImplemented();
}
