//---------------------------------------------------------------------------
#ifndef AnimationBSRRendererH
#define AnimationBSRRendererH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "AnimationBSRContext.h"
#include "SGSimpleAnimationNode.h"
#include "RenderingTool.h"
#include "PatchGraphMergeCandidateSelect.h"
#include "PatchGraphRenderer.h"
#include <qtcore/qtimer.h>
#include "PatchGraphOpticalFlow.h"
#include "PatchGraphMerge.h"
//---------------------------------------------------------------------------

namespace geoxl{

/// Rendering and visualization functionality associated to the AnimationBSR tool
class ANIMATIONRECONSTRUCTION_API AnimationBSRRenderer : public Persistent {
	GEOXL_CLASS( AnimationBSRRenderer )

	private:
		/// Context
		AnimationBSRContext* context;
		/// Rendering tool used
		RenderingTool* rt;
		/// Scene object
		Scene* scene;
		/// Merge selection
		PatchGraphMergeCandidateSelect* selecter;
		/// Patch graph renderer
		StandardPatchGraphRenderer* patchGraphRenderer;
		/// Optical flow test
		PatchGraphMerge* merger;

		/// Current debug point
		int32 monitorPoint;
		/// Current frame (used for selecting patch graphs)
		int32 currentFrame;
		/// Render scaling
		float32 renderScale;

		/// Show the tensor voting sigma environment
		bool showTensorVotingSigma;
		/// Show sampling spacing
		float32 showSamplingSpacing;
		/// Show next merge candidates
		bool showMergeCandidates;
		/// Show height field monomial
		bool showHeightFieldMonomial;
		/// Show optical flow
		bool showOpticalFlow;
		/// Show normals of patch graphs
		bool showNormals;
		/// Index of rendered patch graph from potentially visible ones; -1 for all potetially visible in current frame
		int32 showPGIndex;
		/// draw feature points of current frame
		bool showFeaturePoints;
		/// draw feature correspondences of current frame
		bool drawFeatureCorrespondences;
		/// draw all patch graphs
		bool showAllPatchGraphs;

		/// Debug visualization of the optical flow
		void renderGLOpticalFlow();

	public:
		/// Constructor
		AnimationBSRRenderer();
		/// Destructor
		~AnimationBSRRenderer();
		/// Call rendering functionality
		void render();

		/*** INPUT DATA ACCESS ***/
		/// Get number of frames
		card32 getNumFrames() const;
		/// Get the data animation node
		SGSimpleAnimationNode* getDataAnimationNode() const;
		/// Return index of current frame
		int32 getCurrentFrameIndex() const;
		/// Get data point cloud for frame index
		UnstructuredInCorePointCloud* getDataFrame( card32 index );
		/// Get name of data frame point cloud 
		string getDataFrameName( card32 index ){ char buffer[10]; return std::string( "root/data/frame" ) + itoa( index, buffer, 10 ); }

		/*** MEMBER ACCESS ***/
		inline void setContext( AnimationBSRContext* context ){ this->context = context; }
		inline void setScene( Scene* scene ){ this->scene = scene; }
		inline void setSelecter( PatchGraphMergeCandidateSelect* selecter ){ this->selecter = selecter; }
		inline void setCurrentFrame( const int32 currentFrame ){ this->currentFrame = currentFrame; }
		inline StandardPatchGraphRenderer* getPatchGraphRenderer() const { return patchGraphRenderer; }
		inline void setShowPGIndex( const card32 i ){ showPGIndex = i; }
		inline void setShowHeightFieldMonomial( bool b ){ showHeightFieldMonomial = b; }
		inline void setShowAllPatchGraphs( bool b ){ showAllPatchGraphs = b; }
		inline void setShowMergeCandidates( bool b ){ showMergeCandidates = b; }
		inline void setShowOpticalFlow( bool b ){ showOpticalFlow = b; }
		inline void setMerger( PatchGraphMerge* merger ){ this->merger = merger; }
};
}

#endif