//---------------------------------------------------------------------------
#ifndef Monomial2ndOrderDisplayerH
#define Monomial2ndOrderDisplayerH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "GLEventDisplayer.h"
#include "Object.h"
#include "RenderingTool.h"
#include "Monomials2ndOrder.h"
//---------------------------------------------------------------------------

namespace geoxl {

class GLICPisplayerConfig;

class ANIMATIONRECONSTRUCTION_API Monomial2ndOrderDisplayer : public GLEventObject 
{
	GEOXL_CLASS( Monomial2ndOrderDisplayer )
	private:
		/// Rendering tool
		RenderingTool rt;
		/// Monomial coefficients
		StaticVector<float32,6> coeffs;
		/// Domain
		Vector2f domainU, domainV;
		/// Rendering resolution
		card32 resolution;
		/// Additional points to be visualized
		vector<Vector3f> points;
		/// Additional arrows to be visualized
		Vector3f t_u, t_v;
		/// Saddle point
		bool hasSaddlePoint;
		Vector2f saddleCoords;
	
	public:
		/// Constructor
		Monomial2ndOrderDisplayer();
		/// Destructor
		virtual ~Monomial2ndOrderDisplayer();

		/// Setup
		void setup( StaticVector<float32,6> coeffs, Vector2f domainU, Vector2f domainV, card32 resolution );
		/// Set additional points to be visualized
		void setPoints( vector<Vector3f> points ){ this->points = points; }
		/// Set additional arrows to be visualized
		void setEigenVectors( Vector3f t_u, Vector3f t_v ){ this->t_u = t_u; this->t_v = t_v; }
		/// Setter: saddle point
		void setSaddle( bool hasSaddlePoint, Vector2f saddleCoords ){ this->hasSaddlePoint = hasSaddlePoint; this->saddleCoords = saddleCoords; }

		virtual BoundingBox3f getBoundingBox();
		virtual void drawGL( QGLWidget *widget, Camera &cam );
		virtual GLEventConfiguration *createConfigWindow( QWidget *parent );
		virtual void updateData( GLEventObject *other );
};

class ANIMATIONRECONSTRUCTION_API Monomial2ndOrderDisplayerConfig : public GLEventConfiguration 
{
	Q_OBJECT
	private:
	private slots:
	public:
		Monomial2ndOrderDisplayerConfig( QWidget *parent, GLEventObject *obj );
};

}

#endif