//---------------------------------------------------------------------------
#ifndef PatchGraphRendererH
#define PatchGraphRendererH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PatchGraph;
class PointCloudAnimationSequence;

/**
	opengl display of patch graphs and trajectory sets
*/
class ANIMATIONRECONSTRUCTION_API PatchGraphRenderer : public Persistent {
	GEOXL_ABSTRACT_CLASS( PatchGraphRenderer )

 private:


 public:
	virtual void renderGL(PatchGraph *pg, PointCloudAnimationSequence *originalPoints = NULL, card32 time = 0) = 0;
	virtual void invalidate() = 0;

   virtual void assign(const Object* obj, CopyContext *context = NULL);
};

static const card32 CM_PATCH_GRAPH_COLOR = 0;
static const card32 CM_COLOR_CHANNEL_COLOR = 1;
static const card32 CM_FIXED_COLOR = 2;
static const card32 CM_CLUSTER_COLOR = 3;
static const card32 CM_POTENTIALS = 4;

/**
	standard renderer with various options
*/
class ANIMATIONRECONSTRUCTION_API StandardPatchGraphRenderer : public PatchGraphRenderer {
	GEOXL_CLASS( StandardPatchGraphRenderer )

 private:
	bool showPoints;
	bool showCoords;
	bool showCorrDataPoints;
	bool showAllDataPoints;
	bool showLocalPatches;
	bool showInterpolatedPatches;	
	bool showTopology;
	bool showSampleSpacing;
	bool showCorrespondences;
	bool showAllFrames;
	bool showUrshape;
   Vector3f afInterFrameTransl;
   Vector3f urshapeTransl;
   bool globalTimeTranslation;
   
	bool useDList;
	static const card32 COORDS_SIMPLE = 0;
	static const card32 COORDS_ARROWS = 1;
	static const card32 COORDS_XYZ = 2;
	card32 coordsDrawStyle;
	float32 coordsSize;
	float32 pointSize;
	
	float32 patchSize;
	card32 patchSubdiv;
	bool patchLighting;
	float32 kernelWidthFactor;
	static const card32 PATCH_INTERP_LIN = 0;
	static const card32 PATCH_INTERP_GAUSSIAN = 1;
	static const card32 PATCH_INTERP_COS = 2;
	card32 interpKernel;
	
	card32 pointsColorMode;
	card32 topologyColorMode;
	card32 dataPointsColorMode;
	card32 allDataPointsColorMode;
	
	Vector4f pointsColor;
	Vector4f corrDataPointsColor;
	Vector4f otherDataPointsColor;
	Vector4f topologyColor;
	Vector4f urshapePointColor;
	Vector4f urshapeTopologyColor;
	
	bool extraAlpha;

	float32 minPotValue;
	float32 maxPotValue;
	Vector3f potMask;
	card32 correspondencesOffset;
	
	
	// -- intern temp
	GLuint dlist;		 
	bool needsUpdate;
	
	inline float32 calcWeight(const Vector3f &neighborPos, const Vector3f &pos, float32 kernelSize);
	inline void rebuildDList();

 public:
	StandardPatchGraphRenderer();

	virtual void renderGL(PatchGraph *pg, PointCloudAnimationSequence *dataPoints = NULL, card32 time = 0);
	virtual void invalidate();

	virtual ~StandardPatchGraphRenderer();

	/*** MEMBER ACCESS ***/
	inline void setUseDL( bool b ){ useDList = b; }
	inline void setShowTopology( bool b ){ showTopology = b; }
	inline void setShowCoords( bool b ){ showCoords = b; }
	inline void setShowDataPoints( bool b ){ showCorrDataPoints = b; }
	inline void setShowPoints( bool b ){ showPoints = b; }
	inline void setShowLocalPatches( bool b ){ showLocalPatches = b; }
	inline void setShowAllFrames(bool b) {showAllFrames = b;}
	inline void setShowCorrespondences(bool b) {showCorrespondences = b;}
	inline Vector3f getInterFrameTranslation() {return afInterFrameTransl;}
	inline void setShowAllDataPoints( bool b ){ showAllDataPoints = b; }
	inline void setTopologyColorMode( const card32 mode ){ topologyColorMode = mode; }
	inline void setPointsColorMode( const card32 mode ){ pointsColorMode = mode; }
	inline void setExtraAlpha(bool b) {extraAlpha = b;}
	inline void setShowUrshape(bool b) {showUrshape = b;}

	Vector3f getTranslationVector() {return afInterFrameTransl;}
};


}

#endif