//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "GlobalOptimization2.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PatchGraph.h"
#include "TrajectorySet.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "PatchGraphOpt2.h"
#include "AnimationBSRContext.h"
#include "PatchTypes.h"
#include "InCorePCTopologyGraph.h"
#include "PGOpt2Config.h"
#include "PGOpt2GraphCreator.h"
#include "PGOpt2FramePairVars.h"
#include "PGOpt2FramePair.h"
#include "PGOpt2Frame.h"
#include "OptimizationGraph.h"
#include "PatchTypes.h"
#include "MathTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( GlobalOptimization2 ,0) {
	BEGIN_CLASS_INIT( GlobalOptimization2 );
}

// ----

GlobalOptimization2::GlobalOptimization2() {
}

void GlobalOptimization2::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

GlobalOptimization2::~GlobalOptimization2() {
}




void GlobalOptimization2::optimize(AnimationBSRContext *context, PatchGraph* pg, PointCloudAnimationSequence *data, DebugRenderer *debugRenderer,OptimizationRecord2 *optRecord) 
{
   PatchGraphOpt2 opt;
   context->getGlobalOptConfig()->potentials.configure(&opt);

	const card32 numFrames = pg->getTrajectorySet()->getNumFrames();
   const card32 numPatches = pg->getTrajectorySet()->getNumPatches();
   
   vector< vector<card32> > frameVarIndices(numFrames);
   vector< vector<card32> > edgeVarIndices(numFrames);
   for (card32 frame=0; frame<numFrames; frame++) {
      frameVarIndices[frame].resize(numPatches);
      edgeVarIndices[frame].resize(numPatches);
      for (card32 patch=0; patch<numPatches; patch++) {
         frameVarIndices[frame][patch] = patch;      
         edgeVarIndices[frame][patch] = patch;      
      }
   }

   if (!context->getOptGraph2useUrshape()) {
   
      /// simple optimization
   
      PGOpt2GraphCreator gc;
      PGOpt2FramePairVars *pairVars = gc.createPairVars(
		   pg, 
		   context->getCyclicTime(), false, false, false, 
		   context->getOptGraph2twoWayTemporalEdges()
	   );
      OptimizationGraph *optGraph = gc.createOptGraph(
		   pg, data, pairVars, 
		   context->getCyclicTime(), false, false, false, 
		   context->getOptGraph2twoWayTemporalEdges(), 
		   &frameVarIndices, &edgeVarIndices, false, false, context->getTopology2MinimumVertexDegree()
	   );
   
      opt.optimize(context->getGlobalOptConfig()->params, context, optGraph, pg, data, debugRenderer, optRecord);

      delete pairVars;
      delete optGraph;
   } else {
   
      /// urshape version - first scene to urshape, then urshape to scene optimization
      
      // create new urshape
      pg->createAndClearUrshape(true, 0);     

      // variables
      PGOpt2GraphCreator gc;
      PGOpt2FramePairVars *pairVars;
      OptimizationGraph *optGraph;

      // fit trajectories only (for acceleration) 
      pairVars = gc.createPairVars(
		   pg, 
		   context->getCyclicTime(), false, false, false, 
		   context->getOptGraph2twoWayTemporalEdges(),
		   false
	   );
      optGraph = gc.createOptGraph(
		   pg, data, pairVars, 
		   context->getCyclicTime(), false, false, false, 
		   context->getOptGraph2twoWayTemporalEdges(), 
		   &frameVarIndices, &edgeVarIndices,
		   false, false, context->getTopology2MinimumVertexDegree()
	   );
   
      opt.optimize(context->getGlobalOptConfig()->params, context, optGraph, pg, data, debugRenderer, optRecord);

      delete pairVars;
      delete optGraph;

      // jointly optimize urshape and frames (omit inter-frame links) 
      pairVars = gc.createPairVars(
		   pg, 
		   context->getCyclicTime(), true, true, true, 
		   context->getOptGraph2twoWayTemporalEdges(),
		   true
	   );
      optGraph = gc.createOptGraph(
		   pg, data, pairVars, 
		   context->getCyclicTime(), true, true, true, 
		   context->getOptGraph2twoWayTemporalEdges(), 
		   &frameVarIndices, &edgeVarIndices,
		   false, true, context->getTopology2MinimumVertexDegree()
	   );
   
      opt.optimize(context->getGlobalOptConfig()->params, context, optGraph, pg, data, debugRenderer, optRecord);

      delete pairVars;
      delete optGraph;

   }
}
