//---------------------------------------------------------------------------
#ifndef HoleFilling2H
#define HoleFilling2H
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PatchGraph;
struct OptimizationRecord2;
class AnimationBSRContext;
class DebugRenderer;
class PointCloudAnimationSequence;

///
///   fills in estimates for uninitialized patches using prediction and optimization
///
class ANIMATIONRECONSTRUCTION_API HoleFilling2 : public Persistent {
	GEOXL_CLASS( HoleFilling2 )
 private:
   struct OptPatch {
      card32 frame;
      card32 patch;
   };

   /// looks for uninitialized patches directly adjacent to initialized ones
   void computeNextOptPatches(AnimationBSRContext *context, const PatchGraph *pg, vector<OptPatch> &result);
   /// fills in prediction from neighbors
   void predict(AnimationBSRContext *context, PatchGraph *pg, const vector<OptPatch> &indices);
   /// perform actual optimization of node positions
   void partialOptimization(
		AnimationBSRContext *context, 
		PatchGraph *pg, 
	   PointCloudAnimationSequence *data,
		const vector<OptPatch> &indices, 
		DebugRenderer *debugRenderer, 
		OptimizationRecord2 *optRecord);
   /// (finally) marks patches as initialized
   void markAsInitialized(PatchGraph *pg, const vector<OptPatch> &indices);
   /// finally set topology_depth to zero everywhere (and [debug-only-] check for dangeling uninitialized points)
   void checkInitEverything(PatchGraph *pg);

 public:
	HoleFilling2();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

	virtual void fillHoles( 
		AnimationBSRContext *context,
		PatchGraph* pg, 
	   PointCloudAnimationSequence *data,
		DebugRenderer *debugRenderer,
		OptimizationRecord2 *optRecord
   );

   ~HoleFilling2();
};

}

#endif