//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PairwiseAlignment2.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PatchGraph.h"
#include "PointCloudAnimationSequence.h"
#include "AnimationBSRContext.h"
#include "PGOperationsDebugging.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "PatchGraphRenderer.h"
#include "CorrespondenceExtrapolation.h"
#include "PGOpt2Config.h"
#include "PGOpt2Parameters.h"
#include "PatchGraphOpt2.h"
#include "AnimationBSRRenderer.h"
#include "TrajectorySet.h"
#include "PGOpt2GraphCreator.h"
#include "PGOpt2FramePairVars.h"
#include "OptimizationGraph.h"
#include "TopologyMerging2.h"
#include "PGOpt2Frame.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PairwiseAlignment2 ,0) {
	BEGIN_CLASS_INIT( PairwiseAlignment2 );
}


// ----


PairwiseAlignment2::PairwiseAlignment2() {}

void PairwiseAlignment2::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

PairwiseAlignment2::~PairwiseAlignment2() {}




void PairwiseAlignment2::buildPairsPG(
   const	PatchGraph	*source,					
   const	PatchGraph	*target,					
   const int32 sourceFrameIndex,
   const int32 targetFrameIndex,
   PatchGraph *&resultPG,
   PointCloudAnimationSequence *&resultData)
{
	pAssert(source->getTrajectorySet()->getNumFrames() > 0);
	const PointSet *sourcePS       = source->getTrajectorySet()->getTrajectories();
	const card32 numPts            = sourcePS->getHeight();
	const card32 absDataFrameIndex = sourceFrameIndex	+	source->getStartFrameIndex();
	
	// --- setup trajectories (initialize second frame with first)
	PointSet *ps = new PointSet();
	ps->clearAndSetup(2, numPts, sourcePS->getDescr());
	for (card32 i=0; i<numPts; i++) 
	{
		ps->setVertex(0, i, sourcePS->getDataPointer(sourceFrameIndex, i));
		ps->setVertex(1, i, sourcePS->getDataPointer(sourceFrameIndex, i));
	}

	TrajectorySet *ts = new TrajectorySet();
	ts->clearAndSetup(ps);

	// --- assemble resulting patch graph
	InCorePCTopologyGraph *tg = (InCorePCTopologyGraph*)(source->getTopology()->copy());
	
	resultPG = new PatchGraph();
	resultPG->clearAndSetup(ts, tg);	 

	// --- create data points
	resultData = new PointCloudAnimationSequence();
	{
		PointSet *dataPS0 = new PointSet();
		PointSet *dataPS = new PointSet();
		VertexDescriptor vd;
		vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
		vd.pushAttrib(mVAD("flag_inlier", 1, VAD::DATA_FORMAT_INT32));
		AAT destAAT			= vd.getToken("position");
		AAT	FLAG_INLIER	= vd.getToken("flag_inlier"); 
		const PointSet	*targetPS			= target->getTrajectorySet()->getTrajectories();
		AAT			sourceAAT			= targetPS->getAAT("position");
		const card32	height_target	= targetPS->getHeight();
		dataPS->clearAndSetup(1, height_target, &vd);
		dataPS0->clearAndSetup(1, 0, &vd);
		resultData->addFrame(dataPS0);
		for (card32 id = 0; id < height_target; id++) 
		{
			Vector3f	pos = targetPS->get3f(targetFrameIndex, id, sourceAAT);
			dataPS->set3f(id, destAAT, pos);
			dataPS->set<int32>(id, FLAG_INLIER, 1);
		}
		resultData->addFrame(dataPS);
	}

	resultPG->createPatchAttribute(mVAD("topology_depth", 1,	VAD::DATA_FORMAT_INT32));
	ps = resultPG->getTrajectorySet()->getTrajectories();
	AAT		TOP_DEPTH	= ps->getAAT("topology_depth");
	for (card32 i=0; i<numPts; i++) 
	{
		ps->set<int32>(0, i, TOP_DEPTH, 0);
		ps->set<int32>(1, i, TOP_DEPTH, 0);
	}
}


PointSet *PairwiseAlignment2::alignParts( 
	AnimationBSRContext *context,
	const PatchGraph* pgSource, 
	const PatchGraph* pgTarget, 
	const card32 sourceFrame,
	const card32 targetFrame,
	const map<card32, Vector3f> &featureCorrespondencesSourceToTarget,
	DebugRenderer *debugRenderer,
	OptimizationRecord2 *optRecord)
{
	/// Build the patch graph for optimization
	PatchGraph *alignPG;
	PointCloudAnimationSequence *alignDataPts;
   buildPairsPG(
	   pgSource,					
	   pgTarget,					
	   sourceFrame,
	   targetFrame,
	   //featureCorrespondencesSourceToTarget,
	   alignPG,
	   alignDataPts
	);

   {
      alignPG->createPatchAttribute(mVAD("dataIndex", 1, VAD::DATA_FORMAT_INT32));
      PointSet *alignPS = alignPG->getTrajectorySet()->getTrajectories();
      AAT dataIndexAAT = alignPS->getAAT("dataIndex");      
      const card32 n = alignPS->getNumEntries();
      for (card32 i=0; i<n; i++) {
         alignPS->set<int>(i, dataIndexAAT, -1);
      }
   }

   PatchGraphOpt2 opt;
   context->getPairwiseAlignmentConfig()->potentials.configure(&opt);

   const card32 numPatches = alignPG->getTrajectorySet()->getNumPatches();
   vector< vector<card32> > frameVarIndices(2);
   vector< vector<card32> > edgeVarIndices(2);
   frameVarIndices[1].resize(numPatches);
   edgeVarIndices[0].resize(numPatches);
   edgeVarIndices[1].resize(numPatches);
   for (card32 i=0; i<numPatches; i++) {
      frameVarIndices[1][i] = i;
      edgeVarIndices[0][i] = i;
      edgeVarIndices[1][i] = i;
   }
   PGOpt2GraphCreator gc;
   PGOpt2FramePairVars *pairVars = gc.createPairVars(alignPG, false, false, false, false, false);
   OptimizationGraph *optGraph = gc.createOptGraph(
      alignPG, alignDataPts, pairVars, 
      false, false, false, false, false, 
      &frameVarIndices, &edgeVarIndices,
      false, false, context->getTopology2MinimumVertexDegree()
   );
   
   if (!featureCorrespondencesSourceToTarget.empty()) {
      optGraph->getFrame(1)->setCorrespondences(featureCorrespondencesSourceToTarget);
   }
   
   if (context->getAlignment2FeaturesFirstDataSecond() && !featureCorrespondencesSourceToTarget.empty()) {
      bool markers = context->getPairwiseAlignmentConfig()->potentials.framePotentials.distanceMarkers;
      bool data = context->getPairwiseAlignmentConfig()->potentials.framePotentials.distancePairwise;
      context->getPairwiseAlignmentConfig()->potentials.framePotentials.distanceMarkers = true;
      context->getPairwiseAlignmentConfig()->potentials.framePotentials.distancePairwise = false;
      opt.optimize(context->getPairwiseAlignmentConfig()->params, context, optGraph, alignPG, alignDataPts, debugRenderer, optRecord);
      context->getPairwiseAlignmentConfig()->potentials.framePotentials.distanceMarkers = false;
      context->getPairwiseAlignmentConfig()->potentials.framePotentials.distancePairwise = true;
      opt.optimize(context->getPairwiseAlignmentConfig()->params, context, optGraph, alignPG, alignDataPts, debugRenderer, optRecord);
      context->getPairwiseAlignmentConfig()->potentials.framePotentials.distanceMarkers = markers;
      context->getPairwiseAlignmentConfig()->potentials.framePotentials.distancePairwise = data;
   } else {
      opt.optimize(context->getPairwiseAlignmentConfig()->params, context, optGraph, alignPG, alignDataPts, debugRenderer, optRecord);
   }
   
   delete pairVars;
   delete optGraph;
   
   PointSet *aps = alignPG->getTrajectorySet()->getTrajectories();
   
   PointSet *aligned = new PointSet();   
   aligned->clearAndSetup(1, aps->getHeight(), aps->getDescr());
   for (card32 i=0; i<aps->getHeight(); i++) {
      aligned->setVertex(i, aps->getDataPointer(1, i));
   }
   
	delete alignPG;
	delete alignDataPts;
	
	return aligned;
}

static PatchGraph *copySlice(const PatchGraph *source, card32 frameIndex) {
   const PointSet *sourcePS = source->getTrajectorySet()->getTrajectories();
   PointSet *traj = new PointSet();
   const card32 h = sourcePS->getHeight();
   traj->clearAndSetup(1, h, sourcePS->getDescr());
   for (card32 i=0; i<h; i++) {
      traj->setVertex(i, sourcePS->getDataPointer(frameIndex, i));
   }
   TrajectorySet *ts = new TrajectorySet();
   ts->clearAndSetup(traj);   
   PatchGraph *result = new PatchGraph();
   result->clearAndSetup(ts, (InCorePCTopologyGraph*)(source->getTopology()->copy()));
   return result;
}




void PairwiseAlignment2::reverseAlignment(
   AnimationBSRContext *context, 
   PatchGraph *combinedPG, 
   const card32 whichPartHasBeenMerged, 
   const PatchGraph *pg1, 
   const PatchGraph *pg2, 
	const map<card32, Vector3f> &featureCorrespondences,
	DebugRenderer *debugRenderer)
{
   PatchGraph *sourcePG = NULL;
   const PatchGraph *targetPG = NULL;
   card32 targetFrameIndex = 0xFFFFFFFF;
   int32 numPG1Frames = pg1->getTrajectorySet()->getNumFrames();
   map<card32, Vector3f> featureCorrespondencesSourceToTarget;
   if (whichPartHasBeenMerged == TopologyMerging2::MERGED_LEFT) {
      // left (pg1) has already been merged, i.e. now merge right (pg2)
      sourcePG = copySlice(combinedPG, numPG1Frames); // pg1->width is index of first frame of pg2
      targetPG = pg1;
      targetFrameIndex = numPG1Frames-1;
      
      // in the case of aligning "right-to-left", the indices of the feature points are offset by pg1.height
      const card32 heightPG1 = pg1->getTrajectorySet()->getNumPatches();
      featureCorrespondencesSourceToTarget.clear();
      map<card32, Vector3f>::const_iterator pos = featureCorrespondences.begin();
      map<card32, Vector3f>::const_iterator end = featureCorrespondences.end();
      while (pos != end) {
         pair<card32, Vector3f> entry(*pos);
         entry.first += heightPG1;
         featureCorrespondencesSourceToTarget.insert(entry);
         ++pos;
      }
   } else {
      // right (pg2) has already been merged, i.e. now merge left (pg1)
      sourcePG = copySlice(combinedPG, numPG1Frames-1); // pg1->width-1 is index of last frame of pg1
      targetPG = pg2;
      targetFrameIndex = 0;
      featureCorrespondencesSourceToTarget = featureCorrespondences;
   }

	// Build the patch graph for optimization
	PatchGraph *alignPG;
	PointCloudAnimationSequence *alignDataPts;
   buildPairsPG(
	   sourcePG,					
	   targetPG,					
	   0,
	   targetFrameIndex,
	   //featureCorrespondencesSourceToTarget,
	   alignPG,
	   alignDataPts
	);
	
	// add additional markers
   {
      // first initialize everything to "no correspondence"
      alignPG->createPatchAttribute(mVAD("dataIndex", 1, VAD::DATA_FORMAT_INT32));
      PointSet *alignPS = alignPG->getTrajectorySet()->getTrajectories();
      AAT dataIndexAAT = alignPS->getAAT("dataIndex");      
      const card32 n = alignPS->getNumEntries();
      for (card32 i=0; i<n; i++) {
         alignPS->set<int>(i, dataIndexAAT, -1);
      }

      // then: fill in known parts
      const card32 pg1Height = pg1->getTrajectorySet()->getTrajectories()->getHeight();
      const card32 pg2Height = pg2->getTrajectorySet()->getTrajectories()->getHeight();
      if (whichPartHasBeenMerged == TopologyMerging2::MERGED_LEFT) {
         // deformed left has already been filled in to its right
         for (card32 y=0; y<pg1Height; y++) {
            alignPS->set<int>(1, y, dataIndexAAT, y);
         }
      } else {
         // deformed right has already been filled in to its left
         for (card32 y=0; y<pg2Height; y++) {
            alignPS->set<int>(1, y+pg1Height, dataIndexAAT, y);
         }
      }
   }

   PatchGraphOpt2 opt;
   context->getPairwiseAlignmentConfig()->potentials.configure(&opt);

   const card32 numPatches = alignPG->getTrajectorySet()->getNumPatches();
   vector< vector<card32> > frameVarIndices(2);
   vector< vector<card32> > edgeVarIndices(2);
   frameVarIndices[1].resize(numPatches);
   edgeVarIndices[0].resize(numPatches);
   edgeVarIndices[1].resize(numPatches);
   for (card32 i=0; i<numPatches; i++) {
      frameVarIndices[1][i] = i;
      edgeVarIndices[0][i] = i;
      edgeVarIndices[1][i] = i;
   }
   PGOpt2GraphCreator gc;
   PGOpt2FramePairVars *pairVars = gc.createPairVars(alignPG, false, false, false, false, false);
   OptimizationGraph *optGraph = gc.createOptGraph(
      alignPG, alignDataPts, pairVars, false, false, false, false, false, 
      &frameVarIndices, &edgeVarIndices, false, false, 
      context->getTopology2MinimumVertexDegree()
   );
   OptimizationRecord2 optRecord;   
   opt.optimize(context->getPairwiseAlignmentConfig()->params, context, optGraph, alignPG, alignDataPts, debugRenderer, &optRecord);

   delete pairVars;
   delete optGraph;
   
   {
      const card32 pg1Height = pg1->getTrajectorySet()->getTrajectories()->getHeight();
      const card32 pg2Height = pg2->getTrajectorySet()->getTrajectories()->getHeight();
      
      PointSet *alignedPS         = alignPG->getTrajectorySet()->getTrajectories();
      PointSet *combinedPS        = combinedPG->getTrajectorySet()->getTrajectories();
      const AAT sourcePosAAT      = alignedPS->getAAT("position");
      const AAT sourceNormalAAT   = alignedPS->getAAT("normal");
      const AAT sourceTangentUAAT = alignedPS->getAAT("tangent_u");
      const AAT sourceFlagsAAT    = alignedPS->getAAT("flags");
      const AAT targetPosAAT      = combinedPS->getAAT("position");
      const AAT targetNormalAAT   = combinedPS->getAAT("normal");
      const AAT targetTangentUAAT = combinedPS->getAAT("tangent_u");
      const AAT targetFlagsAAT    = combinedPS->getAAT("flags");
		const AAT targetTopDepthAAT = combinedPS->getAAT("topology_depth"); 
      
      if (whichPartHasBeenMerged == TopologyMerging2::MERGED_LEFT) {
         const card32 targetFrame = pg1->getTrajectorySet()->getNumFrames()-1;
         for (card32 i=0; i<pg2Height; i++) {
            const card32 y = i+pg1Height;
            combinedPS->set3f   (targetFrame, y, targetPosAAT,      alignedPS->get3f   (1, y, sourcePosAAT)); 
            combinedPS->set3f   (targetFrame, y, targetNormalAAT,   alignedPS->get3f   (1, y, sourceNormalAAT)); 
            combinedPS->set3f   (targetFrame, y, targetTangentUAAT, alignedPS->get3f   (1, y, sourceTangentUAAT)); 
            combinedPS->set<int>(targetFrame, y, targetFlagsAAT,    alignedPS->get<int>(1, y, sourceFlagsAAT)); 
            combinedPS->set<int>(targetFrame, y, targetTopDepthAAT, 0);
         }
      } else {
         const card32 targetFrame = pg1->getTrajectorySet()->getNumFrames();
         for (card32 i=0; i<pg1Height; i++) {
            const card32 y = i;
            combinedPS->set3f   (targetFrame, y, targetPosAAT,		  alignedPS->get3f   (1, y, sourcePosAAT)); 
            combinedPS->set3f   (targetFrame, y, targetNormalAAT,	  alignedPS->get3f   (1, y, sourceNormalAAT)); 
            combinedPS->set3f   (targetFrame, y, targetTangentUAAT, alignedPS->get3f   (1, y, sourceTangentUAAT)); 
            combinedPS->set<int>(targetFrame, y, targetFlagsAAT,	  alignedPS->get<int>(1, y, sourceFlagsAAT)); 
            combinedPS->set<int>(targetFrame, y, targetTopDepthAAT, 0);
         }
      }
   }
   
	delete alignPG;
	delete alignDataPts;
}
