//---------------------------------------------------------------------------
#ifndef PairwiseAlignment2H
#define PairwiseAlignment2H
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PatchGraph;
class PointCloudAnimationSequence;
class AnimationBSRContext;
class DebugRenderer;
class PointSet;
class BSROptConfig;
class AnimationBSRRenderer;
struct OptimizationRecord2;


///
///   aligns two frames of a patch graph using energy optimization
///
class ANIMATIONRECONSTRUCTION_API PairwiseAlignment2 : public Persistent {
	GEOXL_CLASS( PairwiseAlignment2 )
 private:
   
   void buildPairsPG(
	   const	PatchGraph *source,					
	   const	PatchGraph *target,					
	   const int32 sourceFrameIndex,
	   const int32 targetFrameIndex,
	   PatchGraph *&resultPG,
	   PointCloudAnimationSequence *&resultData
	);

 public:
	PairwiseAlignment2();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   /// align two patch graphs
   /// returns deformed version of "source" that is aligned with "target" in their mutually overlapping area
   /// the result contains only one frame, which is the deformed "source", keeping the same number of patches and indexing scheme
	virtual PointSet *alignParts( 
		AnimationBSRContext *context,
		const PatchGraph* pgSource, 
		const PatchGraph* pgTarget, 
		const card32 sourceFrame,
		const card32 targetFrame,
		const map<card32, Vector3f> &featureCorrespondencesSourceToTarget,
		DebugRenderer *debugRenderer,
		OptimizationRecord2 *optRecord);

   /// performs reverse alignment: after alignParts has been performed twice (left <-> right and vice versa)
   /// one part has been chosen and written into combinedPG. This method fills in the missing part by performing
   /// the other alignment (the other way round) again, but now using the known correspondences establishe during merging
   /// the result is written into combinedPG
   virtual void reverseAlignment(
      AnimationBSRContext *context, 
      PatchGraph *combinedPG, 
      const card32 whichPartHasBeenMerged, 
      const PatchGraph *pg1, 
      const PatchGraph *pg2, 
		const map<card32, Vector3f> &featureCorrespondences,
		DebugRenderer *debugRenderer
   );

   ~PairwiseAlignment2();
};

}

#endif