//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PatchGraphJoin2.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PatchGraph.h"
#include "TrajectorySet.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "InCorePCTopologyGraph.h"
#include "PGDataPointIndices.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PatchGraphJoin2 ,0) {
	BEGIN_CLASS_INIT( PatchGraphJoin2 );
}

// ----

PatchGraphJoin2::PatchGraphJoin2() {
}

void PatchGraphJoin2::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

PatchGraphJoin2::~PatchGraphJoin2() {
}


void PatchGraphJoin2::createCombinedTrajectorySet( 
	TrajectorySet* ts, 
	InCorePCTopologyGraph* tg,
	const PatchGraph* pg1,
	const PatchGraph* pg2)
{
	card32 patchOffset	= pg1->getTrajectorySet()->getNumPatches();
	card32 frameOffset	= pg1->getTrajectorySet()->getNumFrames();
	int32	frameLeft	= pg1->getTrajectorySet()->getNumFrames() - 1;
	int32	frameRight	= frameLeft + 1;
	AAT		POSITION	= ts->getTrajectories()->getAAT("position");
	PointSet *psSource	= NULL;

	/// We need the folloiwng channel for iterative extrapolation
	ts->createPatchAttribute(mVAD("topology_depth", 1,	VAD::DATA_FORMAT_INT32));
	AAT			TOP_DEPTH			= ts->getTrajectories()->getAAT("topology_depth");

	// Frame 0 .. frameLeft: 
	//  - patch 0 .. offset-1: copy positions, data point assignments
	//  - patch offset .. ts->getNumPatches: reset
	for(int32 frameIndex = 0; frameIndex <= frameLeft; frameIndex++)
	{
		psSource = pg1->getTrajectorySet()->copySingleFrame(frameIndex);
		for(unsigned int patchIndex = 0; patchIndex < patchOffset; patchIndex++)
		{
			/// // Position
			ts->getTrajectories()->setVertex(frameIndex, patchIndex, psSource->getDataPointer(0, patchIndex));
			/// the geometries of these points are fixed
			ts->getTrajectories()->set<int32>(frameIndex, patchIndex, TOP_DEPTH, 0);
			/// Data point assignment
			vector<card32> indices = pg1->getTrajectorySet()->getDataPoints()->getIndices(frameIndex, patchIndex);
			ts->getDataPoints()->setIndices(frameIndex, patchIndex, indices);
		}
		delete psSource; psSource = NULL;
		for (unsigned int patchIndex = patchOffset; patchIndex < ts->getNumPatches(); patchIndex++)
		{
			// Position
			ts->getTrajectories()->set3f(frameIndex, patchIndex, POSITION, makeVector3f( 0,0,0 ));
			/// the geometries of these points are unknown
			ts->getTrajectories()->set<int32>(frameIndex, patchIndex, TOP_DEPTH, -1);
			// Data point assignment
			vector<card32> indices; indices.clear();
			ts->getDataPoints()->setIndices(frameIndex, patchIndex, indices);
		}
		///Create DataPoints.

	}

	// Frame frameRight .. ts->getNumFrames()-1: 
	// - patch 0: .. offset-1: reset
	// - patch offset .. ts->getNumPatches): copy positions, data point assignments
	for(int32 frameIndex = frameRight; frameIndex < int32( ts->getNumFrames()); frameIndex++)
	{
		for(unsigned int patchIndex = 0; patchIndex < patchOffset; patchIndex++)
		{
			// Position
			ts->getTrajectories()->set3f( frameIndex, patchIndex, POSITION, makeVector3f(0,0,0 ));
			/// the geometries of these points are unknown
			ts->getTrajectories()->set<int32>(frameIndex, patchIndex, TOP_DEPTH, -1);
			// Data point assignment
			vector<card32> indices; indices.clear();
			ts->getDataPoints()->setIndices( frameIndex, patchIndex, indices );
		}
		psSource = pg2->getTrajectorySet()->copySingleFrame( frameIndex - frameOffset );
		for ( unsigned int patchIndex = patchOffset; patchIndex < ts->getNumPatches(); patchIndex++ )
		{
			// // Position
			// const Vector3f pos = psSource->get3f( 0, patchIndex - patchOffset, POSITION );
			// ts->getTrajectories()->set3f( frameIndex, patchIndex, POSITION, pos );
			ts->getTrajectories()->setVertex( frameIndex, patchIndex, psSource->getDataPointer( 0, patchIndex - patchOffset) );
			/// the geometries of these points are unknown
			ts->getTrajectories()->set<int32>(frameIndex, patchIndex, TOP_DEPTH, 0);

			// Data point assignment
			vector<card32> indices = pg2->getTrajectorySet()->getDataPoints()->getIndices( frameIndex - frameOffset, patchIndex - patchOffset );
			ts->getDataPoints()->setIndices( frameIndex, patchIndex, indices );
		}
		delete psSource; psSource = NULL;

		///Create DataPoints

	}

	// Copy known topology from pg1 and pg2
	tg->clearAndSetup( pg1->getTopology()->getEdges()->getDescr() );
	// Copy topology from pg1
	for (unsigned int patchIndex = 0; 
		patchIndex < pg1->getTrajectorySet()->getNumPatches(); 
		patchIndex++ )
	{
		for (unsigned int vertexIndex = 0; 
			vertexIndex < pg1->getTopology()->getNumAdjacentVertices( patchIndex ); 
			vertexIndex++ )
		{
			tg->addEdge(patchIndex, pg1->getTopology()->getAdjacentVertex( patchIndex, vertexIndex ) );
		}
	}
	// Copy topology from pg2
	for(unsigned int patchIndex = 0; 
		patchIndex < pg2->getTrajectorySet()->getNumPatches(); 
		patchIndex++ )
	{
		for (unsigned int vertexIndex = 0; 
			vertexIndex < pg2->getTopology()->getNumAdjacentVertices(patchIndex); 
			vertexIndex++ )
		{
			tg->addEdge(patchIndex + patchOffset, pg2->getTopology()->getAdjacentVertex( patchIndex, vertexIndex ) + patchOffset );
		}
	}
}


PatchGraph *PatchGraphJoin2::createCombinedPatchGraph( 
	const PatchGraph* pg1,
	const PatchGraph* pg2)
{
	card32 frameOffset = pg1->getTrajectorySet()->getNumFrames();
	int32 patchOffset = pg1->getTrajectorySet()->getNumPatches();
	card32 numFrames = pg1->getTrajectorySet()->getNumFrames() + pg2->getTrajectorySet()->getNumFrames();
	card32 numPatches = pg1->getTrajectorySet()->getNumPatches() + pg2->getTrajectorySet()->getNumPatches();
	card32 localFrameLeft = pg1->getTrajectorySet()->getNumFrames() - 1;
	card32 localFrameRight = localFrameLeft + 1;

	// Create combined patch graph data structure
	TrajectorySet* ts = new TrajectorySet();
	PointSet* trajectories = new PointSet();
	if(pg1->getTrajectorySet()->getTrajectories()->getWidth() >  pg2->getTrajectorySet()->getTrajectories()->getWidth()){
		trajectories->clearAndSetup( numFrames, numPatches, pg1->getTrajectorySet()->getTrajectories()->getDescr() );
	} else {
		trajectories->clearAndSetup( numFrames, numPatches, pg2->getTrajectorySet()->getTrajectories()->getDescr() );
	}
	ts->setStartFrameIndex( pg1->getStartFrameIndex() );
	ts->clearAndSetup( trajectories );
	InCorePCTopologyGraph* tg = new InCorePCTopologyGraph();
	PatchGraph* combinedPG = new PatchGraph();
	combinedPG->clearAndSetup( ts, tg, false );

	createCombinedTrajectorySet(ts, tg, pg1, pg2);
	
	PatchGraph *result = new PatchGraph();
	result->clearAndSetup(ts, tg, false);
	return result;
}
