//---------------------------------------------------------------------------
#ifndef PatchGraphMerge2H
#define PatchGraphMerge2H
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PatchGraphJoin2;
class PairwiseAlignment2;
class TopologyMerging2;
class HoleFilling2;
class PatchGraphResampler2;
class GlobalOptimization2;
class AnimationBSRContext;
class PatchGraphContainer;
class DebugRenderer;
class AnimationBSRRenderer;
class PatchGraph;
class PointCloudAnimationSequence;


///
///   This is version 2.0 of the patch graph merging routine
///
class ANIMATIONRECONSTRUCTION_API PatchGraphMerge2 : public Persistent {
	GEOXL_CLASS( PatchGraphMerge2 )
 private:
   PatchGraphJoin2      *joinParts;
   PairwiseAlignment2   *alignParts;
   TopologyMerging2     *mergeAlignedRegion;
   HoleFilling2         *fillHoles;
   PatchGraphResampler2 *resampler;
   GlobalOptimization2  *globalOpt;

   card32 debugStopEarly;   

 public:
	PatchGraphMerge2();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

	virtual void mergeTwoFrameRanges(
		AnimationBSRContext *context, 
		const int32 pgIndex1, 
		const int32 pgIndex2, 
      PatchGraphContainer *container,
		DebugRenderer *debugRenderer,
		AnimationBSRRenderer* renderer);

	virtual void performGlobalOptOnly(
		AnimationBSRContext *context, 
		PatchGraph *pg,
		PointCloudAnimationSequence *data,
		DebugRenderer *debugRenderer,
		AnimationBSRRenderer* renderer);

   virtual void recomputeTopologyOnly(
      AnimationBSRContext *context,
      PatchGraph *pg);

   virtual void resampleOnly(
      AnimationBSRContext *context,
      PatchGraph *pg);

   ~PatchGraphMerge2();
};

}

#endif