//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "TopologyMerging2.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PatchGraph.h"
#include "TrajectorySet.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "PatchGraphOpt2.h"
#include "PGOpt2Potential.h"
#include "AnimationBSRContext.h"
#include "PatchTypes.h"
#include "RecomputeTopology2.h"
//#include "PGOpt2RemoveLowDegreeVertices.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( TopologyMerging2 ,0) {
	BEGIN_CLASS_INIT( TopologyMerging2 );
}

// ----

TopologyMerging2::TopologyMerging2() {
}

void TopologyMerging2::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

TopologyMerging2::~TopologyMerging2() {
}

void TopologyMerging2::writeDeformed(PatchGraph *combinedPG, card32 wLeft, card32 hLeft, bool useLeft, const PointSet *deformed) {
   combinedPG->createPatchAttribute(mVAD("topology_depth", 1, VAD::DATA_FORMAT_INT32));
   combinedPG->createPatchAttribute(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
   PointSet *ps = combinedPG->getTrajectorySet()->getTrajectories();
   const card32 upperMiddle = wLeft + (useLeft?1:0);
   const card32 lowerMiddle = wLeft - (useLeft?0:1);
   const card32 width = ps->getWidth();
   const card32 height = ps->getHeight();
   AAT aatFlags = ps->getAAT("flags");
   AAT aatTopologyDepth = ps->getAAT("topology_depth");
   
   if (useLeft) {
      for (card32 y=0; y<hLeft; y++) {
         ps->setVertex(wLeft, y, deformed->getDataPointer(0, y));
      }
   } else {
      for (card32 y=hLeft; y<height; y++) {
         ps->setVertex(wLeft-1, y, deformed->getDataPointer(0, y-hLeft));
      }
   }
   
   for (card32 y=0; y<hLeft; y++) {
      for (card32 x=0; x<upperMiddle; x++) {
         ps->set<int>(x,y, aatFlags, ps->get<int>(x,y, aatFlags) | FLAG_PATCH_INITIALIZED);
         ps->set<int>(x,y, aatTopologyDepth, 0);
      }
   }
   for (card32 y=0; y<hLeft; y++) {
      for (card32 x=upperMiddle; x<width; x++) {
         ps->set<int>(x,y, aatFlags, 0);
         ps->set<int>(x,y, aatTopologyDepth, 2);
      }
   }
   for (card32 y=hLeft; y<height; y++) {
      for (card32 x=0; x<lowerMiddle; x++) {
         ps->set<int>(x,y, aatFlags, 0);
         ps->set<int>(x,y, aatTopologyDepth, 2);
      }
   }
   for (card32 y=hLeft; y<height; y++) {
      for (card32 x=lowerMiddle; x<width; x++) {
         ps->set<int>(x,y, aatFlags, ps->get<int>(x,y, aatFlags) | FLAG_PATCH_INITIALIZED);
         ps->set<int>(x,y, aatTopologyDepth, 0);
      }
   }
}


card32 TopologyMerging2::merge(
   AnimationBSRContext *context, PatchGraph *combinedPG, 
   const PatchGraph *upperLeft, const PatchGraph *lowerRight, 
   const PointSet *deformedLeft,  OptimizationRecord2 *deformLeftRecord,
   const PointSet *deformedRight, OptimizationRecord2 *deformRightRecord)
{
   card32 result = 0xFFFFFFFF;
   float32 leftDeformEnergy = 0;  float32 leftSum = 0;  float32 leftDistE = 0;
   float32 rightDeformEnergy = 0; float32 rightSum = 0; float32 rightDistE = 0;
   for (unsigned i=0; i<deformLeftRecord->energies.size(); i++) {
      float32 e = deformLeftRecord->energies[i].energy;
      if (deformLeftRecord->energies[i].potentialType->inheritsFrom(PGOpt2RigidityPotential::getClass())) {
         leftDeformEnergy = e;
      }
      if (deformLeftRecord->energies[i].potentialType->inheritsFrom(PGOpt2SquaredDistancePotential_Pairwise::getClass())) {
         leftDistE = e;
      }
      leftSum += e;
      if (context->getVerbose()) {
         debugOutput << "left " << deformLeftRecord->energies[i].potentialType->getClassName() << ": " << e << "\n";
      }
   }
   for (unsigned i=0; i<deformRightRecord->energies.size(); i++) {
      float32 e = deformRightRecord->energies[i].energy;;
      if (deformRightRecord->energies[i].potentialType->inheritsFrom(PGOpt2RigidityPotential::getClass())) {
         rightDeformEnergy = e;
      }
      if (deformLeftRecord->energies[i].potentialType->inheritsFrom(PGOpt2SquaredDistancePotential_Pairwise::getClass())) {
         rightDistE = e;
      }
      rightSum += e;
      if (context->getVerbose()) {
         debugOutput << "right " << deformLeftRecord->energies[i].potentialType->getClassName() << ": " << e << "\n";
      }
   }

   bool useLeft;
   if (context->getTopology2MergeSelCriterion() == AnimationBSRContext::MERGE_MIN_TOTAL_ENERGY) {
      useLeft = leftSum < rightSum;
   } if (context->getTopology2MergeSelCriterion() == AnimationBSRContext::MERGE_MIN_DEFORM_ENERGY) {
      useLeft = leftDeformEnergy < rightDeformEnergy;
   } else { // MERGE_MIN_DEFORM_PLUS_DISTANCE_ENERGY
      useLeft = ((leftDeformEnergy + leftDistE) < (rightDeformEnergy + rightDistE));
   }

   if (context->getVerbose()) {
      debugOutput << "leftsum " << leftSum << "\n";
      debugOutput << "rightsum " << rightSum << "\n";
   }
 
   pAssert(upperLeft->getTrajectorySet()->getTrajectories()->getHeight() == deformedLeft->getHeight());
   writeDeformed(combinedPG, upperLeft->getTrajectorySet()->getTrajectories()->getWidth(), deformedLeft->getHeight(), useLeft, useLeft?deformedLeft:deformedRight);
   combinedPG->getTrajectorySet()->invalidateAllAttachments();

   RecomputeTopology2 recTop;
   recTop.setupStandardFilterConfig(RecomputeTopology2::FILTER_INIT_LENGTH_DEV);
   InCorePCTopologyGraph *newTop = recTop.recomputeTopology(context, combinedPG);
   combinedPG->setTopology(newTop);
   //PGOpt2RemoveLowDegreeVertices::process(context, combinedPG);
   
   if (useLeft) {
      return MERGED_LEFT;
   } else {
      return MERGED_RIGHT;
   }
}

void TopologyMerging2::recomputeTopology(AnimationBSRContext *context, PatchGraph *combinedPG) 
{
   RecomputeTopology2 recTop;
   recTop.setupStandardFilterConfig(RecomputeTopology2::FILTER_INIT_LENGTH_DEV);
   InCorePCTopologyGraph *newTop = recTop.recomputeTopology(context, combinedPG);
   combinedPG->setTopology(newTop);
   //PGOpt2RemoveLowDegreeVertices::process(context, combinedPG);
}
