//---------------------------------------------------------------------------
#ifndef TopologyMerging2H
#define TopologyMerging2H
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PatchGraph;
class PointSet;
struct OptimizationRecord2;
class AnimationBSRContext;

///
///   merge topology of two patch graphs that have been aligned geometrically
///
class ANIMATIONRECONSTRUCTION_API TopologyMerging2 : public Persistent {
	GEOXL_CLASS( TopologyMerging2 )
 private:

   void writeDeformed(PatchGraph *combinedPG, card32 wLeft, card32 hLeft, bool useLeft, const PointSet *deformed);

 public:
	TopologyMerging2();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   static const card32 MERGED_LEFT = 0;
   static const card32 MERGED_RIGHT = 1;

   /// merges the topology of two aligned patch graphs.
   /// "combinedPG" is expected to consist of two patch graphs, one in the upper left and one in the lower right portion
   /// everything else beeing uninitialized (zero entries)
   /// "deformedLeft" is the rightmost frame of "upperLeft" (the upper left half of "combinedPG"), deformed to align with the leftmost frame of lowerRight
   /// "deformedRight" is the same but with for the leftmost frame from lowerRight
   /// output is by modifying the topology graph of combinedPG to stitch together the two parts
   /// returns MERGED_LEFT if decision was made to combine deformedLeft with combinedPG
   ///         MERGED_RIGHT otherwise (deformedRight has been taken)
   virtual card32 merge(AnimationBSRContext *context,
                        PatchGraph *combinedPG, const PatchGraph *upperLeft, const PatchGraph *lowerRight, 
                        const PointSet *deformedLeft,  OptimizationRecord2 *deformLeftRecord,
                        const PointSet *deformedRight, OptimizationRecord2 *deformRightRecord);

   /// recompute topology (again) - this is the last step in "merge" (called after reverse alignment)
   virtual void recomputeTopology(AnimationBSRContext *context, PatchGraph *combinedPG);

   ~TopologyMerging2();
};

}

#endif