//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "GridBasedSphericalRangeQuery.h"
//---------------------------------------------------------------------------
#include "GridQuery_templCode.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



GridBasedSphericalRangeQuery::GridBasedSphericalRangeQuery(const PointSet *ps, float32 queryRadius)
: query(queryRadius*2, ps->getBoundingBox().lowerCorner) 
{
	this->ps = ps;
	this->queryRadius = queryRadius;
	this->queryRadiusQuad = queryRadius*queryRadius;
	if (ps == NULL) throw PException("GridBasedSphericalRangeQuery - ps must not be NULL.");
   this->positionAAT= ps->getAAT("position", 3, VAD::DATA_FORMAT_FLOAT32);

   const card32 numPts = ps->getNumEntries();
   for (card32 i=0; i<numPts; i++) {
      Vector3f pos = ps->get3f(i, positionAAT);
      query.addPoint(query.getCell(pos), i);
   }
}

void GridBasedSphericalRangeQuery::getIndices(const Vector3f &targetPoint, vector<int32> &result) {
   result.clear();
   Vector3i initialCell = query.getCell(targetPoint);
   Vector3f initialCellCenter = query.getCellCenter(initialCell);
   Vector3i start, end;
   for (card32 i=0; i<3; i++) {
      if (targetPoint[i] > initialCellCenter[i]) {
         start[i] = 0;
         end[i] = 1;
      } else {
         start[i] = -1;
         end[i] = 0;
      }
   }
   Vector3i gridPos;
   for (gridPos[2]=start[2]+initialCell[2]; gridPos[2]<=end[2]+initialCell[2]; gridPos[2]++) {
      for (gridPos[1]=start[1]+initialCell[1]; gridPos[1]<=end[1]+initialCell[1]; gridPos[1]++) {
         for (gridPos[0]=start[0]+initialCell[0]; gridPos[0]<=end[0]+initialCell[0]; gridPos[0]++) {
            const vector<int32> *cellResult = query.query(gridPos);
            if (cellResult) {
               const size_t nCellR = cellResult->size();
               for (unsigned i=0; i<nCellR; i++) {
                  card32 newIndex = (*cellResult)[i];
                  Vector3f pos = ps->get3f(newIndex, positionAAT);
                  if (normQuad(pos-targetPoint) <= queryRadiusQuad) {
                     result.push_back(newIndex);
                  }
               }
            }
         }
      }
   }
}



