//---------------------------------------------------------------------------
#ifndef GridQueryH
#define GridQueryH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "SparseGrid.h"
#include "BoundingBox.h"
//---------------------------------------------------------------------------

namespace geoxl {

///
///   stores lists of objects in sparse grids, addressed by spatial position
///
template<typename FloatType, typename IntType, unsigned dimension, typename EntryType>
class ANIMATIONRECONSTRUCTION_API GridQuery {
 private:
   class NullClass {};
   typedef vector<EntryType> EntryList;
   typedef EntryList* EntryListPtr;
   SparseGrid< dimension, EntryListPtr, NullClass > grid;
   FloatType cellSize;
   StaticVector<FloatType, dimension> offset;
   
 public:
	GridQuery(FloatType cellSize, const StaticVector<FloatType, dimension> &offset);

   StaticVector<IntType, dimension> getCell(const StaticVector<FloatType, dimension> &pos);
   inline void addPoint(const StaticVector<IntType, dimension> &cell, const EntryType &entry);
   inline void query(const StaticVector<IntType, dimension> &cell, vector<EntryType> &addTo);
   inline const vector<EntryType>* query(const StaticVector<IntType, dimension> &cell);
   BoundingBox<FloatType, dimension> getBoundingBox(const StaticVector<IntType, dimension> &cell);
   StaticVector<FloatType, dimension> getCellCenter(const StaticVector<IntType, dimension> &cell);

   ~GridQuery();
};

}

#endif