//---------------------------------------------------------------------------
#include "GridQuery.h"
//---------------------------------------------------------------------------
#include "SparseGrid_templCode.h"
//---------------------------------------------------------------------------

namespace geoxl {


template<typename FloatType, typename IntType, unsigned dimension, typename EntryType>
GridQuery<FloatType, IntType, dimension, EntryType>::GridQuery(FloatType cellSize, const StaticVector<FloatType, dimension> &offset) {
   this->cellSize = cellSize;
   this->offset = offset;
}

template<typename FloatType, typename IntType, unsigned dimension, typename EntryType>
StaticVector<IntType, dimension> GridQuery<FloatType, IntType, dimension, EntryType>::getCell(const StaticVector<FloatType, dimension> &pos) {
   StaticVector<IntType, dimension> result;
   for (unsigned i=0; i<dimension; i++) {
      result[i] = (IntType)(pos[i]/cellSize - offset[i]);
   }
   return result;
}

template<typename FloatType, typename IntType, unsigned dimension, typename EntryType>
void GridQuery<FloatType, IntType, dimension, EntryType>::addPoint(const StaticVector<IntType, dimension> &cell, const EntryType &entry) {
   EntryListPtr e;
   if (!grid.getObject(cell, e)) {
      e = new EntryList;
      grid.setObject(cell, e);
   }
   e->push_back(entry);
}

template<typename FloatType, typename IntType, unsigned dimension, typename EntryType>
void GridQuery<FloatType, IntType, dimension, EntryType>::query(const StaticVector<IntType, dimension> &cell, vector<EntryType> &addTo) {
   EntryListPtr e;
   if (grid.getObject(cell, e)) {
      EntryList::iterator pos = e->begin();
      EntryList::iterator end = e->end();
      while (pos != end) {
         addTo.push_back(*pos);
      }
   }
}

template<typename FloatType, typename IntType, unsigned dimension, typename EntryType>
const vector<EntryType> *GridQuery<FloatType, IntType, dimension, EntryType>::query(const StaticVector<IntType, dimension> &cell) {
   EntryListPtr e;
   if (grid.getObject(cell, e)) {
      return e;
   } else {
      return NULL;
   }
}


template<typename FloatType, typename IntType, unsigned dimension, typename EntryType>
BoundingBox<FloatType, dimension> GridQuery<FloatType, IntType, dimension, EntryType>::getBoundingBox(const StaticVector<IntType, dimension> &cell) {
   BoundingBox<FloatType, dimension> result;
   for (unsigned i=0; i<dimension; i++) {
      result.lowerCorner[i] = (cell[i] + offset[i])*cellSize;
      result.upperCorner[i] = result.lowerCorner[i] + cellSize;
   }
   return result;
}

template<typename FloatType, typename IntType, unsigned dimension, typename EntryType>
StaticVector<FloatType, dimension> GridQuery<FloatType, IntType, dimension, EntryType>::getCellCenter(const StaticVector<IntType, dimension> &cell) {
   StaticVector<FloatType, dimension> result;
   for (unsigned i=0; i<dimension; i++) {
      result[i] = (cell[i] + offset[i])*cellSize+cellSize/2;
   }
   return result;
}


template<typename FloatType, typename IntType, unsigned dimension, typename EntryType>
GridQuery<FloatType, IntType, dimension, EntryType>::~GridQuery() {
   mpcard n = grid.getNumElements();
   for (mpcard i=0; i<n; i++) {
      delete grid.getEntry(i);
   }
}


}