// Magic Software, Inc.
// http://www.magic-software.com
// Copyright (c) 2000, 2001.  All Rights Reserved
//
// Source code from Magic Software is supplied under the terms of a license
// agreement and may not be copied or disclosed except in accordance with the
// terms of that agreement.  The various license agreements may be found at
// the Magic Software web site.  This file is subject to the license
//
// FREE SOURCE CODE
// http://www.magic-software.com/License/free.pdf

#ifndef MGCEIGEN_H
#define MGCEIGEN_H

namespace geoxl {

template<class FloatType>
class Eigen
{
public:
    inline Eigen<FloatType> (int iSize);
    inline ~Eigen<FloatType> ();

    // set the matrix for eigensolving
    inline FloatType& Matrix (int iRow, int iCol);
    inline void SetMatrix (FloatType** aafMat);

    // get the results of eigensolving (eigenvectors are columns of matrix)
    inline FloatType GetEigenvalue (int i) const;
    inline FloatType GetEigenvector (int iRow, int iCol) const;
    inline FloatType* GetEigenvalue ();
    inline FloatType** GetEigenvector ();

    // solve eigensystem
    inline void EigenStuff2 ();
    inline void EigenStuff3 ();
    inline void EigenStuff4 ();
    inline void EigenStuffN ();
    inline void EigenStuff  ();

    // solve eigensystem, use decreasing sort on eigenvalues
    inline void DecrSortEigenStuff2 ();
    inline void DecrSortEigenStuff3 ();
    inline void DecrSortEigenStuff4 ();
    inline void DecrSortEigenStuffN ();
    inline void DecrSortEigenStuff  ();

    // solve eigensystem, use increasing sort on eigenvalues
    inline void IncrSortEigenStuff2 ();
    inline void IncrSortEigenStuff3 ();
    inline void IncrSortEigenStuff4 ();
    inline void IncrSortEigenStuffN ();
    inline void IncrSortEigenStuff  ();

protected:
    int			m_iSize;
    FloatType** m_aafMat;
    FloatType*	m_afDiag;
    FloatType*	m_afSubd;

    // Householder reduction to tridiagonal form
    inline void Tridiagonal2 (FloatType** aafMat, FloatType* afDiag, FloatType* afSubd);
    inline void Tridiagonal3 (FloatType** aafMat, FloatType* afDiag, FloatType* afSubd);
    inline void Tridiagonal4 (FloatType** aafMat, FloatType* afDiag, FloatType* afSubd);
    inline void TridiagonalN (int iSize, FloatType** aafMat, FloatType* afDiag,
        FloatType* afSubd);

    // QL algorithm with implicit shifting, applies to tridiagonal matrices
    inline bool QLAlgorithm (int iSize, FloatType* afDiag, FloatType* afSubd,
        FloatType** aafMat);

    // sort eigenvalues from largest to smallest
    inline void DecreasingSort (int iSize, FloatType* afEigval,
        FloatType** aafEigvec);

    // sort eigenvalues from smallest to largest
    inline void IncreasingSort (int iSize, FloatType* afEigval,
        FloatType** aafEigvec);
};

} // namespace Mgc

#include "MgcEigen_templCode.h"

#endif
