#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "MultiResolutionPC.h"
//---------------------------------------------------------------------------
#include "PointCloudTools.h"
#include "Commands/PCCResampler.h"
#include "StandardSceneGraphIterator.h"
#include "HierarchicalKNNIterator.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( MultiResolutionPC ,0)
{
	BEGIN_CLASS_INIT( MultiResolutionPC );
}

MultiResolutionPC::MultiResolutionPC(void)
{
}

MultiResolutionPC::~MultiResolutionPC(void)
{
	for( card32 i=1;i<m_PointClouds.size();i++ )
		delete m_PointClouds[i].pc;
}

void MultiResolutionPC::assign(const Object* obj, CopyContext *context)
{
	//Persistent::assign(obj,context);
	const MultiResolutionPC * o = dynamic_cast<const MultiResolutionPC*>(obj);
	pAssert( o != NULL );

	m_PointClouds = o->m_PointClouds;


}
void MultiResolutionPC::setup( PointCloud * pc, card32 minNumberOfPoints, float distanceFactor,
							  float sigma) 
{
	pAssert( m_PointClouds.size() == 0 );
	float medPointDistance = getMedianPointDistance( pc );
	PointCloud * actPC = pc;
	while( true )
	{
		// dublicate point cloud		
		PointCloud * newPC = (PointCloud*)actPC->copy();

		// resample
		PCCResampler::resamplePC(newPC,medPointDistance*pow(distanceFactor,(float)m_PointClouds.size() + 1));
		TPointCloud tpc;
		tpc.pc = newPC;
		tpc.averagePointDistance = getMedianPointDistance(newPC);
		m_PointClouds.push_back(tpc);

		// smooth attributes with gaussian filter
		/*SceneObjectIterator * iterNew = newPC->createIterator( SceneObjectIteratorType::CAP_BASIC_PC|
		SceneObjectIteratorType::CAP_MODIFYABLE_PC);
		BasicPointCloudIterator * bItNew = dynamic_cast<BasicPointCloudIterator*>(iterNew);
		ModifyablePointCloudIterator *mItNew = dynamic_cast<ModifyablePointCloudIterator*>(iterNew);
		AAT posAAT = newPC->getAAT( "position" );
		AAT normalAAT = newPC->getAAT( "normal" );
		AAT colorAAT = newPC->getAAT( "color" );

		float h_2 = pow(averagePointDistance * sigma,2.0f);

		while( !bItNew->atEnd() )
		{
		Vector3f pos = bItNew->get3f(posAAT);
		hIt->setSeekPointAndReset( pos );

		Vector3f color = makeVector3f(0,0,0);
		Vector3f normal = makeVector3f(0,0,0);
		float summed_weight = 0.0f;

		while( !hIt->atEnd() )
		{
		Vector3f nPos = hIt->get3f( posAAT );
		float distance = norm( nPos - pos );
		float weight = exp( -pow(distance,2.0f) / h_2 );
		summed_weight += weight;
		color += hIt->get3f(colorAAT) * weight;
		normal += hIt->get3f(normalAAT) * weight;

		if( weight < 0.01f )
		break;

		hIt->next();
		}

		normal = normalize( normal );
		if( summed_weight > 0.0f )
		color = color / summed_weight;
		mItNew->set3f( normalAAT, normal );
		mItNew->set3f( colorAAT, color );

		bItNew->next();
		}


		delete iter;
		delete hIt;
		delete iterNew;*/

		if( newPC->getNumPoints() <= minNumberOfPoints )
			break;
	}
}

PointCloud *MultiResolutionPC::getPointCloud( card32 i ) 
{
	pAssert( i < m_PointClouds.size() );
	if( i < m_PointClouds.size() )
		return m_PointClouds[i].pc;
	else
		return NULL;
}

float MultiResolutionPC::getPCAveragePointDistance( card32 i ) 
{
	pAssert( i < m_PointClouds.size() );
	if( i < m_PointClouds.size() )
		return m_PointClouds[i].averagePointDistance;
	else
		throw "Out of Range!";
	return 0;
}

PointCloud *MultiResolutionPC::getClosestPointCloud( float averagePointDistance ) 
{
	pAssert( m_PointClouds.size() > 0 );
	if( m_PointClouds.size() == 0 )
		return NULL;

	for( card32 i=0;i<m_PointClouds.size();i++ )
	{
		size_t index = m_PointClouds.size() - i - 1;
		if( m_PointClouds[index].averagePointDistance < averagePointDistance )
		{
			return m_PointClouds[index].pc;
		}
	}
	return m_PointClouds[0].pc;
}
