#ifndef TensorVotingH
#define TensorVotingH

#include "Persistent.h"
#include "UnstructuredInCorePointCloud.h"
#include "AttachedIndexedOctree.h"
#include "PointCloudProcessingDefinitions.h"

namespace geoxl
{
/*
 * Provide functionality to apply tensor voting using different tensor voting fields
 *
 * Required: 
 * - position        (3f)
 *
 * Output:
 * - surfaceness     (1f)
 * 
 * Optional
 * - samplingSpacing (1f) SamplingSpacingEstimator
 * - tensor          (6f) Output
*/
class ANIMATIONRECONSTRUCTION_API TensorVoting : public Persistent
{	
	GEOXL_CLASS( TensorVoting )
	private:
		
		// Voting: init stick tensors
		static Matrix3f computeTensorVote( 
			const Vector3f& p, 
			const Vector3f& np, 
			const float32 epsilon );

		// Vote with stick tensors
		static bool computeTensorVote( 
			Matrix3f& vote,
			const Matrix3f& receiverEigenVectors,
			const Vector3f& receiverPos,
			const Matrix3f& senderTensor,
			const Vector3f& senderPos,
			const float32 epsilon );

	public:
		TensorVoting();
		~TensorVoting();
		
		// Init a stick tensor field; set surfaceness field and save tensors if wanted
		static void tensorVote( 
			bool initTensors,
			PointCloud* pc, 
			const float32 epsilon, 
			const bool saveTensor = false,
			const card32 numKNN = 0,
			const float32 timesSamplingSpacing = -1,
			const card32 mode = ENVIRONMENT_EPSILON );

		// Init a stick tensor field; set surfaceness field and save tensors if wanted
		static void tensorVote( 
			bool initTensors,
			PointSet* ps, 
			const float32 sigma, 
			const bool saveTensor = false,
			const card32 numKNN = 0,
			const float32 timesSamplingSpacing = -1,
			const card32 mode = ENVIRONMENT_EPSILON,
			bool useCachedRangeQuerries = false);


};

}

#endif