//---------------------------------------------------------------------------
#ifndef UniformPointCloudResamplingH
#define UniformPointCloudResamplingH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PointSet;
class PatchGraph;

/**
	resample point sets to get a uniform sample spacing
	works by point removal: the operation iteratively removes points from the point set
	for each point, the number of points within "maxspacing" is computed
	then, the points are removed from the closest pairs until no point can be removed without 
	reducing coverage to zero at some point
*/
class ANIMATIONRECONSTRUCTION_API UniformPointCloudResampling : public Persistent {
	GEOXL_CLASS( UniformPointCloudResampling )

 private:
	float32 maxSpacing;
	
 public:
	UniformPointCloudResampling();

	/// maximum spacing allowed between points (that have been previously closer to some points to be removed)
	void setup(float32 maxSpacing);
   virtual void assign(const Object* obj, CopyContext *context = NULL);

	virtual PointSet *resamplePointSet(const PointSet *ps);
	// /// uses max spacing criterion over all frames (logical "and")
	// virtual PatchGraph *resamplePatchGraph(const PatchGraph *pg);

	~UniformPointCloudResampling();

};

}

#endif
