//---------------------------------------------------------------------------
#ifndef OptimizationGraphH
#define OptimizationGraphH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PGOpt2Frame;
class PGOpt2FramePair;
class PGOpt2FramePairPair;

///
///  This class describes a set of optimization potentials on a set of frames/patches in patch graphs
///
class ANIMATIONRECONSTRUCTION_API OptimizationGraph : public Persistent {
	GEOXL_CLASS( OptimizationGraph )

 private:
   /// frames to be optimized  (e.g. for spatial smoothness)
   /// smoothness and SDF and marker error
	 vector<PGOpt2Frame*> frames;
   /// edges: pair of frames to be optimized (e.g. for spatio-temporal smoothness)
   /// rigidity
	 vector<PGOpt2FramePair*> edges;
   /// inter-edge constraints: pairs of edges to be considered (e.g. necessary for acceleration)
   /// temporal coherence size = width - 2
	 vector<PGOpt2FramePairPair*> interEdgeLinks;
   
	IMPLEMENT_VAR_ARRAY_METHODS_STL(frames)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(edges)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(interEdgeLinks)

 public:
	OptimizationGraph();


   virtual void assign(const Object* obj, CopyContext *context = NULL);

   
   // - setup ----

   /// add frame to graph, this object takes ownership
   void addFrame(PGOpt2Frame *frame);
   /// add edge to graph, this object takes ownership
   void addEdge(PGOpt2FramePair *pair);
   /// add pairs of edges to graph, this object takes ownership
   void addEdgePair(PGOpt2FramePairPair *edgeLink);

   
   // - access ----

   mpcard getNumFrames()           {return frames.size();}
   PGOpt2Frame *getFrame(card32 i) {return frames[i];}

   mpcard getNumEdges()               {return edges.size();}
   PGOpt2FramePair *getEdge(card32 i) {return edges[i];}
   
   mpcard getNumInterEdgeLinks()                   {return interEdgeLinks.size();}
   PGOpt2FramePairPair *getInterEdgeLink(card32 i) {return interEdgeLinks[i];}

	~OptimizationGraph();
};

}

#endif