//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PGOpt2Debugging.h"
//---------------------------------------------------------------------------
#include "InCorePCTopologyGraph.h"
#include "DebugOutput.h"
#include "PointSet.h"
#include "PatchGraph.h"
#include "TrajectorySet.h"
#include "PatchTypes.h"
#include "ImageDisplayer.h"
#include "DebugOutput.h"
#include "AnimationBSRContext.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



void PGOpt2Debugging::computeTopDegHistogram(const InCorePCTopologyGraph *tg, vector<card32> &result) 
{
   result.clear();
   for (card32 i=0; i<tg->getNumVertices(); i++) {
      card32 deg = tg->getNumAdjacentVertices(i);
      if (result.size() <= deg) {
         result.resize(deg+1, 0);
      }
      result[deg]++;
   }
}

void PGOpt2Debugging::ouputHistogram(const vector<card32> &h) {
   debugOutput << "topology degree histogram:\n";
   for (unsigned i=0; i<h.size(); i++) {
      debugOutput << i << "\t" << h[i] << "\n";
   }
}

void PGOpt2Debugging::outputValidityCheck(const PatchGraph *pg, string name) {
	const PointSet *ps = pg->getTrajectorySet()->getTrajectories();
	const card32 width = ps->getWidth();
	const card32 height = ps->getHeight();
	const card32 sizew = (card32)floor(sqrt((float32)height));
	const card32 sizeh = (card32)ceil((float32)height / (float32)sizew);
	const AAT aatNormal = ps->getAAT("normal");
	const AAT aatTangentU = ps->getAAT("tangent_u");
	const AAT aatFlags = ps->getAAT("flags");
	QImage img(sizew*width, sizeh, QImage::Format_ARGB32);
	for (card32 y=0; y<sizeh; y++) {
		for (card32 x=0; x<sizew*width; x++) {
			img.setPixel(x,y, qRgb(128,100,200));
		}
	}
	for (card32 f=0; f<width; f++) {
		for (card32 p=0; p<height; p++) {
			bool flagValid = (ps->get<int32>(f,p, aatFlags) & FLAG_PATCH_INITIALIZED) != 0;
			bool normalValid = (fabs(1.0f-norm(ps->get<float32, 3>(f,p, aatNormal))) < 0.1f);
			bool tangentUValid = (fabs(1.0f-norm(ps->get<float32, 3>(f,p, aatTangentU))) < 0.1f);
			int x = f*sizew + p%sizew;
			int y = p/sizew;
			img.setPixel(x,y, qRgb(flagValid?0:255, normalValid?0:255, tangentUValid?0:255));
		}
	}
	ImageDisplayer::DisplayQImage disp(&img);
	debugOutput.displayEvent(ImageDisplayer::getClass(), name, &disp);
}

void PGOpt2Debugging::searchLongEdges(const geoxl::AnimationBSRContext *context, const geoxl::PatchGraph *pg) {
   float32 sampleSpacing = context->getSamplingSpacing();
   const card32 numVertices = pg->getTopology()->getNumVertices();
   const InCorePCTopologyGraph *tg = pg->getTopology();
   const PointSet *ps = pg->getTrajectorySet()->getTrajectories();
   const AAT posAAT = ps->getAAT("position");
   const card32 numFrames = ps->getWidth();
   for (card32 v=0; v<numVertices; v++) {
      const card32 numNeighbors = tg->getNumAdjacentVertices(v);
      for (card32 i=0; i<numNeighbors; i++) {
         const card32 ni = tg->getAdjacentVertex(v, i);
         float32 el = 0;
         for (card32 f=0; f<numFrames; f++) {
            float32 l = norm(ps->get3f(f, v, posAAT) - ps->get3f(f, ni, posAAT));
            if (l>el) el = l;
         }
         if (el > 7*sampleSpacing) {
            debugOutput << "longEdge from #" << v << " to #" << ni << "; length: " << el << ".\n";
         }
      }
   }
}
void PGOpt2Debugging::outputGraph(const InCorePCTopologyGraph *tg) 
{
   const card32 numVertices = tg->getNumVertices();
   for (card32 v=0; v<numVertices; v++) {
      debugOutput << v << ".\n";
      const card32 numNeighbors = tg->getNumAdjacentVertices(v);
      vector<card32> ns(numNeighbors);
      for (card32 i=0; i<numNeighbors; i++) {
         const card32 ni = tg->getAdjacentVertex(v, i);
         ns[i] = ni;
      }
      sort(ns.begin(), ns.end());
      for (card32 i=0; i<numNeighbors; i++) {
         debugOutput << "\t" << ns[i] << ".\n";
      }
   }
}

void PGOpt2Debugging::outputPoints(const PatchGraph *pg) 
{
   const PointSet *ps = pg->getTrajectorySet()->getTrajectories();
   const card32 numFrames = ps->getWidth();
   const card32 numPoints = ps->getHeight();
   const AAT posAAT = ps->getAAT("position");
   debugOutput << "\n";
   for (card32 p=0; p<numPoints; p++) {
      for (card32 f=0; f<numFrames; f++) {
         Vector3f pos = ps->get3f(f, p, posAAT);
         debugOutput << pos.toString() << "\t";
      }
      debugOutput << "\n";
   }
}
