//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PGOpt2Frame.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ObjectClassProperty.h"
#include "EmbeddedObjectClassProperty.h"
#include "PointCloudAnimationSequence.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PGOpt2Frame ,1) {
	BEGIN_CLASS_INIT( PGOpt2Frame );
   ADD_OBJECT_PROP(patches, 0, PointSet::getClass(), false)
   ADD_OBJECT_PROP(data, 0, PointCloudAnimationSequence::getClass(), false)
   ADD_CARD32_PROP(patchesFrameIndex, 0)
   ADD_CARD32_PROP(dataFrameIndex, 0)
   ADD_FLOAT32_PROP(smoothnessWeight, 1)
}

// ----

PGOpt2Frame::PGOpt2Frame() {
   patches = NULL;
   data = NULL;
   patchesFrameIndex = 0;
   dataFrameIndex = 0;
   smoothnessWeight = 1;
}

void PGOpt2Frame::setup(
   PointSet *patches,
   AttachedDataContainer *patchAttachments,
   PointCloudAnimationSequence *data,
   card32 patchesFrameIndex,
   card32 dataFrameIndex,
   const vector<card32> &patchIndices)
{
   this->patches = patches;
   this->patchAttachments = patchAttachments;
   this->data = data;
   this->patchesFrameIndex = patchesFrameIndex;
   this->dataFrameIndex = dataFrameIndex;
   this->patchIndices = patchIndices;
}

void PGOpt2Frame::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

void PGOpt2Frame::invalidatePatchAttachments() {
   patchAttachments->deleteDelOnWriteData();
}

AttachedIndexedOctree *PGOpt2Frame::fetchPatchOctree(card32 maxPointsPerNode) {
   AttachedIndexedOctree *octree = dynamic_cast<AttachedIndexedOctree*>(patchAttachments->getData("SimpleOctree"));
   if (octree == NULL) {
		octree = new AttachedIndexedOctree();
		octree->setup("SimpleOctree", AttachedData::ADF_DELETEONUPDATE,  NULL);

	   PointSet *singleFrame = new PointSet();
	   singleFrame->clearAndSetup(1, patches->getHeight(), patches->getDescr());
	   const card32 h = patches->getHeight();
	   for (card32 p=0; p<h; p++) {
		   singleFrame->setVertex(0, p, patches->getDataPointer(patchesFrameIndex, p));
	   }
		octree->octreesetup(singleFrame, maxPointsPerNode);
		patchAttachments->attachData(octree);

		delete singleFrame;
   }
   return octree;
}

AttachedIndexedOctree *PGOpt2Frame::fetchDataFrameOctree(card32 maxPointsPerNode) {
   return data->fetchOctree(dataFrameIndex, maxPointsPerNode);
}
