//---------------------------------------------------------------------------
#ifndef PGOpt2FrameH
#define PGOpt2FrameH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "AttachedIndexedOctree.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PointSet;
class PointCloudAnimationSequence;

///
///   one frame in optimization graph
///
class ANIMATIONRECONSTRUCTION_API PGOpt2Frame : public Persistent {
	GEOXL_CLASS( PGOpt2Frame )
 private:
   PointSet                    *patches;
   AttachedDataContainer       *patchAttachments;
   PointCloudAnimationSequence *data;
   card32                       patchesFrameIndex;
   card32                       dataFrameIndex;
   vector<card32>               patchIndices;
   float32                      smoothnessWeight; // additional weight, multiplied with the context's weight
   map<card32, Vector3f>        correspondences; // additional correspondences from patch index to position

 public:
	PGOpt2Frame();
	
	virtual void assign(const Object* obj, CopyContext *context = NULL);

	/// this function takes references only, none of the objects will be deleted
	void setup(
	   PointSet *patches,
      AttachedDataContainer *patchAttachments,
	   PointCloudAnimationSequence *data,
	   card32 patchesFrameIndex,
	   card32 dataFrameIndex,
	   const vector<card32> &patchIndices
	);

   // --- member access

	PointSet* getPatches() const {return patches;}
	void setPatches(PointSet *v) {patches=v;}

	AttachedDataContainer *getPatchAttachments() const {return patchAttachments;}
	void setPatchAttachments(AttachedDataContainer *v) {patchAttachments=v;}

	PointCloudAnimationSequence *getData() const {return data;}
	void setData(PointCloudAnimationSequence *v) {data=v;}

	card32 getPatchesFrameIndex() const {return patchesFrameIndex;}
	void setPatchesFrameIndex(card32 v) {patchesFrameIndex=v;}

	card32 getDataFrameIndex() const {return dataFrameIndex;}
	void setDataFrameIndex(card32 v) {dataFrameIndex=v;}
	
	const vector<card32> &getPatchIndices() {return patchIndices;}
	void setPatchIndices(const vector<card32> &pi) {patchIndices = pi;}
	
	float32 getSmoothnessWeight() const {return smoothnessWeight;}
	void setSmoothnessWeight(float32 v) {smoothnessWeight=v;}

	map<card32, Vector3f> getCorrespondences() const {return correspondences;}
	void setCorrespondences(const map<card32, Vector3f> &v) {correspondences=v;}

	void invalidatePatchAttachments();
	AttachedIndexedOctree *fetchPatchOctree(card32 maxPointsPerNode = 20);	
	AttachedIndexedOctree *fetchDataFrameOctree(card32 maxPointsPerNode = 20);	
};

}

#endif