//---------------------------------------------------------------------------
#ifndef PGOpt2FramePairH
#define PGOpt2FramePairH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "PGOpt2PotentialSelector.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;

class VertexDescriptor;


///
///   optimization between a pair of frames and a subset of their points
///
class ANIMATIONRECONSTRUCTION_API PGOpt2FramePair : public Persistent {
	GEOXL_CLASS( PGOpt2FramePair )
 private:
   /// index of first frame in list of frames (list is global to this object)
   card32 optFrameIndex1;
   /// index of second frame in list of frames (list is global to this object)
   card32 optFrameIndex2;
   /// index of patches that should be included in optimization
   vector<card32> patchIndices;
   /// additional attributes for the optimized patch pairs
   PointSet *patchPairsAttribs;

   /// additional weight for this edge's rigidity (multiplied with the weight set in context)
   float32 rigidityWeight;

	IMPLEMENT_VAR_ARRAY_METHODS_STL(patchIndices)

 public:
	PGOpt2FramePair();
   
   void setup(
      card32 optFrameIndex1,
      card32 optFrameIndex2,
      const	 vector<card32> &patchIndices,
      PointSet *patchPairsAttribs
   );

	virtual void assign(const Object* obj, CopyContext *context = NULL);


   // - member access ----
   
	card32 getOptFrameIndex1() const {return optFrameIndex1;}
	void setOptFrameIndex1(card32 v) {optFrameIndex1=v;}

	card32 getOptFrameIndex2() const {return optFrameIndex2;}
	void setOptFrameIndex2(card32 v) {optFrameIndex2=v;}

	vector<card32> getPatchIndices() const {return patchIndices;}
	void setPatchIndices(const vector<card32> &v) {patchIndices=v;}

   PointSet *getPatchPairsAttribs() {return patchPairsAttribs;}

	float32 getRigidityWeight() const {return rigidityWeight;}
	void setRigidityWeight(float32 v) {rigidityWeight=v;}

   void swapFrames() {swap(optFrameIndex1, optFrameIndex2);}

   ~PGOpt2FramePair();
};


}

#endif