//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PGOpt2GraphCreator.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "OptimizationGraph.h"
#include "PGOpt2Frame.h"
#include "PGOpt2FramePair.h"
#include "PGOpt2FramePairPair.h"
#include "PatchGraph.h"
#include "TrajectorySet.h"
#include "PointCloudAnimationSequence.h"
#include "PGOpt2FramePairVars.h"
#include "DynamicLinearAlgebra_templCode.h"
#include "PointSet_templCode.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PGOpt2GraphCreator ,0) {
	BEGIN_CLASS_INIT( PGOpt2GraphCreator );
}

// ----

PGOpt2GraphCreator::PGOpt2GraphCreator() {}

void PGOpt2GraphCreator::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

PGOpt2GraphCreator::~PGOpt2GraphCreator() {}



PGOpt2FramePairVars *PGOpt2GraphCreator::createPairVars(
	const PatchGraph *pg, 
	bool cyclic, 
	bool urshape, 
	bool urshapeToScene, 
	bool sceneToUrshape,
	bool doubleInterFrameLinks,
	bool omitInterFrameLinks)
{
   card32 numFrames = pg->getTrajectorySet()->getNumFrames();
   card32 resultNumFrames = 0;
   if (!omitInterFrameLinks) resultNumFrames += numFrames-1;
   if (!omitInterFrameLinks && cyclic) resultNumFrames++;
   if (doubleInterFrameLinks) resultNumFrames *= 2;
   if (urshape && urshapeToScene) resultNumFrames += numFrames;
   if (urshape && sceneToUrshape) resultNumFrames += numFrames;
   PGOpt2FramePairVars *result = new PGOpt2FramePairVars();
   VertexDescriptor vd;
   vd.pushAttrib(mVAD("rotation_matrix", 4, VAD::DATA_FORMAT_FLOAT32));
   vd.pushAttrib(mVAD("varIndex_rot", 1, VAD::DATA_FORMAT_INT32));
   result->setup(resultNumFrames, pg->getTrajectorySet()->getNumPatches(), &vd);
   return result;
}


OptimizationGraph *PGOpt2GraphCreator::createOptGraph(
   PatchGraph *pg, 
   PointCloudAnimationSequence *data, 
   PGOpt2FramePairVars *pairVars, 
   bool cyclic,
   bool urshape,
   bool urshapeToScene, 
   bool sceneToUrshape,
   bool doubleInterFrameLinks,
   const vector< vector<card32> > *indicesInFrames,
   const vector< vector<card32> > *indicesInEdges,
   bool omitFramePatches,
   bool omitInterFrameLinks,
   card32 minVertexDegree)
{
   const card32 numFrames = pg->getTrajectorySet()->getNumFrames();
   vector<card32> allUsefulPatchIndices;
   vector< vector<card32> > selectedUsefulPatchIndices(numFrames);
   vector< vector<card32> > selectedUsefulEdgeIndices(numFrames);
   const InCorePCTopologyGraph *tg = pg->getTopology();
   const card32 numPatches = tg->getNumVertices();
   pAssert(numPatches == pg->getTrajectorySet()->getNumPatches());

   for (card32 p=0; p<numPatches; p++) {
      if (tg->getNumAdjacentVertices(p) >= minVertexDegree) {
         allUsefulPatchIndices.push_back(p);
      }
   }

   for (card32 f=0; f<pg->getTrajectorySet()->getNumFrames(); f++) {

      if (indicesInFrames) {
         pAssert(numFrames == (*indicesInFrames).size());
         for (unsigned i=0; i<(*indicesInFrames)[f].size(); i++) {
            const card32 p = (*indicesInFrames)[f][i];
            if (tg->getNumAdjacentVertices(p) >= minVertexDegree) {
               selectedUsefulPatchIndices[f].push_back(p);
            }
         }
      } else {
         selectedUsefulPatchIndices[f] = allUsefulPatchIndices;
      }
      if (indicesInEdges) {
         pAssert(numFrames == (*indicesInEdges).size());
         for (unsigned i=0; i<(*indicesInEdges)[f].size(); i++) {
            const card32 p = (*indicesInEdges)[f][i];
            if (tg->getNumAdjacentVertices(p) >= minVertexDegree) {
               selectedUsefulEdgeIndices[f].push_back(p);
            }
         }
      } else {
         selectedUsefulEdgeIndices[f] = allUsefulPatchIndices;
      }
   
   }
      
   OptimizationGraph *result = new OptimizationGraph;
   card32 frameVarIndex = 0;
   vector<card32> emptyIndices;
   for (card32 f=0; f<pg->getTrajectorySet()->getNumFrames(); f++) {
	   PGOpt2Frame *frame = new PGOpt2Frame();
	   frame->setup(
		   pg->getTrajectorySet()->getTrajectories(),
		   pg->getTrajectorySet()->getAttachments(f),
		   data,
		   f,
		   pg->getStartFrameIndex() + f,
		   omitFramePatches?emptyIndices:(selectedUsefulPatchIndices[f])
		   );
	   result->addFrame(frame);
	   
      if (!omitInterFrameLinks) {
	      if (f != pg->getTrajectorySet()->getNumFrames()-1) {
		      PGOpt2FramePair *pair = new PGOpt2FramePair();
		      pair->setup(f, f+1, selectedUsefulEdgeIndices[f], pairVars->getFramePairVars(frameVarIndex));
		      frameVarIndex++;
		      result->addEdge(pair);
		      if (doubleInterFrameLinks) {
	            PGOpt2FramePair *pairBack = new PGOpt2FramePair();
	            pairBack->setup(f+1, f, selectedUsefulEdgeIndices[f], pairVars->getFramePairVars(frameVarIndex));
	            frameVarIndex++;
	            result->addEdge(pairBack);
		      }
	      } else {
		      if (cyclic) {
			      PGOpt2FramePair *pair = new PGOpt2FramePair();
			      pair->setup(f, 0, selectedUsefulEdgeIndices[f], pairVars->getFramePairVars(frameVarIndex));
			      frameVarIndex++;
			      result->addEdge(pair);
		         if (doubleInterFrameLinks) {
	               PGOpt2FramePair *pairBack = new PGOpt2FramePair();
	               pairBack->setup(0, f, selectedUsefulEdgeIndices[f], pairVars->getFramePairVars(frameVarIndex));
	               frameVarIndex++;
	               result->addEdge(pairBack);
		         }
		      }
	      }

	      if (f < pg->getTrajectorySet()->getNumFrames()-2) {
	         if (doubleInterFrameLinks) {
		         PGOpt2FramePairPair *pairPair = new PGOpt2FramePairPair(f*2, (f+1)*2, selectedUsefulPatchIndices[f]);
	            result->addEdgePair(pairPair);
		         PGOpt2FramePairPair *pairPairBack = new PGOpt2FramePairPair((f+1)*2+1, f*2+1, selectedUsefulPatchIndices[f]);
	            result->addEdgePair(pairPairBack);
            } else {
		         PGOpt2FramePairPair *pairPair = new PGOpt2FramePairPair(f, f+1, selectedUsefulPatchIndices[f]);
	            result->addEdgePair(pairPair);
	         }
	      }      
      }
   }
   if (urshape) {
      card32 urshapeFrameIndex = result->getNumFrames();
      PGOpt2Frame *frame = new PGOpt2Frame();
      frame->setup(
         pg->getUrshape(),
         pg->getUrshapeAttachments(),
         NULL,
         0,
         0,
         allUsefulPatchIndices
      );
      result->addFrame(frame);
      if (sceneToUrshape) {
         card32 nextFrameIndex = result->getNumFrames();
         for (card32 i=0; i<pg->getTrajectorySet()->getNumFrames(); i++) {
            PGOpt2FramePair *pair = new PGOpt2FramePair();
            pair->setup(i, urshapeFrameIndex, allUsefulPatchIndices, pairVars->getFramePairVars(frameVarIndex));
            frameVarIndex++;
            result->addEdge(pair);
         }
      }
      if (urshapeToScene) {
         card32 nextFrameIndex = result->getNumFrames();
         for (card32 i=0; i<pg->getTrajectorySet()->getNumFrames(); i++) {
            PGOpt2FramePair *pair = new PGOpt2FramePair();
            pair->setup(urshapeFrameIndex, i, selectedUsefulPatchIndices[i], pairVars->getFramePairVars(frameVarIndex));
            frameVarIndex++;
            result->addEdge(pair);
         }
      }
   }
   return result;
}


card32 PGOpt2GraphCreator::writeVarIndices(OptimizationGraph *graph) {
   card32 varIndex = 0;
   {
      card32 numFrames = graph->getNumFrames();
      for (card32 f=0; f<numFrames; f++) {
         PGOpt2Frame *frame = graph->getFrame(f);
         card32 frameIndex = frame->getPatchesFrameIndex();
         PointSet *ps = frame->getPatches();
         AAT VAR_POS = ps->getAAT("varIndex_pos");
         AAT VAR_NOR = ps->getAAT("varIndex_nor");
         size_t numPatchIndices = frame->getPatchIndices().size();
         for (card32 pIndex=0; pIndex<numPatchIndices; pIndex++) {
            card32 p = frame->getPatchIndices()[pIndex];
            ps->set<int>(frameIndex, p, VAR_POS, varIndex);
            varIndex += 3;
            ps->set<int>(frameIndex, p, VAR_NOR, varIndex);
            varIndex += 3;
         }
      }
   } {   
      card32 numEdges = graph->getNumEdges();
      for (card32 e=0; e<numEdges; e++) {
         PGOpt2FramePair *pair = graph->getEdge(e);
         PointSet *ps = pair->getPatchPairsAttribs();
         AAT VAR_ROT = ps->getAAT("varIndex_rot");
         size_t numPatchIndices = pair->getPatchIndices().size();
         for (card32 pIndex=0; pIndex<numPatchIndices; pIndex++) {
            card32 p = pair->getPatchIndices()[pIndex];
            ps->set<int>(p, VAR_ROT, varIndex);
            varIndex += 4;
         }
      }
   }
   return varIndex;
}

void PGOpt2GraphCreator::copyDataVars(OptimizationGraph *graph, DVectorF &vars, bool dataToVars) {
   card32 varIndex = 0;
   {
      card32 numFrames = graph->getNumFrames();
      for (card32 f=0; f<numFrames; f++) {
         PGOpt2Frame *frame = graph->getFrame(f);
         card32 frameIndex = frame->getPatchesFrameIndex();
         PointSet *ps = frame->getPatches();
         AAT VAR_POS = ps->getAAT("varIndex_pos");
         AAT VAR_NOR = ps->getAAT("varIndex_nor");
         AAT POS = ps->getAAT("position");
         AAT NOR = ps->getAAT("normal");
         size_t numPatchIndices = frame->getPatchIndices().size();
         for (card32 pIndex=0; pIndex<numPatchIndices; pIndex++) {
            card32 p = frame->getPatchIndices()[pIndex];
            card32 storedVarIndPos = ps->get<int>(frameIndex, p, VAR_POS);
            card32 storedVarIndNor = ps->get<int>(frameIndex, p, VAR_NOR);
            pAssert(storedVarIndPos == varIndex);
            pAssert(storedVarIndNor == varIndex+3);
            pAssert(vars.getDim() > storedVarIndNor+2);
            if (dataToVars) {
               Vector3f pos = ps->get3f(frameIndex, p, POS);
               Vector3f nor = ps->get3f(frameIndex, p, NOR);
               vars[storedVarIndPos  ] = pos[0];
               vars[storedVarIndPos+1] = pos[1];
               vars[storedVarIndPos+2] = pos[2];
               vars[storedVarIndNor  ] = nor[0];
               vars[storedVarIndNor+1] = nor[1];
               vars[storedVarIndNor+2] = nor[2];
            } else {
               Vector3f pos;
               Vector3f nor;
               pos[0] = vars[storedVarIndPos  ];
               pos[1] = vars[storedVarIndPos+1];
               pos[2] = vars[storedVarIndPos+2];
               nor[0] = vars[storedVarIndNor  ];
               nor[1] = vars[storedVarIndNor+1];
               nor[2] = vars[storedVarIndNor+2];
               ps->set3f(frameIndex, p, POS, pos);
               ps->set3f(frameIndex, p, NOR, nor);
            }
            varIndex += 6;
         }
      }
   } {   
      card32 numEdges = graph->getNumEdges();
      for (card32 e=0; e<numEdges; e++) {
         PGOpt2FramePair *pair = graph->getEdge(e);
         PointSet *ps = pair->getPatchPairsAttribs();
         AAT VAR_ROT = ps->getAAT("varIndex_rot");
         AAT ROT = ps->getAAT("rotation_matrix");
         size_t numPatchIndices = pair->getPatchIndices().size();
         for (card32 pIndex=0; pIndex<numPatchIndices; pIndex++) {
            card32 p = pair->getPatchIndices()[pIndex];
            card32 storedVarIndRot = ps->get<int>(p, VAR_ROT);
            pAssert(storedVarIndRot == varIndex);
            pAssert(vars.getDim() > storedVarIndRot+2);
            if (dataToVars) {
               Vector4f rot = ps->get<float32, 4>(p, ROT);
               vars[storedVarIndRot  ] = rot[0];
               vars[storedVarIndRot+1] = rot[1];
               vars[storedVarIndRot+2] = rot[2];
               vars[storedVarIndRot+3] = rot[3];
            } else {
               Vector4f rot;
               rot[0] = vars[storedVarIndRot  ];
               rot[1] = vars[storedVarIndRot+1];
               rot[2] = vars[storedVarIndRot+2];
               rot[3] = vars[storedVarIndRot+3];
               ps->set<float32, 4>(p, ROT, rot);
            }
            varIndex += 4;
         }
      }
   }
}
