//---------------------------------------------------------------------------
#ifndef PGOpt2GraphCreatorH
#define PGOpt2GraphCreatorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "DynamicLinearAlgebra.h"
//---------------------------------------------------------------------------

/// slower but safer if defined
#define PGOPT2GRAPHCREATOR_DEBUG

namespace geoxl {


class OptimizationGraph;
class PatchGraph;
class PointCloudAnimationSequence;
class PGOpt2FramePairVars;


///
///   creates optimization graphs for patch graphs
///
class ANIMATIONRECONSTRUCTION_API PGOpt2GraphCreator : public Persistent {
	GEOXL_CLASS( PGOpt2GraphCreator )
 private:

 public:
	PGOpt2GraphCreator();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   /// creates frame pair variables (rotation matrices etc.); for use with createOptGraph
   PGOpt2FramePairVars *createPairVars(
		const PatchGraph *pg, 
		bool cyclic, 
		bool urshape, 
		bool urshapeToScene, 
		bool sceneToUrshape, 
		bool doubleInterFrameLinks,
		bool omitInterFrameLinks = false
	);

   /// create an opt graph
   /// pg - patch graph for which the graph is created
   /// data - data points
   /// pairVars - container storing the per-edge attributes
   /// cyclic - connect last to first frame
   /// urshape - include an urshape (from pg)
   /// urshapeToScene - connect urshape to every frame in scene
   /// sceneToUrshape - connect every frame in scene to urshape
   /// doubleInterFrameLinks - connects frames back and forth (not just in foreward direction)
   /// indicesInFrames - list of indices to be optimized in every frame, outer vector: frames, inner vector: points
   ///                   NULL means use all (urshape points/normals are always optimized completely)
   OptimizationGraph *createOptGraph(PatchGraph *pg, PointCloudAnimationSequence *data, PGOpt2FramePairVars *pairVars, 
                                     bool cyclic, bool urshape, bool urshapeToScene, bool sceneToUrshape,
                                     bool doubleInterFrameLinks,
                                     const vector< vector<card32> > *indicesInFrames,
                                     const vector< vector<card32> > *indicesInEdges,
                                     bool omitFramePatches = false,
                                     bool omitInterFrameLinks = false,
                                     card32 minVertexDegree = 0);

   card32 writeVarIndices(OptimizationGraph *graph);
   /// copies data from vector to variables stored in graph and vice versa; "dataToVars" selects direction
   void copyDataVars(OptimizationGraph *graph, DVectorF &vars, bool dataToVars);
      

   ~PGOpt2GraphCreator();
};

}

#endif