//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PGOpt2Parameters.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "SeparatorClassProperty.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PGOpt2Parameters, 2 ) {
	BEGIN_CLASS_INIT( PGOpt2Parameters );
   /// please use categories to sort properties
	ADD_SEPARATOR( "numerics" );
	ADD_CARD32_PROP(numIterations,                  0)
	ADD_FLOAT32_PROP(convergenceEps,                0)
	ADD_FLOAT32_PROP(levenbergMarquant,					2)
	ADD_FLOAT32_PROP(minimumEnergyDecay,				0)
	ADD_FLOAT32_PROP(minimumStepsize,					0)

	ADD_SEPARATOR( "likelihood (data term)" );
	ADD_FLOAT32_PROP( weightSquaredDistance,			0)
	ADD_FLOAT32_PROP( weightMarkerError,				0)

	ADD_SEPARATOR( "smoothness" );
	ADD_FLOAT32_PROP( weightPlanarSmoothness,			0)
	ADD_FLOAT32_PROP( weightNormalSmoothness,			0)
	ADD_FLOAT32_PROP( weightLaplacianSmoothness,		0)

	ADD_SEPARATOR( "rigidity" );
	ADD_FLOAT32_PROP( weightPositionRigidity,			0)
	ADD_FLOAT32_PROP( weightNormalRigidity,				0)

	ADD_SEPARATOR( "acceleration" );
	ADD_FLOAT32_PROP( weightPositionCoherence,			0)
	ADD_FLOAT32_PROP( weightNormalCoherence,			0)
	ADD_FLOAT32_PROP( weightRotationCoherence,			0)
	ADD_FLOAT32_PROP( weightColorSimilarity,			0)

	ADD_SEPARATOR( "general" );
	ADD_FLOAT32_PROP( weightlambda,						0)
	ADD_FLOAT32_PROP( tradeoff_sdf,						0)
	ADD_FLOAT32_PROP( overlapping_rate,					0)
	ADD_BOOLEAN_PROP( outputVertexPotentials,       1)

	ADD_SEPARATOR( "cg solver");
	ADD_FLOAT32_PROP(cgStopRes, 0)
	ADD_CARD32_PROP(cgMaxNumIterations, 0)
	ADD_BOOLEAN_PROP(cgDebugging, 0)
	ADD_SEPARATOR( "misc")
	ADD_BOOLEAN_PROP( verbose, 0 )
}

// ----

PGOpt2Parameters::PGOpt2Parameters() {

	numIterations				   = 10;
	convergenceEps				   = 1e-3f;
	levenbergMarquant          = 0.0f;
	minimumEnergyDecay			= 0.99f;
	minimumStepsize				= 1.0f/64;

	weightlambda				   = 0.0f;
	tradeoff_sdf				   = 0.7f;
	overlapping_rate			   = 0.85f;
	weightSquaredDistance		= 1.0f;
	weightMarkerError			   = 10.0f;
	weightPlanarSmoothness		= 1.0f;
	weightNormalSmoothness		= 0.01f;
	weightLaplacianSmoothness	= 1.0f;
	weightPositionRigidity		= 1.0f;
	weightNormalRigidity		   = 0.1f;
	weightPositionCoherence		= 0.1f;
	weightNormalCoherence		= 0.1f;
	weightRotationCoherence		= 0.1f;
	weightColorSimilarity		= 1.0f;

	cgStopRes						= 1E-4f;
   cgMaxNumIterations			= 250;
   cgDebugging						= false;
   outputVertexPotentials     = false;
}

void PGOpt2Parameters::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

PGOpt2Parameters::~PGOpt2Parameters() {
}



