//---------------------------------------------------------------------------
#ifndef PGOpt2ParametersH
#define PGOpt2ParametersH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

///
///   parameters for the optimization (PatchGraphOpt2)
///
class ANIMATIONRECONSTRUCTION_API PGOpt2Parameters : public Persistent {
	GEOXL_CLASS( PGOpt2Parameters )
 private:

 public:
	PGOpt2Parameters();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   ~PGOpt2Parameters();

   /// Number of iterations in the optimization procedure
   card32				numIterations;
   /// threshold for convergence (depends on stoppingCriterion)
   float32				convergenceEps;
   /// amount of identity added to approx. Hessian
   float32           levenbergMarquant;

   /// Parameters used in Armijo rule
   float32				minimumEnergyDecay;
   float32				minimumStepsize;

   ///The lambda parameter used in optimization
   float32	weightlambda;
   /// The trade-off between point-point squared distance potential and
   /// point-plane squared distance potential
   float32	tradeoff_sdf;
   ///the percentage of overlapping region
   float32	overlapping_rate;
   ///weight of squared distance term
   float32	weightSquaredDistance;
   /// weight of marker error potential
   float32	weightMarkerError;
   ///weight of planar smoothness term
   float32	weightPlanarSmoothness;
   ///weight of normal smoothness term
   float32	weightNormalSmoothness;
   ///weight of laplacian smoothness term
   float32	weightLaplacianSmoothness;
   ///weight of position rigidity term
   float32	weightPositionRigidity;
   ///weight of normal rigidity term
   float32	weightNormalRigidity;
   /// weight of position spatial temporal coherence
   float32	weightPositionCoherence;
   /// weight of normal spatial temporal coherence
   float32	weightNormalCoherence;
   /// weight of rotation matrix coherence
   float32	weightRotationCoherence;
   /// weight of color similarity
   float32	weightColorSimilarity;
   /// if true, the computed energy-function / potential is stored in a channel for every vertex
   bool outputVertexPotentials;

   // *** linear system solver ***
   float32	cgStopRes;
   card32	cgMaxNumIterations;
   bool		cgDebugging;

   // *** MISC ***
   bool verbose;
};

}

#endif