//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PGOpt2Potential.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
#include "AnimationBSRContext.h"
#include "PGOpt2Parameters.h"
#include "OptimizationGraph.h"
#include "PGOpt2Frame.h"
#include "InCorePCTopologyGraph.h"
#include "PatchTypes.h"
#include "PointCloudAnimationSequence.h"
#include "PointSetKNNQuery.h"
#include "AttachedIndexedOctree.h"
#include "PGOpt2FramePair.h"
#include "PGOpt2FramePairPair.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_ABSTRACT_CLASS( PGOpt2Potential ,0)
{
	BEGIN_CLASS_INIT( PGOpt2Potential );
}

IMPLEMENT_GEOXL_ABSTRACT_CLASS( PGOpt2SquaredDistancePotential_Global ,0)
{
	BEGIN_CLASS_INIT( PGOpt2SquaredDistancePotential_Global );
}

IMPLEMENT_GEOXL_ABSTRACT_CLASS( PGOpt2SquaredDistancePotential_Pairwise ,0)
{
	BEGIN_CLASS_INIT( PGOpt2SquaredDistancePotential_Pairwise );
}

IMPLEMENT_GEOXL_ABSTRACT_CLASS( PGOpt2MarkerErrorPotential ,0)
{
	BEGIN_CLASS_INIT( PGOpt2MarkerErrorPotential );
}

IMPLEMENT_GEOXL_ABSTRACT_CLASS( PGOpt2SmoothnessPotential ,0)
{
	BEGIN_CLASS_INIT( PGOpt2SmoothnessPotential );
}

IMPLEMENT_GEOXL_ABSTRACT_CLASS( PGOpt2RigidityPotential ,0)
{
	BEGIN_CLASS_INIT( PGOpt2RigidityPotential );
}

IMPLEMENT_GEOXL_ABSTRACT_CLASS( PGOpt2TemporalCoherence ,0)
{
	BEGIN_CLASS_INIT( PGOpt2TemporalCoherence );
}

IMPLEMENT_GEOXL_ABSTRACT_CLASS( PGOpt2STColorSimilarity ,0)
{
	BEGIN_CLASS_INIT( PGOpt2STColorSimilarity );
}

PGOpt2Potential::PGOpt2Potential()
{
}

PGOpt2Potential::~PGOpt2Potential()
{
}

void PGOpt2Potential::assign(const Object* obj, CopyContext *context)
{
   copyObjectProperties(obj, this);
}

void		Quaternion_To_Matrix3f_Opt2(const float	*q, Matrix3f &rotation)
{
	pAssert(q != NULL);
	rotation[0][0] = 2*q[1]*q[1] + (2*q[0]*q[0]-1);
	rotation[1][0] = 2*q[1]*q[2] - 2*q[0]*q[3];
	rotation[2][0] = 2*q[1]*q[3] + 2*q[0]*q[2];
	rotation[0][1] = 2*q[2]*q[1] + 2*q[0]*q[3];
	rotation[1][1] = 2*q[2]*q[2] + (2*q[0]*q[0]-1);
	rotation[2][1] = 2*q[2]*q[3] - 2*q[0]*q[1];
	rotation[0][2] = 2*q[3]*q[1] - 2*q[0]*q[2];
	rotation[1][2] = 2*q[3]*q[2] + 2*q[0]*q[1];
	rotation[2][2] = 2*q[3]*q[3] + (2*q[0]*q[0]-1);
}

void		BinaryInsertion_Sort1(
								 vector<int>	&m_array, //the sorted array
								 int newElement, // the new element to be inserted
								 mpcard &index)	// the position of this element after insertion
{
	if(m_array.size() > 0)
	{
		vector<int>::iterator	begin = m_array.begin(), pos;
		mpcard	left = 0, right = m_array.size()-1;
		if(newElement <= (*begin))
		{
			m_array.insert(begin, newElement);
			index = 0;
		}
		else if(newElement >= m_array[right])
		{
			index = m_array.size();
			m_array.push_back(newElement);
		}
		else
		{
			while (right - left > 1)
			{
				mpcard mid = (right + left)/2;
				if(newElement < m_array[mid])
				{
					right = mid;
				}
				else
				{
					left = mid;
				}
			}
			index = right;
			pos = begin + right;
			m_array.insert(pos, newElement);
		}
	}
	else
	{
		m_array.push_back(newElement);
		index = 0;
	}
}

static	void	UpdateMatrix_by_a_matrix1(
	const DMatrixF		*mat,				
	const DVectorI		*m_indices,
	const	DVectorI		*m_mul,
	const card32			numOfElements,
	SparseMatrixF		*quad)			/// The Hessian of the energy function	
{
	card32 id = 0;
	for(card32 i = 0; i < numOfElements; i++)
	{
		int row = (*m_indices)[i];
		if(row >= 0)
		{
			for(card32 j = 0; j < card32((*m_mul)[i]); j++)
			{
				(*quad)[row+j].updateEntries_Binary(&((*mat)[id]), m_indices, m_mul, numOfElements);
//            (*quad)[row+j].addEntries_Binary(m_indices, &((*mat)[id]), numOfElements);
				id++;
			}
		}
		else
		{
			id+= (*m_mul)[i];
		}
	}
}

static	void	UpdateVector_by_a_vector1(const	DVectorF	*vec,
										 const	DVectorI		*m_indices, 
										 const	DVectorI		*m_mul,
										 const	card32		numOfElements,
										 DVectorF	*lin)
{
	card32 id = 0;
	for(card32 i = 0; i < numOfElements; i++)
	{
		int row = (*m_indices)[i];
		if(row >= 0)
		{
			for(card32 j = 0; j < card32((*m_mul)[i]); j++)
			{
				(*lin)[row+j] += (*vec)[id];
				id++;
			}
		}
		else
		{
			id += (*m_mul)[i];
		}
	}
}


/// Squared distance potential

void PGOpt2SquaredDistancePotential_Global::quadraticApprox(
	const PGOpt2Parameters     *parameters,
	const AnimationBSRContext   *recParameters,
	const InCorePCTopologyGraph *topology,
	OptimizationGraph           *optGraph,
	float32                     &potential,
	DVectorF                    *gradient,
	SparseMatrixF               *quadApprox)
{
	potential = 0.0f;
	float32		w_sdf	= parameters->weightSquaredDistance,
						tradeoff= parameters->tradeoff_sdf; 
										

	/// The dimension of the patch graph
	card32		width	= optGraph->getNumFrames();
		
	DMatrixF	mat;		mat.setDimension(5, 5, false);		
	DVectorF	vec;		vec.setDim(5);									
	DVectorI	m_indices;	m_indices.setDim(2);							
	DVectorI	m_mul;		m_mul.setDim(2);

	for(card32 frameIndex = 0; frameIndex < width; frameIndex++)
	{
		PGOpt2Frame	*frame	             = optGraph->getFrame(frameIndex);
		if (frame->getPatchIndices().empty()) continue;
		card32 patchesFrameIndex          = frame->getPatchesFrameIndex();
		PointSet	*ps				          = frame->getPatches();
		PointCloudAnimationSequence *anim = frame->getData();
		if (anim == NULL) continue;
		PointSet	*dataPS		             = anim->getFrame(frame->getDataFrameIndex());
		/// Attributes
		AAT POSITION	 = ps->getAAT("position");
		AAT NORMAL      = ps->getAAT("normal");
		AAT REF_U		 = ps->getAAT("tangent_u");
		AAT POS_INDEX	 = ps->getAAT("varIndex_pos");
		AAT NOR_INDEX   = ps->getAAT("varIndex_nor");
		AAT FLAG_INLIER = dataPS->getAAT("flag_inlier");
		AAT DATA_POS    = dataPS->getAAT("position");

      AAT SDF_POT = NULL_AAT;
      if (ps->providesAttribute("sdfPot", 1, VAD::DATA_FORMAT_FLOAT32)) {
         SDF_POT = ps->getAAT("sdfPot");
      }

		AttachedIndexedOctree *octree = frame->fetchPatchOctree();
		PointSet *psSlice = ps->copyVerticalSlice(patchesFrameIndex);
		PointSetKNNQuery *pg_knn = new PointSetKNNQuery(psSlice, octree, NULL_VECTOR3F, ps->getAAT("position"));
		
		card32	numOfDataPoints = dataPS->getHeight();
		card32	footIndex;
		for(card32 pointIndex = 0; pointIndex	< numOfDataPoints; pointIndex++)
		{
		   float32 lastPotential = potential;
			int32	flag_inlier	= dataPS->get<int32>(0, pointIndex, FLAG_INLIER);
			if(flag_inlier < 0)
				continue;
			Vector3f	dik		= dataPS->get3f(0, pointIndex, DATA_POS);

			pg_knn->setTargetPoint(dik);
			{
				bool found;
				pg_knn->getNextNearestPointIndex(footIndex, found);
				pAssert(found);
			}
			Vector3f		pik				= ps->get3f(patchesFrameIndex, footIndex, POSITION);
			Vector3f		devik_foot	= dik - pik;
			if (norm(devik_foot) > recParameters->getSamplingSpacing() * recParameters->getTopology2MaxDistanceRelSampleSpacing()) continue;
			
			Vector3f		nik_foot		= ps->get3f(patchesFrameIndex, footIndex, NORMAL); pAssert(fabs(1.0f-norm(nik_foot)) < 0.1f);
			Vector3f		Eik1_foot	= ps->get3f(patchesFrameIndex, footIndex, REF_U); pAssert(fabs(1.0f-norm(Eik1_foot)) < 0.1f);
			Vector3f		Eik2_foot	= nik_foot.crossProduct(Eik1_foot);
			int32			posid_foot	= ps->get<int32>(patchesFrameIndex, footIndex, POS_INDEX);
			int32			norid_foot	= ps->get<int32>(patchesFrameIndex, footIndex, NOR_INDEX);
			
			float32			disik		= devik_foot*nik_foot;
			
			float		buf_v[5];
			m_mul[0]	= 3;		m_mul[1] = 2;
			m_indices[0]= posid_foot;	m_indices[1] = norid_foot;
			for(card32 k= 0; k < 3; k++)
			{
				buf_v[k]= - nik_foot[k];
			}
			buf_v[3]	= devik_foot*Eik1_foot;
			buf_v[4]	= devik_foot*Eik2_foot;
			potential  += disik*disik*w_sdf*(1-tradeoff);
			for(card32 i = 0; i < 5; i++)
			{
				vec[i] = -disik*buf_v[i]*w_sdf*(1-tradeoff);
				for(card32 j = 0; j < 5; j++)
				{
					mat[i][j] = buf_v[i]*buf_v[j]*w_sdf*(1-tradeoff);
				}
			}
			potential  += devik_foot.getSqrNorm()*w_sdf*tradeoff;
			for(card32 i = 0; i < 3; i++)
			{
				vec[i] += devik_foot[i]*w_sdf*tradeoff;
				mat[i][i] += w_sdf*tradeoff;
			}
			UpdateMatrix_by_a_matrix1(&mat, &m_indices, &m_mul, 2, quadApprox);
			UpdateVector_by_a_vector1(&vec, &m_indices, &m_mul, 2, gradient);
			
			if (parameters->outputVertexPotentials && SDF_POT != NULL_AAT) {
			   ps->set1f(patchesFrameIndex, footIndex, SDF_POT, ps->get1f(patchesFrameIndex, footIndex, SDF_POT) + potential - lastPotential);
			}

		}
		delete pg_knn;
		delete psSlice;
	}
}

void PGOpt2SquaredDistancePotential_Pairwise::quadraticApprox(
	const PGOpt2Parameters     *parameters,
	const AnimationBSRContext   *recParameters,
	const InCorePCTopologyGraph *topology,
	OptimizationGraph           *optGraph,
	float32                     &potential,
	DVectorF                    *gradient,
	SparseMatrixF               *quadApprox)
{
	potential = 0.0f;
	float32		w_sdf	= parameters->weightSquaredDistance;

	/// The dimension of the patch graph
	card32		width	= optGraph->getNumFrames();
	/// 
	DMatrixF	mat;		mat.setDimension(5, 5, false);		
	DVectorF	vec;		vec.setDim(5);									
	DVectorI	m_indices;	m_indices.setDim(2);							
	DVectorI	m_mul;		m_mul.setDim(2);


	for(card32 frameIndex = 0; frameIndex < width; frameIndex++)
	{	
		PGOpt2Frame	*frame		= optGraph->getFrame(frameIndex);
		if(frame->getDataFrameIndex() < 0)
			continue;

		PointSet	*ps						    = frame->getPatches();
		card32 patchesFrameIndex          = frame->getPatchesFrameIndex();
		PointCloudAnimationSequence *anim = frame->getData();
		if (anim == NULL) continue;
		PointSet	*dataPS				       = anim->getFrame(frame->getDataFrameIndex());
		
		if(dataPS->getHeight() == 0)
		   continue;
		/// Attributes
		AAT			POSITION			= ps->getAAT("position");
		AAT			NORMAL				= ps->getAAT("normal");
		AAT			REF_U					= ps->getAAT("tangent_u"); 
		AAT			POS_INDEX		= ps->getAAT("varIndex_pos");
		AAT			NOR_INDEX		= ps->getAAT("varIndex_nor");
		AAT			DATA_POS_AAT= dataPS->getAAT("position", 3, VAD::DATA_FORMAT_FLOAT32 );
		AAT			FLAG_INLIER		= dataPS->getAAT("flag_inlier");


		vector<Vector3f>		buf_dev_df_ik;			//Buffer for deviation vectors from data points to patches						
		vector<Vector3f>		buf_dev_fd_ik;			//Buffer for deviation vectors from patches to data points
		vector<Vector3f>		buf_frame_foot_nik;		//Buffer for normals of foot points of the patches
		vector<Vector3f>		buf_frame_foot_Eik1;	//Buffer for tangent_u of foot points of the patches
		vector<int32>			buf_varId_foot_pos;		//
		vector<int32>			buf_varId_foot_nor;
		set<float32>				buf_d2;					// Used to determine the median 

		//patch octree
		AttachedIndexedOctree *octree = frame->fetchPatchOctree();
		PointSet *psSlice = ps->copyVerticalSlice(patchesFrameIndex);
		PointSetKNNQuery*	pg_knn = new PointSetKNNQuery(psSlice, octree, NULL_VECTOR3F, ps->getAAT("position"));
		

		//data octree
		AttachedIndexedOctree *dataOctree = frame->getData()->fetchOctree(frame->getDataFrameIndex());
		PointSetKNNQuery data_knn(dataPS, dataOctree, NULL_VECTOR3F, dataPS->getAAT("position"));

		card32			numOfDataPoints = dataPS->getHeight();
		for(card32 pointIndex = 0; pointIndex	< numOfDataPoints; pointIndex++)
		{
			Vector3f		data_source_pik		= dataPS->get3f(0, pointIndex, DATA_POS_AAT);
			card32		footIndex_Frame, footIndex_Data;

			pg_knn->setTargetPoint(data_source_pik);
			{
				bool found;
				pg_knn->getNextNearestPointIndex(footIndex_Frame, found);
				pAssert(found);
			}

			int32			varId_foot_pos	= ps->get<int32>(patchesFrameIndex, footIndex_Frame, POS_INDEX);
			int32			varId_foot_nor	= ps->get<int32>(patchesFrameIndex, footIndex_Frame, NOR_INDEX);
			/// The foot point can be moved
			if(varId_foot_pos >= 0 || varId_foot_nor >= 0)
			{
				Vector3f		frame_foot_pik	= ps->get3f(patchesFrameIndex, footIndex_Frame, POSITION);
				data_knn.setTargetPoint(frame_foot_pik);
				{
					bool found;
					data_knn.getNextNearestPointIndex(footIndex_Data, found);
					pAssert(found);
				}
				bool	good_correspondence = true;
				
				if(good_correspondence)
				{
					Vector3f		data_foot_pik		= dataPS->get3f(0, footIndex_Data, DATA_POS_AAT);
					Vector3f		frame_foot_nik		= ps->get3f(patchesFrameIndex, footIndex_Frame, NORMAL); pAssert(fabs(1.0f-norm(frame_foot_nik)) < 0.1f);
					Vector3f		frame_foot_Eik1	= ps->get3f(patchesFrameIndex, footIndex_Frame, REF_U);  pAssert(fabs(1.0f-norm(frame_foot_Eik1)) < 0.1f);
					Vector3f		dev_df_ik				= data_source_pik	- frame_foot_pik;
					Vector3f		dev_fd_ik				= frame_foot_pik		- data_foot_pik;

					buf_dev_df_ik.push_back(dev_df_ik);
					buf_dev_fd_ik.push_back(dev_fd_ik);
					buf_frame_foot_nik.push_back(frame_foot_nik);
					buf_frame_foot_Eik1.push_back(frame_foot_Eik1);
					buf_varId_foot_pos.push_back(varId_foot_pos);
					buf_varId_foot_nor.push_back(varId_foot_nor);
					buf_d2.insert(dev_df_ik.getSqrNorm());
				}	
			}
		}
		set<float32>::iterator		iter_float = buf_d2.begin();
		for(card32 pointIndex = 0; pointIndex < card32(buf_d2.size()*parameters->overlapping_rate); pointIndex++)
		{
			pointIndex++;		iter_float++;
		}
		float32 mean_d2 = *iter_float;

		for(card32 pointIndex = 0; pointIndex < buf_d2.size(); pointIndex++)
		{
			Vector3f		dev_df_ik	= buf_dev_df_ik[pointIndex];
			Vector3f		dev_fd_ik	= buf_dev_fd_ik[pointIndex];

			float32	sqrDevdfik		= dev_df_ik.getSqrNorm(),
							sqrDevfdik		= dev_fd_ik.getSqrNorm();

			dev_df_ik.normalize();		dev_fd_ik.normalize();
			float32	angleik			= fabs(dev_df_ik*dev_fd_ik);

			// Check whether data_pik reside in the overlapping region
			if((angleik < 0.7 || sqrDevdfik > 4*sqrDevfdik) &&sqrDevdfik > 4*mean_d2)
			{
				dataPS->set<int32>(0, pointIndex, FLAG_INLIER, -1);
			}
			else
			{
				dataPS->set<int32>(0, pointIndex, FLAG_INLIER, 1);
			}
			if((angleik < 0.7 || sqrDevdfik > 4*sqrDevfdik) &&sqrDevdfik > 2*mean_d2)
			{
				continue;
			}

			int32		varId_foot_pos		=	buf_varId_foot_pos[pointIndex];
			int32		varId_foot_nor		=	buf_varId_foot_nor[pointIndex];
			Vector3f	Eik2_foot				=	buf_frame_foot_nik[pointIndex].crossProduct(buf_frame_foot_Eik1[pointIndex]);
			///In this case we only use point-plane distances because we assume that 
			///we have a very good initial position
			m_mul[0]	= 3;			m_indices[0] = varId_foot_pos;	
			m_mul[1]	= 2;			m_indices[1] = varId_foot_nor;
			float		buf_v[5],		disik		 = buf_dev_df_ik[pointIndex]*buf_frame_foot_nik[pointIndex];

			for(card32 k = 0; k < 3; k++)
			{
				buf_v[k] = - buf_frame_foot_nik[pointIndex][k];
			}
			buf_v[3]		= buf_dev_df_ik[pointIndex]*buf_frame_foot_Eik1[pointIndex];
			buf_v[4]		= buf_dev_df_ik[pointIndex]*Eik2_foot;
			potential += disik*disik*w_sdf*(1.0 - parameters->tradeoff_sdf);
			for(card32 i = 0; i < 5; i++)
			{
				vec[i] = -disik*buf_v[i]*w_sdf*(1.0 - parameters->tradeoff_sdf);
				for(card32 j = 0; j < 5; j++)
				{
					mat[i][j] = buf_v[i]*buf_v[j]*w_sdf*(1.0 - parameters->tradeoff_sdf);
				}
			}
			potential += (buf_dev_df_ik[pointIndex][0]*buf_dev_df_ik[pointIndex][0]
						+buf_dev_df_ik[pointIndex][1]*buf_dev_df_ik[pointIndex][1]
						+buf_dev_df_ik[pointIndex][2]*buf_dev_df_ik[pointIndex][2])*w_sdf*parameters->tradeoff_sdf;
			for(card32 i = 0; i < 3; i++)
			{
				vec[i] += buf_dev_df_ik[pointIndex][i]*w_sdf*parameters->tradeoff_sdf;
				mat[i][i] += w_sdf*parameters->tradeoff_sdf;
			}
			UpdateMatrix_by_a_matrix1(&mat, &m_indices, &m_mul, 2, quadApprox);
			UpdateVector_by_a_vector1(&vec, &m_indices, &m_mul, 2, gradient);
		}
		delete pg_knn;
		delete psSlice;
	}
}

/// Marker error potential

void PGOpt2MarkerErrorPotential::quadraticApprox(
	const PGOpt2Parameters     *parameters,
	const AnimationBSRContext   *recParameters,
	const InCorePCTopologyGraph *topology,
	OptimizationGraph           *optGraph,
	float32                     &potential,
	DVectorF                    *gradient,
	SparseMatrixF               *quadApprox)
{
	potential				= 0.0f;
	float32		w_marker	= parameters->weightMarkerError;

	/// The dimension of the patch graph
	card32		width		= optGraph->getNumFrames();

	/// Buffers for updating sparse matrix 
	DMatrixF		mat;			mat.setDimension(3, 3, false);		
	DVectorF	vec;			vec.setDim(3);									
	DVectorI		m_indices;	m_indices.setDim(1);							
	DVectorI		m_mul;		m_mul.setDim(1);

	
	/// Variables to be used
	Vector3f	pik, dik;

	/// Indies
	int32		ik_posid, ik_depth;

	card32		j, k;
	for(card32 frameIndex = 0; frameIndex < width; frameIndex++)
	{
		PGOpt2Frame	*frame = optGraph->getFrame(frameIndex);
		card32 patchesFrameIndex = frame->getPatchesFrameIndex();
		PointSet	*ps = frame->getPatches();
		if(!ps->providesAttribute("dataIndex"))
			continue;
		PointCloudAnimationSequence *anim = frame->getData();
		if (anim == NULL) continue;
		PointSet	*dataPS = anim->getFrame(frame->getDataFrameIndex());
		/// Attributes
		AAT POSITION	  = ps->getAAT("position"),       
          REF_U		  = ps->getAAT("tangent_u"),		 
          POS_INDEX	  = ps->getAAT("varIndex_pos"),	 
          TOP_DEPTH	  = ps->getAAT("topology_depth"), 
          DATA_INDEX   = ps->getAAT("dataIndex"),
          DATA_POS_AAT = dataPS->getAAT("position");

		vector<card32>	patchIndices = frame->getPatchIndices();

      map<card32, Vector3f> corr = frame->getCorrespondences();
      
      map<card32, Vector3f>::iterator pos = corr.begin();
      map<card32, Vector3f>::iterator end = corr.end();
      
      while (pos != end) {
      
			card32	surfelIndex = pos->first;
			ik_depth		=	ps->get<int32>(patchesFrameIndex, surfelIndex, TOP_DEPTH);
			if(ik_depth >= 2)
				continue;

			ik_posid		= ps->get<int32>(patchesFrameIndex, surfelIndex, POS_INDEX);

			/// get geometry
			pik				= ps->get3f(patchesFrameIndex, surfelIndex,	POSITION);

			dik = pos->second;
			Vector3f		devik	= pik - dik;
			m_mul[0]				= 3;
			m_indices[0]			= ik_posid;
			/// Update Gradient
			potential += devik.getSqrNorm()*w_marker;
			for(j = 0; j < 3; j++)
			{
				vec[j]	= -devik[j]*w_marker;
				for(k = 0; k < 3; k++)
				{
					mat[j][k]	= 0.0f;
				}
				mat[j][j]	+= w_marker;
			}
			UpdateMatrix_by_a_matrix1(&mat, &m_indices, & m_mul, 1, quadApprox);
			UpdateVector_by_a_vector1(&vec, &m_indices, & m_mul, 1, gradient);
      
         ++pos;
      }
      
         		
	   
		
		
		//for(card32 id = 0; id < patchIndices.size(); id++)
		//{
		//	card32	surfelIndex = patchIndices[id];
		//	ik_depth		=	ps->get<int32>(patchesFrameIndex, surfelIndex, TOP_DEPTH);
		//	if(ik_depth >= 2)
		//		continue;

		//	ik_posid		= ps->get<int32>(patchesFrameIndex, surfelIndex, POS_INDEX);

		//	/// get geometry
		//	pik				= ps->get3f(patchesFrameIndex, surfelIndex,	POSITION);

		//	int32	dataIndex = ps->get<int32>(patchesFrameIndex, surfelIndex, DATA_INDEX);
		//	if (dataIndex >= 0 ) 
		//	{
		//		dik = dataPS->get3f(dataIndex, DATA_POS_AAT);
		//		Vector3f		devik	= pik - dik;
		//		m_mul[0]				= 3;
		//		m_indices[0]			= ik_posid;
		//		/// Update Gradient
		//		potential += devik.getSqrNorm()*w_marker;
		//		for(j = 0; j < 3; j++)
		//		{
		//			vec[j]	= -devik[j]*w_marker;
		//			for(k = 0; k < 3; k++)
		//			{
		//				mat[j][k]	= 0.0f;
		//			}
		//			mat[j][j]	+= w_marker;
		//		}
		//		UpdateMatrix_by_a_matrix1(&mat, &m_indices, & m_mul, 1, quadApprox);
		//		UpdateVector_by_a_vector1(&vec, &m_indices, & m_mul, 1, gradient);
		//	}
		//}
	}
}

/// Smoothness potential

void PGOpt2SmoothnessPotential::quadraticApprox(
	const PGOpt2Parameters     *parameters,
	const AnimationBSRContext   *recParameters,
	const InCorePCTopologyGraph *topology,
	OptimizationGraph           *optGraph,
	float32                     &potential,
	DVectorF                    *gradient,
	SparseMatrixF               *quadApprox)
{
	potential = 0.0f;

	/// Controls the smoothness
	float32	lambda	=	parameters->weightlambda;

	/// Get geometry and topology data structure from patch graph

	/// The dimension of the patchgraph
	card32								width				=	optGraph->getNumFrames();

	/// Buffers for updating sparse matrix(planar and normal smoothness) 
	DMatrixF								mat;			mat.setDimension(10, 10, false);		
	DVectorF								vec;			vec.setDim(10);									
	DVectorI								m_indices;		m_indices.setDim(4);							
	DVectorI								m_mul;			m_mul.setDim(4);

	/// Variables to be used
	Vector3f								pik,nik,Eik1,Eik2,pjk,njk,Ejk1,Ejk2,
											rijk,nijk,cen_pik;

	/// Distances
	float32									disijk;

	/// Indies
	int32									ik_posid, ik_norid, jk_posid, jk_norid, ik_depth, jk_depth;
	bool									ik_need_smoothness_flag, jk_need_smoothness_flag;

	for(card32 frameIndex = 0; frameIndex < width; frameIndex++)
	{
		PGOpt2Frame *frame = optGraph->getFrame(frameIndex);
   	const float32 w_pla_s = parameters->weightPlanarSmoothness * frame->getSmoothnessWeight(),
   	              w_nor_s = parameters->weightNormalSmoothness * frame->getSmoothnessWeight(),
		              w_lap_s = parameters->weightLaplacianSmoothness * frame->getSmoothnessWeight();
		card32 patchesFrameIndex = frame->getPatchesFrameIndex();
		PointSet	*ps = frame->getPatches();
		/// Attributes
		AAT					POSITION		= ps->getAAT("position"),				NORMAL			= ps->getAAT("normal"),
								REF_U				= ps->getAAT("tangent_u"),			POS_INDEX	= ps->getAAT("varIndex_pos"),
								NOR_INDEX	= ps->getAAT("varIndex_nor"),		TOP_DEPTH	= ps->getAAT("topology_depth"),	
								AAT_FLAGS	= ps->getAAT("flags");

      AAT SMOOTH_POT = NULL_AAT;
      if (ps->providesAttribute("smoothPot", 1, VAD::DATA_FORMAT_FLOAT32)) {
         SMOOTH_POT = ps->getAAT("smoothPot");
      }

		vector<card32>	patchIndices = frame->getPatchIndices();
		for(card32 id = 0; id < patchIndices.size(); id++)
		{
		   float32 lastPotential = potential;
			card32	surfelIndex = patchIndices[id];
			///number of neighbors
			ik_depth		=	ps->get<int32>(patchesFrameIndex, surfelIndex, TOP_DEPTH);
			if(ik_depth >= 2)
				continue;
			card32			numOfEntries = topology->getNumAdjacentVertices(surfelIndex);
         pAssert(numOfEntries > 0);
         			
			card32			ni, neighbor_id;
			mpcard			i_index;
			card32			j, k;

			ik_posid			= ps->get<int32>(patchesFrameIndex, surfelIndex, POS_INDEX);
			ik_norid			= ps->get<int32>(patchesFrameIndex, surfelIndex, NOR_INDEX);
			ik_need_smoothness_flag = (ps->get<int32>(patchesFrameIndex, surfelIndex, AAT_FLAGS) & FLAG_PATCH_NOSMOOTH) == 0;

			/// get geometry
			pik					= ps->get3f(patchesFrameIndex, surfelIndex,	POSITION);
			nik					= ps->get3f(patchesFrameIndex, surfelIndex,	NORMAL); pAssert(fabs(1.0f-norm(nik)) < 0.1f);
			Eik1					= ps->get3f(patchesFrameIndex, surfelIndex,	REF_U);  pAssert(fabs(1.0f-norm(Eik1)) < 0.1f);
			Eik2					= nik.crossProduct(Eik1);

			/// initialize for Laplacian smoothness potential
			cen_pik[0] = cen_pik[1] = cen_pik[2] = 0.0;
			vector<int32>	m_array;
			bool laplacian_flag = true;
			for(ni = 0; ni < numOfEntries; ni++)
			{
				/// the index of this neighbor
				neighbor_id	= topology->getAdjacentVertex(surfelIndex, ni);
				if(neighbor_id == surfelIndex)
				{
					continue;
				}

				BinaryInsertion_Sort1(m_array, neighbor_id, i_index);
				jk_depth			=	ps->get<int32>(patchesFrameIndex, neighbor_id, TOP_DEPTH);
				if(jk_depth >= 2)
				{
					laplacian_flag = false;
					continue;
				}
				/// the geometry of this neighbor
				pjk					=	ps->get3f(patchesFrameIndex, neighbor_id,	POSITION);
				njk					=	ps->get3f(patchesFrameIndex, neighbor_id,  NORMAL); pAssert(fabs(1.0f-norm(njk)) < 0.1f);
				Ejk1				=	ps->get3f(patchesFrameIndex, neighbor_id, REF_U);      pAssert(fabs(1.0f-norm(Ejk1)) < 0.1f);
				Ejk2				=	njk.crossProduct(Ejk1);

				jk_posid			=	ps->get<int32>(patchesFrameIndex, neighbor_id, POS_INDEX);
				jk_norid			=	ps->get<int32>(patchesFrameIndex, neighbor_id, NOR_INDEX);

				jk_need_smoothness_flag = (ps->get<int32>(patchesFrameIndex, neighbor_id, AAT_FLAGS) & FLAG_PATCH_NOSMOOTH) == 0;

				/// precomputed geometric quantities 
				rijk			=	pik	-	pjk;
				nijk			=	nik	-	njk;
				disijk		=	nik*rijk;

				/// precomputed weights
				float32	sedgeweight = exp(lambda*rijk.getSqrNorm());
				float32	redgeweight = 1.0f;

				float32	weight;

				/************************************************************************/
				/* Planar smoothness smoothness                                         */
				/************************************************************************/
				{
					weight = w_pla_s*sedgeweight/numOfEntries;
					float32	buf_v[8];
					m_mul[0] = m_mul[1] = 3;		m_mul[2] = 2;
					if(surfelIndex < neighbor_id)
					{
						for(k = 0; k < 3; k++)
						{
							buf_v[k]		= nik[k] + lambda*disijk*rijk[k];
							buf_v[3+k]	= -buf_v[k];
						}
						m_indices[0] = ik_posid;			m_indices[1] = jk_posid;
					}
					else
					{
						for(k = 0; k < 3; k++)
						{
							buf_v[3+k]		= nik[k] + lambda*disijk*rijk[k];
							buf_v[k]	= -buf_v[3+k];
						}
						m_indices[1] = ik_posid;			m_indices[0] = jk_posid;
					}
					buf_v[6]			= rijk*Eik1;				buf_v[7]			= rijk*Eik2;			m_indices[2] = ik_norid;
					
					potential += disijk*disijk*weight;
					for(k = 0; k < 8; k++)
					{
						vec[k] = -disijk*buf_v[k]*weight;
						for(j = 0; j < 8; j++)
						{
							mat[k][j] = buf_v[k]*buf_v[j]*weight;
						}
					}
					UpdateMatrix_by_a_matrix1(&mat, &m_indices, & m_mul, 3, quadApprox);
					UpdateVector_by_a_vector1(&vec, &m_indices, & m_mul, 3, gradient);
				}
				/************************************************************************/
				/* Normal smoothness potential                                                                     */
				/************************************************************************/
				{
					weight = w_nor_s*sedgeweight/numOfEntries;

					float32	buf_v[10][3];
					m_mul[0] = m_mul[1] = 3;		m_mul[2] = m_mul[3] = 2;
					if(surfelIndex < neighbor_id)
					{
						for(k = 0; k < 3; k++)
						{
							for(j = 0; j < 3; j++)
							{
								buf_v[k][j]		= lambda*rijk[k]*nijk[j];
								buf_v[3+k][j]	= -buf_v[k][j];
							}
							buf_v[6][k]		= Eik1[k];				buf_v[7][k]		= Eik2[k];
							buf_v[8][k]		= -Ejk1[k];				buf_v[9][k]		= -Ejk2[k];
						}
						m_indices[0] = ik_posid;						m_indices[1] = jk_posid;
						m_indices[2] = ik_norid;						m_indices[3] = jk_norid;
					}
					else
					{
						for(k = 0; k < 3; k++)
						{
							for(j = 0; j < 3; j++)
							{
								buf_v[3+k][j]		= lambda*rijk[k]*nijk[j];
								buf_v[k][j]	= -buf_v[3+k][j];
							}
							buf_v[8][k]		= Eik1[k];				buf_v[9][k]		= Eik2[k];
							buf_v[6][k]		= -Ejk1[k];				buf_v[7][k]		= -Ejk2[k];
						}
						m_indices[1] = ik_posid;						m_indices[0] = jk_posid;
						m_indices[3] = ik_norid;						m_indices[2] = jk_norid;
					}
					potential += nijk.getSqrNorm()*weight;
					for(k = 0; k < 10; k++)
					{
						vec[k] = -(nijk[0]*buf_v[k][0]+nijk[1]*buf_v[k][1]+nijk[2]*buf_v[k][2])*weight;
						for(j = 0; j < 10; j++)
						{
							mat[k][j] = (buf_v[k][0]*buf_v[j][0]
							+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
						}
					}
					UpdateMatrix_by_a_matrix1(&mat, &m_indices, & m_mul, 4, quadApprox);
					UpdateVector_by_a_vector1(&vec, &m_indices, & m_mul, 4, gradient);
				}
				cen_pik -= pjk;
			}
			if(numOfEntries >= 1 && laplacian_flag)
			{
				cen_pik /= m_array.size();
				cen_pik += pik;
				BinaryInsertion_Sort1(m_array, surfelIndex, i_index);
				card32	m_dim = 3*(card32)m_array.size();
				card32 id, m, n;

				float32	**buf_v = new float32*[m_dim];
				for(id = 0; id < m_dim; id++)
				{
					buf_v[id] = new float32[3];
				}
				DMatrixF						mat1;				mat1.setDimension(m_dim, m_dim, false);		
				DVectorF						vec1;				vec1.setDim(m_dim);									
				DVectorI						m_indices1;			m_indices1.setDim((card32)m_array.size());							
				DVectorI						m_mul1;				m_mul1.setDim((card32)m_array.size());
				float32							sumSqrNorm = 0.0;

				for(id = 0; id < m_array.size(); id++)
				{
					if(m_array[id] == surfelIndex)
					{
						for(m = 0; m < 3; m++)
						{
							for(n = 0; n < 3; n++)
							{
								buf_v[3*id + m][n] = lambda*cen_pik[m]*cen_pik[n];
							}
							buf_v[3*id+m][m] += 1.0f;
						}
					}
					else
					{
						pjk = ps->get3f(patchesFrameIndex, m_array[id], POSITION);
						rijk = pik - pjk;
						sumSqrNorm += rijk.getSqrNorm();
						for(m = 0; m < 3; m++)
						{
							for(n = 0; n < 3; n++)
							{
								buf_v[3*id + m][n] = -lambda*rijk[m]*cen_pik[n]/(m_array.size()-1);
							}
							buf_v[3*id+m][m] -= 1.0f/(m_array.size()-1);
						}
					}
					ik_posid = ps->get<int32>(patchesFrameIndex, m_array[id], POS_INDEX);
					m_indices1[id] = ik_posid;
					m_mul1[id] = 3;
				}
				sumSqrNorm /= m_array.size() - 1;
				float32	weight = exp(lambda*sumSqrNorm)*w_lap_s;
				weight = 0.0f;
				//			if(!ik_need_smoothness_flag||!jk_need_smoothness_flag)
				//				weight /= 100.0f;

				potential += cen_pik.getSqrNorm()*weight;
				for(k = 0; k < m_dim; k++)
				{
					vec1[k] = -(cen_pik[0]*buf_v[k][0]+cen_pik[1]*buf_v[k][1]+cen_pik[2]*buf_v[k][2])*weight;	
					for(j = 0; j < m_dim; j++)
					{
						mat1[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
					}
				}
				UpdateMatrix_by_a_matrix1(&mat1, &m_indices1, & m_mul1, (card32)m_array.size(), quadApprox);
				UpdateVector_by_a_vector1(&vec1, &m_indices1, & m_mul1, (card32)m_array.size(), gradient);


				for(id = 0; id < m_dim; id++)
				{
					delete []buf_v[id];
				}
				delete []buf_v;
				
			}
			if (parameters->outputVertexPotentials && SMOOTH_POT != NULL_AAT) {
			   ps->set1f(patchesFrameIndex, surfelIndex, SMOOTH_POT, potential - lastPotential);
			}
		}
	}
}


/// Normal rigidity potential

void PGOpt2RigidityPotential::quadraticApprox(
	const PGOpt2Parameters		*parameters,
	const AnimationBSRContext   *recParameters,
	const InCorePCTopologyGraph *topology,
	OptimizationGraph           *optGraph,
	float32                     &potential,
	DVectorF                    *gradient,
	SparseMatrixF               *quadApprox)
{
	potential = 0.0f;
	
	card32		numOfLinks	=	optGraph->getNumEdges();
	
	/// Buffers for updating sparse matrix 
	DMatrixF		mat;				mat.setDimension(15, 15, false);		
	DVectorF	vec;				vec.setDim(15);									
	DVectorI		m_indices;		m_indices.setDim(5);							
	DVectorI		m_mul;			m_mul.setDim(5);

	
	/// Variables to be used
	Vector3f		pikp,nikp,Eikp1,Eikp2,AikpEikp1,AikpEikp2, 
						pjkp,njkp,Ejkp1,Ejkp2,AikpEjkp1,AikpEjkp2,
						pik,nik,Eik1,Eik2,pjk,njk,Ejk1,Ejk2,
						rijk,rijkp,rijkpc,nijk,nijkp,nijkpc,
						dev_pijk, dev_nijk;
	
	/// Indies
	int32		ik_posid, ik_norid, ikp_posid, ikp_norid, ikp_rotid,
					ik_depth, jk_depth, ikp_depth, jkp_depth,
					jk_posid, jk_norid, jkp_posid, jkp_norid;

	/// Current term rotation matrix
	Matrix3f		Aikp;
	card32		j, k;

	for(card32	edgeIndex = 0; edgeIndex < numOfLinks; edgeIndex++)
	{
		PGOpt2FramePair *edge = optGraph->getEdge(edgeIndex);
   	float32 w_pos_r = parameters->weightPositionRigidity * edge->getRigidityWeight(),			
   	        w_nor_r = parameters->weightNormalRigidity * edge->getRigidityWeight();

		//This maybe modified accordingly
		card32	sourceFrameIndex = edge->getOptFrameIndex1();
		card32	targetFrameIndex = edge->getOptFrameIndex2();
		card32	sourcePatchFrameIndex = optGraph->getFrame(sourceFrameIndex)->getPatchesFrameIndex();
		card32	targetPatchFrameIndex = optGraph->getFrame(targetFrameIndex)->getPatchesFrameIndex();

		PointSet	*sourcePS	= optGraph->getFrame(sourceFrameIndex)->getPatches();
		PointSet	*targetPS	= optGraph->getFrame(targetFrameIndex)->getPatches();
		PointSet	*edge_ps	= edge->getPatchPairsAttribs();
		pAssert(edge_ps->providesAttribute("rotation_matrix")&& edge_ps->providesAttribute("varIndex_rot"));

		/// Attributes
		AAT			POSITION	= sourcePS->getAAT("position"),			POS_INDEX	= sourcePS->getAAT("varIndex_pos"),
						NORMAL		= sourcePS->getAAT("normal"),			NOR_INDEX	= sourcePS->getAAT("varIndex_nor"),
						REF_U			= sourcePS->getAAT("tangent_u"),		TOP_DEPTH	= sourcePS->getAAT("topology_depth"),
						ROTATION	= edge_ps->getAAT("rotation_matrix"),ROT_INDEX	= edge_ps->getAAT("varIndex_rot");

      AAT RIGID_POT = NULL_AAT;
      if (targetPS->providesAttribute("rigidityPot", 1, VAD::DATA_FORMAT_FLOAT32)) {
         RIGID_POT = targetPS->getAAT("rigidityPot");
      }


		vector<card32>	patchIndices = edge->getPatchIndices();

		for(card32 id = 0; id < patchIndices.size(); id++)
		{
		   float32 lastPotential = potential;
			card32	surfelIndex = patchIndices[id];
			ik_depth		=	targetPS->get<int32>(targetPatchFrameIndex, surfelIndex, TOP_DEPTH);
			if(ik_depth >= 2)
				continue;

			/// Get the information of the previous surfel
			pikp				= sourcePS->get3f(sourcePatchFrameIndex, surfelIndex,	POSITION);
			nikp				= sourcePS->get3f(sourcePatchFrameIndex, surfelIndex,  NORMAL); 
			Eikp1			= sourcePS->get3f(sourcePatchFrameIndex, surfelIndex, REF_U);
			Eikp2			= nikp.crossProduct(Eikp1);

			ikp_posid	= sourcePS->get<int32>(sourcePatchFrameIndex, surfelIndex, POS_INDEX);
			ikp_norid		= sourcePS->get<int32>(sourcePatchFrameIndex, surfelIndex, NOR_INDEX);
			ikp_depth	= sourcePS->get<int32>(sourcePatchFrameIndex, surfelIndex, TOP_DEPTH);
			
			/// Get the current rotation matrix
			char			*dataC	= edge_ps->getDataPointer(0, surfelIndex);
			float32	*quatc	= (float*)(dataC+ROTATION.getOffset());
			Quaternion_To_Matrix3f_Opt2(quatc, Aikp);
			ikp_rotid					= edge_ps->get<int32>(0, surfelIndex, ROT_INDEX);

			AikpEikp1	= Aikp*Eikp1;
			AikpEikp2	= Aikp*Eikp2;
			
			/// Get the information of the current surfel
			
			pik			= targetPS->get3f(targetPatchFrameIndex, surfelIndex,	POSITION);
			nik			= targetPS->get3f(targetPatchFrameIndex, surfelIndex,  NORMAL);  pAssert(fabs(1.0f-norm(nik)) < 0.1f);
			Eik1		= targetPS->get3f(targetPatchFrameIndex, surfelIndex, REF_U);       pAssert(fabs(1.0f-norm(Eik1)) < 0.1f);
			Eik2		= nik.crossProduct(Eik1);
			ik_posid	= targetPS->get<int32>(targetPatchFrameIndex, surfelIndex, POS_INDEX);
			ik_norid	= targetPS->get<int32>(targetPatchFrameIndex, surfelIndex, NOR_INDEX);

			/// get the number of neighboring vertices
			card32		numOfEntries = topology->getNumAdjacentVertices(surfelIndex);
         pAssert(numOfEntries > 0);
			for(card32 ni = 0; ni < numOfEntries; ni++)
			{
				/// the index of this neighbor
				card32 neighbor_id	= topology->getAdjacentVertex(surfelIndex, ni);
				if(neighbor_id == surfelIndex)
				{
					continue;
				}

				jk_depth	=	targetPS->get<int32>(targetPatchFrameIndex, neighbor_id, TOP_DEPTH);
				if(jk_depth >= 2)
				{
					continue;
				}

				/// get the information of the previous surfel
				pjkp		=	sourcePS->get3f(sourcePatchFrameIndex, neighbor_id,	POSITION);
				njkp		=	sourcePS->get3f(sourcePatchFrameIndex, neighbor_id,  NORMAL); pAssert(fabs(1.0f-norm(njkp)) < 0.1f);
				Ejkp1		=	sourcePS->get3f(sourcePatchFrameIndex, neighbor_id, REF_U);   pAssert(fabs(1.0f-norm(Ejkp1)) < 0.1f);
				Ejkp2		=	njkp.crossProduct(Ejkp1);
				AikpEjkp1	=	Aikp*Ejkp1;
				AikpEjkp2	=	Aikp*Ejkp2;

				jkp_posid	=  sourcePS->get<int32>(sourcePatchFrameIndex, neighbor_id, POS_INDEX);
				jkp_norid	=	sourcePS->get<int32>(sourcePatchFrameIndex, neighbor_id, NOR_INDEX);
				jkp_depth	=	sourcePS->get<int32>(sourcePatchFrameIndex, neighbor_id, TOP_DEPTH);

				/// get the information of the current surfel
				pjk			=	targetPS->get3f(targetPatchFrameIndex, neighbor_id,	POSITION);
				njk			=	targetPS->get3f(targetPatchFrameIndex, neighbor_id,  NORMAL);   pAssert(fabs(1.0f-norm(njk)) < 0.1f);
				Ejk1		=	targetPS->get3f(targetPatchFrameIndex, neighbor_id, REF_U);        pAssert(fabs(1.0f-norm(Ejk1)) < 0.1f);
				Ejk2		=	njk.crossProduct(Ejk1);

				jk_posid	=	targetPS->get<int32>(targetPatchFrameIndex, neighbor_id, POS_INDEX);
				jk_norid	=	targetPS->get<int32>(targetPatchFrameIndex, neighbor_id, NOR_INDEX);

				/// precomputed geometric quantities 
				rijk			=	pik	-	pjk;
				nijk			=	nik	-	njk;
				
				rijkp			=	pikp		-	pjkp;
				rijkpc		=	Aikp*rijkp;
				nijkp		=	nikp		-	njkp;
				nijkpc		=	Aikp*nijkp;
				dev_pijk	=	rijkpc	-	rijk;
				dev_nijk	=	nijkpc	-	nijk;

				float32	weight;

				/// Position rigidity potential
				if(ikp_rotid>=0 && ikp_depth <= 1 && jkp_depth <= 1)
				{
					weight = w_pos_r/numOfEntries;
					float32	buf_v[15][3];
					m_mul[0] = m_mul[1] = m_mul[2] = m_mul[3] = m_mul[4] = 3;
					if(surfelIndex < neighbor_id)
					{
						if(ikp_posid < ik_posid)
						{
							for(k = 0; k < 3; k++)
							{
								for(j = 0; j < 3; j++)
								{
									buf_v[k][j] = Aikp[k][j];
									buf_v[3+k][j] = -buf_v[k][j];
									buf_v[6+k][j] = 0.0f;			buf_v[9+k][j] = 0.0f;
								}
								buf_v[6+k][k] = -1.0f;			buf_v[9+k][k] = 1.0f;
							}
							m_indices[0] = ikp_posid;			m_indices[1] = jkp_posid;
							m_indices[2] = ik_posid;				m_indices[3] = jk_posid;
						}
						else
						{
							for(k = 0; k < 3; k++)
							{
								for(j = 0; j < 3; j++)
								{
									buf_v[6+k][j] = Aikp[k][j];
									buf_v[9+k][j] = -buf_v[k][j];
									buf_v[k][j] = 0.0f;			buf_v[3+k][j] = 0.0f;
								}
								buf_v[k][k] = -1.0f;			buf_v[3+k][k] = 1.0f;
							}
							m_indices[2] = ikp_posid;			m_indices[3] = jkp_posid;
							m_indices[0] = ik_posid;			m_indices[1] = jk_posid;
						}
					}
					else
					{
						if(ikp_posid < ik_posid)
						{
							for(k = 0; k < 3; k++)
							{
								for(j = 0; j < 3; j++)
								{
									buf_v[3+k][j] = Aikp[k][j];
									buf_v[k][j] = -buf_v[3+k][j];
									buf_v[9+k][j] = 0.0f;			buf_v[6+k][j] = 0.0f;
								}
								buf_v[9+k][k] = -1.0f;			buf_v[6+k][k] = 1.0f;
							}
							m_indices[1] = ikp_posid;			m_indices[0] = jkp_posid;
							m_indices[3] = ik_posid;				m_indices[2] = jk_posid;
						}
						else
						{
							for(k = 0; k < 3; k++)
							{
								for(j = 0; j < 3; j++)
								{
									buf_v[9+k][j] = Aikp[k][j];
									buf_v[6+k][j] = -buf_v[3+k][j];
									buf_v[3+k][j] = 0.0f;			buf_v[k][j] = 0.0f;
								}
								buf_v[3+k][k] = -1.0f;			buf_v[k][k] = 1.0f;
							}
							m_indices[3] = ikp_posid;			m_indices[2] = jkp_posid;
							m_indices[1] = ik_posid;			m_indices[0] = jk_posid;
						}
					}
					buf_v[12][0] = 0.0f;			buf_v[12][1] = -rijkpc[2];	buf_v[12][2] = rijkpc[1];
					buf_v[13][0] = rijkpc[2];	buf_v[13][1] = 0.0f;			buf_v[13][2] = -rijkpc[0];
					buf_v[14][0] = -rijkpc[1];	buf_v[14][1] = rijkpc[0];	buf_v[14][2] = 0.0f;
					m_indices[4] = ikp_rotid;

					potential += dev_pijk.getSqrNorm()*weight;
					for(k = 0; k < 15; k++)
					{
						vec[k] = -(dev_pijk[0]*buf_v[k][0]+dev_pijk[1]*buf_v[k][1]+dev_pijk[2]*buf_v[k][2])*weight;
						for(j = 0; j < 15; j++)
						{
							mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*weight;
						}
					}
					UpdateMatrix_by_a_matrix1(&mat, &m_indices, & m_mul, 5, quadApprox);
					UpdateVector_by_a_vector1(&vec, &m_indices, & m_mul, 5, gradient);
				}

				/// Normal rigidity potential
				if(ikp_rotid>=0 && ikp_depth <= 1 && jkp_depth <= 1)
				{
					weight	 = w_nor_r/numOfEntries;
					float32	buf_v[11][3];
					m_mul[0] = m_mul[1] = m_mul[2] = m_mul[3] = 2;	m_mul[4] = 3;
					if(surfelIndex < neighbor_id)
					{
						if(ikp_norid < ik_norid)
						{
							for(j = 0; j < 3; j++)
							{
								buf_v[0][j] = AikpEikp1[j];
								buf_v[1][j] = AikpEikp2[j];
								buf_v[2][j] = -AikpEjkp1[j];
								buf_v[3][j] = -AikpEjkp2[j];
								buf_v[4][j] = -Eik1[j];
								buf_v[5][j] = -Eik2[j];
								buf_v[6][j] = Ejk1[j];
								buf_v[7][j] = Ejk2[j];
							}
							m_indices[0] = ikp_norid;				m_indices[1] = jkp_norid;
							m_indices[2] = ik_norid;				m_indices[3] = jk_norid;
						}
						else
						{
							for(j = 0; j < 3; j++)
							{
								buf_v[4][j] = AikpEikp1[j];
								buf_v[5][j] = AikpEikp2[j];
								buf_v[6][j] = -AikpEjkp1[j];
								buf_v[7][j] = -AikpEjkp2[j];
								buf_v[0][j] = -Eik1[j];
								buf_v[1][j] = -Eik2[j];
								buf_v[2][j] = Ejk1[j];
								buf_v[3][j] = Ejk2[j];
							}
							m_indices[2] = ikp_norid;				m_indices[3] = jkp_norid;
							m_indices[0] = ik_norid;					m_indices[1] = jk_norid;
						}
					}
					else
					{
						if(ikp_norid < ik_norid)
						{
							for(j = 0; j < 3; j++)
							{
								buf_v[2][j] = AikpEikp1[j];
								buf_v[3][j] = AikpEikp2[j];
								buf_v[0][j] = -AikpEjkp1[j];
								buf_v[1][j] = -AikpEjkp2[j];
								buf_v[6][j] = -Eik1[j];
								buf_v[7][j] = -Eik2[j];
								buf_v[4][j] = Ejk1[j];
								buf_v[5][j] = Ejk2[j];
							}
							m_indices[1] = ikp_norid;				m_indices[0] = jkp_norid;
							m_indices[3] = ik_norid;				m_indices[2] = jk_norid;
						}
						else
						{
							for(j = 0; j < 3; j++)
							{
								buf_v[6][j] = AikpEikp1[j];
								buf_v[7][j] = AikpEikp2[j];
								buf_v[4][j] = -AikpEjkp1[j];
								buf_v[5][j] = -AikpEjkp2[j];
								buf_v[2][j] = -Eik1[j];
								buf_v[3][j] = -Eik2[j];
								buf_v[0][j] = Ejk1[j];
								buf_v[1][j] = Ejk2[j];
							}
							m_indices[3] = ikp_norid;				m_indices[2] = jkp_norid;
							m_indices[1] = ik_norid;				m_indices[0] = jk_norid;
						}
					}
					buf_v[8][0] = 0.0f;				buf_v[8][1] = -nijkpc[2];	buf_v[8][2] = nijkpc[1];
					buf_v[9][0] = nijkpc[2];		buf_v[9][1] = 0.0f;				buf_v[9][2] = -nijkpc[0];
					buf_v[10][0] = -nijkpc[1];	buf_v[10][1] = nijkpc[0];	buf_v[10][2] = 0.0f;
					m_indices[4] = ikp_rotid;

					potential += dev_nijk.getSqrNorm()*weight;
					for(k = 0; k < 11; k++)
					{
						vec[k] = -(dev_nijk[0]*buf_v[k][0] + dev_nijk[1]*buf_v[k][1] + dev_nijk[2]*buf_v[k][2])*weight;	
						for(j = 0; j < 11; j++)
						{
							mat[k][j] = (buf_v[k][0]*buf_v[j][0] + buf_v[k][1]*buf_v[j][1] + buf_v[k][2]*buf_v[j][2])*weight;
						}
					}
					UpdateMatrix_by_a_matrix1(&mat, &m_indices, & m_mul, 5, quadApprox);
					UpdateVector_by_a_vector1(&vec, &m_indices, & m_mul, 5, gradient);
				}
			}

			if (parameters->outputVertexPotentials && RIGID_POT != NULL_AAT) {
			   targetPS->set1f(targetPatchFrameIndex, surfelIndex, RIGID_POT, potential - lastPotential);
			}

		}
	}
}

/// Position temporal coherence

void PGOpt2TemporalCoherence::quadraticApprox(
	const PGOpt2Parameters     *parameters,
	const AnimationBSRContext   *recParameters,
	const InCorePCTopologyGraph *topology,
	OptimizationGraph           *optGraph,
	float32                     &potential,
	DVectorF                    *gradient,
	SparseMatrixF               *quadApprox)
{
	potential = 0.0f;
	/// The dimension of the patch graph
	card32								width				= optGraph->getNumFrames();
	if(width < 3)
		return;

	float32		w_stpc		= parameters->weightPositionCoherence,		w_stnc		= parameters->weightNormalCoherence,
						w_strc		= parameters->weightRotationCoherence;

	/// Controls the smoothness
	float32		lambda	=	parameters->weightlambda;


	/// Buffers for updating sparse matrix 
	DMatrixF		mat;				mat.setDimension(9, 9, false);		
	DVectorF	vec;				vec.setDim(9);									
	DVectorI		m_indices;		m_indices.setDim(4);							
	DVectorI		m_mul;			m_mul.setDim(4);

		
	/// Variables to be used
	Vector3f		pikp,nikp,Eikp1,Eikp2,
						pik, nik, Eik1, Eik2,
						pikn,nikn,Eikn1,Eikn2,
						AikpEikp1,AikpEikp2,
						AikEikp1, AikEikp2, 
						AikEik1,AikEik2,
						rikpk, nikpk, rikpkc, nikpkc, rikkn, nikkn, 
						dev_pikkpkn, dev_nikkpkn;

	/// In dices
	int32		ik_posid, ik_norid, ikp_posid, ikp_norid, ikp_rotid, 
					ik_depth, ikp_depth, 
					ikn_posid, ikn_norid, ik_rotid, ikn_depth;

	/// Current term rotation matrix
	Matrix3f		Aikp, Aik;

	card32		j, k;

	for(card32 interLinkIndex = 0; interLinkIndex < optGraph->getNumInterEdgeLinks(); interLinkIndex++)
	{
		PGOpt2FramePairPair	*interLink = optGraph->getInterEdgeLink(interLinkIndex);

		card32	edgeIndex0 = interLink->getOptFramePairIndex1();
		card32	edgeIndex1 = interLink->getOptFramePairIndex2();
		
		PGOpt2FramePair	*link0	 = optGraph->getEdge(edgeIndex0);
		PGOpt2FramePair	*link1	 = optGraph->getEdge(edgeIndex1);

		PointSet		*linkPS0 = link0->getPatchPairsAttribs();
		PointSet		*linkPS1 = link1->getPatchPairsAttribs();

		card32	frameIndex0		 = link0->getOptFrameIndex1();
		card32	frameIndex1		 = link0->getOptFrameIndex2();
		card32	frameIndex2		 = link1->getOptFrameIndex2();
		card32	patchesFrameIndex0 = optGraph->getFrame(frameIndex0)->getPatchesFrameIndex();
		card32	patchesFrameIndex1 = optGraph->getFrame(frameIndex1)->getPatchesFrameIndex();
		card32	patchesFrameIndex2 = optGraph->getFrame(frameIndex2)->getPatchesFrameIndex();

		pAssert(frameIndex2 == frameIndex1 + 1
			&& frameIndex0 == frameIndex1 - 1);

		PointSet	*framePS0	 = optGraph->getFrame(frameIndex0)->getPatches();
		PointSet	*framePS1	 = optGraph->getFrame(frameIndex1)->getPatches();
		PointSet	*framePS2	 = optGraph->getFrame(frameIndex2)->getPatches();

		/// Attributes
		AAT			POSITION	= framePS0->getAAT("position"),			POS_INDEX	= framePS0->getAAT("varIndex_pos"),
						NORMAL		= framePS0->getAAT("normal"),			NOR_INDEX	= framePS0->getAAT("varIndex_nor"),
						REF_U			= framePS0->getAAT("tangent_u"),		TOP_DEPTH	= framePS0->getAAT("topology_depth"),
						ROTATION	= linkPS0->getAAT("rotation_matrix"),	ROT_INDEX	= linkPS0->getAAT("varIndex_rot");
		
		vector<card32>	patchIndices = interLink->getPatchIndices();

		for(card32 id = 0; id < patchIndices.size(); id++)
		{
			card32	surfelIndex = patchIndices[id];
			ik_depth	=	framePS1->get<int32>(patchesFrameIndex1, surfelIndex, TOP_DEPTH);
			if(ik_depth >= 2)
				continue;
			
			/// Information of the previous surfel
			pikp			= framePS0->get3f(patchesFrameIndex0, surfelIndex,	POSITION);
			nikp			= framePS0->get3f(patchesFrameIndex0, surfelIndex,	NORMAL); pAssert(fabs(1.0f-norm(nikp)) < 0.1f);
			Eikp1		= framePS0->get3f(patchesFrameIndex0, surfelIndex,	REF_U);     pAssert(fabs(1.0f-norm(Eikp1)) < 0.1f);
			Eikp2		= nikp.crossProduct(Eikp1);

			ikp_posid		= framePS0->get<int32>(patchesFrameIndex0, surfelIndex, POS_INDEX);
			ikp_norid			= framePS0->get<int32>(patchesFrameIndex0, surfelIndex, NOR_INDEX);
			ikp_depth		= framePS0->get<int32>(patchesFrameIndex0, surfelIndex, TOP_DEPTH);

			char	*dataC	= linkPS0->getDataPointer(0, surfelIndex);
			float32	*quatc	= (float*)(dataC+ROTATION.getOffset());
			Quaternion_To_Matrix3f_Opt2(quatc, Aikp);
			ikp_rotid		= linkPS0->get<int32>(0, surfelIndex, ROT_INDEX);

			AikpEikp1	= Aikp*Eikp1;
			AikpEikp2	= Aikp*Eikp2;

			/// Information of the current surfel
			ik_posid	= framePS1->get<int32>(patchesFrameIndex1, surfelIndex, POS_INDEX);
			ik_norid	= framePS1->get<int32>(patchesFrameIndex1, surfelIndex, NOR_INDEX);
			pik			= framePS1->get3f(patchesFrameIndex1, surfelIndex,	  POSITION);
			nik			= framePS1->get3f(patchesFrameIndex1, surfelIndex,	  NORMAL);    pAssert(fabs(1.0f-norm(nik)) < 0.1f);
			Eik1			= framePS1->get3f(patchesFrameIndex1, surfelIndex,      REF_U);  pAssert(fabs(1.0f-norm(Eik1)) < 0.1f);
			Eik2			= nik.crossProduct(Eik1);

			/// Information of the next surfel

			pikn			= framePS2->get3f(patchesFrameIndex2, surfelIndex,	  POSITION);
			nikn			= framePS2->get3f(patchesFrameIndex2, surfelIndex,    NORMAL);   pAssert(fabs(1.0f-norm(nikn)) < 0.1f);
			Eikn1		= framePS2->get3f(patchesFrameIndex2, surfelIndex,    REF_U);       pAssert(fabs(1.0f-norm(Eikn1)) < 0.1f);
			Eikn2		= nikn.crossProduct(Eikn1);

			ikn_posid	= framePS2->get<int32>(patchesFrameIndex2, surfelIndex, POS_INDEX);
			ikn_norid		= framePS2->get<int32>(patchesFrameIndex2, surfelIndex, NOR_INDEX);
			ikn_depth	= framePS2->get<int32>(patchesFrameIndex2, surfelIndex, TOP_DEPTH);

			ik_rotid		= linkPS1->get<int32>(0 ,   surfelIndex, ROT_INDEX);
			char	*dataC1	= linkPS1->getDataPointer(0, surfelIndex);
			float32	*quatc1	= (float*)(dataC1+ROTATION.getOffset());
			Quaternion_To_Matrix3f_Opt2(quatc1, Aik);

			AikEik1	=	Aik*Eik1;				AikEik2	= Aik*Eik2;
			AikEikp1	=	Aik*Eikp1;				AikEikp2	= Aik*Eikp2;

			/// Compute inter median variables
			rikpk		=	pikp - pik;				nikpk		= nikp - nik;
			rikkn		=	pik- pikn;				nikkn		= nik - nikn;
			rikpkc		=	Aik*rikpk;				nikpkc		= Aik*nikpk;
			dev_pikkpkn	=	rikpkc - rikkn;
			dev_nikkpkn	=	nikpkc - nikkn;

			if(ikp_depth < 2 && ikn_depth < 2)
			{
				Vector3f		curvik = rikpk - rikkn;
				float32		buf_v[9][3];
				m_mul[0] = m_mul[1] = m_mul[2]  = 3;
				for(k = 0; k < 3; k++)
				{
					for(j = 0; j < 3; j++)
					{
						buf_v[k][j] = 0.0f;
						buf_v[3+k][j] = 0.0f;
						buf_v[6+k][j] = 0.0f;
					}
					buf_v[k][k] = 1.0f;			buf_v[3+k][k] = -2.0f;			buf_v[6+k][k] = 1.0f;
				}
				m_indices[0] = ikp_posid;			m_indices[1] = ik_posid;		m_indices[2] = ikn_posid;
				potential += w_stpc*curvik.getSqrNorm();
				for(k = 0; k < 9; k++)
				{
					vec[k] = -(curvik[0]*buf_v[k][0]+curvik[1]*buf_v[k][1]+curvik[2]*buf_v[k][2])*w_stpc;
					for(j = 0; j < 9; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*w_stpc;
					}
				}
				UpdateMatrix_by_a_matrix1(&mat, &m_indices, & m_mul, 3, quadApprox);
				UpdateVector_by_a_vector1(&vec, &m_indices, & m_mul, 3, gradient);
			}
			if(ikp_depth < 2 && ikn_depth < 2)
			{
				float32	buf_v[9][3];
				m_mul[0] = m_mul[1] = m_mul[2] = 2;	m_mul[3] = 3;
				for(j = 0; j < 3; j++)
				{
					buf_v[0][j] = AikEikp1[j];
					buf_v[1][j] = AikEikp2[j];
					buf_v[2][j] = -AikEik1[j] - Eik1[j];
					buf_v[3][j] = -AikEik2[j] - Eik2[j];
					buf_v[4][j] = Eikn1[j];
					buf_v[5][j] = Eikn2[j];
				}
				m_indices[0] = ikp_norid;			m_indices[1] = ik_norid;			m_indices[3] = ikn_norid;

				buf_v[6][0] = 0.0f;				buf_v[6][1] = -nikpkc[2];	buf_v[6][2] = nikpkc[1];
				buf_v[7][0] = nikpkc[2];		buf_v[7][1] = 0.0f;				buf_v[7][2] = -nikpkc[0];
				buf_v[8][0] = -nikpkc[1];	buf_v[8][1] = nikpkc[0];		buf_v[8][2] = 0.0f;
				m_indices[3] = ik_rotid;
				potential += dev_nikkpkn.getSqrNorm()*w_stnc;
				for(k = 0; k < 9; k++)
				{
					vec[k] = -(dev_nikkpkn[0]*buf_v[k][0]+dev_nikkpkn[1]*buf_v[k][1]+dev_nikkpkn[2]*buf_v[k][2])*w_stnc;	
					for(j = 0; j < 9; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2])*w_stnc;
					}
				}
				UpdateMatrix_by_a_matrix1(&mat, &m_indices, & m_mul, 4, quadApprox);
				UpdateVector_by_a_vector1(&vec, &m_indices, & m_mul, 4, gradient);
			}
			if(ikp_depth < 2)
			{
				float32	buf_v[6][9];
				float32	buf_colume[9];
				m_mul[0] = m_mul[1] = 3;

				for(k = 0; k < 3; k++)
				{
					for(j = 0; j < 3; j++)
					{
						buf_colume[3*k + j] = Aikp[k][j] - Aik[k][j];
					}
					buf_v[0][3*k+0] = 0.0f;				buf_v[0][3*k+1] = -Aikp[k][0];		buf_v[0][3*k+2] = Aikp[k][1];
					buf_v[1][3*k+0] = Aikp[k][0];		buf_v[1][3*k+1] = 0.0f;				buf_v[1][3*k+2] = -Aikp[k][2];
					buf_v[2][3*k+0] = -Aikp[k][1];		buf_v[2][3*k+1] = Aikp[k][2];		buf_v[2][3*k+2] = 0.0f;

					buf_v[3][3*k+0] = 0.0f;				buf_v[3][3*k+1] = Aik[k][0];		buf_v[3][3*k+2] = -Aik[k][1];
					buf_v[4][3*k+0] = -Aik[k][0];		buf_v[4][3*k+1] = 0.0f;				buf_v[4][3*k+2] = Aik[k][2];
					buf_v[5][3*k+0] = Aik[k][1];		buf_v[5][3*k+1] = -Aik[k][2];		buf_v[5][3*k+2] = 0.0f;
				}
				m_indices[0] = ikp_rotid;			m_indices[1] = ik_rotid;
				potential += w_strc*(buf_colume[0]*buf_colume[0] + buf_colume[1]*buf_colume[1] + buf_colume[2]*buf_colume[2]
				+ buf_colume[3]*buf_colume[3] + buf_colume[4]*buf_colume[4] + buf_colume[5]*buf_colume[5]
				+ buf_colume[6]*buf_colume[6] + buf_colume[7]*buf_colume[7] + buf_colume[8]*buf_colume[8]);
				for(k = 0; k < 6; k++)
				{
					vec[k] = -(buf_colume[0]*buf_v[k][0]+buf_colume[1]*buf_v[k][1]+buf_colume[2]*buf_v[k][2]
					+ buf_colume[3]*buf_v[k][3]+buf_colume[4]*buf_v[k][4]+buf_colume[5]*buf_v[k][5]
					+ buf_colume[6]*buf_v[k][6]+buf_colume[7]*buf_v[k][7]+buf_colume[8]*buf_v[k][8])*w_strc;	
					for(j = 0; j < 6; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0]+buf_v[k][1]*buf_v[j][1]+buf_v[k][2]*buf_v[j][2]
						+buf_v[k][3]*buf_v[j][3]+buf_v[k][4]*buf_v[j][4]+buf_v[k][5]*buf_v[j][5]
						+buf_v[k][6]*buf_v[j][6]+buf_v[k][7]*buf_v[j][7]+buf_v[k][8]*buf_v[j][8])*w_strc;
					}
				}
				UpdateMatrix_by_a_matrix1(&mat, &m_indices, &m_mul, 2, quadApprox);
				UpdateVector_by_a_vector1(&vec, &m_indices, &m_mul, 2, gradient);
			}
		}
	}
}

/// Temporal color	similarity

void PGOpt2STColorSimilarity::quadraticApprox(
	const PGOpt2Parameters     *parameters,
	const AnimationBSRContext   *recParameters,
	const InCorePCTopologyGraph *topology,
	OptimizationGraph           *optGraph,
	float32                     &potential,
	DVectorF                    *gradient,
	SparseMatrixF               *quadApprox)
{
	potential		= 0.0f;
		
	card32	width	= optGraph->getNumFrames();

	vector< vector<int32> >	neighbors_id;
	neighbors_id.resize(width);

	for(card32 frameIndex = 0; frameIndex < width; frameIndex++)
	{
		PGOpt2Frame	*frame	             = optGraph->getFrame(frameIndex);
		card32 patchesFrameIndex          = frame->getPatchesFrameIndex();
		PointSet			*ps			       = frame->getPatches();
		PointCloudAnimationSequence *anim = frame->getData();
		if (anim == NULL) continue;
		PointSet			*dataPS = anim->getFrame(frame->getDataFrameIndex());

		pAssert(dataPS->providesAttribute("color")&&dataPS->providesAttribute("gradient_color_r")
			&& dataPS->providesAttribute("gradient_color_g") && dataPS->providesAttribute("gradient_color_b"));

		AAT		DATA_POS			= dataPS->getAAT("position");
		AAT		COLOR					= dataPS->getAAT("color");
		AAT		GRA_COLOR_R	= dataPS->getAAT("gradient_color_r");
		AAT		GRA_COLOR_G	= dataPS->getAAT("gradient_color_g");
		AAT		GRA_COLOR_B	= dataPS->getAAT("gradient_color_b");

		AAT		POSITION			= ps->getAAT("position");
		AAT		POS_INDEX		= ps->getAAT("varIndex_pos");


		neighbors_id[frameIndex].resize(ps->getHeight());
		
		AttachedIndexedOctree *dataOctree = frame->getData()->fetchOctree(frame->getDataFrameIndex());
		PointSetKNNQuery data_knn(dataPS, dataOctree, NULL_VECTOR3F, DATA_POS);

		card32	foot_id;

		set<float32>		buf_sqr2;
		for(card32 surfelIndex = 0; surfelIndex < ps->getHeight(); surfelIndex++)
		{
			Vector3f		pik = ps->get3f(patchesFrameIndex, surfelIndex, POSITION);
			Vector3f		dik;
			data_knn.setTargetPoint(pik);
			{
				bool found;
				data_knn.getNextNearestPointIndex(foot_id, found);
				pAssert(found);
				neighbors_id[frameIndex][surfelIndex] = foot_id;
				dik = dataPS->get3f(0, foot_id, DATA_POS);
			}
			Vector3f		devik = pik - dik;
			buf_sqr2.insert(devik.getSqrNorm());
		}
		set<float32>::iterator	iter_set = buf_sqr2.begin();
		for(card32 ni = 0; ni < card32(0.4*ps->getHeight()); ni++)
		{
			iter_set++;
		}
		float32	mean_d2 = (*iter_set)*2;

		// Do linear regression to find overlapping points
		for(card32 surfelIndex = 0; surfelIndex < ps->getHeight(); surfelIndex++)
		{
			Vector3f		pik = ps->get3f(patchesFrameIndex, surfelIndex, POSITION);
			Vector3f		dik = dataPS->get3f(0, neighbors_id[frameIndex][surfelIndex], DATA_POS);
			Vector3f		devik = pik - dik;
			if(devik.getSqrNorm() > mean_d2)
			{
				neighbors_id[frameIndex][surfelIndex] = -1;
			}
		}
	}
	Vector3f	pik, pikn, cik, cikn, devcikikn,
				gra_cik_r, gra_cik_g, gra_cik_b, 
				gra_cikn_r, gra_cikn_g, gra_cikn_b;
	int32		ik_posid, ikn_posid;

	/// Buffers for updating sparse matrix 
	DMatrixF	mat;		mat.setDimension(6, 6, false);		
	DVectorF	vec;		vec.setDim(6);									
	DVectorI	m_indices;	m_indices.setDim(2);							
	DVectorI	m_mul;		m_mul.setDim(2);

	for(card32 linkIndex = 0; linkIndex < optGraph->getNumEdges(); linkIndex++)
	{
		PGOpt2FramePair	*link= optGraph->getEdge(linkIndex);
		card32	frameIndex0	= link->getOptFrameIndex1();
		card32	frameIndex1	= link->getOptFrameIndex2();
	   if (optGraph->getFrame(frameIndex0)->getData() == NULL) continue;
	   if (optGraph->getFrame(frameIndex1)->getData() == NULL) continue;
      card32   patchesFrameIndex0 = optGraph->getFrame(frameIndex0)->getPatchesFrameIndex();
      card32   patchesFrameIndex1 = optGraph->getFrame(frameIndex1)->getPatchesFrameIndex();
      
		PointSet*	framePS0	= optGraph->getFrame(frameIndex0)->getPatches();
		PointSet*	framePS1	= optGraph->getFrame(frameIndex1)->getPatches();
	
		PointSet*	dataPS0		= optGraph->getFrame(frameIndex0)->getData()->getFrame(optGraph->getFrame(frameIndex0)->getDataFrameIndex());
		PointSet*	dataPS1		= optGraph->getFrame(frameIndex1)->getData()->getFrame(optGraph->getFrame(frameIndex1)->getDataFrameIndex());
		card32      dataPS0FrameIndex = optGraph->getFrame(frameIndex0)->getDataFrameIndex();
		card32      dataPS1FrameIndex = optGraph->getFrame(frameIndex1)->getDataFrameIndex();

		AAT		POSITION			= framePS0->getAAT("position");
		AAT		POS_INDEX		= framePS0->getAAT("varIndex_pos");
		AAT		DATA_POS			= dataPS0->getAAT("position");
		AAT		COLOR					= dataPS0->getAAT("color");
		AAT		GRA_COLOR_R	= dataPS0->getAAT("gradient_color_r");
		AAT		GRA_COLOR_G	= dataPS0->getAAT("gradient_color_g");
		AAT		GRA_COLOR_B	= dataPS0->getAAT("gradient_color_b");

		vector<card32>	patchIndices = link->getPatchIndices();
		for(card32 id = 0; id < patchIndices.size(); id++)
		{
			card32	surfelIndex = patchIndices[id];
			
			pik				= framePS0->get3f(patchesFrameIndex0, surfelIndex, POSITION);
			ik_posid		= framePS0->get<int32>(patchesFrameIndex0, surfelIndex, POS_INDEX);

			pikn			= framePS1->get3f(patchesFrameIndex1, surfelIndex, POSITION);
			ikn_posid		= framePS1->get<int32>(patchesFrameIndex1, surfelIndex, POS_INDEX);
			
			if(neighbors_id[frameIndex0][surfelIndex] >= 0 
				&& neighbors_id[frameIndex1][surfelIndex] >= 0
				&& ik_posid >= 0 && ikn_posid >= 0)
			{
				float32		buf_v[6][3];
				cik			= dataPS0->get3f(dataPS0FrameIndex,	neighbors_id[frameIndex0][surfelIndex], COLOR);
				gra_cik_r	= dataPS0->get3f(dataPS0FrameIndex,	neighbors_id[frameIndex0][surfelIndex], GRA_COLOR_R);
				gra_cik_g	= dataPS0->get3f(dataPS0FrameIndex,	neighbors_id[frameIndex0][surfelIndex], GRA_COLOR_G);
				gra_cik_b	= dataPS0->get3f(dataPS0FrameIndex,	neighbors_id[frameIndex0][surfelIndex], GRA_COLOR_B);
				
				cikn		   = dataPS1->get3f(dataPS1FrameIndex, neighbors_id[frameIndex1][surfelIndex], COLOR);
				gra_cikn_r	= dataPS1->get3f(dataPS1FrameIndex, neighbors_id[frameIndex1][surfelIndex], GRA_COLOR_R);
				gra_cikn_g	= dataPS1->get3f(dataPS1FrameIndex, neighbors_id[frameIndex1][surfelIndex], GRA_COLOR_G);
				gra_cikn_b	= dataPS1->get3f(dataPS1FrameIndex, neighbors_id[frameIndex1][surfelIndex], GRA_COLOR_B);
				devcikikn	= cik - cikn;

				for(card32 k = 0; k < 3; k++)
				{
					buf_v[k][0]		= gra_cik_r[k];
					buf_v[k][1]		= gra_cik_g[k];
					buf_v[k][2]		= gra_cik_b[k];
					buf_v[3+k][0]	= -gra_cikn_r[k];
					buf_v[3+k][1]	= -gra_cikn_g[k];
					buf_v[3+k][2]	= -gra_cikn_b[k];
				}
				potential += devcikikn.getSqrNorm()*parameters->weightColorSimilarity;
				for(card32 k = 0; k < 6; k++)
				{
					vec[k] = -(buf_v[k][0]*devcikikn[0] + buf_v[k][1]*devcikikn[1] + buf_v[k][2]*devcikikn[2])*parameters->weightColorSimilarity;
					for(card32 j = 0; j < 6; j++)
					{
						mat[k][j] = (buf_v[k][0]*buf_v[j][0] + buf_v[k][1]*buf_v[j][1] + buf_v[k][2]*buf_v[j][2])*parameters->weightColorSimilarity;
					}
				}
				m_mul[0]  = m_mul[1] = 3;
				m_indices[0] = ik_posid;
				m_indices[1] = ikn_posid;
				UpdateMatrix_by_a_matrix1(&mat, &m_indices, & m_mul, 2, quadApprox);
				UpdateVector_by_a_vector1(&vec, &m_indices, & m_mul, 2, gradient);
			}
		}
	}
}
