//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PGOpt2PotentialSelector.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "BooleanClassProperty.h"
#include "EmbeddedObjectClassProperty.h"
#include "EmbeddedObjectClassPropertyEditor.h"
#include "SeparatorClassProperty.h"
#include "PatchGraphOpt2.h"
#include "PGOpt2Potential.h"
#include "PGOpt2Processing.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PGOpt2FramePotentialSelector ,0) {
	BEGIN_CLASS_INIT( PGOpt2FramePotentialSelector );
   ADD_SEPARATOR("likelihood")
   ADD_BOOLEAN_PROP(distancePairwise, 0)
   ADD_BOOLEAN_PROP(distanceGlobalOpt, 0)
   ADD_BOOLEAN_PROP(distanceMarkers, 0)
   ADD_BOOLEAN_PROP(colorSimilarity, 0)
   ADD_SEPARATOR("priors")
   ADD_BOOLEAN_PROP(smoothness, 0)
}

IMPLEMENT_GEOXL_CLASS( PGOpt2FramePairPotentialSelector ,0) {
	BEGIN_CLASS_INIT( PGOpt2FramePairPotentialSelector );
   ADD_BOOLEAN_PROP(rigidity, 0)
}

IMPLEMENT_GEOXL_CLASS( PGOpt2ThreeFramePotentialSelector ,0) {
	BEGIN_CLASS_INIT( PGOpt2ThreeFramePotentialSelector );
   ADD_BOOLEAN_PROP(acceleration, 0)
}

IMPLEMENT_GEOXL_CLASS( PGOpt2PotentialSelector ,0) {
	BEGIN_CLASS_INIT( PGOpt2PotentialSelector );

   ADD_EMBEDDED_OBJECT_PROP(framePotentials, 0, PGOpt2FramePotentialSelector::getClass())
   SET_NAME("frame");
   SET_EDITOR(EmbeddedObjectClassPropertyInlineEditor::getClass());

   ADD_EMBEDDED_OBJECT_PROP(framePairPotentials, 0, PGOpt2FramePairPotentialSelector::getClass())
   SET_NAME("pair");
   SET_EDITOR(EmbeddedObjectClassPropertyInlineEditor::getClass());

   ADD_EMBEDDED_OBJECT_PROP(threeFramePotentials, 0, PGOpt2ThreeFramePotentialSelector::getClass())
   SET_NAME("three");
   SET_EDITOR(EmbeddedObjectClassPropertyInlineEditor::getClass());

   ADD_BOOLEAN_PROP(preprocessingEstimateNormals, 0)
   ADD_BOOLEAN_PROP(preprocessingUnifyNormalOrientation, 0)
   ADD_BOOLEAN_PROP(preprocessingComputeRotations, 0)
   ADD_BOOLEAN_PROP(preprocessingWriteVarIndices, 0)
}

// ----

PGOpt2FramePotentialSelector::PGOpt2FramePotentialSelector() {
   distancePairwise = true;
   distanceGlobalOpt = true;
   distanceMarkers = true; 
   smoothness = true;
   colorSimilarity = false;
}

void PGOpt2FramePotentialSelector::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

PGOpt2FramePotentialSelector::~PGOpt2FramePotentialSelector() {
}

// --

PGOpt2FramePairPotentialSelector::PGOpt2FramePairPotentialSelector() {
   rigidity = true;
}

void PGOpt2FramePairPotentialSelector::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

PGOpt2FramePairPotentialSelector::~PGOpt2FramePairPotentialSelector() {
}

// --

PGOpt2ThreeFramePotentialSelector::PGOpt2ThreeFramePotentialSelector() {
   acceleration = true;
}

void PGOpt2ThreeFramePotentialSelector::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

PGOpt2ThreeFramePotentialSelector::~PGOpt2ThreeFramePotentialSelector() {
}

// --

PGOpt2PotentialSelector::PGOpt2PotentialSelector() {
   preprocessingEstimateNormals = true;
   preprocessingUnifyNormalOrientation = true;
   preprocessingComputeRotations = true;
   preprocessingWriteVarIndices = true;
}

void PGOpt2PotentialSelector::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

void PGOpt2PotentialSelector::configure(PatchGraphOpt2 *opt) {
   opt->clearPotentials();
   if (framePotentials.distanceGlobalOpt) {
      opt->addPotential(new PGOpt2SquaredDistancePotential_Global());
   }
   if (framePotentials.distancePairwise) {
      opt->addPotential(new PGOpt2SquaredDistancePotential_Pairwise());
   }
   if (framePotentials.distanceMarkers) {
      opt->addPotential(new PGOpt2MarkerErrorPotential());
   }
   if (framePotentials.smoothness) {
      opt->addPotential(new PGOpt2SmoothnessPotential());
   }
   if (framePotentials.colorSimilarity) {
      opt->addPotential(new PGOpt2STColorSimilarity());
   }
   if (framePairPotentials.rigidity) {
      opt->addPotential(new PGOpt2RigidityPotential());
   }
   if (threeFramePotentials.acceleration) {
      opt->addPotential(new PGOpt2TemporalCoherence());
   }
   opt->clearPreprocessingSteps();
   if (preprocessingEstimateNormals) {
      opt->addPreprocessingStep(new PGOpt2Processing_Normal_Estimation());
   }
   if (preprocessingUnifyNormalOrientation) {
      opt->addPreprocessingStep(new PGOpt2Processing_Normal_Orientation());
   }
   if (preprocessingComputeRotations) {
      opt->addPreprocessingStep(new PGOpt2Processing_Initial_Rotation_Matrices());
   }
   if (preprocessingWriteVarIndices) {
      opt->addPreprocessingStep(new PGOpt2Processing_varIndex());
   }
}

PGOpt2PotentialSelector::~PGOpt2PotentialSelector() {
}


