//---------------------------------------------------------------------------
#ifndef PGOpt2PotentialSelectorH
#define PGOpt2PotentialSelectorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PatchGraphOpt2;

/// component1 : per-frame potentials
class ANIMATIONRECONSTRUCTION_API PGOpt2FramePotentialSelector : public Persistent {
	GEOXL_CLASS( PGOpt2FramePotentialSelector )
 public:
   bool distancePairwise;
   bool distanceGlobalOpt;
   bool distanceMarkers;
   bool colorSimilarity;
   bool smoothness;

	PGOpt2FramePotentialSelector();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   ~PGOpt2FramePotentialSelector();

};

/// component2 : frame-pair potentials
class ANIMATIONRECONSTRUCTION_API PGOpt2FramePairPotentialSelector : public Persistent {
	GEOXL_CLASS( PGOpt2FramePairPotentialSelector )
 public:
   bool rigidity;                            

	PGOpt2FramePairPotentialSelector();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   ~PGOpt2FramePairPotentialSelector();

};

/// component2 : frame-pair potentials
class ANIMATIONRECONSTRUCTION_API PGOpt2ThreeFramePotentialSelector : public Persistent {
	GEOXL_CLASS( PGOpt2ThreeFramePotentialSelector )
 public:
   bool acceleration;

	PGOpt2ThreeFramePotentialSelector();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   ~PGOpt2ThreeFramePotentialSelector();

};

///
///   set of boolean flags to select a subset of potentials
///
class ANIMATIONRECONSTRUCTION_API PGOpt2PotentialSelector : public Persistent {
	GEOXL_CLASS( PGOpt2PotentialSelector )
 public:
   PGOpt2FramePotentialSelector framePotentials;
   PGOpt2FramePairPotentialSelector framePairPotentials;
   PGOpt2ThreeFramePotentialSelector threeFramePotentials;
   
   bool preprocessingEstimateNormals;
   bool preprocessingUnifyNormalOrientation;
   bool preprocessingComputeRotations;
   bool preprocessingWriteVarIndices;
   

	PGOpt2PotentialSelector();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

	virtual void configure(PatchGraphOpt2 *opt);

   ~PGOpt2PotentialSelector();
};

}

#endif