//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PGOpt2Processing.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PGOpt2Frame.h"
#include "PGOpt2FramePair.h"
#include "OptimizationGraph.h"
#include "MgcEigen.h"
#include "InCorePCTopologyGraph.h"
#include "PGOpt2Parameters.h"
#include "PatchGraph.h"
#include "PGOpt2GraphCreator.h"
#include "PatchTypes.h"
#include "PGOpt2FramePairVars.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_ABSTRACT_CLASS( PGOpt2Processing ,0) {
	BEGIN_CLASS_INIT( PGOpt2Processing );
}

IMPLEMENT_GEOXL_CLASS( PGOpt2Processing_varIndex ,0) {
	BEGIN_CLASS_INIT( PGOpt2Processing_varIndex );
}

IMPLEMENT_GEOXL_CLASS( PGOpt2Processing_Normal_Orientation ,0) {
	BEGIN_CLASS_INIT( PGOpt2Processing_Normal_Orientation );
}

IMPLEMENT_GEOXL_CLASS( PGOpt2Processing_Initial_Rotation_Matrices ,0) {
	BEGIN_CLASS_INIT( PGOpt2Processing_Initial_Rotation_Matrices );
}

IMPLEMENT_GEOXL_CLASS( PGOpt2Processing_Normal_Estimation, 0 ) {
	BEGIN_CLASS_INIT( PGOpt2Processing_Normal_Estimation );
}


// ----

PGOpt2Processing::PGOpt2Processing() {
}

void PGOpt2Processing::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

PGOpt2Processing::~PGOpt2Processing() {
}

void PGOpt2Processing_varIndex::process(const PGOpt2Parameters *parameters,
										OptimizationGraph *optGraph, 
										PointCloudAnimationSequence *data, 
										PatchGraph *patchGraph, 
										DebugRenderer *dbgRenderer,
                        		ProcessingContext *context)
{
	
	int32 varIndex = 0;
	for(card32 frameIndex = 0; frameIndex < optGraph->getNumFrames(); frameIndex++)
	{
		PGOpt2Frame *frame             = optGraph->getFrame(frameIndex);
	   const card32 patchesFrameIndex = frame->getPatchesFrameIndex();
		PointSet		*ps                = frame->getPatches();
	
		///Initialize position channels
		AAT			POS_INDEX		= ps->getAAT("varIndex_pos");
		for(card32 surfelIndex = 0; surfelIndex < ps->getHeight(); surfelIndex++)
		{
			ps->set<int32>(patchesFrameIndex, surfelIndex, POS_INDEX, -3);
		}
		vector<card32>	patchIndices = frame->getPatchIndices();
		for(card32 ni = 0; ni < patchIndices.size(); ni++)
		{
			ps->set<int32>(patchesFrameIndex, patchIndices[ni], POS_INDEX, varIndex);
			varIndex += 3;
		}
	}
	for(card32 frameIndex = 0; frameIndex < optGraph->getNumFrames(); frameIndex++)
	{
		PGOpt2Frame	*frame             = optGraph->getFrame(frameIndex);
	   const card32 patchesFrameIndex = frame->getPatchesFrameIndex();
		PointSet		*ps                = frame->getPatches();

		/// Initialize normal channels
		AAT NOR_INDEX = ps->getAAT("varIndex_nor");
		for(card32 surfelIndex = 0; surfelIndex < ps->getHeight(); surfelIndex++)
		{
			ps->set<int32>(patchesFrameIndex, surfelIndex, NOR_INDEX, -3);
		}
		vector<card32>	patchIndices = frame->getPatchIndices();
		for(card32 ni = 0; ni < patchIndices.size(); ni++)
		{
			ps->set<int32>(patchesFrameIndex, patchIndices[ni], NOR_INDEX, varIndex);
			varIndex += 2;
		}
	}
	for(card32 edgeIndex = 0; edgeIndex < optGraph->getNumEdges(); edgeIndex++)
	{
		PGOpt2FramePair *edge = optGraph->getEdge(edgeIndex);
		PointSet        *ps   = edge->getPatchPairsAttribs();

		/// Initialize rotation channels
		AAT		ROT_INDEX		= ps->getAAT("varIndex_rot");
		for(card32 surfelIndex = 0; surfelIndex < ps->getHeight(); surfelIndex++)
		{
			ps->set<int32>(0, surfelIndex, ROT_INDEX, -3);
		}
		vector<card32>	patchIndices = edge->getPatchIndices();
		for(card32 ni = 0; ni < patchIndices.size(); ni++)
		{
			ps->set<int32>(0, patchIndices[ni], ROT_INDEX, varIndex);
			varIndex += 3;
		}
	}
	context->numVariables = varIndex;
}

void PGOpt2Processing_Initial_Rotation_Matrices::process(const PGOpt2Parameters *parameters, 
											   OptimizationGraph *optGraph,
											   PointCloudAnimationSequence *data, 
											   PatchGraph *patchGraph, 
										      DebugRenderer *dbgRenderer,
                        		      ProcessingContext *context)
{
	InCorePCTopologyGraph	*topology = patchGraph->getTopology(); 
	//Normal orientation

	///Compute initial rotation matrix
	geoxl::Eigen<float32>			*eigenSolver= new geoxl::Eigen<float32>(4);

	float32			w_pos_r		= parameters->weightPositionRigidity, 
					w_nor_r		= parameters->weightNormalRigidity;
	
	int32			ik_depth, jk_depth, ikn_depth, jkn_depth;
	Vector3f		pik,  pikn,  pjk,  pjkn,
					nik,  nikn,  njk,  njkn,
					rijk, rijkn, nijk, nijkn;

	DMatrixF		M;	M.setDimension(3, 3, false);

	for(card32 edgeIndex = 0; edgeIndex < optGraph->getNumEdges(); edgeIndex++)
	{
		PGOpt2FramePair	*edge	   = optGraph->getEdge(edgeIndex);
		PointSet		*edge_ps   = edge->getPatchPairsAttribs();
		
		card32		scanIndex1	   = edge->getOptFrameIndex1();
		card32	   scanIndex2	   = edge->getOptFrameIndex2();
		card32      scanIndex1PatchesFrameIndex = optGraph->getFrame(scanIndex1)->getPatchesFrameIndex();
		card32      scanIndex2PatchesFrameIndex = optGraph->getFrame(scanIndex2)->getPatchesFrameIndex();

		PointSet		*scan_ps1  = optGraph->getFrame(scanIndex1)->getPatches();
		PointSet		*scan_ps2  = optGraph->getFrame(scanIndex2)->getPatches();
		
		AAT		REF_N				= scan_ps1->getAAT( "normal" );
		AAT		POSITION		= scan_ps1->getAAT( "position" );
		AAT		TOP_DEPTH	= scan_ps1->getAAT( "topology_depth");

		//take care of this, if there is not "varIndex_rot" channel, we should create it
		AAT		ROTATION		= edge_ps->getAAT( "rotation_matrix");
		
		vector<card32>	patchIndices  =	edge->getPatchIndices();
		for(card32 id = 0; id < patchIndices.size(); id++)
		{
			card32 surfelIndex = patchIndices[id];
			pik			= scan_ps1->get3f(scanIndex1PatchesFrameIndex, surfelIndex,		POSITION);
			nik			= scan_ps1->get3f(scanIndex1PatchesFrameIndex, surfelIndex,		REF_N);
			ik_depth		= scan_ps1->get<int32>(scanIndex1PatchesFrameIndex, surfelIndex,	TOP_DEPTH);
			pikn				= scan_ps2->get3f(scanIndex2PatchesFrameIndex,	surfelIndex,	POSITION);
			nikn			= scan_ps2->get3f(scanIndex2PatchesFrameIndex,	surfelIndex,	REF_N);
			ikn_depth	= scan_ps2->get<int32>(scanIndex2PatchesFrameIndex, surfelIndex, TOP_DEPTH);
			if(ik_depth >= 2 || ikn_depth >= 2)
				continue;

			M.setZero();
			card32	numOfEntries = topology->getNumAdjacentVertices(surfelIndex);
			for(card32 ni = 0; ni < numOfEntries; ni++)
			{
				card32 neighbor_id = topology->getAdjacentVertex(surfelIndex, ni);
				pjk				= scan_ps1->get3f(scanIndex1PatchesFrameIndex, neighbor_id, POSITION);
				njk				= scan_ps1->get3f(scanIndex1PatchesFrameIndex, neighbor_id, REF_N);
				jk_depth			= scan_ps1->get<int32>(scanIndex1PatchesFrameIndex, neighbor_id, TOP_DEPTH);
				pjkn				= scan_ps2->get3f(scanIndex2PatchesFrameIndex, neighbor_id, POSITION);
				njkn				= scan_ps2->get3f(scanIndex2PatchesFrameIndex, neighbor_id, REF_N);
				jkn_depth		= scan_ps2->get<int32>(scanIndex2PatchesFrameIndex, neighbor_id, TOP_DEPTH);
				if(jk_depth >= 2 || jkn_depth >= 2)
					continue;

				rijk					= pik  - pjk;
				rijkn					= pikn - pjkn;
				nijk					= nik  - njk;
				nijkn				= nikn - njkn;
				float	weight		= 1.0f/numOfEntries;
				for(card32 i = 0; i < 3; i++)
				{
					for(card32 j = 0; j < 3; j++)
					{
						M[i][j] += weight*w_pos_r*rijk[i]*rijkn[j]; 
					}
				}
				for(card32 i = 0; i < 3; i++)
				{
					for(card32 j = 0; j < 3; j++)
					{
						M[i][j] += weight*w_nor_r*nijk[i]*nijkn[j];
					}
				}

				eigenSolver->Matrix(0, 0) =		M[0][0] + M[1][1] + M[2][2];
				eigenSolver->Matrix(1, 1) =		M[0][0] - M[1][1] - M[2][2];
				eigenSolver->Matrix(2, 2) =	-	M[0][0] + M[1][1] - M[2][2];				
				eigenSolver->Matrix(3, 3) =	-	M[0][0] - M[1][1] + M[2][2];
				eigenSolver->Matrix(0, 1) =		M[1][2] - M[2][1];	eigenSolver->Matrix(1, 0) =		M[1][2] - M[2][1];
				eigenSolver->Matrix(0, 2) =		M[2][0] - M[0][2];	eigenSolver->Matrix(2, 0) =		M[2][0] - M[0][2];
				eigenSolver->Matrix(0, 3) =		M[0][1] - M[1][0];	eigenSolver->Matrix(3, 0) =		M[0][1] - M[1][0];
				eigenSolver->Matrix(1, 2) =		M[0][1] + M[1][0];	eigenSolver->Matrix(2, 1) =		M[0][1] + M[1][0];
				eigenSolver->Matrix(1, 3) =		M[0][2] + M[2][0];	eigenSolver->Matrix(3, 1) =		M[0][2] + M[2][0];
				eigenSolver->Matrix(2, 3) =		M[1][2] + M[2][1];	eigenSolver->Matrix(3, 2) =		M[1][2] + M[2][1];

				eigenSolver->DecrSortEigenStuff4();
				//  Destruct the eigen-system used for computing rotation matrix
				char			*dataC = edge_ps->getDataPointer(0, surfelIndex);
				float32	*dataF = (float*)(dataC+ROTATION.getOffset());
				for(card32 id = 0; id < 4; id++)
				{
					dataF[ id ] = eigenSolver->GetEigenvector(id, 0);
				}
			}
		}
	}
	
	delete	eigenSolver;
}


struct Entry {
	card32 i;
	float32 w;
	
	Entry() {}
	inline Entry(card32 ni, float32 nw) {i=ni; w=nw;}
	bool operator<(const Entry &e) const {return w < e.w;}
};

void	PGOpt2Processing_Normal_Orientation::process(
   const PGOpt2Parameters *parameters, 
	OptimizationGraph *optGraph,
	PointCloudAnimationSequence *data,
	PatchGraph *patchGraph,
	DebugRenderer *dbgRenderer,
	ProcessingContext *context)
{
   card32 numFrames = optGraph->getNumFrames();
   for (card32 frameIndex=0; frameIndex<numFrames; frameIndex++) {
      PGOpt2Frame *frame = optGraph->getFrame(frameIndex);
      PointSet *ps = frame->getPatches();
      card32 psFrameIndex = frame->getPatchesFrameIndex();
	   InCorePCTopologyGraph *tg = patchGraph->getTopology();
	   const card32 numPatches = ps->getHeight();
   	const AAT NORMAL = ps->getAAT( "normal" );
   	const AAT FLAGS = ps->getAAT( "flags" );
		set<card32> unprocessedPatches;
		for (card32 patch=0; patch<numPatches; patch++) {
			const int32 nflags = ps->get<int32>(psFrameIndex, patch, FLAGS);
			if ((nflags & FLAG_PATCH_INITIALIZED) == 0) continue;
			unprocessedPatches.insert(patch);
		}
		while (!unprocessedPatches.empty()) {
			priority_queue<Entry> patchesQueue;
			patchesQueue.push(Entry(*unprocessedPatches.begin(), 0));
			unprocessedPatches.erase(unprocessedPatches.begin());
			while (!patchesQueue.empty()) {
				card32 nextPatch = patchesQueue.top().i;
				patchesQueue.pop();

				const Vector3f thisNormal = ps->get<float32, 3>(psFrameIndex, nextPatch, NORMAL);

				const card32 numNeighbors = tg->getNumAdjacentVertices(nextPatch);
				for (card32 neighbor=0;	neighbor<numNeighbors; neighbor++) {
					const card32 neighborIndex = tg->getAdjacentVertex(nextPatch, neighbor);
					const int32 nflags = ps->get<int32>(psFrameIndex, neighborIndex, FLAGS);
					if ((nflags & FLAG_PATCH_INITIALIZED) == 0) continue;
					if (unprocessedPatches.find(neighborIndex) != unprocessedPatches.end()) {
						const Vector3f otherNormal = ps->get<float32, 3>(psFrameIndex, neighborIndex, NORMAL);
						if (thisNormal * otherNormal < 0) {
							ps->set3f(psFrameIndex, neighborIndex, NORMAL, -otherNormal);
						}
						unprocessedPatches.erase(neighborIndex);
						patchesQueue.push(Entry(neighborIndex, fabs(normalize(thisNormal) * normalize(otherNormal))));
					}
				}
			}
		}
		// --- make leftmost normal point outwards
		int32 leftIndex = -1;
		float32 xmin = 1.0E30f;
   	const AAT POSITION = ps->getAAT( "position" );
      for (card32 p=0; p<numPatches; p++) {
         Vector3f pos = ps->get<float32, 3>(psFrameIndex, p, POSITION);
         if (pos[0] < xmin) {
            xmin = pos[0];
            leftIndex = p;
         }
      }
      if (leftIndex == -1) {
         warning("empty point cloud");
         return;
      }
      Vector3f leftMostNormal = ps->get<float32, 3>(psFrameIndex, leftIndex, NORMAL);
      bool needsFlip = (leftMostNormal[0]>0);
      if (needsFlip) {
         for (card32 p=0; p<numPatches; p++) {
            ps->set3f(psFrameIndex, p, NORMAL, -ps->get3f(psFrameIndex, p, NORMAL));
         }
      }
	}
	/*{
      PointSet *ps = patchGraph->getTrajectorySet()->getTrajectories();
      const card32 numFrames = ps->getWidth();
      const card32 numPoints = ps->getHeight();
      const AAT normalAAT = ps->getAAT("normal");
      // --- unify over frames
      for (card32 p=0; p<numPoints; p++) {
         card32 numAsZero = 0;
         card32 numNotAsZero = 0;
         Vector3f zeroNormal = ps->get3f(0, p, normalAAT);
         for (card32 f=1; f<numFrames; f++) {
            Vector3f comp = ps->get3f(f, p, normalAAT);
            if (comp*zeroNormal >=0) {
               numAsZero++;
            } else {
               numNotAsZero++;
            }
         }
         if (numAsZero >= numNotAsZero) {
            for (card32 f=1; f<numFrames; f++) {
               Vector3f comp = ps->get3f(f, p, normalAAT);
               if (comp*zeroNormal <0) {
                  ps->set3f(f, p, normalAAT, -comp);
               }            
            }      
         } else {
            for (card32 f=0; f<numFrames; f++) {
               Vector3f comp = ps->get3f(f, p, normalAAT);
               if (comp*zeroNormal >=0) {
                  ps->set3f(f, p, normalAAT, -comp);
               }            
            }      
         }
      }
   }*/
}


void PGOpt2Processing_Normal_Estimation::process(
   const PGOpt2Parameters *parameters, 
	OptimizationGraph *optGraph,
	PointCloudAnimationSequence *data,
	PatchGraph *patchGraph,
	DebugRenderer *dbgRenderer,
	ProcessingContext *context)
{
   PointSet *ps = patchGraph->getTrajectorySet()->getTrajectories();
   InCorePCTopologyGraph *tg = patchGraph->getTopology();
   card32 numFrames = ps->getWidth();
   card32 numPatches = ps->getHeight();
   AAT POS = ps->getAAT("position");
   AAT NORMAL = ps->getAAT("normal");
   AAT TANGENT_U = ps->getAAT("tangent_u");
   AAT FLAGS = ps->getAAT("flags");
   for (card32 p=0; p<numPatches; p++) {
      for (card32 f=0; f<numFrames; f++) {

		   RegressionPlane rp;
		   rp.clear();
		   card32 numAdjacent = tg->getNumAdjacentVertices(p);
		   vector<card32> neighbors(numAdjacent);
         for (card32 i=0; i<numAdjacent; i++) {
            neighbors[i] = tg->getAdjacentVertex(p, i);
         }
		   for (card32 j=0; j<numAdjacent; j++) {
				const int32 nflags = ps->get<int32>(f, neighbors[j], FLAGS);
				if ((nflags & FLAG_PATCH_INITIALIZED) == 0) continue;
			   rp.addPoint( ps->get3f( f, neighbors[j], POS ) );
		   }
		   rp.computePlane();

		   Vector3f normal = normalize(rp.getNormal());
		   Vector3f tangent_u = normalize(rp.getU());
		   ps->set3f(f, p, NORMAL, normal);
		   ps->set3f(f, p, TANGENT_U, tangent_u);
      }
   }
   PointSet *urshape = patchGraph->getUrshape();
   if (urshape) {
      for (card32 p=0; p<numPatches; p++) {
		   RegressionPlane rp;
		   rp.clear();
		   card32 numAdjacent = tg->getNumAdjacentVertices(p);
		   vector<card32> neighbors(numAdjacent);
         for (card32 i=0; i<numAdjacent; i++) {
            neighbors[i] = tg->getAdjacentVertex(p, i);
         }
		   for (card32 j=0; j<numAdjacent; j++) {
				const int32 nflags = ps->get<int32>(0, neighbors[j], FLAGS);
				if ((nflags & FLAG_PATCH_INITIALIZED) == 0) continue;
			   rp.addPoint( ps->get3f( 0, neighbors[j], POS ) );
		   }
		   rp.computePlane();

		   Vector3f normal = normalize(rp.getNormal());
		   Vector3f tangent_u = normalize(rp.getU());
		   urshape->set3f(0, p, NORMAL, normal);
		   urshape->set3f(0, p, TANGENT_U, tangent_u);
      }
   }
}

void PGOpt2Processing_Normal_Estimation::estimateNormals(PatchGraph *pg) 
{
	PGOpt2Processing_Normal_Estimation est;
	est.process(NULL, NULL, NULL, pg, NULL, NULL);
}

void PGOpt2Processing_Normal_Orientation::unifyNormals(PatchGraph *pg) 
{
	const card32 numFrames = pg->getTrajectorySet()->getNumFrames();
   const card32 numPatches = pg->getTrajectorySet()->getNumPatches();
   
   vector< vector<card32> > frameVarIndices(numFrames);
   vector< vector<card32> > edgeVarIndices(numFrames);
   for (card32 frame=0; frame<numFrames; frame++) {
      frameVarIndices[frame].resize(numPatches);
      edgeVarIndices[frame].resize(numPatches);
      for (card32 patch=0; patch<numPatches; patch++) {
         frameVarIndices[frame][patch] = patch;      
         edgeVarIndices[frame][patch] = patch;      
      }
   }

   PGOpt2GraphCreator gc;
   PGOpt2FramePairVars *pairVars = gc.createPairVars(pg, false, false, false, false, false);
   OptimizationGraph *optGraph = gc.createOptGraph(pg, NULL, pairVars, false, false, false, false, false, &frameVarIndices, &edgeVarIndices);
	PGOpt2Processing_Normal_Orientation orient;
	orient.process(NULL, optGraph, NULL, pg, NULL, NULL);
   delete optGraph;
   delete pairVars;
}
