//---------------------------------------------------------------------------
#ifndef PGOpt2ProcessingH
#define PGOpt2ProcessingH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PGOpt2Parameters;
class OptimizationGraph;
class PointCloudAnimationSequence;
class PatchGraph;
class DebugRenderer;

///
///   performs data processing before / after the main optimization takes place
///
class ANIMATIONRECONSTRUCTION_API PGOpt2Processing : public Persistent {
	GEOXL_CLASS( PGOpt2Processing )
 private:

 public:
   struct ProcessingContext {
      int32 numVariables;
      void reset() {numVariables = -1;}
   };
 
	PGOpt2Processing();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual void process(
		const PGOpt2Parameters      *parameters,
		OptimizationGraph           *optGraph,
		PointCloudAnimationSequence *data,	
		PatchGraph                  *patchGraph,	
		DebugRenderer               *dbgRenderer,
		ProcessingContext           *context
   ) = 0;   

   ~PGOpt2Processing();
};



class ANIMATIONRECONSTRUCTION_API PGOpt2Processing_varIndex : public PGOpt2Processing {
	GEOXL_CLASS( PGOpt2Processing_varIndex )

public:

	virtual void process(
		const PGOpt2Parameters      *parameters,
		OptimizationGraph           *optGraph,
		PointCloudAnimationSequence *data,	
		PatchGraph                  *patchGraph,	
		DebugRenderer               *dbgRenderer,
		ProcessingContext           *context
   );  
};



class ANIMATIONRECONSTRUCTION_API PGOpt2Processing_Initial_Rotation_Matrices : public PGOpt2Processing {
	GEOXL_CLASS( PGOpt2Processing_Initial_Rotation_Matrices )
public:

	virtual void process(
		const PGOpt2Parameters      *parameters,
		OptimizationGraph           *optGraph,
		PointCloudAnimationSequence *data,	
		PatchGraph                  *patchGraph,	
		DebugRenderer               *dbgRenderer,
		ProcessingContext           *context
	);

};

class ANIMATIONRECONSTRUCTION_API PGOpt2Processing_Normal_Orientation : public PGOpt2Processing {
	GEOXL_CLASS( PGOpt2Processing_Normal_Orientation )
public:

	virtual void process(
		const PGOpt2Parameters      *parameters,
		OptimizationGraph           *optGraph,
		PointCloudAnimationSequence *data,	
		PatchGraph                  *patchGraph,	
		DebugRenderer               *dbgRenderer,
		ProcessingContext           *context
	);
	// unifies normals for one (whole) patch graph
	static void unifyNormals(PatchGraph *pg);
};

class ANIMATIONRECONSTRUCTION_API PGOpt2Processing_Normal_Estimation : public PGOpt2Processing {
	GEOXL_CLASS( PGOpt2Processing_Normal_Estimation )
public:

	virtual void process(
		const PGOpt2Parameters      *parameters,
		OptimizationGraph           *optGraph,
		PointCloudAnimationSequence *data,	
		PatchGraph                  *patchGraph,	
		DebugRenderer               *dbgRenderer,
		ProcessingContext           *context
	);
	// estimates normals and tangent_u's for one (whole) patch graph
	static void estimateNormals(PatchGraph *pg);
};

}

#endif