//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PatchGraphOpt2.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PGOpt2Processing.h"
#include "PGOpt2Potential.h"
#include "PGOpt2Processing.h"
#include "PGOpt2Frame.h"
#include "PGOpt2FramePair.h"
#include "OptimizationGraph.h"
#include "PGOpt2Parameters.h"
#include "PatchGraph.h"
#include "IterativeSolvers.h"
#include "IterativeSolvers_templCode.h"
#include "PatchGraphRenderer.h"
#include "PGOpt2GraphCreator.h"
#include "Timer.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PatchGraphOpt2 ,0)
{
	BEGIN_CLASS_INIT( PatchGraphOpt2 );

	ADD_VAR_ARRAY_PROP( preprocessing, 0, 
		new ObjectClassProperty( "element" , MetaClass::FIRST_VERSION, NULL, 
                               PGOpt2Processing::getClass(), true, NULL )
	)
	ADD_VAR_ARRAY_PROP( potentials, 0, 
		new ObjectClassProperty( "element" , MetaClass::FIRST_VERSION, NULL, 
                               PGOpt2Potential::getClass(), true, NULL )
	)
	ADD_VAR_ARRAY_PROP( postprocessing, 0, 
		new ObjectClassProperty( "element" , MetaClass::FIRST_VERSION, NULL, 
                               PGOpt2Processing::getClass(), true, NULL )
	)
}


// --


void OptimizationRecord2::debugOutput() {
   for (card32 i=0; i<energies.size(); i++) {
      ::debugOutput << energies[i].potentialType->getClassName() << "\t" << energies[i].energy << "\t";
   }
   ::debugOutput << "\n";
}

float32 OptimizationRecord2::getEnergy(const MetaClass *type) {
   for (card32 i=0; i<energies.size(); i++) {
      if (energies[i].potentialType == type) {
         return energies[i].energy;
      }
   }
   return 0;
}


// ----


PatchGraphOpt2::PatchGraphOpt2()
{
}

PatchGraphOpt2::~PatchGraphOpt2()
{
   clearPreprocessingSteps();
   clearPostprocessingSteps();
   clearPotentials();
}

void PatchGraphOpt2::assign(const Object* obj, CopyContext *context)
{
	Persistent::assign(obj,context);
	const PatchGraphOpt2 * o = dynamic_cast<const PatchGraphOpt2*>(obj);
	pAssert( o != NULL );
   copyObjectProperties(obj, this);
}

void PatchGraphOpt2::clearPreprocessingSteps() 
{
   for (unsigned i=0; i<preprocessing.size(); i++) {
      delete preprocessing[i];
   }
   preprocessing.clear();
}

void PatchGraphOpt2::addPreprocessingStep(PGOpt2Processing *step) {
   preprocessing.push_back(step);
}


void PatchGraphOpt2::clearPostprocessingSteps() 
{
   for (unsigned i=0; i<postprocessing.size(); i++) {
      delete postprocessing[i];
   }
   postprocessing.clear();
}

void PatchGraphOpt2::addPostprocessingStep(PGOpt2Processing *step) {
   postprocessing.push_back(step);
}


void PatchGraphOpt2::clearPotentials() 
{
   for (unsigned i=0; i<potentials.size(); i++) {
      delete potentials[i];
   }
   potentials.clear();
}

void PatchGraphOpt2::addPotential(PGOpt2Potential *pot) {
   potentials.push_back(pot);
}




void PatchGraphOpt2::Projection_Back_Onto_The_Manifold(
	OptimizationGraph *optGraph, 
	DVectorF *displacement)
{
	for(card32 frameIndex = 0; frameIndex < optGraph->getNumFrames(); frameIndex++)
	{
		PGOpt2Frame	*frame	 = optGraph->getFrame(frameIndex);
		PointSet	*scan_ps	    = frame->getPatches();
		card32 psFrameIndex   = frame->getPatchesFrameIndex();

		AAT		POS_INDEX		 = scan_ps->getAAT("varIndex_pos");
		AAT		NOR_INDEX		 = scan_ps->getAAT("varIndex_nor");
		AAT		POSITION			 = scan_ps->getAAT("position");
		AAT		NORMAL				 = scan_ps->getAAT("normal");
		AAT		REF_U					 = scan_ps->getAAT("tangent_u");
		vector<card32>	patchIndices = frame->getPatchIndices();
		for(card32 ni = 0; ni < patchIndices.size(); ni++)
		{
			card32	surfelIndex	 = patchIndices[ni];
			int32	ik_posid	 = scan_ps->get<int32>(psFrameIndex, surfelIndex, POS_INDEX);
			int32	ik_norid	 = scan_ps->get<int32>(psFrameIndex, surfelIndex, NOR_INDEX);

			Vector3f	pik		 = scan_ps->get3f(psFrameIndex, surfelIndex, POSITION);
			Vector3f	nik		 = scan_ps->get3f(psFrameIndex, surfelIndex, NORMAL);
			Vector3f	Eik1	    = scan_ps->get3f(psFrameIndex, surfelIndex, REF_U);
			Vector3f	Eik2	    = nik.crossProduct(Eik1);
			if(ik_posid >= 0)
			{
				for(card32 k = 0; k < 3; k++)
					pik[k] += (*displacement)[ik_posid + k];
				scan_ps->set3f(psFrameIndex, surfelIndex, POSITION, pik);
			}
			if(ik_norid >= 0)
			{
				nik			+=	Eik1*(*displacement)[ik_norid]	+	Eik2*(*displacement)[ik_norid + 1];
				nik.normalize();
				scan_ps->set3f(psFrameIndex, surfelIndex, NORMAL, nik);
				/// Update Ref_U
				Eik1			-=	nik*(Eik1*nik);
				Eik1.normalize();
				scan_ps->set3f(psFrameIndex, surfelIndex, REF_U, Eik1);
			}
		}
	}
	for(card32 edgeIndex = 0; edgeIndex < optGraph->getNumEdges(); edgeIndex++)
	{
		PGOpt2FramePair	*edge	  = optGraph->getEdge(edgeIndex);
		PointSet				*edge_ps = edge->getPatchPairsAttribs();

		AAT		ROT_INDEX		 = edge_ps->getAAT("varIndex_rot");
		AAT		ROTATION			 = edge_ps->getAAT("rotation_matrix");
		vector<card32>	patchIndices = edge->getPatchIndices();
		for(card32 ni = 0; ni < patchIndices.size(); ni++)
		{
			card32	surfelIndex = patchIndices[ni];
			int		ik_rotid    = edge_ps->get<int32>(0, surfelIndex, ROT_INDEX);
			if(ik_rotid >= 0)
			{
				//get current rotation matrix
				char*		dataC	= edge_ps->getDataPointer(0, surfelIndex);
				float32*	quatC	= (float32*)(dataC+ROTATION.getOffset());

				float32		theta		= sqrt((*displacement)[ik_rotid]*(*displacement)[ik_rotid] 
				+ (*displacement)[ik_rotid+1]*(*displacement)[ik_rotid+1] 
				+ (*displacement)[ik_rotid+2]*(*displacement)[ik_rotid+2]);

				float32	quat[4], quatn[4], sina;
				sina								= sin(theta/2);
				quat[0]						= cos(theta/2);
				if(theta > 1e-6)
				{
					for(card32	k = 0; k < 3; k++)
					{
						quat[k + 1]		= sina*(*displacement)[ik_rotid + k]/theta;
					}
				}
				else
				{
					quat[1] = sina;			quat[2] = 0.0f;			quat[3] = 0.0f;
				}
				quatn[0] = quat[0]*quatC[0] - quat[1]*quatC[1] - quat[2]*quatC[2] - quat[3]*quatC[3];
				quatn[1] = quat[0]*quatC[1] + quat[1]*quatC[0] + quat[2]*quatC[3] - quat[3]*quatC[2];
				quatn[2] = quat[0]*quatC[2] + quat[2]*quatC[0] + quat[3]*quatC[1] - quat[1]*quatC[2];
				quatn[3] = quat[0]*quatC[3] + quat[3]*quatC[0] + quat[1]*quatC[2] - quat[2]*quatC[1];
				theta = sqrt(quatn[0]*quatn[0]+quatn[1]*quatn[1]+quatn[2]*quatn[2]+quatn[3]*quatn[3]);
				quatn[0] /= theta;	quatn[1] /= theta;	quatn[2] /= theta;	quatn[3] /= theta;

				for(card32	k = 0; k < 4; k++)
				{
					quatC[k] = quatn[k];
				}
			}
		}
	}
}

void PatchGraphOpt2::optimize(
   const PGOpt2Parameters      *parameters,
   const AnimationBSRContext   *recParameters,
   OptimizationGraph           *optGraph,
   PatchGraph					    *pg,
	PointCloudAnimationSequence *data,	
   DebugRenderer               *dbgRenderer,
   OptimizationRecord2         *optRecord)
{
   if (parameters->outputVertexPotentials) {
      pg->createPatchAttribute(mVAD("sdfPot", 1, VAD::DATA_FORMAT_FLOAT32));
      pg->createPatchAttribute(mVAD("smoothPot", 1, VAD::DATA_FORMAT_FLOAT32));
      pg->createPatchAttribute(mVAD("rigidityPot", 1, VAD::DATA_FORMAT_FLOAT32));
   }
   if (optRecord) optRecord->converged = false;
   DebugVis vis;
   vis.renderer.setShowTopology(true);
   vis.renderer.setShowCoords(false);
   vis.renderer.setShowDataPoints(false);
   vis.renderer.setShowAllDataPoints(true);
   vis.renderer.setUseDL(false);
   vis.renderer.setShowAllFrames(true);
   vis.renderer.setExtraAlpha(false);
   vis.pgVis = pg;
   vis.data = data;
   if (dbgRenderer) {
      dbgRenderer->performRedraw(&vis);
   }

   Timer timer;
	/// Initial processing
	PGOpt2Processing::ProcessingContext context;
	context.reset();
	for(card32 i = 0; i < preprocessing.size(); i++)
	{
		preprocessing[i]->process(parameters, optGraph, data, pg, dbgRenderer, &context);
	}
	
	if (parameters->verbose) {debugOutput << "preprocess [ms]: " << timer.getDeltaValue() << "\n"; debugOutput.flush();}
	
	if (context.numVariables == -1) {
	   throw PException("num variables not set.");
	}

	SparseMatrixF	quadApprox;
	DVectorF		linApprox, direction, diag;
	card32	m_dimension = context.numVariables;
	SparseMatrixF lmMatrix(m_dimension);
	quadApprox.setRows(m_dimension);
	linApprox.setDim(m_dimension);
	direction.setDim(m_dimension);
	diag.setDim(m_dimension);
	
	if (parameters->levenbergMarquant > 0.0f) {
	   for (card32 i=0; i<m_dimension; i++) {
	      lmMatrix[i].pushBackEntry(i, parameters->levenbergMarquant);
	   }
	}

	if (parameters->verbose) {debugOutput << "num variables: " << m_dimension << "\n"; debugOutput.flush();}

	for(card32 step = 0; step < parameters->numIterations; step++)
	{
		quadApprox.refresh();
		linApprox.setZero();
		direction.setZero();
		diag.setZero();

		vector<float32>	energies;
		for(card32 i = 0; i < potentials.size(); i++)
		{
			float32 energy;
			potentials[i]->quadraticApprox(parameters, recParameters, pg->getTopology(), optGraph, energy, &linApprox, &quadApprox);
			energies.push_back(energy);
		}
   	if (parameters->levenbergMarquant > 0.0f) {
   	   quadApprox += lmMatrix;
      }
      if (optRecord) {
         optRecord->energies.clear();
         optRecord->energies.resize(potentials.size());
         for (card32 i=0; i<potentials.size(); i++) {
            optRecord->energies[i].potentialType = potentials[i]->getInstanceClass();
            optRecord->energies[i].energy = energies[i];
         }
      }
   	if (parameters->verbose) {
   	   debugOutput << "   setup [ms]: " << timer.getDeltaValue() << "\n"; 
   	   debugOutput << "   "; 
         for (card32 i=0; i<potentials.size(); i++) {
      	   debugOutput << potentials[i]->getInstanceClass()->getClassName() << ": " << energies[i] << "\t";
         }
    	   debugOutput << "\n";
   	   debugOutput.flush();
   	}

		for(card32 id = 0; id < m_dimension; id++)
		{
		   float32 q = sqrt(quadApprox[id]*quadApprox[id]);
		   pAssert(q != 0);
         
         #if defined(GEOXL_DEBUG)
		   if (q == 0) {
		      warning("preconditioner failed (diagonal contains one or more zeros) - optimization aborted");
		      break;
		   }
			diag[id] = 1.0/q;
			#else 
		   if (q == 0) {
            diag[id] = 1.0f;
		   } else {
   			diag[id] = 1.0f/q;
		   }
			#endif
		}
		
		CGStatistics stats;
		card32 numIterations = parameters->cgMaxNumIterations;
		conjugateGradientsSolve_Diag_Precond<float32>(quadApprox, direction, linApprox, diag,
			numIterations, parameters->cgStopRes, parameters->cgDebugging, &stats);

   	if (parameters->verbose) {
   	   debugOutput << "   solve [ms]: " << timer.getDeltaValue() << "\n";
   	   debugOutput << "   stats: iterations - " << stats.numIterations << ", final res. norm - " << stats.lastResidualNorm << "\n";
   	   debugOutput.flush();
   	}

		Projection_Back_Onto_The_Manifold(optGraph, &direction);
      pg->getTrajectorySet()->invalidateAllAttachments();
		pg->invalidateUrshapeAttachments();

   	if (parameters->verbose) {debugOutput << "   project back [ms]: " << timer.getDeltaValue() << "\n"; debugOutput.flush();}

      if (dbgRenderer) {
         dbgRenderer->performRedraw(&vis);
      }
      
		//Detect convergence
		float32	maxDeviation = -1e10f;
		for(card32 ni = 0; ni < m_dimension; ni++)
		{
			if(maxDeviation < fabs(direction[ni]))
			{
				maxDeviation = fabs(direction[ni]);
			}	
		}
   	if (parameters->verbose) {
   		debugOutput << "   inf-norm of gradient: " << maxDeviation << "\n"; 
   		debugOutput << "   ---> iteration " << step+1 << "\n"; debugOutput.flush();
   	}
		if(maxDeviation < parameters->convergenceEps) {
         if (optRecord) optRecord->converged = true;
			break;
	   }
	   if (stats.numIterations == 0) break; // no change in last cg iteration --> no further progress can be expected
	}

	/// Post processing
	context.reset();
	for(card32 i = 0; i < postprocessing.size(); i++)
	{
		postprocessing[i]->process(parameters, optGraph, data, pg, dbgRenderer, &context);
	}
//   gc.copyDataVars(optGraph, direction, false);
}

void PatchGraphOpt2::DebugVis::debugDraw() const {
	renderer.renderGL(pgVis, data, pgVis->getStartFrameIndex());
}
