//---------------------------------------------------------------------------
#ifndef PatchGraphOpt2H
#define PatchGraphOpt2H
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------
#include "SparseLinearAlgebra.h"
#include "SparseLinearAlgebra_templCode.h"
#include "PGOperationsDebugging.h"
#include "PatchGraphRenderer.h"
//---------------------------------------------------------------------------

namespace geoxl {


class PGOpt2Potential;
class PGOpt2Parameters;
class PGOpt2Processing;
class DebugRenderer;
class OptimizationGraph;
class PointCloudAnimationSequence;
class DebugRenderer;
class PatchGraph;
class AnimationBSRContext;


/// helper class - provides parameters about the optimization process
/// returned (optionally) by the main optimization routine
struct OptimizationRecord2 {
   struct EnergyEntry {
      const MetaClass *potentialType;
      float32 energy;
   };
   vector<EnergyEntry> energies;
   bool converged;
   
   void debugOutput();
   float32 getEnergy(const MetaClass *type);
};

///
/// energy function optimization for patch graphs
///
class ANIMATIONRECONSTRUCTION_API PatchGraphOpt2 : public Persistent {
	GEOXL_CLASS( PatchGraphOpt2 )

 private:
   /// actions performed before optimization
   vector<PGOpt2Processing*> preprocessing;
	/// list of energy functions
	vector<PGOpt2Potential*> potentials;
   /// actions performed after optimization
   vector<PGOpt2Processing*> postprocessing;

	IMPLEMENT_VAR_ARRAY_METHODS_STL(preprocessing)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(potentials)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(postprocessing)

   class DebugVis : public DebugRenderFunction {
    public:
      mutable PatchGraph *pgVis;
      mutable PointCloudAnimationSequence *data;
      mutable StandardPatchGraphRenderer renderer;
      virtual void debugDraw() const;
   };

	void	Projection_Back_Onto_The_Manifold(
		OptimizationGraph	*optGraph,
		DVectorF				*displacement);

 public:
	PatchGraphOpt2();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   /// add a preprocessing step
   /// this object takes ownership
   /// executed the same order in which they were added
   void addPreprocessingStep(PGOpt2Processing *step);
   void clearPreprocessingSteps();

   /// add a postprocessing step
   /// this object takes ownership
   /// executed the same order in which they were added
   void addPostprocessingStep(PGOpt2Processing *step);
   void clearPostprocessingSteps();
   
   /// add a potential
   /// this object takes ownership
   void addPotential(PGOpt2Potential *pot);
   void clearPotentials();

   /// perform the optimization
	virtual void optimize(
	   const PGOpt2Parameters      *parameters,
	   const AnimationBSRContext   *recParameters,
      OptimizationGraph           *optGraph,
	   PatchGraph				       *pg,
		PointCloudAnimationSequence *data,	
      DebugRenderer               *dbgRenderer,
      OptimizationRecord2         *optRecord = NULL
   );

	~PatchGraphOpt2();
};

}

#endif