//---------------------------------------------------------------------------
#ifndef DataPointIndicesH
#define DataPointIndicesH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

/**
	set of original data points associated with a patch graph
*/
class ANIMATIONRECONSTRUCTION_API PGDataPointIndices : public Persistent {
	GEOXL_CLASS( PGDataPointIndices )

private:
	/// storage - indexing scheme is [time][patchNum][indexOfIndex]
	vector< vector< vector<card32> > > indices;

public:
	PGDataPointIndices();
	
	void setup(card32 numFrames, card32 numPatches);
	
	inline mpcard getNumFrames() const;
	inline mpcard getNumPatches() const;
	
	inline mpcard getNumDataPoints(card32 time, card32 patchIndex) const;
	inline card32 getIndex(card32 time, card32 patchIndex, card32 indexOfIndex) const;
	inline vector<card32> getIndices(card32 time, card32 patchIndex) const;
	inline void setIndices(card32 time, card32 patchIndex, const vector<card32> &indices);
	inline vector< vector<card32> > getFullFrame(card32 time) const;
	inline void setFullFrame(card32 time, const vector< vector<card32> > &frame);
	inline void addIndices(card32 time, card32 patchIndex, const vector<card32> &newIndices);
	inline void addIndex(card32 time, card32 patchIndex, card32 newIndex);
	
   virtual void assign(const Object* obj, CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);
};


// ---- inline methods


inline mpcard PGDataPointIndices::getNumFrames() const {
	return indices.size();
}
inline mpcard PGDataPointIndices::getNumPatches() const {
	if (indices.empty()) return 0;
	return indices[0].size();
}

inline mpcard PGDataPointIndices::getNumDataPoints(card32 time, card32 patchIndex) const {
	pAssert(time < indices.size());
	pAssert(patchIndex < indices[time].size());
	return indices[time][patchIndex].size();
}
inline card32 PGDataPointIndices::getIndex(card32 time, card32 patchIndex, card32 indexOfIndex) const {
	pAssert(time < indices.size());
	pAssert(patchIndex < indices[time].size());
	pAssert(indexOfIndex < indices[time][patchIndex].size());
	return indices[time][patchIndex][indexOfIndex];
}

inline vector<card32> PGDataPointIndices::getIndices(card32 time, card32 patchIndex) const {
	pAssert(time < indices.size());
	pAssert(patchIndex < indices[time].size());
	return indices[time][patchIndex];
}

inline void PGDataPointIndices::setIndices(card32 time, card32 patchIndex, const vector<card32> &indices) {
	pAssert(time < this->indices.size());
	pAssert(patchIndex < this->indices[time].size());
	this->indices[time][patchIndex] = indices;
}

inline void PGDataPointIndices::addIndices(card32 time, card32 patchIndex, const vector<card32> &newIndices) {
	pAssert(time < this->indices.size());
	pAssert(patchIndex < this->indices[time].size());
	size_t n = newIndices.size();
	for (unsigned i=0; i<n; i++) {
		this->indices[time][patchIndex].push_back(newIndices[i]);
	}
}

inline void PGDataPointIndices::addIndex(card32 time, card32 patchIndex, card32 newIndex) {
	pAssert(time < this->indices.size());
	pAssert(patchIndex < this->indices[time].size());
	this->indices[time][patchIndex].push_back(newIndex);
}

inline vector< vector<card32> > PGDataPointIndices::getFullFrame(card32 time) const {
	return indices[time];
}

inline void PGDataPointIndices::setFullFrame(card32 time, const vector< vector<card32> > &frame) {
	indices[time] = frame;
}



}

#endif