//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PatchGraphContainer.h"
#include "ClassMethods.h"
#include "NumericalClassProperty.h"
#include "ClassMethods.h"
#include "BinaryObjectStreams.h"
#include "ObjectClassProperty.h"
#include <QtCore/qfileinfo.h>
#include <qtgui/qfiledialog.h>
#include <assert.h>
#include "FeatureCorrespondences.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( PatchGraphContainer, 5 )
{
	BEGIN_CLASS_INIT( PatchGraphContainer )
	ADD_CARD32_PROP( frameStart, 2 )
	ADD_CARD32_PROP( frameEnd, 2 )
	ADD_OBJECT_PROP( animSeq, 0, PointCloudAnimationSequence::getClass(), NULL )
	ADD_VAR_ARRAY_PROP( pgs, 2, new ObjectClassProperty( "element" , 0, NULL, PatchGraph::getClass(), true, NULL ) )
	ADD_NOARGS_METHOD( computeBoundingBoxes )
	ADD_NOARGS_METHOD( clear )
	ADD_OBJECT_PROP( featureCorrespondences, 4, FeatureCorrespondences::getClass(), true );
	ADD_OBJECT_PROP( tempCorrespondences, 5, FeatureCorrespondences::getClass(), true );
	
}

/** 
 * Constructor
 */
PatchGraphContainer::PatchGraphContainer()
{	
	pgs.clear();
	animSeq = new PointCloudAnimationSequence();
	frameStart = 1000;
	frameEnd = -1;
	notifyer = new Notifyer();
	featureCorrespondences = new FeatureCorrespondences;
	tempCorrespondences = new FeatureCorrespondences;
}

/** 
 * Destructor
 */
PatchGraphContainer::~PatchGraphContainer()
{
	clear();
	delete animSeq;
	delete notifyer;
	delete featureCorrespondences;
	delete tempCorrespondences;
}

/** 
 * Clear container
 */
void PatchGraphContainer::clear()
{
	for ( unsigned int patchGraphIndex = 0; patchGraphIndex < pgs.size(); patchGraphIndex++ ){
		PatchGraph* pg = pgs[patchGraphIndex];
		if ( pg != NULL ){
			delete pg;
			pg = NULL;
		}
	}
	pgs.clear();
	notifyer->change();
}

/** 
 * Add a patch graph
 */
void PatchGraphContainer::addPatchGraph( PatchGraph* pg )
{
	pgs.push_back( pg );
	if ( int32( pg->getStartFrameIndex() ) < frameStart ) frameStart = pg->getStartFrameIndex();
	int32 end = pg->getStartFrameIndex() + pg->getTrajectorySet()->getNumFrames() - 1;
	if ( end > frameEnd ) frameEnd = end;
	notifyer->change();
}

/** 
 * Getter: certain patch graph
 */
PatchGraph* PatchGraphContainer::getPatchGraph( const int32 index ) const 
{ 
	if ( index < 0 ) return NULL;
	if ( index >= int32( pgs.size() ) ) return NULL;;
	return pgs[index]; 
}

/** 
 * Delete certain patch graph
 */
void PatchGraphContainer::deletePatchGraph( const card32 index )
{
	assert( index < pgs.size() );
	delete pgs[index];
	pgs.erase( pgs.begin() + index );
	notifyer->change();
}

/**
 * Print info
 */
void PatchGraphContainer::printDebug() const
{
	for ( unsigned int pgIndex = 0; pgIndex < pgs.size(); pgIndex++ ){
		PatchGraph* pg = pgs[pgIndex];
		debugOutput << "Patch Graph " << pgIndex << ": frames: " <<  pg->getStartFrameIndex() << ".." << ( pg->getStartFrameIndex() + pg->getTrajectorySet()->getNumFrames() - 1 ) << ".\n";
	}
	debugOutput.flush();
}

/** 
 * Compute bounding boxes for all patch graphs in container
 */
void PatchGraphContainer::computeBoundingBoxes()
{
	for ( unsigned int pgIndex = 0; pgIndex < pgs.size(); pgIndex++ ){
		pgs[pgIndex]->computeBBox();
	}
}

int32 PatchGraphContainer::getAnyPatchGraphIndexWithStartFrame(card32 startFrame) {
   card32 numPG = getNumPatchGraphs();
   for (card32 i=0; i<numPG; i++) {
      if (getPatchGraph(i)->getStartFrameIndex() == startFrame) {
         return i;
      }
   }
   return -1;
}

void PatchGraphContainer::setNumpgss(card32 size)
{ 
	pgs.resize(size);
}
		
void PatchGraphContainer::deletepgs( card32 index )
{
	pAssert( index < pgs.size()); 
	pgs.erase( pgs.begin()+index );
}
		
char* PatchGraphContainer::getpgs( card32 index )
{
	pAssert( index < pgs.size()); 
	return (char*)&pgs[index];
}
		
mpcard PatchGraphContainer::getNumpgss()
{
	return pgs.size();
}
		
string PatchGraphContainer::getpgsName(card32 index )
{ 
	pAssert( index < pgs.size());
	char bufferStart[10], bufferEnd[10];
	itoa( pgs[index]->getStartFrameIndex(), bufferStart, 10 );
	itoa( pgs[index]->getStartFrameIndex() + pgs[index]->getTrajectorySet()->getNumFrames() -1, bufferEnd, 10 );
	string s = " => ";
	s = bufferStart + s + bufferEnd;
	return s;
}