//---------------------------------------------------------------------------
#ifndef PatchGraphContainerH
#define PatchGraphContainerH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "PatchGraph.h"
#include "PointCloudAnimationSequence.h"
#include "Helpers/Notifyer.h"
#include <qtcore/qobject.h>
#include <assert.h>

namespace geoxl{

class FeatureCorrespondences;

/// Container for a set of patch graphs which need to get merged throughout the animation
class ANIMATIONRECONSTRUCTION_API PatchGraphContainer : public Persistent
{
	GEOXL_CLASS( PatchGraphContainer )

	private:
		/// Container for all patch graphs
		vector<PatchGraph*> pgs;
		/// Frame range
		int32 frameStart, frameEnd;
		/// Notifyer object
		Notifyer* notifyer;
		/// Animation sequence
		PointCloudAnimationSequence* animSeq;
		/// All feature correspondences in the animation
		FeatureCorrespondences * featureCorrespondences;
		FeatureCorrespondences * tempCorrespondences;

	public:
		/// Constructor
		PatchGraphContainer();
		/// Destructor
		~PatchGraphContainer();

		/// Container descriptor
		void setNumpgss(card32 size);
		void deletepgs( card32 index );
		char* getpgs( card32 index );
		mpcard getNumpgss();
		string getpgsName(card32 index );
		
		/// Return certain patch graph
		PatchGraph* getPatchGraph( const int32 index ) const;
		/// Return animation sequence
		inline PointCloudAnimationSequence* getAnimSeq() const { return animSeq; }
		/// Delete certain patch graph
		void deletePatchGraph( const card32 index );
		/// Clear container
		void clear();
		/// Add a patch graph
		void addPatchGraph( PatchGraph* pg );
		/// for controlled destruction (to not delete some pgs)
		void setNULL(card32 i) {pgs[i] = NULL;}
		/// Compute bounding boxes for all patch graphs in container
		void computeBoundingBoxes();
		/// Print info
		void printDebug() const;		

		FeatureCorrespondences * getFeatureCorrespondences() {return featureCorrespondences;}

		FeatureCorrespondences * getTempCorrespondences() const {return tempCorrespondences;}

		
		/*** MEMBER ACCESS ***/
		inline int32 getFrameStart() const { return frameStart; }
		inline int32 getFrameEnd() const { return frameEnd; }
		inline mpcard getNumPatchGraphs() const { return pgs.size(); }
		inline Notifyer* getNotifyer() const { return notifyer; }
		
		/// returns -1 if not found
		int32 getAnyPatchGraphIndexWithStartFrame(card32 startFrame);
};

}

#endif