//---------------------------------------------------------------------------
#ifndef PointCloudAnimationSequenceH
#define PointCloudAnimationSequenceH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "AttachedIndexedOctree.h"
//---------------------------------------------------------------------------
#include <deque>
//---------------------------------------------------------------------------


namespace geoxl {
	using namespace std;


/**
	represents an animation sequence as array of point sets
*/
class ANIMATIONRECONSTRUCTION_API PointCloudAnimationSequence : public Persistent {
	GEOXL_CLASS( PointCloudAnimationSequence )

 private:
	deque<PointSet*> frames;
	deque<AttachedDataContainer*> frameAttachments;

	/// Container descriptor
	void setNumframess(card32 size) { frames.resize(size);}
	void deleteframes( card32 index ){pAssert( index < frames.size()); frames.erase( frames.begin()+index );}
	char* getframes( card32 index ){pAssert( index < frames.size()); return (char*)&frames[index];}
	mpcard getNumframess(){return frames.size();}
	string getframesName(card32 index ){ 
		pAssert( index < frames.size()); 
		QString s = "Frame " + QString::number( index );
		return s.toStdString();
	}

	IMPLEMENT_VAR_ARRAY_METHODS_STL(frameAttachments)
	
 public:
	enum InsertWhere {INSERT_FRONT, INSERT_BACK};
	
 
	PointCloudAnimationSequence();

	void addFrame(const PointSet *frame, InsertWhere where = INSERT_BACK);
	inline mpcard getNumFrames() const {return frames.size();}
	PointSet* getFrame(card32 time);
	const PointSet* getFrame(card32 time) const;
	/// for controlled destruction (avoiding deletion of some frames)
	void setNULL(card32 i) {frames[i] = NULL;}
	/// Remove all frames
	void clear();
	
	// --- attachments
	AttachedDataContainer *getAttachments(card32 frame);
	void invalidateAttachments(card32 frame);
	void invalidateAllAttachments();
	/// helper method - fetch or create an index octree for a frame
	AttachedIndexedOctree *fetchOctree(card32 frame, card32 maxPointsPerNode = 20);	

	void	createDataAttribute(VAD attribute);

	///Added by peter for computing the gradient of color information
	/// we store for each data point a 3 times 3 matrix
	void computeGradientOfColors(const card32 numOfNeighbors);

	virtual ~PointCloudAnimationSequence();
};


}

#endif