//---------------------------------------------------------------------------
#include "QuadraticObjectiveFunction.h"
//---------------------------------------------------------------------------
#include "NonLinearOptimization_templCode.h"
#include "DynamicLinearAlgebra_templCode.h"
#include "SparseLinearAlgebra_templCode.h"
#include "Exceptions.h"
#include "Tools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



template <typename FloatType, typename IntType>
QuadraticObjectiveFunction<FloatType, IntType>
// -------------------------
::QuadraticObjectiveFunction(IntType dim)
// -------------------------
{
	m_quadMat = SparseMatrix<FloatType>(dim);
	m_linVect = nullDVector(dim);
	m_constVal = 0;
}



template <typename FloatType, typename IntType>
QuadraticObjectiveFunction<FloatType, IntType>
// -------------------------
::QuadraticObjectiveFunction(const SparseMatrix<FloatType> &quadMat, const DynamicVector<FloatType> &linVect, const FloatType &constVal)
// -------------------------
{
	m_quadMat = quadMat;
	m_linVect = linVect;
	m_constVal = constVal;
}


template <typename FloatType, typename IntType>
void QuadraticObjectiveFunction<FloatType, IntType>
// -------
::checkDim() const
// -------
{
	if (m_linVect.getDim() != m_quadMat.getRows()) {
		throw ERangeCheck("QuadraticObjectiveFunction::checkDim() - linear and quadratic component do not match.")
	}
}


template <typename FloatType, typename IntType>
IntType QuadraticObjectiveFunction<FloatType, IntType>
// -----------
::getDimension() const
// -----------
{
	checkDim();
	return m_linVect.getDim();
}



template <typename FloatType, typename IntType>
FloatType QuadraticObjectiveFunction<FloatType, IntType>
// ---
::calc(const DynamicVector<FloatType> &pos) const
// ---
{
	DynamicVector qval = m_quadMat * pos;
	FloatType result = ((FloatType)0.5)*(qval*pos);
	result += m_linVect*pos;
	result += m_constVal;
	return result;
}

template <typename FloatType, typename IntType>
void QuadraticObjectiveFunction<FloatType, IntType>
// -----------
::calcGradient(const DynamicVector<FloatType> &pos, DynamicVector<FloatType> &gradient) const
// -----------
{
	gradient = m_quadMat*pos + m_linVect;
}


template <typename FloatType, typename IntType>
void QuadraticObjectiveFunction<FloatType, IntType>
// ----------
::calcHessian(const DynamicVector<FloatType> &pos, SparseMatrix<FloatType> &hessian) const
// ----------
{
	forget(pos);
	return m_quadMat;
}