//---------------------------------------------------------------------------
#ifndef SparseScatterOperationsH
#define SparseScatterOperationsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PTypes.h"
#include "SparseLinearAlgebra.h"
#include "DynamicLinearAlgebra.h"
#include "LinearAlgebra.h"
#include "VariablesNaming.h"
//---------------------------------------------------------------------------

namespace geoxl {

/// This function maps the rows and columns of matrix "dense" to that of the sparse matrix "dest" and then adds the values
/// to dest. The mapping is defined by indicesMapping. The sparse index is stored in the element with index equal to the dense index.
/// If monotonousAscIndices is set to true, it is assumed that indicesMapping is a monotonous ascending sequence (more efficient but does not work if violated)
template <typename FloatType, typename IntType>
void scatterAndAddMatrix(
	const DynamicMatrix<FloatType> &dense, 
	const DynamicVector<IntType> &indicesMapping, 
	SparseMatrix<FloatType> &dest, 
	bool monotonousAscIndices = false
);

template <typename FloatType, typename IntType>
void scatterAndAddDenseMatrixByName(SparseMatrix<FloatType> &dest, const DynamicMatrix<FloatType> &dense, const VAR_AAT &vaat, IntType elementIndex);

template <typename FloatType, typename IntType>
void scatterAndAddVectorByName(DynamicVector<FloatType> &dest, const DynamicVector<FloatType> &source, const VAR_AAT &vaat, IntType elementIndex);

template <typename FloatType>
void convertDenseToSparse(const DynamicMatrix<FloatType> &source, SparseMatrix<FloatType> &dest);

 
//template <typename FloatType, typename IntType>
//void scatterAndAddDenseMatrix(SparseMatrix<FloatType> &dest, const StaticMatrix<FloatType> &dense, const VAR_AAT &vaat, IntType rowIndex, IntType colIndex);

}

#endif
