#ifndef CentroidDescriptorH
#define CentroidDescriptorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl
{
class CentroidSignature;
class HierarchicalKNNIterator;

/**
	centroid descriptor - a new fancy way to deal with geometry
*/
class ANIMATIONRECONSTRUCTION_API CentroidDescriptor : public Persistent
{
	GEOXL_CLASS( CentroidDescriptor )

private:

	vector<string>		m_Attribute;

	card32				m_NumScales;
	card32				m_NumSamplesPerScale;
	float32				m_StartSigma;
	float32				m_BaseSigma;


	float32				m_WindowSize;
	float32				m_WindowSigma;



	// ------------members----------
	HierarchicalKNNIterator		*		m_hIter;
	AAT			m_PosAAT, m_NormalAAT;
	vector<AAT> m_AttributeAAT;
	float			m_CurrentMedianPointDistance;

public:
	CentroidDescriptor(void);
	~CentroidDescriptor(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	void beginSignatureCalculations(PointCloud * pc);
	void endSignatureCalculations();
	CentroidSignature * createSignature(const Vector3f &pos);

	card32 getNumScales() const {return m_NumScales;}
	void setNumScales(card32 v) {m_NumScales=v;}

	card32 getNumSamplesPerScale() const {return m_NumSamplesPerScale;}
	void setNumSamplesPerScale(card32 v) {m_NumSamplesPerScale=v;}

	float32 getStartSigma() const {return m_StartSigma;}
	void setStartSigma(float32 v) {m_StartSigma=v;}

	float32 getBaseSigma() const {return m_BaseSigma;}
	void setBaseSigma(float32 v) {m_BaseSigma=v;}

	float32 getWindowSize() const {return m_WindowSize;}
	void setWindowSize(float32 v) {m_WindowSize=v;}

	float32 getWindowSigma() const {return m_WindowSigma;}
	void setWindowSigma(float32 v) {m_WindowSigma=v;}

	float getCurrentMedianPointDistance() const {return m_CurrentMedianPointDistance;}

	IMPLEMENT_VAR_ARRAY_METHODS_STL( m_Attribute );
};


class ANIMATIONRECONSTRUCTION_API CentroidSignature : public Persistent
{
	GEOXL_CLASS( CentroidSignature )

public:
	CentroidSignature(void);
	~CentroidSignature(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	void setupZero(card32 numScales, card32 numSamplesPerScale);

	Vector4f calculateStabilityInformation();

	Matrix4f calculateTransformation(CentroidSignature * sourceSignature, bool usePCANormal);

	static Matrix4f calculateTransformation(vector<CentroidSignature*> sourceSignatures, vector<CentroidSignature*> destSignatures, bool usePCANormal,
		float descSize);

	float calculateDistance( CentroidSignature * sig);

	float computeHistogramDistance( CentroidSignature * sig);

	/// calculates different distance metrics
	Vector4f calculateDistanceVector( CentroidSignature * sig);

	vector<float32>	m_Distance;
	vector<Vector3f>	m_Centroids;
	vector<float32>	m_CentroidWeights;
	Vector3f				m_PCANormal;	// oriented
	Vector3f				m_FeaturePos;
	float32				m_SummedAreaWeight;

};


}

#endif