#ifndef CmdObjPointCloudDistortionH
#define CmdObjPointCloudDistortionH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "ProcessPointsCmdObj.h"
//---------------------------------------------------------------------------

namespace geoxl
{

/**
	provides a set of distortion operations for point clouds
*/
class ANIMATIONRECONSTRUCTION_API CmdObjPointCloudDistortion : public ProcessPointsCmdObj
{
	GEOXL_CLASS( CmdObjPointCloudDistortion )

private:

	// ----------------- 
	//	properties
	card32	m_DistortionMode;
	float32	m_Strength;				// noise, gravity: relativ to bb
											// twist: angle in rad
	
	card32	m_RandomSeed;			// in order to get reproduceable distortions
	card32	m_NumGravityBalls;

	// twist
	Vector3f	m_RotationCenter;
	Vector3f	m_RotationAxis;

	BoundingBox3f	m_PointCloudBB;


	// ----------------- 
	//	members
	struct TGravityBall
	{
		Vector3f	pos;
		float		strength;
	};
	vector<TGravityBall> m_GravityBalls;

public:
	CmdObjPointCloudDistortion(void);
	~CmdObjPointCloudDistortion(void);

   
	virtual void doAction(Object *context);
   virtual void undoAction(Object *context){}
   virtual bool canUndo() {return false;}
   virtual string getDescription();


	enum EDistortionMode
	{
		DM_Noise = 1,
		DM_Twist = 2,
		DM_GravityBalls = 3,
		DM_Rotate = 4
	};

	virtual void assign(const Object* obj, CopyContext *context = NULL);


	card32 getDistortionMode() const {return m_DistortionMode;}
	void setDistortionMode(card32 v) {m_DistortionMode=v;}

	float32 getStrength() const {return m_Strength;}
	void setStrength(float32 v) {m_Strength=v;}

	card32 getRandomSeed() const {return m_RandomSeed;}
	void setRandomSeed(card32 v) {m_RandomSeed=v;}

	card32 getNumGravityBalls() const {return m_NumGravityBalls;}
	void setNumGravityBalls(card32 v) {m_NumGravityBalls=v;}

	Vector3f getRotationCenter() const {return m_RotationCenter;}
	void setRotationCenter(Vector3f v) {m_RotationCenter=v;}

	Vector3f getRotationAxis() const {return m_RotationAxis;}
	void setRotationAxis(Vector3f v) {m_RotationAxis=v;}

	BoundingBox3f getPointCloudBB() const {return m_PointCloudBB;}
	void setPointCloudBB(BoundingBox3f v) {m_PointCloudBB=v;}


public:

	void prepareDistortion(PointInfo* info, void *context);
	void processDistortion(PointInfo* info, void *context);




};

}

#endif