#ifndef DescriptorClusteringH
#define DescriptorClusteringH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "FeatureDetector.h"
//---------------------------------------------------------------------------

namespace geoxl
{

class PGDescriptorFactory;

/**
	uses descriptorclustering to detect features
	1. builds pca-compressed descriptor with 3 dimensions (uses PGDescriptorFactory)
	2. uses kmeansclustering to reduce 3D deskriptor to n different clusters (IDs)
	3. builds spatial clusters with same deskriptor ID and uses centroids of those clusters as features
*/
class ANIMATIONRECONSTRUCTION_API DescriptorClustering : public FeatureDetector
{
	GEOXL_CLASS( DescriptorClustering )

private:

	PGDescriptorFactory * m_DescriptorFactory;
	card32		m_NumClusters;
	card32		m_NumCells;

public:
	DescriptorClustering(void);
	~DescriptorClustering(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	virtual FeatureList * detectFeatures( PointCloud * pc, float radius ) {throw "Do not use!";}

	virtual vector<FeatureList*> detectFeatures( vector<PointCloud*> pcs, float radius );

};

}

#endif