#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "FDAttribute.h"
//---------------------------------------------------------------------------
#include "VertexDescriptor.h"
#include "StringHelper.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( FDAttribute ,0)
{
	BEGIN_CLASS_INIT( FDAttribute );

	ADD_STRING_PROP( m_AttributeName, 0);
	ADD_CARD32_PROP( m_NumComponents, 0);
	ADD_CARD32_PROP( m_FirstComponent, 0);
	ADD_VAR_ARRAY_PROP( DescriptorData, 0, new Float32ClassProperty("element",MetaClass::FIRST_VERSION+0,NULL));
}

FDAttribute::FDAttribute(void)
{
	m_AttributeName = "test";
	m_NumComponents = 4;
	m_FirstComponent = 0;

}

FDAttribute::~FDAttribute(void)
{
}

void FDAttribute::assign(const Object* obj, CopyContext *context)
{
	FeatureDescriptor::assign(obj,context);
	const FDAttribute * o = dynamic_cast<const FDAttribute*>(obj);
	pAssert( o != NULL );

	m_AttributeName = o->m_AttributeName;
	m_NumComponents = o->m_NumComponents;
	m_FirstComponent = o->m_FirstComponent;


}
void FDAttribute::createDescriptor( const Vector3f &point, float size, PointCloud * pc, TDescriptorContext * context ) 
{
	HierarchicalKNNIterator *hIt = new HierarchicalKNNIterator(pc,32,NULL);
	hIt->setSeekPointAndReset(point);
	m_DescriptorData.resize(m_NumComponents);

	AAT aat = pc->getAAT( m_AttributeName );

	Vector4f vec = hIt->get4f( aat );
	for( card32 i=0;i<m_NumComponents;i++ )
	{
		m_DescriptorData[i] = vec[m_FirstComponent+i];
	}

	delete hIt;
}


void FDAttribute::setNumDescriptorDatas(card32 size) 
{
	m_DescriptorData.resize(size);
}

void FDAttribute::deleteDescriptorData( card32 index ) 
{
	m_DescriptorData.erase( m_DescriptorData.begin() + index );
}

char *FDAttribute::getDescriptorData( card32 index ) 
{
	return (char*)&m_DescriptorData[index];
}

mpcard FDAttribute::getNumDescriptorDatas() 
{
	return m_DescriptorData.size();
}

string FDAttribute::getDescriptorDataName(card32 index ) 
{
	return intToStr(index);
}
