#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "FDColorPCA.h"
//---------------------------------------------------------------------------
#include "PCA.h"
#include "HierarchicalKNNIterator.h"
#include "StringHelper.h"
#include "TopologicalKNNIterator.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( FDColorPCA ,1)
{
	BEGIN_CLASS_INIT( FDColorPCA );
	ADD_VAR_ARRAY_PROP( DescriptorData, 0, new Float32ClassProperty("element",MetaClass::FIRST_VERSION+0,NULL));
	ADD_CARD32_PROP( m_NumRings, 1 );
}

FDColorPCA::FDColorPCA(void)
{
	m_NumRings = 2;
}

FDColorPCA::~FDColorPCA(void)
{
}

void FDColorPCA::assign(const Object* obj, CopyContext *context)
{
	FeatureDescriptor::assign(obj,context);
	const FDColorPCA * o = dynamic_cast<const FDColorPCA*>(obj);
	pAssert( o != NULL );
	m_NumRings = o->m_NumRings;

}
void FDColorPCA::createDescriptor( const Vector3f &point, float size, PointCloud * pc, TDescriptorContext * context ) 
{
	PCA<float,3> pca;

	HierarchicalKNNIterator hIt(pc, 32, NULL );
	AAT posAAT = pc->getAAT( "position" );
	AAT colorAAT = pc->getAAT( "color" );

	hIt.setSeekPointAndReset( point );
	vector<Vector3f> colors;
	vector<Vector3f> centroids( m_NumRings, NULL_VECTOR3F);
	vector<card32> num_samples( m_NumRings ,0 );
	
	while( !hIt.atEnd() )
	{
		float dist = norm( point - hIt.get3f(posAAT));
		if( dist > size )
			break;

		colors.push_back( hIt.get3f( colorAAT ) );

		for( card32 i=0;i<m_NumRings;i++ )
		{
			if( dist < size * (float)(i+1) / (float)(m_NumRings) )
			{
				centroids[i] += colors.back();
				num_samples[i]++;
			}
		}

		hIt.next();
	}
	/*context->tIt->setStartPointAndReset( hIt.getPointSetPointNumber() );
	while( !context->tIt->atEnd() )
	{
		float dist = norm( point - context->tIt->get3f(posAAT));
		if( dist > size )
			break;

		colors.push_back( context->tIt->get3f( colorAAT ) );

		for( card32 i=0;i<m_NumRings;i++ )
		{
			if( dist < size * (float)(i+1) / (float)(m_NumRings) )
			{
				centroids[i] += colors.back();
				num_samples[i]++;
			}
		}

		context->tIt->next();
	}*/

//	Vector3f eigenValues;
//	Matrix3f eigenVectors;

	for( card32 i=0;i<centroids.size();i++ )
		centroids[i] /= max(1,num_samples[i]);
/*	Matrix3f C;
	for( card32 k=0;k<colors.size();k++ )
	{
		for ( unsigned int i = 0; i < 3; i++ ){
			for ( unsigned int j = 0; j <= i; j++ ){
				C[i][j] += ( colors[k][i]-centroid[i]) * (colors[k][j]-centroid[j]);
				C[j][i] = C[i][j];
			}
		}
	}
	C /= max(1,colors.size());
   C.computeEigenStructure( eigenValues, eigenVectors );
*/	

	m_DescriptorData.resize( 3*m_NumRings );
	for( card32 i=0;i<3*m_NumRings;i++ )
	{
		//m_DescriptorData[i] = fabs(eigenValues[i]);
		m_DescriptorData[i] = centroids[i/3][i%3];
	}

}

void FDColorPCA::setNumDescriptorDatas(card32 size) 
{
	m_DescriptorData.resize(size);
}

void FDColorPCA::deleteDescriptorData( card32 index ) 
{
	m_DescriptorData.erase( m_DescriptorData.begin() + index );
}

char *FDColorPCA::getDescriptorData( card32 index ) 
{
	return (char*)&m_DescriptorData[index];
}

mpcard FDColorPCA::getNumDescriptorDatas() 
{
	return m_DescriptorData.size();
}

string FDColorPCA::getDescriptorDataName(card32 index ) 
{
	return intToStr(index);
}
