#ifndef FDHistogramH
#define FDHistogramH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "FeatureDescriptor.h"
//---------------------------------------------------------------------------

namespace geoxl
{

/**
	feature descriptor - creates n histograms from a specified attribute, every
	histogram describes a fixed interval (radius)
*/
class ANIMATIONRECONSTRUCTION_API FDHistogram : public FeatureDescriptor
{
	GEOXL_CLASS( FDHistogram )

private:

	card32	m_NumBins;			// number of bins per histogram
	card32	m_NumHistograms;	// number of circular histograms
	string	m_Attribute;		// histogram attribute 
	card32	m_AttributeCompNumber;
	float32	m_MaxValue;			// maximal attribute value (upper histogram bound)
	float32	m_MinValue;			// minimal attribute value (lower histogram bound)

	IMPLEMENT_VAR_ARRAY_METHODS_STL( m_DescriptorData );

public:
	FDHistogram(void);
	~FDHistogram(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);
	virtual void createDescriptor( const Vector3f &point, float size, PointCloud * pc, TDescriptorContext * context );
	virtual void preparePointCloud( PointCloud * pc );
	virtual mpcard getNumDescriptorValues() {return m_NumHistograms*m_NumBins;}

	card32 getNumBins() const {return m_NumBins;}
	void setNumBins(card32 v) {m_NumBins=v;}

	card32 getNumHistograms() const {return m_NumHistograms;}
	void setNumHistograms(card32 v) {m_NumHistograms=v;}

	string getAttribute() const {return m_Attribute;}
	void setAttribute(string v) {m_Attribute=v;}

	float32 getMaxValue() const {return m_MaxValue;}
	void setMaxValue(float32 v) {m_MaxValue=v;}

	float32 getMinValue() const {return m_MinValue;}
	void setMinValue(float32 v) {m_MinValue=v;}

	card32 getAttributeCompNumber() const {return m_AttributeCompNumber;}
	void setAttributeCompNumber(card32 v) {m_AttributeCompNumber=v;}


};

}

#endif