#ifndef FDSIFTH
#define FDSIFTH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "FeatureDescriptor.h"
//---------------------------------------------------------------------------

namespace geoxl
{

/**
	SIFT-Scale Invariant Feature Transform
	This class implements a modified version of SIFT:
	 - precalculate gradients in objectspace
	 - estimate a local coordinate system (PCA)
	 - for each neighbour
		- transform objectspace gradient into local coordinate system
		- project into u-v-plane
		- estimate orientation via tan2(u,v)
		- add to orientation histogram
	 - select biggest orientation(s)
	 - store rotated orientation histogram(s) (NumSampleRegions^2)
*/
class ANIMATIONRECONSTRUCTION_API FDSIFT : public FeatureDescriptor
{
	GEOXL_CLASS( FDSIFT )

private:

	string			m_GradientAttribute;
	float32			m_OrientationMagnitudeThreshold;
	float32			m_WindowSmoothPercentage;
	card32			m_NumOrientationBins;
	card32			m_NumDesriptorBins;
	card32			m_NumSampleRegions;


	struct TDesc
	{
		vector< vector<float> >	keypoints;
		Vector3f			orientation;
	};

	vector<TDesc>	m_Descriptors;
	Vector3f			m_Point;
	vector<float>	m_OrientationHistogram;
	

public:
	FDSIFT(void);
	~FDSIFT(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	virtual void createDescriptor( const Vector3f &point, float size, PointCloud * pc, TDescriptorContext * context );

	virtual void preparePointCloud( PointCloud * pc );

	virtual float compare( const FeatureDescriptor * other );

	string getGradientAttribute() const {return m_GradientAttribute;}
	void setGradientAttribute(string v) {m_GradientAttribute=v;}

	void drawOrientations(float scale);

	void debugReport();

	card32 getNumOrientationBins() const {return m_NumOrientationBins;}
	void setNumOrientationBins(card32 v) {m_NumOrientationBins=v;}

	card32 getNumDesriptorBins() const {return m_NumDesriptorBins;}
	void setNumDesriptorBins(card32 v) {m_NumDesriptorBins=v;}

	card32 getNumSampleRegions() const {return m_NumSampleRegions;}
	void setNumSampleRegions(card32 v) {m_NumSampleRegions=v;}


};

}

#endif