#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "Feature.h"
//---------------------------------------------------------------------------
#include "FixedArrayClassProperty.h"
#include "StringHelper.h"
#include "FeatureDescriptor.h" 
#include "ObjectListProperty.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( Feature ,4)
{
	BEGIN_CLASS_INIT( Feature );

	ADD_VECTOR3F_PROP( m_Position, 1 );
	ADD_FLOAT32_PROP( m_Magnitude, 1 );
	ADD_CARD32_PROP( m_DetectorID, 1 );
	ADD_CARD32_PROP( m_Color, 1 );
	ADD_FLOAT32_PROP( m_Size, 2 );
	ADD_OBJECT_LIST_PROP( Descriptor, 3, FeatureDescriptor::getClass() );	
}

IMPLEMENT_GEOXL_CLASS( FeatureList, 2 )
{
	BEGIN_CLASS_INIT( FeatureList );

	ADD_OBJECT_LIST_PROP( Feature, 1, Feature::getClass() );
	ADD_VAR_ARRAY_PROP( Distance, 2, new Float32ClassProperty( "",MetaClass::FIRST_VERSION+2, NULL ) );

}

Feature::Feature(void)
{
	setPosition( makeVector3f(0,0,0) );
	setMagnitude( 0 );
	setDetectorID( 0 );
	setColor(0);
	setSize(0);
}

Feature::~Feature(void)
{
	deleteDescriptors( 0, (card32)m_Descriptors.size() );
}

void Feature::assign(const Object* obj, CopyContext *context)
{
	const Feature * f = dynamic_cast<const Feature*>(obj);
	pAssert(f);

	m_Position = f->m_Position;
	m_Magnitude = f->m_Magnitude;
	m_DetectorID = f->m_DetectorID;
	m_Color = f->m_Color;
	m_Size = f->m_Size;

	deleteDescriptors( 0, (card32)m_Descriptors.size() );
	for( card32 i=0;i<f->m_Descriptors.size();i++ )
	{
		FeatureDescriptor * fd = (FeatureDescriptor*)f->m_Descriptors[i]->copy();
		m_Descriptors.push_back(fd);
	}
}

void Feature::addDescriptor( Object * obj ) 
{
	m_Descriptors.push_back( (FeatureDescriptor*)obj );
}

void Feature::deleteDescriptors( card32 begin, card32 end ) 
{
	for( card32 i=begin;i<end;i++ )
		delete m_Descriptors[i];
	m_Descriptors.erase( m_Descriptors.begin() + begin, m_Descriptors.begin() + end);
}

Object *Feature::getDescriptor( card32 index ) 
{
	return m_Descriptors[index];
}

void Feature::moveDescriptor( card32 oldPos, card32 newPos ) 
{
	FeatureDescriptor * f = m_Descriptors[oldPos];
	m_Descriptors.erase( m_Descriptors.begin() + oldPos );
	m_Descriptors.insert( m_Descriptors.begin() + newPos, f );
}

mpcard Feature::getNumDescriptors() 
{
	return m_Descriptors.size();
}

string Feature::getDescriptorName(card32 index ) 
{
	return string("desc") + intToStr(index);
}

FeatureList::FeatureList()
{
}

FeatureList::~FeatureList()
{
	for( card32 i=0;i<m_List.size();i++ )
		delete m_List[i];
}

void FeatureList::assign(const Object* obj, CopyContext *context )
{
	const FeatureList * l = dynamic_cast<const FeatureList*>(obj);
	pAssert(l);

	for( card32 i=0;i<m_List.size();i++ )
		delete m_List[i];
	m_List.clear();

	for( card32 i=0;i<l->m_List.size();i++ )
	{
		Feature * f = (Feature*)l->m_List[i]->copy();
		m_List.push_back( f );
	}
	m_Distance = l->m_Distance;
}

void FeatureList::addFeature( Object * obj )
{
	m_List.push_back( (Feature*)obj );
}

void FeatureList::deleteFeatures( card32 begin, card32 end )
{
	for( card32 i=begin;i<end;i++ )
		delete m_List[i];
	m_List.erase( m_List.begin() + begin, m_List.begin() + end);
}

Object * FeatureList::getFeature( card32 index )
{
	return m_List[index];
}

void FeatureList::moveFeature( card32 oldPos, card32 newPos )
{
	Feature * f = m_List[oldPos];
	m_List.erase( m_List.begin() + oldPos );
	m_List.insert( m_List.begin() + newPos, f );
}

mpcard FeatureList::getNumFeatures()
{
	return m_List.size();
}

string FeatureList::getFeatureName(card32 index )
{
	return string( "Feature[") + intToStr(index) + "]";
}

bool lessfunc_FeatureMagnitude( const Feature *a, const Feature *b)
{
	return a->getMagnitude() < b->getMagnitude();
}

void FeatureList::sortByMagnitude()
{
	sort( m_List.begin(), m_List.end(), lessfunc_FeatureMagnitude );
}

void FeatureList::setNumDistances(card32 size) 
{
	m_Distance.resize(size);
}

void FeatureList::deleteDistance( card32 index ) 
{
	m_Distance.erase( m_Distance.begin() + index );
}

char *FeatureList::getDistance( card32 index ) 
{
	return (char*)&m_Distance[index];
}

mpcard FeatureList::getNumDistances() 
{
	return m_Distance.size();
}

string FeatureList::getDistanceName(card32 index ) 
{
	return intToStr( index );
}

float Feature::compareDetectorSignature( Feature * other ) 
{
	float ret = 0;
	for( card32 i=0;i<m_DetectorSignature.size();i++ )
		ret += pow( m_DetectorSignature[i] - other->m_DetectorSignature[i]  ,2.0f );

	return sqrt(ret);
}

void Feature::replaceDescriptor(card32 index, FeatureDescriptor *desc) 
{
	if( m_Descriptors.size() <= index )
		throw "Bad replace!";

	delete m_Descriptors[index];
	m_Descriptors[index] = desc;
}
