#ifndef FeatureH
#define FeatureH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl
{

class FeatureDescriptor;

/**
	describes where a feature is located and how strong it is.
*/
class ANIMATIONRECONSTRUCTION_API Feature : public Persistent
{
	GEOXL_CLASS( Feature )

	Vector3f		m_Position;		// position in pointcloud space
	float			m_Magnitude;	// defines how strong the feature is (should be in [0,1])
	card32		m_DetectorID;	// optional: 0 = no information, >0 = detector
	card32		m_Color;			// used to colorize features
	float32		m_Size;
	int32			m_PointID;
	vector<float>	m_DetectorSignature;


	vector<FeatureDescriptor*>	m_Descriptors;

public:
	Feature(void);
	~Feature(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	Vector3f getPosition() const {return m_Position;}
	void setPosition(Vector3f v) {m_Position=v;}

	float getMagnitude() const {return m_Magnitude;}
	void setMagnitude(float v) {m_Magnitude=v;}

	card32 getDetectorID() const {return m_DetectorID;}
	void setDetectorID(card32 v) {m_DetectorID=v;}

	card32 getColor() const {return m_Color;}
	void setColor(card32 v) {m_Color=v;}

	float32 getSize() const {return m_Size;}
	void setSize(float32 v) {m_Size=v;}

   void addDescriptor( Object * obj );
   void deleteDescriptors( card32 begin, card32 end );
   Object * getDescriptor( card32 index );
   void moveDescriptor( card32 oldPos, card32 newPos );
   mpcard getNumDescriptors();
   string getDescriptorName(card32 index );
	FeatureDescriptor * getFeatureDescriptor( card32 index ) {return m_Descriptors[index];}
	void replaceDescriptor(card32 index, FeatureDescriptor * desc);

	int32 getPointID() const {return m_PointID;}
	void setPointID(int32 v) {m_PointID=v;}

	void setDetectorSignature( const vector<float> &sig ) {m_DetectorSignature = sig;}
	float compareDetectorSignature( Feature * other );

};


class ANIMATIONRECONSTRUCTION_API FeatureList : public Persistent
{
	GEOXL_CLASS( FeatureList )
private:
	vector<Feature*>	m_List;
	vector<float32>	m_Distance;
public:

	FeatureList();
	~FeatureList();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

	void push_back( Feature * f ) {m_List.push_back( f );}
	mpcard size() const {return m_List.size();}
	Feature * get(int i) {return m_List[i];}
	float dist(int i, int j) {return m_Distance[i*m_List.size()+j];}

	void setupFeatureDistances( vector<float> distance ) {m_Distance = distance;}

	void sortByMagnitude();

private:
   void addFeature( Object * obj );
   void deleteFeatures( card32 begin, card32 end );
   Object * getFeature( card32 index );
   void moveFeature( card32 oldPos, card32 newPos );
   mpcard getNumFeatures();
   string getFeatureName(card32 index );

   void setNumDistances(card32 size);
   void deleteDistance( card32 index );
   char * getDistance( card32 index );
   mpcard getNumDistances();
   string getDistanceName(card32 index );

};

}

#endif