#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "FeatureCorrespondences.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
#include "CorrespondenceList.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( FeatureCorrespondences ,0)
{
	BEGIN_CLASS_INIT( FeatureCorrespondences );

	ADD_VAR_ARRAY_PROP( Frame, 0, new ObjectClassProperty( "", 0, NULL, CorrespondenceList::getClass()));

}

FeatureCorrespondences::FeatureCorrespondences(void)
{
}

FeatureCorrespondences::~FeatureCorrespondences(void)
{
	setNumFrames(0);
}

void FeatureCorrespondences::assign(const Object* obj, CopyContext *context)
{
	//Persistent::assign(obj,context);
	FeatureCorrespondences * o = dynamic_cast<FeatureCorrespondences*>((Object*)obj);
	pAssert( o != NULL );
	
	setNumFrames(0);
	for( card32 i=0;i<o->getNumFrames();i++ )
		m_Frames.push_back( (CorrespondenceList*)o->m_Frames[i]->copy() );

}
void FeatureCorrespondences::setNumFrames(card32 size) 
{
	if( size == m_Frames.size() )
		return;

	if( size < m_Frames.size() )
	{
		for( card32 i=size;i<m_Frames.size();i++ )
			delete m_Frames[i];
		m_Frames.resize(size);
	}
	else
	{
		while( size > m_Frames.size() )
		{
			m_Frames.push_back( new CorrespondenceList );
		}
	}
}

void FeatureCorrespondences::deleteFrame( card32 index ) 
{
	delete m_Frames[index];
	m_Frames.erase( m_Frames.begin() + index );
}

char *FeatureCorrespondences::getFrame( card32 index ) 
{
	return (char*)&m_Frames[index];
}

mpcard FeatureCorrespondences::getNumFrames() 
{
	return m_Frames.size();
}

string FeatureCorrespondences::getFrameName(card32 index ) 
{
	return intToStr(index);
}

card32 FeatureCorrespondences::getNumCorrespondenciesInFrame( card32 frameID ) 
{
	if (m_Frames.empty()) {
		return 0;
	} else {
		return m_Frames[frameID]->getNumCorrespondences();
	}
}

Correspondence * FeatureCorrespondences::getCorrespondence( card32 frameID, card32 index ) 
{
	return m_Frames[frameID]->get(index);
}

void FeatureCorrespondences::setup( card32 numFrames ) 
{
	setNumFrames(0);
	setNumFrames(numFrames);
}
