#ifndef FeatureDescriptorH
#define FeatureDescriptorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl
{

class PointCloud;
class TopologicalKNNIterator;
class LocalParameterization;
class ScatteredDataInterpolation;

struct TDescriptorContext
{
	PointCloud * pc;
	TopologicalKNNIterator *tIt;

	// used in parametric descriptors only
	LocalParameterization	*parameterization;
	ScatteredDataInterpolation	*interpolation;
	card32	parametricImageSize;
	float	averagePointDistance;

};

/**
	base class for feature descriptors
*/
class ANIMATIONRECONSTRUCTION_API FeatureDescriptor : public Persistent
{
	GEOXL_CLASS( FeatureDescriptor )

protected:

	vector<float32>	m_DescriptorData;

public:
	FeatureDescriptor(void);
	~FeatureDescriptor(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	virtual void createDescriptor( const Vector3f &point, float size, PointCloud * pc, TDescriptorContext * context ){}

	virtual void preparePointCloud( PointCloud * pc ) {}

	/// defines a norm between descriptors, default is euklidian norm
	virtual float compare( const FeatureDescriptor * other );

	/// normalize descriptor
	virtual void normalizeDescriptor();

	// descriptor data access methods
	virtual mpcard getNumDescriptorValues() {return m_DescriptorData.size();}
	float * getDescriptorPointer() {return &m_DescriptorData[0];}
	void setupDescriptorData( float * data, card32 numEntries );

	float operator [] (card32 i) const {pAssert( i < m_DescriptorData.size() ); return m_DescriptorData[i]; }

};

}

#endif