#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "HeighfieldParameterization.h"
//---------------------------------------------------------------------------
#include "PointSet.h"
#include "VertexDescriptor.h"
#include "PCA.h"
#include "TopologicalKNNIterator.h"
#include "HierarchicalKNNIterator.h"
#include "UnstructuredInCorePointCloud.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( HeighfieldParameterization ,0)
{
	BEGIN_CLASS_INIT( HeighfieldParameterization );
}

HeighfieldParameterization::HeighfieldParameterization(void)
{
}

HeighfieldParameterization::~HeighfieldParameterization(void)
{
}

void HeighfieldParameterization::assign(const Object* obj, CopyContext *context)
{
	LocalParameterization::assign(obj,context);
}

void HeighfieldParameterization::calculate( PointCloud * pc, const Vector3f &point, float radius ) 
{
	pAssert( dynamic_cast<UnstructuredInCorePointCloud*>(pc) );
	UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);

	
	// setup vertex descriptor
	VertexDescriptor * vd = (VertexDescriptor*)pc->getDescr()->copy();
	if( !vd->providesAttribute( "param_pos" ) )
		vd->pushAttrib( VAD( "param_pos", 2, VAD::DATA_FORMAT_FLOAT32 ) );
	if( !vd->providesAttribute( "height" ) )
		vd->pushAttrib( VAD( "height", 1, VAD::DATA_FORMAT_FLOAT32 ) );
	if( !vd->providesAttribute( "point_index" ) )
		vd->pushAttrib( VAD( "point_index", 1, VAD::DATA_FORMAT_INT32 ) );
	AAT posAAT = vd->getToken( "position" );
	AAT paramAAT = vd->getToken( "param_pos" );
	AAT heightAAT = vd->getToken( "height" );
	AAT pointIndexAAT = vd->getToken( "point_index" );


	//FixedKNNTopologyIterator * topIter = new FixedKNNTopologyIterator(upc->getPointSet(), 
	//	upc->getAttachments(), upc, 8 );
	//TopologicalKNNIterator * tIt = new TopologicalKNNIterator( pc, topIter );
	HierarchicalKNNIterator * hIt = new HierarchicalKNNIterator( pc, 32, NULL );

	// find nearest point and get index
	hIt->setSeekPointAndReset( point );
	//PointIndex pointIndex = hIt->getPointSetPointNumber();

	// init PCA and count # of points in radius
	PCA<float,3> pca;
	static vector<Vector3f> positions;
	Vector3f centroid = makeVector3f(0,0,0);
	card32 numPoints = 0;
	//tIt->setStartPointAndReset(pointIndex);
	float currentDistance;
	do
	{
		currentDistance = norm( hIt->get3f( posAAT ) - point );
		pca.addPoint( hIt->get3f(posAAT) );
		//positions.push_back(hIt->get3f(posAAT));
		//centroid += positions.back();
				
		numPoints++;
		hIt->next();
		if( hIt->atEnd() )
			break;
	}
	while(currentDistance < radius);

	// pca
	Matrix3f covariance = IDENTITY3F * 0;
	Matrix3f eigenVectors;
	Vector3f eigenValues;
/*	for( card32 i=0;i<positions.size();i++ )
	{
		for( card32 j=0;j<3;j++ )
		{

		}
	}*/

	pca.analyze( eigenValues, eigenVectors, centroid );
	Vector3f normal = normalize( eigenVectors[0].crossProduct( eigenVectors[1]) );
	Matrix3f inv_tangentSystem = calcTangentSystem( normal ).transpose();

	Matrix3f rotMat = eigenVectors;
	rotMat[2] = normalize( rotMat[0].crossProduct( rotMat[1]) );

	m_WorldToParametric = expand3To4( rotMat.transpose() ) * makeTranslation4f( -point );
	m_ParametricToWorld = makeTranslation4f( point ) * expand3To4( rotMat );
	
	// create & init pointset
	if( m_PointSet == NULL )
	{
		m_PointSet = new PointSet;
	}
	m_PointSet->clearAndSetup(1, numPoints, vd );

	// transform vertices into tangent space and store them in pointset
	hIt->setSeekPointAndReset(point);
	vector<char> buffer( vd->getSize() );
	Vector2f * param_pos_ptr = (Vector2f*)( (char*)&buffer[0] + vd->getAttributeOffset(vd->getIndex("param_pos")) );
	float * height_ptr = (float*)( (char*)&buffer[0] + vd->getAttributeOffset(vd->getIndex("height")) );
	int * index_ptr = (int*)( (char*)&buffer[0] + vd->getAttributeOffset(vd->getIndex("point_index")) );
	for(card32 i=0;i<numPoints;i++ )
	{
		hIt->getVertex( &buffer[0] );	// copy vertex into buffer
		
		// transform into tangent space
		Vector3f pos = hIt->get3f( posAAT );
		//Vector3f tang_pos = eigenVectors.transpose() * (pos - point);
		Vector3f tang_pos = shrink4To3( m_WorldToParametric * expand3To4(pos));
		*param_pos_ptr = makeVector2f( tang_pos[0],tang_pos[1] );
		*height_ptr = tang_pos[2];
		*index_ptr = hIt->getCurrentPointIndex();
		
		// store in pointset
		m_PointSet->setVertex( i, &buffer[0] );

		hIt->next();
	}


	//delete tIt;
//	delete topIter;
	delete hIt;
}
