#ifndef HeighfieldParameterizationH
#define HeighfieldParameterizationH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "LocalParameterization.h"
//---------------------------------------------------------------------------

namespace geoxl
{

/**
	calculate PCA to estimate a local coordinate system, which is used for heightfield 
	parameterization. height is stored in an additional channel 'height'
*/
class ANIMATIONRECONSTRUCTION_API HeighfieldParameterization : public LocalParameterization
{
	GEOXL_CLASS( HeighfieldParameterization )

private:

	Matrix4f	m_WorldToParametric;
	Matrix4f	m_ParametricToWorld;

public:
	HeighfieldParameterization(void);
	~HeighfieldParameterization(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	virtual void calculate( PointCloud * pc, const Vector3f &point, float radius );

	/// returns transformation matrix from world to local coordinate frame from current parameterization
	Matrix4f getWorldToParametric4f() {return m_WorldToParametric;}

	/// returns transformation matrix from local coordinate frame to world from current parameterization
	Matrix4f getParametricToWorld4f() {return m_ParametricToWorld;}


};

}

#endif