#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "LocalExtremaDetector.h"
//---------------------------------------------------------------------------
#include "PointCloudIterators.h"
#include "TopologicalKNNIterator.h"
#include "ProgressWindow.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( LocalExtremaDetector ,0)
{
	BEGIN_CLASS_INIT( LocalExtremaDetector );
}

LocalExtremaDetector::LocalExtremaDetector(void)
{
	m_MinMagnitude = 0.1f;
	m_ComponentNumber = 0;
	m_Attribute = "color";
}

LocalExtremaDetector::~LocalExtremaDetector(void)
{
}

void LocalExtremaDetector::assign(const Object* obj, CopyContext *context)
{
	FeatureDetector::assign(obj,context);
}

bool isBigger( const Feature* a, const Feature *b )
{
	return a->getMagnitude() > b->getMagnitude();
}

FeatureList * LocalExtremaDetector::detectFeatures( PointCloud * pc, float radius ) 
{
	pAssert( dynamic_cast<UnstructuredInCorePointCloud*>(pc) );
	pAssert( pc->providesAttribute( m_Attribute ) );
	UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);


	// setup topolgy and iterators
/*	FixedKNNTopologyIterator * topIter = new FixedKNNTopologyIterator(upc->getPointSet(), 
		upc->getAttachments(), upc, 8 );
	TopologicalKNNIterator * tIt = new TopologicalKNNIterator( pc, topIter );
	SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | 
		SceneObjectIteratorType::CAP_INDEXED_PC );	
	BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	IndexedPointCloudIterator * iIt = dynamic_cast<IndexedPointCloudIterator*>(iter);

	// gather set of local extremas
	FeatureList * ret = new FeatureList;
	AAT posAAT = pc->getAAT( "position" );
	vector<char> buffer( pc->getDescr()->getSize() );
	float * attributePointer = (float*)((char*) &buffer[0] + 
		pc->getDescr()->getAttributeOffset( pc->getDescr()->getIndex( m_Attribute ) ) +
		sizeof(float)*m_ComponentNumber);
	//progressWindow->pushStep( true, "detection" );
	card32 index=0;
	while( !bIt->atEnd() && progressWindow->progress( (float)index/(float)pc->getNumPoints()*100.0f ) )
	{
		index++;
		Vector3f currentPoint = bIt->get3f( posAAT );
		bIt->getVertex( &buffer[0] );
		float currentValue = *attributePointer;
		tIt->setStartPointAndReset( iIt->getCurrentPointIndex() );
		tIt->next();
		tIt->getVertex( &buffer[0] );
		bool minima = *attributePointer > currentValue;
		tIt->next();
		float last_dist = norm( currentPoint - tIt->get3f( posAAT ) );
		while( true )
		{
			float dist = norm( currentPoint - tIt->get3f( posAAT ) );
			int detectorID = 0;
			if( dist > radius )
			{
				Feature *f = new Feature;
				f->setDetectorID(detectorID);
				f->setPosition( currentPoint );
				f->setMagnitude( 1.0f );		// best feature
				ret->push_back( f );
			}

			tIt->getVertex( &buffer[0] );
			bool isMaxima = !minima && (currentValue > *attributePointer);
			bool isMinima = minima && (currentValue < *attributePointer);
			if( !isMaxima && !isMinima )
			{
				if( last_dist > radius * m_MinMagnitude )	// force minimum 
				{
					Feature *f = new Feature;
					f->setDetectorID(detectorID);
					f->setPosition(currentPoint);
					f->setMagnitude( last_dist / radius );
					ret->push_back( f );
				}
				break;	// next point
			}


			tIt->next();
			if( tIt->atEnd() )
			{
				if( last_dist > radius * m_MinMagnitude )	// force minimum 
				{
					Feature *f = new Feature;
					f->setDetectorID(detectorID);
					f->setPosition(currentPoint);
					f->setMagnitude( last_dist / radius );
					ret->push_back( f );
				}
				break;	// next point
			}
			last_dist = dist;
		}
		
		bIt->next();
	}

	//bool b;
	//progressWindow->popSteps(b);*/


	SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | 
		SceneObjectIteratorType::CAP_INDEXED_PC );	
	BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	IndexedPointCloudIterator * iIt = dynamic_cast<IndexedPointCloudIterator*>(iter);
	AAT curvAAT = pc->getAAT( "curvature" );
	AAT posAAT = pc->getAAT( "position" );

	float summed_curv = 0.0f;
	vector<float> curvatures;
	vector<Vector3f> positions;
	vector<int> flags;
	while( !bIt->atEnd() )
	{
		float curv = fabs( bIt->get4f( curvAAT )[0] );
		if( curv > 0.05f )
		{
			curvatures.push_back( summed_curv + curv );
			flags.push_back(1);
			positions.push_back( bIt->get3f( posAAT ));
			summed_curv += curv;
		}
		bIt->next();
	}

	FeatureList * ret = new FeatureList;
	for( card32 i=0;i<1000;i++ )
	{
		float r = (float)rand() / (float)RAND_MAX * summed_curv;
		int a = 0;
		int b = (int)curvatures.size()-1;
		while( true )
		{
			if( (curvatures[a] <= r && curvatures[a+1] >= r) || a == b)
			{
				if( flags[a] )
				{
					Feature * f = new Feature;
					f->setPosition( positions[a] );
					f->setColor( 0xff0000 );
					ret->push_back( f );
					flags[a] = 0;
				}
				break;
			}
			int m = (a+b) / 2;
			if( curvatures[m] < r )
			{
				a = m;
			}
			else
			{
				b = m;
			}
		}

	}


//	ret->sortByMagnitude();

	return ret;
}
