#ifndef LocalParameterizationH
#define LocalParameterizationH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl
{

class PointSet;

/**
	estimates a local parameterization for a point cloud around a point.
	results are stored in a PointSet which includes all parameterized vertices
	and an additional channel 'param_pos'
*/
class ANIMATIONRECONSTRUCTION_API LocalParameterization : public Persistent
{
	GEOXL_ABSTRACT_CLASS( LocalParameterization )

protected:

	PointSet *		m_PointSet;

public:
	LocalParameterization(void);
	~LocalParameterization(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	/// calculates a parameterization for point cloud 'pc' around a point 'point'.
	virtual void calculate( PointCloud * pc, const Vector3f &point, float radius ) = 0;

	/// returns NULL or a PointSet with all parameterized points. Every point has 
	/// an additional attribute 'param_pos' holding parametric coordinates.
	/// call calculate() _before_ using this pointset, LocalParameterization owns 
	/// this PointSet!
	PointSet * getParametricPoints() {return m_PointSet;}



};

}

#endif