#ifndef MLSMultiScaleFeatureDetectorH
#define MLSMultiScaleFeatureDetectorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "FeatureDetector.h"
//---------------------------------------------------------------------------

namespace geoxl
{

/**
	extract features using a moving least squares multi-scale representation
*/
class ANIMATIONRECONSTRUCTION_API MLSMultiScaleFeatureDetector : public FeatureDetector
{
	GEOXL_CLASS( MLSMultiScaleFeatureDetector )

private:

	card32	m_NumberOfScales;
	float32	m_H0;	// h of scale 0
	float32	m_F;	// h = h0 * F^n
	float32	m_AveragePointDistance;	// will be calculated if == 0
	
	string	m_Attribute;	// = "" uses normaldifference


public:
	MLSMultiScaleFeatureDetector(void);
	~MLSMultiScaleFeatureDetector(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	virtual FeatureList * detectFeatures( PointCloud * pc, float radius );

	card32 getNumberOfScales() const {return m_NumberOfScales;}
	void setNumberOfScales(card32 v) {m_NumberOfScales=v;}

	float32 getH0() const {return m_H0;}
	void setH0(float32 v) {m_H0=v;}

	float32 getF() const {return m_F;}
	void setF(float32 v) {m_F=v;}

	float32 getAveragePointDistance() const {return m_AveragePointDistance;}
	void setAveragePointDistance(float32 v) {m_AveragePointDistance=v;}

	string getAttribute() const {return m_Attribute;}
	void setAttribute(string v) {m_Attribute=v;}

};

}

#endif