#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "MultiFrameCorrespondenceValidation.h"
//---------------------------------------------------------------------------
#include "FeatureCorrespondences.h"
#include "CorrespondenceList.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( MultiFrameCorrespondenceValidation ,0)
{
	BEGIN_CLASS_INIT( MultiFrameCorrespondenceValidation );
}

MultiFrameCorrespondenceValidation::MultiFrameCorrespondenceValidation(void)
{
}

MultiFrameCorrespondenceValidation::~MultiFrameCorrespondenceValidation(void)
{
}

void MultiFrameCorrespondenceValidation::assign(const Object* obj, CopyContext *context)
{
	//Persistent::assign(obj,context);
	const MultiFrameCorrespondenceValidation * o = dynamic_cast<const MultiFrameCorrespondenceValidation*>(obj);
	pAssert( o != NULL );

}

struct TMultiFrameCorr
{
	vector< Correspondence* > list;
	vector< card32 > frameIDs;
};

void MultiFrameCorrespondenceValidation::validateCorrspondences( FeatureCorrespondences * correspondences ) 
{
	vector<Correspondence*> lastFrame,thisFrame;
	for( card32 frame=0;frame<correspondences->getNumFrames()-1;frame++ )
	{
		for( card32 c=0;c<correspondences->getNumCorrespondenciesInFrame( frame );c++ )
		{
			Correspondence * corr = correspondences->getCorrespondence( frame, c );
			if( corr->getFlags() == 0 )
			{
				thisFrame.push_back(corr);
			}
		}

		for( card32 i=0;i<lastFrame.size();i++ )
		{
			bool foundConnection = false;
			for( card32 c=0;c<correspondences->getNumCorrespondenciesInFrame( frame );c++ )
			{
				Correspondence * corr = correspondences->getCorrespondence( frame, c );
				if( corr->getFlags() == 0 )
				{
					if( norm( lastFrame[i]->getPosition2() - corr->getPosition1() ) < 1e-10f )
					{
						foundConnection = true;
					}
				}
			}
			if( !foundConnection )
			{
				Correspondence * bestCorr = NULL;
				float bestCorrDist = 1e20f;

				for( card32 c=0;c<correspondences->getNumCorrespondenciesInFrame( frame );c++ )
				{
					Correspondence * corr = correspondences->getCorrespondence( frame, c );
					if( corr->getFlags() == Correspondence::CF_Initial )
					{
						if( norm( lastFrame[i]->getPosition2() - corr->getPosition1() ) < 1e-10f )
						{
							float movement = norm( corr->getPosition2() - corr->getPosition1() );
							if( movement < bestCorrDist )
							{
								bestCorrDist = movement;
								bestCorr = corr;
							}
						}
					}
				}

				if( bestCorr )
					if( bestCorrDist < 0.1f )
						bestCorr->setFlags( Correspondence::CF_Guessed );

			}
		}
		lastFrame = thisFrame;

	}
	/*vector< TMultiFrameCorr > corrList, finishedCorrList;
	for( card32 frame=0;frame<correspondences->getNumFrames()-1;frame++ )
	{
		// search connected corrspondences 
		for( card32 c=0;c<correspondences->getNumCorrespondenciesInFrame( frame );c++ )
		{
			Correspondence * corr = correspondences->getCorrespondence( frame, c );
			bool foundConnection = false;
			for( card32 i=0;i<corrList.size();i++ )
			{
				if( corrList[i].frameIDs.back() == frame - 1 )
					if( norm( corrList[i].list.back()->getPosition2() - corr->getPosition1() ) < 1e-10f )
					{
						corrList[i].frameIDs.push_back(frame);
						corrList[i].list.push_back( corr );
						foundConnection = true;
					}
			}
			if( !foundConnection )
			{
				TMultiFrameCorr mfc;
				mfc.frameIDs.push_back(frame);
				mfc.list.push_back(corr);
				corrList.push_back(mfc);
			}
		}

		// split finished correspondences into a seperate container
		vector< TMultiFrameCorr > tmpList;
		for( card32 i=0;i<corrList.size();i++ )
		{
			if( corrList[i].frameIDs.back() == frame )
			{
				tmpList.push_back( corrList[i] );
			}
			else
			{
				finishedCorrList.push_back( corrList[i] );
			}
		}
		corrList = tmpList;
	}

	for( card32 i=0;i<corrList.size();i++ )
		finishedCorrList.push_back( corrList[i] );

	Vector3f colors[8];
	colors[0] = makeVector3f(0,0,1);
	colors[1] = makeVector3f(1,0,0);
	colors[2] = makeVector3f(1,0.2f,0);
	colors[3] = makeVector3f(1,0.4f,0);
	colors[4] = makeVector3f(1,0.6f,0);
	colors[5] = makeVector3f(1,0.8f,0);
	colors[6] = makeVector3f(1,1,0);
	colors[7] = makeVector3f(1,1,1);
	
	for( card32 i=0;i<finishedCorrList.size();i++ )
	{
		TMultiFrameCorr mfc = finishedCorrList[i];

		for( card32 j=0;j<mfc.list.size();j++ )
			mfc.list[j]->setColor( colors[ min(7,mfc.list.size() - 1) ] );

		
	}*/
}
