#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIFeatureTest.h"
//---------------------------------------------------------------------------
#include "CmdObjPointCloudDistortion.h"
#include "SceneGraphState.h"
#include "PointCloudTools.h"
#include "FeatureDetector.h"
#include "FeatureDescriptor.h"
#include "StandardChannels.h"
#include "HeighfieldParameterization.h"
#include "SplatPullPushInterpolation.h"
#include "VertexArray.h"
#include "PointSet.h"
#include "LinearTransformPriors.h"
#include "SpectralCorrespondenceValidation.h"
#include "MLSMultiScaleFeatureDetector.h"
#include "FDAttribute.h"
#include "RegistrationTools.h"
#include "FDSIFT.h"
#include "SelectionTools.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "RenderingTool.h"
#include "MultiResolutionPC.h"
#include "MultiScaleGenerator.h"
#include "FeatureRegistration.h"
#include "Feature.h"
#include "FDHistogram.h"
#include "ProgressWindow.h"
#include "RGBAImage.h"
#include "ImageDisplayer.h"
#include "FastFeatures.h"
#include "Monomials2ndOrder.h"
#include "StringHelper.h"
#include "PGDescriptorFactory.h"
#include "DescriptorClustering.h"
#include "NormalDistribution.h"
#include <float.h>
#include "TableOutputTools.h"
#include "FDAverage.h"
#include "DynamicLinearAlgebraTools.h"
#include "DynamicLinearAlgebraTools_templCode.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( PCIFeatureTest ,0)
{
	BEGIN_CLASS_INIT( PCIFeatureTest );
}

PCIFeatureTest::PCIFeatureTest(void)
{
	m_DistortionMode = CmdObjPointCloudDistortion::DM_Twist;
	m_DistortionStrength = 0.0f;
	m_DistortionNumGravityBalls = 10;
	m_DistortionRandomSeed = 15091980;

	/*m_FeatureDetector = new MLSMultiScaleFeatureDetector;
	MLSMultiScaleFeatureDetector * mls = dynamic_cast<MLSMultiScaleFeatureDetector*>(m_FeatureDetector);
	mls->setH0( 1.5f );
	mls->setNumberOfScales(4);
/*	m_FeatureDescriptor = new FDAttribute;
	FDAttribute * fd = dynamic_cast<FDAttribute*>(m_FeatureDescriptor);
	fd->setNumComponents(1);*/
//	m_FeatureDescriptor = new FDSIFT;
	m_FeatureDetector = new DescriptorClustering;
	m_FeatureDescriptor = new FDHistogram;
	m_HideInitialPairs = false;
	m_ShowPairsPercentage = 100.0f;
	m_DrawGradients = false;
	m_GradientLength = 0.1f;
	m_RelativeFeatureSize = 0.2f;

	m_ScreenWidth = 400;
	m_ScreenHeight = 400;
	m_DrawFeatureScale = false;

	m_FeaturesA = NULL;
	m_FeaturesB = NULL;
	m_Features = new FeatureList;
	m_PointCloudA = m_PointCloudB = NULL;

	m_SpectralSigma = 2.0f;
	m_SpectralAddDiff = 1.0f;
	m_SpectralScaleDiff = 0.0;
	m_DescClusterRadius = 10.0f;

	/*vector<SplatPullPushInterpolation::TWorkingVertex> data;
	SplatPullPushInterpolation::TWorkingVertex v1,v2,v3;
	v1.paramPos = makeVector2f(0.1f, 0.27f);
	v1.attribute = makeVector4f(1.0f,0.0f,0.0f,0.0f);
	v2.paramPos = makeVector2f(0.65f, 0.41f);
	v2.attribute = makeVector4f(1.0f,1.0f,0.0f,0.0f);
	v3.paramPos = makeVector2f(0.51f, 0.78f);
	v3.attribute = makeVector4f(0.0f,0.0f,1.0f,0.0f);
	data.push_back( v1 );
	data.push_back( v2 );
	data.push_back( v3 );
	SplatPullPushInterpolation test;
	test.createImage3f( 64,data,false);*/

}

PCIFeatureTest::~PCIFeatureTest(void)
{
	delete m_FeaturesA;
	delete m_FeaturesB;
	delete m_Features;
}

void PCIFeatureTest::assign(const Object* obj, CopyContext *context)
{
	PCInteractionTool::assign(obj,context);
	const PCIFeatureTest * o = dynamic_cast<const PCIFeatureTest*>(obj);
	pAssert( o != NULL );

	m_DistortionMode = o->m_DistortionMode;
	m_DistortionStrength = o->m_DistortionStrength;
	m_DistortionNumGravityBalls = o->m_DistortionNumGravityBalls;
	m_DistortionRandomSeed = o->m_DistortionRandomSeed;
	m_HideInitialPairs = o->m_HideInitialPairs;
	m_ShowPairsPercentage = o->m_ShowPairsPercentage;
	m_DrawGradients = o->m_DrawGradients;
	m_GradientLength = o->m_GradientLength;
	m_RelativeFeatureSize = o->m_RelativeFeatureSize;
	m_DrawFeatureScale = o->m_DrawFeatureScale;
	delete m_FeaturesA;
	delete m_FeaturesB;
	delete m_Features;
	m_FeaturesA = NULL;
	m_FeaturesB = NULL;
	m_Features = NULL;
}

static vector<Vector3f> patchVertices;
static vector<Vector3f> patchNormals;
static vector<Vector3f> patchColors;
static vector<float> patchNormalDist;
static Vector3f extremePoint;

void PCIFeatureTest::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
	if( buttonsState.getLeft() )
	{
		Vector3f point;
		string pcName;

		if( getPointFromCursor(x,y, point, pcName ) )
		{
			PointCloud * pc = getPointCloud( getScene(), pcName );
			Vector3f lastpoint = point;

			if( pc )
			{
				MultiScaleGenerator mgen;
				MultiResolutionPC mresPC;
				mresPC.setup( pc, 100);
				float patchSamplingSpace = 3.0f;
				float sourceSamplingSpace = 0;
				PointCloud * newPC = mgen.smoothPC( pc, &mresPC, 10.0f, patchSamplingSpace, sourceSamplingSpace);
				addPointCloud( getScene(), newPC, "test" );
				return;
			}
			/*MultiResolutionPC mrpc;

			if( pc == NULL )
			{
				return;
			}
			mrpc.setup( pc, 50 );

			FastFeatures ff;
			vector<Vector3f> pts = ff.detectFeatures( &mrpc );
			delete m_Features;
			m_Features = new FeatureList;
			for( card32 i=0;i<pts.size();i++)
			{
				Feature *f = new Feature;
				f->setPosition( pts[i] );
				f->setSize( 0.01f );
				f->setColor( 0x00ff00 );
				m_Features->push_back( f );
			}*/

/*			float featureSize = getAveragePointDistance(pc) * 10.0f;

			HeighfieldParameterization param;
			param.calculate( pc, point, featureSize );
			PointSet * tmpPS = param.getParametricPoints();

			SplatPullPushInterpolation interp;
			TemplatedImage< StaticVector<float,3> > * param_image = interp.createImage3f( 32, tmpPS, "curvature",0, true );

			static RGBAFloat32Image * image = NULL;
			if( image == NULL )
				image = new RGBAFloat32Image(param_image->getSizeX(),param_image->getSizeY() );

			Vector3f * src = param_image->getPixels();
			Vector4f * dst = image->getPixels();
			for( card32 i=0;i<param_image->getSizeX() * param_image->getSizeY();i++ )
			{
				dst[i] = expand3To4( src[i] );
			}

			ImageDisplayer::DisplayRGBAFloat32Image ev(image);
			debugOutput.displayEvent( ImageDisplayer::getClass(), "muh", &ev );

			delete param_image;*/
			
			patchVertices.clear();

			HeighfieldParameterization hparam;
			float radius = getMedianPointDistance( pc ) * 100.0f * m_RelativeFeatureSize;
			for( card32 it = 0;it<1;it++)
			{
			
			hparam.calculate(pc, point, radius);
			PointSet * ps = hparam.getParametricPoints();
			vector<Vector3f> points;
			AAT paramposAAT = ps->getAAT( "param_pos" );
			AAT heightAAT = ps->getAAT( "height" );
			for( card32 i=0;i<ps->getNumEntries();i++ )
			{
				Vector2f ppos = ps->get2f(i,paramposAAT);
				float height = ps->get1f(i, heightAAT);
				points.push_back( makeVector3f(ppos[0], ppos[1], height)/radius );
			}

			StaticVector<float,6> coeffs,coeffs2, diffCoeffs;
			float sigma = sqrt( -log(0.05f) );
			float h1_2 = sigma*sigma;
			float h2_2 = pow( sigma * 2.25f, 2.0f);
			Monomials2ndOrder::computeCoeffs( points, coeffs, sigma*sigma/ pow(2.25f,2.0f));
			Monomials2ndOrder::computeCoeffs( points, coeffs2, sigma*sigma);
			diffCoeffs = coeffs - coeffs2;

			vector<Vector3f> diffPoints;
			card32 patchRes = 16;
			float sampleRadius = 0.25f;
			for( card32 x=0;x<patchRes;x++)
			{
				for( card32 y=0;y<patchRes;y++)
				{
					Vector2f paramPos = (makeVector2f(x,y)/patchRes - makeVector2f(0.5f,0.5f)) * sampleRadius;
					diffPoints.push_back( makeVector3f( paramPos[0], paramPos[1], Monomials2ndOrder::evaluate(diffCoeffs, paramPos)));
				}
			}

			Monomials2ndOrder::computeCoeffs( diffPoints, diffCoeffs);

			
			//coeffs = coeffs2 - coeffs;

			//todo!
			// normal difference
			// small radius
			// move center to discrete min/max?
			// scale points!
		

/*
			const card32 NUM_NEIGHBORS = ps->getNumEntries();
			coeffs[0] = 0.0f; coeffs[1] = 0.0f; coeffs[2] = 0.0f; coeffs[3] = 0.0f; coeffs[4] = 0.0f; coeffs[5] = 0.0f;
			StaticVector<float32, 6> cd;
			StaticMatrix<float32, 6,6> dm;
			for (card32 i=0; i<6;i++) {
				for (card32 j=0; j<6;j++) {
					dm[i][j] = 0.0f;
				}
			}

			AAT paramPosAAT = ps->getAAT( "param_pos" );
			AAT heightAAT = ps->getAAT( "height" );

			for (card32 pIndex=0; pIndex<NUM_NEIGHBORS; pIndex++)
			{
				Vector2f paramPos = ps->get2f( pIndex, paramPosAAT );
				const float32 u = paramPos[0];
				const float32 v = paramPos[1];
				const float32 n = ps->get1f( pIndex, heightAAT);
				const float32 w = ((exp(-(u*u+v*v)/h2_2)) - (exp(-(u*u+v*v)/h1_2))) / (float32)NUM_NEIGHBORS;

				// c[j] += b[j](uv(i)) * n(i) * w(uv(i));
				cd[0] =       w;
				cd[1] = u   * w;
				cd[2] = v   * w;
				cd[3] = u*u * w;
				cd[4] = u*v * w;
				cd[5] = v*v * w;

				coeffs[0] += cd[0] * n;
				coeffs[1] += cd[1] * n;
				coeffs[2] += cd[2] * n;
				coeffs[3] += cd[3] * n;
				coeffs[4] += cd[4] * n;
				coeffs[5] += cd[5] * n;

				// 1 u v uu uv vv

				dm[0][0] += w;     dm[0][1] += u*w;     dm[0][2] += v*w;     dm[0][3] += u*u*w;     dm[0][4] += u*v*w;     dm[0][5] += v*v*w;
				dm[1][0] += w*u;   dm[1][1] += u*w*u;   dm[1][2] += v*w*u;   dm[1][3] += u*u*w*u;   dm[1][4] += u*v*w*u;   dm[1][5] += v*v*w*u;
				dm[2][0] += w*v;   dm[2][1] += u*w*v;   dm[2][2] += v*w*v;   dm[2][3] += u*u*w*v;   dm[2][4] += u*v*w*v;   dm[2][5] += v*v*w*v;
				dm[3][0] += w*u*u; dm[3][1] += u*w*u*u; dm[3][2] += v*w*u*u; dm[3][3] += u*u*w*u*u; dm[3][4] += u*v*w*u*u; dm[3][5] += v*v*w*u*u;
				dm[4][0] += w*u*v; dm[4][1] += u*w*u*v; dm[4][2] += v*w*u*v; dm[4][3] += u*u*w*u*v; dm[4][4] += u*v*w*u*v; dm[4][5] += v*v*w*u*v;
				dm[5][0] += w*v*v; dm[5][1] += u*w*v*v; dm[5][2] += v*w*v*v; dm[5][3] += u*u*w*v*v; dm[5][4] += u*v*w*v*v; dm[5][5] += v*v*w*v*v;
			}

			StaticMatrix<float32, 6,6> invDm;
			bool success;
			invDm = invertMatrix(dm, &success, 1e-7f);
			if (success) {
				coeffs = invDm * coeffs;
			} else {
				coeffs[0] = 0; coeffs[1] = 0; coeffs[2] = 0;
				coeffs[3] = 0; coeffs[4] = 0; coeffs[5] = 0;
			}
*/

			Matrix4f paramToWorld = hparam.getParametricToWorld4f();
			extremePoint = makeVector3f(0,0,0);
			const float A = diffCoeffs[3];
			const float B = diffCoeffs[4];
			const float C = diffCoeffs[5];
			const float D = diffCoeffs[1];
			const float E = diffCoeffs[2];
			float detH = 4.0f * A * C - B*B;
			if( detH > 0 )
			{
				float cu = (-2.0f * C * D + B * E) / detH;
				float cv = (-2.0f * A * E + B * D) / detH;
				extremePoint = makeVector3f(cu,cv,Monomials2ndOrder::evaluate( coeffs, makeVector2f(cu,cv))*radius);
				extremePoint = shrink4To3( paramToWorld * expand3To4(extremePoint));
				lastpoint = point;
				point = extremePoint;
			}

			patchVertices.clear();
			patchNormals.clear();
			patchColors.clear();
			patchNormalDist.clear();
			float minT = 1e10f;
			float maxT = -1e10f;
			for( card32 x=0;x<patchRes;x++)
			{
				for( card32 y=0;y<patchRes;y++)
				{
					Vector2f paramPos = (makeVector2f(x,y)/patchRes - makeVector2f(0.5f,0.5f)) * 0.5f;
					patchVertices.push_back( makeVector3f( paramPos[0], paramPos[1], Monomials2ndOrder::evaluate(coeffs,paramPos)));
					float derivateU = 2.0f * coeffs[3] * paramPos[0]
									+ coeffs[4] * paramPos[1]
									+ coeffs[1];
					float derivateV = 2.0f * coeffs[5] * paramPos[1]
									+ coeffs[4] * paramPos[0]
									+ coeffs[2];

					Vector3f normal = makeVector3f(1,0,derivateU).crossProduct(makeVector3f(0,1,derivateV));
					normal = normalize(normal);

					patchNormals.push_back( normal );

					/*Vector3f n = normal;
					Vector3f s = patchVertices.back();

					float a =	coeffs2[3] * sqr(n[0]) + 
								coeffs2[4] * n[0] * n[1] + 
								coeffs2[5] * sqr(n[1]);
					float b =	coeffs2[3] * 2.0f * n[0]*s[0] +
								coeffs2[4] * ( n[0]*s[1] + n[1]*s[0] ) +
								coeffs2[5] * 2.0f * n[1]*s[1] +
								coeffs2[1] * n[0] +
								coeffs2[2] * n[1] -
								n[2];
					float c =	coeffs2[3] * sqr(s[0]) +
								coeffs2[4] * s[0] * s[1] + 
								coeffs2[5] * sqr(s[1]) + 
								coeffs2[1] * s[0] + 
								coeffs2[2] * s[1] + 
								coeffs2[0] -
								s[2];

					Vector3f color = makeVector3f(1,1,1);
					float tmp = sqr(b) - 4.0f*a*c;
					float t;
					if( fabs(tmp)<1e-15f )
					{
						t = -b / 2.0f / a;
						t *= radius;
						color = makeVector3f(1,1,1) * t;
						minT = min( t, minT );
						maxT = max( t, maxT );
						patchNormalDist.push_back( t );
					}
					else
					{
						if( tmp > 0 )
						{
							t = (tmp-b) / 2.0f / a;
							float t2 = (-tmp-b) / 2.0f / a;
							if( fabs(t2) < fabs(t) )
								t = t2;
							t *= radius;
							color = makeVector3f(1,1,1) * t;
							minT = min( t, minT );
							patchNormalDist.push_back( t );
							maxT = max( t, maxT );
						}
						else
						{
							color = makeVector3f(1e10f,0,0);
							patchNormalDist.push_back( 100*radius );
						}
					}


					patchColors.push_back( color );*/
					patchColors.push_back( makeVector3f(1,1,1) * 0.7f);
					patchNormalDist.push_back(0);

				}
			
			}
			

			debugOutput << "\n--------------\nMinT: " << minT << "\nMaxT: " << maxT << "\n";
			Matrix3f paramToWorld3f = shrink4To3( paramToWorld);
			for( card32 i=0;i<patchVertices.size();i++ )
			{
				patchVertices[i] = shrink4To3( paramToWorld * expand3To4(patchVertices[i]*radius));			
				patchNormals[i] = paramToWorld3f * patchNormals[i];
				if( patchColors[i][0] < 1e9f)
				{
					patchColors[i] = (patchColors[i]-makeVector3f(1,1,1) * minT) * (maxT - minT);
				}
			}
			
			if( detH <= 0 )
			{
				debugOutput << "no minima/maxima found\n";
				break;
			}

			if( norm(extremePoint - lastpoint) > radius * 0.5f )
			{
				debugOutput << "|extremePoint - lastpoint| / radius = " << norm(extremePoint - lastpoint) / radius
					<< "\n";
				break;
			}

			}



		/*	for( card32 i=0;i<m_Descriptors.size();i++ )
			{
				delete m_Descriptors[i];
			}
			m_Descriptors.clear();


			float featureSize = getAveragePointDistance(pc) * 10.0f;

			FDSIFT * sift = new FDSIFT;
			TDescriptorContext dc;
			dc.interpolation = NULL;
			dc.parameterization = NULL;
			dc.pc = pc;
			dc.tIt = NULL;
			if( !pc->providesAttribute( "gradient" ) )
				sift->preparePointCloud(pc);
			sift->createDescriptor( point, featureSize, pc, &dc );
			sift->debugReport();

			m_Descriptors.push_back(sift);*/
		}
	}

}

void PCIFeatureTest::mouseMoved(int32 x, int32 y) 
{
}

void PCIFeatureTest::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
}

void PCIFeatureTest::areaResize(card32 width, card32 height) 
{
	m_ScreenWidth = width;
	m_ScreenHeight = height;
}

void PCIFeatureTest::keyDown(GeneralKey key) 
{
}

void PCIFeatureTest::keyUp(GeneralKey key) 
{
}

void PCIFeatureTest::connectToSceneImpl(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget, QWidget* toolWidget) 
{
	m_SceneBB = getScenePCBBox(getScene());
	m_RenderTool = new RenderingTool;
	m_RenderTool->init();
}

static MultiResolutionPC * mres = NULL;

void drawPairs( const vector<FeaturePair> pairs, Vector3f color, const Matrix4f &ctm1, const Matrix4f &ctm2 )
{
	glBegin( GL_LINES );
	glColor3fv(&color[0]);
	for( card32 i=0;i<pairs.size();i++ )
	{
		Vector3f pos1 = pairs[i].m_Feature1->getPosition();
		Vector3f pos2 = pairs[i].m_Feature2->getPosition();
		pos1 = shrink4To3(ctm1 * expand3To4( pos1) );
		pos2 = shrink4To3(ctm2 * expand3To4( pos2) );

		glVertex3fv( &pos1[0] );
		glVertex3fv( &pos2[0] );
	}
	glEnd();
}

void PCIFeatureTest::glDrawTool(GLContext *glContext) 
{
	glPushAttrib( GL_ALL_ATTRIB_BITS );
	glEnable( GL_DEPTH_TEST );

	if( patchVertices.size() != 0)
	{
		glColor3f(1,0,0);
		glBegin( GL_LINES );
		for( card32 i=0;i<patchVertices.size();i++)
		{
			glVertex3fv( patchVertices[i].data() );
			Vector3f tmp = patchVertices[i] + patchNormals[i] * patchNormalDist[i];
			glVertex3fv( tmp.data() );
		}
		glEnd();


		card32 patchRes = sqrt( (float)patchVertices.size() ); 
		glEnable( GL_LIGHTING );
		// Light 0
		float ambient0[] = { 0.0f, 0.0f, 0.0f, 1.0f };
		float diffuse0[] = { 0.6f, 0.6f, 0.8f, 1.0f };
		float specular0[] = { 0.7f, 0.7f, 0.7f, 1.0f };
		float light_pos0[] = { 0.0f, 1.0f, 1.0f, 0.0f };
		glLightfv( GL_LIGHT0, GL_AMBIENT, ambient0 );
		glLightfv( GL_LIGHT0, GL_DIFFUSE, diffuse0 );
		glLightfv( GL_LIGHT0, GL_SPECULAR, specular0 );
		glLightfv( GL_LIGHT0, GL_POSITION, light_pos0 );
		// Light 1
		float ambient1[] = { 0.0f, 0.0f, 0.0f, 1.0f };
		float diffuse1[] = { 0.8f, 0.8f, 0.6f, 1.0f };
		float specular1[] = { 0.7f, 0.7f, 0.7f, 1.0f };
		float light_pos1[] = { 0.0f, -1.0f, -1.0f, 0.0f };
		glLightfv( GL_LIGHT1, GL_AMBIENT, ambient1 );
		glLightfv( GL_LIGHT1, GL_DIFFUSE, diffuse1 );
		glLightfv( GL_LIGHT1, GL_SPECULAR, specular1 );
		glLightfv( GL_LIGHT1, GL_POSITION, light_pos1 );

		glEnable( GL_LIGHT0 );
		glEnable( GL_LIGHT1 );		//glPointSize( 5.0f);
		//glEnable( GL_COLOR_MATERIAL );
		glBegin( GL_TRIANGLES );
		glColor3f(1,0,0);
		for( card32 x=0;x<patchRes-1;x++ )
		{
			for( card32 y=0;y<patchRes-1;y++ )
			{
				card32 i1 = (x) * patchRes + (y);
				card32 i2 = (x+1) * patchRes + (y);
				card32 i3 = (x) * patchRes + (y+1);
				card32 i4 = (x+1) * patchRes + (y+1);
				glColor3fv( patchColors[i1].data() );
				glNormal3fv( patchNormals[i1].data() );
				glVertex3fv( patchVertices[i1].data() );
				glColor3fv( patchColors[i2].data() );
				glNormal3fv( patchNormals[i2].data() );
				glVertex3fv( patchVertices[i2].data() );
				glColor3fv( patchColors[i3].data() );
				glNormal3fv( patchNormals[i3].data() );
				glVertex3fv( patchVertices[i3].data() );

				glColor3fv( patchColors[i2].data() );
				glNormal3fv( patchNormals[i2].data() );
				glVertex3fv( patchVertices[i2].data() );
				glColor3fv( patchColors[i4].data() );
				glNormal3fv( patchNormals[i4].data() );
				glVertex3fv( patchVertices[i4].data() );
				glColor3fv( patchColors[i3].data() );
				glNormal3fv( patchNormals[i3].data() );
				glVertex3fv( patchVertices[i3].data() );

			}
		}
		glEnd();
		glDisable( GL_LIGHTING );
		glDisable( GL_DEPTH_TEST );

		glPointSize(5.0f);
		glColor3f(0,0,0);
		glBegin( GL_POINTS );
		glVertex3fv( extremePoint.data() );
		glEnd();
		glDepthFunc( GL_LEQUAL );
		glPointSize(3.0f);
		glColor3f(1,1,1);
		glBegin( GL_POINTS );
		glVertex3fv( extremePoint.data() );
		glEnd();
		glDepthFunc( GL_LESS );
	}

	for( card32 i=0;i<m_Features->size();i++ )
	if( !m_DrawFeatureScale )
	{
		card32 color = m_Features->get(i)->getColor();
		//rt->setRenderColor( 1,0,0 );
		m_RenderTool->setRenderColor( color & 0xff, (color >> 8) & 0xff, (color >> 16) & 0xff );
		Vector3f pos = m_Features->get(i)->getPosition();
		float size = m_RelativeFeatureSize;
		m_RenderTool->renderSphere( pos, size );
	}
	else
	{

		for( card32 i=0;i<m_Features->size();i++ )
		{
			card32 color = m_Features->get(i)->getColor();
			m_RenderTool->setRenderColor( (color >> 16) & 0xff,
				(color >> 8) & 0xff,color & 0xff );
			Vector3f pos = m_Features->get(i)->getPosition();
			m_RenderTool->renderSphere( pos, m_Features->get(i)->getSize() );
		}
		
	}

	glPopAttrib();

	glPushAttrib( GL_ALL_ATTRIB_BITS );
	glEnable( GL_DEPTH_TEST );
	glDepthFunc( GL_LEQUAL );

	
	Matrix4f ctm1, ctm2;
	ctm1 = ctm2 = IDENTITY4F;
	if( m_PointCloudA )
		ctm1 = m_PointCloudA->getTransformation();

	if( m_PointCloudB )
		ctm2 = m_PointCloudB->getTransformation();

	if( !m_HideInitialPairs )
	{
		drawPairs( m_InitialPairs, makeVector3f(1,1,1) * 0.8f, ctm1, ctm2 );
	}

	// TODO: limit pairs
	drawPairs( m_Pairs, makeVector3f(1,0,0) , ctm1, ctm2 );


	for( card32 i=0;i<m_Descriptors.size();i++ )
	{
		FDSIFT * sift = dynamic_cast<FDSIFT*>(m_Descriptors[i]);
		if( sift )
			sift->drawOrientations( m_GradientLength );

	}

	if( m_DrawGradients )
	{
		// get point cloud from scene
		PointCloud * pc = NULL;
		StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
		it->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
		while ( !it->atEnd() )
		{
			SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( it->getNode() );
			if( sgon )
			{
				pc = dynamic_cast<PointCloud*>( sgon->getSceneObject() );
				if( pc )
					break;
			}
			it->next();
		}

		if( pc && pc->providesAttribute("gradient"))
		{
			AAT gradAAT = pc->getAAT("gradient");
			AAT posAAT = pc->getAAT("position");
			SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC );
			BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);

			

			bIt->reset();
			glBegin( GL_LINES );
			glColor3f(0,1,0);
			while( !bIt->atEnd() )
			{
				Vector3f pos = bIt->get3f( posAAT );
				Vector3f grad = bIt->get3f( gradAAT );
				float l=norm( grad );
				//l = atan2(l,1.0f);
				float gradLength = l * m_GradientLength;
				Vector3f pos2 = pos + normalize(grad) * gradLength;

				glColor3f(1,0,0);
				glVertex3fv( &pos[0] );
				glColor3f(1,1,0);
				glVertex3fv( &pos2[0] );

				bIt->next();
			}
			glEnd();

			delete iter;

		}

	}

	if( mres != NULL )
	{
		BoundingBox3f bb = getScenePCBBox( getScene() );
		float dist = bb.getMaxSideLength() * 1.05f;
		for( card32 i=0;i<mres->getNumberOfPointClouds();i++ )
		{
			PointCloud * pc = mres->getPointCloud(i);
			SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC );

			BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);

			AAT posAAT = pc->getAAT( "position" );
			AAT normalAAT = pc->getAAT( "normal" );
			AAT colorAAT = pc->getAAT( "color" );

			Vector3f light = normalize(makeVector3f(1,1,1));

			//glPointSize( 1 << i );
			//glBegin( GL_POINTS );
			glBegin( GL_LINES );
			while( !bIt->atEnd() )
			{
				Vector3f normal = bIt->get3f(normalAAT);
				Vector3f color = bIt->get3f(colorAAT)
					* max(0, normal * light * 0.5f + 0.5f);
				//glColor3fv( &color[0] );
				glColor3f(0,1,0);
				Vector3f pos = bIt->get3f(posAAT);
				pos[0] += ((float)i+1.0f)*dist;
				glVertex3fv( &pos[0] );
				pos += normal * dist * 0.1f;
				glVertex3fv( &pos[0] );
				bIt->next();
			}
			glEnd();
		}
	}

	if( m_Paths.size() )
	{
		glLineWidth(3.0f);
		for( card32 i=0;i<m_Paths.size();i++ )
		{
			glBegin( GL_LINES );
			for( card32 j=0;j<m_Paths[i].size()-1;j++ )
			{
				glColor3f( 1.0f, (float)j / (float)m_Paths[i].size(), 0.0f);
				glVertex3fv( &m_Paths[i][j][0] );
				glVertex3fv( &m_Paths[i][j+1][0] );
			}
			glEnd();
		}
	}

	glPopAttrib();
}

void PCIFeatureTest::distortPointClouds() 
{
	// add ref_position and copy from original position (if ref_position doesn't already exists)
	preparePointClouds();

	CmdObjPointCloudDistortion * distort = new CmdObjPointCloudDistortion;
	distort->setDestChannel( "position" );
	distort->setSourceChannel( "position" );

	distort->setDistortionMode( m_DistortionMode );
	distort->setStrength( m_DistortionStrength );
	distort->setRandomSeed( m_DistortionRandomSeed );
	distort->setNumGravityBalls( m_DistortionNumGravityBalls );

	BoundingBox3f bb = m_SceneBB;
	distort->setPointCloudBB( bb );
	distort->setRotationAxis( normalize(bb.upperCorner - bb.lowerCorner));
	distort->setRotationCenter( bb.getCenter() );

	getScene()->getCommandHistory()->executeCommand(getScene(), distort, false);
}

void PCIFeatureTest::preparePointClouds() 
{
   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() == 0 )
		return;

	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		PointCloud * pc = getPointCloud( getScene(), selection[m] );
		if( !pc->providesAttribute( "ref_position" ) )
		{
			VertexDescriptor * newAttr = new VertexDescriptor;
			newAttr->pushAttrib( VAD( "ref_position", 3, VAD::DATA_FORMAT_FLOAT32));
			PointCloud * tmp_pc = pc->appendFields( newAttr );
			replacePointCloud( getScene(), selection[m], tmp_pc );
			pc = tmp_pc;


			SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC );
			pAssert( iter != NULL );

			BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
			ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);

			AAT posAAT = pc->getAAT( "position" );
			AAT refposAAT = pc->getAAT( "ref_position" );

			while( !bIt->atEnd() )
			{
				mIt->set3f( refposAAT, bIt->get3f(posAAT));
				bIt->next();
			}


		}
	}
}


void PCIFeatureTest::testDetector(int numberOfTests) 
{

	BoundingBox3f bb = m_SceneBB;
	if( m_FeatureDetector == NULL )
		return;

   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() != 1 )
		return;

	// get point cloud from scene
	PointCloud * pc = NULL;
	string pcName;
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
//		checkAttribute( getScene(),selection[m], "gradient",3, VAD::DATA_FORMAT_FLOAT32 );
//		checkAttribute( getScene(),selection[m], "curvature",2, VAD::DATA_FORMAT_FLOAT32 );
		pc = getPointCloud( getScene(), selection[m] );
		pcName = selection[m];
		if( pc )
			break;
	}

	if( !pc )
		return;


//	calculateCurvature(pc, 30, "curvature" );
//	calculateGradients( pc, 200, "gradient");
//	return;

	// create ref_position channel (if not already created)
	/*if( !pc->providesAttribute( VERTEX_CHANNEL_NAME_REFERENCE_POSITION ) )
	{
		// append channel	
		VertexDescriptor * newAttr = new VertexDescriptor;
		newAttr->pushAttrib( VAD( VERTEX_CHANNEL_NAME_REFERENCE_POSITION, 3, VAD::DATA_FORMAT_FLOAT32));
		PointCloud * tmp_pc = pc->appendFields( newAttr );
		replacePointCloud( getScene(), pcName, tmp_pc );
		pc = tmp_pc;

		// copy position to ref_position
		SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC );
		pAssert( iter != NULL );
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
		ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);
		AAT posAAT = pc->getAAT( "position" );
		AAT refposAAT = pc->getAAT( "ref_position" );
		while( !bIt->atEnd() )
		{
			mIt->set3f( refposAAT, bIt->get3f(posAAT));
			bIt->next();
		}
	}*/


	for( int iTest = 0;iTest<numberOfTests;iTest++ )
	{
		// distort 
		/*CmdObjPointCloudDistortion * distort = new CmdObjPointCloudDistortion;
		distort->setDestChannel( "position" );
		distort->setSourceChannel( "ref_position" );

		distort->setDistortionMode( m_DistortionMode );
		if( numberOfTests > 1 )
			distort->setStrength( m_DistortionStrength * (float)iTest / (float)(numberOfTests-1) );
		else
			distort->setStrength( m_DistortionStrength );
		distort->setRandomSeed( m_DistortionRandomSeed);
		distort->setNumGravityBalls( m_DistortionNumGravityBalls );

		distort->setPointCloudBB( bb );
		distort->setRotationAxis( normalize(bb.upperCorner - bb.lowerCorner));
		distort->setRotationCenter( bb.getCenter() );

		getScene()->getCommandHistory()->executeCommand(getScene(), distort, false);

		pc = getPointCloud( getScene(), pcName );
		if( !pc )
		{
			debugOutput << "Couldn't find distorted point cloud!\n";
			return;
		}*/

		// generate multi-resolution and multi-scale represenation
/*		MultiResolutionPC mres;
		mres.setup( pc, 100, 2.0f, 1.0f );

		
		card32 numberOfScales = 5;
		MLSMultiScaleFeatureDetector * mlsDet = dynamic_cast<MLSMultiScaleFeatureDetector*>(m_FeatureDetector);
		if( mlsDet )
			numberOfScales = mlsDet->getNumberOfScales();
		MultiScaleGenerator gen;
		gen.setMinNumberOfSamples(30);
		gen.setNumberOfScales(numberOfScales);
		gen.addVertexAttributef( "color",3);
		if( mlsDet )
		{
			gen.setF( mlsDet->getF() );
			gen.setH0( mlsDet->getH0() );
		}
		PointCloud * mscale = gen.create( pc, &mres );
		pc->assign( mscale );
		pc->clearAttachments();

		calculateGradients( pc, 20, "gradient", "",0,0.0f,"pos1" );

		m_Features = m_FeatureDetector->detectFeatures( pc, 0 );*/
		float averagePointDistance = getAveragePointDistance( pc );
		FeatureRegistration * fr = setupRegistration( averagePointDistance );
		delete m_Features;
		m_Features = fr->calculateFeatures( pc, FeatureRegistration::DETECT_POS );
		delete fr;

		float maxF = 1e-20f;
		float minF = 1e20f;
		for( card32 i=0;i<m_Features->size();i++ )
		{
			maxF = max( maxF, m_Features->get(i)->getMagnitude());
			minF = min( minF, m_Features->get(i)->getMagnitude());
		}

		debugOutput << "MaxF: " << maxF << " MinF: " << minF << "\n";
		for( card32 i=0;i<m_Features->size();i++ )
			m_Features->get(i)->setMagnitude(m_Features->get(i)->getMagnitude() / maxF );

		debugOutput << "Number of detected feature: " << m_Features->size() << "\n";

		redrawScene();
		
	}
}

void PCIFeatureTest::testDescriptor( int numberOfTests, int numberOfSamples, float radius ) 
{

/*	BoundingBox3f bb = m_SceneBB;
	float featureSize = norm( bb.upperCorner - bb.lowerCorner) * radius;
	if( m_FeatureDetector == NULL )
		return;

	if( m_FeatureDescriptor == NULL )
		return;

   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() != 1 )
		return;

	// get point cloud from scene
	PointCloud * pc = NULL;
	string pcName;
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		pc = getPointCloud( getScene(), selection[m] );
		pcName = selection[m];
		if( pc )
			break;
	}

	if( !pc )
		return;

	checkAttribute( getScene(), pcName, "intAtt", 1, VAD::DATA_FORMAT_INT32 );
	pc = getPointCloud( getScene(), pcName );

	// create curvature channel (if not already created)
	if( !pc->providesAttribute( "test") )
	{
		// append channel	
		VertexDescriptor * newAttr = new VertexDescriptor;
		newAttr->pushAttrib( VAD( "test", 4, VAD::DATA_FORMAT_FLOAT32));
		PointCloud * tmp_pc = pc->appendFields( newAttr );
		replacePointCloud( getScene(), pcName, tmp_pc );
		pc = tmp_pc;
	}

	// create ref_position channel (if not already created)
	if( !pc->providesAttribute( VERTEX_CHANNEL_NAME_REFERENCE_POSITION ) )
	{
		// append channel	
		VertexDescriptor * newAttr = new VertexDescriptor;
		newAttr->pushAttrib( VAD( VERTEX_CHANNEL_NAME_REFERENCE_POSITION, 3, VAD::DATA_FORMAT_FLOAT32));
		PointCloud * tmp_pc = pc->appendFields( newAttr );
		replacePointCloud( getScene(), pcName, tmp_pc );
		pc = tmp_pc;

		// copy position to ref_position
		SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC );
		pAssert( iter != NULL );
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
		ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);
		AAT posAAT = pc->getAAT( "position" );
		AAT refposAAT = pc->getAAT( "ref_position" );
		while( !bIt->atEnd() )
		{
			mIt->set3f( refposAAT, bIt->get3f(posAAT));
			bIt->next();
		}
	}

	SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC );
	BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	// select sample points
	delete m_Features;
	vector<int> featureIndex;
	pAssert( iter != NULL );
	card32 blockSize = pc->getNumPoints() / numberOfSamples;
	for( card32 i=0;i<(card32)numberOfSamples;i++ )
	{
		card32 index = min(i * blockSize + rand() % blockSize,pc->getNumPoints()-1);
		Feature *f = new Feature;
		f->setColor( 0xff0000 );
		m_Features.push_back(f);
		featureIndex.push_back(index);
	}

	/*m_Features = m_FeatureDetector->detectFeatures( pc, featureSize );
	HierarchicalKNNIterator hIt(pc,32,NULL);
	vector<int> featureIndex;
	for( card32 i=0;i<m_Features.size();i++ )
	{
		hIt.setSeekPointAndReset( m_Features[i].getPosition() );
		featureIndex.push_back( hIt.getPointSetPointNumber() );
	}*/
	


	// estimate normals and curvature
/*	int numNeighbours = 50;
	PCNormalEstimator ne;
	ne.setComputeTangents(false);
	ne.estimateNormals(pc, numNeighbours );
	ne.unifyNormals(pc,numNeighbours);
	calculateCurvature( pc, numNeighbours, "test" );
	
	// calculate initial feature descriptors
	TDescriptorContext dc;
	dc.pc = pc;
	dc.parametricImageSize = 32;
	dc.interpolation = new SplatPullPushInterpolation;
	dc.parameterization = new HeighfieldParameterization;
	dc.tIt = NULL;

	m_FeatureDescriptor->preparePointCloud( pc );

	vector<FeatureDescriptor*> initialDescriptors;
	RandomAccessPointCloudIterator * rIt = dynamic_cast<RandomAccessPointCloudIterator*>(iter);
	AAT posAAT = pc->getAAT( "position" );
	for( card32 i=0;i<m_Features.size();i++ )
	{
		rIt->goToIndex( featureIndex[i] );
		FeatureDescriptor* fd = (FeatureDescriptor*)m_FeatureDescriptor->copy();
		fd->createDescriptor( bIt->get3f(posAAT), featureSize, pc, &dc );
		initialDescriptors.push_back( fd );
	}


	for( int iTest = 0;iTest<numberOfTests;iTest++ )
	{
		// distort 
		CmdObjPointCloudDistortion * distort = new CmdObjPointCloudDistortion;
		distort->setDestChannel( "position" );
		distort->setSourceChannel( "ref_position" );

		distort->setDistortionMode( m_DistortionMode );
		if( numberOfTests > 1 )
			distort->setStrength( m_DistortionStrength * (float)iTest / (float)(numberOfTests-1) );
		else
			distort->setStrength( m_DistortionStrength );
		distort->setRandomSeed( m_DistortionRandomSeed);
		distort->setNumGravityBalls( m_DistortionNumGravityBalls );

		distort->setPointCloudBB( bb );
		distort->setRotationAxis( normalize(bb.upperCorner - bb.lowerCorner));
		distort->setRotationCenter( bb.getCenter() );

		getScene()->getCommandHistory()->executeCommand(getScene(), distort, false);

		pc = getPointCloud( getScene(), pcName );
		if( !pc )
		{
			debugOutput << "Couldn't find distorted point cloud!\n";
			return;
		}

		// estimate normals and curvature
		int numNeighbours = 20;
		PCNormalEstimator ne;
		ne.setComputeTangents(false);
		ne.estimateNormals(pc, numNeighbours );
		ne.unifyNormals(pc,numNeighbours);
		calculateCurvature( pc, numNeighbours, "test" );

		m_FeatureDescriptor->preparePointCloud( pc );

		// calculate new descriptors
		SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC );
		pAssert( iter != NULL );
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
		RandomAccessPointCloudIterator * rIt = dynamic_cast<RandomAccessPointCloudIterator*>(iter);
		AAT posAAT = pc->getAAT( "position" );
		vector<FeatureDescriptor*> descriptors;
		for( card32 i=0;i<m_Descriptors.size();i++ )
			delete m_Descriptors[i];		
		m_Descriptors.clear();
		for( card32 i=0;i<m_Features.size();i++ )
		{
			rIt->goToIndex( featureIndex[i] );
			FeatureDescriptor* fd = (FeatureDescriptor*)m_FeatureDescriptor->copy();
			fd->createDescriptor( bIt->get3f(posAAT), featureSize, pc, &dc );
			m_Features[i]->setPosition(bIt->get3f(posAAT));
			descriptors.push_back( fd );
			m_Descriptors.push_back( fd );
		}

		// calculate average distance from initial frame to new frame
		float distance = 0.0f;
		for( card32 i=0;i<descriptors.size();i++ )
		{
			distance += descriptors[i]->compare( initialDescriptors[i] );
		}
		distance /= (float)descriptors.size();

		debugOutput << "average distance: " << distance << "\n";

		// colorize features to visualize big differences
		for( card32 i=0;i<descriptors.size();i++ )
		{
			float fDist = descriptors[i]->compare( initialDescriptors[i] );
			card32 color = 0;
			if( fDist < distance )
				color = 0x00ff00;
			else
				if( fDist < distance * 2.0f )
					color = 0x008800;
				else
					if( fDist < distance * 4.0f )
						color = 0xff8800;
					else
						color = 0xff0000;
			m_Features[i]->setColor( color );

		}

		redrawScene();
		//for( card32 i=0;i<descriptors.size();i++ )
		//	delete descriptors[i];
		
	}

	for( card32 i=0;i<initialDescriptors.size();i++ )
		delete initialDescriptors[i];
*/
}

void PCIFeatureTest::testRegistration(int maxNumberOfPairs, float featureRadius, float pruningThreshold) 
{
	calculateFeatures( featureRadius );
	buildFeaturePairs( maxNumberOfPairs, pruningThreshold );
		

	// setup spectral c.v.
	/*SpectralCorrespondenceValidation spectralCV;
	card32 numberOfPairs = min((card32)pairs.size(),(card32)maxNumberOfPairs);
	spectralCV.setup(numberOfPairs);
	vector<TFeaturePair> initialPairs;
	for( card32 i=0;i<numberOfPairs; )
	{
		if( pairs.empty() )
			break;
		TFeaturePair pair = pairs.top();
		pairs.pop();
		//if( numPairs1[ pair.i1 ] > 10 || numPairs2[ pair.i2 ] > 10 )
		//	continue;
		initialPairs.push_back( pair );
		spectralCV.setCorrespondencePair(i, pair.i1, pair.i2 );
		spectralCV.setMatrixEntry(i,i, 0 );
		m_Pairs.push_back( pair );
		debugOutput << "initial pair: (" << pair.i1 << "," << pair.i2 << "," << pair.score << ")\n";
		i++;
	}
	m_InitialPairs = initialPairs;

	float maxDifference = norm(m_SceneBB.upperCorner - m_SceneBB.lowerCorner);
	float scaleDifference = 1.0f / maxDifference;

	for( card32 i=0;i<initialPairs.size();i++ )
	{
		for( card32 j=i+1;j<initialPairs.size();j++ )
		{
			float difference1 = norm(initialPairs[i].pos1 - initialPairs[j].pos1);
			float difference2 = norm(initialPairs[i].pos2 - initialPairs[j].pos2);
			//float conf = exp( -abs(difference1 - difference2)*featureSize);
			float conf = max(0,1.0f - pow(abs(difference1 - difference2) * scaleDifference,2.0f));
			spectralCV.setMatrixEntry(i,j, conf );
		}
	}

	spectralCV.calculate();

	vector<SpectralCorrespondenceValidation::TFeaturePair> tmpPairs;
	tmpPairs = spectralCV.getValidPairs();
	m_Pairs.clear();
	for( card32 i=0;i<tmpPairs.size();i++ )
	{
		TFeaturePair p;
		p.i1 = tmpPairs[i].i1;
		p.i2 = tmpPairs[i].i2;
		p.pos1 = m_Features[ p.i1 ]->getPosition();
		p.pos2 = features[ p.i2 ]->getPosition();
		p.score = 0;
		m_Pairs.push_back( p );
	}


	redrawScene();
	for( card32 i=0;i<descriptors.size();i++ )
		delete descriptors[i];
	
	for( card32 i=0;i<initialDescriptors.size();i++ )
		delete initialDescriptors[i];*/

}

bool PCIFeatureTest::getPointFromCursor( int x, int y, Vector3f &retPoint, string &retPC )
{
   PointCloudIterator* it = getClosestPointIteratorFromAllPoints(x, y, m_ScreenWidth, m_ScreenHeight, 5, 
			&getScene()->getRootState()->staticState->camera, 
			&getScene()->getRootState()->staticState->viewFrustum,
			getScene(),
			retPC
	);

	if( it == NULL )
		return false;

	BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(it);
	PointCloud * pc = getPointCloud( getScene(), retPC );

	pAssert( bIt != NULL );
	pAssert( pc != NULL );

	retPoint = bIt->get3f( pc->getAAT( "position") );
	delete it;
	return true;
}

static float oldAveragePointDist = 0;
struct TTmpFeatureAsdf
{
	int numPoints;
	Feature * f;
};

bool compareFeatureSupport( const TTmpFeatureAsdf &a, const TTmpFeatureAsdf &b )
{
	return a.numPoints < b.numPoints;
}

void PCIFeatureTest::calculateFeatures( float featureRadius ) 
{
	BoundingBox3f bb = m_SceneBB;
//	float featureSize = norm( bb.upperCorner - bb.lowerCorner) * featureRadius;
	if( m_FeatureDetector == NULL )
		return;

	if( m_FeatureDescriptor == NULL )
		return;

   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() < 1 )
		return;

	// get pointclouds from scene
	PointCloud * pcA = NULL;
	PointCloud * pcB = NULL;
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		if( pcA != NULL )
		{
			pcB = getPointCloud( getScene(), selection[m] );
			m_PointCloudB_Name = selection[m];
		}
		else
		{
			pcA = getPointCloud( getScene(), selection[m] );
			m_PointCloudA_Name = selection[m];
		}
		if( pcB )
			break;
	}

	if( !pcA )
		return;

	m_PointCloudA = pcA;
	m_PointCloudB = pcB;

	float averagePointDistance = getAveragePointDistance( pcA );
	oldAveragePointDist = averagePointDistance;
	FeatureRegistration * fr = setupRegistration( averagePointDistance );

	calculateCurvatureForAlleClouds( 3.0f, 2.0f, true );

	/*checkAttribute(pcA, "curvature", 4, VAD::DATA_FORMAT_FLOAT32 );
	calculateCurvature(pcA, 30, "curvature", averagePointDistance );
	if( pcB )
	{
		checkAttribute(pcB, "curvature", 4, VAD::DATA_FORMAT_FLOAT32 );
		calculateCurvature(pcB, 30, "curvature", averagePointDistance );
	}*/
	
	vector<PointCloud*> clouds;
	clouds.push_back( pcA );
	if( pcB )
		clouds.push_back( pcB );

	delete m_FeaturesA;
	delete m_FeaturesB;

	vector<FeatureList*> features;
	
	if( dynamic_cast<MLSMultiScaleFeatureDetector*>( m_FeatureDetector ) )
	{
		features.push_back( fr->calculateFeatures( pcA, FeatureRegistration::DETECT_POS ) );
		if( pcB ) 
			features.push_back(fr->calculateFeatures( pcB, FeatureRegistration::DETECT_POS ));
	}
	else
	{
		features = m_FeatureDetector->detectFeatures( clouds, m_DescClusterRadius*averagePointDistance);
	}


	// remove border features
	for( card32 i=0;i<features.size();i++ )
	{
		FeatureList * fl = removeBorderFeatures( features[i], featureRadius * averagePointDistance, clouds[i] ); 
		delete features[i];
		features[i] = fl;
		/*
		vector< TTmpFeatureAsdf > tmpfeatures;
		HierarchicalKNNIterator hIt( clouds[i] ,32 , NULL );
		AAT posAAT = clouds[i]->getAAT( "position" );
		for( card32 j=0;j<features[i]->size();j++ )
		{
			TTmpFeatureAsdf f;
			f.f = features[i]->get(j);
			hIt.setSeekPointAndReset( f.f->getPosition() );
			f.numPoints = 0;
			while( !hIt.atEnd() )
			{
				if( norm( hIt.get3f(posAAT) -  f.f->getPosition() ) > featureRadius * averagePointDistance )
					break;
				f.numPoints++;
				hIt.next();
			}

			tmpfeatures.push_back(f);
		}


		sort( tmpfeatures.begin(), tmpfeatures.end(), compareFeatureSupport );
		int minNumberOfPoints = tmpfeatures[ tmpfeatures.size() / 2 ].numPoints * 0.9f;
		FeatureList * fl = new FeatureList;
		for( card32 j=0;j<tmpfeatures.size();j++ )
			if( tmpfeatures[j].numPoints >= minNumberOfPoints )
				fl->push_back( (Feature*)tmpfeatures[j].f->copy() );

		delete features[i];
		features[i] = fl;*/
	}

	m_FeaturesA = features[0];
	if( pcB )
		m_FeaturesB = features[1];

	debugOutput << "#features A: " << m_FeaturesA->size() << "\n";
	if( pcB )
		debugOutput << "#features B: " << m_FeaturesB->size() << "\n";

	delete m_Features;
	m_Features = new FeatureList;

	for( card32 i=0;i<m_FeaturesA->size();i++ )
	{
		m_Features->push_back((Feature*)m_FeaturesA->get(i)->copy());
	}

	if( pcB )
	for( card32 i=0;i<m_FeaturesB->size();i++ )
	{
		m_Features->push_back((Feature*)m_FeaturesB->get(i)->copy());
	}
}

void PCIFeatureTest::buildFeaturePairs( int maxNumberOfPairs, float pruningThreshold ) 
{
	if( m_FeaturesA == NULL )
		return;

	FeatureRegistration fr;
	m_InitialPairs = fr.buildFeaturePairs( m_FeaturesA, m_FeaturesB, pruningThreshold, 3000 );

	debugOutput << "Number of pairs: " << (mpcard)m_InitialPairs.size() << "\n";

	if( m_InitialPairs.size() == 0 )
		return;

	// build difference histogram
	float maxDist = m_InitialPairs.back().m_DescriptorDistance;
	float maxMag = 0.0f;
	int numBins = 100;
	
	vector<float> histogram(numBins,0);
	for( card32 i=0;i<m_InitialPairs.size();i++ )
	{
		int binID = min( m_InitialPairs[i].m_DescriptorDistance / maxDist , 1.0f ) * (float)(numBins-1.0f);
		histogram[binID] += 1.0f;
		maxMag = max(maxMag,histogram[binID]);
	}

	static RGBAFloat32Image * image = NULL;
	float imageHeight = 40.0f;
	if( image == NULL )
		image = new RGBAFloat32Image( (card32)histogram.size(), imageHeight);

	image->clear( makeVector4f(0,0,0,0));
	for( card32 i=0;i<image->getSizeX();i++ )
	{
		for( card32 j=0;j<histogram[i] / maxMag * imageHeight;j++ )
			image->setPixel(i,imageHeight-1-j, makeVector4f(1,1,1,1));
	}

	ImageDisplayer::DisplayRGBAFloat32Image ev(image);
	debugOutput.displayEvent( ImageDisplayer::getClass(), "muh", &ev );


}

void PCIFeatureTest::validateFeaturePairs() 
{
	FeatureRegistration fr;
	fr.setSceneBB( m_SceneBB );
	fr.setSpectralSigma( m_SpectralSigma );
	fr.setSpectralAddDiff( m_SpectralAddDiff );
	fr.setSpectralScaleDiff( m_SpectralScaleDiff );

	m_Pairs = fr.validateFeaturePairs( m_InitialPairs, m_PointCloudA, m_PointCloudB );
	
	Matrix4f transformation = fr.calculateTransformation( m_Pairs );

	SGObjectNode * node = dynamic_cast<SGObjectNode*>( getSceneGraphNodeByName( getScene(), m_PointCloudB_Name ) );
	if( node )
	{
		node->getSceneObject()->setTransformation( transformation);
	}
	
	// estimate centroid of features:

}


FeatureRegistration *PCIFeatureTest::setupRegistration(float averagePointDistance) 
{
	FeatureRegistration * fr = new FeatureRegistration;

	fr->setFeatureRadius( 20.0f );
	card32 numberOfScales = 4;
	float F = 2.25f;
	float H0 = 1.5f;
	MLSMultiScaleFeatureDetector * mlsDet = dynamic_cast<MLSMultiScaleFeatureDetector*>(m_FeatureDetector);
	if( mlsDet )
	{
		numberOfScales = mlsDet->getNumberOfScales();
		F = mlsDet->getF();
		H0 = mlsDet->getH0();
	}

	FDSIFT * sift = dynamic_cast<FDSIFT*>( m_FeatureDescriptor );
	if( sift )
	{
		fr->setupSIFT( sift->getGradientAttribute(), numberOfScales, H0, F, averagePointDistance, sift->getNumSampleRegions(),
			sift->getNumOrientationBins() );
	}
	FDHistogram * histo = dynamic_cast<FDHistogram*>(m_FeatureDescriptor);
	if( histo )
		fr->setupHistogram( "curvature", numberOfScales, H0, F, averagePointDistance, histo->getNumHistograms(), 
		histo->getNumBins()); 
//		fr->setupHistogram( "gray", numberOfScales, H0, F, averagePointDistance, histo->getNumHistograms(), 
//		histo->getNumBins()); 
	
	return fr;
}

void PCIFeatureTest::testGeodeticDistances() 
{
   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() != 1 )
		return;

	// get point cloud from scene
	PointCloud * pc = NULL;
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		pc = getPointCloud( getScene(), selection[m] );
		if( pc )
			break;
	}

	if( !pc )
		return;

	UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
	FixedKNNTopologyIterator topologyIter( upc->getPointSet(), pc->getAttachments(), pc, 10 );

	SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | 
		SceneObjectIteratorType::CAP_RANDOM_ACCESS_PC );
	pAssert( iter != NULL );
	BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	RandomAccessPointCloudIterator * rIt = dynamic_cast<RandomAccessPointCloudIterator*>(iter);

	AAT posAAT = pc->getAAT( "position" );
	rIt->goToIndex( rand() % pc->getNumPoints() );
	Vector3f startPoint = bIt->get3f(posAAT);

	vector<Vector3f> points;
	for( card32 i=0;i<10;i++ )
	{
		rIt->goToIndex( rand() % pc->getNumPoints() );
		points.push_back(bIt->get3f(posAAT));
	}

/*	progressWindow->pushStep( true, "test" );
	for( card32 i=0;i<points.size();i++ )
	{
		calculateGeodeticDistances( pc, &topologyIter, startPoint, points );
		progressWindow->progress( (float)i / (float)points.size() * 100.0f );
	}
	bool aborted;
	progressWindow->popSteps(aborted);*/

	calculateGeodeticDistances( pc, &topologyIter, startPoint, points, &m_Paths );

}

void PCIFeatureTest::recalculateFeatureDescriptors( float featureRadius ) 
{
	TDescriptorContext dc;
	dc.interpolation = NULL;
	dc.parameterization = NULL;
	dc.parametricImageSize = 64;
//	dc.pc = mscale;
	dc.pc = m_PointCloudA;
	dc.tIt = NULL;
	dc.averagePointDistance = oldAveragePointDist;

	for( card32 i=0;i<m_FeaturesA->size();i++ )
	{
		FeatureDescriptor * fd = (FeatureDescriptor*)
			((FeatureDescriptor*)m_FeaturesA->get(i)->getDescriptor(0))->copy();

		fd->createDescriptor( m_FeaturesA->get(i)->getPosition(), 
			featureRadius * oldAveragePointDist, dc.pc, &dc );

		m_FeaturesA->get(i)->replaceDescriptor(0,fd);
		
	}

	dc.pc = m_PointCloudB;

	for( card32 i=0;i<m_FeaturesB->size();i++ )
	{
		FeatureDescriptor * fd = (FeatureDescriptor*)
			((FeatureDescriptor*)m_FeaturesB->get(i)->getDescriptor(0))->copy();

		fd->createDescriptor( m_FeaturesB->get(i)->getPosition(), 
			featureRadius * oldAveragePointDist, dc.pc, &dc );

		m_FeaturesB->get(i)->replaceDescriptor(0,fd);
	}
}

void PCIFeatureTest::createMultiScalePointClouds() 
{
	BoundingBox3f bb = m_SceneBB;
//	float featureSize = norm( bb.upperCorner - bb.lowerCorner) * featureRadius;
	if( m_FeatureDetector == NULL )
		return;

	if( m_FeatureDescriptor == NULL )
		return;

   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() < 1 )
		return;

	// get pointclouds from scene
	PointCloud * pc = NULL;
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		pc = getPointCloud( getScene(), selection[m] );
		if( pc )
			break;
	}

	if( !pc)
		return;

	float averagePointDistance = getAveragePointDistance( pc );

	MultiResolutionPC mres;
	mres.setup( pc, 100, 2.0f, 1.0f );
	
	MultiScaleGenerator gen;
	gen.setMinNumberOfSamples(50);
	gen.setupPosition();

	card32 numberOfScales = 5;
	MLSMultiScaleFeatureDetector * mlsDet = dynamic_cast<MLSMultiScaleFeatureDetector*>(m_FeatureDetector);
	if( mlsDet )
	{
		numberOfScales = mlsDet->getNumberOfScales();
		gen.setF( mlsDet->getF() );
		gen.setH0( mlsDet->getH0() );
	}

	PointCloud * mscalepc = gen.create( pc, &mres, averagePointDistance );

	for( card32 i=0;i<numberOfScales;i++ )
	{
		UnstructuredInCorePointCloud * newPC = new UnstructuredInCorePointCloud;
		PointSet * newPS = new PointSet;
		VertexDescriptor * vd = new VertexDescriptor;
		vd->pushAttrib( VAD( "position",3, VAD::DATA_FORMAT_FLOAT32 ) );
		vd->pushAttrib( VAD( "normal",3, VAD::DATA_FORMAT_FLOAT32 ) );
		vd->pushAttrib( VAD( "flags",1, VAD::DATA_FORMAT_INT32 ) );
		newPS->clearAndSetup(1, mscalepc->getNumPoints(), vd );

		AAT sourcePos = mscalepc->getAAT( string("pos") + intToStr(i));
		AAT sourceNormal = mscalepc->getAAT( string("normal") + intToStr(i));
		PointSet * srcPS = ((UnstructuredInCorePointCloud*)mscalepc)->getPointSet();
		AAT destPos = newPS->getAAT( "position" );
		AAT destNormal = newPS->getAAT( "normal" );

		for( card32 p=0;p<mscalepc->getNumPoints();p++ )
		{
			Vector3f pos = srcPS->get3f(p, sourcePos );
			newPS->set3f( p, destPos, pos );
			Vector3f normal = srcPS->get3f(p, sourceNormal );
			newPS->set3f( p, destNormal, normal );
		}
		newPC->getPointSet()->assign( newPS );
		delete newPS;

		addPointCloud( getScene(), newPC, string("test") + intToStr(i));
	}


}

bool compareCurv( const Vector4f &v1, const Vector4f &v2)
{
	return v1[0] < v2[0];
}

void PCIFeatureTest::calculateCurvatureForAlleClouds( float sigma, float normalSigma, bool median ) 
{
   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() == 0 )
		return;

	vector<PointCloud*> clouds;
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		PointCloud * pc = getPointCloud( getScene(), selection[m] );
		if( pc )
			clouds.push_back(pc);
	}
	calculateCurvaturePC(clouds, sigma, normalSigma, median);
}

void PCIFeatureTest::calculateCurvaturePC( vector<PointCloud *> clouds, float sigma,float normalSigma, bool median) 
{
	//vector<Vector4f> curvatures;
	vector<float> meanCurv, gaussianCurv;
	card32 numNANs = 0;
	for( card32 i=0;i<clouds.size();i++ )
	{
		PointCloud * pc = clouds[i];
		checkAttribute( pc, "curvature", 4 , VAD::DATA_FORMAT_FLOAT32 );
		AAT posAAT = pc->getAAT( "position" );
		AAT normalAAT = pc->getAAT( "normal" );
		AAT destAAT = pc->getAAT( "curvature" );

		float averagePointDistance = geoxl::getAveragePointDistance( pc, 4 );
		float sigma_2 = pow( sigma*averagePointDistance, 2.0f );
		float inv_sigma_2 = 1.0f / sigma_2;
		float maxRad = sqrt( -log(0.05f) * sigma_2 );

		float normal_sigma_2 = pow( normalSigma*averagePointDistance, 2.0f );
		float normal_inv_sigma_2 = 1.0f / normal_sigma_2;
		float normal_maxRad = sqrt( -log(0.05f) * normal_sigma_2 );

		HierarchicalKNNIterator hIt( pc, 32, NULL);
		SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC );
		pAssert( iter != NULL );
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
		ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);

		
		card32 actPoint = 0;
		card32 currentIndex = 0;
		progressWindow->pushStep(true,"curv");
		while( !bIt->atEnd() )
		{
			vector<Vector3f> posBuffer,posBufferNormal;

			Vector3f pos = bIt->get3f( posAAT );
			hIt.setSeekPointAndReset( pos );
			PCA<float,3> pca;
			PCA<float,3> normal_pca;

			posBuffer.push_back( makeVector3f(0,0,0) );
			posBufferNormal.push_back( makeVector3f(0,0,0) );

			int actPoint = 1;
			while(!hIt.atEnd() )
			{
				hIt.next();

				Vector3f tmpPos = hIt.get3f( posAAT );

				float dist = norm( tmpPos - pos);

				if( dist > maxRad && dist > normal_maxRad && actPoint >= 5)
					break;

				if( dist <= maxRad || actPoint < 5 )
				{
					posBuffer.push_back( tmpPos );
					pca.addPoint( tmpPos );
				}

				if( dist <= normal_maxRad || actPoint < 5)
				{
					posBufferNormal.push_back( tmpPos );
					normal_pca.addPoint( tmpPos );
				}

				actPoint++;
			}

			Vector3f eigenValues, centroid;
			Matrix3f eigenVectors;
			pca.analyze( eigenValues, eigenVectors, centroid );
			
			Vector3f normal = normalize( eigenVectors[0].crossProduct( eigenVectors[1] ) );
			Matrix3f tangentSystem = calcTangentSystem( normal );
			Matrix3f invTangentSystem = tangentSystem.transpose();

			Vector3f eigenValuesN, centroidN;
			Matrix3f eigenVectorsN;
			normal_pca.analyze( eigenValuesN, eigenVectorsN, centroidN );
			
			Vector3f normalN = normalize( eigenVectorsN[0].crossProduct( eigenVectorsN[1] ) );
			Matrix3f tangentSystemN = calcTangentSystem( normalN );
			Matrix3f invTangentSystemN = tangentSystem.transpose();
			


			// transform points into lokal coordinate frame
			for( card32 i=0;i<posBuffer.size();i++ )
			{
				posBuffer[i] = invTangentSystem * ( posBuffer[i] - pos) / averagePointDistance;
			}
			for( card32 i=0;i<posBufferNormal.size();i++ )
			{
				posBufferNormal[i] = invTangentSystemN * ( posBufferNormal[i] - pos) / averagePointDistance;
			}

			StaticVector<float32, 6> coeff;
			vector< StaticVector<float32, 6> > coeffsDerivs;
			GLTMonomials2ndOrder::computeCoeffs( posBuffer, coeff, coeffsDerivs, sigma );

			float meanCurvature =	pow( coeff[3] , 2.0f ) + 
											2.0f * pow( coeff[4] , 2.0f ) + 
											2.0f * pow( coeff[5] , 2.0f );
	
			float extrema = 0.0f;

			float f_x = coeff[1];
			float f_y = coeff[2];
			float f_xy = coeff[3];
			float f_xx = 2.0f * coeff[4];
			float f_yy = 2.0f * coeff[5];

			float K = (f_xx*f_yy - pow( f_xy, 2.0f ))
				/ pow(1.0f + pow(f_x,2.0f) + pow(f_y,2.0f),2.0f );

			float H = ((1.0f + pow(f_x,2.0f))*f_yy - 2.0f * f_x * f_y * f_xy + (1.0f + pow(f_y,2.0f))*f_xx)
				/ pow(1.0f + pow(f_x,2.0f) + pow(f_y,2.0f),2.0f/3.0f ) / 2.0f;


			Vector4f res_vec;
			float signH = H<0 ? -1:1;
			H = sqrt( fabs(H) ) * signH;
			float signK = K<0 ? -1 : 1;
			K = sqrt( fabs(K) ) * signK;
			res_vec[ 0 ] = H;
			res_vec[ 1 ] = K;
			mIt->set4f( destAAT, res_vec );
			//curvatures.push_back(res_vec);
			meanCurv.push_back( H );
			gaussianCurv.push_back( K );


			// calculate normal
			GLTMonomials2ndOrder::computeCoeffs( posBufferNormal, coeff, coeffsDerivs, normalSigma );
			float derivate_u = coeff[1];
			float derivate_v = coeff[2];

			Vector3f newNormal = normalize( makeVector3f(1,0, derivate_u).crossProduct(makeVector3f(0,1, derivate_v)) );
			newNormal = tangentSystemN * newNormal;
			Vector3f oldNormal = bIt->get3f( normalAAT );
			if( newNormal * oldNormal < 0.0f )
				newNormal = -newNormal;
			mIt->set3f( normalAAT, newNormal );

			for( card32 c=0;c<3;c++ )
				if( _isnan( newNormal[c] ) )
				{
					numNANs++;
					mIt->set3f( normalAAT, makeVector3f(1,0,0) );
					break;
				}


			bIt->next();

			if( currentIndex++ % 300 == 1 )
				if( !progressWindow->progress( 100.0f * (float)currentIndex / pc->getNumPoints() ) )
					break;
		}
		delete iter;
		progressWindow->popSteps();

	}

	if( numNANs )
		debugOutput << "num nans: " << numNANs << "\n";

	sort( meanCurv.begin(), meanCurv.end() );
	sort( gaussianCurv.begin(), gaussianCurv.end() );
	
	
	float minH = meanCurv[0];
	float maxH = meanCurv.back();
	float minK = gaussianCurv[0];
	float maxK = gaussianCurv.back();
	if( median )
	{
		minH = meanCurv[ (float)meanCurv.size() * 0.05f ];
		maxH = meanCurv[ (float)meanCurv.size() * 0.95f ];
		minK = gaussianCurv[ (float)gaussianCurv.size() * 0.05f ];
		maxK = gaussianCurv[ (float)gaussianCurv.size() * 0.95f ];
	}

	for ( unsigned i = 0; i < clouds.size(); i++ )
	{
		PointCloud * pc = clouds[i];
		AAT destAAT = pc->getAAT( "curvature" );

		SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC );
		pAssert( iter != NULL );
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
		ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);

		while( !bIt->atEnd() )
		{
			Vector4f curv = bIt->get4f( destAAT );
			float tmpMean = min(1.0f, max(0.0f,(curv[0] - minH) / (maxH - minH ) ));
			curv[0] = ( fabs(tmpMean) );
			float tmpGaussian = min(1.0f, max(0.0f,(curv[1] - minK) / (maxK - minK ) ));
			curv[1] = ( fabs(tmpGaussian) );
			mIt->set4f( destAAT, curv );
			bIt->next();
		}

		delete iter;
	}
}



void PCIFeatureTest::testDetectorStability_Rigid( PointCloud * pc, int noiseLevel, float noise, int numTests ) 
{
	BoundingBox3f bb = geoxl::getPCBBox( pc );
	float bb_diag = norm( bb.lowerCorner - bb.upperCorner);
	vector<float> distanceHistogram(6,0);
	vector<float> ambigousHistogram(distanceHistogram.size(),0);
	float noCoorespondence = 0.0f;
	float averagePointDistance = geoxl::getAveragePointDistance(pc,4);

	float curvature_sigma = max(3.0f, noise/averagePointDistance*bb_diag);
	float normal_sigma = max(2.0f,noise/averagePointDistance*bb_diag);

	for( int test=0;test<numTests;test++ )
	{
		vector<FeatureList*> features;
		vector<PointCloud*> clouds;
		static int testI = 0;

		Matrix3f randRot = makeRotX3f( rnd01() * 2.0f * M_PI )
			* makeRotY3f( rnd01() * 2.0f * M_PI )
			* makeRotZ3f( rnd01() * 2.0f * M_PI );
		Vector3f randTransl = makeVector3f( rnd01() - 0.5f, rnd01() - 0.5f,rnd01() - 0.5f )*bb_diag;

		// create noisy point clouds
		for( card32 a=0;a<2;a++ )
		{
			UnstructuredInCorePointCloud * pcCopy = new UnstructuredInCorePointCloud;
			pcCopy->clear();
			pcCopy->getPointSet()->assign( ((UnstructuredInCorePointCloud*)pc)->getPointSet() );
			VertexArray va( pcCopy->getPointSet() );
			for( card32 i=0;i<va.getNumElements();i++ )
			{
				float noise_strength = min(NormalDistribution::randomNumber( 0, bb_diag * noise ), 2.0f * bb_diag * noise );
				float theta = rnd01() * 2.0f * M_PI;
				float phi = acos( rnd01()*2.0f-1.0f ) - M_PI/2.0f;
				Vector3f noise_vec = makeVector3f( sin(theta)*cos(phi), cos(theta)*cos(phi), sin(phi)) * noise_strength;
				
				Vector3f tmp = va.getPosition3f(i);
				if( noise > 0 )
					tmp += noise_vec;

				if( a == 1 )
				{
					tmp = randRot * tmp + randTransl;
					va.setNormal3f( i, randRot * va.getNormal3f(i) );
				}
				va.setPosition3f( i, tmp );
				

			}
			if( test == 0 && a == 0 )
			{
				//PointCloud * pcShowCopy = (PointCloud*)pcCopy->copy();
				//addPointCloud( getScene(), pcShowCopy, string("NoisyPC") + intToStr( testI ));
				testI++;
			}

			pcCopy->clearAttachments();
			clouds.push_back(pcCopy);
		}


		// detect features
		float averagePointDistance = getAveragePointDistance( clouds[0] );
		oldAveragePointDist = averagePointDistance;
		FeatureRegistration * fr = setupRegistration( averagePointDistance );

		
		calculateCurvaturePC( clouds, curvature_sigma, normal_sigma, true );
		//addPointCloud( getScene(), clouds[0], string("NoisyPC") + numToStr(noise));
		//delete clouds[1];
		//return;
	
		if( dynamic_cast<MLSMultiScaleFeatureDetector*>( m_FeatureDetector ) )
		{
			for( card32 a=0;a<clouds.size();a++ )
			{
				FeatureList * f = fr->calculateFeatures( clouds[a], FeatureRegistration::DETECT_POS );
				features.push_back(f);
			}
		}
		else
		{
			features = m_FeatureDetector->detectFeatures( clouds, m_DescClusterRadius*averagePointDistance);
		}

		// -----------------------------------------
		// analyse

		// find corresponding features
		FeatureList * featuresA = features[0], *featuresB = features[1];
		

		for( card32 i=0;i<featuresA->size();i++ )
		{
			vector<int> his( distanceHistogram.size() , 0 );
			for( card32 j=0;j<featuresB->size();j++ )
			{
				Vector3f featureBPos = featuresB->get(j)->getPosition();
				featureBPos = randRot.transpose() * (featureBPos-randTransl);
				float dist = norm( featuresA->get(i)->getPosition() - featureBPos );
				if( featuresA->get(i)->compareDetectorSignature( featuresB->get(j) ) > 0.1f )
				{
					continue;
				}

				
				for( card32 k=0;k<his.size();k++ )
				{
					if( dist < (float)(k+1) * averagePointDistance )
					{
						his[k]++;
						break;
					}
				}
			}

			for( card32  k=0;k<his.size();k++ )
			{
				if( his[k] > 0 )
				{
					if( his[k] == 1 )
						distanceHistogram[k] += 1.0f / (float)numTests;
					else
						ambigousHistogram[k] += 1.0f / (float)numTests;
					break;
				}

				if( k == his.size()-1)
					noCoorespondence +=  1.0f / (float)numTests;
			}


		}

		
		for( card32 a=0;a<clouds.size();a++)
			delete clouds[a];

		delete m_Features;
		m_Features = new FeatureList;

		for( card32 i=0;i<featuresA->size();i++ )
		{
			Feature * f = (Feature*)featuresA->get(i)->copy();
			f->setColor( 0x00ff00 );
			m_Features->push_back(f);
		}

		for( card32 i=0;i<featuresB->size();i++ )
		{
			Feature * f = (Feature*)featuresB->get(i)->copy();
			f->setColor( 0x0000ff );
			m_Features->push_back(f);
		}
		delete fr;
	}

	for( card32 i=0;i<distanceHistogram.size();i++ )
	{
		tableOutputTool->setData( "Eindeutige Features", i, noiseLevel, distanceHistogram[i] );
		tableOutputTool->setData( "Nicht eindeutige Features", i, noiseLevel, ambigousHistogram[i] );
		tableOutputTool->setRowName( "Eindeutige Features", i, "<=" + intToStr(i+1) );
		tableOutputTool->setRowName( "Nicht eindeutige Features", i, "<=" + intToStr(i+1) );
	}
	tableOutputTool->setData( "Eindeutige Features", (card32)distanceHistogram.size(), noiseLevel, noCoorespondence );
	tableOutputTool->setRowName( "Eindeutige Features", (card32)distanceHistogram.size(), ">" + intToStr(distanceHistogram.size()) );
	

	/*debugOutput << "Featuretest-------------------------\n\nSettings:\n"
		<< " - noise = " << noise << "\n - curvature_sigma = " << curvature_sigma
		<< "\nResults: -------------\nnon-ambigous:\n";

	for( card32 i=0;i<distanceHistogram.size();i++ )
		debugOutput << distanceHistogram[i] << "\n";

	debugOutput << "ambigous:\n";
	for( card32 i=0;i<ambigousHistogram.size();i++ )
		debugOutput << ambigousHistogram[i] << "\n";

	debugOutput << "no correspondence:" << noCoorespondence << "\n";*/
	message( "" );


}

void PCIFeatureTest::testDetectorStability_Rigid() 
{
   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() != 1 )
		return;

	PointCloud * pc = NULL;
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		pc = getPointCloud( getScene(), selection[m] );
		if( pc )
			break;
	}

	vector<float> noiseLevels;
	noiseLevels.push_back( 0 );
	noiseLevels.push_back( 0.001f );
	noiseLevels.push_back( 0.002f );
	noiseLevels.push_back( 0.004f );
	noiseLevels.push_back( 0.008f );
	noiseLevels.push_back( 0.016f );


	tableOutputTool->clear();
	
	for( card32 i=0;i<noiseLevels.size();i++ )
	{
		float noise = noiseLevels[i];
		tableOutputTool->setColumnName( "Eindeutige Features", i, floatToStr( noise ) );
		tableOutputTool->setColumnName( "Nicht eindeutige Features", i, floatToStr( noise ) );
		testDetectorStability_Rigid(pc, i, noise , 5 );
		debugOutput.flush();
	}

	debugOutput << tableOutputTool->outputSimpleTable( "Eindeutige Features", TableOutputTools::DF_FloatComma );
	debugOutput << tableOutputTool->outputLatexTable( "Eindeutige Features", TableOutputTools::DF_FloatPoint );
	debugOutput << tableOutputTool->outputSimpleTable( "Nicht eindeutige Features", TableOutputTools::DF_FloatComma );
	debugOutput << tableOutputTool->outputLatexTable( "Nicht eindeutige Features", TableOutputTools::DF_FloatPoint );

	//testDetectorStability_Rigid(pc, 0.001f , 1 );
}

void PCIFeatureTest::calculateAllFeatures(float featureRadius) 
{
	Timer timer;
	card32 startTime = timer.getCurrentValue();
	BoundingBox3f bb = m_SceneBB;
	if( m_FeatureDetector == NULL )
		return;

	if( m_FeatureDescriptor == NULL )
		return;

   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() < 2 )
		return;

	for( card32 i=0;i<m_Clouds.size();i++ )
		delete m_Clouds[i].features;
	m_Clouds.clear();
	
	// get pointclouds from scene
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		TGlobalRegistrationCloud cloud;
		cloud.pc = getPointCloud( getScene(), selection[m] );
		cloud.name = selection[m];
		if( cloud.pc )
			m_Clouds.push_back(cloud);
	}

	if( m_Clouds.size() < 2 )
		return;

	float averagePointDistance = getAveragePointDistance( m_Clouds[0].pc );
	oldAveragePointDist = averagePointDistance;
	FeatureRegistration * fr = setupRegistration( averagePointDistance );

	calculateCurvatureForAlleClouds( 3.0f, 2.0f, true );

	vector<PointCloud*> clouds;
	for( card32 i=0;i<m_Clouds.size();i++)
		clouds.push_back( m_Clouds[i].pc );
	
	if( dynamic_cast<MLSMultiScaleFeatureDetector*>( m_FeatureDetector ) )
	{
		for( card32 i=0;i<clouds.size();i++ )
			m_Clouds[i].features = fr->calculateFeatures( clouds[i], FeatureRegistration::DETECT_POS );
	}
	else
	{
		vector<FeatureList*> features = m_FeatureDetector->detectFeatures( clouds, m_DescClusterRadius*averagePointDistance);
		for( card32 i=0;i<m_Clouds.size();i++ )
			m_Clouds[i].features = features[i];
	}

	for( card32 i=0;i<m_Clouds.size();i++ )
	{
		FeatureList * fl = removeBorderFeatures( m_Clouds[i].features, featureRadius * averagePointDistance / 2.0f,
			m_Clouds[i].pc );
		delete m_Clouds[i].features;
		m_Clouds[i].features = fl;
	}
	
	// remove border features
	/*for( card32 i=0;i<m_Clouds.size();i++ )
	{
		vector< TTmpFeatureAsdf > features;
		HierarchicalKNNIterator hIt( m_Clouds[i].pc ,32 , NULL );
		AAT posAAT = m_Clouds[i].pc->getAAT( "position" );
		for( card32 j=0;j<m_Clouds[i].features->size();j++ )
		{
			TTmpFeatureAsdf f;
			f.f = m_Clouds[i].features->get(j);
			hIt.setSeekPointAndReset( f.f->getPosition() );
			f.numPoints = 0;
			while( !hIt.atEnd() )
			{
				if( norm( hIt.get3f(posAAT) -  f.f->getPosition() ) > featureRadius * averagePointDistance )
					break;
				f.numPoints++;
				hIt.next();
			}

			features.push_back(f);
		}


		sort( features.begin(), features.end(), compareFeatureSupport );
		int minNumberOfPoints = features[ features.size() / 2 ].numPoints * 0.75f;
		FeatureList * fl = new FeatureList;
		for( card32 j=0;j<features.size();j++ )
			if( features[j].numPoints >= minNumberOfPoints )
				fl->push_back( (Feature*)features[j].f->copy() );

		debugOutput << "num feautures: " << m_Clouds[i].features->size() <<
			" num remaining: " << fl->size() << "\n";
		delete m_Clouds[i].features;
		m_Clouds[i].features = fl;
	}*/
	

	delete m_Features;
	m_Features = new FeatureList;

	for( card32 c=0;c<m_Clouds.size();c++ )
	{

		for( card32 i=0;i<m_Clouds[c].features->size();i++ )
		{
			m_Features->push_back((Feature*)m_Clouds[c].features->get(i)->copy());
		}
	}

	card32 deltaTime = timer.getCurrentValue() - startTime;
	debugOutput << "Calculate all feature points: " << convertTimeToString(deltaTime) << "\n";
}


void PCIFeatureTest::registerAllClouds(float featureRadius, int maxNumberOfPairs, float pruningThreshold) 
{
	Timer timer;
	card32 startTime = timer.getCurrentValue();
	if( m_Clouds.size()==0 )
		return;

	bool createdResultPC = false;
	float averagePointDistance = geoxl::getAveragePointDistance( m_Clouds[0].pc );

	vector<TGlobalRegistrationCloud> clouds = m_Clouds;
	
	FeatureRegistration * fr = setupRegistration( averagePointDistance );
	fr->setSceneBB( m_SceneBB );
	fr->setSpectralSigma( m_SpectralSigma );
	fr->setSpectralAddDiff( m_SpectralAddDiff );
	fr->setSpectralScaleDiff( m_SpectralScaleDiff );
	debugOutput << "Settings:\n"
		<< "Sigma: " << m_SpectralSigma
		<< "\nAdd: " << m_SpectralAddDiff
		<< "\n";
	while( clouds.size() > 1 )
	{
		debugOutput << "num clouds: " << (mpcard)clouds.size() << "\n";
		// find best match for first cloud
		float bestScore = -1;
		card32 bestCloud = 0;
		Matrix4f transfBtoA;
		for( card32 i=1;i<clouds.size();i++ )
		{
			vector<FeaturePair> pairs = fr->buildFeaturePairs( clouds[0].features, clouds[i].features, pruningThreshold, maxNumberOfPairs );
			vector<FeaturePair> validpairs = fr->validateFeaturePairs( pairs, clouds[0].pc, clouds[i].pc );
			Matrix4f transformation = fr->calculateTransformation( validpairs );

			float score = testOverlap(	clouds[0].pc, clouds[i].pc, transformation );
			if( score > bestScore )
			{
				bestCloud = i;
				bestScore = score;
				transfBtoA = transformation;
			}

			debugOutput << "------------\nNum pairs: " << (mpcard) pairs.size()
				<< "\nNum valid pairs: " << (mpcard) validpairs.size()
				<< "\nScore : " << score << "\n";
		}

		// -----------------------------------------------------------
		// create resulting point cloud
		if( !createdResultPC )
		{
			TGlobalRegistrationCloud newCloud;
			newCloud.pc = (PointCloud*)clouds[0].pc->copy();
			newCloud.features = new FeatureList;
			newCloud.features->assign( clouds[0].features );
			clouds[0] = newCloud;
			createdResultPC = true;
		}

		// -----------------------------------------------------------
		// merge clouds with best score
		PointSet * ps0 = ((UnstructuredInCorePointCloud*)clouds[0].pc)->getPointSet();
		PointSet * ps1 = ((UnstructuredInCorePointCloud*)clouds[bestCloud].pc)->getPointSet();
		AAT posAAT = ps0->getAAT( "position" );
		AAT normalAAT = ps0->getAAT( "normal" );

		Matrix3f normalTransformation = shrink4To3( transfBtoA );
		
		// copy points into res pc
		vector<char> buffer( ps1->getDescr()->getSize() );
		for( card32 i=0;i<ps1->getNumEntries();i++ )
		{
			ps1->getVertex(i, &buffer[0] );
			ps0->addPoint( &buffer[0] );

			// transform point and normal
			card32 index = ps0->getNumEntries()-1;
			ps0->set3f( index, posAAT, shrink4To3( transfBtoA * expand3To4(ps0->get3f(index,posAAT))));
			ps0->set3f( index, normalAAT, normalTransformation * ps0->get3f(index,normalAAT));
		}

		// merge featuresets
		for( card32 i=0;i<clouds[bestCloud].features->size();i++ )
		{
			Feature * f = (Feature*)clouds[bestCloud].features->get(i)->copy();
			f->setPosition( shrink4To3( transfBtoA * expand3To4(f->getPosition() )));
			clouds[0].features->push_back(  f );
		}

		// set transformation
		SGObjectNode * node = dynamic_cast<SGObjectNode*>( getSceneGraphNodeByName( getScene(), clouds[bestCloud].name) );
		if( node )
		{
			node->getSceneObject()->setTransformation( transfBtoA );
		}
		
		clouds[0].pc->clearAttachments();
		clouds.erase( clouds.begin() + bestCloud );
		
		/*// recalulate descriptors
		TDescriptorContext dc;
		dc.interpolation = new SplatPullPushInterpolation;
		dc.parameterization = new HeighfieldParameterization;
		dc.parametricImageSize = 32;
		dc.pc = clouds[0].pc;
		dc.tIt = NULL;
		dc.averagePointDistance = averagePointDistance;

		for( card32 i=0;i<clouds[0].features->size();i++ )
		{
			FeatureDescriptor * fd = (FeatureDescriptor *)m_FeatureDescriptor->copy(); 
			fd->createDescriptor( clouds[0].features->get(i)->getPosition(), featureRadius * averagePointDistance, dc.pc, &dc );
			fd->normalizeDescriptor();
			clouds[0].features->get(i)->deleteDescriptors(0, clouds[0].features->get(i)->getNumDescriptors() );
			clouds[0].features->get(i)->addDescriptor( fd );
		}*/

	}

	if( getPointCloud(  getScene(), "result" )  == NULL)
		addPointCloud( getScene(), clouds[0].pc, "result" );
	else
		replacePointCloud( getScene(), "result", clouds[0].pc );

	card32 deltaTime = timer.getCurrentValue() - startTime;
	debugOutput << "Global Registration: " << convertTimeToString(deltaTime) << "\n";

}

float PCIFeatureTest::testOverlap( PointCloud * pcA, PointCloud *pcB, Matrix4f transformBtoA ) 
{
	VertexArray va( ((UnstructuredInCorePointCloud*)pcB)->getPointSet() );
	float averagePointDistance = geoxl::getAveragePointDistance( pcB );
	float sigma = 1.5f * averagePointDistance;
	float inv_sigma_2 = 1.0f / (sigma*sigma);

	HierarchicalKNNIterator hIt( pcA, 32, NULL );
	AAT posAAT = pcA->getAAT("position");

	float ret = 0;
	for( card32 i=0;i<va.getNumElements();i++ )
	{
		Vector3f pos = shrink4To3( transformBtoA * expand3To4( va.getPosition3f(i) ) );
		hIt.setSeekPointAndReset(pos);
		while( !hIt.atEnd() )
		{
			Vector3f tmpPos = hIt.get3f( posAAT );
			float dist_2 = normQuad( tmpPos - pos );
			float weight = exp(-dist_2 * inv_sigma_2);
			if( weight < 0.1f )
				break;

			ret += weight;
			
			hIt.next();
		}
	}

	return ret;
}

void PCIFeatureTest::testDescriptorStability() 
{
   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() != 1 )
		return;

	PointCloud * pc = NULL;
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		pc = getPointCloud( getScene(), selection[m] );
		if( pc )
			break;
	}

	// detect features
	float averagePointDistance = getAveragePointDistance( pc );
	float descSize = averagePointDistance * 5.0f;
	oldAveragePointDist = averagePointDistance;
	FeatureRegistration * fr = setupRegistration( averagePointDistance );


	vector<PointCloud*> clouds;
	clouds.push_back( pc );
	calculateCurvaturePC( clouds, 3.0f, 2.0f, true );

	FeatureList * features = NULL;
	if( dynamic_cast<MLSMultiScaleFeatureDetector*>( m_FeatureDetector ) )
	{
		features = fr->calculateFeatures( pc, FeatureRegistration::DETECT_POS );
	}
	else
	{
		vector<FeatureList*> tmp_features = m_FeatureDetector->detectFeatures( clouds, m_DescClusterRadius*averagePointDistance);
		features = tmp_features[0];
	}


	FeatureList * fl = removeBorderFeatures( features, descSize, pc );
	delete features;
	features = fl;


	vector<float> noiseLevels;
	noiseLevels.push_back( 0 );
	noiseLevels.push_back( 0.001f );
	noiseLevels.push_back( 0.002f );
	noiseLevels.push_back( 0.004f );
	noiseLevels.push_back( 0.008f );
	noiseLevels.push_back( 0.016f );

	int numRings = 1;
	vector<TDescTest> tests;
	FDAverage fda;
	fda.setAttribute( "curvature" );
	fda.setNumComponents(1);
	fda.setNumRings( numRings );
	tests.push_back( TDescTest((FeatureDescriptor *) fda.copy(), "Durchschnitt",descSize) );
	FDHistogram fdhis;
	fdhis.setAttribute( "curvature" );
	fdhis.setAttributeCompNumber(0);
	fdhis.setNumBins( 8 );
	fdhis.setNumHistograms( numRings );
	tests.push_back( TDescTest((FeatureDescriptor *) fdhis.copy(), "Radialhistogram",descSize) );

	
	tableOutputTool->clear();
	/*for( card32 i=0;i<tests.size();i++ )
	{
		tableOutputTool->setRowName( "Summierter Fehler", i, tests[i].rowName );
		tableOutputTool->setRowName( "Anzahl falscher Korrespondenzen", i, tests[i].rowName );
	}
	
	for( card32 i=0;i<noiseLevels.size();i++ )
	{
		float noise = noiseLevels[i];
		tableOutputTool->setColumnName( "Summierter Fehler", i, floatToStr( noise ) );
		tableOutputTool->setColumnName( "Anzahl falscher Korrespondenzen", i, floatToStr( noise ) );
		testDescriptorStability(features, pc, tests, i, noise , 5 );
	}

	debugOutput << "Num Rings: " << numRings << "\ndescSize: " << descSize / averagePointDistance << "\n\n";
	debugOutput << tableOutputTool->outputSimpleTable( "Summierter Fehler", TableOutputTools::DF_FloatComma ) << "\n";
	debugOutput << tableOutputTool->outputLatexTable( "Summierter Fehler", TableOutputTools::DF_FloatPoint ) << "\n";
	debugOutput << tableOutputTool->outputSimpleTable( "Anzahl falscher Korrespondenzen", TableOutputTools::DF_FloatComma ) << "\n";
	debugOutput << tableOutputTool->outputLatexTable( "Anzahl falscher Korrespondenzen", TableOutputTools::DF_FloatPoint ) << "\n";*/
	testDescriptorStability(features, pc, tests, 0, 0 , 1 );
	for( card32 i=0;i<tests.size();i++ )
	{
		DynamicPCA<float> pca( &tests[i].pcatest_descriptors[0], (card32)tests[i].pcatest_descriptors.size() );
		DVectorF eigen = pca.calcEigenValues();
		float summedEigen = 0;
		for( card32 j=0;j<eigen.getDim();j++ )
			summedEigen += eigen[j];

		for( card32 j=0;j<eigen.getDim();j++ )
		{
			tableOutputTool->setRowName( tests[i].rowName, j, intToStr(j));
			tableOutputTool->setData( tests[i].rowName, j, 0, eigen[j] / summedEigen );
		}
		tableOutputTool->setColumnName(tests[i].rowName, 0, tests[i].rowName );
		debugOutput << tableOutputTool->outputSimpleTable( tests[i].rowName, TableOutputTools::DF_FloatComma ) << "\n";
		debugOutput << tableOutputTool->outputLatexTable( tests[i].rowName, TableOutputTools::DF_FloatPoint ) << "\n";


	}

}

PointCloud *PCIFeatureTest::getNoisyPointCloud( PointCloud * pc, float noise, float bb_diag, Matrix3f &randRot, Vector3f &randTransl) 
{
	randRot = makeRotX3f( rnd01() * 2.0f * M_PI / 100.0f )
			* makeRotY3f( rnd01() * 2.0f * M_PI / 100.0f )
			* makeRotZ3f( rnd01() * 2.0f * M_PI / 100.0f );
	randTransl = makeVector3f( rnd01() - 0.5f, rnd01() - 0.5f,rnd01() - 0.5f )*bb_diag;
	//randRot = IDENTITY3F;
	//randTransl = NULL_VECTOR3F;

	UnstructuredInCorePointCloud * pcCopy = new UnstructuredInCorePointCloud;
	pcCopy->clear();
	pcCopy->getPointSet()->assign( ((UnstructuredInCorePointCloud*)pc)->getPointSet() );
	VertexArray va( pcCopy->getPointSet() );
	for( card32 i=0;i<va.getNumElements();i++ )
	{
		float noise_strength = min(NormalDistribution::randomNumber( 0, bb_diag * noise ), 2.0f * bb_diag * noise );
		float theta = rnd01() * 2.0f * M_PI;
		float phi = acos( rnd01()*2.0f-1.0f ) - M_PI/2.0f;
		Vector3f noise_vec = makeVector3f( sin(theta)*cos(phi), cos(theta)*cos(phi), sin(phi)) * noise_strength;
		
		Vector3f tmp = va.getPosition3f(i);
		if( noise > 0 )
			tmp += noise_vec;

		tmp = randRot * tmp + randTransl;
		va.setNormal3f( i, randRot * va.getNormal3f(i) );

		va.setPosition3f( i, tmp );
		

	}
	return pcCopy;

}


struct TTestPruningPair
{
	bool correct;
	float dist;
	bool operator < ( const TTestPruningPair &o ) const
	{
		return dist < o.dist;
	}
};

void PCIFeatureTest::testDescriptorStability(FeatureList * features, PointCloud * pc, vector<TDescTest> &test, int noiseLevel, float noise, int numTests) 
{
	BoundingBox3f bb = geoxl::getPCBBox( pc );
	float bb_diag = norm( bb.lowerCorner - bb.upperCorner);
	bool noiseFeatureMode = true;

	Matrix3f randRot;
	Vector3f randTrans;
	PointCloud * npc1 = getNoisyPointCloud( pc, noiseFeatureMode ? 0 : noise, bb_diag,randRot,randTrans );
	float averagePointDistance = geoxl::getAveragePointDistance(npc1);
	float curvature_sigma = max(3.0f, noise/averagePointDistance*bb_diag);
	float normal_sigma = max(2.0f,noise/averagePointDistance*bb_diag);
	vector<PointCloud*> clouds;
	clouds.push_back( npc1);
	calculateCurvaturePC( clouds, curvature_sigma, normal_sigma, true );

	TDescriptorContext dc;
	dc.interpolation = new SplatPullPushInterpolation;
	dc.parameterization = new HeighfieldParameterization;
	dc.parametricImageSize = 64;
	dc.pc = npc1;
	dc.tIt = NULL;
	dc.averagePointDistance = averagePointDistance;

	vector< vector< FeatureDescriptor*> > referenceDescriptors;
	// calculate reference descriptors
	for( card32 i=0;i<test.size();i++ )
	{
		FeatureDescriptor * fd = test[i].fd;
		vector< FeatureDescriptor*> descs;
		for( card32 j=0;j<features->size();j++ )
		{
			FeatureDescriptor * fdnew = (FeatureDescriptor *)fd->copy();
			float noise_strength = min(NormalDistribution::randomNumber( 0, bb_diag * noise ), 2.0f * bb_diag * noise );
			float theta = rnd01() * 2.0f * M_PI;
			float phi = acos( rnd01()*2.0f-1.0f ) - M_PI/2.0f;
			Vector3f noise_vec = makeVector3f( sin(theta)*cos(phi), cos(theta)*cos(phi), sin(phi)) * noise_strength;
			//if( !noiseFeatureMode )
				noise_vec *= 0;
			fdnew->createDescriptor( randRot * features->get(j)->getPosition() + randTrans + noise_vec, test[i].descSize , dc.pc, &dc );
			fdnew->normalizeDescriptor();
			descs.push_back(fdnew);
		}
		referenceDescriptors.push_back(descs);
	}

	for( card32 t=0;t<(card32)numTests;t++)
	{
		// create noisy point cloud and estimate curvature+normals

		PointCloud * npc2 = pc;
		randRot = IDENTITY3F;
		randTrans = NULL_VECTOR3F;
		if( !noiseFeatureMode)
			npc2 = getNoisyPointCloud( pc, noise,bb_diag,randRot,randTrans );
		dc.pc = npc2;
		vector<PointCloud*> clouds;
		clouds.push_back( npc2);
		if( !noiseFeatureMode )
			calculateCurvaturePC( clouds, curvature_sigma, normal_sigma, true );
		
		for( card32 i=0;i<test.size();i++ )
		{
			FeatureDescriptor * fd = test[i].fd;
			// calculate descriptors
			vector<FeatureDescriptor*> descriptors;
			for( card32 j=0;j<features->size();j++ )
			{
				FeatureDescriptor * fdnew = (FeatureDescriptor *)fd->copy();
				float noise_strength = min(NormalDistribution::randomNumber( 0, bb_diag * noise ), 2.0f * bb_diag * noise );
				float theta = rnd01() * 2.0f * M_PI;
				float phi = acos( rnd01()*2.0f-1.0f ) - M_PI/2.0f;
				Vector3f noise_vec = makeVector3f( sin(theta)*cos(phi), cos(theta)*cos(phi), sin(phi)) * noise_strength;
				//if( !noiseFeatureMode )
					noise_vec *= 0;
				fdnew->createDescriptor( randRot * features->get(j)->getPosition() + randTrans + noise_vec, test[i].descSize , dc.pc, &dc );
				fdnew->normalizeDescriptor();
				DVectorF dvecdesc;
				dvecdesc.setDim( fdnew->getNumDescriptorValues() );
				for( card32 di = 0; di<fdnew->getNumDescriptorValues(); di++ )
					dvecdesc[di] = (*fdnew)[di];
				test[i].pcatest_descriptors.push_back( dvecdesc );
				descriptors.push_back(fdnew);
			}

			// sum distances of correct pairs
			float summed_distance = 0;
			for( card32 j=0;j<features->size();j++ )
			{
				summed_distance += descriptors[j]->compare( referenceDescriptors[i][j] );
			}
			summed_distance /= (float)numTests;

			// build set of all correspondences between compatible features
			vector< TTestPruningPair > pairs;
			for( card32 j=0;j<features->size();j++ )
			{
				for( card32 k=0;k<features->size();k++ )
				{
					if( features->get(j)->compareDetectorSignature( features->get(k) ) < 0.1f )
					{
						TTestPruningPair pair;
						pair.correct = j == k;
						pair.dist = descriptors[j]->compare( referenceDescriptors[i][k] );
						pairs.push_back( pair );
					}
				}
			}

			// count incorrect pairs for a initialset which contains #feature/2 correct pairs
			sort( pairs.begin(), pairs.end() );
			card32 numIncorrect = 0;
			card32 numCorrect = 0;
			for( card32 j=0;j<pairs.size();j++ )
			{
				if( pairs[j].correct )
					numCorrect++;
				else
					numIncorrect++;
				if( numCorrect >= features->size() / 2 )
					break;
			}

			// report to table
			tableOutputTool->addData( "Summierter Fehler", i, noiseLevel, summed_distance );
			tableOutputTool->addData( "Anzahl falscher Korrespondenzen", i, noiseLevel, (float)numIncorrect / (float)numTests );
	

		}


		if( !noiseFeatureMode )
			delete npc2;
	}
	delete npc1;
}

FeatureList *PCIFeatureTest::removeBorderFeatures( FeatureList * fl, float radius, PointCloud * pc ) 
{
	FeatureList * ret = new FeatureList;
	HeighfieldParameterization param;
	int numSlices = 6;

	for( card32 i=0;i<fl->size();i++ )
	{
		param.calculate(pc, fl->get(i)->getPosition(), radius );
		PointSet * ps = param.getParametricPoints();
		AAT paramPosAAT = ps->getAAT("param_pos");

		vector<float> histogram( numSlices,0);
		for( card32 j=0;j<ps->getNumEntries();j++ )
		{
			Vector2f parampos = ps->get2f( j, paramPosAAT );
			float angle = (atan2(parampos[0],parampos[1]) + M_PI) / 2.0f / M_PI * (float)numSlices;
			int slice = (angle + numSlices);
			slice = slice % numSlices;
			histogram[slice]++;
		}
		sort( histogram.begin(), histogram.end() );
		float median = histogram[ numSlices / 2];
		if( histogram[0] > median * 0.75f )
		{
			// valid feature
			ret->push_back( (Feature*)fl->get(i)->copy() );
		}

	}
	return ret;
}

void PCIFeatureTest::testValidation() 
{
   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();
	if( selection.size() != 1 )
		return;

	PointCloud * pc = NULL;
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		pc = getPointCloud( getScene(), selection[m] );
		if( pc )
			break;
	}

	// detect features
	float averagePointDistance = getAveragePointDistance( pc );
	oldAveragePointDist = averagePointDistance;
	FeatureRegistration * fr = setupRegistration( averagePointDistance );

	vector<PointCloud*> clouds;
	clouds.push_back( pc );
	calculateCurvaturePC( clouds, 3.0f, 2.0f, true );

	FeatureList * features = NULL;
	if( dynamic_cast<MLSMultiScaleFeatureDetector*>( m_FeatureDetector ) )
	{
		features = fr->calculateFeatures( pc, FeatureRegistration::DETECT_POS );
	}
	else
	{
		vector<FeatureList*> tmp_features = m_FeatureDetector->detectFeatures( clouds, m_DescClusterRadius*averagePointDistance);
		features = tmp_features[0];
	}



	vector<float> noiseLevels;
//	noiseLevels.push_back( 0 );
//	noiseLevels.push_back( 0.001f );
//	noiseLevels.push_back( 0.002f );
//	noiseLevels.push_back( 0.004f );
	noiseLevels.push_back( 0.008f );
//	noiseLevels.push_back( 0.016f );

	int nk = 20;
	vector<int> numIncorrect;
	BoundingBox3f bb = geoxl::getPCBBox( pc );
	float bb_diag = norm( bb.lowerCorner - bb.upperCorner);

	tableOutputTool->clear();
	progressWindow->pushStep(true, "test" );
	for( card32 n=0;n<noiseLevels.size();n++ )
	{
		float noise = noiseLevels[n];
		tableOutputTool->setRowName( "Validierung", n, intToStr( noise * 1000.0f ) );
		// create noisy features sets
		FeatureList * featuresA = new FeatureList;
		FeatureList * featuresB = new FeatureList;

		for( card32 i=0;i<features->size();i++ )
		{
			Feature * f1 = (Feature*)features->get(i)->copy();
			Feature * f2 = (Feature*)features->get(i)->copy();

			for( card32 j=0;j<2;j++ )
			{
				float noise_strength = min(NormalDistribution::randomNumber( 0, bb_diag * noise ), 2.0f * bb_diag * noise );
				float theta = rnd01() * 2.0f * M_PI;
				float phi = acos( rnd01()*2.0f-1.0f ) - M_PI/2.0f;
				Vector3f noise_vec = makeVector3f( sin(theta)*cos(phi), cos(theta)*cos(phi), sin(phi)) * noise_strength;
				if( j == 0 )
					f1->setPosition( f1->getPosition() + noise_vec );
				else
					f2->setPosition( f2->getPosition() + noise_vec );
			}
			featuresA->push_back( f1 );
			featuresB->push_back( f2 );

		}

		debugOutput << "num Feautres: " << features->size() << "\n";
		
		for( card32 i=9;i<10;i++ )
		{
			int nf = i*40;
			tableOutputTool->setColumnName( "Validierung", i, floatToStr( nf ) );

			FeatureRegistration fr;
			fr.setSceneBB( m_SceneBB );
			fr.setSpectralSigma( m_SpectralSigma );
			fr.setSpectralAddDiff( m_SpectralAddDiff );
			fr.setSpectralScaleDiff( m_SpectralScaleDiff );
			
			int numTests = 100;
			for( card32 t=0;t<(card32)numTests;t++ )
			{

				// build initial set
				vector<FeaturePair> pairs;
				for( card32 j=0;j<(card32)nk && j<features->size();j++ )
				{
					FeaturePair pair;
					pair.m_DescriptorDistance = 0;
					pair.m_Feature1 = featuresA->get(j);
					pair.m_Feature2 = featuresB->get(j);
					pair.m_Index1 = j;
					pair.m_Index2 = j;
					pairs.push_back( 	pair );
				}

				// add incorrect assignments
				map<int,int> usedcorr;
				for( card32 j=0;j<(card32)nf;j++ )
				{
					card32 i1 = rand() % features->size();
					card32 i2 = rand() % features->size();
					while( i1 == i2 || usedcorr.find( i1 * features->size()+i2) != usedcorr.end() )
					{
						i1 = rand() % features->size();
						i2 = rand() % features->size();
					}

					usedcorr[ i1 * features->size()+i2 ] = 1;
					FeaturePair pair;
					pair.m_DescriptorDistance = 0;
					pair.m_Feature1 = featuresA->get(i1);
					pair.m_Feature2 = featuresB->get(i2);
					pair.m_Index1 = i1;
					pair.m_Index2 = i2;
					pairs.push_back( 	pair );

				}

				// validate
				vector<FeaturePair> result = fr.validateFeaturePairs( pairs, pc, pc );

				// get result
				int numIncorrect = 0;
				for( card32 j=0;j<result.size();j++ )
				{
					numIncorrect += result[j].m_Index1 !=result[j].m_Index2 ? 1 : 0;
				}
				
				tableOutputTool->addData( "Validierung", n, i, (float)numIncorrect/(float)result.size()/(float)numTests );

			}

		}
		progressWindow->progress( (n+1) * 100 / (card32)noiseLevels.size() );
		delete featuresA;
		delete featuresB;
	}
	progressWindow->popSteps();

	debugOutput << tableOutputTool->outputSimpleTable( "Validierung", TableOutputTools::DF_FloatComma ) << "\n";
	debugOutput << "\n";
	debugOutput << tableOutputTool->outputLatexTable( "Validierung", TableOutputTools::DF_FloatPoint ) << "\n";
//	testDescriptorStability(features, pc, tests, 0, 0 , 1 );



}

vector<Matrix4f> transformationClipboard;

void PCIFeatureTest::copyTransformationsToClipboard() 
{
	transformationClipboard.clear();
   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();

	// get pointclouds from scene
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		SGObjectNode * node = dynamic_cast<SGObjectNode*>( getSceneGraphNodeByName( getScene(), selection[m] ) );
		if( node )
		{
			transformationClipboard.push_back( node->getSceneObject()->getTransformation());
		}
	}
}

void PCIFeatureTest::pasteTransformationsFromClipboard() 
{
   vector<string> selection = getScene()->getRootState()->staticState->getSelectedNodes();

	// get pointclouds from scene
	card32 i=0;
	for ( unsigned m = 0; m < selection.size(); m++ )
	{
		if( m >= transformationClipboard.size() )
			break;

		SGObjectNode * node = dynamic_cast<SGObjectNode*>( getSceneGraphNodeByName( getScene(), selection[m] ) );
		if( node )
		{
			node->getSceneObject()->setTransformation(transformationClipboard[i]);
			i++;
		}
	}
}
