#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIGlobalRegistration.h"
//---------------------------------------------------------------------------
#include "MultiResolutionPC.h"
#include "MultiScaleGenerator.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "ClassMethods.h"
#include "StringHelper.h"
#include "VertexArray.h"
#include "SelectionTools.h"
#include "RegistrationTools.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "Monomials2ndOrder.h"
#include "SeparatorClassProperty.h"
#include "TableOutputTOols.h"
#include "FastFeature.h"
#include "FeatureAttachment.h"
#include "FDCombined.h"
#include "FDHistogram.h"
#include "DebugRenderer.h"
#include "SpectralCorrespondenceValidation.h"
#include "DebugRenderer.h"
#include "CentroidDescriptor.h"
#include "SuperIterator_templCode.h"
#include "TableOutputTools.h"
#include "progressWindow.h"
#include "TopologyAttachment.h"
#include "FastFeature.h"
#include "FeatureAttachment.h"
#include "ImageDisplayer.h"
#include "RGBAImage.h"
#include "AnnSearch.h"
#include "CachedRangeQuerry.h"
#include "PointSet_templCode.h"
#include <qtCore/QDateTime>
#include "AttachedIndexedOctree.h"
#include "UniformPointCloudResampling.h"
#include "PointCloudTools.h"
#include "../externalIncludes/opencv/cxtypes.h"
#include "../externalIncludes/opencv/cxcore.h"
//---------------------------------------------------------------------------
using namespace geoxl;

static string describeShowFeaturesMode(card32 v) {
	if (v==0) return "SHOW ALL";
	if (v==1) return "SHOW MINIMA";
	if (v==2) return "SHOW MAXIMA";
	return "undefined";
};

static string describeInteractionMode(card32 v)
{
	switch(v)
	{
	case 0:
		return "DrawSignature";
	case 1:
		return "Register";
	case 2:
		return "analyseNeighbours";
	case 3:
		return "show similarity";
	case 4:
		return "select features";
	case 5:
		return "manual corresp.";
	}
	return "undefined";
}

static string describeTVSettings(card32 v)
{
	switch(v)
	{
	case 0:
		return "Dragon";
	case 1:
		return "Armadillo";
	}
	return "undefined";
}

IMPLEMENT_GEOXL_CLASS( PCIGlobalRegistration ,0)
{
	BEGIN_CLASS_INIT( PCIGlobalRegistration );

/*	ADD_CARD32_PROP( m_MaxNumInitialCorrespondences,0);
	ADD_FLOAT32_PROP(m_FeatureRadius,0);
	ADD_OBJECT_PROP( m_Descriptor, 0, FeatureDescriptor::getClass(), true );
*/


	ADD_SEPARATOR("corners");
	ADD_FLOAT32_PROP(m_TVCP_MinEigenProduct,0);
	ADD_SEPARATOR("tv");
	ADD_FLOAT32_PROP(m_TVSigma,0);
	ADD_FLOAT32_PROP(m_TVCurvSigma,0);
	ADD_FLOAT32_PROP(m_TVMaxAngle,0);
	ADD_FLOAT32_PROP(m_TVLP_MinLineLength,0);
	ADD_FLOAT32_PROP(m_TVLP_SamplesPercent,0);
	ADD_FLOAT32_PROP(m_TVLP_MaxInitialDistance,0);
	ADD_CARD32_PROP(m_TVLP_MinNumLineSamples,0);
	ADD_FLOAT32_PROP(m_TVLP_LineSmoothingSigma,0);
	ADD_FLOAT32_PROP(m_TVLP_MaxLineJoinDistance,0);
	ADD_FLOAT32_PROP(m_TVLP_MaxLineJoinAngle,0);
	ADD_FLOAT32_PROP(m_TVLP_MaxLineDistance,0);
	ADD_FLOAT32_PROP(m_TVLP_MaxLineToLineDistance,0);
	ADD_FLOAT32_PROP(m_TVLP_MinLineLengthForIntersection,0);
	ADD_CARD32_PROP_UPDATE_DESCR( m_TVDefaultSettings, 0, &tvSettingsChanged, describeTVSettings, 2);

	
/*	ADD_SEPARATOR("misc");
	ADD_FLOAT32_PROP( m_Sigma, 0 );
	ADD_FLOAT32_PROP( m_PatchSamplingSpace, 0 );
	ADD_FLOAT32_PROP( m_SourceSamplingSpace, 0 );
	ADD_CARD32_PROP( m_CurrentScale, 0 );
	ADD_FLOAT32_PROP(m_CurvatureSigma,0);
	ADD_FLOAT32_PROP(m_TestMinDifferenceStrength,0);
	ADD_BOOLEAN_PROP(m_ShowFeatureRadius,0);
	
	ADD_FLOAT32_PROP(m_FDistance,0);
	ADD_FLOAT32_PROP(m_FDistanceDeviation,0);
	ADD_FLOAT32_PROP(m_FMaxDistance,0);

	ADD_SEPARATOR("multi scale");
	ADD_CARD32_PROP( m_NumberOfScales, 0 );
	ADD_FLOAT32_PROP( m_StartScale, 0 );
	ADD_FLOAT32_PROP( m_Scale_F, 0 );

	ADD_SEPARATOR("rendering");
	ADD_CARD32_PROP_UPDATE_DESCR( m_ShowFeatureMode, 0, NULL, describeShowFeaturesMode, 3);
	ADD_BOOLEAN_PROP( m_ShowDroppedFeatures, 0);

	ADD_SEPARATOR("tests");
	ADD_CARD32_PROP( m_FS_NumTests, 0 );
	ADD_FLOAT32_PROP( m_FS_NoiseAmplitude, 0 );
	ADD_FLOAT32_PROP( m_FS_ResamplingPercentage, 0 );

	ADD_FLOAT32_PROP(m_CD_WindowSize,0);
	ADD_FLOAT32_PROP(m_CD_StartSigma,0);
	ADD_CARD32_PROP_UPDATE_DESCR( m_InteractionMode,0, NULL, describeInteractionMode, 6 );
*/
//	ADD_NOARGS_METHOD_DESC( testScale, "test smoothing");
//	ADD_NOARGS_METHOD_DESC( testFeatureStability, "test feat. stability");
//	ADD_NOARGS_METHOD_DESC( buildScalePointClouds, "create scale space point clouds");
//	ADD_NOARGS_METHOD_DESC( calculateFeatures, "calc features");
//	ADD_NOARGS_METHOD_DESC( calculateFeatureSignatures, "extract signatures");
//	ADD_NOARGS_METHOD_DESC( removeBoringFeatures, "drop boring signatures");
//	ADD_NOARGS_METHOD_DESC( registerAllClouds, "global registration");
//	ADD_NOARGS_METHOD_DESC( calculateCentroidDescriptors, "calc centroiddescriptors");
//	ADD_NOARGS_METHOD_DESC( estimateMaxima, "estimateMaxima");
//	ADD_NOARGS_METHOD_DESC(resetTransformations,"resetTransformations");
//	ADD_NOARGS_METHOD_DESC(bench,"bench stability");
	ADD_NOARGS_METHOD(descriptorClustering);
//	ADD_NOARGS_METHOD(showFeatureLines);
//	ADD_NOARGS_METHOD(ransacSymmetryPlaneEstimation);
//	ADD_NOARGS_METHOD(distanceTransform);
	ADD_NOARGS_METHOD(tensorVoting);
	ADD_NOARGS_METHOD(initLinePoints);
	ADD_NOARGS_METHOD(stepLinePoints);
	ADD_NOARGS_METHOD(estimateLines);
//	ADD_NOARGS_METHOD(multiStepLinePoints);
	ADD_NOARGS_METHOD(drawDistributions);
	ADD_NOARGS_METHOD(ransacPlaneExtraction);
	ADD_NOARGS_METHOD(initCornerPoints);
	ADD_NOARGS_METHOD(stepCornerPoints);
	ADD_NOARGS_METHOD(goodOldMSFeatures);

}

PCIGlobalRegistration::PCIGlobalRegistration(void)
{
	m_TestMinDifferenceStrength = 0;
	m_Sigma = 10.0f;
	m_PatchSamplingSpace = 4.0f;
	m_SourceSamplingSpace = 1.0f;
	m_CurrentScale = 5;	

	m_ScreenWidth = 800;
	m_ScreenHeight = 600;

	m_ShowDroppedFeatures = false;
	m_ShowFeatureMode = 0;
	
	m_Scale_F = 2.0f;
	m_StartScale = 2.0f;
	m_NumberOfScales = 5;

	m_FS_NumTests = 1;;
	m_FS_NoiseAmplitude = 0.01f;
	m_FS_ResamplingPercentage = 1.0f;

	m_FeatureRadius = 10.0f;
	m_CurvatureSigma = 5.0f;
	m_ShowFeatureRadius = true;
	m_FDistanceDeviation = 1.0f;
	m_FDistance = 4.0;
	m_FMaxDistance = 15.0f;

	m_MaxNumInitialCorrespondences = 200;

	FDHistogram * fd1 = new FDHistogram;
	fd1->setAttribute( "curvature");
	fd1->setNumHistograms(2);
	fd1->setNumBins(16);
	fd1->setMinValue(0);
	fd1->setMaxValue(1);
	fd1->setAttributeCompNumber(0);

	FDHistogram * fd2 = new FDHistogram;
	fd2->setAttribute( "curvature");
	fd2->setNumHistograms(2);
	fd2->setNumBins(16);
	fd2->setMinValue(0);
	fd2->setMaxValue(1);
	fd2->setAttributeCompNumber(1);

	FDCombined * fd = new FDCombined;
	fd->addDesc( fd1 );
	fd->addDesc( fd2 );
	m_Descriptor = fd1;

	m_CD_WindowSize = 30.0f;
	m_CD_StartSigma = 0.5f;

	m_InteractionMode = IM_SelectManualCorrespondence;

	m_TVDefaultSettings = TVD_Dragon;
	tvSettingsChanged();

}

PCIGlobalRegistration::~PCIGlobalRegistration(void)
{
}

void PCIGlobalRegistration::assign(const Object* obj, CopyContext *context)
{
	PCInteractionTool::assign(obj,context);
	const PCIGlobalRegistration * o = dynamic_cast<const PCIGlobalRegistration*>(obj);
	pAssert( o != NULL );

}

static vector<UnstructuredInCorePointCloud*> mrsPC;
static float medianDistance = 0;
static float mgen_F = 0;

void PCIGlobalRegistration::testScale() 
{
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	while ( !it->atEnd() )
	{
		SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( it->getNode() );
		if( sgon )
		{
			PointCloud * pc = dynamic_cast<PointCloud*>( sgon->getSceneObject() );
			if( pc )
			{
				MultiScaleGenerator mgen;
				float patchSamplingSpace = 3.0f;
				float sourceSamplingSpace = 0;
				mgen_F = pow( 2.0f, 0.25f);
				mgen.setF( m_Scale_F);
				mgen.setH0( m_StartScale );
				mgen.setNumberOfScales( m_NumberOfScales );
				medianDistance = getMedianPointDistance(pc);
				mgen.createFast(pc);
				/*if( getPointCloud( getScene(), "test") != NULL)
					replacePointCloud( getScene(), "test", newPC );
				else
					addPointCloud( getScene(), newPC, "test" );*/
				break;				
			}
		}
		it->next();
	}
	delete it;
}

static vector<Vector3f> testpoints;

void PCIGlobalRegistration::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
	if( buttonsState.getRight() && m_InteractionMode == IM_SelectFeaturePoints )
	{
		m_FeatureCurvPoints.clear();
		m_FeatureCurvRadius.clear();
		debugRenderer->clearDebugData( "featureSelect");
	}

	if( buttonsState.getRight() && m_InteractionMode == IM_SelectManualCorrespondence )
		m_ManualCorrespondence.clear();

	if( buttonsState.getLeft() )
	{
		Vector3f point;
		string pcName;
		PointCloud * pc;

		if( getIntersectionFromCursor(x,y, point, pcName, &pc ) )
		{
			if( pc )
			{
				if( m_InteractionMode == IM_SelectManualCorrespondence )
				{
					m_ManualCorrespondence.push_back(point);
					return;
				}
				if( m_InteractionMode == IM_SelectFeaturePoints )
				{
					static Vector3f center = NULL_VECTOR3F;
					if( center == NULL_VECTOR3F)
					{
						center = point;
						return;
					}

					float tmpRadius = norm( point - center);
					debugRenderer->beginRenderJob("featureSelect");
					debugRenderer->addSphere( center, tmpRadius, makeVector3f(1,0,0));
					debugRenderer->endRenderJob();

					HierarchicalKNNIterator hIt(pc,32,NULL);
					hIt.setSeekPointAndReset(center);
					AAT curvAAT = pc->getAAT("curvature");
					AAT posAAT = pc->getAAT("position");
					Vector2f curv = NULL_VECTOR2F;
					int index = 0;
					while( !hIt.atEnd() )					
					{
						if( norm( center - hIt.get3f(posAAT)) > tmpRadius ) 
							break;
						curv += hIt.get2f(curvAAT);
						index++;
						hIt.next();
					}
					curv /= (float)index;
					hIt.setSeekPointAndReset(center);
					float radius = 0;
					while( index-- > 0 )
					{
						Vector2f tmp = hIt.get2f(curvAAT);
						radius = max(radius,norm(curv - tmp));
						hIt.next();
					}

					if( radius < 0.15f )
						radius = 0.15f;
					
					//m_FeatureCurvPoints.push_back( curv );
					//m_FeatureCurvRadius.push_back( radius );
					for( card32 i=0;i<1;i++ )
					{
						m_FeatureCurvPoints.push_back( curv );
						m_FeatureCurvRadius.push_back( radius );
					}
					center = NULL_VECTOR3F;
					return;
				}

				if( !pc->providesAttribute("curvature") )
					calculateCurvature( pc, 40, "curvature" );

				if( m_InteractionMode == IM_Registration ||
					m_InteractionMode == IM_DrawSignature )
				{

				if( m_ClickSignatures.size() >= 2 )
					m_ClickSignatures.clear();

				if( m_InteractionMode == IM_DrawSignature)
					if( m_ClickSignatures.size() >= 1 )
						m_ClickSignatures.clear();

				if( m_ClickSignatures.size() == 1)
					if( m_ClickSignatures[0].pc == pc)
						return;

				TClickRegisterTest tmp;
				tmp.pc = pc;
				tmp.pcName = pcName;

				CentroidDescriptor cd;
				cd.setWindowSize(m_CD_WindowSize);
				cd.setStartSigma(m_CD_StartSigma);
				cd.setNumScales(1);
				cd.setNumSamplesPerScale(10);
				cd.setBaseSigma(2.0f);
				cd.beginSignatureCalculations(pc);
				tmp.sig = cd.createSignature(point);
				card32 numSigmaTests = 20;
				/*debugSignatures.clear();

				for( card32 i=0;i<numSigmaTests;i++ )
				{
					cd.setStartSigma( pow(1.2f,-(float)i) );
					CentroidSignature * sig = cd.createSignature(point);
					debugSignatures.push_back(sig);
				}
				debugSignatures.back()->calculateTransformation( debugSignatures.back(),true);
				cd.endSignatureCalculations();
				debugSignatures.clear();
				return;*/
				
				m_ClickSignatures.push_back(tmp);
				cd.endSignatureCalculations();

				if( m_ClickSignatures.size() == 1 )
					debugRenderer->clearDebugData("test");
				debugRenderer->beginRenderJob("test");
				for( card32 i=0;i<tmp.sig->m_Centroids.size()-1;i++ )
				{
					debugRenderer->addLine(tmp.sig->m_Centroids[i],tmp.sig->m_Centroids[i+1]);
				}
				debugRenderer->endRenderJob();

				if( m_ClickSignatures.size() == 2)
				{
					// try to minimize distance
					TClickRegisterTest t1,t2;
					t1 = m_ClickSignatures[0];
					t2 = m_ClickSignatures[1];
					float bestDistance = t1.sig->calculateDistance( t2.sig );
					HierarchicalKNNIterator hIt( t2.pc,32,NULL );
					hIt.setSeekPointAndReset(point);
					hIt.next();
					AAT posAAT = t2.pc->getAAT("position");
					CentroidDescriptor cd;
					cd.setWindowSize( m_CD_WindowSize );
					cd.beginSignatureCalculations(t2.pc);
					card32 numIterations = 0;
					while( !hIt.atEnd())
					{
						Vector3f pos = hIt.get3f(posAAT);
						CentroidSignature * sig = cd.createSignature(pos);
						float tmpDist = sig->calculateDistance(t1.sig);
						if( tmpDist < bestDistance )
						{
							bestDistance = tmpDist;
							t2.sig = sig;
						}
						else
							delete sig;

						if( numIterations++ > 200 )
							break;
						hIt.next();
					}
					cd.endSignatureCalculations();

					Matrix4f mat2to1 = m_ClickSignatures[0].sig->calculateTransformation(m_ClickSignatures[1].sig,true);
					SGObjectNode * node = dynamic_cast<SGObjectNode*>( getSceneGraphNodeByName( getScene(), m_ClickSignatures[1].pcName ) );
					if( node )
					{
						node->getSceneObject()->setTransformation( mat2to1 );
					}

				}




				}

					if( m_InteractionMode == IM_AnalyseLocalNeighbourhood )
					{
						checkAttribute(pc,"test",4, VAD::DATA_FORMAT_FLOAT32);
						CentroidDescriptor cd;
						cd.setWindowSize(m_CD_WindowSize);
						cd.setStartSigma(m_CD_StartSigma);
						cd.setNumScales(1);
						cd.setNumSamplesPerScale(10);
						cd.setBaseSigma(2.0f);
						cd.beginSignatureCalculations(pc);

						CentroidSignature * sig = cd.createSignature(point);
						Vector4f sigStability = sig->calculateStabilityInformation();
						VertexArray var( pc );

						HierarchicalKNNIterator hIt( pc, 32, NULL);
						hIt.setSeekPointAndReset(point);
						hIt.next();

						AAT posAAT = pc->getAAT("position");
						AAT testAAT = pc->getAAT("test");
						card32 index = 0;
						while( index++ < 100 )
						{	
							Vector3f pos = hIt.get3f(posAAT);
							CentroidSignature * tmpSig = cd.createSignature(pos);

							Vector4f tmpStability = tmpSig->calculateStabilityInformation();

							card64 pointIndex = hIt.getCurrentPointIndex();

							Vector4f diff = sigStability - tmpStability;
							diff[2] = tmpSig->calculateDistance(sig);
							diff[3] = 1;
							var.set<float,4>(pointIndex, testAAT,diff);
							hIt.next();
							delete tmpSig;
						}

						while( index++ < 150 )
						{
							hIt.next();
							card64 pointIndex = hIt.getCurrentPointIndex();
							var.set<float,4>(pointIndex, testAAT,makeVector4f(0,0,0,0));
						}
						cd.endSignatureCalculations();
						delete sig;
						pc->clearAttachments();

					}

					if(m_InteractionMode == IM_DrawSimilarity)
					{
						checkAttribute(pc, "sigInfo",4,VAD::DATA_FORMAT_FLOAT32);

						SuperIterator iter( (PointCloudIterator*)pc->createIterator(0), pc->getDescr() );
						AAT posAAT = pc->getAAT("position");
						AAT sigInfoAAT = pc->getAAT("sigInfo");

						static vector<CentroidSignature*> signatures;
						if( signatures.size() != pc->getNumPoints() )
						{
							signatures = vector<CentroidSignature*>(pc->getNumPoints(),NULL);

							CentroidDescriptor cd;
							cd.setWindowSize( m_CD_WindowSize );
							cd.beginSignatureCalculations(pc);

							progressWindow->pushStep(true,"sig");
							card32 index = 0;
							while( !iter.atEnd() )
							{
								Vector3f pos = iter.get<Vector3f>(posAAT);
								CentroidSignature * sig = cd.createSignature( pos );
								delete signatures[index];
								signatures[index] = sig;

								iter.next();
								index++;
								if( index % 100 == 0)
									progressWindow->progress( (float)index / (float)pc->getNumPoints() * 100.0f );
							}
							progressWindow->popSteps();
							cd.endSignatureCalculations();
						}

						HierarchicalKNNIterator hIt(pc,32,NULL);
						hIt.setSeekPointAndReset(point);
						CentroidSignature * sig = signatures[hIt.getCurrentPointIndex()];
						iter.reset();
						card32 index = 0;
						while( !iter.atEnd() )
						{
							//float sigDist = sig->calculateDistance( signatures[index++] );
							Vector4f sigDist4f = sig->calculateDistanceVector( signatures[index++] );
							iter.set<Vector4f>( sigInfoAAT, sigDist4f );

							iter.flush();
							iter.next();
						}

						pc->clearAttachments();
					}

				/*
				if( !pc->providesAttribute("curvature") )
					calculateCurvature( pc, 40, "curvature" );

				CentroidDescriptor cd;
				cd.setWindowSize(m_CD_WindowSize);
				cd.beginSignatureCalculations(pc);
				CentroidSignature * sig = cd.createSignature( point );
				cd.endSignatureCalculations();

				debugRenderer->beginRenderJob("test");
				BoundingBox3f bb;
				float medianPointDistance = getMedianPointDistance(pc);
				bb.lowerCorner = point + makeVector3f(1,1,1) * -cd.getWindowSize() * medianPointDistance;
				bb.upperCorner = point + makeVector3f(1,1,1) * cd.getWindowSize() * medianPointDistance;
				debugRenderer->addBoundingBox( bb, makeVector3f(1,0,0) );
				for( card32 i=0;i<sig->m_Centroids.size()-1;i++ )
				{
					debugRenderer->addLine( sig->m_Centroids[i],sig->m_Centroids[i+1] );
				}
				debugRenderer->endRenderJob();

				/*float medianDistance = getMedianPointDistance( pc );
				HierarchicalKNNIterator hIt( pc, 32, NULL);

				card32 numPoints = 10;
				float sigma = 1.0f / (float)numPoints * 0.5f;

				float radius = medianDistance * 40.0f;
				float windowSigma = medianDistance * 2.0f;
				float maxRadius = radius + sqrt( -log(0.01f) * pow(windowSigma,2.0f) );

				float minWeight = 0.01f;
				testpoints = vector<Vector3f>( numPoints, point* minWeight);
				vector<float> weight(numPoints,minWeight);
				AAT posAAT = pc->getAAT( "position");
				AAT curvAAT = pc->getAAT( "curvature" );

				hIt.setSeekPointAndReset( point );

				while( !hIt.atEnd() )
				{
					Vector3f pos = hIt.get3f( posAAT );
					Vector2f curv = hIt.get2f( curvAAT );

					float dist = norm( point - pos);
					if( dist > maxRadius )
						break;

					pos -= point;

					float pointWeight = exp( -pow( max(0,dist - radius)  ,2.0f)/pow(windowSigma,2.0f) );

					for( card32 i=0;i<numPoints;i++ )
					{
						float center = ((float)i+0.5f) / (float)numPoints;
						float curvWeight = exp( -pow(fabs(curv[0] - center) / sigma ,2.0f) );
						testpoints[i] += pos * pointWeight * curvWeight;
						weight[i] += pointWeight * curvWeight;
					}

					hIt.next();
				}

				for( card32 i=0;i<numPoints;i++ )
				{
					if( weight[i] > 0 )
						testpoints[i] /= weight[i];

					testpoints[i] += point;
				}*/

								
			}
		}
	}

}
/*
bool PCIGlobalRegistration::getPointFromCursor( int x, int y, Vector3f &retPoint, string &retPC )
{
	PointCloudIterator* it = getClosestPointIteratorFromAllPoints(x, y, m_ScreenWidth, m_ScreenHeight, 5, 
		&getScene()->getRootState()->staticState->camera, 
		&getScene()->getRootState()->staticState->viewFrustum,
		getScene(),
		retPC
		);

	if( it == NULL )
		return false;

	BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(it);
	PointCloud * pc = getPointCloud( getScene(), retPC );

	pAssert( bIt != NULL );
	pAssert( pc != NULL );

	retPoint = bIt->get3f( pc->getAAT( "position") );
	delete it;
	return true;
}*/

void PCIGlobalRegistration::mouseMoved(int32 x, int32 y) 
{
}

void PCIGlobalRegistration::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
}

void PCIGlobalRegistration::areaResize(card32 width, card32 height) 
{
	m_ScreenWidth = width;
	m_ScreenHeight = height;
}

void PCIGlobalRegistration::keyDown(GeneralKey key) 
{
}

void PCIGlobalRegistration::keyUp(GeneralKey key) 
{
}

void PCIGlobalRegistration::glDrawTool(GLContext *glContext)
{
	glLineWidth( 3.0f );
	glBegin( GL_LINE_STRIP );
	for( card32 i=0;i<testpoints.size();i++ )
	{
		glColor3f( (float)i / (float)testpoints.size(),0,1.0f - (float)i / (float)testpoints.size() );
		glVertex3fv( testpoints[i].data() );
	}
	glEnd();

	static int narf = 0;
	if( debugSignatures.size() )
	{
		narf++;
		narf = narf % debugSignatures.size();
		glBegin( GL_LINE_STRIP );
		for( card32 i=0;i<debugSignatures[narf ]->m_Centroids.size();i++ )
		{
			glColor3f( 1,0,0 );
			glVertex3fv( debugSignatures[narf]->m_Centroids[i].data() );
		}
		glEnd();
	}

		
	vector<Vector3f> colorTable( 10 );
	colorTable[0] = makeVector3f(0.5,0,0 );
	colorTable[1] = makeVector3f(0.5,1,0 );
	colorTable[2] = makeVector3f(0.0,0,0.5 );
	colorTable[3] = makeVector3f(0,1,0.5 );
	colorTable[4] = makeVector3f(0,0.5,0 );
	colorTable[5] = makeVector3f(0,1,0 );
	colorTable[6] = makeVector3f(0.5,0.5,0 );
	colorTable[7] = makeVector3f(1,1,0 );
	colorTable[8] = makeVector3f(0,0.5,0.5 );
	colorTable[9] = makeVector3f(0,1,1 );

	glEnable( GL_DEPTH_TEST );
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	card32 pc_index = 0;
	while ( !it->atEnd() )
	{
		SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( it->getNode() );
		if( sgon )
		{
			PointCloud * pc = dynamic_cast<PointCloud*>( sgon->getSceneObject() );
			if( pc )
			{
				FeatureAttachment * featureAttachment = (FeatureAttachment*)pc->getAttachments()->getData( 
					FeatureAttachment::attachmentName() );
				if( featureAttachment )
				{
					for( card32 i=0;i<featureAttachment->getNumFeatures();i++ )
					{
						Vector3f color;
						FastFeature *f = featureAttachment->getFeature(i);
						color = colorTable[f->m_Level % colorTable.size() ];
						//if( m_TestMinDifferenceStrength <= f->m_Difference )
							drawFeature( f, color );
					}
					pc_index++;
				}
			}
		}
		it->next();
	}



	glEnable( GL_DEPTH_TEST );


	static card32 lastScale = 1000;
	if( lastScale != m_CurrentScale //&& m_CurrentScale < mrsPC.size())
		&& m_CurrentScale < MultiScaleGenerator::ffInfo.diffClouds.size() )
	{
		float patchSize = pow( mgen_F, (float)m_CurrentScale ) * medianDistance * m_Sigma;
		//PointCloud * newPC = createPatchMesh( mrsPC[m_CurrentScale], patchSize );
		PointCloud * newPC = (PointCloud *)MultiScaleGenerator::ffInfo.diffClouds[m_CurrentScale]->copy();
		newPC->setMaterialIndex(5);
		PointCloud * tmp = getPointCloud( getScene(), "root/view" );
		if( tmp )
		{
			tmp->assign( newPC);
			tmp->clearAttachments();
			delete newPC;
		}
		else
			addPointCloud( getScene(), newPC, "root/view");
		lastScale = m_CurrentScale;

	}
	
	/*StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	while ( !it->atEnd() )
	{
		SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( it->getNode() );
		if( sgon )
		{
			PointCloud * pc = dynamic_cast<PointCloud*>( sgon->getSceneObject() );
			if( pc && pc->providesAttribute( string("position") + intToStr(m_CurrentScale)))
			{
				static card32 lastScale = 1000;
				if( lastScale != m_CurrentScale)
				{
					UnstructuredInCorePointCloud * newPC = new UnstructuredInCorePointCloud;
					VertexDescriptor vd;
					vd.pushAttrib( VAD( "position", 3, VAD::DATA_FORMAT_FLOAT32));
					vd.pushAttrib( VAD( "normal", 3, VAD::DATA_FORMAT_FLOAT32));
					vd.pushAttrib( VAD( "color", 3, VAD::DATA_FORMAT_FLOAT32));
					vd.pushAttrib( VAD( "flags", 1, VAD::DATA_FORMAT_INT32));
					vd.pushAttrib( VAD( "diff", 1, VAD::DATA_FORMAT_FLOAT32));
					newPC->clearAndSetup( &vd, pc->getNumPoints());
					newPC->setMaterialIndex( pc->getMaterialIndex() );
					VertexArray var( newPC );
					VertexArray srcVar( pc );
					AAT posAAT = pc->getAAT( string("position") + intToStr(m_CurrentScale) );
					AAT normalAAT = pc->getAAT( string("normal") + intToStr(m_CurrentScale) );
					AAT diffAAT = newPC->getAAT( "diff");
					AAT pos2AAT;
					bool calcDifference = pc->providesAttribute(string("position") + intToStr(m_CurrentScale+1));
					if( calcDifference )
						pos2AAT = pc->getAAT( string("position") + intToStr(m_CurrentScale+1) );
					for( card32 i=0;i<srcVar.getNumElements();i++ )
					{
						var.setPosition3f(i, srcVar.get<float,3>(i, posAAT));
						var.setNormal3f(i, srcVar.get<float,3>(i, normalAAT));
						var.setColor3f(i,makeVector3f(1,1,1));
						var.setFlags(i, srcVar.getFlags(i) );
						if( calcDifference )
						{
							StaticVector<float,1> difference;
							difference[0] = srcVar.get<float,3>(i, normalAAT) * (srcVar.get<float,3>(i, posAAT) - srcVar.get<float,3>(i, pos2AAT));
							var.set(i, diffAAT,difference);
						}
					}
					PointCloud * tmp = getPointCloud( getScene(), "root/view" );
					if( tmp )
					{
						tmp->assign( newPC);
						tmp->clearAttachments();
						delete newPC;
					}
					else
						addPointCloud( getScene(), newPC, "root/view");
					lastScale = m_CurrentScale;
				}
			}
		}
		it->next();
	}
	delete it;*/
}


 /*- indizes holen 
 - beim holen gewichtete Zentroiden bestimmen
 - positionen kopieren in res-level-cache 
 - distanz zu mittelpunkt cachen 
 
 - 
 
 
 */

UnstructuredInCoreTriangleMesh *PCIGlobalRegistration::createPatchMesh( PointCloud * pc, float patchSize ) 
{
	UnstructuredInCoreTriangleMesh * mesh = new UnstructuredInCoreTriangleMesh;
	PointSet * ps = mesh->getPointSet();
	PointSet * triangles = mesh->getTriangles();

	int numRadialQuads = 16;
	int numRings = 7;

	int numVerticesPerPatch = 1 + numRadialQuads * (numRings - 1);
	int numTrianglesPerPatch = numRadialQuads + 2*numRadialQuads * (numRings - 1);
	
	VertexDescriptor vd;
	vd.pushAttrib( VAD("position",3,VAD::DATA_FORMAT_FLOAT32) );
	vd.pushAttrib( VAD("normal",3,VAD::DATA_FORMAT_FLOAT32) );
	vd.pushAttrib( VAD("flags",1,VAD::DATA_FORMAT_INT32) );

	ps->clearAndSetup(1, numVerticesPerPatch * pc->getNumPoints(), &vd);

	VertexDescriptor vd_tri;
	vd_tri.pushAttrib( VAD("index",3,VAD::DATA_FORMAT_INT32) );

	triangles->clearAndSetup(1, numTrianglesPerPatch * pc->getNumPoints(), &vd_tri);

	VertexArray points( ps );
	VertexArray tri( triangles );
	
	PointSet * srcps = ((UnstructuredInCorePointCloud*)pc)->getPointSet();
	card32 r_posOffset = pc->getDescr()->getAttributeOffset(pc->getDescr()->getIndex( "position" ));
	card32 r_localFrameOffset = pc->getDescr()->getAttributeOffset(pc->getDescr()->getIndex( "localFrame"));
	card32 r_coeffOffset = pc->getDescr()->getAttributeOffset(pc->getDescr()->getIndex(  "coeff" ));
	card32 r_scalePatchOffset = pc->getDescr()->getAttributeOffset(pc->getDescr()->getIndex(  "scalePatch" ));

	for(card32 i=0;i<pc->getNumPoints();i++ )
	{
		char * dataPtr = srcps->getDataPointer(i);
		Vector3f position = *((Vector3f*)(dataPtr + r_posOffset));
		Matrix3f paramToWorld = *((Matrix3f*)(dataPtr + r_localFrameOffset));
		StaticVector<float,6> coeffs = *((StaticVector<float,6>*)(dataPtr + r_coeffOffset));
		float scalePatch = *((float32*)(dataPtr + r_scalePatchOffset));

		vector<Vector3f> positions, normals;

		// vertices
		for( int a = 0;a<numRadialQuads;a++ )
		{
			float angle = (float)a / (float)numRadialQuads * M_PI * 2.0f;
			for( int b = 0;b<numRings;b++ )
			{
				if( a > 0 && b == 0 )
					continue;

				Vector2f paramPos = makeVector2f( sin(angle),cos(angle) ) * (float)b / (float)(numRings-1) * patchSize / scalePatch;
				float u = paramPos[0];
				float v = paramPos[1];
				Vector3f samplePos = makeVector3f(u,v, Monomials2ndOrder::evaluate(coeffs, paramPos)) * scalePatch;
//				Vector3f samplePos = makeVector3f(u,v, 0) * scalePatch;
				positions.push_back( paramToWorld * samplePos + position);

				float derivateU = 2.0f * coeffs[3] * u + coeffs[4] * v + coeffs[1];
				float derivateV = 2.0f * coeffs[5] * v + coeffs[4] * u + coeffs[2];
				Vector3f normal = -normalize( makeVector3f(1,0, derivateU).crossProduct( makeVector3f(0,1,derivateV) ) );
				normal = paramToWorld * normal;

				normals.push_back( -normal);

			}
		}

		for( card32 j=0;j<positions.size();j++ )
		{
			points.setPosition3f(i*numVerticesPerPatch + j, positions[j]);
			points.setNormal3f(i*numVerticesPerPatch + j, normals[j]);
		}

		if( numVerticesPerPatch != positions.size() )
			debugOutput << "num vertices: " << numVerticesPerPatch << "\n Num actual vertices: " << (mpcard) positions.size() << "\n";
		card32 startVertexIndex = i*numVerticesPerPatch;
		Vector3i start = makeVector3i(1,1,1) * startVertexIndex;
		card32 startTri = i*numTrianglesPerPatch;

		// triangles
		for( card32 j=0;j<(card32)numRadialQuads;j++ ) // inner ring
		{
			tri.setIndex3i(startTri + j, makeVector3i(0, j*(numRings-1)+1, ((j+1)%numRadialQuads)*(numRings-1)+1) + start );
			//tri.setIndex3i(startTri + j, start );

		}

		card32 currentTri = startTri + numRadialQuads;
		for( card32 j=1;j<(card32)numRings-1;j++)
		{
			for( card32 k=0;k<(card32)numRadialQuads;k++ )
			{
				card32 i1 = 1 + k * (numRings-1) + j-1;
				card32 i2 = 1 + ((k+1)%numRadialQuads) * (numRings-1) + j-1;
				card32 i3 = 1 + ((k+1)%numRadialQuads) * (numRings-1) + j;
				card32 i4 = 1 + k * (numRings-1) + j;
				tri.setIndex3i( currentTri++, makeVector3i(i1,i2,i3) + start );			
				tri.setIndex3i( currentTri++, makeVector3i(i1,i3,i4) + start );			
			}
		}

	}

	return mesh;
}

void PCIGlobalRegistration::testFeatureStability() 
{
	const vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
	PointCloud * pc = NULL;
	for( card32 i=0;i<selection.size();i++ )
	{
		pc = getPointCloud( getScene(), selection[i] );
		if( pc )
			break;
	}
	if( pc == NULL )
		return;

	float stdDeviation = getPCBBox( pc ).getMaxSideLength() * m_FS_NoiseAmplitude;
	UnstructuredInCorePointCloud * pcCopy = (UnstructuredInCorePointCloud*)createNoisyPointCloud( pc, stdDeviation, stdDeviation * 3.0f);

	MultiScaleGenerator mgen;
	mgen.setF( m_Scale_F);
	mgen.setH0( m_StartScale );
	mgen.setNumberOfScales( m_NumberOfScales );
	medianDistance = getMedianPointDistance(pc);
	mgen.createFast(pc,medianDistance);
	mgen.createFast(pcCopy,medianDistance);

	FeatureAttachment * fa1 = (FeatureAttachment*)pc->getAttachments()->getData( FeatureAttachment::attachmentName() );
	FeatureAttachment * fa2 = (FeatureAttachment*)pcCopy->getAttachments()->getData( FeatureAttachment::attachmentName() );

	map<int,int> statistics;
	map<int,int> numTotal;
	for( card32 i=0;i<fa1->getNumFeatures();i++ )
	{
		FastFeature * f1 = fa1->getFeature(i);
		if( !f1->m_Valid )
			continue;

		numTotal[f1->m_Level]++;

		for( card32 j=0;j<fa2->getNumFeatures();j++ )
		{
			FastFeature * f2 = fa2->getFeature(j);
			if( f1->m_Level != f2->m_Level )
				continue;

			if( !f1->m_Valid )
				continue;

			float dist = norm( f1->m_Pos - f2->m_Pos );
			if( dist < f1->m_Sigma * 0.5f )
			{
				statistics[ f1->m_Level ]++;
				break;
			}

		}
	}

	for( map<int,int>::iterator iter = statistics.begin(); iter != statistics.end(); ++iter )
	{
		debugOutput << "Level: " << iter->first << " num corr. " << iter->second << 
			" numTotal " << numTotal[iter->first] << "\n";
	}

	//delete pcCopy;
	addPointCloud( getScene(), pcCopy, "noise" );
}

void PCIGlobalRegistration::drawFeature( FastFeature * f, const Vector3f &color ) 
{
	card32 numSegments = 10;
	if(	!m_ShowDroppedFeatures && !f->m_Valid )
		return;
	if( m_ShowFeatureMode == 1 && !f->m_IsMin)
		return;
	if( m_ShowFeatureMode == 2 && f->m_IsMin)
		return;
	if( m_ShowFeatureRadius )
	{
		glLineWidth( 2.0f );
		glBegin( GL_LINE_STRIP );
		glColor3fv(color.data());
		for( card32 j=0;j<numSegments+1;j++ )
		{
			float angle = (float)(j%numSegments)/(float)numSegments * 2.0f * M_PI;
			Vector3f lpos = makeVector3f( sin(angle), cos(angle),0) * f->m_Radius;
			lpos = f->m_TangentFrame * lpos + f->m_Pos + f->m_Normal*f->m_Radius*0.5f;
			glVertex3fv( lpos.data() );

		}
		glEnd();
	}

	glPointSize( 3.0f );
	glBegin( GL_POINTS );
	Vector3f tmpPos = f->m_Pos + f->m_Normal*f->m_Radius*0.5f;
	glVertex3fv( tmpPos.data() );
	glEnd();

	/*glBegin( GL_LINES );
	Vector3f orgPos = f->m_Pos - f->m_Movement;
	glColor3f( 0.5f,0.5f,0.5f );
	glVertex3fv( orgPos.data() );
	glColor3f(1,1,0);
	glVertex3fv( f->m_Pos.data() );
	glEnd();*/

	glLineWidth( 1.0f );
}

void PCIGlobalRegistration::buildScalePointClouds() 
{
	const vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
	PointCloud * pc = NULL;
	for( card32 i=0;i<selection.size();i++ )
	{
		pc = getPointCloud( getScene(), selection[i] );
		if( pc )
			break;
	}
	if( pc == NULL )
		return;

	for( card32 i=0;i<MultiScaleGenerator::ffInfo.multiScaleClouds.size();i++ )
	{
		PointCloud *spc = MultiScaleGenerator::ffInfo.multiScaleClouds[i];
		card32 r_posOffset = spc->getDescr()->getAttributeOffset(spc->getDescr()->getIndex( "position" ));
		card32 r_localFrameOffset = spc->getDescr()->getAttributeOffset(spc->getDescr()->getIndex( "localFrame"));
		card32 r_coeffOffset = spc->getDescr()->getAttributeOffset(spc->getDescr()->getIndex(  "coeff" ));
		card32 r_scalePatchOffset = spc->getDescr()->getAttributeOffset(spc->getDescr()->getIndex(  "scalePatch" ));

		vector<char> vbuffer( spc->getDescr()->getSize() );
		Vector3f *posPtr = (Vector3f*)(&vbuffer[0] + r_posOffset);
		Matrix3f *framePtr = (Matrix3f*)(&vbuffer[0] + r_localFrameOffset);
		StaticVector<float,6> *coeffPtr = (StaticVector<float,6>*)(&vbuffer[0] + r_coeffOffset);
		float *scalePtr = (float*)(&vbuffer[0] + r_scalePatchOffset);

		UnstructuredInCorePointCloud * upc = (UnstructuredInCorePointCloud*)pc->copy();
		VertexArray va_ret( upc );
		HierarchicalKNNIterator hIt( spc, 32, NULL );
		for( card32 j=0;j<va_ret.getNumElements();j++ )
		{
			Vector3f pos = va_ret.getPosition3f(j);
			Vector3f normal = va_ret.getNormal3f(j);

			hIt.setSeekPointAndReset(pos);
			float summed_weight = 0;
			card32 numSamples = 0;
			Vector3f smoothPos = NULL_VECTOR3F;
			Vector3f smoothNormal = NULL_VECTOR3F;
			float inv_sigma_sqr_level = MultiScaleGenerator::ffInfo.info[i].inv_sigma_2;
			while( !hIt.atEnd() )
			{
				hIt.getVertex(&vbuffer[0]);

				float dist_2 = normQuad(*posPtr - pos);
				float weight = exp( -dist_2 * inv_sigma_sqr_level );

				Vector3f tmpPos, tmpNormal;
				MultiScaleGenerator::projectPointOntoSurface( *posPtr, *framePtr, framePtr->transpose(), *coeffPtr, *scalePtr, pos, tmpPos, tmpNormal);

				smoothPos += tmpPos * weight;
				smoothNormal += tmpNormal * weight;
				summed_weight += weight;


				if( weight < 0.01f && numSamples++ < 3)
					break;
				hIt.next();
			}

			smoothPos /= summed_weight;
			smoothNormal /= summed_weight;

			if( smoothNormal * normal < 0)
				smoothNormal = -smoothNormal;
			va_ret.setPosition3f(j,smoothPos);
			va_ret.setNormal3f(j, smoothNormal);

		}

		addPointCloud(getScene(), upc, "scale");

	}
}

void PCIGlobalRegistration::calculateFeatureSignatures() 
{
	const vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
	PointCloud * pc = NULL;
	float medianPointDistance;
	bool firstCloud = true;
	TDescriptorContext dc;
	dc.tIt = NULL;
	for( card32 i=0;i<selection.size();i++ )
	{
		pc = getPointCloud( getScene(), selection[i] );
		if( !pc )
			continue;
		FeatureAttachment * featureAtt = (FeatureAttachment *)pc->getAttachments()->getData( FeatureAttachment::attachmentName());
		if( !featureAtt )
			continue;
		if( firstCloud )
		{
			medianPointDistance = getMedianPointDistance(pc);
			firstCloud = false;
			dc.averagePointDistance = medianPointDistance;
		}
		dc.pc = pc;
		calculateCurvature( pc, m_CurvatureSigma * medianPointDistance, "curvature");
		featureAtt = (FeatureAttachment *)pc->getAttachments()->getData( FeatureAttachment::attachmentName());
		
		//pc->getAttachments()->deleteDelOnWriteData();

		for( card32 j=0;j<featureAtt->getNumFeatures();j++ )
		{
			FastFeature * f = featureAtt->getFeature(j);
			if( f->m_Valid )
			{
				FeatureDescriptor * fd = (FeatureDescriptor*)m_Descriptor->copy();
				fd->createDescriptor( f->m_Pos, medianPointDistance * m_FeatureRadius, pc, &dc );
				delete f->m_Descriptor;
				f->m_Descriptor = fd;
			}
		}

	}
}

void PCIGlobalRegistration::removeBoringFeatures() 
{
	registerTwoClouds();
}

vector<PCIGlobalRegistration::TCorrespondence> PCIGlobalRegistration::buildInitialSet( PointCloud * pc1, PointCloud * pc2) 
{
	FeatureAttachment * fa1 = (FeatureAttachment*)pc1->getAttachments()->getData( FeatureAttachment::attachmentName());
	FeatureAttachment * fa2 = (FeatureAttachment*)pc2->getAttachments()->getData( FeatureAttachment::attachmentName());
	return buildInitialSet(fa1,fa2);
}


vector<PCIGlobalRegistration::TCorrespondence> PCIGlobalRegistration::buildInitialSet( FeatureAttachment * fa1, FeatureAttachment * fa2 ) 
{
	vector<TCorrespondence> ret;
	for( card32 i=0;i<fa1->getNumFeatures();i++ )
	{
		FastFeature *f1 = fa1->getFeature(i);
		if( !f1->m_Valid )
			continue;
		vector<TCorrespondence> tmpCorr;
		for( card32 j=0;j<fa2->getNumFeatures();j++ )
		{
			FastFeature *f2 = fa2->getFeature(j);
			if( !f2->m_Valid )
				continue;
			TCorrespondence corr;
			if( f1->m_Level != f2->m_Level || f1->m_IsMin != f2->m_IsMin )
				continue;
			corr.f1 = f1;
			corr.f2 = f2;
			corr.distance = f1->m_Descriptor->compare( f2->m_Descriptor );
			corr.i1 = i;
			corr.i2 = j;
			tmpCorr.push_back( corr );
		}
		sort( tmpCorr.begin(),tmpCorr.end());

		float max_dist = tmpCorr[0].distance * 1.2f;
		vector<TCorrespondence> tmpResCor;
		for( card32 j=0;j<tmpCorr.size();j++ )
		{
			if( tmpCorr[j].distance > max_dist )
			{
				break;
			}
			tmpResCor.push_back( tmpCorr[j] );
		}

		if( tmpResCor.size() < 30 )
			for( card32 j=0; j<tmpResCor.size();j++ )
				ret.push_back(tmpResCor[j]);
	}
	sort(ret.begin(), ret.end());
	if( ret.size() > m_MaxNumInitialCorrespondences )
		ret.resize( m_MaxNumInitialCorrespondences );

	/*debugRenderer->beginRenderJob( "test");
	for( card32 i=0;i<ret.size();i++ )
		debugRenderer->addLine( ret[i].f1->m_Pos, ret[i].f2->m_Pos);
	debugRenderer->endRenderJob();*/

	return ret;
}

void PCIGlobalRegistration::registerTwoClouds() 
{
	const vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
	PointCloud * pc1 = NULL, *pc2 = NULL;
	float medianDistance = 0;
	string p2_name;
	for( card32 i=0;i<selection.size();i++ )
	{
		if( pc1 == NULL )
		{
			pc1 = getPointCloud( getScene(), selection[i] );
			if( pc1)
				medianDistance = getMedianPointDistance(pc1);
		}
		else
		{
			pc2 = getPointCloud( getScene(), selection[i] );
			if( pc2 )
			{
				p2_name = selection[i];
				break;
			}
		}
	}
	if( pc2 == NULL )
		return;

	vector<TCorrespondence> initialset = buildInitialSet(pc1,pc2);
	Matrix4f transf;
	validateInitialSet(initialset, medianDistance, transf);

	SGObjectNode * node = dynamic_cast<SGObjectNode*>( getSceneGraphNodeByName( getScene(), p2_name) );
	if( node )
	{
		node->getSceneObject()->setTransformation( transf );
	}


}

void PCIGlobalRegistration::calculateFeatures( PointCloud * pc, float medianDistance) 
{
	MultiScaleGenerator mgen;
	mgen.setF( m_Scale_F);
	mgen.setH0( m_StartScale );
	mgen.setNumberOfScales( m_NumberOfScales );
	mgen.createFast(pc, medianDistance);
}

void PCIGlobalRegistration::calculateFeatures() 
{
	float medianDistance;
	const vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
	PointCloud * pc = NULL;
	bool firstCloud = true;
	for( card32 i=0;i<selection.size();i++ )
	{
		pc = getPointCloud( getScene(), selection[i]);
		if( pc == NULL )
			continue;
		if( firstCloud )
		{
			medianDistance = getMedianPointDistance(pc);
			firstCloud = false;
		}
		calculateFeatures(pc, medianDistance);
	}
}

vector<PCIGlobalRegistration::TCorrespondence> PCIGlobalRegistration::validateInitialSet( const vector<TCorrespondence>& initialset, float medianPointDistance, Matrix4f &transf ) 
{
	SpectralCorrespondenceValidation validater;
	validater.setAddDiff( 2.0f * medianPointDistance );
	validater.setSigma( 2.0f * medianPointDistance);
	validater.setScaleDiff(0);
	validater.setMinConfidence(0);

	map<card32,card32> idToEntry1, idToEntry2;
	map<card32,card32> entryToID1,entryToID2;	
	vector<Vector3f> points1, points2;
	for( card32 i=0;i<initialset.size();i++ )
	{
		card32 index1 = initialset[i].i1;
		card32 index2 = initialset[i].i2;

		if( idToEntry1.find( index1 ) == idToEntry1.end() )
		{
			idToEntry1[index1] = (card32)points1.size();
			points1.push_back( initialset[i].f1->m_Pos );
		}
		if( idToEntry2.find( index2 ) == idToEntry2.end() )
		{
			idToEntry2[index2] = (card32)points2.size();
			points2.push_back( initialset[i].f2->m_Pos );
		}
	}

	validater.setup((card32)initialset.size(), (card32)points1.size(), (card32)points2.size() );

	for( card32 i=0;i<initialset.size(); )
	{
		TCorrespondence corr = initialset[i];
		validater.setCorrespondencePair(i, idToEntry1[corr.i1], idToEntry2[corr.i2], 
			corr.i1, corr.i2, 0.0f, corr.f1->m_Pos, corr.f2->m_Pos );
		i++;
	}

	for( card32 i=0;i<points1.size();i++ )
	{
		for( card32 j=i+1;j<points1.size();j++ )
		{
			validater.setDistanceEntry1( i,j, norm( points1[i]-points1[j] ), norm( points1[i]-points1[j] ));
		}
	}
	for( card32 i=0;i<points2.size();i++ )
	{
		for( card32 j=i+1;j<points2.size();j++ )
		{
			validater.setDistanceEntry2( i,j, norm( points2[i]-points2[j] ), norm( points2[i]-points2[j] ));
		}
	}

	validater.calculate();
	vector<SpectralCorrespondenceValidation::TFeaturePair> validPairs = validater.getValidPairs();
	transf = calculateTransformation( validPairs );
	/*debugRenderer->beginRenderJob("valid");
	for( card32 i=0;i<validPairs.size();i++ )
	{
		SpectralCorrespondenceValidation::TFeaturePair p = validPairs[i];
		debugRenderer->addLine (p.p1, p.p2 );
	}*/

	vector<TCorrespondence> ret;
	return ret;
}

Matrix4f PCIGlobalRegistration::calculateTransformation( const vector<SpectralCorrespondenceValidation::TFeaturePair> &pairs ) 
{
	// estimate centroids of features
	Vector3f centroidA = NULL_VECTOR3F, centroidB = NULL_VECTOR3F;
	for( card32 i=0;i<pairs.size();i++ )
	{
		centroidA += pairs[i].p1 / (float)pairs.size();
		centroidB += pairs[i].p2 / (float)pairs.size();
	}

	// calculate cross-covariance matrix
	Matrix3f C;
	C *= 0;
	for( card32 i=0;i<pairs.size();i++ )
	{
		for( card32 j=0;j<3;j++ )
		{
			for( card32 k=0;k<3;k++ )
			{
				C[j][k] += (pairs[i].p1[j] - centroidA[j])
					* (pairs[i].p2[k] - centroidB[k]);
			}
		}
	}

	// Prepare OpenCV Datasructure CvMat
	Matrix3f U,V;
	Vector3f eigenValues;
	CvMat cvMatInput, cvMat_U, cvMat_W, cvMat_Vt;
	cvInitMatHeader( &cvMatInput, 3, 3, CV_32FC1, C.data());
	cvInitMatHeader( &cvMat_W, 3, 1, CV_32FC1, eigenValues.data() );
	cvInitMatHeader( &cvMat_U, 3, 3, CV_32FC1, U.data() );
	cvInitMatHeader( &cvMat_Vt, 3, 3, CV_32FC1, V.data() );

	// Do SVD
	cvSVD(&cvMatInput, &cvMat_W, &cvMat_U, &cvMat_Vt, CV_SVD_MODIFY_A | CV_SVD_U_T );

	Matrix3f rotation = U*V;
	Matrix4f ret = makeTranslation4f(centroidA) * expand3To4(rotation) * makeTranslation4f(-centroidB);

	return ret;
}

struct TRegCloud
{
	bool operator < (const TRegCloud &o) const {return pc->getNumPoints() < o.pc->getNumPoints();}
	PointCloud * pc;
	string		name;
	bool aligned;
};

void PCIGlobalRegistration::registerAllClouds() 
{
	const vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
	vector<TRegCloud> clouds;
	float medianDistance = 0;
	for( card32 i=0;i<selection.size();i++ )
	{
		PointCloud * pc = getPointCloud( getScene(), selection[i] );
		if( pc )
		{
			if( clouds.size() == 0 )
				medianDistance = getMedianPointDistance(pc);
			TRegCloud cloud;
			cloud.name = selection[i];
			cloud.pc = pc;
			cloud.aligned = (clouds.size() == 0);
			clouds.push_back(cloud);
		}
				
	}

	if( clouds.size() < 2)
		return;

	FeatureAttachment * currentFeatures = (FeatureAttachment*)clouds[0].pc->getAttachments()->getData(FeatureAttachment::attachmentName())->copy();

	do 
	{
		bool allAligned = true;
		bool non_aligned = true;
	
		for( card32 i=1;i<clouds.size();i++ )
		{
			if( clouds[i].aligned )
				continue;

			FeatureAttachment * pc_fa = (FeatureAttachment*)clouds[i].pc->getAttachments()->getData(FeatureAttachment::attachmentName());
			vector<TCorrespondence> initialset = buildInitialSet(currentFeatures,pc_fa);
			Matrix4f transf;
			vector<TCorrespondence> validSet = validateInitialSet(initialset, medianDistance, transf);

			if( validSet.size() > 5 )
			{
				SGObjectNode * node = dynamic_cast<SGObjectNode*>( getSceneGraphNodeByName( getScene(), clouds[i].name) );
				if( node )
				{
					node->getSceneObject()->setTransformation( transf );
				}
				currentFeatures->addFeatureSet( pc_fa, transf );
				non_aligned = false;
			}
			else
				allAligned = false;
		}

		if( allAligned || non_aligned)
			break;

	} while(true);

}

void PCIGlobalRegistration::calculateCentroidDescriptors() 
{
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
		}
		if( pc )
			break;
		sceneIt->next();
	}

	delete sceneIt;

	if( pc == NULL )
		return;

	checkAttribute(pc, "sigInfo",4,VAD::DATA_FORMAT_FLOAT32);

	SuperIterator iter( (PointCloudIterator*)pc->createIterator(0), pc->getDescr() );
	CentroidDescriptor cd;
	cd.setWindowSize( m_CD_WindowSize );
	cd.beginSignatureCalculations(pc);
	AAT posAAT = pc->getAAT("position");
	AAT sigInfoAAT = pc->getAAT("sigInfo");

	while( !iter.atEnd() )
	{
		Vector3f pos = iter.get<Vector3f>(posAAT);
		CentroidSignature * sig = cd.createSignature( pos );

		Vector4f sigInfo = sig->calculateStabilityInformation();
		iter.set( sigInfoAAT, sigInfo );

		iter.flush();
		iter.next();
	}
	cd.endSignatureCalculations();
	pc->clearAttachments();
}

void PCIGlobalRegistration::estimateMaxima() 
{
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
		}
		if( pc )
			break;
		sceneIt->next();
	}

	delete sceneIt;

	if( pc == NULL )
		return;

	if( !pc->providesAttribute("sigInfo") )
		return;

	SuperIterator iter( (PointCloudIterator*)pc->createIterator(0), pc->getDescr() );
	AAT posAAT = pc->getAAT("position");
	AAT sigInfoAAT = pc->getAAT("sigInfo");
	HierarchicalKNNIterator hIt(pc,32,NULL);
	float maxRad = m_CD_WindowSize * getMedianPointDistance(pc) * 0.5f;
	debugRenderer->beginRenderJob("test");
	while( !iter.atEnd() )
	{
		Vector3f pos = iter.get<Vector3f>(posAAT);
		Vector4f sigInfo = iter.get<Vector4f>(sigInfoAAT);
		hIt.setSeekPointAndReset(pos);
		hIt.next();
		bool maximum = true;
		while( !hIt.atEnd() && maximum )
		{
			if( norm(pos - hIt.get3f(posAAT)) > maxRad )
				break;

			Vector4f tmp = hIt.get4f( sigInfoAAT );
			maximum = sigInfo[0] > tmp[0];
			
			hIt.next();
		}

		if( maximum )
			debugRenderer->addSphere( pos, maxRad * 0.25f, makeVector3f(0.5f,0,0));

		hIt.setSeekPointAndReset(pos);
		hIt.next();
		maximum = true;
		while( !hIt.atEnd() && maximum )
		{
			if( norm(pos - hIt.get3f(posAAT)) > maxRad )
				break;

			Vector4f tmp = hIt.get4f( sigInfoAAT );
			maximum = sigInfo[1] > tmp[1];

			hIt.next();
		}

		if( maximum )
			debugRenderer->addSphere( pos, maxRad * 0.25f, makeVector3f(0,0.5f,0));

		iter.next();
	}
	debugRenderer->endRenderJob();
	pc->clearAttachments();
}

void PCIGlobalRegistration::resetTransformations() 
{
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			objNode->getSceneObject()->setTransformation( IDENTITY4F );
		}
		sceneIt->next();
	}
}

void PCIGlobalRegistration::benchStability( PointCloud * pc, float attribute_noise, float feature_pos_noise, int cd_numScales, int cd_numSamples, float startSigma, 
														 float baseSigma, float cd_radius, int row, int column ) 
{
	// -----------------------------
	// create pc copy with noisy attribute
	// estimate a subset of points
	card32 numTestPoints = 400;
	srand(0);
	float prob = (float32)numTestPoints / (float)pc->getNumPoints();
	vector<Vector3f> samples;

	PointCloud * pc2 = (PointCloud*)pc->copy();
	SuperIterator sIter( pc2->createIterator(0), pc2->getDescr());
	AAT attrAAT = pc2->getAAT("curvature");
	AAT posAAT = pc2->getAAT("position");
	while( !sIter.atEnd() )
	{
		float attr = sIter.get<float>(attrAAT);
		attr += (rnd01() - 0.5f)*2.0f*attribute_noise;
		sIter.set(attrAAT,attr);

		if( rnd01() < prob )
			samples.push_back(sIter.get<Vector3f>(posAAT));

		sIter.flush();
		sIter.next();
	}

	CentroidDescriptor cd1,cd2;
	cd1.setWindowSize( cd_radius );
	cd2.setWindowSize( cd_radius );
	cd1.setNumScales(cd_numScales);
	cd2.setNumScales(cd_numScales);
	cd1.setNumSamplesPerScale(cd_numSamples);
	cd2.setNumSamplesPerScale(cd_numSamples);
	cd1.setStartSigma(startSigma);
	cd2.setStartSigma(startSigma);
	cd1.setBaseSigma(baseSigma);
	cd2.setBaseSigma(baseSigma);

	cd1.beginSignatureCalculations(pc);
	cd2.beginSignatureCalculations(pc2);

	for( card32 i=0;i<samples.size();i++ )
	{
		CentroidSignature * sig1 = cd1.createSignature(samples[i]);
		CentroidSignature * sig2 = cd2.createSignature(samples[i] + (makeVector3f(rnd01(),rnd01(),rnd01())*2 - makeVector3f(1,1,1))*feature_pos_noise );

		Matrix4f mat = sig1->calculateTransformation(sig2,true);
		Matrix3f mat_rot = shrink4To3(mat);

		float trans_error = norm( shrink4To3(mat[3]));
		float good_rot_angle = 2.5f;
		float cos_angle = cos( good_rot_angle / 180.0f * M_PI );
		float rot_fine = (mat[0][0] > cos_angle) && (mat[1][1] > cos_angle) && (mat[2][2] > cos_angle) ? 1 : 0;
	
		tableOutputTool->addData( "translation", row,column, trans_error / (float)samples.size() );
		tableOutputTool->addData( "rotation", row, column, rot_fine / (float)samples.size() );

		delete sig1;
		delete sig2;
	}

	cd1.endSignatureCalculations();
	cd2.endSignatureCalculations();

	delete pc2;
}

void PCIGlobalRegistration::bench() 
{
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	vector<float> startScale;
	startScale.push_back( 1.0f);
	vector<float> radi;
	radi.push_back( 20.0f );
	radi.push_back( 30.0f );
	radi.push_back( 40.0f );

	BoundingBox3f bb = getPCBBox(pc);
	
	for( card32 i=0;i<startScale.size();i++ )
	{
		for( card32 j=0;j<radi.size();j++ )
		{
			benchStability(pc, 0.05f, bb.getMaxSideLength() * 0.0f, 8, 8, startScale[i], 2.0f, radi[j], i,j);
		}
	}
	
	debugOutput << tableOutputTool->outputSimpleTable( "translation", TableOutputTools::DF_FloatPoint ) << "\n";
	debugOutput << tableOutputTool->outputSimpleTable( "rotation", TableOutputTools::DF_FloatPoint ) << "\n";
}

void PCIGlobalRegistration::descriptorClustering()
{
	debugRenderer->clearDebugData( "featureSelect");
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	checkAttribute(pc,"clusterID",1,VAD::DATA_FORMAT_INT32);

	// setup clusters
	vector< Vector3f > clusterColors(16);
	clusterColors[0] = makeVector3f(1,0,0);
	clusterColors[1] = makeVector3f(1,1,0);
	clusterColors[2] = makeVector3f(1,0,1);
	clusterColors[3] = makeVector3f(1,1,1);
	clusterColors[4] = makeVector3f(0,1,0);
	clusterColors[5] = makeVector3f(0,1,1);
	clusterColors[6] = makeVector3f(0,0,1);
	clusterColors[7] = makeVector3f(1,0.5,0);
	clusterColors[8] = makeVector3f(0,0.5,0);
	clusterColors[9] = makeVector3f(1,0.5,1);
	clusterColors[10] = makeVector3f(0,0.5,1);
	clusterColors[11] = makeVector3f(0.5,0.5,0.5);

	
	float innerRadius = 0.1f;
	float outerRadius = 0.05f;
	float completeRadius = innerRadius + outerRadius;
	float inv_sigma_2 = 1.0f / pow( outerRadius / 2.0f,2.0f );
	vector< Vector2f > clusterPoints;
	/*for( card32 i=0;i<3;i++ )
	{
		for( card32 j=0;j<3;j++ )
		{
			clusterPoints.push_back( makeVector2f(0.5f+i,0.5f+j) / 3.0f);
		}
	}*/

	clusterPoints = m_FeatureCurvPoints;

	SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC );
	pAssert( iter != NULL );
	BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);

	AAT colorAAT = pc->getAAT( "color" );
	AAT curvAAT = pc->getAAT( "curvature");
	AAT clusterAAT = pc->getAAT("clusterID");

	while( !bIt->atEnd() )
	{
		Vector2f curv = bIt->get2f(curvAAT);

		int clusterID = 0;

		for( card32 i=0;i<clusterPoints.size();i++)
		{
			float dist = norm(curv - clusterPoints[i]);
			if( dist < m_FeatureCurvRadius[i] )
			{
				clusterID |= (1 << i);
			}
		}
		
		mIt->set1i( clusterAAT, clusterID );
		

		bIt->next();
	}


	UnstructuredInCorePointCloud * upc = (UnstructuredInCorePointCloud*)pc;
	FixedKNNTopologyIterator topologyIter( upc->getPointSet(), upc->getAttachments(), upc, 8 );

	//InCorePCTopologyGraph * top = pgc->getPatchGraph(i)->getTopology();
	PointSet * ps = upc->getPointSet();
	AAT posAAT = ps->getAAT( "position" );
	AAT descAAT = ps->getAAT( "curvature" );
	AAT normalAAT = ps->getAAT( "normal");
	HierarchicalKNNIterator hIt( pc, 32, NULL );

	FeatureAttachment * featureAttachment = NULL;
	featureAttachment = dynamic_cast<FeatureAttachment*>( pc->getAttachments()->getData(FeatureAttachment::attachmentName()) );
	if( !featureAttachment )
	{
		featureAttachment = new FeatureAttachment;
		featureAttachment->setup(FeatureAttachment::attachmentName(), AttachedData::ADF_PERSISTENT );
		pc->getAttachments()->attachData( featureAttachment );
	}

	featureAttachment->clearFeatures();

	vector<card32> visited( ps->getNumEntries(), 0 );
	for( card32 j=0;j<ps->getNumEntries();j++ )
	{
		//if( visited[j] != 0 )
		//	continue;

		int32 clusterID = ps->get1i( j, clusterAAT );
		Vector2f curv = ps->get2f(j,curvAAT);
		Vector3f normal = ps->get3f(j,normalAAT);

		if( clusterID == 0)
			continue;

		for( card32 i=0;i<clusterPoints.size();i++ )
		{
			card32 clusterMask = 1 << i;
			if( (clusterID & clusterMask) == 0 )
				continue;

			completeRadius = m_FeatureCurvRadius[i];
			float sigma = m_FeatureCurvRadius[i] * 0.5f;
			inv_sigma_2 = 1.0f / pow( sigma,2.0f);


			// start clustering
			list<int> queue;
			vector<int> clusterIndices;
			vector<Vector3f> points;
			vector<float> weight;
			queue.push_back(j);
			visited[j] |= clusterMask;
			clusterIndices.push_back( j );
			points.push_back( ps->get3f(j, posAAT ) );
			weight.push_back(1.0f);
			//Vector3f mean_desc = ps->get3f(j, descAAT );

			while( !queue.empty() )
			{
				int node = queue.front();
				queue.pop_front();

				topologyIter.goToIndex( node );
				for( card32 n=0;n<topologyIter.getNumNeighbours();n++ )
				{
					int32 neighbour = topologyIter.getNeighbour(n);
					int tmpClusterID = ps->get1i(neighbour, clusterAAT );
					if( tmpClusterID != clusterID && tmpClusterID >= 0)
						continue;

					float32 dist = norm( ps->get2f(neighbour, curvAAT) - clusterPoints[i]);
					if( dist > completeRadius )
						continue;

					if( (visited[neighbour] & clusterMask) == 0 )
					{
						queue.push_back( neighbour );
						visited[neighbour] |= clusterMask;
						clusterIndices.push_back( neighbour );
						points.push_back( ps->get3f(neighbour, posAAT ) );

						//float w = exp( -pow(max(0.0f,dist-innerRadius),2.0f)*inv_sigma_2 );
						float w = exp( -pow(dist,2.0f)*inv_sigma_2 );
						weight.push_back( w );
						//mean_desc += ps->get3f(neighbour, descAAT );
					}

				}

			}

			float summed_weight = 0;
			Vector3f fpos = NULL_VECTOR3F;
			for( card32 k=0;k<points.size();k++)
			{
				fpos += points[k] * weight[k];
				summed_weight += weight[k];
			}

			if( summed_weight < 5.0f )
				continue;

			fpos /= summed_weight;

			//mean_desc /= clusterIndices.size();

			PCA<float,3> pca;

			for( card32 k=0;k<points.size();k++ )
			{
				pca.addPoint(points[k]);
			}

			Vector3f centroid, eigenValues;
			Matrix3f eigenVectors;
			pca.analyze(eigenValues, eigenVectors, centroid );

			FastFeature * f = new FastFeature;
			f->m_Pos = fpos;
			f->m_Radius = sqrt(eigenValues[0]);
			f->m_Valid = true;
			f->m_TangentFrame = eigenVectors;
			f->m_Normal = eigenVectors[2];
			if( normal * f->m_Normal < 0  )
				f->m_Normal = -f->m_Normal;
			f->m_Level = i;
			f->m_Weight = summed_weight;
			featureAttachment->addFeature(f);
		}

	}
		/*vector<DVectorF> points;
		for( card32 k=0;k<clusterPoints.size();k++ )
		{
		DVectorF vec;
		vec.setDim(3);
		for( card32 l=0;l<3;l++ )
		vec[l] = clusterPoints[k][l];
		points.push_back( vec );
		}
		DynamicPCA<float32> pca( &points[0], points.size() );
		DynamicVector<float32> mean = pca.calcMean();
		Vector3f centroid = makeVector3f( mean[0],mean[1],mean[2] );*/
		/*PCA<float,3> pca;
		for( card32 k=0;k<clusterPoints.size();k++ )
		{
			pca.addPoint( clusterPoints[k] );
		}
		Vector3f centroid, eigenValues;
		Matrix3f eigenVectors;
		pca.analyze( eigenValues, eigenVectors, centroid );

		//DynamicVector<float32> eigenValues = pca.calcEigenValues();
		vector<float> signature;
		signature.push_back( fabs( eigenValues[0] ) );
		signature.push_back( fabs( eigenValues[1] ) );
		signature.push_back( mean_desc[0] );
		signature.push_back( mean_desc[1] );
		signature.push_back( mean_desc[2] );
		signature.push_back( clusterID );
		//norm( eigenValues ), sqrt( absf( eigenValues[1] / max(eigenValues[2],1e-10f) )));
		FDAttribute * fa = new FDAttribute;
		fa->setNumComponents( signature.size() );
		fa->setupDescriptorData( &signature[0], signature.size() );

		/*/ /* check if centroid is near the surface
		hIt.setSeekPointAndReset( centroid );
		if( norm( centroid - hIt.get3f( posAAT ) ) >  averagePointDistance * 1.5f )
		continue;*/

		/*Feature * f = new Feature;
		f->setPosition( centroid );
		f->setPointID(-1);
		f->addDescriptor( fa );
		f->setSize( fabs( eigenValues[0] ) );
		vector<float> fdesc;
		fdesc.push_back( clusterID );
		card32 clusterColor[8];
		/*clusterColor[0] = 0xFF0000;
		clusterColor[1] = 0xFFFF00;
		clusterColor[2] = 0x00FFFF;
		clusterColor[3] = 0x0000FF;
		clusterColor[4] = 0xFF00FF;
		clusterColor[5] = 0xFFFFFF;
		clusterColor[6] = 0x000000;
		clusterColor[7] = 0x00FF00;*/

//	}	

}

struct TTmpLineGR
{
	FastFeature *f1,*f2;
	float dist;
};

void PCIGlobalRegistration::showFeatureLines()
{
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;


	debugRenderer->clearDebugData("test");
	debugRenderer->beginRenderJob("test");
	static vector<TTmpLineGR> lines;
	Vector3f colors[3];
	colors[0] = makeVector3f(1,0,0);
	colors[1] = makeVector3f(0,1,0);
	colors[2] = makeVector3f(0,0,1);
	static float lastFMaxDistance = 0;
	int imgSize = 128;
	RGBAFloat32Image his2d(imgSize,imgSize);
	his2d.clear( makeVector4f(0,0,0,0) );
	if( lines.size() == 0 || lastFMaxDistance != m_FMaxDistance)
	{
		lastFMaxDistance = m_FMaxDistance;
		lines.clear();
		FeatureAttachment * featureAttachment = NULL;
		featureAttachment = dynamic_cast<FeatureAttachment*>( pc->getAttachments()->getData(FeatureAttachment::attachmentName()) );
		if( featureAttachment == NULL )
			return;

		for( card32 i=0;i<featureAttachment->getNumFeatures();i++ )
		{
			FastFeature * f1 = featureAttachment->getFeature(i);
			for( card32 j=i+1;j<featureAttachment->getNumFeatures();j++ )
			{
				FastFeature * f2 = featureAttachment->getFeature(j);
				
				float dist = norm( f1->m_Pos - f2->m_Pos);
				if( dist > m_FMaxDistance )
					continue;

				TTmpLineGR l;
				if( f1->m_Level > f2->m_Level )
				{
					l.f1 = f2;
					l.f2 = f1;
				}
				else
				{
					l.f1 = f1;
					l.f2 = f2;
				}
				l.dist = dist;

				debugRenderer->addLine( l.f1->m_Pos, l.f2->m_Pos, colors[ l.f1->m_Level ], colors[ l.f2->m_Level ],2);
				lines.push_back(l);

				float tmpRad = l.f1->m_Radius;
				tmpRad /= m_FMaxDistance;
				tmpRad *= (float)(imgSize-1);
				tmpRad = min((float)imgSize-1.0f,tmpRad);
				int x = tmpRad;
				int y = min( imgSize-1, dist / m_FMaxDistance * ((float)imgSize-1.0f));
				Vector4f inc = makeVector4f( l.f1->m_Level == 0 && l.f2->m_Level == 0,
					l.f1->m_Level == 0 && l.f2->m_Level == 1,
					l.f1->m_Level == 1 && l.f2->m_Level == 1,0);
				his2d.setPixel(x,y, his2d.getPixel(x,y) + inc);

			}
		}
	}
	else
	{
		for( card32 i=0;i<lines.size();i++ )
		{
			if( fabs(lines[i].dist - m_FDistance) < m_FDistanceDeviation )
			{
				TTmpLineGR l = lines[i];
				debugRenderer->addLine( l.f1->m_Pos + l.f1->m_Normal*l.f1->m_Radius*0.5f , 
					l.f2->m_Pos+ l.f2->m_Normal*l.f2->m_Radius*0.5f, 
					colors[ l.f1->m_Level ], colors[ l.f2->m_Level ],2);
			}
		}
	}
	debugRenderer->endRenderJob();
	ImageDisplayer::DisplayRGBAFloat32Image ev(&his2d);
	debugOutput.displayEvent(ImageDisplayer::getClass(), "his2d", &ev);
}

struct TRansacPlane
{
	Plane plane;
	float score;
	
	int l1;
	int l2;

	bool operator < (const TRansacPlane& o) const {return score < o.score;}
};

void PCIGlobalRegistration::ransacSymmetryPlaneEstimation()
{
	debugRenderer->clearDebugData( "rplanes");
	debugRenderer->beginRenderJob("rplanes");
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	FeatureAttachment * featureAttachment = NULL;
	featureAttachment = dynamic_cast<FeatureAttachment*>( pc->getAttachments()->getData(FeatureAttachment::attachmentName()) );
	if( featureAttachment == NULL )
		return;

	// setup ANN and plane array
	float maxFeatureRadiusDifference = 0.25f;
	vector<ANNpoint> featurePoints;
	vector<TRansacPlane> planes;
	vector<TTmpLineGR> lines;
	vector< vector<TTmpLineGR> > flines(featureAttachment->getNumFeatures());
	for( card32 i=0;i<featureAttachment->getNumFeatures();i++ )
	{
		ANNpoint point = featureAttachment->getFeature(i)->m_Pos.data();
		featurePoints.push_back(point);
		vector<TTmpLineGR> tmpLines;

		if( m_ManualCorrespondence.size() == 0)
		for( card32 j=i+1;j<featureAttachment->getNumFeatures();j++ )
		{
			FastFeature * f1 = featureAttachment->getFeature(i);
			FastFeature * f2 = featureAttachment->getFeature(j);
			/*if( norm( f1->m_Pos - f2->m_Pos) > 5.0f )
			if( fabs(f1->m_Weight - f2->m_Weight) < 2.0f)
			if( f1->m_Level == f2->m_Level )
				//if( fabs(f1->m_Radius-f2->m_Radius) <= maxFeatureRadiusDifference )
				{
					TRansacPlane p;
					p.plane.point = (f1->m_Pos + f2->m_Pos) * 0.5f;
					p.plane.normal = normalize(f1->m_Pos - f2->m_Pos);
					p.score = 0;
					planes.push_back(p);
//					debugRenderer->addLine( f1->m_Pos, f2->m_Pos );
				}*/

			if( norm( f1->m_Pos - f2->m_Pos) < m_FMaxDistance && norm( f1->m_Pos - f2->m_Pos) > m_FDistanceDeviation )
			{
				TTmpLineGR l;
				l.f1 = f1;
				l.f2 = f2;
				l.dist = norm( f1->m_Pos - f2->m_Pos);
				lines.push_back(l);
				flines[i].push_back(l);
				flines[j].push_back(l);
			}
		}
	}

	UnstructuredInCorePointCloud * lpc = new UnstructuredInCorePointCloud;
	VertexDescriptor * vd = new VertexDescriptor;
	vd->pushAttrib( VAD("position",3,VAD::DATA_FORMAT_FLOAT32));
	vd->pushAttrib( VAD("color",3,VAD::DATA_FORMAT_FLOAT32));
	vd->pushAttrib( VAD("flags",1,VAD::DATA_FORMAT_INT32));
	lpc->clearAndSetup(vd,0);
	PointSet * ps = lpc->getPointSet();
	vector<char> vertexB(vd->getSize(),0);
	Vector3f *posPtr = (Vector3f*)&vertexB[0];
	Vector3f *colorPtr = (Vector3f*)&vertexB[sizeof(Vector3f)];

	map<card32,float> volume;
	int vdim = 16;
	float vdimf = vdim;

	for( card32 i=0;i<flines.size();i++ )
	{
		for( card32 j=0;j<flines[i].size();j++ )
		{
			for( card32 k=j+1;k<flines[i].size();k++ )
			{
				float d1 = flines[i][j].dist;
				float d2 = flines[i][k].dist;
				float d3 = 0;
				if( flines[i][j].f1 == flines[i][k].f1 )
					d3 = norm(flines[i][j].f2->m_Pos - flines[i][k].f2->m_Pos);
				if( flines[i][j].f2 == flines[i][k].f1 )
					d3 = norm(flines[i][j].f1->m_Pos - flines[i][k].f2->m_Pos);
				if( flines[i][j].f2 == flines[i][k].f2 )
					d3 = norm(flines[i][j].f1->m_Pos - flines[i][k].f1->m_Pos);
				if( flines[i][j].f1 == flines[i][k].f2 )
					d3 = norm(flines[i][j].f2->m_Pos - flines[i][k].f1->m_Pos);

				int i1 = min(vdimf,d1 / m_FMaxDistance * vdimf);
				int i2 = min(vdimf,d2 / m_FMaxDistance * vdimf);
				int i3 = min(vdimf,d3 / 2.0f / m_FMaxDistance * vdimf);

				volume[i1 + i2*vdim + i3*vdim*vdim] += 0.1f;
			}
		}
	}

	for( map<card32,float>::iterator iter = volume.begin();iter!=volume.end();++iter)
	{
		int i1 = iter->first % vdim;
		int i2 = (iter->first/vdim) % vdim;
		int i3 = (iter->first/vdim/vdim) % vdim;
		*posPtr = makeVector3f(i1,i2,i3) / vdim;
		*colorPtr = makeVector3f(1,1,1) * (iter->second);
		ps->addPoint( &vertexB[0]);
	}

	addPointCloud(getScene(), lpc, "test");

	return;
	
	AnnSearch ann;
	ann.buildSearchStructure( featureAttachment->getNumFeatures(), 3, &featurePoints[0] );

	if( m_ManualCorrespondence.size() != 0)
	{
		lines.clear();
		for( card32 i=0;i<m_ManualCorrespondence.size()/2;i++ )
		{
			TTmpLineGR l;
			for( card32 j=0;j<2;j++)
			{
				float dist;
				int index = ann.getNearestNeighborAndDistance(m_ManualCorrespondence[i*2+j].data(), dist);
				if( j==0)
					l.f1 = featureAttachment->getFeature(index);
				else
					l.f2 = featureAttachment->getFeature(index);
			}
			lines.push_back(l);
			debugRenderer->addLine( l.f1->m_Pos,l.f2->m_Pos, makeVector3f(0,0,1),makeVector3f(0,0,1),1);
		}
	}

	
	vector<TRansacPlane> testedPlanes;
	
	float mirrorSigma = m_FDistanceDeviation;
	float inv_mirrorSigma_2 = 1.0f / pow( mirrorSigma, 2.0f);

	debugOutput << "num lines: " << lines.size() << "\n";

	card32 numCanditates = 0;
	for( card32 i=0;i<5000000 && i<lines.size()*lines.size();i++ )
	{
		/*int i1 = rand() % lines.size();
		int i2;
		do 
		{
			i2 = rand() % lines.size();
		} while(i2 == i1);*/
		int i1 = i % (unsigned)lines.size();
		int i2 = i / (unsigned)lines.size();
		if( i1 == i2 )
			continue;
		TTmpLineGR l1 = lines[i1];
		TTmpLineGR l2 = lines[i2];

		if( fabs(l1.dist - l2.dist) > m_FDistanceDeviation )
			continue;

		Vector3f middle1 = (l1.f1->m_Pos + l1.f2->m_Pos) * 0.5f;
		Vector3f middle2 = (l2.f1->m_Pos + l2.f2->m_Pos) * 0.5f;

		TRansacPlane p;
		p.l1 = i1;
		p.l2 = i2;
		p.plane.point = (middle1 + middle2) * 0.5f;
		p.plane.normal = normalize(middle2 - middle1);
		Vector3f p11 = p.plane.calculateMirrorPoint( l1.f1->m_Pos );
		Vector3f p12 = p.plane.calculateMirrorPoint( l1.f2->m_Pos );
		if( norm(l2.f1->m_Pos - p11) + norm(l2.f2->m_Pos - p12) > m_FDistanceDeviation )
			continue;
		if( l1.f1->m_Level != l2.f1->m_Level || l1.f2->m_Level != l2.f2->m_Level)
			continue;
		//if( fabs( l1.f1->m_Weight - l2.f1->m_Weight ) > 15.0f || fabs( l1.f2->m_Weight - l2.f2->m_Weight ) > 15.0f )
		//	continue;
		if( norm(middle1 - middle2) < m_FMaxDistance )
			continue;

		//debugRenderer->addLine( l1.f1->m_Pos + l1.f1->m_Normal*l1.f1->m_Radius*0.5f, l2.f1->m_Pos
		//	+ l2.f1->m_Normal*l2.f1->m_Radius*0.5f);
	
		numCanditates++;
		//continue;
		if( numCanditates > 500 )
			break;

		//TRansacPlane p = planes[ rand() % planes.size() ];		
		p.score = 0;
		// test candidate
		vector<float> quaddistances(10);
		vector<int> indices(quaddistances.size());
		for( card32 j=0;j<featureAttachment->getNumFeatures();j++ )
		{
			FastFeature * f = featureAttachment->getFeature(j);
			Vector3f pos = f->m_Pos;
			Vector3f mirroredPoint = p.plane.calculateMirrorPoint(pos);

			ann.getkNearestNeighbors( mirroredPoint.data(), (card32)quaddistances.size(), &quaddistances[0], &indices[0] );
			float bestFeatureDist = 1e20f;

			for( card32 k=0;k<indices.size();k++ )
			{
				FastFeature * f2 = featureAttachment->getFeature(indices[k]);
				if( f2->m_Level == f->m_Level )
					if( fabs(f->m_Weight - f2->m_Weight) < 5.0f)
					//if(fabs(f->m_Radius-f2->m_Radius) <= maxFeatureRadiusDifference )
					{
						bestFeatureDist = quaddistances[k];
						break;
					}
			}

			p.score += exp( -bestFeatureDist * inv_mirrorSigma_2 );
		}
		testedPlanes.push_back(p);
	}

	sort( testedPlanes.begin(), testedPlanes.end());

	BoundingBox3f bb = getPCBBox(pc);
	float size = bb.getMaxSideLength() * 0.3f;
	int numPlanes = 3;
	if( (int)(testedPlanes.size()) > numPlanes )
		testedPlanes.erase(testedPlanes.begin(), testedPlanes.begin() + testedPlanes.size() -  numPlanes);
	for( card32 i=0;i<testedPlanes.size();i++ )
	{
		TRansacPlane p = testedPlanes[i];
		Matrix3f localFrame = calcTangentSystem(p.plane.normal);
		debugRenderer->addQuad( p.plane.point + localFrame[0]*size,
										p.plane.point + localFrame[1]*size,
										p.plane.point - localFrame[0]*size,
										p.plane.point - localFrame[1]*size, makeVector3f( 1,(float)(i) / (float)numPlanes,0) );
		debugRenderer->addLine( lines[p.l1].f1->m_Pos,lines[p.l1].f2->m_Pos, makeVector3f( 1,(float)(i) / (float)numPlanes,0),
			makeVector3f( 1,(float)(i) / (float)numPlanes,0),2);
		debugRenderer->addLine( lines[p.l2].f1->m_Pos,lines[p.l2].f2->m_Pos, makeVector3f( 1,(float)(i) / (float)numPlanes,0),
			makeVector3f( 1,(float)(i) / (float)numPlanes,0),2);
		debugRenderer->addLine( (lines[p.l2].f1->m_Pos + lines[p.l2].f2->m_Pos)*0.5f,
			(lines[p.l1].f1->m_Pos + lines[p.l1].f2->m_Pos)*0.5f,
			makeVector3f( 1,(float)(i) / (float)numPlanes,0),
			makeVector3f( 1,(float)(i) / (float)numPlanes,0),2);
		debugOutput << "score: " << p.score << "\n";
							

	}
	debugRenderer->endRenderJob();

}

void PCIGlobalRegistration::distanceTransform()
{
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	checkAttribute(pc,"test",4, VAD::DATA_FORMAT_FLOAT32);

	SceneObjectIterator * iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | SceneObjectIteratorType::CAP_MODIFYABLE_PC );
	pAssert( iter != NULL );
	BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(iter);

	HierarchicalKNNIterator * hIt = new HierarchicalKNNIterator(pc,32,NULL);

	AAT posAAT = pc->getAAT( "position" );
	AAT curvAAT = pc->getAAT( "curvature");
	AAT testAAT = pc->getAAT("test");

	Vector2f orgCurv = m_FeatureCurvPoints[0];
	float sigma = 0.2f;
	float maxDist = 2.0f * sigma;
	float inv_sigma_2 = 1.0f / sigma / sigma;
	float weightThreshold = 0.05f;

	float medPointDistance = getMedianPointDistance(pc);

	while( !bIt->atEnd())
	{
		Vector3f pos = bIt->get3f(posAAT);
		Vector4f curv = bIt->get4f(curvAAT);
		Vector4f test4f = makeVector4f(0,0,0,0);

		if( exp( - normQuad( makeVector2f(curv[0],curv[1]) - orgCurv ) * inv_sigma_2 ) > 0.8f )
		{
			test4f[1] = 1;
			float dist = 0;
			float summed_weight = 0;
			hIt->setSeekPointAndReset(pos);
			card32 numPoints = 0;
			Vector3f dir = NULL_VECTOR3F;
			float dirNorm = 0;
			bool dirFound = false;
			while( !hIt->atEnd() )
			{
				Vector3f tmpPos = hIt->get3f(posAAT);
				Vector4f tmpCurv = hIt->get4f(curvAAT);

				float tmpdist = norm( tmpPos - pos );
				if( dirFound )
					if( tmpdist > dirNorm + 1.0f* medPointDistance)
						break;

				float curvDist_2 = norm( makeVector2f(tmpCurv[0],tmpCurv[1]) - orgCurv );
				float weight = exp( -curvDist_2*inv_sigma_2 );
				if( weight < weightThreshold )
				{
					if( !dirFound )
					{
						dir = tmpPos - pos;
						dirFound = true;
						dirNorm = norm(dir);
					}
					else
					{
						if( dir * (tmpPos - pos) < 0.52f)
						{
							test4f[0] = 1.0f;
						}
					}
					/*weight = weightThreshold - weight;
					float pos_dist = norm( pos - tmpPos );
					dist = pos_dist;
					dist += pos_dist * weight;
					summed_weight += weight;*/
				}
			
				if( numPoints > 300 )
				{
					test4f[2] = 1;
					break;
				}
				
				/*if( summed_weight > 1.0f)
				{
					dist /= summed_weight;
					break;
				}*/

				hIt->next();
				numPoints++;
			}

			//test4f[0] = dist;
			mIt->set4f( testAAT, test4f);
		}
		else
		{
			mIt->set4f( testAAT, test4f);
		}

		bIt->next();
	}
	
	delete iter;
	delete hIt;
}

void PCIGlobalRegistration::tensorVoting()
{
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	checkAttribute( pc, "tensor", 9, VAD::DATA_FORMAT_FLOAT32 );
	checkAttribute( pc, "dir",3, VAD::DATA_FORMAT_FLOAT32);
	checkAttribute( pc, "edge", 3, VAD::DATA_FORMAT_FLOAT32);
	checkAttribute( pc, "eigen", 3, VAD::DATA_FORMAT_FLOAT32);

	SuperIterator iter( pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | SceneObjectIteratorType::CAP_MODIFYABLE_PC ), pc->getDescr()  );

	AAT posAAT = pc->getAAT( "position" );
	AAT curvAAT = pc->getAAT( "curvature");
	AAT colorAAT = pc->getAAT("color");
	AAT tensorAAT = pc->getAAT("tensor");
	AAT dirAAT = pc->getAAT("dir");
	AAT edgeAAT = pc->getAAT("edge");
	AAT normalAAT = pc->getAAT("normal");
	AAT eigenAAT = pc->getAAT("eigen");

	float medPointDistance = getMedianPointDistance(pc);
	float cacheRadius = 2.7f * medPointDistance;
	float sigma = m_TVSigma * medPointDistance;
	float radius = 2.0f * sigma;
	float inv_sigma_2 = 1.0f / sigma / sigma;
	float radius_2 = radius * radius;
	float minDistanceQuad = pow( 0.1f * medPointDistance, 2.0f);

	float curvSigma = m_TVCurvSigma;
	float inv_curvSigma_2 = 1.0f / curvSigma / curvSigma;

	CachedRangeQuerry querry(pc);
	debugRenderer->clearDebugData("tensorVoting");
	debugRenderer->beginRenderJob("tensorVoting");

	// ------------------------------------------
	// step 1: compute initial tensors
	progressWindow->pushStep(true, "tv");
	card32 index = 0;
	while( !iter.atEnd() )
	{
		Vector3f pos = iter.get<Vector3f>(posAAT);
		Vector4f originCurv = iter.get<Vector4f>(curvAAT);
		float originMaxCurv = max(0.0f,1.0f - max(originCurv[0],-originCurv[1]));
		float origin_curv_weight = exp( -originMaxCurv*originMaxCurv*inv_curvSigma_2 );


		PointSet * ps = querry.getSpherePoints( pos, radius, cacheRadius );
		const card32 numNeighbours = ps->getNumEntries();
		Matrix3f tensor;
		tensor *= 0;
		float summedWeight = 0;
		for( card32 i=0;i<numNeighbours;i++ )
		{
			Vector3f tmpPos = ps->get3f(i,posAAT);
			Vector3f diffVec = tmpPos - pos;
			float distQuad = normQuad(diffVec);
			if( distQuad > radius_2 || distQuad < minDistanceQuad )
				continue;


			Vector4f curv = ps->get4f(i,curvAAT);
			float maxCurv = max(0.0f,1.0f - max(curv[0],-curv[1]));
			float curv_weight = exp( -maxCurv*maxCurv*inv_curvSigma_2 );
			
			float weight = exp( -distQuad * inv_sigma_2 ) * curv_weight * origin_curv_weight;

			diffVec = normalize( diffVec);
			tensor += (outerProduct( diffVec, diffVec )) * weight;
			summedWeight += exp( -distQuad * inv_sigma_2 );
		}

		if( summedWeight > 0.0f )
			tensor /= summedWeight;

		Matrix3f eigenVectors;
		Vector3f eigenValues;
		tensor.computeEigenStructure( eigenValues, eigenVectors );

		iter.set( colorAAT, eigenValues);
		iter.set( tensorAAT,tensor);
		iter.set( dirAAT, eigenVectors[0]);
		iter.flush();


		/*if( eigenValues[0] * 0.8 < eigenValues[1])
			debugRenderer->addEllipsoid( pos, eigenVectors, makeVector3f(eigenValues[0],eigenValues[1],eigenValues[2]) * medPointDistance * 5.0f,
			makeVector3f(1.0f,0.4f,0.2f));*/
		//if( eigenValues[0] * 0.8f > eigenValues[1] )
		//	debugRenderer->addLine( pos - eigenVectors[0] * eigenValues[0] * medPointDistance * 3.0f,pos + eigenVectors[0] * eigenValues[0] * medPointDistance * 3.0f );
		
		if( index % 20 == 0 )
			progressWindow->progress((float)index/(float)pc->getNumPoints()*100.0f);
		index++;
		iter.next();
	}
	progressWindow->popSteps();

	bool debug1 = false;
	srand(QDateTime::currentDateTime().toTime_t());

	// --------------------------------------------
	// Step 2: Voting
	iter.reset();
	index = 0;
	progressWindow->pushStep(true, "tv");
	while( !iter.atEnd())
	{
		Vector3f pos = iter.get<Vector3f>(posAAT);

		// caluclate tensor normal
		Matrix3f myTensor = iter.get<Matrix3f>(tensorAAT);
		Matrix3f eigenVectors;
		Vector3f eigenValues;
		myTensor.computeEigenStructure( eigenValues, eigenVectors );
		Vector3f myTensorNormal = -eigenVectors[0];
		Vector3f origindir = iter.get<Vector3f>(dirAAT);
		Vector3f inv_eigen;
		for( int i=0;i<3;i++ )
			if( eigenValues[i] > 1e-20f )
				inv_eigen[i] = 1.0f / eigenValues[i];
			else
				inv_eigen[i] = 0;
		Matrix3f anisotropicNormalDist = eigenVectors * makeScale3f(inv_eigen);




		PointSet * ps = querry.getSpherePoints( pos, radius, cacheRadius );
		const card32 numNeighbours = ps->getNumEntries();
		Matrix3f tensor;
		tensor *= 0;

		float summed_weight = 0;
		for( card32 i=0;i<numNeighbours;i++ )
		{
			Vector3f tmpPos = ps->get3f(i,posAAT);
			Vector3f diffVec = tmpPos - pos;
			float distQuad = normQuad(diffVec);
			if( distQuad > radius_2 )
				continue;

			if( pos == tmpPos )
			{
				StaticVector<float,9> nTensorVec = ps->get<float,9>(i,tensorAAT);
				Matrix3f nTensor;
				memcpy( nTensor.data(), nTensorVec.data(), 9*sizeof(float) );
				tensor += nTensor;
				continue;
			}

			Vector3f transformedDiff = anisotropicNormalDist * diffVec;
			float distQuadAnisotropic = normQuad(transformedDiff);



			Vector3f dir = ps->get3f(i,dirAAT);

			StaticVector<float,9> nTensorVec = ps->get<float,9>(i,tensorAAT);
			Matrix3f nTensor;
			memcpy( nTensor.data(), nTensorVec.data(), 9*sizeof(float) );

			// compute weight
			float dirAngle = fabs(origindir * dir);
			float maxAngle_2 = pow( (float)cos(m_TVMaxAngle/180.0f*M_PI),2.0f);
			float weight = exp( -distQuad ) * exp( -pow(1.0f - dirAngle,2.0f)/maxAngle_2);
			//summed_weight += exp( -(1.0f - dirAngle)*2.0f);
			//debugOutput << "weight: " << weight << "\n";

//			if( rand() % 50 == 0  )
//				debugOutput << "Tensor: ---------------\n" << nTensor[0] << "\n" << nTensor[1] << "\n" << nTensor[2] << "\n" << "--------\n";
			
			tensor += nTensor * weight;
		}

		//if( summed_weight > 0)
		//	tensor /= summed_weight;
		tensor.computeEigenStructure( eigenValues, eigenVectors );
//		debugOutput << "Tensor: ---------------\n" << tensor[0] << "\n" << tensor[1] << "\n" << tensor[2] << "\n" << "--------\n";


		//debugRenderer->addEllipsoid( pos + makeVector3f(0.5f,0,0)*medPointDistance, eigenVectors, makeVector3f(eigenValues[0],eigenValues[1],eigenValues[2]) * medPointDistance * 0.25f,
		//	makeVector3f(0.4f,0.4f,1.0f));
		float length = 0;
		if( eigenValues[1] > 1e-20f )
			length = eigenValues[0]/eigenValues[1] - 1.0f;
		//if( eigenValues[0] > 0.001f)
			//if( eigenValues[0] * 0.9f > eigenValues[1] )
		Vector3f col = makeVector3f(length*3.0f,eigenValues[0],0);
		if( length > 0.01f)
				debugRenderer->addLine( pos - eigenVectors[0] * length * medPointDistance*3.0f,pos + eigenVectors[0] * length * medPointDistance*3.0f,
				col, col, 1.0f);
				//debugRenderer->addEllipsoid( pos, eigenVectors, makeVector3f(eigenValues[0],eigenValues[1],eigenValues[2]) * medPointDistance * 0.07f,
				//	makeVector3f(1.0f,0.4f,0.2f));

		iter.set( edgeAAT, eigenVectors[0] * length);
		iter.set( colorAAT, makeVector3f(summed_weight,length,eigenValues[0]*eigenValues[1]*eigenValues[2]));
		iter.set( eigenAAT, eigenValues);
		//iter.set( tensorAAT,tensor);
		iter.flush();


		if( index % 20 == 0 )
			progressWindow->progress((float)index/(float)pc->getNumPoints()*100.0f);
		index++;
		iter.next();
	}
	progressWindow->popSteps();

	/*progressWindow->pushStep(true, "tv");
	index = 0;
	iter.reset();
	while( !iter.atEnd() )
	{
		Vector3f pos = iter.get<Vector3f>(posAAT);
		//Vector3f originDir = iter.get<Vector3f>(dirAAT);
		Vector3f normal = iter.get<Vector3f>(normalAAT);
		
		Matrix3f localCoordSystem = calcTangentSystem(normal).transpose();

		PointSet * ps = querry.getSpherePoints( pos, radius, cacheRadius );
		const card32 numNeighbours = ps->getNumEntries();
		float summedWeight = 0;
		vector<float> histogram(18,0);
		for( card32 i=0;i<numNeighbours;i++ )
		{
			Vector3f tmpPos = ps->get3f(i,posAAT);
			Vector3f diffVec = tmpPos - pos;
			float distQuad = normQuad(diffVec);
			if( distQuad > radius_2 || distQuad < minDistanceQuad )
				continue;

			Vector3f edge = ps->get3f(i,edgeAAT);
			float length = norm(edge) * exp(-distQuad * inv_sigma_2);

			Vector3f localEdge = localCoordSystem * edge;
			float angle = atan2( localEdge[0],localEdge[1] ) / M_PI * (float)histogram.size();
			if( angle < 0)
				angle += M_PI;
			float fracAngle = angle - (int)angle;
			histogram[(int)angle % histogram.size()] += (1.0f -fracAngle)*length;
			histogram[((int)angle+1)%histogram.size()] += fracAngle*length;
		}

		unsigned maxEntry = 0;
		for( unsigned i=0;i<histogram.size();i++ )
		{
			if( histogram[i] > histogram[maxEntry])
				maxEntry = i;
		}

		histogram[(maxEntry+1)%histogram.size()] = 0;
		histogram[(maxEntry+histogram.size()+1)%histogram.size()] = 0;
		unsigned maxEntry2 = (maxEntry + histogram.size() / 2)%histogram.size();
		for( unsigned i=0;i<histogram.size();i++ )
		{
			if( histogram[i] > histogram[maxEntry2] && maxEntry2 != maxEntry)
				maxEntry2 = i;
		}

		Vector3f col = makeVector3f( histogram[maxEntry2] / histogram[maxEntry], histogram[maxEntry], histogram[maxEntry2]  );
		iter.set( colorAAT, col);
		iter.flush();

		if( index % 20 == 0 )
			progressWindow->progress((float)index/(float)pc->getNumPoints()*100.0f);
		index++;
		iter.next();
	}
	progressWindow->popSteps(aborted);*/



	return;
	
	/*progressWindow->pushStep(true, "tv");
	index = 0;
	iter.reset();
	while( !iter.atEnd() )
	{
		Vector3f pos = iter.get<Vector3f>(posAAT);
		//Vector3f originDir = iter.get<Vector3f>(dirAAT);


		PointSet * ps = querry.getSpherePoints( pos, radius, cacheRadius );
		const card32 numNeighbours = ps->getNumEntries();
		Vector3f dir = makeVector3f(0,0,0);
		Vector3f edge = iter.get<Vector3f>(edgeAAT);
		float summedWeight = 0;
		for( card32 i=0;i<numNeighbours;i++ )
		{
			Vector3f tmpPos = ps->get3f(i,posAAT);
			Vector3f diffVec = tmpPos - pos;
			float distQuad = normQuad(diffVec);
			if( distQuad > radius_2 || distQuad < minDistanceQuad )
				continue;

			Vector3f edge2 = ps->get3f(i,edgeAAT);
			float cos_angle_2 = pow(1.0f - edge2 * edge,2.0f);
			float inv_maxAngle_2 = 1.0f / pow((float)cos(10.0f/180.0f*M_PI),2.0f);

			float weight = exp( -distQuad * inv_sigma_2 ) * exp( -cos_angle_2 * inv_maxAngle_2 ) ;

			dir += ps->get3f(i,edgeAAT)*weight;
			summedWeight += exp( -distQuad * inv_sigma_2 );
		}

		float length = iter.get<Vector3f>(colorAAT)[1];

		//iter.set( );
		iter.set<Vector3f>(colorAAT, makeVector3f( norm(dir),0,0 ));
		iter.flush();

		Vector3f dirn = normalize(dir);


		//if( eigenValues[0] > 0.01f)
		//debugRenderer->addEllipsoid( pos, eigenVectors, makeVector3f(eigenValues[0],eigenValues[1],eigenValues[2]) * medPointDistance * 5.0f,
		//	makeVector3f(1.0f,0.4f,0.2f));
		//if( eigenValues[0] * 0.8f > eigenValues[1] )
		//if( length > 0.2f )
		Vector3f col = makeVector3f(1,1,0)*min(1.0f,length*4.0f);
			debugRenderer->addLine( pos - dirn * medPointDistance ,pos + dirn * medPointDistance, col, col, 2 );

		if( index % 20 == 0 )
			progressWindow->progress((float)index/(float)pc->getNumPoints()*100.0f);
		index++;
		iter.next();
	}
	progressWindow->popSteps(aborted);



	debugRenderer->endRenderJob();

	//delete iter;*/
}

void PCIGlobalRegistration::drawDistributions()
{
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	SuperIterator iter( pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | SceneObjectIteratorType::CAP_MODIFYABLE_PC ), pc->getDescr()  );
	HierarchicalKNNIterator hIt(pc,32,NULL);

	AAT posAAT = pc->getAAT( "position" );

	float medPointDistance = getMedianPointDistance(pc);
	float cacheRadius = 2.7f * medPointDistance;

	float sigma = 5.0f * medPointDistance;
	float inv_sigma_2 = 1.0f / sigma / sigma;
	float maxrad_2 = 4.0f * sigma * sigma;

	debugRenderer->clearDebugData("test");
	debugRenderer->beginRenderJob("test");
	while( !iter.atEnd())
	{
		if( rnd01() < 0.01f )
		{
			Matrix3f C;
			vector<Vector3f> positions;

			Vector3f pos = iter.get<Vector3f>(posAAT);
			hIt.setSeekPointAndReset(pos);
			hIt.next();
			Vector3f centroid = pos;
			float summedWeight = 1.0f;

			while(!hIt.atEnd())
			{
				Vector3f tmpPos = hIt.get3f(posAAT);
				float distQuad = normQuad(tmpPos - pos);
				if( distQuad > maxrad_2)
					break;

				float weight = exp(-distQuad * inv_sigma_2);
				centroid += tmpPos * weight;
				summedWeight += weight;
				positions.push_back(tmpPos);

				hIt.next();
			}
			centroid /= summedWeight;

			for( unsigned i=0;i<positions.size();i++ )
			{
				C += outerProduct(positions[i] - centroid,positions[i] - centroid);
			}
			C /= positions.size();
			Vector3f eigenValues;
			Matrix3f eigenVectors;
			C.computeEigenStructure(eigenValues,eigenVectors);
			Vector3f scale = makeVector3f( sqrt(eigenValues[0]),sqrt(eigenValues[1]),sqrt(eigenValues[2]) );
			Vector3f color = eigenValues[1] * 0.2f > eigenValues[2] ? makeVector3f(0,0.3f,0.9f) : makeVector3f(1,0.5f,0);
			debugRenderer->addFastEllipse(centroid,eigenVectors,scale,color);
			//debugRenderer->addSphere(centroid,medPointDistance*3.0f,makeVector3f(1,0.5f,0));

		}
		iter.next();
	}
	debugRenderer->endRenderJob();

}

void PCIGlobalRegistration::initLinePoints()
{
	m_LinePoints.clear();

	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	debugRenderer->clearDebugData("tensorVoting");
	debugRenderer->clearDebugData("linePoints");
	debugRenderer->beginRenderJob("linePoints");

	SuperIterator iter( pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | SceneObjectIteratorType::CAP_MODIFYABLE_PC ), pc->getDescr()  );
	HierarchicalKNNIterator hIt(pc,32,NULL);

	AAT posAAT = pc->getAAT( "position" );
	AAT edgeAAT = pc->getAAT("edge");
	float medPointDistance = getMedianPointDistance(pc);
	
	// fetch points with high line length
	vector<unsigned> linePointSet;
	card32 index = 0;
	while(!iter.atEnd())
	{
		Vector3f edge = iter.get<Vector3f>(edgeAAT);
		float length = norm(edge);

		if( length > m_TVLP_MinLineLength )
		{
			linePointSet.push_back(index);
			debugRenderer->addPoint(iter.get<Vector3f>(posAAT),makeVector3f(0,0.5f,1));
		}

		iter.next();
		index++;
	}

	// select random subset
	card32 numSeedPoints = linePointSet.size() * m_TVLP_SamplesPercent;
	for( unsigned i=0;i<numSeedPoints;i++ )
	{
		card32 lIndex = rand() % linePointSet.size();
		card32 index = linePointSet[lIndex];
		iter.goToIndex(index);

		TLinePoint lp;
		lp.pos = iter.get<Vector3f>(posAAT);
		Vector3f lineDir = iter.get<Vector3f>(edgeAAT);
		lp.lineLength = norm(lineDir);
		lp.lineDir = lineDir / lp.lineLength;
		lp.movement = 1e20f;
		m_LinePoints.push_back(lp);
		linePointSet.erase(linePointSet.begin() + lIndex);

		Vector3f col = makeVector3f(1,0.5f,0);
		debugRenderer->addLine(lp.pos - lp.lineDir * medPointDistance,lp.pos + lp.lineDir * medPointDistance,col,col,2);
		//debugRenderer->addSphere(lp.pos, medPointDistance * 0.5f, col);
	}

	debugRenderer->endRenderJob();
}

void PCIGlobalRegistration::stepLinePoints()
{
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	debugRenderer->clearDebugData("linePoints");
	debugRenderer->beginRenderJob("linePoints");

	SuperIterator iter( pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | SceneObjectIteratorType::CAP_MODIFYABLE_PC ), pc->getDescr()  );
	HierarchicalKNNIterator hIt(pc,32,NULL);

	AAT posAAT = pc->getAAT( "position" );
	AAT edgeAAT = pc->getAAT("edge");
	float medPointDistance = getMedianPointDistance(pc);

	float sigma = m_TVSigma * medPointDistance;
	float inv_sigma_2 = 1.0f / sigma / sigma;
	float maxrad_2 = sigma * sigma * 4.0f;

	float stepSize = 0.80f;
	CachedRangeQuerry querry(pc);
	progressWindow->pushStep(true,"lines");

	for( unsigned i=0;i<m_LinePoints.size();i++ )
	{
		for( unsigned j=0;j<100;j++ )
		{
			TLinePoint lp = m_LinePoints[i];
			if( lp.movement < medPointDistance * 0.01f )
			{
				Vector3f col = makeVector3f(0.4f,1.0f,0.1f);
				debugRenderer->addLine(lp.pos - lp.lineDir * medPointDistance,lp.pos + lp.lineDir * medPointDistance,col,col,2);
				break;
			}

			PointSet * cache = querry.getSpherePoints(lp.pos, 2.0f*sigma, 3.0f * medPointDistance);
			
			// calculate average line direction
			Vector3f lineDir = NULL_VECTOR3F;
			Matrix3f lineCovariance = IDENTITY3F * 0;
			for( unsigned k=0;k<cache->getNumEntries();k++ )
			{
				Vector3f pos = cache->get3f(k,posAAT);
				float dist_2 = normQuad(lp.pos - pos);
				if( dist_2 > maxrad_2)
					continue;

				Vector3f dir = cache->get3f(k,edgeAAT);
				//if( dir * lp.lineDir < 0)
				//	dir = -dir;
				
				float weight = exp( -dist_2 * inv_sigma_2);// * max(0.0f, fabs(dir*lp.lineDir));
				lineDir += dir * weight;
				lineCovariance += outerProduct(dir,dir)*weight;
			}
			//lineDir = normalize(lineDir);
			//m_LinePoints[i].lineDir = lineDir;
			Vector3f lineEigenValues;
			Matrix3f lineEigenVectors;
			lineCovariance.computeEigenStructure(lineEigenValues,lineEigenVectors);
			lineDir = lineEigenVectors[0];
			m_LinePoints[i].lineDir = lineEigenVectors[0];
			lp.lineDir = lineDir;

			
			// estimate gradient & hessian matrix
			Matrix3f H = IDENTITY3F * 0;
			Vector3f gradient = NULL_VECTOR3F;
			float functionValue=0;
			for( unsigned k=0;k<cache->getNumEntries();k++ )
			{
				Vector3f pos = cache->get3f(k,posAAT);
				float dist_2 = normQuad(lp.pos - pos);
				if( dist_2 > maxrad_2)
					continue;

				Vector3f dir = cache->get3f(k,edgeAAT);

				float dirWeight = pow(normalize(dir)*lineDir,2.0f);
				float length = norm(dir);

				float tmpFV = exp( -dist_2 * inv_sigma_2)* dirWeight* length;
				functionValue += tmpFV;
				Vector3f x = pos - lp.pos;

				Vector3f tmpGradient = makeVector3f( 
					-2.0f * x[0] * tmpFV,
					-2.0f * x[1] * tmpFV,
					-2.0f * x[2] * tmpFV				
					);
				gradient += tmpGradient;
				H[0][0] += -2.0f * tmpFV - 2.0f*x[0]* tmpGradient[0];
				H[0][1] += 2.0f*x[0]* tmpGradient[1];
				H[0][2] += 2.0f*x[0]* tmpGradient[2];

				H[1][0] += 2.0f*x[1]* tmpGradient[0];
				H[1][1] += -2.0f * tmpFV - 2.0f*x[1]* tmpGradient[1];
				H[1][2] += 2.0f*x[1]* tmpGradient[2];

				H[2][0] += 2.0f*x[2]* tmpGradient[0];
				H[2][1] += 2.0f*x[2]* tmpGradient[1];
				H[2][2] += -2.0f * tmpFV - 2.0f*x[2]* tmpGradient[2];

			}

			Vector3f stepColor = makeVector3f(1,1,1);
			Vector3f step = invertMatrix(H) * gradient * stepSize;
			step = step - lineDir * (step*lineDir);

			lp.movement = norm(step);
			lp.pos += step;
			m_LinePoints[i] = lp;

		}
		Vector3f col = makeVector3f(1,0.5f,0);
		debugRenderer->addLine(m_LinePoints[i].pos - m_LinePoints[i].lineDir * medPointDistance,m_LinePoints[i].pos + m_LinePoints[i].lineDir * medPointDistance,col,col,2);

		if( i%10 == 0)
			if( !progressWindow->progress((float)i/(float)m_LinePoints.size()*100.0f) )
				break;
	}
	progressWindow->popSteps();

	debugRenderer->endRenderJob();
	redrawScene();
}

void PCIGlobalRegistration::multiStepLinePoints()
{
	for( card32 i=0;i<10;i++ )
		stepLinePoints();
}

struct TRNode
{
	BoundingBox3f	bb;
	int level;
};

struct TRPlane
{
	Plane p;
	float score;
	bool operator < (const TRPlane &o) const {return score < o.score;}
};

void PCIGlobalRegistration::ransacPlaneExtraction()
{
	// algorithm:
	// - top down approax (overlapping octree)
	// - sampling (constant #points per bb)
	// - extract planes -> refine with original data?
	// - define a overlapping grid, test planes in each cell -> corner point

	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	debugRenderer->clearDebugData("linePoints");
	debugRenderer->beginRenderJob("linePoints");

	float medPointDistance = getMedianPointDistance(pc);
	float sigma = 1.5f * medPointDistance;
	float cellSize = 5.0f * medPointDistance;
	card32 numSamplePoints = 1000;
	card32 numRansacTests = 100;

	float inv_sigma_2 = 1.0f / sigma / sigma;

	// create position only point set
	PointSet * ps = new PointSet;
	VertexDescriptor * vd = new VertexDescriptor;
	vd->pushAttrib(VAD("position",3,VAD::DATA_FORMAT_FLOAT32));
	ps->clearAndSetup( 1, pc->getNumPoints(), vd );
	VertexArray va(pc);
	AAT posAAT = ps->getAAT("position");
	for( unsigned i=0;i<va.getNumElements();i++ )
		ps->set3f(i, posAAT, va.getPosition3f(i));
	
	list<TRNode> nodes;
	TRNode n;
	n.bb = getPCBBox(pc);
	n.bb.makeBoundingCube();
	n.level = 0;
	nodes.push_back(n);
	AttachedIndexedOctree * octree = new AttachedIndexedOctree;
	octree->octreesetup(ps,32);
	int currentLevel = n.level;

	vector<TRPlane> foundPlanes;
	

	// plane extraction
	while( !nodes.empty() )
	{
		// get top element
		TRNode node = *nodes.begin();
		nodes.pop_front();
		BoundingBox3f bb = node.bb;
		bb.addBorder( bb.getMaxSideLength() / 2.0f );

		if( node.level == 2 )
			break;

		if( currentLevel != node.level )
		{
			currentLevel = node.level;

			// rebuild octree
			delete octree;
			octree = new AttachedIndexedOctree;
			octree->octreesetup(ps,32);
		}

		// gather samples in bb
		vector<Vector3f> samples;
		list<AIOctreeNode*> octreeNodes;
		octreeNodes.push_back( octree->getOctree() );
		while( !octreeNodes.empty() )
		{
			AIOctreeNode * ocNode = *octreeNodes.begin();
			octreeNodes.pop_front();
			
			if( bb.intersectBoundingBox( ocNode->getContent().m_bb ) == BVQueryDisjoint )
				continue;

			if( ocNode->isLeaf() )
			{
				for( unsigned i=0;i<ocNode->getContent().m_index.size();i++ )
				{
					samples.push_back( ps->get3f( ocNode->getContent().m_index[i], posAAT ) ); 
				}
			}
			else
			{
				for( unsigned i=0;i<8;i++ )
					octreeNodes.push_back( ocNode->getChild(i) );
			}
		}

		if( samples.size() < numSamplePoints /2 )
			continue;

		// subsample
		if( samples.size()> 2*numSamplePoints )
		{
			for( unsigned i=0;i<numSamplePoints;i++ )
			{
				unsigned sampleIndex = rand() % (samples.size()-i);
				Vector3f tmp = samples[i];
				samples[i] = samples[sampleIndex];
				samples[sampleIndex] = tmp;
			}
			samples.resize(numSamplePoints);
		}
		
		// select random candiates and calc score
		vector<TRPlane> candidates;
		for( unsigned testNr=0;testNr<numRansacTests;testNr++ )
		{
			// draw sample points
			unsigned i1 = rand() % samples.size();
			unsigned i2,i3;
			do
			{
				i2 = rand() % samples.size();
			} while(i2 == i1);

			Vector3f v1 = samples[i2] - samples[i1];
			Vector3f normal;
			float nlength = 0;
			do
			{
				i3 = rand() % samples.size();
				Vector3f v2 = samples[i3] - samples[i1];
				normal = v1.crossProduct(v2);
				nlength = norm(normal);
			} while(i3 == i1 || i2 == i1 || nlength < 0.01f*medPointDistance);

			// setup plane
			Plane p;
			normal.normalize();
			p.setup( normal, samples[i1] );
			
			// score
			float score = 0;
			for( unsigned i=0;i<samples.size();i++ )
			{
				score += exp( -pow(p.calculateSignedDistance( samples[i] ), 2.0f) * inv_sigma_2 );
			}

			debugOutput << "score:" << score << "\n";
			if( score > (float)samples.size()*0.025f )
			{
				TRPlane rp;
				rp.p = p;
				rp.score = score;
				candidates.push_back( rp );
			}
		}

		// pca from supporting samples
		sort(candidates.begin(),candidates.end());

		if( candidates.size() > 0 )
		for( int i=(int)candidates.size()-1;i>0;i-- )
		{
			Vector3f col = makeVector3f(rnd01(),rnd01(),rnd01());
			debugOutput << "plane score:" << candidates[i].score << "\n";
			Plane p = candidates[i].p;
			PCA<float,3> pca;
			float tmpScore = 0;
			for( unsigned j=0;j<samples.size();j++)
			{
				float tmp = exp( -pow(p.calculateSignedDistance( samples[j] ), 2.0f) * inv_sigma_2 );
				if( tmp > 0.1f )
				{
					tmpScore += tmp;
					pca.addPoint(samples[j]);
				}
			}
			debugOutput << "plane tmpscore:" << tmpScore << "\n";
			if( tmpScore < (float)numSamplePoints * 0.025f )
				continue;
			Vector3f centroid,eigenValues;
			Matrix3f eigenVectors;
			pca.analyze(eigenValues, eigenVectors, centroid);

			//Plane p;
			p.setup(eigenVectors[2],centroid);
			debugOutput << "Plane: " << p.getNormal() << "-" << p.getPoint() << "\n";
			for( unsigned j=0;j<samples.size();)
			{
				float tmp = exp( -pow(p.calculateSignedDistance( samples[j] ), 2.0f) * inv_sigma_2 );
				if( tmp > 0.1f )
				{
					/*Vector3f vec1 = eigenVectors[0]*medPointDistance*0.5f;
					Vector3f vec2 = eigenVectors[1]*medPointDistance*0.5f;
					debugRenderer->addQuad( samples[j] - vec1 - vec2,
													samples[j] - vec1 + vec2,
													samples[j] + vec1 + vec2,
													samples[j] + vec1 - vec2, col );*/
					samples.erase(samples.begin()+j);
					tmpScore += tmp;
				}
				else
					j++;
			}

			debugRenderer->addQuad( centroid - eigenVectors[0]*bb.getMaxSideLength()*0.5f
														- eigenVectors[1]*bb.getMaxSideLength()*0.5f ,
											centroid - eigenVectors[0]*bb.getMaxSideLength()*0.5f
														+ eigenVectors[1]*bb.getMaxSideLength()*0.5f,
											centroid + eigenVectors[0]*bb.getMaxSideLength()*0.5f
														+ eigenVectors[1]*bb.getMaxSideLength()*0.5f,
											centroid + eigenVectors[0]*bb.getMaxSideLength()*0.5f
														- eigenVectors[1]*bb.getMaxSideLength()*0.5f,
														makeVector3f(1,0,1));

		}

		// push child nodes on stack
		for( unsigned i=0;i<8;i++ )
		{
			TRNode child;
			child.bb = node.bb;
			child.level = node.level + 1;
			for( unsigned j=0;j<3;j++ )
			{
				float center = (child.bb.lowerCorner[j]+child.bb.upperCorner[j])*0.5f;
				if( (i & (1 >> j)) == 0 )
					child.bb.lowerCorner[j] = center;
				else
					child.bb.upperCorner[j] = center;
			}
			
			nodes.push_back(child);
		}

	}

	debugRenderer->endRenderJob();
}

void PCIGlobalRegistration::tvSettingsChanged()
{
	switch(m_TVDefaultSettings)
	{
	case TVD_Dragon:
		m_TVSigma = 3.0f;
		m_TVCurvSigma = 0.76f;
		m_TVMaxAngle = 10.0f;
		m_TVLP_MinLineLength = 0.1f;
		m_TVLP_SamplesPercent = 0.25f;
		m_TVLP_MaxInitialDistance = 3.0f;
		m_TVLP_MinNumLineSamples = 10;
		m_TVLP_LineSmoothingSigma = 5.0f;
		m_TVLP_MaxLineJoinDistance = 10.0f;
		m_TVLP_MaxLineJoinAngle = 30.0f;
		m_TVLP_MaxLineDistance = 20.0f;
		m_TVLP_MaxLineToLineDistance = 10.0f;
		m_TVLP_MinLineLengthForIntersection = 20.0f;
		break;
	case TVD_Armadillo:
		m_TVSigma = 4.0f;
		m_TVCurvSigma = 0.76f;
		m_TVMaxAngle = 10.0f;
		m_TVLP_MinLineLength = 0.1f;
		m_TVLP_SamplesPercent = 0.25f;
		m_TVLP_MinNumLineSamples = 10;
		m_TVLP_MaxInitialDistance = 3.0f;
		m_TVLP_MaxLineJoinAngle = 30.0f;
		m_TVLP_MaxLineDistance = 20.0f;
		m_TVLP_MaxLineToLineDistance = 10.0f;
		m_TVLP_MinLineLengthForIntersection = 20.0f;
		break;
	}
	ObjectViewsTable::update(this);
}

struct TMinDistTmp
{
	Vector3f pos;
	float dist;
	int index;
};

void PCIGlobalRegistration::estimateLines()
{
	if( m_LinePoints.size() == 0)
		return;

	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	debugRenderer->clearDebugData("linePoints");
	debugRenderer->beginRenderJob("linePoints");

	// setup upc and hIt
	UnstructuredInCorePointCloud tmpPC;
	VertexDescriptor * vd = new VertexDescriptor;
	vd->pushAttrib(VAD("position",3,VAD::DATA_FORMAT_FLOAT32));
	vd->pushAttrib(VAD("entry",1,VAD::DATA_FORMAT_INT32));
	tmpPC.clearAndSetup(vd,(card32)m_LinePoints.size());
	PointSet * ps = tmpPC.getPointSet();
	AAT posAAT = ps->getAAT("position");
	AAT entryAAT = ps->getAAT("entry");
	for( unsigned i=0;i<ps->getNumEntries();i++ )
	{
		ps->set3f(i,posAAT,m_LinePoints[i].pos);
		ps->set1i(i,entryAAT,i);
	}

	float medPointDistance = getMedianPointDistance(pc);
	float minCosAngle = cos(10.0f/180.0f*M_PI);
	float maxInitialDistance = m_TVLP_MaxInitialDistance * medPointDistance;

	// resample ps
	UniformPointCloudResampling resampler;
	float maxSampleSpacing = 0.5f*medPointDistance;
	resampler.setup(maxSampleSpacing);
	PointSet * newPS = resampler.resamplePointSet(ps);
	tmpPC.getPointSet()->assign(newPS);
	ps = newPS;
	
	// smooth remaining line samples
	HierarchicalKNNIterator * tmphIt = new HierarchicalKNNIterator(&tmpPC,32,NULL);
	vector<Vector3f> newPositions(ps->getNumEntries());
	float resSigma = maxSampleSpacing / 2.0f;
	float inv_resSigma_2 = 1.0f / resSigma / resSigma;
	for( unsigned i=0;i<ps->getNumEntries();i++)
	{
		Vector3f newPos = NULL_VECTOR3F;
		Vector3f pos = ps->get3f(i,posAAT);
		tmphIt->setSeekPointAndReset(pos);
		float summedWeight = 0;
		while( !tmphIt->atEnd())
		{
			Vector3f tmpPos = tmphIt->get3f(posAAT);
			if( norm(tmpPos-pos) > 2.0f * resSigma)
				break;
			float weight = exp(-normQuad(tmpPos-pos)*inv_resSigma_2);
			newPos += tmpPos * weight;
			summedWeight += weight;
			tmphIt->next();
		}
		newPos /= summedWeight;
		newPositions[i] = newPos;
	}
	for( unsigned i=0;i<ps->getNumEntries();i++)
	{
		ps->set3f(i,posAAT, newPositions[i]);
	}
	delete tmphIt;
	ps->invalidateLastNumFlagged();
	tmpPC.clearAttachments();
	
	// setup line graph
	HierarchicalKNNIterator hIt(&tmpPC,32,NULL);
	vector<int> lineGraph(m_LinePoints.size()*2,-1);
	for( unsigned i=0;i<ps->getNumEntries();i++ )
	{
		int entry = ps->get1i(i,entryAAT);
		
		if( lineGraph[entry*2+0] != -1 && lineGraph[entry*2+1] != -1)
			continue;

		const Vector3f pos = m_LinePoints[entry].pos;
		hIt.setSeekPointAndReset(pos);
		Vector3f dir = m_LinePoints[entry].lineDir;
		hIt.next();
		while( !hIt.atEnd())
		{
			int e2 = hIt.get1i(entryAAT);
			Vector3f tmpDir = pos - m_LinePoints[e2].pos;

			if( norm(tmpDir)>maxInitialDistance)
				break;

			if( lineGraph[entry*2+0] == -1 )
			{
				if( tmpDir*dir > 0 )
				{
					lineGraph[entry*2+0] = e2;
					//debugRenderer->addLine(pos,m_LinePoints[e2].pos);
				}
			}
			if( lineGraph[entry*2+1] == -1 )
			{
				if( tmpDir*dir < 0 )
				{
					lineGraph[entry*2+1] = e2;
					//debugRenderer->addLine(pos,m_LinePoints[e2].pos);
				}
			}

			if( lineGraph[entry*2+0] != -1 && lineGraph[entry*2+1] != -1)
				break;

			hIt.next();
		}


	}

		
	vector< vector<unsigned> > lines;
	vector< bool > visited(m_LinePoints.size(),false);
	for( unsigned i=0;i<lineGraph.size()/2;i++)
	{
		if( visited[i] )
			continue;
		visited[i] = true;
		
		// grow back
		unsigned tmp = i;
		vector<unsigned> tmpLine;
		tmpLine.push_back(tmp);
		unsigned dir = 0;
		while( lineGraph[tmp*2+dir]!=-1 )
		{
			int neighbour = lineGraph[tmp*2+dir];
			if( visited[neighbour] )
				break;

			if( lineGraph[neighbour*2+0] == tmp || lineGraph[neighbour*2+1] == tmp)
			{
				tmpLine.insert(tmpLine.begin(),neighbour);
				visited[neighbour] = true;
				if( lineGraph[neighbour*2+0] == tmp )
					dir = 1;
				else
					dir = 0;
				tmp = neighbour;
			}
			else
			{
				/*Vector3f col = makeVector3f(1,1,0);
				debugRenderer->addLine(m_LinePoints[neighbour].pos,m_LinePoints[tmp].pos,col,col,3);
				Vector3f col2 = makeVector3f(0.5,1,0);
				if(lineGraph[neighbour*2+0] =! -1 )
					debugRenderer->addLine(m_LinePoints[neighbour].pos,m_LinePoints[lineGraph[neighbour*2+0]].pos,col2,col2,3);
				if(lineGraph[neighbour*2+1] =! -1 )
					debugRenderer->addLine(m_LinePoints[neighbour].pos,m_LinePoints[lineGraph[neighbour*2+1]].pos,col2,col2,3);*/
				break;
			}
		}

		// grow forward
		tmp = i;
		dir = 1;
		while( lineGraph[tmp*2+dir]!=-1 )
		{
			int neighbour = lineGraph[tmp*2+dir];
			if( visited[neighbour] )
				break;

			if( lineGraph[neighbour*2+0] == tmp || lineGraph[neighbour*2+1] == tmp)
			{
				tmpLine.push_back(neighbour);
				visited[neighbour] = true;
				if( lineGraph[neighbour*2+0] == tmp )
					dir = 1;
				else
					dir = 0;
				tmp = neighbour;
			}
			else
			{
/*				Vector3f col = makeVector3f(1,1,0);
				debugRenderer->addLine(m_LinePoints[neighbour].pos,m_LinePoints[tmp].pos,col,col,3);
				Vector3f col2 = makeVector3f(0.5,1,0);
				if(lineGraph[neighbour*2+0] =! -1 )
					debugRenderer->addLine(m_LinePoints[neighbour].pos,m_LinePoints[lineGraph[neighbour*2+0]].pos,col2,col2,3);
				if(lineGraph[neighbour*2+1] =! -1 )
					debugRenderer->addLine(m_LinePoints[neighbour].pos,m_LinePoints[lineGraph[neighbour*2+1]].pos,col2,col2,3);*/
				break;
			}
		}

		if( tmpLine.size() >= m_TVLP_MinNumLineSamples)
		{
			lines.push_back(tmpLine);
			/*Vector3f col = makeVector3f(rnd01(),rnd01(),rnd01());
			for( unsigned j=0;j<tmpLine.size()-1;j++ )
			{
				debugRenderer->addLine(m_LinePoints[tmpLine[j]].pos,m_LinePoints[tmpLine[j+1]].pos, col, col, 2);
			}*/
		}

	}

	// smooth lines endings
	vector<Vector3f> smoothedEndingDirections(lines.size()*2);
	float lineSmoothingSigma = m_TVLP_LineSmoothingSigma * medPointDistance;
	float inv_lineSmoothingSigma_2 = 1.0f / lineSmoothingSigma / lineSmoothingSigma;
	for( unsigned i=0;i<lines.size();i++ )
	{
		// front
		PCA<float,3> pca;
		Vector3f pos0 = m_LinePoints[lines[i][0]].pos;
		for( unsigned j=0;j<lines[i].size();j++ )
		{
			Vector3f tmpPos = m_LinePoints[lines[i][j]].pos;
			if( norm(tmpPos-pos0)>2.0f*lineSmoothingSigma)
				break;
			float weight = exp( -normQuad(tmpPos-pos0)*inv_lineSmoothingSigma_2);
			pca.addPoint(tmpPos,weight );		
		}
		Matrix3f eigenVectors;
		Vector3f centroid, eigenValues;
		pca.analyze(eigenValues,eigenVectors,centroid);
		smoothedEndingDirections[i*2+0] = eigenVectors[0];
		if( eigenVectors[0] * (centroid - pos0) > 0 )
			smoothedEndingDirections[i*2+0] = -eigenVectors[0];
		//debugRenderer->addLine(pos0,pos0 + smoothedEndingDirections[i*2+0]*lineSmoothingSigma, makeVector3f(0.2f,0.5f,1.0f),
		//	makeVector3f(0.2f,0.5f,1.0f),2.0f);

		// back
		pos0 = m_LinePoints[lines[i].back()].pos;
		pca.clear();
		for( int j=(int)lines[i].size()-1;j>=0;j-- )
		{
			Vector3f tmpPos = m_LinePoints[lines[i][j]].pos;
			if( norm(tmpPos-pos0)>2.0f*lineSmoothingSigma)
				break;
			float weight = exp( -normQuad(tmpPos-pos0)*inv_lineSmoothingSigma_2);
			pca.addPoint(tmpPos,weight );		
		}
		pca.analyze(eigenValues,eigenVectors,centroid);
		smoothedEndingDirections[i*2+1] = eigenVectors[0];
		if( eigenVectors[0] * (centroid - pos0) > 0 )
			smoothedEndingDirections[i*2+1] = -eigenVectors[0];
		//debugRenderer->addLine(pos0,pos0 + smoothedEndingDirections[i*2+1]*lineSmoothingSigma, makeVector3f(0.2f,0.5f,1.0f),
		//	makeVector3f(0.2f,0.5f,1.0f),2.0f);


	}

	// join lines
	unsigned currentLine = 0;
	float maxJoinDistance = medPointDistance * m_TVLP_MaxLineJoinDistance;
	while( currentLine < lines.size() )
	{
		Vector3f begin_pos0 = m_LinePoints[ lines[currentLine][0] ].pos;
		Vector3f begin_pos1 = m_LinePoints[ lines[currentLine].back() ].pos;
		Vector3f lineDir0 = smoothedEndingDirections[currentLine*2+0];
		Vector3f lineDir1 = smoothedEndingDirections[currentLine*2+1];

		// find best line begin/end
		int best0 = -1;
		int best1 = -1;
		float bestCosAngle0 = cos(m_TVLP_MaxLineJoinAngle/180.0f*M_PI);
		float bestCosAngle1 = bestCosAngle0;
		bool isBack0=false, isBack1=false;
		for( unsigned i=currentLine+1;i<lines.size();i++ )
		{
			// check front end
			Vector3f front_pos = m_LinePoints[lines[i][0]].pos;
			Vector3f front_dir = smoothedEndingDirections[i*2+0];
			if( norm(begin_pos0-front_pos) < maxJoinDistance )
				if( -front_dir * lineDir0 > bestCosAngle0)	// must point into opposite directions
				{
					bestCosAngle0 = -front_dir * lineDir0;
					best0 = i;
					isBack0 = false;
				}
			
			if( norm(begin_pos1-front_pos) < maxJoinDistance )
				if( -front_dir * lineDir1 > bestCosAngle1)	// must point into opposite directions
				{
					bestCosAngle1 = -front_dir * lineDir1;
					best1 = i;
					isBack1 = false;
				}

			// check back end
			Vector3f back_pos = m_LinePoints[lines[i].back()].pos;
			Vector3f back_dir = smoothedEndingDirections[i*2+1];
			if( norm(begin_pos0-back_pos) < maxJoinDistance )
				if( -back_dir * lineDir0 > bestCosAngle0)	// must point into opposite directions
				{
					bestCosAngle0 = -back_dir * lineDir0;
					best0 = i;
					isBack0 = true;
				}

				if( norm(begin_pos1-back_pos) < maxJoinDistance )
					if( -back_dir * lineDir1 > bestCosAngle1)	// must point into opposite directions
					{
						bestCosAngle1 = -back_dir * lineDir1;
						best1 = i;
						isBack1 = true;
					}
		}

		if( best0 != -1 && best0 != best1 )
		{
			if( isBack0 )
			{
//				debugRenderer->addLine(m_LinePoints[lines[currentLine][0]].pos,
//					m_LinePoints[lines[best0].back()].pos, makeVector3f(0,1,0),makeVector3f(0,1,0),4);
				lines[currentLine].insert(lines[currentLine].begin(),lines[best0].begin(), lines[best0].end());
				smoothedEndingDirections[currentLine*2+0] = smoothedEndingDirections[best0*2+0];
			}
			else
			{
//				debugRenderer->addLine(m_LinePoints[lines[currentLine][0]].pos,
//					m_LinePoints[lines[best0][0]].pos, makeVector3f(1,1,0),makeVector3f(1,0,0),4);
				for( unsigned i=0;i<lines[best0].size();i++)
				{
					lines[currentLine].insert(lines[currentLine].begin(),lines[best0][i]);
				}
				smoothedEndingDirections[currentLine*2+0] = smoothedEndingDirections[best0*2+1];
			}
			lines.erase(lines.begin() + best0);
			smoothedEndingDirections.erase(smoothedEndingDirections.begin() + 2*best0);
			smoothedEndingDirections.erase(smoothedEndingDirections.begin() + 2*best0);
			if( best1 != -1 && best1 > best0)
				best1--;
		}

		if( best1 != -1 && best0 != best1 )
		{
			if( isBack1 )
			{
//				debugRenderer->addLine(m_LinePoints[lines[currentLine].back()].pos,
//					m_LinePoints[lines[best1].back()].pos, makeVector3f(1,0,0),makeVector3f(1,0,0),4);
				size_t last = lines[best1].size()-1;
				for( unsigned i=0;i<lines[best1].size();i++)
				{
					lines[currentLine].insert(lines[currentLine].end(),lines[best1][last - i]);
				}
				smoothedEndingDirections[currentLine*2+1] = smoothedEndingDirections[best1*2+0];
			}
			else
			{
//				debugRenderer->addLine(m_LinePoints[lines[currentLine].back()].pos,
//					m_LinePoints[lines[best1][0]].pos, makeVector3f(0,0,1),makeVector3f(0,0,1),4);
				lines[currentLine].insert(lines[currentLine].end(),lines[best1].begin(), lines[best1].end());
				smoothedEndingDirections[currentLine*2+1] = smoothedEndingDirections[best1*2+1];
			}
			lines.erase(lines.begin() + best1);
			smoothedEndingDirections.erase(smoothedEndingDirections.begin() + 2*best1);
			smoothedEndingDirections.erase(smoothedEndingDirections.begin() + 2*best1);
		}

		if( best1 == -1 && best0 == -1 || best0 == best1 )
			currentLine++;

	}

	for( unsigned i=0;i<lines.size();i++ )
	{
		Vector3f col = makeVector3f(rnd01(),rnd01(),rnd01());
		for( unsigned j=0;j<lines[i].size()-1;j++ )
		{
			debugRenderer->addLine(m_LinePoints[lines[i][j]].pos,m_LinePoints[lines[i][j+1]].pos,col,col,2);
		}
		/*Vector3f pos0 = m_LinePoints[lines[i][0]].pos;
		debugRenderer->addLine(pos0,pos0 + smoothedEndingDirections[i*2+0]*lineSmoothingSigma, makeVector3f(0.2f,0.5f,1.0f),
			makeVector3f(0.2f,0.5f,1.0f),2.0f);
		pos0 = m_LinePoints[lines[i].back()].pos;
		debugRenderer->addLine(pos0,pos0 + smoothedEndingDirections[i*2+1]*lineSmoothingSigma, makeVector3f(0.2f,0.5f,1.0f),
			makeVector3f(0.2f,0.5f,1.0f),2.0f);*/
	}

	// gather line ends
	vector<TLinePoint> lineEnds;
	for( unsigned i=0;i<lines.size();i++ )
	{
		if( lines[i].size() < m_TVLP_MinLineLengthForIntersection)
			continue;

		TLinePoint lp;
		lp.pos = m_LinePoints[lines[i][0]].pos;
		lp.lineDir = smoothedEndingDirections[i*2+0];
		lineEnds.push_back(lp);
		lp.pos = m_LinePoints[lines[i].back()].pos;
		lp.lineDir = smoothedEndingDirections[i*2+1];
		lineEnds.push_back(lp);
	}

	// test for "intersections"
	float maxPointDist = m_TVLP_MaxLineDistance * medPointDistance;
	float maxLinePointDist = m_TVLP_MaxLineToLineDistance * medPointDistance;
	FeatureAttachment * featureAttachment = NULL;
	featureAttachment = dynamic_cast<FeatureAttachment*>( pc->getAttachments()->getData(FeatureAttachment::attachmentName()) );
	if( !featureAttachment )
	{
		featureAttachment = new FeatureAttachment;
		featureAttachment->setup(FeatureAttachment::attachmentName(), AttachedData::ADF_PERSISTENT );
		pc->getAttachments()->attachData( featureAttachment );
	}

	for( unsigned i=0;i<lines.size();i++ )
	{
		map<unsigned,TMinDistTmp> intersections;
		for( unsigned j=0;j<lines[i].size();j++ )
		{
			TLinePoint lp = m_LinePoints[lines[i][j]];
			for( unsigned k=0;k<lineEnds.size();k++ )					
			{
				if( norm(lineEnds[k].pos - lp.pos) < maxPointDist && k/2 != i && fabs(lineEnds[k].lineDir * lp.lineDir) < cos(45.0f/180.0f*M_PI) )
				{
					float tmpDist = sqrt( normQuad(lp.pos-lineEnds[k].pos)-pow((lp.pos-lineEnds[k].pos)*lineEnds[k].lineDir,2.0f) );
					if( tmpDist < maxLinePointDist)
					{
						if( intersections.find(k) == intersections.end())
						{
							TMinDistTmp tmp;
							tmp.dist = tmpDist;
							tmp.index = k;
							tmp.pos = lp.pos;
							intersections[k] = tmp;
						}
						else
							if( intersections[k].dist > tmpDist)
							{
								intersections[k].dist = tmpDist;
								intersections[k].pos = lp.pos;
							}
					}
				}
			}
		}
		for( map<unsigned,TMinDistTmp>::iterator iter=intersections.begin();iter!=intersections.end();++iter)
		{
			debugRenderer->addSphere(iter->second.pos, medPointDistance*3.5f, makeVector3f(1,0,0));
			FastFeature * f = new FastFeature;
			f->m_Pos = iter->second.pos;
			f->m_Normal = computePCANormal(pc,f->m_Pos,5.0f*medPointDistance);
			featureAttachment->addFeature(f);
		}
	}

	for( unsigned i=0;i<lines.size();i++ )
	{
		Vector3f pos = m_LinePoints[ lines[i][lines[i].size()/2]].pos;
		debugRenderer->addSphere(pos, medPointDistance*3.5f, makeVector3f(1,0,0));
		FastFeature * f = new FastFeature;
		f->m_Pos = pos;
		f->m_Normal = computePCANormal(pc,f->m_Pos,5.0f*medPointDistance);
		featureAttachment->addFeature(f);

	}

	debugRenderer->endRenderJob();
	
}

Vector3f PCIGlobalRegistration::computePCANormal( PointCloud * pc, const Vector3f &pos, float sigma )
{
	float inv_sigma_2 = 1.0f / sigma / sigma;
	float rad_2 = sigma * sigma * 4.0f;

	PCA<float,3> pca;
	
	unsigned i=0;
	AAT posAAT = pc->getAAT("position");
	AAT normalAAT = pc->getAAT("normal");
	HierarchicalKNNIterator hIt(pc,32,NULL);
	hIt.setSeekPointAndReset(pos);
	Vector3f pointNormal = hIt.get3f(normalAAT);
	while(!hIt.atEnd())
	{
		Vector3f tmpPos = hIt.get3f(posAAT);
		float dist_2 = normQuad(tmpPos - pos);
		if( dist_2 > rad_2 && i > 10)
			break;

		pca.addPoint(tmpPos, exp(-dist_2*inv_sigma_2));

		i++;
		hIt.next();
	}
	Matrix3f eigenVectors;
	Vector3f centroid, eigenValues;
	pca.analyze(eigenValues, eigenVectors, centroid);

	Vector3f normal = normalize(eigenVectors[0].crossProduct(eigenVectors[1]));
	if( pointNormal * normal < 0)
		normal *= -1.0f;
	return normal;
}

void PCIGlobalRegistration::stepCornerPoints()
{
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	debugRenderer->clearDebugData("linePoints");
	debugRenderer->beginRenderJob("linePoints");

	SuperIterator iter( pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | SceneObjectIteratorType::CAP_MODIFYABLE_PC ), pc->getDescr()  );
	HierarchicalKNNIterator hIt(pc,32,NULL);

	AAT posAAT = pc->getAAT( "position" );
	AAT colorAAT = pc->getAAT("color");
	float medPointDistance = getMedianPointDistance(pc);

	float sigma = m_TVSigma * medPointDistance;
	float inv_sigma_2 = 1.0f / sigma / sigma;
	float maxrad_2 = sigma * sigma * 4.0f;

	float stepSize = 0.80f;
	CachedRangeQuerry querry(pc);
	progressWindow->pushStep(true,"lines");

	for( unsigned i=0;i<m_CornerPoints.size();i++ )
	{
		Vector3f oldPos = m_CornerPoints[i].pos;
		for( unsigned j=0;j<100;j++ )
		{
			TCornerPoint cp = m_CornerPoints[i];
			if( cp.movement < medPointDistance*0.01f )
			{
				Vector3f col = makeVector3f(0.4f,1.0f,0.1f);
				debugRenderer->addPoint(cp.pos,col);
				break;
			}

			PointSet * cache = querry.getSpherePoints(cp.pos, 2.0f*sigma, 3.0f * medPointDistance);


			// estimate gradient & hessian matrix
			Matrix3f H = IDENTITY3F * 0;
			Vector3f gradient = NULL_VECTOR3F;
			float functionValue=0;
			for( unsigned k=0;k<cache->getNumEntries();k++ )
			{
				Vector3f pos = cache->get3f(k,posAAT);
				float dist_2 = normQuad(cp.pos - pos);
				if( dist_2 > maxrad_2)
					continue;

				float eigenProd = cache->get3f(k,colorAAT)[2];

				float tmpFV = exp( -dist_2 * inv_sigma_2)* eigenProd;
				functionValue += tmpFV;
				Vector3f x = pos - cp.pos;

				Vector3f tmpGradient = makeVector3f( 
					-2.0f * x[0] * tmpFV,
					-2.0f * x[1] * tmpFV,
					-2.0f * x[2] * tmpFV				
					);
				gradient += tmpGradient;
				H[0][0] += -2.0f * tmpFV - 2.0f*x[0]* tmpGradient[0];
				H[0][1] += 2.0f*x[0]* tmpGradient[1];
				H[0][2] += 2.0f*x[0]* tmpGradient[2];

				H[1][0] += 2.0f*x[1]* tmpGradient[0];
				H[1][1] += -2.0f * tmpFV - 2.0f*x[1]* tmpGradient[1];
				H[1][2] += 2.0f*x[1]* tmpGradient[2];

				H[2][0] += 2.0f*x[2]* tmpGradient[0];
				H[2][1] += 2.0f*x[2]* tmpGradient[1];
				H[2][2] += -2.0f * tmpFV - 2.0f*x[2]* tmpGradient[2];

			}

			Vector3f stepColor = makeVector3f(1,1,1);
			Vector3f step = invertMatrix(H) * gradient * stepSize;
			//step = step - lineDir * (step*lineDir);

			cp.movement = norm(step);
			cp.pos += step;
			m_CornerPoints[i] = cp;

		}
		Vector3f col = makeVector3f(1,0.5f,0);
		debugRenderer->addPoint(m_CornerPoints[i].pos,col);
		debugRenderer->beginRenderJob("test");
		debugRenderer->addLine(m_CornerPoints[i].pos, oldPos);
		debugRenderer->beginRenderJob("linePoints");

		if( i%10 == 0)
			if( !progressWindow->progress((float)i/(float)m_CornerPoints.size()*100.0f) )
				break;
	}
	progressWindow->popSteps();

	debugRenderer->endRenderJob();
	redrawScene();	
}

void geoxl::PCIGlobalRegistration::initCornerPoints()
{
	m_CornerPoints.clear();

	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	debugRenderer->clearDebugData("tensorVoting");
	debugRenderer->clearDebugData("linePoints");
	debugRenderer->beginRenderJob("linePoints");

	SuperIterator iter( pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | SceneObjectIteratorType::CAP_MODIFYABLE_PC ), pc->getDescr()  );
	HierarchicalKNNIterator hIt(pc,32,NULL);

	AAT posAAT = pc->getAAT( "position" );
	AAT colorAAT = pc->getAAT("color");
	float medPointDistance = getMedianPointDistance(pc);

	// fetch points with big eigenvalues
	vector<unsigned> cornerPointSet;
	card32 index = 0;
	while(!iter.atEnd())
	{
		float eigenProd = iter.get<Vector3f>(colorAAT)[2];

		if( eigenProd > m_TVCP_MinEigenProduct )
		{
			cornerPointSet.push_back(index);
			debugRenderer->addPoint(iter.get<Vector3f>(posAAT),makeVector3f(0,0.5f,1));
		}

		iter.next();
		index++;
	}

	// select random subset
	card32 numSeedPoints = cornerPointSet.size() * m_TVLP_SamplesPercent;
	for( unsigned i=0;i<numSeedPoints;i++ )
	{
		card32 lIndex = rand() % cornerPointSet.size();
		card32 index = cornerPointSet[lIndex];
		iter.goToIndex(index);

		TCornerPoint cp;
		cp.pos = iter.get<Vector3f>(posAAT);
		cp.movement = 1e20f;
		m_CornerPoints.push_back(cp);
		cornerPointSet.erase(cornerPointSet.begin() + lIndex);

		Vector3f col = makeVector3f(1,0.5f,0);
		debugRenderer->addPoint(cp.pos,col);
		//debugRenderer->addSphere(lp.pos, medPointDistance * 0.5f, col);
	}

	debugRenderer->endRenderJob();
}

void PCIGlobalRegistration::goodOldMSFeatures()
{
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	PointCloud * pc = NULL;
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				break;
		}
		sceneIt->next();
	}

	if( !pc )
		return;

	checkAttribute(pc,"spos",3,VAD::DATA_FORMAT_FLOAT32);
	checkAttribute(pc,"snormal",3,VAD::DATA_FORMAT_FLOAT32);

	debugRenderer->clearDebugData("mlsFeatures");
	debugRenderer->beginRenderJob("mlsFeatures");

	SuperIterator iter( pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | SceneObjectIteratorType::CAP_MODIFYABLE_PC ), pc->getDescr()  );
	//HierarchicalKNNIterator hIt(pc,32,NULL);

	AAT posAAT = pc->getAAT( "position" );
	AAT colorAAT = pc->getAAT("color");
	AAT normalAAT = pc->getAAT("normal");
	AAT sposAAT = pc->getAAT("spos");
	AAT snormalAAT = pc->getAAT("snormal");
	float medPointDistance = getMedianPointDistance(pc);

	CachedRangeQuerry querry(pc);

	float sigmaA = 3.0f * medPointDistance;
	float sigmaB = 2.25f * sigmaA;
	float radA = 2.0f * sigmaA;
	float radB = 2.0f * sigmaB;
	float radA_2 = radA * radA;
	float radB_2 = radB * radB;
	float inv_sigmaA_2 = 1.0f / sigmaA / sigmaA;
	float inv_sigmaB_2 = 1.0f / sigmaB / sigmaB;


	progressWindow->pushStep(true,"mscale");
	unsigned index=0;
	while( !iter.atEnd() )
	{
		Vector3f pos = iter.get<Vector3f>(posAAT);
		
		vector<Vector3f> *cache = querry.getSpherePositions(pos, radB, 2.7f );
		
		// setup pca
		PCA<float,3> pcaA,pcaB;
		for( unsigned i=0;i<cache->size();i++ )
		{
			float dist_2 = normQuad(pos - (*cache)[i]);

			if( dist_2 > radB_2 )
				continue;

			pcaB.addPoint( (*cache)[i], exp( -dist_2 * inv_sigmaB_2) );

			if( dist_2 > radB_2 )
				continue;

			pcaA.addPoint( (*cache)[i], exp( -dist_2 * inv_sigmaA_2) );
		}

		// estimate planes
		Matrix3f eigenVectors;
		Vector3f eigenValues, centroid;
		pcaA.analyze(eigenValues, eigenVectors, centroid);
		Plane planeA,planeB;
		planeA.setup(normalize(eigenVectors[0].crossProduct(eigenVectors[1])),centroid);

		pcaB.analyze(eigenValues, eigenVectors, centroid);
		planeB.setup(normalize(eigenVectors[0].crossProduct(eigenVectors[1])),centroid);
		
		// project pos onto planes
		Vector3f pos0 = pos - planeA.getNormal() * (planeA.getNormal()*(pos-planeA.getPoint()));
		Vector3f pos1 = pos - planeB.getNormal() * (planeB.getNormal()*(pos-planeB.getPoint()));

		Vector3f normal = iter.get<Vector3f>(normalAAT);
		if( planeB.getNormal() * normal < 0 )
			planeB.setNormal( -planeB.getNormal() );

		float distance = (pos1-pos0)*planeB.getNormal();
		iter.set(colorAAT, makeVector3f(distance,0,0));
		iter.set(sposAAT, pos1);
		iter.set(snormalAAT, planeB.getNormal() );
		iter.flush();

		iter.next();
		index++;
		if( index % 10 == 0 )
			progressWindow->progress((float)index/(float)pc->getNumPoints()*100.0f);
	}
	progressWindow->popSteps();

	HierarchicalKNNIterator hIt(pc,32,NULL);
	iter.reset();
	float minFeatureRad_2 = sigmaB * sigmaB;
	while( !iter.atEnd() )
	{
		Vector3f pos = iter.get<Vector3f>(posAAT);
		Vector3f col = iter.get<Vector3f>(colorAAT);
		float scaleDifference = col[0];
		hIt.setSeekPointAndReset(pos);
		hIt.next();
		bool isMin = true;
		bool isMax = true;
		while( !hIt.atEnd() && (isMax || isMin) )
		{
			float dist_2 = normQuad( hIt.get3f(posAAT) - pos );
			if( dist_2 > radB_2 )
				break;

			isMax = isMax & (hIt.get3f(colorAAT)[0] < scaleDifference);
			isMin = isMin & (hIt.get3f(colorAAT)[0] > scaleDifference);

			hIt.next();
		}

		if( isMin )
			debugRenderer->addSphere( pos, medPointDistance * 2.5f, makeVector3f(1,0.5f,0));
		if( isMax )
			debugRenderer->addSphere( pos, medPointDistance * 2.5f, makeVector3f(0.1f,0.25f,1));

		iter.next();
	}


	debugRenderer->endRenderJob();
}
