#ifndef PGDescriptorFactoryH
#define PGDescriptorFactoryH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PGDescriptorChannels.h"
#include "FeatureDescriptor.h"
//---------------------------------------------------------------------------

namespace geoxl
{

class UnstructuredInCorePointCloud;
class PatchGraph;
class TopologyIterator;

/**
	creates multiple descriptors for every patchgraph
*/
class ANIMATIONRECONSTRUCTION_API PGDescriptorFactory : public PGDescriptorChannels
{
	GEOXL_CLASS( PGDescriptorFactory )

private:

	// properties
	vector<FeatureDescriptor*>		m_Descriptors;
	bool									m_UseDataPoints;
	float32								m_DescriptorSize;
	
	// members
	UnstructuredInCorePointCloud *	m_CurrentFramePointCloud;	// holds current frame (projected data points or pg points only)
	int32				m_CurrentFrameIndex;
	PatchGraph *	m_CurrentPG;
	card32		m_NumDescriptorComponents;
	vector<Component>	m_Components;
	AAT			m_PosAAT;
	TDescriptorContext m_DescriptorContext;
	PointCloudIterator * m_TopologyIterator;
	UnstructuredInCorePointCloud * m_CurrentPC;


	void addDescriptor( Object * obj );
	void deleteDescriptors( card32 begin, card32 end );
	Object * getDescriptor( card32 index );
	void moveDescriptor( card32 oldPos, card32 newPos );
	mpcard getNumDescriptors();
	string getDescriptorName(card32 index );

	void buildCurrentPointCloud(const PatchGraph *pg, const PointCloudAnimationSequence *data, const int32 frameIndex);
	void clearData();

protected:

   virtual void startComputeDescriptors( const vector<PatchGraph*> &pgs, const PointCloudAnimationSequence *data, vector<Component> &components);
   virtual void computeDescriptor( const PatchGraph *pg, const PointCloudAnimationSequence *data, const int32 frameIndex, const card32 patchIndex, DVectorF &result);

	virtual void startComputeDescriptors(vector<PointCloud*> pcs, vector<Component> &components);
	virtual void computeDescriptor( PointCloud * pc, card32 pointIndex, DVectorF &result );


public:
	PGDescriptorFactory(void);
	~PGDescriptorFactory(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	float32 getDescriptorSize() const {return m_DescriptorSize;}
	void setDescriptorSize(float32 v) {m_DescriptorSize=v;}

	void setupFunkhouserChrom();
	void setupFunkhouserRGB();
	void setupHistogramChrom();
	void setupHistogramRGB();
	void setupHistogramChrom_Curvature();
	void setupHistogramCurvature();

};

}

#endif