#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "RegistrationContext.h"
//---------------------------------------------------------------------------
#include "FDHistogram.h"
#include "MLSMultiScaleFeatureDetector.h"
#include "DescriptorClustering.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( RegistrationContext ,8)
{
	BEGIN_CLASS_INIT( RegistrationContext );

	ADD_OBJECT_PROP( m_FeatureDetector, 0, FeatureDetector::getClass(), true );
	ADD_OBJECT_PROP( m_FeatureDescriptor, 0, FeatureDescriptor::getClass(), true );

	ADD_FLOAT32_PROP( m_FeatureRadius, 0 );
	ADD_FLOAT32_PROP( m_AveragePointDistance, 0 );

	ADD_BOOLEAN_PROP( m_UseGeodeticDistances, 0);

	ADD_FLOAT32_PROP( m_MaxMovement,1);
	ADD_FLOAT32_PROP( m_PruningThreshold,1);
	ADD_BOOLEAN_PROP( m_ValidateCorrespondences, 2);

	ADD_FLOAT32_PROP( m_Validate_ConfSigma, 3 );
	ADD_FLOAT32_PROP( m_Validate_ConflictScale, 3 );
	ADD_FLOAT32_PROP( m_Validate_ConflictAdd, 3 );

	ADD_FLOAT32_PROP( m_SelfConfidenceSigma,5);
	ADD_FLOAT32_PROP( m_SelfConfidenceScale,5);

	ADD_FLOAT32_PROP( m_TPS_Gamma, 4 );
	SET_USER_EDIT(false);
	ADD_FLOAT32_PROP( m_TPS_q, 4 );
	SET_USER_EDIT(false);
	ADD_FLOAT32_PROP( m_TPS_Alpha_Tolerance, 4 );
	SET_USER_EDIT(false);

	ADD_BOOLEAN_PROP(m_UseExistingCorrespondenceAsValid, 7 );

	ADD_FLOAT32_PROP(m_MinConfidence,8);

	ADD_CARD32_PROP( m_NumDescriptorClusters, 6);
	ADD_CARD32_PROP( m_NumDescriptorSpaceCells, 6);


}

RegistrationContext::RegistrationContext(void)
{
	m_FeatureDescriptor = new FDHistogram;
	m_FeatureDetector = new DescriptorClustering;
	
	m_FeatureRadius = 5.0f;
	m_AveragePointDistance = 0.0f;

	m_UseGeodeticDistances = true;

	m_MaxMovement = 0.2f;
	m_PruningThreshold = 0.01f;
	m_ValidateCorrespondences = true;

	m_Validate_ConfSigma = 2.0f;
	m_Validate_ConflictScale = 0.0f;
	m_Validate_ConflictAdd = 1.0f;

	m_TPS_Gamma = 0.5f;
	m_TPS_q = 0.0001f;
	m_TPS_Alpha_Tolerance = 25.0f;

	m_SelfConfidenceSigma = 2.0f;
	m_SelfConfidenceScale = 10.0f;

	m_NumDescriptorClusters = 8.0f;
	m_NumDescriptorSpaceCells = 32.0f;

	m_UseExistingCorrespondenceAsValid = false;

	m_MinConfidence = 1e-10f;

}

RegistrationContext::~RegistrationContext(void)
{
}

void RegistrationContext::assign(const Object* obj, CopyContext *context)
{
	//Persistent::assign(obj,context);
	const RegistrationContext * o = dynamic_cast<const RegistrationContext*>(obj);
	pAssert( o != NULL );

	m_FeatureDetector = o->m_FeatureDetector;
	m_FeatureDescriptor = o->m_FeatureDescriptor;
	
	m_FeatureRadius = o->m_FeatureRadius;
	m_AveragePointDistance = o->m_AveragePointDistance;

	m_UseGeodeticDistances = o->m_UseGeodeticDistances;
	m_ValidateCorrespondences = o->m_ValidateCorrespondences;

	m_TPS_Gamma = o->m_TPS_Gamma;
	m_TPS_q = o->m_TPS_q;
	m_TPS_Alpha_Tolerance = o->m_TPS_Alpha_Tolerance;

	m_UseExistingCorrespondenceAsValid = o->m_UseExistingCorrespondenceAsValid;

}