#ifndef RegistrationContextH
#define RegistrationContextH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl
{

class FeatureDetector;
class FeatureDescriptor;
	
/**
	context for feature registration
*/
class ANIMATIONRECONSTRUCTION_API RegistrationContext : public Persistent
{
	GEOXL_CLASS( RegistrationContext )

private:

	FeatureDetector	*		m_FeatureDetector;
	FeatureDescriptor	*		m_FeatureDescriptor;

	float32						m_FeatureRadius;
	float32						m_AveragePointDistance;

	bool							m_UseGeodeticDistances;
	
	float32						m_MaxMovement;
	float32						m_PruningThreshold;
	bool							m_ValidateCorrespondences;

	float32						m_Validate_ConfSigma;
	float32						m_Validate_ConflictScale;
	float32						m_Validate_ConflictAdd;

	float32						m_TPS_Gamma;
	float32						m_TPS_q;
	float32						m_TPS_Alpha_Tolerance;

	float32						m_SelfConfidenceSigma;
	float32						m_SelfConfidenceScale;

	card32						m_NumDescriptorClusters;
	card32						m_NumDescriptorSpaceCells;
	
	bool							m_UseExistingCorrespondenceAsValid;
	
	float32						m_MinConfidence;




public:
	RegistrationContext(void);
	~RegistrationContext(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	FeatureDetector * getFeatureDetector() const {return m_FeatureDetector;}

	FeatureDescriptor * getFeatureDescriptor() const {return m_FeatureDescriptor;}


	float32 getFeatureRadius() const {return m_FeatureRadius;}
	void setFeatureRadius(float32 v) {m_FeatureRadius=v;}

	float32 getAveragePointDistance() const {return m_AveragePointDistance;}
	void setAveragePointDistance(float32 v) {m_AveragePointDistance=v;}


	bool getUseGeodeticDistances() const {return m_UseGeodeticDistances;}
	void setUseGeodeticDistances(bool v) {m_UseGeodeticDistances=v;}

	float getMaxMovement() const {return m_MaxMovement;}
	void setMaxMovement(float v) {m_MaxMovement=v;}

	float getPruningThreshold() const {return m_PruningThreshold;}
	void setPruningThreshold(float v) {m_PruningThreshold=v;}

	bool getValidateCorrespondences() const {return m_ValidateCorrespondences;}
	void setValidateCorrespondences(bool v) {m_ValidateCorrespondences=v;}

	float32 getValidate_ConfSigma() const {return m_Validate_ConfSigma;}
	void setValidate_ConfSigma(float32 v) {m_Validate_ConfSigma=v;}

	float32 getValidate_ConflictScale() const {return m_Validate_ConflictScale;}
	void setValidate_ConflictScale(float32 v) {m_Validate_ConflictScale=v;}

	float32 getValidate_ConflictAdd() const {return m_Validate_ConflictAdd;}
	void setValidate_ConflictAdd(float32 v) {m_Validate_ConflictAdd=v;}

	float32 getTPS_Gamma() const {return m_TPS_Gamma;}
	void setTPS_Gamma(float32 v) {m_TPS_Gamma=v;}

	float32 getTPS_q() const {return m_TPS_q;}
	void setTPS_q(float32 v) {m_TPS_q=v;}

	float32 getTPS_Alpha_Tolerance() const {return m_TPS_Alpha_Tolerance;}
	void setTPS_Alpha_Tolerance(float32 v) {m_TPS_Alpha_Tolerance=v;}

	float32 getSelfConfidenceSigma() const {return m_SelfConfidenceSigma;}
	void setSelfConfidenceSigma(float32 v) {m_SelfConfidenceSigma=v;}

	float32 getSelfConfidenceScale() const {return m_SelfConfidenceScale;}
	void setSelfConfidenceScale(float32 v) {m_SelfConfidenceScale=v;}

	card32 getNumDescriptorClusters() const {return m_NumDescriptorClusters;}
	void setNumDescriptorClusters(card32 v) {m_NumDescriptorClusters=v;}

	card32 getNumDescriptorSpaceCells() const {return m_NumDescriptorSpaceCells;}
	void setNumDescriptorSpaceCells(card32 v) {m_NumDescriptorSpaceCells=v;}

	bool getUseExistingCorrespondenceAsValid() const {return m_UseExistingCorrespondenceAsValid;}
	void setUseExistingCorrespondenceAsValid(bool v) {m_UseExistingCorrespondenceAsValid=v;}

	float32 getMinConfidence() const {return m_MinConfidence;}
	void setMinConfidence(float32 v) {m_MinConfidence=v;}

};

}

#endif