#ifndef RegistrationToolsH
#define RegistrationToolsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include <animationReconstruction.h>

//---------------------------------------------------------------------------

namespace geoxl
{

/// calculates curvature for each point. curvature is stored in destAttribute
/// which has to be a 2-float32 channel (H,K).
/// averagePointDistance will be calculated if not specified
void ANIMATIONRECONSTRUCTION_API calculateCurvature( PointCloud * pc, int numNeighbours, string destAttribute,
								float averagePointDistance=0.0f);

void ANIMATIONRECONSTRUCTION_API calculateCurvature( PointCloud * pc, float32 sigma, string destAttribute );

/// calculates world space gradients
/// destAttrbute has to be a 3f channel
/// averagePointDistance will be calculated if not specified
void calculateGradients( PointCloud * pc, int numNeighbours, string destAttribute, 
								string sourceAttribute="",card32 sourceCompIndex=0,float averagePointDistance=0.0f,
								string positionAttribute="position");

/// calculates geodetic distances between startPoint and every other point
/// when paths is specified, all geodetic paths are returned
vector<float> calculateGeodeticDistances( PointCloud * pc,PointCloudIterator * topologyIter, 
													  const Vector3f startPoint, const vector<Vector3f> points,
													  vector< vector<Vector3f> > * paths = NULL);


/// creates a gray-channel from color-channel
void createGrayChannel( PointCloud * pc );

/// creates a noisy copy of the input point cloud
/// noise: new_point = point + normal * max(min(gaussianNoise(stdDeviation),maxAmplitude),-maxAmplitude)
PointCloud * createNoisyPointCloud( PointCloud * pc, float stdDeviation, float maxAmplitude );

}

#endif