//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCAddOutliers.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointCloud.h"
#include "VertexDescriptor.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PCCAddOutliers ,0) {
	BEGIN_CLASS_INIT( PCCAddOutliers );
	ADD_CARD32_PROP(numOutliers, 0)
	ADD_VECTOR3F_PROP(bb.lowerCorner, 0)
	ADD_VECTOR3F_PROP(bb.upperCorner, 0)
}

// ----

PCCAddOutliers::PCCAddOutliers() {
   numOutliers = 5000;
   bb.lowerCorner = makeVector3f(-1,-1,-1);
   bb.upperCorner = makeVector3f( 1, 1, 1);
}

void PCCAddOutliers::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

PCCAddOutliers::~PCCAddOutliers() {
}




void PCCAddOutliers::execute(Scene *scene) 
{
   vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
	for ( unsigned m = 0; m < selection.size(); m++ ){
		PointCloud *pc = getPointCloud( scene, selection[m] );
      SceneObjectIterator *it = pc->createIterator(SOIT::CAP_DYNAMIC_PC);
      if (it == NULL) {
         warning("selected object is not dynamic");
         continue;
      }
      const VertexDescriptor *vd = pc->getDescr();
      if (!vd->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
         warning("no position channel found");
         continue;
      }
      char *buffer = new char[vd->getSize()];
      ZeroMemory(buffer, vd->getSize());
      AAT posAAT = vd->getToken("position");
      VAD posVAD = vd->getAttribute("position");
      
      DynamicPointCloudIterator *din = dynamic_cast<DynamicPointCloudIterator*>(it);
      for (card32 i=0; i<numOutliers; i++) {
         Vector3f v = makeVector3f(rnd01(), rnd01(), rnd01());
         v[0] *= bb.upperCorner[0] - bb.lowerCorner[0];
         v[1] *= bb.upperCorner[1] - bb.lowerCorner[1];
         v[2] *= bb.upperCorner[2] - bb.lowerCorner[2];
         v += bb.lowerCorner;
         posVAD.setAnyAttrib((card8*)buffer, makeVector4d(v[0], v[1], v[2], 0), posAAT);
         din->add(buffer);
      }
      
      delete[] buffer;
      delete it;      
	}
}
