//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCAnimationCreator.h"
//---------------------------------------------------------------------------
#include "FixedArrayClassProperty.h"
#include "StringClassProperty.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra_templCode.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "StringHelper.h"
#include "SGSimpleAnimationNode.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------
IMPLEMENT_GEOXL_CLASS( PCCAnimationCreator, 3 )
{
	BEGIN_CLASS_INIT( PCCAnimationCreator )
	ADD_STRING_PROP( referenceNode, 0 )
	ADD_STRING_PROP( animNodeName, 0 )
	ADD_VECTOR3F_PROP( translation, 0 )
	ADD_CARD32_PROP( numFrames, 0 )
	ADD_FLOAT32_PROP( angleX, 1 )
	ADD_FLOAT32_PROP( angleY, 1 )
	ADD_FLOAT32_PROP( angleZ, 1 )
	ADD_BOOLEAN_PROP( selectionOnly, 2 )
	ADD_BOOLEAN_PROP( offsetCopy, 3 )
	ADD_VECTOR3F_PROP( baseOffset, 3 )
	ADD_VECTOR3F_PROP( offsetDirection, 3 )
	ADD_FLOAT32_PROP( offsetAmplitude, 3 )
	ADD_FLOAT32_PROP( offsetPhase, 3 )
	ADD_FLOAT32_PROP( offsetPhaseSpeed, 3 )
}

/// Constructor
PCCAnimationCreator::PCCAnimationCreator()
{
	referenceNode = "root/pc1";
	animNodeName = "root/data";
	translation = makeVector3f( 0.0f, 0.0f, 0.0f ); 
	numFrames = 16;
	angleX = 0.0f;
	angleY = 3.0f;
	angleZ = 0.0f;
	selectionOnly = false;
   offsetCopy = false;
	baseOffset = makeVector3f(1.55f, 0,0);
	offsetDirection = makeVector3f(1.0f, 0,0);
	offsetAmplitude = 0.5f;
	offsetPhase = (float32)(M_PI);
	offsetPhaseSpeed = (float32)(M_PI/16.0);
}

/** 
 * Destructor
 */
PCCAnimationCreator::~PCCAnimationCreator()
{

}

/** 
 * Execute command
 */
void PCCAnimationCreator::execute( Scene *scene )
{
	// Find animation node
	SGAnimationNode* animNode = getAnimationNode( scene );
	if ( animNode == NULL ){
		// Determine name
		QString name = QString::fromStdString( animNodeName );
		card32 index = name.indexOf( "root/" );
		if ( index >= 0 ){
			name = name.right( name.length() - 5 );
		}
		// Create new animation node
		animNode = new SGSimpleAnimationNode();
		animNode->setName( qString2STLString(name) );
		scene->getRootNode()->addChildNode( animNode );
	}
	// Delete all existing children
	for ( unsigned int childIndex = 0; childIndex < animNode->getNumChildNodes( scene->getRootState() ); childIndex++ ){
		animNode->getChildNode( scene->getRootState(), childIndex );	
	}

	// Get reference point cloud
	PointCloud* referencePC = getReferencePointCloud( scene );
	if ( referencePC == NULL ){
		debugOutput << "execute: invalid reference point cloud.\n";
		debugOutput.flush();
		return;
	}
	if ( !referencePC->providesAttribute( "position" ) ||
		 !referencePC->providesAttribute( "flags" ) ){
		debugOutput << "execute: reference point cloud misses attribute.\n";
		debugOutput.flush();
	}

	Matrix3f rotationX = makeMatrix3f( 1, 0,              0,
									   0, cos( angleX/180.0f*M_PI ),  sin( angleX/180.0f*M_PI ),
									   0, -sin( angleX/180.0f*M_PI ), cos( angleX/180.0f*M_PI ) );
	Matrix3f rotationY = makeMatrix3f( cos( angleY/180.0f*M_PI ), 0, -sin( angleY/180.0f*M_PI ),
									   0,             1, 0,
									   sin( angleY/180.0f*M_PI ), 0, cos( angleY/180.0f*M_PI ) );
	Matrix3f rotationZ = makeMatrix3f( cos( angleZ/180.0f*M_PI ), sin( angleZ/180.0f*M_PI ), 0,
									   sin( angleZ/180.0f*M_PI ), cos( angleZ/180.0f*M_PI ), 0,
									   0,             0,             1 );

	Matrix3f R = makeMatrix3f( 1,0,0,0,1,0,0,0,1 );
	Vector3f t = makeVector3f( 0,0,0 );
	for ( unsigned int frameIndex = 0; frameIndex < numFrames; frameIndex++ ){
		R = rotationZ * rotationY * rotationX * R;
		t = t + translation;
		
		UnstructuredInCorePointCloud* pc = new UnstructuredInCorePointCloud();
		VertexDescriptor vd;
		vd.pushAttrib( mVAD( "position", 3, VAD::DATA_FORMAT_FLOAT32 ) );
		vd.pushAttrib( mVAD( "flags", 1, VAD::DATA_FORMAT_INT32 ) );
		pc->clearAndSetup( &vd, referencePC->getNumPoints()*(offsetCopy?2:1) );
		AAT POSITION = pc->getAAT( "position" );

		SceneObjectIterator *newItSO = referencePC->createIterator( SOIT( SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC ) );
		BasicPointCloudIterator* basicIt = dynamic_cast<BasicPointCloudIterator*>( newItSO );
		ModifyablePointCloudIterator* modIt = dynamic_cast<ModifyablePointCloudIterator*>( newItSO );
		AAT REFERENCE_POSITION = referencePC->getAAT( "position" );
		AAT REFERENCE_FLAGS = referencePC->getAAT( "flags" );
		basicIt->reset();
		card32 pointIndex = 0;
		while( !basicIt->atEnd() ){
			Vector3f pos = basicIt->get3f( REFERENCE_POSITION );
			Vector3f transformation = translation * frameIndex;
			if ( basicIt->get1i( REFERENCE_FLAGS ) != 0 || !selectionOnly ){
				// Modify
				pc->getPointSet()->set3f( pointIndex, POSITION, R * pos + t );
			} else {
				// Keep
				pc->getPointSet()->set3f( pointIndex, POSITION, pos );
			}
			pointIndex++;
			basicIt->next();
		}

      if (offsetCopy) {
         card32 pBIndex = referencePC->getNumPoints();
		   basicIt->reset();
		   pointIndex = 0;
		   while( !basicIt->atEnd() ){
			   Vector3f pos = basicIt->get3f( REFERENCE_POSITION );
			   Vector3f transformation = translation * frameIndex;
			   if ( basicIt->get1i( REFERENCE_FLAGS ) != 0 || !selectionOnly ){
				   // Modify
				   float32 offW = offsetAmplitude*sin(frameIndex*offsetPhaseSpeed + offsetPhase);
				   pc->getPointSet()->set3f( pBIndex + pointIndex, POSITION, R * (pos + t + baseOffset + offsetDirection*offW));
			   } else {
				   // Keep
				   pc->getPointSet()->set3f( pBIndex + pointIndex, POSITION, pos );
			   }
			   pointIndex++;
			   basicIt->next();
		   }
      }
		delete newItSO;

		// Add newly created point cloud to animation node
		SGObjectNode* frame0Node = new SGObjectNode();
		frame0Node->clearAndSetup( pc );
		char buffer[3];
		std::string frameName = std::string( "frame" ) + itoa( frameIndex, buffer, 10 );
		frame0Node->setName( frameName );
		animNode->addChildNode( frame0Node );
	}

}

/**
* Get the data animation node
*/
SGAnimationNode* PCCAnimationCreator::getAnimationNode( Scene* scene )
{
	// Find data node
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	SGAnimationNode* animNode = NULL;
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	while ( !it->atEnd() ){
		if ( it->getNameWithPath() == animNodeName ){
			animNode = dynamic_cast<SGAnimationNode*>( it->getNode() );
			if ( animNode != NULL ){
				break;            
			}
		}
		it->next();
	}
	delete it;
	return animNode;
}

/** 
 * Get reference point cloud
 */
PointCloud* PCCAnimationCreator::getReferencePointCloud( Scene* scene )
{
	return getPointCloud( scene, referenceNode );
}
