//---------------------------------------------------------------------------
#ifndef PCCAnimationCreatorH
#define PCCAnimationCreatorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
#include "LinearAlgebra.h"
#include "DebugOutput.h"
#include "PCCommand.h"
#include "Scene.h"
#include "PointCloud.h"
#include "SGAnimationNode.h"
//---------------------------------------------------------------------------

namespace geoxl {


/// Creates an animation from a single frame; select part of a point cloud and define translation and number of frames to be created in 'animNodeName'
class ANIMATIONRECONSTRUCTION_API PCCAnimationCreator : public PCCommand 
{
	GEOXL_CLASS( PCCAnimationCreator )

	private:
		/// Name of node containing reference point cloud
		string referenceNode;
		/// Name of node containing result animation
		string animNodeName;
		/// Translation vector applied at each frame
		Vector3f translation;
		/// Number of frames to be created
		card32 numFrames;
		/// Rotation angles
		float32 angleX, angleY, angleZ;
		/// modifies only selected portions
		bool selectionOnly;
		/// creates a second copy moving in sin waves
		bool offsetCopy;
		Vector3f baseOffset;
		Vector3f offsetDirection;
		float32 offsetAmplitude;
		float32 offsetPhase;
		float32 offsetPhaseSpeed;

		/// Get the data animation node
		SGAnimationNode* getAnimationNode( Scene* scene );
		/// Get reference point cloud
		PointCloud* getReferencePointCloud( Scene* scene );

	public:
		/// Constructor
		PCCAnimationCreator();
		/// Destructor
		~PCCAnimationCreator();
		/// Execute command
		virtual void execute(Scene *scene);
};


}

#endif
 