//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCConvertPointCloudTypeInList.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "StringClassProperty.h"
#include "ClassTypeClassProperty.h"
#include "PointCloudTools.h"
#include "SGListNode.h"
#include "SGObjectNode.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PCCConvertPointCloudTypeInList , 0 ) {
	BEGIN_CLASS_INIT( PCCConvertPointCloudTypeInList );
	ADD_STRING_PROP(targetList, 0)
	ADD_METACLASS_PROP( newType, 0, PointCloud::getClass() );
	ADD_STRING_PROP(newPrefix, 0)
}

// ----

PCCConvertPointCloudTypeInList::PCCConvertPointCloudTypeInList() {
	targetList = "root/data";
	newType = UnstructuredInCorePointCloud::getClass();
	newPrefix = "conv_";
}

void PCCConvertPointCloudTypeInList::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

PCCConvertPointCloudTypeInList::~PCCConvertPointCloudTypeInList() {
}




void PCCConvertPointCloudTypeInList::execute(Scene *scene)
{
   vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
   if (selection.size() == 0) {
      warning("PCCAnimRename::execute() - nothing selected.");
      return;
   }
   SceneGraphState *myState = scene->getRootState()->createTraversalState();
   myState->staticState->defaultTraversalMode = SGStaticState::tmAllNodes;
   myState->dynamicState->traversalMode = SGDynamicState::tmAllNodes;
   for (card32 i=0; i<selection.size(); i++) {
      SceneGraphNode *node = getSceneGraphNodeByName(scene, selection[i]);
      if (node == NULL) {
         warning("PCCAnimRename::execute() - inconsistency in selection (cannot find node).");
         continue;
      }
      max_platform_card numNodes = node->getNumChildNodes(myState);
      for (max_platform_card n=0; n<numNodes; n++) {
         SceneGraphNode *currentNode = node->getChildNode(myState, n);
         if (dynamic_cast<SGObjectNode*>(currentNode)) {
				SceneObject *obj = dynamic_cast<SGObjectNode*>(currentNode)->getSceneObject();
				PointCloud *pc = dynamic_cast<PointCloud*>(obj);
				if (pc) {
					SceneObjectIterator *it = obj->createIterator(SOIT::CAP_BASIC_PC);
					BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
					PointCloud *tmp = (PointCloud*)(newType->newInstance());
					PointCloud *target = tmp->appendFields(pc->getDescr());
					delete tmp;
					pAssert(pc->getDescr()->equal(target->getDescr()));
					SceneObjectIterator *tit = target->createIterator(SOIT::CAP_DYNAMIC_PC);
					DynamicPointCloudIterator *dtit = dynamic_cast<DynamicPointCloudIterator*>(tit);
					if (!tit) {
						delete it;
						throw ECommandFailed("newType is not dynamic.");
					}
					if (it) {
						pAssert(bit);
						pAssert(dtit);
						char *buffer = new char[pc->getDescr()->getSize()];
						while (!bit->atEnd()) {
							bit->getVertex(buffer);
							bit->next();
							dtit->add(buffer);
						}
						delete[] buffer;
					}
					delete it;
					delete tit;
					
					SGObjectNode *newNode = new SGObjectNode();
					newNode->clearAndSetup(target);
					newNode->setName(newPrefix + currentNode->getName());
					
					if (!node->addChildNode(newNode)) {
						delete newNode;
						throw ECommandFailed("targetList does not accept new child nodes.");
					}
					
				}
         }
      }      
   }   
   delete myState;

}
