#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCPrepareAnimPointCloud.h"
//---------------------------------------------------------------------------
#include "StringClassProperty.h"
#include "NumericalClassProperty.h"
#include "PointCloudTools.h"
#include "UnstructuredInCorePointCloud.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( PCCPrepareAnimPointCloud ,0)
{
	BEGIN_CLASS_INIT( PCCPrepareAnimPointCloud );
   ADD_FLOAT32_PROP(minIntensity, 0)
   ADD_BOOLEAN_PROP(deleteZeroPosPoints, 0)
   ADD_BOOLEAN_PROP(deleteLowIntensityPoints, 0)
}

PCCPrepareAnimPointCloud::PCCPrepareAnimPointCloud() {
   minIntensity = 10000.0f;
   deleteZeroPosPoints = true;
   deleteLowIntensityPoints = true;
}

PCCPrepareAnimPointCloud::~PCCPrepareAnimPointCloud() {
}

void PCCPrepareAnimPointCloud::execute(Scene *scene) {
   vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
   if (selection.size() == 0) {
      warning("PCCPrepareAnimPointCloud::execute() - nothing selected.");
      return;
   }
   for (unsigned i=0; i<selection.size(); i++) {
      PointCloud *pc = getPointCloud(scene, selection[i]);
      if (pc == NULL) {
         warning("PCCPrepareAnimPointCloud::execute() - selected node is not a point cloud.");
         continue;
      }
      UnstructuredInCorePointCloud *newpc = new UnstructuredInCorePointCloud();
      newpc->clearAndSetup(pc->getDescr(), 0);
      newpc->assignSettings(pc);
      SceneObjectIterator *it = pc->createIterator(SOIT::CAP_BASIC_PC);
      BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(it);
      SceneObjectIterator *targetIt = newpc->createIterator(SOIT::CAP_DYNAMIC_PC);
      DynamicPointCloudIterator *dit = dynamic_cast<DynamicPointCloudIterator*>(targetIt);
      if (!bit) {delete it; continue;}
      char *buffer = new char[pc->getDescr()->getSize()];
      AAT posAAT; bool usePos;
      AAT intensityAAT; bool useIntensity;
      if (pc->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
         usePos = true;
         posAAT = pc->getAAT("position");
      }
      if (pc->getDescr()->providesAttribute("intensity", 1, VAD::DATA_FORMAT_FLOAT32)) {
         useIntensity = true;
         intensityAAT = pc->getAAT("intensity");
      }
      while (!bit->atEnd()) {
         bool doit = true;
         if (usePos && deleteZeroPosPoints) {
            if (bit->get3f(posAAT) == NULL_VECTOR3F) {
               doit = false;
            }
         }
         if (useIntensity && deleteLowIntensityPoints) {
            if (bit->get1f(intensityAAT) < minIntensity) {
               doit = false;
            }
         }
         if (doit) {
            bit->getVertex(buffer);
            dit->add(buffer);
         }
         bit->next();
      }
      delete[] buffer;
      delete targetIt;
      delete it;
      
      pc->clear();

      SceneObjectIterator *finalTargetIt = pc->createIterator(SOIT::CAP_DYNAMIC_PC);
      DynamicPointCloudIterator *finalDit = dynamic_cast<DynamicPointCloudIterator*>(finalTargetIt);
      PointSet *ps = newpc->getPointSet();
      
      for (card32 i=0; i<ps->getNumEntries();i++) {
         finalDit->add(ps->getDataPointer(i));
      }
      
      delete finalTargetIt;
   }
}
