//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCResamplePointCloud.h"
//---------------------------------------------------------------------------
#include "UniformPointCloudResampling.h"
#include "SceneObjectIterator.h"
#include "PointCloudIterators.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS( PCCResamplePointCloud ,0)
{
	BEGIN_CLASS_INIT( PCCResamplePointCloud );
	ADD_FLOAT32_PROP(maxSpacing, 0)
}


// --


PCCResamplePointCloud::PCCResamplePointCloud() {
	maxSpacing = 0.1f;
}

void PCCResamplePointCloud::rekursiveDownsampling(HierarchicalIterator *hit, SceneObjectIterator *it) {
	if (hit->isLeaf()) {
		ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(it);
		pAssert(modIt);
		BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(it);
		pAssert(bIt);
		PointSet *ps = modIt->lockBlockReadWrite();
		UniformPointCloudResampling resampler;
		resampler.setup(maxSpacing);
		PointSet *newPS = resampler.resamplePointSet(ps);
		ps->assign(newPS);
		bIt->unlock();
	} else {
		card32 numChildren = hit->getNumChildren();
		for (card32 i=0; i<numChildren; i++) {
			if (hit->goToChild(i)) {
				rekursiveDownsampling(hit, it);
				bool s = hit->goToParent();
				pAssert(s);
			}
		}
	}	
}

void PCCResamplePointCloud::execute(Scene *scene) {
   unsigned int m;
   vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
   for ( m = 0; m < selection.size(); m++ ){
      PointCloud *pc = getPointCloud( scene, selection[m] );
		if (pc != NULL) {
			SceneObjectIterator *it = pc->createIterator(SOIT::CAP_HIERARCHICAL || SOIT::CAP_MODIFYABLE_PC || SOIT::CAP_BASIC_PC);
			if (it) {
				HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(it);
				pAssert(hit);
				rekursiveDownsampling(hit, it);
				delete it;
			}
		}
   }
	
}
