//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIAnimationBSR.h"
//---------------------------------------------------------------------------
#include "ObjectClassProperty.h"
#include "SeparatorClassProperty.h"
#include "ClassMethods.h"
#include "Scene.h"
#include "SelectionTools.h"
#include "DebugOutput.h"
#include "SceneGraphState.h"
#include "GeometricAlignment.h"
//#include "ReconstructionInitialization.h"
#include "StandardSceneGraphIterator.h"
#include "PointSet_templCode.h"
#include "SGObjectNode.h"
#include "ProgressWindow.h"
#include "PatchGraphTopologicalResampling.h"
#include "BSROptContext.h"
#include "FeatureRegistration.h"
#include "GeometricAlignment.h"
#include "BSROptConfig.h"
#include "InterfaceGlobals.h"
#include "PatchTypes.h"
#include "PGNeighborhoodExtrapolator4D.h"
#include "UrshapeCreator.h"
#include "PGDescriptorFactory.h"
#include "PatchGraphMerge2.h"
#include "PGOpt2Debugging.h"
#include "StringHelper.h"
#include "InterfaceGlobals.h"
#include "PGOpt2Processing.h"
#include "PatchGraphCleanupOptionsDialog.h"
#include "PointSetTools.h"
#include "PGOpt2RemoveLowDegreeVertices.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
#include <time.h>
#include <stdio.h>
#include <dos.h>
#include <QtGui/QInputDialog>
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS( PCIAnimationBSR, 6 )
{
	BEGIN_CLASS_INIT( PCIAnimationBSR )
	ADD_SEPARATOR( "Settings" )
	ADD_OBJECT_PROP( context, 0, AnimationBSRContext::getClass(), true )
	ADD_SEPARATOR( "Reconstruction" )
	ADD_OBJECT_PROP( extractor, 0, PatchGraphExtract::getClass(), true )
	ADD_OBJECT_PROP( featureRegistration, 1, FeatureRegistration::getClass(), true );
	ADD_OBJECT_PROP( mergeCandidateSelecter, 0, PatchGraphMergeCandidateSelect::getClass(), true )
	ADD_OBJECT_PROP( merger, 0, PatchGraphMerge::getClass(), true )
	ADD_OBJECT_PROP( merger2, 6, PatchGraphMerge2::getClass(), true )
	ADD_OBJECT_PROP( triangleMesher, 4, PatchGraphTriangleMesh::getClass(), true );
	ADD_SEPARATOR( "Debugging" )
	ADD_OBJECT_PROP( animBSRRenderer, 0, AnimationBSRRenderer::getClass(), true ); 
	ADD_OBJECT_PROP( debugger, 0, AnimationBSRDebugger::getClass(), true ); 
	ADD_OBJECT_PROP( descriptorChannels, 5, PGDescriptorChannels::getClass(), true );
	ADD_NOARGS_METHOD( loadScene )
	ADD_NOARGS_METHOD( saveScene )
	ADD_NOARGS_METHOD( extract )
	//ADD_NOARGS_METHOD( autoSelectNextMerge )
	ADD_NOARGS_METHOD( applyNextMerge )
	//ADD_NOARGS_METHOD( applyMerges )
	ADD_NOARGS_METHOD( mergeRange )
	ADD_NOARGS_METHOD( globalOpt )
	//ADD_NOARGS_METHOD( resample )
	//ADD_NOARGS_METHOD( downsample )
	//ADD_NOARGS_METHOD( extrapolate )
	//ADD_NOARGS_METHOD( createUrshapes )
	ADD_NOARGS_METHOD( descriptorTest )
	ADD_NOARGS_METHOD( recomputeTopology )
	ADD_NOARGS_METHOD( resampleContainer )
	ADD_NOARGS_METHOD( recomputePGFromData )
	ADD_NOARGS_METHOD( removeUnsupported )
}

PatchGraphCleanupOptionsDialog *PCIAnimationBSR::cleanupDialog = NULL;

/**
 * Constructor
 */
PCIAnimationBSR::PCIAnimationBSR()
{
   if (cleanupDialog == NULL) {
      cleanupDialog = new PatchGraphCleanupOptionsDialog(InterfaceGlobals::getMainWindow(), Qt::Dialog | Qt::MSWindowsFixedSizeDialogHint);
   }
	context = new AnimationBSRContext();

	// Extract
	extractor = new PatchGraphExtract();
	extractor->setContext( context );

	// Merge
	merger = new PatchGraphMerge();
	merger->addAlignMergeExtrapolateStep(new GeometricAlignment());
	merger->setContext( context );

   merger2 = new PatchGraphMerge2();

	// Merge candidates
	mergeCandidateSelecter = new PatchGraphMergeCandidateSelect();
	mergeCandidateSelecter->setContext( context );

	// Post processing: triangle mesh
	triangleMesher = new PatchGraphTriangleMesh();
	triangleMesher->setContext( context );

	// Rendering
	animBSRRenderer = new AnimationBSRRenderer();
	animBSRRenderer->setContext( context );
	animBSRRenderer->setSelecter( mergeCandidateSelecter );
	animBSRRenderer->setMerger( merger );

	// Debugging
	debugger = new AnimationBSRDebugger();
	debugger->setExtractor( extractor );
	debugger->setContext( context );
	debugPainter = NULL;
	drawMode = dmNormal;

	featureRegistration = new FeatureRegistration;
	featureRegistration->setAnimationBSRContext( context );

	descriptorChannels = new PGDescriptorChannelsTest;

   // Settings for mw
	animBSRRenderer->setShowAllPatchGraphs( false );
	mergeCandidateSelecter->setCand1( 0 );
	mergeCandidateSelecter->setCand1( 1 );
	//context->setUseOldPipeline(false);
	
	// Settings for pj
	context->setSaveAfterEachMerge( true );
}

/**
 * Destructor
 */
PCIAnimationBSR::~PCIAnimationBSR()
{
	delete context;
	delete extractor;
	delete merger;
	delete merger2;
	delete mergeCandidateSelecter;
	delete animBSRRenderer;
	delete debugger;
	delete featureRegistration;
	delete triangleMesher;
	delete descriptorChannels;
}

/** 
 * Extract patch graphs from input point cloud
 */
void PCIAnimationBSR::extract()
{
	extractor->tensorVoteAllFrames();
	extractor->extractPatchGraphAllFrames();
	context->getPGContainer()->computeBoundingBoxes();
	mergeCandidateSelecter->select();
}

/** 
 * Apply next merge operation
 */
void PCIAnimationBSR::applyNextMerge()
{
	// Select next merge candidates
	if ( context->getVerbose() ) { debugOutput << "Merging:\n"; debugOutput.flush(); }
   if (context->getUseOldPipeline()) {
   	merger->merge_optimized2(context, featureRegistration, mergeCandidateSelecter->getCand1(), mergeCandidateSelecter->getCand2(), this, true);
   } else {
      merger2->mergeTwoFrameRanges(context, mergeCandidateSelecter->getCand1(), mergeCandidateSelecter->getCand2(), context->getPGContainer(), this, animBSRRenderer);
   }
	if ( context->getVerbose() ) { debugOutput << " " << context->getPGContainer()->getNumPatchGraphs() << " patch graphs remaining.\n"; debugOutput.flush(); }
   mergeCandidateSelecter->select();
}

/** 
 * Apply several merge steps (numMergeSteps in context)
 */
void PCIAnimationBSR::applyMerges()
{
	float32 start = clock();
	progressWindow->pushStep( true, "Merging" );
	progressWindow->progress( 0 );	
	if ( context->getNumMergeSteps() >= 0 ){
		mergeCandidateSelecter->select();
		for ( int32 mergeStep = 0; mergeStep < context->getNumMergeSteps(); mergeStep++ ){
			debugOutput << "Merging step " << ( mergeStep + 1 ) << "/" << context->getNumMergeSteps() << ":\n"; debugOutput.flush();

         if (context->getUseOldPipeline()) {
		      if ( !merger->merge_optimized2( context, featureRegistration, mergeCandidateSelecter->getCand1(), mergeCandidateSelecter->getCand2(), this, context->getVerbose() ) ){
			      debugOutput << "Error during merge operation.\n"; debugOutput.flush();			
			      break;
		      }
         } else {
		      merger2->mergeTwoFrameRanges( context, mergeCandidateSelecter->getCand1(), mergeCandidateSelecter->getCand2(), context->getPGContainer(), this, animBSRRenderer );
         }

			debugOutput << " " << context->getPGContainer()->getNumPatchGraphs() << " patch graphs remaining.\n"; debugOutput.flush();
			mergeCandidateSelecter->select();
			progressWindow->progress( card32( float32( mergeStep + 1 ) * 100.0f / float32( context->getNumMergeSteps() ) ) );
		}
	} else {
		card32 numSteps = context->getPGContainer()->getNumPatchGraphs() - abs( context->getNumMergeSteps() );;
		card32 step = 0;
		while( int32( context->getPGContainer()->getNumPatchGraphs() ) > abs( context->getNumMergeSteps() ) ){
			debugOutput << "Merging step " << ( step + 1 ) << "/" << numSteps << ".\n"; debugOutput.flush();
			mergeCandidateSelecter->select();


			if (context->getUseOldPipeline()) {
				if ( !merger->merge_optimized2( context, featureRegistration, mergeCandidateSelecter->getCand1(), mergeCandidateSelecter->getCand2(), this, context->getVerbose() ) ){
					debugOutput << "Error during merge operation.\n"; debugOutput.flush();			
					break;
				}
			} else {
				merger2->mergeTwoFrameRanges( context, mergeCandidateSelecter->getCand1(), mergeCandidateSelecter->getCand2(), context->getPGContainer(), this, animBSRRenderer );
			}

			debugOutput << " " << context->getPGContainer()->getNumPatchGraphs() << " patch graphs remaining.\n"; debugOutput.flush();
			progressWindow->progress( card32( float32( step + 1 ) * 100.0f / float32( numSteps ) ) );
			step++;

			if ( context->getSaveAfterEachMerge() ){
				context->savePGContainer( "mergeSave" + QString::number( step ) + ".pgc.object" );
			}
		}
	}
	debugOutput << "Merge: time required: " << ( clock() - start ) / CLK_TCK << ".\n"; debugOutput.flush();
	progressWindow->popSteps( );	
}

/** 
 * Merge a range of frames
 */
void PCIAnimationBSR::mergeRange()
{
	PatchGraphContainer *pgc = context->getPGContainer();
	bool ok;
	int lower = QInputDialog::getInteger( 
		InterfaceGlobals::getMainWindow(), 
		"mergeRange()", "Select lower frame index", 
		0, 0, pgc->getAnimSeq()->getNumFrames()-1, 1, &ok, 0
	);
	if (!ok) return;
	int upper = QInputDialog::getInteger( 
		InterfaceGlobals::getMainWindow(), 
		"mergeRange()", "Select upper frame index", 
		pgc->getAnimSeq()->getNumFrames()-1, 0, pgc->getAnimSeq()->getNumFrames()-1, 1, &ok, 0
	);
	if (!ok) return;

   if (QMessageBox::question(InterfaceGlobals::getMainWindow(), "Perform initial smoothing?", 
                             "Apply initial optimization (smoothing) to all patch graphs in range?", QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes) {
      bool storeUrshapeFlag = context->getOptGraph2useUrshape();
      context->setOptGraph2useUrshape(false);

      PointCloudAnimationSequence *animSeq = context->getPGContainer()->getAnimSeq();
      for (card32 i=0; i<context->getPGContainer()->getNumPatchGraphs(); i++) {
         PatchGraph *pg = context->getPGContainer()->getPatchGraph(i);
         if ((int)(pg->getStartFrameIndex())>upper || (int)(pg->getStartFrameIndex()+pg->getTrajectorySet()->getNumFrames()-1)<lower) continue;
	      merger2->performGlobalOptOnly(context, pg, animSeq, this, animBSRRenderer);
      }   
      
      context->setOptGraph2useUrshape(storeUrshapeFlag);
   }

	vector< vector<pair<card32, card32> > > merges;
	int32 step = 2;
	int level = 0;
	do {
		merges.push_back( vector<pair<card32, card32> >() );
		for (int i=lower; (i+(step/2))<=upper; i+=step) {
			pair<card32, card32> p(i, i+step/2);
			merges[level].push_back(p);
		}
		level++;
		step *= 2;
	} while (step/2 <= upper);
   
	if ( context->getVerbose() ){ debugOutput << "Range merge: operations:\n"; debugOutput.flush(); }
	card32 numMerges = 0;
	for (card32 x=0; x<merges.size(); x++) {
		if ( context->getVerbose() ){ debugOutput << " "; debugOutput.flush(); }
		for (card32 y=0; y<merges[x].size(); y++) {
			if ( context->getVerbose() ){ debugOutput << merges[x][y].first << ","<< merges[x][y].second << "   "; }
			numMerges++;
		}
		if ( context->getVerbose() ){ debugOutput <<"\n"; }
	}
   
	Timer timer;
   
	card32 merge = 0;
	progressWindow->pushStep( true, "Merging" );
	progressWindow->progress( 0 );	

	for (card32 x=0; x<merges.size(); x++) {
		for (card32 y=0; y<merges[x].size(); y++) {
			if ( context->getVerbose() ) { debugOutput << "Range merge: step " << (merge + 1 ) << "/" << numMerges << ":\n"; debugOutput.flush(); }
			int32 pg1 = pgc->getAnyPatchGraphIndexWithStartFrame(merges[x][y].first);
			int32 pg2 = pgc->getAnyPatchGraphIndexWithStartFrame(merges[x][y].second);
			if (pg1 == -1 || pg2 == -1) continue;
			if (context->getUseOldPipeline()) {
      			merger->merge_optimized2(context, featureRegistration, pg1, pg2, this, true);
      		} else {
      			merger2->mergeTwoFrameRanges(context, pg1, pg2, context->getPGContainer(), this, animBSRRenderer);
      		}
			debugOutput << " " << context->getPGContainer()->getNumPatchGraphs() << " patch graphs remaining.\n"; debugOutput.flush();

			progressWindow->progress( card32( float32( merge + 1 ) * 100.0f / float32( numMerges ) ) );
			merge++;

			if ( context->getSaveAfterEachMerge() ){
				context->savePGContainer( "mergeSave" + QString::number( merge ) + ".pgc.object" );
			}
		}
	}
   
	progressWindow->popSteps( );	
	debugOutput << "Finished range merge. Overall time: " << timer.getDeltaValue()/1000.0f << "sec.\n\n";
}

/** 
* Automatically select next merge candidates
*/
void PCIAnimationBSR::autoSelectNextMerge()
{
	// Select next merge candidates
	mergeCandidateSelecter->select();
	PatchGraph* pg = context->getPGContainer()->getPatchGraph( mergeCandidateSelecter->getCand1() );
	if ( pg != NULL ){ 
		scene->getRootState()->dynamicState->time = pg->getStartFrameIndex();
	}
}

void PCIAnimationBSR::redrawView(void *context) {
   PCIAnimationBSR *cThis = (PCIAnimationBSR*)context;
	cThis->redrawScene();
}

/**
 * Currently: Just for debugging...
 */
void PCIAnimationBSR::globalOpt() {
	if (context->getPGContainer()->getNumPatchGraphs() == 0) {
		warning("no pgs found");
		return;
	}
	bool ok = true;
	int pgIndex = QInputDialog::getInteger( 
		InterfaceGlobals::getMainWindow(), 
		"globalOpt()", "Select index of patch graph to optimize, use -1 for all patch graphs", 
		-1, -1, (int)(context->getPGContainer()->getNumPatchGraphs())-1, 1, &ok, 0
	);
	if (!ok) return;
	PointCloudAnimationSequence *animSeq = context->getPGContainer()->getAnimSeq();
	if (pgIndex >=0) {
		PatchGraph *pg = context->getPGContainer()->getPatchGraph(pgIndex);
		merger2->performGlobalOptOnly(context, pg, animSeq, this, animBSRRenderer);
   } else {
		for (pgIndex=0; pgIndex<(int)(context->getPGContainer()->getNumPatchGraphs()); pgIndex++) {
			PatchGraph *pg = context->getPGContainer()->getPatchGraph(pgIndex);
			merger2->performGlobalOptOnly(context, pg, animSeq, this, animBSRRenderer);
		}
   }
}

void	PCIAnimationBSR::resample()
{
	if ( context->getVerbose() ) { debugOutput << "Merging:\n"; debugOutput.flush(); }
	if ( context->getVerbose() ) { debugOutput << " Using settings: " << AnimationBSRContext::getContextDataSetDescr( context->getDataSet() ) << ".\n"; debugOutput.flush(); }

	PatchGraph	*pg = context->getPGContainer()->getPatchGraph(0);
	if(pg == NULL)
		return;

	PatchGraphTopologicalResampling	*resampler = new PatchGraphTopologicalResampling();
	PatchGraph	*newpg = resampler->fullResampling_Modified(pg, 2*context->getSamplingSpacing(), 2.0f, 8, 1);
	pg->assign(newpg);
	delete resampler;
	delete newpg;
}

void	PCIAnimationBSR::downsample()
{
	if ( context->getVerbose() ) { debugOutput << "Merging:\n"; debugOutput.flush(); }
	if ( context->getVerbose() ) { debugOutput << " Using settings: " << AnimationBSRContext::getContextDataSetDescr( context->getDataSet() ) << ".\n"; debugOutput.flush(); }

	PatchGraph	*pg = context->getPGContainer()->getPatchGraph(0);
	if(pg == NULL)
		return;

	PatchGraphTopologicalResampling	*resampler = new PatchGraphTopologicalResampling();
	PatchGraph	*newpg = resampler->fullDownsampling(pg, context->getSamplingSpacing(), 8);
	pg->assign(newpg);
	delete resampler;
	delete newpg;
}

void PCIAnimationBSR::extrapolate() {
	if (context->getPGContainer()->getNumPatchGraphs() == 0) {
		warning("no pgs found");
		return;
	}
	bool ok = true;
	int pgIndex = QInputDialog::getInteger( 
		InterfaceGlobals::getMainWindow(), 
		"globalOpt()", "Select index of patch graph to extrapolate:", 
		0, 0, (int)(context->getPGContainer()->getNumPatchGraphs())-1, 1, &ok, 0
	);
	if (!ok) return;
	PatchGraph *pg = context->getPGContainer()->getPatchGraph(pgIndex);
		
	PGNeighborhoodExtrapolator4D::extrapolateNeighbors(context, pg);
}

void PCIAnimationBSR::createUrshapes() {
	PatchGraphContainer *pgc = context->getPGContainer();
	const card32 numPGs = pgc->getNumPatchGraphs();
	for (card32 i=0; i<numPGs; i++) {
		PatchGraph *pg = pgc->getPatchGraph(i);
		UrshapeCreator::checkCreateAvUrshape(pg);
	}
}

void PCIAnimationBSR::descriptorTest() {
	if( descriptorChannels == NULL )
	{
		message( "create a descriptorChannels object first!" );
		return;
	}
   message("PCIAnimationBSR::descriptorTest() - computing a\n"
           "descriptor for every point in the patch graph.\n"
           "the result PCA-compressed to three dimensions\n"
           "and stored in the color channel of the patch graph.\n\n");

	Timer timer;
	card32 startTime = timer.getCurrentValue();

	PatchGraphContainer *pgc = context->getPGContainer();
	PointCloudAnimationSequence *data = context->getPGContainer()->getAnimSeq();
	const card32 numPGs = pgc->getNumPatchGraphs();

	vector<PatchGraph*> allpg;
	for (card32 i=0; i<numPGs; i++) {
      pgc->getPatchGraph(i)->createPatchAttribute(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	   allpg.push_back(pgc->getPatchGraph(i));
   }

   descriptorChannels->computeDescriptors(allpg, data, "descriptor", 3);
   //PGDescriptorChannelsTest testcode;
   //testcode.computeDescriptors(allpg, data, "descriptor", 3);

   Vector3f minv = makeVector3f( 1e30f, 1e30f, 1e30f);
   Vector3f maxv = makeVector3f(-1e30f,-1e30f,-1e30f);
	for (card32 i=0; i<numPGs; i++) {
		PatchGraph *pg = pgc->getPatchGraph(i);
      pg->createPatchAttribute(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
      PointSet *ps = pg->getTrajectorySet()->getTrajectories();
      AAT COL = ps->getAAT("color");
      AAT DESCR = ps->getAAT("descriptor");
      const card32 numPoints = ps->getNumEntries();
      for (card32 p=0; p<numPoints; p++) {
         Vector3f t = ps->get3f(p, DESCR);
         minv[0] = min(minv[0], t[0]);
         maxv[0] = max(maxv[0], t[0]);
         minv[1] = min(minv[1], t[1]);
         maxv[1] = max(maxv[1], t[1]);
         minv[2] = min(minv[2], t[2]);
         maxv[2] = max(maxv[2], t[2]);
      }
	}
	for (card32 i=0; i<numPGs; i++) {
		PatchGraph *pg = pgc->getPatchGraph(i);
      PointSet *ps = pg->getTrajectorySet()->getTrajectories();
      AAT COL = ps->getAAT("color");
      AAT DESCR = ps->getAAT("descriptor");
      const card32 numPoints = ps->getNumEntries();
      for (card32 p=0; p<numPoints; p++) {
         Vector3f c = ps->get3f(p, DESCR);
         c -= minv;
         c[0] /= max(maxv[0] - minv[0], 1e-5f);
         c[1] /= max(maxv[1] - minv[1], 1e-5f);
         c[2] /= max(maxv[2] - minv[2], 1e-5f);
         ps->set3f(p, COL, c);
      }
	}

	debugOutput << "descriptor calculation: " << timer.getDeltaValue()/1000 << "s\n";

	featureRegistration->calculateAllFeatures_Clustering();

	debugOutput << "complete feature preprocessing: " << (timer.getCurrentValue() - startTime) / 1000 << "s\n";
	
	/*PointSet * colorPosPS = new PointSet;
	VertexDescriptor * vd = new VertexDescriptor;
	vd->pushAttrib( VAD("position", 3, VAD::DATA_FORMAT_FLOAT32 ) );
	colorPosPS->clearAndSetup( 1, 0, vd );
	AAT posAAT = colorPosPS->getAAT( "position" );

	for( card32 i=0;i<data->getNumFrames();i++ )
	{
		AAT colorAAT = data->getFrame(i)->getAAT( "color" );
		for( card32 j=0;j<data->getFrame(i)->getNumEntries();j++ )
		{
			Vector3f vec = data->getFrame(i)->get3f(j, colorAAT );
			colorPosPS->addPoint( (char*)&vec[0] );
		}
	}

	UnstructuredInCorePointCloud * pc = new UnstructuredInCorePointCloud;
	pc->getPointSet()->assign( colorPosPS );

	addPointCloud( getScene(), pc, "testrgb" );
*/

}


/** 
* Get number of frames
*/
card32 PCIAnimationBSR::getNumFrames() const
{
	SGSimpleAnimationNode* animDataNode = getDataAnimationNode();
	if ( animDataNode == NULL ){
		return 0;
	}
	return animDataNode->getNumChildNodes( scene->getRootState() );
}

/**
* Get the data animation node
*/
SGSimpleAnimationNode* PCIAnimationBSR::getDataAnimationNode() const
{
	// Find data node
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	SGSimpleAnimationNode* animDataNode = NULL;
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	while ( !it->atEnd() ){
		if ( it->getNameWithPath() == "root/data" ){
			animDataNode = dynamic_cast<SGSimpleAnimationNode*>( it->getNode() );
			if ( animDataNode != NULL ){
				break;            
			}
		}
		it->next();
	}
	delete it;
	return animDataNode;
}

/**
* Return index of current frame
*/
int32 PCIAnimationBSR::getCurrentFrameIndex() const
{
	return scene->getRootState()->dynamicState->time;
}

/**
* Get data point cloud for frame index
*/
UnstructuredInCorePointCloud* PCIAnimationBSR::getDataFrame( card32 index )
{
	return dynamic_cast<UnstructuredInCorePointCloud*>( getPointCloud( scene, getDataFrameName( index ) ) ); 
}

/** 
* Load patch graph container
*/
void PCIAnimationBSR::loadScene()
{
	QString filename = QFileDialog::getOpenFileName( NULL, "Choose a file", "D:/LocalProject/data/Objects/AnimationReconstruction", "Patch graph containers (*.object)" );
	if ( !filename.isNull() ){
		loadSceneFromFile( filename );

      pAssert(cleanupDialog);
      bool ok = cleanupDialog->exec();
      PatchGraphCleanupSettings settings = cleanupDialog->getSettings();
      if (ok) {
   		preparePGContainer(context->getPGContainer(), settings);
	      if (settings.outputValidityGraph) {
	         for (card32 i=0; i<context->getPGContainer()->getNumPatchGraphs(); i++) {
		         PGOpt2Debugging::outputValidityCheck(context->getPGContainer()->getPatchGraph(i), intToStr(i));
	         }
	      }
      }

      mergeCandidateSelecter->setCand1(0);
      mergeCandidateSelecter->setCand1(1);
	}
}

/** 
* Load patch graph container from given filename
*/
void PCIAnimationBSR::loadSceneFromFile( const QString& filename )
{
	context->loadPGContainer( filename );
	if (context->getPGContainer() != NULL) {
	   //for ( unsigned int n = 0; n < context->getPGContainer()->getNumPatchGraphs(); n++ ){
		   //PatchGraph* pg = context->getPGContainer()->getPatchGraph( n );
		   //if ( pg->getTrajectorySet()->getNumFrames() <= 1 ){
			//   extractor->detectAndRemoveIsolatedVertices( pg );
		   //}
	   //}
	   context->getPGContainer()->computeBoundingBoxes();
	   //mergeCandidateSelecter->select();
	}
}

/** 
* Save patch graph container
*/
void PCIAnimationBSR::saveScene()
{
	QString filename = QFileDialog::getSaveFileName( NULL, "Choose a file", "D:/LocalProject/data/Objects/AnimationReconstruction", "Patch graph containers (*.object)" );
	if ( !filename.isNull() ){
		context->savePGContainer( filename );
	}
}

void PCIAnimationBSR::glDrawTool(GLContext *glContext)
{
	featureRegistration->debugOnly_Scene = getScene();
	if (drawMode == dmNormal) {
		// Call visualization functionality
		animBSRRenderer->setCurrentFrame( getCurrentFrameIndex() );
		animBSRRenderer->render();
		triangleMesher->drawGL();
	} else {
		if (debugPainter != NULL) {
			debugPainter->debugDraw();
		}
	}
}

void PCIAnimationBSR::performRedraw(const DebugRenderFunction *painter) {
   drawMode = dmDebug;
   debugPainter = painter;
   try {
      redraw();   
   } catch (...) {
      debugPainter = NULL;
      drawMode = dmNormal;
      throw;
   }
   debugPainter = NULL;
   drawMode = dmNormal;
}


/**
* User interaction
*/
void PCIAnimationBSR::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
}

/*** PCInteractable ***/
void PCIAnimationBSR::assign(const Object* obj, CopyContext *context)
{
	PCInteractionTool::assign(obj,context);
	const PCIAnimationBSR * o = dynamic_cast<const PCIAnimationBSR*>(obj);
	pAssert( o != NULL );
}

/**
* User interaction
*/
void PCIAnimationBSR::mouseMoved(int32 x, int32 y) 
{
}

/**
* User interaction
*/
void PCIAnimationBSR::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
}

/**
* User interaction
*/
void PCIAnimationBSR::areaResize(card32 width, card32 height) 
{
}

/**
* User interaction
*/
void PCIAnimationBSR::keyDown(GeneralKey key) 
{
	if ( key.getkeyAlphaNumKey() == 'A' ){
	} else if ( key.getkeyAlphaNumKey() == 'B' ){
	}
	redraw();
}

/**
* User interaction
*/
void PCIAnimationBSR::keyUp(GeneralKey key) 
{
}

void PCIAnimationBSR::connectToSceneImpl(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget, QWidget* toolWidget)
{
	debugger->setScene( scene );
	animBSRRenderer->setScene( scene );
	extractor->setScene( scene );
	triangleMesher->setScene( scene );
}

void PCIAnimationBSR::activateTool() 
{

}

void PCIAnimationBSR::deactivateTool() 
{

}


void PCIAnimationBSR::preparePGContainer(PatchGraphContainer *cont, PatchGraphCleanupSettings &settings) {
	debugOutput << "\npreparePGContainer\n";
	const card32 numPGs = cont->getNumPatchGraphs();
	for (card32 pgi=0; pgi<numPGs; pgi++) {
		PatchGraph *pg = cont->getPatchGraph(pgi);
		PointSet *ps = pg->getTrajectorySet()->getTrajectories();
		const card32 width = ps->getWidth();
		const card32 height = ps->getHeight();
		const AAT flagsAAT = ps->getAAT("flags");
		const AAT normalAAT = ps->getAAT("normal");
		const AAT tangentUAAT = ps->getAAT("tangent_u");
		card32 uninit = 0;
		card32 unnormNormal = 0;
		card32 unnormTangent = 0;
		for (card32 p=0; p<height; p++) {
			for (card32 f=0; f<width; f++) {
				int32 flags = ps->get<int>(f,p, flagsAAT);
				Vector3f normal = ps->get<float32, 3>(f,p, normalAAT);
				Vector3f tangent_u = ps->get<float32, 3>(f,p, tangentUAAT);
				if (!(flags & FLAG_PATCH_INITIALIZED)) {
					uninit++;
					if (settings.initPatches) ps->set<int>(f,p, flagsAAT, flags | FLAG_PATCH_INITIALIZED);
				} 
				if (fabs(1.0f-norm(normal)) > 0.1f) {
					unnormNormal++;
				} else if (fabs(1.0f-norm(tangent_u)) > 0.1f) {
					Vector3f newTangentU = makeOrthonormalVector(normal);
					if (settings.fixTangents) ps->set<float32, 3>(f,p, tangentUAAT, newTangentU);
					unnormTangent++;
				}
			}
		}
		if (unnormNormal > 0) {
		   if (settings.fixNormals) {
			   debugOutput << string("patch graph ")+intToStr(pgi)+string(": some patches do not contain normals - recomputing.");
			   PGOpt2Processing_Normal_Estimation::estimateNormals(pg);
			   PGOpt2Processing_Normal_Orientation::unifyNormals(pg);
			 } else {
			   debugOutput << string("patch graph ")+intToStr(pgi)+string(": some patches do not contain normals (not corrected).");
			 }
		}
		debugOutput << "Patch Graph #" << pgi << ":\n";
		debugOutput << "points flagged as uninitialized: " << uninit << "\n";
		debugOutput << "unnormalized normals: " << unnormNormal << "\n";
		if (unnormNormal == 0) {
			debugOutput << "unnormalized tangents: " << unnormTangent << "\n";
		}
	}
	if (settings.removeOutliers) {
	   const float32 maxDataDistance = context->getSamplingSpacing() * context->getDataPointsInfluence();
	   debugOutput << "removing outliers - using maximum distance: " << maxDataDistance << " which is computed as context->getSamplingSpacing() * context->getDataPointsInfluence().\n\n";
	   
	   PointCloudAnimationSequence *data = cont->getAnimSeq();
	   const card32 numDataFrames = data->getNumFrames();
	   const card32 numPGs = cont->getNumPatchGraphs();
	   for (card32 f=0; f<numDataFrames; f++) {
	      PatchGraph *pg = NULL;
	      
	      for (card32 i=0; i<numPGs; i++) {
	         PatchGraph *candPG = cont->getPatchGraph(i);
	         if (candPG->getStartFrameIndex()<=f && candPG->getStartFrameIndex()+candPG->getTrajectorySet()->getNumFrames() > f) {
	            if (pg != NULL) {
	               warning("PCIAnimationBSR::preparePGContainer() [removeOutliers] - two patch graphs cover the same data frame: ignored, please reextract pgs with latest version.");
	               continue;
	            } else {
	               pg = candPG;
	            }
	         }
         }	 
		  // Check if patch graph exists for data frame (might be false with some datasets)
		  if ( pg != NULL ){
			 pAssert(f>=pg->getStartFrameIndex());
			 const card32 pgFrame = f - pg->getStartFrameIndex();
			 PointSet *psPatchesFrame = pg->getTrajectorySet()->copySingleFrame(pgFrame);
			 AttachedIndexedOctree *octree = pg->getTrajectorySet()->fetchOctree(pgFrame);
			 AAT patchesPosAAT = psPatchesFrame->getAAT("position");
			 PointSetKNNQuery knn(psPatchesFrame, octree, NULL_VECTOR3F, patchesPosAAT);
	         
			 PointSet *psData = data->getFrame(f);
			 const card32 numDataPoints = psData->getNumEntries();
			 AAT dataPosAAT = psData->getAAT("position");
			 vector<bool> keepPoint(numDataPoints);

			 card32 delStatistics = 0;
			 for (card32 d=0; d<numDataPoints; d++) {
				Vector3f dataPos = psData->get3f(d, dataPosAAT);
				knn.setTargetPoint(dataPos);
				card32 nextIndex;
				bool found;
				knn.getNextNearestPointIndex(nextIndex, found);
				if (found) {
				   Vector3f patchPos = psPatchesFrame->get3f(nextIndex, patchesPosAAT);
				   if (norm(patchPos - dataPos) <= maxDataDistance) {
					   keepPoint[d] = true;
				   } else {
					   keepPoint[d] = false;
					   delStatistics++;
				   }
				} else {
				   keepPoint[d] = false;
				   delStatistics++;
				}
			 }
			 removePoints(psData, keepPoint);
			 debugOutput << "frame: "<< f <<" - removed " << delStatistics << " out of " << numDataPoints << " data points.\n";
			 debugOutput.flush();
	         
			 delete psPatchesFrame;
		  }
	   }
	}
}

static void setAllValid(PatchGraph *pg) {
   PointSet *ps = pg->getTrajectorySet()->getTrajectories();
   const card32 n = ps->getNumEntries();
   const AAT aatFlags = ps->getAAT("flags");
   for (card32 i=0; i<n; i++) {
      ps->set1i(i, aatFlags, ps->get1i(i, aatFlags) | FLAG_PATCH_INITIALIZED);
   }
}

void PCIAnimationBSR::recomputeTopology() 
{
	if (context->getPGContainer()->getNumPatchGraphs() == 0) {
		warning("no pgs found");
		return;
	}
	bool ok = true;
	int pgIndex = QInputDialog::getInteger( 
		InterfaceGlobals::getMainWindow(), 
		"globalOpt()", "Select index of patch graph to optimize, use -1 for all patch graphs", 
		-1, -1, (int)(context->getPGContainer()->getNumPatchGraphs())-1, 1, &ok, 0
	);
	if (!ok) return;
	PointCloudAnimationSequence *animSeq = context->getPGContainer()->getAnimSeq();
	if (pgIndex >=0) {
		PatchGraph *pg = context->getPGContainer()->getPatchGraph(pgIndex);
		setAllValid(pg);
		merger2->recomputeTopologyOnly(context, pg);
   } else {
		for (pgIndex=0; pgIndex<(int)(context->getPGContainer()->getNumPatchGraphs()); pgIndex++) {
			PatchGraph *pg = context->getPGContainer()->getPatchGraph(pgIndex);
			setAllValid(pg);
   		merger2->recomputeTopologyOnly(context, pg);
		}
   }
}

void PCIAnimationBSR::resampleContainer() 
{
   bool ok = true;
	int startFrame = QInputDialog::getInteger( 
		InterfaceGlobals::getMainWindow(), 
		"resampleContainer()", "Please enter start index of frame range to be kept:", 
		0, 0, 10000000, 1, &ok, 0
	);
	if (!ok) return;            
	int endFrame = QInputDialog::getInteger( 
		InterfaceGlobals::getMainWindow(), 
		"resampleContainer()", "Please enter end index of frame range to be kept:", 
		(int)(context->getPGContainer()->getNumPatchGraphs())-1, 0, 10000000, 1, &ok, 0
	);
	if (!ok) return;
	int mod = QInputDialog::getInteger( 
		InterfaceGlobals::getMainWindow(), 
		"resampleContainer()", "Please enter modulus (every n-th frame is kept):", 
		1, 1, 10000000, 1, &ok, 0
	);
	if (!ok) return;
	vector<int> keep;
	for (int pgIndex=0; pgIndex<(int)(context->getPGContainer()->getNumPatchGraphs()); pgIndex++) {
		PatchGraph *pg = context->getPGContainer()->getPatchGraph(pgIndex);
      int startFrameIndex = pg->getStartFrameIndex();
      if (startFrameIndex >= startFrame && startFrameIndex <= endFrame) {
         int rel = startFrameIndex - startFrame;
         if (mod <= 1 || ((rel % mod) == 0)) {
            keep.push_back(pgIndex);
         }
      }      
	}
   PatchGraphContainer *pgCont = new PatchGraphContainer();
   PatchGraphContainer *old = context->getPGContainer();
   PointCloudAnimationSequence *animSeq = context->getPGContainer()->getAnimSeq();
   card32 frameNumber = 0;
	for (unsigned i=0; i<keep.size(); i++) {
      PatchGraph *pg = old->getPatchGraph(keep[i]);
      int start = pg->getStartFrameIndex();
      int end = (int)pg->getStartFrameIndex() + (int)pg->getTrajectorySet()->getNumFrames();
      pg->setStartFrameIndex(frameNumber);
      pgCont->addPatchGraph(pg);
      old->setNULL(keep[i]);
      for (int i=start; i<end; i++) {
         pgCont->getAnimSeq()->addFrame(animSeq->getFrame(i));
         animSeq->setNULL(i);
      }
      frameNumber += pg->getTrajectorySet()->getNumFrames();
	}
	context->setPGContainer(pgCont);
}

void PCIAnimationBSR::recomputePGFromData() 
{
	PointCloudAnimationSequence *anim = context->getPGContainer()->getAnimSeq();
	for (int pgIndex=0; pgIndex<(int)(context->getPGContainer()->getNumPatchGraphs()); pgIndex++) {
		if (context->getVerbose()) {
			debugOutput << "recomputePG pg " << pgIndex << "\n";
			debugOutput.flush();
		}
		PatchGraph *pg = context->getPGContainer()->getPatchGraph(pgIndex);
		pg->getTrajectorySet()->invalidateAllAttachments();
		PointSet *ps = pg->getTrajectorySet()->getTrajectories();
		card32 numFrames = ps->getWidth();
		if (numFrames != 1) {
			warning("works only on single frame pgs.");
			continue;
		}
		card32 startFrame = pg->getStartFrameIndex();
		pg->getTrajectorySet()->getTrajectories()->changeHeight(anim->getFrame(startFrame)->getHeight());
		pg->getTrajectorySet()->setSingleFrame(0, anim->getFrame(startFrame));
		ps = pg->getTrajectorySet()->getTrajectories();
		card32 n = ps->getNumEntries();
		AAT flagsAAT = ps->getAAT("flags");
		for (card32 i=0; i<n; i++) {
			ps->set1i(i, flagsAAT, FLAG_PATCH_INITIALIZED);
		}		
		merger2->resampleOnly(context, pg);
	}
}

void PCIAnimationBSR::removeUnsupported() 
{
	if (context->getPGContainer()->getNumPatchGraphs() == 0) {
		warning("no pgs found");
		return;
	}
	bool ok = true;
	int pgIndex = QInputDialog::getInteger( 
		InterfaceGlobals::getMainWindow(), 
		"removeUnsupported()", "Select index of patch graph edit, use -1 for all patch graphs", 
		-1, -1, (int)(context->getPGContainer()->getNumPatchGraphs())-1, 1, &ok, 0
	);
	if (!ok) return;
	int support = QInputDialog::getInteger( 
		InterfaceGlobals::getMainWindow(), 
		"removeUnsupported()", "specify min. number of frames with data supporting a surfel needed to keep the surfel", 
		1, 1, 1000, 1, &ok, 0
	);
	if (!ok) return;
	PointCloudAnimationSequence *animSeq = context->getPGContainer()->getAnimSeq();
	if (pgIndex >=0) {
		PatchGraph *pg = context->getPGContainer()->getPatchGraph(pgIndex);
      PGOpt2RemoveLowDegreeVertices::removeUnsupported(context, support, pg, animSeq);
   } else {
		for (pgIndex=0; pgIndex<(int)(context->getPGContainer()->getNumPatchGraphs()); pgIndex++) {
			PatchGraph *pg = context->getPGContainer()->getPatchGraph(pgIndex);
         PGOpt2RemoveLowDegreeVertices::removeUnsupported(context, support, pg, animSeq);
		}
   }
}
