//---------------------------------------------------------------------------
#ifndef PCIAnimationBSRH
#define PCIAnimationBSRH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
#include "PointCloud.h"
#include "RenderingTool.h"
//#include "ReconstructionDefinitions.h"
//#include "PointCloudTools.h"
#include "PatchGraphRenderer.h"
#include "PatchGraphExtract.h"
#include "SGSimpleAnimationNode.h"
#include "PatchGraphMerge.h"
#include "PatchGraphBSROpt.h"
#include "PatchGraphMerge.h"
#include "PatchGraphMergeCandidateSelect.h"
#include "AnimationBSRContext.h"
#include "AnimationBSRRenderer.h"
#include "AnimationBSRDebugger.h"
#include <qtcore/qobject.h>
#include "PGOperationsDebugging.h"
#include "PatchGraphTriangleMesh.h"
//---------------------------------------------------------------------------

namespace geoxl {

class FeatureRegistration;
class BSROptConfig;
class PGDescriptorChannels;
class PatchGraphMerge2;
class PatchGraphCleanupOptionsDialog;
struct PatchGraphCleanupSettings;

/// Extract patch graphs from point clouds based on tensor voting
class ANIMATIONRECONSTRUCTION_API PCIAnimationBSR : public PCInteractionTool, 
                                                    public DebugRenderer // mw - added this functionality for easier testing; see PGOperationsDebugging.h
{
	GEOXL_CLASS( PCIAnimationBSR )

	private:
	   // --- persistent properties
		/// Context
		AnimationBSRContext* context;
		/// Extractor
		PatchGraphExtract* extractor;
		/// Merge Tool
		PatchGraphMerge* merger;
		PatchGraphMerge2* merger2;
		/// Select the next candidate for merging
		PatchGraphMergeCandidateSelect* mergeCandidateSelecter;
		/// Rendering and visualization tool
		AnimationBSRRenderer* animBSRRenderer;
		/// Debugging functionality
		AnimationBSRDebugger* debugger;
		/// registration tool 
		FeatureRegistration* featureRegistration;
		/// Triangle mesh post processing
		PatchGraphTriangleMesh* triangleMesher;

		PGDescriptorChannels * descriptorChannels;

      // --- temp. properties
      /// temporary object for displaying debug events
      const DebugRenderFunction *debugPainter;
      /// controls display of debugging events (internal)
      enum DrawMode {dmNormal, dmDebug};
      /// controls display of debugging events (internal)
      DrawMode drawMode;

		static void redrawView(void *context);
		static PatchGraphCleanupOptionsDialog *cleanupDialog;

		void preparePGContainer(PatchGraphContainer *cont, PatchGraphCleanupSettings &settings);


	public:
		/*** CLASS MANAGEMENT ***/
		/// Constructor
		PCIAnimationBSR();
		// Destructor
		~PCIAnimationBSR();

		/*** MAIN OPERATIONS ***/
		/// Extract patch graphs from input point cloud
		void extract();
		/// Automatically select next merge candidates
		void autoSelectNextMerge();
		/// Apply next merge operation
		void applyNextMerge();
		/// merge a specified range of frames
		void mergeRange();
		/// Apply several merge steps (numMergeSteps in context; negative values indicate goal number of patch graphs)
		void applyMerges();

		/*** SCENE MANAGEMENT ***/
		/// Save patch graph container
		void saveScene();
		/// Load patch graph container
		void loadScene();
		
		/*** DEBUGGING ***/
		void globalOpt();
		void resample();
		void downsample();
		void extrapolate();
		void createUrshapes();
		void descriptorTest();
		void recomputeTopology();
		void resampleContainer();
		void recomputePGFromData();
		void removeUnsupported();
		
		/*** INPUT DATA ACCESS ***/
		/// Get number of frames
		card32 getNumFrames() const;
		/// Get the data animation node
		SGSimpleAnimationNode* getDataAnimationNode() const;
		/// Return index of current frame
		int32 getCurrentFrameIndex() const;
		/// Get data point cloud for frame index
		UnstructuredInCorePointCloud* getDataFrame( card32 index );
		/// Get name of data frame point cloud 
		string getDataFrameName( card32 index ){ char buffer[10]; return std::string( "root/data/frame" ) + itoa( index, buffer, 10 ); }
		/// Load patch graph container from given filename
		void loadSceneFromFile( const QString& filename );

		/*** PCINTERACTION ***/
		virtual void assign(const Object* obj, CopyContext *context = NULL);
		virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
		virtual void mouseMoved(int32 x, int32 y);
		virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
		virtual void areaResize(card32 width, card32 height);
		virtual void keyDown(GeneralKey key);
		virtual void keyUp(GeneralKey key);
		virtual void connectToSceneImpl(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget, QWidget* toolWidget);
		virtual void activateTool();
		virtual void deactivateTool();
		virtual void glDrawTool(GLContext *glContext);
      virtual bool forceCamByDefault() {return true;}
		void redraw(){ redrawScene(); }

      /// can be called by suboperations for debugging purpose (show single steps & progress)
      virtual void performRedraw(const DebugRenderFunction *painter);
};

}

#endif