//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIPatchGraphBSR.h"
//---------------------------------------------------------------------------
#include "Scene.h"
#include "DebugOutput.h"
#include "BinaryObjectStreams.h"
#include "Monomials2ndOrder.h"
#include "FileDialogs.h"
#include "InterfaceGlobals.h"
#include "SeparatorClassProperty.h"
#include "ObjectClassProperty.h"
#include "BooleanClassProperty.h"
#include "NumericalClassProperty.h"
#include "ClassMethods.h"
#include "ImplicitPatchGraphMeshing.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "UnstructuredInCorePointCloud.h"
#include "PatchGraph.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "TrajectorySet.h"
#include "PGDataPointIndices.h"
#include "PointCloudAnimationSequence.h"
#include "TopologyAttachment.h"
#include "InCorePCTopologyGraph.h"
#include "UnifyNormals.h"
#include "PatchGraphTopologicalResampling.h"
#include "ColorTools.h"
#include "Random.h"
#include "MultiResolutionPGOpt.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
#include <qtcore/qfileinfo.h>
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PCIPatchGraphBSR, 1 )
{
	BEGIN_CLASS_INIT( PCIPatchGraphBSR )
	ADD_SEPARATOR("scene")
	ADD_OBJECT_PROP(pgContainer, 0, PatchGraphContainer::getClass(), true)
	ADD_SEPARATOR("computation")
	ADD_OBJECT_PROP(optimizer, 0, PatchGraphBSROpt::getClass(), true)
	ADD_BOOLEAN_PROP(peterOptimize, 1)
   ADD_CARD32_PROP(whichFrame, 0)
	ADD_BOOLEAN_PROP(allFrames, 0)
	ADD_BOOLEAN_PROP(useUrshape, 0)
	ADD_BOOLEAN_PROP(multiResConstrain2ndFrameDataPoints, 0)
	ADD_SEPARATOR("rendering")
	ADD_OBJECT_PROP(patchGraphRenderer, 0, StandardPatchGraphRenderer::getClass(), true)
	ADD_BOOLEAN_PROP(showSteps, 0)
	ADD_SEPARATOR("configuration")

	ADD_BOOLEAN_PROP(ccGaussianPrjLikelihood,					0)
	ADD_BOOLEAN_PROP(ccSimpleICPLikelihood,						0)
	ADD_BOOLEAN_PROP(ccProjectionSmoothness,					0)
   ADD_BOOLEAN_PROP(ccSpringSTSmoothness,					0)
   ADD_BOOLEAN_PROP(ccGreenTensorSTSmoothness,		0)
	ADD_BOOLEAN_PROP(ccPatchLSSmoothing,						0)
	ADD_BOOLEAN_PROP(ccCorrespInterpolTest,						0)
	ADD_SEPARATOR("new configuration")
	ADD_BOOLEAN_PROP(ccSquaredDistancePotential,			0)
	ADD_BOOLEAN_PROP(ccMarkerErrorPotential,						0)
	ADD_BOOLEAN_PROP(ccPlanarSmoothnessPotential,			0)
	ADD_BOOLEAN_PROP(ccNormalSmoothnessPotential,		0)
	ADD_BOOLEAN_PROP(ccCircularSmoothnessPotential,     0)
	ADD_BOOLEAN_PROP(ccLaplacianSmoothnessPotential,	0)
	ADD_BOOLEAN_PROP(ccPositionRigidityPotential,				0)
	ADD_BOOLEAN_PROP(ccNormalRigidityPotential,					0)
	ADD_BOOLEAN_PROP(ccSTPositionCoherence,					0)
	ADD_BOOLEAN_PROP(ccSTNormalCoherence,						0)
	ADD_BOOLEAN_PROP(ccSTRotationCoherence,					0)


	ADD_SEPARATOR("output")
	ADD_OBJECT_PROP(meshing, 0, ImplicitPatchGraphMeshing::getClass(), true)
	ADD_STRING_PROP(outputMeshNode, 0)
	ADD_SEPARATOR("tests & debugging")
	//ADD_STRING_PROP(corrTestSource, 0)
	//ADD_FLOAT32_PROP(probCorr, 0)
	//ADD_FLOAT32_PROP(rotDeg, 0)
	ADD_FLOAT32_PROP(resamplingTestDist, 0)
	ADD_BOOLEAN_PROP(resamplingShowClustering, 0)
	ADD_CARD32_PROP(monitorPoint, 0)
	ADD_BOOLEAN_PROP(monitorShowCoordinateFrames, 0);
	ADD_BOOLEAN_PROP(monitorShowMonomials, 0)
	ADD_FLOAT32_PROP(renderScale, 0)
	
	ADD_NOARGS_METHOD(loadScene)
	ADD_NOARGS_METHOD(saveScene)
	ADD_NOARGS_METHOD(optimize)
	ADD_NOARGS_METHOD(makeMesh)
	//ADD_NOARGS_METHOD(createCorrTest)
	ADD_NOARGS_METHOD(createUrshape)
	ADD_NOARGS_METHOD(resamplingTest)
}

/**
 * Constructor
 */
PCIPatchGraphBSR::PCIPatchGraphBSR()
{
	pgContainer = new PatchGraphContainer();
	patchGraphRenderer = new StandardPatchGraphRenderer();
	patchGraphRenderer->setShowPoints( true );
	patchGraphRenderer->setShowTopology( true );
	patchGraphRenderer->setShowCoords( false );
	patchGraphRenderer->setShowDataPoints( false );
	patchGraphRenderer->setUseDL( false );
	patchGraphRenderer->setShowLocalPatches( false );
	patchGraphRenderer->setShowAllFrames( true );
	optimizer = new PatchGraphBSROpt();
	optimizer->addPotential(new BOPGaussianPrjLikelihood());
   optimizer->addPotential(new BOPPrjSmoothness());
	optimizer->addPotential(new BOPSpringBasedRigidity());
	peterOptimize = true;
   whichFrame = 0;
   allFrames = true;   
   useUrshape = false;
   multiResConstrain2ndFrameDataPoints = false;
   
	// Debugging
	showSteps = true;
	monitorShowCoordinateFrames = false;
	monitorShowMonomials = false;
	rt = new RenderingTool();
	rt->init();
	monitorPoint = 0;
	renderScale = 0.1f;
	meshing = new ImplicitPatchGraphMeshing();
	outputMeshNode = "root/recMesh";
	corrTestSource = "root/pc1";
	probCorr = 0.1f;
	rotDeg = 30;
	resamplingTestDist = 0.25f;
	resamplingShowClustering = true;

	ccGaussianPrjLikelihood					= false;
	ccSimpleICPLikelihood						= false;
	ccProjectionSmoothness					= false;
	ccSpringSTSmoothness					= false;
	ccSquaredDistancePotential			= true;
	ccMarkerErrorPotential						= false;
	ccNormalSmoothnessPotential		= true;
	ccPlanarSmoothnessPotential			= true;
	ccCircularSmoothnessPotential		= false;
	ccLaplacianSmoothnessPotential	= false;
	ccPositionRigidityPotential				= true;
	ccNormalRigidityPotential				= true;
	ccSTPositionCoherence					= true;
	ccSTNormalCoherence						= false;
	ccSTRotationCoherence					= false;
	ccPatchLSSmoothing						= false;
	ccCorrespInterpolTest						= false;
   ccGreenTensorSTSmoothness		= false;

}

/**
* Destructor
*/
PCIPatchGraphBSR::~PCIPatchGraphBSR()
{
	if ( pgContainer != NULL ){
		delete pgContainer;
		pgContainer = NULL;
	}
	if ( patchGraphRenderer != NULL ){
		delete patchGraphRenderer;
		patchGraphRenderer = NULL;
	}
	if ( optimizer != NULL ){
		delete optimizer;
		optimizer = NULL;
	}
	if ( rt != NULL ){
		delete rt;
		rt = NULL;
	}
	delete meshing;
}

void PCIPatchGraphBSR::loadScene() {
	string filename = FileDialogs::getOpenFileName(
		InterfaceGlobals::getMainWindow(), "Import Patch Graph Container", 
      "*.object", NULL, "", FileDialogs::IMPORT_DIR);
	if ( filename != "" ) {
		loadPatchGraphContainer( QString(filename.c_str()) );
		redraw();
	}
}

void PCIPatchGraphBSR::saveScene() {
	string filename = FileDialogs::getSaveFileName(
		InterfaceGlobals::getMainWindow(), "Save Patch Graph Container", 
      "*.object", NULL, "", FileDialogs::IMPORT_DIR);
	if ( filename != "" ) {
		savePatchGraphContainer( QString(filename.c_str()) );
	}
}

void PCIPatchGraphBSR::optimize() {
	 changeConfig();
//	if (peterOptimize) {
		peter_optimize(0);
//	} else {
//		optimize(0);
//	}
}

void PCIPatchGraphBSR::makeMesh() {
	if (pgContainer->getNumPatchGraphs() > 0) {
      UnifyNormals un;
      un.unifyNormals(pgContainer->getPatchGraph(0));
      UnstructuredInCoreTriangleMesh *result = meshing->createMesh(pgContainer->getPatchGraph(0), 0);
      if (getPointCloud(getScene(), outputMeshNode) != NULL) {
   	   replacePointCloud(getScene(), outputMeshNode, result);
   	} else {
   	   addPointCloud(getScene(), result, outputMeshNode);
   	}
	}    
}

void PCIPatchGraphBSR::createCorrTest() {
	UnstructuredInCorePointCloud *test = 
		dynamic_cast<UnstructuredInCorePointCloud*>(getPointCloud(getScene(), corrTestSource));
	if (test != NULL) {
		AttachedDataContainer *attachments = test->getAttachments();
		TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(attachments->getData("TopologyAtt"));
		InCorePCTopologyGraph *tg = NULL;
		if (topAtt != NULL) {
			tg = topAtt->getTopology();
			if (tg == NULL) {
				warning("no topology found");
			   return;
			}
		} else {
			warning("no topology found");
			return;
		}
		if (test) {
			Matrix3f M = makeRotX3f(rotDeg/180.0f*M_PI) * makeRotY3f(rotDeg/180.0f*M_PI);
			const PointSet *ps = test->getPointSet();
			PointSet *copy = (PointSet*)ps->copy();
			PointSet *rot = (PointSet*)ps->copy();
			PGDataPointIndices *dp = new PGDataPointIndices();
			dp->setup(1, rot->getNumEntries());
			AAT POS = rot->getAAT("position");
			card32 n = rot->getNumEntries();
			for (card32 i=0; i<n; i++) {
				if (rnd01() < probCorr) {
					vector<card32> dpi(1);
					dpi[0] = i;
					dp->setIndices(0, i, dpi);
				}
				Vector3f v = rot->get3f(i, POS);
				v = M * v;
				rot->set3f(i, POS, v);
			}
			PointCloudAnimationSequence *data = new PointCloudAnimationSequence();
			data->addFrame(rot);
			delete rot;
			TrajectorySet *ts = new TrajectorySet();
			ts->clearAndSetup(copy, dp);
			PatchGraph *pg = new PatchGraph();
			pg->clearAndSetup(ts, (InCorePCTopologyGraph*)tg->copy());
			pg->createPatchAttribute(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
			pg->createPatchAttribute(mVAD("tangent_u", 3, VAD::DATA_FORMAT_FLOAT32));
			pg->createPatchAttribute(mVAD("hfieldCoeffs", 6, VAD::DATA_FORMAT_FLOAT32));
			pg->createPatchAttribute(mVAD("corrCoeffs", 6, VAD::DATA_FORMAT_FLOAT32));
			PointSet *ps2 = pg->getTrajectorySet()->getTrajectories();
			AAT CCOEFF = ps2->getAAT("corrCoeffs");
			for (card32 i=0; i<n; i++) {
				StaticVector<float32, 6> v;
				v[0] = 0; v[1] = 0; v[2] = 0;
				v[3] = 0; v[4] = 0; v[5] = 0;
				ps2->set<float32, 6>(i, CCOEFF, v);
			}
			
			PatchGraphContainer *cont = new PatchGraphContainer();
			cont->addPatchGraph( pg );
			delete pgContainer;
			pgContainer = cont;
		}
	} else {
		warning("no unstructured in-core point cloud with given name found");
	}
}

void PCIPatchGraphBSR::changeConfig() 
{
	optimizer->clearPotentials();
	if (ccPatchLSSmoothing) 
	{
		optimizer->addPotential(new BOPDataLeastSquares());
      optimizer->getContext()->optMode = BSROptContext::MODE_FIT_COEFFICIENTS;
	}
	if (ccGaussianPrjLikelihood) 
	{
		BOPGaussianPrjLikelihood *bop= new BOPGaussianPrjLikelihood();
		optimizer->addPotential( bop );
      optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	if (ccSimpleICPLikelihood) 
	{
		BOPSimpleICPLikelihood *bop= new BOPSimpleICPLikelihood();
		optimizer->addPotential( bop );
      optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	if (ccProjectionSmoothness) 
	{
		BOPPrjSmoothness *bop= new BOPPrjSmoothness();
		optimizer->addPotential( bop );
      optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	if (ccSpringSTSmoothness) 
	{
		BOPSpringBasedRigidity *bop = new BOPSpringBasedRigidity();
		optimizer->addPotential( bop );
      optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	optimizer->potential_flags.push_back(ccSquaredDistancePotential);
	if(ccSquaredDistancePotential)
	{
		BOPSquaredDistancePotential *bop = new BOPSquaredDistancePotential();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	optimizer->potential_flags.push_back(ccMarkerErrorPotential);
	if(ccMarkerErrorPotential)
	{
		BOPMarkerErrorPotential *bop = new BOPMarkerErrorPotential();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	optimizer->potential_flags.push_back(ccPlanarSmoothnessPotential);
	if(ccPlanarSmoothnessPotential)
	{
		BOPPlanarSmoothnessPotential *bop = new BOPPlanarSmoothnessPotential();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	optimizer->potential_flags.push_back(ccNormalSmoothnessPotential);
	if(ccNormalSmoothnessPotential)
	{
		BOPNormalSmoothnessPotential *bop = new BOPNormalSmoothnessPotential();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	optimizer->potential_flags.push_back(ccCircularSmoothnessPotential);
	if(ccCircularSmoothnessPotential)
	{
		BOPCircularSmoothnessPotential *bop = new BOPCircularSmoothnessPotential();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	optimizer->potential_flags.push_back(ccLaplacianSmoothnessPotential);
	if(ccLaplacianSmoothnessPotential)
	{
		BOPLaplacianSmoothnessPotential *bop = new BOPLaplacianSmoothnessPotential();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	optimizer->potential_flags.push_back(ccPositionRigidityPotential);
	if(ccPositionRigidityPotential)
	{
		BOPPositionRigidityPotential *bop = new BOPPositionRigidityPotential();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	optimizer->potential_flags.push_back(ccNormalRigidityPotential);
	if(ccNormalRigidityPotential)
	{
		BOPNormalRigidityPotential *bop = new BOPNormalRigidityPotential();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	optimizer->potential_flags.push_back(ccSTPositionCoherence);
	if(ccSTPositionCoherence)
	{
		BOPSTPositionCoherence *bop = new BOPSTPositionCoherence();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	optimizer->potential_flags.push_back(ccSTNormalCoherence);
	if(ccSTNormalCoherence)
	{
		BOPSTNormalCoherence *bop = new BOPSTNormalCoherence();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	optimizer->potential_flags.push_back(ccSTRotationCoherence);
	if(ccSTNormalCoherence)
	{
		BOPSTNormalCoherence *bop = new BOPSTNormalCoherence();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
	if(ccSTRotationCoherence)
	{
		BOPSTRotationCoherence *bop = new BOPSTRotationCoherence();
		optimizer->addPotential(bop);
		optimizer->getContext()->optMode = BSROptContext::MODE_MOVE_POINTS;
	}
}

void PCIPatchGraphBSR::createUrshape() {
	PatchGraph* pg = pgContainer->getPatchGraph( 0 );
   pg->createAndClearUrshape(true, 0);
}

void PCIPatchGraphBSR::resamplingTest() {
	PatchGraph* pg = pgContainer->getPatchGraph( 0 );
   PatchGraphTopologicalResampling resampling;
   resampling.setup(true, 0);
   vector< vector<card32> > clusters;
   PatchGraph *newPG = resampling.fullResampling(pg, resamplingTestDist, &clusters);
   if (resamplingShowClustering) {
		pg->createPatchAttribute(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
		PointSet *ps = pg->getTrajectorySet()->getTrajectories();
		AAT COL = ps->getAAT("color");
		const card32 numFrames = ps->getWidth();
		const card32 numPatches = ps->getHeight();
		for (unsigned c=0; c<clusters.size(); c++) {
			Vector3f col = shrink4To3(rainbowGradient(rnd01()));
			for (unsigned pi=0; pi<clusters[c].size(); pi++) {
				const card32 p = clusters[c][pi];
				for (card32 f=0; f<numFrames; f++) {
					ps->set3f(f, p, COL, col);			
				}
			}   
		}
		
	} else {
	   pg->assign(newPG);
	}
	delete newPG;
}

/** 
 * Load a patch graph container
 */
void PCIPatchGraphBSR::loadPatchGraphContainer( const QString filename )
{
	QFileInfo fileInfo( filename );
	if ( fileInfo.absoluteDir().exists() ){
		BinaryInputObjectStream in_str( filename.toLatin1() );
		in_str.readObject( (Persistent*&)pgContainer );
	} else {
		debugOutput << "loadPGContainer: directory does not exist!\n";
		debugOutput.flush();
		return;
	}
	debugOutput << "Loaded patch graph container with " << pgContainer->getNumPatchGraphs() << " patch graphs.\n";
	debugOutput.flush();
}

/** 
* Load patch graph container
*/
void PCIPatchGraphBSR::savePatchGraphContainer( const QString filename )
{
	QFileInfo fileInfo( filename );
	if ( fileInfo.absoluteDir().exists() ){
		BinaryOutputObjectStream out_str( filename.toLatin1() );
		out_str.writeObject( pgContainer );
	} else {
		debugOutput << "savePGContainer: directory does not exist!\n";
		debugOutput.flush();
		return;
	}
	debugOutput << "Saved patch graph container with " << pgContainer->getNumPatchGraphs() << " patch graphs.\n";
	debugOutput.flush();
}

/** 
 * Apply optimization to patch graph 'index' in patch graph container
 */
void PCIPatchGraphBSR::optimize( const card32 index )
{
	PatchGraph* pg = pgContainer->getPatchGraph( index );
	PointCloudAnimationSequence* animSeq = pgContainer->getAnimSeq();
	if ( pg != NULL )
	{
		// TODO: determine index set in a more sophisticated way
		BSROptVariableIndexSet indexSet; indexSet.clear();
		// Used for 3D reconstruction test: use all patches in first frame
		const card32 startFrame = allFrames?0:whichFrame;
		const card32 endFrame = allFrames?pg->getTrajectorySet()->getNumFrames():whichFrame+1;
		for ( card32 frame = startFrame; frame < endFrame; frame++) 
		{
	      for ( unsigned int patchIndex = 0; patchIndex < pg->getTrajectorySet()->getNumPatches(); patchIndex++ )
			{
		      BSROptPointIndex pointIndex;
		      pointIndex.frame = frame;
		      pointIndex.point = patchIndex;
		      indexSet.push_back( pointIndex );
	      }
      }
      if (useUrshape) 
		{
	      for ( unsigned int patchIndex = 0; patchIndex < pg->getTrajectorySet()->getNumPatches(); patchIndex++ )
			{
		      BSROptPointIndex pointIndex;
		      pointIndex.frame = BSROPT_FRAME_URSHAPE;
		      pointIndex.point = patchIndex;
		      indexSet.push_back( pointIndex );
	      }
	      if (!pg->getUrshape()) 
			{
            pg->createAndClearUrshape(true, 0);
         }
      }
      if (optimizer->getContext()->multiGrid >= 1) {
			MultiResolutionPGOpt mg;
			vector<bool> removable;
			vector<bool> *remPtr = NULL;
			if (multiResConstrain2ndFrameDataPoints) {
				PointSet *ps = pg->getTrajectorySet()->getTrajectories();
				if (ps->getWidth() >= 2) {
					removable.resize(ps->getHeight());
					PGDataPointIndices *dp = pg->getTrajectorySet()->getDataPoints();
					for (card32 i=0; i<ps->getHeight(); i++) {
						removable[i] = (dp->getNumDataPoints(1, i) == 0);
					}				
					remPtr = &removable;
				}
			}
			mg.setupAndBuildLowresTopology(animSeq, pg, optimizer->getContext()->lowresSampleSpacing, remPtr, true);
//			if (optimizer->getContext()->multiGrid == 2) {
//				mg.longEdgesOpt(optimizer, &indexSet,  &redrawView, this);
//			} else {
   		mg.twoResolutionOpt(optimizer, &indexSet,  &redrawView, this);
//			}
		} else {
			optimizer->compute( &indexSet, animSeq, pg, &redrawView, this );
		}
		patchGraphRenderer->invalidate();
	} else 
	{
		debugOutput << "optimize: invalid patch graph!"; debugOutput.flush();
	}
}

/************************************************************************/
/* PETER: The following codes are added for testing                                                           */
/************************************************************************/

void	PCIPatchGraphBSR::peter_optimize(const card32 index)
{
	PatchGraph* pg = pgContainer->getPatchGraph( index );
	PointCloudAnimationSequence* animSeq = pgContainer->getAnimSeq();

	if ( pg != NULL )
	{
		// TODO: determine index sets in a more sophisticated way
		// Three index sets: position, normal and rotation
		BSROptVariableIndexSet indexSet_pos;	indexSet_pos.clear();
		BSROptVariableIndexSet indexSet_nor;	indexSet_nor.clear();
		BSROptVariableIndexSet indexSet_rot;	indexSet_rot.clear();
		vector<card32>				indices_data;		indices_data.push_back(0); 

		// Used for 3D reconstruction test: use all patches in first frame
/*		for (card32 point = 0; point < pg->getTrajectorySet()->getNumPatches(); point++ )
		{
			BSROptPointIndex pointIndex;
			pointIndex.frame	= 1;
			pointIndex.point	= point;
			indexSet_pos.push_back(pointIndex);
			indexSet_nor.push_back(pointIndex);
		}
		for (card32 point = 0; point < pg->getTrajectorySet()->getNumPatches(); point++ )
		{
			BSROptPointIndex pointIndex;
			pointIndex.frame	= 0;
			pointIndex.point	= point;
			indexSet_rot.push_back(pointIndex);
		}*/

		PointSet		*ps = pg->getTrajectorySet()->getTrajectories();
		card32		width = ps->getWidth(),
							height = ps->getHeight();

		for(card32 frameIndex = 0; frameIndex < width; frameIndex++)
		{
			indices_data.push_back(frameIndex);
			for(card32 surfelIndex = 0; surfelIndex < height; surfelIndex++)
			{
				BSROptPointIndex pointIndex;
				pointIndex.frame	= frameIndex;
				pointIndex.point	= surfelIndex;
				indexSet_pos.push_back(pointIndex);
				indexSet_nor.push_back(pointIndex);
				if(frameIndex < width -1)
				{
					indexSet_rot.push_back(pointIndex);
				}
			}
		}
			
//		optimizer->optimization(&indexSet_pos, &indexSet_nor, &indexSet_rot, &indices_data,
//														animSeq, pg, 1, &redrawView, this );
		patchGraphRenderer->invalidate();
	} 
	else 
	{
		debugOutput << "optimize: invalid patch graph!"; debugOutput.flush();
	}
}

void	PCIPatchGraphBSR::peter_loadPatchGraphContainer( const QString filename )
{

}

void	PCIPatchGraphBSR::peter_savePatchGraphContainer( const QString filename )
{

}

/************************************************************************/
/* PETER: The above codes are added for testing                                                               */
/************************************************************************/


void PCIPatchGraphBSR::redrawView(void *context) {
   PCIPatchGraphBSR *cThis = (PCIPatchGraphBSR*)context;
   if (cThis->showSteps) {
      cThis->redrawScene();
   }
}

void PCIPatchGraphBSR::glDrawTool(GLContext *glContext)
{
	rt->enableLighting();
	// Render patch graph
	card32 currentFrame = getCurrentFrameIndex();
	if ( pgContainer != NULL ){
		card32 pgCounter = 0;
		for ( card32 patchGraphIndex = 0; patchGraphIndex < pgContainer->getNumPatchGraphs(); patchGraphIndex++ ){
			PatchGraph* pg = pgContainer->getPatchGraph( patchGraphIndex );
			PointCloudAnimationSequence* animSeq = pgContainer->getAnimSeq();
			if ( pg != NULL ){
			card32 localFrameIndex = currentFrame - pg->getStartFrameIndex();
				if ( pg->containsFrame( localFrameIndex ) ){
					patchGraphRenderer->renderGL( pg, animSeq, currentFrame - pg->getStartFrameIndex() );
					AAT POSITION = pg->getTrajectorySet()->getTrajectories()->getAAT( "position" );
					AAT REF_U = pg->getTrajectorySet()->getTrajectories()->getAAT( "tangent_u" );
					AAT REF_N = pg->getTrajectorySet()->getTrajectories()->getAAT( "normal" );
					AAT COEFF = pg->getTrajectorySet()->getTrajectories()->getAAT( "hfieldCoeffs" );
					Vector3f pos = pg->getTrajectorySet()->getTrajectories()->get3f( localFrameIndex, monitorPoint, POSITION );
					if ( monitorShowCoordinateFrames ){
						Vector3f u = pg->getTrajectorySet()->getTrajectories()->get3f( localFrameIndex, monitorPoint, REF_U );
						Vector3f n = pg->getTrajectorySet()->getTrajectories()->get3f( localFrameIndex, monitorPoint, REF_N );
						Vector3f v = n.crossProduct(u);
						rt->setRenderColor( 1,0,0 );
						rt->renderArrow( pos, pos + u * renderScale, 1.0f );
						rt->setRenderColor( 0,1,0 );
						rt->renderArrow( pos, pos + v * renderScale, 1.0f );
						rt->setRenderColor( 0,0,1 );
						rt->renderArrow( pos, pos + n * renderScale, 1.0f );
					}
					if ( monitorShowMonomials ){
						Matrix3f T, invT;
						T[0] = pg->getTrajectorySet()->getTrajectories()->get3f( localFrameIndex, monitorPoint, REF_U );
						T[2] = pg->getTrajectorySet()->getTrajectories()->get3f( localFrameIndex, monitorPoint, REF_N );
						T[1] = T[2].crossProduct(T[0]);
						StaticVector<float32,6> coeff = pg->getTrajectorySet()->getTrajectories()->get<float32,6>( localFrameIndex, monitorPoint, COEFF );
						T[2] = pg->getTrajectorySet()->getTrajectories()->get3f( localFrameIndex, monitorPoint, REF_N );
						for ( int i = -10; i <= 10; i++ ){
							for ( int j = -10; j <= 10; j++ ){
								// Sampling
								float32 u0 = float32(i)/10.0f * renderScale;
								float32 v0 = float32(j)/10.0f * renderScale;
								float32 u1 = float32(i+1)/10.0f * renderScale;
								float32 v1 = float32(j+1)/10.0f * renderScale;
								float32 n0 = Monomials2ndOrder::evaluate( coeff, makeVector2f( u0, v0 ) );
								Vector3f p0 = T * makeVector3f( u0, v0, n0 ) + pos;
								float32 n1 = Monomials2ndOrder::evaluate( coeff, makeVector2f( u1, v0 ) );
								Vector3f p1 = T * makeVector3f( u1, v0, n1 ) + pos;
								float32 n2 = Monomials2ndOrder::evaluate( coeff, makeVector2f( u0, v1 ) );
								Vector3f p2 = T * makeVector3f( u0, v1, n1 ) + pos;
								rt->setRenderColor( 1,1,0 );
								rt->renderLine( p0, p1 );
								rt->renderLine( p0, p2 );
							}
						}
					}
				}
			}
		}
	}
}

/**
* User interaction
*/
void PCIPatchGraphBSR::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
}

/*** PCInteractable ***/
void PCIPatchGraphBSR::assign(const Object* obj, CopyContext *context)
{
	PCInteractionTool::assign(obj,context);
	const PCIPatchGraphBSR * o = dynamic_cast<const PCIPatchGraphBSR*>(obj);
	pAssert( o != NULL );
}

/**
* User interaction
*/
void PCIPatchGraphBSR::mouseMoved(int32 x, int32 y) 
{
}

/**
* User interaction
*/
void PCIPatchGraphBSR::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
}

/**
* User interaction
*/
void PCIPatchGraphBSR::areaResize(card32 width, card32 height) 
{
}

/**
* User interaction
*/
void PCIPatchGraphBSR::keyDown(GeneralKey key) 
{
	if ( key.getkeyAlphaNumKey() == 'a' ){
		monitorPoint++;
		redraw();
	} else if ( key.getkeyAlphaNumKey() == 'b' ){
		monitorPoint--;
		redraw();
	}
}

/**
* User interaction
*/
void PCIPatchGraphBSR::keyUp(GeneralKey key) 
{
}

void PCIPatchGraphBSR::activateTool() 
{
}

void PCIPatchGraphBSR::deactivateTool() 
{
}

