//---------------------------------------------------------------------------
#ifndef PCIPatchGraphBSRH
#define PCIPatchGraphBSRH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
#include "PointCloud.h"
#include "PatchGraphRenderer.h"
#include "Persistent.h"
#include "PatchGraphContainer.h"
#include "PatchGraphRenderer.h"
#include "BSROptContext.h"
#include "PatchGraphBSROpt.h"
#include "RenderingTool.h"
//---------------------------------------------------------------------------

namespace geoxl {

class ImplicitPatchGraphMeshing;

/// Main tool for the BSR-based animation reconstruction
class ANIMATIONRECONSTRUCTION_API PCIPatchGraphBSR : public PCInteractionTool {
	GEOXL_CLASS( PCIPatchGraphBSR )

 public:
	/// Container for patch graphs
	PatchGraphContainer* pgContainer;
	
	/// Renderer
	StandardPatchGraphRenderer* patchGraphRenderer;
	
	/// visual feedback
	bool									showSteps;
	
	/// Optimizer
	PatchGraphBSROpt*		optimizer;
	
	bool peterOptimize;
  
	/// frame to be optimized
	card32							whichFrame;
	
	/// optimize all frames?
	bool									allFrames;
	
	/// use an urshape during optimization
	bool									useUrshape;
	/// option for testing constrained simplification in multi-grid opt
	bool multiResConstrain2ndFrameDataPoints;

	// Debugging: show coordinate frames
	bool									monitorShowCoordinateFrames;
	
	// Debugging: show monomials
	bool									monitorShowMonomials;
	
	/// Debugging: monitored point
	card32							monitorPoint;
	
	/// Debugging: rendering functionality
	RenderingTool*				rt;
	
	/// Scaling factor of all renderings
	float32							renderScale;

	// configuration
	bool									ccGaussianPrjLikelihood;
	bool									ccSimpleICPLikelihood;
	bool									ccProjectionSmoothness;
	bool									ccSpringSTSmoothness;
	bool									ccGreenTensorSTSmoothness;
	bool									ccPatchLSSmoothing;
	bool									ccCorrespInterpolTest;
	bool									ccSquaredDistancePotential;
	bool									ccMarkerErrorPotential;
	bool									ccNormalSmoothnessPotential;
	bool									ccPlanarSmoothnessPotential;
	bool									ccCircularSmoothnessPotential;
	bool									ccLaplacianSmoothnessPotential;
	bool									ccPositionRigidityPotential;
	bool									ccNormalRigidityPotential;
	bool									ccSTPositionCoherence;
	bool									ccSTNormalCoherence;
	bool									ccSTRotationCoherence;
	
	/// final output
	ImplicitPatchGraphMeshing *meshing;
	string								outputMeshNode;
	
	/// tests & debugging
	string								corrTestSource;
	float32							probCorr;
	float32							rotDeg;
	float32 resamplingTestDist;
	bool resamplingShowClustering;

   static void redrawView(void *context);

 private:
	void									loadScene();
	void									saveScene();
	void									optimize();
	void									makeMesh();
	void									createCorrTest();
	void									changeConfig();
	void									createUrshape();
	void resamplingTest();

	 public:
	/*** CLASS MANAGEMENT ***/
	/// Constructor
	PCIPatchGraphBSR();
	// Destructor
	~PCIPatchGraphBSR();

	/// Load a patch graph container
	void									loadPatchGraphContainer( const QString filename );
	
	/// Load patch graph container
	void									savePatchGraphContainer( const QString filename );

	/// Apply optimization to patch graph 'index' in patch graph container
	void									optimize( const card32 index );
	
	/************************************************************************/
	/* PETER: The following codes are added for testing                                                           */
	/************************************************************************/

	void									peter_loadPatchGraphContainer( const QString filename );
	void									peter_savePatchGraphContainer( const QString filename );
	void									peter_optimize(const card32 index);

	/************************************************************************/
	/* PETER: The above codes are added for testing                                                               */
	/************************************************************************/

	/// Return index of current frame
	int32								getCurrentFrameIndex(){ return scene->getRootState()->dynamicState->time; }

	/*** PCINTERACTION ***/
	virtual void						assign(const Object* obj, CopyContext *context = NULL);
	virtual void						mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
	virtual void						mouseMoved(int32 x, int32 y);
	virtual void						mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
	virtual void						areaResize(card32 width, card32 height);
	virtual void						keyDown(GeneralKey key);
	virtual void						keyUp(GeneralKey key);
	virtual void						activateTool();
	virtual void						deactivateTool();
	virtual void						glDrawTool(GLContext *glContext);
	void									redraw(){ redrawScene(); }
};

}

#endif