//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIPatchGraphReconstruction3D.h"
//---------------------------------------------------------------------------
#include "ObjectClassProperty.h"
#include "PatchGraphRenderer.h"
#include "PatchGraph.h"
#include "BooleanClassProperty.h"
#include "SeparatorClassProperty.h"
#include "StringClassProperty.h"
#include "PointCloud.h"
#include "UnstructuredInCorePointCloud.h"
#include "PointSet.h"
#include "InCorePCTopologyGraph.h"
#include "PatchGraphCreator3D.h"
#include "ClassMethods.h"
#include "TopologyAttachment.h"
#include "RefFrameEst3D.h"
#include "PointCloudAnimationSequence.h"
#include "PatchApprox3D.h"
#include "HFBFBivariateMonomials.h"
#include "ComputeSampleSpacing.h"
#include "UnifyNormals.h"
#include "PatchTypes.h"
#include "PatchGraphRoutines.h"
#include "RegularVolume.h"
#include "BoundingBox.h"
#include "BoundingBox_templCode.h"
#include "Timer.h"
#include "GLVolumeDisplayer.h"
#include "GLEventDisplayer.h"
#include "TrajectorySet.h"
#include "ImplicitPatchGraphMeshing.h"
#include "PointCloudTools.h"
#include "UnstructuredInCoreTriangleMesh.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS(PCIPatchGraphReconstruction3D, 5)
{
	BEGIN_CLASS_INIT(PCIPatchGraphReconstruction3D);
	ADD_SEPARATOR("input")
	ADD_STRING_PROP(inputNode, 0)
	ADD_SEPARATOR("reconstruction")
	ADD_BOOLEAN_PROP(in2d, 0)
	ADD_BOOLEAN_PROP(flipNormals, 3)
	ADD_BOOLEAN_PROP(regularized, 4);
	ADD_BOOLEAN_PROP(fromNeighbors, 0)
	ADD_CARD32_PROP(patchDegree, 5)
	ADD_SEPARATOR("visualization")
	ADD_OBJECT_PROP(renderer, 0, PatchGraphRenderer::getClass(), true)
	ADD_BOOLEAN_PROP(showReconstruction, 0)
	ADD_SEPARATOR("output")
	ADD_STRING_PROP(outputMeshNode, 1)
	ADD_OBJECT_PROP(reconstruction, 0, PatchGraph::getClass(), true)
	ADD_OBJECT_PROP(dataPoints, 0, PointCloudAnimationSequence::getClass(), true)

	ADD_SEPARATOR("rec. pipeline")
	ADD_OBJECT_PROP(patchGraphCreator, 2, PatchGraphCreator3D::getClass(), true)
	ADD_OBJECT_PROP(compSpacing, 2, ComputeSampleSpacing::getClass(), true)
	ADD_OBJECT_PROP(refEst, 2, RefFrameEst3D::getClass(), true)
	ADD_OBJECT_PROP(unifyNormals, 2, UnifyNormals::getClass(), true)
	ADD_OBJECT_PROP(patchApprox, 2, PatchApprox3D::getClass(), true)
	ADD_OBJECT_PROP(meshing, 2, ImplicitPatchGraphMeshing::getClass(), true)
	ADD_OBJECT_PROP(basisFunctions, 2, HFBFBivariateMonomials::getClass(), true)

	ADD_NOARGS_METHOD(performReconstruction)
	ADD_NOARGS_METHOD(computeMesh)
}


// --


PCIPatchGraphReconstruction3D::PCIPatchGraphReconstruction3D() {
	renderer = new StandardPatchGraphRenderer();
	inputNode = "root/pc1";
	reconstruction = NULL;
	dataPoints = NULL;
	in2d = false;
	fromNeighbors = false;
	flipNormals = true;
	regularized = true;
	outputMeshNode = "root/recMesh";
	patchDegree = 2;

	patchGraphCreator = new PatchGraphCreator3D;
	patchGraphCreator->setup(true, PatchGraphCreator3D::PATCH_HEIGHTFIELD_QUAD, PatchGraphCreator3D::PATCH_CORR_NONE);
	compSpacing = new ComputeSampleSpacing;
	refEst = new RefFrameEst3D;
	unifyNormals = new UnifyNormals;
	patchApprox = new PatchApprox3D;
   meshing = new ImplicitPatchGraphMeshing;
   basisFunctions = new HFBFBivariateMonomials;
   basisFunctions->setup(patchDegree);
	patchApprox->setup(basisFunctions);
}

void PCIPatchGraphReconstruction3D::assign(const Object* obj, CopyContext *context) {
	PCInteractionTool::assign(obj,context);
}

PCIPatchGraphReconstruction3D::~PCIPatchGraphReconstruction3D() {
	delete renderer;
	delete reconstruction;
	delete dataPoints;
}


void PCIPatchGraphReconstruction3D::getEnv(PointSet *&ps, InCorePCTopologyGraph *&tg) {
	tg = NULL;
	ps = NULL;
   PointCloud *pc = getPointCloud(scene, inputNode);
   if (!pc) return;
	UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
	if (!upc) return;
	ps = upc->getPointSet();
   AttachedDataContainer *att = upc->getAttachments();
   TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(att->getData("TopologyAtt"));
   if (topAtt) {
	   tg = topAtt->getTopology();
	}
}

bool PCIPatchGraphReconstruction3D::drawGLSceneRendering() {
	if (renderer != NULL && reconstruction != NULL) {
		renderer->renderGL(reconstruction, dataPoints);
	}
	return true;
}


void PCIPatchGraphReconstruction3D::performReconstruction() {
	PointSet *ps;
	InCorePCTopologyGraph *tg;
	getEnv(ps, tg);
	if (ps != NULL && tg != NULL) {
		patchGraphCreator->setup(true, PatchGraphCreator3D::PATCH_HEIGHTFIELD_QUAD, PatchGraphCreator3D::PATCH_CORR_NONE);
		delete reconstruction;
		delete dataPoints;
		reconstruction = NULL;

		dataPoints = patchGraphCreator->createScene(ps);
		reconstruction = patchGraphCreator->createPatchGraph(ps, tg, in2d);

		compSpacing->estimatePatchSamplingDistance(reconstruction);

		if (fromNeighbors) {
			refEst->estimateReferenceFrameFromNeighbors(reconstruction);
		} else {
			refEst->estimateReferenceFrameFromDataPoints(reconstruction, dataPoints);
		}

		unifyNormals->unifyNormals(reconstruction);
		if (flipNormals) {
			unifyNormals->flipNormals(reconstruction);
		}

		basisFunctions->setup(patchDegree);
		patchApprox->setup(basisFunctions);
      
      if (regularized) {
   		patchApprox->computeCoeffsRegularized(reconstruction, dataPoints);
      } else {
	   	patchApprox->computeCoeffsUnregularized(reconstruction, dataPoints);
		}
	}
}

void PCIPatchGraphReconstruction3D::computeMesh() {
	if (reconstruction != NULL) {
      UnstructuredInCoreTriangleMesh *result = meshing->createMesh(reconstruction, 0);
      if (getPointCloud(getScene(), outputMeshNode) != NULL) {
   	   replacePointCloud(getScene(), outputMeshNode, result);
   	} else {
   	   addPointCloud(getScene(), result, outputMeshNode);
   	}
	}    
}

	