//---------------------------------------------------------------------------
#ifndef PCIPatchGraphReconstruction3DH
#define PCIPatchGraphReconstruction3DH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------


namespace geoxl {

class PatchGraphRenderer;
class PatchGraph;
class InCorePCTopologyGraph;
class PointSet;
class PointCloudAnimationSequence;

class PatchGraphCreator3D;
class ComputeSampleSpacing;
class RefFrameEst3D;
class UnifyNormals;
class PatchApprox3D;
class ImplicitPatchGraphMeshing;
class HFBFBivariateMonomials;


/**
	tool for 3d reconstruction with patch graphs
*/
class ANIMATIONRECONSTRUCTION_API PCIPatchGraphReconstruction3D : public PCInteractionTool {
	GEOXL_CLASS( PCIPatchGraphReconstruction3D )

 private:
	// input
	string inputNode;
	// rendering
	PatchGraphRenderer *renderer;
	bool showReconstruction;
	// reconstruction
	bool in2d;
	bool fromNeighbors;
	bool flipNormals;
	bool regularized;
	card32 patchDegree;
	
	// output
	string outputMeshNode;
	PatchGraph *reconstruction;
	PointCloudAnimationSequence *dataPoints;

	// pipeline
	PatchGraphCreator3D *patchGraphCreator;
	ComputeSampleSpacing *compSpacing;
	RefFrameEst3D *refEst;
	UnifyNormals *unifyNormals;
	PatchApprox3D *patchApprox;
   ImplicitPatchGraphMeshing *meshing;
   HFBFBivariateMonomials *basisFunctions;
	
	void getEnv(PointSet *&ps, InCorePCTopologyGraph *&tg);
	void performReconstruction();
	void computeMesh();

 public:
	PCIPatchGraphReconstruction3D();

   virtual void assign(const Object* obj, CopyContext *context = NULL);


   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {}
   virtual void mouseMoved(int32 x, int32 y) {}
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {}
   virtual void areaResize(card32 width, card32 height) {}

   virtual void keyDown(GeneralKey key) {}
   virtual void keyUp(GeneralKey key) {}

   virtual bool drawGLSceneRendering();

	~PCIPatchGraphReconstruction3D();
};


}

#endif