#ifndef ARAPDeformationContextH
#define ARAPDeformationContextH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "ASAPDeformationContext.h"
#include "ASAPDeformationSolver.h"
#include "ASAPDeformationEvaluator.h"
#include "FastSparseMatrix.h"
//---------------------------------------------------------------------------

namespace geoxl
{

/**
	plz comment your class
*/
class DEFORMATION_API ARAPDeformationContext : public ASAPDeformationContext
{
	GEOXL_CLASS( ARAPDeformationContext )

private:
	FastSparseMatrixFloatPrecision* A_fac;
	vector<Vector3f> lastConstraintAnchors;
	mpcard currentConstraintID;
	vector<const PositionARConstraint*> allConstraints;



public:
	ARAPDeformationContext(void);
	~ARAPDeformationContext(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual void initializeFromShape(const SceneObject *obj, Vector2f timeInterval, const ARSettings *settings);

  // virtual void addPositionConstraint(const PositionARConstraint *c);
   //virtual void addSpatialDerivativeConstraint(const SpatialDerivativeARConstraint *c){}
   virtual void beginConstraints();

   void writeLinearSystem();

   virtual void addPositionConstraint(const PositionARConstraint *c);
   virtual void solve();
   virtual DeformationSolver *createSolver();
   virtual DeformationEvaluator *createEvaluator();

   virtual Matrix3f getSpatialGradient(const ARConstraintAnchor &at);


};



class DEFORMATION_API ARAPDeformationSolver : public ASAPDeformationSolver {
	GEOXL_CLASS( ARAPDeformationSolver )
private:

public:
	//ARAPDeformationSolver();

	virtual DeformationEvaluator *createEvaluator(){return new ASAPDeformationEvaluator;}
	AROptimizationContext *createOptContext(){return new ARAPDeformationContext;}


};





}

#endif