//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "initDeformation.h"
//---------------------------------------------------------------------------
#include "SGDeformationConstraintsList.h"
#include "SGDeformationNodeAR.h"
#include "UnitTests.h"
#include "SceneGraphEntity.h"
#include "SGEShape.h"
#include "SGEPointCloud.h"
#include "SGESurfelGraph.h"
#include "SGEDeformationSequence.h"
#include "SGEVolumeGridDeformationSequence.h"
#include "AROptimizationTool.h"
#include "DeformationSolver.h"
#include "DeformationEvaluator.h"
#include "ARSettings.h"
#include "BackendSettings.h"
#include "RigidDeformationSolver.h"
#include "RigidDeformationEvaluator.h"
#include "PCIAnimationReconstruction2.h"
#include "PCCImportDepthAnimation.h"
#include "CameraParameterAttachment.h"
#include "ICPConstraints.h"
#include "SmallScaleReconstruction.h"

// Reconstruction
#include "SurfelGraphInit2.h"
#include "SurfelOptimizer.h"
#include "ShapeReconstruction.h"
#include "SurfelGraphTriangulator.h"

// rendering
#include "PCCNodesTools.h"		

#include "ARMergeSettings.h"
#include "ARPipelineOptions.h"
#include "ARDataConstraintsSettings.h"
#include "ARVisualizationSettings.h"
#include "ARGlobalOptSettings.h"

#include "ARSequenceEvaluator.h"
#include "ARPointCloudSeqEval.h"

#include "PCCDataResampling.h"
#include "PCCSetErrorQuadrics.h"
#include "PCCRenameNode.h"
#include "PCCBoundaryDetector.h"

#include "MMDeformationSolver.h"
#include "MMNodes.h"
#include "MMNeighborContainer.h"
#include "MMNode.h"
#include "MMDeformations.h"
#include "AttachedMMDeformationContext.h"
#include "AROptimizationContext.h"
#include "ARDeformationOptimizationContext.h"
#include "MMDeformationContext.h"
#include "RigidDeformationContext.h"
#include "ARShapeOptimizationContext.h"
#include "SurfelOptimizationContext.h"

#include "MMSampler.h"
#include "MMEdgeContractionSampler.h"
#include "MMTopologyGridSampler.h"
#include "MMGridSampler.h"
#include "MMFurthestPointSampler.h"

#include "PCCRemoveSmallPatches.h"
#include "PCCTopologicalResampling.h"

#ifdef GEOXL_GUI
//-- gui related includes
#endif

#include "UnitTests.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



void geoxl::initDeformation()
{
	ARConstraintAnchor::init(Persistent::getClass());
	GeneralARConstraint::init(Persistent::getClass());
		VectorValuedARConstraint::init(GeneralARConstraint::getClass());
			PositionARConstraint::init(VectorValuedARConstraint::getClass());
			VelocityVectorARConstraint::init(VectorValuedARConstraint::getClass());
			InstantaneousVelocityARConstraint::init(VectorValuedARConstraint::getClass());
			ARHalfSpaceConstraint::init(VectorValuedARConstraint::getClass());
	   MatrixValuedARConstraint::init(GeneralARConstraint::getClass());
			SpatialDerivativeARConstraint::init(MatrixValuedARConstraint::getClass());
			SimpleSpatialDerivativeARConstraint::init(MatrixValuedARConstraint::getClass());

	SGDeformationConstraintsList::init(SceneGraphNode::getClass());
	SGDeformationNodeAR::init(SceneGraphNode::getClass());
   PCIAnimationReconstruction2::init(PCInteractionTool::getClass());

	PCCDataResampling::init(PCCommand::getClass());
	PCCSetErrorQuadrics::init(PCCommand::getClass());
	PCCBoundaryDetector::init(PCCommand::getClass());
	PCCTopologicalResampling::init(PCCommand::getClass());
   
   SceneGraphEntity::init(Persistent::getClass());
      SGEShape::init(SceneGraphEntity::getClass());
         SGEPointCloud::init(SGEShape::getClass());
         SGESurfelGraph::init(SGEShape::getClass());
      SGEDeformationSequence::init(SceneGraphEntity::getClass());
         SGEVolumeGridDeformationSequence::init(SGEDeformationSequence::getClass());
   
   AROptimizationTool::init(Persistent::getClass());
      DeformationSolver::init(AROptimizationTool::getClass());
			MMDeformationSolver::init(DeformationSolver::getClass());
			RigidDeformationSolver::init(DeformationSolver::getClass());
      DeformationEvaluator::init(AROptimizationTool::getClass());
			RigidDeformationEvaluator::init(DeformationEvaluator::getClass());
    MMTopology::init(Persistent::getClass());	
   MMNodes::init(Persistent::getClass());
   MMNeighborContainer::init(Persistent::getClass());
      MMNode::init(MMNeighborContainer::getClass());
   MMDeformations::init(Persistent::getClass());
   AttachedMMDeformationContext::init(AttachedData::getClass());
   AROptimizationContext::init(Persistent::getClass());
      ARDeformationOptimizationContext::init(AROptimizationContext::getClass());
         MMDeformationContext::init(ARDeformationOptimizationContext::getClass());
      ARShapeOptimizationContext::init(AROptimizationContext::getClass());
         SurfelOptimizationContext::init(ARShapeOptimizationContext::getClass());
   MMSampler::init(Persistent::getClass());
      MMEdgeContractionSampler::init(MMSampler::getClass());
      MMTopologyGridSampler::init(MMSampler::getClass());
      MMGridSampler::init(MMSampler::getClass());
      MMFurthestPointSampler::init(MMSampler::getClass());

   ARSettings::init(Persistent::getClass());
	ARMergeSettings::init(Persistent::getClass());
	ARPipelineOptions::init(Persistent::getClass());
	ARDataConstraintsSettings::init(Persistent::getClass());
	ARVisualizationSettings::init(Persistent::getClass());
	ARGlobalOptSettings::init(Persistent::getClass());
	BackendSettings::init( Persistent::getClass() );

	PCCNodesTools::init(PCCommand::getClass());	

	PCCImportDepthAnimation::init(PCCommand::getClass());
	CameraParameterAttachment::init(AttachedData::getClass());

	ICPConstraints::init( Persistent::getClass() );

   // Reconstruction
   SurfelGraphInit2::init( Persistent::getClass() );
   ShapeReconstruction::init( Persistent::getClass() );
   SurfelOptimizer::init( ShapeReconstruction::getClass() );
   SurfelGraphTriangulator::init( Persistent::getClass() );
   SmallScaleReconstruction::init( Persistent::getClass() );

	ARSequenceEvaluator::init(Persistent::getClass());
      ARPointCloudSeqEval::init(ARSequenceEvaluator::getClass());
	
	PCCRenameNode::init(PCCommand::getClass());

	PCCRemoveSmallPatches::init(PCCommand::getClass());
}

//---------------------------------------------------------------------------

void geoxl::shutdownDeformation()
{
	ARConstraintAnchor::shutdown();
	GeneralARConstraint::shutdown();
	VectorValuedARConstraint::shutdown();
	PositionARConstraint::shutdown();
	VelocityVectorARConstraint::shutdown();
	InstantaneousVelocityARConstraint::shutdown();

	SGDeformationConstraintsList::shutdown();
	SGDeformationNodeAR::shutdown();
	PCIAnimationReconstruction2::shutdown();

	PCCDataResampling::shutdown();
	PCCSetErrorQuadrics::shutdown();
	PCCBoundaryDetector::shutdown();
	PCCTopologicalResampling::shutdown();

	SceneGraphEntity::shutdown();
	SGEShape::shutdown();
	SGEPointCloud::shutdown();
	SGESurfelGraph::shutdown();
	SGEDeformationSequence::shutdown();
	SGEVolumeGridDeformationSequence::shutdown();

	AROptimizationTool::shutdown();
	DeformationSolver::shutdown();
	DeformationEvaluator::shutdown();
	MMDeformationSolver::shutdown();
	RigidDeformationSolver::shutdown();
	RigidDeformationEvaluator::shutdown();

	MMNodes::shutdown();
	MMNeighborContainer::shutdown();
	MMNode::shutdown();
	MMDeformations::shutdown();
	AttachedMMDeformationContext::shutdown();
	AROptimizationContext::shutdown();
	ARDeformationOptimizationContext::shutdown();
	MMDeformationContext::shutdown();
	RigidDeformationContext::shutdown();
	ARShapeOptimizationContext::shutdown();
	SurfelOptimizationContext::shutdown();

	MMSampler::shutdown();
	MMEdgeContractionSampler::shutdown();
	MMTopologyGridSampler::shutdown();
	MMGridSampler::shutdown();
	MMFurthestPointSampler::shutdown();

	ARSettings::shutdown();
	ARMergeSettings::shutdown();
	ARPipelineOptions::shutdown();
	ARDataConstraintsSettings::shutdown();
	ARVisualizationSettings::shutdown();
	ARGlobalOptSettings::shutdown();
	BackendSettings::shutdown();

	PCCImportDepthAnimation::shutdown();
	CameraParameterAttachment::shutdown();

	ICPConstraints::shutdown();

	// Reconstruction
	SurfelGraphInit2::shutdown();
	ShapeReconstruction::shutdown();
	SurfelOptimizer::shutdown();
	SurfelGraphTriangulator::shutdown();
	SmallScaleReconstruction::shutdown();

	PCCNodesTools::shutdown();		

	ARSequenceEvaluator::shutdown();
	ARPointCloudSeqEval::shutdown();

	PCCRenameNode::shutdown();

	PCCRemoveSmallPatches::shutdown();
}

