#include "stdafx.h"
#include "MMSimpleCG.h"

using namespace geoxl;
									   /* this is for compatibility */
MMSimpleCG::MMSimpleCG(int n, int forgetit, int forgetit2) {
	this->x=new double[n];
	this->g=new double[n];
	this->m=new double[n];
	
	this->alpha = 0;
	
	this->N = n;
	
	xcurr = nullDVector<double>(N);
	gcurr = nullDVector<double>(N);

	sx = scalarDVector<double>(N,1);
	
	cnls = cng = cnf = cnd = 0;
	tol.setMaxBacktrackIter(5);
	tol.setStepTol(0.00001);
	tol.setMaxIter(10000);
	tol.setLineTol(0.01);
	tol.setFTol(0.0000001);
}

MMSimpleCG::~MMSimpleCG(){
	delete[] x;
	delete[] g;
	delete[] m;
}

double MMSimpleCG::computeStep(DVectorD sk, int method){
	//secant:
	
	double alpha = 0, alpha_curr = 0, alpha_old = 0,  sigma = 0, sigma0 = 0, sigma_old = 0;
	
	DVectorD temp = nullDVector<double>(N), temp2 = nullDVector<double>(N);
	
	int iter = 0;
	
	int maxIter = tol.getMaxBacktrackIter();
	double stepTol = tol.getLineTol();
	
	alpha_curr = 0;
   alpha = 0.00001;
   
	setX(xcurr);
	
	countG();
	countF();
	
	float32 fstart = f;
	
	get_g(&temp);

	sigma0 = temp*sk;
	sigma = sigma0;
	
	if(sigma>0.000001){
		cout << "error: not a descent direction" << endl;
	}

	while(iter<maxIter && fabs(sigma)>stepTol*abs(sigma0)){
		iter++;
		if(method == 1){
			alpha_curr = alpha_curr;
			
			setX(xcurr);
			d2a = countD2A(&sk,alpha_curr);
			
			/* setX(xcurr + sk*(alpha+0.001));
			countG();
			get_g(&temp2);
			sigma_old = (temp2*sk - sigma)/0.001;*/	
			
			if(fabs(d2a)>0.0000001)
				alpha_curr = alpha_curr - sigma/d2a;
			else{
				alpha_curr = 0;
				break;
			}
			
			setX(xcurr + sk*alpha_curr);
			countG();
			get_g(&temp2);
			
			sigma = temp2*sk;
		}
		else{
			alpha_old = alpha_curr;
			alpha_curr= alpha;

			sigma_old = sigma;

			setX(xcurr + sk*alpha);

			countG();
			get_g(&temp2);

			sigma = temp2*sk;

			alpha = (sigma*alpha_old - sigma_old*alpha)/(sigma - sigma_old);
		}
	}
	
	setX(xcurr + sk*alpha_curr);
	countF();
	
	//cout << "F start = " << fstart << " current f = " << f << endl;
	
	if(f>fstart)
		alpha_curr = 0;
	
	return alpha_curr;
}

double MMSimpleCG::computeStep2(DVectorD sk){
	//secant:
	
	double alpha = 0, alpha_curr = 0, alpha_old = 0,  sigma = 0, sigma0 = 0, sigma_old = 0;
	
	DVectorD temp = nullDVector<double>(N), temp2 = nullDVector<double>(N);
	
	int iter = 0;
	
	int maxIter = tol.getMaxBacktrackIter();
	double stepTol = tol.getLineTol();
	
	alpha_curr = 0;
   alpha = 0.00001;
   
	setX(xcurr);
	
	countG();
	
	countF();
	float32 fstart = f;
	
	get_g(&temp);

	sigma0 = temp*sk;
	sigma = sigma0;
	
	while(iter<maxIter && fabs(sigma)>stepTol*abs(sigma0)){
		//iter++;
		alpha_old = alpha_curr;
		alpha_curr=alpha;
		
		sigma_old = sigma;
		
		setX(xcurr + sk*alpha);

		countG();
		get_g(&temp2);
		
		sigma = temp2*sk;

		alpha = (sigma*alpha_old - sigma_old*alpha)/(sigma - sigma_old);
	}
	
	setX(xcurr + sk*alpha);
	countF();
	
	if(f>=fstart)
		alpha = 0;
	
	return alpha_curr;
}

int MMSimpleCG::CGoptimize(bool used2a = false){

	DVectorD d = nullDVector<double>(N), r = nullDVector<double>(N);
	DVectorD s = nullDVector<double>(N), minv = nullDVector<double>(N);
	
	countG();
	countF();
	
	cout << "Running Nonlinear CG with " << ((used2a)?"d2a derivative ":"secant ") << "linesearch and preconditioning. Starting f = " <<  f << endl;
	
	double fold = f;
	double delta0, delta_old, delta_new, delta_mid;
	
	int restart = 50, k = 0;
	
	get_negg(&r);
	
	getX(&xcurr);

	countM();
	get_minv(&minv);
	
	s = minv.componentProduct(r);

	d = s;
	
	delta_new = r*d;
	delta0 = delta_new;
	
	int maxIter = tol.getMaxIter();
	
	double alpha = 0, beta = 0;
	
	for(int i=0; i < maxIter; i++, k++){
		
		alpha = computeStep(d,((used2a)?1:0));
		
		//alpha = computeStep2(d);
		
		xcurr = xcurr + d*alpha;
		
		setX(xcurr);
		countF();	
		countG();
		get_negg(&r);
		
		if(alpha == 0 || abs(f-fold) < tol.getStepTol() || norm(r)<tol.getFTol()){
			// converged!
			
			cout << "Coverged a = " << alpha << " abs(f-fold) " << abs(f-fold) << " norm(r) " << norm(r) << "cnd = " << cnd << endl;
			
			cnls = i;
			setX(xcurr);			
			return 1;
		}	
		
		fold = f;
		
		delta_old = delta_new;
		delta_mid = r*s;
		
		countM();
		
		get_minv(&minv);
		
      
		s = minv.componentProduct(r);
		
		delta_new = r*s;
		
		beta = (delta_new - delta_mid)/delta_old;
		
		if( k == restart || beta <= 0){
			d = s;
			k = 0;		
		}
		else{
			d = s + d*beta;
		}
		
		
	}
		
	//didn't converge within maxIter	iterations
	return -1;
}

int MMSimpleCG::CGoptimize2(bool used2a = false){

	DVectorD dcurr = nullDVector<double>(N),rcurr = nullDVector<double>(N), rprev = nullDVector<double>(N);
	
	countG();
	countF();
	
	double fold = f;
	
	get_g(&dcurr);
	dcurr = -dcurr;
	rcurr = dcurr;
	
	int maxIter = tol.getMaxIter();
	
	double alpha = 0, beta = 0;
	
	for(int i=0; i < maxIter; i++){
		
		alpha = computeStep(dcurr,((used2a)?1:0));
		//alpha = computeStep(dcurr);
		
		xcurr = xcurr + dcurr*alpha;
		
		rprev = rcurr;
		
		setX(xcurr);
		countF();	
		
		if(alpha == 0 || abs(f-fold) < tol.getFTol() || norm(rprev)<tol.getFTol()){
			// converged!
			
			cnls = i;
			setX(xcurr);			
			return 1;
		}	
		
		//cout << "F = " << "\t" << f << endl;
		
		fold = f;
		
		countG();
		
		get_g(&rcurr);
		
		//cout << "G = " << "\t" << norm(rcurr) << endl;
		
		rcurr = -rcurr;
		
		beta = max(rcurr*(rcurr - rprev)/(rprev*rprev),0);
		
		dcurr = rcurr + dcurr*beta;
	}
		
	//didn't converge within maxIter	iterations
	return -1;
}

void MMSimpleCG::countG(){
	cng++;
	this->G();
}

void MMSimpleCG::countF(){
	cnf++;
	this->F();
}


void MMSimpleCG::countM(){
	cnm++;
	this->M();
}

double MMSimpleCG::countD2A(DVectorD *d, double alpha){
	cnd++;
	
	return this->D2A(d,  alpha);
}
