#ifndef MMSimpleCG_h
#define MMSimpleCG_h

#include "TOLS.h"
#include "math.h"
#include "DynamicLinearAlgebra.h"

using namespace geoxl;

class MMSimpleCG{
private:

 DVectorD xcurr;
 DVectorD gcurr;

 void countF();
 
 void countG();
 
 void countM();
 
 double countD2A(DVectorD *d, double alpha);
 
public: 
	TOLS tol;

	//preconditioner	
	DVectorD sx;

	int cnls;
	int cnf, cng, cnm;

	double *x;
	double f;
	double *g;
	double *m;
	
	double alpha;
	double d2a;
	double cnd;

	int N;

	MMSimpleCG(int, int forgetit, int forgetit2);

	virtual ~MMSimpleCG();

	int CGoptimize(bool);
	
	int CGoptimize2(bool);

	double computeStep(DVectorD sk, int method = 0);
	
	double computeStep2(DVectorD sk);

	void setX(DVectorD X){
		for(int i=0;i<N;i++){
			x[i] = X[i];	
		}
	}
	
	void getX(DVectorD *X){
		for(int i=0;i<N;i++){
			(*X)[i] = x[i];	
		}
	}
	
	void get_g(DVectorD *X){
		for(int i=0;i<N;i++){
			(*X)[i] = g[i];	
		}
	}
	
	void get_negg(DVectorD *g_new){
		for(int i=0;i<N;i++){
			(*g_new)[i] = -g[i];	
		}
	}
	
	void get_minv(DVectorD *m_new){
		for(int i=0;i<N;i++){
			(*m_new)[i] = 1.0/m[i];	
			//(*m_new)[i] = 1.0;	
		}
	}
	
	void get_m(DVectorD *m_new){
		for(int i=0;i<N;i++){
			(*m_new)[i] = m[i];	
		}
	}

	virtual void F() = 0;	// f=F(x)
	virtual void G() = 0;	// g=Grad F(x)
	virtual void M() = 0;	// m=diag Hess (x)
	virtual double D2A(DVectorD *d, double alpha) = 0;	// d2a = d^2f(x+alpha*d)/d alpha^2
	

};

#endif 