//---------------------------------------------------------------------------
#ifndef ARConstraintsInterfaceH
#define ARConstraintsInterfaceH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

namespace geoxl {

class GeneralARConstraint;
class PositionARConstraint;
class VelocityVectorARConstraint;
class InstantaneousVelocityARConstraint;
class ARHalfSpaceConstraint;
class SpatialDerivativeARConstraint;
class SimpleSpatialDerivativeARConstraint;


/// this is the interface to a class that takes positional quadratic constraints
/// (for surface reconstruction / deformation fitting)
class ARConstraintsInterface {
 public:
	/// adds an arbitrary constraint, will dispatch accordingly (less efficient)
	virtual void addGeneralConstraint(const GeneralARConstraint *c) = 0;
	/// adds a soft position constraint
   virtual void addPositionConstraint(const PositionARConstraint *c) = 0;
	/// adds a half space constraint
	virtual void addHalfSpaceConstraint(const ARHalfSpaceConstraint *c) = 0;
	/// adds a soft constraint on the local jacobian matrix
	virtual void addSpatialDerivativeConstraint(const SpatialDerivativeARConstraint *c) = 0;
	/// adds a soft constraint on the local jacobian matrix (simplified)
	virtual void addSimpleSpatialDerivativeConstraint(const SimpleSpatialDerivativeARConstraint *c) = 0;
};


/// this is the interface to a class that takes positional and time dependent (velocity) quadratic constriants
/// (for deformation fitting only)
class ARTimeVariantConstraintsInterface : public ARConstraintsInterface {
 public:
	/// add a soft constraint to specify the displacement between two frames
   virtual void addVelocityConstraint(const VelocityVectorARConstraint *c) = 0;
   /// add a soft constraint to specify the instantaneous velocity at a certain time
   virtual void addInstantaneousVelocityConstraint(const InstantaneousVelocityARConstraint *c) = 0;
};


}

#endif