//---------------------------------------------------------------------------
#ifndef DeformationToolsH
#define DeformationToolsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "RegularData.h"
//---------------------------------------------------------------------------


// DeformationTools - mw 10/11/2007
//
// misc. routines and helpers for the deformation project


namespace geoxl {

class InCorePCTopologyGraph;
class ARSequence;
class ARSettings;


bool DEFORMATION_API compareDescr( 
   const VertexDescriptor *other, 
   const VertexDescriptor *mustHave
);

DEFORMATION_API string endWithSlash(const string &str);
DEFORMATION_API string endWithBackSlash(const string &str);
   
/// fetches an attached octree (creates it if not present)
/// works only for UnstructuredInCorePointClouds and 
/// GenericSwapAllOutOfCorePointCloud that contain an UnstructuredInCorePointCloud
DEFORMATION_API AttachedIndexedOctree *fetchAndAttachOctree(PointCloud *pc, card32 createNumPtsPerLeaf = 20);
   
/// converts the position channel of the point cloud to a dynamic vector
DEFORMATION_API DVectorF convertPCToVector(PointCloud *pc);
   
/// fetch attached topology
DEFORMATION_API InCorePCTopologyGraph *getTopology(PointCloud *pc);   
   
/// put / replace attached topology
DEFORMATION_API void attachTopology(PointCloud *pc, InCorePCTopologyGraph *tg, bool visible = true);


/// replaces attached topology of target with that attached to source
/// if source has no topology, target will end up with no topology as well
DEFORMATION_API void assignTopology(PointCloud *source, PointCloud *target);   
   
DEFORMATION_API Matrix4d svdPseudoInvert4d(Matrix4d m, const float64 zeroTolerance = 1e-5);
DEFORMATION_API Matrix3d svdPseudoInvert3d(Matrix3d m, const float64 zeroTolerance = 1e-5);
   
DEFORMATION_API void prunePoints(
   UnstructuredInCorePointCloud *inPC, InCorePCTopologyGraph *inTG,
   UnstructuredInCorePointCloud *&outPC, InCorePCTopologyGraph *&outTG,
   const vector<bool> &toKeep
);

DEFORMATION_API void prunePoints(
   PointCloud *pc, InCorePCTopologyGraph *tg,
   const vector<bool> &toKeep
);

DEFORMATION_API void prunePointsAndReplace(ARSequence *seq, const vector<bool> &toKeep);

/// merges the two point clouds and their topology, including new edges between pc1 and pc2 
DEFORMATION_API PointCloud *mergePointClouds(PointCloud *pc1, PointCloud *pc2, ARSettings *settings);

DEFORMATION_API void removePointsAtEnd(PointCloud *pc, card32 endPtsStartIndex);

/// recomputes and unifies normals and reference frames
/// assumes topology attachment
DEFORMATION_API void recomputeNormalsAndFrames(PointCloud *pc, ARSettings *settings);

/// Helper for recomputeNormalsAndFrames: Estimate normals from topological neighborhood via PCA
DEFORMATION_API void recomputeFrames( PointCloud *pc, ARSettings *settings );

/// Helper for recomputeNormalsAndFrames: unify normal directions via region growing
DEFORMATION_API void recomputeUnifyNormals( PointCloud *pc, ARSettings *settings );

/// Helper for recomputeNormalsAndFrames: unify normal directions via Hoppe's minimal spanning tree
DEFORMATION_API void recomputeUnifyNormalsHoppe( PointCloud *pc, ARSettings *settings );

/// Helper for recomputeNormalsAndFrames: Find all topological neighbors of point 
/// 'pIndex' (directly connected or within 'influenceRadius')
DEFORMATION_API set<card32> getNeighborIndices(
	const card32 index,
	PointSet* ps,
	InCorePCTopologyGraph* topology,
	const float32 influenceRadius = -1 );	// if < 0, only directly connected neighbors are used

DEFORMATION_API PointSet *copySegment(PointSet *ps, mpcard startIndex, mpcard endIndex);

   
}

#endif