//---------------------------------------------------------------------------
#ifndef AROptimizationToolH
#define AROptimizationToolH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "AROptimizationContext.h"
//---------------------------------------------------------------------------

namespace geoxl {

class ARSettings;
class ARDeformationOptimizationContext;


///   mw - 09/27/2007
///
///   A tool that operates on a deformation sequence and an urshape to parametrize the deformation
///
class DEFORMATION_API AROptimizationTool : public Persistent {
	GEOXL_ABSTRACT_CLASS( AROptimizationTool )
 public:
	/// default: copy object properties...
	virtual void assign(const Object* obj, CopyContext *context = NULL);
	
	/// this method creates a context object that needs to be setup by the user of the deformation tool
	/// given this object, the tool itself can be initialized by calling "setup()"
	virtual AROptimizationContext *createOptContext() = 0;

   /// specifies the domain and the deformation context the tool is working on
	/// it also specifies a settings object that must not be deleted until
	/// while the tool is working
	/// the context is _not_ owned (i.e. finally deleted) by the AROptimizationTool
	/// it can be used after the optimization, e.g. for evaluation
	virtual void setup(AROptimizationContext *context, const ARSettings *settings) = 0;
};


}

#endif