//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "DeformationSolver.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ARConstraints.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_ABSTRACT_CLASS( DeformationSolver , 0 ) {
	BEGIN_CLASS_INIT( DeformationSolver );
}

void DeformationSolver::addGeneralConstraint(const GeneralARConstraint *c) 
{
	switch (c->getConstraintTypesHint()) {
		case GeneralARConstraint::POSITION_CONSTRAINT: {
			addPositionConstraint((PositionARConstraint*)c);
			break;
		}
		case GeneralARConstraint::VELOCITY_CONSTRAINT: {
			addVelocityConstraint((VelocityVectorARConstraint*)c);
			break;
		}
		case GeneralARConstraint::INST_VELOCITY_CONSTRAINT: {
			addInstantaneousVelocityConstraint((InstantaneousVelocityARConstraint*)c);
			break;
		}
		case GeneralARConstraint::HALF_SPACE_CONSTRAINT: {
			addHalfSpaceConstraint((ARHalfSpaceConstraint*)c);
			break;
		}
		case GeneralARConstraint::SPATIAL_DERIVATIVE_CONSTRAINT: {
			addSpatialDerivativeConstraint((SpatialDerivativeARConstraint*)c);
			break;
		}
		case GeneralARConstraint::SIMPLE_SPATIAL_DERIVATIVE_CONSTRAINT: {
			addSimpleSpatialDerivativeConstraint((SimpleSpatialDerivativeARConstraint*)c);
			break;
		}
		default: {
			throw ERangeCheck("DeformationSolver::addGeneralConstraint() - unknown constraint type.");
		}
	}

}

/// default implementation might not be optimal in general
void DeformationSolver::setIdentityMap() {
   beginConstraints();
   solve();
}

void DeformationSolver::solveGroup(GroupDeformationContext *groupContext, vector<DeformationSolver*> group) {
   solve();
   for (size_t i=0; i<group.size(); i++) {
      group[i]->solve();
   }
}
