#ifndef MMConstraintH
#define MMConstraintH

#include "GlobalDefinitions.h"
#include "MMNeighborContainer.h"

namespace geoxl {

/**
  * Holds for a constraint anchor point position, the indices of the nodal neighbors,
  * together with the shape functions evaluated at the anchor point.
  * We cache this information for efficient solving.
  */
	class DEFORMATION_API MMConstraint /*: public MMNeighborContainer*/ {
	private: 
		MMNeighborContainer * neighbors;
		bool neighborsOwner;

	public:


		MMConstraint(MMNeighborContainer *n, bool owner)  {
			neighbors = n;
			neighborsOwner = owner;
		}

		MMConstraint(MMConstraint &copy) /*: MMNeighborContainer(copy)*/ {			
			if (copy.neighborsOwner) {
				neighbors = new MMNeighborContainer(*copy.neighbors);
				neighborsOwner = true;
			}
			else {
				neighbors = copy.neighbors;
				neighborsOwner = false;
			}
		}

		virtual ~MMConstraint() {
			if (neighborsOwner) delete neighbors; 
			neighbors = 0;
			// I'm not responsible for deleting the neighbors
		}

		MMNeighborContainer * getNeighbors() { return neighbors; }
		void setNeighbors(MMNeighborContainer *n) { neighbors = n; }
			
};


}


#endif