#ifndef MMConstraintsH
#define MMConstraintsH

#include "GlobalDefinitions.h"

#include "MMPositionConstraint.h"
#include "MMHalfSpaceConstraint.h"
#include <vector>
#include <hash_map>
using namespace stdext;
using namespace std;

struct eqpair
{
  bool operator()(const pair< size_t, size_t > &p1, const pair< size_t, size_t > &p2) const
  {
    return p1.first<p2.first || (p1.first==p2.first && p1.second < p2.second);
  }
};

struct pairhash {
	size_t operator()(const pair< size_t, size_t > &key) const {
		//hash<int> H;
		return hash_value(key.first)+hash_value(key.second);
	}
};

struct MyHashTraits
{
	static const size_t bucket_size = 4;
	static const size_t min_buckets = 8;
	size_t operator()(const pair< size_t, size_t > & key) const
	{
		return hash_func(key);
	}
	bool operator()(const pair< size_t, size_t > & lhs, const pair< size_t, size_t > & rhs) const
	{
		return order_func(lhs, rhs);
	}

	private:

		pairhash hash_func;
		eqpair order_func;

};

class ABMatrix3f{
public:
	size_t  b;
	geoxl::Matrix3f m;

	ABMatrix3f(  const size_t &bb, const geoxl::Matrix3f &ma){
		 b=bb; m=ma;
	}
};

class ABMatrix3fVec{
public:
	size_t  a, b;
	geoxl::Matrix3f m;
	ABMatrix3fVec(){}
	ABMatrix3fVec(const size_t &aa, const size_t &bb, const geoxl::Matrix3f &ma){
		a=aa; b=bb; m=ma;
	}
};

class FastGetAB{
  public:
	FastGetAB(){
	}
	vector<vector<ABMatrix3f>> data;

	void resize(const size_t numNodes){
		data.resize(numNodes);
	}
	void clear(){
		data.clear();
		data.swap(data);
	}

	inline void createNewOrAddToExistingMatrix(const size_t &a, const size_t &b, const geoxl::Matrix3f &ma){
		size_t vecSize=data[a].size();
		for (size_t i=0; i<vecSize; i++)
		{
			if (data[a][i].b==b)
			{
				data[a][i].m+=ma;
				return;
			}
		}
		data[a].push_back(ABMatrix3f(b,ma));
	}

	size_t size(){
		size_t dataNum=data.size();
		size_t res=0;
		for (size_t i=0; i<dataNum; i++)
			res+=data[i].size();
		return res;
	}

};

class FastGetABIterator{

	size_t indexA, indexB, arrayAsize;
	const FastGetAB * object;
	bool isAtEnd;

public:
	FastGetABIterator(const FastGetAB &obj){
		object=&obj;
		indexA=0;
		indexB=0;
		arrayAsize= object->data.size();
		isAtEnd=false;
	}
	inline bool atEnd() const{
		return isAtEnd;
	}
	inline void begin(){
		while (indexA<arrayAsize && object->data[indexA].empty())
			indexA++;
		if (indexA>=arrayAsize)
			isAtEnd=true;
	}
	inline void next(){
		indexB++;
		if (indexB>=object->data[indexA].size())
		{
			indexA++;
			while (indexA<arrayAsize && object->data[indexA].empty())
				indexA++;
			indexB=0;
		}
		if (indexA>=arrayAsize)
			isAtEnd=true;
	}

	inline size_t getA()const{return indexA;}
	inline size_t getB()const{return object->data[indexA][indexB].b;}
	inline const geoxl::Matrix3f getMatrix() const{return object->data[indexA][indexB].m;}


};

typedef hash_map< pair< size_t, size_t >, geoxl::Matrix3f , MyHashTraits > MyMap; 
//typedef map< pair< size_t, size_t >, geoxl::Matrix3f > MyMap; 


namespace geoxl {

	class ARSettings;
	class MMDeformations;

	/**
	  * Holds for every frame, the cached constraints
	  * information.
	  */
	class DEFORMATION_API MMConstraints {

	protected:
		vector<MMPositionConstraint*> positionConstraints;
		vector<MMHalfSpaceConstraint*> halfSpaceConstraints;

		MyMap positionConstraintsQuadric;
		hash_map< card32, Vector3f > positionConstraintsLinear;
		float32 positionConstraintsConstant;

		FastGetAB fastGetIt;
		size_t numNodes;

	public:

		MMConstraints(const size_t numFrames, const size_t myFrameNum);
		virtual ~MMConstraints();

		void setNumNodes(size_t num);

		MMConstraints(MMConstraints &copy);

		inline mpcard nbPositionConstraints() {
			return positionConstraints.size();
		}

		inline mpcard nbHalfSpaceConstraints() {
			return halfSpaceConstraints.size();
		}

		inline MMPositionConstraint* getPositionConstraint(card32 index) {
			return positionConstraints[index];
		}

		inline MMHalfSpaceConstraint* getHalfSpaceConstraint(card32 index) {
			return halfSpaceConstraints[index];
		}

		inline void addHalfSpaceConstraint(MMHalfSpaceConstraint *c) {
			halfSpaceConstraints.push_back(c);
		}

		virtual void addPositionConstraint(MMPositionConstraint *c);
		//inline void addPositionConstraint(MMPositionConstraint *c) {
		//	positionConstraints.push_back(c);
		//}
		virtual float32 fCorrespondences(MMDeformations * deformations, float32 tempCorrespondenceParam);
		virtual void gCorrespondences(vector<Vector3f> &gradient, MMDeformations * deformations, float32 tempCorrespondenceParam);

		virtual void clearConstraints();

		void printMemUsage();

		//MMConstraints* copy() const;
	};
}


#endif