#include "StdAfx.h"
#include "MMConstraintsOOC.h"
#include "MMDeformations.h"
#include "ARSettings.h"
#include "BinaryObjectStreams.h"
#include "StringHelper.h"
#include <QtCore/QUuid>
#include <QtCore/QString>
using namespace geoxl;

mpcard MMConstraintsOOC::constraintsInMem = 0;
mpcard MMConstraintsOOC::maxConstraintsInMem = 2;
queue<MMConstraintsOOC*> MMConstraintsOOC::lruQueue;
bool MMConstraintsOOC::useConstraintsOOC=false;
string MMConstraintsOOC::OOCPathFilename="c:/ooc/AR2-constraints-temp";

MMConstraintsOOC::MMConstraintsOOC(const size_t numFrames, const size_t myFrameNum) 
:  MMConstraints(numFrames, myFrameNum){
	dataAfterLoading= new vector<ABMatrix3fVec>;
	positionConstraintsConstant = 0;
	numNodes=0;
	dataOnDisk=false;
	dataReloaded=false;
// 
// 	char tempName[10000];
// 	unsigned result = GetTempFileNameA(path.c_str(), "ADD", 0, tempName);
// 	fileName=OOCPathFilename + intToStr(myFrameNum) + "_" + tempName + ".tmp";

	QUuid uid;
	uid=uid.createUuid();
	
	
	fileName=OOCPathFilename + intToStr(myFrameNum) + "_UID" +uid.toString().toStdString() + ".tmp";
	cout << " created ooc constraint filename " << fileName << flush;

	constraintsInMem++;
	lruQueue.push(this);
	checkSwapOut();
}


MMConstraintsOOC::~MMConstraintsOOC() {
	clearConstraints();
	if (!dataOnDisk)
	{
		constraintsInMem--;
	}
	delete dataAfterLoading;
}

void MMConstraintsOOC::clearConstraints() {
	for (card32 i=0; i<positionConstraints.size(); i++) {
		delete positionConstraints[i];
	}
	positionConstraints.clear();
	for (card32 i=0; i<halfSpaceConstraints.size(); i++) {
		delete halfSpaceConstraints[i];
	}
	halfSpaceConstraints.clear();

	positionConstraintsConstant = 0;
	positionConstraintsLinear.clear();
	positionConstraintsQuadric.clear();
	fastGetIt.clear();
	delete dataAfterLoading;
	dataAfterLoading = new vector<ABMatrix3fVec>;
	dataReloaded=false;  //!!!!
	numNodes=0;


	//to clear ooc data just write num of constraints =0 to file 
	BinaryOutputObjectStream out(fileName.c_str());
	out.write(0);
	
}

float32 MMConstraintsOOC::fCorrespondences(MMDeformations * deformations, float32 tempCorrespondenceParam) {
	checkSwapIn();
	
	// constant term
	float32 result = positionConstraintsConstant;

	// linear terms
	for (hash_map< card32, Vector3f >::const_iterator iter=positionConstraintsLinear.begin(); iter!=positionConstraintsLinear.end(); ++iter) {
		const card32 index = iter->first;
		const Vector3f &deformation = deformations->getDeformation(index);
		result+=iter->second*deformation;
	}

	// 	// quadratic terms
	// 	for (MyMap::const_iterator iter=positionConstraintsQuadric.begin(); iter!=positionConstraintsQuadric.end(); ++iter) {
	// 		const card32 index1 = iter->first.first;
	// 		const card32 index2 = iter->first.second;
	// 		const Vector3f &deformation1 = deformations->getDeformation(index1);
	// 		const Vector3f &deformation2 = deformations->getDeformation(index2);
	// 		result+=deformation1*(iter->second*deformation2);
	// 	}


	if (dataReloaded)
	{
		size_t numEntries=dataAfterLoading->size();
		for (mpcard i=0; i<numEntries;i++)
		{
			const mpcard index1 = (dataAfterLoading[0])[i].a;
			const mpcard index2 = dataAfterLoading[0][i].b;
			const Vector3f &deformation1 = deformations->getDeformation(index1);
			const Vector3f &deformation2 = deformations->getDeformation(index2);
			result+=deformation1*( dataAfterLoading[0][i].m*deformation2);
		}
	}else{
		// quadratic terms FastGetABIterator
		FastGetABIterator iter(fastGetIt);
		for (iter.begin(); !iter.atEnd(); iter.next()) {
			const mpcard index1 = iter.getA();
			const mpcard index2 = iter.getB();
			const Vector3f &deformation1 = deformations->getDeformation(index1);
			const Vector3f &deformation2 = deformations->getDeformation(index2);
			result+=deformation1*(iter.getMatrix()*deformation2);
		}
	}

	// final result
	return tempCorrespondenceParam*result;
}

void MMConstraintsOOC::gCorrespondences(vector<Vector3f> &gradient, MMDeformations * deformations, float32 tempCorrespondenceParam) {
	checkSwapIn();
	
	// linear terms
	for (hash_map< card32, Vector3f >::const_iterator iter=positionConstraintsLinear.begin(); iter!=positionConstraintsLinear.end(); ++iter) {
		const card32 index = iter->first;
		if (!deformations->isActive(index)) continue;
		const card32 realIndex = deformations->getRealIndex(index);
		gradient[realIndex]+=iter->second*tempCorrespondenceParam;
	}

	// quadratic terms
	// 	for (MyMap::const_iterator iter=positionConstraintsQuadric.begin(); iter!=positionConstraintsQuadric.end(); ++iter) {
	// 		const card32 index1 = iter->first.first;
	// 		const card32 index2 = iter->first.second;
	// 		const Vector3f &deformation1 = deformations->getDeformation(index1);
	// 		const Vector3f &deformation2 = deformations->getDeformation(index2);
	// 		
	// 		const card32 realIndex1 = deformations->getRealIndex(index1);
	// 		const card32 realIndex2 = deformations->getRealIndex(index2);
	// 		
	// 		gradient[realIndex1]+=(iter->second*deformation2)*tempCorrespondenceParam;
	// 		gradient[realIndex2]+=(iter->second*deformation1)*tempCorrespondenceParam;
	// 	}

	if (dataReloaded)
	{
		size_t numEntries=dataAfterLoading->size();
		for (mpcard i=0; i<numEntries;i++)
		{
			const mpcard index1 = dataAfterLoading[0][i].a;
			const mpcard index2 = dataAfterLoading[0][i].b;
			const Vector3f &deformation1 = deformations->getDeformation(index1);
			const Vector3f &deformation2 = deformations->getDeformation(index2);

			const card32 realIndex1 = deformations->getRealIndex(index1);
			const card32 realIndex2 = deformations->getRealIndex(index2);

			gradient[realIndex1]+=(dataAfterLoading[0][i].m*deformation2)*tempCorrespondenceParam;
			gradient[realIndex2]+=(dataAfterLoading[0][i].m*deformation1)*tempCorrespondenceParam;
		}
	}
	else{

		// quadratic terms
		FastGetABIterator iter(fastGetIt);
		for (iter.begin(); !iter.atEnd(); iter.next()) {
			const mpcard index1 = iter.getA();
			const mpcard index2 = iter.getB();
			const Vector3f &deformation1 = deformations->getDeformation(index1);
			const Vector3f &deformation2 = deformations->getDeformation(index2);

			const card32 realIndex1 = deformations->getRealIndex(index1);
			const card32 realIndex2 = deformations->getRealIndex(index2);

			gradient[realIndex1]+=(iter.getMatrix()*deformation2)*tempCorrespondenceParam;
			gradient[realIndex2]+=(iter.getMatrix()*deformation1)*tempCorrespondenceParam;
		}
	}
}

void geoxl::MMConstraintsOOC::moveToDisk()
{
	debugOutput << " moveToDisk() " << fileName << "\n";
	debugOutput.flush();
	FastGetABIterator iter(fastGetIt);
	BinaryOutputObjectStream out(fileName.c_str());
	if (dataReloaded)
	{
		mpcard numEntries = dataAfterLoading->size();
		out.writeMaxPlatformCard(numEntries);
		for (mpcard i=0; i<numEntries;i++)
		{
			const mpcard index1 = dataAfterLoading[0][i].a;
			const mpcard index2 = dataAfterLoading[0][i].b;
			const Matrix3f m = dataAfterLoading[0][i].m;
			out.writeMaxPlatformCard(index1); out.writeMaxPlatformCard(index2);out.write(m);
		}
		
		/*vector<ABMatrix3fVec> testi;
		debugOutput <<" cap testi: " << testi.capacity() <<"\n";

		
		debugOutput <<" cap bevor clear: " << dataAfterLoading->capacity() <<"\n";
		dataAfterLoading->clear();
		debugOutput <<" cap nach clear: " << dataAfterLoading->capacity() <<"\n";*/
		delete dataAfterLoading;
		dataAfterLoading = new vector<ABMatrix3fVec>;
		/*debugOutput <<" cap nach delete/new: " << dataAfterLoading->capacity() <<"\n";
		debugOutput.flush();*/
	}else{
		mpcard numConstr=fastGetIt.size();
		out.writeMaxPlatformCard(numConstr);
		for (iter.begin(); !iter.atEnd(); iter.next()) {
			const mpcard index1 = iter.getA();
			const mpcard index2 = iter.getB();
			const Matrix3f m = iter.getMatrix();
			out.writeMaxPlatformCard(index1); out.writeMaxPlatformCard(index2);out.write(m);
		}
		fastGetIt.clear();
	}
	dataOnDisk=true;
	constraintsInMem--;
}

void geoxl::MMConstraintsOOC::reloadFromDisk()
{
	if (dataOnDisk)
	{
		debugOutput << " reloadFromDisk() " << fileName << "\n";
		debugOutput.flush();
		BinaryInputObjectStream in(fileName.c_str());
		card32 numConstr;
		in.read(numConstr);
		dataReloaded=numConstr>0;
		//debugOutput << " MMConstraintsOOC::reloadFromDisk() reloading " << numConstr << " constraints\n"; debugOutput.flush();
		card32 index1, index2;
		Matrix3f ma;
		dataAfterLoading->resize(numConstr);
		for (card32 i=0; i<numConstr; i++)
		{
			in.read(index1); in.read(index2); in.read(ma);
			dataAfterLoading[0][i]=(ABMatrix3fVec(index1, index2, ma));
		}
		constraintsInMem++;
	}
	dataOnDisk=false;
}

void geoxl::MMConstraintsOOC::checkSwapIn()
{
	if (dataOnDisk)
	{
		reloadFromDisk();
		checkSwapOut();
		if (this!=lruQueue.back())
		{
			lruQueue.push(this);
		}
	}
}

void geoxl::MMConstraintsOOC::checkSwapOut()
{
	if (constraintsInMem>maxConstraintsInMem)
	{
		//swap lru out
		MMConstraintsOOC* swapOutCand= lruQueue.front(); lruQueue.pop();
		swapOutCand->moveToDisk();
	}
}

void geoxl::MMConstraintsOOC::addPositionConstraint( MMPositionConstraint *c )
{
	if (dataReloaded)
	{
		throw PException(string("Assertion failed: MMConstraintsOOC::addPositionConstraint"));
	}
	checkSwapIn();
	MMConstraints::addPositionConstraint(c);
}