//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "MMDeformationContext.h"
//---------------------------------------------------------------------------
#include "SGEShape.h"
#include "SGAnimationNode.h"
#include "SGObjectNode.h"
#include "StringHelper.h"
#include "ARSettings.h"
#include "SGEDeformationSequence.h"
#include "SGEMeshlessDeformationSequence.h"
#include "PCCNodesTools.h"
#include "SGEPointCloud.h"
#include "MMNodes.h"
#include "InCorePCTopologyGraph.h"
#include "TopologyAttachment.h"
#include "SGRelativeTimeAnimationNode.h"
#include "PointWithDist.h"
#include "LinearAlgebra.h"
#include "AttachedMMDeformationContext.h"
#include "MMEdgeContractionSampler.h"
#include "MMFurthestPointSampler.h"
#include "MMGridSampler.h"
#include "MMTopologyGridSampler.h"
#include "Timer.h"
#include "MMConstraintsOOC.h"
#include "MMDeformationSolver.h"
#include "MMDeformationEvaluator.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( MMDeformationContext , 2 ) {
	BEGIN_CLASS_INIT( MMDeformationContext );
	ADD_OBJECT_PROP(nodes, 0, MMNodes::getClass(), true)
	ADD_VAR_ARRAY_PROP( deformations, 0, new ObjectClassProperty("", 0, NULL, MMDeformations::getClass(), true, NULL) )
	//ADD_VAR_ARRAY_PROP( ddeformations, 1, new ObjectClassProperty("", 0, NULL, MMDeformations::getClass(), true, NULL) )
	ADD_CARD32_PROP(realNbUnknowns, 0)
	ADD_BOOLEAN_PROP(haveTopology, 0)	
	ADD_BOOLEAN_PROP(nearestBaseIndexHack, 0)
	ADD_BOOLEAN_PROP(identity, 0)
    //ADD_FLOAT64_PROP(alpha, 2)
}

// ----

MMDeformationContext::MMDeformationContext() {
	//urshape = NULL;
	nodes = 0;
	realNbUnknowns = 0;
	haveTopology = false;
	nearestBaseIndexHack = false;
	identity = false;
}



MMDeformationContext::MMDeformationContext(MMDeformationContext &copy) {
	nodes = new MMNodes(*copy.getNodes());
	const size_t nbdeformations = copy.deformations.size();
	for (card32 i=0; i<nbdeformations; i++) {
		deformations.push_back(new MMDeformations(*copy.getDeformations(i)));
	}
	const size_t nbconstraints = copy.constraints.size();
	for (card32 i=0; i<nbconstraints; i++) {
		constraints.push_back(new MMConstraints(*copy.getConstraints(i)));
	}
	//for (card32 i=0; i<copy.nbHalfSpaceConstraints(); i++) {
	//	halfSpaceConstraints.push_back((ARHalfSpaceConstraint*)copy.getHalfSpaceConstraint(i)->copy());
	//}
	hash_map<mpint, MMNeighborContainer*>::const_iterator iter;
	for (iter=copy.cached_neighbors.begin(); iter!=copy.cached_neighbors.end(); ++iter) {
		cached_neighbors[iter->first]=new MMNeighborContainer(*(iter->second));
	}
	//for (card32 i=0; i<copy.no_id_neighbors.size(); i++) {
	//	no_id_neighbors.push_back(new MMNeighborContainer(*copy.no_id_neighbors[i]));
	//}
	realNbUnknowns = copy.realNbUnknowns;
	haveTopology = copy.haveTopology;
	nearestBaseIndexHack =copy.nearestBaseIndexHack;
	identity = copy.identity;
}

MMDeformationContext::~MMDeformationContext() {
	for (unsigned i=0; i<deformations.size(); i++) {
		delete deformations[i];
	}
	deformations.clear();
	for (unsigned i=0; i<ddeformations.size(); i++) {
		delete ddeformations[i];
	}
	ddeformations.clear();

	delete nodes;
	for (unsigned i=0; i<constraints.size(); i++) {
		delete constraints[i];
	}
	constraints.clear();
	//delete urshape;
	hash_map<mpint, MMNeighborContainer*>::const_iterator iter;
	for (iter=cached_neighbors.begin(); iter!=cached_neighbors.end(); ++iter) {
		delete iter->second;
	}
	for (card32 i=0; i<no_id_neighbors.size(); i++) {
		delete no_id_neighbors[i];
	}
	no_id_neighbors.clear();
	cached_neighbors.clear();
	//for (card32 i=0; i<halfSpaceConstraints.size(); i++) {
	//	delete halfSpaceConstraints[i];
	//}
	//halfSpaceConstraints.clear();
}

void MMDeformationContext::createDDeformations() {

	if (ddeformations.size() != deformations.size()){

		const size_t nbdeformations = deformations.size();

		for (card32 i=0; i<nbdeformations; i++) {
			ddeformations.push_back(new MMDeformations(*getDeformations(i)));
		}
	}
}

void MMDeformationContext::reinitialize(MMDeformationContext &copy) {
	// delete everything
	for (unsigned i=0; i<deformations.size(); i++) {
		delete deformations[i];
	}
	
	for (unsigned i=0; i<ddeformations.size(); i++) {
		delete ddeformations[i];
	}
	
	delete nodes;
	for (unsigned i=0; i<constraints.size(); i++) {
		delete constraints[i];
	}
	constraints.clear();
	//for (card32 i=0; i<halfSpaceConstraints.size(); i++) {
//		delete halfSpaceConstraints[i];
	//}
	//halfSpaceConstraints.clear();
	//delete urshape;
	hash_map<mpint, MMNeighborContainer*>::const_iterator iter;
	for (iter=cached_neighbors.begin(); iter!=cached_neighbors.end(); ++iter) {
		delete iter->second;
	}
	cached_neighbors.clear();
	for (card32 i=0; i<no_id_neighbors.size(); i++) {
		delete no_id_neighbors[i];
	}
	no_id_neighbors.clear();
	// copy everything
	nodes = new MMNodes(*copy.getNodes());
	const size_t nbdeformations = copy.deformations.size();
	for (card32 i=0; i<nbdeformations; i++) {
		deformations.push_back(new MMDeformations(*copy.getDeformations(i)));
	}
	
//	map<mpint, MMNeighborContainer*>::const_iterator iter;
	for (iter=copy.cached_neighbors.begin(); iter!=copy.cached_neighbors.end(); ++iter) {
		cached_neighbors[iter->first]=new MMNeighborContainer(*(iter->second));
	}
	const size_t nbconstraints = copy.constraints.size();
	for (card32 i=0; i<nbconstraints; i++) {
		//MMNeighborContainer * neighbors = copy.getNeighbors(copy.getConstraints(i)->getTopologicalAnchor(), posAt);
		//MMConstraint * constraint = new MMConstraint((const PositionARConstraint*)c->copy(), neighbors);
		MMConstraints * newConstraints = new MMConstraints(*copy.getConstraints(i));
		//for (card32 j=0; j<newConstraints->nbConstraints(); j++) {
		//	MMConstraint * newConstraint = newConstraints->getConstraint(j);
		//	newConstraint->setNeighbors(getNeighbors(newConstraint->getConstraint()->getAnchor().getTopologicalAnchor(), newConstraint->getNeighbors()->getPos()));
		//}
		constraints.push_back(newConstraints);
	}
	//for (card32 i=0; i<copy.nbHalfSpaceConstraints(); i++) {
	//	halfSpaceConstraints.push_back((ARHalfSpaceConstraint*)copy.getHalfSpaceConstraint(i)->copy());
	//}
	//for (card32 i=0; i<copy.no_id_neighbors.size(); i++) {
	//	no_id_neighbors.push_back(new MMNeighborContainer(*copy.no_id_neighbors[i]));
	//}
	realNbUnknowns = copy.realNbUnknowns;
	haveTopology=copy.haveTopology;
	nearestBaseIndexHack = copy.nearestBaseIndexHack;
	identity = copy.identity;
}


MMDeformations *MMDeformationContext::createDeformationStorageObject(PointCloud *pc, const ARSettings *settings) 
{
	return new MMDeformations(nbNodes());
}

void MMDeformationContext::checkAndAddTopology(UnstructuredInCorePointCloud * pc) {
	// check if there is already a topology
	AttachedDataContainer *attachments = pc->getAttachments();
	TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(attachments->getData("TopologyAtt"));
	if (topAtt) {
		// warning("There was already a topology attachment.");
		return;
	}
	warning("Creating a new topology attachment in MMDeformationContext.");

	PointSet *ps = pc->getPointSet();
	if (!ps->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
		error("No position channel");
		return;
	}
	InCorePCTopologyGraph *top = new InCorePCTopologyGraph();
	VertexDescriptor vd;
	vd.pushAttrib(VAD("vindex", 2, VAD::DATA_FORMAT_INT32));
	top->clearAndSetup(&vd);
	AAT POS = ps->getAAT("position");
	card32 n = ps->getNumEntries();
    HierarchicalKNNIterator knn(pc, 20, NULL);
	card32 k = 20; // TODO: make this a param
	for (card32 i=0; i<n; i++) {
		Vector3f pos = ps->get3f(i, POS);
	   knn.setSeekPointAndReset(pos);
	   card32 j = 0;
	   while (!knn.atEnd() && j < k) {
			int32 i2 = knn.getPointSetPointNumber();
			if (i != i2) {
				top->addEdge(i, i2);
			}
			knn.next();
			j++;
	   }		
	}
	
	
	topAtt = new TopologyAttachment();
	topAtt->setup("TopologyAtt", AttachedData::ADF_PERSISTENT);
	topAtt->setTopology(top);
	attachments->attachData(topAtt);
	
}

void MMDeformationContext::initializeFromShape(const SceneObject *obj, Vector2f timeInterval, const ARSettings *settings) {
   baseResolution = 0;
   {
      SceneObjectIterator *it = const_cast<SceneObject*>(obj)->createIterator(SOIT::CAP_HIERARCHICAL);
      HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(it);
      if (hit) {
         baseResolution = hit->getBoundingBox().getMaxSideLength() / settings->gridSize;
      }
      delete it;
   }
   
	int numFrames = (int)(timeInterval[1] - timeInterval[0])+1;
	debugOutput << "initializeFromShape for " << timeInterval << " frames.\n";
	if (numFrames < 1) {
		throw ERangeCheck("MMDeformationContext::initializeFromShape() - timeIntervall invalid.");
	}
	if (identity) {
		nodes = new MMNodes();
	}
	else {
		
	const UnstructuredInCorePointCloud *pc = dynamic_cast<const UnstructuredInCorePointCloud*>(obj);

	if (!pc) {
		throw PException("MMDeformationContext::initializeFromShape() - obj is not of correct type.");
	}
	//delete urshape;
	PointCloud * urshape = (PointCloud*)(pc->copy());
	UnstructuredInCorePointCloud * incore_urshape =  dynamic_cast<UnstructuredInCorePointCloud*>(urshape);
	
	if (!incore_urshape) {
		delete urshape;
		throw PException("MMDeformationContext::initializeFromShape() - obj is not of correct type.");
	}
	Timer t;
	checkAndAddTopology(incore_urshape);
	debugOutput << "MMDeformationContext::checkAndAddTopology: " << t.getDeltaValue() << "\n";
	
	MMSampler *sampler = (MMSampler*)settings->mmNodesSampler->newInstance();
	
	//MMEdgeContractionSampler sampler;
	//MMFurthestPointSampler sampler;
	//MMGridSampler sampler;
	//MMTopologyGridSampler sampler;

	nodes = sampler->createNodes(settings, incore_urshape);
	
	//delete nodes;

	sampler->initializeCache(cached_neighbors, haveTopology);
	
	delete sampler;
	delete incore_urshape;
	
	cout << "sampler.createNodes time: " << t.getDeltaValue() << " ms" <<endl;

	/*float32 offset = settings->nodalOffset;
	float32 radius = settings->nodalRadius;
	
	PCCNodesTools *nd = new PCCNodesTools(offset,radius,1);

//	vector<Vector3f> nodePositions;
	//getNodes(radius, nodePositions);
	UnstructuredInCorePointCloud *uc = nd->edgeContractionSampleMMNodesPointCloud(incore_urshape, offset, radius);

	//nd->testNodes(uc,incore_urshape,1,radius);
	
	//addPointCloud(scene,uc,"Deformation Nodes");

	float32 newRadius = nd->testNodes(uc,incore_urshape,4,radius);
	
	//PointSet * ps = uc->getPointSet();
	//cout << "Total number of nodes: " << ps->getNumEntries() << endl;
	
	

	// for now sample some nodes in the [0,1]^3 interval
	//nodes = new MMNodes(uc, newRadius + 0.01);
	if(newRadius>radius)
		nodes = new MMNodes(uc, newRadius + 0.1);
	else
		nodes = new MMNodes(uc, radius);
	//nodes = new MMNodes(uc, radius);
	//nodes = new MMNodes(nodePositions, 2.5*radius);*/
	
	// copy the nodes into nodes
	/*
	AAT posAAT = uc->getAAT("position");
	for (card32 i=0; i<uc->getNumPoints(); i++) {
		Vector3f pos = ps->get3f(i, posAAT);
		nodes->addNode(new MMNode(pos, 3*radius));
	}
	*/
	
	
	/*float32 step = 0.2f;
	for (float32 x=0; x<=1; x+=step) {
		for (float32 y=0; y<=1; y+=step) {
			for (float32 z=0; z<=1; z+=step) {
				Vector3f pos; pos[0] = x; pos[1] = y; pos[2] = z;
				nodes->addNode(new MMNode(pos, 3*step));
			}
		}
	}*/
	nodes->initializeNodes();
	cout << "nodes->initializeNodes(): " << t.getDeltaValue() << endl;
	
	}
	
	for (unsigned i=0; i<constraints.size(); i++) {
		delete constraints[i];
	}
	
	for (unsigned i=0; i<deformations.size(); i++) {
		delete deformations[i];
	}
	
	for (unsigned i=0; i<ddeformations.size(); i++) {
		delete ddeformations[i];
	}
	
	
	constraints.clear();
	deformations.clear();
	
	deformations.resize(numFrames);
	constraints.resize(numFrames);
	MMConstraintsOOC::maxConstraintsInMem=settings->maxConstraintsInMem;
	MMConstraintsOOC::OOCPathFilename=settings->constraintsTempFileDir;
	for (int i=0; i<numFrames; i++) {
		deformations[i] = createDeformationStorageObject(0, settings);
		if (MMConstraintsOOC::useConstraintsOOC && settings->useConstraintsOOC)
			constraints[i] = new MMConstraintsOOC(numFrames, i);
		else
			constraints[i] = new MMConstraints(numFrames, i);
	}
	
	setWindow(0,numFrames);
	computeRealIndices();

	nearestBaseIndexHack = settings->nearestBaseIndexHack;
}

void MMDeformationContext::writeNodesVisToSceneGraph(Scene *scene, string path, string subgraphName) {
	// TODO: fix this
	SGListNode *list = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(scene, path));
	if (!list) {
		throw PException("MMDeformationContext::writeNodesVisToSceneGraph() - no list exists at the given path");
	}
	SceneGraphNode *tnode = getSceneGraphNodeByName(scene, path+subgraphName);
	SGListNode *node = dynamic_cast<SGListNode*>(tnode);
	if (tnode && !node) {
	   throw PException("MMDeformationContext::writeNodesVisToSceneGraph() - the given <<path+subgraphName>> is already occupied by a non-list node.");
	}
	if (!tnode) {
	   node = new SGRelativeTimeAnimationNode();
	   tnode = node;
	   node->setName(subgraphName);
	   list->addChildNode(node);
	   SGRelativeTimeAnimationNode *rtList = dynamic_cast<SGRelativeTimeAnimationNode*>(list);
	   if (rtList) {
	      rtList->setDuration(rtList->getChildIndex(subgraphName), deformations.size());
	   }
	}
    node->clear();	
	for (unsigned i=0; i<deformations.size(); i++) {
		SGObjectNode *sgNode = new SGObjectNode();
		UnstructuredInCorePointCloud * pc = getDisplacedNodes(i);
		
		sgNode->clearAndSetup(pc);
		sgNode->setName(string("frame")+intToStr(i));
		pc->setMaterialIndex(4);
		node->addChildNode(sgNode);
	}
}


void MMDeformationContext::computeRealDeformations() {
	const card32 nbframes = nbFrames();
	for (card32 frame=0; frame<nbframes; frame++) {
		MMDeformations * deformations = getDeformations(frame);
		if (!deformations->isActive()) continue;
		//if(!getFrameActive(frame))
		//	continue;
			
		const card32 nbnodes = nbNodes();
		for (card32 i=0; i < nbnodes; i++) {
			Vector3f& real_deformation = deformations->getRealDeformation(i);
			real_deformation.setZero();
			MMNode * node = getNodes()->getNode(i);
			for (card32 j=0; j < node->nbNeighbors(); j++) {
				const card32 neighbor_index = node->getNeighborIndex(j);	
				const Vector3f &neighbor_deformation = deformations->getDeformation(neighbor_index);
				//cout << neighbor_deformation[0] << " " << neighbor_deformation[1] << " " << neighbor_deformation[2] << endl;
				const float32 shapeValue = node->getShapeValue(j);
				real_deformation+=neighbor_deformation*shapeValue;
			}
		}
	}
}

void MMDeformationContext::getDeformationAndGradient(const Vector3f &pos, mpint id, Vector3f &deformation, Matrix3f &gradient, bool computeGradient, card32 frame) {
	// first compute neighbors
	//MMNeighborContainer neighbors(pos);
	//nodes->computeNeighbors(&neighbors);
	bool owner;
	MMNeighborContainer * neighbors = getNeighbors(id, pos, owner);
	// compute deformation
	//deformation = getDeformation(pos, *neighbors, frame);
	deformation = getDeformation(neighbors->getPos(), *neighbors, frame);
	if (computeGradient) {
		//gradient = getDeformationGradient(pos, deformation, *neighbors, frame);
		gradient = getDeformationGradient(neighbors->getPos(), deformation, *neighbors, frame);
	}
	if (owner) delete neighbors;
}

Vector3f MMDeformationContext::getDeformation(const Vector3f &pos, mpint id, card32 frame) {
	bool owner;
	MMNeighborContainer * neighbors = getNeighbors(id, pos, owner);
	//assert(neighbors->nbNeighbors()>0);
	//const Vector3f result =  getDeformation(pos, *neighbors, frame);
	const Vector3f result =  getDeformation(neighbors->getPos(), *neighbors, frame);
	if (owner) delete neighbors;
	/*map<int,MMNeighborContainer*>::const_iterator iter;
	
	if(id == ARConstraintAnchor::NO_TOP_ANCHOR){
		MMNeighborContainer *neighbors = new MMNeighborContainer(pos);
		Vector3f result = getDeformation(pos, *neighbors, frame);
		return result;
	}	
	
	iter = cached_neighbors.find(id);
	if (iter!=cached_neighbors.end()) {
		//cout << "retreiving cache: " << id << endl;
		return getDeformation(pos, *(iter->second), frame);
	}
	else {
		//cout << "not in cache: " << id << endl;
		MMNeighborContainer *neighbors = new MMNeighborContainer(pos);
		Vector3f result = getDeformation(pos, *neighbors, frame);
		cached_neighbors[id]=neighbors;
		return result;
	}*/
	return result;
}

MMNeighborContainer * MMDeformationContext::getNeighbors(mpint id, const Vector3f &pos, bool &owner) {

	// first check if valid id and if pos corresponds to the id pos;
	bool idGiven = (id != ARConstraintAnchor::NO_TOP_ANCHOR);
	hash_map<mpint,MMNeighborContainer*>::const_iterator iter;
	bool idExists = false;
	if (idGiven) {
		iter = cached_neighbors.find(id);
		idExists = (iter!=cached_neighbors.end());
	}
	bool idClean = true;
	if (idExists) {
		if ((iter->second->getPos()-pos).getSqrNorm() > 0.0000001f) idClean = false;
	}
	
	if ((!idGiven) || (!idClean)){
		MMNeighborContainer *neighbors = 0;
		//assert(false);
		//cout << "No anchor" << endl;
		if (haveTopology) {
			if (nearestBaseIndexHack) {
				owner = false;
				id = nodes->getClosestBaseIndex(pos);
				hash_map<mpint,MMNeighborContainer*>::const_iterator iter = cached_neighbors.find(id);
				pAssert(iter!=cached_neighbors.end());
				neighbors = iter->second;
				//neighbors = new MMNeighborContainer(*iter->second);
				//neighbors->setBaseIndex(id);
				//neighbors->clearShapeValues(pos, getNodes());				
			}
			else {
				owner = true;					
				id = nodes->getClosestBaseIndex(pos);				
				//neighbors = new MMNeighborContainer(pos, id);
				//nodes->computeNeighbors(neighbors);
				hash_map<mpint,MMNeighborContainer*>::const_iterator iter = cached_neighbors.find(id);
				pAssert(iter!=cached_neighbors.end());

				neighbors = new MMNeighborContainer(*iter->second);
				neighbors->clearShapeValues(pos, nodes);
				//nodes->computeNeighbors(neighbors);
				//cout << "Neighbors: " << neighbors->nbNeighbors() << endl;
				//bool success;
				//nodes->evaluateNodeShapeFunctions(neighbors, success);
			}
		}
		else {
			neighbors = new MMNeighborContainer(pos);
			neighbors->setBaseIndex(id);
			nodes->computeNeighbors(neighbors);		
			owner = true;
		}
		if (idGiven && !idExists) {
			owner = false;
			cached_neighbors[id] = neighbors;
		}
		return neighbors;		
	}	
	else if (idGiven && !idExists) {
		error("Id given for neighborcontainer, but id does not exist");
		return 0;
	}
	else {
		//cout << "retreiving cache: " << id << endl;
		MMNeighborContainer * cont = iter->second;
		//float32 sqrDist = (cont->getPos()-pos).getSqrNorm();
		owner = false;
		return iter->second;		
	}	
}

Vector3f MMDeformationContext::getDeformation(const Vector3f &pos, MMNeighborContainer &neighbors, card32 frame) {
	// first compute node neighbors
	//MMNeighborContainer neighbors(pos);
	/*if (neighbors.nbNeighbors()==0) nodes->computeNeighbors(&neighbors);
	//cout << "nbneighbors: " << neighbors.nbNeighbors() << endl;
	if (neighbors.nbNeighbors()<5) {
		cout << "Error: only " << neighbors.nbNeighbors() << " neighbors. Returning 0 deformation." << endl;
		return makeVector3f(0,0,0);
	}
	
	bool success;
	nodes->evaluateNodeShapeFunctions(&neighbors, success);
	Vector3f result; result.setZero();
	if (!success) {
		cout << "deformation failed due to failed shape function evaluation, returning 0 deformation" << endl;
		return result;
	}*/

	if (!neighbors.getShapeValuesInitialized()) {
		bool success;
		nodes->evaluateNodeShapeFunctions(&neighbors, success);		
		//neighbors.setShapeValuesInitialized();
	}
	
	Vector3f result; result.setZero();
	float32 totValue = 0;
	for (card32 j=0; j<neighbors.nbNeighbors(); j++) {
		const card32 neighbor_index = neighbors.getNeighborIndex(j);	
		const Vector3f &neighbor_deformation = getDeformations(frame)->getDeformation(neighbor_index);
		
		const float32 shapeValue = neighbors.getShapeValue(j);
		
		totValue += shapeValue;
		
		result+=neighbor_deformation*shapeValue;
	}
	
	if(fabs(totValue-1)>0.2){
		cout << "Wrong results at position " << pos[0] << " " << pos[1] << " " << pos[2] << " \t shape value = " << totValue << endl;
	}
	
	return result;
}

Matrix3f MMDeformationContext::getDeformationGradient(const Vector3f &pos, const Vector3f &myDeformation, MMNeighborContainer &neighbors, card32 frame) {
	// first compute node neighbors
	/*if (neighbors.nbNeighbors()==0) nodes->computeNeighbors(&neighbors);
	//MMNeighborContainer neighbors(pos);
	// nodes->computeNeighbors(&neighbors);
	//cout << "nbneighbors: " << neighbors.nbNeighbors() << endl;
	if (neighbors.nbNeighbors()<5) {
		cout << "Error: only " << neighbors.nbNeighbors() << " neighbors. Returning 0 deformation." << endl;
		return makeMatrix3f(1,0,0,0,1,0,0,0,1);
	}
	bool success;	
	nodes->evaluateNodeGradientShapeFunctions(&neighbors, success);
	Matrix3f result; result.setZero();
	if (!success) {
		cout << "deformation failed due to failed shape function evaluation, returning 0 deformation gradient" << endl;
		return result;
	}*/

	if (!neighbors.getShapeValuesInitialized()) {
		bool success;
		nodes->evaluateNodeShapeFunctions(&neighbors, success);
		//neighbors.setGradientShapeValuesInitialized();
	}

	if (!neighbors.getGradientShapeValuesInitialized()) {
		bool success;
		nodes->evaluateNodeGradientShapeFunctions(&neighbors, success);
		//neighbors.setGradientShapeValuesInitialized();
	}
	Matrix3f result; /*result.setZero();*/
	result = makeMatrix3f(1,0,0,0,1,0,0,0,1);
	for (card32 j=0; j<neighbors.nbNeighbors(); j++) {
		const card32 neighbor_index = neighbors.getNeighborIndex(j);	
		const Vector3f &neighbor_deformation = getDeformations(frame)->getDeformation(neighbor_index);
		const Vector3f shapeValue = neighbors.getGradientShapeValues2(j); 
		//cout << shapeValue[0] << " " << shapeValue[1] << " " << shapeValue[2] << endl;
		//for (card32 k=0; k<3; k++) result[k]+=(myDeformation-neighbor_deformation)*shapeValue[k];
		for (card32 k=0; k<3; k++) result[k]+=(neighbor_deformation)*shapeValue[k];
	}
	return result;
}

void MMDeformationContext::visualize(Scene * scene) {
	// let's add a pointcloud for the urshape nodes
	UnstructuredInCorePointCloud * pc = new UnstructuredInCorePointCloud();
	PointSet * ps = new PointSet();
	VertexDescriptor vd;
	vd.pushAttrib( mVAD( "position", 3, VAD::DATA_FORMAT_FLOAT32 ) );
	ps->clearAndSetup( 1,nodes->nbNodes(), &vd );
	pc->setPointSet( ps );
	for (card32 i=0; i<nodes->nbNodes(); i++) {
		const Vector3f &pos = nodes->getNode(i)->getPos()+getDeformation(i,0);		
		ps->addPoint((char*)&pos);
	}
	addPointCloud( scene, pc, "root/deformed_nodes" );
}

struct MyVertexStruct {
   Vector3f pos;
   float32 r;
};

UnstructuredInCorePointCloud* MMDeformationContext::getDisplacedNodes(card32 frame) {
	UnstructuredInCorePointCloud * pc = new UnstructuredInCorePointCloud();
	PointSet * ps = new PointSet();
	VertexDescriptor vd;
	vd.pushAttrib( mVAD( "position", 3, VAD::DATA_FORMAT_FLOAT32 ) );
	vd.pushAttrib( mVAD( "radius", 1, VAD::DATA_FORMAT_FLOAT32 ) );
	ps->clearAndSetup( 1, 0 , &vd );
	pc->setPointSet( ps );
	//
	for (card32 i=0; i<nodes->nbNodes(); i++) {
		Vector3f pos = nodes->getNode(i)->getPos();
		float32 radius = nodes->getNode(i)->getRadius();
		pos += getDeformation(i,frame);
		MyVertexStruct vs;
		vs.pos = pos;
		vs.r = radius;
		ps->addPoint((char*)&vs);
	}
	debugOutput << "nbNodes = " << nodes->nbNodes() << "\n";
	return pc;
}
/*
bool compare_xyz(PointWithDist &p1, PointWithDist &p2) {
      if (p1.pos[0]<p2.pos[0]) return true;
	  else if (p1.pos[0]==p2.pos[0]) {
		  if (p1.pos[1]<p2.pos[1]) return true;
		  else if (p1.pos[1]==p2.pos[1]) {
			  return p1.pos[2]<p2.pos[2];
		  }
		  else return false;
	  }
	  else return false;
} 
bool compare_dist(PointWithDist &p0, PointWithDist &p1) {
     return p0.sqrDist>p1.sqrDist;
} 
*/

void MMDeformationContext::getNodes(const UnstructuredInCorePointCloud *pc, const float radius, vector<Vector3f> &nodes) {
	/*// get the vertex with the smallest (x,y,z) coordinate
	// make this a node
	//const UnstructuredInCorePointCloud *pc = dynamic_cast<const UnstructuredInCorePointCloud*>(urshape);
	AAT posAAT = pc->getAAT("position");
	AAT normalAAT = pc->getAAT("normal");
	const PointSet *ps = pc->getPointSet();
	const card32 nbpoints = ps->getNumEntries();

	vector<PointWithDist> uncoveredPoints;
	for (card32 i=0; i<nbpoints; i++) {
		uncoveredPoints.push_back(PointWithDist(ps->get3f(i, posAAT), ps->get3f(i, normalAAT), 100000.0f));
	}
	sort(uncoveredPoints.begin(), uncoveredPoints.end(), compare_xyz);

	while (!uncoveredPoints.empty()) {
		// create new node from first position
		const Vector3f &newpos = uncoveredPoints[0].pos;
		const Vector3f &normal = uncoveredPoints[0].normal;
		nodes.push_back(newpos);
		nodes.push_back(newpos + normal*(0.5f*radius));
		nodes.push_back(newpos - normal*(0.5f*radius));
		// go over all points and update dist and add if not too close to new node
		vector<PointWithDist> temp;
		for (card32 i=0; i<uncoveredPoints.size(); i++)  {
			PointWithDist &p = uncoveredPoints[i];
			float32 sqrDist = (p.pos-newpos).getSqrNorm();
			if (sqrDist>radius*radius) {
				p.sqrDist = p.sqrDist < sqrDist?p.sqrDist:sqrDist;
				temp.push_back(p);
			}
		}
		uncoveredPoints = temp;
		sort(uncoveredPoints.begin(), uncoveredPoints.end(), compare_dist);
	}*/
}

void MMDeformationContext::setWindow(card32 startFrame, card32 endFrame) {
	card32 index = 0;
	const card32 nbframes = nbFrames();
	
	if(endFrame<0)
		endFrame = nbframes;
		
	//framesActive.clear();
	
	for (card32 i=0; i<nbframes; i++) {	
		if(i>=startFrame && i<=endFrame){
			deformations[i]->setActive(true);
			//framesActive.push_back(true);
		}
		else{
			deformations[i]->setActive(false);
			//framesActive.push_back(false);
		}
	}
}

//bool MMDeformationContext::getFrameActive(card32 i) {
//	return framesActive[i];
//}


void MMDeformationContext::computeRealIndices() {
	card32 index = 0;
	const card32 nbframes = nbFrames();
	for (card32 i=0; i<nbframes; i++) {
		MMDeformations * deformations = getDeformations(i);
		if (!deformations->isActive()) continue;
		//if(getFrameActive(i)){
			//MMDeformations * deformations = getDeformations(i);
			const card32 nbnodes = deformations->nbDeformations();
			for (card32 j=0; j<nbnodes; j++) {
				const bool active = deformations->isActive(j);
				if (active) {
					deformations->setRealIndex(j, index);
					index++;
				}
			}
		//}
	}
	// this is the actual number of deformation vector unknowns
	setRealNbUnknowns(index);
}

void MMDeformationContext::setActive(const card32 frame, const card32 node, const bool active) {
	if (frame<0 || frame>=nbFrames()) return;
	if (node<0 || node>=nbNodes()) return;

	getDeformations(frame)->setActive(node, active);
}

float32 MMDeformationContext::getDuration() 
{
   card32 n = nbFrames();
   return (n==0) ? 0 : n-1;
}

void MMDeformationContext::clearConstraints() {
	for (card32 i=0; i<constraints.size(); i++) constraints[i]->clearConstraints();
	//for (card32 i=0; i<halfSpaceConstraints.size(); i++) {
	//	delete halfSpaceConstraints[i];
	//}
	//halfSpaceConstraints.clear();
}

Vector3f MMDeformationContext::getDeformation(card32 index, card32 frame){
	MMNode * node = nodes->getNode(index);
	MMDeformations * deformation = deformations[frame];
	const card32 nbneighbors = node->nbNeighbors();
	Vector3f result = makeVector3f(0,0,0);
	for (card32 i=0; i<nbneighbors; i++) {
		const float32 shapeValue = node->getShapeValue(i);
		const Vector3f &u = deformation->getDeformation(node->getNeighborIndex(i));
		result+=(u*shapeValue);
	}
	return result;
}

void MMDeformationContext::write(OutputObjectStream *s) const 
{
   ARDeformationOptimizationContext::write(s);
   mpcard n = cached_neighbors.size();
   s->writeMaxPlatformCard(n);
   stdext::hash_map<mpint, MMNeighborContainer*>::const_iterator pos = cached_neighbors.begin();
   stdext::hash_map<mpint, MMNeighborContainer*>::const_iterator end = cached_neighbors.end();
   while (pos != end) {
      s->writeMaxPlatformInt(pos->first);
      s->writeObject(pos->second);
      ++pos;
   }   
}

void MMDeformationContext::read(InputObjectStream *s) 
{
   ARDeformationOptimizationContext::read(s);
   mpcard n;
   s->readMaxPlatformCard(n);
   for (mpcard i=0; i<n; i++) {
      pair<mpint, MMNeighborContainer*> p;
      s->readMaxPlatformInt(p.first);
      s->readAnyObject(p.second);
      cached_neighbors.insert(p);
   }   
}

DeformationSolver *MMDeformationContext::createSolver() 
{
   return new MMDeformationSolver();
}

DeformationEvaluator *MMDeformationContext::createEvaluator() 
{
   return new MMDeformationEvaluator();
}

float32 MMDeformationContext::getBaseResolution() 
{
   return baseResolution;
}
