//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "MMDeformationEvaluator.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( MMDeformationEvaluator , 0 ) {
	BEGIN_CLASS_INIT( MMDeformationEvaluator );
}

MMDeformationEvaluator::MMDeformationEvaluator() : context(0) {
}

MMDeformationEvaluator::~MMDeformationEvaluator() {
	context = 0;
}

AROptimizationContext *MMDeformationEvaluator::createOptContext() 
{
	throw PException("MMDeformationEvaluator::createDeformationOptContext() - please use MMDeformationSolver to create context.");
}


void MMDeformationEvaluator::setup(AROptimizationContext *context, const ARSettings *settings) 
{
	if (!dynamic_cast<MMDeformationContext*>(context)) {
		throw ETypeMismatch("MMDeformationSolver::setup() - context does not have the correct dynamic type.");
	}
	this->context = (MMDeformationContext*)context;
	this->settings = settings;	
}

Vector3f MMDeformationEvaluator::getPosition(const ARConstraintAnchor &at) {
	if (context->isIdentity()) return at.getPos();
	int32 topAnchor = at.getTopologicalAnchor();	
	const Vector3f &deformation = context->getDeformation(at.getPos(), topAnchor, time);
	return at.getPos()+deformation;
}

Vector3f MMDeformationEvaluator::getVelocity(const ARConstraintAnchor &at){
   notImplemented();
   return makeVector3f(0,0,0);
}

Vector3f MMDeformationEvaluator::getAcceleration(const ARConstraintAnchor &at){
   notImplemented();
   return makeVector3f(0,0,0);
}

Matrix3f MMDeformationEvaluator::getSpatialGradient(const ARConstraintAnchor &at) {
	Vector3f deformation;
	Matrix3f gradient;
	if (context->isIdentity()) {
		gradient.setIdentity();
	}
	else context->getDeformationAndGradient(at.getPos(), at.getTopologicalAnchor(), deformation, gradient, true, time);
    return gradient;
}

void MMDeformationEvaluator::beginQuery(float32 time) 
{
	this->time = time;	
}

void MMDeformationEvaluator::endQuery() {}



