//---------------------------------------------------------------------------
#ifndef MMDeformationSolverH
#define MMDeformationSolverH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "DeformationSolver.h"
#include "MMDeformationContext.h"

//---------------------------------------------------------------------------

namespace geoxl {

//class MeshlessCachedConstraints;
//class SGEMeshlessDeformationSequence;
//class UnstructuredInCorePointCloud;
//class AttachedNodeInformation;



class DEFORMATION_API MMDeformationSolver : public DeformationSolver {
	GEOXL_CLASS( MMDeformationSolver )
 private:

	// Stores the nodes, the deformations for each frame, etc.
	MMDeformationContext * context;
	const ARSettings *settings;

	
	float32 tempCorrespondenceParam;
	
	// Objective function evaluations for all different energies we have
	float32 fRigidAndVolume();
	float32 fCorrespondences();
	float32 fAcceleration();
	float32 fVelocity();
	float32 fHalfSpace();

	// Gradient function evaluations for all different energies we have
	void gRigidAndVolume(std::vector<Vector3f> &gradient);
	void gCorrespondences(std::vector<Vector3f> &gradient);
	void gAcceleration(std::vector<Vector3f> &gradient);
	void gVelocity(std::vector<Vector3f> &gradient);
	void gHalfSpace(std::vector<Vector3f> &gradient);

	// Diagonal entries of the Hessian (Preconditioning)
	void mRigidAndVolume(vector<Vector3f> &conditioner);
	void mCorrespondences(std::vector<Vector3f> &conditioner);
	void mAcceleration(std::vector<Vector3f> &conditioner);
	void mVelocity(std::vector<Vector3f> &conditioner);
	
	// Diagonal entries of the Hessian (Preconditioning)
	double d2aRigidAndVolume();
	double d2aCorrespondences();

	//void mAcceleration(std::vector<Vector3f> &conditioner);
	//void mVelocity(std::vector<Vector3f> &conditioner);
	
	
	
 public:

	 MMDeformationSolver();

	 ~MMDeformationSolver();

	virtual void addPositionConstraint(const PositionARConstraint *c);
	virtual void addVelocityConstraint(const VelocityVectorARConstraint *c);
	virtual void addHalfSpaceConstraint(const ARHalfSpaceConstraint *c);
	virtual void addInstantaneousVelocityConstraint(const InstantaneousVelocityARConstraint *c);
   virtual void addSpatialDerivativeConstraint(const SpatialDerivativeARConstraint *c);
   virtual void addSimpleSpatialDerivativeConstraint(const SimpleSpatialDerivativeARConstraint *c);

	AROptimizationContext *createOptContext();
	virtual void setup(AROptimizationContext *context, const ARSettings *settings);

	 virtual void solve();
	 // compute function
	 float32 func();
	 // compute gradient
	 void grad(std::vector<Vector3f> &gradient);
	 
	 // compute (diagonal) preconditioner: diagonal entries of the Hessian matrix
	 void precon(std::vector<Vector3f> &conditioner);
	 
	 // compute the second order derivative in the direction ddeformations
	 double d2a();

	/// create an evaluator that is compatible with this solver (will work on the same context)
	virtual DeformationEvaluator *createEvaluator();
	
	/// call this method to begin specifying constraints
	virtual void beginConstraints();
	  
	MMDeformationContext* getContext() { return context; }

	virtual void setIdentityMap() { 
		context->setIdentity(); 
	}
};

}

#endif