#ifndef MMDeformationsH
#define MMDeformationsH

#include "GlobalDefinitions.h"
#include <vector>
#include <assert.h>

namespace geoxl {

class DEFORMATION_API MMDeformations : public Persistent {
   GEOXL_CLASS(MMDeformations)
   
 private:
	std::vector<Vector3f> deformations;
	std::vector<Vector3f> real_deformations;
	// keep for each deformation whether it is active
	std::vector<bool> active;
	std::vector<card32> realIndex;
	bool frameActive;

 public:

	//MMDeformations(MMDeformations &copy) {
	//	for (card32 i=0; i<copy.nbDeformations(); i++) {
	//		deformations.push_back(copy.getDeformation(i));
	//		real_deformations.push_back(copy.getRealDeformation(i));
	//		active.push_back(copy.isActive(i));
	//		realIndex.push_back(copy.getRealIndex(i));
	//	}
	//}

   MMDeformations() {
	   frameActive = true;
   }
   
   ~MMDeformations() {
		deformations.clear();
		real_deformations.clear();
		realIndex.clear();
   }
   
	MMDeformations(card32 nbnodes) {
		frameActive = true;
		Vector3f zero; zero.setZero();
		deformations.resize(nbnodes, zero);
		real_deformations.resize(nbnodes, zero);
		// set all nodes to active
		active.resize(nbnodes, true);
		// set the indices to -1, i.e., we don't know which index
		// these unknowns have yet.
		//realIndex.resize(nbnodes, -1);
		for (card32 i=0; i<nbnodes; i++) {
			realIndex.push_back(i);
		}
	}

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

	MMDeformations * copy() {
		return new MMDeformations(*this);
	}

	inline mpcard nbDeformations() const {
		return deformations.size();
	}

	inline Vector3f& getDeformation(card32 index) {
		return deformations[index];
	}

	inline Vector3f& getRealDeformation(card32 index) {
		return real_deformations[index];
	}

	inline void setActive(card32 index, bool isActive) {
		active[index] = isActive;
	}

	inline void setActive(bool isActive) {
		frameActive = isActive;
	}

	inline bool isActive(card32 index) const {
		return active[index];
	}

	inline bool isActive() const {
		return frameActive;
	}

	inline void setRealIndex(card32 index, card32 realIndex) {
		this->realIndex[index] = realIndex;
	}

	card32 getRealIndex(card32 index) const {
		assert(index<realIndex.size());
		
		return realIndex[index];
	}
};
}

#endif