//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "MMNeighborContainer.h"
#include "MMNodes.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( MMNeighborContainer, 1 ) {
	BEGIN_CLASS_INIT( MMNeighborContainer );
	ADD_VECTOR3F_PROP( pos, 0 )
    ADD_CARD32_PROP(baseIndex, 1)
	ADD_BOOLEAN_PROP(shapeValuesInitialized, 1)
	ADD_BOOLEAN_PROP(gradientShapeValuesInitialized, 1)
//	ADD_VAR_ARRAY_PROP( neighbors, 0, new ObjectClassProperty("", 0, NULL, NeighborInformation::getClass(), true, NULL) )
}


void MMNeighborContainer::write(OutputObjectStream *s) const 
{
   mpcard n = neighbors.size();
   s->writeMaxPlatformCard(n);
   for (mpcard i=0; i<n; i++) {
      neighbors[i].write(s);
   }
   Persistent::write(s);
}

void MMNeighborContainer::read(InputObjectStream *s) 
{
   mpcard n;
   s->readMaxPlatformCard(n);
   neighbors.resize(n);
   for (mpcard i=0; i<n; i++) {
      neighbors[i].read(s);
   }
   Persistent::read(s);
}

void MMNeighborContainer::clearShapeValues(const Vector3f &pos, MMNodes * nodes) {
	this->pos = pos;
	shapeValuesInitialized = false;
	gradientShapeValuesInitialized = false;

	for (card32 i=0; i<neighbors.size(); i++) {
		NeighborInformation &n = neighbors[i];
		const card32 neighbor_index = n.index;
		MMNode *neighbor = nodes->getNode(neighbor_index);
		const Vector3f &neighbor_pos = neighbor->getPos();
		const float32 neighbor_radius = neighbor->getRadius();
		const float32 neighbor_sqr_dist = (pos-neighbor->getPos()).getSqrNorm();
		const float32 neighbor_dist = sqrt(neighbor_sqr_dist);				
		float32 weight = 1.0f - neighbor_sqr_dist/(neighbor_radius*neighbor_radius); weight = weight*weight*weight;
		Vector3f gradientWeight = (pos-neighbor_pos)*3*(1-neighbor_sqr_dist/(neighbor_radius*neighbor_radius))*(1-neighbor_sqr_dist/(neighbor_radius*neighbor_radius))*(-2.0/(neighbor_radius*neighbor_radius));
		n.distance = neighbor_dist;
		n.weight = weight;
		n.gradientWeight = gradientWeight;		
	}
}
