#ifndef MMPositionConstraintH
#define MMPositionConstraintH

#include "GlobalDefinitions.h"
#include "ARConstraints.h"
#include "MMNeighborContainer.h"
#include "MMConstraint.h"

namespace geoxl {

/**
  * Holds for a constraint anchor point position, the indices of the nodal neighbors,
  * together with the shape functions evaluated at the anchor point.
  * We cache this information for efficient solving.
  */
	class DEFORMATION_API MMPositionConstraint : public MMConstraint {
	private: 
		const PositionARConstraint * constraint;
		Vector3f deformation;

	public:


		MMPositionConstraint(const PositionARConstraint *c, MMNeighborContainer *n, bool owner) : MMConstraint(n, owner), constraint(c) {
			deformation = c->getToPoint()-c->getAnchor().getPos() ;		
		}

		MMPositionConstraint(MMPositionConstraint &copy) : MMConstraint(copy) {
			constraint = (PositionARConstraint*)copy.constraint->copy();
			//constraint = copy.constraint;
			deformation = copy.deformation;			
		}

		~MMPositionConstraint() {
			delete constraint; constraint = 0;			
		}

		const PositionARConstraint * getConstraint() { return constraint; }
	
		
		float32 getCovarianceElement(unsigned row, unsigned col){
			return constraint->covariance[row][col];
		}
		
		inline const Matrix3f &getCovariance() const {return constraint->covariance;}	
		

		inline Vector3f getDeformation() const {
			return deformation;
			//const Vector3f &posAt = constraint->getAnchor().getPos();
			//const Vector3f &posTo = constraint->getToPoint();
			//return posTo-posAt;
		}

		//MMConstraint * copy() const {
		//	MMConstraint * result = new MMConstraint((PositionARConstraint*)constraint->copy());
		//	return result;
		//}
	
};


}


#endif