#include "StdAfx.h"
#include "MMTopology.h"

#include "PointSetKNNQuery.h"
#include "AttachedIndexedOctree.h"
#include "InCorePCTopologyGraph.h"
#include "PointSet.h"
#include "TopologyRangeSearch.h"
#include <assert.h>

using namespace geoxl;


IMPLEMENT_GEOXL_CLASS( MMTopology , 0 ) {
	BEGIN_CLASS_INIT( MMTopology );
	ADD_OBJECT_PROP(ps, 0, PointSet::getClass(), true);
	ADD_OBJECT_PROP(octree, 0, AttachedIndexedOctree::getClass(), true);
//	ADD_OBJECT_PROP(query, 0, PointSetKNNQuery::getClass(), true);
	ADD_OBJECT_PROP(topGraph, 0, InCorePCTopologyGraph::getClass(), true);
//	ADD_OBJECT_PROP(posAAT, 0, AAT::getClass(), true);
}

MMTopology::MMTopology() {
	ps = 0;
	octree = 0;
	query = 0;
	topGraph = 0;
}

void MMTopology::write(OutputObjectStream *s) const {
   Persistent::write(s);
}

void MMTopology::read(InputObjectStream *s) {
   Persistent::read(s);
   // ugly hack, but needs to be compatible
   delete octree;
   octree = NULL;   
}

MMTopology::MMTopology(PointSet *ps, InCorePCTopologyGraph *topGraph) {
	this->ps = (PointSet*)ps->copy();
	this->topGraph = (InCorePCTopologyGraph*)topGraph->copy();

	octree = new AttachedIndexedOctree();
	octree->setup("SimpleOctree", AttachedData::ADF_DELETEONUPDATE,  NULL);
	octree->octreesetup(this->ps, 20);

	posAAT = this->ps->getAAT("position");
	// TODO: we should keep this, for the correspondences.
	query = new PointSetKNNQuery(this->ps, octree, Vector3f(), posAAT);
}

MMTopology::MMTopology(MMTopology &other) {
	posAAT = other.posAAT;
	if (other.octree != NULL) {
      octree = new AttachedIndexedOctree();
   		octree->assign(other.octree);
	} else {
		octree = NULL;
   }
   if (other.ps != NULL) {
      ps = (PointSet*)(other.ps->copy());
	   query = new PointSetKNNQuery(ps, octree, Vector3f(), posAAT);
	} else {
	   ps = NULL;
	   query = NULL;
	}
   if (other.topGraph!=NULL) {
	   topGraph = (InCorePCTopologyGraph*)other.topGraph->copy();
   }
   else {
	   topGraph = 0;
   }

}

MMTopology::~MMTopology() {
	delete ps; ps = 0;
	delete query; query = 0;
	delete octree; octree = 0;
	delete topGraph; topGraph = 0;
	// do i have to delete the topGraph?
}

void MMTopology::getReachableIndices(const card32 index, const Vector3f &pos, const float32 radius, std::set<card32> &result) {
	TopNeighborhoodSphereSearch search;
	search.setup(topGraph,ps,index,pos/*ps->get3f(index, posAAT)*/,radius,posAAT);
	while (!search.atEnd()) {
		const card32 index = search.getCurrentIndex();
		result.insert(index);
		search.next();
	}
}

void MMTopology::getReachableIndices(const Vector3f &pos, const float32 radius, std::set<card32> &result) {
   pAssert(ps);
   if (!octree) {
	   octree = new AttachedIndexedOctree();
	   octree->setup("SimpleOctree", AttachedData::ADF_DELETEONUPDATE,  NULL);
	   octree->octreesetup(ps, 20);
   }
	if (!query) {
		posAAT = posAAT = ps->getAAT("position");
		query = new PointSetKNNQuery(ps, octree, Vector3f(), posAAT);
	}
	// get nearest point index
	query->setTargetPoint(pos);
	card32 index;
	bool found;
	query->getNextNearestPointIndex(index, found);
	assert(found);
	getReachableIndices(index, pos, radius, result);
}

bool MMTopology::isNeighbor(const card32 geometryBaseIndex, const card32 nodeBaseIndex, const Vector3f &nodePos, const float32 nodeRadius) {
	TopNeighborhoodSphereSearch search;
	search.setup(topGraph,ps,nodeBaseIndex,nodePos/*ps->get3f(index, posAAT)*/,nodeRadius,posAAT);
	while (!search.atEnd()) {
		const card32 index = search.getCurrentIndex();
		if (index==geometryBaseIndex) return true;
		search.next();
	}
	return false;
}

card32 MMTopology::getClosestBaseIndex(const Vector3f &pos) {
   pAssert(ps);
   if (!octree) {
	   octree = new AttachedIndexedOctree();
	   octree->setup("SimpleOctree", AttachedData::ADF_DELETEONUPDATE,  NULL);
	   octree->octreesetup(ps, 20);
   }
	if (!query) {
	   pAssert(ps);
	   pAssert(octree);
		posAAT = posAAT = ps->getAAT("position");
		query = new PointSetKNNQuery(ps, octree, Vector3f(), posAAT);
	}
	query->setTargetPoint(pos);
	card32 index;
	bool found;
	query->getNextNearestPointIndex(index, found);
	return index;
}