#ifndef NodeInformationH
#define NodeInformationH

#include "GlobalDefinitions.h"
#include <vector>
#include "NeighborInformation.h"


namespace geoxl {


/**
  * Holds for a node, the indices of its neighbors,
  * together with the shape functions evaluated at this node
  * together with the coefficients for the gradient of the displacement
  * field evaluated at this node. We cache this information for efficient
  * solving.
  */
class DEFORMATION_API NodeInformation {
	//GEOXL_CLASS( NodeInformation )

private:

	Vector3f pos;
	float32 radius;
	std::vector<NeighborInformation> neighbors;
	
public:

	NodeInformation(const Vector3f &p, const float32 &r) : pos(p), radius(r) {}

	Vector3f &getPos() {
		return pos;
	}

	float32& getRadius() {
		return radius;
	}

	mpcard nbNeighbors() const {
		return neighbors.size();
	}

	void addNeighbor(card32 index, float32 distance, float32 weight, const Vector3f &gradientWeight) {
		neighbors.push_back(NeighborInformation(index, distance, weight, gradientWeight));
	}

	void setShapeValue(card32 index, float32 value) {
		neighbors[index].shapeValue = value;
	}

	void setGradientShapeValue(card32 index, const Vector3f &value) {
		neighbors[index].gradientShapeValue = value;
	}
	
	card32 getNeighborIndex(card32 index) const {
		return neighbors[index].index;
	}

	float32 getWeight(card32 index) const {
		return neighbors[index].weight;
	}

	float32 getDistance(card32 index) const {
		return neighbors[index].distance;
	}

	float32 getShapeValue(card32 index) const {
		return neighbors[index].shapeValue;
	}

	const Vector3f& getGradientShapeValues(card32 index) const {
		return neighbors[index].gradientShapeValue;
	}	

};

}

#endif