//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "RigidDeformationContext.h"
//---------------------------------------------------------------------------
#include "SGERigidDeformationSequence.h"
#include "SGListNode.h"
#include "SGTransformationNode.h"
#include "SGEShape.h"
#include "SGRelativeTimeAnimationNode.h"
#include "RigidDeformationSolver.h"
#include "RigidDeformationEvaluator.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( RigidDeformationContext , 0 ) {
	BEGIN_CLASS_INIT( RigidDeformationContext );
}

// ----

RigidDeformationContext::RigidDeformationContext() {
}


RigidDeformationContext::~RigidDeformationContext() {
}




void RigidDeformationContext::initializeFromShape(const SceneObject *obj, Vector2f timeInterval, const ARSettings *params) 
{
	forget(obj);
	forget(params);
	int numFrames = (int)(timeInterval[1] - timeInterval[0]) + 1;
	if (numFrames <= 0) throw ERangeCheck("RigidDeformationContext::initializeFromShape() - time interval invalid.");
	transformations.resize(numFrames, IDENTITY4F);
	inverseTransformations.resize(numFrames, IDENTITY4F);
}

float32 RigidDeformationContext::getDuration() 
{
   size_t n = transformations.size();
   pAssert(n == inverseTransformations.size());
   return (n==0) ? 0 : n-1;
}

void RigidDeformationContext::externalSetup(card32 numTransformations) 
{
   transformations.resize(numTransformations);
   inverseTransformations.resize(numTransformations);
}

void RigidDeformationContext::setTransformation(card32 i, Matrix4f t) 
{
   transformations[i] = t;
   inverseTransformations[i] = invertMatrix(t);
}

DeformationSolver *RigidDeformationContext::createSolver() 
{
   return new RigidDeformationSolver();
}

DeformationEvaluator *RigidDeformationContext::createEvaluator() 
{
   return new RigidDeformationEvaluator();
}
