//---------------------------------------------------------------------------
#ifndef RigidDeformationContextH
#define RigidDeformationContextH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "ARDeformationOptimizationContext.h"
//---------------------------------------------------------------------------

namespace geoxl {

class RigidDeformationSolver;
class RigidDeformationEvaluator;


///
///   context for RigidDeformationSolver
///   stores list of transformations
///
class DEFORMATION_API RigidDeformationContext : public ARDeformationOptimizationContext {
	GEOXL_CLASS( RigidDeformationContext )
 private:
	vector<Matrix4f> transformations;
	vector<Matrix4f> inverseTransformations;
	
 public:
	RigidDeformationContext();
	
	void externalSetup(card32 numTransformations);
	void setTransformation(card32 i, Matrix4f t);

	virtual void initializeFromShape(const SceneObject *obj, Vector2f timeInterval, const ARSettings *params);
	virtual float32 getDuration();
	
	inline card32 getNumTransformations() {return (card32)transformations.size();}
	inline const Matrix4f &getTransformation(card32 i) {return transformations[i];}
	inline const Matrix4f &getInverseTransformation(card32 i) {return inverseTransformations[i];}

	virtual DeformationSolver *createSolver();
	virtual DeformationEvaluator *createEvaluator();
	virtual float32 getBaseResolution() {return 0;}

   ~RigidDeformationContext();
};

}

#endif