//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "RigidDeformationEvaluator.h"
//---------------------------------------------------------------------------
#include "RigidDeformationContext.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( RigidDeformationEvaluator , 0 ) {
	BEGIN_CLASS_INIT( RigidDeformationEvaluator );
}

// ----

RigidDeformationEvaluator::RigidDeformationEvaluator() {
	context = NULL;
}

RigidDeformationEvaluator::~RigidDeformationEvaluator() {}




AROptimizationContext *RigidDeformationEvaluator::createOptContext() 
{
	return new RigidDeformationContext();
}

void RigidDeformationEvaluator::setup(AROptimizationContext *context, const ARSettings *settings) 
{
	RigidDeformationContext *c = dynamic_cast<RigidDeformationContext*>(context);
	if (!c) {
		throw ETypeMismatch("RigidDeformationSolver::setup() - wrong type for context.");
	}
	this->context = c;
}

void RigidDeformationEvaluator::beginQuery(float32 time) {
	this->time = (int32)time;
	if (!context) {
		throw EInvalidState("RigidDeformationEvaluator::beginQuery() - context is not set.");
	}
	if (time<0) {
		throw ERangeCheck("RigidDeformationEvaluator::beginQuery() - time must be >= 0.");
	}
	if (time>=context->getNumTransformations()) {
		throw ERangeCheck("RigidDeformationEvaluator::beginQuery() - time must be < #frames.");
	}
} 

Vector3f RigidDeformationEvaluator::getPosition(const ARConstraintAnchor &at) 
{
	return transformVector3f(context->getTransformation(time), at.getPos());
}

Vector3f RigidDeformationEvaluator::getVelocity(const ARConstraintAnchor &at) 
{
	if (context->getNumTransformations() == 1) return NULL_VECTOR3F;
	int32 time1 = time-1;
	int32 time2 = time;
	if (time1<0) {
		time1++;
		time2++;
		pAssert(time2 < (int32)context->getNumTransformations());
	}
	return transformVector3f(context->getTransformation(time2), at.getPos())
		  - transformVector3f(context->getTransformation(time1), at.getPos());
}

Matrix3f RigidDeformationEvaluator::getSpatialGradient(const ARConstraintAnchor &at) 
{
	Matrix3f result;
	Matrix4f full = context->getTransformation(time);
	result[0][0] = full[0][0];	result[1][0] = full[1][0]; result[2][0] = full[2][0];
	result[0][1] = full[0][1];	result[1][1] = full[1][1];	result[2][1] = full[2][1];
	result[0][2] = full[0][2];	result[1][2] = full[1][2];	result[2][2] = full[2][2];
	return result;	
}

Vector3f RigidDeformationEvaluator::invertDeformation(
   const Vector3f &position, 
   const ARConstraintAnchor &closeUrshapePoint, 
   Matrix3f *linApprox,
   Matrix3f *invLinApprox,
   float32 precision,
   card32 maxIterations) 
{
   forget(precision);
   forget(maxIterations);
   Matrix4f inv = context->getInverseTransformation(time);
   if (linApprox) *linApprox = shrink4To3(context->getTransformation(time));
   if (invLinApprox) *invLinApprox = shrink4To3(inv);
	return transformVector3f(inv, position);
}

void RigidDeformationEvaluator::endQuery() {}
