//---------------------------------------------------------------------------
#ifndef RigidDeformationEvaluatorH
#define RigidDeformationEvaluatorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "DeformationEvaluator.h"
//---------------------------------------------------------------------------

namespace geoxl {

class RigidDeformationContext;


///
///   Evalutor for rigid deformation optimization (just performs matrix/vector multiplication)
///
class DEFORMATION_API RigidDeformationEvaluator : public DeformationEvaluator {
	GEOXL_CLASS( RigidDeformationEvaluator )
 private:
   RigidDeformationContext *context;
   int32 time;
   
 public:
	RigidDeformationEvaluator();

	virtual AROptimizationContext *createOptContext();
	virtual void setup(AROptimizationContext *context, const ARSettings *settings);

   virtual void beginQuery(float32 frame);

	virtual Vector3f getPosition(const ARConstraintAnchor &at);
   virtual Vector3f getVelocity(const ARConstraintAnchor &at);
   virtual Matrix3f getSpatialGradient(const ARConstraintAnchor &at);
   virtual Vector3f invertDeformation(
      const Vector3f &position, 
      const ARConstraintAnchor &closeUrshapePoint, 
      Matrix3f *linApprox = NULL,
      Matrix3f *invLinApprox = NULL,
      float32 precision = 1e-5,
      card32 maxIterations = 10);

   virtual void endQuery();

   ~RigidDeformationEvaluator();
};

}

#endif