//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SurfelGraphInit.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "SGEShape.h"
#include "InCorePCTopologyGraph.h"
#include "ProgressWindow.h"
#include "PointSetKNNQuery.h"
#include "TopologyAttachment.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS( SurfelGraphInit, 2 ) 
{
	BEGIN_CLASS_INIT( SurfelGraphInit )
		/*
	ADD_BOOLEAN_PROP( doPreprocessData, 0 )
	ADD_BOOLEAN_PROP( doCreateSurfelGraph, 1 )
	ADD_BOOLEAN_PROP( doCreate, 0 ); 
	ADD_BOOLEAN_PROP( doResample, 0 )
	ADD_BOOLEAN_PROP( doInitFrames, 2 )
	*/
}

SurfelGraphInit::SurfelGraphInit()
{
	doPreprocessData = true;
	doCreateSurfelGraph = true;
	doCreate = true;
	doResample = true;
	doInitFrames = true;
	settings = NULL;
}

SurfelGraphInit::~SurfelGraphInit()
{
}

void SurfelGraphInit::createSurfelGraph( SGESurfelGraph* surfels )
{
	if ( surfels == NULL ) {
		debugOutput << "createSurfelGraph: error.\n"; debugOutput.flush();
		return;
	}
	// Create initial point cloud
	UnstructuredInCorePointCloud* surfelPC = dynamic_cast<UnstructuredInCorePointCloud*>( getPointCloud( surfels->getScene(), surfels->getPath() ) );
	PointSet* ps = NULL;
	if ( surfelPC == NULL ){
		surfelPC = new UnstructuredInCorePointCloud();
		ps = new PointSet();
		VertexDescriptor vd;
		vd.pushAttrib( mVAD( "position", 3, VAD::DATA_FORMAT_FLOAT32 ) );
		vd.pushAttrib( mVAD( "normal", 3, VAD::DATA_FORMAT_FLOAT32 ) );
		vd.pushAttrib( mVAD( "flags", 1, VAD::DATA_FORMAT_INT32 ) );
		vd.pushAttrib( mVAD( "tangent_u", 3, VAD::DATA_FORMAT_FLOAT32 ) );
		vd.pushAttrib( mVAD( "tangent_v", 3, VAD::DATA_FORMAT_FLOAT32 ) );
		vd.pushAttrib( mVAD( "coeffs", SURFEL_NUM_BASIS_FUNCTIONS, VAD::DATA_FORMAT_FLOAT32 ) );
		ps->clearAndSetup( 1,1, &vd );
		surfelPC->setPointSet( ps );
		addPointCloud( surfels->getScene(), surfelPC, surfels->getPath() );
	} else {
		ps = surfelPC->getPointSet();
	}
	// Topology
	AttachedDataContainer *attachments = surfelPC->getAttachments();
	TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(attachments->getData("TopologyAtt"));
	if ( topAtt == NULL ){
		topAtt = new TopologyAttachment();
		topAtt->setup( "TopologyAtt", AttachedData::ADF_PERSISTENT );
		attachments->attachData( topAtt );
		InCorePCTopologyGraph* topology = new InCorePCTopologyGraph();
		VertexDescriptor topotVD;
		topotVD.pushAttrib( VAD( "vindex", 2, VAD::DATA_FORMAT_INT32 ) );
		topology->clearAndSetup( &topotVD );
		topAtt->setTopology( topology );
	}
	// Octree
	AttachedIndexedOctree* octree = dynamic_cast<AttachedIndexedOctree*>( attachments->getData( "SimpleOctree" ) );
	if ( octree == NULL ){
		octree = PointSetKNNQuery::createOctree( ps, 32 );
		octree->setup( "SimpleOctree", AttachedData::ADF_DELETEONUPDATE );
		attachments->attachData( octree );
	}
}

bool SurfelGraphInit::initialize( SGESurfelGraph* surfels, SGEPointCloud* data )
{
	// Process data
	if ( data == NULL ){
		debugOutput << "initialize: error.\n"; debugOutput.flush();
		return false;
	}
	UnstructuredInCorePointCloud* dataPC = dynamic_cast<UnstructuredInCorePointCloud*>( data->getPointCloud() );
	if ( dataPC == NULL ){
		debugOutput << "initialize: error.\n"; debugOutput.flush();
		return false;
	}
	PointSet* dataPS = dataPC->getPointSet();
	if ( doPreprocessData ) if ( !applyTensorVoting( data ) ) {
		debugOutput << "initialize: error.\n"; debugOutput.flush();
		return false;
	}

	// Process surfels
	if ( surfels == NULL ){
		debugOutput << "initialize: error.\n"; debugOutput.flush();
		return false;
	}
	if ( doCreateSurfelGraph ) createSurfelGraph( surfels );
	string msg;
	if ( !surfels->check( msg ) ){
		debugOutput << "Invalid surfel surface SGE: " << msg << "\n"; debugOutput.flush();
		return false;
	}
	if ( doCreate ) if ( !createSurfels( surfels, data ) ) {
		debugOutput << "initialize: error.\n"; debugOutput.flush();
		return false;
	}
	if ( doResample ) if ( !resample( surfels ) ) {
		debugOutput << "initialize: error.\n"; debugOutput.flush();
		return false;
	}
	if ( doInitFrames ) if ( !initFrames( surfels ) ) {
		debugOutput << "intialize: error.\n"; debugOutput.flush();
		return false;
	}
	return true;
}

bool SurfelGraphInit::applyTensorVoting( SGEPointCloud* data )
{
	return false;
	/// NOT USED ANY MORE
	/*
	if ( settings == NULL ){
		debugOutput << "applyTensorVoting: error.\n"; debugOutput.flush();
		return false;
	}
	if ( data == NULL ){
		debugOutput << "applyTensorVoting: error.\n"; debugOutput.flush();
		return false;
	}

	checkAttribute( data->getScene(), data->getPath(), "position", 3, VAD::DATA_FORMAT_FLOAT32 );

	string msg;
	if ( !data->check( msg ) ){
		debugOutput << "applyTensorVoting: invalid data: " << msg << ".\n"; debugOutput.flush();
		return false;
	}
	UnstructuredInCorePointCloud* dataPC = dynamic_cast<UnstructuredInCorePointCloud*>( data->getPointCloud() );
	PointSet* dataPS = dataPC->getPointSet();
	TensorVoting::tensorVote( true, dataPS, settings->recSigma );
	return true;
	*/
}

bool SurfelGraphInit::createSurfels( SGESurfelGraph* surfels, SGEPointCloud* data )
{
	if ( data == NULL ) {
		debugOutput << "createSurfels: error.\n"; debugOutput.flush();
		return false;
	}
	string msg;
	if ( !data->check( msg ) ) {
		debugOutput << "createSurfels: error.\n"; debugOutput.flush();
		return false;
	}
	UnstructuredInCorePointCloud* dataPC = dynamic_cast<UnstructuredInCorePointCloud*>( data->getPointCloud() );
	PointSet* dataPS = dataPC->getPointSet();
	AAT DATA_POSITION = dataPS->getAAT( "position" );

	// Count number of surface points
	card32 numSurfacePoints = dataPS->getNumEntries();

	if ( surfels == NULL ){
		debugOutput << "createSurfels: error.\n"; debugOutput.flush();
		return false;
	}
	if ( !surfels->check( msg ) ){
		debugOutput << "createSurfels: invalid surfels: " << msg << ".\n"; debugOutput.flush();
		return false;
	}
	PointSet* surfelPS = surfels->getPointSet();
	surfelPS->changeHeight( numSurfacePoints );
	InCorePCTopologyGraph* topology = surfels->getTopology();
	topology->clearAndSetup( topology->getEdges()->getDescr() );
	AAT SURFEL_POSITION = surfelPS->getAAT( "position" );
	debugOutput << "Created " << surfelPS->getNumEntries() << " surfels from " << dataPS->getNumEntries() << " data points.\n"; debugOutput.flush();

	// Create initial topology graph
	progressWindow->pushStep( true, "Topology" );
	AttachedIndexedOctree* surfelOctree = PointSetKNNQuery::createOctree( surfelPS, 32 );
	PointSetKNNQuery knn( surfelPS, surfelOctree, makeVector3f( 0,0,0 ), SURFEL_POSITION );
	for ( unsigned int pIndex = 0; pIndex < surfelPS->getNumEntries(); pIndex++ ){
		const Vector3f& p = surfelPS->get3f( pIndex, SURFEL_POSITION );
		knn.setTargetPoint( p );
		bool found = true;
		card32 nIndex = pIndex;
		card32 numNeighbors = 0;
		while( ( numNeighbors < 8 ) && found ){
			if ( nIndex != pIndex ) {
				topology->addEdge( pIndex, nIndex );
				numNeighbors++;
			}
			knn.getNextNearestPointIndex( nIndex, found );
		}
		progressWindow->progress( card32( float32( pIndex ) / float32( surfelPS->getNumEntries() ) * 100 ) );  
	}
	progressWindow->popSteps();
	delete surfelOctree;
	return true;
}

bool SurfelGraphInit::resample( SGESurfelGraph* surfels )
{
	if ( surfels == NULL ){
		debugOutput << "resample: error.\n"; debugOutput.flush();
		return false;
	}
	string msg;
	if ( !surfels->check( msg ) ){
		debugOutput << "resample: invalid surfels: " << msg << ".\n"; debugOutput.flush();
		return false;
 	}
	PointSet* surfelPS = surfels->getPointSet();
	InCorePCTopologyGraph* topology = surfels->getTopology();
	AttachedIndexedOctree* surfelOctree = PointSetKNNQuery::createOctree( surfelPS, 32 );
	AAT POSITION = surfelPS->getAAT( "position" );
	AAT FLAGS = surfelPS->getAAT( "flags" );
	AAT COEFFS = surfelPS->getAAT( "coeffs" );
	PointSetKNNQuery knn( surfelPS,  surfelOctree, makeVector3f( 0,0,0 ), POSITION );

	// Init
	card32 oldNumSurfels = surfelPS->getNumEntries();
	for ( unsigned int pIndex = 0; pIndex < surfelPS->getNumEntries(); pIndex++ ){
		surfelPS->set1i( pIndex, FLAGS, 0 );
	}

	// Mark prune points
	progressWindow->pushStep( true, "Resampling" );
	float32 sqrSurfelSpacing = sqr( settings->topResamplingSurfelSpacing );
	card32 numSurfels = 0;
	vector<int32> mapOldToNew; mapOldToNew.resize( surfelPS->getNumEntries() );
	for ( unsigned int pIndex = 0; pIndex < surfelPS->getNumEntries(); pIndex++ ){
		if ( surfelPS->get1i( pIndex, FLAGS ) == 0 ){
			const Vector3f& p = surfelPS->get3f( pIndex, POSITION );
			mapOldToNew[pIndex] = numSurfels;
			knn.setTargetPoint( p );
			float32 sqrD = 0;
			card32 nIndex = pIndex;
			bool found = true;
			Vector3f np = p;
			while( ( sqrD < sqrSurfelSpacing ) && found ){
				// Mark neighbors as invalid
				if ( ( sqrD < sqrSurfelSpacing ) && ( nIndex > pIndex ) ){
					surfelPS->set1i( nIndex, FLAGS, 1 );			
				}
				// Inc
				knn.getNextNearestPointIndex( nIndex, found );
				np = surfelPS->get3f( nIndex, POSITION );
				sqrD = ( np - p ) * ( np - p );
			}
			numSurfels++;									// Surfel is kept
		} else {
			mapOldToNew[pIndex] = -1;
		}
		progressWindow->progress( card32( float32( pIndex ) * 100 / float32( surfelPS->getNumEntries() ) ) );
	}
	progressWindow->popSteps();

	// Prune marked surfels
	StaticVector<float32,SURFEL_NUM_BASIS_FUNCTIONS> coeffs;
	for ( unsigned int n = 0; n < SURFEL_NUM_BASIS_FUNCTIONS; n++ ) coeffs[n] = 0;
	vector<card32> mapNewToOld; mapNewToOld.resize( numSurfels );
	card32 newSurfelIndex = 0;
	for ( unsigned int pIndex = 0; pIndex < surfelPS->getNumEntries(); pIndex++ ){
		if ( surfelPS->get1i( pIndex, FLAGS ) == 0 ){
			mapNewToOld[newSurfelIndex] = pIndex;
			surfelPS->set3f( newSurfelIndex, POSITION, surfelPS->get3f( pIndex, POSITION ) );
			surfelPS->set<float32,SURFEL_NUM_BASIS_FUNCTIONS>( newSurfelIndex, COEFFS, coeffs ); 
			newSurfelIndex++;	
		}
	}
	surfelPS->changeHeight( numSurfels );
	surfelOctree->octreesetup( surfelPS, 32 );

	// Update topology: TODO: naive approach: old topology discarded!
	topology->clearAndSetup( topology->getEdges()->getDescr() );
	knn = PointSetKNNQuery( surfelPS,  surfelOctree, makeVector3f( 0,0,0 ), POSITION );
	float32 avgValence = 0;
	for ( unsigned int pIndex = 0; pIndex < surfelPS->getNumEntries(); pIndex++ ){
		const Vector3f& p = surfelPS->get3f( pIndex, POSITION );
		knn.setTargetPoint( p );
		card32 nIndex = pIndex; 
		Vector3f np = p;
		float32 sqrD = 0;
		bool found = true;
		card32 numNeighbors = 0;
		while( ( sqrD < sqr( settings->topRelRadius * settings->topSurfelSpacing ) ) && found ){
			if ( pIndex != nIndex ){
				topology->addEdge( pIndex, nIndex );
				numNeighbors++;
				if (numNeighbors >= settings->topMaxNumNeighbors) break;
			}
			knn.getNextNearestPointIndex( nIndex, found );
			np = surfelPS->get3f( nIndex, POSITION );
			sqrD = ( np - p ) * ( np - p );
		}
		avgValence += numNeighbors;
	}
	avgValence /= float32( surfelPS->getNumEntries() );
	debugOutput << "Average valence in topology graph: " << avgValence << ".\n"; debugOutput.flush();

	// Reset flags
	for ( unsigned int pIndex = 0; pIndex < surfelPS->getNumEntries(); pIndex++ ){
		surfelPS->set1i( pIndex, FLAGS, 0 );
	}

	debugOutput << "Resampled from " << oldNumSurfels << " to " << numSurfels << " surfels.\n"; debugOutput.flush();
	delete surfelOctree;
	return true;
}

bool SurfelGraphInit::initFrames( SGESurfelGraph* surfels )
{
	if ( surfels == NULL ) {
		debugOutput << "initFrames: error.\n"; debugOutput.flush();
		return false;
	}
	string msg;
	if ( !surfels->check( msg ) ){
		debugOutput << "initFrames: invalid surfels: " << msg << ".\n"; debugOutput.flush();
		return false;
	}
	PointSet* surfelPS = surfels->getPointSet();
	InCorePCTopologyGraph* topology = surfels->getTopology();
	AAT POSITION = surfelPS->getAAT( "position" );
	AAT NORMAL = surfelPS->getAAT( "normal" );
	AAT TANGENT_U = surfelPS->getAAT( "tangent_u" );
	AAT TANGENT_V = surfelPS->getAAT( "tangent_v" );
	PCA3f pca;
	Matrix3f eigenVectors;
	Vector3f eigenValues, centroid;
	progressWindow->pushStep( true, "Frame Estimation" );
	for ( unsigned int pIndex = 0; pIndex < surfelPS->getNumEntries(); pIndex++ ){
		const Vector3f& p = surfelPS->get3f( pIndex, POSITION );
		pca.clear();
		pca.addPoint( p );
		for ( unsigned int n = 0; n < topology->getNumAdjacentVertices( pIndex ); n++ ){
			const card32 nIndex = topology->getAdjacentVertex( pIndex, n );
			const Vector3f& np = surfelPS->get3f( nIndex, POSITION );
			pca.addPoint( np );
		}
		pca.analyze( eigenValues, eigenVectors, centroid );
		surfelPS->set3f( pIndex, NORMAL, eigenVectors[2] );
		surfelPS->set3f( pIndex, TANGENT_U, eigenVectors[0] );
		surfelPS->set3f( pIndex, TANGENT_V, eigenVectors[1] );
		progressWindow->progress( card32( float32( pIndex + 1 ) * 100 / float32( surfelPS->getNumEntries() ) ) );
	}
	progressWindow->popSteps();
	
	return true;
}