//---------------------------------------------------------------------------
#ifndef SurfelGraphInit2H
#define SurfelGraphInit2H
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class SGESurfelGraph;
class SGEPointCloud;
class ARSettings;
class InCorePCTopologyGraph;

///
///   (mw 12/31/07) - This is a copy & paste revision of Philipp's SurfelGraphInit
///   (became necessary because of some pipeline changes)
///   
///   changes
///     - no tensor voting (handled externally)
///     - works on generic point clouds (incl. OOC)
///     - computes topology according to settings
///     - errors cause exceptions (easier to use)
///
class DEFORMATION_API SurfelGraphInit2 : public Persistent {
	GEOXL_CLASS( SurfelGraphInit2 )
 private:
	void createSurfelGraph( SGESurfelGraph* surfels, ARSettings* settings );
	void createSurfels( SGESurfelGraph* surfels, SGEPointCloud* data, ARSettings *settings );
	void resample( SGESurfelGraph* surfels, ARSettings *settings );
	void initChannels( SGESurfelGraph* surfels, ARSettings* settings );

 public:
	SurfelGraphInit2();
	virtual void assign(const Object* obj, CopyContext *context = NULL);

	// Apply the complete initialization routine; throws an exception if anything goes wrong
	void initialize( SGESurfelGraph* surfels, SGEPointCloud* data,  ARSettings *settings );

	/// remove points and corresponding topology edges from small connected regions
	static void removeSmallComponents( PointSet * ps, InCorePCTopologyGraph * topology, card32 minConnectedPoints, ARSettings* settings );

   ~SurfelGraphInit2();
};

}

#endif