//---------------------------------------------------------------------------
#ifndef SurfelOptimizerH
#define SurfelOptimizerH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "ShapeReconstruction.h"
#include "ARSettings.h"
#include "SurfelOptimizationContext.h"
#include "ARConstraints.h"
//---------------------------------------------------------------------------

namespace geoxl {

///
///   Optimization functionality for a surfel graph
///
class DEFORMATION_API SurfelOptimizer: public ShapeReconstruction
{
	GEOXL_CLASS( SurfelOptimizer )

	private:
		// Internal helper variables
		float32 f;
		SparseMatrix<float32> H;				// Hessian of linear system
		DynamicVector<float32> variables;		// Variables contain serialized coefficients for each surfel
		DynamicVector<float32> gradient;		// Computed gradient of the problem
		float32 startBuildMatrixTime;			// Time required to build matrix
		DynamicVector<float32> b;				// Right hand side for soling linear system
		AttachedIndexedOctree* surfelOctree;	// Octree over surfel point set

		void addConsistencyConstraints();

		const ARSettings* settings;
		SurfelOptimizationContext* context;

	public:
		SurfelOptimizer();
		~SurfelOptimizer();


		/// this method creates a context object that needs to be setup by the user of the deformation tool
		/// given this object, the tool itself can be initialized by calling "setup()"
		AROptimizationContext *createOptContext();

		/// specifies the domain and the deformation context the tool is working on
		/// it also specifies a settings object that must not be deleted until
		/// while the tool is working
		/// the ownership of the context is transferred to the AROptimizationTool
		void setup( AROptimizationContext* context, const ARSettings* settings );

		void beginConstraints();
		void addPositionConstraint( const PositionARConstraint* posConstraint );
		void addHalfSpaceConstraint( const ARHalfSpaceConstraint* hsConstraint );
		void addGeneralConstraint( const GeneralARConstraint* generalConstraint );
   	void addSpatialDerivativeConstraint(const SpatialDerivativeARConstraint *c);
	   void addSimpleSpatialDerivativeConstraint(const SimpleSpatialDerivativeARConstraint *c);
		void solve();
		
		ShapeEvaluator *createEvaluator();
};

}

#endif