//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SGEPointCloud.h"
//---------------------------------------------------------------------------
#include "DeformationTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( SGEPointCloud , 0 ) {
	BEGIN_CLASS_INIT( SGEPointCloud );
}

// ----

SGEPointCloud::SGEPointCloud() {
   pointChannels = new VertexDescriptor();
}

SGEPointCloud::~SGEPointCloud() {
   delete pointChannels;
}




const VertexDescriptor *SGEPointCloud::getPointChannels() const
{
   return pointChannels;
}

void SGEPointCloud::setPointChannels(VertexDescriptor *cn)
{
   if (pointChannels != cn) {
      delete pointChannels;
   }
   pointChannels = cn;
}

bool SGEPointCloud::check(string &msg) const
{
   if (!SGEShape::check(msg)) {
      return false;
   }
   SceneObject *so = getSceneObject();
   PointCloud *upc = dynamic_cast<PointCloud*>(so);
   if (!upc) {
      msg = "scene object is not a PointCloud (or descendant)";
      return false;
   }	
   if (!compareDescr(upc->getDescr(), pointChannels)) {
      msg = "attribute channels of the point cloud do not match the requirements.";
      return false;
   } 
   return true;
}

PointCloud *SGEPointCloud::getPointCloud() const
{
   SceneObject *so = getSceneObject();
   PointCloud *pc = dynamic_cast<PointCloud*>(so);
   if (!pc) {
      throw EInvalidState("SGEPointCloud::getPointCloud() - SceneObject is not a point cloud (or NULL).");
   }	
   return pc;
}
