//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SGEShape.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PointCloudTools.h"
#include "SGObjectNode.h"
#include "SceneObject.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( SGEShape , 0 ) {
	BEGIN_CLASS_INIT( SGEShape );
}

// ----

SGEShape::SGEShape() {
}

SGEShape::~SGEShape() {
}




SceneObject *SGEShape::getSceneObject() const
{
	Matrix4f forget;
	SceneGraphNode *node;
   getSGNodeTransformation(getScene(), getPath(), forget, node);
	SGObjectNode *on = dynamic_cast<SGObjectNode*>(node);
	if (!on) {
		throw EInvalidState("SGEShape::getSceneObject() - path is not pointing to a single object node (call check to validate scene graph content)");
	}
	SceneObject *so = on->getSceneObject();
	if (so == NULL) {
		throw EInvalidState("SGEShape::getSceneObject() - path is pointing to an object node that does not contain an actual object (getSceneObject() == NULL, call check to validate scene graph content)");
	}
   return so;
}

SceneObjectIterator *SGEShape::createIterator(SceneObjectIteratorType type) const
{
	SceneObject *so = getSceneObject();
	return so->createIterator(type);
}

bool SGEShape::check(string &msg) const
{
	if (!SceneGraphEntity::check(msg)) return false;
	Matrix4f forget;
	SceneGraphNode *node;
   getSGNodeTransformation(getScene(), getPath(), forget, node);
	SGObjectNode *on = dynamic_cast<SGObjectNode*>(node);
	if (!on) {
		msg = "path is not pointing to a single object node (check scene graph structure)";
		return false;
	}
	SceneObject *so = on->getSceneObject();
	if (so == NULL) {
		msg = "path is pointing to an object node that does not contain an actual object (getSceneObject() == NULL)";
		return false;
	}
   return true;
}

Matrix4f SGEShape::getBaseTransform() const
{
	return SceneGraphEntity::getBaseTransform() * getSceneObject()->getTransformation();
}
