//---------------------------------------------------------------------------
#ifndef SGESurfelGraphH
#define SGESurfelGraphH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "SGEShape.h"
//---------------------------------------------------------------------------

namespace geoxl {

class InCorePCTopologyGraph;

#define SURFEL_NUM_BASIS_FUNCTIONS 3

///   mw - 09/27/2007
///
///   encapsulates scene graph representation of a surfel graph, 
///   i.e. an unstructured point cloud with attached topology
///
///   additional assumptions:
///     - scene object is of type UnstructuredInCorePointCloud or decendant
///     - an InCorePCTopologyGraph is attached to the scene object in a TopologyAttachment
///     - all the channels specified in properties [surfel|graph]channels are present
class DEFORMATION_API SGESurfelGraph : public SGEShape {
	GEOXL_CLASS( SGESurfelGraph )
 private:
	/// attribute channels the surfels are expected to have
	VertexDescriptor *surfelChannels;
	/// attribute channels the topology graph is expected to have
	VertexDescriptor *graphChannels;
	
 public:
	SGESurfelGraph();

	/// property access: channels, ownership transferred
	void setSurfelChannels(VertexDescriptor *cn);
	/// property access: channels
	const VertexDescriptor *getSurfelChannels() const;
   /// property access: channels, ownership transferred
   void setGraphChannels(VertexDescriptor *cn);
   /// property access: channels
   const VertexDescriptor *getGraphChannels() const;
		
	virtual bool check(string &msg) const;

	/// get point cloud object
	UnstructuredInCorePointCloud *getPointCloud() const;
	/// directly get the point set of the unstructured point cloud
	PointSet *getPointSet() const;
	/// directly get the topology graph
	InCorePCTopologyGraph *getTopology() const;
	
   ~SGESurfelGraph();
};

}

#endif