//---------------------------------------------------------------------------
#ifndef SceneGraphEntityH
#define SceneGraphEntityH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

///   mw - 09/27/2007
///
///   this class encapsulates a piece of information stored in the scene graph
///   (e.g. a deformation sequence, an urshape, a data point sequence...)
///
///   specific assumptions:
///     - valid scene graph path
class DEFORMATION_API SceneGraphEntity : public Persistent {
	GEOXL_CLASS( SceneGraphEntity )
 private:
	Scene *scene;
	string path;
	
 public:
	SceneGraphEntity();
	virtual void assign(const Object* obj, CopyContext *context = NULL);
	
	/// setup new object
	virtual void setup(Scene *s, string p);

	/// property access: scene
	Scene *getScene() const {return scene;}
	/// property access: scene
	void setScene(Scene *s) {scene = s;}
	/// property access: path
	string getPath() const {return path;}
	/// property access: path
	void setPath(string p) {path = p;}

	/// checks whether the information stored in the scene graph matches the
	/// expectations of the using module (channels, organization, ...)
	/// msg contains an error description if return value is false (i.e. something is wrong)
	/// override in decendant classes to perform additional checks
	virtual bool check(string &msg) const;

	/// get the referenced scene graph node	
	virtual SceneGraphNode *getSceneGraphNode() const;
	
	/// get transformation matrix of scene graph entity (induced by traversing a path to the root)
	virtual Matrix4f getBaseTransform() const;

   ~SceneGraphEntity();
};

}

#endif