//---------------------------------------------------------------------------
#ifndef ARMergeSequencesH
#define ARMergeSequencesH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "Delegate.h"
#include "AROptToolWithFeedback.h"
#include "Delegate.h"
//---------------------------------------------------------------------------

namespace geoxl {

class ARSequence;
class ARSettings;
class ShapeReconstruction;
class DeformationSolver;
class ARMergeSettings;
class ARDeformationOptimizationContext;
class ARConstraintsInterface;
class DeformationEvaluator;
class InCorePCTopologyGraph;


///
///   this class takes two animations (urshape with deformation sequence)
///   and merges them into a single one, given a (possibly empty) estimate
///   of sparse correspondences
///
class DEFORMATION_API ARMergeSequences : public AROptToolWithFeedback {
 private:
   AROptToolWithFeedback *visTool;
   PointCloud *myVisPC;
   Vector4f myVisColor;
   
   void show(PointCloud *showPC, Vector4f showColor, Delegate repaint, ARSettings *settings);
   
   UnstructuredInCorePointCloud *mergeAlignedShapes(
      PointCloud *shapeA, 
      PointCloud *shapeB, 
      ARSettings *settings);

   enum WhereIsTheUrshapeType {URSHAPE_IN_A, URSHAPE_IN_B};

   //card32 checkTopology4(
   //   PointCloud *urshape,
   //   card32 seqBStart,
   //   bool deform_AP_to_B,
   //   PointCloud *nonUrshapeFrame,
   //   DeformationSolver *deformationSolver,
   //   ARSettings *settings,
   //   vector<bool> &keepVertices
   //);
   card32 checkTopology5(
      PointCloud *urshape,
      card32 seqBStart,
      bool deform_AP_to_B,
      PointCloud *nonUrshapeFrame,
      DeformationSolver *deformationSolver,
      ARSettings *settings,
      vector<bool> &keepVertices,
	   ARSequence *seqA, 
	   ARSequence *seqB
   );
   void checkTopologyLater(
      ARSequence *seq,
      DeformationSolver *deformationSolver,
      ARSettings *settings
   );

   ARDeformationOptimizationContext *recomputeDeformation3( 
      PointCloud *newUrshape, 
      ARSequence *seqA, 
      ARSequence *seqB,
      card32 seqBStart,
      card32 seqBStartTime,
      WhereIsTheUrshapeType whereIsTheUrshape,
      DeformationSolver *solver,
      ARSettings *settings);

   void initPropChannel(PointCloud *shape, int32 frame);
   void setupDefRefitChannel(PointCloud *p, int32 frameIndex);
   
   void removeSmallComponents(PointCloud *pc, ARSettings *settings);
   
 public:
	ARMergeSequences();

   ARSequence *mergeSequences(
			Scene* scene,
		ARSequence *seqA, 
		ARSequence *seqB, 
		card32 seqAStartTime,
		card32 seqBStartTime,
      const vector<PointCloud*> &dataPoints,
	   PointCloud *urshapePropagationInput,
	   PointCloud *&urshapePropagationOutput,
		ARMergeSettings *settings, 
		Delegate visualFeedback = NULL);

	virtual void glDrawFeedback(GLContext *glContext, ARVisualizationSettings *visSettings);	

   ~ARMergeSequences();
};

void ARMergeSequences_DebugOutputTimings();

}

#endif