//---------------------------------------------------------------------------
#ifndef ARSequenceH
#define ARSequenceH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class ARDeformationOptimizationContext;
class DeformationEvaluator;
class PointCloud;
class ShapeReconstruction;
class DeformationSolver;
class ARSettings;


///
///   This class represents an animation of a deformable object
///   The final output of the Animation Reconstruction 2.0 pipeline is of this type
///
class DEFORMATION_API ARSequence : public Persistent {
	GEOXL_CLASS( ARSequence )
 private:
   ARDeformationOptimizationContext *deformation;
   PointCloud *urshape;
   
 public:
	ARSequence();

   // this object only stores references
   // objects are not owned	
	void clearAndSetup(
	   ARDeformationOptimizationContext *deformation,
      PointCloud *urshape);
   void setupFromSceneGraph(
      Scene *scene,
      string urshapePath, string deformationPath, 
      ShapeReconstruction *shapeSolver, DeformationSolver *deformSolver);

	virtual void assign(const Object* obj, CopyContext *context = NULL);

	ARDeformationOptimizationContext *getDeformation() {return deformation;}
	const ARDeformationOptimizationContext *getDeformation() const {return deformation;}
	void setDeformation(ARDeformationOptimizationContext *v) {deformation=v;}
	
	PointCloud* getUrshape() {return urshape;}
	const PointCloud* getUrshape() const {return urshape;}
	void setUrshape(PointCloud* v) {urshape=v;}
	
	float32 getDuration();
	PointCloud *evaluate(float32 time, DeformationEvaluator *evaluator, ARSettings *settings);
	PointCloud *evaluateStart(DeformationEvaluator *evaluator, ARSettings *settings);
	PointCloud *evaluateEnd(DeformationEvaluator *evaluator, ARSettings *settings);
};

}

#endif