//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ARGlobalDeformationOptimization.h"
//---------------------------------------------------------------------------
#include "ARVisualizationSettings.h"
#include "ARSequence.h"
#include "ARSettings.h"
#include "AROptimizationContext.h"
#include "ARShapeOptimizationContext.h"
#include "ARTransfConstraintsToUrshape.h"
#include "ICPConstraints.h"
#include "DeformationEvaluator.h"
#include "AREvaluateDeformation.h"
#include "DeformationSolver.h"
#include "ARDeformationOptimizationContext.h"
#include "ARPruneUnsupportedOutliers.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



ARGlobalDeformationOptimization::ARGlobalDeformationOptimization() {
}

void ARGlobalDeformationOptimization::globalOpt(
   ARSequence *seq, 
   DeformationSolver *deformOpt, 
   const vector<PointCloud*> &dataPoints,
   ARSettings *settings,
   ARPruneUnsupportedOutliers *unsupportedPtsCulling
   )
{
   ARDeformationOptimizationContext *context = seq->getDeformation();

   deformOpt->setup(context, settings);
   DeformationEvaluator *deformEval = deformOpt->createEvaluator();
   deformOpt->beginConstraints();

   pAssert(seq->getDuration()+1 == dataPoints.size());
   const size_t numFrames = dataPoints.size();
   for (card32 i=0; i<numFrames; i++) {
      ICPConstraints icp;
      icp.setConstraintMode(settings->icpConstraintsMode);
      icp.setDistanceOutliersMode(settings->icpNearFieldCulling?ICP_OUTLIERS_DISTANCE:ICP_OUTLIERS_NONE);
      icp.setOutliersDistanceFactor(settings->icpNearFieldRelativeCutOff);
      icp.setCullNormalOutliers(false);
      icp.setOutlierNormals(-1.0f);
      deformEval->setup(seq->getDeformation(), settings);
      
      // TODO: context is used in eval and opt at the same time
      PointCloud *deformedReference = AREvaluateDeformation::evaluatePC(
         seq->getUrshape(), 
         context, 
         deformEval,
         i,
         settings);
      if (unsupportedPtsCulling) {
         ARPruneUnsupportedOutliers::resetCounters(seq->getUrshape());
      }

		/// xxx -here TODOCons


	   icp.buildICPConstraints(
		   settings, deformOpt, seq->getUrshape(),
		   deformedReference,
		   dataPoints[i],
		   i,
		   NULL, false,
		   settings->icpSkipPercentage,
		   unsupportedPtsCulling != NULL
	   );
	   delete deformedReference;
      if (unsupportedPtsCulling) {
         unsupportedPtsCulling->addNextFrame(seq->getUrshape());
         ARPruneUnsupportedOutliers::resetCounters(seq->getUrshape());
      }
   }
   delete deformEval;

   deformOpt->solve();
}

void ARGlobalDeformationOptimization::glDrawFeedback(GLContext *glContext, ARVisualizationSettings *visSettings) 
{
   forget(visSettings);
}
