//---------------------------------------------------------------------------
#ifndef ARGlobalOptimizationH
#define ARGlobalOptimizationH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "AROptToolWithFeedback.h"
#include "Delegate.h"
//---------------------------------------------------------------------------

namespace geoxl {

class ARSequence;
class DeformationSolver;
class ShapeReconstruction;
class Scene;
class ARSettings;

///
///   this class performs iterative global optimization of shape & deformation
///
class DEFORMATION_API ARGlobalOptimization : public AROptToolWithFeedback {
 private:

   AROptToolWithFeedback *visTool;
   PointCloud *myVisPC;
   Vector4f myVisColor;
   vector<PointCloud*> getDataPoints(Scene *scene, string dataPath, card32 startFrame, card32 endFrame);
   void show(PointCloud *showPC, Vector4f showColor, Delegate repaint, ARSettings *settings);

 public:
	ARGlobalOptimization();

   void performGlobalOpt(
	   /// sequence to be optimized
	   ARSequence *seq, 
	   /// deformation optimizer
	   DeformationSolver *deformSolver,
	   /// shape optimizer
	   ShapeReconstruction *shapeSolver, 
      /// input data
      Scene *scene,
      string dataPath,
      /// parameters
	   ARSettings *settings,
      /// range that is optimized
      card32 startFrame,
      card32 endFrame,
      Delegate visualFeedback = NULL,
	   /// optional: turn off urshape deformation optimization
	   bool optimizeUrshapeDeformation = true,
	   /// optional: turn off shape optimization
	   bool optimizeShape = true,
	   /// optional: turn off deformation optimization
	   bool optimizeDeformation = true,
	   /// optimal turn off unsupported points deletion
	   bool removeUnsupportedPoints = true
   );

	virtual void glDrawFeedback(GLContext *glContext, ARVisualizationSettings *visSettings);	

   ~ARGlobalOptimization();
};

}

#endif