//---------------------------------------------------------------------------
#ifndef ARPairwiseMatchingH
#define ARPairwiseMatchingH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "ARConstraintsSource.h"
#include "Delegate.h"
#include "AROptToolWithFeedback.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PointCloud;
class ARMergeSettings;
class DeformationSolver;
class ARDeformationOptimizationContext;

///
///   This class deforms one point cloud (w/topology) to match another one
///   If the rigid solver is used, rigid ICP is performed
///
class DEFORMATION_API ARPairwiseMatching : public AROptToolWithFeedback {
 private:
	PointCloud *visPC1, *visPC2;
	bool visAlternateColor;
	AROptToolWithFeedback *subTool;

 public:
	ARPairwiseMatching();
	
	virtual void glDrawFeedback(GLContext *glContext, ARVisualizationSettings *visSettings);
		
   virtual void matchGeometry(
		Scene* scene,
		card32 frameNo,
      /// point cloud to be deformed
      PointCloud *referenceShape, 
      /// point cloud to be matched (partially)
      PointCloud *matchingShape, 
      /// deformation solver to be used
      DeformationSolver *solver,
      /// parameters for solver
      const ARMergeSettings *settings,
      /// resulting deformed point cloud (positions and normals deformed, other channels copied)
      PointCloud *&deformedGeometry,
      /// resulting deformation (deforms referenceShape into matching shape)
      ARDeformationOptimizationContext *&deformation,
      /// output the number of iterations needed, e.g. for detecting convergence issues
      card32 &numIterationsNeeded,
      /// this delegate is called after each ICP iteration (if not NULL)
      Delegate callForRepaint = NULL,
      bool alternateColor = false);

	virtual ~ARPairwiseMatching();
};

}

#endif