//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ARRecomputeTopology.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "InCorePCTopologyGraph.h"
#include "DeformationTools.h"
#include "PointSetKNNQuery.h"
#include "ARSettings.h"
#include "TopologyAttachment.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



void ARRecomputeTopology::recomputeTopology(UnstructuredInCorePointCloud *pc, const ARSettings *settings)
{
   PointSet *ps = pc->getPointSet();
   const AAT psPosAAT = ps->getDescr()->getToken("position", 3, VAD::DATA_FORMAT_FLOAT32);
   InCorePCTopologyGraph *tp = new InCorePCTopologyGraph();
	VertexDescriptor vd;
	vd.pushAttrib(VAD("vindex", 2, VAD::DATA_FORMAT_INT32));
   tp->clearAndSetup(&vd);

   AttachedIndexedOctree *octree = fetchAndAttachOctree(pc);
   PointSetKNNQuery knn(ps, octree, NULL_VECTOR3F, psPosAAT);

   const card32 numPoints = ps->getNumEntries();
   for (card32 i=0; i<numPoints; i++) {
      Vector3f targetPoint = ps->get<float32, 3>(i, psPosAAT);
      knn.setTargetPoint(targetPoint);

      card32 k=0;
      while (k <= settings->topMaxNumNeighbors) {
         card32 neighborIndex;
         bool found;
         knn.getNextNearestPointIndex(neighborIndex, found);
         if (neighborIndex == i) continue;
         if (!found) break;
         Vector3f nextPos = ps->get<float32, 3>(neighborIndex, psPosAAT);
         if (norm(nextPos-targetPoint) > settings->topSurfelSpacing * settings->topRelRadius) break;
         
         tp->addEdge(i, neighborIndex);
         
         k++;
      }   
   }      
   
	AttachedDataContainer *attachments = pc->getAttachments();
	TopologyAttachment *topAtt = dynamic_cast<TopologyAttachment*>(attachments->getData("TopologyAtt"));
	if (topAtt == NULL) {
		topAtt = new TopologyAttachment();
	   topAtt->setup("TopologyAtt", AttachedData::ADF_PERSISTENT);
		attachments->attachData(topAtt);
	}
	topAtt->setTopology(tp);
}
