//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ARTransfConstraintsToUrshape.h"
//---------------------------------------------------------------------------
#include "DeformationEvaluator.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------




ARTransfConstraintsToUrshape::ARTransfConstraintsToUrshape(
   DeformationEvaluator *evaluator,
   ARConstraintsInterface *receiver) 
{
   this->evaluator = evaluator;
   this->receiver = receiver;
}

void ARTransfConstraintsToUrshape::addGeneralConstraint(const GeneralARConstraint *c) 
{
   const PositionARConstraint *pcr = dynamic_cast<const PositionARConstraint*>(c);
   if (!pcr) {
      const ARHalfSpaceConstraint *hscr = dynamic_cast<const ARHalfSpaceConstraint*>(c);
      if (!hscr) {
         throw ETypeMismatch("ARTransfConstraintsToUrshape::addGeneralConstraint() - supports only position and halfspace constraints.");
      } else {
         addHalfSpaceConstraint(hscr);
      }      
   } else {
      addPositionConstraint(pcr);
   }
}

void ARTransfConstraintsToUrshape::addPositionConstraint(const PositionARConstraint *c) 
{
   PositionARConstraint transformedConstraint;
   transformedConstraint.anchor = c->anchor;
   transformedConstraint.time = c->time;
   Matrix3f invLinApprox;
   transformedConstraint.toPoint = evaluator->invertDeformation(c->toPoint, c->anchor, NULL, &invLinApprox);
   transformedConstraint.covariance = invLinApprox.transpose() * c->covariance * invLinApprox;

   receiver->addPositionConstraint(&transformedConstraint);
}

void ARTransfConstraintsToUrshape::addHalfSpaceConstraint(const ARHalfSpaceConstraint *c) 
{
   ARHalfSpaceConstraint transformedConstraint;
   transformedConstraint.anchor = c->anchor;
   transformedConstraint.time = c->time;
   Matrix3f linApprox;
   Matrix3f invLinApprox;
   transformedConstraint.halfSpaceFootPoint = evaluator->invertDeformation(c->halfSpaceFootPoint, c->anchor, &linApprox, &invLinApprox);
   transformedConstraint.halfSpaceNormal = linApprox.transpose() * c->halfSpaceNormal; // transform by inv-transpose of inv
   transformedConstraint.penaltyWeight = c->penaltyWeight;
   
   receiver->addHalfSpaceConstraint(&transformedConstraint);
}

void ARTransfConstraintsToUrshape::addSpatialDerivativeConstraint(const SpatialDerivativeARConstraint *c) {
   notImplemented();
}

void ARTransfConstraintsToUrshape::addSimpleSpatialDerivativeConstraint(const SimpleSpatialDerivativeARConstraint *c) 
{
   notImplemented();
}
