//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ARUrshapeDeformationOpt.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "DeformationEvaluator.h"
#include "DeformationSolver.h"
#include "ARSequence.h"
#include "PointCloud.h"
#include "ARSettings.h"
#include "ARDeformationOptimizationContext.h"
#include "ICPConstraints.h"
#include "AREvaluateDeformation.h"
#include "ARTransfConstraintsToUrshape.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



ARUrshapeDeformationOpt::ARUrshapeDeformationOpt() 
{
}

ARUrshapeDeformationOpt::~ARUrshapeDeformationOpt() 
{
}


void ARUrshapeDeformationOpt::glDrawFeedback(GLContext *glContext, ARVisualizationSettings *visSettings) 
{
   forget(glContext);
}


PointCloud *ARUrshapeDeformationOpt::optimizeUrshape(
   ARSequence *seq, 
   DeformationSolver *deformationSolver,
   const vector<PointCloud*> &dataPoints,
   ARSettings *settings)
{
   DeformationEvaluator *deformEvaluator = deformationSolver->createEvaluator();
   
   ARDeformationOptimizationContext *urshapeDeformationContext = 
      (ARDeformationOptimizationContext*)deformationSolver->createOptContext();

   urshapeDeformationContext->initializeFromShape(seq->getUrshape(), makeVector2f(0,0), settings);
   deformationSolver->setup(urshapeDeformationContext, settings);
   deformationSolver->beginConstraints();

   const card32 numFrames = seq->getDuration()+1;
   for (card32 i=0; i<numFrames; i++) {
      ICPConstraints icp;
      icp.setConstraintMode(settings->icpConstraintsMode);
      icp.setDistanceOutliersMode(settings->icpNearFieldCulling?ICP_OUTLIERS_DISTANCE:ICP_OUTLIERS_NONE);
      icp.setOutliersDistanceFactor(settings->icpNearFieldRelativeCutOff);
      icp.setCullNormalOutliers(false);
      deformEvaluator->setup(seq->getDeformation(), settings);
      
      PointCloud *deformedReference = AREvaluateDeformation::evaluatePC(
         seq->getUrshape(), 
         seq->getDeformation(), 
         deformEvaluator,
         i,
         settings);

      deformEvaluator->beginQuery(i);
      ARTransfConstraintsToUrshape adaptor(deformEvaluator, deformationSolver);
      
	   icp.buildICPConstraints(
		   settings, &adaptor, seq->getUrshape(),
		   deformedReference,
		   dataPoints[i],
		   0,
		   NULL, false,
		   settings->icpSkipPercentage
	   );
	   delete deformedReference;
      deformEvaluator->endQuery();
   }
   deformationSolver->solve();

   PointCloud *result = AREvaluateDeformation::evaluatePC(seq->getUrshape(), urshapeDeformationContext, deformEvaluator, 0, settings);

   delete deformEvaluator;
   delete urshapeDeformationContext;
   return result;
}