//---------------------------------------------------------------------------
#ifndef ARUrshapeOptimizationH
#define ARUrshapeOptimizationH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "AROptToolWithFeedback.h"
#include "Delegate.h"
//---------------------------------------------------------------------------

namespace geoxl {

class ARVisualizationSettings;
class ARSequence;
class ARSettings;
class ShapeReconstruction;
class DeformationEvaluator;


///
///   computes an optimal urshape given a deformation sequence
///
class DEFORMATION_API ARUrshapeOptimization : public AROptToolWithFeedback {
 private:
   PointCloud *visPC;
   
 public:
	ARUrshapeOptimization();
	
	virtual void glDrawFeedback(GLContext *glContext, ARVisualizationSettings *visSettings);
	
	PointCloud *optimizeUrshape(
	   /// sequence to be optimized
	   ARSequence *seq, 
	   /// shape optimizer
	   ShapeReconstruction *shapeOpt, 
	   /// deformation evaluator to be used to transform the constraints
	   /// (will be setup with the context from seq)
      DeformationEvaluator *deformEval,
      /// input data points
      const vector<PointCloud*> &dataPoints,
      /// parameters
	   ARSettings *settings
   );
	PointCloud *optimizeSingleFrame(
	   /// sequence to be optimized
	   PointCloud *pc, 
	   /// shape optimizer
	   ShapeReconstruction *shapeOpt, 
      /// input data points
      PointCloud *dataPoints,
      /// parameters
	   ARSettings *settings,
	   /// feedback
	   Delegate callForRedraw = NULL
   );
};

}

#endif