//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "DataToUrshape.h"
//---------------------------------------------------------------------------
#include "ARSequence.h"
#include "ARSettings.h"
#include "PointCloud.h"
#include "DeformationEvaluator.h"
#include "UnstructuredInCorePointCloud.h"
#include "ARDeformationOptimizationContext.h"
#include "AREvaluateDeformation.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "PointSetKNNQuery.h"
#include "AttachedIndexedOctree.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


struct VStruct {
   Vector3f position;
   Vector3f normal;
};

UnstructuredInCorePointCloud *DataToUrshape::transfD2U(
   ARSequence *seq,
   vector<PointCloud*> data,
   DeformationEvaluator *deformEval,
   ARSettings *settings)
{
   UnstructuredInCorePointCloud *result = new UnstructuredInCorePointCloud();
   VertexDescriptor vd;
   vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
   vd.pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
   result->clearAndSetup(&vd, 0);
   AAT positionAATResult = result->getAAT("position");
   AAT normalAATResult = result->getAAT("normal");
   PointSet *resultPS = result->getPointSet();
   
   const card32 numFrames = seq->getDuration()+1;
   pAssert(numFrames == data.size());
   deformEval->setup(seq->getDeformation(), settings);
   for (card32 i=0; i<numFrames; i++) {
      UnstructuredInCorePointCloud *deformedReference = AREvaluateDeformation::evaluatePC(
         seq->getUrshape(),
         seq->getDeformation(),
         deformEval,
         i,
         settings);

      PointSet *deformedReferencePS = deformedReference->getPointSet();
      AAT positionAATDRef = deformedReferencePS->getAAT("position");

      deformEval->beginQuery(i);
      AttachedIndexedOctree* octree = PointSetKNNQuery::createOctree( deformedReferencePS, 32 );
      PointSetKNNQuery knn( deformedReferencePS, octree, makeVector3f( 0,0,0 ), positionAATDRef );

      UnstructuredInCorePointCloud *dataPC = dynamic_cast<UnstructuredInCorePointCloud*>(data[i]);
      if (!dataPC) continue;
      PointSet *dataPS = dataPC->getPointSet();
      card32 numDataPts = dataPS->getNumEntries();
      
      AAT positionAATData = dataPS->getAAT("position");
      AAT normalAATData = NULL_AAT;
      if (dataPS->providesAttribute("normal")) {
         normalAATData = dataPS->getAAT("normal");
      }
      VStruct v;
      v.position = NULL_VECTOR3F;
      v.normal = NULL_VECTOR3F;
      for (card32 p=0; p<numDataPts; p++) {
         Vector3f dataPos = dataPS->get3f(p, positionAATData);
         knn.setTargetPoint(dataPos);
         card32 nextIndex;
         bool found;
         knn.getNextNearestPointIndex(nextIndex, found);
         if (found) {
            ARConstraintAnchor anch;
            anch.topologicalAnchor = nextIndex;
            anch.pos = deformedReferencePS->get3f(nextIndex, positionAATDRef);
            v.position = deformEval->invertDeformation(dataPos, anch, NULL, NULL, 1e-5f, settings->dataToUsNumIterations);
            if (normalAATData != NULL_AAT) {
               v.normal = invertMatrix(deformEval->getSpatialGradient(anch)).transpose()*dataPS->get3f(p, normalAATData);
            }
            resultPS->addPoint((const char*)&v);
         }
      }
      
      delete octree;
      delete deformedReference;
   }
   return result;
}