//---------------------------------------------------------------------------
#ifndef ARDataConstraintsSettingsH
#define ARDataConstraintsSettingsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

///
///   This class controls which data constraints are used in the merging process
///
class DEFORMATION_API ARDataConstraintsSettings : public Persistent {
	GEOXL_CLASS( ARDataConstraintsSettings )
 private:
   // --- geometry matching
	bool useGeometry;
	//vector<string> geometryDataPaths; - use path in ARMergeSettings

   // --- feature matching
	bool useFeatures;
	vector<string> featureCorrPaths;

   // --- photometric matching
	bool usePhotoConsistency;
	bool useOpticalFlow;
	vector<string> imageDataPaths;

   // --- silhouette constraints
	bool useSilhouettes;
	vector<string> depthImagesPaths;

	//IMPLEMENT_VAR_ARRAY_METHODS_STL(geometryDataPaths)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(featureCorrPaths)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(imageDataPaths)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(depthImagesPaths)

 public:
	ARDataConstraintsSettings();

	void setPresets();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   ~ARDataConstraintsSettings();
};

}

#endif