//---------------------------------------------------------------------------
#ifndef ARMergeSettingsH
#define ARMergeSettingsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "BackendSettings.h"
//---------------------------------------------------------------------------

namespace geoxl {

class ARPipelineOptions;
class ARSettings;
class ARDataConstraintsSettings;
class DeformationSolver;
class ShapeReconstruction;
class SurfelGraphInit2;
class ARConstraintsSource;
class ARVisualizationSettings;
class ARSequenceEvaluator;
class ARGlobalOptSettings;

///
///   this class contains all parameters necessary to control merging of sequences
///
class DEFORMATION_API ARMergeSettings : public Persistent {
	GEOXL_CLASS( ARMergeSettings )
 private:
	void getPathIntern(string whichPath, Scene *scene, card32 frame, string &framePath, SceneGraphNode *&node);
 
 public:
	// -- input & output
	string constraintsDataPath;   // folder with data to be used in optimization
	string urshapesDataPath;      // folder with data to be used to create initial urshapes
	string globalOptDataPath;     // folder with data to be used in global optimization
	string urshapesPath;          // folder that stores the computed urshapes
	string deformationsPath;      // folder that stores the computed deformations
	string resultsPath;           // folder in which to put the results
	// -- configuration
	ARPipelineOptions         *pipelineOptions;
	ARSettings                *optimizationSettings;
	ARDataConstraintsSettings *dataConstraintsSettings;
	ARGlobalOptSettings       *globalOptSettings;
	ARVisualizationSettings   *visualizationSettings;
	BackendSettings     *backendSettings;
	// -- solvers
	DeformationSolver			  *deformationSolver;
	ShapeReconstruction		  *shapeSolver;
	SurfelGraphInit2  		  *initUrshapes;
	ARSequenceEvaluator       *finalOutput;

	// ----

	ARMergeSettings();

	void setPresets();
	virtual void assign(const Object* obj, CopyContext *context = NULL);

   void getConstraintsDataPath(Scene *scene, card32 frame, string &framePath, SceneGraphNode *&node);
   void getUrshapesDataPath(Scene *scene, card32 frame, string &framePath, SceneGraphNode *&node);
   void getGlobalOptDataPath(Scene *scene, card32 frame, string &framePath, SceneGraphNode *&node);
   void getUrshapesPath(Scene *scene, card32 frame, string &framePath, SceneGraphNode *&node);
   void getDeformationsPath(Scene *scene, card32 frame, string &framePath, SceneGraphNode *&node);
   void getResultsPath(Scene *scene, card32 frame, string &framePath, SceneGraphNode *&node);
	
   ~ARMergeSettings();
};

}

#endif