//---------------------------------------------------------------------------
#ifndef ARPipelineOptionsH
#define ARPipelineOptionsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

///
///   fundamental options for the merging pipeline
///   (overall merging strategy)
///
class DEFORMATION_API ARPipelineOptions : public Persistent {
	GEOXL_CLASS( ARPipelineOptions )
 private:
   bool unused_rigidMode;
	bool unused_alignWithConstrData;
 public:
	enum MergeMode {
		mmExtractSurfacesFirst, mmExtractSurfacesLast
	};

   // --- general options	
	MergeMode mode;                   // controls whether surface reconstruction is called regularly during merging (mmExtractSurfacesLast = no reconstruction at all, only in global opt)
	bool deletePartialResults;        // if not set, all intermediate merging results are kept (for debugging purpose)
   card32 minKeepPartialResultsSize; // minimum size of a partial result to be kept
	bool urshapePropagation;          // propagates the urshape across the binary merging boundaries in order to improve the pairwise merging stability
	bool urshapeDeformationOpt;       // trys to deform the urshape to better match the data
   bool optimizeShapeWhileMerging;   // perform shape opt during merging (independent of intial urshapes)?
   bool mergeInPowersOfTwo;          // always performs splits at power-of-two boundaries
   bool lateTopologyCheck;           // perform top check after hole filling
   bool noTopologyCheck;             // do not check for topological changes (saves time & mem)
	bool matchingOnly;                // in order to use the tool for deformable matching, all other optimizations can be disabled
	
	// --- debugging options
	card32 maximumMergeRange;
	static const card32 DEFAULT_PIPELINE = 0;
	static const card32 STOP_AFTER_MERGE = 1;
	card32 bailOut;
	bool savePartialResults;
	string savePartialResultsPath;
	bool recomputeDeformationsInMerge; // should be true
	
	ARPipelineOptions();

	void setPresets();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   ~ARPipelineOptions();
};

}

#endif