#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCRemoveSmallPatches.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ARMergeSequences.h"
#include "DeformationTools.h"
#include "SurfelGraphInit2.h"
#include "VertexArray.h"
#include "InCorePCTopologyGraph.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( PCCRemoveSmallPatches ,0)
{
	BEGIN_CLASS_INIT( PCCRemoveSmallPatches );
	ADD_STRING_PROP(pcName,0 )
	ADD_CARD32_PROP(minConnectedPoints, 0)
	ADD_CARD32_PROP(top_maxNN, 0)
	ADD_FLOAT32_PROP(top_spacingFactor,0 )

	ADD_NOARGS_METHOD(mathUnitTest);
}

PCCRemoveSmallPatches::PCCRemoveSmallPatches(void)
{
	pcName="root/pc1";
	minConnectedPoints=100;
	top_maxNN=4;
	top_spacingFactor=3.0f;
}

PCCRemoveSmallPatches::~PCCRemoveSmallPatches(void)
{
}

void PCCRemoveSmallPatches::assign(const Object* obj, CopyContext *context)
{
	PCCommand::assign(obj,context);
	const PCCRemoveSmallPatches * o = dynamic_cast<const PCCRemoveSmallPatches*>(obj);
	pAssert( o != NULL );

	copyObjectProperties(obj,this);
}

void geoxl::PCCRemoveSmallPatches::execute( Scene *scene )
{
	UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(getPointCloud(scene,pcName));
	if (!upc) return;

	PointSet* ps=upc->getPointSet();

	HierarchicalKNNIterator hIt(upc,32,NULL);
	VertexArray varr(upc);
	AAT posAAT = upc->getAAT("position");


	InCorePCTopologyGraph tg;
	VertexDescriptor vd;
	vd.pushAttrib(VAD("vindex", 2, VAD::DATA_FORMAT_INT32));

	tg.clearAndSetup(&vd);

	debugOutput << "Computing median point distance..."; debugOutput.flush();
		float medPointDistance = getMedianPointDistance(upc);
	debugOutput << "done.\n Creating topology..."; debugOutput.flush();

	float32 allowedDist = medPointDistance * top_spacingFactor;
	const card32 numPoints=varr.getNumElements();
	
	for( unsigned i=0;i<numPoints;i++ )
	{
		Vector3f pos = varr.getPosition3f(i);
		hIt.setSeekPointAndReset(pos);
		hIt.next(); //don't use self
		for( card32 j=0;j< top_maxNN; j++ )
		{
			card32 n = hIt.getCurrentPointIndex();
			Vector3f npos = hIt.get3f(posAAT);
			if( i != n && norm(npos-pos)< allowedDist)
				tg.addEdge(i,n);
			hIt.next();
		}
	}
	debugOutput << "done.\n Calling removeSmallComponents...\n"; debugOutput.flush();
	SurfelGraphInit2::removeSmallComponents(ps, &tg, 100 , NULL);
	debugOutput << "done.\n "; debugOutput.flush();

}


void geoxl::PCCRemoveSmallPatches::execute_ar( Scene *scene, UnstructuredInCorePointCloud *upc )
{
	PointSet* ps=upc->getPointSet();

	HierarchicalKNNIterator hIt(upc,32,NULL);
	VertexArray varr(upc);
	AAT posAAT = upc->getAAT("position");


	InCorePCTopologyGraph tg;
	VertexDescriptor vd;
	vd.pushAttrib(VAD("vindex", 2, VAD::DATA_FORMAT_INT32));

	tg.clearAndSetup(&vd);

	float medPointDistance = getMedianPointDistance(upc);

	float32 allowedDist = medPointDistance * top_spacingFactor;
	const card32 numPoints=varr.getNumElements();

	for( unsigned i=0;i<numPoints;i++ )
	{
		Vector3f pos = varr.getPosition3f(i);
		hIt.setSeekPointAndReset(pos);
		hIt.next(); //don't use self
		for( card32 j=0;j< top_maxNN; j++ )
		{
			card32 n = hIt.getCurrentPointIndex();
			Vector3f npos = hIt.get3f(posAAT);
			if( i != n && norm(npos-pos)< allowedDist)
				tg.addEdge(i,n);
			hIt.next();
		}
	}
	SurfelGraphInit2::removeSmallComponents(ps, &tg, minConnectedPoints, NULL);
}

void geoxl::PCCRemoveSmallPatches::mathUnitTest()
{
//	FastSparseMatrix m;m.preCondCGSolverTest();
}
