//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PCCTopologicalResampling.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "UnstructuredInCorePointCloud.h"
#include "ARTopologicalResampling.h"
#include "InCorePCTopologyGraph.h"
#include "DeformationTools.h"
#include "PointCloudTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS(PCCTopologicalResampling, 0)
{
	BEGIN_CLASS_INIT(PCCTopologicalResampling)
   ADD_FLOAT32_PROP(sampleSpacing, 0)
   ADD_FLOAT32_PROP(relMaxConnectSpacing, 0)
   ADD_CARD32_PROP(maxNumNeighbours, 0)
}

//---------------------------------------------------------------------------

PCCTopologicalResampling::PCCTopologicalResampling() {
   sampleSpacing = 0.01f;
   relMaxConnectSpacing = 3;
   maxNumNeighbours = 8;
}

PCCTopologicalResampling::~PCCTopologicalResampling() {}

void PCCTopologicalResampling::assign(const Object* obj, CopyContext *context)
{
	copyObjectProperties(obj, this);
}

//---------------------------------------------------------------------------

void PCCTopologicalResampling::execute(Scene *scene) 
{
   unsigned int m;
   vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
   for ( m = 0; m < selection.size(); m++ ) {
      PointCloud *pc = getPointCloud( scene, selection[m] );
      UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
      if (!upc) {
         warning("works only on unstructured-in-core point clouds");
         continue;
      }
      InCorePCTopologyGraph *tg = getTopology(upc);
      if (!tg) {
         warning("no topology");
         continue;
      }
      UnstructuredInCorePointCloud *res = ARTopologicalResampling::resample(
         upc, 
         sampleSpacing,
         relMaxConnectSpacing,
         maxNumNeighbours);
      upc->assign(res);
      delete res;
   }
}
