#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "CameraParameterAttachment.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( CameraParameterAttachment ,0)
{
	BEGIN_CLASS_INIT( CameraParameterAttachment );
	ADD_MATRIX3F_PROP(intrinsicMatrix, 0);
	ADD_MATRIX3F_PROP(rotationMatrix, 0);
	ADD_VECTOR3F_PROP(translationVector, 0);
	ADD_MATRIX4F_PROP(projectionMatrix, 0);
}

CameraParameterAttachment::CameraParameterAttachment(void)
{
	intrinsicMatrix=IDENTITY3F;
	rotationMatrix=IDENTITY3F;
	translationVector=NULL_VECTOR3F;
	projectionMatrix=IDENTITY4F;
}

CameraParameterAttachment::~CameraParameterAttachment(void)
{
}

void CameraParameterAttachment::assign(const Object* obj, CopyContext *context)
{
	copyObjectProperties(obj, this);
}

geoxl::CameraParameterAttachment::CameraParameterAttachment( Matrix3f intrinsicM,Matrix3f rotationM, Vector3f translationV,Matrix4f projectionM )
{
	intrinsicMatrix=intrinsicM;
	rotationMatrix=rotationM;
	translationVector=translationV;
	projectionMatrix=projectionM;
}

geoxl::Vector2f geoxl::CameraParameterAttachment::projectWorldtoImage( const Vector3f xyz )
{
	const float imageWidth = 1024.0f;
	const float imageHeight = 768.0f;
	
	float u,v,w;
	float x = -xyz[0];
	float y = xyz[1];
	float z = xyz[2];

	const Matrix4f projMatrix = projectionMatrix;

	u = projMatrix[0][0]*x + 
		projMatrix[0][1]*y + 
		projMatrix[0][2]*z + 
		projMatrix[0][3];

	v = projMatrix[1][0]*x + 
		projMatrix[1][1]*y + 
		projMatrix[1][2]*z + 
		projMatrix[1][3];

	w = projMatrix[2][0]*x + 
		projMatrix[2][1]*y + 
		projMatrix[2][2]*z + 
		projMatrix[2][3];

	u /= w;
	v /= w;

	// image (0,0) is bottom lefthand corner
	v = (double) imageHeight - v - 1.0;

	return makeVector2f(u,v);
} 

geoxl::Vector3f geoxl::CameraParameterAttachment::projectImageToWorld( const Vector3f &uvz )
{
	float c0, c1, c2;

	const float u = uvz[0];
	const float v = 768.0f - uvz[1] - 1.0f;
	const float z = uvz[2];
	const Matrix4f projMatrix = projectionMatrix;

	c0 = z*projMatrix[0][2] + projMatrix[0][3];
	c1 = z*projMatrix[1][2] + projMatrix[1][3];
	c2 = z*projMatrix[2][2] + projMatrix[2][3];

	Vector3f ret;
	ret[1] = u*(c1*projMatrix[2][0] - projMatrix[1][0]*c2) +
		v*(c2*projMatrix[0][0] - projMatrix[2][0]*c0) +
		projMatrix[1][0]*c0 - c1*projMatrix[0][0];

	ret[1] /= v*(projMatrix[2][0]*projMatrix[0][1] - projMatrix[2][1]*projMatrix[0][0]) +
		u*(projMatrix[1][0]*projMatrix[2][1] - projMatrix[1][1]*projMatrix[2][0]) +
		projMatrix[0][0]*projMatrix[1][1] - projMatrix[1][0]*projMatrix[0][1];

	ret[0] = (ret[1])*(projMatrix[0][1] - projMatrix[2][1]*u) + c0 - c2*u;
	ret[0] /= projMatrix[2][0]*u - projMatrix[0][0];
	ret[0] = - ret[0];
	ret[2] = z;

	return ret;
}