//---------------------------------------------------------------------------
#ifndef PCCDataResamplingH
#define PCCDataResamplingH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
//---------------------------------------------------------------------------

namespace geoxl {

///
///   This class creates a resampled version of input data nodes for initial matching
///
class DEFORMATION_API PCCDataResampling : public PCCommand {
	GEOXL_CLASS( PCCDataResampling )

 public:
	/// point removal - good results but slow
	static const card32 METHOD_POINT_REMOVAL = 0;
	/// grid decimation - fast and inaccurate
	static const card32 METHOD_GRID = 1;
	/// grid decimation followed by point removal
	static const card32 METHOD_HYBRID = 2;
	/// add offset before quantization?
	bool avoidZeroCrossing;
	
 private:	
	/// where to read the input from
	string sourceFolder;
	/// where to put the result (must exists)
	string targetFolder;
	/// additional prefix to filenames in case of OOC PCs (no effect otherwise)
	string filenamePrefix;
	/// new (guaranteed) point distance
   float32 maxDist;
   /// frames to skip (every k-th)
   card32 frameResampling;
	/// how to perform the resampling (default: grid - it's really fast)
   card32 method;
   
 public:
	PCCDataResampling();
	
	void setup(
		string sourceFolder,
		string targetFolder,
		float32 maxDist,
		card32 frameResampling
	);

	virtual void assign(const Object* obj, CopyContext *context = NULL);
	virtual void execute(Scene *scene);

	string getSourceFolder() const {return sourceFolder;}
	void setSourceFolder(string v) {sourceFolder=v;}
	string getTargetFolder() const {return targetFolder;}
	void setTargetFolder(string v) {targetFolder=v;}
	float32 getMaxDist() const {return maxDist;}
	void setMaxDist(float32 v) {maxDist=v;}
	card32 getMethod() const {return method;}
	void setMethod(card32 v) {method=v;}
	card32 getFrameResampling() const {return frameResampling;}
	void setFrameResampling(card32 v) {frameResampling=v;}



   ~PCCDataResampling();
};

}

#endif