//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCNodeTest.h"
#include "ClassMethods.h"
#include "PointCloudTools.h"
#include "SGListNode.h"
#include "UnstructuredInCorePointCloud.h"
#include "SGObjectNode.h"
#include "SGEMeshlessUrshape.h"
#include "MMDeformationContext.h"
#include "SGDeformationConstraintsList.h"
#include "MMDeformationSolver.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS( PCCNodeTest , 0 ) {
	BEGIN_CLASS_INIT( PCCNodeTest );
	ADD_STRING_PROP(urshapePath, 0)
	ADD_NOARGS_METHOD(setupTestCase)
}


PCCNodeTest::PCCNodeTest() {
	urshapePath = "root/urshape";
}


PCCNodeTest::~PCCNodeTest() {
}


void PCCNodeTest::execute(Scene *scene) {
	//SGEMeshlessUrshape * urshape = new SGEMeshlessUrshape();
	//urshape->setScene(scene);
	//urshape->setPath("root/urshape");
	//urshape->tempTest();

   MMDeformationContext *context = new MMDeformationContext();
   string sourceNode = "root/pc1";
   PointCloud *pcDomain = getPointCloud(scene, sourceNode);
	context->initializeFromShape(pcDomain, makeVector2f(0,0), 0);
	addPointCloud(scene, context->getNodes()->getPointCloud(), "nodal sampling");

	MMDeformationSolver solver;
	solver.setup(context, 0);
	PositionARConstraint * constraint = new PositionARConstraint();
	ARConstraintAnchor anchor; anchor.setPos(context->getNodes()->getNode(0)->getPos());
	constraint->setup(anchor, makeVector3f(3,3,3), makeMatrix3f(1,0,0,1,0,0,1,0,0),0);
	solver.addPositionConstraint(constraint);
	
	solver.solve();

	UnstructuredInCorePointCloud * result  = dynamic_cast<UnstructuredInCorePointCloud*>(pcDomain->copy());
	AAT posAAT = result->getAAT("position");
	PointSet *ps = result->getPointSet();
	for (card32 i=0; i<result->getNumPoints(); i++) {
		if ((i%1000)==0) cout << "Deforming point : " << i << " " << result->getNumPoints() << endl;
		Vector3f pos = ps->get3f(i, posAAT);
		pos+=solver.getContext()->getDeformation(pos,0);
		ps->set3f(i, posAAT, pos);
	}
	addPointCloud(scene, result, "deformed geometry");

	//MMDeformationContext *context = ;
}

void PCCNodeTest::setupTestCase() {
	Scene *s = getScene();
	SGListNode *node = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(scene, urshapePath));
	if (node == NULL) {
		error("does not work");
		return;
	}
	UnstructuredInCorePointCloud *newPC = new UnstructuredInCorePointCloud();
	VertexDescriptor vd;
	vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	vd.pushAttrib(mVAD("radius", 1, VAD::DATA_FORMAT_FLOAT32));
	newPC->clearAndSetup(&vd, 0);
	SGObjectNode *newSO = new SGObjectNode();
	newSO->clearAndSetup(newPC);
	newSO->setName("mmNodes");
	if (node->getNumChildNodes(NULL) != 1) {
		warning("not the right number of child nodes.");
	}
	node->addChildNode(newSO);
}