#ifndef PCCNodesToolsH
#define PCCNodesToolsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "SGEShape.h"
//---------------------------------------------------------------------------
#include "SGEMeshlessUrshape.h"
//---------------------------------------------------------------------------

namespace geoxl {

class InCorePCTopologyGraph;
class VertexDescriptor;
class AttachedIndexedOctree;

///
///   mo - 11/121/2007
///
///   Helper functions for creating a set of deformation nodes (as a pointCloud) from the geometry.
///   Random Sampling with given offset (in the both positive and negative normal direction) and
///	Topological Sampling, which does a breadth-first search on a given topology to retrieve all points within a certain radius.

typedef pair<float32,card32> edgeType;

enum REFINE_TYPE { 
	RANDOM, 
	TOP_SEARCH,
	TOP_SEARCH2
};

	
class Prioritize {
public:
	int operator() ( edgeType& p1, 
		edgeType& p2 ) {
			return p1.first > p2.first;
	}
	
	
};

/*
bool operator< ( edgeType& a, edgeType& b )
{
  return a.first < b.first;
}*/

class DEFORMATION_API PCCNodesTools : public PCCommand {
	GEOXL_CLASS( PCCNodesTools )
	
 private:
	float32 offset;
	float32 radius;
	int32 numNodes;
	int32 minNeighbors;
	
	card32 type;
	
	bool PCCNodesTools::vertexMerge(int32 v1, int32 v2, int32 edgeIndex, PointSet *edges, vector< set<card32> > *edgeTable, float32 radius, PointSet *geometry, priority_queue< edgeType, vector<edgeType>, Prioritize> *edgeIndices, vector<bool> *to_keep,vector<card32> *counter, bool useQueue);
	
	unsigned getOther(Vector2i both, unsigned given) const;

 public:
	PCCNodesTools();

	PCCNodesTools(float32 offset, float32 radius, int32 minNeighbors = 2, int32 numNodes = 1000, card32 type = TOP_SEARCH);

	virtual void assign(const Object* obj, CopyContext *context = NULL);
	virtual void execute(Scene *scene);

	/// get point cloud object for the mmNodes part
	UnstructuredInCorePointCloud *createMMNodesPointCloud() const ;	
	
	// generate a pointcloud from the parameter set. If a pointer to map is provided, it will also output the mapping between original and kept points
	UnstructuredInCorePointCloud *getToKeep(UnstructuredInCorePointCloud *geometry, vector<bool> to_keep, float32 offset, map<int32,int32> *keepers = NULL, vector< set<card32> > *edgeTable = NULL, PointSet *edges = NULL);	
	
	// assigns radii to each Node (assumes only position channel) so that each has exactly nNeighbors within the radius.
	// Adds a radius channel if it doesn't exist
	void setRadii(UnstructuredInCorePointCloud *mmNodes,int32 nNeighbors);
	
	//makes sure all of the geometry points are close to some deformation nodes, 
	//returns the radius which ensures that each geometry point has at least nNeighbors node points within it.
	float32 testNodes(UnstructuredInCorePointCloud *mmNodes, UnstructuredInCorePointCloud *geometry, int32 nNeighbors, float32 radius);
	
	// get point cloud object for the mmNodes part
	UnstructuredInCorePointCloud *randomSampleMMNodesPointCloud(UnstructuredInCorePointCloud *geometry, float32 offset, int32 numNodes);	
	
	UnstructuredInCorePointCloud *edgeContractionSampleMMNodesPointCloud(UnstructuredInCorePointCloud *geometry, float32 offset, float32 radius, bool useQueue = true);
	
	// get the topological sampling. Assumes that a InCorePCTopologyGraph exists as an attachment to the geometry
	UnstructuredInCorePointCloud *topSampleMMNodesPointCloud(UnstructuredInCorePointCloud *geometry, float32 offset, float32 radius);	
	
	void copyConnections(card32 oldPoint, card32 newNeighbor, vector< set<card32> > *edgeTable , PointSet *edges);
	
	void setTopology(UnstructuredInCorePointCloud* uc, vector< set<card32> > *edgeTable, PointSet *edges, map<int32,int32> *keepers );
	
	void addVerifiedTopology(UnstructuredInCorePointCloud *geometry, float32 euclRadius, float32 topRadius);
	
	void verify(UnstructuredInCorePointCloud* pointcloud, Vector3f position, float32 radius, set<card32> *euclNeighbors, set<card32> *verified);
	
	int32 getTopAnchor(PointSet *ps, AttachedIndexedOctree* octree, Vector3f position, AAT positionAAT, AAT baseAAT);
	
	void output();
	
	~PCCNodesTools();
};

}

#endif