//---------------------------------------------------------------------------
#ifndef PCIAnimationReconstruction2H
#define PCIAnimationReconstruction2H
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------

namespace geoxl {

class ARMergeSettings;
class ARMergeSequences;
class AROptToolWithFeedback;
class ARSettings;
class ARSequence;
class DeformationEvaluator;


///
///   Main user interface for animation reconstruction 2.0
///
class DEFORMATION_API PCIAnimationReconstruction2 : public PCInteractionTool {
	GEOXL_CLASS( PCIAnimationReconstruction2 )
 private:
	// --- settings
	card32 startFrame;
	card32 endFrame;
	ARMergeSettings *mergeSettings;
	
	// --- temporary data
	AROptToolWithFeedback *currentToolToDraw;
	bool mergeAborted;
	
	void setPresets();
	
	// --- buttons
	void clean();
	//void mmSolver();
	void mergeRange();
   void globalOpt();
   void computeDataWeight();
	void computeGlobalResidual();
	void testRigidities();
	void convertResidualsToRigidities();
	void adaptDeformationToRigidities();
   //void globalOptWindowed();
   void outputResult();
	void loadSettings();
	void saveSettings();
	void upsampleUrshape();
	void backend();
	void outputDefNodes();
	void upsampleAndGlobalOpt();
	void dataToUrshape();
	void test();
	void computeFrameResiduals();
	
	// --- helper functions
	void recursiveMerge(
	   card32 startFramePow2, 
	   card32 endFramePow2,
      PointCloud *&urshapePropagation);
	void initUrshape(card32 frame);
	void mergeSequences(
	   card32 seqAFirstFrame, card32 seqALastFrame,
	   card32 seqBFirstFrame, card32 seqBLastFrame,
	   PointCloud *urshapePropagationInput, PointCloud *&urshapePropagationOutput
	);
   bool urshapeExists(card32 startIndex, card32 endIndex);
   card32 splitRange(card32 startFrame, card32 endFrame);
   // Apply optimization on shape in backend
   void backendOptimizeShape( 
	   UnstructuredInCorePointCloud* pc, 
	   ARSettings* settings,
	   ARSequence* seq,
	   DeformationEvaluator* evaluator,
	   vector<PointCloud*>& dataPoints );
	
 public:
	PCIAnimationReconstruction2();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

	virtual bool forceCamByDefault() {return true;}

	virtual void glDrawTool(GLContext *glContext);
   virtual bool drawGLSceneRendering();

   ~PCIAnimationReconstruction2();



   static string makeUrshapeName(card32 startIndex, card32 endIndex);
   static string makeDeformationName(card32 startIndex, card32 endIndex);
};
		
}

#endif