//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "InitEditor.h"
//---------------------------------------------------------------------------

/*
	Please separate between system code and interface code ( editors ), in 
	order to build a non-interface commandline version of geoxl.

File structure:
	includes
		..
		..
	interface includes
		..
		..
	initEditor()
	{
		init geoxl classes
		..
		..
		init geoxl class editors
		..
		..
	}
	shutdownEditor()
	{
		shutdown geoxl classes
		..
		..
		shutdown geoxl class editors
		..
		..
				
	}
*/
//---------------------------------------------------------------------------

#include "PCCommand.h"
#include "CommandObject.h"
#include "CommandObjMacro.h"
#include "OnSceneChangeCallback.h"

// command objects
#include "CmdObjFilter.h"
#include "CmdObjMappingOnClick.h"
#include "CmdObjSpecialDraw.h"
#include "CmdObjGeneratePC.h"
#include "CmdObjDrawOnClick.h"
#include "CmdObjSelection.h"
#include "CmdObjTransfEdit.h"
#include "CmdObjPointCloudOperations.h"
#include "CmdObjScriptProgram.h"
#include "SceneGraphCmdObjs.h"

// interaction tools
#include "PCInteractionTool.h"
#include "PCICameraController.h"

#include "PCICameraManager.h"
#include "PCIDrawingTool.h"
#include "PCIFilterTool.h"
#include "PCIMappingTool.h"
#include "PCIPointInfo.h"
#include "PCISelectionTool.h"
#include "PCITransfEdit.h"
#include "PCIManualAlignment.h"

//PCCommands
#include "PCCParametricResampling.h"
#include "PCCPointCloudCreator.h"
#include "PCCPointGenerator.h"
#include "PCCTriangleMeshSampler.h"
#include "PCCAnimatedTriangleMeshSampler.h"
#include "PCCImporter.h"
#include "PCCAnimationImporter.h"
#include "PCCSwapChannels.h"
#include "PCCAttributeModifier.h"
#include "PCCAutoAlign.h"
#include "PCCComputeFlags.h"
#include "PCCSetMaterial.h"
#include "PCCImplicitMesher.h"
//#include "PCCPointScript.h"
#include "PCCResampler.h"
#include "PCCPointNoise.h"
#include "PCCEstimateSampleSpacing.h"
#include "PCCDirectoryImporter.h"
#include "PCCConvToSwapAllOOC.h"
#include "PCCSortListNode.h"
#include "PCCAnimRename.h"
#include "PCCImportVolumeData.h"
#include "PCCVolumeGenerator.h"
#include "PCCNormalEstimatorPCA.h"

//Importer
#include "PointCloudImporterReader.h"
#include "PointCloudImporterReader2DImages.h"
#include "PointCloudImporterReader2DImagesPlusAlpha.h"
#include "PointCloudImporterReaderMSDepthBMP.h"
#include "PointCloudImporterReaderCustomASCII.h"
#include "PointCloudImporterReaderSMFOBJ.h"
#include "PointCloudImporterReaderPLY.h"
#include "PointCloudImporterReaderVRML.h"
#include "PointCloudImporterReaderBSM.h"
#include "PointCloudImporterAdder.h"
#include "PointCloudImporterReaderOFF.h"
#include "PointCloudImporterReaderCustom3DScan.h"
#include "PointCloudImporterReaderBinary.h"

#include "SceneClassEditors.h"
#include "PCSceneMaterialBrowser.h"
#include "CmdObjScriptProgramEditorWidget.h"
#include "Scene.h"
#include "SceneGraphEditor.h"
#include "GEOXLEvent.h"
#include "SceneInterface.h"
#include "SceneEditorOptions.h"

// debug
#include "TestRenderer.h"

// ---------------------------------------------------------------------------------
//					Interface Includes
// ---------------------------------------------------------------------------------

#ifdef GEOXL_GUI

#include "SceneEditorWidget.h"
#include "SceneEditorOptionsEditorWidget.h"

#include "PCIDrawingToolEditorWidget.h"
#include "PCISelectionToolEditorWidget.h"
#include "PCITransfEditEditorWidget.h"
#include "PCIFilterToolEditorWidget.h"
#include "PCIMappingToolEditorWidget.h"
#include "PCICameraManagerEditorWidget.h"

#include "PCCImporterEditorWidget.h"
#include "PCCAnimationImporterEditorWidget.h"
#include "PCCPointCloudCreatorEditorWidget.h"
#include "PointCloudImporterReaderCustomASCIIEditorWidget.h"

#endif	// end of #ifdef GEOXL_GUI

//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


void geoxl::initEditor() {

// ---------------------------------------------------------------------------------
//					Class Registration ( no Editors ! )
// ---------------------------------------------------------------------------------

	OnSceneChangeCallback::init( Persistent::getClass() );

   // commmand objects
   CmdObjChangePCFlags::init(CommandObject::getClass());
   CmdObjChangeSelection::init(CommandObject::getClass());
   CmdObjChangeSelectionOnClick::init(CommandObject::getClass());
   CmdObjChangeView::init(CommandObject::getClass());
   CmdObjDeletePCs::init(CommandObject::getClass());
   CmdObjDrawOnClick::init(CommandObject::getClass());
   CmdObjFilter::init(CommandObject::getClass());
   CmdObjLoadMap::init(CommandObject::getClass());
   CmdObjMappingOnClick::init(CommandObject::getClass());
   CmdObjPCCommand::init(CommandObject::getClass());
   CmdObjPointCloudOperations::init(CommandObject::getClass());
   CmdObjRenamePC::init(CommandObject::getClass());
   CmdObjSelectionByAttribute::init(CommandObject::getClass());
   CmdObjSelectionByButton::init(CommandObject::getClass());
   CmdObjSelectionOnArea::init(CommandObject::getClass());
   CmdObjSpecialDraw::init(CommandObject::getClass());
   CmdObjTransfEdit::init(CommandObject::getClass());
   CmdObjApplyRegisterTransform::init(CommandObject::getClass());
   CmdObjGeneratePC::init(CommandObject::getClass());
   CmdObjScriptProgram::init(CommandObject::getClass());
   CO_SGAddNode::init(CommandObject::getClass());
   CO_SGDeleteNode::init(CommandObject::getClass());
   CO_SGMoveNode::init(CommandObject::getClass());
   CO_SGMoveSelection::init(CommandObject::getClass());
   CO_SGGroupSelection::init(CommandObject::getClass());
   CO_SGUnGroupSelection::init(CommandObject::getClass());
   CO_SGChangeSelection::init(CommandObject::getClass());
   CmdObjAppendFields::init(CommandObject::getClass());
   CmdObjConvertType::init(CommandObject::getClass());
	CmdObjSGChangeDefaultTraversalMode::init(CommandObject::getClass());

   // commands
   PCCommand::init(Persistent::getClass());
	PCCAttributeModifier::init(PCCommand::getClass());
	PCCAutoAlign::init(PCCommand::getClass());
	PCCComputeFlags::init(PCCommand::getClass());
	PCCParametricResampling::init(PCCommand::getClass());
	PCCPointCloudCreator::init(PCCommand::getClass());
	PCCPointGenerator::init(PCCommand::getClass());
	PCCTriangleMeshSampler::init(PCCommand::getClass());
	PCCAnimatedTriangleMeshSampler::init(PCCommand::getClass());
	//	PCCPointScript::init(PCCommand::getClass());
	PCCImporter::init(PCCommand::getClass());
	PCCAnimationImporter::init(PCCImporter::getClass());
	PCCSwapChannels::init(PCCommand::getClass());
	PCCSetMaterial::init(PCCommand::getClass());
	PCCImplicitMesher::init(PCCommand::getClass() );
	PCCResampler::init( PCCommand::getClass() );
	PCCPointNoise::init( PCCommand::getClass() );
	PCCEstimateSampleSpacing::init(PCCommand::getClass());
	PCCDirectoryImporter::init(PCCommand::getClass());
	PCCConvToSwapAllOOC::init(PCCommand::getClass());
	PCCSortListNode::init(PCCommand::getClass());
	PCCAnimRename::init(PCCommand::getClass());
	PCCImportVolumeData::init(PCCommand::getClass());
	PCCVolumeGenerator::init(PCCommand::getClass());
	PCCNormalEstimatorPCA::init(PCCommand::getClass());	


	// Importer
	PointCloudImporterAdder::init(Persistent::getClass());
	PointCloudImporterAdderParametricDynamic::init(PointCloudImporterAdder::getClass());
	PointCloudImporterAdderDynamic::init(PointCloudImporterAdder::getClass());
	PointCloudImporterAdderMesh::init(PointCloudImporterAdder::getClass());
	PointCloudImporterReader::init(Persistent::getClass());
	PointCloudImporterReader2DImages::init(PointCloudImporterReader::getClass());
	PointCloudImporterReader2DImagesPlusAlpha::init(PointCloudImporterReader::getClass());
	PointCloudImporterReaderMSDepthBMP::init(PointCloudImporterReader::getClass());
	PointCloudImporterReaderCustomASCII::init(PointCloudImporterReader::getClass());
	PointCloudImporterReaderSMFOBJ::init(PointCloudImporterReader::getClass());
	PointCloudImporterReaderPLY::init(PointCloudImporterReader::getClass());
	PointCloudImporterReaderVRML::init(PointCloudImporterReader::getClass());
	PointCloudImporterReaderBSM::init(PointCloudImporterReader::getClass());
	PointCloudImporterReaderOFF::init(PointCloudImporterReader::getClass());
	PointCloudImporterReaderCustom3DScan::init(PointCloudImporterReader::getClass());
	PointCloudImporterReaderBinary::init(PointCloudImporterReader::getClass());


   // interactive tools
   PCInteractionTool::init(Persistent::getClass());
        PCICameraController::init(PCInteractionTool::getClass());
		PCICameraManager::init(PCInteractionTool::getClass());
		PCIDrawingTool::init(PCInteractionTool::getClass());
		PCIFilterTool::init(PCInteractionTool::getClass());
		PCIMappingTool::init(PCInteractionTool::getClass());
		PCISelectionTool::init(PCInteractionTool::getClass());
		PCIPointInfo::init(PCInteractionTool::getClass());
		PCITransfEdit::init(PCInteractionTool::getClass() );
		PCIManualAlignment::init(PCInteractionTool::getClass() );

   // geoxl events
   GEOXLEvent::init( Persistent::getClass() );
      ModeChangeEvent::init( GEOXLEvent::getClass() );
      SpecifiedModeChangeEvent::init( GEOXLEvent::getClass() );
      CommandObjectEvent::init( GEOXLEvent::getClass() );
      StringEvent::init( GEOXLEvent::getClass() );

   // init geoxl class editors
   SceneInterface::init( AttachedData::getClass() );
   SceneToolbar::init( Persistent::getClass() );
   SceneToolbarElement::init( Persistent::getClass() );
      SceneToolButton::init( SceneToolbarElement::getClass() );
      SceneToolbarCheckBox::init( SceneToolbarElement::getClass() );
      ToolbarSplitter::init( SceneToolbarElement::getClass() );

	SceneEditorOptions::init( AttachedData::getClass() );

	//debug
	TestRenderer::init(Renderer::getClass() );

// ---------------------------------------------------------------------------------
//					Editor Class Registration 
// ---------------------------------------------------------------------------------
#ifdef GEOXL_GUI

   // scene editors
   SceneClassEditor::init(ClassEditor::getClass());
      SceneGraphEditor::init(SceneClassEditor::getClass());
      PCSceneMaterialBrowser::init(SceneClassEditor::getClass());	

   CmdObjScriptProgramEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )CmdObjScriptProgram::getClass()->getGUIDescriptor())->addClassEditor(
		CmdObjScriptProgramEditor::getClass(), true);

  // register PCSceneEditor as application for Scene
	SceneEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )Scene::getClass()->getGUIDescriptor())->addClassEditor(
		SceneEditor::getClass(), true);

	SceneEditorOptionsEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )SceneEditorOptions::getClass()->getGUIDescriptor())->addClassEditor(
		SceneEditorOptionsEditor::getClass(), true);


	// PCI Editors
	PCIDrawingToolEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )PCIDrawingTool::getClass()->getGUIDescriptor())->addClassEditor(
		PCIDrawingToolEditor::getClass(), true);

	PCISelectionToolEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )PCISelectionTool::getClass()->getGUIDescriptor())->addClassEditor(
		PCISelectionToolEditor::getClass(), true);

	PCITransfEditEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )PCITransfEdit::getClass()->getGUIDescriptor())->addClassEditor(
		PCITransfEditEditor::getClass(), true);

	PCIFilterToolEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )PCIFilterTool::getClass()->getGUIDescriptor())->addClassEditor(
		PCIFilterToolEditor::getClass(), true);

	PCIMappingToolEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )PCIMappingTool::getClass()->getGUIDescriptor())->addClassEditor(
		PCIMappingToolEditor::getClass(), true);

	PCICameraManagerEditor::init(ClassEditor::getClass() );
	((GUIDescriptor * )PCICameraManager::getClass()->getGUIDescriptor())->addClassEditor(
		PCICameraManagerEditor::getClass(), true);


	PCCPointCloudCreatorEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )PCCPointCloudCreator::getClass()->getGUIDescriptor())->addClassEditor(
		PCCPointCloudCreatorEditor::getClass(), true);

	// Importer Editors
	PCCImporterEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )PCCImporter::getClass()->getGUIDescriptor())->addClassEditor(
		PCCImporterEditor::getClass(), true);

	PCCAnimationImporterEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )PCCAnimationImporter::getClass()->getGUIDescriptor())->addClassEditor(
		PCCAnimationImporterEditor::getClass(), true);

	PointCloudImporterReaderCustomASCIIEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )PointCloudImporterReaderCustomASCII::getClass()->getGUIDescriptor())->addClassEditor(
		PointCloudImporterReaderCustomASCIIEditor::getClass(), true);

#endif	// end of #ifdef GEOXL_GUI

}


void geoxl::shutdownEditor() {

	OnSceneChangeCallback::shutdown();

   // commmand objects
   CmdObjChangePCFlags::shutdown();
   CmdObjChangeSelection::shutdown();
   CmdObjChangeSelectionOnClick::shutdown();
   CmdObjChangeView::shutdown();
   CmdObjDeletePCs::shutdown();
   CmdObjDrawOnClick::shutdown();
   CmdObjFilter::shutdown();
   CmdObjLoadMap::shutdown();
   CmdObjMappingOnClick::shutdown();
   CmdObjPCCommand::shutdown();
   CmdObjPointCloudOperations::shutdown();
   CmdObjRenamePC::shutdown();
   CmdObjSelectionByAttribute::shutdown();
   CmdObjSelectionByButton::shutdown();
   CmdObjSelectionOnArea::shutdown();
   CmdObjSpecialDraw::shutdown();
   CmdObjTransfEdit::shutdown();
   CmdObjApplyRegisterTransform::shutdown();
   CmdObjGeneratePC::shutdown();
   CmdObjScriptProgram::shutdown();
	CmdObjSGChangeDefaultTraversalMode::shutdown();
   CmdObjAppendFields::shutdown();
   CmdObjConvertType::shutdown();
	CO_SGAddNode::shutdown();
   CO_SGDeleteNode::shutdown();
   CO_SGMoveNode::shutdown();
   CO_SGMoveSelection::shutdown();
   CO_SGGroupSelection::shutdown();
   CO_SGUnGroupSelection::shutdown();
   CO_SGChangeSelection::shutdown();

   // commands
   PCCommand::shutdown();
	PCCAttributeModifier::shutdown();
	PCCAutoAlign::shutdown();
	PCCComputeFlags::shutdown();
	PCCParametricResampling::shutdown();
	PCCPointCloudCreator::shutdown();
	PCCPointGenerator::shutdown();
	PCCTriangleMeshSampler::shutdown();
	PCCAnimatedTriangleMeshSampler::shutdown();
	//	PCCPointScript::shutdown();
	PCCImporter::shutdown();
	PCCAnimationImporter::shutdown();
	PCCSwapChannels::shutdown();
	PCCSetMaterial::shutdown();
	PCCImplicitMesher::shutdown();
	PCCResampler::shutdown();
	PCCPointNoise::shutdown();
	PCCEstimateSampleSpacing::shutdown();
	PCCDirectoryImporter::shutdown();
	PCCConvToSwapAllOOC::shutdown();
	PCCSortListNode::shutdown();
	PCCAnimRename::shutdown();
	PCCImportVolumeData::shutdown();
	PCCVolumeGenerator::shutdown();
	PCCNormalEstimatorPCA::shutdown();	


   // Importer
	PointCloudImporterAdder::shutdown();
	PointCloudImporterAdderParametricDynamic::shutdown();
	PointCloudImporterAdderDynamic::shutdown();
	PointCloudImporterAdderMesh::shutdown();
	PointCloudImporterReader::shutdown();
	PointCloudImporterReader2DImages::shutdown();
	PointCloudImporterReader2DImagesPlusAlpha::shutdown();
	PointCloudImporterReaderMSDepthBMP::shutdown();
	PointCloudImporterReaderCustomASCII::shutdown();
	PointCloudImporterReaderSMFOBJ::shutdown();
	PointCloudImporterReaderPLY::shutdown();
	PointCloudImporterReaderVRML::shutdown();
	PointCloudImporterReaderBSM::shutdown();
	PointCloudImporterReaderOFF::shutdown();
	PointCloudImporterReaderCustom3DScan::shutdown();
   PointCloudImporterReaderBinary::shutdown();


   // interactive tools
   PCInteractionTool::shutdown();
   PCICameraController::shutdown();
   PCIDrawingTool::shutdown();
   PCIFilterTool::shutdown();
   PCICameraManager::shutdown();
   PCIMappingTool::shutdown();
   PCISelectionTool::shutdown();
   PCIPointInfo::shutdown();
   PCITransfEdit::shutdown();
	PCIManualAlignment::shutdown();

   // geoxl events
   GEOXLEvent::shutdown();
   ModeChangeEvent::shutdown();
   SpecifiedModeChangeEvent::shutdown();
   CommandObjectEvent::shutdown();
   StringEvent::shutdown();

   // init geoxl class editors
   SceneInterface::shutdown();
   SceneToolbar::shutdown();
   SceneToolbarElement::shutdown();
   SceneToolButton::shutdown();
   SceneToolbarCheckBox::shutdown();
   ToolbarSplitter::shutdown();

	SceneEditorOptions::shutdown();

	//debug
	TestRenderer::shutdown();


// ---------------------------------------------------------------------------------
//					Editor Class Registration 
// ---------------------------------------------------------------------------------
#ifdef GEOXL_GUI

   // scene editors
   SceneClassEditor::shutdown();
   SceneGraphEditor::shutdown();
   PCSceneMaterialBrowser::shutdown();

   CmdObjScriptProgramEditor::shutdown();

	SceneEditor::shutdown();
	SceneEditorOptionsEditor::shutdown();

	// PCI Editors
	PCIDrawingToolEditor::shutdown();
	PCISelectionToolEditor::shutdown();
	PCITransfEditEditor::shutdown();
	PCIFilterToolEditor::shutdown();
	PCIMappingToolEditor::shutdown();
	PCICameraManagerEditor::shutdown();


	PCCPointCloudCreatorEditor::shutdown();

	// Importer Editors
	PCCImporterEditor::shutdown();
	PCCAnimationImporterEditor::shutdown();
	PointCloudImporterReaderCustomASCIIEditor::shutdown();

#endif	// end of #ifdef GEOXL_GUI

}