//---------------------------------------------------------------------------
#ifndef PointSetANNQueryH
#define PointSetANNQueryH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "ANN.h"
//---------------------------------------------------------------------------

namespace geoxl {
	using namespace std;

class EDITOR_API PointSetANNQuery
{
protected:
	ANNkd_tree*	kdTree;
	card32 numPoints;
	card32 k;

	float32 *distances;
	int32 *indexes;
	ANNpoint* data;

	card32 position;

	PointSetANNQuery(){}

public:
	PointSetANNQuery(const PointSet *ps, const card32 &k);
	~PointSetANNQuery(void);

	/// reset position in space where search starts
	void setTargetPoint(const Vector3f &targetPosition);
	/// get next nearest point. found is true if another point has been found. if found is false, nextIndex is undefined.
	void getNextNearestPointIndex(card32 &nextIndex, bool &found);
	/// returns array of indices
	int32 * getAllIndices(card32 &returnedNumOfIndices);
	/// get nearest Neighbor to given point
	int32 getNearestPointIndex(const Vector3f &targetPosition) const;
	/// get nearest Neighbor and Distance to given point
	int32 getNearestPointIndexAndSqrDistance(const Vector3f &targetPosition, float32 &returnedDistance);

};

class EDITOR_API PointSetRangeQuery : public PointSetANNQuery
{
private:
	float32 sqrRad;
	card32 lastK;
public:
	PointSetRangeQuery(const PointSet *ps, const float32& sqrRad);

	/// reset position in space where search starts
	void setTargetPoint(const Vector3f &targetPoint);
	/// reset position in space where search starts, sets range to tempSqrRad for this query ONLY
	void setTargetPointRadius(const Vector3f &targetPoint, const float32 &tempSqrRad);
	/// get next nearest point. found is true if another point has been found. if found is false, nextIndex is undefined.
	void getNextNearestPointIndex(card32 &nextIndex, bool &found);
	/// change search square radius
	void setRangeSqrRad(const float32 &sqrRad);
};

}

#endif
