//---------------------------------------------------------------------------
#ifndef CmdObjFilterH
#define CmdObjFilterH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "CommandObject.h"
#include "PointCloud.h"
#include "ViewFrustum.h"
#include "Camera.h"
#include "UnstructuredInCorePointCloud.h"
#include "IteratorAdaptors.h"
#include "AttachedMemoryManager.h"
#include "HierarchicalKNNIterator.h"
#include "Scene.h"
//---------------------------------------------------------------------------

namespace geoxl {


class EDITOR_API CmdObjFilter : public CommandObject {

 private:
   static MetaClass *CmdObjFilterMetaClass;
   static Object* newInstance();

   UnstructuredInCorePointCloud* tempCloud;
   PointCloudIterator* tmpIt;

   AAT ATTRIB;

   void averageFilter3f(IteratorAdaptor *it, float32 radius);
   void  medianFilter3f(IteratorAdaptor *it, float32 radius);
   void   gaussFilter3f(IteratorAdaptor *it, float32 radius);

   struct clickPoint {
      int32 x;
      int32 y;
   };

   void filterHightField(Scene *scene);
   std::vector<float32> hightField;

   void averageFilter1f(IteratorAdaptor *it);
   void  medianFilter1f(IteratorAdaptor *it);
   void   gaussFilter1f(IteratorAdaptor *it);

   void regenerateNormals(IteratorAdaptor *it, PointCloud * pc);

   HierarchicalKNNIterator* kNN_Iterator_TempCloud;
   bool use_kNN;
   card32 kNN_numPoints;

   bool use_Heightmap;
   Vector3f u,v,n;
   Matrix4f ctm;
   Matrix4f inv;
   Vector3f mean;

 protected:
   vector<clickPoint> clickList;

   int32 clickX, clickY;
   card32 tolerance, mode,channel, areaWidth, areaHeight;
   float32 strength;
   bool drawScene;

   Camera cam;
   ViewFrustum viewFrust;


 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   void filterScene(Scene *scene);
   virtual void setup(const int32 &x, const int32 &y, const card32 &tol, const card32 &mode, const card32 &channel,
                      Camera &ca, ViewFrustum &vfr, const card32 &w, const card32 &h, const float32 &str,
                      const bool &scene, const bool &use_kNN, const card32 &kNN_numPoints, const bool &use_Height);


   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();

   virtual bool mergeCommand(const CommandObject *nextCmd);

};


}//ns






#endif
