//---------------------------------------------------------------------------
#ifndef CmdObjMappingOnClickH
#define CmdObjMappingOnClickH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "CommandObject.h"
#include "PointCloud.h"
#include "ViewFrustum.h"
#include "Camera.h"
#include "PointCloud.h"
#include "ParametricInCorePointCloud.h"
#include "GeometricRange.h"
#include "IteratorAdaptors.h"
#include "LinearAlgebra.h"
#include "PCTexture.h"
//---------------------------------------------------------------------------

namespace geoxl {

class Scene;


class EDITOR_API CmdObjMappingOnClick : public CommandObject {

 private:
   static MetaClass *CmdObjMappingOnClickMetaClass;
   static Object* newInstance();

   struct clickPoint {
      int32 x;
      int32 y;
   };

   Vector3f mean,eigenVal;
   Matrix3f eigenVec;
   bool localValid;

 protected:
   vector<clickPoint> clickList;

   int32 clickX, clickY;
   card32 tolerance, selectMode, areaWidth, areaHeight;

   Camera cam;
   ViewFrustum viewFrust;
   Vector3f m_col;
   bool fade,col,pos,pnorm,raise,drag;
   float32 scale;
   string texName;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   virtual void setup(int32 x, int32 y, card32 tol, card32 mode, Camera &ca, ViewFrustum &vfr, card32 w, card32 h, string color, bool fade,bool col, bool pos, bool pnorm, bool raise , bool drag,float32 scale);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   /// Use SVD to find parametric coords
   bool makeLocalCoords(Scene* scene, GeometricRange* range, bool notMod ,card32 pcnumber,Vector3f &u,Vector3f &v,Vector3f &w);

   GeometricRange* getSelectionSphereRangeNotMod(Scene* scene);



   virtual string getDescription();

   virtual bool mergeCommand(const CommandObject *nextCmd);

};






class EDITOR_API CmdObjLoadMap : public CommandObject {

 private:
   static MetaClass *CmdObjLoadMapMetaClass;
   static Object* newInstance();


 protected:
   string fileName;


 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void setup(string file);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();

};







}//ns


#endif
