//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "CmdObjPointCloudOperations.h"
//---------------------------------------------------------------------------
#include "SceneGraphState.h"
#include "EmbeddedObjectClassProperty.h"
#include "MathTools.h"
#include "MathTools_templCode.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "SelectionTools.h"
#include "PointCloudTools.h"
#include "ObjectViewsTable.h"
#include "ProgressWindow.h"
#include "ParametricInCorePointCloud.h"
#include "HierarchicalInCorePointCloud.h"
#include "HierarchicalOutOfCorePointCloud.h"
#include "UnstructuredInCoreTriangleMesh.h"
//#include "LinearAlgebra.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* CmdObjPointCloudOperations::CmdObjPointCloudOperationsMetaClass = NULL;

const MetaClass* CmdObjPointCloudOperations::getInstanceClass() const {
   return getClass();
}

Object* CmdObjPointCloudOperations::newInstance() {
   return new CmdObjPointCloudOperations;
}

const MetaClass* CmdObjPointCloudOperations::getClass() {
   if (CmdObjPointCloudOperationsMetaClass==NULL) {
      CmdObjPointCloudOperationsMetaClass = new MetaClass("CmdObjPointCloudOperations", "no description.",
                                  newInstance, sizeof(CmdObjPointCloudOperations));
   }
   return CmdObjPointCloudOperationsMetaClass;
}

void CmdObjPointCloudOperations::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION +1);
   CmdObjPointCloudOperationsMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(CmdObjPointCloudOperations)
   ADD_CARD32_PROP(Operation ,0)
   ADD_STRING_PROP(ChannelName, 1);
}

void CmdObjPointCloudOperations::shutdown() {
   delete CmdObjPointCloudOperationsMetaClass;
}


void CmdObjPointCloudOperations::setup(card32 mode) {
   Operation = mode;
}

void CmdObjPointCloudOperations::setup(string remChannelName)
{
	Operation = OP_REMOVE_CHANNEL;
	ChannelName = remChannelName;

}


void CmdObjPointCloudOperations::doAction(Object *context) {
   Scene *scene = dynamic_cast<Scene*>(context); pAssert(scene);

   switch (Operation)
   {
      case OP_DELETE: deletePoints(scene); break;
      case OP_COPY: copyPointsToNewCloud(scene,OLDP_DESELECT); break;
      case OP_SPLIT: copyPointsToNewCloud(scene,OLDP_DELETE); break;
      case OP_JOIN: joinPointClouds(scene); break;
      case OP_COPY_TO_ONE: copyPointsToNewCloud(scene,OLDP_DESELECT); joinPointClouds(scene); break;
      case OP_SPLIT_TO_ONE: copyPointsToNewCloud(scene,OLDP_DELETE); joinPointClouds(scene); break;
	  case OP_REMOVE_CHANNEL: removeChannel(scene); break;
      default: break;
   };

   ObjectViewsTable::update(scene);
}


void CmdObjPointCloudOperations::undoAction(Object *context) {
   forget(context);
   throw ECannotUndo("CmdObjPointCloudOperations does not provide any undo.");
}

bool CmdObjPointCloudOperations::canUndo() {
   return false;
}


string CmdObjPointCloudOperations::getDescription() {
   string result;
   switch (Operation) {
      case OP_DELETE: result = "delete selected points"; break;
      case OP_COPY: result = "duplicate selected points to different new clouds"; break;
      case OP_SPLIT: result = "split selected points to different new clouds"; break;
      case OP_JOIN: result = "join selected clouds"; break;
      case OP_COPY_TO_ONE: result = "duplicate selected points to single new cloud"; break;
      case OP_SPLIT_TO_ONE: result = "split selected points to single new cloud"; break;
	  case OP_REMOVE_CHANNEL: result = "remove attribute channel from selected clouds"; break;
      default: break;
   };
   return result;
}


void CmdObjPointCloudOperations::deletePoints(Scene *scene) {

   // not nice but working. deleting PointClouds in "while (!sceneIterator->atEnd())..." causes node chaos and doesn't work
   vector<string> deleteClouds;
   deleteClouds.clear();

   StandardSceneGraphIterator* sceneIterator = new StandardSceneGraphIterator();
   sceneIterator->clearAndSetup(scene->getRootNode(), scene->getRootState());

   // init progress window
   progressWindow->pushStep(true, "delete selected points");
   progressWindow->progress(0);
   float32 numSelPoints = 1.0f;

   while (!sceneIterator->atEnd()) {

      // prepare PointCloud
      SGObjectNode* SGN = dynamic_cast<SGObjectNode*>(sceneIterator->getNode());
      if (SGN==NULL) { sceneIterator->next(); continue; }
      PointCloud* pc = dynamic_cast<PointCloud*>(SGN->getSceneObject());
      if (pc==NULL) { sceneIterator->next(); continue; }
      if (!pc->providesAttribute("flags")) { sceneIterator->next(); continue; }

		//special case UnstructuredInCoreTriangleMesh
		UnstructuredInCoreTriangleMesh* triMesh= dynamic_cast<UnstructuredInCoreTriangleMesh*>(pc);
		if (triMesh)
		{
			triMesh->deleteMarkedTriangles();
		}else{
			//special case ParametricInCorePointCloud
			ParametricInCorePointCloud* paramPC = dynamic_cast<ParametricInCorePointCloud*>(pc);

			if (paramPC)
			{
				paramPC->invalidateSelectedPoints();
			}else{

				if (!checkCloudTypeIsDynamic(pc)) { sceneIterator->next(); continue; }


				// prepare Iterators
				SceneObjectIterator *newItSO = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_DYNAMIC_PC | SOIT::CAP_HIERARCHICAL));
				if (newItSO==NULL) { sceneIterator->next(); continue; }
				PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
				if (newIt == NULL) { delete newItSO; sceneIterator->next(); continue; }
				DynamicPointCloudIterator *dynIt = dynamic_cast<DynamicPointCloudIterator*>(newIt);
				if (dynIt == NULL) { delete newItSO; sceneIterator->next(); continue; }

				IteratorAdaptor *sit = getSelectionIteratorAdaptor(pc, newIt);
				if (sit==NULL) { delete newItSO; sceneIterator->next(); continue; }

				// needed for percentage display in progress window
				HierarchicalIterator* hierIt = dynamic_cast<HierarchicalIterator*>(newIt);
				if (hierIt==NULL) { delete sit; delete newItSO; sceneIterator->next(); continue; }
				card64 tmpNSP = 0;
				bool ok = hierIt->getNodeInfo(HierarchicalIterator::niFlaggedInSubtree, &PF_FLAG_SELECTED, &tmpNSP);
				pAssert(ok);
				card64 counter = numSelPoints;
				numSelPoints += tmpNSP;
				float32 factor = 100.0f/numSelPoints;

				while(!sit->atEnd()) {
					// delete selected points in cloud
					dynIt->deleteCurrent();
					// sit->next() is in deleteCurrent()!

					// update progress window
					counter++;
					if (counter%1000==0)
						if (!progressWindow->progress(factor*counter)) { delete sit; delete newItSO; delete sceneIterator; return; };
				};
				delete sit;
				delete newIt;
			}

		}

      // if cloud empty, ask for delete
      if (pc->isEmpty()) {
         string cloudName = sceneIterator->getNameWithPath();
         string msg = "PointCloud " + cloudName + " is empty now!\nDo you want to delete it?";
			int tmp=QMessageBox::question(NULL, "Info...", msg.c_str(), "&Yes","&No", QString(), 0, 1);
         if (tmp==0) {
            // deselect deleted cloud
            set<string> selN = scene->getRootState()->staticState->selectedNodes;
            selN.erase(cloudName);
            scene->getRootState()->staticState->selectedNodes = selN;
            // delete this PointCloud at end, don't delete it here because of iterator mess
            deleteClouds.push_back(cloudName);
         };
      };

      sceneIterator->next();
   }; // end while sceneIterator
   delete sceneIterator;

   // finally delete clouds
   for (card32 i=0; i<deleteClouds.size(); i++) {
      deletePointCloud(scene, deleteClouds[i]);
   };

   // finish progress window
   bool dummy;
   progressWindow->popSteps(dummy);
}


void CmdObjPointCloudOperations::copyPointsToNewCloud(Scene *scene, card32 oldp) {

   set<string> selectClouds = scene->getRootState()->staticState->selectedNodes;
   selectClouds.clear();

   // not nice but working. adding/deleting PointClouds in "while (!sceneIterator->atEnd())..." causes node chaos and doesn't work
   vector<string> deleteClouds;
   deleteClouds.clear();
   vector<PointCloud*> addClouds;
   addClouds.clear();
   vector<string> addCloudsName;
   addCloudsName.clear();

   StandardSceneGraphIterator* sceneIterator = new StandardSceneGraphIterator();
   sceneIterator->clearAndSetup(scene->getRootNode(), scene->getRootState());

   // init progress window
   progressWindow->pushStep(true, "copy points to new cloud");
   progressWindow->progress(0);
   float32 numSelPoints = 1.0f;

   while (!sceneIterator->atEnd()) {

      // prepare input PC
      SGObjectNode* SGN = dynamic_cast<SGObjectNode*>(sceneIterator->getNode());
      if (SGN==NULL) { sceneIterator->next(); continue; }
      PointCloud* inputPC = dynamic_cast<PointCloud*>(SGN->getSceneObject());
      if (inputPC==NULL) { sceneIterator->next(); continue; }
      if (!inputPC->providesAttribute("flags")) { sceneIterator->next(); continue; }
      AAT FLAGS = inputPC->getAAT("flags");

      // prepare input iterators
      SceneObjectIterator *inItSO = inputPC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_DYNAMIC_PC | SOIT::CAP_HIERARCHICAL));
      if (inItSO==NULL) { sceneIterator->next(); continue; }
      PointCloudIterator *inIt = dynamic_cast<PointCloudIterator*>(inItSO);
      if (inIt==NULL) { delete inItSO; sceneIterator->next(); continue; }
      BasicPointCloudIterator *inBasicIt = dynamic_cast<BasicPointCloudIterator*>(inIt);
      ModifyablePointCloudIterator *inModIt = dynamic_cast<ModifyablePointCloudIterator*>(inIt);
      DynamicPointCloudIterator *inDynIt = dynamic_cast<DynamicPointCloudIterator*>(inIt);
      if ((inDynIt==NULL) || (inModIt==NULL) || (inBasicIt==NULL)) { delete inItSO; sceneIterator->next(); continue; }

      IteratorAdaptor *sit = getSelectionIteratorAdaptor(inputPC, inIt);
      if (sit==NULL) { delete inItSO; sceneIterator->next(); continue; }

      // needed for percentage display in progress window
      HierarchicalIterator* hierIt = dynamic_cast<HierarchicalIterator*>(inIt);
      if (hierIt == NULL) { delete sit; delete inItSO; sceneIterator->next(); continue; }
      card64 tmpNSP = 0;
      bool ok = hierIt->getNodeInfo(HierarchicalIterator::niFlaggedInSubtree, &PF_FLAG_SELECTED, &tmpNSP);
      pAssert(ok);
      card64 counter = numSelPoints;
      numSelPoints += (tmpNSP*2);
      float32 factor = 100.0f/numSelPoints;

      // prepare output PC
      PointCloud *outPC = NULL;

/*
// old method:
// not good! crashes with hierarchical clouds!
         outPC = (PointCloud*)inputPC->copy();
         outPC->clear();
*/


      // make new cloud of right type and with same settings
      switch (calcPCPriority(inputPC)) {

         case 4:
            pAssert(false);
            break;

            // todo: implement correct behavior for HierarchicalOutOfCorePointCloud

            /*
            HierarchicalOutOfCorePointCloud* HOOCPC = dynamic_cast<HierarchicalOutOfCorePointCloud*>(inputPC);
            if (HOOCPC==NULL) { delete inItSO; sceneIterator->next(); continue; }

            MultiObjectStream *mobj = HOOCPC->getOctree()->getAlloc()->getMultiObjectStream();
            if (dynamic_cast<FileBasedMultiObjectStream*>(mobj) == NULL) { delete inItSO; sceneIterator->next(); continue; };
            string swapFileName = dynamic_cast<FileBasedMultiObjectStream*>(mobj)->getFilename();
            AnsiString newFileName = ExtractFilePath(swapFileName) + "copied" + ExtractFileName(swapFileName);

            BlockedMultiObjectStream *multiObj = new BlockedMultiObjectStream(newFileName, blockSize);
            MHOutOfCorePointSetOctreeNodeBase::NodeAllocator *nAlloc
               = new MHOutOfCorePointSetOctreeNode::NodeAllocator(
               MHOutOfCorePointSetOctreeNode::getClass(),
               multiObj,
               maxNumNodesInMem
            );
            // mw - 10/18/07 - moved in constructor - multiObj->addGlobalObjectPointer(nAlloc, 1);

            outPC = new HierarchicalOutOfCorePointCloud();

            ((HierarchicalOutOfCorePointCloud*)outPC)->clearAndSetup(HOOCPC->getDescr(), nAlloc);
            MHStandardSettings settings;
            ((HierarchicalOutOfCorePointCloud*)inputPC)->getOctree()->getSettings(settings);
            MHStandardSettings settings2 = settings->copy();
            ((HierarchicalOutOfCorePointCloud*)outPC)->getOctree()->setSettings(settings2);

            break;
            */

         case 3: {
            outPC = new HierarchicalInCorePointCloud();
            ((HierarchicalInCorePointCloud*)outPC)->clearAndSetup(inputPC->getDescr());
            MHStandardSettings inSettings;
            ((HierarchicalInCorePointCloud*)inputPC)->getOctree()->getSettings(inSettings);
            MHStandardSettings outSettings;
            outSettings.maxNumObjectsPerLeaf = inSettings.maxNumObjectsPerLeaf;
            outSettings.multiResGridSize = inSettings.multiResGridSize;
            ((HierarchicalInCorePointCloud*)outPC)->getOctree()->setSettings(outSettings);
         };
         break;

         case 2: {
            outPC = new UnstructuredInCorePointCloud();
            ((UnstructuredInCorePointCloud*)outPC)->clearAndSetup(inputPC->getDescr(), 0);
            };
            break;

         default: {
            if (dynamic_cast<const ParametricInCorePointCloud*>(inputPC)) {
               outPC = new ParametricInCorePointCloud();
               ((ParametricInCorePointCloud*)outPC)->clearAndSetup(0, 0, inputPC->getDescr());
            } else {
               pAssert(false);
               // unknown cloud type! implement in calcPCPriority too, if necessary.
            }
            };
            break;

      }; // end switch calcPCPriority
      if (outPC == NULL) { delete sit; delete inItSO; sceneIterator->next(); continue; }
      outPC->assignSettings(inputPC);

      // prepare output iterators
      SceneObjectIterator *outItSO = outPC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_DYNAMIC_PC));
      if (outItSO==NULL) { delete sit; delete inItSO; delete outPC; sceneIterator->next(); continue; }
      PointCloudIterator *outIt = dynamic_cast<PointCloudIterator*>(outItSO);
      if (outIt==NULL) { delete sit; delete inItSO; delete outItSO; delete outPC; sceneIterator->next(); continue; }
      DynamicPointCloudIterator *outDynIt = dynamic_cast<DynamicPointCloudIterator*>(outIt);
      if (outDynIt == NULL) { delete sit; delete inItSO; delete outItSO; delete outPC; sceneIterator->next(); continue; }

      // add points to new cloud
      int32 pointcount=0;
      char *buffer = new char[inputPC->getDescr()->getSize()];
      while(!sit->atEnd()) {
         inBasicIt->getVertex(buffer);
         outDynIt->add(buffer);
         sit->next();
         pointcount++;

         // update progress window
         counter++;
         if (counter%1000==0)
            if (!progressWindow->progress(factor*counter)) { delete[] buffer; delete sit; delete inItSO; delete outItSO; delete outPC; delete sceneIterator; return; };
      };
      delete[] buffer;

      // manage old points, don't do this in above loop, it won't work cause of iterator mess
      sit->reset();
      while(!sit->atEnd()) {
         switch (oldp)
         {
            case OLDP_DESELECT: inModIt->set1i(FLAGS, inBasicIt->get1i(FLAGS) & (0xFFFFFFFF ^ PF_FLAG_SELECTED)); sit->next(); break;
            case OLDP_DELETE: inDynIt->deleteCurrent(); break;
            default: break;
         };

         // update progress window
         counter++;
         if (counter%1000==0)
            if (!progressWindow->progress(factor*counter)) { delete[] buffer; delete sit; delete inItSO; delete outItSO; delete outPC; delete sceneIterator; return; };
      };

      delete sit;
      delete inItSO;
      delete outItSO;

      if (pointcount!=0) {
         // if some points where copied from this cloud

         string newCloudName = sceneIterator->getNode()->getName() + "Copy";
         string newCloudNameWithPath = sceneIterator->getNameWithPath() + "Copy";

         if (inputPC->isEmpty()) {
            // if source cloud empty, ask for delete
				string oldCloudNameWithPath = sceneIterator->getNameWithPath();
            string msg = "PointCloud " + oldCloudNameWithPath + " is empty now!\nDo you want to delete it?";
				int tmp=QMessageBox::question(NULL, "Info...", msg.c_str(), "&Yes","&No", QString(), 0, 1);
	         if (tmp==0) {
               // delete old cloud, add new cloud
               deleteClouds.push_back(oldCloudNameWithPath);
               addClouds.push_back(outPC);
               addCloudsName.push_back(newCloudName);
            } else {
               // add new cloud
               addClouds.push_back(outPC);
               addCloudsName.push_back(newCloudName);
            };
         } else {
            // source cloud not empty, add new point cloud to scene
            addClouds.push_back(outPC);
            addCloudsName.push_back(newCloudName);
         };

      } else {
         // else: no points copied, delete empty outPC
         delete outPC;
      };

      sceneIterator->next();
   }; // end while sceneIterator
   delete sceneIterator;

   // delete clouds if necessary
   for (card32 i=0; i<deleteClouds.size(); i++) {
      deletePointCloud(scene, deleteClouds[i]);
   };

   // add copied clouds
   for (card32 i=0; i<addClouds.size(); i++) {
      string name = addPointCloudAndReturnNodeName(scene, addClouds[i], addCloudsName[i]);
      // new point clouds are selected after copy
      selectClouds.insert(scene->getRootNode()->getName() + "/" + name);
   };

   // finally set selection
   scene->getRootState()->staticState->selectedNodes = selectClouds;

   // finish progress window
   bool dummy;
   progressWindow->popSteps(dummy);
};


bool CmdObjPointCloudOperations::joinTwoClouds(PointCloud* basePC, PointCloud* addPC) {

   if ((basePC==NULL) || (addPC==NULL) || (basePC==addPC)) return false;
   if (!checkCloudTypeIsDynamic(basePC)) return false;
// todo: not equal but compatible!
   if ( !(basePC->getDescr()->equal(addPC->getDescr())) )
      throw PException("CmdObjPointCloudOperations::joinTwoClouds() - Descriptors not compatible. Join failed!");

   // output iterators
   SceneObjectIterator *newIt = basePC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_DYNAMIC_PC));
   if (newIt == NULL) return false;
   DynamicPointCloudIterator *outDynIt = dynamic_cast<DynamicPointCloudIterator*>(newIt);
   if (outDynIt == NULL) {delete newIt; return false;}

   // input iterators
   SceneObjectIterator *inIt = addPC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC));
   if (inIt == NULL) return false;
   BasicPointCloudIterator *inBasicIt = dynamic_cast<BasicPointCloudIterator*>(inIt);
   ModifyablePointCloudIterator *inModIt = dynamic_cast<ModifyablePointCloudIterator*>(inIt);
   if ((inBasicIt == NULL) || (inModIt == NULL)) {delete newIt; delete inIt; return false;}

   AAT POSITION = addPC->getAAT("position");
	AAT AAT_FLAGS;
   AAT NORMAL;
   bool noFlags = !addPC->providesAttribute("flags");
   if (!noFlags) AAT_FLAGS = addPC->getAAT("flags");
   bool hasnormal = addPC->providesAttribute("normal");
   if (hasnormal) NORMAL = addPC->getAAT("normal");

   // init progress window
   progressWindow->pushStep(true, "join two clouds");
   progressWindow->progress(0);
   card64 counter = 0;
   float32 factor = 100.0f/(((float32)addPC->getNumPoints())+1.0f);

   // transformation matrix
   Matrix4f trans = invertMatrix(basePC->getTransformation()) * addPC->getTransformation();
   Matrix3f transRot = shrink4To3(trans);
   char *buffer = new char[basePC->getDescr()->getSize()];

   // add points to base cloud
   while(!inBasicIt->atEnd()) {
      if (noFlags || !(inBasicIt->get1i(AAT_FLAGS) & PF_FLAG_INVALID)) {
         inModIt->set3f(POSITION, transformVector3f(trans, inBasicIt->get3f(POSITION)));
         inModIt->set3f(NORMAL, transRot*inBasicIt->get3f(NORMAL));
         inBasicIt->getVertex(buffer);
         outDynIt->add(buffer);
      };
      inBasicIt->next();

      // update progress window
      counter++;
      if (counter%1000==0)
         if (!progressWindow->progress(factor*counter)) { delete[] buffer; delete newIt; delete inIt; return false; };
   };

   delete[] buffer;
   delete newIt;
   delete inIt;

   // finish progress window
   bool dummy;
   progressWindow->popSteps(dummy);

   return true;
};


void CmdObjPointCloudOperations::joinPointClouds(Scene* scene) {

   vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
   if (selection.size()<2) return;

   vector<string> PCDeleteList;  // list of joined PointClouds which can be deleted at end
   string baseCloudSel = "";
   card32 basePriority = 0;
   PointCloud* tmpPC = NULL;

   // find highest priority cloud
   for (card32 m=0; m<selection.size(); m++ ){

      tmpPC = getPointCloud(scene, selection[m]);
      if (tmpPC==NULL) continue;

      card32 nextPriority = calcPCPriority(tmpPC);
      if (nextPriority > basePriority) {
         basePriority = nextPriority;
         baseCloudSel = selection[m];
      } else if (nextPriority == basePriority) {
         PointCloud* pc1 = getPointCloud(scene, baseCloudSel);
         if (pc1==NULL) throw PException("CmdObjPointCloudOperations::joinPointClouds() - node object is no point cloud");  // this may never happen
         card64 n1 = pc1->getNumPoints();
         card64 n2 = tmpPC->getNumPoints();
         if (n1<n2) baseCloudSel = selection[m];
      }
   };

   // get base cloud
   tmpPC = getPointCloud(scene, baseCloudSel);
   if (tmpPC==NULL) {
      throw PException("CmdObjPointCloudOperations::joinPointClouds() - Join failed, basePointCloud not found!");
   }

   // join every cloud to base cloud (tmpPC)
   for (card32 i=0; i<selection.size(); i++ ){
      if (selection[i]!=baseCloudSel) {
         if (joinTwoClouds(tmpPC, getPointCloud(scene, selection[i]))) {
            PCDeleteList.push_back(selection[i]);
         }
      }
   };

   // set new name for joined cloud
   SGObjectNode* SGONode = dynamic_cast<SGObjectNode*>(getSceneGraphNodeByName(scene, baseCloudSel));
   string path = baseCloudSel.substr(0, baseCloudSel.length() - SGONode->getName().length());
   SGONode->setName("joined point cloud");

   // select only joined cloud
   scene->getRootState()->staticState->selectedNodes.clear();
   scene->getRootState()->staticState->selectedNodes.insert(path + SGONode->getName());

   // delete added clouds
   for (card32 j=0; j<PCDeleteList.size(); j++) {
      deletePointCloud(scene, PCDeleteList[j]);
   }

};

void CmdObjPointCloudOperations::removeChannel(Scene *scene)
{
	
	
	set<string> selectedClouds = scene->getRootState()->staticState->selectedNodes;
	if (selectedClouds.empty())
		return;
	string name;
	set<string>::iterator pos = selectedClouds.begin();
    set<string>::iterator end = selectedClouds.end();
   
	StandardSceneGraphIterator* sceneIterator = new StandardSceneGraphIterator();
	sceneIterator->clearAndSetup(scene->getRootNode(),scene->getRootState());

	while (pos!=end)
	{
		name=*pos;
		++pos;
		while (!sceneIterator->atEnd())
		{
			int a=0;
			//check for selected nodes
			if (sceneIterator->getNameWithPath()!=name)
			{sceneIterator->next(); continue;}
			
			//prepare input PC if PC
			SGObjectNode* SGN = dynamic_cast<SGObjectNode*> (sceneIterator->getNode());
			if (SGN == NULL) break;
			PointCloud* inputPC = dynamic_cast<PointCloud*> (SGN->getSceneObject());
			if (inputPC == NULL) break;
			if (!inputPC->providesAttribute(ChannelName))
				break;
			
			SceneObjectIterator *inItSO=inputPC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC |
				SOIT::CAP_DYNAMIC_PC | SOIT::CAP_HIERARCHICAL));
			if (inItSO==NULL) break;
			PointCloudIterator* inIt=dynamic_cast<PointCloudIterator*> (inItSO);
			if (inIt==NULL) break;
			BasicPointCloudIterator* inBasicIt = dynamic_cast<BasicPointCloudIterator*> (inIt);
	        ModifyablePointCloudIterator *inModIt = dynamic_cast<ModifyablePointCloudIterator*>(inIt);
		    DynamicPointCloudIterator *inDynIt = dynamic_cast<DynamicPointCloudIterator*>(inIt);
			if ((inDynIt==NULL) || (inModIt==NULL) || (inBasicIt==NULL)) { delete inItSO; break; }
			

			//prepare output VD
			const VertexDescriptor* inVD=inputPC->getDescr();
			VertexDescriptor* outVD= new VertexDescriptor();
			for (card32 j=0;j<inVD->getNumAttribs();j++)
			{
				VAD vad1=inVD->getAttribute(j);
				if (vad1.name!= ChannelName)
				{
					outVD->pushAttrib(vad1);
				}
			}
			if (!outVD->getNumAttribs())
			{
				delete outVD;delete inItSO; break;
			}

			
			//prepare output PC
			PointCloud *outPC = NULL;

			switch (calcPCPriority(inputPC)){
				
				case 4:
					pAssert(false);
					break;
					//todo: implement correct behavior for Hier. OOC PC (see Copy)

				case 3:
					{
						outPC=new HierarchicalInCorePointCloud();
						((HierarchicalInCorePointCloud*)outPC)->clearAndSetup(outVD);
						MHStandardSettings inSettings;
						((HierarchicalInCorePointCloud*)inputPC)->getOctree()->getSettings(inSettings);
						MHStandardSettings outSettings;
						outSettings.maxNumObjectsPerLeaf=inSettings.maxNumObjectsPerLeaf;
						outSettings.multiResGridSize=inSettings.multiResGridSize;
						((HierarchicalInCorePointCloud*)outPC)->getOctree()->setSettings(outSettings);
					};
					break;
				case 2:
					{
						outPC=new UnstructuredInCorePointCloud();
						((UnstructuredInCorePointCloud*)outPC)->clearAndSetup(outVD,0);
						
					};
					break;

				default:
					if (dynamic_cast<const ParametricInCorePointCloud*> (inputPC)) {
						outPC=new ParametricInCorePointCloud();
						((ParametricInCorePointCloud*)outPC)->clearAndSetup(0,0, outVD);
						
						

					}else{
						pAssert(false);
						//unknown cloud Type. Implement in calcPriority too, if necessary.

					};
					break;
			}; //end switch

			if (outPC == NULL ) {delete inItSO; break;}
			outPC->assignSettings(inputPC);
			outPC->getAttachments()->assign(inputPC->getAttachments());
			

			SceneObjectIterator *outItSO = outPC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_DYNAMIC_PC));
			if (outItSO==NULL) { delete inItSO; delete outPC; break; }
			PointCloudIterator *outIt = dynamic_cast<PointCloudIterator*>(outItSO);
			if (outIt==NULL) { delete inItSO; delete outItSO; delete outPC; break; }
			DynamicPointCloudIterator *outDynIt = dynamic_cast<DynamicPointCloudIterator*>(outIt);
			if (outDynIt == NULL) { delete inItSO; delete outItSO; delete outPC; break; }
			
			//copy points to new cloud

			card8 *iBuffer=new card8[inputPC->getDescr()->getSize()];
			card8 *oBuffer=new card8[outPC->getDescr()->getSize()];
			
			while (!inBasicIt->atEnd())
			{
				inBasicIt->getVertex((char*)iBuffer);
				for (card32 j=0; j<inputPC->getDescr()->getNumAttributes(); j++)
				{
					
					if (inputPC->getDescr()->getAttribute(j).name == ChannelName)
					continue;
					card32 size=inputPC->getDescr()->getAttribute(j).getSize();
					AAT aat1= inputPC->getAAT(inputPC->getDescr()->getAttribute(j).name);
					AAT aat2= outPC->getAAT(inputPC->getDescr()->getAttribute(j).name);
					card8 *buffer=new card8[size];
					
					for (card32 n=0; n<size; n++)
					{
						buffer[n]= (iBuffer+aat1.getOffset())[n];
					}
					for (card32 n=0; n<size; n++)
					{
						(oBuffer+aat2.getOffset())[n]=buffer[n];
					}
					delete[] buffer;
					
				}
				outDynIt->add((char*)oBuffer);
				
				inBasicIt->next();
			}
			delete[] iBuffer;
			delete[] oBuffer;
			delete inItSO;
			delete outItSO;
			

			//setup 
			
			sceneIterator->getNode()->setName("delete_me_now");
			addPointCloud(scene, outPC,name );
			deletePointCloud(scene,sceneIterator->getNameWithPath());
			break;
		}
		sceneIterator->clearAndSetup(scene->getRootNode(),scene->getRootState());
	}

	delete sceneIterator;


}






card32 CmdObjPointCloudOperations::calcPCPriority(const PointCloud *pc) {
   card32 prio = 1;
   if (dynamic_cast<const UnstructuredInCorePointCloud *>(pc)) prio = 2;
   if (dynamic_cast<const HierarchicalInCorePointCloud *>(pc)) prio = 3;
   if (dynamic_cast<const HierarchicalOutOfCorePointCloud *>(pc)) prio = 4;
   return prio;
};

