//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "pAssert.h"
#include "CmdObjScriptProgram.h"
#include "StringHelper.h"
#include "Scene.h"
#include "PointCloud.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "SceneGraphNode.h"
#include "SGListNode.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


MetaClass* CmdObjScriptProgram::CmdObjScriptProgramMetaClass = NULL;

/****************************************************************************
 * CLASS CmdObjScriptProgram
 ****************************************************************************/
CmdObjScriptProgram::CmdObjScriptProgram()
{
  initVars();
}

void CmdObjScriptProgram::initVars()
{
  scriptEngine.setCmdObjScript(this);
}

const MetaClass* CmdObjScriptProgram::getInstanceClass() const {
   return getClass();
}

Object* CmdObjScriptProgram::newInstance() {
   return new CmdObjScriptProgram;
}

const MetaClass* CmdObjScriptProgram::getClass() {
   if (CmdObjScriptProgramMetaClass == NULL) {
      CmdObjScriptProgramMetaClass = new MetaClass("CmdObjScriptProgram", "no description.",
                                       newInstance, sizeof(CmdObjScriptProgram));
   }
   return CmdObjScriptProgramMetaClass;
}

void CmdObjScriptProgram::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION+1);
   CmdObjScriptProgramMetaClass->copyParentProperties();

   //TODO
   //CmdObjScriptProgramMetaClass->getGUIDescriptor()->addClassEditor(CmdObjScriptProgramEditor::getClass(), true);
}

void CmdObjScriptProgram::shutdown() {
   delete CmdObjScriptProgramMetaClass;
}

void CmdObjScriptProgram::doAction(Object *context)
{
  Scene *scene = dynamic_cast<Scene*>(context); pAssert(scene);
  scriptEngine.setScene(scene);
  scriptEngine.execute(false);
}

void CmdObjScriptProgram::undoAction(Object *context)
{
  forget(context);
}

bool CmdObjScriptProgram::canUndo()
{
  return false;
}

std::string CmdObjScriptProgram::getDescription()
{
  return "Script program";
}

void CmdObjScriptProgram::assign(const Object* obj, CopyContext *context)
{
  pAssert(obj->getInstanceClass() == getInstanceClass());

  CmdObjScriptProgram* other = (CmdObjScriptProgram*)obj;

  cmdObjects.assign(&(other->cmdObjects), context);
  scriptEngine.assign(&(other->scriptEngine), context);
  initVars();
}

void CmdObjScriptProgram::addCommand(const CommandObject* command)
{
  cmdObjects.addObject(command->copy());
}

void CmdObjScriptProgram::write(OutputObjectStream *s) const
{
  s->writeObject(&cmdObjects);
  s->writeObject(&scriptEngine);
}

void CmdObjScriptProgram::read(InputObjectStream *s)
{
  Persistent* obj;
  if (s->getClassVersion(getInstanceClass()) < MetaClass::FIRST_VERSION+1) {
    // downward compatibility
    cmdObjects.clear();
    card32 numCommands;
    s->read(numCommands);

    for (card32 i = 0; i != numCommands; ++i) {
      Persistent* obj;
      s->readObject(obj);
      string name;
      s->readString(name);
      cmdObjects.addObject(obj, name, true);
    }
  } else {
    // latest file version
    s->readObject(obj);
    ObjectDictionary* dictionary = (ObjectDictionary*)obj;
    cmdObjects.assign(dictionary);
    delete obj;
  }

  s->readObject(obj);
  scriptEngine.assign(obj);
  delete obj;
}

bool CmdObjScriptProgram::getCmdObj(const std::string name, CommandObject*& cmdObj) const
{
  Object* obj;
  if (!cmdObjects.getObjectSafe(name, obj))
    return false;

  cmdObj = dynamic_cast<CommandObject*>(obj);
  if (cmdObj != NULL) return true;
    else return false;
}

CmdObjScriptProgram::~CmdObjScriptProgram()
{
}

/****************************************************************************
 * CLASS PointCloudObjSymTab
 ****************************************************************************/
bool PointCloudObjSymTab::getValue(const std::string name, Object* &value)
{
  if (name.size() == 0) return false;

  if (name == "@scene") {
    value = cmdScript->getScene();
    return true;
  }

  return false;
}


/****************************************************************************
 * CLASS CmdObjScriptEngine
 ****************************************************************************/
CmdObjScriptEngine::CmdObjScriptEngine()
{
  ScriptEngine::ScriptEngine();
  cmdObjScriptProg = NULL;
  initVars();
}

void CmdObjScriptEngine::initVars()
{
  ScriptEngine::initVars();

  scene = NULL;

  executeCommand.setCmdScriptEngine(this);
  scriptCommands.registerCommand(&executeCommand, this);
  pCloudCommand.setCmdScriptEngine(this);
  scriptCommands.registerCommand(&pCloudCommand, this);

  if (cmdObjScriptProg != NULL) 
    dictionarySymbols.init(getCmdObjScriptProg()->getDictionary());
    
  pointCloudSymbols.init(this);
  objectTables.addTable(&dictionarySymbols);
  objectTables.addTable(&pointCloudSymbols);
}

void CmdObjScriptEngine::setCmdObjScript(CmdObjScriptProgram* cmdObjScriptProg)
{
  this->cmdObjScriptProg = cmdObjScriptProg;
  initVars();
}

/****************************************************************************
 * CLASS ExecuteCommand
 ****************************************************************************/
void ExecuteCommand::execute(std::string parameters, bool syntaxCheck)
{
  if (!syntaxCheck && !script->isActive()) return;

  extractToChar(parameters, ' ');

  if (parameters == "")
    throw EParserException("EXECUTE: No parameters");

  CommandObject* cmdObj;
  if (!cmdScript->getCmdObjScriptProg()->getCmdObj(parameters, cmdObj)) {
    throw EParserException("Execute: Invalid command object name (" + parameters +")");
  }

  if (!syntaxCheck)
    cmdObj->doAction(cmdScript->getScene());
}


/****************************************************************************
 * CLASS PCloudCommand
 ****************************************************************************/
void PCloudCommand::execute(std::string parameters, bool syntaxCheck)
{
  if (!syntaxCheck && !script->isActive()) return;

  extractToChar(parameters, ' ');
  if (parameters == "")
    throw EParserException("PCLOUD: No parameters");

  string subCommand = strLCase(extractToChar(parameters, ' '));

  if (subCommand == "number") {
    if (!syntaxCheck) {
      SGLegacyPCSceneAdaptor sceneLA(cmdScript->getScene());
      card32 num = sceneLA.getNumPointClouds();
      cmdScript->getNumTables()->setValue(parameters, num);
    }
    return;
  }

  if (subCommand == "select") {
    if (!syntaxCheck) {
      if (strLCase(parameters) == "all") {
        vector<card32> selVec;
        SGLegacyPCSceneAdaptor sceneLA(cmdScript->getScene());
        card32 numClouds = sceneLA.getNumPointClouds();
        for (card32 i = 0; i < numClouds; ++i)
          selVec.push_back(i);
        sceneLA.setSelectedPCs(selVec);
        return;

      } else {

        vector<card32> selVec;
        while (parameters != "") {
          string pcNumStr = extractToChar(parameters, ',');
          card32 num = cmdScript->getExpParser()->evaluate(pcNumStr);
          selVec.push_back(num);
        }
        SGLegacyPCSceneAdaptor sceneLA(cmdScript->getScene());
        sceneLA.setSelectedPCs(selVec);
      }
    }
    return;
  }

  if (subCommand == "delete") {
    if (!syntaxCheck) {
      if (strLCase(parameters) == "all") {
         SGListNode *root = dynamic_cast<SGListNode*>(cmdScript->getScene()->getRootNode());
         if (root) {
            while (root->getNumChildNodes(NULL) > 0) {
               root->removeNode(0);
            }
         }
      } else {
         vector<card32> selVec;
         while (parameters != "") {
           string pcNumStr = extractToChar(parameters, ',');
           card32 num = cmdScript->getExpParser()->evaluate(pcNumStr);
           selVec.push_back(num);
         }
         SGLegacyPCSceneAdaptor sceneLA(cmdScript->getScene());
         sceneLA.deletePointClouds(selVec);
      }
    }
    return;
  }

  if (subCommand == "getname") {
    string cloudStr = trimStr(extractToChar(parameters, ','));
    string targetStr = trimStr(parameters);

    card32 cloudNum = cmdScript->getExpParser()->evaluate(cloudStr);
    if (!syntaxCheck) {
      SGLegacyPCSceneAdaptor sceneLA(cmdScript->getScene());
      if (!script->getStringTables()->setValue(targetStr, sceneLA.getNode(cloudNum)->getName()))
        throw EParserException("PCLOUD GETNAME: Could not set " + targetStr);
    }
    return;
  }

  if (subCommand == "setname") {
    string cloudStr = trimStr(extractToChar(parameters, ','));
    string targetStrExp = trimStr(parameters);

    card32 cloudNum = cmdScript->getExpParser()->evaluate(cloudStr);
    string targetStr;
    if (!script->getStringTables()->evaluate(targetStrExp, targetStr))
      throw("PCLOUD SETNAME: Invalid string expression " + targetStrExp);

    if (!syntaxCheck) {
      SGLegacyPCSceneAdaptor sceneLA(cmdScript->getScene());
      sceneLA.getNode(cloudNum)->setName(targetStr);
    }
    return;
  }

  throw EParserException("PCLOUD " + strUCase(subCommand) + " is not a valid command.");
}
