//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "CmdObjSelection.h"
//---------------------------------------------------------------------------
#include "SceneGraphState.h"
#include "EmbeddedObjectClassProperty.h"

#include "MathTools.h"
#include "MathTools_templCode.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "SelectionTools.h"
#include "PointCloudTools.h"
#include "ProgressWindow.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------




// --------------- CmdObjChangeSelectionOnClick ---------------


MetaClass* CmdObjChangeSelectionOnClick::CmdObjChangeSelectionOnClickMetaClass = NULL;

const MetaClass* CmdObjChangeSelectionOnClick::getInstanceClass() const {
   return getClass();
}

Object* CmdObjChangeSelectionOnClick::newInstance() {
   return new CmdObjChangeSelectionOnClick;
}

const MetaClass* CmdObjChangeSelectionOnClick::getClass() {
   if (CmdObjChangeSelectionOnClickMetaClass==NULL) {
      CmdObjChangeSelectionOnClickMetaClass = new MetaClass("CmdObjChangeSelectionOnClick", "no description.",
                                  newInstance, sizeof(CmdObjChangeSelectionOnClick));
   }
   return CmdObjChangeSelectionOnClickMetaClass;
}

void CmdObjChangeSelectionOnClick::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   CmdObjChangeSelectionOnClickMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( CmdObjChangeSelectionOnClick )
   ADD_BOOLEAN_PROP(multiselect, 0)
   ADD_BOOLEAN_PROP(surfaceMode, 0)
	ADD_CARD32_PROP(areaWidth, 0)
	ADD_CARD32_PROP(areaHeight, 0)
	ADD_CARD32_PROP(selectMode, 0)
	ADD_CARD32_PROP(brushSize, 0)
	ADD_INT32_PROP(clickX, 0)
	ADD_INT32_PROP(clickY, 0)
	ADD_EMBEDDED_OBJECT_PROP(cam, 0, Camera::getClass());
	ADD_EMBEDDED_OBJECT_PROP(viewFrust, 0, ViewFrustum::getClass());

// *** clickList handled separately in assign/write/read

}

void CmdObjChangeSelectionOnClick::shutdown() {
   delete CmdObjChangeSelectionOnClickMetaClass;
}

void CmdObjChangeSelectionOnClick::assign(const Object* obj, CopyContext *context) {
   CommandObject::assign(obj, context);
   const CmdObjChangeSelectionOnClick *tobj = dynamic_cast<const CmdObjChangeSelectionOnClick*>(obj);
   pAssert(tobj != NULL);
   clickList = tobj->clickList;
}

void CmdObjChangeSelectionOnClick::write(OutputObjectStream *s) const {
   CommandObject::write(s);
   card32 n = (card32)clickList.size();
   s->write(n);
   for (card32 i=0; i<n; i++) {
      s->write(clickList[i].x);
      s->write(clickList[i].y);
   }
}

void CmdObjChangeSelectionOnClick::read(InputObjectStream *s) {
   CommandObject::read(s);
   card32 n;
   s->read(n);
   clickList.resize(n);
   for (card32 i=0; i<n; i++) {
      s->read(clickList[i].x);
      s->read(clickList[i].y);
   }
}


void CmdObjChangeSelectionOnClick::setup(int32 x, int32 y, bool multsel, card32 tol, card32 mode, Camera &ca, ViewFrustum &vfr, card32 w, card32 h, bool sphsel)
{
  clickX = x;
  clickY = y;

  clickPoint p;
  p.x = clickX;
  p.y = clickY;
  clickList.push_back(p);

  multiselect = multsel;
  brushSize = tol;
  selectMode = mode;
  cam = ca;
  viewFrust = vfr;
  areaWidth = w;
  areaHeight = h;

  surfaceMode = sphsel;
};


void CmdObjChangeSelectionOnClick::doAction(Object *context) {
   Scene *scene = dynamic_cast<Scene*>(context); pAssert(scene);

   for (card32 i=0; i<clickList.size(); i++)
   {
      clickX = clickList[i].x;
      clickY = clickList[i].y;

      switch (selectMode)
      {
         case SELMODE_SINGLECLOUD: selectCloud(scene); break;
         case SELMODE_SINGLEPOINT: selectPoint(scene); break;
         case SELMODE_BRUSHCIRCLE: selectPointsBrush(scene); break;
         case SELMODE_BRUSHQUAD: selectPointsBrush(scene); break;
         default: break;
      };
   };
}


void CmdObjChangeSelectionOnClick::undoAction(Object *context) {
   forget(context);
   throw ECannotUndo("CmdObjChangeSelectionOnClick does not provide any undo.");
}

bool CmdObjChangeSelectionOnClick::canUndo() {
   return false;
}


string CmdObjChangeSelectionOnClick::getDescription() {
   string result;
   if ((selectMode!=SELMODE_SINGLECLOUD) && multiselect) result = "de";
   result += "select ";
   switch (selectMode)
   {
      case SELMODE_SINGLECLOUD: result += "Cloud:"; break;
      case SELMODE_SINGLEPOINT: result += "Point:"; break;
      case SELMODE_BRUSHCIRCLE: result += "CircleBrush:"; break;
      case SELMODE_BRUSHQUAD: result += "QuadBrush:"; break;
      default: result += ":"; break;
   };
   for (card32 i=0; i<clickList.size(); i++) {
      result += " (";
      char buff[1000];
      string sX = itoa(clickList[i].x,buff,10);
      result += sX;
      result += ",";
      string sY = itoa(clickList[i].y,buff,10);
      result += sY;
      result += ")";
   }
   return result;
}


void CmdObjChangeSelectionOnClick::selectCloud(Scene *scene) {

   Vector3f retPointPos = NULL_VECTOR3F;
   string retCloudName = "";

   // find cloud which was clicked on
   if (!getClosestPointPositionFromAllPoints(clickX, clickY, areaWidth, areaHeight, 5, &cam, &viewFrust, scene, retPointPos, retCloudName)) return;

   if (!multiselect) {
      // select only single cloud
      scene->getRootState()->staticState->selectedNodes.clear();
      scene->getRootState()->staticState->selectedNodes.insert(retCloudName);
   } else {
      // search for clicked cloud in selected clouds
      // if found deselect cloud, if not found select cloud
      if (testPointCloudIsSelected(scene, retCloudName)) scene->getRootState()->staticState->selectedNodes.erase(retCloudName);
         else scene->getRootState()->staticState->selectedNodes.insert(retCloudName);
   }

}


void CmdObjChangeSelectionOnClick::selectPoint(Scene* scene) {

   // find point which was clicked on
   string CloudName = "";
   PointCloudIterator *returnedIt = getClosestPointIteratorFromAllPoints(clickX, clickY, areaWidth, areaHeight, 5, &cam, &viewFrust, scene, CloudName);
   if (returnedIt==NULL) return;

   PointCloud* PC = getPointCloud(scene, CloudName);
   if ((PC==NULL)||(!PC->providesAttribute("flags"))) { delete returnedIt; return; }
   AAT FLAGS = PC->getAAT("flags");

   BasicPointCloudIterator *baseIt = dynamic_cast<BasicPointCloudIterator*>(returnedIt);
   ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(returnedIt);
   if ((modIt==NULL) || (baseIt==NULL)) { delete returnedIt; return; }

   // select or deselect point
   if (multiselect) modIt->set1i(FLAGS, baseIt->get1i(FLAGS) & (0xFFFFFFFF ^ PF_FLAG_SELECTED));
      else modIt->set1i(FLAGS, baseIt->get1i(FLAGS) | PF_FLAG_SELECTED);

   delete returnedIt;
}


void CmdObjChangeSelectionOnClick::selectPointsBrush(Scene* scene) {

   // geometric range which defines selection region in 3D
   GeometricRange* range = NULL;

   if (surfaceMode) { // if surface mode
      switch (selectMode)
      {
         case SELMODE_BRUSHCIRCLE: range = getNearestSphereRangeFromMouseClick(clickX,clickY,areaWidth,areaHeight,brushSize,&cam,&viewFrust,scene); break;
         case SELMODE_BRUSHQUAD: range = getNearestClippedRayPyramidalRangeFromMouseClick(clickX-brushSize,clickY-brushSize,clickX+brushSize,clickY+brushSize,areaWidth,areaHeight,&cam,&viewFrust,scene); break;
         default: break;
      };
   } else { // if not surface mode
      switch (selectMode)
      {
         case SELMODE_BRUSHCIRCLE: range = getRayConeRangeFromMouseClick(clickX,clickY,areaWidth,areaHeight,brushSize,&cam,&viewFrust); break;
         case SELMODE_BRUSHQUAD: range = getRayPyramidalRangeFromMouseClick(clickX-brushSize,clickY-brushSize,clickX+brushSize,clickY+brushSize,areaWidth,areaHeight,&cam,&viewFrust); break;
         default: break;
      };
   };

   if (range==NULL) return;

   StandardSceneGraphIterator* sceneIterator = new StandardSceneGraphIterator();
   sceneIterator->clearAndSetup(scene->getRootNode(), scene->getRootState());

   while (!sceneIterator->atEnd()) {
      // iterate through visible point clouds only
      SGObjectNode* SGN = dynamic_cast<SGObjectNode*>(sceneIterator->getNode());
      if ((SGN==NULL) || !SGN->visible()) { sceneIterator->next(); continue; }
      PointCloud* PC = dynamic_cast<PointCloud*>(SGN->getSceneObject());
      if ((PC==NULL) || (!PC->providesAttribute("flags"))) { sceneIterator->next(); continue; }
      AAT FLAGS = PC->getAAT("flags");

      SceneObjectIterator *newItSO = PC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL));
      PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
      if ((newIt==NULL)||(newItSO==NULL)) { delete newItSO; sceneIterator->next(); continue; };
      ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(newIt);
      if (modIt==NULL) { delete newItSO; sceneIterator->next(); continue; }

      // cloud transformation matrix and geometric range iterator adpator
      range->setTransformation(sceneIterator->getState()->dynamicState->ctm);
      IteratorAdaptor *rangeIt = getGeoRangeIteratorAdaptor(PC, newIt, range);
      if (rangeIt==NULL) { delete newItSO; sceneIterator->next(); continue; };

      // set point selection flag
      while (!rangeIt->atEnd() )
      {
         if (multiselect) rangeIt->set1i(FLAGS, rangeIt->get1i(FLAGS) & (0xFFFFFFFF ^ PF_FLAG_SELECTED));
            else rangeIt->set1i(FLAGS, rangeIt->get1i(FLAGS) | PF_FLAG_SELECTED);
         rangeIt->next() ;
      }

      delete rangeIt;
      delete newItSO;
      sceneIterator->next();
   }; // end while sceneIterator

   delete sceneIterator;
   delete range;
}


bool CmdObjChangeSelectionOnClick::mergeCommand(const CommandObject *nextCmd) {
   if (nextCmd->getInstanceClass() == CmdObjChangeSelectionOnClick::getClass()) {
      const CmdObjChangeSelectionOnClick *nextCmdMyType = dynamic_cast<const CmdObjChangeSelectionOnClick*>(nextCmd);
      pAssert(nextCmdMyType != NULL);

      if ((this->multiselect==nextCmdMyType->multiselect) &&
         (this->brushSize==nextCmdMyType->brushSize) &&
         (this->selectMode==nextCmdMyType->selectMode) &&
         (this->cam==nextCmdMyType->cam) &&
         (this->viewFrust==nextCmdMyType->viewFrust) &&
         (this->areaWidth==nextCmdMyType->areaWidth) &&
         (this->areaHeight==nextCmdMyType->areaHeight) &&
         (this->surfaceMode==nextCmdMyType->surfaceMode))
         {
            clickPoint p;
            p.x = nextCmdMyType->clickX;
            p.y = nextCmdMyType->clickY;
            this->clickList.push_back(p);
            return true;
         }
       else return false;

   } else {
      return false;
   }
}





// --------------- CmdObjSelectionByButton ---------------


MetaClass* CmdObjSelectionByButton::CmdObjSelectionByButtonMetaClass = NULL;

const MetaClass* CmdObjSelectionByButton::getInstanceClass() const {
   return getClass();
}

Object* CmdObjSelectionByButton::newInstance() {
   return new CmdObjSelectionByButton;
}

const MetaClass* CmdObjSelectionByButton::getClass() {
   if (CmdObjSelectionByButtonMetaClass==NULL) {
      CmdObjSelectionByButtonMetaClass = new MetaClass("CmdObjSelectionByButton", "no description.",
                                  newInstance, sizeof(CmdObjSelectionByButton));
   }
   return CmdObjSelectionByButtonMetaClass;
}

void CmdObjSelectionByButton::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   CmdObjSelectionByButtonMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(CmdObjSelectionByButton)
   ADD_CARD32_PROP(button, 0)
}

void CmdObjSelectionByButton::shutdown() {
   delete CmdObjSelectionByButtonMetaClass;
}


void CmdObjSelectionByButton::setup(card32 btn) {
   button = btn;
}


void CmdObjSelectionByButton::doAction(Object *context) {
   Scene *scene = dynamic_cast<Scene*>(context); pAssert(scene);

   if (button==BUTTON_NOCLOUDS) {
      // deselect all nodes
      scene->getRootState()->staticState->selectedNodes.clear();

   } else if (button==BUTTON_ALLCLOUDS) {
      // select all clouds
      StandardSceneGraphIterator* sceneIterator = new StandardSceneGraphIterator();
      sceneIterator->clearAndSetup(scene->getRootNode(), scene->getRootState());
      while (!sceneIterator->atEnd()) {
         // iterate through all point clouds
         SGObjectNode* SGN = dynamic_cast<SGObjectNode*>(sceneIterator->getNode());
         if (SGN==NULL) { sceneIterator->next(); continue; }
         PointCloud* PC = dynamic_cast<PointCloud*>(SGN->getSceneObject());
         if (PC==NULL) { sceneIterator->next(); continue; }
         scene->getRootState()->staticState->selectedNodes.insert(sceneIterator->getNameWithPath());
         sceneIterator->next();
      }; // end while sceneIterator
      delete sceneIterator;

   } else if (button==BUTTON_NOPOINTS) {
      // clear point selection via hierarchy
      StandardSceneGraphIterator* sceneIterator = new StandardSceneGraphIterator();
      sceneIterator->clearAndSetup(scene->getRootNode(), scene->getRootState());

      // init progress window
      progressWindow->pushStep(true, "deselect all points");
      progressWindow->progress(0);
      float32 numSelPoints = 1.0f;

      while (!sceneIterator->atEnd()) {
         // iterate through all point clouds
         SGObjectNode* SGN = dynamic_cast<SGObjectNode*>(sceneIterator->getNode());
         if (SGN==NULL) { sceneIterator->next(); continue; }
         PointCloud* PC = dynamic_cast<PointCloud*>(SGN->getSceneObject());
         if ((PC==NULL) || (!PC->providesAttribute("flags"))) { sceneIterator->next(); continue; }
         AAT FLAGS = PC->getAAT("flags");

         SceneObjectIterator *newItSO = PC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL));
         if (newItSO==NULL) { sceneIterator->next(); continue; }
         PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
         if (newIt==NULL) { delete newItSO; continue; };
         ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(newIt);
         if (modIt==NULL) { delete newItSO; sceneIterator->next(); continue; }

         IteratorAdaptor *sit = getSelectionIteratorAdaptor(PC, newIt);
         if (sit==NULL) { delete newItSO; sceneIterator->next(); continue; }

         // needed for percentage display in progress window
         HierarchicalIterator* hierIt = dynamic_cast<HierarchicalIterator*>(newIt);
         if (hierIt==NULL) { delete sit; delete newItSO; sceneIterator->next(); continue; }
         card64 tmpNSP = 0;
         bool ok = hierIt->getNodeInfo(HierarchicalIterator::niFlaggedInSubtree, &PF_FLAG_SELECTED, &tmpNSP);
         pAssert(ok);
         card64 counter = numSelPoints;
         numSelPoints += tmpNSP;
         float32 factor = 100.0f/numSelPoints;

         while(!sit->atEnd()) {
            // clear point selection
            sit->set1i(FLAGS, sit->get1i(FLAGS) & (0xFFFFFFFF ^ PF_FLAG_SELECTED));
            sit->next();
            
            // update progress window
            counter++;
            if (counter%1000==0)
               if (!progressWindow->progress(factor*counter)) { delete sit; delete newItSO; delete sceneIterator; return; };
         };

         delete sit;
         delete newItSO;
         sceneIterator->next();
      }; // end while sceneIterator
      delete sceneIterator;

      // finish progress window
      bool dummy;
      progressWindow->popSteps(dummy);

   } else if ((button==BUTTON_ALLPOINTS)||(button==BUTTON_INVERTPOINTS)) {
      // point operations
      StandardSceneGraphIterator* sceneIterator = new StandardSceneGraphIterator();
      sceneIterator->clearAndSetup(scene->getRootNode(), scene->getRootState());

      // init progress window
      string message;
      switch (button) {
         case BUTTON_INVERTPOINTS: message = "invert"; break;
         case BUTTON_ALLPOINTS: message = "select"; break;
         default: message="error"; break;
      };
      message += " all points";
      progressWindow->pushStep(true, message);
      progressWindow->progress(0);
      float32 numPoints = 1.0f;

      while (!sceneIterator->atEnd()) {
         // iterate through all point clouds
         SGObjectNode* SGN = dynamic_cast<SGObjectNode*>(sceneIterator->getNode());
         if (SGN==NULL) { sceneIterator->next(); continue; }
         PointCloud* PC = dynamic_cast<PointCloud*>(SGN->getSceneObject());
         if ((PC==NULL) || (!PC->providesAttribute("flags"))) { sceneIterator->next(); continue; }
         AAT FLAGS = PC->getAAT("flags");

         SceneObjectIterator *newIt = PC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC));
         if (newIt == NULL) { sceneIterator->next(); continue; }
         BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
         ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(newIt);
         if ((modIt == NULL)||(basicIt == NULL)) { delete newIt; sceneIterator->next(); continue ;}

         // needed for percentage display in progress window
         card64 counter = numPoints;
         numPoints += PC->getNumPoints();
         float32 factor = 100.0f / numPoints;

         // iterate through all points in cloud
         while(!basicIt->atEnd()) {
            switch (button) {
               case BUTTON_INVERTPOINTS: modIt->set1i(FLAGS, basicIt->get1i(FLAGS) ^ PF_FLAG_SELECTED); break;  // invert selection
               case BUTTON_ALLPOINTS: modIt->set1i(FLAGS, basicIt->get1i(FLAGS) | PF_FLAG_SELECTED); break;  // select all points
               default: break;
            };
            basicIt->next();

            // update progress window
            counter++;
            if (counter%1000==0)
               if (!progressWindow->progress(factor*counter)) { delete newIt; delete sceneIterator; return; };
         };

         delete newIt;
         sceneIterator->next();
      }; // end while sceneIterator
      delete sceneIterator;

      // finish progress window
      bool dummy;
      progressWindow->popSteps(dummy);

   }; // end else button

}


void CmdObjSelectionByButton::undoAction(Object *context) {
   forget(context);
   throw ECannotUndo("CmdObjSelectionByButton does not provide any undo.");
}

bool CmdObjSelectionByButton::canUndo() {
   return false;
}


string CmdObjSelectionByButton::getDescription() {
   string result;
   switch (button) {
      case BUTTON_INVERTPOINTS: result = "invert point selection"; break;
      case BUTTON_ALLCLOUDS: result = "select all clouds"; break;
      case BUTTON_ALLPOINTS: result = "select all points"; break;
      case BUTTON_NOCLOUDS: result = "clear cloud selection"; break;
      case BUTTON_NOPOINTS: result = "clear point selection"; break;
      default: break;
   };
   return result;
}


bool CmdObjSelectionByButton::mergeCommand(const CommandObject *nextCmd) {
   if (this->button==BUTTON_INVERTPOINTS) return false;
   if (nextCmd->getInstanceClass() == CmdObjSelectionByButton::getClass()) {
      const CmdObjSelectionByButton *nextCmdMyType = dynamic_cast<const CmdObjSelectionByButton*>(nextCmd);
      pAssert(nextCmdMyType != NULL);
      if ((this->button)==(nextCmdMyType->button)) return true;
       else return false;

   } else {
      return false;
   }
}





// --------------- CmdObjSelectionOnArea ---------------


MetaClass* CmdObjSelectionOnArea::CmdObjSelectionOnAreaMetaClass = NULL;

const MetaClass* CmdObjSelectionOnArea::getInstanceClass() const {
   return getClass();
}

Object* CmdObjSelectionOnArea::newInstance() {
   return new CmdObjSelectionOnArea;
}

const MetaClass* CmdObjSelectionOnArea::getClass() {
   if (CmdObjSelectionOnAreaMetaClass==NULL) {
      CmdObjSelectionOnAreaMetaClass = new MetaClass("CmdObjSelectionOnArea", "no description.",
                                  newInstance, sizeof(CmdObjSelectionOnArea));
   }
   return CmdObjSelectionOnAreaMetaClass;
}

void CmdObjSelectionOnArea::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   CmdObjSelectionOnAreaMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( CmdObjSelectionOnArea )
   ADD_BOOLEAN_PROP(multiselect, 0)
	ADD_CARD32_PROP(areaWidth, 0)
	ADD_CARD32_PROP(areaHeight, 0)
	ADD_CARD32_PROP(selectMode, 0)
	ADD_INT32_PROP(clickX1, 0)
	ADD_INT32_PROP(clickY1, 0)
	ADD_INT32_PROP(clickX2, 0)
	ADD_INT32_PROP(clickY2, 0)
	ADD_EMBEDDED_OBJECT_PROP(cam, 0, Camera::getClass());
	ADD_EMBEDDED_OBJECT_PROP(viewFrust, 0, ViewFrustum::getClass());

// *** rectList handled separately in assign/write/read

}

void CmdObjSelectionOnArea::shutdown() {
   delete CmdObjSelectionOnAreaMetaClass;
}

void CmdObjSelectionOnArea::assign(const Object* obj, CopyContext *context) {
   CommandObject::assign(obj, context);
   const CmdObjSelectionOnArea *tobj = dynamic_cast<const CmdObjSelectionOnArea*>(obj);
   pAssert(tobj != NULL);
   rectList = tobj->rectList;
}

void CmdObjSelectionOnArea::write(OutputObjectStream *s) const {
   CommandObject::write(s);
   card32 n = (card32)rectList.size();
   s->write(n);
   for (card32 i=0; i<n; i++) {
      s->write(rectList[i].x1);
      s->write(rectList[i].y1);
      s->write(rectList[i].x2);
      s->write(rectList[i].y2);
   }
}

void CmdObjSelectionOnArea::read(InputObjectStream *s) {
   CommandObject::read(s);
   card32 n;
   s->read(n);
   rectList.resize(n);
   for (card32 i=0; i<n; i++) {
      s->read(rectList[i].x1);
      s->read(rectList[i].y1);
      s->read(rectList[i].x2);
      s->read(rectList[i].y2);
   }
}


void CmdObjSelectionOnArea::setup(int32 x1, int32 y1, int32 x2, int32 y2, bool multsel, card32 tol, card32 mode, Camera &ca, ViewFrustum &vfr, card32 w, card32 h)
{
  clickX1 = x1;
  clickY1 = y1;
  clickX2 = x2;
  clickY2 = y2;

  rectangle r;
  r.x1 = clickX1;
  r.y1 = clickY1;
  r.x2 = clickX2;
  r.y2 = clickY2;
  rectList.push_back(r);

  multiselect = multsel;
  tolerance = tol;
  selectMode = mode;
  cam = ca;
  viewFrust = vfr;
  areaWidth = w;
  areaHeight = h;
};


void CmdObjSelectionOnArea::doAction(Object *context) {
   Scene *scene = dynamic_cast<Scene*>(context); pAssert(scene);

   for (card32 i=0; i<rectList.size(); i++)
   {
      clickX1 = rectList[i].x1;
      clickY1 = rectList[i].y1;
      clickX2 = rectList[i].x2;
      clickY2 = rectList[i].y2;

      // geometric range which defines selection region in 3D
      GeometricRange* range = NULL;
      switch (selectMode)
      {
         case SELMODE_AREARECTANGLE: range = getRayPyramidalRangeFromMouseClick(clickX1,clickY1,clickX2,clickY2,areaWidth,areaHeight,&cam,&viewFrust); break;
         case SELMODE_AREACIRCLE: range = getRayConeRangeFromMouseClick((clickX1+clickX2)/2, (clickY1+clickY2)/2, areaWidth, areaHeight, sqrt((float)((clickX2-clickX1)*(clickX2-clickX1)) + ((clickY2-clickY1)*(clickY2-clickY1)))/2, &cam, &viewFrust); break;
         default: break;
      };
      if (range==NULL) return;

      // init progress window
      progressWindow->pushStep(true, "select area");
      progressWindow->progress(0);
      float32 numPoints = 1.0f;

      StandardSceneGraphIterator* sceneIterator = new StandardSceneGraphIterator();
      sceneIterator->clearAndSetup(scene->getRootNode(), scene->getRootState());

      while (!sceneIterator->atEnd()) {
         // iterate through visible point clouds only
         SGObjectNode* SGN = dynamic_cast<SGObjectNode*>(sceneIterator->getNode());
         if ((SGN==NULL) || !SGN->visible()) { sceneIterator->next(); continue; }
         PointCloud* PC = dynamic_cast<PointCloud*>(SGN->getSceneObject());
         if ((PC==NULL) || (!PC->providesAttribute("flags"))) { sceneIterator->next(); continue; }
         AAT FLAGS = PC->getAAT("flags");

         SceneObjectIterator *newItSO = PC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL));
         PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
         if ((newIt==NULL)||(newItSO==NULL)) { delete newItSO; sceneIterator->next(); continue; };
         ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(newIt);
         if (modIt == NULL) { delete newItSO; sceneIterator->next(); continue; }

         // cloud transformation matrix and geometric range iterator adpator
         range->setTransformation(sceneIterator->getState()->dynamicState->ctm);
         IteratorAdaptor *rangeIt = getGeoRangeIteratorAdaptor(PC, newIt, range);
         if (rangeIt==NULL) { delete newItSO; sceneIterator->next(); continue; }

         // needed for percentage display in progress window
         card64 counter = numPoints;
         numPoints += PC->getNumPoints();
         float32 factor = 100.0f / numPoints;

         // select or deselect points in range
         while (!rangeIt->atEnd() )
         {
            if (multiselect) rangeIt->set1i(FLAGS, rangeIt->get1i(FLAGS) & (0xFFFFFFFF ^ PF_FLAG_SELECTED));
            else rangeIt->set1i(FLAGS, rangeIt->get1i(FLAGS) | PF_FLAG_SELECTED);
            rangeIt->next() ;

            // update progress window
            counter++;
            if (counter%1000==0)
               if (!progressWindow->progress(factor*counter)) { delete rangeIt; delete newItSO; delete sceneIterator; return; };
         }

         delete rangeIt;
         delete newItSO;

         sceneIterator->next();
      }; // end while sceneIterator

      delete sceneIterator;
      delete range;

      // finish progress window
      bool dummy;
      progressWindow->popSteps(dummy);

   }; // end for rectlist

}


void CmdObjSelectionOnArea::undoAction(Object *context) {
   forget(context);
   throw ECannotUndo("CmdObjSelectionOnArea does not provide any undo.");
}

bool CmdObjSelectionOnArea::canUndo() {
   return false;
}


string CmdObjSelectionOnArea::getDescription() {

   string result;
   if (multiselect) result = "de";
   result += "select ";
   switch (selectMode)
   {
      case SELMODE_AREARECTANGLE: result += "RectangleArea: "; break;
      case SELMODE_AREACIRCLE: result += "CircleArea: "; break;
      default: result += ":"; break;
   };

   for (card32 i=0; i<rectList.size(); i++) {
      char buff[1000];
      string sX1 = itoa(rectList[i].x1,buff,10);
      string sY1 = itoa(rectList[i].y1,buff,10);
      string sX2 = itoa(rectList[i].x2,buff,10);
      string sY2 = itoa(rectList[i].y2,buff,10);
      result = result + "(" + sX1 + "," + sY1 + "," + sX2 + "," + sY2 + ") ";
   }
   return result;
}


bool CmdObjSelectionOnArea::mergeCommand(const CommandObject *nextCmd) {
   if (nextCmd->getInstanceClass() == CmdObjSelectionOnArea::getClass()) {
      const CmdObjSelectionOnArea *nextCmdMyType = dynamic_cast<const CmdObjSelectionOnArea*>(nextCmd);
      pAssert(nextCmdMyType != NULL);

      if ((this->multiselect==nextCmdMyType->multiselect) &&
         (this->tolerance==nextCmdMyType->tolerance) &&
         (this->selectMode==nextCmdMyType->selectMode) &&
         (this->cam==nextCmdMyType->cam) &&
         (this->viewFrust==nextCmdMyType->viewFrust) &&
         (this->areaWidth==nextCmdMyType->areaWidth) &&
         (this->areaHeight==nextCmdMyType->areaHeight))
         {
            rectangle r;
            r.x1 = nextCmdMyType->clickX1;
            r.y1 = nextCmdMyType->clickY1;
            r.x2 = nextCmdMyType->clickX2;
            r.y2 = nextCmdMyType->clickY2;
            this->rectList.push_back(r);
            return true;
         }
       else return false;

   } else {
      return false;
   }
}





// --------------- CmdObjSelectionByAttribute ---------------


MetaClass* CmdObjSelectionByAttribute::CmdObjSelectionByAttributeMetaClass = NULL;

const MetaClass* CmdObjSelectionByAttribute::getInstanceClass() const {
   return getClass();
}

Object* CmdObjSelectionByAttribute::newInstance() {
   return new CmdObjSelectionByAttribute;
}

const MetaClass* CmdObjSelectionByAttribute::getClass() {
   if (CmdObjSelectionByAttributeMetaClass==NULL) {
      CmdObjSelectionByAttributeMetaClass = new MetaClass("CmdObjSelectionByAttribute", "no description.",
                                  newInstance, sizeof(CmdObjSelectionByAttribute));
   }
   return CmdObjSelectionByAttributeMetaClass;
}

void CmdObjSelectionByAttribute::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   CmdObjSelectionByAttributeMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( CmdObjSelectionByAttribute )
   ADD_STRING_PROP(AttribName, 0)
   ADD_VECTOR4F_PROP(AttribValues, 0)
	ADD_CARD32_PROP(Tolerance, 0)
	ADD_BOOLEAN_PROP(deselect, 0)
}

void CmdObjSelectionByAttribute::shutdown() {
   delete CmdObjSelectionByAttributeMetaClass;
}


void CmdObjSelectionByAttribute::setup(string attrname, Vector4f attrval, card32 tol, bool desel) {
   AttribName = attrname;
   AttribValues = attrval;
   Tolerance = tol;
   deselect = desel;
}


void CmdObjSelectionByAttribute::doAction(Object *context) {
   Scene *scene = dynamic_cast<Scene*>(context); pAssert(scene);

   float32 tol255 = Tolerance/255.0f;

   // init progress window
   progressWindow->pushStep(true, "select by attribute");
   progressWindow->progress(0);
   float32 numPoints = 1.0f;

   StandardSceneGraphIterator* sceneIterator = new StandardSceneGraphIterator();
   sceneIterator->clearAndSetup(scene->getRootNode(), scene->getRootState());

   while (!sceneIterator->atEnd()) {
      // iterate through all point clouds
      SGObjectNode* SGN = dynamic_cast<SGObjectNode*>(sceneIterator->getNode());
      if (SGN==NULL) { sceneIterator->next(); continue; }
      PointCloud* PC = dynamic_cast<PointCloud*>(SGN->getSceneObject());
      if ((PC==NULL) || (!PC->providesAttribute("flags"))) { sceneIterator->next(); continue; }
      AAT FLAGS = PC->getAAT("flags");
      if (!PC->providesAttribute(AttribName)) { sceneIterator->next(); continue; }
      AAT ATTRIB = PC->getAAT(AttribName);

      SceneObjectIterator *newItSO = PC->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC));
      BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newItSO);
      ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(newItSO);
      if ((basicIt==NULL) || (modIt==NULL)) { delete newItSO; sceneIterator->next(); continue; };

      // needed for percentage display in progress window
      card64 counter = numPoints;
      numPoints += PC->getNumPoints();
      float32 factor = 100.0f / numPoints;

      // iterate through all points in cloud
      while(!basicIt->atEnd()) {

         if (AttribName=="color") {
            Vector3f tmpColor = basicIt->get3f(ATTRIB);
            if (((tmpColor[0] >= AttribValues[0]-tol255) && (tmpColor[0] <= AttribValues[0]+tol255)) &&
                ((tmpColor[1] >= AttribValues[1]-tol255) && (tmpColor[1] <= AttribValues[1]+tol255)) &&
                ((tmpColor[2] >= AttribValues[2]-tol255) && (tmpColor[2] <= AttribValues[2]+tol255)))
               {
                  if (deselect) modIt->set1i(FLAGS, basicIt->get1i(FLAGS) & (0xFFFFFFFF ^ PF_FLAG_SELECTED));
                     else modIt->set1i(FLAGS, basicIt->get1i(FLAGS) | PF_FLAG_SELECTED);
            };

         };

         basicIt->next();

         // update progress window
         counter++;
         if (counter%1000==0)
            if (!progressWindow->progress(factor*counter)) { delete newItSO; delete sceneIterator; return; };
      };

      delete newItSO;
      sceneIterator->next();
   };  // end while sceneIterator
   delete sceneIterator;

   // finish progress window
   bool dummy;
   progressWindow->popSteps(dummy);

}


void CmdObjSelectionByAttribute::undoAction(Object *context) {
   Scene *scene = dynamic_cast<Scene*>(context); pAssert(scene);
   forget(scene);
   throw ECannotUndo("CmdObjSelectionByAttribute does not provide any undo.");
}

bool CmdObjSelectionByAttribute::canUndo() {
   return false;
}


string CmdObjSelectionByAttribute::getDescription() {
   string result;
   if (deselect) result = "de";
   result += "select by attribute: ";
   result += AttribName;
   char buff[1000];
   string V1 = itoa(AttribValues[0],buff,10);
   string V2 = itoa(AttribValues[1],buff,10);
   string V3 = itoa(AttribValues[2],buff,10);
   string V4 = itoa(AttribValues[3],buff,10);
   result = result + "(" + V1 + "," + V2 + "," + V3 + "," + V4 + ")";
   string T = itoa(Tolerance,buff,10);
   result = result + "  tolerance: " + T;
   return result;
}


bool CmdObjSelectionByAttribute::mergeCommand(const CommandObject *nextCmd) {
   if (nextCmd->getInstanceClass() == CmdObjSelectionByAttribute::getClass()) {
      const CmdObjSelectionByAttribute *nextCmdMyType = dynamic_cast<const CmdObjSelectionByAttribute*>(nextCmd);
      pAssert(nextCmdMyType != NULL);
      if ((this->AttribName==nextCmdMyType->AttribName) &&
          (this->AttribValues==nextCmdMyType->AttribValues) &&
          (this->Tolerance==nextCmdMyType->Tolerance) &&
          (this->deselect==nextCmdMyType->deselect))
         {
            return true;
         }
       else return false;

   } else {
      return false;
   }
}


