//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "CmdObjSpecialDraw.h"
//---------------------------------------------------------------------------
#include "FixedArrayClassProperty.h"

#include "StringClassProperty.h"
#include "ObjectClassProperty.h"
#include "PCCommand.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "EmbeddedObjectClassProperty.h"
#include "PointCloud.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "PointCloudIterators.h"
#include "IteratorAdaptors.h"
#include "SelectionIteratorAdaptor.h"
#include "GeometricRange.h"
#include "GeometricRangeIteratorAdaptor.h"
#include "UnstructuredInCorePointCloud.h"
#include "VertexDescriptor.h"
#include "ObjectViewsTable.h"
#include "DebugOutput.h"
#include "MHSelectionIteratorAdaptor.h"
#include "MHGeoRangeIteratorAdaptor.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "PointCloudTools.h"
#include "SelectionTools.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* CmdObjSpecialDraw::CmdObjSpecialDrawMetaClass = NULL;

const MetaClass* CmdObjSpecialDraw::getInstanceClass() const {
   return getClass();
}

Object* CmdObjSpecialDraw::newInstance() {
   return new CmdObjSpecialDraw;
}

const MetaClass* CmdObjSpecialDraw::getClass() {
   if (CmdObjSpecialDrawMetaClass==NULL) {
      CmdObjSpecialDrawMetaClass = new MetaClass("CmdObjSpecialDraw", "no description.",
                                  newInstance, sizeof(CmdObjSpecialDraw));
   }
   return CmdObjSpecialDrawMetaClass;
}


void CmdObjSpecialDraw::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   CmdObjSpecialDrawMetaClass->copyParentProperties();

   BEGIN_CLASS_INIT(CmdObjSpecialDraw)
   ADD_CARD32_PROP(mode ,0)
   ADD_FLOAT32_FIXED_ARRAY_PROP( m_col ,0,3 ,1 )
   ADD_FLOAT32_FIXED_ARRAY_PROP( m_normal ,0,3 ,1 )
   ADD_INT32_PROP(flags ,0)

}

void CmdObjSpecialDraw::shutdown() {
   delete CmdObjSpecialDrawMetaClass;
}


// ----
void CmdObjSpecialDraw::setup(card32 m, Vector3f color, Vector3f normal, int32 f)
{
  mode = m;

  m_col=color;
  m_normal=normal;
  flags=f;

};


void CmdObjSpecialDraw::doAction(Object *context) {
   Scene *scene = dynamic_cast<Scene*>(context); pAssert(scene);
   checkSceneAllAttributes(scene); //slow but stable
   SGLegacyPCSceneAdaptor sceneLA(scene);

   if(mode==0){
      for(card32 i=0; i<sceneLA.getNumPointClouds();i++){

         PointCloud *pc = sceneLA.getPointCloud(i);
//       PointCloudIterator *it = pc->createIterator(CAP_MODIFYABLE_PC);
         SceneObjectIterator *itSO = pc->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_HIERARCHICAL);
         PointCloudIterator *it = dynamic_cast<PointCloudIterator*>(itSO);
         if (it==NULL) { delete itSO; continue; };
         IteratorAdaptor *sit = getSelectionIteratorAdaptor(pc, it);
         // replaces: SelectionIteratorAdaptor *sit = new SelectionIteratorAdaptor(it, pc->getDescr());


         AAT COLOR = pc->getAAT("color");
            while(!sit->atEnd()) {
               sit->set3f(COLOR, m_col);
               sit->next();
            };
         delete sit;
         delete it;
      }
  }
  if(mode==1){
     for(card32 i=0; i<sceneLA.getNumPointClouds();i++){

         PointCloud *pc = sceneLA.getPointCloud(i);
         SceneObjectIterator *itSO = pc->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC);
         PointCloudIterator *it = dynamic_cast<PointCloudIterator*>(itSO);
         if (it==NULL) { delete itSO; continue; };
         NormalIteratorAdaptor *nit = new NormalIteratorAdaptor(it,pc->getDescr(),m_normal);
         AAT COLOR = pc->getAAT("color");
            while(!nit->atEnd()) {
                  nit->set3f(COLOR,m_col);
                  nit->next();
              };
         delete nit;
         delete it;
     }
  }

  if(mode==2){
     for(card32 i=0; i<sceneLA.getNumPointClouds();i++){
         string pcName = sceneLA.getNameWithPath( i );
         checkAttribute( scene, pcName, "color", 3, VAD::DATA_FORMAT_FLOAT32 );

         PointCloud *pc = sceneLA.getPointCloud(i);

         SceneObjectIterator *newItSO = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC));
         PointCloudIterator *newIt = dynamic_cast<PointCloudIterator*>(newItSO);
         if (newIt==NULL) { delete newItSO; continue; };

         if (newIt == NULL) continue;
         BasicPointCloudIterator *baseIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
         if (baseIt == NULL) {delete newIt; continue;}
         ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(newIt);
         if (modIt == NULL) {delete newIt; continue;}

         AAT FLAGS = pc->getAAT("flags");
         while(!baseIt->atEnd()) {

           // modIt->set1i(FLAGS, baseIt->get1i(FLAGS) | flags);
            modIt->set1i(FLAGS, baseIt->get1i(FLAGS) & (0xFFFFFFFF ^ flags)); // clear flag selection
            baseIt->next();
         };

         delete newIt;
     }
  }

}

void CmdObjSpecialDraw::undoAction(Object *context) {
   forget(context);
   throw ECannotUndo("CmdObjSpecialDraw does not provide any undo.");
}


bool CmdObjSpecialDraw::canUndo() {
   return false;
}

string CmdObjSpecialDraw::getDescription() {
   string result;
   result = "";
   if (mode==0)result += "fill selection";
   if (mode==1)result += "fill similar normals";
   if (mode==2)result += "clear FLAG_MODIFIED";
   return result;
}







