//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "GEOXLEvent.h"
//---------------------------------------------------------------------------
#include "StringClassProperty.h"
#include "ObjectClassProperty.h"
#include "CommandObject.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------

using namespace geoxl;
using namespace std;

MetaClass* GEOXLEvent::GEOXLEventMetaClass = NULL;

const MetaClass* GEOXLEvent::getInstanceClass() const {
   return getClass();
}

Object* GEOXLEvent::newInstance() {
   return new GEOXLEvent;
}

const MetaClass* GEOXLEvent::getClass() {
   if (GEOXLEventMetaClass==NULL) {
      GEOXLEventMetaClass = new MetaClass("XRTEvent", "no description.",
                                  newInstance, sizeof(GEOXLEvent));
   }
   return GEOXLEventMetaClass;
}

void GEOXLEvent::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   GEOXLEventMetaClass->copyParentProperties();
}

void GEOXLEvent::shutdown() {
   delete GEOXLEventMetaClass;
}

// ----

MetaClass* ModeChangeEvent::ModeChangeEventMetaClass = NULL;

const MetaClass* ModeChangeEvent::getInstanceClass() const {
   return getClass();
}

Object* ModeChangeEvent::newInstance() {
   return new ModeChangeEvent;
}

const MetaClass* ModeChangeEvent::getClass() {
   if (ModeChangeEventMetaClass==NULL) {
      ModeChangeEventMetaClass = new MetaClass("ModeChangeEvent", "no description.",
                                  newInstance, sizeof(ModeChangeEvent));
   }
   return ModeChangeEventMetaClass;
}

void ModeChangeEvent::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   ModeChangeEventMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(ModeChangeEvent)
	ADD_STRING_PROP( m_ModeClassName, 0 );
}

void ModeChangeEvent::shutdown() {
   delete ModeChangeEventMetaClass;
}


// ----

MetaClass* SpecifiedModeChangeEvent::SpecifiedModeChangeEventMetaClass = NULL;

const MetaClass* SpecifiedModeChangeEvent::getInstanceClass() const {
   return getClass();
}

Object* SpecifiedModeChangeEvent::newInstance() {
   return new SpecifiedModeChangeEvent;
}

const MetaClass* SpecifiedModeChangeEvent::getClass() {
   if (SpecifiedModeChangeEventMetaClass==NULL) {
      SpecifiedModeChangeEventMetaClass = new MetaClass("SpecifiedModeChangeEvent", "no description.",
                                  newInstance, sizeof(SpecifiedModeChangeEvent));
   }
   return SpecifiedModeChangeEventMetaClass;
}

void SpecifiedModeChangeEvent::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   SpecifiedModeChangeEventMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(SpecifiedModeChangeEvent)
   ADD_OBJECT_PROP(m_Object ,0, Object::getClass(), true)
}

void SpecifiedModeChangeEvent::shutdown() {
   delete SpecifiedModeChangeEventMetaClass;
}

// ----

MetaClass* CommandObjectEvent::CommandObjectEventMetaClass = NULL;

const MetaClass* CommandObjectEvent::getInstanceClass() const {
   return getClass();
}

Object* CommandObjectEvent::newInstance() {
   return new CommandObjectEvent;
}

const MetaClass* CommandObjectEvent::getClass() {
   if (CommandObjectEventMetaClass==NULL) {
      CommandObjectEventMetaClass = new MetaClass("CommandObjectEvent", "no description.",
                                  newInstance, sizeof(CommandObjectEvent));
   }
   return CommandObjectEventMetaClass;
}

void CommandObjectEvent::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   CommandObjectEventMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(CommandObjectEvent)
   ADD_OBJECT_PROP(m_CommandObject ,0, CommandObject::getClass(), true)
}

void CommandObjectEvent::shutdown() {
   delete CommandObjectEventMetaClass;
}


// ----

MetaClass* StringEvent::StringEventMetaClass = NULL;

const MetaClass* StringEvent::getInstanceClass() const {
   return getClass();
}

Object* StringEvent::newInstance() {
   return new StringEvent;
}

const MetaClass* StringEvent::getClass() {
   if (StringEventMetaClass==NULL) {
      StringEventMetaClass = new MetaClass("StringEvent", "no description.",
                                  newInstance, sizeof(StringEvent));
   }
   return StringEventMetaClass;
}

void StringEvent::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   StringEventMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT(StringEvent)
	ADD_STRING_PROP(m_String,0)
}

void StringEvent::shutdown() {
   delete StringEventMetaClass;
}


// ----


void GEOXLEvent::assign(const Object* obj, CopyContext *context)
{
   forget(context);
   GEOXLEvent * event = (GEOXLEvent *)obj;
   m_Source = event->m_Source;
   m_Dest = event->m_Dest;
}

void ModeChangeEvent::assign(const Object* obj, CopyContext *context)
{
   GEOXLEvent::assign( obj, context );
   ModeChangeEvent * ev = (ModeChangeEvent *)obj;
   m_ModeClassName = ev->m_ModeClassName;
}

SpecifiedModeChangeEvent::SpecifiedModeChangeEvent()
{
   m_Object = NULL;
}

SpecifiedModeChangeEvent::~SpecifiedModeChangeEvent()
{
   delete m_Object;
}

void SpecifiedModeChangeEvent::assign(const Object* obj, CopyContext *context)
{
   GEOXLEvent::assign( obj, context );
   SpecifiedModeChangeEvent * ev = (SpecifiedModeChangeEvent*)obj;
   m_Object = ev->m_Object;
}

void SpecifiedModeChangeEvent::setObject( Object * obj )
{
   delete m_Object;
   m_Object = obj;
}

CommandObjectEvent::CommandObjectEvent()
{
   m_CommandObject = NULL;
}

CommandObjectEvent::~CommandObjectEvent()
{
   delete m_CommandObject;
}

void CommandObjectEvent::assign(const Object* obj, CopyContext *context)
{
   GEOXLEvent::assign( obj, context );
   CommandObjectEvent * ev = (CommandObjectEvent*)obj;
   m_CommandObject = ev->m_CommandObject;
}

void CommandObjectEvent::setCommandObject( CommandObject * obj )
{
   delete m_CommandObject;
   m_CommandObject = obj;
}
