//---------------------------------------------------------------------------
#ifndef GEOXLEventH
#define GEOXLEventH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include <string>
//---------------------------------------------------------------------------


namespace geoxl
{
   using namespace std;


class EDITOR_API GEOXLEvent : public Persistent
{
private:
   static MetaClass *GEOXLEventMetaClass;
   static Object* newInstance();

   string   m_Source;
   string   m_Dest;

   card32   m_Context;  // non-persistent & undefined, must be set before sending the event

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   inline void setSource( string source ) { m_Source = source; }
   inline string getSource() {return m_Source;}
   inline void setDest( string dest ) { m_Dest = dest; }
   inline string getDest() {return m_Dest;}

   inline void setContext( card32 context ) { m_Context = context; }
   inline card32 getContext() { return m_Context; }

};

class EDITOR_API ModeChangeEvent : public GEOXLEvent
{
private:
   static MetaClass *ModeChangeEventMetaClass;
   static Object* newInstance();

   string   m_ModeClassName;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   inline string getModeClassName() { return m_ModeClassName; }
   inline void setModeClassName( string className ) { m_ModeClassName = className; }
};

class EDITOR_API SpecifiedModeChangeEvent : public GEOXLEvent
{
private:
   static MetaClass *SpecifiedModeChangeEventMetaClass;
   static Object* newInstance();

   Object   *  m_Object;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   SpecifiedModeChangeEvent();
   ~SpecifiedModeChangeEvent();

   /// deletes old object, then sets obj
   void setObject( Object * obj );
   Object * getObject() { return m_Object; }

};

class CommandObject;

class EDITOR_API CommandObjectEvent : public GEOXLEvent
{

private:
   static MetaClass *CommandObjectEventMetaClass;
   static Object* newInstance();

   CommandObject   *  m_CommandObject;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   CommandObjectEvent();
   ~CommandObjectEvent();

   /// deletes old object, then sets cmdobj
   void setCommandObject( CommandObject * cmd );
   CommandObject * getCommandObject() {return m_CommandObject; }

};

/**
   StringEvent - Simple Event that holds a string
*/

class EDITOR_API StringEvent : public GEOXLEvent
{
private:
   static MetaClass *StringEventMetaClass;
   static Object* newInstance();

   string m_String;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   string getString() { return m_String; }
   void setString(string str)  { m_String = str; }

};

}

#endif
