//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "MeshExporter.h"
//---------------------------------------------------------------------------
#include "UnstructuredInCoreTriangleMesh.h"
#include "PointSet_templCode.h"
#include "LinearAlgebra_templCode.h"

//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------
#include "StringHelper.h"



MeshExporter::MeshExporter(PointCloud* pc, ofstream* outf)
{
  this->pc = pc;
  this->outf = outf;
}

void MeshExporter::exportPointCloud()
{
	UnstructuredInCoreTriangleMesh *mesh = dynamic_cast<UnstructuredInCoreTriangleMesh*>(pc);
	if ( mesh != NULL ){
		// Export triangle mesh
		PointSet* vertex   = mesh->getPointSet();
		card32 numVert= vertex->getNumEntries();
		const VertexDescriptor* vertexDesc = vertex->getDescr();
		AAT POSITION = vertexDesc->getToken("position");
		for (card32 i=0; i<numVert; i++){
			Vector3f vec3f = vertex->get<float32,3>(i, POSITION);
			*outf << "v " << vec3f[0] << " " << vec3f[1] << " " << vec3f[2] << endl;
		}
		PointSet* triangle = mesh->getTriangles();
		card32 numTri= triangle->getNumEntries();
		const VertexDescriptor* triDesc = triangle->getDescr();
		AAT INDEX = triDesc->getToken("index");
		for (card32 i=0; i<numTri; i++){
			Vector3i vec3i = triangle->get<int32,3>(i, INDEX);
			*outf << "f " << intToStr(vec3i[0]+1) << " " << intToStr(vec3i[1]+1) << " " << intToStr(vec3i[2]+1) << endl;
		}
	} else {
		// Export point cloud
		if ( pc == NULL ){
			return;
		}
		SceneObjectIterator *newItSO = pc->createIterator( SOIT( SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC ) );
		BasicPointCloudIterator* basicIt = dynamic_cast<BasicPointCloudIterator*>( newItSO );
		ModifyablePointCloudIterator* modIt = dynamic_cast<ModifyablePointCloudIterator*>( newItSO );
		AAT POSITION = pc->getAAT( "position" );
		basicIt->reset();
		while( !basicIt->atEnd() ){
			Vector3f vec3f = basicIt->get3f( POSITION );
			*outf << "v " << vec3f[0] << " " << vec3f[1] << " " << vec3f[2] << endl;
			basicIt->next();
		}
		delete newItSO;
	}
}

