#ifndef NormalEstimatorH
#define NormalEstimatorH

#include "Persistent.h"
#include "PointCloud.h"
#include "PointCloudProcessingDefinitions.h"

namespace geoxl
{

#define ENVIRONMENT_KNN 0
#define ENVIRONMENT_EPSILON 1
#define ENVIRONMENT_TIMES_SAMPLING_SPACING 2

/*
 * Estimate normals in point sets and point clouds
 *
 * Required: 
 * - position        (3f)
 *
 * Output:
 * - normal          (3f) NormalEstimator, NormalUnifier
 * 
 * Optional
 * - samplingSpacing (1f) SamplingSpacingEstimator
 * - tangent_u       (3f) Output
 * - tangent_v       (3f) Output
 */
class EDITOR_API NormalEstimator : public Persistent
{	
	private:

	public:

		/// Estimate normals in point set
		///  mode == NORMAL_ESTIMATE_KNN: numKNN knn-neighbors used
		///  mode == NORMAL_ESTIMATE_EPSILON: all knn-neighbors withing epsilon-ball used
		///  mode == NORMAL_ESTIMATE_TIMES_SAMPLING_SPACING: epsilon = samplingSpacing * timesSamplingSpacing
		/// saveTangents: write to tangent_u, tangent_v
		static void estimateNormals( 
			PointSet* ps, 
			const card32 numKNN, 
			float32 epsilon = -1, 
			float32 timesSamplingSpacing = -1, 
			card32 mode = ENVIRONMENT_KNN,
			bool saveTangents = false );

		/// Estimate normals in point set
		///  mode == NORMAL_ESTIMATE_KNN: numKNN knn-neighbors used
		///  mode == NORMAL_ESTIMATE_EPSILON: all knn-neighbors withing epsilon-ball used
		///  mode == NORMAL_ESTIMATE_TIMES_SAMPLING_SPACING: epsilon = samplingSpacing * timesSamplingSpacing
		/// saveTangents: write to tangent_u, tangent_v
		static void estimateNormals( 
			PointCloud* pc, 
			const card32 numKNN,
			float32 epsilon = -1, 
			float32 timesSamplingSpacing = -1, 
			card32 mode = ENVIRONMENT_KNN, 
			bool saveTangents = false );
};

}

#endif