//---------------------------------------------------------------------------
#ifndef PCCommandH
#define PCCommandH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
#include "Persistent.h"
#include "OnSceneChangeCallback.h"
#include "PCCommand.h"
//---------------------------------------------------------------------------

namespace geoxl {

class GLContext;
class Scene;

static const card32 PCCOMMAND_CHANGE_VIEW = 1;

class EDITOR_API PCCommand : public Persistent {	
	GEOXL_ABSTRACT_CLASS( PCCommand )

	protected:
		OnSceneChangeCallback *onChange;
		QWidget *sceneEditorWidget;
		Scene* scene;
	
	public:
		/// Constructor
		PCCommand();

		virtual void execute(Scene *scene) = 0;

		/// called once to connect tool with scene and editor
		/// scene: point cloud scene
		/// onChange: callback to notify editor of changes
		/// doRedraw: callback to cause redraw of current editor view of scene
		/// sceneEditorWidget: top level widget that contains the editor (used to attach child windows to the right parents, do not cast to SceneEditorWidget to access it directly)
		virtual void connectScene( Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget );
		/// override this method to provide user feedback, e.g. handles / bounding boxes etc...
		virtual void glDrawTool(GLContext *glContext) {forget(glContext);}
		/// call to cause immediate redraw
		virtual void redrawScene() {if (onChange != NULL) { onChange->changed( PCCOMMAND_CHANGE_VIEW );}}
		/// call to write a frame as bitmap to disc (if activated in main editor, otherwise nothing happens)
		virtual void writeFrameToDisk() {if (onChange != NULL) {onChange->changed(PCI_WRITE_FRAME);}}

		virtual void assign(const Object* obj, CopyContext *context = NULL);

		inline void setScene( Scene* scene) { this->scene = scene; }
		inline void setCallback( OnSceneChangeCallback* callback ){ this->onChange = callback; }
		inline Scene* getScene() const { return scene; }
		inline OnSceneChangeCallback* getCallback() const { return onChange; }

};

}

#endif
 