//---------------------------------------------------------------------------
#include "stdafx.h"
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------
#include "SelectionTools.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "SceneEditorWidget.h"
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_ABSTRACT_CLASS( PCInteractionTool, 0 )
{
	BEGIN_CLASS_INIT( PCInteractionTool )
}

PCInteractionTool::PCInteractionTool() 
{
   scene = NULL;
   onChange = NULL;
	screenSizeX = 0;
	screenSizeY = 0;
}

void PCInteractionTool::connectScene(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget) 
{
   this->scene = scene;
   this->onChange = onChange;
   this->sceneEditorWidget = sceneEditorWidget;
}

PCInteractionTool::~PCInteractionTool() 
{
   delete onChange;
};

bool PCInteractionTool::forceCamByDefault() 
{
   return false;
}

bool PCInteractionTool::getIntersectionFromCursor( int x, int y, Vector3f &retPoint, string &pcName, PointCloud ** pc ) 
{
   PointCloudIterator* it = getClosestPointIteratorFromAllPoints(x, y, screenSizeX, screenSizeY, 4,
			&getScene()->getRootState()->staticState->camera, 
			&getScene()->getRootState()->staticState->viewFrustum,
			getScene(),
			pcName,
			&retPoint
	);

	if( it == NULL )
		return false;

	BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(it);
	*pc = getPointCloud( getScene(), pcName );

	pAssert( bIt != NULL );
	pAssert( *pc != NULL );

	//retPoint = bIt->get3f( (*pc)->getAAT( "position") );
	delete it;
	return true;
}

vector<PointCloud*> geoxl::PCInteractionTool::getAllPointCloudsFromScene()
{
	vector<PointCloud*> ret;

	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( getScene()->getRootNode(), getScene()->getRootState() );
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			PointCloud * pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				ret.push_back(pc);
		}
		sceneIt->next();
	}

	return ret;
}

vector<PointCloud*> geoxl::PCInteractionTool::getSelectedPointCloudsFromScene()
{
	vector<PointCloud*> ret;
	vector<string> sel = getScene()->getRootState()->staticState->getSelectedNodes();
	for( unsigned i=0;i<sel.size();i++ )
	{
		PointCloud * pc = dynamic_cast<PointCloud*>(
			getPointCloud(getScene(), sel[i]));
		if( pc )
			ret.push_back( pc );
	}

	return ret;
}

void PCInteractionTool::mouseDownPassToCam(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->mouseDownPassToCam(x, y, buttonsState, modifiersState);
   }
}

void PCInteractionTool::mouseMovedPassToCam(int32 x, int32 y) 
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->mouseMovedPassToCam(x, y);
   }
}

void PCInteractionTool::mouseUpPassToCam(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->mouseUpPassToCam(x, y, buttonsState, modifiersState);
   }
}

void PCInteractionTool::areaResizePassToCam(card32 width, card32 height) 
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->areaResizePassToCam(width, height);
   }
}

void PCInteractionTool::mouseWheelRotatedPassToCam(int32 rotatedDelta, Modifiers modifiersState) 
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->mouseWheelRotatedPassToCam(rotatedDelta, modifiersState);
   }
}

void PCInteractionTool::keyDownPassToCam(GeneralKey key)
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->keyDownPassToCam(key);
   }
}

void PCInteractionTool::keyUpPassToCam(GeneralKey key)
{
   SceneEditorWidget *w = dynamic_cast<SceneEditorWidget*>(sceneEditorWidget);
   if (w) {
      w->keyUpPassToCam(key);
   }
}
