//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PickFromPointSet.h"
//---------------------------------------------------------------------------
#include "Camera.h"
#include "ViewFrustum.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


float32 geoxl::computeMinOnScreenDistancePoint(
   int32 x,
   int32 y,
   const PointSet *ps,
   ViewFrustum &vf,
   Camera &cam,
   card32 viewWidth,
   card32 viewHeight,
   mpcard &closestIndex,
   Matrix4f baseTransform,
   string positionChannel)
{
   if (!ps || ps->getNumEntries() == 0) throw ERangeCheck("computeMinOnScreenDistancePoint() - point set NULL or empty.");

	Matrix4f viewMatrix = cam.getViewMatrix();
	Matrix4f projMatrix = vf.getProjectionMatrix();
	Matrix4f mvMatrix = projMatrix * viewMatrix * baseTransform;
	float32 tanOfHalfViewingAngle = tan(M_PI*vf.getVerticalFieldOfView()/180.0f/2.0f);
	Vector3f camViewDir = cam.getOrthoNormViewDirection();
	Vector3f camRightDir = cam.getOrthoNormRightDirection();
	Vector3f camUpDir = cam.getOrthoNormUpDirection();
	Vector3f camPos = cam.getPosition();

   Matrix4f T = vf.getProjectionMatrix();
   AAT posAAT = ps->getAAT(positionChannel);
   mpcard n = ps->getNumEntries();
   float32 minDist = 1E30f;
   closestIndex = 0;
   for (mpcard i=0; i<n; i++) {
      Vector3f wc = ps->get3f(i, posAAT);

	   Vector3f proj = projectiveTransformVector3f(mvMatrix, wc);
	   Vector2f click;
	   click[0] = ((float32)x)/(viewWidth*0.5f)-1.0f;
	   click[1] = ((float32)y)/(viewHeight*0.5f)-1.0f;
	   Vector2f proj2 = makeVector2f(proj[0], proj[1]);
	   //float32 depth = (pos-camPos)*camViewDir;
	   float32 dist = norm(click-proj2)*viewHeight*0.5f;
      if (dist < minDist) {
         minDist = dist;
         closestIndex = i;
      }
   }
   return minDist;
}
