//---------------------------------------------------------------------------
#ifndef SceneCommandObjectsH
#define SceneCommandObjectsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "CommandObject.h"
#include "PointCloud.h"
#include "ViewFrustum.h"
#include "Camera.h"
//---------------------------------------------------------------------------


namespace geoxl {

class Scene;

// CmdObjects List for faster search... ;-)

// CmdObjDeletePCs
// CmdObjRenamePC
// CmdObjPCCommand
// CmdObjChangeView
// CmdObjChangePCFlags
// CmdObjChangeSelection
// CmdObjApplyRegisterTransform


// ----

class EDITOR_API CmdObjDeletePCs : public CommandObject {
 private:
   static MetaClass *CmdObjDeletePCsMetaClass;
   static Object* newInstance();

   vector<string> m_deletedPCs;

   string getDPC(mpcard index) const;
   void setDPC(mpcard index, string n);
   mpcard getNumDPC() const;
   void setNumDPC(mpcard size);
   void pushDPC();
   void popDPC();                    

	void setNumDeletedPointClouds(mpcard size) {m_deletedPCs.resize(size);}
	void deleteDeletedPointCloud( mpcard index ) {pAssert(index<m_deletedPCs.size() );m_deletedPCs.erase(m_deletedPCs.begin() + index);}
	char * getDeletedPointCloud( mpcard index ) {pAssert(index<m_deletedPCs.size() );return (char*)&m_deletedPCs[index];}
	mpcard getNumDeletedPointClouds() {return (mpcard)m_deletedPCs.size();}
	string getDeletedPointCloudName(mpcard index ) {return m_deletedPCs[index];}

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   // virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual void setup(vector<string> deletedPCs);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};



class EDITOR_API CmdObjRenamePC : public CommandObject {
 private:
   static MetaClass *CmdObjRenamePCMetaClass;
   static Object* newInstance();

   string m_oldName;
   string m_newName;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   // virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual void setup(string oldName, string newName);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();

};


class PCCommand;

class EDITOR_API CmdObjPCCommand : public CommandObject {
 private:
   static MetaClass *CmdObjPCCommandMetaClass;
   static Object* newInstance();

 protected:
    PCCommand *m_cmd;

 public:
   CmdObjPCCommand();

   virtual void setup(const PCCommand *cmd);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   // virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();

   ~CmdObjPCCommand();

};



class EDITOR_API CmdObjChangeView : public CommandObject {
 private:
   static MetaClass *CmdObjChangeViewMetaClass;
   static Object* newInstance();

 protected:
   ViewFrustum viewFrustum, old_vf;
   Camera cam, old_cam;
   bool setViewFrustum;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void setup(const ViewFrustum &vf, const Camera &cam, bool setViewFrustum = false);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
   virtual bool mergeCommand(const CommandObject *nextCmd);

};



class EDITOR_API CmdObjChangePCFlags : public CommandObject {
 private:
   static MetaClass *CmdObjChangePCFlagsMetaClass;
   static Object* newInstance();

 protected:
   string targetName;
   card32 oldFlags, flags;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void setup(card32 flags, string targetName);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};


class EDITOR_API CmdObjChangeSelection : public CommandObject {
 private:
   static MetaClass *CmdObjChangeSelectionMetaClass;
   static Object* newInstance();

   string getSel(mpcard index) const;
   void setSel(mpcard index, string n);
   mpcard getNumSel() const;
   void setNumSel(mpcard size);
   void pushSel();
   void popSel();

	void setNumSelections(mpcard size) {m_selection.resize(size);}
	void deleteSelection( mpcard index ) {pAssert( index < m_selection.size()); m_selection.erase( m_selection.begin() + index );}
	char * getSelection( mpcard index ) {pAssert( index < m_selection.size()); return (char*)&m_selection[index];}
	mpcard getNumSelections() {return (mpcard)m_selection.size();}
	string getSelectionName(mpcard index ) {pAssert( index < m_selection.size()); return m_selection[index];}


   string getOSel(mpcard index) const;
   void setOSel(mpcard index, string n);
   mpcard getNumOSel() const;
   void setNumOSel(mpcard size);
   void pushOSel();
   void popOSel();

	void setNumOldSelections(mpcard size) {m_oldSelection.resize(size);}
	void deleteOldSelection( mpcard index ) {pAssert( index < m_oldSelection.size()); m_oldSelection.erase( m_oldSelection.begin() + index );}
	char * getOldSelection( mpcard index ) {pAssert( index < m_oldSelection.size()); return (char*)&m_oldSelection[index];}
	mpcard getNumOldSelections() {return (mpcard)m_oldSelection.size();}
	string getOldSelectionName(mpcard index ) {pAssert( index < m_oldSelection.size()); return m_oldSelection[index];}


protected:
   vector<string> m_selection, m_oldSelection;

   void setSelection(Scene *scene, const vector<string> &selection);
   vector<string> getSelection(Scene *scene);

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void setup(vector<string> selection);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};



class EDITOR_API CmdObjApplyRegisterTransform : public CommandObject {
 struct doUndoItem {
      Matrix4f d;
      Matrix4f u;
   };

 private:
   static MetaClass *CmdObjApplyRegisterTransformMetaClass;
   static Object* newInstance();

 protected:
   Matrix4f d,u;
   string target;
   int32 gridRes,numIterations;
   bool varRes;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void setup(Matrix4f do_trans,Matrix4f undo_trans,string pc,int numIt, int gridRes);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();

   virtual bool mergeCommand(const CommandObject *nextCmd);
};


}

#endif
