//---------------------------------------------------------------------------
#ifndef SceneGraphCmdObjsH
#define SceneGraphCmdObjsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "CommandObject.h"
//---------------------------------------------------------------------------
#include <string>
#include <vector>
#include <set>
//---------------------------------------------------------------------------


namespace geoxl {
   using namespace std;
   

class EDITOR_API CO_SGAddNode : public CommandObject {
 private:
   static MetaClass *CO_SGAddNodeMetaClass;
   static Object* newInstance();

   card32 action;
   string target;
   const MetaClass *newNodeType;
   string undo_newNodeName;

 public:
   static const card32 ADD = 0;
   static const card32 INSERT = 1;

   CO_SGAddNode(string target = "", const MetaClass *newNodeType = NULL, card32 action = 0);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};



class EDITOR_API CO_SGDeleteNode : public CommandObject {
 private:
   static MetaClass *CO_SGDeleteNodeMetaClass;
   static Object* newInstance();

   vector<string> target;

 public:
   CO_SGDeleteNode() {}
   CO_SGDeleteNode(vector<string> target);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};



class EDITOR_API CO_SGMoveNode : public CommandObject {
 private:
   static MetaClass *CO_SGMoveNodeMetaClass;
   static Object* newInstance();

   string target;
   card32 action;

 public:
   static const card32 UP = 1;
   static const card32 DOWN = 2;

   CO_SGMoveNode() {action = 0;}
   CO_SGMoveNode(string target, card32 action);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};



class EDITOR_API CO_SGMoveSelection : public CommandObject {
 private:
   static MetaClass *CO_SGMoveSelectionMetaClass;
   static Object* newInstance();

   string target;
   card32 action;

 public:
   static const card32 MOVE = 1;
   static const card32 LINK = 2;

   CO_SGMoveSelection() {action = 0;}
   CO_SGMoveSelection(string target, card32 action);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};



class EDITOR_API CO_SGGroupSelection : public CommandObject {
 private:
   static MetaClass *CO_SGGroupSelectionMetaClass;
   static Object* newInstance();

   string parent;
   vector<string> children;

 public:
   CO_SGGroupSelection() {}
   CO_SGGroupSelection(string parent, vector<string> children);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};



class EDITOR_API CO_SGUnGroupSelection : public CommandObject {
 private:
   static MetaClass *CO_SGUnGroupSelectionMetaClass;
   static Object* newInstance();

   string target;

 public:
   CO_SGUnGroupSelection() {}
   CO_SGUnGroupSelection(string target);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};



class EDITOR_API CO_SGChangeSelection : public CommandObject {
 private:
   static MetaClass *CO_SGChangeSelectionMetaClass;
   static Object* newInstance();

   set<string> prevSelection;
   set<string> selection;

 public:
   CO_SGChangeSelection() {}
   CO_SGChangeSelection(set<string> selection);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();
   virtual bool mergeCommand(const CommandObject *nextCmd);

   virtual string getDescription();
};

class EDITOR_API CmdObjConvertType : public CommandObject {
 private:
   static MetaClass *CmdObjConvertTypeMetaClass;
   static Object* newInstance();

   int32 action;
   bool delOriginal;
   string filename_depreciated; // not used anylonger
   card32 allocatorIndex; // instead, the allocator is chosen by index from staticState.allocators
   int32 blockSize_depreciated; // not used anylonger
   int32 maxNumNodesInMem;
   int32 maxLeafPts;
   int32 gridSize;

 public:
   CmdObjConvertType(int32 action = 0, bool delOriginal = false, card32 allocatorIndex = 0, 
                     int32 maxNumNodesInMem = 0, int32 maxLeafPts = 0, int32 gridSize = 0);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};


class EDITOR_API CmdObjAppendFields : public CommandObject {
 private:
   static MetaClass *CmdObjAppendFieldsMetaClass;
   static Object* newInstance();

   VertexDescriptor *newFields;

 public:
   CmdObjAppendFields(VertexDescriptor *newFields = NULL);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};



class EDITOR_API CmdObjSGChangeDefaultTraversalMode : public CommandObject {
 private:
   static MetaClass *CmdObjSGChangeDefaultTraversalModeMetaClass;
   static Object* newInstance();

	SGStaticState::TraversalModeType newMode;
   SGStaticState::TraversalModeType oldMode;

 public:
	 CmdObjSGChangeDefaultTraversalMode(SGStaticState::TraversalModeType mode = SGStaticState::tmAllNodes);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doAction(Object *context);
   virtual void undoAction(Object *context);
   virtual bool canUndo();

   virtual string getDescription();
};

}

#endif
