//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCAnimatedTriangleMeshSampler.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
#include "PointCloudTools.h"
#include "TriangleMeshPointCloud.h"
#include "VertexDescriptor.h"
#include "NumericalClassProperty.h"
#include "StringClassProperty.h"
#include "PointSet.h"
#include "Random.h"
#include "SGObjectNode.h"
#include "MathTools.h"
#include "ProgressWindow.h"
#include "PointSet_templCode.h"
#include "SGSimpleAnimationNode.h"
#include "PCCTriangleMeshSampler.h"
#include "UnstructuredInCoreTriangleMesh.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------
MetaClass* PCCAnimatedTriangleMeshSampler::PCCAnimatedTriangleMeshSamplerMetaClass = NULL;
//---------------------------------------------------------------------------

/**
 * Constructor
 */
PCCAnimatedTriangleMeshSampler::PCCAnimatedTriangleMeshSampler() 
{
   expNumPoints = 50000;
   sourceNode = "root/tri";
   destNode = "root/data";
}

void PCCAnimatedTriangleMeshSampler::execute(Scene *scene) 
{
	// Find source node
	SceneGraphNode *sourceN = getSceneGraphNodeByName( scene, sourceNode );
	SGAnimationNode* s = dynamic_cast<SGAnimationNode*>( sourceN );
	if ( s == NULL ){
		throw PException( "PCCAnimatedTriangleMeshSampler::execute() - invalid source node." );
	}
	
	// Find/create destNode
	SceneGraphNode *destN = getSceneGraphNodeByName( scene, destNode );
	SGAnimationNode* d = dynamic_cast<SGAnimationNode*>( destN );
	if ( d == NULL ){
		// Create a new destNode - todo: use given name
		d = new SGSimpleAnimationNode();
		d->setName( "data" );
		SGListNode *rootNode = dynamic_cast<SGListNode*>(scene->getRootNode());
		rootNode->addNode( d );
	}
	for ( unsigned int frameID = 0; frameID < s->getNumChildNodes( scene->getRootState() ); frameID++ ){
		SGObjectNode* sgObjectNode = dynamic_cast<SGObjectNode*>( s->getChildNode( scene->getRootState(), frameID ) );
		UnstructuredInCoreTriangleMesh* sourceMesh = dynamic_cast<UnstructuredInCoreTriangleMesh*>( sgObjectNode->getSceneObject() );
		string frameName = "frame" + QString::number( frameID ).toStdString() + "";
		SGObjectNode* frameNode = new SGObjectNode();
		frameNode->setName( frameName );
		d->addNode( frameNode );
		// Create point cloud
		VertexDescriptor vd = *(sourceMesh->getDescr());
		UnstructuredInCorePointCloud* frame = new UnstructuredInCorePointCloud();
		frame->clearAndSetup( &vd, 0 );
		frameNode->clearAndSetup( frame );
		sampleMeshRandomly( scene, 
			                destNode + "/" + frameName, 
							sourceMesh, 
							frame, 
							expNumPoints,
			                false, 
							-1,false );
	}
}

//---------------------------------------------------------------------------

const MetaClass* PCCAnimatedTriangleMeshSampler::getInstanceClass() const 
{
	return getClass();
}

Object* PCCAnimatedTriangleMeshSampler::newInstance() 
{
	return new PCCAnimatedTriangleMeshSampler;
}

const MetaClass* PCCAnimatedTriangleMeshSampler::getClass() 
{
	if (PCCAnimatedTriangleMeshSamplerMetaClass==NULL) {
		PCCAnimatedTriangleMeshSamplerMetaClass = new MetaClass("PCCAnimatedTriangleMeshSampler", "no description.",
			newInstance, sizeof(PCCAnimatedTriangleMeshSampler));
	}
	return PCCAnimatedTriangleMeshSamplerMetaClass;
}

void PCCAnimatedTriangleMeshSampler::init(const MetaClass *parentClass) 
{
	((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
	PCCAnimatedTriangleMeshSamplerMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( PCCAnimatedTriangleMeshSampler )
	ADD_STRING_PROP( sourceNode, 0 )
	ADD_STRING_PROP( destNode, 0 )
	ADD_CARD64_PROP( expNumPoints, 0 )
}

void PCCAnimatedTriangleMeshSampler::shutdown() 
{
	delete PCCAnimatedTriangleMeshSamplerMetaClass;
}