//---------------------------------------------------------------------------
#ifndef PCCAttributeModifierH
#define PCCAttributeModifierH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
#include "LinearAlgebra.h"
#include "DebugOutput.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
//---------------------------------------------------------------------------
#include "Scene.h"
#include "PointCloud.h"
//---------------------------------------------------------------------------

namespace geoxl {

#define OPERATION_MULTIPLICATION 0 
#define OPERATION_ADDITION 1 
#define OPERATION_SET 2
#define OPERATION_COPY 3

class EDITOR_API PCCAttributeModifier : public PCCommand 
{
	GEOXL_CLASS( PCCAttributeModifier )
	public:

	private:	
		Matrix4f muliplicationMatrix;	/// Multiplication: matrix	
		Vector4f additionVector;		/// Addition: vector
		string attributeID;				/// ID string for the modified attribute
		bool homogenious;				/// Use homogeneous transformation (only for dimension 1-3)
		card32 modificationType;		/// Modification type
		string copyFromAttribute;		/// Attribute to copy data from
		card32 copyFromCoordinate;		/// Coordinate in copyFromAttribute
		card32 copyToCoordinate;		/// Coordinate in copyToAttribute
		float32 alpha;					/// Rotation angle (rotateX, rotateY, rotateZ)

		void rotate( const Matrix3f& R );

   public:
      PCCAttributeModifier();
	  ~PCCAttributeModifier();

      /// Execute command
      virtual void execute(Scene *scene);

	  void rotateX();
	  void rotateY();
	  void rotateZ();
	  void identifyTransformation();
};


}

#endif
 