//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "PCCAutoAlign.h"
//---------------------------------------------------------------------------
#include "BoundingBox.h"
#include "BoundingBox_templCode.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra_templCode.h"
#include "BooleanClassProperty.h"

#include "PointCloudIterators.h"
#include "PointCloudTools.h"

#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

MetaClass* PCCAutoAlign::PCCAutoAlignMetaClass = NULL;

const MetaClass* PCCAutoAlign::getInstanceClass() const {
   return getClass();
}

Object* PCCAutoAlign::newInstance() {
   return new PCCAutoAlign;
}

const MetaClass* PCCAutoAlign::getClass() {
   if (PCCAutoAlignMetaClass==NULL) {
      PCCAutoAlignMetaClass = new MetaClass("PCCAutoAlign", "no description.",
                                  newInstance, sizeof(PCCAutoAlign));
   }
   return PCCAutoAlignMetaClass;
}

void PCCAutoAlign::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   PCCAutoAlignMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(PCCAutoAlign)
   ADD_BOOLEAN_PROP( center , 0)
   ADD_BOOLEAN_PROP( rescale , 0)
   ADD_BOOLEAN_PROP( adjustCamera , 0)
}

void PCCAutoAlign::shutdown() {
   delete PCCAutoAlignMetaClass;
}


// ------------------------------------------------------------

PCCAutoAlign::PCCAutoAlign() {
   center = true;
   rescale = true;
   adjustCamera = false;
}

void PCCAutoAlign::execute(Scene *scene) {
   if (!rescale && !center) return;

   // calc bb
   BoundingBox3f bb;
   bool posset = false;

   unsigned int m;
   vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
   PointCloud* pc;
   for ( m = 0; m < selection.size(); m++ ){
      pc = getPointCloud( scene, selection[m] );

      if (!pc->providesAttribute("position")) continue;
      AAT POSITION = pc->getAAT("position");
      SceneObjectIterator *newIt = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
      if (newIt == NULL) continue;
      BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
      if (basicIt == NULL) {delete newIt; continue;}

      while(!basicIt->atEnd()) {
         Vector3f pos = basicIt->get3f(POSITION);
         if (posset) {
            bb.addPoint(pos);
         } else {
            bb = BoundingBox3f(pos);
            posset = true;
         }
         basicIt->next();
      };

      delete newIt;
   }

   // compute transform
   Matrix4f transform = IDENTITY4F;
   Vector3f mean = bb.getCenter();
   if (center) {
      transform = makeTranslation4f(-mean);
   }
   float32 diameter = bb.getMaxSideLength();
   if (rescale && diameter > 1E-10) {
      float32 s = 2.0/diameter;
      transform = makeScale4f(s,s,s) * transform;
   }

   if ( adjustCamera ){
      /// Only adjust camera
      Camera& cam = scene->getRootState()->staticState->camera;
      cam.setup( cam.getPosition(), mean, cam.getUp() );
   } else {
      // Modify point data of selectec points
      for ( m = 0; m < selection.size(); m++ ){
         pc = getPointCloud( scene, selection[m] );

         if (!pc->providesAttribute("position")) continue;
         AAT POSITION = pc->getAAT("position");

         SceneObjectIterator *newIt = pc->createIterator(SOIT(SOIT::CAP_MODIFYABLE_PC));
         if (newIt == NULL) continue;
         BasicPointCloudIterator *baseIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
         if (baseIt == NULL) {delete newIt; continue;}
         ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(newIt);
         if (modIt == NULL) {delete newIt; continue;}

         while(!baseIt->atEnd()) {
            Vector3f pos = baseIt->get3f(POSITION);
            Vector3f newPos = transformVector3f(transform, pos);
            modIt->set3f(POSITION, newPos);
            baseIt->next();
         };

         delete newIt;
      }
   }
}

