//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCDirectoryImporter.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "PCCImporter.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS(PCCDirectoryImporter, 0)
{
	BEGIN_CLASS_INIT(PCCDirectoryImporter);
   ADD_STRING_PROP(directory, 0)
   ADD_STRING_PROP(fileEnding, 0)
   ADD_OBJECT_PROP(importer, 0, PCCImporter::getClass(), true);
   ADD_BOOLEAN_PROP(ignoreExceptions, 0)
   ADD_BOOLEAN_PROP(outputFiles, 0)
}

PCCDirectoryImporter::PCCDirectoryImporter()
{
   fileEnding = ".dat";
   importer = new PCCImporter();
   ignoreExceptions = false;
   outputFiles = true;
}

PCCDirectoryImporter::~PCCDirectoryImporter()
{
   delete importer;
}

void PCCDirectoryImporter::assign(const Object* obj, CopyContext *context)
{
	copyObjectProperties(obj, this);
}


void PCCDirectoryImporter::execute(Scene *scene) 
{
   importer->connectScene(scene, getCallback(), sceneEditorWidget);
   recursiveCollect(directory);
}

void PCCDirectoryImporter::recursiveCollect(string file) 
{
   QFileInfo info(file.c_str());
   if (info.isDir()) {
      if (outputFiles) debugOutput << file << "\n";
      //debugOutput.flush();
      QDir dir(file.c_str());
      for (unsigned i=0; i<dir.count(); i++) {
         if (!(dir[i] == QString(".") || dir[i] == QString(".."))) {
            recursiveCollect(qString2STLString(dir.absoluteFilePath(dir[i])));
         }
      }
   } else {
      if (info.isFile() && info.exists() && info.isReadable()) {
         bool readme = ((file.size() > 0) && (file.size() >= fileEnding.size()));
         if (readme) {
            for (unsigned i=0; i<fileEnding.size(); i++) {
               if (file[file.size()-fileEnding.size()+i] != fileEnding[i]) {
                  return;
               }
            }
         }
         if (outputFiles) debugOutput << file << "\n";
         //debugOutput.flush();
         try {
            importer->setInputFileName(qString2STLString(QDir::toNativeSeparators(file.c_str())));
            importer->execute(scene);
         } catch (...) {
            if (!ignoreExceptions) {
               throw;
            } else {
               warning("PCCDirectoryImporter::recursiveCollect(): importer has thrown an exception - ignored.");
            }
         }
      }
   }
}
