//---------------------------------------------------------------------------
#ifndef PCCImporterH
#define PCCImporterH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Scene.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
//---------------------------------------------------------------------------

namespace geoxl {


class PointCloudImporterReader;
class SGListNode;


class EDITOR_API PCCImporter : public PCCommand {


private:

	static MetaClass *PCCImporterMetaClass;
	static Object* newInstance();

protected:
	/// filename of import file
	std::string m_InputFileName;
	/// choose subtype of PointCloudImporterReader to define the type of importer you want to use
   PointCloudImporterReader* m_ImporterReader;
	/// name of target point cloud
	std::string m_TargetName;
	/// if true write points directly at coordinate position in parameter domain.
	bool paramWriteByCoords;
   /// attribute channel containing the parametric coordinates
	std::string paramCoordsChannel;
	/// scaling of parameter x coords
   float64 scalePCoordsX;
	/// scaling of parameter y coords
   float64 scalePCoordsY;
	/// offset of parameter x coords
   float64 offsetPCoordsX;
	/// offset of parameter y coords
   float64 offsetPCoordsY;

	/// execute import by reader/adder
	void execImporter(PointCloud *targetPC);
	/// execute import of geoxl .object files, imports the PointClouds or Scenes to the root node
	void execGEOXLObjectImport(SGListNode *listnode);


 public:

   PCCImporter();
   ~PCCImporter();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void execute(Scene *scene);

	inline std::string getInputFileName() { return m_InputFileName; };
	inline void setInputFileName(std::string fn) { m_InputFileName = fn; };

	inline PointCloudImporterReader* getImporterReader() { return m_ImporterReader; };
	void setImporterReader(PointCloudImporterReader* newImp); // not inline
	inline std::string getTargetName() { return m_TargetName; };
	inline void setTargetName(std::string tn) { m_TargetName = tn; }
	inline bool getParamWriteByCoords() { return paramWriteByCoords; }
	inline void setParamWriteByCoords(bool pw) { paramWriteByCoords = pw; }
	inline std::string getParamCoordsChannel() { return paramCoordsChannel; }
	inline void setParamCoordsChannel(std::string c) { paramCoordsChannel = c; }
	inline float64 getPCoordsScaleX() { return scalePCoordsX; }
	inline void setPCoordsScaleX(float64 s) { scalePCoordsX = s; }
	inline float64 getPCoordsScaleY() { return scalePCoordsY; }
	inline void setPCoordsScaleY(float64 s) { scalePCoordsY = s; }
	inline float64 getPCoordsOffsetX() { return offsetPCoordsX; }
	inline void setPCoordsOffsetX(float64 s) { offsetPCoordsX = s; }
	inline float64 getPCoordsOffsetY() { return offsetPCoordsY; }
	inline void setPCoordsOffsetY(float64 s) { offsetPCoordsY = s; }

};


}

#endif
