//---------------------------------------------------------------------------
#ifndef PCCNormalEstimatorPCAH
#define PCCNormalEstimatorPCAH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PCCommand.h"
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PointCloud;
class MemoryManager;

/// Estimate PointCloud Normals by PCA, uses on selected PCs
/// Adds Channels normal, tangent_u and tangent_v if needed 
class EDITOR_API PCCNormalEstimatorPCA : public PCCommand {
   private:
      /*--- META CLASS --- */
      static MetaClass *PCCNormalEstimatorPCAMetaClass;
      static Object* newInstance();

      /*--- MEMBERS --- */
      bool computeTangents;
      // Number of nearest neighbors used for PCA
      card32 nearestNeighbors;
		/// Consistently orient normals for a laser scan with camera position in  (0,0,0)
		bool camInOrigin;
      // unify normal orientation
      bool unifyNormals;
	  // use improved normalization algorithm
	  bool useImprovedAlgorithm;
      // use orientation of normal previously stored in normal channel or recompute orientations?
      bool usePreviousOrientation;
		/// scale computed tangent vectors with individually average point distance
		bool scaleTangentVectors;

		void unifyNormalsByCameraPosition( PointCloud *pc );

   public:
      /// Contructor
      PCCNormalEstimatorPCA();

      /// compatibility with former version
      virtual void read(InputObjectStream *s);

      /// Execute command
      virtual void execute( Scene *scene );

      bool getComputeTangents() {return computeTangents;}
      void setComputeTangents(bool b) {computeTangents = b;}

      /*--- META CLASS ---*/
      virtual const MetaClass *getInstanceClass() const;
      static const MetaClass *getClass();
      static void init(const MetaClass *parentClass);
      static void shutdown();
};


}


#endif
