//---------------------------------------------------------------------------
#ifndef PCCPointCloudCreatorH
#define PCCPointCloudCreatorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <string>
#include "PCCommand.h"
#include "VertexDescriptor.h"
//---------------------------------------------------------------------------

namespace geoxl {


class EDITOR_API PCCPointCloudCreator : public PCCommand {
 private:
   static MetaClass *PCCPointCloudCreatorMetaClass;
   static Object* newInstance();

   std::string m_pcloud_name;
   std::string ooc_filename_depreciated; // not used anylonger
   card32 allocatorIndex;					  // replaced by allocator index (indexing to static scene graph state / allocators)
   VertexDescriptor *m_vd;
   card32 m_type, maxNodeInMem, 
          blockSize_depreciated;         // not used anylonger (same as ooc_filename)
   /// hierarchy termination criterion
   max_platform_card maxNumObjectsPerLeaf;
   /// number of points per bounding box side length for grid stratified multi-res point sets
   card32 multiResGridSize;

 public:
   PCCPointCloudCreator();
   ~PCCPointCloudCreator();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void execute(Scene *scene);
	/// does the real work and is called by execute. Needed for PCCAnimationImporter.
   virtual void executeCreator(Scene *scene);

	inline void setName(string s) { m_pcloud_name = s; };
	inline string getName() { return m_pcloud_name; };
	inline void setType(card32 t) { m_type = t; };
	inline card32 getType() { return m_type; };
	inline void setMaxNumObjectsPerLeaf(max_platform_card x) { maxNumObjectsPerLeaf = x; };
	inline max_platform_card getMaxNumObjectsPerLeaf() { return maxNumObjectsPerLeaf; };
	inline void setMultiResGridSize(card32 x) { multiResGridSize = x; };
	inline card32 getMultiResGridSize() { return multiResGridSize; };
	inline void setMaxNodeInMem(card32 x) { maxNodeInMem = x; };
	inline card32 getMaxNodeInMem() { return maxNodeInMem; };
	inline void setAllocatorIndex(card32 x) { allocatorIndex = x; };
	inline card32 getAllocatorIndex() { return allocatorIndex; };
	inline VertexDescriptor* getVertexDescriptor() { return m_vd; };

};


}

#endif

