//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCPointNoise.h"
//---------------------------------------------------------------------------
#include "Random.h"
#include "BooleanClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "NumericalClassProperty.h"
#include "ClassMethods.h"
#include "SeparatorClassProperty.h"
#include "DebugOutput.h"
#include "MathTools.h"
#include "PointCloudIterators.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "UnstructuredInCorePointCloud.h"
#include "PointCloudTools.h"
#include "SceneGraphState.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "SGSimpleAnimationNode.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

static string describeModificationCoordinate(card32 q)
{
	if (q == 0) return "1st Coordinate";
	else if (q == 1) return "2nd Coordinate";
	else if (q == 2) return "3rd Coordinate";
	else if (q == 3) return "4th Coordinate";
	else if (q == 4) return "All Coordinates";
	else return "Undefined";
}

static string describeModificationType( card32 q )
{
	if (q == 0) return "Random noise";
	else if ( q == 1 ) return "Gaussian noise";
	else return "Undefined";
}

IMPLEMENT_GEOXL_CLASS( PCCPointNoise, 2 )
{
	BEGIN_CLASS_INIT( PCCPointNoise )
	ADD_CARD32_PROP_UPDATE_DESCR( modificationType,0 ,NULL , describeModificationType, 2 )
	ADD_CARD32_PROP_UPDATE_DESCR( modificationCoordinate,0 , NULL, describeModificationCoordinate, 5 )
	ADD_STRING_PROP( attributeID, 0 )
	ADD_FLOAT32_PROP( normalVariance, 0 )
	ADD_FLOAT32_PROP( scale, 0 )
	ADD_BOOLEAN_PROP( modifyAllFrames, 1 );
	ADD_SEPARATOR( "Outliers" )
	ADD_FLOAT32_PROP( addOutliersRatio, 2 )
	ADD_NOARGS_METHOD( addOutliers )
	SET_METHOD_AS_CMD_OBJ( true )
}

PCCPointNoise::PCCPointNoise()
{
	modificationType = 1;
	attributeID = "position";
	modificationCoordinate = 4;
	normalVariance = 0.01f;
	scale = 0.5;
	modifyAllFrames = false;
	addOutliersRatio = 0.2f;
}

void PCCPointNoise::addOutliers()
{
	unsigned int m; 
	vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
	PointCloud* pc;
	for ( m = 0; m < selection.size(); m++ ){
		pc = getPointCloud( scene, selection[m] );	
		if ( pc != NULL ){
			BoundingBox3f& bbox = getPCBBox( pc );
			Vector3f d = bbox.upperCorner - bbox.lowerCorner;
			const card32 numNewPoints = card32( pc->getNumPoints() * addOutliersRatio );
			PointSet* newPS = new PointSet();
			newPS->clearAndSetup( 1, numNewPoints, pc->getDescr() );
			AAT POSITION = newPS->getAAT( "position" );
			for ( unsigned int n = 0; n < numNewPoints; n++ ){
				Vector3f p; 
				for ( unsigned int i = 0; i < 3; i++ ){
					p[i] = bbox.lowerCorner[i] + rand()%1000/1000.0f * d[i];
				}
				newPS->set3f( n, POSITION, p );
			}
			SceneObjectIterator* it = pc->createIterator( SOIT::CAP_BASIC_PC | SOIT::CAP_DYNAMIC_PC );
			DynamicPointCloudIterator* dynIt = dynamic_cast<DynamicPointCloudIterator*>( it );
			dynIt->add( newPS );
			delete it;
			delete newPS;
		}
	}
}

void PCCPointNoise::execute(Scene *scene) {

   unsigned int m; 
   vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
   PointCloud* pc;

   if( !modifyAllFrames )
	{
		for ( m = 0; m < selection.size(); m++ ){
			pc = getPointCloud( scene, selection[m] );

			if (pc && pc->providesAttribute(attributeID)) 
			{
				modifyPointCloud( pc );

			}
		}
	}
	else
	{
		StandardSceneGraphIterator * sgIt = new StandardSceneGraphIterator;

		sgIt->clearAndSetup( scene->getRootNode(), scene->getRootState() );

		while( !sgIt->atEnd() )
		{
			SGAnimationNode * animNode = dynamic_cast<SGAnimationNode*>(sgIt->getNode());
			if( animNode)
			{
				for( card32 i=0;i<animNode->getNumChildNodes(scene->getRootState());i++ )
				{
					SGObjectNode * node = dynamic_cast<SGObjectNode*>(animNode->getChildNode(scene->getRootState(), i ));
					if( node )
					{
						PointCloud * pc = dynamic_cast<PointCloud *>(node->getSceneObject());
						if( pc )
							modifyPointCloud( pc );
					}
				}
			}
				
			sgIt->next();
		}


		delete sgIt;
	}
}

void PCCPointNoise::modifyPointCloud( PointCloud * pc ) 
{
   SceneObjectIterator *newIt=NULL;

   /// Create iterator
   newIt = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC));
   
   if (newIt) {
      BasicPointCloudIterator* basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
      ModifyablePointCloudIterator* modIt = dynamic_cast<ModifyablePointCloudIterator*>(newIt);
      VertexAttribDescriptor attribDesc = pc->getDescr()->getAttribute( attributeID );

      if (basicIt && modIt) {
         while (!basicIt->atEnd()) {

            Vector4f v = makeVector4f(0,0,0,0);

            /// Read attribute
            if ( attribDesc.dataFormat == VertexAttribDescriptor::DATA_FORMAT_CARD8 ) {
               //Application->MessageBox("Not implemented yet.", "Cannot Execute Command...", MB_OK | MB_ICONERROR);
			 WARNINGMESSAGE("Not implemented yet.");
               return;
            } else if ( attribDesc.dataFormat == VertexAttribDescriptor::DATA_FORMAT_INT32 ) {
               //Application->MessageBox("Not implemented yet.", "Cannot Execute Command...", MB_OK | MB_ICONERROR);
			 WARNINGMESSAGE("Not implemented yet.");
               return;
            } else if ( attribDesc.dataFormat == VertexAttribDescriptor::DATA_FORMAT_FLOAT32 ) {
               if ( attribDesc.numComponents == 1 ){
                  float32 attVec = basicIt->get1f( pc->getAAT(attributeID) );
                  v[0] = attVec;
               } else if ( attribDesc.numComponents == 2 ){
                  Vector2f attVec = basicIt->get2f( pc->getAAT(attributeID) );
                  for ( int i = 0; i < 2; i++ ) v[i] = attVec[i];
               } else if ( attribDesc.numComponents == 3 ){
                  Vector3f attVec = basicIt->get3f( pc->getAAT(attributeID) );
                  for ( int i = 0; i < 3; i++ ) v[i] = attVec[i];
               } else if ( attribDesc.numComponents == 4 ){
                  v = basicIt->get4f( pc->getAAT(attributeID) );
               } else {
                  //Application->MessageBox("Invalid number of components.", "Cannot Execute Command...", MB_OK | MB_ICONERROR);
                  WARNINGMESSAGE("Invalid number of components.");
				return;
               }
            } else {
               //Application->MessageBox("Invalid attribute format.", "Cannot Execute Command...", MB_OK | MB_ICONERROR);
			 WARNINGMESSAGE("Invalid attribute format.");
               return;
            }

            /// Modify selected component
            if ( modificationType == 0 ){
               /// Random noise
               if( ( modificationCoordinate == 0 ) || ( modificationCoordinate == 4 ) )
                  v[0] += (rnd01()-0.5f) * scale;
               if( ( modificationCoordinate == 1 ) || ( modificationCoordinate == 4 ) )
                  v[1] += (rnd01()-0.5f) * scale;
               if( ( modificationCoordinate == 2 ) || ( modificationCoordinate == 4 ) )
                  v[2] += (rnd01()-0.5f) * scale;
               if( ( modificationCoordinate == 3 ) || ( modificationCoordinate == 4 ) )
                  v[3] += (rnd01()-0.5f) * scale;
            } if ( modificationType == 1 ){
               /// Gaussian noise
               if( ( modificationCoordinate == 0 ) || ( modificationCoordinate == 4 ) )
                  v[0] += NormalDistribution::randomNumber( 0, normalVariance );
               if( ( modificationCoordinate == 1 ) || ( modificationCoordinate == 4 ) )
                  v[1] += NormalDistribution::randomNumber( 0, normalVariance );
               if( ( modificationCoordinate == 2 ) || ( modificationCoordinate == 4 ) )
                  v[2] += NormalDistribution::randomNumber( 0, normalVariance );
               if( ( modificationCoordinate == 3 ) || ( modificationCoordinate == 4 ) )
                  v[3] += NormalDistribution::randomNumber( 0, normalVariance );
            }

            /// Write attribute back
            if ( attribDesc.dataFormat == VertexAttribDescriptor::DATA_FORMAT_CARD8 ) {
               //Application->MessageBox("Not implemented yet.", "Cannot Execute Command...", MB_OK | MB_ICONERROR);
			 WARNINGMESSAGE("Not implemented yet.");
               return;
            } else if ( attribDesc.dataFormat == VertexAttribDescriptor::DATA_FORMAT_INT32 ) {
               //Application->MessageBox("Not implemented yet.", "Cannot Execute Command...", MB_OK | MB_ICONERROR);
			 WARNINGMESSAGE("Not implemented yet.");
               return;
            } else if ( attribDesc.dataFormat == VertexAttribDescriptor::DATA_FORMAT_FLOAT32 ) {
               if ( attribDesc.numComponents == 1 ){
                  float32 attVec = v[0];
                  modIt->set1f( pc->getAAT(attributeID), attVec );
               } else if ( attribDesc.numComponents == 2 ){
                  Vector2f attVec;
                  for ( int i = 0; i < 2; i++ ) attVec[i] = v[i];
                  modIt->set2f( pc->getAAT(attributeID), attVec );
               } else if ( attribDesc.numComponents == 3 ){
                  Vector3f attVec;
                  for ( int i = 0; i < 3; i++ ) attVec[i] = v[i];
                  modIt->set3f( pc->getAAT(attributeID), attVec );
               } else if ( attribDesc.numComponents == 4 ){
                  modIt->set4f( pc->getAAT(attributeID), v );
               } else {
                  //Application->MessageBox("Invalid number of components.", "Cannot Execute Command...", MB_OK | MB_ICONERROR);
				 WARNINGMESSAGE("Invalid number of components.");
                  return;
               }
            } else {
               //Application->MessageBox("Invalid attribute format.", "Cannot Execute Command...", MB_OK | MB_ICONERROR);
			  WARNINGMESSAGE("Invalid attribute format.");
               return;
            }
            basicIt->next();
         }
      }
      delete newIt;
   }
}
