//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCSortListNode.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "SGListNode.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PCCSortListNode , 0 ) {
	BEGIN_CLASS_INIT( PCCSortListNode );
   ADD_BOOLEAN_PROP(descending, 0)
   ADD_BOOLEAN_PROP(byLengthFirst, 0)
}

// ----

PCCSortListNode::PCCSortListNode() {
   descending = false;
   byLengthFirst = true;
}

void PCCSortListNode::assign(const Object* obj, CopyContext *context) {
	copyObjectProperties(obj, this);
}

struct Entry {
   string name;
   card32 index;
   bool sortByLengthFirst;
   SceneGraphNode *node;
   
   bool operator<(const Entry &other) {
      if (sortByLengthFirst) {
         if (name.length() < other.name.length()) {
            return true;
         } else if (name.length() > other.name.length()) {
            return false;
         }
      }
      return name < other.name;
   }
};

void PCCSortListNode::execute(Scene *scene) {
   vector<string> selection = scene->getRootState()->staticState->getSelectedNodes();
   for (unsigned m=0; m<selection.size(); m++) {
      SGListNode *targetNode = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(scene, selection[m]));
      if (targetNode) {
         SGListNode keepOneReference;
         vector<Entry> toSort;
         const mpcard numNodes = targetNode->getNumChildNodes(NULL);
         for (mpcard i=0; i<numNodes; i++) {
            Entry e;
            e.name = targetNode->getChildNode(NULL, i)->getName();
            e.index = i;
            e.sortByLengthFirst = byLengthFirst;
            e.node = targetNode->getChildNode(NULL, i);
            keepOneReference.addChildNode(e.node);
            toSort.push_back(e);
         }
         targetNode->clear();
         if (descending) {
            sort(toSort.rbegin(), toSort.rend());
         } else {
            sort(toSort.begin(), toSort.end());
         }
         for (unsigned i=0; i<toSort.size(); i++) {
            targetNode->addChildNode(toSort[i].node);
         }
      }
   }
   
}


PCCSortListNode::~PCCSortListNode() {
}



