#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCImportVolumeData.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "FileNamePropertyEditor.h"
#include "MultiResolutionInCoreVolume.h"
#include "RegularInCoreVolume.h"
#include "RegularVolume.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( PCCImportVolumeData ,1)
{
	BEGIN_CLASS_INIT( PCCImportVolumeData );
	ADD_STRING_PROP(m_FileName, 0);
	SET_EDITOR(FileNamePropertyEditor::getClass());
	ADD_STRING_PROP(m_PointCloudName,0);
	ADD_BOOLEAN_PROP(m_MultiRes, 1);

}

PCCImportVolumeData::PCCImportVolumeData(void)
{
	m_PointCloudName = "volume1";
	m_MultiRes = false;
}

PCCImportVolumeData::~PCCImportVolumeData(void)
{
}

void PCCImportVolumeData::assign(const Object* obj, CopyContext *context)
{
	const PCCImportVolumeData * o = dynamic_cast<const PCCImportVolumeData*>(obj);
	pAssert( o != NULL );

	copyObjectProperties(obj,this);
}

void geoxl::PCCImportVolumeData::execute( Scene * scene )
{
	importAndAddToScene(m_FileName, scene);
}

PointCloud* geoxl::PCCImportVolumeData::importAndAddToScene( string filename, Scene * scene )
{

	RegularVolume * volumeData = new RegularVolume;
	if( !volumeData->loadFromMHDRaw(filename.c_str()) )
	{
		debugOutput << "PCCImportVolumeData::importAndAddToScene() - Couldn't load file: " << filename << "\n";
		return NULL;
	}
	if(!m_MultiRes)
	{
		RegularInCoreVolume * volume = new RegularInCoreVolume;
		volume->setVoxels(volumeData);
		addSceneObject(scene,volume,m_PointCloudName);
		return volume;
	}
	else
	{
		MultiResolutionInCoreVolume * volume = new MultiResolutionInCoreVolume;
		Vector3i dim = volumeData->getExtends();
		volume->setVoxels(volumeData);
		addSceneObject(scene,volume,m_PointCloudName);
		return volume;
	}
	return NULL;
}