//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterAdder.h"
//---------------------------------------------------------------------------
#include "ProgressWindow.h"
#include "PointCloudImporterReader.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "Timer.h" 
#include "PointSet.h"
#include "VertexArray.h"
#include "PointSet_templCode.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* PointCloudImporterAdder::PointCloudImporterAdderMetaClass = NULL;

const MetaClass* PointCloudImporterAdder::getInstanceClass() const {
   return getClass();
}

const MetaClass* PointCloudImporterAdder::getClass() {
   if (PointCloudImporterAdderMetaClass == NULL) {
      PointCloudImporterAdderMetaClass = new MetaClass("PointCloudImporterAdder", "no description.",
                                       NULL, sizeof(PointCloudImporterAdder));
   }
   return PointCloudImporterAdderMetaClass;
}

void PointCloudImporterAdder::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   PointCloudImporterAdderMetaClass->copyParentProperties();

}

void PointCloudImporterAdder::shutdown() {
   delete PointCloudImporterAdderMetaClass;
}


PointCloudImporterAdder::PointCloudImporterAdder() {
};




// ---------- Parametric Dynamic Adder----------

MetaClass* PointCloudImporterAdderParametricDynamic::PointCloudImporterAdderParametricDynamicMetaClass = NULL;

const MetaClass* PointCloudImporterAdderParametricDynamic::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterAdderParametricDynamic::newInstance() {
   return new PointCloudImporterAdderParametricDynamic;
}

const MetaClass* PointCloudImporterAdderParametricDynamic::getClass() {
   if (PointCloudImporterAdderParametricDynamicMetaClass == NULL) {
      PointCloudImporterAdderParametricDynamicMetaClass = new MetaClass("PointCloudImporterAdderParametricDynamic", "no description.",
                                       newInstance, sizeof(PointCloudImporterAdderParametricDynamic));
   }
   return PointCloudImporterAdderParametricDynamicMetaClass;
}

void PointCloudImporterAdderParametricDynamic::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   PointCloudImporterAdderParametricDynamicMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( PointCloudImporterAdderParametricDynamic )
	ADD_BOOLEAN_PROP(writeByParameterCoordinates, 0)
	ADD_STRING_PROP(coordinateChannel, 0)
	ADD_FLOAT64_PROP(scalePCoordsX, 0)
	ADD_FLOAT64_PROP(scalePCoordsY, 0)
	ADD_FLOAT64_PROP(offsetPCoordsX, 0)
	ADD_FLOAT64_PROP(offsetPCoordsY, 0)
}

void PointCloudImporterAdderParametricDynamic::shutdown() {
   delete PointCloudImporterAdderParametricDynamicMetaClass;
}


PointCloudImporterAdderParametricDynamic::PointCloudImporterAdderParametricDynamic(bool writeByParameterCoordinates, string coordinateChannel,
                                                         float64 scalePCoordsX, float64 scalePCoordsY,
                                                         float64 offsetPCoordsX, float64 offsetPCoordsY)
{
   this->writeByParameterCoordinates = writeByParameterCoordinates;
   this->coordinateChannel = coordinateChannel;
   this->scalePCoordsX = scalePCoordsX;
   this->scalePCoordsY = scalePCoordsY;
   this->offsetPCoordsX = offsetPCoordsX;
   this->offsetPCoordsY = offsetPCoordsY;
};


void PointCloudImporterAdderParametricDynamic::doImport(PointCloudImporterReader *imp, PointCloud *target) {

   pAssert(target->supportsIteratorType(SOIT::CAP_DYNAMIC_PARAMETRIC_PC));

   SceneObjectIterator *newIt = target->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_DYNAMIC_PARAMETRIC_PC);
   if (newIt == NULL) return;
   BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
   if (basicIt == NULL) {delete newIt; return ;}
   ModifyablePointCloudIterator *modIt = dynamic_cast<ModifyablePointCloudIterator*>(newIt);
   if (modIt == NULL) {delete newIt; return ;}
   DynamicParametricPointCloudIterator *dynparIt = dynamic_cast<DynamicParametricPointCloudIterator*>(newIt);
   if (dynparIt == NULL) {delete newIt; return ;}

   // init progress window
   progressWindow->pushStep(true, "import (param dyn)");
   progressWindow->progress(0);

   card32 width = imp->getWidth();
   card32 height = imp->getHeight();
   dynparIt->setSize(width, height);

   const VertexDescriptor* VDSource = imp->getVertexDescriptor();
   VertexDescriptor* VDTarget = (VertexDescriptor*)target->getDescr();

   if (!writeByParameterCoordinates) {

      vector<card8> card8vector;
      const card8* point = NULL;
      card64 numPoints = 0;

      while (!imp->isFinished()) {

			// update progress window
			numPoints++;
         if (numPoints % 10000 == 0) {
            if (!progressWindow->progress(imp->getProgressPercentage())) { 
					// escape pressed: abort
					delete newIt;
					return;
				};
			};

			point = imp->getNextPoint();
			if (point!=NULL) {
				VDTarget->convertVertices((const char*)point, 1, *VDSource, card8vector);
				modIt->setVertex((const char*)&card8vector[0]);

				basicIt->next();
				pAssert(!( (basicIt->atEnd()) && (!imp->isFinished()) ));
			};

      };

   } else {

      ParametricPointCloudIterator *parIt = dynamic_cast<ParametricPointCloudIterator*>(newIt);
		if (parIt == NULL) {delete newIt; bool dummy; progressWindow->popSteps(dummy); return;}

      if (target->providesAttribute("flags")) {
         AAT FLAGS = target->getAAT("flags");
         for (card32 y=0; y<height; y++) {
            for (card32 x=0; x<width; x++) {
               parIt->goTo(x,y);
               modIt->set1i(FLAGS, PF_FLAG_INVALID);
            }
         }
      }

      if (!VDSource->providesAttribute(coordinateChannel)) {
         WARNINGMESSAGE("PointCloudImporterAdderParametricDynamic::doImport()\nCoordinateChannel not found in source VertexDescriptor!");
			bool dummy;
			progressWindow->popSteps(dummy);
			return;
      }

      VertexAttribDescriptor coordsVad = VDSource->getAttribute(coordinateChannel);
      AAT COORDS_AAT = VDSource->getToken(coordinateChannel);

      if (coordsVad.numComponents != 2) {
         WARNINGMESSAGE("PointCloudImporterAdderParametricDynamic::doImport()\nCoordinateChannel must have dimension 2.");
			bool dummy;
			progressWindow->popSteps(dummy);
			return;
      }

      vector<card8> card8vector;
      const card8* point = NULL;
      card64 numPoints = 0;

      while (!imp->isFinished()) {

			// update progress window
         numPoints++;
			if (numPoints % 10000 == 0) {
            if (!progressWindow->progress(imp->getProgressPercentage())) { 
					// escape pressed: abort
					delete newIt;
					return;
				};
			};

         point = imp->getNextPoint();
			if (point!=NULL) {
				Vector4d coords = coordsVad.getAnyAttrib(point, COORDS_AAT);
				VDTarget->convertVertices((const char*)point, 1, *VDSource, card8vector);

				card32 x = (card32)(scalePCoordsX * coords[0] + offsetPCoordsX);
				card32 y = (card32)(scalePCoordsY * coords[1] + offsetPCoordsY);
				parIt->goTo(x,y);
				modIt->setVertex((const char*)&card8vector[0]);
			};

      };

   }

   delete newIt;

   // finish progress window
   bool dummy;
   progressWindow->popSteps(dummy);
}




// ---------- Dynamic Adder ----------

MetaClass* PointCloudImporterAdderDynamic::PointCloudImporterAdderDynamicMetaClass = NULL;

const MetaClass* PointCloudImporterAdderDynamic::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterAdderDynamic::newInstance() {
   return new PointCloudImporterAdderDynamic;
}

const MetaClass* PointCloudImporterAdderDynamic::getClass() {
   if (PointCloudImporterAdderDynamicMetaClass == NULL) {
      PointCloudImporterAdderDynamicMetaClass = new MetaClass("PointCloudImporterAdderDynamic", "no description.",
                                       newInstance, sizeof(PointCloudImporterAdderDynamic));
   }
   return PointCloudImporterAdderDynamicMetaClass;
}

void PointCloudImporterAdderDynamic::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   PointCloudImporterAdderDynamicMetaClass->copyParentProperties();
}

void PointCloudImporterAdderDynamic::shutdown() {
   delete PointCloudImporterAdderDynamicMetaClass;
}


PointCloudImporterAdderDynamic::PointCloudImporterAdderDynamic() {
};


void PointCloudImporterAdderDynamic::doImport(PointCloudImporterReader *imp, PointCloud *target) {

   Timer timer;
   debugOutput << "PointCloudImporterAdderDynamic::doImport() - import started\n";

   pAssert(target->supportsIteratorType(SOIT::CAP_DYNAMIC_PC));

   SceneObjectIterator *newIt = target->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_DYNAMIC_PC);
   if (newIt == NULL) return;
   BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
   if (basicIt == NULL) {delete newIt; return ;}
   DynamicPointCloudIterator *dynIt = dynamic_cast<DynamicPointCloudIterator*>(newIt);
   if (dynIt == NULL) {delete newIt; return ;}

   // init progress window
   progressWindow->pushStep(true, "import (dyn)");
   progressWindow->progress(0);

	const VertexDescriptor* VDSource = imp->getVertexDescriptor();
   VertexDescriptor* VDTarget = (VertexDescriptor*)target->getDescr();

   vector<card8> card8vector;
   const card8* point = NULL;
   card64 numPoints = 0;
	
	if( !VDTarget->equal( VDSource ) ) // mb - if descriptors are equal, no conversion is needed
	{
		while (!imp->isFinished()) {

			// update progress window
			numPoints++;
			if (numPoints % 10000 == 0) {
				if (!progressWindow->progress(imp->getProgressPercentage())) { 
					// escape pressed: abort
					delete newIt;
					return;
				}
			}

			point = imp->getNextPoint();
			if (point!=NULL) {
				VDTarget->convertVertices((const char*)point, 1, *VDSource, card8vector);
				const char* c = (const char*)&card8vector[0];
				dynIt->add(c);
			}

		}
	}
	else
	{
		while (!imp->isFinished()) {

			// update progress window
			numPoints++;
			if (numPoints % 10000 == 0) {
				if (!progressWindow->progress(imp->getProgressPercentage())) { 
					// escape pressed: abort
					delete newIt;
					return;
				}
			}

			point = imp->getNextPoint();
			if (point!=NULL) 
			{
				dynIt->add( (const char*)point);
			}
		}
	}

   debugOutput << "PointCloudImporterAdderDynamic::doImport() - finished adding points (" << timer.getDeltaValue() << "ms)\n";
   delete newIt;

   debugOutput << "PointCloudImporterAdderDynamic::doImport() - deleted iterator/finished hierarchy creation [if applicable] (additional " << timer.getDeltaValue() << "ms)\n";

	// finish progress window
   bool dummy;
   progressWindow->popSteps(dummy);
}




// ---------- Mesh Adder ----------

MetaClass* PointCloudImporterAdderMesh::PointCloudImporterAdderMeshMetaClass = NULL;

const MetaClass* PointCloudImporterAdderMesh::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterAdderMesh::newInstance() {
   return new PointCloudImporterAdderMesh;
}

const MetaClass* PointCloudImporterAdderMesh::getClass() {
   if (PointCloudImporterAdderMeshMetaClass == NULL) {
      PointCloudImporterAdderMeshMetaClass = new MetaClass("PointCloudImporterAdderMesh", "no description.",
                                       newInstance, sizeof(PointCloudImporterAdderMesh));
   }
   return PointCloudImporterAdderMeshMetaClass;
}

void PointCloudImporterAdderMesh::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   PointCloudImporterAdderMeshMetaClass->copyParentProperties();
}

void PointCloudImporterAdderMesh::shutdown() {
   delete PointCloudImporterAdderMeshMetaClass;
}


PointCloudImporterAdderMesh::PointCloudImporterAdderMesh() {
};


void PointCloudImporterAdderMesh::doImport(PointCloudImporterReader *imp, PointCloud *target) {

	pAssert(target->supportsIteratorType(SOIT::CAP_DYNAMIC_PC | SOIT::CAP_TRIANGLE_MESH));

   SceneObjectIterator *newIt = target->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_DYNAMIC_PC);
   if (newIt == NULL) return;
   BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
   if (basicIt == NULL) {delete newIt; return ;}
   DynamicPointCloudIterator *dynIt = dynamic_cast<DynamicPointCloudIterator*>(newIt);
   if (dynIt == NULL) {delete newIt; return ;}

	// vertex data
	const VertexDescriptor* VDSource = imp->getVertexDescriptor();
   VertexDescriptor* VDTarget = (VertexDescriptor*)target->getDescr();

	// triangle data
	const VertexDescriptor* TDSource = imp->getTriangleDescriptor();
   VertexDescriptor* TDTarget = (VertexDescriptor*)((UnstructuredInCoreTriangleMesh*)target)->getTriangleDescr();
	if ((TDSource==NULL)||(TDTarget==NULL)) {
		WARNINGMESSAGE("PointCloudImporterAdderMesh::doImport()\nFound no triangle descriptor!");
		delete newIt;
		return;
	};
	PointSet* TrianglePS = ((UnstructuredInCoreTriangleMesh*)target)->getTriangles();

	// init progress window
   progressWindow->pushStep(true, "import (mesh)");
   progressWindow->progress(0);

	vector<card8> card8vectorP;
   const card8* point = NULL;

	vector<card8> card8vectorT;
   const card8* tri = NULL;

	card64 numcount = 0;
   card64 numPoints = 0;
   card64 numTriangles = 0;

	Vector3i startVertexIndex3i = makeVector3i(1,1,1)*target->getNumPoints();
	AAT indexAAT = TDTarget->getToken( "index" );
	bool unambiguousTriangulation = false;

   while (!imp->isFinished()) {

		// update progress window
		numcount++;
      if (numcount % 10000 == 0) {
         if (!progressWindow->progress(imp->getProgressPercentage())) { 
				// escape pressed: abort
				delete newIt;
				return;
			};
		};

      point = imp->getNextPoint();
		if (point!=NULL) {
			VDTarget->convertVertices((const char*)point, 1, *VDSource, card8vectorP);
			const char* c = (const char*)&card8vectorP[0];
			dynIt->add(c);
			numPoints++;
		};

      tri = imp->getNextTriangle();
		if (tri!=NULL) {
			TDTarget->convertVertices((const char*)tri, 1, *TDSource, card8vectorT);
			const char* c = (const char*)&card8vectorT[0];
			Vector3i * iPtr = (Vector3i*)(c + indexAAT.getOffset());
			unambiguousTriangulation |= (*iPtr)[2] < 0;
			*iPtr += startVertexIndex3i;
			TrianglePS->addPoint(c);
			numTriangles++;
		};

	};

   delete newIt;

	if (TrianglePS->getNumEntries()<1) {
		WARNINGMESSAGE("PointCloudImporterAdderMesh::doImport()\nFound no triangle/face/mesh data in file! Imported vertex data nevertheless, but triangle PointSet is empty.");
	};

	// mb - triangulate the polygon by adding a new point to the pointcloud (center of mass).
	// For every polygon edge a triangle is created to this new point.
	// Due to the design of the importer, the new point is added outside.
	// A negative vertex index indicates such a new point.
	if( unambiguousTriangulation )
	{
		vector<char> buffer(VDTarget->getSize(),0);
		VertexArray trianglesVA(TrianglePS);
		PointSet * ps = ((UnstructuredInCorePointCloud*)target)->getPointSet();

		Vector3f * posPtr = (Vector3f*)(&buffer[0] + target->getAAT("position").getOffset());
		Vector3f * normalPtr = NULL;
		AAT posAAT = target->getAAT("position");
		AAT normalAAT, colorAAT;
		if( target->providesAttribute("normal"))
		{
			normalPtr = (Vector3f*)(&buffer[0] + target->getAAT("normal").getOffset());
			normalAAT = target->getAAT("normal");
		}
		Vector3f * colorPtr = NULL;
		if( target->providesAttribute("color"))
		{
			colorPtr = (Vector3f*)(&buffer[0] + target->getAAT("color").getOffset());
			colorAAT = target->getAAT("color");
		}

		for( unsigned i=0;i<trianglesVA.getNumElements(); )
		{
			// init buffer
			*posPtr = makeVector3f(0,0,0);
			if( normalPtr )
				*normalPtr = makeVector3f(0,0,0);
			if( colorPtr )
				*colorPtr = makeVector3f(0,0,0);

			// find next polygon
			bool found = false;
			while( i < trianglesVA.getNumElements() )
			{
				if( trianglesVA.getIndex3i(i)[2] < 0)
				{
					found = true;
					break;
				}
				i++;
			}

			if( !found )
				break;

			// calculate center of mass and reset triangle indices
			card32 numPolyPoints = 0;
			int newVertexIndex = ps->getNumEntries();
			int polyID = trianglesVA.getIndex3i(i)[2];
			while( i < trianglesVA.getNumElements() )
			{
				Vector3i tri = trianglesVA.getIndex3i(i);
				if( tri[2] != polyID )
					break;

				tri[2] = newVertexIndex;
				trianglesVA.setIndex3i(i, tri);

				numPolyPoints++;
				*posPtr += ps->get3f(tri[0], posAAT);
				if( normalPtr )
					*normalPtr += ps->get3f(tri[0], normalAAT);
				if( colorPtr )
					*colorPtr += ps->get3f(tri[0], colorAAT);

				i++;
			}

			*posPtr /= numPolyPoints;
			if( normalPtr )
				*normalPtr /= numPolyPoints;
			if( colorPtr )
				*colorPtr /= numPolyPoints;

			// add new vertex
			ps->addPoint(&buffer[0]);
			
		}
	}

	// mb - calculate vertex normals
	//(dynamic_cast<UnstructuredInCoreTriangleMesh*>(target))->calculateVertexNormals();
	
	// finish progress window
   bool dummy;
   progressWindow->popSteps(dummy);
}
