//---------------------------------------------------------------------------
#ifndef PointCloudImporterAdderH
#define PointCloudImporterAdderH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PointCloudImporterReader;

class EDITOR_API PointCloudImporterAdder : public Persistent {

 private:
   static MetaClass *PointCloudImporterAdderMetaClass;

 public:
   PointCloudImporterAdder();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

	/// import point data to target cloud using subclass of PointCloudImporterReader
   virtual void doImport(PointCloudImporterReader *imp, PointCloud *target) = 0;

};


/// adds to parametric-dynamic clouds
class EDITOR_API PointCloudImporterAdderParametricDynamic : public PointCloudImporterAdder {

 private:
   static MetaClass *PointCloudImporterAdderParametricDynamicMetaClass;
   static Object* newInstance();

   /// mw - added: write points directly at coordinate position in parameter domain
   bool writeByParameterCoordinates;
   /// attribute channel containing the parametric coordinates
   string coordinateChannel;
   float64 scalePCoordsX, scalePCoordsY;
   float64 offsetPCoordsX, offsetPCoordsY;


public:
   PointCloudImporterAdderParametricDynamic(bool writeByParameterCoordinates = false, string coordinateChannel = "",
                               float64 scalePCoordsX = 1, float64 scalePCoordsY = 1,
                               float64 offsetPCoordsX = 0, float64 offsetPCoordsY = 0);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doImport(PointCloudImporterReader *imp, PointCloud *target);

};


/// adds to dynamic clouds
class EDITOR_API PointCloudImporterAdderDynamic : public PointCloudImporterAdder {

 private:
   static MetaClass *PointCloudImporterAdderDynamicMetaClass;
   static Object* newInstance();


 public:
   PointCloudImporterAdderDynamic();
   
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doImport(PointCloudImporterReader *imp, PointCloud *target);

};


/// adds to triangle meshes
class EDITOR_API PointCloudImporterAdderMesh : public PointCloudImporterAdder {

 private:
   static MetaClass *PointCloudImporterAdderMeshMetaClass;
   static Object* newInstance();


 public:
   PointCloudImporterAdderMesh();
   
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void doImport(PointCloudImporterReader *imp, PointCloud *target);

};


}

#endif
