//---------------------------------------------------------------------------
#ifndef PointCloudImporterReaderH
#define PointCloudImporterReaderH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {


class EDITOR_API PointCloudImporterReader : public Persistent {

 private:
   static MetaClass *PointCloudImporterReaderMetaClass;


 protected:
	/// buffer for point data
   card8* buffer;
	/// buffer for triangle data
   card8* tribuffer;

	// propertys:
	/// width of imported set of points, may be unknown (0) before end of import
   mpcard width;
	/// true if width is known
   bool IKnowWidth;
	/// height of imported set of points, may be unknown (0) before end of import
	mpcard height;
	/// true if height is known
	bool IKnowHeight;
	/// true if import has been started
   bool ImportStarted;
	/// true if import is finished
	bool ImportFinished;
	/// holds the VertexDescriptor used to import point data
   VertexDescriptor* myVD;
	/// holds the Triangle VertexDescriptor used to import point data, standard NULL, please overwrite if triangles are supported
   VertexDescriptor* myTD;


public:
   PointCloudImporterReader();
   ~PointCloudImporterReader();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	/// returns VertexDescriptor used for import
   const VertexDescriptor* getVertexDescriptor();
	/// returns Triangle VertexDescriptor used for import
   const VertexDescriptor* getTriangleDescriptor();


	/// make preparations to begin import, returns true if successfull
   virtual bool beginImport( string filename, bool readTrianglesIfPossible=false ) = 0;
	/// returns next point, according to VertexDescriptor
   virtual const card8* getNextPoint() = 0;
	/// returns next triangle for mesh import
	virtual const card8* getNextTriangle() = 0;
	/// returns true if import is finished
   inline bool isFinished() { return ImportFinished; };
	/// end import, should reset all so import can be started again
   virtual void endImport() = 0;

	/// returns width of imported set of points. attention: if unknown then possibly runs through whole file to find out and resets importer
   virtual mpcard getWidth() = 0;
	/// returns height of imported set of points. attention: if unknown then possibly runs through whole file to find out and resets importer
	virtual mpcard getHeight() = 0;

	/// returns percentage value of how far import has advanced till yet
	virtual mpcard getProgressPercentage() = 0;

	/// returns true if import-reader is capable of returning triangle mesh data
   virtual bool supportsTriangleImport() = 0;
};

}

#endif
