//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader2DImages.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


MetaClass* PointCloudImporterReader2DImages::PointCloudImporterReader2DImagesMetaClass = NULL;

const MetaClass* PointCloudImporterReader2DImages::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterReader2DImages::newInstance() {
   return new PointCloudImporterReader2DImages;
}

const MetaClass* PointCloudImporterReader2DImages::getClass() {
   if (PointCloudImporterReader2DImagesMetaClass == NULL) {
      PointCloudImporterReader2DImagesMetaClass = new MetaClass("PointCloudImporterReader2DImages", "no description.",
                                       newInstance, sizeof(PointCloudImporterReader2DImages));
   }
   return PointCloudImporterReader2DImagesMetaClass;
}

void PointCloudImporterReader2DImages::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   PointCloudImporterReader2DImagesMetaClass->copyParentProperties();
}

void PointCloudImporterReader2DImages::shutdown() {
   delete PointCloudImporterReader2DImagesMetaClass;
}

PointCloudImporterReader2DImages::PointCloudImporterReader2DImages() {
	invWidth = 0;
	invHeight = 0;
   inv255 = 1.0f/255.0f;

   myImage = NULL;
	x = 0;
   y = 0;

	myVD = new VertexDescriptor();
	myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
	myVD->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
}

PointCloudImporterReader2DImages::~PointCloudImporterReader2DImages() {
   delete myImage;
}

void PointCloudImporterReader2DImages::assign(const Object* obj, CopyContext *context) {
   PointCloudImporterReader::assign(obj,context);

   const PointCloudImporterReader2DImages *other = dynamic_cast<const PointCloudImporterReader2DImages*>(obj);
   pAssert(other != NULL);
}


bool PointCloudImporterReader2DImages::beginImport(string filnam, bool readTrianglesIfPossible) {

   // call beginImport just once
   if (ImportStarted) return false;
	pAssert(buffer == NULL);

	if ( (!myVD->providesAttribute("position")) || (!myVD->providesAttribute("color")) || (!myVD->providesAttribute("flags")) ) {
		WARNINGMESSAGE("PointCloudImporterReader2DImages::beginImport()\nVertex descriptor doesn't have one or more needed attributes (position,color,flags)!");
		return false;
	}

	string ext = extractFileExt(strUCase(filnam));

   if ( !((ext==".BMP")||(ext==".JPG")||(ext==".PNG")) ) {
		// no valid image
		WARNINGMESSAGE("PointCloudImporterReader2DImages::beginImport()\nFile ending is not a valid image type (must be BMP, JPG or PNG)!");
		return false;
	}

	delete myImage;
	myImage = new QImage(QImage(QString(filnam.c_str())).convertToFormat(QImage::Format_RGB32));
	if (myImage->isNull()) {
		WARNINGMESSAGE("PointCloudImporterReader2DImages::beginImport()\nFile \"" + filnam + "\" is not a valid image file!");
		return false;
	};

	width = myImage->width();
	IKnowWidth = true;
	height = myImage->height();
	IKnowHeight = true;
	invWidth = 1.0f;//((float32)width);
	invHeight = 1.0f;//((float32)height);

	x = 0;
	y = 0;

	ImportStarted = false;
	ImportFinished = false;

	delete[] buffer;
	buffer = new card8[myVD->getSize()];

	return true;

}


void PointCloudImporterReader2DImages::endImport() {
	x = 0;
	y = 0;
	ImportStarted = false;
	ImportFinished = false;
	delete[] buffer;
	buffer = NULL;
};


mpcard PointCloudImporterReader2DImages::getWidth() {
   if (IKnowWidth && !ImportStarted) {
      return width;
   } else {
      pAssert(false);
      return 0;
   }
}


mpcard PointCloudImporterReader2DImages::getHeight() {
   if (IKnowHeight && !ImportStarted) {
      return height;
   } else {
      pAssert(false);
      return 0;
   }
}


mpcard PointCloudImporterReader2DImages::getProgressPercentage() {
	return ((y*width+x)*100)/(width*height);
};


const card8* PointCloudImporterReader2DImages::getNextPoint() {

   if (ImportFinished) return NULL;
   pAssert(buffer!=NULL);
   ImportStarted = true;
 
	QRgb pixel = myImage->pixel(x,y);
   Vector3f color = makeVector3f(qRed(pixel),qGreen(pixel),qBlue(pixel))*inv255;

   Vector3f pos;
   pos[0] = x * invWidth;
   pos[1] = y * invWidth;
   pos[2] = 0.0f;

   StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;

	*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("position")))) = pos;
   *((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("color")))) = color;
   *((StaticVector<int32, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("flags")))) = NULL_FLAG;
	if (myVD->providesAttribute("normal")) *((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("normal")))) = makeVector3f(0.0f, 0.0f, 1.0f);

   x++;
   if (x >= width) {
      x = 0;
      y++;
      if (y >= height) ImportFinished = true;
   };

	return buffer;
}


bool PointCloudImporterReader2DImages::supportsTriangleImport() {
	return false;
}


const card8* PointCloudImporterReader2DImages::getNextTriangle() {
	return NULL;
}
