//---------------------------------------------------------------------------
#ifndef PointCloudImporterReader2DImagesH
#define PointCloudImporterReader2DImagesH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader.h"
#include <QtGui/QImage.h>
//---------------------------------------------------------------------------

namespace geoxl {

/// Import Reader for Image files, supported types are: BMP,JPG,PNG
class EDITOR_API PointCloudImporterReader2DImages : public PointCloudImporterReader {

 private:
   static MetaClass *PointCloudImporterReader2DImagesMetaClass;
   static Object* newInstance();

	/// 1/width, needed in getNextPoint() but calculated just once for speed purpose
   float32 invWidth;
	/// 1/height, needed in getNextPoint() but calculated just once for speed purpose
	float32 invHeight;
	/// 1/255, needed in getNextPoint() but calculated just once for speed purpose
	float32 inv255;
	/// image which is actually imported
	QImage* myImage;
	/// actual import x position
   card32 x;
	/// actual import y position
	card32 y;


public:
   PointCloudImporterReader2DImages();
   ~PointCloudImporterReader2DImages();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual bool beginImport(string filenam, bool readTrianglesIfPossible = false);
   virtual const card8* getNextPoint();
	virtual const card8* getNextTriangle();
   virtual void endImport();

   virtual mpcard getWidth();
   virtual mpcard getHeight();

	virtual mpcard getProgressPercentage();

	virtual bool supportsTriangleImport();

};


}

#endif
