//---------------------------------------------------------------------------
#ifndef PointCloudImporterReader2DImagesPlusAlphaH
#define PointCloudImporterReader2DImagesPlusAlphaH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader.h"
#include <QtGui/QImage.h>
//---------------------------------------------------------------------------

namespace geoxl {

/// Import Reader for Image files plus separate Alpha Image. Alpha-Filename has to be Color-Filename + "-mask". Alpha values are written to separate "alpha"-1-card8-attribute channel! Supported image types are: BMP,JPG,PNG
class EDITOR_API PointCloudImporterReader2DImagesPlusAlpha : public PointCloudImporterReader {

 private:
   static MetaClass *PointCloudImporterReader2DImagesPlusAlphaMetaClass;
   static Object* newInstance();

	/// 1/width, needed in getNextPoint() but calculated just once for speed purpose
   float32 invWidth;
	/// 1/height, needed in getNextPoint() but calculated just once for speed purpose
	float32 invHeight;
	/// 1/255, needed in getNextPoint() but calculated just once for speed purpose
	float32 inv255;
	/// image which is actually imported
	QImage* myImage;
	/// depth image which is actually imported
	QImage* myAlphaImage;
	/// actual import x position
   card32 x;
	/// actual import y position
	card32 y;


public:
   PointCloudImporterReader2DImagesPlusAlpha();
   ~PointCloudImporterReader2DImagesPlusAlpha();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual bool beginImport(string filenam, bool readTrianglesIfPossible = false);
   virtual const card8* getNextPoint();
	virtual const card8* getNextTriangle();
   virtual void endImport();

   virtual mpcard getWidth();
   virtual mpcard getHeight();

	virtual mpcard getProgressPercentage();

	virtual bool supportsTriangleImport();

};


}

#endif
