//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderCustom3DScan.h"
//---------------------------------------------------------------------------
#include "ProgressWindow.h"
#include "StringHelper.h"
#include <fstream>
#include <sstream>
#include <QtGui/QImage.h>
#include <QtCore/QFile.h>
#include "DebugOutput.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------
IMPLEMENT_GEOXL_CLASS( PointCloudImporterReaderCustom3DScan, 0 )
{
	BEGIN_CLASS_INIT( PointCloudImporterReaderCustom3DScan )
}

/**
 * Constructor
 */
PointCloudImporterReaderCustom3DScan::PointCloudImporterReaderCustom3DScan() 
{
	pointProgress = 0;
	pointImportFinished = false;
	Positions.clear();
	Colors.clear();
	Normals.clear();

	delete myVD;
	myVD = new VertexDescriptor();
	myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
}

/**
 * Destructor                                                                     
 **/
PointCloudImporterReaderCustom3DScan::~PointCloudImporterReaderCustom3DScan() 
{
}


bool PointCloudImporterReaderCustom3DScan::beginImport( string filename, bool readTrianglesIfPossible ) 
{
	// call beginImport just once
	if (ImportStarted) return false;
	pAssert(buffer == NULL);
	pointImportFinished = false;

	if ( (!myVD->providesAttribute("position")) || (!myVD->providesAttribute("color")) || (!myVD->providesAttribute("flags")) || (!myVD->providesAttribute("normal"))) {
		WARNINGMESSAGE("PointCloudImporterReaderCustom3DScan::beginImport()\nVertex descriptor doesn't have one or more needed attributes (position,color,flags,normal).");
		return false;
	}

	string ext = extractFileExt( strUCase( filename ) );
	if ( ext.compare( ".PNT" ) != 0 ) {
		// no .off
		WARNINGMESSAGE("PointCloudImporterReaderCustom3DScan::beginImport()\nFile ending has to be .PNT, is " + ext + ".");
		return false;
	};

	// clear caches
	Positions.clear();
	Colors.clear();
	Normals.clear();

	// open file
	ifstream in;
	in.clear();
	in.open( filename.c_str(),ifstream::in );
	if (!in.good()) { 
		WARNINGMESSAGE("PointCloudImporterReaderCustom3DScan::beginImport()\nError opening file \"" + filename + "\"!"); 
		return false;
	};
	in.seekg(-1, ios::end);
	card64 fileSize = in.tellg();
	in.seekg(0, ios::beg);

	// init progress window
	//progressWindow->pushStep(true, "loading .pnt");
	//progressWindow->progress(0);

	//code to read one frame from file
	ScanPoint3D* datablock = NULL;
	int size;
	// Open file
	std::fstream file;
	file.open( filename.c_str(), std::ios_base::in|std::ios_base::binary );
	// Read number of points
	file.read( (char*)&size, sizeof(int) );
	// Read data
	datablock = new ScanPoint3D[size];
	file.read( (char*)datablock, size * sizeof( ScanPoint3D ) );
	// Close file
	file.close();
	// Save points
	Positions.clear();
	Normals.clear();
	Colors.clear();
	for ( unsigned long n = 0; n < unsigned long( size ); n++ ){
		Positions.push_back( makeVector3f( datablock[n].px, datablock[n].py, datablock[n].pz ) );
		Normals.push_back( makeVector3f( datablock[n].nx, datablock[n].ny, datablock[n].nz ) );
		Colors.push_back( makeVector3f( datablock[n].r, datablock[n].g, datablock[n].b ) );
	}
	// Free allocated mem
	delete[] datablock;

	//progressWindow->progress(98);

	width = 1;
	height = Positions.size();
	IKnowWidth = true;
	IKnowHeight = true;
	ImportStarted = false;
	ImportFinished = false;
	pointProgress = 0;

	delete[] buffer;
	buffer = new card8[myVD->getSize()];

	// finish progress window
	//bool dummy;
	//progressWindow->popSteps(dummy);

	return true;
}


void PointCloudImporterReaderCustom3DScan::endImport() 
{
	pointProgress = 0;
	Positions.clear();
	Colors.clear();
	Normals.clear();
	ImportFinished = false;
	ImportStarted = false;
	pointImportFinished = false;
	delete[] buffer;
	buffer = NULL;
	delete[] tribuffer;
	tribuffer = NULL;
};


mpcard PointCloudImporterReaderCustom3DScan::getWidth() {
   if (IKnowWidth && !ImportStarted) {
      return width;
   } else {
      pAssert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderCustom3DScan::getHeight() {
   if (IKnowHeight && !ImportStarted) {
      return height;
   } else {
      pAssert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderCustom3DScan::getProgressPercentage() 
{
	size_t sum = Positions.size();
	if ( sum > 0 ){
		return ( pointProgress*100 ) / sum;
	} else {
		return 0;
	}
};


const card8* PointCloudImporterReaderCustom3DScan::getNextPoint() 
{
	if (pointImportFinished||ImportFinished) return NULL;
	pAssert(buffer!=NULL);

	ImportStarted = true;
	StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;

	*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("position")))) = Positions[pointProgress];
	*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("normal")))) = Normals[pointProgress];
	*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("color")))) = Colors[pointProgress];
	*((StaticVector<int32, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("flags")))) = NULL_FLAG;

	pointProgress++;
	if (pointProgress >= Positions.size()) pointImportFinished = true;
	if ( pointImportFinished ) ImportFinished = true;
	return buffer;	
}


bool PointCloudImporterReaderCustom3DScan::supportsTriangleImport() 
{
	return false;
}

void PointCloudImporterReaderCustom3DScan::assign(const Object* obj, CopyContext *context) 
{
	PointCloudImporterReader::assign(obj,context);
	const PointCloudImporterReaderCustom3DScan *other = dynamic_cast<const PointCloudImporterReaderCustom3DScan*>(obj);
	pAssert(other != NULL);
}