//---------------------------------------------------------------------------
#ifndef PointCloudImporterReaderCustom3DScanH
#define PointCloudImporterReaderCustom3DScanH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader.h"
//---------------------------------------------------------------------------

namespace geoxl {

struct ScanPoint3D
{
	float px,py,pz;			//position in 3d
	float nx,ny,nz;			//normal
	unsigned char r,g,b;	//color (only grayscale)
	unsigned int x,y;		//camera pixel position
	short pcol;				//projector column number
};

/// Import Reader for the Dresden Scanner (Gumhold)
class EDITOR_API PointCloudImporterReaderCustom3DScan : public PointCloudImporterReader 
{
	GEOXL_CLASS( PointCloudImporterReaderCustom3DScan )

	private:
		/// import point progress
		card32 pointProgress;
		/// import triangle progress
		card32 triangleProgress;
		/// true if point import is finished
		bool pointImportFinished;
		/// holds all positions
		vector<Vector3f> Positions;
		/// holds all colors
		vector<Vector3f> Colors;
		/// holds all normals
		vector<Vector3f> Normals;

	public:
		/// Constructor
		PointCloudImporterReaderCustom3DScan();
		/// Destructor
		~PointCloudImporterReaderCustom3DScan();

		virtual bool beginImport( string filenam, bool readTrianglesIfPossible = false );
		virtual const card8* getNextPoint();
		virtual const card8* getNextTriangle(){ return NULL; }
		virtual void endImport();
		virtual mpcard getWidth();
		virtual mpcard getHeight();
		virtual mpcard getProgressPercentage();
		virtual bool supportsTriangleImport();

		virtual void assign( const Object* obj, CopyContext *context = NULL );

};

}

#endif
