//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderCustomASCII.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
#include "ImportTools.h"
#include <sstream>
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


MetaClass* PointCloudImporterReaderCustomASCII::PointCloudImporterReaderCustomASCIIMetaClass = NULL;

const MetaClass* PointCloudImporterReaderCustomASCII::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterReaderCustomASCII::newInstance() {
   return new PointCloudImporterReaderCustomASCII;
}

const MetaClass* PointCloudImporterReaderCustomASCII::getClass() {
   if (PointCloudImporterReaderCustomASCIIMetaClass == NULL) {
      PointCloudImporterReaderCustomASCIIMetaClass = new MetaClass("PointCloudImporterReaderCustomASCII", "no description.",
                                       newInstance, sizeof(PointCloudImporterReaderCustomASCII));
   }
   return PointCloudImporterReaderCustomASCIIMetaClass;
}

void PointCloudImporterReaderCustomASCII::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   PointCloudImporterReaderCustomASCIIMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( PointCloudImporterReaderCustomASCII )
	ADD_CARD32_PROP(HeaderType, 0)
	ADD_CARD32_PROP(PresetType, 0)
	ADD_CARD32_PROP(IgnoreFirstLinesNum, 0)
	ADD_STRING_PROP(Separator,1)
}

void PointCloudImporterReaderCustomASCII::shutdown() {
   delete PointCloudImporterReaderCustomASCIIMetaClass;
}

PointCloudImporterReaderCustomASCII::PointCloudImporterReaderCustomASCII() {
	tmpFileName = "";
   progress = 0;

   HeaderType = 0;
   PresetType = 0;
   Separator= " ";
   IgnoreFirstLinesNum = 0;
   CommentMarkers.clear();
	IgnoreColumns.clear();

	myVD = new VertexDescriptor();
	myVD->clear();
//	myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
}

PointCloudImporterReaderCustomASCII::~PointCloudImporterReaderCustomASCII() {
}

void PointCloudImporterReaderCustomASCII::assign(const Object* obj, CopyContext *context) {
	PointCloudImporterReader::assign(obj,context);
   const PointCloudImporterReaderCustomASCII *other = dynamic_cast<const PointCloudImporterReaderCustomASCII*>(obj);
   pAssert(other != NULL);
   this->HeaderType = other->HeaderType;
   this->IgnoreFirstLinesNum = other->IgnoreFirstLinesNum;
   this->CommentMarkers = other->CommentMarkers;
   this->IgnoreColumns = other->IgnoreColumns;
   this->Separator= other->Separator;
}

void PointCloudImporterReaderCustomASCII::write(OutputObjectStream *s) const {
   PointCloudImporterReader::write(s);
   card32 n = (card32)CommentMarkers.size();
   s->write(n);
   for (card32 i=0; i<n; i++) {
      s->writeString(CommentMarkers[i]);
   };
   n = (card32)IgnoreColumns.size();
   s->write(n);
   for (card32 j=0; j<n; j++) {
      s->write(IgnoreColumns[j]);
   };

   s->writeString(Separator);

}

void PointCloudImporterReaderCustomASCII::read(InputObjectStream *s) {
   PointCloudImporterReader::read(s);
   card32 n;
   s->read(n);
   CommentMarkers.resize(n);
   for (card32 i=0; i<n; i++) {
      s->readString(CommentMarkers[i]);
   }
   s->read(n);
   IgnoreColumns.resize(n);
   for (card32 j=0; j<n; j++) {
      s->read(IgnoreColumns[j]);
   }

	if (s->getClassVersion(PointCloudImporterReaderCustomASCII::getClass()) >= MetaClass::FIRST_VERSION + 1) {
		s->readString(Separator);
	}
		
}


bool PointCloudImporterReaderCustomASCII::beginImport(string filnam, bool readTrianglesIfPossible) {

	tmpFileName = filnam;

	// call beginImport just once
   if (ImportStarted) return false;
	pAssert(buffer == NULL);

	{
		QFileInfo qfi(filnam.c_str());
		fileSize = qfi.size();
	}

	in.clear();
	in.open(filnam.c_str(),ifstream::in);
	if (!in.good()) { 
		WARNINGMESSAGE("PointCloudImporterReaderCustomASCII::beginImport()\nError opening file \"" + filnam + "\"!"); 
		return false;
	};

	//in.seekg(-1, ios::end);
	//fileSize = in.tellg();
	//in.seekg(0, ios::beg);

   // skip first # lines
   for (card32 i=0; i<IgnoreFirstLinesNum; i++) skipToEOL(in);

   IKnowWidth = true;
   IKnowHeight = true;
   string forgetMe;

   switch (HeaderType)
   {
      case HT_USERDEFINED:  {
                  if ((width==0) || (height==0)) {
                     IKnowWidth = false;
                     IKnowHeight = false;
                  };
               }
               break;
      case HT_WIDTHHEIGHT:  {
                  in >> width;
                  in >> height;
               }
               break;
      case HT_WIDTHWIDTHHEIGHTHEIGHT:  {
                  in >> forgetMe;
                  in >> width;
                  in >> forgetMe;
                  in >> height;
               }
               break;
		case HT_NUMPOINTS: {
						width = 1;
                  in >> height;
               }
               break;
      default: {
                  IKnowWidth = false;
                  IKnowHeight = false;
               }
               break;
   };

   ImportStarted = false;
   ImportFinished = false;
   progress = 0;

   delete[] buffer;
   buffer = new card8[myVD->getSize()];

	return true;
}


void PointCloudImporterReaderCustomASCII::endImport() {
	progress = 0;
	ImportFinished = false;
	ImportStarted = false;
	if (in.is_open()) in.close();
	delete[] buffer;
	buffer = NULL;
};


mpcard PointCloudImporterReaderCustomASCII::getWidth() {
   if (IKnowWidth && !ImportStarted) {
      return width;
   } else {

      width = 1;
      height = 2;

		while (!isFinished()) {
			getNextPoint();
			height++;
      };

      height -= 2;
      card32 h = height;

      endImport();
      if (!beginImport(tmpFileName)) throw PException("PointCloudImporterReaderCustomASCII::getWidth() - critical error!!");

      width = 1;
      height = h;
      IKnowWidth = true;
      IKnowHeight = true;

      return width;
   };
}


mpcard PointCloudImporterReaderCustomASCII::getHeight() {
   if (IKnowHeight && !ImportStarted) {
      return height;
   } else {

      width = 1;
      height = 2;

      while (!isFinished()) {
			getNextPoint();
         height++;
      };

      height -= 2;
      card32 h = height;

      endImport();
      if (!beginImport(tmpFileName)) throw PException("PointCloudImporterReaderCustomASCII::getHeight() - critical error!!");

      width = 1;
      height = h;
      IKnowWidth = true;
      IKnowHeight = true;

      return height;
   };
}


mpcard PointCloudImporterReaderCustomASCII::getProgressPercentage() {
	if (IKnowWidth&&IKnowHeight) return (progress*100)/(width*height);
	// hopefully doesn't crash
	card64 sp = in.tellg();
	return (sp*100)/fileSize;
};


const card8* PointCloudImporterReaderCustomASCII::getNextPoint() {

   if (ImportFinished) return NULL;
   pAssert(buffer!=NULL);
   pAssert(in.is_open());

	ImportStarted = true;
	

	card32 const nAttribs = myVD->getNumAttribs();
   if (nAttribs == 0) { ImportFinished = true; return NULL; };

	string line;
	bool CLFound = true;
	// take just lines which are not beginning with comment marker
	while (CLFound) {
		getline(in, line);
		line = removeDblSpace(line); // remove multiple spaces between values and make them single space
		line = trimStr(line); // remove beginning and ending spaces from line
		
		CLFound = false;
	   for (card32 j=0; j<CommentMarkers.size(); j++) {
			if (line.size()<CommentMarkers[j].size()) continue;
			for (card32 k=0; k<CommentMarkers[j].size(); k++)
			{
				if (line[k]!=(CommentMarkers[j])[k]) { CLFound = false; break; }
				CLFound = true;
			};
		};
	};

	// ignore specified columns
	card32 columncount = 0;
	vector<string> columns = splitString(line, Separator);
	line = "";
	for (card32 j=0; j<columns.size(); j++) {
		bool found = false;
		for (card32 k=0; k<IgnoreColumns.size(); k++) {
			if (IgnoreColumns[k]==j) found = true;
		}
		if (!found) {
			line = line + columns[j].c_str() + " ";
			columncount++;
		};
	}

//	line = removeDblSpace(line); // remove multiple spaces between values and make them single space

	// test if line is too short and jump over invalid lines
	card32 nAttrColumns = 0;
	for (card32 i=0; i<nAttribs; i++) {
		VAD at = myVD->getAttribute(i);
		nAttrColumns += at.numComponents;
	};
	if (columncount<nAttrColumns) {
		// jump over lines which line don't fit importer settings
		if (in.eof()) ImportFinished = true;
		return NULL;
	};

	card8 *bufferPos = buffer;
	istringstream lineStream;
	lineStream.str(line);

	for (card32 i=0; i<nAttribs; i++) {

      VAD at = myVD->getAttribute(i);

		switch (at.dataFormat)
      {
         case VAD::DATA_FORMAT_CARD8:
            for (card32 i=0; i<at.numComponents; i++)
            {
               card8 value;
               lineStream >> value;
               *((card8*)(bufferPos)) = value;
					bufferPos += sizeof(card8);
//               ((card8*)bufferPos)++;
            }
            break;
         case VAD::DATA_FORMAT_FLOAT32:
            for (card32 i=0; i<at.numComponents; i++)
            {
               float32 value;
               lineStream >> value;
               *((float32*)(bufferPos)) = value;
					bufferPos += sizeof(float32);
//               ((float32*)bufferPos)++;
            }
            break;
         case VAD::DATA_FORMAT_INT32:
            for (card32 i=0; i<at.numComponents; i++)
            {
               int32 value;
               lineStream >> value;
               *((int32*)(bufferPos)) = value;
					bufferPos += sizeof(int32);
//               ((int32*)bufferPos)++;
            }
            break;
         default: pAssert(false); break;
      }  // end switch
   }; // end for nAttribs

   progress++;

	if (IKnowWidth&&IKnowHeight) {
		if (progress >= (width*height)) ImportFinished = true;
	}
   if (in.eof()) ImportFinished = true;

	return buffer;
}


bool PointCloudImporterReaderCustomASCII::supportsTriangleImport() {
	return false;
}


const card8* PointCloudImporterReaderCustomASCII::getNextTriangle() {
	return NULL;
}


