#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderCustomASCIIEditorWidget.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderCustomASCII.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( PointCloudImporterReaderCustomASCII )

PointCloudImporterReaderCustomASCIIEditorWidget::PointCloudImporterReaderCustomASCIIEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	setWindowIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/ASCIIImport.png").c_str()));

	// set presets, has to be done before connect
	ui.CBPresets->addItem("empty preset");
	ui.CBPresets->addItem("position:3f, color:3f, flag:1i");
	ui.CBPresets->addItem("SPH-Data (id:1i, fluid:1ub, m:1f, position:3f, v:3f, rho:1f, T:1f, p:1f, e:1f)");
	ui.CBPresets->addItem("Zoller-Frhlich ASCII (position:3f, intensity:1f, param_coords:2i)");
	ui.CBPresets->addItem("Lucy format (position:3f, normal:3f)");
	ui.CBPresets->addItem("Cedarburg .xyz (position:3f, separator=tab)");
	ui.CBPresets->setCurrentIndex(0);

	// set column separators (file), feel free to insert some more if needed
	ui.CBSeparator->addItem(" (space)"); // space character has to be index 0, don't change!
	ui.CBSeparator->addItem("\t(tab)"); // space character has to be index 1, don't change!
	ui.CBSeparator->addItem(",");
	ui.CBSeparator->addItem("/");
	ui.CBSeparator->addItem(";");
	ui.CBSeparator->setCurrentIndex(0);

	// used Qt auto-connect feature for other slots
	QWidget::connect(ui.RBNone,SIGNAL(clicked()),this,SLOT(sltRadioButtonPressed()));
	QWidget::connect(ui.RBUserDefined,SIGNAL(clicked()),this,SLOT(sltRadioButtonPressed()));
	QWidget::connect(ui.RBWidthHeight,SIGNAL(clicked()),this,SLOT(sltRadioButtonPressed()));
	QWidget::connect(ui.RBWidthHeight2,SIGNAL(clicked()),this,SLOT(sltRadioButtonPressed()));
	QWidget::connect(ui.RBNumPoints,SIGNAL(clicked()),this,SLOT(sltRadioButtonPressed()));
	QWidget::connect(ui.CBPresets,SIGNAL(currentIndexChanged(int)),this,SLOT(sltCBPresetsCurrentIndexChanged(int)));
	QWidget::connect(ui.CBSeparator,SIGNAL(currentIndexChanged(int)),this,SLOT(sltCBSeparatorCurrentIndexChanged(int)));


	// make layout to add VertexDescriptorEditor
	QVBoxLayout *layout1 = new QVBoxLayout;
	layout1->setMargin(0);
	layout1->setSpacing(0);
	ui.VertexDescrBox->setLayout(layout1);

	// init the rest
	vdEditor = NULL;
	vdEditorWidget = NULL;
	presetChangedNotByUser = false;
}

PointCloudImporterReaderCustomASCIIEditorWidget::~PointCloudImporterReaderCustomASCIIEditorWidget(void)
{
}

void PointCloudImporterReaderCustomASCIIEditorWidget::setup( PointCloudImporterReaderCustomASCII * obj )
{
	m_Object = obj;
}

void PointCloudImporterReaderCustomASCIIEditorWidget::updateEditor() {
	ui.SBWidth->setEnabled(false);
	ui.SBHeight->setEnabled(false);
	switch (m_Object->getHeaderType()) {
		case PointCloudImporterReaderCustomASCII::HT_USERDEFINED: ui.RBUserDefined->setChecked(true); ui.SBWidth->setEnabled(true); ui.SBHeight->setEnabled(true); break;
		case PointCloudImporterReaderCustomASCII::HT_WIDTHHEIGHT: ui.RBWidthHeight->setChecked(true); break;
		case PointCloudImporterReaderCustomASCII::HT_WIDTHWIDTHHEIGHTHEIGHT: ui.RBWidthHeight2->setChecked(true); break;
		case PointCloudImporterReaderCustomASCII::HT_NUMPOINTS: ui.RBNumPoints->setChecked(true); break;
		default: ui.RBNone->setChecked(true); break;
	};
	ui.SBWidth->setValue(m_Object->getHeaderWidth());
	ui.SBHeight->setValue(m_Object->getHeaderHeight());
	ui.SBIgnoreFirstLines->setValue(m_Object->getIgnoreFirstLines());
	setCBCommentMarkers(m_Object->getCommentMarkers());
	setCBColumns(m_Object->getIgnoreColumns());
	setCBSeparator(m_Object->getSeparator());

	presetChangedNotByUser = true;
	ui.CBPresets->setCurrentIndex(m_Object->getPresetType());
	presetChangedNotByUser = false;

   // VertexDescriptor Editor
	if (vdEditor != NULL) {
		ui.VertexDescrBox->layout()->removeWidget(vdEditorWidget);
      delete vdEditorWidget;
      delete vdEditor;
      vdEditor = NULL;
      vdEditorWidget = NULL;
   }
	VertexDescriptor *vd = m_Object->getImportVD();
	const MetaClass *mc = vd->getInstanceClass();
   vdEditor = (ClassEditor*)(mc->getGUIDescriptor()->getDefaultClassEditor()->newInstance());
	vdEditorWidget = vdEditor->makeEditor(ui.VertexDescrBox, vd);
	ui.VertexDescrBox->layout()->addWidget(vdEditorWidget);
   vdEditorWidget->show();
   vdEditor->update();
}


// ---------- user functions ----------

void PointCloudImporterReaderCustomASCIIEditorWidget::setCBCommentMarkers(vector<string> list) {
	ui.CBCommentMarkers->clear();
	for (card32 i=0; i<list.size(); i++) {
		ui.CBCommentMarkers->addItem(QString(list[i].c_str()));
	};
	if (ui.CBCommentMarkers->currentIndex()>=0) ui.BTRemoveLines->setEnabled(true);
		else ui.BTRemoveLines->setEnabled(false);
};

vector<string> PointCloudImporterReaderCustomASCIIEditorWidget::getCBCommentMarkers() {
	vector<string> r;
	for (int32 i=0; i<ui.CBCommentMarkers->count(); i++) {
		r.push_back(qPrintable(ui.CBCommentMarkers->itemText(i)));
	}
	return r;
};


void PointCloudImporterReaderCustomASCIIEditorWidget::setCBColumns(vector<card32> list) {
	ui.CBColumns->clear();
	for (card32 i=0; i<list.size(); i++) {
		ui.CBColumns->addItem(QString(list[i]));
	};
	if (ui.CBColumns->currentIndex()>=0) ui.BTRemoveColumns->setEnabled(true);
		else ui.BTRemoveColumns->setEnabled(false);
};

vector<card32> PointCloudImporterReaderCustomASCIIEditorWidget::getCBColumns() {
	vector<card32> r;
	for (int32 i=0; i<ui.CBColumns->count(); i++) {
		QString s = ui.CBColumns->itemText(i);
		bool ok = false;
		int32 strint = s.toInt(&ok);
		if (strint<0) ok = false;
		if (!ok) continue;
		r.push_back((card32)strint);
	}
	return r;
};


void PointCloudImporterReaderCustomASCIIEditorWidget::setCBSeparator(string sep) {
	if (sep==" ") sep=" (space)";
	if (sep=="\t") sep="\t(tab)";
	int32 index = ui.CBSeparator->findText(sep.c_str());
	if (index!=-1) ui.CBSeparator->setCurrentIndex(index);
	else {
		ui.CBSeparator->addItem(sep.c_str());
		index = ui.CBSeparator->findText(sep.c_str());
	}
};


bool PointCloudImporterReaderCustomASCIIEditorWidget::ColumnIsInColumnList(card32 col, vector<card32> colList) {
	for (card32 i=0; i<colList.size(); i++) {
		if (colList[i]==col) return true;
	}
	return false;
}



// ---------- slots ----------

void PointCloudImporterReaderCustomASCIIEditorWidget::sltCBPresetsCurrentIndexChanged(int index) {

	if (presetChangedNotByUser) return;

	// standard preset
	m_Object->setPresetType(index);
	m_Object->setHeaderType(PointCloudImporterReaderCustomASCII::HT_NONE);
	m_Object->setHeaderWidth(0);
	m_Object->setHeaderHeight(0);
	m_Object->setIgnoreFirstLines(0);
	vector<string> commentmarkers;
	m_Object->setCommentMarkers(commentmarkers);
	vector<card32> ignorecolumns;
	m_Object->setIgnoreColumns(ignorecolumns);
	setCBSeparator(" ");
	VertexDescriptor *vd = m_Object->getImportVD();
   vd->clear();

   switch (index)
   {
      case 1:  {
                vd->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
                vd->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
                vd->pushAttrib(mVAD("flag", 1, VAD::DATA_FORMAT_FLOAT32));
               }
               break;
      case 2:  {
                vd->pushAttrib(mVAD("id", 1, VAD::DATA_FORMAT_INT32));
                vd->pushAttrib(mVAD("fluid", 1, VAD::DATA_FORMAT_CARD8));
                vd->pushAttrib(mVAD("m", 1, VAD::DATA_FORMAT_FLOAT32));
                vd->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
                vd->pushAttrib(mVAD("v", 3, VAD::DATA_FORMAT_FLOAT32));
                vd->pushAttrib(mVAD("rho", 1, VAD::DATA_FORMAT_FLOAT32));
                vd->pushAttrib(mVAD("T", 1, VAD::DATA_FORMAT_FLOAT32));
                vd->pushAttrib(mVAD("p", 1, VAD::DATA_FORMAT_FLOAT32));
                vd->pushAttrib(mVAD("e", 1, VAD::DATA_FORMAT_FLOAT32));
               }
               break;
      case 3:  {
                vd->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
                vd->pushAttrib(mVAD("intensity", 1, VAD::DATA_FORMAT_FLOAT32));
                vd->pushAttrib(mVAD("param_coords", 2, VAD::DATA_FORMAT_INT32));
					 m_Object->setHeaderType(PointCloudImporterReaderCustomASCII::HT_USERDEFINED);
					 m_Object->setHeaderWidth(1400);
					 m_Object->setHeaderHeight(8000);
					 m_Object->setIgnoreFirstLines(1);
               }
               break;
      case 4:  {
                vd->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
                vd->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
					 m_Object->setHeaderType(PointCloudImporterReaderCustomASCII::HT_USERDEFINED);
					 m_Object->setHeaderWidth(1);
					 m_Object->setHeaderHeight(15000000);
					 m_Object->setIgnoreFirstLines(5);
               }
               break;
      case 5:  {
                vd->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
					 m_Object->setIgnoreFirstLines(4);
					 setCBSeparator("\t");
               }
               break;

      default: break;
   };
	updateEditor();
}


void PointCloudImporterReaderCustomASCIIEditorWidget::on_SBWidth_valueChanged(int value) {
	m_Object->setHeaderWidth(value);
}


void PointCloudImporterReaderCustomASCIIEditorWidget::on_SBHeight_valueChanged(int value) {
	m_Object->setHeaderHeight(value);
}


void PointCloudImporterReaderCustomASCIIEditorWidget::on_SBIgnoreFirstLines_valueChanged(int value) {
	m_Object->setIgnoreFirstLines(value);
}


void PointCloudImporterReaderCustomASCIIEditorWidget::on_CBCommentMarkers_currentIndexChanged(int index) {
	if (index>=0) {
		ui.BTRemoveLines->setEnabled(true);
	} else {
		ui.BTRemoveLines->setEnabled(false);
	}
	m_Object->setCommentMarkers(getCBCommentMarkers());
}


void PointCloudImporterReaderCustomASCIIEditorWidget::on_BTRemoveLines_clicked() {
	if (ui.CBCommentMarkers->currentIndex()>=0) ui.CBCommentMarkers->removeItem(ui.CBCommentMarkers->currentIndex());
}


void PointCloudImporterReaderCustomASCIIEditorWidget::on_CBColumns_currentIndexChanged(int index) {
	if (index>=0) {
		QString s = ui.CBColumns->itemText(index);
		bool ok = false;
		int32 strint = s.toInt(&ok);
		if (strint<0) ok = false;
		if (!ok) { ui.CBColumns->removeItem(index); return; }
		ui.BTRemoveColumns->setEnabled(true);
	} else {
		ui.BTRemoveColumns->setEnabled(false);
	}
	m_Object->setIgnoreColumns(getCBColumns());
}


void PointCloudImporterReaderCustomASCIIEditorWidget::on_BTRemoveColumns_clicked() {
	if (ui.CBColumns->currentIndex()>=0) ui.CBColumns->removeItem(ui.CBColumns->currentIndex());
}


void PointCloudImporterReaderCustomASCIIEditorWidget::sltRadioButtonPressed() {
	if (ui.RBUserDefined->isChecked()) {
		ui.SBWidth->setEnabled(true);
		ui.SBHeight->setEnabled(true);
	} else {
		ui.SBWidth->setEnabled(false);
		ui.SBHeight->setEnabled(false);
	}

	if (ui.RBUserDefined->isChecked()) m_Object->setHeaderType(PointCloudImporterReaderCustomASCII::HT_USERDEFINED);
	else if (ui.RBWidthHeight->isChecked()) m_Object->setHeaderType(PointCloudImporterReaderCustomASCII::HT_WIDTHHEIGHT);
	else if (ui.RBWidthHeight2->isChecked()) m_Object->setHeaderType(PointCloudImporterReaderCustomASCII::HT_WIDTHWIDTHHEIGHTHEIGHT);
	else if (ui.RBNumPoints->isChecked()) m_Object->setHeaderType(PointCloudImporterReaderCustomASCII::HT_NUMPOINTS);
	else m_Object->setHeaderType(PointCloudImporterReaderCustomASCII::HT_NONE);
}


void PointCloudImporterReaderCustomASCIIEditorWidget::sltCBSeparatorCurrentIndexChanged(int index)
{
	string sep = " ";
   if (index==1) sep = "\t";
	if (index>1) sep = qPrintable(ui.CBSeparator->itemText(index));
	m_Object->setSeparator(sep);
};


void PointCloudImporterReaderCustomASCIIEditorWidget::on_BTGenerateSampleFile_clicked() {
	// init html text
	QString line = "<html><head><meta name=\"qrichtext\" content=\"1\"></head><body style=\"white-space:pre-wrap; font-size:8.25pt; font-weight:400; font-style:normal; text-decoration:none;\"><p style=\"margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">";

	// print ignored first lines
	for (card32 i=0; i<m_Object->getIgnoreFirstLines(); i++) {
		line = line + "--- ignored line " + QString().setNum(i) + "<br>";
	};

	// print header line
	switch (m_Object->getHeaderType()) {
		case PointCloudImporterReaderCustomASCII::HT_WIDTHHEIGHT: line += "%width %height<br>"; break;
		case PointCloudImporterReaderCustomASCII::HT_WIDTHWIDTHHEIGHTHEIGHT: line += "widthtext %width heighttext %height<br>"; break;
		case PointCloudImporterReaderCustomASCII::HT_NUMPOINTS: line += "%numberOfPoints<br>"; break;
		default: break;
	};

	// print sample line according to VertexDescriptor and ignored columns
	card32 nAttribs = m_Object->getImportVD()->getNumAttribs();
	card32 nAttrColumns = 0;
	for (card32 i=0; i<nAttribs; i++) {
		VAD at = m_Object->getImportVD()->getAttribute(i);
		nAttrColumns += at.numComponents;
	};
	vector<card32> ignoredColumns = m_Object->getIgnoreColumns();
	card32 nColumns = nAttrColumns + (card32)ignoredColumns.size();
	card32 columnsCounter = 0;
	card32 ignoredColumnsCounter = 0;
	for (card32 i=0; i<nAttribs; i++) {

		VAD at =  m_Object->getImportVD()->getAttribute(i);
		switch (at.dataFormat)
		{
			case VAD::DATA_FORMAT_CARD8:
				for (int32 j=0; j<((int32)at.numComponents); j++) {
					if (ColumnIsInColumnList(columnsCounter,ignoredColumns)) {
						line += "%IGNORED";
						columnsCounter++;
						if (columnsCounter<nColumns) line += m_Object->getSeparator().c_str();
						j--;
						continue;
					};
					line = line + "%" + (at.name).c_str() + "C" + QString().setNum(j);
					columnsCounter++;
					if (columnsCounter<nColumns) line += m_Object->getSeparator().c_str();
					};
				break;
			case VAD::DATA_FORMAT_FLOAT32:
				for (int32 j=0; j<((int32)at.numComponents); j++) {
					if (ColumnIsInColumnList(columnsCounter,ignoredColumns)) {
						line += "%IGNORED";
						columnsCounter++;
						if (columnsCounter<nColumns) line += m_Object->getSeparator().c_str();
						j--;
						continue;
					};
					line = line + "%" + (at.name).c_str() + "F" + QString().setNum(j);
					columnsCounter++;
					if (columnsCounter<nColumns) line += m_Object->getSeparator().c_str();
				};
				break;
			case VAD::DATA_FORMAT_INT32:
				for (int32 j=0; j<((int32)at.numComponents); j++) {
					if (ColumnIsInColumnList(columnsCounter,ignoredColumns)) {
						line += "%IGNORED";
						columnsCounter++;
						if (columnsCounter<nColumns) line += m_Object->getSeparator().c_str();
						j--;
						continue;
					};
					line = line + "%" + (at.name).c_str() + "I" + QString().setNum(j);
					columnsCounter++;
					if (columnsCounter<nColumns) line += m_Object->getSeparator().c_str();
				};
				break;
			default: pAssert(false); break;
		}  // end switch
	}; // end for nAttribs
	line += "<br>";

	// print sample comment lines
	vector<string> commentMarkers = m_Object->getCommentMarkers();
	for (card32 i=0; i<commentMarkers.size(); i++) {
		line = line + commentMarkers[i].c_str() + "  this comment lines will be jumped over<br>";
	}

	ui.TESampleFile->setHtml(line+"</p></body></html>");
};


#include "PointCloudImporterReaderCustomASCIIEditorWidget_moc.cpp"
