//---------------------------------------------------------------------------
#ifndef PointCloudImporterReaderMSDepthBMPH
#define PointCloudImporterReaderMSDepthBMPH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader.h"
#include <QtGui/QImage.h>
//---------------------------------------------------------------------------

namespace geoxl {


// see: http://research.microsoft.com/IVM/3DVideoDownload/TestProjection.doc
typedef struct {
	double m_K[3][3]; // 3x3 intrinsic matrix
	double m_RotMatrix[3][3]; // rotation matrix
	double m_Trans[3]; // translation vector

	double m_ProjMatrix[4][4]; // projection matrix
} CalibStruct;

/// Import Reader for Microsoft Depth Bitmaps (BMP), needs two files: color-file has to start with "color", depth-file has to have the same name but starting with "depth", e.g. "colorMyImage.bmp" and "depthMyImage.bmp".
class EDITOR_API PointCloudImporterReaderMSDepthBMP : public PointCloudImporterReader {

 private:
   static MetaClass *PointCloudImporterReaderMSDepthBMPMetaClass;
   static Object* newInstance();

	/// 1/width, needed in getNextPoint() but calculated just once for speed purpose
   float32 invWidth;
	/// 1/height, needed in getNextPoint() but calculated just once for speed purpose
	float32 invHeight;
	/// 1/255, needed in getNextPoint() but calculated just once for speed purpose
	float32 inv255;
	/// image which is actually imported
	QImage* myImage;
	/// depth image which is actually imported
	QImage* myDepthImage;
	/// actual import x position
   card32 x;
	/// actual import y position
	card32 y;
	/// 1/MinZ, needed in getNextPoint() but calculated just once for speed purpose
   float32 invMinZ;
	/// 1/MaxZ, needed in getNextPoint() but calculated just once for speed purpose
	float32 invMaxZ;

	// propertys:
	/// minimum z-Value
   float32 MinZ;
	/// maximum z-Value
	float32 MaxZ;

	/// uses old one-image import if true, otherwise imports all 8 images and projects them into the reference system
	bool oldImport;

	card32 positionOffset, colorOffset;
	
	// -------------------------------------------
	//			new importer
	static const int	m_NumCameras = 1;
	QImage*		m_ColorImage[m_NumCameras];
	QImage*		m_DepthImage[m_NumCameras];
	string		m_SettingsFileName;
	card32		actCamera;


	// globals (for illustration only)
	CalibStruct m_CalibParams[8];
	static const int m_Width = 1024;
	static const int m_Height = 768;

	double DepthLevelToZ( unsigned char d );
	double projXYZtoUV(double projMatrix[4][4], 	double x, double y, double z, double *u, double *v);
	void projUVZtoXY(double projMatrix[4][4], double u, double v, double z, double *x, double *y);
	void computeProjectionMatrices();
	bool readCalibrationFile(char *fileName);



 public:
   PointCloudImporterReaderMSDepthBMP();
   ~PointCloudImporterReaderMSDepthBMP();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual bool beginImport(string filenam, bool readTrianglesIfPossible = false);
   virtual const card8* getNextPoint();
	virtual const card8* getNextTriangle();
   virtual void endImport();

   virtual mpcard getWidth();
   virtual mpcard getHeight();

	virtual mpcard getProgressPercentage();

	virtual bool supportsTriangleImport();

	void setup( string settingsFilename ) {m_SettingsFileName = settingsFilename;}
};


}

#endif
