//---------------------------------------------------------------------------
#ifndef PointCloudImporterReaderOFFH
#define PointCloudImporterReaderOFFH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader.h"
//---------------------------------------------------------------------------

namespace geoxl {

/// Import Reader for OFF (offset) file format
class EDITOR_API PointCloudImporterReaderOFF : public PointCloudImporterReader 
{
	GEOXL_CLASS( PointCloudImporterReaderOFF )

	private:
		/// import point progress
		card32 pointProgress;
		/// import triangle progress
		card32 triangleProgress;
		/// true if point import is finished
		bool pointImportFinished;
		/// true if triangle import is finished
		bool triangleImportFinished;
		/// holds all positions
		vector<Vector3f> Positions;
		/// holds all colors
		vector<Vector3f> Colors;
		/// holds all normals
		vector<Vector3f> Normals;
		/// holds all faces
		vector<Vector3i> Faces;
		/// holds all texture coordinates
		vector<Vector2f> TextureCoords;

	public:
		/// Constructor
		PointCloudImporterReaderOFF();
		/// Destructor
		~PointCloudImporterReaderOFF();

		virtual bool beginImport( string filenam, bool readTrianglesIfPossible = false );
		virtual const card8* getNextPoint();
		virtual const card8* getNextTriangle();
		virtual void endImport();
		virtual mpcard getWidth();
		virtual mpcard getHeight();
		virtual mpcard getProgressPercentage();
		virtual bool supportsTriangleImport();

		virtual void assign( const Object* obj, CopyContext *context = NULL );

};

}

#endif
