//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderPLY.h"
//---------------------------------------------------------------------------
#include "StringHelper.h"
#include <sstream>
#include <QtGui/QImage.h>
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


MetaClass* PointCloudImporterReaderPLY::PointCloudImporterReaderPLYMetaClass = NULL;

const MetaClass* PointCloudImporterReaderPLY::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterReaderPLY::newInstance() {
   return new PointCloudImporterReaderPLY;
}

const MetaClass* PointCloudImporterReaderPLY::getClass() {
   if (PointCloudImporterReaderPLYMetaClass == NULL) {
      PointCloudImporterReaderPLYMetaClass = new MetaClass("PointCloudImporterReaderPLY", "no description.",
                                       newInstance, sizeof(PointCloudImporterReaderPLY));
   }
   return PointCloudImporterReaderPLYMetaClass;
}

void PointCloudImporterReaderPLY::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   PointCloudImporterReaderPLYMetaClass->copyParentProperties();

	BEGIN_CLASS_INIT(PointCloudImporterReaderPLY);
	ADD_BOOLEAN_PROP(useUnambigousTriangulation,1);
}

void PointCloudImporterReaderPLY::shutdown() {
   delete PointCloudImporterReaderPLYMetaClass;
}

PointCloudImporterReaderPLY::PointCloudImporterReaderPLY() {
	progress = 0;
	binIn = NULL;
	binReadBuff = new card8[8];
	// asciiIn init?
	Elements.clear();
	PlyImportType = PLY_INVALID;
	actuallyImportedElement = 0;
	SumOfElementCounts = 0;
	pointImportFinished = false;
	triangleImportFinished = false;
	ActualTriangles.clear();
	useUnambigousTriangulation = false;
}

PointCloudImporterReaderPLY::~PointCloudImporterReaderPLY() {
	delete[] binReadBuff;
}

void PointCloudImporterReaderPLY::assign(const Object* obj, CopyContext *context) {
	PointCloudImporterReader::assign(obj,context);
   const PointCloudImporterReaderPLY *other = dynamic_cast<const PointCloudImporterReaderPLY*>(obj);
   pAssert(other != NULL);
}


bool PointCloudImporterReaderPLY::beginImport(string filnam, bool readTrianglesIfPossible) {

	// call beginImport just once
   if (ImportStarted) return false;
	pAssert(buffer == NULL);
	pAssert(tribuffer == NULL);

	string ext = extractFileExt(strUCase(filnam));
   if (ext!=".PLY") {
		WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nFile ending is not .PLY!");
		return false;
	};

	// clear caches
	Elements.clear();
	ActualTriangles.clear();
	pointImportFinished = false;
	triangleImportFinished = false;
	currentFaceID = 1;

	// init triangle descriptor if neccessary
	if (readTrianglesIfPossible)  {
		if (myTD==NULL) myTD = new VertexDescriptor();
		if (!myTD->providesAttribute("index")) myTD->pushAttrib(mVAD("index", 3, VAD::DATA_FORMAT_INT32));
		delete[] tribuffer;
		tribuffer = new card8[myTD->getSize()];
	} else {
		triangleImportFinished = true;
	};

	// read file header
	ifstream in;
	in.clear();
	in.open(filnam.c_str(),ifstream::in);
	if (!in.good()) { 
		WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nError opening file \"" + filnam + "\"!"); 
		return false;
	};
	in.seekg(0, ios::beg);

	string line; // holds actual read line from file
	getline(in, line);
	if (line!="ply") {
		WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nFile \""+ filnam + "\" is not a valid PLY file!"); 
		in.close();
		return false;
	};
	string x;
	bool ascii = false;
	bool bigEndian = false;
	bool littleEndian = false;
	bool needsFirstElement = true;
	element actualElement;
	bool headerFinished = false;
   while (!headerFinished) {
		if (in.eof()) {
			WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nend_header not found!");
			in.close();
			return false;
		};
		getline(in, line);
		line = removeDblSpace(line); // remove multiple spaces between values and make them single space
		line = trimStr(line); // remove beginning and ending spaces from line
		if (line.empty()) continue; // jump over empty lines
		istringstream lineStream;
		lineStream.str(line);
		x = ""; // has to be done due to problems if lineStream is empty!
		lineStream >> x;

		if (x=="format") {
			// format descriptor
			lineStream >> x;
			float32 y;
			lineStream >> y;
			if (y!=1.0f) continue; // just ply-version 1.0 is supported
			
			if (x=="ascii") {
				ascii = true;
			} else if (x=="binary_little_endian") {
				littleEndian = true;
			} else if (x=="binary_big_endian") {
				bigEndian = true;
			} else {
				// unsupported format
				WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nUnsupported PLY file format \"" + x + "\"!");
				in.close();
				return false;
			};

		} else if (x=="element") {
			// defines next element
			lineStream >> x;
			card32 y;
			lineStream >> y;
			if (!needsFirstElement) Elements.push_back(actualElement);
			actualElement = element(x,y);
			SumOfElementCounts += y;
			needsFirstElement = false;

		} else if (x=="property") {
			// defines next property of actual element
			if (needsFirstElement) { 
				WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nFile is not a valid PLY file, missing element definer!"); 
				in.close();
				return false;
			};
			lineStream >> x;
			if (x=="list") {
				// list property
				lineStream >> x;
				card32 lct = calcPropertyType(x);
				string y;
				lineStream >> y;
				card32 pt = calcPropertyType(y);
				string z;
				lineStream >> z;
				actualElement.elementProperties.push_back(elementProperty(pt,z,lct));
			} else {
				// scalar property
				card32 t = calcPropertyType(x);
				string y;
				lineStream >> y;
				actualElement.elementProperties.push_back(elementProperty(t,y));
			};

		} else if (x=="end_header") {
			// end of header
			Elements.push_back(actualElement);
			headerFinished = true;
			continue;

		} else if (x=="comment") {
			// skip comment lines
			continue;

		} else {
			// unknown command in header
			//pAssert(false);
			continue;

		}; // end if x

	}; // end while !in.eof

	// end loading of header
	in.close();

	// analyse header data
	delete myVD;
	myVD = new VertexDescriptor();

	card32 positionCount = 0;
	card32 normalCount = 0;
	card32 colorCount = 0;
	card32 VertexCount = 0;
	card32 FaceCount = 0;

	for (card32 i=0; i<Elements.size(); i++) {

		// make attributes for vertex data
		if (Elements[i].elementName=="vertex") {
			// vertex element
			for (card32 j=0; j<(Elements[i].elementProperties.size()); j++) {
				// all position coordinates?
				string propName = Elements[i].elementProperties[j].propertyName;
				if (propName=="x") positionCount++;
				else if (propName=="y") positionCount++;
				else if (propName=="z") positionCount++;
				// all normal coordinates?
				else if (propName=="nx") normalCount++;
				else if (propName=="ny") normalCount++;
				else if (propName=="nz") normalCount++;
				// all colors?
				else if (propName=="red") colorCount++;
				else if (propName=="green") colorCount++;
				else if (propName=="blue") colorCount++;
				else {
					// maybe automatically make VD-Attribute for other Propertys?
				};
			}
			// adapt VertexDescriptor
			if ((positionCount==3) && (!myVD->providesAttribute("position"))) myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
			if ((normalCount==3) && (!myVD->providesAttribute("normal"))) myVD->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
			if ((colorCount==3) && (!myVD->providesAttribute("color"))) myVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
			VertexCount = Elements[i].elementCount;

		} else if (Elements[i].elementName=="face") {
			// face element
			for (card32 j=0; j<(Elements[i].elementProperties.size()); j++) {
				string propName = Elements[i].elementProperties[j].propertyName;
				if (propName=="vertex_indices") {
					// if face with valid vertex_indices
					FaceCount = Elements[i].elementCount;
				}
			}

		} else if (Elements[i].elementName=="") {
			// maybe others elements like "face" or "egde"

		}; // end if elementName
	}

	if ((!myVD->providesAttribute("position")) || (VertexCount==0)) {
		WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nFile does not define \"vertex x,y,z\" positions! Special ply-formats are not supportet!");
		return false;
	};
	if (!myVD->providesAttribute("flags")) myVD->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));


	if (ascii) {
		// start ascii import
		PlyImportType = PLY_ASCII;
		in.clear();
		asciiIn.open(filnam.c_str(),ifstream::in);
		if (!asciiIn.good()) { 
			WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nError opening ascii file \"" + filnam + "\"!"); 
			asciiIn.close();
			return false;
		};
		asciiIn.seekg(0, ios::beg);
		// jump over header
		line = "";
		while (line!="end_header") {
			if (asciiIn.eof()) {
				WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nend_header not found!");
				asciiIn.close();
				return false;
			};
			getline(asciiIn, line);
			line = removeDblSpace(line);
			line = trimStr(line);
		};

	} else {
		// start binary import
		if (littleEndian) PlyImportType = PLY_BINLittleEndian;
		else if (bigEndian) PlyImportType = PLY_BINBigEndian;
		else {
			WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nUnsupported PLY file format!");
			return false;
		}

	   binIn = fopen(filnam.c_str(), "rb");
		clearerr(binIn);
		if (ferror(binIn)){
			WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nError opening binary file \"" + filnam + "\"!");
			fclose(binIn);
			return false;
		}
		// jump over header
	   char hB[1000];
		hB[0]='x';
		while (!((hB[0]=='e')&&(hB[1]=='n')&&(hB[2]=='d')&&(hB[3]=='_')&&(hB[4]=='h')&&(hB[5]=='e')&&(hB[6]=='a')&&(hB[7]=='d')&&(hB[8]=='e')&&(hB[9]=='r'))) {
			// read stringline
			fgets(hB, 1000, binIn);
			if (feof(binIn)) {
				WARNINGMESSAGE("PointCloudImporterReaderPLY::beginImport()\nend_header not found!");
				fclose(binIn);
				return false;
			}
		};
	}; // end start binary import

	width = 1;
	IKnowWidth = true;
	height = VertexCount;
	IKnowHeight = true;

	ImportStarted = false;
	ImportFinished = false;
	progress = 0;

	delete[] buffer;
	buffer = new card8[myVD->getSize()];

	actuallyImportedElement = 0;

	return true;

}


void PointCloudImporterReaderPLY::endImport() {

	switch (PlyImportType) {
		case PLY_ASCII: asciiIn.close(); break;
		case PLY_BINLittleEndian: fclose(binIn); break;
		case PLY_BINBigEndian: fclose(binIn); break;
		default: break;
	}
	Elements.clear();
	actuallyImportedElement = 0;

	pointImportFinished = false;
	triangleImportFinished = false;
	ActualTriangles.clear();

	progress = 0;
	ImportFinished = false;
	ImportStarted = false;
	delete[] buffer;
	buffer = NULL;
};


mpcard PointCloudImporterReaderPLY::getWidth() {
   if (IKnowWidth && !ImportStarted) {
      return width;
   } else {
      pAssert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderPLY::getHeight() {
   if (IKnowHeight && !ImportStarted) {
      return height;
   } else {
      pAssert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderPLY::getProgressPercentage() {
	card64 totalCount = 0;
	for (card32 i=0; i<actuallyImportedElement; i++) {
		totalCount += Elements[i].elementCount;
	}
	totalCount += progress;
	return (totalCount*100)/SumOfElementCounts;
};


const card8* PointCloudImporterReaderPLY::getNextPoint() {

	if (ImportFinished || pointImportFinished || (!ActualTriangles.empty())) return NULL;
	if (pointImportFinished && triangleImportFinished) { 
		ImportFinished = true;
		return NULL;
	};
   pAssert(buffer!=NULL);

   ImportStarted = true;

	bool bigEndian = (PlyImportType==PLY_BINBigEndian);
	bool importAscii = (PlyImportType==PLY_ASCII);

	string actualElementName = Elements[actuallyImportedElement].elementName;
	bool returnNull = false;

	int8 tmpChar = 0;
	card8 tmpUChar = 0;
	int16 tmpShort = 0;
	card16 tmpUShort = 0;
	int32 tmpInt = 0;
	card32 tmpUInt = 0;
	float32 tmpFloat = 0;
	float64 tmpDouble = 0;

	// handle specific elements
	Vector3f tmpPos;
	Vector3f tmpNormal;
	Vector3f tmpColor;
	if (actualElementName=="vertex") {
		tmpPos = makeVector3f(0.0f,0.0f,0.0f);
		tmpNormal = makeVector3f(0.0f,0.0f,0.0f);
		tmpColor = makeVector3f(0.0f,1.0f,0.0f);
	};
	vector<card32> tmpTriIndices;

	// begin reading
	for (card32 k=0; k<(Elements[actuallyImportedElement].elementProperties.size()); k++) {
		// for all properties:
		card32 actualElementPropertyDataType = Elements[actuallyImportedElement].elementProperties[k].propertyDataType;
		string actualElementPropertyName = Elements[actuallyImportedElement].elementProperties[k].propertyName;

		if (Elements[actuallyImportedElement].elementProperties[k].propertyListCountDataType==PTINVALID) {
			// scalar property value

			// read property data from file
			if (importAscii) {
				// import data in ascii mode
				switch (actualElementPropertyDataType) {
					case PTCHAR:	asciiIn >> tmpChar; break;
					case PTUCHAR:	asciiIn >> tmpUChar; break;
					case PTSHORT:	asciiIn >> tmpShort; break;
					case PTUSHORT:	asciiIn >> tmpUShort; break;
					case PTINT:		asciiIn >> tmpInt; break;
					case PTUINT:	asciiIn >> tmpUInt; break;
					case PTFLOAT:	asciiIn >> tmpFloat; break;
					case PTDOUBLE:	asciiIn >> tmpDouble; break;
					default: pAssert(false); continue;
				};

			} else {
				// import data in binary mode
				switch (actualElementPropertyDataType) {
					case PTCHAR:	tmpChar = readChar(); break;
					case PTUCHAR:	tmpUChar = readUChar(); break;
					case PTSHORT:	tmpShort = readShort(bigEndian); break;
					case PTUSHORT:	tmpUShort = readUShort(bigEndian); break;
					case PTINT:		tmpInt = readInt(bigEndian); break;
					case PTUINT:	tmpUInt = readUInt(bigEndian); break;
					case PTFLOAT:	tmpFloat = readFloat(bigEndian); break;
					case PTDOUBLE:	tmpDouble = readDouble(bigEndian); break;
					default: pAssert(false); continue;
				};
			}

			// map data to attributes
			if (actualElementName=="vertex") { 
				// mapping of vertex element data

				if ( (actualElementPropertyName=="x")||(actualElementPropertyName=="y")||(actualElementPropertyName=="z")
					||(actualElementPropertyName=="nx")||(actualElementPropertyName=="ny")||(actualElementPropertyName=="nz")
					||(actualElementPropertyName=="red")||(actualElementPropertyName=="green")||(actualElementPropertyName=="blue")) {
					// if position, normal or color data

					switch (actualElementPropertyDataType) {
						case PTCHAR:	tmpFloat = (float32)tmpChar / 255.0f; break;
						case PTUCHAR:	tmpFloat = (float32)tmpUChar / 255.0f; break;
						case PTSHORT:	tmpFloat = (float32)tmpShort; break;
						case PTUSHORT:	tmpFloat = (float32)tmpUShort; break;
						case PTINT:		tmpFloat = (float32)tmpInt; break;
						case PTUINT:	tmpFloat = (float32)tmpUInt; break;
						case PTFLOAT:	break;
						case PTDOUBLE:	tmpFloat = (float32)tmpDouble; break;
						default: pAssert(false); continue;
					};

					if (actualElementPropertyName=="x") tmpPos[0] = tmpFloat;
					else if (actualElementPropertyName=="y") tmpPos[1] = tmpFloat;
					else if (actualElementPropertyName=="z") tmpPos[2] = tmpFloat;
					else if (actualElementPropertyName=="nx") tmpNormal[0] = tmpFloat;
					else if (actualElementPropertyName=="ny") tmpNormal[1] = tmpFloat;
					else if (actualElementPropertyName=="nz") tmpNormal[2] = tmpFloat;
					else if (actualElementPropertyName=="red") tmpColor[0] = tmpFloat;
					else if (actualElementPropertyName=="green") tmpColor[1] = tmpFloat;
					else if (actualElementPropertyName=="blue") tmpColor[2] = tmpFloat;
				
				} else {
					// maybe automatically make VD-Attribute for unknown Propertys?

				}; // property name switch


			} else if (actualElementName=="") { 
				// place for mapping of other element data
				returnNull = true;

			} else { 
				// unknown elements
				// maybe automatically map VD-Attribute for unknown Elements&Propertys?
				returnNull = true;

			}; // end of map data to attributes


		} else {
			// list property value

			// read property components count
			card32 compcount = 0;
			if (importAscii) {
				// import data in ascii mode
				switch (Elements[actuallyImportedElement].elementProperties[k].propertyListCountDataType) {
					case PTCHAR:	asciiIn >> tmpChar; compcount = (card32)tmpChar; break;
					case PTUCHAR:	asciiIn >> tmpUChar; compcount = (card32)tmpUChar; break;
					case PTSHORT:	asciiIn >> tmpShort; compcount = (card32)tmpShort; break;
					case PTUSHORT:	asciiIn >> tmpUShort; compcount = (card32)tmpUShort; break;
					case PTINT:		asciiIn >> tmpInt; compcount = (card32)tmpInt; break;
					case PTUINT:	asciiIn >> tmpUInt; compcount = tmpUInt; break;
					case PTFLOAT:	asciiIn >> tmpFloat; compcount = (card32)tmpFloat; break;
					case PTDOUBLE:	asciiIn >> tmpDouble; compcount = (card32)tmpDouble; break;
					default: pAssert(false); continue;
				};

			} else {
				// import data in binary mode
				switch (Elements[actuallyImportedElement].elementProperties[k].propertyListCountDataType) {
					case PTCHAR:	tmpChar = readChar(); compcount = (card32)tmpChar; break;
					case PTUCHAR:	tmpUChar = readUChar(); compcount = (card32)tmpUChar; break;
					case PTSHORT:	tmpShort = readShort(bigEndian); compcount = (card32)tmpShort; break;
					case PTUSHORT:	tmpUShort = readUShort(bigEndian); compcount = (card32)tmpUShort; break;
					case PTINT:		tmpInt = readInt(bigEndian); compcount = (card32)tmpInt; break;
					case PTUINT:	tmpUInt = readUInt(bigEndian); compcount = tmpUInt; break;
					case PTFLOAT:	tmpFloat = readFloat(bigEndian); compcount = (card32)tmpFloat; break;
					case PTDOUBLE:	tmpDouble = readDouble(bigEndian); compcount = (card32)tmpDouble; break;
					default: pAssert(false); continue;
				};
			}

			// read list property data from file
			int32 rootTriIndex = -1; // used for face data
			for (card32 j=0; j<compcount ;j++) {

				if (importAscii) {
					// import data in ascii mode
					switch (actualElementPropertyDataType) {
						case PTCHAR:	asciiIn >> tmpChar; break;
						case PTUCHAR:	asciiIn >> tmpUChar; break;
						case PTSHORT:	asciiIn >> tmpShort; break;
						case PTUSHORT:	asciiIn >> tmpUShort; break;
						case PTINT:		asciiIn >> tmpInt; break;
						case PTUINT:	asciiIn >> tmpUInt; break;
						case PTFLOAT:	asciiIn >> tmpFloat; break;
						case PTDOUBLE:	asciiIn >> tmpDouble; break;
						default: pAssert(false); continue;
					};

				} else {
					// import data in binary mode
					switch (actualElementPropertyDataType) {
						case PTCHAR:	tmpChar = readChar(); break;
						case PTUCHAR:	tmpUChar = readUChar(); break;
						case PTSHORT:	tmpShort = readShort(bigEndian); break;
						case PTUSHORT:	tmpUShort = readUShort(bigEndian); break;
						case PTINT:		tmpInt = readInt(bigEndian); break;
						case PTUINT:	tmpUInt = readUInt(bigEndian); break;
						case PTFLOAT:	tmpFloat = readFloat(bigEndian); break;
						case PTDOUBLE:	tmpDouble = readDouble(bigEndian); break;
						default: pAssert(false); continue;
					};
				}

				// map list data to attributes
				if (actualElementName=="face") { 
					// mapping of vertex element data

					if (actualElementPropertyName=="vertex_indices") {
						// if valid triangle index

						switch (actualElementPropertyDataType) {
							case PTCHAR:	tmpUInt = (card32)tmpChar; break;
							case PTUCHAR:	tmpUInt = (card32)tmpUChar; break;
							case PTSHORT:	tmpUInt = (card32)tmpShort; break;
							case PTUSHORT:	tmpUInt = (card32)tmpUShort; break;
							case PTINT:		tmpUInt = (card32)tmpInt; break;
							case PTUINT:	break;
							case PTFLOAT:	tmpUInt = (card32)tmpFloat; break;
							case PTDOUBLE:	tmpUInt = (card32)tmpDouble; break;
							default: pAssert(false); continue;
						};
						tmpTriIndices.push_back(tmpUInt);

					} else {
						// maybe automatically make VD-Attribute for unknown Propertys?

					}; // end of property name switch

				} else if (actualElementName=="") { 
					// place for mapping of other element data
					returnNull = true;

				} else { 
					// unknown elements
					// maybe automatically map VD-Attribute for unknown Elements&Propertys?
					returnNull = true;

				}; // end of map data to list attributes

			}; // end for compcount


		}; // end of read nolist/list property

	}; // end for all properties


	// finally handle specific elements
	if (actualElementName=="vertex") {
		// vertex: write found data to buffer
		*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("position")))) = tmpPos;
		if (myVD->providesAttribute("normal"))
			*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("normal")))) = tmpNormal;
		if (myVD->providesAttribute("color"))
			*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("color")))) = tmpColor;
		StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;
		*((StaticVector<int32, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("flags")))) = NULL_FLAG;

	} else if ((actualElementName=="face") && !triangleImportFinished) {
		// face: add to current triangle list and return null point
      if (tmpTriIndices.size() >= 3) 
		{
			currentFaceID++;
			if( tmpTriIndices.size() > 3 )
				debugOutput << ":";
			if( !useUnambigousTriangulation || tmpTriIndices.size() == 3)
			{
				for (card32 i=0; i<=tmpTriIndices.size()-3; i++) 
				{
					ActualTriangles.push_back(makeVector3i(tmpTriIndices[0], tmpTriIndices[1+i], tmpTriIndices[2+i]));
				}
			}
			else
			{
				// mb - triangulate the polygon by adding a new point to the pointcloud (center of mass).
				// For every polygonedge a triangle is created to this new point.
				// Due to the design of the importer, the new point is added outside.
				// A negative vertex index indicates such a new point.
				debugOutput << ".";
				for (card32 i=0; i<tmpTriIndices.size(); i++) 
				{
					ActualTriangles.push_back(makeVector3i(tmpTriIndices[i], tmpTriIndices[(i+1)%tmpTriIndices.size()], -currentFaceID));
				}

			}
      }
		returnNull = true;

	} else {
		// not used till yet
		returnNull = true;
	};

	progress++;
	if (progress >= Elements[actuallyImportedElement].elementCount) {
		actuallyImportedElement++;
		progress = 0;
	}

	if (actuallyImportedElement>=Elements.size()) pointImportFinished = true;
	if (importAscii) {
		if (asciiIn.eof()) pointImportFinished = true;
	} else {
		if (feof(binIn)) pointImportFinished = true;
	}

	if (pointImportFinished && triangleImportFinished) ImportFinished = true;
	if (returnNull) return NULL;
	else return buffer;

}


bool PointCloudImporterReaderPLY::supportsTriangleImport() {
	return true;
}


const card8* PointCloudImporterReaderPLY::getNextTriangle() {
	if (ActualTriangles.empty()) {
		if (pointImportFinished) { 
			triangleImportFinished = true;
			ImportFinished = true;
		} else {
			return NULL;
		}
	};
   if (triangleImportFinished||ImportFinished) return NULL;
   pAssert(tribuffer!=NULL);

   ImportStarted = true;

   *((StaticVector<int32, 3>*)(tribuffer + myTD->getAttributeOffset(myTD->getIndex("index")))) = ActualTriangles[ActualTriangles.size()-1];
	ActualTriangles.pop_back();

	if (pointImportFinished && triangleImportFinished) ImportFinished = true;
	return tribuffer;
}


const card32 PointCloudImporterReaderPLY::calcPropertyType(string s) {
	if (s=="char") return PTCHAR;
		else if (s=="uchar") return PTUCHAR; 
		else if (s=="uint8") return PTUCHAR; 
		else if (s=="short") return PTSHORT;
		else if (s=="ushort") return PTUSHORT;
		else if (s=="int") return PTINT;
		else if (s=="int32") return PTINT;
		else if (s=="uint") return PTUINT;
		else if (s=="float") return PTFLOAT;
		else if (s=="float32") return PTFLOAT;
		else if (s=="double") return PTDOUBLE;
		else return PTINVALID;
};
