//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderSMFOBJ.h"
//---------------------------------------------------------------------------
#include "ProgressWindow.h"
#include "StringHelper.h"
#include <fstream>
#include <sstream>
#include <QtGui/QImage.h>
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


MetaClass* PointCloudImporterReaderSMFOBJ::PointCloudImporterReaderSMFOBJMetaClass = NULL;

const MetaClass* PointCloudImporterReaderSMFOBJ::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterReaderSMFOBJ::newInstance() {
   return new PointCloudImporterReaderSMFOBJ;
}

const MetaClass* PointCloudImporterReaderSMFOBJ::getClass() {
   if (PointCloudImporterReaderSMFOBJMetaClass == NULL) {
      PointCloudImporterReaderSMFOBJMetaClass = new MetaClass("PointCloudImporterReaderSMFOBJ", "no description.",
                                       newInstance, sizeof(PointCloudImporterReaderSMFOBJ));
   }
   return PointCloudImporterReaderSMFOBJMetaClass;
}

void PointCloudImporterReaderSMFOBJ::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   PointCloudImporterReaderSMFOBJMetaClass->copyParentProperties();
}

void PointCloudImporterReaderSMFOBJ::shutdown() {
   delete PointCloudImporterReaderSMFOBJMetaClass;
}

PointCloudImporterReaderSMFOBJ::PointCloudImporterReaderSMFOBJ() {
	pointProgress = 0;
	triangleProgress = 0;
	pointImportFinished = false;
	triangleImportFinished = false;
	Positions.clear();
	Colors.clear();
	Normals.clear();
	Faces.clear();
	TextureCoords.clear();

	delete myVD;
	myVD = new VertexDescriptor();
	myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
}

PointCloudImporterReaderSMFOBJ::~PointCloudImporterReaderSMFOBJ() {
}

void PointCloudImporterReaderSMFOBJ::assign(const Object* obj, CopyContext *context) {
	PointCloudImporterReader::assign(obj,context);
   const PointCloudImporterReaderSMFOBJ *other = dynamic_cast<const PointCloudImporterReaderSMFOBJ*>(obj);
   pAssert(other != NULL);
}


bool PointCloudImporterReaderSMFOBJ::beginImport(string filnam, bool readTrianglesIfPossible) {

	// call beginImport just once
   if (ImportStarted) return false;
	pAssert(buffer == NULL);
	pAssert(tribuffer == NULL);
	pointImportFinished = false;
	triangleImportFinished = false;

	if ( (!myVD->providesAttribute("position")) || (!myVD->providesAttribute("color")) || (!myVD->providesAttribute("flags")) || (!myVD->providesAttribute("normal"))) {
		WARNINGMESSAGE("PointCloudImporterReaderSMFOBJ::beginImport()\nVertex descriptor doesn't have one or more needed attributes (position,color,flags,normal).");
		return false;
	}

	string ext = extractFileExt(strUCase(filnam));
   if (!((ext==".SMF")||(ext==".OBJ"))) {
		// no .smf or .obj
		WARNINGMESSAGE("PointCloudImporterReaderSMFOBJ::beginImport()\nFile ending has to be .SMF or .OBJ");
		return false;
	};

	// init triangle descriptor if neccessary
	if (readTrianglesIfPossible)  {
		if (myTD==NULL) myTD = new VertexDescriptor();
		if (!myTD->providesAttribute("index")) myTD->pushAttrib(mVAD("index", 3, VAD::DATA_FORMAT_INT32));
		delete[] tribuffer;
		tribuffer = new card8[myTD->getSize()];
	} else {
		triangleImportFinished = true;
	};

	// clear caches
	Positions.clear();
	Colors.clear();
	Normals.clear();
	Faces.clear();
	TextureCoords.clear();

	// open file
	ifstream in;
	in.clear();
	in.open(filnam.c_str(),ifstream::in);
	if (!in.good()) { 
		WARNINGMESSAGE("PointCloudImporterReaderSMFOBJ::beginImport()\nError opening file \"" + filnam + "\"!"); 
		return false;
	};
	in.seekg(-1, ios::end);
	card64 fileSize = in.tellg();
	in.seekg(0, ios::beg);

   // init progress window
   progressWindow->pushStep(true, "loading smf/obj");
   progressWindow->progress(0);

	// load from file to caches
	string line;
	card64 lineCount = 0;
	string x;
	float32 v1, v2, v3;
	bool bindColorToVertex = true;
	bool bindNormalToVertex = false;
	bool hasObjTexture = false;
	bool objMtlDefined = false;

   while (!in.eof()) {
		getline(in, line);
		lineCount++;
		line = removeDblSpace(line); // remove multiple spaces between values and make them single space
		line = trimStr(line); // remove beginning and ending spaces from line
		istringstream lineStream;
		lineStream.str(line);
		x = ""; // has to be done due to problems if lineStream is empty!
		lineStream >> x;

		if (x=="v") {
			// vertex position
		   lineStream >> v1; lineStream >> v2; lineStream >> v3;
			Positions.push_back(makeVector3f(v1,v2,v3));
		} else if ((x=="f")||(x=="t")) {
			// face:  "f p p p" or "f p/t/n ..."
			// attention: "t" is a quick-hack for smf's with triangles, don't know if there will be some problems with other formats
			vector<int32> pindices;
			vector<int32> tindices;
			vector<int32> nindices;
			if ((line.find_first_of("f ")==0)||(line.find_first_of("t ")==0)) line = line.substr(2,line.length());
			vector<string> faceEntries = splitString(line, " ");
			for (card32 i=0; i<faceEntries.size(); i++) {
				vector<string> faceEntryIndexes = splitString(faceEntries[i], "/");
				// position
				int32 posIdx = 0;
				bool success = strToInt(faceEntryIndexes[0], posIdx);
				if (success) pindices.push_back(posIdx);
				if (x=="f") pindices.back()--; // idx begins with 1 but vertex list with 0 if "f"
/*				if (faceEntryIndexes.size()>2) {
					// texture
					int32 texIdx = 0;
					success = strToInt(faceEntryIndexes[1], texIdx);
					if (success) tindices.push_back(texIdx);
					if (x=="f") tindices.back()--; // idx begins with 1 but vertex list with 0 if "f"
					// normal
					int32 normIdx = 0;
					success = strToInt(faceEntryIndexes[2], normIdx);
					if (success) nindices.push_back(normIdx);
					if (x=="f") nindices.back()--; // idx begins with 1 but vertex list with 0 if "f"
				} else if (faceEntryIndexes.size()>1) {
					// normal
					int32 normIdx = 0;
					success = strToInt(faceEntryIndexes[1], normIdx);
					if (success) nindices.push_back(normIdx);
					if (x=="f") nindices.back()--; // idx begins with 1 but vertex list with 0 if "f"
				}
*/			}
         if (pindices.size() >= 3) {
				// force triangles (mabye quads in file)
            for (card32 i=0; i<=pindices.size()-3; i++) {
					Faces.push_back(makeVector3i(pindices[0], pindices[1+i], pindices[2+i]));
            }
         }
			// todo: handle texture information
			// todo: handle normal information

		} else if (x=="c") {
			// color
	      lineStream >> v1; lineStream >> v2; lineStream >> v3;
	      Colors.push_back(makeVector3f(v1,v2,v3));
		} else if (x=="n") {
			// normal (smf only)
	      lineStream >> v1; lineStream >> v2; lineStream >> v3;
	      Normals.push_back(makeVector3f(v1,v2,v3));
		  bindNormalToVertex = true;
		} else if (x=="vn") {
			// normal (obj only)
	      lineStream >> v1; lineStream >> v2; lineStream >> v3;
	      Normals.push_back(makeVector3f(v1,v2,v3));
			bindNormalToVertex = true;
		} else if (x=="vt") {
			// texture coordinates (obj only)
	      lineStream >> v1; lineStream >> v2;
	      TextureCoords.push_back(makeVector2f(v1,v2));
			hasObjTexture = true;
			bindColorToVertex = true;
		} else if ((x=="mtllib") || (x=="usemtl")) {
			// .obj material library files not supported!
			objMtlDefined = true;
		} else if (x=="bind") {
			lineStream >> x;
			if (x=="c") {
				lineStream >> x;
				if (x=="vertex") {
					bindColorToVertex = true;
				} else if (x=="face") {
					bindColorToVertex = false;
				} else {
					// not a supported binding
					continue;
				}
			} else if (x=="n") {
				lineStream >> x;
				if (x=="vertex") {
					bindNormalToVertex = true;
				} else if (x=="face") {
					bindNormalToVertex = false;
				} else {
					// not a supported binding
					continue;
				}
			} else {
				// not a supported binding
				continue;
			};
		} else {
			// jump over comments and unsupported or invalid lines
			continue;
		};

		// update progress window
		if (lineCount % 10000 == 0) {
			// hopefully doesn't crash
			card64 sp = in.tellg();
         if (!progressWindow->progress((sp*100)/fileSize)) { 
				// escape pressed: abort
				in.close();
				return false;
			};
		};

	}; // end while !in.eof

	// end loading from file
	in.close();
   progressWindow->progress(96);


	if (bindNormalToVertex) {
		// one normal per vertex in file
		pAssert(Normals.size()>=Positions.size());

		// todo: lookup face normal indices, without that maybe results in wrong normal, see above faceEntryIndexes[2]


	} else {
		// one normal per face or no normals in file, so calculate vertex normals from faces
		Vector3f tempVF;

		if (Normals.empty()) {
			// calc one normal per face
			for (card32 i=0; i<Faces.size(); i++) {
				tempVF = (Positions[(Faces[i])[1]]-Positions[(Faces[i])[0]]).crossProduct(Positions[(Faces[i])[2]]-Positions[(Faces[i])[0]]);
				tempVF = normalize(tempVF);
				Normals.push_back(tempVF);
			};
		};

		pAssert(Normals.size()>=Faces.size());
		vector<Vector3f> tmpNormals;

		// initialize with zero-Normals first
		for (card32 i=0; i<Positions.size(); i++) {
			tmpNormals.push_back(makeVector3f(0.0f,0.0f,0.0f));
		};

		// for every face add face normal to edge vertices normal
		for (card32 i=0; i<Faces.size(); i++) {
			tempVF = Normals[i];
			tmpNormals[(Faces[i])[0]] += tempVF;
			tmpNormals[(Faces[i])[1]] += tempVF;
			tmpNormals[(Faces[i])[2]] += tempVF;
		}

		// finally add normalized normals
		Normals.clear();
		for (card32 i=0; i<Positions.size(); i++) {
			Normals.push_back(normalize(tmpNormals[i]));
		};
		tmpNormals.clear();

	};	// end if bindNormalToVertex

   progressWindow->progress(98);

	if (bindColorToVertex) {
		// one color per vertex, no colors in file or obj texture
		if (Colors.empty()) {
			bool foundTextureFile = false;
			if (hasObjTexture && objMtlDefined) {
				// test if texture exists without obj-material-file
				string FileNameTex = filnam.replace(filnam.size()-3, 3, "bmp");
				if (geoxl::FileExists(FileNameTex.c_str())) foundTextureFile = true;
			}
			if ((hasObjTexture && !objMtlDefined) || foundTextureFile) {
				// is obj with texture and no obj-material-file defined
				bool textureFileNotValid = false;
				pAssert(TextureCoords.size()>=Positions.size());
				string FileNameTex = filnam.replace(filnam.size()-3, 3, "bmp");
				if (!geoxl::FileExists(FileNameTex.c_str())) {
					WARNINGMESSAGE("PointCloudImporterReaderSMFOBJ::beginImport()\nFile not found! Textured .obj must have a .bmp with same name!\nAll vertexes are set to white.");
					textureFileNotValid = true;
					// finish progress window
//					bool dummy;
//					progressWindow->popSteps(dummy);
//					return false;
				};
				QImage *myImage = new QImage(QImage(QString(FileNameTex.c_str())).convertToFormat(QImage::Format_RGB32));
				if (myImage->isNull()) {
					WARNINGMESSAGE("PointCloudImporterReaderSMFOBJ::beginImport()\nTexture file is not a valid picture!\nAll vertexes are set to white.");
					textureFileNotValid = true;
				   // finish progress window
//					bool dummy;
//					progressWindow->popSteps(dummy);
//					return false;
				};
				if (!textureFileNotValid) {
					card32 iW = myImage->width();
					card32 iH = myImage->height();
					float32 inv255 = 1.0f/255.0f;

					// todo: lookup face color indices, without that maybe results in wrong color, see above faceEntryIndexes[1]

					// calculate colors
					for (card32 n=0; n<Positions.size(); n++)
					{
						card32 tX = (TextureCoords[n])[0] * iW;
						card32 tY = (1.0f - (TextureCoords[n])[1]) * iH;

						QRgb pixel = myImage->pixel(tX,tY);
						Colors.push_back(makeVector3f(qRed(pixel),qGreen(pixel),qBlue(pixel))*inv255);
					};

					if (!myVD->providesAttribute("texcoord")) myVD->pushAttrib(mVAD("texcoord", 2, VAD::DATA_FORMAT_FLOAT32));
				} else {
					// texture file invalid, so set white for every vertex
					for (card32 i=0; i<Positions.size(); i++) {
						Colors.push_back(makeVector3f(1.0f,1.0f,1.0f));
					};
				};

			} else {
				// no texture or an obj-material-file is defined (not supported), so set white for every vertex
				for (card32 i=0; i<Positions.size(); i++) {
					Colors.push_back(makeVector3f(1.0f,1.0f,1.0f));
				};
			}
		}; // end if colors.empty()
		pAssert(Colors.size()>=Positions.size());

	} else {
		// one color per face in file, so calculate vertex colors from faces
		Vector3f tempVF;

		pAssert(Colors.size()>=Faces.size());
		vector<Vector3f> tmpColors;

		// initialize with black first
		for (card32 i=0; i<Positions.size(); i++) {
			tmpColors.push_back(makeVector3f(0.0f,0.0f,0.0f));
		};

		// for every face add face color to edge vertices color
		for (card32 i=0; i<Faces.size(); i++) {
			tempVF = Colors[i];
			tmpColors[(Faces[i])[0]] += tempVF;
			tmpColors[(Faces[i])[1]] += tempVF;
			tmpColors[(Faces[i])[2]] += tempVF;
		}

		// finally add "normalized" colors
		Colors.clear();
		for (card32 i=0; i<Positions.size(); i++) {
			Colors.push_back(tmpColors[i]/3.0f);
		};
		tmpColors.clear();

	};	// end if bindColorToVertex

	width = 1;
	height = Positions.size();
	IKnowWidth = true;
	IKnowHeight = true;
	ImportStarted = false;
	ImportFinished = false;
	pointProgress = 0;
	triangleProgress = 0;

	delete[] buffer;
	buffer = new card8[myVD->getSize()];

	// finish progress window
   bool dummy;
   progressWindow->popSteps(dummy);

	return true;

}


void PointCloudImporterReaderSMFOBJ::endImport() {
	pointProgress = 0;
	triangleProgress = 0;
	Positions.clear();
	Colors.clear();
	Normals.clear();
	Faces.clear();
	TextureCoords.clear();
	ImportFinished = false;
	ImportStarted = false;
	pointImportFinished = false;
	triangleImportFinished = false;
	delete[] buffer;
	buffer = NULL;
	delete[] tribuffer;
	tribuffer = NULL;
};


mpcard PointCloudImporterReaderSMFOBJ::getWidth() {
   if (IKnowWidth && !ImportStarted) {
      return width;
   } else {
      pAssert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderSMFOBJ::getHeight() {
   if (IKnowHeight && !ImportStarted) {
      return height;
   } else {
      pAssert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderSMFOBJ::getProgressPercentage() {
	return ((pointProgress+triangleProgress)*100)/((card32)Positions.size()+(card32)Faces.size());
};


const card8* PointCloudImporterReaderSMFOBJ::getNextPoint() {

   if (pointImportFinished||ImportFinished) return NULL;
   pAssert(buffer!=NULL);

   ImportStarted = true;
   StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;

	*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("position")))) = Positions[pointProgress];
   *((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("normal")))) = Normals[pointProgress];
   *((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("color")))) = Colors[pointProgress];
   *((StaticVector<int32, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("flags")))) = NULL_FLAG;
	if (myVD->providesAttribute("texcoord")) *((StaticVector<float32, 2>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("texcoord")))) = TextureCoords[pointProgress];

   pointProgress++;
   if (pointProgress >= Positions.size()) pointImportFinished = true;
	if (pointImportFinished && triangleImportFinished) ImportFinished = true;
   return buffer;	
}


bool PointCloudImporterReaderSMFOBJ::supportsTriangleImport() {
	return true;
}

const card8* PointCloudImporterReaderSMFOBJ::getNextTriangle() {
	if (Faces.empty()) { 
		triangleImportFinished = true;
		if (pointImportFinished && triangleImportFinished) ImportFinished = true;
	};
   if (triangleImportFinished||ImportFinished) return NULL;
   pAssert(tribuffer!=NULL);

   ImportStarted = true;

   *((StaticVector<int32, 3>*)(tribuffer + myTD->getAttributeOffset(myTD->getIndex("index")))) = Faces[triangleProgress];

	triangleProgress++;
   if (triangleProgress >= Faces.size()) triangleImportFinished = true;
	if (pointImportFinished && triangleImportFinished) ImportFinished = true;
	return tribuffer;
}
