//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReaderVRML.h"
//---------------------------------------------------------------------------
#include "ProgressWindow.h"
#include "StringHelper.h"
#include <fstream>
//#include <QtGui/QImage.h>
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


MetaClass* PointCloudImporterReaderVRML::PointCloudImporterReaderVRMLMetaClass = NULL;

const MetaClass* PointCloudImporterReaderVRML::getInstanceClass() const {
   return getClass();
}

Object* PointCloudImporterReaderVRML::newInstance() {
   return new PointCloudImporterReaderVRML;
}

const MetaClass* PointCloudImporterReaderVRML::getClass() {
   if (PointCloudImporterReaderVRMLMetaClass == NULL) {
      PointCloudImporterReaderVRMLMetaClass = new MetaClass("PointCloudImporterReaderVRML", "no description.",
                                       newInstance, sizeof(PointCloudImporterReaderVRML));
   }
   return PointCloudImporterReaderVRMLMetaClass;
}

void PointCloudImporterReaderVRML::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   PointCloudImporterReaderVRMLMetaClass->copyParentProperties();
}

void PointCloudImporterReaderVRML::shutdown() {
   delete PointCloudImporterReaderVRMLMetaClass;
}

PointCloudImporterReaderVRML::PointCloudImporterReaderVRML() {
	pointProgress = 0;
	triangleProgress = 0;
	pointImportFinished = false;
	triangleImportFinished = false;
	Positions.clear();
	Colors.clear();
	diffColor = makeVector3f(1.0f,1.0f,1.0f);
	Faces.clear();

	delete myVD;
	myVD = new VertexDescriptor();
	myVD->pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
//	myVD->pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));
	myVD->pushAttrib(mVAD("flags", 1, VAD::DATA_FORMAT_INT32));
}

PointCloudImporterReaderVRML::~PointCloudImporterReaderVRML() {
}

void PointCloudImporterReaderVRML::assign(const Object* obj, CopyContext *context) {
	PointCloudImporterReader::assign(obj,context);
   const PointCloudImporterReaderVRML *other = dynamic_cast<const PointCloudImporterReaderVRML*>(obj);
   pAssert(other != NULL);
}


bool PointCloudImporterReaderVRML::beginImport(string filnam, bool readTrianglesIfPossible) {

	// call beginImport just once
   if (ImportStarted) return false;
	pAssert(buffer == NULL);
	pAssert(tribuffer == NULL);
	pointImportFinished = false;
	triangleImportFinished = false;

	if (!myVD->providesAttribute("position")) {
		WARNINGMESSAGE("PointCloudImporterReaderVRML::beginImport()\nVertex descriptor doesn't have needed attribute \"position\".");
		return false;
	}

	string ext = extractFileExt(strUCase(filnam));
   if (!(ext==".WRL")) {
		// no .smf or .obj
		WARNINGMESSAGE("PointCloudImporterReaderVRML::beginImport()\nFile ending has to be .WRL!");
		return false;
	};

	// init triangle descriptor if neccessary
	if (readTrianglesIfPossible)  {
		if (myTD==NULL) myTD = new VertexDescriptor();
		if (!myTD->providesAttribute("index")) myTD->pushAttrib(mVAD("index", 3, VAD::DATA_FORMAT_INT32));
		delete[] tribuffer;
		tribuffer = new card8[myTD->getSize()];
	} else {
		triangleImportFinished = true;
	};

	// clear caches
	Positions.clear();
	Colors.clear();
	diffColor = makeVector3f(1.0f,1.0f,1.0f);
	Faces.clear();

	// open file
	ifstream in;
	in.clear();
	in.open(filnam.c_str(),ifstream::in);
	if (!in.good()) { 
		WARNINGMESSAGE("PointCloudImporterReaderVRML::beginImport()\nError opening file \"" + filnam + "\"!"); 
		return false;
	};
	in.seekg(-1, ios::end);
	card64 fileSize = in.tellg();
	in.seekg(0, ios::beg);

	// first line
	string line;
	getline(in, line);
	line = removeDblSpace(line); // remove multiple spaces between values and make them single space
	line = trimStr(line); // remove beginning and ending spaces from line
	if (line!="#VRML V1.0 ascii") {
		if (QMessageBox::No == QMessageBox::warning(NULL, "Warning!", "PointCloudImporterReaderVRML::beginImport()\n\nFile is not a valid \"VRML V1.0 ascii\" file!\n\nTry to load it anyway (results may be undefined)?", QMessageBox::Yes|QMessageBox::Default, QMessageBox::No|QMessageBox::Escape))
			return false;
	}

   // init progress window
   progressWindow->pushStep(true, "loading wrl");
   progressWindow->progress(0);

	// load from file to caches
   static const card32 EAT_STUFF = 0;
   static const card32 GET_MAT = 1;
   static const card32 GET_POINTS = 2;
   static const card32 GET_FACES = 3;
   static const card32 GET_COLORS = 4;

   card32 mode = EAT_STUFF;
	card32 wordCount = 0;

   while (!in.eof()) {
		string token;
      in >> token;
      if (token == "diffuseColor") {
         mode = GET_MAT;
      } else if (token == "point") {
         string fence;
         in >> fence;
         mode = GET_POINTS;
      } else if (token == "color") {
         string fence;
         in >> fence;
			if (fence=="[") mode = GET_COLORS;
      } else if ((token == "coordIndex") && readTrianglesIfPossible) {
         string fence;
         in >> fence;
         mode = GET_FACES;
      }

      switch (mode) {
         case GET_MAT: {
            in >> diffColor[0];
            in >> diffColor[1];
            in >> diffColor[2];
            mode = EAT_STUFF;
            break;
         }
         case GET_POINTS: {
            do {
               string sep;
               in >> sep;
					if (sep == "]") { mode = EAT_STUFF; break; }; // hack! cause "... v3, ]" is valid VRML (problem is "," after last point vertex)
					float32 tmpFloat;
					bool succ = strToFloat(sep, tmpFloat);
					if (!succ) { pAssert(false); };
               Vector3f p;
               p[0] = tmpFloat;
               in >> p[1];
               in >> p[2];
               Positions.push_back(p);
               in >> sep;
               if ((sep=="]") || (sep==",]")) mode = EAT_STUFF;
					// update progress window
					wordCount++;
					if (wordCount % 10000 == 0) {
						wordCount = 0;
						// hopefully doesn't crash
						card64 sp = in.tellg();
						if (!progressWindow->progress((sp*100)/fileSize)) { 
							// escape pressed: abort
							in.close();
							return false;
						};
					};
            } while (mode == GET_POINTS && !in.eof());
            break;
         }
         case GET_COLORS: {
            do {
               string sep;
               in >> sep;
					if (sep == "]") { mode = EAT_STUFF; break; }; // hack! cause "... c3, ]" is valid VRML (problem is "," after last point vertex)
					float32 tmpFloat;
					bool succ = strToFloat(sep, tmpFloat);
					if (!succ) { pAssert(false); };
               Vector3f c;
               c[0] = tmpFloat;
               in >> c[1];
               in >> c[2];
               Colors.push_back(c);
               in >> sep;
               if ((sep=="]") || (sep==",]")) mode = EAT_STUFF;
					// update progress window
					wordCount++;
					if (wordCount % 10000 == 0) {
						wordCount = 0;
						// hopefully doesn't crash
						card64 sp = in.tellg();
						if (!progressWindow->progress((sp*100)/fileSize)) { 
							// escape pressed: abort
							in.close();
							return false;
						};
					};
            } while (mode == GET_COLORS && !in.eof());
            break;
         }
         case GET_FACES: {
				vector< vector<int> > tmpfaces;
            do {
               float32 numf;
					int32 num;
               vector<int> face;
               string sep;
               do {
						in >> numf;
						if (in.fail()) {
							in.clear();
							in >> sep;
							if (sep=="]") { mode = EAT_STUFF; break; } // hack! cause "... ix,\n]" is possible (problem is the "\n")
						}
						num = (int32)numf;
						in >> sep;
                  if (num != -1) {
                     face.push_back(num);
                  }
               } while (num != -1);
               tmpfaces.push_back(face);
               if ((sep=="]") || (sep==",]")) mode = EAT_STUFF;
					// update progress window
					wordCount++;
					if (wordCount % 10000 == 0) {
						wordCount = 0;
						// hopefully doesn't crash
						card64 sp = in.tellg();
						if (!progressWindow->progress((sp*100)/fileSize)) { 
							// escape pressed: abort
							in.close();
							return false;
						};
					};
            } while (mode == GET_FACES && !in.eof());

				wordCount = 0;
				progressWindow->progress(98);

            /// store triangles
            for (card32 f = 0; f < tmpfaces.size(); f++) {
               vector<int> currentFace = tmpfaces[f];
               if (currentFace.size() < 3) continue;
               for (card32 triNum = 0; triNum <=currentFace.size()-3; triNum++) {
                  Vector3i faceIdx;
                  faceIdx[0] = currentFace[0];
                  faceIdx[1] = currentFace[triNum+1];
                  faceIdx[2] = currentFace[triNum+2];
						Faces.push_back(faceIdx);
					}
					// update progress window
					if (f % 10000 == 0) {
						if (!progressWindow->progress(((int32)f)/((int32)tmpfaces.size()))) { 
							// escape pressed: abort
							in.close();
							return false;
						};
					};
				};
				tmpfaces.clear();
            break;
         }	// end get_faces
         default: break;
		} // end switch mode

		// update progress window
		wordCount++;
		if (wordCount % 20000 == 0) {
			wordCount = 0;
			// hopefully doesn't crash
			card64 sp = in.tellg();
         if (!progressWindow->progress((sp*100)/fileSize)) { 
				// escape pressed: abort
				in.close();
				return false;
			};
		};

	} // end while !in.eof

	// end loading from file
	in.close();

	// todo: calculate vertex normals!

	width = 1;
	height = Positions.size();
	IKnowWidth = true;
	IKnowHeight = true;
	ImportStarted = false;
	ImportFinished = false;
	pointProgress = 0;
	triangleProgress = 0;

	delete[] buffer;
	buffer = new card8[myVD->getSize()];

	// finish progress window
   bool dummy;
   progressWindow->popSteps(dummy);

	return true;

}


void PointCloudImporterReaderVRML::endImport() {
	pointProgress = 0;
	triangleProgress = 0;
	Positions.clear();
	Colors.clear();
	diffColor = makeVector3f(1.0f,1.0f,1.0f);
	Faces.clear();
	ImportFinished = false;
	ImportStarted = false;
	pointImportFinished = false;
	triangleImportFinished = false;
	delete[] buffer;
	buffer = NULL;
	delete[] tribuffer;
	tribuffer = NULL;
};


mpcard PointCloudImporterReaderVRML::getWidth() {
   if (IKnowWidth && !ImportStarted) {
      return width;
   } else {
      pAssert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderVRML::getHeight() {
   if (IKnowHeight && !ImportStarted) {
      return height;
   } else {
      pAssert(false);
      return 0;
   }
}


mpcard PointCloudImporterReaderVRML::getProgressPercentage() {
	return ((pointProgress+triangleProgress)*100)/((card32)Positions.size()+(card32)Faces.size());
};


const card8* PointCloudImporterReaderVRML::getNextPoint() {

   if (pointImportFinished||ImportFinished) return NULL;
   pAssert(buffer!=NULL);

   ImportStarted = true;
   StaticVector<int32, 1> NULL_FLAG; NULL_FLAG[0] = 0;

	*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("position")))) = Positions[pointProgress];
//   *((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("normal")))) = Normals[pointProgress];
	if (Colors.size()>=Positions.size()){
		*((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("color")))) = Colors[pointProgress];
	} else {
	   *((StaticVector<float32, 3>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("color")))) = diffColor;
	}
   *((StaticVector<int32, 1>*)(buffer + myVD->getAttributeOffset(myVD->getIndex("flags")))) = NULL_FLAG;

   pointProgress++;
   if (pointProgress >= Positions.size()) pointImportFinished = true;
	if (pointImportFinished && triangleImportFinished) ImportFinished = true;
   return buffer;	
}


bool PointCloudImporterReaderVRML::supportsTriangleImport() {
	return true;
}

const card8* PointCloudImporterReaderVRML::getNextTriangle() {
	if (Faces.empty()) { 
		triangleImportFinished = true;
		if (pointImportFinished && triangleImportFinished) ImportFinished = true;
	};
   if (triangleImportFinished||ImportFinished) return NULL;
   pAssert(tribuffer!=NULL);

   ImportStarted = true;

   *((StaticVector<int32, 3>*)(tribuffer + myTD->getAttributeOffset(myTD->getIndex("index")))) = Faces[triangleProgress];

	triangleProgress++;
   if (triangleProgress >= Faces.size()) triangleImportFinished = true;
	if (pointImportFinished && triangleImportFinished) ImportFinished = true;
	return tribuffer;
}
