//---------------------------------------------------------------------------
#ifndef PointCloudImporterReaderVRMLH
#define PointCloudImporterReaderVRMLH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PointCloudImporterReader.h"
//---------------------------------------------------------------------------

namespace geoxl {

/// Import Reader for very simple(!) VRML files
/// notice: just reads points and faces not considering other geometry or object grouping etc
/// for color of all points the last found diffuse color will be taken, per-vertex-colors are not supported till yet
class EDITOR_API PointCloudImporterReaderVRML : public PointCloudImporterReader {

 private:
   static MetaClass *PointCloudImporterReaderVRMLMetaClass;
   static Object* newInstance();

	/// import point progress
   card32 pointProgress;
	/// import triangle progress
   card32 triangleProgress;
	/// true if point import is finished
	bool pointImportFinished;
	/// true if triangle import is finished
	bool triangleImportFinished;
	/// holds all positions
   vector<Vector3f> Positions;
	/// holds all colors if in file
   vector<Vector3f> Colors;
	/// holds actual color
   Vector3f diffColor;
	/// holds all faces
   vector<Vector3i> Faces;


 public:
   PointCloudImporterReaderVRML();
   ~PointCloudImporterReaderVRML();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual bool beginImport(string filenam, bool readTrianglesIfPossible = false);
   virtual const card8* getNextPoint();
	virtual const card8* getNextTriangle();
   virtual void endImport();

   virtual mpcard getWidth();
   virtual mpcard getHeight();

	virtual mpcard getProgressPercentage();

	virtual bool supportsTriangleImport();

};

}

#endif
