#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "CmdObjScriptProgramEditorWidget.h"
#include "CmdObjScriptProgram.h"
//---------------------------------------------------------------------------
#include <qtGui/qmessagebox.h>
#include "Configuration.h"
#include "FileDialogs.h"
#include "ObjectBrowserWidget.h"
#include "BinaryObjectStreams.h"
#include "CommandObjMacro.h"
#include "ClassSelectionDialog.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( CmdObjScriptProgram )

CmdObjScriptProgramEditorWidget::CmdObjScriptProgramEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	// set header for command lists
   QTableWidgetItem* itemPrototype = new QTableWidgetItem();
   itemPrototype->setFlags(Qt::ItemIsSelectable);
   ui.TWCommands->setItemPrototype(itemPrototype);

   ui.TWCommands->setHorizontalHeaderLabels(QStringList() << "Name" << "Class" << "Description");

	// button icons
	ui.BTBottom->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/down_bottom_arrow.bmp").c_str()));
	ui.BTDown->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/down_arrow.bmp").c_str()));
	ui.BTUp->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/up_arrow.bmp").c_str()));
	ui.BTTop->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/up_top_arrow.bmp").c_str()));

	// make layout to add command object property editors
	QVBoxLayout *layout = new QVBoxLayout;
	layout->setMargin(0);
	layout->setSpacing(0);
	ui.PropertyFrame->setLayout(layout);

	// init the rest
	objectEditor = NULL;
	objectEditorWidget = NULL;
	updating = false;
}

CmdObjScriptProgramEditorWidget::~CmdObjScriptProgramEditorWidget(void)
{
}

void CmdObjScriptProgramEditorWidget::setup( CmdObjScriptProgram * obj )
{
	cmdScript = obj;

   this->dictionary = cmdScript->getDictionary();
}

void CmdObjScriptProgramEditorWidget::updateEditor()
{
   QTableWidgetItem* item;
   Object* obj;
   
   updating = true;
   ui.TWCommands->setRowCount(dictionary->getNumObjects());
   for (card32 i = 0; i < dictionary->getNumObjects(); ++i) {
      obj = dictionary->getObject(i);
      
      item = new QTableWidgetItem();
      item->setText(dictionary->getObjectName(i).c_str());
      ui.TWCommands->setItem(i,0,item);
      
      item = new QTableWidgetItem();
      item->setFlags(item->flags() & ~Qt::ItemIsEditable);
      item->setText(obj->getInstanceClass()->getClassName().c_str());
      ui.TWCommands->setItem(i,1,item);
      
      string description;
      CommandObject* cmdObj = dynamic_cast<CommandObject*>(obj);
      if (cmdObj != NULL) description = cmdObj->getDescription();
      
      item = new QTableWidgetItem();
      item->setFlags(item->flags() & ~Qt::ItemIsEditable);
      item->setText(description.c_str());
      ui.TWCommands->setItem(i,2,item);
   }

   ui.TWCommands->resizeColumnsToContents();
   ui.TWCommands->resizeRowsToContents();
   updating = false;
}

void CmdObjScriptProgramEditorWidget::updateSelectedItemEditor(Object *obj)
{
	if (objectEditor != NULL) {
		ui.PropertyFrame->layout()->removeWidget(objectEditorWidget);
		delete objectEditorWidget;
		delete objectEditor;
		objectEditor = NULL;
		objectEditorWidget = NULL;
	}
	if (obj == NULL) return;

	const MetaClass *mc = obj->getInstanceClass();
	
	const MetaClass* classEditorMC = mc->getGUIDescriptor()->getDefaultClassEditor();
	if (classEditorMC == NULL) return;
	
	objectEditor = (ClassEditor*)(classEditorMC->newInstance());
	objectEditorWidget = objectEditor->makeEditor(ui.PropertyFrame, obj);
	ui.PropertyFrame->layout()->addWidget(objectEditorWidget);
	objectEditorWidget->show();
	objectEditor->update();
}

void CmdObjScriptProgramEditorWidget::on_BTEditScript_clicked()
{
	CmdObjScriptEngine* script = cmdScript->getScript();
	
	const GUIDescriptor * gui = script->getInstanceClass()->getGUIDescriptor();
	if( gui->getNumApplications() == 0 )
	{
		ClassEditor * ce = (ClassEditor*)gui->getDefaultClassEditor()->newInstance();
		if (ce->standAloneEditorApplication()) {
		   ObjectViewsTable::bindView( script, ce );
		   QWidget * w = ce->makeEditor( this , script );
		   w->setWindowFlags( Qt::Window );
		   w->setWindowTitle( "Script editor" );
		   w->setAttribute( Qt::WA_DeleteOnClose );
		   w->show();
		} else {
	      ObjectBrowserWidget *newBrowser = new ObjectBrowserWidget(script, ce, this);
	      QWidget * w = ce->makeEditor( newBrowser, script );
	      newBrowser->setupEditor(w);		
	      newBrowser->show();
	      ObjectViewsTable::bindView( script, ce );
	     }
	}
	else
	{
		GUIDescriptor::ApplicationPtr app = gui->getApplication(0);
		app.startProc( script );
	}
}

void CmdObjScriptProgramEditorWidget::on_BTAddMacro_clicked()
{
   string filename = FileDialogs::getOpenFileName(this, "Choose macro file", "*.object" );

   if (filename != "") {
      BinaryInputObjectStream in(filename.c_str());
      CommandObjMacro *newMacro = NULL;
      in.readObject((Persistent*&)newMacro);
      if (newMacro->getInstanceClass() != CommandObjMacro::getClass()) {
         QMessageBox::critical(this, "Wrong file type", "Imported object is not of type CommandObjMacro.");
         Object *del = newMacro;
         newMacro = NULL;
         delete del;
      } else {
         for (unsigned i = 0; i < newMacro->numCommands(); ++i)
            cmdScript->addCommand(newMacro->getCommand(i));

         updateEditor();
         if (newMacro != NULL) delete newMacro;
      }
   }
}

void CmdObjScriptProgramEditorWidget::on_BTBottom_clicked()
{
   if (ui.TWCommands->currentRow() >= 0) {
      dictionary->moveObject(ui.TWCommands->currentRow(), dictionary->getNumObjects()-1);
      updateEditor();
      ui.TWCommands->setCurrentCell(dictionary->getNumObjects()-1, ui.TWCommands->currentColumn());
   }
}

void CmdObjScriptProgramEditorWidget::on_BTDown_clicked()
{
   if (ui.TWCommands->currentRow() >= 0 && ui.TWCommands->currentRow() < (int)dictionary->getNumObjects()-1) {
      dictionary->moveObject(ui.TWCommands->currentRow(), ui.TWCommands->currentRow()+1);
      updateEditor();
      ui.TWCommands->setCurrentCell(ui.TWCommands->currentRow()+1, ui.TWCommands->currentColumn());
   }
}

void CmdObjScriptProgramEditorWidget::on_BTUp_clicked()
{
   if (ui.TWCommands->currentRow() > 0) {
      dictionary->moveObject(ui.TWCommands->currentRow(), ui.TWCommands->currentRow()-1);
      updateEditor();
      ui.TWCommands->setCurrentCell(ui.TWCommands->currentRow()-1, ui.TWCommands->currentColumn());
   }
}

void CmdObjScriptProgramEditorWidget::on_BTTop_clicked()
{
   if (ui.TWCommands->currentRow() >= 0) {
      dictionary->moveObject(ui.TWCommands->currentRow(), 0);
      updateEditor();
      ui.TWCommands->setCurrentCell(0, ui.TWCommands->currentColumn());
   }
}


void CmdObjScriptProgramEditorWidget::on_BTNew_clicked()
{
	ClassSelectionDialog* classSelectionDialog = new ClassSelectionDialog(this);
	classSelectionDialog->init(Object::getClass(), false);

	classSelectionDialog->move(QCursor::pos());
	if (classSelectionDialog->exec() == QDialog::Accepted) {
		MetaClass * mc = classSelectionDialog->getSelectedClass();
		dictionary->newObject(mc);
		updateEditor();
	}
	delete classSelectionDialog;
}

void CmdObjScriptProgramEditorWidget::on_BTDelete_clicked()
{
   if (ui.TWCommands->currentRow() >= 0) {
      QTableWidgetItem* item = ui.TWCommands->item(ui.TWCommands->currentRow(), 0);
      dictionary->deleteObject(item->text().toLatin1().data());
      updateEditor();
   }
}

void CmdObjScriptProgramEditorWidget::on_BTDuplicate_clicked()
{
   if (dictionary->getNumObjects() == 0) return;

   Object *obj = NULL;
   if (ui.TWCommands->currentRow() >= 0) {
      QTableWidgetItem* item = ui.TWCommands->item(ui.TWCommands->currentRow(), 0);
      obj = dictionary->getObject(item->text().toLatin1().data());
      dictionary->addObject(obj->copy());
      updateEditor();
   }
}

void CmdObjScriptProgramEditorWidget::on_TWCommands_currentCellChanged(int row, int column, int prevRow, int prevColumn)
{
   if (row == prevRow) return;

   Object* obj;
   if (row < 0) obj = NULL; else obj = dictionary->getObject(row);
   updateSelectedItemEditor(obj);
}

void CmdObjScriptProgramEditorWidget::on_TWCommands_cellChanged(int row, int column)
{
   if (updating) return;
   
   QTableWidgetItem* item = ui.TWCommands->item(row, column);
   string newName = item->text().toLatin1().data();
   if (!dictionary->renameObject(dictionary->getObjectName(row), newName)) {
      QMessageBox::critical(this, "Invalid name", "The name " + item->text() + " for the object is invalid.\nEvery objects needs a unique name.");
      updating = true;
      item->setText(dictionary->getObjectName(row).c_str());
      updating = false;
   }
}

#include "CmdObjScriptProgramEditorWidget_moc.cpp"
