#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "GeoXLGLWidget.h"
//---------------------------------------------------------------------------
#include "Scene.h"
#include "SelectionTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;


GeoXLGLWidget::GeoXLGLWidget(QWidget* parent)
:QGLWidget(parent)
{
	setMouseTracking(true);
	setAcceptDrops(true);

}

GeoXLGLWidget::~GeoXLGLWidget(void)
{
}


void GeoXLGLWidget::initializeGL()
{
}

void GeoXLGLWidget::paintGL()
{
	 emit sigRepaint();
}

void GeoXLGLWidget::resizeGL(int width, int height)
{
	 emit sigResize(QSize(width,height));
}

void GeoXLGLWidget::mousePressEvent(QMouseEvent *event)
{
	m_dragStartPosition = event->pos();
	 emit sigMousePress(event);
	 //updateGL();
	QGLWidget::mousePressEvent(event);
}

void GeoXLGLWidget::mouseReleaseEvent (QMouseEvent * event){
	emit sigMouseRelease(event);
	//updateGL();
	QGLWidget::mouseReleaseEvent(event);
}

void geoxl::GeoXLGLWidget::wheelEvent( QWheelEvent * event )
{
	emit sigWheelEvent(event);
	QGLWidget::wheelEvent(event);
}

void GeoXLGLWidget::mouseMoveEvent(QMouseEvent *e)
{ 
	emit sigMouseMove(e);
	//updateGL();
	if ((e->buttons() & Qt::LeftButton && (e->modifiers() & Qt::AltModifier)))
	{
		if ((e->pos() - m_dragStartPosition).manhattanLength()
			 < QApplication::startDragDistance())
		{

			string pc = getClickedCloud( m_dragStartPosition.x(), m_dragStartPosition.y(),
				width(),height(), &scene->getRootState()->staticState->camera, 
				&scene->getRootState()->staticState->viewFrustum, scene );

			QDrag *drag = new QDrag(this);
			QMimeData *mimeData = new QMimeData;

			QByteArray pointCloudName( pc.c_str() );
			mimeData->setData("text/plain", pointCloudName);
			drag->setMimeData(mimeData);

			Qt::DropAction dropAction = drag->start(Qt::CopyAction | Qt::MoveAction);
		}
	}
	QGLWidget::mouseMoveEvent(e);
}

GLContext * GeoXLGLWidget::getContext(){
	 makeCurrent();
	 pAssert(wglGetCurrentDC());  pAssert(wglGetCurrentContext());
	 return new GLContext(wglGetCurrentDC(),wglGetCurrentContext(),NULL,this);
}


void GeoXLGLWidget::dragEnterEvent(QDragEnterEvent *event)
{
	if (event->mimeData()->hasFormat("geoxl/material"))
		event->acceptProposedAction();
}

void GeoXLGLWidget::dropEvent(QDropEvent *event)
{
	emit sigDropEvent( (QMimeData*)event->mimeData(), event->pos() );
	event->acceptProposedAction();
}


#include "GeoXLGLWidget_moc.cpp"



