#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "ImportAnimationAssistantWidget.h"
#include "ClassEditor.h"
#include "PCCommand.h"
#include "Scene.h"
#include "ExceptionHandling.h"
//---------------------------------------------------------------------------

using namespace geoxl;


ImportAnimationAssistantWidget::ImportAnimationAssistantWidget(QWidget *parent, Scene* scene)
:QWidget(parent)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	// used Qt auto-connect feature for other slots
	QWidget::connect(ui.PBCancel, SIGNAL(clicked()), this , SLOT(close()));
	
	// set widget attributes
	setWindowFlags( Qt::Window );
	setAttribute(Qt::WA_DeleteOnClose);
	setWindowModality(Qt::WindowModal);
	setWindowIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/geoxl.png").c_str()));
	setWindowTitle("Animation Import Assistant");

	// make importer editor
	const MetaClass *newImporterClass = MetaClass::getClassByName("PCCAnimationImporter");
	m_PCCAnimationImporter = (PCCommand*)newImporterClass->newInstance();
	const MetaClass *newImporterEditorClass = newImporterClass->getGUIDescriptor()->getDefaultClassEditor();
	m_PCCAnimationImporterEditor = (ClassEditor*)(newImporterEditorClass->newInstance());
	m_PCCAnimationImporterEditorWidget = m_PCCAnimationImporterEditor->makeEditor(ui.GBAnimationImportSettings, m_PCCAnimationImporter);

	QVBoxLayout *layout2 = new QVBoxLayout;
	layout2->setMargin(2);
	layout2->setSpacing(2);
	layout2->addWidget(m_PCCAnimationImporterEditorWidget);
   m_PCCAnimationImporterEditorWidget->show();
	ui.GBAnimationImportSettings->setLayout(layout2);

	// init the rest
	ui.LabelImporterIcon->setPixmap(QPixmap(QString((getConfigurationString("IMAGE_PATH")+"/ImportAssistant.png").c_str())));
	ui.PBExecute->setIcon(QIcon(QString((getConfigurationString("IMAGE_PATH")+"/ImportAssistantOk.png").c_str())));
	this->scene = scene;
   int xp = x();
   int yp = y();
   if (xp < 0) {xp=0;}
   if (yp < 0) {yp=0;}
   move(xp,yp);
}

ImportAnimationAssistantWidget::~ImportAnimationAssistantWidget(void)
{
}


// ---------- user functions ----------



// ---------- slots ----------

void ImportAnimationAssistantWidget::on_PBExecute_clicked() {
   STD_GLOBAL_EXCEPTION_HANDLER_START
   
   CmdObjPCCommand *newCmdObj = new CmdObjPCCommand();
   newCmdObj->setup(m_PCCAnimationImporter);
	scene->getCommandHistory()->executeCommand(scene, newCmdObj, false);

	close();
	
	STD_GLOBAL_EXCEPTION_HANDLER_END
}



#include "ImportAnimationAssistantWidget_moc.cpp"
