//---------------------------------------------------------------------------
#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "ImportAssistantWidget.h"
#include "ClassEditor.h"
#include "PCCommand.h"
#include "Scene.h"
#include "ExceptionHandling.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



ImportAssistantWidget::ImportAssistantWidget(QWidget *parent, Scene* scene) : QWidget(parent)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	// used Qt auto-connect feature for other slots
	QWidget::connect(ui.PBCancel, SIGNAL(clicked()), this , SLOT(close()));
	
	// set widget attributes
	setWindowFlags( Qt::Window );
	setAttribute(Qt::WA_DeleteOnClose);
	setWindowModality(Qt::WindowModal);
	setWindowIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/geoxl.png").c_str()));
	setWindowTitle("Import Assistant");

	// make creator editor
	const MetaClass *newCreatorClass = MetaClass::getClassByName("PCCPointCloudCreator");
	m_PCCCreator = (PCCommand*)newCreatorClass->newInstance();
	const MetaClass *newCreatorEditorClass = newCreatorClass->getGUIDescriptor()->getDefaultClassEditor();
	m_PCCCreatorEditor = (ClassEditor*)(newCreatorEditorClass->newInstance());
	m_PCCCreatorEditorWidget = m_PCCCreatorEditor->makeEditor(ui.GBCreateNew, m_PCCCreator);

	QVBoxLayout *layout1 = new QVBoxLayout;
	layout1->setMargin(2);
	layout1->setSpacing(2);
	layout1->addWidget(m_PCCCreatorEditorWidget);
   m_PCCCreatorEditorWidget->show();
	ui.GBCreateNew->setLayout(layout1);

	// make importer editor
	const MetaClass *newImporterClass = MetaClass::getClassByName("PCCImporter");
	m_PCCImporter = (PCCommand*)newImporterClass->newInstance();
	const MetaClass *newImporterEditorClass = newImporterClass->getGUIDescriptor()->getDefaultClassEditor();
	m_PCCImporterEditor = (ClassEditor*)(newImporterEditorClass->newInstance());
	m_PCCImporterEditorWidget = m_PCCImporterEditor->makeEditor(ui.GBImportSettings, m_PCCImporter);

	QVBoxLayout *layout2 = new QVBoxLayout;
	layout2->setMargin(2);
	layout2->setSpacing(2);
	layout2->addWidget(m_PCCImporterEditorWidget);
   m_PCCImporterEditorWidget->show();
	ui.GBImportSettings->setLayout(layout2);

	// init the rest
	ui.LabelImporterIcon->setPixmap(QPixmap(QString((getConfigurationString("IMAGE_PATH")+"/ImportAssistant.png").c_str())));
	ui.PBExecute->setIcon(QIcon(QString((getConfigurationString("IMAGE_PATH")+"/ImportAssistantOk.png").c_str())));
	this->scene = scene;

   int xp = x();
   int yp = y();
   if (xp < 0) {xp=0;}
   if (yp < 0) {yp=0;}
   move(xp,yp);
}

ImportAssistantWidget::~ImportAssistantWidget(void)
{
}


// ---------- user functions ----------



// ---------- slots ----------

void ImportAssistantWidget::on_PBExecute_clicked() {
   STD_GLOBAL_EXCEPTION_HANDLER_START
	if (ui.GBCreateNew->isChecked()) {
      CmdObjPCCommand *newCmdObj = new CmdObjPCCommand();
      newCmdObj->setup(m_PCCCreator);
		scene->getCommandHistory()->executeCommand(scene, newCmdObj, false);
	}

   CmdObjPCCommand *newCmdObj = new CmdObjPCCommand();
   newCmdObj->setup(m_PCCImporter);
	scene->getCommandHistory()->executeCommand(scene, newCmdObj, false);

	close();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}



#include "ImportAssistantWidget_moc.cpp"
