//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCCPointCloudCreatorEditorWidget.h"
//---------------------------------------------------------------------------
#include "PCCPointCloudCreator.h"
#include "ObjectBrowserWidget.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_CLASS_EDITOR( PCCPointCloudCreator )

PCCPointCloudCreatorEditorWidget::PCCPointCloudCreatorEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	m_Object = NULL;

	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//Example: easy access to QtDesigner buttons, etc. by ui object
	//QWidget::connect(ui.loadButton,SIGNAL(clicked()),this,SLOT(sltLoadFile()));

	ui.CBType->addItems(QStringList()<<"Unstructured"<<"Parametric"<<"Hierachical In-Core"<<"Hierachical Out-Of-Core"<<"Unstructured In-Core Triangle-Mesh");
	ui.CBType->setCurrentIndex(0);

}

PCCPointCloudCreatorEditorWidget::~PCCPointCloudCreatorEditorWidget(void)
{
}

void PCCPointCloudCreatorEditorWidget::setup( PCCPointCloudCreator * obj )
{
	m_Object = obj;
}

void PCCPointCloudCreatorEditorWidget::updateEditor()
{
	if (m_Object==NULL) return;
	ui.LEName->setText(QString(m_Object->getName().c_str()));
	ui.CBType->setCurrentIndex(m_Object->getType());
	ui.SBMaxNumObjectsPerLeaf->setValue(m_Object->getMaxNumObjectsPerLeaf());
	ui.SBMultiResGridSize->setValue(m_Object->getMultiResGridSize());
	ui.SBAllocatorIndex->setValue(m_Object->getAllocatorIndex());
	ui.SBMaxNodeInMem->setValue(m_Object->getMaxNodeInMem());
}


// ---------- slots ----------

void PCCPointCloudCreatorEditorWidget::on_LEName_textEdited(QString s) {
	m_Object->setName(qPrintable(s));
}


void PCCPointCloudCreatorEditorWidget::on_CBType_currentIndexChanged(int index) {
	if (m_Object==NULL) return;
	if (index>=0) {
		m_Object->setType(index);
		if ((index==2)||(index==3)) ui.GBHierarchy->setEnabled(true);
			else ui.GBHierarchy->setEnabled(false);
		if (index==3) ui.GBOutOfCore->setEnabled(true);
			else ui.GBOutOfCore->setEnabled(false);
	}
}


void PCCPointCloudCreatorEditorWidget::on_PBEditVD_clicked() {
	ClassEditor * ce = (ClassEditor*)m_Object->getVertexDescriptor()->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
	ObjectViewsTable::bindView( m_Object->getVertexDescriptor(), ce );
   ObjectBrowserWidget *browser = new ObjectBrowserWidget(m_Object->getVertexDescriptor(), ce, this, Qt::Window);
	QWidget * w = ce->makeEditor( browser->getEditorContainer(), m_Object->getVertexDescriptor() );
   browser->setupEditor(w);
   browser->show();

//	w->setWindowFlags( Qt::Window );
//	w->setWindowTitle( "VD" );
//	w->move( QCursor::pos() );
//	w->setAttribute(Qt::WA_DeleteOnClose);
//	w->show();
}


void PCCPointCloudCreatorEditorWidget::on_SBMaxNumObjectsPerLeaf_valueChanged(int value) {
	m_Object->setMaxNumObjectsPerLeaf(value);
}


void PCCPointCloudCreatorEditorWidget::on_SBMultiResGridSize_valueChanged(int value) {
	m_Object->setMultiResGridSize(value);
}


void PCCPointCloudCreatorEditorWidget::on_SBAllocatorIndex_valueChanged(int value) {
	m_Object->setAllocatorIndex(value);
}


void PCCPointCloudCreatorEditorWidget::on_SBMaxNodeInMem_valueChanged(int value) {
	m_Object->setMaxNodeInMem(value);
}



#include "PCCPointCloudCreatorEditorWidget_moc.cpp"
