#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCICameraManagerEditorWidget.h"
//---------------------------------------------------------------------------
#include "PCICameraManager.h"
#include "Scene.h"
#include "StandardSceneGraphIterator.h"
#include "SGCameraNode.h"
#include "ClassSelectionDialog.h"
#include "CameraController.h"
#include <qtGUI/qinputdialog>
#include "elementEditors/objectElementEditor.h"
#include "SGCameraNode.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( PCICameraManager )

PCICameraManagerEditorWidget::PCICameraManagerEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	ui.setupUi(this);

	m_ClassDlg = new ClassSelectionDialog(this);
	m_ClassDlg->init( CameraController::getClass() );
	m_CameraEditorWidget = NULL;
	m_CameraEditor = NULL;
}

PCICameraManagerEditorWidget::~PCICameraManagerEditorWidget(void)
{
	delete m_CameraEditor;
}

void PCICameraManagerEditorWidget::setup( PCICameraManager * obj )
{
	m_Object = obj;
}

void PCICameraManagerEditorWidget::updateEditor()
{
	ui.listCameras->clear();

	Scene * scene = m_Object->getScene();
   StandardSceneGraphIterator sceneIt;
   sceneIt.clearAndSetup(scene->getRootNode(), scene->getRootState(), SGDynamicState::tmRendering, SceneGraphNode::FLAG_VISIBLE, 0);
	int currentItemIndex = -1;

   while (!sceneIt.atEnd()) 
	{
		SGCameraNode * camNode = dynamic_cast<SGCameraNode*>(sceneIt.getNode());
		if( camNode )
		{
			if( sceneIt.getNameWithPath() == m_Object->getSelectedCamera() )
				currentItemIndex = ui.listCameras->count();
			ui.listCameras->addItem( sceneIt.getNameWithPath().c_str() );
		}

		sceneIt.next();
	}

	if( currentItemIndex >= 0 )
		ui.listCameras->setCurrentRow( currentItemIndex );
}

#include "PCICameraManagerEditorWidget_moc.cpp"
void PCICameraManagerEditorWidget::on_pbAdd_clicked() 
{
	QString name = QInputDialog::getText( this, "camera name", "Enter camera name \n(do not enter a scenegraph-path!)", QLineEdit::Normal, "camera1" );
	if( !name.isEmpty() )
	{
		if( m_ClassDlg->exec() == QDialog::Accepted )
		{
			MetaClass * mc = m_ClassDlg->getSelectedClass();
			CameraController * camContr = (CameraController*)mc->newInstance();

			SGCameraNode * node = new SGCameraNode;
			node->setCameraController( camContr );
			node->setName( name.toLatin1().constData() );
			m_Object->getScene()->getRootNode()->addChildNode( node );
		}
	}
	updateEditor();
}

void PCICameraManagerEditorWidget::on_pbAddPanCam_clicked() 
{
	QString name = QInputDialog::getText( this, "camera name", "Enter camera name \n(do not enter a scenegraph-path!)", QLineEdit::Normal, "panoramicCamera1" );
	if( !name.isEmpty() )
	{
		if( m_ClassDlg->exec() == QDialog::Accepted )
		{
			MetaClass * mc = m_ClassDlg->getSelectedClass();
			CameraController * camContr = (CameraController*)mc->newInstance();

			SGCameraNode * node = new SGCameraNode(1); // Type 1: Panoramic Camera
			node->setCameraController( camContr );
			node->setName( name.toLatin1().constData() );
			m_Object->getScene()->getRootNode()->addChildNode( node );
		}
	}
	updateEditor();

}

void PCICameraManagerEditorWidget::on_listCameras_currentTextChanged(const QString &currText) 
{
	m_Object->setSelectedCamera( currText.toLatin1().constData() );
	delete m_CameraEditorWidget;
	m_CameraEditorWidget = NULL;
	delete m_CameraEditor;
	m_CameraEditor = NULL;
	CameraController * camCtrl = m_Object->getCurrentCameraController();
	if( camCtrl )
	{
	m_CameraEditor = (ClassEditor*)camCtrl->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()
		->newInstance();
	m_CameraEditorWidget = m_CameraEditor->makeEditor(NULL, m_Object->getCurrentCameraController());
	m_CameraEditorWidget->hide();
	}

}

void PCICameraManagerEditorWidget::on_pbEdit_clicked() 
{
	if( m_CameraEditorWidget )
		m_CameraEditorWidget->show();
}

