#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIDrawingToolEditorWidget.h"
//---------------------------------------------------------------------------
#include "PCIDrawingTool.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( PCIDrawingTool )

PCIDrawingToolEditorWidget::PCIDrawingToolEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//Example: easy access to QtDesigner buttons, etc. by ui object
	//QWidget::connect(ui.loadButton,SIGNAL(clicked()),this,SLOT(sltLoadFile()));

}

PCIDrawingToolEditorWidget::~PCIDrawingToolEditorWidget(void)
{
}

void PCIDrawingToolEditorWidget::setup( PCIDrawingTool * obj )
{
	this->obj = obj;
}

void PCIDrawingToolEditorWidget::updateEditor()
{
	 col=obj->getColor();
	 ui.redBox->setValue(col[0]);
	 ui.greenBox->setValue(col[1]);
	 ui.blueBox->setValue(col[2]);
	 ui.brushBox->setValue(obj->tol);
	 ui.surfaceBox->setChecked(obj->getMode()!=0);
	 ui.fadeBox->setChecked(obj->fade);
	 QColor color(col[0]*255.0f, col[1]*255.0f, col[2]*255.0f);

	 ui.colorButton->setPalette(QPalette(color));
	 ui.colorButton->setAutoFillBackground(true);

	 ui.attNameEdit->setText(obj->m_AttributeName.c_str());
	 ui.attributePaintBox->setChecked(obj->m_VariableAttributePainting);
	 ui.attrVal->setValue(obj->m_BrushIntensity);
}

void PCIDrawingToolEditorWidget::on_redBox_valueChanged(double value){
	 col[0]=value;
	 sendColor();
}

void PCIDrawingToolEditorWidget::on_greenBox_valueChanged(double v){
	 col[1]=v;
	 sendColor();

}

void PCIDrawingToolEditorWidget::on_blueBox_valueChanged(double v){
	 col[2]=v;
	 sendColor();
}

void PCIDrawingToolEditorWidget::sendColor(){
  obj->setColor(col);
  updateEditor();
}

void PCIDrawingToolEditorWidget::on_brushBox_valueChanged(int v){
	 obj->tol=v;

}

void PCIDrawingToolEditorWidget::on_surfaceBox_clicked(){
	if(ui.surfaceBox->isChecked()){
      obj->setMode(1);
   }else{
      obj->setMode(0);
   };
    
}

void PCIDrawingToolEditorWidget::on_fadeBox_clicked(){
	 obj->fade=ui.fadeBox->isChecked();
}

void PCIDrawingToolEditorWidget::on_fillSelButton_clicked(){
	obj->fillSelection();
}



#include "PCIDrawingToolEditorWidget_moc.cpp"

void geoxl::PCIDrawingToolEditorWidget::on_colorButton_clicked()
{

	Vector3f color3f = obj->getColor();
	QColor color = QColorDialog::getColor(QColor(color3f[0],color3f[1],color3f[2]));
	if (!color.isValid()) return;

	col = makeVector3f(color.red(),color.green(),color.blue())/255.0f;
	sendColor();

}

void geoxl::PCIDrawingToolEditorWidget::on_attributePaintBox_clicked(bool b)
{
	obj->m_VariableAttributePainting=b;
}

void geoxl::PCIDrawingToolEditorWidget::on_attNameEdit_textChanged()
{
	QString test= ui.attNameEdit->text();
	std::string s=test.toLatin1().constData();
	obj->m_AttributeName= s;
}

void geoxl::PCIDrawingToolEditorWidget::on_attrVal_valueChanged( double v)
{
	obj->m_BrushIntensity=v;
}