#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCISelectionToolEditorWidget.h"
//---------------------------------------------------------------------------
#include "PCISelectionTool.h"
#include "SelectionTools.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( PCISelectionTool )

PCISelectionToolEditorWidget::PCISelectionToolEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	// used Qt auto-connect feature for slots
}

PCISelectionToolEditorWidget::~PCISelectionToolEditorWidget(void)
{
}

void PCISelectionToolEditorWidget::setup( PCISelectionTool * obj )
{
	m_SelTool = obj;
}

void PCISelectionToolEditorWidget::updateEditor()
{
	updateButtons();

	ui.CBSurfaceMode->setChecked(m_SelTool->getSurfaceMode());
	ui.CBDeselect->setChecked(m_SelTool->getDeselect());

	Vector3f color = m_SelTool->getSelColor()*255.0f;
	ui.BTAttribColorSelector->setPalette(QPalette(QColor(color[0],color[1],color[2])));

	ui.SBBrushSize->setValue(m_SelTool->getBrushSize());
	ui.SBAttribColorTolerance->setValue(m_SelTool->getTolerance());
}


// ---------- user functions ----------

void PCISelectionToolEditorWidget::updateButtons() {
	card32 mode = m_SelTool->getMode();
	if (mode!=SELMODE_SINGLECLOUD) ui.BTSingleCloud->setChecked(false);
	if (mode!=SELMODE_SINGLEPOINT) ui.BTSinglePoint->setChecked(false);
	if (mode!=SELMODE_BRUSHCIRCLE) ui.BTBrushCircle->setChecked(false);
	if (mode!=SELMODE_BRUSHQUAD) ui.BTBrushQuadratic->setChecked(false);
	if (mode!=SELMODE_AREACIRCLE) ui.BTAreaCircle->setChecked(false);
	if (mode!=SELMODE_AREARECTANGLE) ui.BTAreaRectangle->setChecked(false);
	switch (mode) {
		case SELMODE_SINGLECLOUD: ui.BTSingleCloud->setChecked(true); break;
		case SELMODE_SINGLEPOINT: ui.BTSinglePoint->setChecked(true); break;
		case SELMODE_BRUSHCIRCLE: ui.BTBrushCircle->setChecked(true); break;
		case SELMODE_BRUSHQUAD: ui.BTBrushQuadratic->setChecked(true); break;
		case SELMODE_AREACIRCLE: ui.BTAreaCircle->setChecked(true); break;
		case SELMODE_AREARECTANGLE: ui.BTAreaRectangle->setChecked(true); break;
      default: break;
   };
}


// ---------- slots ----------

void PCISelectionToolEditorWidget::on_BTSingleCloud_clicked() {
	m_SelTool->setMode(SELMODE_SINGLECLOUD);
	updateButtons();
}


void PCISelectionToolEditorWidget::on_BTSinglePoint_clicked() {
	m_SelTool->setMode(SELMODE_SINGLEPOINT);
	updateButtons();
}


void PCISelectionToolEditorWidget::on_BTBrushCircle_clicked() {
	m_SelTool->setMode(SELMODE_BRUSHCIRCLE);
	updateButtons();
}


void PCISelectionToolEditorWidget::on_BTBrushQuadratic_clicked() {
	m_SelTool->setMode(SELMODE_BRUSHQUAD);
	updateButtons();
}


void PCISelectionToolEditorWidget::on_SBBrushSize_valueChanged(int value) {
	m_SelTool->setBrushSize(value);
}


void PCISelectionToolEditorWidget::on_CBSurfaceMode_stateChanged(int state) {
	m_SelTool->setSurfaceMode(state==Qt::Checked);
}


void PCISelectionToolEditorWidget::on_BTAreaCircle_clicked() {
	m_SelTool->setMode(SELMODE_AREACIRCLE);
	updateButtons();
}


void PCISelectionToolEditorWidget::on_BTAreaRectangle_clicked() {
	m_SelTool->setMode(SELMODE_AREARECTANGLE);
	updateButtons();
}


void PCISelectionToolEditorWidget::on_BTClearClouds_clicked() {
	m_SelTool->clearCloudSelection();
}


void PCISelectionToolEditorWidget::on_BTAllClouds_clicked() {
   m_SelTool->selectAllClouds();
}


void PCISelectionToolEditorWidget::on_BTClearPoints_clicked() {
	m_SelTool->clearPointSelection();
}


void PCISelectionToolEditorWidget::on_BTAllPoints_clicked() {
   m_SelTool->selectAllPoints();
}


void PCISelectionToolEditorWidget::on_BTInvertPoints_clicked() {
    m_SelTool->invertSelection();
}


void PCISelectionToolEditorWidget::on_BTAttribColorSelector_clicked() {
	Vector3f color3f = m_SelTool->getSelColor()*255.0f;
	QColor color = QColorDialog::getColor(QColor(color3f[0],color3f[1],color3f[2]));
	if (!color.isValid()) return;
	ui.BTAttribColorSelector->setPalette(QPalette(color));
	color3f = makeVector3f(color.red(),color.green(),color.blue())/255.0f;
	m_SelTool->setSelColor(color3f);
}


void PCISelectionToolEditorWidget::on_SBAttribColorTolerance_valueChanged(int value) {
	m_SelTool->setTolerance(value);
}


void PCISelectionToolEditorWidget::on_BTAttribColorExecute_clicked() {
	m_SelTool->selectByColor();
}


void PCISelectionToolEditorWidget::on_CBDeselect_stateChanged(int state) {
   m_SelTool->setDeselect(state==Qt::Checked);
}


#include "PCISelectionToolEditorWidget_moc.cpp"
