#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "PCITransfEditEditorWidget.h"
//---------------------------------------------------------------------------
#include "PCITransfEdit.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( PCITransfEdit )

PCITransfEditEditorWidget::PCITransfEditEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	// used Qt auto-connect feature for other slots
	QWidget::connect(ui.RBMove, SIGNAL(toggled(bool)), this, SLOT(sltTransfModeToggled(bool)));
	QWidget::connect(ui.RBRotate, SIGNAL(toggled(bool)), this, SLOT(sltTransfModeToggled(bool)));
	QWidget::connect(ui.RBScale, SIGNAL(toggled(bool)), this, SLOT(sltTransfModeToggled(bool)));
}

PCITransfEditEditorWidget::~PCITransfEditEditorWidget(void)
{
}

void PCITransfEditEditorWidget::setup( PCITransfEdit * obj )
{
	m_TransfEdit = obj;
}

void PCITransfEditEditorWidget::updateEditor()
{
	ui.CBWorldCoordinates->setChecked(m_TransfEdit->getAlignWorld());
	ui.CBLockPivot->setChecked(m_TransfEdit->getLockRotScaleCent());

	ui.CBStepwiseMoveBy->setChecked(m_TransfEdit->getSteppedMovement());
	ui.SBMoveBy->setValue(m_TransfEdit->getSteppedMovementValue());
	ui.CBStepwiseRotateBy->setChecked(m_TransfEdit->getSteppedRotation());
	ui.SBRotateBy->setValue(m_TransfEdit->getSteppedRotationDegrees());

	ui.CBShowWorldCS->setChecked(m_TransfEdit->getShowWorldCoordinateSystem());
	ui.SBWorldCSSize->setValue(m_TransfEdit->getCSWorldScale());
	ui.CBShowPivotCS->setCheckState((Qt::CheckState)m_TransfEdit->getShowPivotCoordinateSystem()); // hack: 0 off, 1 partially on, 2 on
	ui.SBPivotCSSize->setValue(m_TransfEdit->getCSPivotScale());

	if (m_TransfEdit->getTransPoints()) {
		ui.BTAffectClouds->setChecked(false);
		ui.BTAffectPoints->setChecked(true);
	} else {
		ui.BTAffectClouds->setChecked(true);
		ui.BTAffectPoints->setChecked(false);
	}

	ui.BTMovePivot->setChecked(m_TransfEdit->getMoveRotScaleCent());

	switch (m_TransfEdit->getMode()) {
		case 0: ui.RBMove->setChecked(true); ui.DirectValuesLabel1->setText("move by"); ui.DirectValuesLabel2->setText("(distance)"); break;
		case 1: ui.RBRotate->setChecked(true); ui.DirectValuesLabel1->setText("rotate by"); ui.DirectValuesLabel2->setText("(degrees)"); break;
		case 2: ui.RBScale->setChecked(true); ui.DirectValuesLabel1->setText("scale by"); ui.DirectValuesLabel2->setText("(1=identity)"); break;
		default: pAssert(false); break;
	};

}



// ---------- slots ----------

void PCITransfEditEditorWidget::sltTransfModeToggled(bool) {

	if (ui.RBScale->isChecked()) {
		m_TransfEdit->setMode(2);
		ui.DirectValuesLabel1->setText("scale by");
		ui.DirectValuesLabel2->setText("(1=identity)");
	} else if (ui.RBRotate->isChecked()) {
		m_TransfEdit->setMode(1);
		ui.DirectValuesLabel1->setText("rotate by");
		ui.DirectValuesLabel2->setText("(degrees)");
	} else {
		m_TransfEdit->setMode(0);
		ui.DirectValuesLabel1->setText("move by");
		ui.DirectValuesLabel2->setText("(distance)");
	}

}


void PCITransfEditEditorWidget::on_BTAffectClouds_clicked() {
	ui.BTAffectClouds->setChecked(true);
	ui.BTAffectPoints->setChecked(false);
	m_TransfEdit->setTransPoints(false);
}


void PCITransfEditEditorWidget::on_BTAffectPoints_clicked() {
	ui.BTAffectClouds->setChecked(false);
	ui.BTAffectPoints->setChecked(true);
	m_TransfEdit->setTransPoints(true);
}


void PCITransfEditEditorWidget::on_CBWorldCoordinates_stateChanged(int state) {
	m_TransfEdit->setAlignWorld(state==Qt::Checked);
}


void PCITransfEditEditorWidget::on_BTApplyDirectValues_clicked() {
	m_TransfEdit->executeXYZ(ui.SBDirectX->value(), ui.SBDirectY->value(), ui.SBDirectZ->value());
}


void PCITransfEditEditorWidget::on_BTMovePivot_clicked(bool checked) {
	m_TransfEdit->setMoveRotScaleCent(checked);
}


void PCITransfEditEditorWidget::on_BTCenterOnSelectedClouds_clicked() {
   m_TransfEdit->centerRotScaleCentToSelClouds();
}


void PCITransfEditEditorWidget::on_BTCenterOnSelectedPoints_clicked() {
	m_TransfEdit->centerRotScaleCentToSelPoints();
}


void PCITransfEditEditorWidget::on_CBLockPivot_stateChanged(int state) {
	m_TransfEdit->setLockRotScaleCent(state==Qt::Checked);
}


void PCITransfEditEditorWidget::on_CBStepwiseMoveBy_stateChanged(int state) {
	m_TransfEdit->setSteppedMovement(state==Qt::Checked);
}


void PCITransfEditEditorWidget::on_CBStepwiseRotateBy_stateChanged(int state) {
	m_TransfEdit->setSteppedRotation(state==Qt::Checked);
}


void PCITransfEditEditorWidget::on_SBMoveBy_valueChanged(double value) {
	m_TransfEdit->setSteppedMovementValue(value);
}


void PCITransfEditEditorWidget::on_SBRotateBy_valueChanged(double value) {
	m_TransfEdit->setSteppedRotationDegrees(value);
}


void PCITransfEditEditorWidget::on_CBShowWorldCS_stateChanged(int state) {
	m_TransfEdit->setShowWorldCoordinateSystem(state==Qt::Checked);
}


void PCITransfEditEditorWidget::on_CBShowPivotCS_stateChanged(int state) {
	m_TransfEdit->setShowPivotCoordinateSystem(state); // hack: 0 off, 1 partially on, 2 on
}


void PCITransfEditEditorWidget::on_SBWorldCSSize_valueChanged(double value) {
	m_TransfEdit->setCSWorldScale(value);
}


void PCITransfEditEditorWidget::on_SBPivotCSSize_valueChanged(double value) {
	m_TransfEdit->setCSPivotScale(value);
}


#include "PCITransfEditEditorWidget_moc.cpp"
