#include "StdAfx.h"
//---------------------------------------------------------------------------
#include <qtGui/qApplication.h>
#include "PCSceneMaterialBrowserWidget.h"
#include "ClassSelectionDialog.h"
#include "GLMaterial.h"
#include "StringHelper.h"
#include "ClassEditor.h"
#include "GLShaderMaterialEditorWidget.h"
#include "BinaryObjectStreams.h"
#include "InterfaceGlobals.h"
//---------------------------------------------------------------------------
using namespace geoxl;

PCSceneMaterialBrowserWidget::PCSceneMaterialBrowserWidget(QWidget * parent , Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	setupUi(this);
    
	/*ootIsDecorated=false and itemsExpandable=false (maybe also uniformRowHeights=true), 
	then you can add columns with treeWidget->setHeaderLabels().*/
	
	QHBoxLayout * l1 = new QHBoxLayout( ViewFrame );
	treeWidget = new MatTreeWidget( ViewFrame );
	l1->addWidget( treeWidget );
	l1->setMargin(0);
	l1->setSpacing(0);
	connect( treeWidget, SIGNAL(itemDoubleClicked(QTreeWidgetItem*, int)), SLOT(treeWidget_itemDoubleClicked(QTreeWidgetItem*, int)));
	
	
	treeWidget->setHeaderLabels(QStringList() << "#" << "Name");
	treeWidget->setRootIsDecorated(false);
	treeWidget->setItemsExpandable(false);
	m_Scene = NULL;
   m_GLContext = NULL;
}


 //  classPopup = makeClassPopup(this, cvOnClassSelection, GLMaterial::getClass());


Object * PCSceneMaterialBrowserWidget::getObject()
{
   return m_Scene;
}

void PCSceneMaterialBrowserWidget::apply()
{
}

void PCSceneMaterialBrowserWidget::update()
{
   pAssert( m_Scene != NULL );
	treeWidget->clear();
   for( card32 i=0;i<m_Scene->getNumGLMaterials();i++ )
   {
      GLMaterial * mat = m_Scene->getGLMaterial(i);
		QStringList s;
		s << intToStr(i).c_str();
		s << QString(mat->getName().c_str());
		QTreeWidgetItem *li = new QTreeWidgetItem(treeWidget, s);
   }
}

void PCSceneMaterialBrowserWidget::setup( Scene * scene )
{
   m_Scene = scene;
}

void PCSceneMaterialBrowserWidget::on_addButton_clicked()
{
   pAssert( m_Scene != NULL);
	ClassSelectionDialog	*	m_ClassSelectionDialog=new ClassSelectionDialog(this);
	m_ClassSelectionDialog->init( GLMaterial::getClass() , false );
		m_ClassSelectionDialog->move( QCursor::pos() );
		if( m_ClassSelectionDialog->exec() == QDialog::Accepted )
		{
			MetaClass * mc = m_ClassSelectionDialog->getSelectedClass();

			m_Scene->addGLMaterial( (GLMaterial *)mc->newInstance());

		}
  delete m_ClassSelectionDialog;
  update();

}


void PCSceneMaterialBrowserWidget::on_editButton_clicked()
{
   pAssert( m_Scene!=NULL );

	if (!treeWidget->currentIndex().isValid()) {
		 QMessageBox::information( this, "Error", "select a material first" );
	}else{
      GLMaterial * mat = m_Scene->getGLMaterial( treeWidget->currentIndex().row() );
      if( mat )
      {
        /* if( (m_Browser == NULL ) || ( FindComponent( "ObjectBrowser" )==NULL ) )
         {
            m_Browser = new TObjectBrowser(this);
            m_Browser->Name = "ObjectBrowser";
         }
         m_Browser->Show();*/
         ClassEditor * classEditor = (ClassEditor *)mat->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
			QWidget* browser= classEditor->makeEditor(dynamic_cast<QWidget*>(parent()),mat);
			browser->setWindowFlags(Qt::Dialog);

         // UGLY  (needed to set gl context)
         GLShaderMaterialEditorWidget * shmatEditor = dynamic_cast<GLShaderMaterialEditorWidget*>(browser);
			if( shmatEditor ){
            shmatEditor->setTestContext( m_GLContext );
				shmatEditor->updateEditor();
			}
			browser->setAttribute(Qt::WA_DeleteOnClose);
			browser->show();
			if (browser->x() < 0 || browser->y() < 0) {
			   QPoint me = mapToGlobal(QPoint(0,0));
			   browser->move(me.x()+100, me.y()+10);
			}
      }
   }

}

void PCSceneMaterialBrowserWidget::treeWidget_itemDoubleClicked(QTreeWidgetItem* item, int col){
	 on_editButton_clicked();
}

void PCSceneMaterialBrowserWidget::on_loadButton_clicked(){
	QString filename = QFileDialog::getOpenFileName(this, "open file","", "*.material.object" );
	 if (filename.isEmpty()) return;
	 BinaryInputObjectStream in(qPrintable(filename));
    Persistent *instance = NULL;
    in.readObject(instance);
	 if( dynamic_cast<GLMaterial*>(instance) == NULL ){
		  QMessageBox::information( this, "Error", "this is not a material" );
		  delete instance;
	 }
		else
      {
         m_Scene->addGLMaterial( (GLMaterial *)instance );
         update();
      }
}

void PCSceneMaterialBrowserWidget::on_saveButton_clicked(){
	if (!treeWidget->currentIndex().isValid()) {
		 QMessageBox::information( this, "Error", "select a material first" );
	}else{
			QString filename = QFileDialog::getSaveFileName( this, "save file",  "", "*.material.object" );
			if( !filename.isEmpty() )
			{
				if( !filename.contains( ".material.object", Qt::CaseInsensitive ) )
					filename += ".material.object";
				BinaryOutputObjectStream out( filename.toLatin1().constData() );
				out.writeObject( m_Scene->getGLMaterial( treeWidget->currentIndex().row() ) );
			}
	}
}


/*


void  PCSceneMaterialBrowserWidget::ButtonSaveClick()
{
   pAssert( m_Scene!=NULL );
   if( MatList->ItemIndex < 0 )
      Application->MessageBox("select a material first","Info", MB_OK | MB_ICONERROR);
   else
   {
      if( SaveDialog->Execute() )
      {
         BinaryOutputObjectStream out(SaveDialog->FileName.c_str());
         out.writeObject( m_Scene->getGLMaterial( MatList->ItemIndex ) );
      }
   }
   forget(Sender);
}
//---------------------------------------------------------------------------


*/

MatTreeWidget::MatTreeWidget(QWidget * parent)
:QTreeWidget(parent)
{
	m_drawItemID = -1;
}

void MatTreeWidget::mousePressEvent(QMouseEvent *e)
{
	m_drawItemID = -1;
	if (e->button() == Qt::LeftButton)
	{
		QTreeWidgetItem * item = this->itemAt( e->pos() );
		if( item != NULL )
		{
			m_drawItemID = this->indexOfTopLevelItem( item );
			m_dragStartPosition = e->pos();
		}
	}

	QTreeWidget::mousePressEvent(e);
}

void MatTreeWidget::mouseMoveEvent(QMouseEvent *e)
{
	if ((e->buttons() & Qt::LeftButton) && (m_drawItemID >= 0))
	{
		if ((e->pos() - m_dragStartPosition).manhattanLength()
			 < QApplication::startDragDistance())
			return;

		QDrag *drag = new QDrag(this);
		QMimeData *mimeData = new QMimeData;

		QByteArray matData;
		matData.resize( 4 );
		char * tmp = (char*)&m_drawItemID;
		matData[0] = tmp[0];
		matData[1] = tmp[1];
		matData[2] = tmp[2];
		matData[3] = tmp[3];
		mimeData->setData("geoxl/material", matData);
		drag->setMimeData(mimeData);

		Qt::DropAction dropAction = drag->start(Qt::CopyAction | Qt::MoveAction);
	}
	QTreeWidget::mouseMoveEvent(e);

}



#include "PCSceneMaterialBrowserWidget_moc.cpp"
