//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PointCloudConversionForms.h"
//---------------------------------------------------------------------------
#include "Tools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------




PointCloudConversionWidget::PointCloudConversionWidget(QWidget* parent)
   : QDialog(parent)
{
	ui.setupUi(this);
	ui.rb_unstructured->setChecked(true);
	ui.spinBoxAllocatorIndex->setEnabled(false);
   ui.sb_maxNodes->setEnabled(false);
   ui.sb_maxLeaf->setEnabled(false);
   ui.sb_mrGrid->setEnabled(false);
}

void PointCloudConversionWidget::execute(QWidget* Owner, int &action, bool &delOriginal,
                                        card32 &allocatorIndex, int &maxNodesInMem,
                                        int &pointsInLeaf, int &mrGrid, card32 numAllocatorsInSGState) {
   PointCloudConversionWidget *diag = new PointCloudConversionWidget(Owner);
   if (numAllocatorsInSGState <=0) {
		diag->ui.spinBoxAllocatorIndex->setRange(-1, -1);
	} else {
	   diag->ui.spinBoxAllocatorIndex->setRange(0, numAllocatorsInSGState-1);
	}
   diag->show();
   if (diag->exec()==QDialog::Accepted) {
	   if (diag->ui.rb_unstructured->isChecked()) {
         action = UNSTRUCTURED;
	   } else if (diag->ui.rb_hierin->isChecked()) {
         action = HIERARCHICAL;
      } else {
         if (numAllocatorsInSGState == 0) {
				QMessageBox::critical(
					diag, "Cannot create out-of-core point cloud.", 
					"A hierarchical out-of-core point cloud cannot be created because\n"
					"no allocators are availabe. Allocators can be created in the \"allocators\"\n"
					" section of the static scene graph state (you can view the scene graph state\n"
					" using the \"View\" menue of the scene editor).",
					QMessageBox::Ok, QMessageBox::NoButton
				);
				action = CANCEL;
         } else {
				action = OUT_OF_CORE;
			}
      }
	  delOriginal = diag->ui.cb_delOriginal->isChecked();
	  allocatorIndex = diag->ui.spinBoxAllocatorIndex->value();
      maxNodesInMem = diag->ui.sb_maxNodes->value();
      pointsInLeaf = diag->ui.sb_maxLeaf->value();
      mrGrid = diag->ui.sb_mrGrid->value();
   } else {
      action = CANCEL;
      delOriginal = false;
		allocatorIndex = 0;
      maxNodesInMem = 0;
      pointsInLeaf = 0;
      mrGrid = 0;
   }
}
void  PointCloudConversionWidget::on_rb_unstructured_clicked()
   
{
   ui.spinBoxAllocatorIndex->setEnabled(false);
   ui.sb_maxNodes->setEnabled(false);
   ui.sb_maxLeaf->setEnabled(false);
   ui.sb_mrGrid->setEnabled(false);
   
}
//---------------------------------------------------------------------------

void PointCloudConversionWidget::on_rb_hierout_clicked()

{
   ui.spinBoxAllocatorIndex->setEnabled(true);
   ui.sb_maxNodes->setEnabled(true);
   ui.sb_maxLeaf->setEnabled(true);
   ui.sb_mrGrid->setEnabled(true);
   
}
//---------------------------------------------------------------------------

void PointCloudConversionWidget::on_rb_hierin_clicked()
{
   ui.spinBoxAllocatorIndex->setEnabled(false);
   ui.sb_maxNodes->setEnabled(false);
   ui.sb_maxLeaf->setEnabled(true);
   ui.sb_mrGrid->setEnabled(true);
}
//---------------------------------------------------------------------------
#include "PointCloudConversionForms_moc.cpp"

//---------------------------------------------------------------------------

