#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SceneEditorOptionsEditorWidget.h"
//---------------------------------------------------------------------------
#include "SceneEditorOptions.h"
//---------------------------------------------------------------------------
#include "FreeCameraController.h"
#include "ExaminerCameraController.h"
#include "CameraPathRecorder.h"
#include "WalkthroughCameraController.h"
#include "Configuration.h"
#include <qtGUI/qcolordialog.h>
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( SceneEditorOptions )

SceneEditorOptionsEditorWidget::SceneEditorOptionsEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//Example: easy access to QtDesigner buttons, etc. by ui object
	//QWidget::connect(ui.loadButton,SIGNAL(clicked()),this,SLOT(sltLoadFile()));

	ui.pbBGColor->setAutoFillBackground(true);
}

SceneEditorOptionsEditorWidget::~SceneEditorOptionsEditorWidget(void)
{
}

void SceneEditorOptionsEditorWidget::setup( SceneEditorOptions * obj )
{
	m_Object = obj;
}

void SceneEditorOptionsEditorWidget::updateEditor()
{
	ui.cbDefaultCamera->setCurrentIndex(0);
	if( m_Object->getDefaultCameraClass() == ExaminerCameraController::getClass() )
		ui.cbDefaultCamera->setCurrentIndex(1);
	if( m_Object->getDefaultCameraClass() == CameraPathRecorder::getClass() )
		ui.cbDefaultCamera->setCurrentIndex(2);
	if( m_Object->getDefaultCameraClass() == WalkthroughCameraController::getClass() )
		ui.cbDefaultCamera->setCurrentIndex(3);

	Vector3f bg_col = m_Object->getBackGroundColor();
	QPalette pal;
	pal.setBrush( QPalette::Button, toColor( bg_col ) );
	ui.pbBGColor->setPalette( pal );

	ui.cbUseSoftwareVertexBuffers->setChecked( m_Object->getUseSoftwareVertexBuffers() );

	int redrawDelay = m_Object->getRedrawDelay();
	if( redrawDelay <= -1 )
		ui.cbRedrawMode->setCurrentIndex(0);
	if( redrawDelay == 0)
		ui.cbRedrawMode->setCurrentIndex(1);

	if( redrawDelay > 0 && redrawDelay <= 10)
		ui.cbRedrawMode->setCurrentIndex(2);
	if( redrawDelay > 10 && redrawDelay <= 20)
		ui.cbRedrawMode->setCurrentIndex(3);
	if( redrawDelay > 20 && redrawDelay <= 40)
		ui.cbRedrawMode->setCurrentIndex(4);
	if( redrawDelay > 40)
		ui.cbRedrawMode->setCurrentIndex(5);

	ui.bUseBoundingBoxSelection->setChecked( m_Object->getUseBBSelection() );
}

void SceneEditorOptionsEditorWidget::on_cbDefaultCamera_currentIndexChanged(int index)
{
	switch( index )
	{
	case 0:
		m_Object->setDefaultCameraClass( (MetaClass*)FreeCameraController::getClass() );
		break;
	case 1:
		m_Object->setDefaultCameraClass( (MetaClass*)ExaminerCameraController::getClass() );
		break;
	case 2:
		m_Object->setDefaultCameraClass( (MetaClass*)CameraPathRecorder::getClass() );
		break;
	case 3:
		m_Object->setDefaultCameraClass( (MetaClass*)WalkthroughCameraController::getClass() );
		break;
	}
}

void SceneEditorOptionsEditorWidget::on_pbSave_clicked()
{
	SceneEditorOptions * opt = (SceneEditorOptions*)Configuration->getData("SCENE_EDITOR_OPTIONS");
	opt->assign( m_Object );
}

void SceneEditorOptionsEditorWidget::on_pbLoad_clicked()
{
	SceneEditorOptions * opt = (SceneEditorOptions*)Configuration->getData("SCENE_EDITOR_OPTIONS");
	m_Object->assign( opt );
	updateEditor();
}

void SceneEditorOptionsEditorWidget::on_pbBGColor_clicked()
{
	QColor col = QColorDialog::getColor( toColor( m_Object->getBackGroundColor() ) ,this );
	if( col.isValid() )
	{
		m_Object->setBackGroundColor( toVec3f( col ) );
		updateEditor();
	}
}


Vector3f SceneEditorOptionsEditorWidget::toVec3f( QColor col )
{
	return makeVector3f( col.red(), col.green(), col.blue() ) /255.0f;
}

QColor SceneEditorOptionsEditorWidget::toColor( Vector3f col )
{
	return 	QColor( col[0]*255.0f,col[1]*255.0f,col[2]*255.0f );

}

#include "SceneEditorOptionsEditorWidget_moc.cpp"
void SceneEditorOptionsEditorWidget::on_cbUseSoftwareVertexBuffers_clicked() 
{
	m_Object->setUseSoftwareVertexBuffers( ui.cbUseSoftwareVertexBuffers->checkState() == Qt::Checked );
}

void SceneEditorOptionsEditorWidget::on_cbRedrawMode_currentIndexChanged(int index) 
{
	int redrawDelay = -1;
	switch(index)
	{
	case 1:
		redrawDelay = 0;
		break;
	case 2:
		redrawDelay = 10;
		break;
	case 3:
		redrawDelay = 20;
		break;
	case 4:
		redrawDelay = 40;
		break;
	case 5:
		redrawDelay = 100;
		break;
	}
	m_Object->setRedrawDelay(redrawDelay);
}

void SceneEditorOptionsEditorWidget::on_bUseBoundingBoxSelection_clicked()
{
	m_Object->setUseBBSelection(ui.bUseBoundingBoxSelection->isChecked());
}