#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCICameraManager.h"
//---------------------------------------------------------------------------
#include "CameraController.h"
#include "Camera.h"
#include "SGCameraNode.h"
#include "StandardSceneGraphIterator.h"
#include "SceneGraphState.h"
#include "StringClassProperty.h"
#include "PointCloudTools.h"
#include "ExaminerCameraController.h"
#include "FreeCameraController.h"
#include "WalkthroughCameraController.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( PCICameraManager ,0)
{
	BEGIN_CLASS_INIT( PCICameraManager );
	ADD_STRING_PROP( m_SelectedCamera,0);
}

PCICameraManager::PCICameraManager(void)
{
}

PCICameraManager::~PCICameraManager(void)
{
}

void PCICameraManager::assign(const Object* obj, CopyContext *context)
{
	PCInteractionTool::assign(obj,context);
	const PCICameraManager * o = dynamic_cast<const PCICameraManager*>(obj);
	pAssert( o != NULL );

}
void PCICameraManager::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
	CameraController * camCon = getCurrentCameraController();
	if( camCon )
		camCon->mouseDown( x,y, buttonsState, modifiersState );
   update();
}

void PCICameraManager::mouseMoved(int32 x, int32 y) 
{
	CameraController * camCon = getCurrentCameraController();
	if( camCon )
		camCon->mouseMoved( x,y);
   update();
}

void PCICameraManager::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) 
{
	CameraController * camCon = getCurrentCameraController();
	if( camCon )
		camCon->mouseUp( x,y, buttonsState, modifiersState );
   update();
}

void PCICameraManager::areaResize(card32 width, card32 height) 
{
	CameraController * camCon = getCurrentCameraController();
	if( camCon )
		camCon->areaResize( width, height );
   update();
}


void PCICameraManager::keyDown(GeneralKey key) 
{
	CameraController* camCon = getCurrentCameraController();
	if ( key.getKey() == keyF5 ){
		BoundingBox3f bb = getScenePCBBox( scene );
		if ( camCon != NULL ){
			Camera* camera = camCon->getCamera();
			Vector3f dir = normalize( camera->getLookAt() - camera->getPosition() );
			Vector3f newPos = bb.getCenter() + dir * 2.0f*bb.getMaxSideLength();
			camCon->setup( newPos, bb.getCenter(), camera->getUp() );  
		}
		ViewFrustum * currentViewFrustum = getCurrentViewFrustum();
		currentViewFrustum->setZNearClip( 0.1 * bb.getMaxSideLength() );
		currentViewFrustum->setZFarClip( 10.0 * bb.getMaxSideLength() );
	} else {
		if( camCon ) camCon->keyDown( key );
	}
	update();
}

void PCICameraManager::keyUp(GeneralKey key) 
{
	CameraController * camCon = getCurrentCameraController();
	if( camCon )
		camCon->keyUp( key );
   update();
}

void PCICameraManager::connectScene(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget) 
{
	PCInteractionTool::connectScene(scene,onChange, sceneEditorWidget);
}

void PCICameraManager::activateTool() 
{
	m_MainCamera = getScene()->getRootState()->staticState->camera;
	m_MainFrustum = getScene()->getRootState()->staticState->viewFrustum;
	update();
}

void PCICameraManager::deactivateTool() 
{
	// restore camera in scenegraphstate
	Camera cam = m_MainCamera;
	ViewFrustum vf = m_MainFrustum;

	getScene()->getRootState()->staticState->camera = cam;
	getScene()->getRootState()->staticState->viewFrustum.setZNearClip( vf.getZNearClip());
	getScene()->getRootState()->staticState->viewFrustum.setZFarClip( vf.getZFarClip());
	getScene()->getRootState()->staticState->viewFrustum.setVerticalFieldOfView( vf.getVerticalFieldOfView());
}


SGCameraNode * PCICameraManager::getCameraNode( string nodename ) 
{
   StandardSceneGraphIterator sceneIt;
   sceneIt.clearAndSetup(scene->getRootNode(), scene->getRootState(), SGDynamicState::tmRendering, SceneGraphNode::FLAG_VISIBLE, 0);

   while (!sceneIt.atEnd()) 
	{
		if( sceneIt.getNameWithPath() == nodename )
		{
			SGCameraNode * ret = dynamic_cast<SGCameraNode*>(sceneIt.getNode());
			return ret;
		}

		sceneIt.next();
	}
	return NULL;
}

Camera * PCICameraManager::getCurrentCamera() 
{
	SGCameraNode * node = getCameraNode( m_SelectedCamera );
	if( node )
		return node->getCamera();
	else
		return NULL;
}

CameraController* PCICameraManager::getCurrentCameraController() 
{
	SGCameraNode * node = getCameraNode( m_SelectedCamera );
	if( node )
		return node->getCameraController();
	else
		return NULL;
}

ViewFrustum * PCICameraManager::getCurrentViewFrustum() 
{
	SGCameraNode * node = getCameraNode( m_SelectedCamera );
	if( node )
		return node->getViewFrustum();
	else
		return NULL;

}

void PCICameraManager::update() 
{
	Camera cam = m_MainCamera;
	ViewFrustum vf = m_MainFrustum;
	if( m_SelectedCamera.length() )
	{
		Camera * currentCamera = getCurrentCamera();
		if( currentCamera != NULL )
			cam = *currentCamera;

		ViewFrustum * currentViewFrustum = getCurrentViewFrustum();
		if( currentViewFrustum != NULL )
			vf = *currentViewFrustum;

	}

	getScene()->getRootState()->staticState->camera = cam;
	getScene()->getRootState()->staticState->viewFrustum.setZNearClip( vf.getZNearClip());
	getScene()->getRootState()->staticState->viewFrustum.setZFarClip( vf.getZFarClip());
	getScene()->getRootState()->staticState->viewFrustum.setVerticalFieldOfView( vf.getVerticalFieldOfView());
	
}
void PCICameraManager::setTime(int time) 
{
	CameraController * ctrl = getCurrentCameraController();
	if( ctrl )
		ctrl->setTime(time);
}
