#ifndef PCICameraManagerH
#define PCICameraManagerH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
#include "Camera.h"
#include "ViewFrustum.h"
//---------------------------------------------------------------------------

namespace geoxl
{

class SGCameraNode;
class CameraController;
class PCICameraManagerEditorWidget;

/**
	works similar to PCICameraController, but without owning a controller.
	user-input is transfered to a CameraController located in scene graph.
*/
class EDITOR_API PCICameraManager : public PCInteractionTool
{
	GEOXL_CLASS( PCICameraManager )

private:

	SGCameraNode * getCameraNode( string nodename );
	Camera * getCurrentCamera();
	CameraController * getCurrentCameraController();
	ViewFrustum * getCurrentViewFrustum();
	void update();

	string	m_SelectedCamera;	// path in scenegraph
	Camera	m_MainCamera;		// holds camera from scenegraphstate ( saved when activateTool() is called )
	ViewFrustum m_MainFrustum;	// holds frustum from scenegraphstate ( saved when activateTool() is called )

	friend PCICameraManagerEditorWidget;

public:
	PCICameraManager(void);
	~PCICameraManager(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void connectScene(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget);

	virtual void activateTool();
	virtual void deactivateTool();

	void setSelectedCamera( string selCam ) {m_SelectedCamera = selCam;}
	string getSelectedCamera() {return m_SelectedCamera;}

	void setTime(int time);

};

}

#endif