//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PCIDrawingTool.h"
//---------------------------------------------------------------------------
//#include "PCIDrawingToolEditor.h"
#include "CameraController.h"
#include "ExaminerCameraController.h"
#include "ObjectClassProperty.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "DebugOutput.h"
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "NumericalClassProperty.h"
#include "PointSet.h"
#include "SceneCommandObjects.h"
#include "CommandHistory.h"
#include "SelectionIteratorAdaptor.h"
#include "ObjectViewsTable.h"
#include "CmdObjDrawOnClick.h"
#include "CmdObjSpecialDraw.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "MathTools.h"
#include "PointCloudTools.h"
#include "SelectionTools.h"
#include "BooleanClassProperty.h"
#include "NumericalClassProperty.h"
#include "NumericalClassProperty_templCode.h"
#include "StringClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "LinearAlgebra_templCode.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* PCIDrawingTool::PCIDrawingToolMetaClass = NULL;

const MetaClass* PCIDrawingTool::getInstanceClass() const {
   return getClass();
}

Object* PCIDrawingTool::newInstance() {
   return new PCIDrawingTool;
}

const MetaClass* PCIDrawingTool::getClass() {
   if (PCIDrawingToolMetaClass == NULL) {
      PCIDrawingToolMetaClass = new MetaClass("PCIDrawingTool", "no description.",
                                       newInstance, sizeof(PCIDrawingTool));
   }
   return PCIDrawingToolMetaClass;
}

void PCIDrawingTool::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   PCIDrawingToolMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(PCIDrawingTool)
	 ADD_CARD32_PROP(w2 ,0)
	 ADD_CARD32_PROP(h2 ,0)
	 ADD_CARD32_PROP(w ,0)
	 ADD_CARD32_PROP(h ,0)
	 ADD_CARD32_PROP(tolerance ,0)
	 ADD_CARD32_PROP(mode ,0)
	 ADD_CARD32_PROP(tol ,0)
	 ADD_FLOAT32_PROP( WidthToHeight ,0)
	 ADD_FLOAT32_PROP( m_BrushIntensity ,0)
	 ADD_STRING_PROP(m_AttributeName , 0 );
	 ADD_BOOLEAN_PROP(m_VariableAttributePainting  , 0)
	 ADD_BOOLEAN_PROP(m_AdditiveBlending  , 0)
	 ADD_BOOLEAN_PROP( fade , 0)
	 ADD_BOOLEAN_PROP( pick , 0)
	 ADD_FLOAT32_FIXED_ARRAY_PROP( m_col, 0 , 3, 1)
	 ADD_FLOAT32_FIXED_ARRAY_PROP( pick_col, 0 , 3, 1)
	 ADD_FLOAT32_FIXED_ARRAY_PROP( pick_normal, 0 , 3, 1)
	 ADD_FLOAT32_FIXED_ARRAY_PROP( m_normal, 0 , 3, 1)
	 ADD_FLOAT32_FIXED_ARRAY_PROP( transform, 0 , 16, 4)
	 ADD_BOOLEAN_FIXED_ARRAY_PROP(m_Mask,0,4,1)


}

void PCIDrawingTool::shutdown() {
   delete PCIDrawingToolMetaClass;
}


// ----

PCIDrawingTool::PCIDrawingTool(){

   leftMBdown=false;
   tolerance=400;
   tol=20;
   mode=0;
   m_col=makeVector3f(0.2f,0.5f,1.0f);
   m_normal=makeVector3f(1.0f,0.0f,0.0f);
   pick_col=makeVector3f(0.5f,0.5f,0.5f);
   pick_normal=makeVector3f(0.0f,0.0f,0.0f);

   fade=false;
   pick=false;

   m_VariableAttributePainting = false;
   m_AttributeName = "rigidityWeight";
   m_Mask[0] = true;
   m_Mask[1] = true;
   m_Mask[2] = true;
   m_Mask[3] = true;
   m_BrushIntensity = 1.0f;
   m_AdditiveBlending = false;


};

void PCIDrawingTool::assign( const Object * obj, CopyContext * context )
{
   const PCIDrawingTool * t = dynamic_cast<const PCIDrawingTool*>(obj);
   pAssert( t != NULL );
   w2 = t->w2;
   h2 = t->h2;
   w = t->w;
   h = t->h;
   mouseMoveX = t->mouseMoveX;
   mouseMoveY = t->mouseMoveY;
   tolerance = t->tolerance;
   ctrlPressed = t->ctrlPressed;
   leftMBdown = t->leftMBdown;
   WidthToHeight = t->WidthToHeight;
   mode = t->mode;
   transform = t->transform;
   m_col = t->m_col;
   m_VariableAttributePainting = t->m_VariableAttributePainting;
   m_Mask[0] = t->m_Mask[0];
   m_Mask[1] = t->m_Mask[1];
   m_Mask[2] = t->m_Mask[2];
   m_Mask[3] = t->m_Mask[3];
   m_AttributeName = t->m_AttributeName;
   m_BrushIntensity = t->m_BrushIntensity;
   m_AdditiveBlending = t->m_AdditiveBlending;
   tol = t->tol;
   fade = t->fade;
   pick = t->pick;
   pick_col = t->pick_col;
   pick_normal = t->pick_normal;
   m_normal = t->m_normal;

   forget(context);
}


Vector3f PCIDrawingTool::getColor(){
   return m_col;
}

void PCIDrawingTool::setColor(Vector3f s){
   m_col=s;
}

            
void PCIDrawingTool::pickCol(card32 x,card32 y, bool click){


// MH - 30.01.2006 - changed SelectionTools.h and therefore this code had to be reworked
/*
   int32 selectedCloud = x;
//   int32 selectedPoint = getNearestPoint(selectedCloud, y, w, h, 20, &scene->getRootRenderState()->cam, &scene->getRootRenderState()->viewFrustum, scene);
   PointCloudIterator *newIt = getSelectedPoint(selectedCloud, y, w, h, 10, &scene->getRootState()->staticState->camera, &scene->getRootState()->staticState->viewFrustum, scene, CloudName);
   if (selectedCloud==-1){
      if(click) {
         return;
      }else{
         pick_col=makeVector3f(0.5,0.5,0.5);
         pick_normal=makeVector3f(0.0,0.0,0.0);
         return;
      }
   }
   SGLegacyPCSceneAdaptor sceneLA(scene);
   PointCloud *pc = sceneLA.getPointCloud(selectedCloud);
*/


   string CloudName;
   PointCloudIterator *returnedIt = getClosestPointIteratorFromAllPoints(x, y, w, h, 5, &scene->getRootState()->staticState->camera, &scene->getRootState()->staticState->viewFrustum, scene, CloudName);

   if (returnedIt==NULL) {
      if(click) {
         return;
      }else{
         pick_col=makeVector3f(0.5,0.5,0.5);
         pick_normal=makeVector3f(0.0,0.0,0.0);
         return;
      }
   }

   PointCloud* pc = getPointCloud(scene, CloudName);
   if ((pc==NULL) || (!pc->providesAttribute("color"))) { delete returnedIt; return; };
   AAT COLOR = pc->getAAT("color");
   AAT NORMAL;
   if (pc->providesAttribute("normal")) NORMAL = pc->getAAT("normal");

   //PointCloudIterator *newIt = pc->createIterator(PointCloudIteratorType(0));
   //if (newIt == NULL) return;
   BasicPointCloudIterator *baseIt = dynamic_cast<BasicPointCloudIterator*>(returnedIt);
   if (baseIt == NULL) {delete returnedIt; return ;}

   // zum entspr Punkt durchlaufen
/*   for (int32 n=0; n<selectedPoint; n++) {
      baseIt->next();
   }
  */
   if(click){
      m_col=baseIt->get3f(COLOR);
      if (pc->providesAttribute("normal")) m_normal=baseIt->get3f(NORMAL);
   }else{
      pick_col=baseIt->get3f(COLOR);
      if (pc->providesAttribute("normal")) pick_normal=baseIt->get3f(NORMAL);
   }

   delete returnedIt;

   ObjectViewsTable::update(this);


   /*     OLD:
      int32 selectedCloud = x;
   int32 selectedPoint = getNearestPoint(selectedCloud, y, w, h, 20, &scene->getRootRenderState()->cam, &scene->getRootRenderState()->viewFrustum, scene);
   if (selectedPoint==-1){
      if(click) {
         return;
      }else{
         pick_col=makeVector3f(0.5,0.5,0.5);
         pick_normal=makeVector3f(0.0,0.0,0.0);
         return;
      }
   }

   PointCloud *pc = scene->getPointCloud(selectedCloud);
   if (!pc->providesAttribute("color")) return;
   AAT COLOR = pc->getAAT("color");

   if (!pc->providesAttribute("normal")) return;
   AAT NORMAL = pc->getAAT("normal");

   PointCloudIterator *newIt = pc->createIterator(PointCloudIteratorType(0));
   if (newIt == NULL) return;
   BasicPointCloudIterator *baseIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
   if (baseIt == NULL) {delete newIt; return ;}

   // zum entspr Punkt durchlaufen
   for (int32 n=0; n<selectedPoint; n++) {
      baseIt->next();
   }

   if(click){
      m_col=baseIt->get3f(COLOR);
      m_normal=baseIt->get3f(NORMAL);
   }else{
      pick_col=baseIt->get3f(COLOR);
      pick_normal=baseIt->get3f(NORMAL);
   }

   delete newIt;

   ObjectViewsTable::update(this);


   */


}

void PCIDrawingTool::fillSide(){
   CmdObjSpecialDraw *newCmd = new CmdObjSpecialDraw();
   newCmd->setup(1, m_col, m_normal, 0);
   scene->getCommandHistory()->executeCommand(scene, newCmd, false);
}

void PCIDrawingTool::fillSelection(){
   CmdObjSpecialDraw *newCmd = new CmdObjSpecialDraw();
   newCmd->setup(0, m_col, m_normal, 0);
   scene->getCommandHistory()->executeCommand(scene, newCmd, false);
}


void PCIDrawingTool::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {
   forget(buttonsState);
   forget(modifiersState);
   leftMBdown=true;

   card32 i=0;
   SGLegacyPCSceneAdaptor sceneLA(scene);
   for (i=0; i<sceneLA.getNumPointClouds(); i++)
   {
      string pcName = sceneLA.getNameWithPath( i );
      checkAttribute( scene, pcName, "color", 3, VAD::DATA_FORMAT_FLOAT32 );
   };

   if(pick){
      pick=false;
      pickCol(x,y,true);
      return;
   }

   CmdObjDrawOnClick *newCmd = new CmdObjDrawOnClick();
   newCmd->setup(x, y, tol, mode, (scene->getRootState()->staticState->camera), (scene->getRootState()->staticState->viewFrustum), w, h, m_col,fade,
      m_VariableAttributePainting, m_AttributeName, m_Mask[0],m_Mask[1],m_Mask[2],m_Mask[3],m_BrushIntensity, m_AdditiveBlending);
   scene->getCommandHistory()->executeCommand(scene, newCmd, false);

}

void PCIDrawingTool::mouseMoved(int32 x, int32 y) {

   mouseMoveX=x;
   mouseMoveY=y;

   if(pick){
      pickCol(x,y,false);
      return;
   }

   if (leftMBdown){
      CmdObjDrawOnClick *newCmd = new CmdObjDrawOnClick();
      newCmd->setup(x, y, tol, mode, (scene->getRootState()->staticState->camera), (scene->getRootState()->staticState->viewFrustum), w, h, m_col,fade,
      m_VariableAttributePainting, m_AttributeName, m_Mask[0],m_Mask[1],m_Mask[2],m_Mask[3],m_BrushIntensity, m_AdditiveBlending);
      scene->getCommandHistory()->executeCommand(scene, newCmd, true);
   }


}

void PCIDrawingTool::paintOldStyle(card32 x,card32 y){

      card32 sqrdist=0;
      card32 i=0;
      card32 quadTol= tol*tol;

      SGLegacyPCSceneAdaptor sceneLA(scene);
      for (i=0; i<sceneLA.getNumPointClouds(); i++)
      {
         Matrix4f curtransmatr = sceneLA.getState(i)->dynamicState->ctm;
         Matrix4f projmatr = (sceneLA.getState(i)->staticState->viewFrustum).getProjectionMatrix();
         Matrix4f viewmatr = (sceneLA.getState(i)->staticState->camera).getViewMatrix();
         transform = projmatr*viewmatr*curtransmatr;

         PointCloud *pc = sceneLA.getPointCloud(i);
         if (!pc->providesAttribute("position")) return;
         AAT POSITION = pc->getAAT("position");
         AAT COLOR = pc->getAAT("color");

         SceneObjectIterator *newIt = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC ));
         if (newIt == NULL) return;
         BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
         if (basicIt == NULL) {delete newIt; return;}
         ModifyablePointCloudIterator *modyIt = dynamic_cast<ModifyablePointCloudIterator*>(newIt);
         if (modyIt == NULL) {delete newIt; return;}

         card32 w2mx=w2-x;
         card32 h2my=h2-y;

         while (!basicIt->atEnd() )
         {
            Vector4f homPos = transform * (expand3To4(basicIt->get3f(POSITION)));
            float32 wmatrix = homPos[3];

            if (fabs(wmatrix) > 0.001f)
            {
               int32 pointXmx = w2mx + (( homPos[0]/wmatrix) * w2);
               int32 pointYmy = h2my + (( homPos[1]/wmatrix) * h2);

               sqrdist = ((pointXmx)*(pointXmx))+((pointYmy)*(pointYmy));
               float32 fading;

               if (sqrdist<quadTol)
               {
                  if (fade){
                     fading=(float)sqrdist/(float)quadTol;
                     modyIt->set3f(COLOR, blend(basicIt->get3f(COLOR),m_col,fading));

                  }
                  else
                     modyIt->set3f(COLOR, m_col);
               }

            }
           basicIt->next() ;
         }

         delete newIt;

      }; // end for


}

void PCIDrawingTool::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {

   leftMBdown=false;

   forget(x+y);
   forget(buttonsState);
   forget(modifiersState);
}

void PCIDrawingTool::areaResize(card32 width, card32 height) {
   w= width;
   h= height;

   h2=height/2;
   w2=width/2;

   WidthToHeight = ((float)height)/((float)width);
}



void PCIDrawingTool::keyDown(GeneralKey key) {
   forget(key);
}

void PCIDrawingTool::keyUp(GeneralKey key) {
   forget(key);
}


void PCIDrawingTool::connectScene(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget) {
   PCInteractionTool::connectScene(scene, onChange, sceneEditorWidget);

}

void PCIDrawingTool::glDrawTool(GLContext *glContext) {
   forget(glContext);

   if(pick) return;
      glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
         glLoadIdentity();
         glMatrixMode(GL_PROJECTION);
         glPushMatrix();
            glLoadIdentity();
            glPushAttrib(GL_ALL_ATTRIB_BITS);
               glDisable(GL_LIGHTING);
               glLineWidth(3.0f);

               float32 posX =   (mouseMoveX-w*0.5f) / (w*0.5f);
               float32 posY = -((mouseMoveY-h*0.5f) / (h*0.5f));

					glEnable(GL_BLEND);
					glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                  float32 radi = tol / (h*0.5f);

				  
					  //Vector4f colo=makeVector4f();
                  glColor4f(m_col[0], m_col[1], m_col[2], 0.8f);
                  drawCircle(posX,posY,radi);


            glPopAttrib();
         glMatrixMode(GL_PROJECTION);
         glPopMatrix();
      glMatrixMode(GL_MODELVIEW);
      glPopMatrix();

}

void PCIDrawingTool::drawCircle(float32 x1, float32 y1, float32 r) {
   float32 DEG2RAD = 3.141592654f/180.0f;
   glBegin(GL_LINE_LOOP);
      for (int32 i=0; i < 360; i+=10)
      {
         float32 degInRad = i*DEG2RAD;
         glVertex2f(x1+cos(degInRad)*WidthToHeight*r, y1+sin(degInRad)*r);
      }
   glEnd();  
};






