//---------------------------------------------------------------------------
#ifndef PCIDrawingToolH
#define PCIDrawingToolH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PCIDrawingToolEditorWidget;


class EDITOR_API PCIDrawingTool : public PCInteractionTool {
 private:
   static MetaClass *PCIDrawingToolMetaClass;
   static Object* newInstance();

   card32 w2,h2,w,h,mouseMoveX,mouseMoveY;
   card32 tolerance;
   bool ctrlPressed;
   bool leftMBdown;

   float32 WidthToHeight;
   card32 mode;
   Matrix4f transform;
   Vector3f m_col;

   // ------- changes mb: added variable attribute + component-mask ----------------------
   // monochrome painting only ( TODO :-) )
   bool     m_VariableAttributePainting;  /// activates this modus
   string   m_AttributeName;
   bool     m_Mask[4];           /// works like OpenGL's colormask - false = don't modify this component
   float32  m_BrushIntensity;    /// scales brush ( usefull when paintng "position" )
   bool     m_AdditiveBlending;  /// new value = old value + brushIntensity
   // -------------------------------------------------------------------------------------


   friend   PCIDrawingToolEditorWidget;   /// mb

 public:

   card32 tol;
   bool fade;
   bool pick;
   Vector3f pick_col, pick_normal,m_normal;

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign( const Object * obj, CopyContext * context = NULL );

   PCIDrawingTool();
   Vector3f getColor();
   void setColor(Vector3f);
   card32 getMode(){return mode;};
   void setMode(card32 m){mode=m;};

   void pickCol(card32 x,card32 y,bool click);


   void fillSelection();
   void fillSide();

   void paintOldStyle(card32 x,card32 y);
//   void paint(card32 x,card32 y);

   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);

   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   virtual void connectScene(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget);
   virtual void glDrawTool(GLContext *glContext);
   void drawCircle(float32 x1, float32 y1, float32 r);




};


}

#endif
