//---------------------------------------------------------------------------
#include "StdAfx.h"

//---------------------------------------------------------------------------
#include "PCIFilterTool.h"
//---------------------------------------------------------------------------
#include "ObjectClassProperty.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "DebugOutput.h"
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "NumericalClassProperty.h"
#include "PointSet.h"
#include "SceneCommandObjects.h"
#include "CommandHistory.h"
#include "ParametricInCorePointCloud.h"
#include "CmdObjFilter.h"
#include "LinearAlgebra_templCode.h"
//---------------------------------------------------------------------------
#include <algorithm>
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


MetaClass* PCIFilterTool::PCIFilterToolMetaClass = NULL;

const MetaClass* PCIFilterTool::getInstanceClass() const {
   return getClass();
}

Object* PCIFilterTool::newInstance() {
   return new PCIFilterTool;
}

const MetaClass* PCIFilterTool::getClass() {
   if (PCIFilterToolMetaClass == NULL) {
      PCIFilterToolMetaClass = new MetaClass("PCIFilterTool", "no description.",
                                       newInstance, sizeof(PCIFilterTool));
   }
   return PCIFilterToolMetaClass;
}

void PCIFilterTool::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION);
   PCIFilterToolMetaClass->copyParentProperties();

   //PCIFilterToolMetaClass->getGUIDescriptor()->addClassEditor(PCIFilterToolEditor::getClass(), true);
}

void PCIFilterTool::shutdown() {
   delete PCIFilterToolMetaClass;
}

// ----


PCIFilterTool::PCIFilterTool(){
   leftMBdown=false;

   mode=2;
   channel=0;
   size=50;
   strength=0.1f;
   use_kNN=true;
   use_Heightmap=false;
   kNN_numPoints=10;

};

void PCIFilterTool::filterArea(card32 x,card32 y){
   CmdObjFilter *newCmd = new CmdObjFilter();
   newCmd->setup(x, y, size, mode,  channel ,(scene->getRootState()->staticState->camera),
                  (scene->getRootState()->staticState->viewFrustum), w, h, strength, false, use_kNN, kNN_numPoints, use_Heightmap);

   scene->getCommandHistory()->executeCommand(scene, newCmd, true);
}

void PCIFilterTool::filterScene(){
   CmdObjFilter *newCmd = new CmdObjFilter();
   newCmd->setup(0, 0, size, mode,  channel ,(scene->getRootState()->staticState->camera),
                  (scene->getRootState()->staticState->viewFrustum), w, h, strength, true,  use_kNN, kNN_numPoints, use_Heightmap);

   scene->getCommandHistory()->executeCommand(scene, newCmd, false);
}


void PCIFilterTool::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {
   forget(buttonsState);
   forget(modifiersState);
   leftMBdown=true;

   filterArea(x,y);

}

void PCIFilterTool::mouseMoved(int32 x, int32 y) {

   mouseMoveX=x;
   mouseMoveY=y;
   if(leftMBdown)
         filterArea(x,y);

}

void PCIFilterTool::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {

   leftMBdown=false;

   forget(x+y);
   forget(buttonsState);
   forget(modifiersState);
}

void PCIFilterTool::areaResize(card32 width, card32 height) {
   w= width;
   h= height;
   WidthToHeight = ((float)height)/((float)width);
}



void PCIFilterTool::keyDown(GeneralKey key) {
   forget(key);
}

void PCIFilterTool::keyUp(GeneralKey key) {
   forget(key);
}


void PCIFilterTool::connectScene(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget) {
   PCInteractionTool::connectScene(scene, onChange, sceneEditorWidget);
}

void PCIFilterTool::glDrawTool(GLContext *glContext) {
   forget(glContext);

     glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
         glLoadIdentity();
         glMatrixMode(GL_PROJECTION);
         glPushMatrix();
            glLoadIdentity();
            glPushAttrib(GL_ALL_ATTRIB_BITS);
               glDisable(GL_LIGHTING);
               glLineWidth(1.0f);

               float32 posX =   (mouseMoveX-w*0.5f) / (w*0.5f);
               float32 posY = -((mouseMoveY-h*0.5f) / (h*0.5f));


               float32 radi = size / (h*0.5f);
               glColor4f(1.0f,0.9f,0.1f,1.0f);
               drawCircle(posX,posY,radi);

               radi = 2* size / (h*0.5f);
               glColor4f(1.0f,0.3f,0.1f,1.0f);
               drawCircle(posX,posY,radi);


            glPopAttrib();
         glMatrixMode(GL_PROJECTION);
         glPopMatrix();
      glMatrixMode(GL_MODELVIEW);
     glPopMatrix();
}


void PCIFilterTool::drawCircle(float32 x1, float32 y1, float32 r) {
   float32 DEG2RAD = 3.141592654f/180.0f;
   glBegin(GL_LINE_LOOP);
      for (int32 i=0; i < 360; i+=10)
      {
         float32 degInRad = i*DEG2RAD;
         glVertex2f(x1+cos(degInRad)*WidthToHeight*r, y1+sin(degInRad)*r);
      }
   glEnd();  
};
