#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIManualAlignment.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "ClassMethods.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( PCIManualAlignment ,0)
{
	BEGIN_CLASS_INIT( PCIManualAlignment );
	ADD_NOARGS_METHOD_DESC( clear, "clear list of corresponding points and restart" );
	ADD_NOARGS_METHOD_DESC( align, "try to align corresponding points (you need 4 points at least)" );
	ADD_NOARGS_METHOD_DESC( basePlaneAlignment, "pure magic");
}

PCIManualAlignment::PCIManualAlignment(void)
{
	clear();
}

PCIManualAlignment::~PCIManualAlignment(void)
{
}

void PCIManualAlignment::assign(const Object* obj, CopyContext *context)
{
	PCInteractionTool::assign(obj,context);
	const PCIManualAlignment * o = dynamic_cast<const PCIManualAlignment*>(obj);
	pAssert( o != NULL );

	copyObjectProperties(obj,this);
}

void geoxl::PCIManualAlignment::mouseDown( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState )
{
	if( buttonsState.getLeft() || buttonsState.getRight() )
	{
		bool cloud1 = buttonsState.getLeft();
		if( modifiersState.getCtrl() )
		{
			if( cloud1 && m_PointList1.size())
				m_PointList1.pop_back();
			if( !cloud1 && m_PointList2.size())
				m_PointList2.pop_back();
			if( m_PointList1.size() == 0 )
				m_PC1 = NULL;
			if( m_PointList2.size() == 0 )
				m_PC2 = NULL;
			return;
		}

		string pcName;
		PointCloud * pc;
		Vector3f point;
		if( getIntersectionFromCursor(x,y,point,pcName,&pc) )
		{
			if( cloud1 )
			{
				if( m_PC1 == NULL )
					m_PC1 = pc;
				if( m_PC1 == pc )
					m_PointList1.push_back(point);
			}
			else
			{
				if( m_PC2 == NULL && m_PC1 != pc)
					m_PC2 = pc;
				if( m_PC2 == pc )
					m_PointList2.push_back(point);
			}
				
		}


	}
}

void geoxl::PCIManualAlignment::glDrawTool( GLContext *glContext )
{
	// transform points based on point cloud transformation
	vector<Vector3f> points1,points2;
	for( unsigned i=0;i<m_PointList1.size();i++ )
	{
		points1.push_back( transformVector3f(m_PC1->getTransformation(),m_PointList1[i]) );
	}

	for( unsigned i=0;i<m_PointList2.size();i++ )
	{
		points2.push_back( transformVector3f(m_PC2->getTransformation(),m_PointList2[i]) );
	}


	glLineWidth(1.0f);
	glBegin(GL_LINES);
	glColor3f(0.65f,0.65f,0.65f);
	for( unsigned i=0;i<points1.size() && i<points2.size();i++ )
	{
		glVertex3fv( points1[i].data());
		glVertex3fv( points2[i].data());
	}
	glEnd();


	glPointSize(5.0f);
	glBegin(GL_POINTS);
	glColor3f(1,0.5,0);
	for( unsigned i=0;i<points1.size();i++ )
	{
		glVertex3fv( points1[i].data());
	}
	glColor3f(0,0.75,1);
	for( unsigned i=0;i<points2.size();i++ )
	{
		glVertex3fv( points2[i].data());
	}
	glEnd();
}

void geoxl::PCIManualAlignment::clear()
{
	m_PC1 = NULL;
	m_PC2 = NULL;
	m_PointList1.clear();
	m_PointList2.clear();
}

void geoxl::PCIManualAlignment::align()
{
	if( m_PointList1.size() != m_PointList2.size() )
	{
		message("you have to select exactly the same number of points in each point cloud");
		return;
	}

	if( m_PointList1.size() < 4 )
	{
		message("not enough points");
		return;		
	}

	// transform points based on point cloud transformation
	vector<Vector3f> points1,points2;
	for( unsigned i=0;i<m_PointList1.size();i++ )
	{
		points1.push_back( transformVector3f(m_PC1->getTransformation(),m_PointList1[i]) );
		points2.push_back( transformVector3f(m_PC2->getTransformation(),m_PointList2[i]) );
	}
	
	// compute centroids
	Vector3f centroid1 = NULL_VECTOR3F;
	Vector3f centroid2 = NULL_VECTOR3F;
	for( unsigned i=0;i<points1.size();i++ )
	{
		centroid1 += points1[i];
		centroid2 += points2[i];
	}
	centroid1 /= m_PointList1.size();
	centroid2 /= m_PointList2.size();

	// calculate cross-covariance matrix
	Matrix3f C = IDENTITY3F * 0;
	for( card32 i=0;i<points1.size();i++ )
	{
		for( card32 j=0;j<3;j++ )
		{
			for( card32 k=0;k<3;k++ )
			{
				C[j][k] += (points1[i][j] - centroid1[j])
					* (points2[i][k] - centroid2[k]);
			}
		}
	}
	C /= points1.size();

	// Prepare OpenCV Datastructure CvMat
	Matrix3f U,V;
	Vector3f eigenValues;
	CvMat cvMatInput, cvMat_U, cvMat_W, cvMat_Vt;
	cvInitMatHeader( &cvMatInput, 3, 3, CV_32FC1, C.data());
	cvInitMatHeader( &cvMat_W, 3, 1, CV_32FC1, eigenValues.data() );
	cvInitMatHeader( &cvMat_U, 3, 3, CV_32FC1, U.data() );
	cvInitMatHeader( &cvMat_Vt, 3, 3, CV_32FC1, V.data() );

	// Do SVD
	cvSVD(&cvMatInput, &cvMat_W, &cvMat_U, &cvMat_Vt, CV_SVD_MODIFY_A | CV_SVD_U_T );

	Matrix3f rotation = U*V;
	Matrix4f transformation = makeTranslation4f(centroid1) * expand3To4(rotation) * makeTranslation4f(-centroid2);


	m_PC2->setTransformation( transformation * m_PC2->getTransformation() );

}

void geoxl::PCIManualAlignment::basePlaneAlignment()
{
	if( m_PointList1.size() < 3 )
	{
		message("not enough points");
		return;		
	}

	Vector3f normal = normalize((m_PointList1[2] - m_PointList1[0]).crossProduct(m_PointList1[1] - m_PointList1[0]));
	Matrix3f tangentSystem = calcTangentSystem(normal);

	m_PC1->setTransformation(expand3To4(tangentSystem.transpose()));

}