//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCISelectionTool.h"
//---------------------------------------------------------------------------
#include "ObjectClassProperty.h"
#include "Scene.h"
#include "SceneGraphState.h"
#include "CmdObjSelection.h"
#include "CommandHistory.h"
#include "PointCloudTools.h"
#include "SelectionTools.h"
#include "SGLegacyPCSceneAdaptor.h"
#include "ObjectViewsTable.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

MetaClass* PCISelectionTool::PCISelectionToolMetaClass = NULL;

const MetaClass* PCISelectionTool::getInstanceClass() const {
   return getClass();
}

Object* PCISelectionTool::newInstance() {
   return new PCISelectionTool;
}

const MetaClass* PCISelectionTool::getClass() {
   if (PCISelectionToolMetaClass == NULL) {
      PCISelectionToolMetaClass = new MetaClass("PCISelectionTool", "no description.",
                                       newInstance, sizeof(PCISelectionTool));
   }
   return PCISelectionToolMetaClass;
}

void PCISelectionTool::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   PCISelectionToolMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( PCISelectionTool )
   ADD_CARD32_PROP(mode, 0)
   ADD_CARD32_PROP(tolerance, 0)
   ADD_CARD32_PROP(brushsize, 0)
   ADD_BOOLEAN_PROP(ctrlPressed, 0)
   ADD_BOOLEAN_PROP(SurfaceMode, 0)
   ADD_VECTOR3F_PROP(SelColor, 0)
}

void PCISelectionTool::shutdown() {
   delete PCISelectionToolMetaClass;
}

// --------------------------------------------------------------------

PCISelectionTool::PCISelectionTool() {
   tolerance = 12;
   brushsize = 12;

   mode = SELMODE_SINGLECLOUD;

   ctrlPressed = false;
   leftMBdown = false;
   rightMBdown = false;
   SurfaceMode = true;

   leftMBdownX = 0;
   leftMBdownY = 0;
   leftMBmoveY = 0;
   leftMBmoveX = 0;

   SelColor = makeVector3f(0.0f,0.0f,0.0f);

   w = 100;
   h = 100;
   WidthToHeight = 1.0f;

}


void PCISelectionTool::mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {

   if (buttonsState.getLeft()) leftMBdown=true;
   if (buttonsState.getRight()) rightMBdown=true;

   leftMBdownX = x;
   leftMBdownY = y;
   leftMBmoveX = x;
   leftMBmoveY = y;

   if (leftMBdown && (mode!=SELMODE_AREARECTANGLE) && (mode!=SELMODE_AREACIRCLE)) {   // nicht bei Area-Select
      CmdObjChangeSelectionOnClick *newCmd = new CmdObjChangeSelectionOnClick();
      newCmd->setup(x, y, ctrlPressed, brushsize, mode, (scene->getRootState()->staticState->camera), (scene->getRootState()->staticState->viewFrustum), w, h, SurfaceMode);
      scene->getCommandHistory()->executeCommand(scene, newCmd, true);
   };

   forget(buttonsState);
   forget(modifiersState);
}

void PCISelectionTool::mouseMoved(int32 x, int32 y) {

   leftMBmoveX = x;
   leftMBmoveY = y;

   if (leftMBdown){

      if ((mode!=SELMODE_SINGLECLOUD)&&(mode!=SELMODE_AREARECTANGLE)&&(mode!=SELMODE_AREACIRCLE)) {   // nicht bei Cloud- oder Area-Select
         CmdObjChangeSelectionOnClick *newCmd = new CmdObjChangeSelectionOnClick();
         newCmd->setup(x, y, ctrlPressed, brushsize, mode, (scene->getRootState()->staticState->camera), (scene->getRootState()->staticState->viewFrustum), w, h, SurfaceMode);
         scene->getCommandHistory()->executeCommand(scene, newCmd, true);
      }
   };

}

void PCISelectionTool::mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {

   if ((mode==SELMODE_AREARECTANGLE)||(mode==SELMODE_AREACIRCLE)) {   // nur bei Area-Select
      CmdObjSelectionOnArea *newCmd = new CmdObjSelectionOnArea();
      newCmd->setup(leftMBdownX, leftMBdownY, x, y, ctrlPressed, brushsize, mode, (scene->getRootState()->staticState->camera), (scene->getRootState()->staticState->viewFrustum), w, h);
      scene->getCommandHistory()->executeCommand(scene, newCmd, true);
   };

   leftMBdown = false;
   rightMBdown = false;
   leftMBdownX = 0;
   leftMBdownY = 0;
   leftMBmoveX = x;
   leftMBmoveY = y;

   forget(buttonsState);
   forget(modifiersState);
}

void PCISelectionTool::areaResize(card32 width, card32 height) {
   h = height;
   w = width;
   if (w==0) { WidthToHeight = 0.0f; return; };
   WidthToHeight = ((float)height)/((float)width);
}



void PCISelectionTool::keyDown(GeneralKey key) {
   if (key.getKey() == keyControl) ctrlPressed = true;
   ObjectViewsTable::update(this);
}

void PCISelectionTool::keyUp(GeneralKey key) {
   if (key.getKey() == keyControl) ctrlPressed = false;
   ObjectViewsTable::update(this);
}


void PCISelectionTool::invertSelection(){
   CmdObjSelectionByButton *newCmd = new CmdObjSelectionByButton();
   newCmd->setup(BUTTON_INVERTPOINTS);
   scene->getCommandHistory()->executeCommand(scene, newCmd, true);
   redrawScene();
}

void PCISelectionTool::selectAllClouds(){
   CmdObjSelectionByButton *newCmd = new CmdObjSelectionByButton();
   newCmd->setup(BUTTON_ALLCLOUDS);
   scene->getCommandHistory()->executeCommand(scene, newCmd, true);
   redrawScene();
}

void PCISelectionTool::selectAllPoints(){
   CmdObjSelectionByButton *newCmd = new CmdObjSelectionByButton();
   newCmd->setup(BUTTON_ALLPOINTS);
   scene->getCommandHistory()->executeCommand(scene, newCmd, true);
   redrawScene();
}

void PCISelectionTool::clearCloudSelection(){
   CmdObjSelectionByButton *newCmd = new CmdObjSelectionByButton();
   newCmd->setup(BUTTON_NOCLOUDS);
   scene->getCommandHistory()->executeCommand(scene, newCmd, true);
   redrawScene();
}

void PCISelectionTool::clearPointSelection(){
   CmdObjSelectionByButton *newCmd = new CmdObjSelectionByButton();
   newCmd->setup(BUTTON_NOPOINTS);
   scene->getCommandHistory()->executeCommand(scene, newCmd, true);
   redrawScene();
}


void PCISelectionTool::selectByColor(){
   Vector4f avalues = expand3To4(SelColor);
   avalues[3] = 0;
   CmdObjSelectionByAttribute *newCmd = new CmdObjSelectionByAttribute();
   newCmd->setup("color",avalues,tolerance,ctrlPressed);
   scene->getCommandHistory()->executeCommand(scene, newCmd, true);
   redrawScene();
}


void PCISelectionTool::connectScene(Scene *scene, OnSceneChangeCallback *onChange, QWidget *sceneEditorWidget) {
   PCInteractionTool::connectScene(scene, onChange, sceneEditorWidget);
}


void PCISelectionTool::glDrawTool(GLContext *glContext) {
   forget(glContext);

   if ((mode==SELMODE_BRUSHCIRCLE) || (mode==SELMODE_AREARECTANGLE) || (mode==SELMODE_BRUSHQUAD) || (mode==SELMODE_AREACIRCLE))
   {
      glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
         glLoadIdentity();
         glMatrixMode(GL_PROJECTION);
         glPushMatrix();
            glLoadIdentity();
            glPushAttrib(GL_ALL_ATTRIB_BITS);
               glDisable(GL_LIGHTING);
               glLineWidth(1.0f);

               float32 posX =   (leftMBmoveX-w*0.5f) / (w*0.5f);
               float32 posY = -((leftMBmoveY-h*0.5f) / (h*0.5f));

               if (mode==SELMODE_BRUSHCIRCLE) {
                  float32 radi = brushsize / (h*0.5f);
                  glColor4f(1.0f,0.9f,0.1f,1.0f);
                  drawCircle(posX,posY,radi);
               };

               if (mode==SELMODE_BRUSHQUAD) {
                  float32 delta = brushsize / (h*0.5f);
                  glColor4f(1.0f,0.9f,0.1f,1.0f);
                  drawRectangle(posX-delta*WidthToHeight, posY-delta, posX+delta*WidthToHeight, posY+delta);
               };

               if ((mode==SELMODE_AREARECTANGLE) && leftMBdown) {
                  float32 posX2 =  (leftMBdownX-w*0.5f) / (w*0.5f);
                  float32 posY2 =-((leftMBdownY-h*0.5f) / (h*0.5f));
                  glEnable(GL_BLEND);
                  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                  glColor4f(1.0f,0.9f,0.1f,0.4f);
                  drawFilledRectangle(posX, posY, posX2, posY2);
                  glColor4f(1.0f,0.9f,0.0f,0.9f);
                  drawRectangle(posX, posY, posX2, posY2);
               };

               if ((mode==SELMODE_AREACIRCLE) && leftMBdown) {
                  float32 centX =  ((float32)leftMBdownX + leftMBmoveX - w) / w;
                  float32 centY = -((float32)leftMBdownY + leftMBmoveY - h) / h;

                  float32 dX = ((float32)leftMBmoveX-leftMBdownX)/(w*WidthToHeight);
                  float32 dY = ((float32)leftMBmoveY-leftMBdownY)/h;
                  float32 radi = sqrt(dX*dX+dY*dY);

                  glEnable(GL_BLEND);
                  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                  glColor4f(1.0f,0.9f,0.1f,0.4f);
                  drawFilledCircle(centX, centY, radi);
                  glColor4f(1.0f,0.9f,0.0f,0.9f);
                  drawCircle(centX,centY,radi);
               };

            glPopAttrib();
         glMatrixMode(GL_PROJECTION);
         glPopMatrix();
      glMatrixMode(GL_MODELVIEW);
      glPopMatrix();
   };
}


void PCISelectionTool::drawRectangle(float32 x1, float32 y1, float32 x2, float32 y2) {
   glBegin(GL_LINE_LOOP);
     glVertex2f(x2, y2);
     glVertex2f(x2, y1);
     glVertex2f(x1, y1);
     glVertex2f(x1, y2);
   glEnd();
};

void PCISelectionTool::drawFilledRectangle(float32 x1, float32 y1, float32 x2, float32 y2) {
   glBegin(GL_QUADS);
      glVertex2f(x2, y2);
      glVertex2f(x2, y1);
      glVertex2f(x1, y1);
      glVertex2f(x1, y2);
   glEnd();
};

void PCISelectionTool::drawCircle(float32 x1, float32 y1, float32 r) {
   float32 DEG2RAD = 3.141592654f/180.0f;
   glBegin(GL_LINE_LOOP);
      for (int32 i=0; i < 360; i+=10)
      {
         float32 degInRad = i*DEG2RAD;
         glVertex2f(x1+cos(degInRad)*WidthToHeight*r, y1+sin(degInRad)*r);
      }
   glEnd();
};

void PCISelectionTool::drawFilledCircle(float32 x1, float32 y1, float32 r) {
   float32 DEG2RAD = 3.141592654f/180.0f;
   glBegin(GL_POLYGON);
      for (int32 i=0; i < 360; i+=10)
      {
         float32 degInRad = i*DEG2RAD;
         glVertex2f(x1+cos(degInRad)*WidthToHeight*r, y1+sin(degInRad)*r);
      }
   glEnd();
};


