//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "InitFoundation.h"
//---------------------------------------------------------------------------

/*
	Please separate between system code and interface code ( editors ), in 
	order to build a non-interface commandline version of geoxl.

File structure:
	includes
		..
		..
	interface includes
		..
		..
	initFoundation()
	{
		init geoxl classes
		..
		..
		init geoxl class editors
		..
		..
	}
	shutdownFoundation()
	{
		shutdown geoxl classes
		..
		..
		shutdown geoxl class editors
		..
		..
				
	}
*/
//---------------------------------------------------------------------------
#include "Camera.h"
#include "ViewFrustum.h"
#include "CameraController.h"
#include "CameraPathRecorder.h"
#include "ClassCreator.h"
#include "ClassEditor.h"
#include "CubeMap.h"
#include "ExaminerCameraController.h"
#include "FreeCameraController.h"
#include "PanoramicViewFrustum.h"
#include "Persistent.h"
#include "PropertyBrowser.h"
#include "PropertyEditor.h"
#include "RGBAImage.h"
#include "ManualImageFeatures.h"
#include "StandardTimeController.h"
#include "WalkthroughCameraController.h"
#include "ScriptEngine.h"
#include "StreamClassTable.h"
#include "MultiObjectStream.h"
#include "BlockedMultiObjectStream.h"
#include "MultiFileMultiObjectStream.h"
#include "MultiFileMultiObjectStreamMT.h"
#include "EventDisplayer.h"
#include "EventCounterDisplay.h"
#include "ObjectDictionary.h"
#include "ProgressWindow.h"
#include "CommandObject.h"
#include "CommandHistory.h"
#include "CommandObjMacro.h"
#include "LegacyClassNameTranslation.h"
#include "FilterMask.h"
#include "AttachedData.h"
#include "FileDialogs.h" // StandardDirectoriesConfig
#include "UserSettings.h"
#include "MemoryMultiObjectStream.h"
#include "CmdObjMemberMethod.h"
#include "IterativeSolvers.h"
#include "UnitTests.h"
#include "Plane.h"
#include "NonLinearIterativeSolvers.h"
#include "SimpleAttachment.h"
#include "ScalarDerivative.h"
#include "PartialDerivative.h"
// ---------------------------------------------------------------------------------
//					Interface Includes
// ---------------------------------------------------------------------------------

#ifdef GEOXL_GUI

#include "BooleanPropertyEditor.h"
#include "ObjectPropertyEditor.h"
#include "FixedArrayPropertyEditor.h"

//#include "AttachedDataContainerEditor.h"
#include "ObjectDictionaryEditorWidget.h"
#include "RGBAImageEditorWidget.h"
#include "CameraPathRecorderEditorWidget.h"
#include "RGBACubeMapEditorWidget.h"
#include "ManualImageFeaturesEditorWidget.h"
#include "ScriptEngineEditorWidget.h"
#include "CommandHistoryEditorWidget.h"
#include "CommandObjMacroEditorWidget.h"
#include "ClassTypeClassProperty.h"
#include "ClassTypeClassPropertyEditorWidget.h"
#include "AttachedDataContainerEditorWidget.h"
#include "UserSettingsEditorWidget.h"
#include "TableOutputTools.h"
#include "foundation_resources_rcc.h"

#endif	// end of #ifdef GEOXL_GUI

//---------------------------------------------------------------------------

using namespace geoxl;


void geoxl::initFoundation()
{

// unit tests

	UnitTests::addTest(iterativeSolvers_unitTest, "iterativeSolvers");
	UnitTests::addTest(NonLinearIterativeSolvers_unitTest, "NonLinearIterativeSolvers");
   UnitTests::addTest(ScalarDerivative_unitTest, "ScalarDerivative_unitTest");
   UnitTests::addTest(PartialDerivative_unitTest, "PartialDerivative_unitTest");

// ---------------------------------------------------------------------------------
//					Class Registration ( no Editors ! )
// ---------------------------------------------------------------------------------

	// -----------------
	// system base
	Object::init(NULL);
      Persistent::init(Object::getClass());
      ClassEditor::init(Object::getClass());
      ClassCreator::init(Object::getClass());
      PropertyEditor::init(Object::getClass());

			// -----------------
			// Persistent derivates
         StreamClassTable::init(Persistent::getClass());
         Camera::init(Persistent::getClass());
         ViewFrustum::init(Persistent::getClass());
		 PanoramicViewFrustum::init(ViewFrustum::getClass());

         CameraController::init(Persistent::getClass());
            ExaminerCameraController::init(CameraController::getClass());
            CameraPathRecorder::init(CameraController::getClass());
            WalkthroughCameraController::init(CameraController::getClass());
            FreeCameraController::init( CameraController::getClass() );

         RGBAImage::init(Persistent::getClass());
         RGBImage::init(Persistent::getClass());
         OneByteImage::init(Persistent::getClass());
         RGBAFloat32Image::init(Persistent::getClass() );
         RGBACubeMap::init(Persistent::getClass());
		ManualImageFeatures::init(Persistent::getClass());


         StandardTimeController::init(Persistent::getClass());
         ScriptEngine::init(Persistent::getClass());

         MultiObjectStream::init(Persistent::getClass());
				FileBasedMultiObjectStream::init(MultiObjectStream::getClass());
	            BlockedMultiObjectStream::init(FileBasedMultiObjectStream::getClass());
		         MultiFileMultiObjectStream::init(FileBasedMultiObjectStream::getClass());
			      MultiFileMultiObjectStreamMT::init(FileBasedMultiObjectStream::getClass());
            MemoryMultiObjectStream::init(MultiObjectStream::getClass());
			
			BlockedMultiObjectStreamCreator::init(ClassCreator::getClass());
			MultiFileMultiObjectStreamCreator::init(ClassCreator::getClass());
			MultiFileMultiObjectStreamMTCreator::init(ClassCreator::getClass());
			
			MultiFileMultiObjectStreamIndex::init(Persistent::getClass());
			
         ObjectDictionary::init(Persistent::getClass());

         CommandObject::init(Persistent::getClass());
				CmdObjMemberMethod::init(CommandObject::getClass());
         CommandHistory::init(Persistent::getClass());
         CommandObjMacro::init(Persistent::getClass());

         FilterMask::init( Persistent::getClass() );
            FixedFilterMask_9x9::init( FilterMask::getClass() );
            FixedFilterMask_21x21::init( FilterMask::getClass() );

         AttachedData::init(Persistent::getClass());
            StringAttachment::init(AttachedData::getClass());
				StandardDirectoriesConfig::init(AttachedData::getClass());
				UserSettings::init(AttachedData::getClass());
				SimpleAttachment::init(AttachedData::getClass());
         AttachedDataContainer::init(Persistent::getClass());
		 Plane::init( Persistent::getClass() );


// ---------------------------------------------------------------------------------
//					Editor Class Registration 
// ---------------------------------------------------------------------------------
#ifdef GEOXL_GUI
			// ------------------------
			// property editors
         BooleanPropertyEditor::init(PropertyEditor::getClass());
         ObjectPropertyEditor::init(PropertyEditor::getClass());
			FixedArrayPropertyEditor::init(PropertyEditor::getClass());

			// ------------------------
			// custom editors
			PropertyBrowser::init(ClassEditor::getClass());
			ObjectDictionaryEditor::init(ClassEditor::getClass());
				((GUIDescriptor * )ObjectDictionary::getClass()->getGUIDescriptor())->addClassEditor(
				ObjectDictionaryEditor::getClass(), true);
			RGBAImageEditor::init(ClassEditor::getClass());
				((GUIDescriptor * )RGBAImage::getClass()->getGUIDescriptor())->addClassEditor(
				RGBAImageEditor::getClass(), true);
			CameraPathRecorderEditor::init(ClassEditor::getClass());
				((GUIDescriptor * )CameraPathRecorder::getClass()->getGUIDescriptor())->addClassEditor(
				CameraPathRecorderEditor::getClass(), true);
			RGBACubeMapEditor::init(ClassEditor::getClass());
				((GUIDescriptor * )RGBACubeMap::getClass()->getGUIDescriptor())->addClassEditor(
				RGBACubeMapEditor::getClass(), true);

				ManualImageFeaturesEditor::init(ClassEditor::getClass());
	((GUIDescriptor * )ManualImageFeatures::getClass()->getGUIDescriptor())->addClassEditor(
		ManualImageFeaturesEditor::getClass(), true);

         ScriptEngineEditor::init(ClassEditor::getClass());
				((GUIDescriptor * )ScriptEngine::getClass()->getGUIDescriptor())->addClassEditor(
				ScriptEngineEditor::getClass(), true);
				((GUIDescriptor * )ScriptEngine::getClass()->getGUIDescriptor())->setAutoApplyEditChanges(false);
         CommandHistoryEditor::init(ClassEditor::getClass());
				((GUIDescriptor * )CommandHistory::getClass()->getGUIDescriptor())->addClassEditor(
				CommandHistoryEditor::getClass(), true);
         CommandObjMacroEditor::init(ClassEditor::getClass());
				((GUIDescriptor * )CommandObjMacro::getClass()->getGUIDescriptor())->addClassEditor(
				CommandObjMacroEditor::getClass(), true);
		  ClassTypeClassPropertyEditor::init(ClassEditor::getClass());
				((GUIDescriptor * )ClassTypeClassProperty::getClass()->getGUIDescriptor())->addClassEditor(
				ClassTypeClassPropertyEditor::getClass(), true);
		  AttachedDataContainerEditor::init(ClassEditor::getClass());
				((GUIDescriptor * )AttachedDataContainer::getClass()->getGUIDescriptor())->addClassEditor(
				AttachedDataContainerEditor::getClass(), true);
		  //UserSettingsEditor::init(ClassEditor::getClass()); // mw - removed to allow for easier changes (add properties)
			//	((GUIDescriptor * )UserSettings::getClass()->getGUIDescriptor())->addClassEditor(
			//	UserSettingsEditor::getClass(), true);

      EventDisplayer::init(Object::getClass());
	      EventCounterDisplay::init(EventDisplayer::getClass());



      //ScriptEngineDebug::init(Object::getClass());

	  ProgressWindow::init(Object::getClass());

   qInitResources();


   /// for web browser component (exception otherwise)
//   CoInitialize(NULL);
//   initOnlineHelpForm();
 
   

#endif	// end of #ifdef GEOXL_GUI

	LegacyClassNameTranslation::registerTranslation("PCCommandObject", CommandObject::getClass());
   LegacyClassNameTranslation::registerTranslation("PCCommandObjectScript", CommandHistory::getClass());

}

void geoxl::shutdownFoundation()
{
// ---------------------------------------------------------------------------------
//					Class Shutdown
// ---------------------------------------------------------------------------------
   Object::shutdown();
   ClassEditor::shutdown();
   Persistent::shutdown();
   StreamClassTable::shutdown();
   Camera::shutdown();
   ViewFrustum::shutdown();
   PanoramicViewFrustum::shutdown();
   CameraController::shutdown();
   ExaminerCameraController::shutdown();
   FreeCameraController::shutdown();
   CameraPathRecorder::shutdown();
   WalkthroughCameraController::shutdown();
   RGBAImage::shutdown();
   RGBImage::shutdown();
   OneByteImage::shutdown();
   RGBAFloat32Image::shutdown();
   RGBACubeMap::shutdown();
   ManualImageFeatures::shutdown();
   StandardTimeController::shutdown();
   ScriptEngine::shutdown();
   MultiObjectStream::shutdown();
   FileBasedMultiObjectStream::shutdown();
   BlockedMultiObjectStream::shutdown();
	MultiFileMultiObjectStream::shutdown();
	MultiFileMultiObjectStreamMT::shutdown();
   MemoryMultiObjectStream::shutdown();
	BlockedMultiObjectStreamCreator::shutdown();
	MultiFileMultiObjectStreamCreator::shutdown();
	MultiFileMultiObjectStreamMTCreator::shutdown();
	MultiFileMultiObjectStreamIndex::shutdown();
   ObjectDictionary::shutdown();
   CommandObject::shutdown();
   CmdObjMemberMethod::shutdown();
   CommandHistory::shutdown();
   CommandObjMacro::shutdown();
   FilterMask::shutdown();
   FixedFilterMask_9x9::shutdown();
   FixedFilterMask_21x21::shutdown();
   AttachedData::shutdown();
   StringAttachment::shutdown();
	StandardDirectoriesConfig::shutdown();
	UserSettings::shutdown();
	SimpleAttachment::shutdown();
   AttachedDataContainer::shutdown();
   Plane::shutdown();


// ---------------------------------------------------------------------------------
//					Editor Class Shutdown
// ---------------------------------------------------------------------------------
#ifdef GEOXL_GUI

	// ------------------------
	// property editors
	BooleanPropertyEditor::shutdown();
   ObjectPropertyEditor::shutdown();
	FixedArrayPropertyEditor::shutdown();

	// ------------------------
	// class editors
	PropertyBrowser::shutdown();
	ObjectDictionaryEditor::shutdown();
	RGBAImageEditor::shutdown();
   CameraPathRecorderEditor::shutdown();
   RGBACubeMapEditor::shutdown();
   ManualImageFeaturesEditor::shutdown();
   ScriptEngineEditor::shutdown();
   CommandHistoryEditor::shutdown();
   CommandObjMacroEditor::shutdown();
	ClassTypeClassPropertyEditor::shutdown();
	AttachedDataContainerEditor::shutdown();
	//UserSettingsEditor::shutdown();

   EventCounterDisplay::shutdown();
   EventDisplayer::shutdown();

   ProgressWindow::shutdown();
	delete progressWindow;

   //ScriptEngineDebug::shutdown();


   qCleanupResources();

#endif

   delete tableOutputTool;
   tableOutputTool = NULL;
}
