#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "AttachedDataContainerEditorWidget.h"
//---------------------------------------------------------------------------
#include "AttachedData.h"
//---------------------------------------------------------------------------
#include "ObjectViewsTable.h"
#include "ClassSelectionDIalog.h"
#include <qtGUI/qInputDialog>
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( AttachedDataContainer )

AttachedDataContainerEditorWidget::AttachedDataContainerEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	m_SubObjectWidget = NULL;

	connect( ui.treeWidget, SIGNAL(itemDoubleClicked( QTreeWidgetItem *, int )),SLOT(sltItemDoubleClicked( QTreeWidgetItem *, int )));

	m_ClassSelectionDialog = new ClassSelectionDialog(this);
	m_ClassSelectionDialog->init( AttachedData::getClass() );
}

AttachedDataContainerEditorWidget::~AttachedDataContainerEditorWidget(void)
{
}

void AttachedDataContainerEditorWidget::setup( AttachedDataContainer * obj )
{
	m_Object = obj;
}

void AttachedDataContainerEditorWidget::updateEditor()
{
	ui.treeWidget->clear();
	vector<AttachedData*> all;
	m_Object->getAll(all);
	for( card32 i=0;i<all.size();i++ )
	{
		QTreeWidgetItem * item = new QTreeWidgetItem( ui.treeWidget );
		item->setText(0, all[i]->getName().c_str() );
		const MetaClass * mc = all[i]->getInstanceClass();
		item->setText(1, mc->getClassName().c_str() );
	}
}

void AttachedDataContainerEditorWidget::on_pbEdit_clicked()
{
	if( ui.treeWidget->currentItem() != NULL )
	{
		QTreeWidgetItem * item = ui.treeWidget->currentItem();
		AttachedData * data = m_Object->getData( item->text(0).toLatin1().constData() );
		if( data != NULL )
			openEditor( data, data->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor() );
	}
}

void AttachedDataContainerEditorWidget::on_pbAdd_clicked()
{
	if( m_ClassSelectionDialog->exec() == QDialog::Accepted )
	{
		MetaClass * mc = m_ClassSelectionDialog->getSelectedClass();
		AttachedData * data = (AttachedData *)mc->newInstance();

		QString name = QInputDialog::getText( this, "input", "please enter a valid attachment-name" );
		if( name.isEmpty() )
			delete data;
		else
		{
			data->setup( name.toLatin1().constData(), AttachedData::ADF_PERSISTENT );
			m_Object->attachData( data );
			updateEditor();
		}
	}
}

void AttachedDataContainerEditorWidget::on_pbDelete_clicked()
{
	if( ui.treeWidget->currentItem() != NULL )
	{
		QTreeWidgetItem * item = ui.treeWidget->currentItem();
		m_Object->removeData( item->text(0).toLatin1().constData() );
		updateEditor();
	}
}

void AttachedDataContainerEditorWidget::on_pbUpdate_clicked()
{
	updateEditor();
}


void AttachedDataContainerEditorWidget::openEditor( Object * obj, const MetaClass * mc )
{
	delete m_SubObjectWidget;
	ClassEditor * ce = (ClassEditor*)mc->newInstance();
	QWidget * w = ce->makeEditor( this, obj );
	w->setWindowFlags( Qt::Window );
	w->setAttribute( Qt::WA_DeleteOnClose );
	connect( w, SIGNAL( destroyed( QObject* ) ), SLOT( sltEditorDestroyed( QObject* ) ) );
	w->move( QCursor::pos() );
	w->show();
	m_SubObjectWidget = w;
	ObjectViewsTable::bindView( obj, ce );

}

void AttachedDataContainerEditorWidget::sltEditorDestroyed(QObject *obj)
{
	m_SubObjectWidget = NULL;
}

void AttachedDataContainerEditorWidget::sltItemDoubleClicked ( QTreeWidgetItem * item, int column )
{
	on_pbEdit_clicked();
}



#include "AttachedDataContainerEditorWidget_moc.cpp"