#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "CommandHistoryEditorWidget.h"
//---------------------------------------------------------------------------
#include "CommandHistory.h"
#include "CommandObject.h"
#include "CommandObjMacro.h"
#include "BinaryObjectStreams.h"
#include "ObjectViewsTable.h"
#include <QtGui/QGroupBox.h>
#include <QtGui/QInputDialog.h>
#include <QtGui/QFileDialog.h>
#include <QtGui/QMessageBox.h>
#include "StringHelper.h"
#include "FileDialogs.h"
//---------------------------------------------------------------------------
using namespace geoxl;


// ******************** Import Dialog ********************

CommandHistoryImportDialog::CommandHistoryImportDialog(QWidget *parent)
	: QDialog(parent)
{
	setWindowTitle("CommandHistoryImportDialog");

	// make radiobuttons
	radioButton1 = new QRadioButton(tr("Add To Undo List"));
	radioButton1->setChecked(true);
	radioButton2 = new QRadioButton(tr("Add And Execute Immediately"));
	radioButton3 = new QRadioButton(tr("Create New Scene"));
	QVBoxLayout *radiobuttonLayout = new QVBoxLayout;
	radiobuttonLayout->addWidget(radioButton1);
	radiobuttonLayout->addWidget(radioButton2);
	radiobuttonLayout->addWidget(radioButton3);
	QGroupBox *radioGroup = new QGroupBox("Options");
	radioGroup->setLayout(radiobuttonLayout);

	// make buttons
	QPushButton *okButton = new QPushButton(tr("&OK"));
	QPushButton *cancelButton = new QPushButton(tr("&Cancel"));
	connect(okButton, SIGNAL(clicked()), this, SLOT(sltOkButtonClicked()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
	QHBoxLayout *buttonLayout = new QHBoxLayout;
	buttonLayout->addStretch(1);
	buttonLayout->addWidget(okButton);
	buttonLayout->addWidget(cancelButton);

	mainLayout = new QVBoxLayout;
	mainLayout->addWidget(radioGroup);
	mainLayout->addLayout(buttonLayout);
	setLayout(mainLayout);

	// TODO: delete items at end???
};


void CommandHistoryImportDialog::sltOkButtonClicked() {
	if (radioButton1->isChecked()) {
		done(1); // be aware that 1 is also QDialog::Accepted
	} else if (radioButton2->isChecked()) {
		done(2);
	} else if (radioButton3->isChecked()) {
		done(3);
	} else {
		reject();
	}
};



// ******************** CommandHistoryEditor ********************

IMPLEMENT_CLASS_EDITOR( CommandHistory )

CommandHistoryEditorWidget::CommandHistoryEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	// call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	// Example: easy access to QtDesigner buttons, etc. by ui object
	QWidget::connect(ui.BTLoad, SIGNAL(clicked()), this, SLOT(sltBTLoadClicked()));
	QWidget::connect(ui.BTSave, SIGNAL(clicked()), this, SLOT(sltBTSaveClicked()));
	QWidget::connect(ui.CBSaveWithScene, SIGNAL(stateChanged(int)), this, SLOT(sltCBSaveWithSceneStateChanged(int)));
	QWidget::connect(ui.BTMakeMacro, SIGNAL(clicked()), this, SLOT(sltBTMakeMacroClicked()));
	QWidget::connect(ui.LVDo, SIGNAL(clicked(QModelIndex)), this, SLOT(sltLVDoChanged(QModelIndex)));
	QWidget::connect(ui.LVUndo, SIGNAL(clicked(QModelIndex)), this, SLOT(sltLVUndoChanged(QModelIndex)));
	QWidget::connect(ui.CBShowLineNumber, SIGNAL(stateChanged(int)), this, SLOT(sltCBShowLineNumberChanged(int)));

	// set header for command lists
	ui.LVDo->setColumnCount(4);
	ui.LVDo->setHeaderLabels(QStringList() << "line" << "class" << "description" << "");
	ui.LVDo->resizeColumnToContents(0);
	ui.LVDo->hideColumn(0);
	ui.LVUndo->setColumnCount(4);
	ui.LVUndo->setHeaderLabels(QStringList() << "line" << "class" << "description" << "");
	ui.LVUndo->resizeColumnToContents(0);
	ui.LVUndo->hideColumn(0);

	// make layout to add command object property editors
	QVBoxLayout *layout1 = new QVBoxLayout;
	layout1->setMargin(0);
	layout1->setSpacing(0);
	ui.TabDo->setLayout(layout1);
	QVBoxLayout *layout2 = new QVBoxLayout;
	layout2->setMargin(0);
	layout2->setSpacing(0);
	ui.TabUndo->setLayout(layout2);

	// button icons
	ui.BTLoad->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/OpenFolder.png").c_str()));
	ui.BTSave->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/SaveDisk.png").c_str()));
	ui.BTMakeMacro->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/small_class.bmp").c_str()));

	// init the rest
   doEditor = NULL;
   undoEditor = NULL;
   doEditorWidget = NULL;
   undoEditorWidget = NULL;
   context = NULL;
   coiContext = NULL;
}

CommandHistoryEditorWidget::~CommandHistoryEditorWidget(void)
{
   delete doEditor;
   delete undoEditor;
}

void CommandHistoryEditorWidget::setup( CommandHistory * obj )
{
	m_CmdHist = obj;
}

void CommandHistoryEditorWidget::updateEditor()
{
	updateSelectedItemEditor(doEditor, doEditorWidget, ui.TabDo, NULL);
	updateSelectedItemEditor(undoEditor, undoEditorWidget, ui.TabUndo, NULL);
	fillList(m_CmdHist->doList, ui.LVDo);
	fillList(m_CmdHist->undoList, ui.LVUndo);

	CommandObject *doObj = NULL;
	if (ui.LVDo->currentIndex().isValid()) {
		doObj = m_CmdHist->doList[ui.LVDo->currentIndex().row()];
	}
	updateSelectedItemEditor(doEditor, doEditorWidget, ui.TabDo, doObj);

   CommandObject *undoObj = NULL;
   if (ui.LVUndo->currentIndex().isValid()) {
      undoObj = m_CmdHist->undoList[ui.LVUndo->currentIndex().row()];
   }
	updateSelectedItemEditor(undoEditor, undoEditorWidget, ui.TabUndo, undoObj);

	ui.CBSaveWithScene->setChecked(m_CmdHist->writeCommands);
}


// ---------- user functions ----------

void CommandHistoryEditorWidget::setContext(Object *cont) {
   CommandObjectContextInterface* test = dynamic_cast<CommandObjectContextInterface*>(cont);
   if (test != NULL) {
      context = cont;
      coiContext = test;
		ui.BTLoad->setEnabled(true);
		ui.BTMakeMacro->setEnabled(true);
   } else {
      context = NULL;
      coiContext = NULL;
		ui.BTLoad->setEnabled(false);
		ui.BTMakeMacro->setEnabled(false);
   }
}


void CommandHistoryEditorWidget::fillList(const vector<CommandObject*> &cmds, QTreeWidget *treeview) {
	int32 ind = treeview->currentIndex().row();
	treeview->clear();
	for (card32 i = 0; i < cmds.size(); ++i) {
		QStringList s;
		s << intToStr(i).c_str();
		s << QString(cmds[i]->getInstanceClass()->getClassName().c_str());
		s << QString(cmds[i]->getDescription().c_str());
		QTreeWidgetItem *li = new QTreeWidgetItem(treeview, s);
	}
	QList<QTreeWidgetItem *> foundItems = treeview->findItems(QString(intToStr(ind).c_str()), Qt::MatchExactly|Qt::MatchWrap, 0);
	if ( !foundItems.empty() ) {
		treeview->setCurrentItem(foundItems.first());
	};
}


void CommandHistoryEditorWidget::updateSelectedItemEditor(ClassEditor *&selEditor, QWidget *&selEditorWidget, QWidget* tab, CommandObject *obj) {
	if (selEditor != NULL) {
		tab->layout()->removeWidget(selEditorWidget);
      delete selEditorWidget;
      delete selEditor;
      selEditor = NULL;
      selEditorWidget = NULL;
   }
   if (obj == NULL) return;
   const MetaClass *mc = obj->getInstanceClass();
   selEditor = (ClassEditor*)(mc->getGUIDescriptor()->getDefaultClassEditor()->newInstance());
   selEditorWidget = selEditor->makeEditor(tab, obj);
	tab->layout()->addWidget(selEditorWidget);
   selEditorWidget->show();
   selEditor->update();
}



// ---------- slots ----------

void CommandHistoryEditorWidget::sltBTLoadClicked() {
	if (context == NULL) return;

	string fname = FileDialogs::getOpenFileName(this, "Choose a file", "Command Object Scripts (*.cmds.object);;All Files (*.*)");
	if (fname != "") {

		BinaryInputObjectStream in(fname.c_str());

		Persistent* obj;
		try {
			in.readObject(obj);
         if (obj->getInstanceClass() != CommandObjMacro::getClass() &&
				obj->getInstanceClass() != CommandObject::getClass()) {
				QMessageBox::warning(this, "Wrong File Type...", "Imported object is not of type CommandObjMacro or CommandObject.");
            Object *del = obj;
            obj = NULL;
            delete del;
         } // end if

         // Script list
         if (obj->getInstanceClass() == CommandObjMacro::getClass()) {
            CommandObjMacro* newScript = (CommandObjMacro*)obj;

				// ask user how to import
				CommandHistoryImportDialog *importDialog = new CommandHistoryImportDialog(this);
				int importResult = importDialog->exec();
            delete importDialog;

				if (importResult==1) {
					// Add to undo list
					for (int32 i = newScript->numCommands()-1; i >= 0; --i) {
						m_CmdHist->undoList.push_back((CommandObject*)newScript->getCommand(i)->copy());
					}
					fillList(m_CmdHist->undoList, ui.LVUndo);
				}

				if (importResult==3) {
					// Create new scene
					coiContext->clear();
					m_CmdHist = coiContext->getCommandHistory();
				}

				if ((importResult==2) || (importResult==3)) {
					// Execute immediately
					for (card32 i = 0; i < newScript->numCommands(); ++i) {
						CommandObject* command = (CommandObject*)newScript->getCommand(i)->copy();
						m_CmdHist->doList.push_back(command);
						command->doAction(context);
					}
					fillList(m_CmdHist->doList, ui.LVDo);
				}
				ObjectViewsTable::update(context);
			} // end if script list


         // Script program
         if (obj->getInstanceClass() == CommandObject::getClass()) {
            CommandObject *cmdObj = (CommandObject*)obj;

				// ask user how to import
				CommandHistoryImportDialog *importDialog = new CommandHistoryImportDialog(this);
				int importResult = importDialog->exec();
            delete importDialog;

				if (importResult==1) {
					// Add to undo list
					m_CmdHist->undoList.push_back((CommandObject*)cmdObj->copy());
					fillList(m_CmdHist->undoList, ui.LVUndo);
				}

				if (importResult==3) {
					// Create new scene
					coiContext->clear();
					m_CmdHist = coiContext->getCommandHistory();
				}

				if ((importResult==2) || (importResult==3)) {
					// Execute immediately
					CommandObject* command = (CommandObject*)cmdObj->copy();
					m_CmdHist->doList.push_back(command);
					command->doAction(context);
					fillList(m_CmdHist->doList, ui.LVDo);
				}
				ObjectViewsTable::update(context);

			} // end if script program

		} catch(...) {
			delete obj;
			throw;
		}

		delete obj;

	};	// end if !fname.isEmpty()
}


void CommandHistoryEditorWidget::sltBTSaveClicked() {
	string fname = FileDialogs::getSaveFileName(this, "Save File...", "Command Object Scripts (*.cmds.object);;All Files (*.*)");
	if(fname != "") {
		BinaryOutputObjectStream out(fname.c_str());
      //out.writeObject(history);
      CommandObjMacro macro;
		for (unsigned i = 0; i != m_CmdHist->doList.size(); ++i) {
			macro.addCommand(m_CmdHist->doList[i]);
		}
      out.writeObject(&macro);
   }
}


void CommandHistoryEditorWidget::sltCBSaveWithSceneStateChanged(int state) {
	m_CmdHist->writeCommands = ui.CBSaveWithScene->isChecked();
}


void CommandHistoryEditorWidget::sltBTMakeMacroClicked() {
   if (context == NULL) return;
   CommandObjMacro* scriptList = new CommandObjMacro();
	for (unsigned i = 0; i != m_CmdHist->doList.size(); ++i) {
		scriptList->addCommand(m_CmdHist->doList[i]);
	};
	bool ok;
   QString name = QInputDialog::getText(this, tr("Macro Name"), tr("Please enter the macro name (empty = choose name automatically):"), QLineEdit::Normal, "", &ok);
	coiContext->addMacro(scriptList, name.toLatin1().constData());
}


void CommandHistoryEditorWidget::sltLVDoChanged(QModelIndex index) {
	CommandObject *doObj = NULL;
	if (index.isValid()) {
		doObj = m_CmdHist->doList[index.row()];
	};
	updateSelectedItemEditor(doEditor, doEditorWidget, ui.TabDo, doObj);
}


void CommandHistoryEditorWidget::sltLVUndoChanged(QModelIndex index) {
   CommandObject *undoObj = NULL;
	if (index.isValid()) {
		undoObj = m_CmdHist->undoList[index.row()];
	};
   updateSelectedItemEditor(undoEditor, undoEditorWidget, ui.TabUndo, undoObj);
}


void CommandHistoryEditorWidget::sltCBShowLineNumberChanged(int state) {
	if (ui.CBShowLineNumber->isChecked()) {
		ui.LVDo->showColumn(0);
		ui.LVUndo->showColumn(0);
	} else {
		ui.LVDo->hideColumn(0);
		ui.LVUndo->hideColumn(0);
	};
}


#include "CommandHistoryEditorWidget_moc.cpp"


