#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "CommandObjMacroEditorWidget.h"
//---------------------------------------------------------------------------
#include "CommandObjMacro.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------

using namespace geoxl;


IMPLEMENT_CLASS_EDITOR( CommandObjMacro )


CommandObjMacroEditorWidget::CommandObjMacroEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	//Example: easy access to QtDesigner buttons, etc. by ui object
	QWidget::connect(ui.BTDuplicate ,SIGNAL(clicked()),this,SLOT(sltBTDuplicateClicked()));
	QWidget::connect(ui.BTDelete ,SIGNAL(clicked()),this,SLOT(sltBTDeleteClicked()));
	QWidget::connect(ui.BTBottom ,SIGNAL(clicked()),this,SLOT(sltBTBottomClicked()));
	QWidget::connect(ui.BTDown ,SIGNAL(clicked()),this,SLOT(sltBTDownClicked()));
	QWidget::connect(ui.BTUp ,SIGNAL(clicked()),this,SLOT(sltBTUpClicked()));
	QWidget::connect(ui.BTTop ,SIGNAL(clicked()),this,SLOT(sltBTTopClicked()));
	QWidget::connect(ui.LVCommands, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), this, SLOT(sltLVCommandsChanged(QTreeWidgetItem*,QTreeWidgetItem*)));
	QWidget::connect(ui.CBShowLineNumber, SIGNAL(stateChanged(int)), this, SLOT(sltCBShowLineNumberChanged(int)));

	// set header for command lists
	ui.LVCommands->setColumnCount(4);
	ui.LVCommands->setHeaderLabels(QStringList() << "line" << "class" << "description" << "");
	ui.LVCommands->resizeColumnToContents(0);
	ui.LVCommands->hideColumn(0);

	// button icons
	ui.BTBottom->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/down_bottom_arrow.bmp").c_str()));
	ui.BTDown->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/down_arrow.bmp").c_str()));
	ui.BTUp->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/up_arrow.bmp").c_str()));
	ui.BTTop->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/up_top_arrow.bmp").c_str()));

	// make layout to add command object property editors
	QVBoxLayout *layout = new QVBoxLayout;
	layout->setMargin(0);
	layout->setSpacing(0);
	ui.PropertyFrame->setLayout(layout);

	// init the rest
	commandEditor = NULL;
	commandEditorWidget = NULL;
}

CommandObjMacroEditorWidget::~CommandObjMacroEditorWidget(void)
{
}

void CommandObjMacroEditorWidget::setup( CommandObjMacro * obj )
{
	m_CmdObjMacro = obj;
}

void CommandObjMacroEditorWidget::updateEditor()
{
	int32 ind = ui.LVCommands->currentIndex().row();
	ui.LVCommands->clear();
	for (card32 i = 0; i < m_CmdObjMacro->commands.size(); ++i) {
		QStringList s;
		s << intToStr(i).c_str();
		s << QString(m_CmdObjMacro->commands[i]->getInstanceClass()->getClassName().c_str());
		s << QString(m_CmdObjMacro->commands[i]->getDescription().c_str());
		QTreeWidgetItem *li = new QTreeWidgetItem(ui.LVCommands, s);
	}
	
   QTreeWidgetItem* item = ui.LVCommands->topLevelItem(ind);
   ui.LVCommands->setCurrentItem(item);
}


// ---------- user functions ----------

void CommandObjMacroEditorWidget::updateSelectedItemEditor(CommandObject *cmdObj)
{
	if (commandEditor != NULL) {
		ui.PropertyFrame->layout()->removeWidget(commandEditorWidget);
		delete commandEditorWidget;
		delete commandEditor;
		commandEditor = NULL;
		commandEditorWidget = NULL;
	}
	if (cmdObj == NULL) return;

	const MetaClass *mc = cmdObj->getInstanceClass();
	commandEditor = (ClassEditor*)(mc->getGUIDescriptor()->getDefaultClassEditor()->newInstance());
	commandEditorWidget = commandEditor->makeEditor(ui.PropertyFrame, cmdObj);
	ui.PropertyFrame->layout()->addWidget(commandEditorWidget);
	commandEditorWidget->show();
	commandEditor->update();
}


// ---------- slots ----------

void CommandObjMacroEditorWidget::sltBTDuplicateClicked() {
	CommandObject *cmdObj = NULL;
	if (ui.LVCommands->currentIndex().isValid()) {
		cmdObj = m_CmdObjMacro->commands[ui.LVCommands->currentIndex().row()];
	   m_CmdObjMacro->addCommand(cmdObj);
		updateEditor();
	}
}

void CommandObjMacroEditorWidget::sltBTDeleteClicked() {
	CommandObject *cmdObj = NULL;
	int32 currentRow = ui.LVCommands->currentIndex().row();
	if (ui.LVCommands->currentIndex().isValid()) {
		cmdObj = m_CmdObjMacro->commands[currentRow];
		delete cmdObj;
		m_CmdObjMacro->commands.erase(m_CmdObjMacro->commands.begin() + currentRow);

      updateEditor();
	}
}

void CommandObjMacroEditorWidget::sltBTBottomClicked() {
	CommandObject *cmdObj = NULL;
	int32 currentRow = ui.LVCommands->currentIndex().row();
	if (ui.LVCommands->currentIndex().isValid() && (currentRow < ((int32)m_CmdObjMacro->commands.size()-1))) {
		cmdObj = m_CmdObjMacro->commands[currentRow];
		m_CmdObjMacro->commands.erase(m_CmdObjMacro->commands.begin() + currentRow);
		m_CmdObjMacro->commands.push_back(cmdObj);
		
		updateEditor();
      QTreeWidgetItem* item = ui.LVCommands->topLevelItem(ui.LVCommands->topLevelItemCount()-1);
      ui.LVCommands->setCurrentItem(item);
	}
}

void CommandObjMacroEditorWidget::sltBTDownClicked() {
	int32 currentRow = ui.LVCommands->currentIndex().row();
	if (ui.LVCommands->currentIndex().isValid() && (currentRow < ((int32)m_CmdObjMacro->commands.size()-1))) {
		swap(m_CmdObjMacro->commands[currentRow], m_CmdObjMacro->commands[currentRow + 1]);

		updateEditor();
      QTreeWidgetItem* item = ui.LVCommands->topLevelItem(currentRow + 1);
      ui.LVCommands->setCurrentItem(item);
	}
}

void CommandObjMacroEditorWidget::sltBTUpClicked() {
	int32 currentRow = ui.LVCommands->currentIndex().row();
	if (ui.LVCommands->currentIndex().isValid() && (currentRow >= 1)) {
		swap(m_CmdObjMacro->commands[currentRow], m_CmdObjMacro->commands[currentRow - 1]);

		updateEditor();
      QTreeWidgetItem* item = ui.LVCommands->topLevelItem(currentRow - 1);
      ui.LVCommands->setCurrentItem(item);
	}
}

void CommandObjMacroEditorWidget::sltBTTopClicked() {
	CommandObject *cmdObj = NULL;
	int32 currentRow = ui.LVCommands->currentIndex().row();
	if (ui.LVCommands->currentIndex().isValid() && (currentRow >= 0)) {
		cmdObj = m_CmdObjMacro->commands[currentRow];
		m_CmdObjMacro->commands.erase(m_CmdObjMacro->commands.begin() + currentRow);
		m_CmdObjMacro->commands.insert(m_CmdObjMacro->commands.begin(), cmdObj);

		updateEditor();
      QTreeWidgetItem* item = ui.LVCommands->topLevelItem(0);
      ui.LVCommands->setCurrentItem(item);
	}
}

void CommandObjMacroEditorWidget::sltLVCommandsChanged(QTreeWidgetItem* currentItem, QTreeWidgetItem* previousItem) {
	CommandObject *cmdObj = NULL;
	int index = ui.LVCommands->indexOfTopLevelItem(currentItem);
	if (index != -1) {
		cmdObj = m_CmdObjMacro->commands[index];
		updateSelectedItemEditor(cmdObj);
	};
}

void CommandObjMacroEditorWidget::sltCBShowLineNumberChanged(int state) {
	if (ui.CBShowLineNumber->isChecked()) {
		ui.LVCommands->showColumn(0);
	} else {
		ui.LVCommands->hideColumn(0);
	};
}


#include "CommandObjMacroEditorWidget_moc.cpp"