//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "DebugOutput.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra_templCode.h"
#include "DynamicLinearAlgebra_templCode.h"
#include "UserSettings.h"
#include "Configuration.h"
#include "SparseLinearAlgebra_templCode.h"
//---------------------------------------------------------------------------
#include "DebugWindows.h"

using namespace geoxl;
using namespace std;

DebugOutput geoxl::debugOutput;

// TODO: let user chose between debugWindow or cout or maybe write in file

DebugOutput::DebugOutput()
{
	debugMutex = new QMutex(QMutex::Recursive);
}
DebugOutput::~DebugOutput()
{
	delete debugMutex;
}

DebugOutput& DebugOutput::operator<<(bool b) {
	debugMutex->lock();
   (*debugWindow) << b;
    debugMutex->unlock();
   return *this;
}

DebugOutput& DebugOutput::operator<<(int i)  {
	debugMutex->lock();
   (*debugWindow) << i;
    debugMutex->unlock();
   return *this;
}

DebugOutput& DebugOutput::operator<<(unsigned int i)  {
	debugMutex->lock();
   (*debugWindow) << i;
    debugMutex->unlock();
   return *this;
}

DebugOutput& DebugOutput::operator<<(geoxl::int64 i)  {
	debugMutex->lock();
   (*debugWindow) << i;
    debugMutex->unlock();
   return *this;
}

DebugOutput& DebugOutput::operator<<(card64 i)  {
	debugMutex->lock();
   (*debugWindow) << i;
    debugMutex->unlock();
   return *this;
}

DebugOutput& DebugOutput::operator<<(long double d)  {
	debugMutex->lock();
   (*debugWindow) << d;
    debugMutex->unlock();
   return *this;
}


DebugOutput& DebugOutput::operator<<(float f)  {
	debugMutex->lock();
   (*debugWindow) << (long double)f;
    debugMutex->unlock();
   return *this;
}

DebugOutput& DebugOutput::operator<<(double d)  {
	debugMutex->lock();
   (*debugWindow) << (long double)d;
    debugMutex->unlock();
   return *this;
}


DebugOutput& DebugOutput::operator<<(string s)
{
	debugMutex->lock();
/*   string tmp;
   string newline;
   for( int i=0;i<(int)s.length();i++ ) {
      if (s[i] == '\n') {
         if (!newline.empty()) {
            (*debugWindow) << newline;
         }
			(*debugWindow).cr();
			newline.clear();
      } else {
         newline.push_back(s[i]);
      }
   }
   if (!newline.empty()) {
      (*debugWindow) << newline;
   }*/
	(*debugWindow) << s;
	 debugMutex->unlock();
   return *this;
}

// mb 3.5.05
DebugOutput& DebugOutput::operator<<(char* s)
{
	debugMutex->lock();
   string tmpS = s;
   *this << tmpS;
    debugMutex->unlock();
   return *this;
}

DebugOutput& DebugOutput::operator<<(const Vector2f &v)  {
	debugMutex->lock();
	(*this) << "(" << (long double)v[0] << " / " << (long double)v[1] << ")";
	 debugMutex->unlock();
	return *this;
}

DebugOutput& DebugOutput::operator<<(const Vector3f &v)  {
	debugMutex->lock();
	(*this) << "(" << (long double)v[0] << " / " << (long double)v[1] << " / " << (long double)v[2] << ")";
	debugMutex->unlock();
	return *this;
}

DebugOutput& DebugOutput::operator<<(const Vector3i &v)  {
	debugMutex->lock();
	(*this) << "(" << int32(v[0]) << " / " << (int32)v[1] << " / " << (int32)v[2] << ")";
	debugMutex->unlock();
	return *this;
}

DebugOutput& DebugOutput::operator<<(const Vector4f &v)  {
	debugMutex->lock();
	(*this) << "(" << (long double)v[0] << " / " << (long double)v[1] << " / " << (long double)v[2] << " / " << (long double)v[3] << ")";
	debugMutex->unlock();
   return *this;
}

DebugOutput& DebugOutput::operator<<(const Vector6f &v)  {
	debugMutex->lock();
	(*this) << "(" << (long double)v[0] << " / " << (long double)v[1] << " / " << (long double)v[2] << " / " << (long double)v[3] << " / " << (long double)v[4] <<" / " << (long double)v[5] <<")";
	debugMutex->unlock();
   return *this;
}

DebugOutput& DebugOutput::operator<<(const Matrix4f &m)  {
	debugMutex->lock();
   (*this) << "         " << m[0][0] << "  " << m[0][1] << "  " << m[0][2]<< "  " << m[0][3];
   (*this).cr();
   (*this) << "         " << m[1][0] << "  " << m[1][1] << "  " << m[1][2]<< "  " << m[1][3];
   (*this).cr();
   (*this) << "         " << m[2][0] << "  " << m[2][1] << "  " << m[2][2]<< "  " << m[2][3];
   (*this).cr();
   (*this) << "         " << m[3][0] << "  " << m[3][1] << "  " << m[3][2]<< "  " << m[3][3];
    debugMutex->unlock();
   return *this;
}

DebugOutput& DebugOutput::operator<<(const Matrix3f &m)  {
	debugMutex->lock();
   (*this) << " " << m[0][0] << "  " << m[0][1] << "  " << m[0][2];
   (*this).cr();
   (*this) << " " << m[1][0] << "  " << m[1][1] << "  " << m[1][2];
   (*this).cr();
   (*this) << " " << m[2][0] << "  " << m[2][1] << "  " << m[2][2];
    debugMutex->unlock();
   return *this;
}


DebugOutput& DebugOutput::operator<<(const CvMat &m) {
	debugMutex->lock();	
	for (int32 i=0;i<m.rows;i++ )
	{
		(*this).cr();
		for (int32 j=0;j<m.cols;j++)
		{
			if( j )
				(*this) << ",   ";
			(*this) << cvmGet(&m,i,j);
		}
	}
	(*this).cr();
	debugMutex->unlock();
	return *this;
}


DebugOutput& DebugOutput::operator<<(const BoundingBox3f &bbox )  {
	debugMutex->lock();
	(*this) << "(" << bbox.lowerCorner[0] << "," << bbox.lowerCorner[1] << "," << bbox.lowerCorner[2] << ")/";
	(*this) << "(" << bbox.upperCorner[0] << "," << bbox.upperCorner[1] << "," << bbox.upperCorner[2] << ")";
	debugMutex->unlock();
	return *this;
}

DebugOutput& DebugOutput::operator<<(const StaticVector<float32,7> &v)
{
	debugMutex->lock();
	(*this) << "(" << v[0] << "," << v[1] << "," << v[2] << "," << v[3] << "," << v[4] << "," << v[5] << "," << v[6] << ")";
	debugMutex->unlock();
	return *this;
}


DebugOutput& DebugOutput::operator<<(const DynamicVector<float> &v)
{
	debugMutex->lock();
	(*this) << "(";
	for( unsigned i=0;i<v.size();i++ )
	{
		if( i )
			(*this) << ",";
		(*this) << v[i];
	}
	(*this) << ")";
	debugMutex->unlock();
	return *this;	
}

DebugOutput& DebugOutput::operator<<(const DynamicMatrix<float> &m)
{
	debugMutex->lock();
   for ( unsigned int r = 0; r < m.getRows(); r++ ){
      for ( unsigned int c = 0; c < m.getColumns(); c++ ){
         (*this) << m[c][r] << " ";
      }
      (*this).cr();
   }
    debugMutex->unlock();
   return *this;
}

DebugOutput& DebugOutput::operator<<(SparseMatrixF &A)
{
	debugMutex->lock();
	float32 value;
	card32 index;
	for ( unsigned int r = 0; r < A.getRows(); r++ ){
		SparseVectorF& row = A[r];
		for ( unsigned int i = 0; i < row.getNumSparseEntries(); i++ ){
			row.getSparseEntry( i, index, value );
			(*this) << index << ": " << value << ", ";
		}
		(*this).cr();
	}
	debugMutex->unlock();
	return *this;
}


void DebugOutput::writeIdented(unsigned ident, string s) {
	debugMutex->lock();
   string identsp;
   for (unsigned i = 0; i<ident; i++) identsp += " ";
   (*debugWindow) << (identsp + s);
	debugMutex->unlock();
}

void DebugOutput::cr()  {
	debugMutex->lock();
   (*debugWindow).cr();
	debugMutex->unlock();
}

void DebugOutput::flush()
{
	debugMutex->lock();
	debugWindow->flush();
	debugMutex->unlock();
}


void DebugOutput::displayEvent(const MetaClass *eventType /* of type EventDisplayer */, string name, const EventDisplayerEvent *event) {
	debugMutex->lock();
	bool doDisplay = true;
	UserSettings * u = dynamic_cast<UserSettings*>( Configuration->getData( "USER_SETTINGS" ) );
	if( u )
	{
		doDisplay = u->getDisplayDebugEvents();
	}
	if (doDisplay) {
		(*debugWindow).displayEvent(eventType, name, event);
	}
	debugMutex->unlock();
}


void geoxl::warning(string msg) {	
   debugWindow->warningMessage(msg);
}
   

void geoxl::error(string msg) {
   debugWindow->errorMessage(msg);
}

void geoxl::message(string msg) {
   debugOutput << msg;
   debugWindow->show();
}