//---------------------------------------------------------------------------
#ifndef DebugOutputH
#define DebugOutputH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <string>
#include <externalIncludes\opencv\cxcore.h>
#include "LinearAlgebra.h"
#include "DynamicLinearAlgebra.h"
#include "BoundingBox.h"
#include "SparseLinearAlgebra.h"
#include <QtCore/QMutex>
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;

class EventDisplayerEvent;
class MetaClass;

class FOUNDATION_API DebugOutput { 
private:
 QMutex * debugMutex;

 public:	  
   DebugOutput();
   ~DebugOutput();
   DebugOutput& operator<<(bool b);
   DebugOutput& operator<<(int i);
   DebugOutput& operator<<(unsigned int i);
   // this is necessary for 64-bit builds
   DebugOutput& operator<<(int64 i);
   DebugOutput& operator<<(card64 i);
   DebugOutput& operator<<(long double d);
   DebugOutput& operator<<(float f);
   DebugOutput& operator<<(double d);
   DebugOutput& operator<<(string s);
   DebugOutput& operator<<(char* s);
   DebugOutput& operator<<(const Vector2f &v);
   DebugOutput& operator<<(const Vector3f &v);
   DebugOutput& operator<<(const Vector4f &v);
   DebugOutput& operator<<(const Vector6f &v);
   DebugOutput& operator<<(const Vector3i &v);
   DebugOutput& operator<<(const Matrix4f &m);
   DebugOutput& operator<<(const Matrix3f &m);
   DebugOutput& operator<<(const BoundingBox3f &m);
   DebugOutput& operator<<(const StaticVector<float32,7> &v);
   DebugOutput& operator<<(const DynamicVector<float> &m);
   DebugOutput& operator<<(const DynamicMatrix<float> &m);
   DebugOutput& operator<<(const CvMat &m);
   DebugOutput& operator<<(SparseMatrixF &A);

   void writeIdented(unsigned ident, string s);
   void cr();
	void flush();

   void displayEvent(const MetaClass *eventType /* of type EventDisplayer */, string name, const EventDisplayerEvent *event);
};

extern FOUNDATION_API DebugOutput debugOutput;

/// printout warning message on debug console, pop-up window if non-visible.
void FOUNDATION_API error(string msg);

/// printout error message on debug console, pop-up window if non-visible.
void FOUNDATION_API warning(string msg);

/// printout message on debug console, pop-up window if non-visible.
void FOUNDATION_API message(string msg);

}

#endif
