//---------------------------------------------------------------------------
#include "stdafx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "DebugWindows.h"
//---------------------------------------------------------------------------
#include "Tools.h"
#include "StringHelper.h"
#include "PAssert.h"
#include "EventViewerWidget.h"
//---------------------------------------------------------------------------
#include <fstream>
#include <Qt/QTimer.h>
#include <qtCore/qCoreApplication>
#include <sstream>
#include <iostream>
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------
DebugWindow *geoxl::debugWindow = NULL;

DebugWindow::DebugWindow(QWidget *parent)
:QWidget(parent)
{
	stdColor = QColor(0,0,0);				 // not used, initialized only just-in-case
	stdBackground = QColor(192,192,192); // not used, initialized only just-in-case
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	//easy access to QtDesigner buttons, etc. by ui object
	QWidget::connect(ui.BTExec, SIGNAL(clicked()), this, SLOT(sltBTExecClicked()));

	QWidget::connect(ui.CBLogConsole, SIGNAL(stateChanged(int)), this, SLOT(sltCBLogConsoleStateChanged(int)));
	QWidget::connect(ui.CBLogEvents, SIGNAL(stateChanged(int)), this, SLOT(sltCBLogEventsStateChanged(int)));

	timer = new QTimer(this);
	timer->setInterval(500);
	timer->setSingleShot(true);
	timer->stop();
	eventViewer = new EventViewerWidget(ui.tabEvents, 0, false);
	((QVBoxLayout*)(ui.tabEvents->layout()))->insertWidget(0,eventViewer);
	connect(timer, SIGNAL(timeout()), this, SLOT(textTimer()));

   QDateTime currentTime = QDateTime::currentDateTime();
   string now = qString2STLString(currentTime.toString());
   (*this) << "*** GEOXL started - ";
   (*this) << "System Time: ";
   (*this) << now;
   (*this) << "\n\n";
   
   ui.splitter->setStretchFactor(0,1);
   ui.splitter->setStretchFactor(1,0);

	startTimer(500);
}

DebugWindow::~DebugWindow() {
   try {
		// std::string LogFileName necessary cause of wayward QString
		std::string LogFileName = ui.LEConsoleLogFilename->text().toAscii();
		ofstream out((getConfigurationString("BIN_PATH")+"/"+LogFileName).c_str(), ios_base::app);
      QDateTime currentTime = QDateTime::currentDateTime();
      string now = qString2STLString(currentTime.toString());
      out << "\n*** GEOXL closed -";
      out << "System Time: " << now << "\n\n";
   } catch (...) {
   }
}


// ------- user functions -------

void DebugWindow::operator<<(bool b) {
   if (b) (*this) << "true"; else (*this) << "false";
}

void DebugWindow::operator<<(int i) {
   (*this) << intToStr(i);
}

void DebugWindow::operator<<(unsigned int i) {
   (*this) << intToStr((int)i);
}

void DebugWindow::operator<<(geoxl::int64 i) {
   (*this) << intToStr(i);
}

void DebugWindow::operator<<(card64 i) {
   (*this) << intToStr((int64)i);
}

void DebugWindow::operator<<(long double d) {
   std::string s;
   ostringstream os;
   os << d;
   s = os.str();
   (*this) << s;
}

void DebugWindow::operator<<(std::string s) {

	textBuffer += s;
	if( textBuffer.size() > 2000 )
		flushText();
}

static int errNum = 1;

static const QColor SIGNAL_COLOR(255,255,255);
static const QColor SIGNAL_BACKGROUND(30,40,128);


void DebugWindow::warningMessage(string message) {
	ui.textEditWarnings->setFontWeight(QFont::Bold);
	ui.textEditWarnings->setTextColor(QColor(128,128,0));
	QString str;
	str += QString("(");
	ui.textEditWarnings->insertPlainText(str);
	ui.textEditWarnings->setTextColor(QColor(128,128,0));
	str = QString::number(errNum++);
	str += QString(") ");
	ui.textEditWarnings->insertPlainText(str);
	ui.textEditWarnings->setTextColor(QColor(30,40,128));
	str = QString("warning: ");
	ui.textEditWarnings->insertPlainText(str);
	ui.textEditWarnings->setFontWeight(QFont::Normal);
	ui.textEditWarnings->setTextColor(QColor(0,0,0));
	str = message.c_str();
	str += "\n";
	ui.textEditWarnings->insertPlainText(str);
	ui.tabWidget->setCurrentIndex(3);
	show();
	raise();
	QPalette pal = ui.labelWarnings->palette();
	stdColor = pal.windowText().color();
	stdBackground = pal.background().color();
	pal.setColor(QPalette::Background, SIGNAL_BACKGROUND);
	pal.setColor(QPalette::WindowText, SIGNAL_COLOR);
	ui.labelWarnings->setPalette(pal);
	timer->start();
}

void DebugWindow::errorMessage(string message) {
	ui.textEditErrors->setFontWeight(QFont::Bold);
	ui.textEditErrors->setTextColor(QColor(128,0,0));
	QString str;
	str += QString("(");
	ui.textEditErrors->insertPlainText(str);
	ui.textEditErrors->setTextColor(QColor(128,0,0));
	str = QString::number(errNum++);
	str += QString(") ");
	ui.textEditErrors->insertPlainText(str);
	ui.textEditErrors->setTextColor(QColor(30,40,128));
	str = QString("error: ");
	ui.textEditErrors->insertPlainText(str);
	ui.textEditErrors->setFontWeight(QFont::Normal);
	ui.textEditErrors->setTextColor(QColor(0,0,0));
	str = message.c_str();
	str += "\n";
	ui.textEditErrors->insertPlainText(str);
	ui.tabWidget->setCurrentIndex(3);
	show();
	raise();
	QPalette pal = ui.labelErrors->palette();
	stdColor = pal.windowText().color();
	stdBackground = pal.background().color();
	pal.setColor(QPalette::Background, SIGNAL_BACKGROUND);
	pal.setColor(QPalette::WindowText, SIGNAL_COLOR);
	ui.labelErrors->setPalette(pal);
	timer->start();
}

void DebugWindow::textTimer() {
	timer->stop();
	QPalette pal = ui.labelErrors->palette();
	pal.setColor(QPalette::Background, stdBackground);
	pal.setColor(QPalette::WindowText, stdColor);
	ui.labelWarnings->setPalette(pal);
	ui.labelErrors->setPalette(pal);
}



void DebugWindow::operator<<(char* s) {

	if( s != NULL)
		textBuffer += s;
	if( textBuffer.size() > 2000 )
		flushText();
}

void DebugWindow::cr() {

	textBuffer += "\n";
	if (ui.CBLogConsole->isChecked()) {
		// output to file
      try {
			// std::string LogFileName necessary cause of wayward QString
			std::string LogFileName = ui.LEConsoleLogFilename->text().toAscii();
			ofstream out((getConfigurationString("BIN_PATH")+"/"+LogFileName).c_str(), ios_base::app);
         out << "\n";
      } catch (...) {
      }
	}
}


void DebugWindow::displayEvent(const MetaClass *eventType, string name, const EventDisplayerEvent *event) {
   string log;
   const char *logPtr = NULL;
	if (ui.CBLogEvents->isChecked()) {
		log = qString2STLString(ui.LEEventsLogFilename->text());
		logPtr = log.c_str();
   }
	eventViewer->displayEvent(eventType, name, event, logPtr);
}

void DebugWindow::flush()
{
	flushText();
	ui.TEConsoleOutput->repaint();
	ui.TEEventsOutput->repaint();
	QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
}


// ------- slots -------

void DebugWindow::sltBTExecClicked() {
   (*this) << "> ";
   // std::string text necessary cause of wayward QString
	std::string text = ui.TEConsoleInput->toPlainText().toAscii();
	(*this) << text;
   cr();
	ui.TEConsoleInput->clear();
}

void DebugWindow::sltCBLogConsoleStateChanged(int state) {
	ui.LEConsoleLogFilename->setEnabled(state==Qt::Checked);
}

void DebugWindow::sltCBLogEventsStateChanged(int state) {
	ui.LEEventsLogFilename->setEnabled(state==Qt::Checked);
}

static bool processingFlushEvent = false;

void geoxl::DebugWindow::timerEvent( QTimerEvent * e )
{
	if( !processingFlushEvent )
	{
		processingFlushEvent = true;
		flushText();
		processingFlushEvent = false;
	}
}

void geoxl::DebugWindow::flushText()
{
	if( textBuffer.size() )
	{
		ui.TEConsoleOutput->moveCursor( QTextCursor::End );
		QString qs = textBuffer.c_str();
		qs.replace(QRegExp("a[\n]"), "\r\n");
		ui.TEConsoleOutput->insertPlainText(qs);

		if (ui.tabWidget->currentIndex()==1) {
			ui.TEEventsOutput->insertPlainText(qs);
   		ui.TEEventsOutput->moveCursor(QTextCursor::End, QTextCursor::MoveAnchor);
		}


		if (ui.CBLogConsole->isChecked()) {
			// output to file
			try {
				// std::string LogFileName necessary cause of wayward QString
				std::string LogFileName = ui.LEConsoleLogFilename->text().toAscii();
				ofstream out((getConfigurationString("BIN_PATH")+"/"+LogFileName).c_str(), ios_base::app);
				out << textBuffer;
			} catch (...) {
			}
		}

		textBuffer = "";
		ui.TEConsoleOutput->moveCursor(QTextCursor::End, QTextCursor::MoveAnchor);

	}
}

void geoxl::DebugWindow::showPage(card32 page) {
   if (page >= 4) {
      page = 4;
      //QList<int> sizes = ui.splitter->sizes();
      //sizes[1] += sizes[0];
      //sizes[0] = 0;
      //ui.splitter->setSizes(sizes);
   }
   ui.tabWidget->setCurrentIndex(page);

}

#include "DebugWindows_moc.cpp"
