//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "EventViewerWidget.h"																				  
//---------------------------------------------------------------------------
#include "EventDisplayer.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
#include <QtGui/QVBoxLayout>
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


EventViewerWidget::EventViewerWidget(QWidget *parent, Qt::WFlags f, bool standAlone) 
	: QWidget(parent, f)
{
	ui.setupUi(this);
	QVBoxLayout *l = new QVBoxLayout();
	l->setMargin(0);
	l->setSpacing(0);
	ui.evFrame->setLayout(l);
	if (standAlone) {
		l->addStretch(1);
	}
}

void EventViewerWidget::displayEvent(
	const MetaClass *eventType, 
	string name, 
	const EventDisplayerEvent *event,
	const char *logFile) 
{
	size_t result = -1;
   for (int32 i=0; i<(int32)eventViewers.size(); i++) {
      if (eventViewers[i].name == name) {
         result = i;
         break;
      }
   }

   if (result == -1) {
      pAssert(eventType != NULL);
		EventDisplayer *evDisp = dynamic_cast<EventDisplayer*>(eventType->newInstance());
      pAssert(evDisp != NULL);
      EventEntry e;
      e.name = name;
      e.evDisp = evDisp;
		e.evWidget = evDisp->makeWidget(ui.evFrame, name);
		QVBoxLayout* l = (QVBoxLayout*)ui.evFrame->layout();
		l->insertWidget(0, e.evWidget);

		QWidget::connect(e.evWidget, SIGNAL(sigCloseWidget(string)), this, SLOT(sltDeleteEventDisplayer(string)));
      result = eventViewers.size();
      eventViewers.push_back(e);
      updateEvLabel();
   }

   EventDisplayer *evDisp = eventViewers[result].evDisp;

   evDisp->displayEvent(event, logFile);
}

void EventViewerWidget::sltDeleteEventDisplayer(string name) {
	int32 result = -1;
   for (int32 i=0; i<(int32)eventViewers.size(); i++) {
      if (eventViewers[i].name == name) {
         result = i;
         break;
      }
   }
   if (result > -1) {
		ui.evFrame->layout()->removeWidget(eventViewers[result].evWidget);
      delete eventViewers[result].evWidget;
      delete eventViewers[result].evDisp;
      eventViewers.erase(eventViewers.begin() + result);
   }
   updateEvLabel();
}

void EventViewerWidget::updateEvLabel() {
	if (eventViewers.size()	== 0) {
		ui.labelNumEvs->setText("(no events)");
	} else {
		ui.labelNumEvs->setText(QString("(") + QString(intToStr(eventViewers.size()).c_str()) + QString(" events)"));
	}
}

EventViewerWidget::~EventViewerWidget() {
   for (card32 i=0; i<eventViewers.size(); i++) {
      delete eventViewers[i].evDisp;
   }
}

#include "EventViewerWidget_moc.cpp"