//---------------------------------------------------------------------------
#ifndef ExceptionHandlingH
#define ExceptionHandlingH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Exceptions.h"
//---------------------------------------------------------------------------
#include <exception>
//---------------------------------------------------------------------------


namespace geoxl {

// -- helper functions that display exceptions on the console

void FOUNDATION_API displayPException(PException *e);

#ifdef TARGET_PLATFORM_MSVC2005FF
void FOUNDATION_API displayOSException(unsigned int code, EXCEPTION_RECORD info);
#endif

void FOUNDATION_API displayGeneralException();

void FOUNDATION_API displaySTLException(std::exception e);


// -- structured win32 exception wrappper

#ifdef TARGET_PLATFORM_MSVC2005FF
class SE_Exception {
 public:
   unsigned int nSE;
	EXCEPTION_RECORD info;
	
   SE_Exception(unsigned int n, EXCEPTION_RECORD exceptionInfo) : nSE(n), info(exceptionInfo) {}
   ~SE_Exception() {}
};

FOUNDATION_API void SE_wrapper_trans_func( unsigned int u, _EXCEPTION_POINTERS* pExp );
#endif

/// call this once in main to setup handling of structured exception (i.e. WIN32-Exceptions)
FOUNDATION_API void setStructureExceptionHandler();


// -- macros for "global" exception handling (i.e. in qt-slots that receive input from qapp directly)


#define STD_GLOBAL_EXCEPTION_HANDLER_START \
try {\



#ifdef TARGET_PLATFORM_MSVC2005FF

#define STD_GLOBAL_EXCEPTION_HANDLER_END \
} catch (PException &e) {\
	displayPException(&e);\
} catch (SE_Exception e) {\
	displayOSException(e.nSE, e.info);\
} catch (std::exception e) {\
	displaySTLException(e);\
} catch (...) {\
	displayGeneralException();\
}

#else

#define STD_GLOBAL_EXCEPTION_HANDLER_END \
} catch (PException &e) {\
	displayPException(&e);\
} catch (...) {\
	displayGeneralException();\
}

#endif

}

#endif