//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "FileDialogs.h"
//---------------------------------------------------------------------------
#include "StringClassProperty.h"
#include "Configuration.h"
#include "AttachedData.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
#include <QtGui/QFileDialog>
#include <QtCore/qdir.h>
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


IMPLEMENT_GEOXL_CLASS(StandardDirectoriesConfig, 0) 
{
	BEGIN_CLASS_INIT( StandardDirectoriesConfig );
	ADD_STRING_PROP( openDir, 0 );
	ADD_STRING_PROP( saveDir, 0 );
	ADD_STRING_PROP( importDir, 0 );
	ADD_STRING_PROP( exportDir, 0 );
}


// ---

void StandardDirectoriesConfig::assign(const Object* obj, CopyContext *context) {
	const StandardDirectoriesConfig *other = dynamic_cast<const StandardDirectoriesConfig*>(obj);
	pAssert(other);
	this->openDir = other->openDir;
	this->saveDir = other->saveDir;
	this->importDir = other->importDir;
	this->exportDir = other->exportDir;
}


void FileDialogs::getDir(string &dir, InitDirType t) {
	if (t == CURRENT_DIR) {
		dir = qString2STLString(QDir::currentPath());
		return;
	} else {
		StandardDirectoriesConfig *att = dynamic_cast<StandardDirectoriesConfig*>(Configuration->getData("StandardDirectories"));
		if (att != NULL) {
			switch (t) {
			 case OPEN_DIR: dir = att->openDir; break;
			 case SAVE_DIR: dir = att->saveDir; break;
			 case IMPORT_DIR: dir = att->importDir; break;
			 case EXPORT_DIR: dir = att->exportDir; break;
			}
		}
	}
	dir = removeFilenameFromPath(dir);
	QDir dirTest(dir.c_str());
	if (!dirTest.exists()) {
		dir = qString2STLString(QDir::currentPath());
	}
}

void FileDialogs::storeDir(string dir, InitDirType t) {
	if (t == CURRENT_DIR) {
		return;
	} else {
		StandardDirectoriesConfig *att = dynamic_cast<StandardDirectoriesConfig*>(Configuration->getData("StandardDirectories"));
		if (att == NULL) {
			att = new StandardDirectoriesConfig();
			att->setup("StandardDirectories", AttachedData::ADF_PERSISTENT);
			Configuration->attachData(att);
		}
		switch (t) {
		 case OPEN_DIR: att->openDir = dir; break;
		 case SAVE_DIR: att->saveDir = dir; break;
		 case IMPORT_DIR: att->importDir = dir; break;
		 case EXPORT_DIR: att->exportDir = dir; break;
		}
	}
}

string FileDialogs::getFileNameIntern(
	QWidget *parent, string caption, 
   string filter, string *selectedFilter, string dir, InitDirType initDir, bool open, string defaultName)
 {
	QString qcaption(caption.c_str());
	QString qfilter(filter.c_str());
	QString qselectedFilter;
	if (dir == "") {
		getDir(dir, initDir);	
	}
	dir += defaultName;
	QString qdir(dir.c_str());
	QString filename;
	if (open) {
		filename = QFileDialog::getOpenFileName(parent, qcaption, qdir, qfilter, &qselectedFilter );
	} else {
		filename = QFileDialog::getSaveFileName(parent, qcaption, qdir, qfilter, &qselectedFilter );
	}
	if( !filename.isEmpty() ) {
		string result = qString2STLString(filename);
		if (selectedFilter != NULL) *selectedFilter = qString2STLString(qselectedFilter);
		storeDir(removeFilenameFromPath(qString2STLString(filename)), initDir);
		return result;	
	} else {
		return string("");
	}  
 }


string FileDialogs::getOpenFileName(
	QWidget *parent, string caption, 
   string filter, string *selectedFilter, string dir, InitDirType initDir, string defaultName)
 {
	return getFileNameIntern(parent, caption, filter, selectedFilter, dir, initDir, true, defaultName);
 }

string FileDialogs::getSaveFileName(QWidget *parent, string caption, 
	string filter, string *selectedFilter, string dir, InitDirType initDir, string defaultName)
{
	return getFileNameIntern(parent, caption, filter, selectedFilter, dir, initDir, false, defaultName);
}

vector<string> FileDialogs::getOpenFileNames(
	QWidget *parent, string caption, 
	string filter, string *selectedFilter, string dir, 
	InitDirType initDir, string defaultName)
{
	QString qcaption(caption.c_str());
	QString qfilter(filter.c_str());
	QString qselectedFilter;
	if (dir == "") {
		getDir(dir, initDir);	
	}
	dir += defaultName;
	QString qdir(dir.c_str());
	QStringList filenames;
	filenames = QFileDialog::getOpenFileNames(parent, qcaption, qdir, qfilter, &qselectedFilter );
	vector<string> result;
	for( int i=0;i<filenames.size();i++ )
		result.push_back(qString2STLString(filenames[i]));
	if (selectedFilter != NULL) *selectedFilter = qString2STLString(qselectedFilter);
	if (!result.empty())
		storeDir(removeFilenameFromPath(result[0]), initDir);
	return result;	
}
