//---------------------------------------------------------------------------
#ifndef MethodBrowserH 
#define MethodBrowserH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "ClassEditor.h"
#include "Delegate.h"
//---------------------------------------------------------------------------
#include <QtGui/QScrollArea.h>
#include <string>
#include <vector>
//---------------------------------------------------------------------------

class QFrame;

namespace geoxl
{
	using namespace std;

class NoArgsClassMethod;
class MethodBrowserWidget;


class FOUNDATION_API MethodBrowser : public ClassEditor {
 public:
   /// delegate(string methodName) : bool
   /// if return value is false for any of the events, the method is not called by this widget
   typedef DelegateTempl<string, bool> MethodClickedNotifyEvent;
 private:
	static MetaClass *MethodBrowserMetaClass;
   static Object* newInstance();

	MethodBrowserWidget  *m_Widget;
	Object	*	m_Object;

 public:
	MethodBrowser() {m_Widget = NULL; m_Object = NULL;}

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual QWidget *makeEditor(QWidget *parent, Object *obj);
	virtual void apply() {}
   virtual void update();
   virtual card32 getPreferredWidth();
   virtual card32 getPreferredHeight();
   virtual Object *getObject();

   virtual void addOnMethodClickedEvent(MethodClickedNotifyEvent ev);

	virtual bool standAloneEditorApplication();

	virtual ~MethodBrowser();
};



class FOUNDATION_API MethodBrowserCustomButton : public QPushButton {
	Q_OBJECT
	
 private:
	int index;

 private slots:
	void receiveClick();
		
 public:
	MethodBrowserCustomButton ( int index, QWidget * parent = 0 );
	MethodBrowserCustomButton ( int index, const QString & text, QWidget * parent = 0 );
	MethodBrowserCustomButton ( int index, const QIcon & icon, const QString & text, QWidget * parent = 0 );

 signals:
	void methodClicked(int index);
};



class FOUNDATION_API MethodBrowserWidget : public QScrollArea
{
	Q_OBJECT

 private:

	Object* m_Object;
	vector<const NoArgsClassMethod*> methods;
	int defaultHeight;
	
	QFrame *frame;
	QVBoxLayout *layout;
   vector<MethodBrowser::MethodClickedNotifyEvent> onMethodClicked;
	
 private slots:
	void methodClicked(int index);

 public:
	MethodBrowserWidget(QWidget * parent = NULL);
	~MethodBrowserWidget(void);

	void setup( Object* obj );
	int getDefaultHeight() {return defaultHeight;}
   virtual void addOnMethodClickedEvent(MethodBrowser::MethodClickedNotifyEvent ev);
	
	void updateEditor();


};


}

#endif