#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ObjectDictionaryEditorWidget.h"
//---------------------------------------------------------------------------
#include <qtGui/qmenu.h>
#include <qtGui/qmouseevent>
#include <qtCore/qurl>
#include <qtGui/qfiledialog.h>
#include <qtCore/qstringlist.h>
#include <qtGui/qmessagebox.h>
#include "ObjectDictionary.h"
#include "ClassSelectionDialog.h"
#include "Configuration.h"
#include "MetaClass.h"
#include "Object.h"
#include "ObjectViewsTable.h"
#include "BinaryObjectStreams.h"
#include "StringHelper.h"
#include "ObjectBrowserWidget.h"
#include "FileDialogs.h"
#include "ScriptEngine.h"
#include "ScriptDictionaryAdapter.h"
#include "ScriptEngineDebugWidget.h"
#include "ExceptionHandling.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( ObjectDictionary )

ObjectDictionaryEditorWidget::ObjectDictionaryEditorWidget(QWidget * parent)
:QListWidget(parent)
{
	setViewMode( QListView::IconMode );
	setSpacing( 7 );
	setMovement( QListView::Free );
	setAcceptDrops( true );
	
	m_ItemMenu = new QMenu( this ); 
	m_ItemMenu->hide();

	m_EditorsMenu = m_ItemMenu->addMenu( "&Editors" );
   m_ExecuteScriptAction = m_ItemMenu->addAction( "Execute &Script" );
   m_DebugScriptAction = m_ItemMenu->addAction( "De&bug Script" );
//	m_MethodsAction = m_ItemMenu->addAction( "&Methods" );
//	m_ApplicationsAction = m_ItemMenu->addAction( "&Applications" );
	m_ItemMenu->addSeparator();
	m_NewAction = m_ItemMenu->addAction( "&New Object" );
	m_NewAction->setShortcut( QKeySequence( Qt::Key_N | Qt::CTRL ) );
	m_OpenAction = m_ItemMenu->addAction( "&Open Object" );
	m_OpenAction->setShortcut( QKeySequence( Qt::Key_O | Qt::CTRL ) );
	m_ItemMenu->addSeparator();
   m_NewFolderAction = m_ItemMenu->addAction("New &Folder");
	m_ItemMenu->addSeparator();
	//m_SaveAction = m_ItemMenu->addAction( "&Save" );
	m_SaveAsAction = m_ItemMenu->addAction( "Save &As" );
	m_RenameAction = m_ItemMenu->addAction( "&Rename" );
	m_RenameAction->setShortcut( QKeySequence( Qt::Key_F2 ) );
	m_DeleteAction = m_ItemMenu->addAction( "&Delete Object" );
	m_DeleteAction->setShortcut( QKeySequence( Qt::Key_Delete ) );

	connect( m_ItemMenu, SIGNAL(triggered(QAction*)),SLOT(sltMenuHandler(QAction*)));

	m_ClassSelectionDialog = new ClassSelectionDialog(this);
	m_ClassSelectionDialog->init( Object::getClass() , false );

}

ObjectDictionaryEditorWidget::~ObjectDictionaryEditorWidget(void)
{
}

void ObjectDictionaryEditorWidget::sltMenuHandler( QAction * action )
{
   STD_GLOBAL_EXCEPTION_HANDLER_START
	// ----------------------------- new ------------------------------
	if( action == m_NewAction )
	{
		m_ClassSelectionDialog->move( QCursor::pos() );
		if( m_ClassSelectionDialog->exec() == QDialog::Accepted )
		{
			MetaClass * mc = m_ClassSelectionDialog->getSelectedClass();
			Object * obj = mc->newInstance();
			addObject( obj, "" );
		}
	}

	// ----------------------------- open ------------------------------
	if( action == m_OpenAction )
	{
		vector<string> filenames = FileDialogs::getOpenFileNames(this, "open file(s)", "*.object" );
		for( unsigned i=0;i<filenames.size();i++ ) {
			loadFile( filenames[i] );
			//debugOutput << "loaded object: " << filenames[i] << "\n";
	   }
	}
	// ----------------------------- new folder ------------------------------
	if( action == m_NewFolderAction )
	{
      addObject(new ObjectDictionary(), "");
	}

	// ----------------------------- editor ------------------------------
	if( m_EditorClasses.find( action ) != m_EditorClasses.end() )
	{
		ClassEditor * ce = (ClassEditor*)m_EditorClasses[action].mc->newInstance();
   	ObjectBrowserWidget *newBrowser = new ObjectBrowserWidget(m_EditorClasses[action].obj, ce, this);
		QWidget * w = ce->makeEditor( newBrowser, m_EditorClasses[action].obj );
   	newBrowser->setupEditor(w);		
		newBrowser->show();
		ObjectViewsTable::bindView( m_EditorClasses[action].obj, ce );
	}

	// ----------------------------- save ------------------------------
	if(/*( action == m_SaveAction)||*/( action == m_SaveAsAction))
	{
		QListWidgetItem * item = currentItem();
		if( item )
		{
			string objName = item->text().toLatin1().constData();
			Object * obj = m_ObjectDict->getObject( objName );
			pAssert( obj != NULL );
			string filename = FileDialogs::getSaveFileName( this, "save file", "*.object", NULL, "", FileDialogs::SAVE_DIR, objName  );
			if( filename != "" )
			{
				if( !QString(filename.c_str()).contains( ".object", Qt::CaseInsensitive ) )
					filename += ".object";
				saveFile( obj, filename );
			}
		}
	}

   if (action == m_RenameAction) {
		QListWidgetItem * item = currentItem();
		if( item )
		{
			string objName = item->text().toLatin1().constData();
			bool ok;
         QString newName = QInputDialog::getText(this, tr("Object name"), tr("Please enter the new object name"), QLineEdit::Normal, objName.c_str(), &ok);
			if( ok )
			{
			   bool success = m_ObjectDict->renameObject(objName, newName.toLatin1().constData());
			   pAssert( success );
			   item->setText(newName);
			}
      }
   }

	// ----------------------------- delete ------------------------------
	if (action == m_DeleteAction) {
		QListWidgetItem * item = currentItem();
		if( item )
		{
			string objName = item->text().toLatin1().constData();

			if( QMessageBox::question( this, "Question", QString("Do you want to delete '") + objName.c_str() + "'?", 
				QMessageBox::Yes,QMessageBox::No) == QMessageBox::Yes )
			{
				Object * obj = m_ObjectDict->getObject( objName );
				pAssert( obj != NULL );

				delete item;
				m_ObjectDict->deleteObject( obj );
			}
		}
	}

	// ----------------------------- execute script ------------------------------
   if (action == m_ExecuteScriptAction) {
      QListWidgetItem * item = currentItem();
      if( item )
      {
         string objName = item->text().toLatin1().constData();
			Object * obj = m_ObjectDict->getObject( objName );
			pAssert( obj != NULL );
			
			ScriptEngine * script = dynamic_cast<ScriptEngine*>(obj);
			if (script != NULL) {
            ScriptDictionaryAdapter adapter;
            adapter.init(m_ObjectDict, script);
            try {
               script->execute(false);
            } catch(PException &e) {
               std::string msg = "Error in line " + intToStr(script->getLine() + 1) + ": \n" + e.getMsg();
               QMessageBox::critical( this, "GEOXLscript error", QString(msg.c_str()));
            }
            
            adapter.done();
			}
	   }
   }

	// ----------------------------- debug script ------------------------------
   if (action == m_DebugScriptAction) {
      QListWidgetItem * item = currentItem();
      if( item )
      {
         string objName = item->text().toLatin1().constData();
			Object * obj = m_ObjectDict->getObject( objName );
			pAssert( obj != NULL );
			
			ScriptEngine * script = dynamic_cast<ScriptEngine*>(obj);
			if (script != NULL) {
            ScriptDictionaryAdapter *adapter = new ScriptDictionaryAdapter;
            adapter->init(m_ObjectDict, script);
            try {
               ScriptEngineDebugWidget* debugger = new ScriptEngineDebugWidget(script);
               debugger->registerScrDicAdapter(adapter);
               // The debugger will release the script dictionary adapter
               debugger->start();
            } catch(PException &e) {
               std::string msg = "Error in line " + intToStr(script->getLine() + 1) + ": \n" + e.getMsg();
               QMessageBox::critical( this, "GEOXLscript error", QString(msg.c_str()));
            }
			}
	   }
   }
   STD_GLOBAL_EXCEPTION_HANDLER_END
}


void ObjectDictionaryEditorWidget::updateEditor()
{
	clear();
	for( unsigned i=0;i<m_ObjectDict->getNumObjects();i++ )
	{
		const MetaClass * sceneClass = MetaClass::getClassByName( "Scene" );
		QIcon icon(m_ObjectDict->getObject(i)->getInstanceClass()->getGUIDescriptor()->getIcon().c_str());
		QListWidgetItem * item = new QListWidgetItem( icon, m_ObjectDict->getObjectName(i).c_str(), this );
	}

	// HACK: OOC
	/*static bool openfirstscene = false;
	if( m_ObjectDict->getNumObjects() > 0 && !openfirstscene)
	{
		QMouseEvent * e = new QMouseEvent(QMouseEvent::MouseButtonDblClick, QPoint(10,10),  Qt::LeftButton,0,0);
		mouseDoubleClickEvent(e);
		openfirstscene = true;
	}*/


}

void ObjectDictionaryEditorWidget::mousePressEvent( QMouseEvent * e)
{
   STD_GLOBAL_EXCEPTION_HANDLER_START
	if( e->button() == Qt::RightButton )
	{
		QListWidgetItem * item = itemAt( e->pos() );
		if( item )
			setCurrentItem( item );
		buildMenu( item );

		m_ItemMenu->exec( mapToGlobal(e->pos()) );
	}
	else
	{
		QListWidget::mousePressEvent( e );
	}
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void ObjectDictionaryEditorWidget::loadFile( string filename )
{
	BinaryInputObjectStream in(filename.c_str());
   Persistent *instance = NULL;
   in.readObject(instance);
   debugOutput << "loaded object: " << filename << "\n";
	string objName = removePathFromFilename(filename);
	if( objName.find( ".object") == objName.length() - 7 )
		objName = objName.substr( 0, objName.length() - 7 );
   addObject(instance, objName );
}

void ObjectDictionaryEditorWidget::saveFile( Object * obj, string filename )
{
  BinaryOutputObjectStream out(filename.c_str());
  out.writeObject((Persistent*)obj);
}

void ObjectDictionaryEditorWidget::addObject( Object * obj, string name )
{
	string generatedName = m_ObjectDict->addObject( obj, name, true );
	const MetaClass * sceneClass = MetaClass::getClassByName( "Scene" );
	bool isSceneObject = obj->getInstanceClass()->inheritsFrom( sceneClass );
	QIcon icon(obj->getInstanceClass()->getGUIDescriptor()->getIcon().c_str());
	QListWidgetItem * item = new QListWidgetItem( icon, generatedName.c_str(), this );
	setCurrentItem( item );
}



void ObjectDictionaryEditorWidget::buildMenu( QListWidgetItem * item )
{
	if( item == NULL )
	{
		m_EditorsMenu->setEnabled(false);
		//m_SaveAction->setEnabled(false);
		m_SaveAsAction->setEnabled(false);
		m_RenameAction->setEnabled(false);
		m_DeleteAction->setEnabled(false);
		m_ExecuteScriptAction->setEnabled(false);
		m_DebugScriptAction->setEnabled(false);
	}
	else
	{
		//m_SaveAction->setEnabled(true);
		m_SaveAsAction->setEnabled(true);
		m_RenameAction->setEnabled(true);
		m_DeleteAction->setEnabled(true);
		m_EditorsMenu->clear();
		m_EditorClasses.clear();
		string objectName = item->text().toLatin1().constData();

		Object * obj = m_ObjectDict->getObject( objectName );
		pAssert( obj != NULL );

		const GUIDescriptor * gd = obj->getInstanceClass()->getGUIDescriptor();
		m_EditorsMenu->setEnabled( gd->getNumClassEditors() > 0 );
		for( unsigned i=0;i<gd->getNumClassEditors();i++ )
		{
			MetaClass * mc = (MetaClass*)gd->getClassEditor(i);
			QAction * action = m_EditorsMenu->addAction( mc->getClassName().c_str() );
			TMetaInfo mi;
			mi.mc = mc;
			mi.obj = obj;
			mi.name = objectName;
			mi.isDefault = (i == 0) && (gd->getNumApplications() == 0);
			m_EditorClasses[ action ] = mi;
			if( mi.isDefault )
			{
				QFont font;
				font.setBold(true);
				action->setFont( font );
			}
		}

      ScriptEngine* script = dynamic_cast<ScriptEngine*>(obj);
      if (script != NULL) {
         m_ExecuteScriptAction->setEnabled(true);
         m_DebugScriptAction->setEnabled(true);
      } else {
         m_ExecuteScriptAction->setEnabled(false);
         m_DebugScriptAction->setEnabled(false);
      }

		// TODO add methods and applications to itemmenu menu

	}
}

void ObjectDictionaryEditorWidget::mouseDoubleClickEvent( QMouseEvent * e )
{
   STD_GLOBAL_EXCEPTION_HANDLER_START
	QListWidgetItem * item = itemAt(e->pos() );
	if( item != NULL )
	{
		setCurrentItem( item );
		string objectName = item->text().toLatin1().constData();

		Object * obj = m_ObjectDict->getObject( objectName );
		pAssert( obj != NULL );

		const GUIDescriptor * gui = obj->getInstanceClass()->getGUIDescriptor();
		if( gui->getNumApplications() == 0 )
		{
			ClassEditor * ce = (ClassEditor*)gui->getDefaultClassEditor()->newInstance();
			if (ce->standAloneEditorApplication()) {
			   ObjectViewsTable::bindView( obj, ce );
			   QWidget * w = ce->makeEditor( this , obj );
			   w->setWindowFlags( Qt::Window );
			   w->setWindowTitle( objectName.c_str() );
			   w->setAttribute( Qt::WA_DeleteOnClose );
			   w->show();
			} else {
   	      ObjectBrowserWidget *newBrowser = new ObjectBrowserWidget(obj, ce, this);
		      QWidget * w = ce->makeEditor( newBrowser, obj );
   	      newBrowser->setupEditor(w);		
		      newBrowser->show();
		      ObjectViewsTable::bindView( obj, ce );
		     }

		}
		else
		{
			GUIDescriptor::ApplicationPtr app = gui->getApplication(0);
			app.startProc( obj );
		}
	}
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void ObjectDictionaryEditorWidget::dragEnterEvent( QDragEnterEvent *e)
{
   STD_GLOBAL_EXCEPTION_HANDLER_START
	if( e->mimeData()->hasUrls() )
		e->acceptProposedAction();
	else
		QListWidget::dragEnterEvent( e );
   STD_GLOBAL_EXCEPTION_HANDLER_END
}

void ObjectDictionaryEditorWidget::dragMoveEvent( QDragMoveEvent *e )
{
   STD_GLOBAL_EXCEPTION_HANDLER_START
	if( e->mimeData()->hasUrls() )
		e->acceptProposedAction();
	else
		QListWidget::dragMoveEvent( e );
   STD_GLOBAL_EXCEPTION_HANDLER_END
	
}

void ObjectDictionaryEditorWidget::dropEvent( QDropEvent * e)
{
   STD_GLOBAL_EXCEPTION_HANDLER_START
	if( e->mimeData()->hasUrls() )
	{
		for( int i=0;i<e->mimeData()->urls().size();i++ )
			loadFile( string( e->mimeData()->urls()[i].toLocalFile().toLatin1().constData()) );
	}
	else
		QListWidget::dropEvent( e );
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void ObjectDictionaryEditorWidget::load()
{
	sltMenuHandler( m_OpenAction );
}

void ObjectDictionaryEditorWidget::save()
{
	sltMenuHandler( m_SaveAsAction );
}

Object* ObjectDictionaryEditorWidget::getSelectedObject()
{
	QListWidgetItem * item = currentItem();
	if( item )
	{
		string objName = item->text().toLatin1().constData();
		Object * obj = m_ObjectDict->getObject( objName );
		pAssert( obj != NULL );
		return obj;
	} else return NULL;
}


#include "ObjectDictionaryEditorWidget_moc.cpp"