//---------------------------------------------------------------------------
#ifndef PerformanceGraphWidgetH
#define PerformanceGraphWidgetH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Object.h"
#include "PTypes.h"
#include <QtGui/QWidget.h>
#include <deque>
//---------------------------------------------------------------------------

namespace geoxl {

class FOUNDATION_API PerformanceGraphWidget : public QWidget {

	Q_OBJECT

	protected:
		/// size of the displayed grid
		card32 gridSize;
		/// minimum displayed value, not used at the moment and hardcoded to 0
		card32 minValue;
		/// maximum displayed value, can be set by user, but will be automatically set to the highest found value of input
		card32 maxValue;
		/// width of the legend displayed left of the graph, is automatically enlarged if legend doesn't fit the space
		card32 legendOffset;
		/// name of the graph
		string graphName;

		/// holds the next value which will (only) be set by calling doStep();
		card32 nextValue;
		/// the width of one step in x-direction, so to say the x-scaling of the graph
		card32 stepSize;

		/// the size hint of the window
		QSize widgetSize;
		/// the graph background color
		QColor backgroundColor;
		/// the graph grid color
		QColor gridColor;
		/// the graph legend text color
		QColor legendColor;
		/// the graph legend2 text color
		QColor legend2Color;
		/// the graph line color
		QColor graphColor;

		/// holds the points of the graph, will be cut in doStep() if the graph gets broader than the window
		deque<card32> graphPoints;

		void drawGrid(QPainter &painter);
		void drawLegend(QPainter &painter);
		void drawGraph(QPainter &painter);

		void paintEvent(QPaintEvent* event);


	public:
		PerformanceGraphWidget(QWidget* parent = NULL);
		~PerformanceGraphWidget();

		QSize minimumSizeHint() const;
		QSize sizeHint() const;

		/// sets gridSize
		inline void setGridSize(card32 g) { gridSize = g; };
		// setMinValue not used till yet, see minValue
		// inline void setMinValue(card32 m) { minValue = m; };
		/// sets maxValue
		inline void setMaxValue(card32 m) { maxValue = m; };
		/// sets legend offset
		inline void setLegendOffset(card32 o) { legendOffset = o; };
		/// sets graphName
		inline void setGraphName(string n) { graphName = n; };
		/// sets stepsize
		inline void setStepSize(card32 s) { stepSize = s; };
		/// sets the widgets minimumSizeHint and sizeHint
		inline void setWidgetSize(card32 w, card32 h) { widgetSize = QSize(w,h); };
		/// sets graph background color
		inline void setBackgroundColor(QColor &c) { backgroundColor = c; };
		/// sets graph grid color
		inline void setGridColor(QColor &c) { gridColor = c; };
		/// sets graph legend text color
		inline void setLegendColor(QColor &c) { legendColor = c; };
		/// sets graph legend2 text color
		inline void setLegend2Color(QColor &c) { legend2Color = c; };
		/// sets graph line color
		inline void setGraphColor(QColor &c) { graphColor = c; };

		// please implement get-methods on demand


		/// sets the next value which will be appended to the graph by doStep(). if called without doStep() afterwards nothing happens.
		inline void setNextValue(card32 s) { nextValue = s; };

		/// appends the next value and updates the graph
		inline void doStep() {
			if (nextValue>maxValue) maxValue = nextValue;
			if (nextValue<minValue) nextValue = minValue;
			graphPoints.push_back(nextValue);
			// left-clip graph if it gets too broad
			int32 tooMuch = (int32)graphPoints.size() - ((card32)width()-legendOffset)/stepSize;
			if (tooMuch>1000) {
				tooMuch /= 2;
				for (int32 i=0; i<tooMuch; i++) {
					graphPoints.pop_front();
				};
				maxValue = 10;
				for (unsigned i=0; i<graphPoints.size(); i++) {
					if (graphPoints[i] > maxValue) maxValue = graphPoints[i];
				};
			}
			repaint();
		};


};

}


#endif
