//---------------------------------------------------------------------------
#ifndef ProgressWindowH
#define ProgressWindowH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Object.h"
#include "Timer.h"
#include <list>
#include <stack>
#include <string>
//---------------------------------------------------------------------------


namespace geoxl {

/// Class for the global progress window of GEOXL
class FOUNDATION_API ProgressWindow : public Object {
  public:
    class WeightElem {
      public:
        WeightElem(bool progressAvailable = true, float32 weight = 1.0f, std::string stepName = "") {
          this->stepName = stepName;
          this->weight = weight;
          this->progressAvailable = progressAvailable;
        }
        std::string stepName;
        float32 weight;
        bool progressAvailable;
    };
    typedef std::list<WeightElem> WeightList;

  private:
    static MetaClass *ProgressWindowMetaClass;
    static Object* newInstance();

    typedef std::stack<WeightList> WeightStack;

    bool isVisible, isAborted;
    card32 startTime, lastProgressTime;
    Timer timer;

    bool toolProgressAvailable, overallProgressAvailable;

    /// Percentage for the steps that are fully done
    WeightStack weightStack;
    std::stack<float32> percentageDoneStack;
    std::stack<card32> updateIntervalStack;

    /// Updates the form of the progress window
    void initForm();

  public:
    virtual const MetaClass *getInstanceClass() const;
    static const MetaClass *getClass();
    static void init(const MetaClass *parentClass);
    static void shutdown();

    ProgressWindow();
    ~ProgressWindow();

    /// Pushes a list of steps onto the operations stack
    void pushSteps(WeightList steps);
    /// Pushes a single step onto the operations stack
    void pushStep(bool progressAvailable = true, std::string stepName = "");
    /// Moves to the next step in the list
    void nextStep(bool& aborted);
    /// Pops the step list on the top of the stack
    void popSteps(bool& aborted);
	 void popSteps() {bool aborted;popSteps(aborted);}
	 
	 bool getIsAborted() {return isAborted;}

    /// Displays the tool message given
    void showMessage(std::string msg);
    /// Updates the progress window and returns false if ESC was pressed.
    bool progress(card32 position);
    /// Sets the minimum interval between GUI-updates for this level in the stack
    void setUpdateInterval(card32 value);
};

/// Global instance for the progress window.
/**
 * Use this instead of your own instance to make sure that there aren't several
 * progress windows displayed.
 */
extern FOUNDATION_API ProgressWindow* progressWindow;

}

#endif