#include "StdAfx.h"
#include "Configuration.h"
//---------------------------------------------------------------------------
#include "ScriptEngineEditorWidget.h"
//---------------------------------------------------------------------------
#include "ScriptEngine.h"
#include <QtGui/QTextCursor.h>
#include <QtGui/QMessageBox.h>
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( ScriptEngine )

ScriptEngineEditorWidget::ScriptEngineEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	//Example: easy access to QtDesigner buttons, etc. by ui object
	QWidget::connect(ui.BTSyntaxCheck, SIGNAL(clicked()), this, SLOT(sltBTSyntaxCheckClicked()));
	QWidget::connect(ui.BTApply, SIGNAL(clicked()), this, SLOT(sltBTApplyClicked()));

	// button icons
	ui.BTSyntaxCheck->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/Zoom.bmp").c_str()));
	ui.BTApply->setIcon(QIcon((getConfigurationString("IMAGE_PATH")+"/OK.bmp").c_str()));

}

ScriptEngineEditorWidget::~ScriptEngineEditorWidget(void)
{
}

void ScriptEngineEditorWidget::setup( ScriptEngine * obj )
{
	m_ScriptEngine = obj;
}

void ScriptEngineEditorWidget::updateEditor()
{
	ui.TEScriptCode->clear();
	for (card32 i = 0; i < m_ScriptEngine->code.size(); ++i) {
		ui.TEScriptCode->append(QString(m_ScriptEngine->code[i].c_str()));
	}
}


// ---------- slots ----------

void ScriptEngineEditorWidget::sltBTSyntaxCheckClicked() {
	sltBTApplyClicked();
	bool errorFound = false;
	try {
		m_ScriptEngine->execute(true);
	} catch (EParserException &e) {

		errorFound = true;

		// select error text
		QTextCursor tc = ui.TEScriptCode->textCursor();
		tc.movePosition(QTextCursor::Start);
		pAssert(!tc.atEnd());
		ui.TEScriptCode->setTextCursor(tc);
		ui.TEScriptCode->find(QString(e.getMsg().c_str()));

		QMessageBox::warning(this, "Warning", e.getMsg().c_str());
	}

	if (!errorFound)
		QMessageBox::information(this, "Info", "No syntax error found.");
};

void ScriptEngineEditorWidget::sltBTApplyClicked() {
	m_ScriptEngine->code.clear();

	QTextCursor tc = ui.TEScriptCode->textCursor();
	tc.movePosition(QTextCursor::Start);
	if (tc.atEnd()) return;

	do {
		tc.select(QTextCursor::LineUnderCursor);
		m_ScriptEngine->code.push_back( (char*)qPrintable(tc.selectedText()) );
	} while (tc.movePosition(QTextCursor::Down));

}


#include "ScriptEngineEditorWidget_moc.cpp"