//---------------------------------------------------------------------------
#ifndef BisectionSearchH
#define BisectionSearchH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <vector>
#include "PTypes.h"
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;

/** search in a monotonicly raising list of entries: search for first entry in a vector wich has a value
    larger than searchValue. class EntryType must define operators <= and >= for comparison with NumericalType*/
template<class EntryType, class NumericalType>
inline card32 bisectionSearch(const vector<EntryType> &entries, card32 lower, card32 upper, NumericalType searchValue) {
   while (upper - lower > 1) {
      card32 mid = (upper + lower) / 2;
      if (entries[mid] <= searchValue) {
         lower = mid;
      } else {
         upper = mid;
      }
   }
   if (entries[lower] >= searchValue) {
      return lower;
   } else {
      return upper;
   }
}

/// returns index of first value >= search value
template<class ElemPtrType, class NumericalType>
inline card32 arrayBisectionSearch(const ElemPtrType entries, card32 lower, card32 upper, NumericalType searchValue) {
   while (upper - lower > 1) {
      card32 mid = (upper + lower) / 2;
      if (entries[mid] <= searchValue) {
         lower = mid;
      } else {
         upper = mid;
      }
   }
   if (entries[lower] >= searchValue) {
      return lower;
   } else {
      return upper;
   }
}

}

#endif
