#ifndef BIVARIATE_POLYNOMIAL_H
#define BIVARIATE_POLYNOMIAL_H

namespace geoxl{

/// Polynomial of arbitray degree and dimension
template<class Type, int Degree>
class BivariatePolynomial
{
	private:
		/// Polynomial parameters
		Type param[(Degree+1)*(Degree+1)];
		/// Index access for parameters
		inline int I( int i, int j ){ return i * ( Degree + 1 ) + j; }

	public:
      /// Constructor
      BivariatePolynomial();
		/// Evaluate polynomial at position x
		Type eval( Type x, Type y );
		/// Debug: init polynomial to specific function
		void init();
		/// Setter: parameters
		inline void setParam( int i, int j, Type p ){ param[I(i,j)] = p; }

      /// Inverse transformation ( back to world coordinates )
      Matrix3f invTransformation;
      /// Inverse translation ( back to world coordinates )
      Vector3f invTranslation;
      /// Extend of polynomial
      Vector2f dMin, dMax;
};

/**
 * Evaluate polynomial at position ( x,y )
 */
template<class Type, int Degree>
BivariatePolynomial<Type,Degree>::BivariatePolynomial()
{
   for ( unsigned int n = 0; n < (Degree+1)*(Degree+1); n++ )
      param[n] = 0;
}

/**
 * Evaluate polynomial at position ( x,y )
 */
template<class Type, int Degree>
Type BivariatePolynomial<Type,Degree>::eval( Type x, Type y )
{
	Type result = 0;
	for ( int i = 0; i <= Degree; i++ ){
		for ( int j = 0; j <= Degree; j++ ){
			result = result + param[I(i,j)] * power( x , i ) * power( y , j );
		}
	}
	return result;
}

/**
 * Debug: init polynomial to specific function
 */
template<class Type, int Degree>
void BivariatePolynomial<Type,Degree>::init()
{
	setParam( 0, 0, 0 );
	setParam( 0, 1, 0 );
	setParam( 0, 2, 0 );
	setParam( 1, 0, 0 );
	setParam( 1, 1, 0 );
	setParam( 1, 2, 0 );
	setParam( 2, 0, 0 );
	setParam( 2, 1, 0 );
	setParam( 2, 2, 0 );
}
		
}

#endif