#ifndef LINE_H
#define LINE_H
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace geoxl
{

/// PS_INFRONT: entire object is on the positive side of plane
/// PS_BEHIND: entire object is on the negative side of plane
/// PS_COINCIDING: entire object is on the plane
/// PS_SPANNING: plane splits object into two parts
enum EPlaneSide { PS_INFRONT = 0, PS_BEHIND = 1, PS_COINCIDING = 2, PS_SPANNING = 3};

/// Container for a plane defined by a normal and one point within the plane.
class FOUNDATION_API Line
{
	protected:
		/// Centroid of the given points
		Vector3f point;
		/// Normal of the regression plane
		Vector3f dir;

	public:
		Line();
		Line( Vector3f d, Vector3f p );
		~Line();
		Line( const Line &other ) { dir = other.getDirection(); point = other.getPoint(); }
		
		void operator = ( const Line &other ){ dir = other.getDirection(); point = other.getPoint(); }
		void clear();

		inline Vector3f getPoint() const { return point; }
		inline Vector3f getDirection() const { return dir; }
		inline void setDirection( Vector3f d ){ dir = d; }
		inline void setPoint( Vector3f p ){ point = p; }
};

}

#endif
