//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "Plane.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS( Plane, 0 )
{
	BEGIN_CLASS_INIT( Plane );
	ADD_VECTOR3F_PROP( normal, 0 );
	ADD_VECTOR3F_PROP( point, 0 );
}

Plane::Plane()
{
	clear();
}

Plane::Plane( Vector3f n, Vector3f p )
{ 
	clear();
	normal = n; 
	normal.normalize(),
	point = p;
}

Plane::~Plane()
{
}

void Plane::clear()
{
	for ( unsigned int n = 0; n < 3; n++ ){
		point[n] = 0;
		normal[n] = 0;
	}
}

float Plane::getDistanceFromPlane( const Vector3f p ) const
{
	return ( normal * p - normal * point );
}

float Plane::calculateSignedDistance( const Vector3f &p ) const
{
	return p*normal - point*normal;
}

float Plane::calculateLineIntersection( const Vector3f &a, const Vector3f &b ) const
{
	float t = (b-a)*normal;
	if( t == 0.0f )
		return 1e20f;
	t = (normal * point - a*normal) / t;
	return t;
}

EPlaneSide Plane::classify( const Line &line ) const
{
	float d1 = calculateSignedDistance( line.getPoint() );
	float d2 = calculateSignedDistance( line.getPoint()+line.getDirection() );

	const float epsilon = 1e-20f;

	if( d1 > epsilon && d2 > epsilon )
		return PS_INFRONT;

	if( d1 < -epsilon && d2 < -epsilon )
		return PS_BEHIND;

	if( (d1 > epsilon && d2 < -epsilon) ||
		(d2 > epsilon && d1 < -epsilon) )
		return PS_SPANNING;

	return PS_COINCIDING;
}

void Plane::split( const Line &line, Line &positive, Line &negative ) const
{
	pAssert( classify( line ) == PS_SPANNING );

	float d1 = calculateSignedDistance( line.getPoint() );

	Vector3f xm = calculateIntersection( line );

	const float epsilon = 1e-20f;

	if( d1 > epsilon )
	{
		positive = Line( xm, line.getPoint() - xm );
		negative = Line( line.getPoint() + line.getDirection(), xm - ( line.getPoint() + line.getDirection() ) );
	}
	else
	{
		positive = Line( line.getPoint() + line.getDirection(), xm - ( line.getPoint() + line.getDirection() ) );
		negative = Line( xm, line.getPoint() - xm );
	}

}

Vector3f Plane::calculateIntersection( const Line& line ) const
{
	return line.getPoint() + line.getDirection() * ((point*normal - line.getPoint()*normal) / (normal * line.getDirection()) );

}

Vector3f Plane::calculateMirrorPoint( const Vector3f &p)
{
	return p - normal * ((p - point)*normal*2.0f);
}
