#ifndef PlaneH
#define PlaneH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "Line.h"
//---------------------------------------------------------------------------

namespace geoxl
{

/// Container for a plane defined by a normal and one point within the plane.
	class FOUNDATION_API Plane : public Persistent
{
	GEOXL_CLASS( Plane )
	public:
		Vector3f point;
		Vector3f normal;

		Plane();
		Plane( Vector3f n, Vector3f p );
		~Plane();
		inline void setup( Vector3f n, Vector3f p ){ normal = n; point = p; }
		void clear();

		// 3D only
		EPlaneSide classify( const Line &line ) const;
		void split( const Line &line, Line &positive, Line &negative ) const;
		Vector3f calculateIntersection( const Line &plane ) const;

		inline void operator=( const Plane& p ){ normal = p.getNormal(); point = p.getPoint(); }
		float calculateSignedDistance( const Vector3f &p ) const;
		float calculateLineIntersection( const Vector3f &a, const Vector3f &b ) const;

		Vector3f calculateMirrorPoint( const Vector3f &p);

		Plane flip() const {
			return Plane( -normal, point );			  
		}

		/*** MEMBER ACCESS ***/
		float getDistanceFromPlane( const Vector3f p ) const;
		inline Vector3f getPoint() const { return point; }
		inline Vector3f getNormal() const { return normal; }
		inline float getDistance() const { return normal * point; }
		inline void setNormal( Vector3f n ){ normal = n; normal.normalize(); }
		inline void setPoint( Vector3f p ){ point = p; }

		
};
}

#endif
