//---------------------------------------------------------------------------
#ifndef PolygonH
#define PolygonH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "Plane.h"
#include "pAssert.h"
//---------------------------------------------------------------------------

namespace geoxl
{
   using namespace std;

/**
   required methods for type Vertex:
    operator +
    operator -
    operator *
    Vector3f getPos3f() const
*/

#define COINCIDING_EPSILON 0.0000001f

template<class Vertex> class Polygon
{
private:

   vector<Vertex> m_Points;

public:

   typedef Polygon<Vertex> MyType;


   Polygon();
   Polygon( const Polygon &p );
   void operator = ( const Polygon &p );

   void addVertex( const Vertex &vec );
   inline card32 getNumVertices() const;
   void clear() {m_Points.clear();}

   inline Vertex & operator [] ( card32 i );
   inline const Vertex & operator [] ( card32 i ) const;

   inline bool isValid() const;

   EPlaneSide classify( const Plane &plane ) const;

   /// calculates normalized normal. if *this is a degenerated polygon, then valid is false
   /// and return vector is (0,0,0)
   Vector3f calculateNormal( bool &valid );

   /// calculates polygon-plane, throws exception if polygon is invalid
   Plane calculatePlane() const;

   /// cut polygon at plane, polygon-part on the negative plane-side will be removed
   /// when the entire polygon is on the negative plane-side the polygon will have no points
   Polygon cut( const Plane &plane ) const;

   /// returns true if poly intersect *this
   /// *** untested ***
   bool intersect( const MyType &poly );
};

class PolyVector3f : public Vector3f
{
public:
   inline Vector3f getPos3f() const {return makeVector3f( (*this)[0],(*this)[1],(*this)[2] );}
   inline void assign( const Vector3f & v ) {(*this)[0] = v[0];(*this)[1] = v[1];(*this)[2] = v[2];}

   static PolyVector3f create( const Vector3f &v ) {PolyVector3f ret; ret.assign(v); return ret;}
};

typedef Polygon< PolyVector3f > SimplePolygon;


}
#include "Polygon_templCode.h"



#endif
