#ifndef REGRESSION_PLANE_H
#define REGRESSION_PLANE_H

// Xrt
#include "ParametricPlane.h"
#include "PCA.h"
#include "PCA_templCode.h"

namespace geoxl
{

/// Implementation of a regression plane. The class allows to add an arbitrary number of templated points in order to compute the associated regression plane via the covariance matrix.
class FOUNDATION_API RegressionPlane : public ParametricPlane
{
	private:
      /// PCA estimator
      PCA<float,3> pca;

   public:
		/// Constructor
		RegressionPlane();
		/// Destructor
		~RegressionPlane();
		/// Reinitialize the point set and all matrices
		void clear();
		/// Add a point to the regression set
		void addPoint( Vector3f p );
		/// Compute the repression plane
		void computePlane();
};

}

#endif
