//---------------------------------------------------------------------------
#ifndef SVDH
#define SVDH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "DynamicLinearAlgebra.h"
//---------------------------------------------------------------------------
#include "DebugOutput.h"
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;


/// Calc fast Covariance Matrix and SVD on DynamicVectors using OpenCV
/// currently instantiated explicitly - allows only float32 and float64
template <typename FloatType>
class FOUNDATION_API SVD {

 private:

 public:

   SVD();

   static const int getType();

   static void computeSVD(const DynamicMatrix<FloatType> &input,
      DynamicMatrix<FloatType> &U,
      DynamicVector<FloatType> &sv,
      DynamicMatrix<FloatType> &Vt
   );

   static void computeSVD(const Matrix3f &input,
      Matrix3f &U,
      Vector3f &sv,
      Matrix3f &Vt
   );

   static void computeSVDPseudoInverseMatrix(const DynamicMatrix<FloatType> &input,
      DynamicMatrix<FloatType> &returnedPseudoInverse,
	  float64 &returnedInversedConditionNumberOfA 
   );

   static void computeSVDPseudoInverseMatrix(const Matrix3f &input,
      Matrix3f &returnedPseudoInverse,
	  float64 &returnedInversedConditionNumberOfA 
   );

	static void invertMatrix(const Matrix3f &input, Matrix3f &output, float32 eps = 10e-30f);

	static void solve(const Matrix3f &A, const Vector3f &b, Vector3f &x, float32 eps = 10e-30f);

   static inline void MatrixToArray(const DynamicMatrix<FloatType> &source, FloatType* target);
   static inline void ArrayToMatrix(FloatType* source, DynamicMatrix<FloatType> &target);

   static inline void MatrixToArray(const vector<Vector3f> &source, FloatType* target);
   static inline void ArrayToMatrix(FloatType* source, vector<Vector3f> &target);

	static inline void MatrixToArray(const Matrix3f &source, FloatType* target);
   static inline void ArrayToMatrix(FloatType* source, Matrix3f &target);

   static void computeSVD_NoMatV(const DynamicMatrix<FloatType> &input,
	   DynamicMatrix<FloatType> &U,
	   DynamicVector<FloatType> &sv);

   static void computeSVD_NoMatV(const Matrix3f &input,
	   Matrix3f &U,
	   Vector3f &sv);
};


}

#endif
