//---------------------------------------------------------------------------
#ifndef CameraControllerH
#define CameraControllerH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "system/IEditor.h"
#include "Camera.h"
#include "DependentObject.h"
#include "KeyboardController.h"
#include "MouseController.h"
#include "TimeDependentController.h"
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace geoxl {


class FOUNDATION_API CameraController : public Persistent, public IEditor, public KeyboardController, public MouseController, public TimeDependentController {
 private:
   static MetaClass *CameraControllerMetaClass;
   MonitoringObject *notifyOnChange;

 protected:
   Camera *camera;
   void cameraChange();

 public:
   CameraController();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void bindCamera(Camera *cam);
   virtual Object *getObject();
   void setNotifyOnCameraChange(MonitoringObject *notifyOnChange);
   MonitoringObject *getNotifyOnCameraChange();
   virtual void mouseWheelRotated(int32 rotatedDelta, Modifiers modifiersState){};

   /*** Member access ***/
   virtual void setTime( float32 currentTime );
   virtual void setup( const Vector3f& position, const Vector3f& ref, const Vector3f& up ) = 0;
   Camera* getCamera(){ return camera; };
};


}










#endif
