//---------------------------------------------------------------------------
#ifndef CameraPathRecorderH
#define CameraPathRecorderH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "CameraController.h"
//---------------------------------------------------------------------------

namespace geoxl {

class CameraPathRecorderEditorWidget;


class FOUNDATION_API CameraPathRecorder : public CameraController {
 private:
   static MetaClass *CameraPathRecorderMetaClass;
   static Object *newInstance();

   class CameraPos {
    public:
      Vector3f pos;
      Vector3f lookAt;
      Vector3f up;
   };

   enum RecorderMode {rmRecording, rmPlaying};

   class ChangesMonitor : public MonitoringObject {
    public:
      CameraPathRecorder *me;
      virtual void changed(Object* obj);
   };
   friend ChangesMonitor;

 public:
   enum Interpolation {interpLinear, interpHermite, interpRotX, interpRotY, interpRotZ};
 private:

   RecorderMode mode;
   vector<CameraPos> cameraPositions;
   ChangesMonitor notifyMe;
   CameraController *subController;
   float32 currentTime;
   float32 pathTime;
   Interpolation interpType;

   void setCameraByTime();

 protected:
   void setSubController(CameraController *camCtrl);
   CameraController *getSubController();

 public:
   CameraPathRecorder();

   // CameraController
   virtual void bindCamera(Camera *cam);

   // Persistent:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   // KeyboardController:
   virtual void keyDown(GeneralKey key);
   virtual void keyUp(GeneralKey key);

   // MouseController:
   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void mouseMoved(int32 x, int32 y);
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState);
   virtual void areaResize(card32 width, card32 height);
	virtual void mouseWheelRotated(int32 rotatedDelta, Modifiers modifiersState);

   // TimeDependentController
   virtual void setTime(float32 currentTime);
   virtual void setup( const Vector3f& position, const Vector3f& ref, const Vector3f& up );

   // Editor:
   virtual void update() {}
   virtual void apply();

   // Properties:
   float32 getPathTime();
   void setPathTime(float32 time);
   Interpolation getInterpolationType();
   void setInterpolationType(Interpolation interpType);

   // Commands:
   void gotoKeyFrame(card32 frameNum);
   void addKeyFrame(card32 frameNum);
   void removeKeyFrame(card32 frameNum);
   void setRecordFrames(bool rec);
   card32 getNumKeyFrames();

   ~CameraPathRecorder();
 friend CameraPathRecorderEditorWidget;
};

}

#endif
