//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "CommandHistory.h"
//---------------------------------------------------------------------------
#include "CommandObject.h"
#include "ObjectStreams.h"
#include "ObjectViewsTable.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* CommandHistory::CommandHistoryMetaClass = NULL;

const MetaClass* CommandHistory::getInstanceClass() const {
   return getClass();
}

Object* CommandHistory::newInstance() {
   return new CommandHistory;
}

const MetaClass* CommandHistory::getClass() {
   if (CommandHistoryMetaClass==NULL) {
      CommandHistoryMetaClass = new MetaClass("CommandHistory", "no description.",
                                  newInstance, sizeof(CommandHistory));
   }
   return CommandHistoryMetaClass;
}

void CommandHistory::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   CommandHistoryMetaClass->copyParentProperties();
	// mh - 13.03.2006 - moved custom editor init to InitFoundation
}

void CommandHistory::shutdown() {
   delete CommandHistoryMetaClass;
}


// ----


CommandHistory::CommandHistory() {
   writeCommands = true;
}

void CommandHistory::write(OutputObjectStream *s) const {
   s->writeBool(writeCommands);
   if (writeCommands) {
      const card32 ndo = (card32)doList.size();
      s->write(ndo);
      for (card32 i=0; i<ndo; i++) {
         s->writeObject(doList[i]);
      }
      const card32 nundo = (card32)undoList.size();
      s->write(nundo);
      for (card32 i=0; i<nundo; i++) {
         s->writeObject(undoList[i]);
      }
   }
}

void CommandHistory::read(InputObjectStream *s) {
   if (s->getVersion() > 120) {
      s->readBool(writeCommands);
      if (s->getClassVersion(getClass()) < MetaClass::FIRST_VERSION + 1) {
         Persistent *scene;
         s->readObject(scene);
      }
      if (writeCommands) {
         card32 ndo;
         s->read(ndo);
         doList.resize(ndo, NULL);
         for (card32 i=0; i<ndo; i++) {
            s->readObject((Persistent*&)doList[i]);
         }

         card32 nundo;
         s->read(nundo);
         undoList.resize(nundo, NULL);
         for (card32 i=0; i<nundo; i++) {
            s->readObject((Persistent*&)undoList[i]);
         }
      }
   }
}

void CommandHistory::executeCommand(Object *context, CommandObject *cObj, bool tryMerge) {
   cObj->doAction(context);
   if (tryMerge && doList.size() > 0) {
      if (doList.back()->mergeCommand(cObj)) {
         delete cObj;
      } else {
         doList.push_back(cObj);
      }

   } else {
      doList.push_back(cObj);
   }
   for (card32 i=0; i<undoList.size(); i++) {
      delete undoList[i];
   }
   undoList.clear();
   ObjectViewsTable::update(this);
}

void CommandHistory::undo(Object *context) {
   if (doList.size() == 0) throw ECannotUndo("CommandHistory::undo() - command history is empty.");
   doList.back()->undoAction(context);
   undoList.push_back(doList.back());
   doList.pop_back();
   ObjectViewsTable::update(this);
}
bool CommandHistory::canUndo() {
   if (doList.size() == 0) return false;
   if (!doList.back()->canUndo()) return false;
   return true;
}

void CommandHistory::redo(Object *context) {
   if (undoList.size() == 0) throw ECannotUndo("CommandHistory::undo() - no commands found to undo.");
   undoList.back()->doAction(context);
   doList.push_back(undoList.back());
   undoList.pop_back();
   ObjectViewsTable::update(this);
}
bool CommandHistory::canRedo() {
   if (undoList.size() == 0) return false;
   return true;
}

void CommandHistory::repeatLatest(Object *context) {
   doList.back()->doAction(context);
   doList.push_back((CommandObject*)(doList.back()->copy()));
   ObjectViewsTable::update(this);
}
bool CommandHistory::canRepeat() {
   return doList.size() > 0;
}

CommandHistory::~CommandHistory() {
   for (card32 i=0; i<doList.size(); i++) {
      delete doList[i];
   }
   for (card32 i=0; i<undoList.size(); i++) {
      delete undoList[i];
   }
}

