//---------------------------------------------------------------------------
#ifndef CommandHistoryH
#define CommandHistoryH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {


class CommandObject;
class CommandHistoryEditorWidget;

/// This class represents a list of actions that have been performed on an Object ("context").
/// The actions have been recorded an can (probably) be done/undone/repeated or the whole list of commands can be reexecuted. 
class FOUNDATION_API CommandHistory : public Persistent {

 private:
   static MetaClass *CommandHistoryMetaClass;
   static Object* newInstance();

   bool writeCommands;

   vector<CommandObject*> doList;
   vector<CommandObject*> undoList;

 public:
   CommandHistory();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   virtual void executeCommand(Object *context, CommandObject *cObj, bool tryMerge = true);

   virtual bool canUndo();
   virtual void undo(Object *context);

   virtual void redo(Object *context);
   virtual bool canRedo();

   virtual void repeatLatest(Object *context);
   virtual bool canRepeat();

   ~CommandHistory();

friend CommandHistoryEditorWidget;

};

}

#endif
