//---------------------------------------------------------------------------
#ifndef CubeMapH
#define CubeMapH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "RGBAImage.h"
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace geoxl {

template <class PixelType>
class FOUNDATION_API CubeMap : public Persistent {

 private:
   inline float32 scale(int32 v);

 protected:
   TemplatedImage<PixelType> *sides[6];

   static PixelType lookup(TemplatedImage<PixelType> *img, float32 x, float32 y);

 public:
   typedef card32 CubeMapSide;

   static const CubeMapSide CMS_POSITIVE_X = 0;
   static const CubeMapSide CMS_NEGATIVE_X = 1;
   static const CubeMapSide CMS_POSITIVE_Y = 2;
   static const CubeMapSide CMS_NEGATIVE_Y = 3;
   static const CubeMapSide CMS_POSITIVE_Z = 4;
   static const CubeMapSide CMS_NEGATIVE_Z = 5;

   CubeMap();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual void assignSide(card32 index, const TemplatedImage<PixelType> *img);
   TemplatedImage<PixelType> *getSide(CubeMapSide side) {return sides[side];}
   PixelType lookupCubeMap(const Vector3f &dir);
   Vector3f getPixelDirection(CubeMapSide index, int32 x, int32 y);
   virtual ~CubeMap();
};


class FOUNDATION_API RGBACubeMap : public CubeMap<Vector4ub> {

 private:
   static MetaClass *RGBACubeMapMetaClass;
   static Object* newInstance();

   void interactiveLoadCubeMapFromFile();

 public:
   RGBACubeMap();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assignSide(card32 index, const TemplatedImage<Vector4ub> *img);

   // "cross format: .. -y ..
   //                -x +z +x
   //                .. +y ..
   //                .. -z ..
   void loadFromCrossBMP(const char *filename);
   void loadFromAngularBMP(const char * filename );
   void setupTestMap();

   void prefilterDiffuse(card32 mapSize);
};

}

#endif
