//---------------------------------------------------------------------------
#ifndef ExpressionParserH
#define ExpressionParserH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <string>
#include <vector>
#include <stack>
#include <map>
//---------------------------------------------------------------------------
#include "Exceptions.h"
#include "SymbolTables.h"
//---------------------------------------------------------------------------


namespace geoxl {

/// Allows to parse and evaluate arithmetic expressions
class FOUNDATION_API ExpressionParser {
  private:
    NumTableHandler* symbolTables;
    std::stack<std::string> operatorStack;
    std::stack<ExpressionNum> operandStack;
    std::string operatorChars;
    std::map<std::string, unsigned> operatorPrecedence;
    typedef std::map<std::string, unsigned>::iterator PrecedenceIt;

    /// Returns true if the token given is a valid operator and false otherwise
    bool tokenIsOperator(const std::string& token) const;
    /// Gets the next token from the expression given
    void ExpressionParser::getNextToken(std::string& expression,
      bool& isOperator, std::string& token);
    /// Handles the operator given
    void handleOperator(std::string token);
    /// Reads the top operand on the operand stack, pops it and returns it
    ExpressionNum popTopOperand(std::string callingOperator);
    /// Handles the operator at the top of the operator stack
    bool handleTopOperator();

  public:
    ExpressionParser();
    /// Connects the symbol table handler given to the expression parser
    void setSymbolTableHandler(NumTableHandler* numTables)
      { symbolTables = numTables; }
    /// Evaluates the arithmetic expression and returns its result
    ExpressionNum evaluate(std::string expression);
};

class FOUNDATION_API EParserException : public PException {
 public:
   EParserException(const string msg) : PException(msg) {};
   virtual string getTypeString() {return "EParserException";}
};

class FOUNDATION_API EParserOperandException : public EParserException {
 public:
   EParserOperandException(const string msg) : EParserException(msg) {};
   virtual string getTypeString() {return "EParserOperandException";}
};


}

#endif
