//---------------------------------------------------------------------------
#ifndef FileBlockChainAllocatorH
#define FileBlockChainAllocatorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PTypes.h"
#include "FileBlockAllocator.h"
//---------------------------------------------------------------------------

namespace geoxl {


class FileBlockChainAllocator {
 private:
   FileBlockAllocator *baseAlloc;

   card64 allocatedSize;

   struct Header {
      /// number of blocks that the allocation consists of
      card64 numSegments;
      /// number of next block in chain
      card64 nextSegment;
      /// size that has been specified for allocation (without blocking overhead)
      card64 dataSize;
   };

   max_platform_card calcNumSegments(max_platform_card size);
   card64 getBlockInChain(card64 startBlock, card64 index, bool updateNumSegments, card64 newNumSegments, card64 newDataSize);
   void attachBlocksToChain(card64 lastOldChain, max_platform_card additionalBlocks, max_platform_card overallNumBlocks, card64 newDataSize);
   max_platform_card deleteBlocksIntern(card64 blockID);

   max_platform_card getMaxDataPerBlock() {return (max_platform_card)baseAlloc->getDataSizePerBlock() - sizeof(Header);}

   void debugDisplay();

 public:
   FileBlockChainAllocator(FileBlockAllocator *baseAlloc);

   card64 allocBlock(max_platform_card size);
   void reallocBlock(card64 blockID, max_platform_card newSize);
   void deleteBlock(card64 blockID);
   max_platform_card getMaxSize(card64 blockID);

   void writeBuffer(card64 blockID, const card8 *buffer, max_platform_card size);
   void readBuffer(card64 blockID, card8 *buffer, max_platform_card size);

   ~FileBlockChainAllocator();

   /// performs functionality test (debugging)
   static bool classTest();
};


}

#endif
 