//---------------------------------------------------------------------------
#ifndef FreeCameraControllerH
#define FreeCameraControllerH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "CameraController.h"
//---------------------------------------------------------------------------

namespace geoxl {

/// Implementation of a camera controller that allows free rotation in
/// each direction and resetting the reference point in each spacial direction
class FOUNDATION_API FreeCameraController : public CameraController {
	
	GEOXL_CLASS( FreeCameraController )
 
	private:
		/// Current mouse state (buttons) 
	   enum MouseState { neutral, dragDist, dragAngles };
	   MouseState state;
	   /// Last x coordinate of mouse
	   int32 lastX;
	   /// Last y coordinate of mouse
	   int32 lastY;
	   /// Rotation speed
	   float32 speed;

	   Vector3f eye, up, ref;

	protected:
	   /// Event: controller state has changed
	   void changed();
	   /// Compute and set the camera parameters
	   void calcCamera();
	   /// Rotate the camera around axis ( ref ^ up )
	   void rotateUp( float s );
	   /// Rotate the camera around axis up
	   void rotateSide( float s );

	public:
		/// Constructor
		FreeCameraController();

		// KeyboardController:
		virtual void keyDown(GeneralKey key);
		virtual void keyUp(GeneralKey key);

		// MouseController:
		virtual void mouseDown( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState );
		virtual void mouseMoved( int32 x, int32 y );
		virtual void mouseUp( int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState );
		virtual void areaResize( card32 width, card32 height );
		virtual void mouseWheelRotated(int32 rotatedDelta, Modifiers modifiersState);

		// Editor:
		virtual void update();
		virtual void apply();
		void setYRotationAngle( float32 alpha );
		inline Vector3f getPosition() const { return camera->getPosition(); }
		inline Vector3f getLookAt() const { return camera->getLookAt(); }
		inline Vector3f getUp() const { return camera->getUp(); }

		// Properties:
		void setToCamera( Camera* cam, Vector3f newCenter );
		void setup( const Vector3f& position, const Vector3f& ref, const Vector3f& up );
		/// Assign camera
		void bindCamera(Camera *cam);

		inline void setPosition( const Vector3f& p ){ 
			this->eye = p; camera->setPosition( p ); changed(); 
		}
		inline void setUp( const Vector3f& u ){ 
			this->up = u; camera->setUp( u ); changed(); 
		}
		inline void setLookAt( const Vector3f& l ){ 
			this->ref = l; camera->setLookAt( l ); changed(); 
		}
};

}

#endif
