//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "MemoryManager.h"
//---------------------------------------------------------------------------
#include "PAssert.h"
#include "AttachedData.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MemoryManager::MemoryManager( card32 maxMemory )
{
   m_UsedMemory = 0;
   m_MaxMemory = maxMemory;
}

MemoryManager::~MemoryManager()
{
   while( !m_Objects.empty() )
      delete  *m_Objects.begin();
}

void MemoryManager::setMaxMemory( card32 maxMemory )
{
   free( maxMemory );
   m_MaxMemory = maxMemory;
}

void MemoryManager::free( card32 maxUsedMemory )
{
   while( m_UsedMemory > maxUsedMemory )
      if( m_Objects.size() )
         delete (*m_Objects.begin());
}

void MemoryManager::add( MemoryManagedObject * obj )
{
   pAssert( obj->m_MyManager == NULL );

   pAssert( m_MaxMemory > obj->getSize() );

   free( m_MaxMemory - obj->getSize() );

   m_UsedMemory += obj->getSize();

   m_Objects.push_back( obj );
   obj->m_MyManager = this;
   obj->m_MyIterator = m_Objects.end();
   obj->m_MyIterator--;
}

void MemoryManager::remove( MemoryManagedObject * obj )
{
   m_UsedMemory -= obj->getSize();
   m_Objects.erase( obj->m_MyIterator );
}

void MemoryManager::touched( MemoryManagedObject * obj )
{
   m_Objects.erase( obj->m_MyIterator );
   m_Objects.push_back( obj );
   obj->m_MyIterator = m_Objects.end();
   obj->m_MyIterator--;
}

void MemoryManager::resized( MemoryManagedObject * obj, card32 oldSize, card32 newSize )
{
   pAssert( m_UsedMemory >= oldSize );
   m_UsedMemory -= oldSize;
   m_UsedMemory += newSize;
   touched( obj );
}

MemoryManagedObject* MemoryManager::createNewMMObject(const AttachedData* data){
   MemoryManagedObject* mmo=new MemoryManagedObject(((AttachedData*)data)->getSize(), this);
   return mmo;
}
                                                         

MemoryManagedObject::MemoryManagedObject( card32 size, MemoryManager * manager )
{
   m_MyManager = NULL;
   m_MySize = size;
   if( manager )
      manager->add(this);
   m_MyOwner = NULL;
}

MemoryManagedObject::~MemoryManagedObject()
{
   if( m_MyOwner )
      m_MyOwner->onDeleteMMO(this);
   if( m_MyManager )
      m_MyManager->remove( this );
}

void MemoryManagedObject::touched()
{
   if( m_MyManager )
      m_MyManager->touched( this );
}

void MemoryManagedObject::setSize(card32 size)
{
   card32 oldSize = m_MySize;
   m_MySize = size;
   if( m_MyManager )
      m_MyManager->resized( this, oldSize, size );
}

// ----------------------------------------------

/*MemoryManagedParameter::MemoryManagedParameter( card32 size, MemoryManager * manager, AttachedData * data )
:MemoryManagedObject(size,manager)
{
   m_AttachedData = data;
}

MemoryManagedParameter::~MemoryManagedParameter()
{

}*/


