//---------------------------------------------------------------------------
#ifndef MemoryManagerH
#define MemoryManagerH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <list>
#include "PTypes.h"
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;

class AttachedData;
class MemoryManagedObject;


class FOUNDATION_API MemoryManager
{
private:
   friend MemoryManagedObject;

   virtual void add( MemoryManagedObject * obj );
   virtual void remove( MemoryManagedObject * obj );
   virtual void touched( MemoryManagedObject * obj );
   virtual void resized( MemoryManagedObject * obj, card32 oldSize, card32 newSize );

   list<MemoryManagedObject*>  m_Objects;
   card32               m_MaxMemory;
   card32               m_UsedMemory;

public:


   MemoryManager( card32 maxMemory );
   virtual ~MemoryManager();

   void setMaxMemory( card32 maxMemory );
   inline card32 getMaxMemory() const {return m_MaxMemory;}
   inline card32 getUsedMemory() const {return m_UsedMemory;}

   virtual MemoryManagedObject* createNewMMObject(const AttachedData*);

   virtual void free( card32 maxUsedMemory );

protected:



};

class MMOOwner;

class FOUNDATION_API MemoryManagedObject
{
   friend MemoryManager;

private:
   MemoryManager *   m_MyManager;
   list<MemoryManagedObject*>::iterator  m_MyIterator;
   card32            m_MySize;
   MMOOwner       *  m_MyOwner;

public:

   MemoryManagedObject( card32 size, MemoryManager * manager);
   virtual ~MemoryManagedObject();

   void setOwner( MMOOwner * owner ) {m_MyOwner = owner;}
   MMOOwner * getOwner() const {return m_MyOwner;}

   void touched();
   inline card32 getSize() const {return m_MySize;};
   void setSize( card32 size);

};

/// interfaceclass
class FOUNDATION_API MMOOwner
{
public:
   virtual void onDeleteMMO(MemoryManagedObject*) = 0;
};


/*class FOUNDATION_API MemoryManagedParameter : public MemoryManagedObject
{
private:
   AttachedData   *    m_AttachedData;
public:

   MemoryManagedParameter( card32 size, MemoryManager * manager, AttachedData * data );
   virtual ~MemoryManagedParameter();

};*/



}
#endif
