#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "MemoryMultiObjectStream.h"
//---------------------------------------------------------------------------
#include "MemoryObjectStreams.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------


#ifdef _DEBUG
//#define HEFTY_DEBUG
#endif

IMPLEMENT_GEOXL_CLASS( MemoryMultiObjectStream ,0)
{
	BEGIN_CLASS_INIT( MemoryMultiObjectStream );
}

MemoryMultiObjectStream::MemoryMultiObjectStream(void)
{
	currentID = 1;
}

void MemoryMultiObjectStream::sync() {}

MemoryMultiObjectStream::ObjectID MemoryMultiObjectStream::createObjectEntry() {
	storage[currentID] = NULL;
	return currentID++;
}

void MemoryMultiObjectStream::updateObject(ObjectID id, const MultiPartPersistent *obj) {
	#ifdef HEFTY_DEBUG
	_CrtCheckMemory();
	#endif
	Iterator pos = storage.find(id);
	if (pos == storage.end()) {
		throw PException("MemoryMultiObjectStream::updateObject() - id not found.");
	}
	delete pos->second;
	MemoryOutputObjectStream s;
   const card32 n = (card32)globalObjectPointers.size();
   for (card32 i=0; i<n; i++) {
      s.addGlobalObject(globalObjectPointers[i].obj, globalObjectPointers[i].index);
   }
	s.writeObject(obj);
	BinaryContainer *buf = new BinaryContainer;
	*buf = *(s.closeAndGetMemoryBuffer());
	pos->second = buf;
	#ifdef HEFTY_DEBUG
	_CrtCheckMemory();
	#endif
}

MultiPartPersistent *MemoryMultiObjectStream::getObject(ObjectID id) {
	#ifdef HEFTY_DEBUG
	_CrtCheckMemory();
	#endif
	Iterator pos = storage.find(id);
	if (pos == storage.end()) {
		throw PException("MemoryMultiObjectStream::updateObject() - id not found.");
	}

	MemoryInputObjectStream s(&((*(pos->second))[0]), pos->second->size());
   const card32 n = (card32)globalObjectPointers.size();
   for (card32 i=0; i<n; i++) {
      s.addGlobalObject(globalObjectPointers[i].obj, globalObjectPointers[i].index);
   }
	Persistent *result;
	s.readObject(result);
	#ifdef HEFTY_DEBUG
	_CrtCheckMemory();
	#endif
	return (MultiPartPersistent*)result;
}

void MemoryMultiObjectStream::deleteObject(ObjectID id) {
	#ifdef HEFTY_DEBUG
	_CrtCheckMemory();
	#endif
	Iterator pos = storage.find(id);
	if (pos == storage.end()) {
		throw PException("MemoryMultiObjectStream::updateObject() - id not found.");
	}
	delete pos->second;
	pos->second = NULL;
	storage.erase(pos);	
	#ifdef HEFTY_DEBUG
	_CrtCheckMemory();
	#endif
}

MemoryMultiObjectStream::Status MemoryMultiObjectStream::getStatus() {
	return statusConnected;
}

MemoryMultiObjectStream::~MemoryMultiObjectStream(void) {
	Iterator pos = storage.begin();
	Iterator end = storage.end();
	while (pos != end) {
		delete pos->second;
		++pos;
	}							 
}