//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "MemoryObjectStreams.h"
//---------------------------------------------------------------------------
#include "PAssert.h"
#include "StreamClassTable.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MemoryOutputObjectStream::MemoryOutputObjectStream(bool perObjectClassVersion, StreamClassTable *classTable, bool doWriteClassTable)
: OutputObjectStream(perObjectClassVersion, classTable, doWriteClassTable)
{
   closed = false;
   currentPos = 0;
   initStream();
}

void MemoryOutputObjectStream::writeClassTableHeader() {
   pAssert(!closed);
   classTablePointerPos = currentPos;
   //write<max_platform_card>(0); // bug! writes card32
   write<card32>(0); // quick fix - does not fully use x64
}


void MemoryOutputObjectStream::writeClassTable() {
   pAssert(!closed);
   max_platform_card classTablePos = currentPos;
   classTable->write(this);
   max_platform_card endPos = currentPos;
   currentPos = classTablePointerPos;
//   write<max_platform_card>(classTablePos); // bug! writes card32
   write<card32>(classTablePos); // quick fix - does fully use x64
   currentPos = endPos;
}

void MemoryOutputObjectStream::checkExpand(card32 length) {
   if (mem.size() == 0) {
      mem.resize(length);
   } else if (mem.size() < currentPos+length) {
      max_platform_card newSize = currentPos+length;
      max_platform_card doubleSize = (max_platform_card)mem.size() * 2;
      if (newSize < doubleSize) newSize = doubleSize;
      mem.resize(newSize);
   }
}

void MemoryOutputObjectStream::writeBuffer(const void* buf, mpcard length) {
   pAssert(!closed);
   checkExpand(length);
   for (max_platform_card i=0; i<length; i++) {
      mem[i+currentPos] = ((card8*)buf)[i];
   }
   currentPos += length;
}


const vector<card8> *MemoryOutputObjectStream::closeAndGetMemoryBuffer() {
   pAssert(!closed);
   writeClassTable();
   closed = true;
   mem.resize(currentPos);
   return &mem;
}


MemoryOutputObjectStream::~MemoryOutputObjectStream() {}




MemoryInputObjectStream::MemoryInputObjectStream(const card8 *mem, max_platform_card memSize, StreamClassTable *streamClassTable)
: InputObjectStream(streamClassTable)
{
   this->mem = mem;
   this->memSize = memSize;
   currentPos = 0;
   initStream();
}


void MemoryInputObjectStream::readClassTable() {
   max_platform_card classTablePos;
//   read<max_platform_card>(classTablePos);  // bug! reads card32 
   card32 fix;
   read<card32>(fix);  // quick-hack for x64
   classTablePos = fix;

   max_platform_card savedPos = currentPos;

   currentPos = classTablePos;

   classTable->read(this);

   currentPos = savedPos;
}


void MemoryInputObjectStream::readBuffer(void* buf, mpcard length) {
   if (length==0) return;
   pAssert(currentPos+length <= memSize);
   for (max_platform_card i=0; i<length; i++) {
      ((card8*)buf)[i] = ((card8*)mem)[i+currentPos];
   }
   currentPos += length;
}


MemoryInputObjectStream::~MemoryInputObjectStream() {}


