//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "MultiFileMultiObjectStreamMT.h"
#include "NumericalClassProperty.h"
#include "StringClassProperty.h"
#include "BooleanClassProperty.h"
#include "MemoryObjectStreams.h"
#include "EventCounterDisplay.h"
#include "DebugOutput.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


IMPLEMENT_GEOXL_CLASS( MultiFileMultiObjectStreamMT , 1 )
{
	BEGIN_CLASS_INIT( MultiFileMultiObjectStreamMT);
   ADD_STRING_PROP(baseFileName, 0)
   ADD_CARD32_PROP(maximumQueueLength, 0)
   ADD_BOOLEAN_PROP(relativePath, 1)
	MultiFileMultiObjectStreamMTMetaClass->getGUIDescriptor()->addClassCreator(MultiFileMultiObjectStreamMTCreator::getClass());
}

// -

IMPLEMENT_GEOXL_CLASS( MultiFileMultiObjectStreamMTCreator, 1 )
{
	BEGIN_CLASS_INIT( MultiFileMultiObjectStreamMTCreator );
	ADD_STRING_PROP(baseFileName, 0)
   ADD_CARD32_PROP(maximumQueueLength, 0)
   ADD_BOOLEAN_PROP(relativePath, 1)   
}

// --


void MFMTLoadStoreThread::checkBufferSize() {
	while (fetchBuffer.size() > maximumQueueLength) {
		if (fetchBuffer.back().second != NULL)	{
			fetchBuffer.back().second->shallowDelete();
		}
		fetchBuffer.pop_back();
	}
}

void MFMTLoadStoreThread::queueStatistics() {
   EventCounterDisplay::NewValue v;
   v.value = commands.getNumAvailableMessageSlots();
   debugOutput.displayEvent(EventCounterDisplay::getClass(), string("ls thread queue length"), &v);
}


void MFMTLoadStoreThread::run() {
	bool stopped = false;
	do {
		try {
			ThreadCommandQueue::Message *msg = commands.getMessage();
			if (dynamic_cast<StopCommand*>(msg)) {
				try {
					// remove old data
					FetchBufferType::iterator pos = fetchBuffer.begin();
					FetchBufferType::iterator end = fetchBuffer.end();
					while (pos != end) {
						if (pos->second != NULL) {
							pos->second->shallowDelete();
						}
						++pos;
					}
					fetchBuffer.clear();
				} catch (...) {}
				
				stopped = true;
				commands.answerMessage();
				
			} else if (dynamic_cast<AddUpdateJobCommand*>(msg)) {
				
				AddUpdateJobCommand *cmd = dynamic_cast<AddUpdateJobCommand*>(msg);
				multiObj->updateObject(cmd->id, *(cmd->serializedData));

				// remove old data
				FetchBufferType::iterator pos = fetchBuffer.begin();
				FetchBufferType::iterator end = fetchBuffer.end();
				while (pos != end) {
					if (pos->first == cmd->id) {
						if (pos->second != NULL) {
							pos->second->shallowDelete();
						}
						fetchBuffer.erase(pos);
						break;
					}
					++pos;
				}

				delete cmd->serializedData;
				delete cmd; // async
			
			} else if (dynamic_cast<FetchJobCommand*>(msg)) {
			
				FetchJobCommand *cmd = dynamic_cast<FetchJobCommand*>(msg);

				bool needFetch = true;
				FetchBufferType::iterator pos = fetchBuffer.begin();
				FetchBufferType::iterator end = fetchBuffer.end();
				while (pos != end) {
					if (pos->first == cmd->id) {
						fetchBuffer.push_front(*pos);
						fetchBuffer.erase(pos);
						needFetch = false;
						break;
					}
					++pos;
				}

				if (needFetch) {
					pair<ObjectID, MultiPartPersistent*> entry;
					entry.first = cmd->id;
					entry.second = multiObj->getObject(cmd->id);
					fetchBuffer.push_front(entry);
					checkBufferSize();
				}
				
				delete cmd;	
					
			}  else if (dynamic_cast<GetObjectCommand*>(msg)) {
			
				
				GetObjectCommand *cmd = dynamic_cast<GetObjectCommand*>(msg);
				cmd->error = false;
				try {
					ObjectID id = cmd->id;
					cmd->obj_answer = NULL;
					bool needFetch = true;
					FetchBufferType::iterator pos = fetchBuffer.begin();
					FetchBufferType::iterator end = fetchBuffer.end();
					while (pos != end) {
						if (pos->first == id) {
							cmd->obj_answer = pos->second;
							fetchBuffer.erase(pos);	// avoid handing out the same reference twice
							needFetch = false;
							break;
						}
						++pos;
					}
				
					if (needFetch) {
						cmd->obj_answer = multiObj->getObject(id);
					}
				} catch (...) {
					cmd->error = true;
				}
				commands.answerMessage();
			} else if (dynamic_cast<CreateObjectEntryCommand*>(msg)) {
				CreateObjectEntryCommand *cmd = dynamic_cast<CreateObjectEntryCommand*>(msg);
				cmd->error = false;			
				try {
					cmd->id_answer = multiObj->createObjectEntry();
				} catch (...) {
					cmd->error = true;
				}
				commands.answerMessage();
			} else if (dynamic_cast<QueryStatusCommand*>(msg)) {
				QueryStatusCommand *cmd = dynamic_cast<QueryStatusCommand*>(msg);
				cmd->error = false;			
				try {
					cmd->status_answer = multiObj->getStatus();
				} catch (...) {
					cmd->error = true;
				}
				commands.answerMessage();
			} else if (dynamic_cast<SyncCommand*>(msg)) {
				multiObj->sync();
				delete msg;
			}  else if (dynamic_cast<AddGlobalObjectPointerCommand*>(msg)) {
				AddGlobalObjectPointerCommand *cmd = dynamic_cast<AddGlobalObjectPointerCommand*>(msg);
				multiObj->addGlobalObjectPointer(cmd->obj, cmd->index);
				delete cmd;
			}  else if (dynamic_cast<DeleteObjectCommand*>(msg)) {
				DeleteObjectCommand *cmd = dynamic_cast<DeleteObjectCommand*>(msg);
				multiObj->deleteObject(cmd->id);
				delete cmd;
			}  else if (dynamic_cast<QueryObjectAvailabilityCommand*>(msg)) {
				QueryObjectAvailabilityCommand *cmd = dynamic_cast<QueryObjectAvailabilityCommand*>(msg);
				cmd->error = false;			
				try {
					cmd->available_answer = false;
					FetchBufferType::iterator pos = fetchBuffer.begin();
					FetchBufferType::iterator end = fetchBuffer.end();
					while (pos != end) {
						if (pos->first == cmd->id) {
							cmd->available_answer = true;
							break;
						}
						++pos;
					}
				} catch (...) {
					cmd->error = true;
				}
				commands.answerMessage();
			}
		} catch (...) {}
	} while (!stopped);
}

MFMTLoadStoreThread::MFMTLoadStoreThread(string baseFileName, card32 maximumQueueLength, GloballyUniqueID initialGUID)
 : commands(maximumQueueLength) 
{
	multiObj = new MultiFileMultiObjectStream(baseFileName, false, &initialGUID);
	this->maximumQueueLength = maximumQueueLength;
}

void MFMTLoadStoreThread::addUpdateJob(ObjectID id, BinaryContainer *serializedData) {
	if (id == NULL) throw PException("MFMTLoadStoreThread::addUpdateJob() - id must not be NULL.");
	AddUpdateJobCommand *msg = new AddUpdateJobCommand;
	msg->id = id;
	msg->serializedData = serializedData;
	queueStatistics();
	commands.sendAsyncMessage(msg);
}

void MFMTLoadStoreThread::addFetchJob(ObjectID id) {
	if (id == 0) return;
	FetchJobCommand *msg = new FetchJobCommand();
	msg->id = id;
	queueStatistics();
	commands.sendAsyncMessage(msg);
}

MultiPartPersistent *MFMTLoadStoreThread::getObject(ObjectID id) {
	if (id == NULL) throw PException("MFMTLoadStoreThread::getObject() - id must not be NULL.");
	GetObjectCommand msg;
	msg.id = id;
	queueStatistics();
	commands.sendMessageAndReceiveReply(&msg);
	if (msg.error) throw PException("MFMTLoadStoreThread::getObject() - Error in allocator thread.");
	return msg.obj_answer;
}

MFMTLoadStoreThread::ObjectID MFMTLoadStoreThread::createObjectEntry() {
	CreateObjectEntryCommand msg;
	queueStatistics();
	commands.sendMessageAndReceiveReply(&msg, true);
	if (msg.error) throw PException("MFMTLoadStoreThread::getObject() - Error in allocator thread.");
	return msg.id_answer;
}

FileBasedMultiObjectStream::Status MFMTLoadStoreThread::getStatus() {
	QueryStatusCommand msg;
	queueStatistics();
	commands.sendMessageAndReceiveReply(&msg, true);
	if (msg.error) throw PException("MFMTLoadStoreThread::getObject() - Error in allocator thread.");
	return msg.status_answer;
}

void MFMTLoadStoreThread::sync() {
	SyncCommand *cmd = new SyncCommand;
	queueStatistics();
	commands.sendAsyncMessage(cmd);
}

void MFMTLoadStoreThread::deleteObject(ObjectID id) {
	if (id == 0) return;
	DeleteObjectCommand *cmd = new DeleteObjectCommand();
	cmd->id = id;
	queueStatistics();
	commands.sendAsyncMessage(cmd);
}

bool MFMTLoadStoreThread::objectAvailable(ObjectID id) {
	if (id == 0) return true;
	QueryObjectAvailabilityCommand cmd;
	cmd.id = id;
	queueStatistics();
	commands.sendMessageAndReceiveReply(&cmd, true);
	if (cmd.error) throw PException("MFMTLoadStoreThread::getObject() - Error in allocator thread.");
	return cmd.available_answer;
}

void MFMTLoadStoreThread::addGlobalObjectPointer(Persistent *obj, card32 index) {
	AddGlobalObjectPointerCommand *msg = new AddGlobalObjectPointerCommand();
	msg->obj = obj;
	msg->index = index;
	queueStatistics();
	commands.sendAsyncMessage(msg);
}



MFMTLoadStoreThread::~MFMTLoadStoreThread() {
	StopCommand msg;
	queueStatistics();
	commands.sendMessageAndReceiveReply(&msg);
	wait();
	queueStatistics();
	delete multiObj;	
}


// ---



MultiFileMultiObjectStreamMT::MultiFileMultiObjectStreamMT(string baseFileName, card32 maximumQueueLength, bool relativePath) {
	globalID.createNewID();
	this->baseFileName = baseFileName;
	this->maximumQueueLength = maximumQueueLength;	
	this->relativePath = relativePath;
	loadStoreThread = NULL;
}

MultiObjectStream *MultiFileMultiObjectStreamMT::createOOCAwareCopy(string idPrefix, bool insertOriginal, string idSuffix) {
	string newFilename = removeFilenameFromPath(baseFileName) + idPrefix + (insertOriginal?removePathFromFilename(baseFileName):string("")) + idSuffix;
	return new MultiFileMultiObjectStreamMT(newFilename, maximumQueueLength, relativePath);
}

void MultiFileMultiObjectStreamMT::write(OutputObjectStream *s) const {
	string saveFileName = baseFileName;
	bool matches = false;
	if (relativePath) {
		MultiFileMultiObjectStream::truncatePathRelativeTo(
			allDirSeperatorsToDos(removeFilenameFromPath(s->getFileName())), 
			allDirSeperatorsToDos(baseFileName), 
			const_cast<MultiFileMultiObjectStreamMT*>(this)->baseFileName,
			matches
		);
		#ifndef WIN32
		this needs to be fixed for unix (dos directory separators)
		#endif
	}

	FileBasedMultiObjectStream::write(s);
	s->write(globalID);

	s->writeBool(matches);
	const_cast<MultiFileMultiObjectStreamMT*>(this)->baseFileName = saveFileName;	
}



void MultiFileMultiObjectStreamMT::read(InputObjectStream *s) {
	FileBasedMultiObjectStream::read(s);
	s->read(globalID);

	bool matches = false;
	if (s->getClassVersion(MultiFileMultiObjectStreamMT::getClass()) >= MetaClass::FIRST_VERSION + 1) {
		s->readBool(matches);
	}
	if (relativePath && matches) {
		baseFileName = allDirSeperatorsToDos(removeFilenameFromPath(s->getFileName()) + baseFileName);
		#ifndef WIN32
		this needs to be fixed for unix (dos directory separators)
		#endif
	}
}


void MultiFileMultiObjectStreamMT::createThread() {
	if (loadStoreThread == NULL) {
		loadStoreThread = new MFMTLoadStoreThread(baseFileName, maximumQueueLength, globalID);
		QThread *mainThread = QThread::currentThread();
		pAssert(mainThread);
		QThread::Priority prio = mainThread->priority();
		switch (prio) {
			case QThread::IdlePriority    	  : prio =	QThread::IdlePriority;  break;
			case QThread::LowestPriority  	  : prio =	QThread::IdlePriority;  break;
			case QThread::LowPriority	   	  : prio =	QThread::LowestPriority;  break;
			case QThread::NormalPriority  	  : prio =	QThread::LowPriority;  break;
			case QThread::HighPriority	   	  : prio =	QThread::LowPriority;  break;
			case QThread::HighestPriority		  : prio =	QThread::LowPriority;  break;
			case QThread::TimeCriticalPriority : prio =	QThread::LowPriority;  break;
			case QThread::InheritPriority		  : prio =	QThread::LowPriority;  break;
			default: pAssert(false); break;
		}																
		loadStoreThread->start(prio);
		const card32 n = (card32)globalObjectPointers.size();
		for (card32 i=0; i<n; i++) {
			loadStoreThread->addGlobalObjectPointer(globalObjectPointers[i].obj, globalObjectPointers[i].index);
		}
	}
}


MultiFileMultiObjectStreamMT::~MultiFileMultiObjectStreamMT() {
	delete loadStoreThread;
}

void MultiFileMultiObjectStreamMT::addGlobalObjectPointer(Persistent *obj, card32 index) {
	if (loadStoreThread == NULL) createThread();
	FileBasedMultiObjectStream::addGlobalObjectPointer(obj, index);
	loadStoreThread->addGlobalObjectPointer(obj, index);
}


MultiFileMultiObjectStreamMT::ObjectID MultiFileMultiObjectStreamMT::createObjectEntry() {
	if (loadStoreThread == NULL) createThread();
	return loadStoreThread->createObjectEntry();
}

void MultiFileMultiObjectStreamMT::updateObject(ObjectID id, const MultiPartPersistent *obj) {
	if (loadStoreThread == NULL) createThread();
	MemoryOutputObjectStream s;
   const card32 n = (card32)globalObjectPointers.size();
   for (card32 i=0; i<n; i++) {
      s.addGlobalObject(globalObjectPointers[i].obj, globalObjectPointers[i].index);
   }
	s.writeObject(obj);
	BinaryContainer *buf = new BinaryContainer;
	*buf = *(s.closeAndGetMemoryBuffer());
	loadStoreThread->addUpdateJob(id, buf);
}

MultiPartPersistent *MultiFileMultiObjectStreamMT::getObject(ObjectID id) {
	if (loadStoreThread == NULL) createThread();
	return loadStoreThread->getObject(id);
}

void MultiFileMultiObjectStreamMT::deleteObject(ObjectID id) {
	if (loadStoreThread == NULL) createThread();
	loadStoreThread->deleteObject(id);
}

void MultiFileMultiObjectStreamMT::sync() {
	if (loadStoreThread == NULL) createThread();
	loadStoreThread->sync();
}

MultiFileMultiObjectStreamMT::Status MultiFileMultiObjectStreamMT::getStatus() {
	if (loadStoreThread == NULL) createThread();
	return loadStoreThread->getStatus();
}

bool MultiFileMultiObjectStreamMT::objectAvailable(ObjectID id) {
	if (loadStoreThread == NULL) createThread();
	return loadStoreThread->objectAvailable(id);
}

void MultiFileMultiObjectStreamMT::prefetchObject(ObjectID id) {
	if (loadStoreThread == NULL) createThread();
	loadStoreThread->addFetchJob(id);
}

string MultiFileMultiObjectStreamMT::getFilename() const {
	return baseFileName + string("_index.mfSwap");
}



MultiFileMultiObjectStreamMTCreator::MultiFileMultiObjectStreamMTCreator() {
	baseFileName = "C:\\temp\\test";
	maximumQueueLength = 50;
	relativePath = true;
}
Object *MultiFileMultiObjectStreamMTCreator::makeObject() {
	return new MultiFileMultiObjectStreamMT(baseFileName, maximumQueueLength, relativePath);
}

 


#include "MultiFileMultiObjectStreamMT_moc.cpp"