#ifndef PCA_H
#define PCA_H

// Xrt
#include "LinearAlgebra.h"

// System
#include <vector>

namespace geoxl {

template <class FloatType, unsigned Dim>
class PCA
{
   private:
      typedef StaticVector<FloatType,Dim> Point;
      typedef StaticMatrix<FloatType,Dim,Dim> Matrix;

      Matrix C;					/// Covariance matrix      
      Point centroid;			// Centroid
	  vector<Point> points;
	  vector<FloatType> weights;

   public:
      /// Constructor
      PCA();
      /// Reinitialize PCA
      void clear();
      /// Add new point
      void addPoint( Point p );
		/// Add new weighted point (don't mix with unweighted addPoint()!)
		void addPoint( Point p, FloatType weight);
      /// Solve eigen system
      void analyze( Point& eigenValues, Matrix& eigenVectors, Point& centroid );
};

typedef PCA<float32,2> PCA2f;
typedef PCA<float32,3> PCA3f;

}

#include "PCA_templCode.h"


#endif
