//---------------------------------------------------------------------------
#ifndef PriorityQueueH
#define PriorityQueueH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <set>
#include <vector>
#include "pAssert.h"
//---------------------------------------------------------------------------

namespace geoxl
{
   using namespace std;


/**
   priority queue with remove/insert methods
   KeyType and DataType have to implement a < operator
   hint: add a unique id to dataType and implement the < operator
   with that unique id to prevent collisions
*/
template<class KeyType,class DataType> class PriorityQueue
{
private:

   struct TElement
   {
      KeyType  key;
      vector<DataType> data;
      bool operator < ( const TElement &other ) const
      {
         return( key < other.key );
      }

   };

   set<TElement>  m_Data;
   typedef typename set<TElement>::iterator SetIterator;

public:

   void insert( KeyType key, DataType data );
   void remove( KeyType key, DataType data );

   ///
   DataType pop_first();
   DataType pop_last();

	KeyType getFirstKey();

	void clear();
	bool isEmpty();

};

//#pragma option push -w-8092

template<class KeyType,class DataType>
void PriorityQueue<KeyType,DataType>::insert( KeyType key, DataType data )
{
   TElement element;
   element.key = key;
   element.data.push_back( data );
	SetIterator iter = m_Data.find( element );
	if( iter == m_Data.end() )
		m_Data.insert( element );
	else
		static_cast<vector<DataType> >(iter->data).push_back( data );
}

template<class KeyType,class DataType>
void PriorityQueue<KeyType,DataType>::remove( KeyType key, DataType data )
{
   TElement element;
   element.key = key;
   element.data.push_back(data);
   SetIterator iter = m_Data.find( element );
   pAssert( iter != m_Data.end() );
	if( iter->data.size() == 1 )
		m_Data.erase( iter );
	else
	{
		for( card32 i=0;i<iter->data.size();i++ )
		{
			if( iter->data[i] == data )
			{
				static_cast< vector<DataType> > (iter->data).erase(iter->data.begin() + i);
				break;
			}
		}
	}
}

//#pragma option pop

template<class KeyType,class DataType>
DataType PriorityQueue<KeyType,DataType>::pop_first()
{
   pAssert( !m_Data.empty() );
   TElement ret = *m_Data.begin();
	if( ret.data.size() == 1 )
		m_Data.erase( m_Data.begin() );
	else
		static_cast< vector<DataType> > (m_Data.begin()->data).erase(static_cast< vector<DataType> > (m_Data.begin()->data).begin());
   return ret.data[0];
}

template<class KeyType,class DataType>
DataType PriorityQueue<KeyType,DataType>::pop_last()
{
   pAssert( !m_Data.empty() );
   SetIterator last = m_Data.end();
   last--;
   TElement ret = *last;
	if( ret.data.size() == 1 )
		m_Data.erase( last );
	else
		last->data.erase(last->data.begin());
   return ret.data[0];
}

template<class KeyType,class DataType>
KeyType PriorityQueue<KeyType,DataType>::getFirstKey()
{
   pAssert( !m_Data.empty() );
   TElement ret = *m_Data.begin();
	return ret.key;
}

template<class KeyType,class DataType>
void PriorityQueue<KeyType,DataType>::clear()
{
	m_Data.clear();
}

template<class KeyType,class DataType>
bool PriorityQueue<KeyType,DataType>::isEmpty()
{
	return m_Data.empty();
}
}
#endif
