//---------------------------------------------------------------------------
#ifndef RGBAImageH
#define RGBAImageH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "LinearAlgebra.h"
#include "TemplatedImage.h"
#include <QtGui/QImage>
//---------------------------------------------------------------------------

namespace geoxl {


class RGBImage;


class FOUNDATION_API RGBAImage : public TemplatedImage<Vector4ub> {
 private:
   static MetaClass *RGBAImageMetaClass;
   static Object *newInstance();

   inline bool checkBorder(card32 x, card32 y, Vector4ub &avCol);

   void interactiveLoadFromFile();

 public:
   RGBAImage() {}
   RGBAImage(card32 sizeX, card32 sizeY) : TemplatedImage<Vector4ub>(sizeX, sizeY) {}

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   // mark all pixels with color cMask as transparent, alpha component == index 3
   void chromaMask(Vector4ub cMask);
   // set color of pixels at the border to neighboring color values
   void maskBorder();
   // zoom down image with anti-aliasing, ignoring alpha == 0 entries
   void downsampleWBorder(card32 factor, RGBAImage *dest, Vector4ub maskColor);
   // zoom down with anti-aliasing, no chroma keying
   void downsampleAA(card32 factor, RGBAImage *dest);
   // drop alpha channel (color index 3)
   RGBImage *convertToRGB();
   // change from rgba to argb format (in place)
   void convertToBGRA();

   // mb - converts image to normal + height map
   // RGB = normal, A = height
   // height = (R + G + B) / 3
   void convertToNormalMap();

   void loadGLTexture();
   void loadGLTextureMipmapped();
   void loadGLSubTexture(card32 x, card32 y);
   void readPixelsGL(card32 x=0, card32 y=0);
   void drawPixelsGL(float32 x=0.0f, float32 y=0.0f);
   void loadFromFile(const char* bmpFile);
   void copyFromBMP(QImage *img, card32 x, card32 y, card32 width, card32 height);
   RGBAImage* flipY();
};

class FOUNDATION_API RGBImage : public TemplatedImage<Vector3ub> {
 private:
   static MetaClass *RGBImageMetaClass;
   static Object *newInstance();

 public:
   RGBImage() {}
   RGBImage(card32 sizeX, card32 sizeY) : TemplatedImage<Vector3ub>(sizeX, sizeY) {}

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   // change from rgb to bgr format (in place)
   void convertToBGR();

   void loadGLTexture();
   void loadGLTextureMipmapped();
   void loadGLSubTexture(card32 x, card32 y);
};

class FOUNDATION_API OneByteImage : public TemplatedImage<card8> {
 private:
   static MetaClass *OneByteImageMetaClass;
   static Object *newInstance();

 public:
   OneByteImage() {}
   OneByteImage(card32 sizeX, card32 sizeY) : TemplatedImage<card8>(sizeX, sizeY) {}

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   void loadGLTexture();
   void loadGLTextureMipmapped();
   void loadGLSubTexture(card32 x, card32 y);
   void readPixelsFromDepthGL(card32 x=0, card32 y=0);
   void drawPixelsGL(float32 x=0.0f, float32 y=0.0f);
};

class FOUNDATION_API RGBAFloat32Image : public TemplatedImage<Vector4f>
{
private:
   static MetaClass *RGBAFloat32ImageMetaClass;
   static Object* newInstance();

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   RGBAFloat32Image() {}
   RGBAFloat32Image( card32 sizeX, card32 sizeY ) : TemplatedImage<Vector4f>(sizeX, sizeY) {}

};


}

#endif

