//---------------------------------------------------------------------------
#ifndef StringHelperH
#define StringHelperH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <string>
#include <vector>
#include "ExpressionParser.h"
//---------------------------------------------------------------------------

class QString;

namespace geoxl {

/// Convert the string given to a number. Returns true if successful.
bool FOUNDATION_API strToNum(std::string s, ExpressionNum& num);
/// Converts the number given to a string.
std::string FOUNDATION_API numToStr(ExpressionNum num);
/// Convert the string given to a int. Returns true if successful.
bool FOUNDATION_API strToInt(std::string s, int& num);
/// Convert the string given to a float. Returns true if successful.
bool FOUNDATION_API strToFloat(std::string s, float& num);
/// Converts num to a string
std::string FOUNDATION_API intToStr(int64 i);
/// Converts float to a string
std::string FOUNDATION_API floatToStr(float f);
/// Trims the string (deletes spaces at the beginning and the end)
std::string FOUNDATION_API trimStr(std::string s);
/// Deletes the leading spaces in the string and returns the result
std::string FOUNDATION_API lTrimStr(std::string s);
/// Deletes the trailing spaces in the string and returns the result
std::string FOUNDATION_API rTrimStr(std::string s);
/// Converts the string into lower case and returns the result
std::string FOUNDATION_API strLCase(std::string s);
/// Converts the string into upper case and returns the result
std::string FOUNDATION_API strUCase(std::string s);
/// Removes all double spaces from the string (i.e. makes them single spaces)
std::string FOUNDATION_API removeDblSpace(std::string s);
/// Extracts the characters up to a certain character (see detailed descr.)
std::string FOUNDATION_API extractToChar(std::string& s, char c);

/// Splits s into substrings, splitted by splitchars
/// Example: splitString( "hello world!\n bye", " \n" ) ----> "hello", "world!", "bye"
std::vector<std::string> FOUNDATION_API splitString( const std::string &s, const std::string &splitchars);

/// Example: "dir/test.abc" -> "test.abc"
std::string FOUNDATION_API removePathFromFilename( const std::string &filename );
/// Example: "dir/test.abc" -> "dir/"
std::string FOUNDATION_API removeFilenameFromPath( const std::string &filename );

/// Cuts string ( by removing all characters behind maxLength), if length > maxLength 
std::string FOUNDATION_API cutString( const std::string str, card32 maxLength );

/// Replaces every substring in str with rStr
std::string FOUNDATION_API replaceSubStrings( const std::string str, const std::string substring, const std::string rStr );

/// returns a string with format [y years][d days ][h:][m:]s.ms calculated from time, time in ms
std::string FOUNDATION_API getTimeString( card32 time, bool milliseconds=false );

/// returns string with the file name ending or "." if no ending exists, e.g. input:"dir/myFile.txt" output:".txt"
std::string FOUNDATION_API extractFileExt( const std::string &s );

std::string FOUNDATION_API qString2STLString(const QString &str);

/// converts all '/' characters to '\'
std::string FOUNDATION_API allDirSeperatorsToDos(const string filename);

/// converts all '\' characters to '/'
std::string FOUNDATION_API allDirSeperatorsToUnix(const string filename);

/// pads string with zeros in front if length is smaller than numZeroes
std::string FOUNDATION_API fillNumberStringWithZeros(const string number, unsigned numZeros);

/// converts time (in ms) into "[h hours] [m minutes] [s seconds] ms ms"
std::string FOUNDATION_API convertTimeToString( card32 delta_time );

}

#endif
