//---------------------------------------------------------------------------
#ifndef SymbolTablesH
#define SymbolTablesH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <vector>
#include "GlobalDefinitions.h"
#include "Object.h"
//---------------------------------------------------------------------------

namespace geoxl {

typedef long double ExpressionNum;

template <class T>
class SymbolTableBase {
  public:
    virtual bool getValue(const std::string name, T& value) = 0;
    virtual bool setValue(const std::string name, T value) = 0;
    virtual bool removeSymbol(const std::string name) { return false; }
};

typedef SymbolTableBase<ExpressionNum> NumSymbolTable;
typedef SymbolTableBase<std::string> StringSymbolTable;
typedef SymbolTableBase<Object*> ObjectSymbolTable;

template <class T>
class SymTableHandlerBase {
  protected:
    std::vector<SymbolTableBase<T>*> symbolTables;

  public:
    void addTable(SymbolTableBase<T>* symbolTable)
      { symbolTables.push_back(symbolTable); }
    void removeTable(SymbolTableBase<T>* symbolTable)
      { symbolTables.erase(find(symbolTables.begin(), symbolTables.end(), symbolTable)); } 
    void clearTables()
      { symbolTables.clear(); }
    bool getValue(const std::string name, T& value);
    bool setValue(const std::string name, T value);
    bool removeSymbol(const std::string name);
};

typedef FOUNDATION_API SymTableHandlerBase<ExpressionNum> NumTableHandler;
typedef FOUNDATION_API SymTableHandlerBase<Object*> ObjectTableHandler;

class FOUNDATION_API StringTableHandler : public SymTableHandlerBase<std::string>
{
  public:
    bool evaluate(const std::string strExpr, std::string& result);
};

//---------------------------------------------------------------------------
template <class T>
bool SymTableHandlerBase<T>::getValue(const std::string name, T& value)
{
  for (card32 i = 0; i < symbolTables.size(); ++i) {
    bool success = symbolTables[i]->getValue(name, value);
    if (success) return true;
  }
  return false;
}

template <class T>
bool SymTableHandlerBase<T>::setValue(const std::string name, T value)
{
  for (card32 i = 0; i < symbolTables.size(); ++i) {
    bool success = symbolTables[i]->setValue(name, value);
    if (success) return true;
  }
  return false;
}

template <class T>
bool SymTableHandlerBase<T>::removeSymbol(const std::string name)
{
  for (card32 i = 0; i < symbolTables.size(); ++i) {
    bool success = symbolTables[i]->removeSymbol(name);
    if (success) return true;
  }
  return false;
}

}


#endif
