#ifndef TableOutputToolsH
#define TableOutputToolsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl
{

/**
	this is a helper class for table data. holds multiple tables accessed by a unique name
	and offers string output for different table formats (latex, simple table for exel copy/paste).
*/
class FOUNDATION_API TableOutputTools 
{

public:
	TableOutputTools(void);
	~TableOutputTools(void);

	/// erase every table
	void clear();

	/// set table entry (you don't have to care about table dimensions)
	void setData( string tableName, int row, int column, float data );

	/// adds to existing table entry (which is initially zero)
	void addData( string tableName, int row, int column, float data );

	void setRowName( string tableName, int row, string rowName );
	void setColumnName( string tableName, int column, string columnName );


	enum DataEntryFormat
	{
		DF_FloatPoint, // 1.5
		DF_FloatComma,	// 1,5
		DF_Int			// 1
	};

	/// generates an output string with simple table format
	/// which looks like this:
	/// Tabelle: tableName
	/// 1 0 1 2 1 0
	/// 0 1 0 0 1 1
	/// 6 8 2 7 2 1 
	/// ...
	string outputSimpleTable( string tableName, DataEntryFormat dataEntryFormat );

	/// generates an output string for latex tables
	string outputLatexTable( string tableName, DataEntryFormat dataEntryFormat );

protected:

	void setupTable( string tableName, int numRows, int numColumns );
	string convertNumber(float n, DataEntryFormat format );

	struct TTable
	{
		vector<float> table;
		int numRows, numColumns;
		vector<string> rowNames, columnNames;
	};

	map<string,TTable> m_Tables;

};

extern FOUNDATION_API TableOutputTools * tableOutputTool;

}

#endif