//---------------------------------------------------------------------------
#ifndef TimerH
#define TimerH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PTypes.h"
//---------------------------------------------------------------------------


namespace geoxl {

class FOUNDATION_API Timer {
 private:
   card32 lastTime;

 public:
   Timer();
   card32 getCurrentValue();
   card32 getDeltaValue(card32 lastValue);
   card32 getDeltaValue();
   ~Timer();
};

#define PEAK_TIMER_START( id ) \
	Timer timer_##id ; 

#define PEAK_TIMER_END( id ) \
	static card32 peaktime_##id = 0; card32 tmptime_##id = timer_##id .getDeltaValue(); \
	if( peaktime_##id < tmptime_##id ) \
{ debugOutput << "peak time[ " << #id << "]" << convertTimeToString( tmptime_##id ) << "\n"; peaktime_##id = tmptime_##id; debugOutput.flush(); }


#define GEOXL_TIMER_START( id ) \
	Timer geoxl_timer_##id ; \
	geoxl_timer_##id##.getDeltaValue();

#define GEOXL_TIMER_END( id ) \
	debugOutput << "Timings[" << #id << "]: " << convertTimeToString(  geoxl_timer_##id .getDeltaValue() ) << "\n";

#define GEOXL_TIMER_STOP( id ) GEOXL_TIMER_END( id )

#define GEOXL_AVERAGE_TIMER_DEFINE(id) \
	static unsigned geoxl_average_timer_variable_##id = 0; \
	static unsigned geoxl_average_timer_numMeasurements_##id = 0; \
	static unsigned geoxl_average_timer_begin_##id; \
	static Timer geoxl_average_timer_##id ; 

#define GEOXL_AVERAGE_TIMER_START(id) \
	geoxl_average_timer_begin_##id = geoxl_average_timer_##id##.getCurrentValue();
	
#define GEOXL_AVERAGE_TIMER_STOP(id) \
	geoxl_average_timer_variable_##id += (geoxl_average_timer_##id##.getCurrentValue() - geoxl_average_timer_begin_##id); \
	geoxl_average_timer_numMeasurements_##id ++;

#define GEOXL_AVERAGE_TIMER_REPORT_AND_RESET(id) \
	if( geoxl_average_timer_numMeasurements_##id > 0 ) \
	debugOutput << "Timer[ " << #id << "] - Report " << convertTimeToString( geoxl_average_timer_variable_##id / geoxl_average_timer_numMeasurements_##id ) << \
	" total time:" << convertTimeToString( geoxl_average_timer_variable_##id ) << "\n"; \
	else \
	debugOutput << "Timer[ " << #id << "] - No measurements were done!\n"; \
	geoxl_average_timer_variable_##id = 0; \
	geoxl_average_timer_numMeasurements_##id = 0; 


}

#endif
 