//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "AttachedData.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"
#include "PAssert.h"
#include "MemoryManager.h"
#include "NumericalClassProperty.h"
#include "NumericalClassProperty_templCode.h"
#include "StringClassProperty.h"
#include "Tools.h"
#include "HashTables_templCode.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* AttachedData::AttachedDataMetaClass = NULL;

const MetaClass* AttachedData::getInstanceClass() const {
   return getClass();
}

Object* AttachedData::newInstance() {
   return new AttachedData;
}

const MetaClass* AttachedData::getClass() {
   if (AttachedDataMetaClass==NULL) {
      AttachedDataMetaClass = new MetaClass("AttachedData", "no description.",
                                  newInstance, sizeof(AttachedData));
   }
   return AttachedDataMetaClass;
}

void AttachedData::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass,MetaClass::FIRST_VERSION + 0);
   AttachedDataMetaClass->copyParentProperties();

	BEGIN_CLASS_INIT( AttachedData );
	ADD_STRING_PROP( m_name, 0 );
	SET_EDIT_READONLY( true );
	ADD_CARD32_PROP( m_flags, 0 );

}

void AttachedData::shutdown() {
   delete AttachedDataMetaClass;
}

// ----

AttachedData::AttachedData():m_flags(0), m_memManObj(NULL),m_MyContainer(NULL){
}

AttachedData::~AttachedData()
{
   if( m_MyContainer )
      m_MyContainer->removeData( m_name );
}

void AttachedData::setup(const std::string& name, const card32 flags, MemoryManager* mm){
   m_flags=flags;
   m_name=name;
   if(mm){
      m_memManObj=mm->createNewMMObject(this);
   }
}

void AttachedData::assign(const Object* obj, CopyContext *context)
{
   //Persistent::assign(obj, context );
   AttachedData * data = (AttachedData *)obj;
   m_name = data->m_name;
   m_flags = data->m_flags;
   m_MyContainer = NULL;
   forget(context);
}

// ============================================================================

MetaClass* AttachedDataContainer::AttachedDataContainerMetaClass = NULL;

const MetaClass* AttachedDataContainer::getInstanceClass() const {
   return getClass();
}

Object* AttachedDataContainer::newInstance() {
   return new AttachedDataContainer;
}

const MetaClass* AttachedDataContainer::getClass() {
   if (AttachedDataContainerMetaClass==NULL) {
      AttachedDataContainerMetaClass = new MetaClass("AttachedDataContainer", "no description.",
                                  newInstance, sizeof(AttachedDataContainer));
   }
   return AttachedDataContainerMetaClass;
}

void AttachedDataContainer::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   AttachedDataContainerMetaClass->copyParentProperties();

//   AttachedDataContainerMetaClass->getGUIDescriptor()->addClassEditor(AttachedDataContainerEditor::getClass(), true);
}

void AttachedDataContainer::shutdown() {
   delete AttachedDataContainerMetaClass;
}


// ----

MetaClass* StringAttachment::StringAttachmentMetaClass = NULL;

const MetaClass* StringAttachment::getInstanceClass() const {
   return getClass();
}

Object* StringAttachment::newInstance() {
   return new StringAttachment;
}

const MetaClass* StringAttachment::getClass() {
   if (StringAttachmentMetaClass==NULL) {
      StringAttachmentMetaClass = new MetaClass("StringAttachment", "no description.",
                                  newInstance, sizeof(StringAttachment));
   }
   return StringAttachmentMetaClass;
}

void StringAttachment::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   StringAttachmentMetaClass->copyParentProperties();
//   StringAttachmentMetaClass->addProperty( new StringClassProperty( "data",MetaClass::FIRST_VERSION + 0,
//	   (StringElementPointer)&StringAttachment::m_String) );
	BEGIN_CLASS_INIT(StringAttachment);
	ADD_STRING_PROP( m_String, 0 );
}

void StringAttachment::shutdown() {
   delete StringAttachmentMetaClass;
}


// ----


max_platform_card AttachedDataContainer::MyHashTable::hashFunction(string key) const {
   card32 hk = 0;
   const card32 n = (card32)key.size();
   for (card32 i=0; i<n; i++) {
      const card8 c = key[i];
      hk <<= 1;
      hk ^=c;
   }
   return (max_platform_card)hk;
}

AttachedDataContainer::MyHashTable::MyHashTable(card32 fillFactor, card32 startSizeIndex)
: HashTable<string, AttachedData*, StringComp>(fillFactor, startSizeIndex) {}


AttachedDataContainer::AttachedDataContainer() {
   m_dataHashTable = new MyHashTable(4 /* fill factor */, 1 /* initial size */);
}

static void writeCallback(void *context, string name, AttachedData *data) {
   OutputObjectStream *s = (OutputObjectStream*)context;
   if( data->getFlags() & AttachedData::ADF_PERSISTENT )
      s->writeObject((Persistent*)data);
}

static void countDataCallback( void * context, string name, AttachedData * data )
{
   card32 * numData = (card32*)context;
   if( data->getFlags() & AttachedData::ADF_PERSISTENT )
      numData[0]++;
}

void AttachedDataContainer::write(OutputObjectStream *s) const
{
   card32 numData=0;
   m_dataHashTable->forEachEntry(&numData, countDataCallback);
   s->write(numData);
   m_dataHashTable->forEachEntry(s, writeCallback);
}

void AttachedDataContainer::read(InputObjectStream *s) {
   clear();
   card32 count;
   s->read(count);
   for (card32 i=0; i<count; i++) {
      AttachedData *d;
      s->readObject((Persistent*&)d);
      attachData(d);
   }
}


static void copyCallback(void* context, string name, AttachedData *data) {
   AttachedDataContainer *thisInst = (AttachedDataContainer*)context;
   if ((data->getFlags() & AttachedData::ADF_PERSISTENT) != 0) {
      thisInst->attachData((AttachedData*)(data->copy()));
   }
}

// in class AttachedData to allow access to destructor
void AttachedData::delCallback(string name, AttachedData *data) {
   delete data;
}


void AttachedDataContainer::clear() {
   m_dataHashTable->forEachEntry(AttachedData::delCallback);
   delete m_dataHashTable;
   m_dataHashTable = new MyHashTable(4 /* fill factor */, 1 /* initial size */);
}

void AttachedDataContainer::assign(const Object* obj, CopyContext *context) {
   const AttachedDataContainer *other = dynamic_cast<const AttachedDataContainer*>(obj);
   if (other == NULL) {
      pAssert(false);
   } else {
      clear();
      other->m_dataHashTable->forEachEntry(this, copyCallback);
   }
   forget(context);
}

static void deleteOnUpdateCallBack(void* context, string name, AttachedData *data) {
   AttachedDataContainer *cThis = (AttachedDataContainer*)context;
   AttachedData::ChangeType change = cThis->temp_change_for_callback;
   if ((data->getFlags() & AttachedData::ADF_DELETEONUPDATE) != 0
       || (((data->getFlags() & AttachedData::ADF_QUERY_DELETE_ON_UPDATE) != 0)) && data->queryDelete(change)) {
      cThis->deleteData(name);
   }
}

void AttachedDataContainer::deleteDelOnWriteData(AttachedData::ChangeType change) {
   temp_change_for_callback = change;
   m_dataHashTable->forEachEntry(this, deleteOnUpdateCallBack);
}

void getAllCallback( void * context, string key, AttachedData * data )
{
   vector<AttachedData*> * pVec = (vector<AttachedData*> *)context;
   pVec->push_back( data );
   forget(key);
}

void AttachedDataContainer::getAll( vector<AttachedData*> &ret )
{
   m_dataHashTable->forEachEntry( (void *)&ret, getAllCallback );
}


AttachedData* AttachedDataContainer::getData(const string& name) const {
   AttachedData *result = NULL;
   bool success = m_dataHashTable->getData(name, result);
   if (success) {
      return result;
   } else {
      return NULL;
   }
}

void AttachedDataContainer::attachData(AttachedData* data){
   pAssert( data->m_MyContainer == NULL );
   const string name = data->getName();
   AttachedData *oldData = NULL;
   if (m_dataHashTable->getData(name, oldData)) {
      oldData->m_MyContainer = NULL;
      delete oldData;
   }
   data->m_MyContainer = this;
   m_dataHashTable->addElement(data->getName(), data);
}

void AttachedDataContainer::deleteData(const string &name) {
   AttachedData *oldData = NULL;
   if (m_dataHashTable->getData(name, oldData)) {
      oldData->m_MyContainer = NULL;
      delete oldData;
      m_dataHashTable->deleteElement(name);
   } else {
      pAssert(false);
   }
}

void AttachedDataContainer::removeData(const string &name)
{
   AttachedData *oldData = NULL;
   if (m_dataHashTable->getData(name, oldData)) {
      m_dataHashTable->deleteElement(name);
   } else {
      pAssert(false);
   }
}

AttachedDataContainer::~AttachedDataContainer() {
   m_dataHashTable->forEachEntry(AttachedData::delCallback);
   delete m_dataHashTable;
}


// --



void StringAttachment::assign(const Object* obj, CopyContext *context)
{
   AttachedData::assign( obj, context );
   const StringAttachment * att = dynamic_cast<const StringAttachment*>(obj);
   pAssert( att );
   m_String = att->m_String;
}




geoxl::mpcard geoxl::AttachedDataContainer::getSizeInMemory()
{
	vector<AttachedData*> att;
	getAll(att);
	card32 ret = 0;
	for( unsigned i=0;i<att.size();i++ )
	{
		ret += att[i]->getSize();
	}
	return ret;
}