//---------------------------------------------------------------------------
#ifndef ClassPropertyParameters_templCodeH
#define ClassPropertyParameters_templCodeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "PAssert.h"
#include "DynamicLinearAlgebra.h"
#include "NumericalClassProperty.h"
//---------------------------------------------------------------------------


namespace geoxl {

class ClassProperty;


inline ClassPropertyParameters::ClassPropertyParameters(const ClassPropertyParameters &c) {
   this->parameters = c.parameters;
}

inline const ClassPropertyParameters ClassPropertyParameters::operator=(const ClassPropertyParameters &c) {
   this->parameters = c.parameters;
   return *this;
}

inline bool ClassPropertyParameters::operator==(const ClassPropertyParameters &c) {
   return this->parameters == c.parameters;
}

inline void ClassPropertyParameters::addParameter(string name) {
   parameters.push_back(name);
}

inline card32 ClassPropertyParameters::getNumParameters() const {
   return (card32)parameters.size();
}

inline void ClassPropertyParameters::setNumParameters(card32 n) {
   parameters.resize(n);
}

inline const string &ClassPropertyParameters::operator[](card32 index) const {
   pAssert(index < parameters.size());
   return parameters[index];
}

inline string &ClassPropertyParameters::operator[](card32 index) {
   pAssert(index < parameters.size());
   return parameters[index];
}


// ----


/*
template <typename FloatType>
const DynamicVector<FloatType> CPPFloatParameterAccess::getParameterVector<FloatType>() {
   const card32 n = access.size();
   vector<FloatType> resultVec;
   for (card32 i=0; i<n; i++) {
      const ClassProperty *prop = access[i].prop;
      Object *target = access[i].target;

      const NumericalClassProperty<float32> *n32prop = dynamic_cast< const NumericalClassProperty<float32>* >(prop);
      if (n32prop != NULL) {
         resultVec.push_back((FloatType)(n32prop->getValue(target)));
      } else {
         const NumericalClassProperty<float64> *n64prop = dynamic_cast< const NumericalClassProperty<float64>* >(prop);
         if (n64prop != NULL) {
            resultVec.push_back((FloatType)(n64prop->getValue(target)));
         } else {
            const ArrayClassProperty<float32> *a32prop = dynamic_cast< const ArrayClassProperty<float32>* >(prop);
            if (a32prop != NULL) {
               const card32 kArr = a32prop->getNumEntries();
               for (card32 k=0; k<nArr; k++) {
                  resultVec.push_back((FloatType)(a32prop->getValue(target, k)));
               }
            } else {
               const ArrayClassProperty<float64> *a64prop = dynamic_cast< const ArrayClassProperty<float64>* >(prop);
               if (a64prop != NULL) {
                  const card32 kArr = a64prop->getNumEntries();
                  for (card32 k=0; k<nArr; k++) {
                     resultVec.push_back((FloatType)(a64prop->getValue(target, k)));
                  }
               }
            }
         }
      }
   }
   const nr = resultVec.size();
   DynamicVector<FloatType> result = DynamicVector<FloatType>(nr);
   for (card32 i=0; i<nr; i++) {
      result[i] = resultVec[i];
   }
   return result;
}

template <typename FloatType>
void CPPFloatParameterAccess::setParameterVector<FloatType>(const DynamicVector<FloatType> &params) {
   const card32 n = access.size();
   const card32 np = params.getDim();
   card32 pPos = 0;
   for (card32 i=0; (i<n && pPos <np); i++) {
      const ClassProperty *prop = access[i].prop;
      Object *target = access[i].target;

      const NumericalClassProperty<float32> *n32prop = dynamic_cast< const NumericalClassProperty<float32>* >(prop);
      if (n32prop != NULL) {
         n32prop->setValue(target, (float32)params[pPos++]);
      } else {
         const NumericalClassProperty<float64> *n64prop = dynamic_cast< const NumericalClassProperty<float64>* >(prop);
         if (n64prop != NULL) {
            n64prop->setValue(target, (float64)params[pPos++]);
         } else {
            const ArrayClassProperty<float32> *a32prop = dynamic_cast< const ArrayClassProperty<float32>* >(prop);
            if (a32prop != NULL) {
               const card32 nArr = a32prop->getNumEntries(target);
               for (card32 k=0; k<nArr; k++) {
                  a32prop->setValue(target, k, (float32)params[pPos++]);
               }
            } else {
               const ArrayClassProperty<float64> *a64prop = dynamic_cast< const ArrayClassProperty<float64>* >(prop);
               if (a64prop != NULL) {
                  const card32 nArr = a64prop->getNumEntries(target);
                  for (card32 k=0; k<nArr; k++) {
                     a64prop->setValue(target, k, (float64)params[pPos++]);
                  }
               }
            }
         }
      }
   }
   pAssert(pPos == np);
}


}

*/

template <typename FloatType>
const DynamicVector<FloatType> CPPFloatParameterAccess::getParameterVector() {
   const card32 n = access.size();
   vector<FloatType> resultVec;
   for (card32 i=0; i<n; i++) {
      const ClassProperty *prop = access[i].prop;
      Object *target = access[i].target;

      const NumericalClassProperty<float32> *n32prop = dynamic_cast< const NumericalClassProperty<float32>* >(prop);
      if (n32prop != NULL) {
         resultVec.push_back((FloatType)(n32prop->getValue(target)));
      } else {
         const NumericalClassProperty<float64> *n64prop = dynamic_cast< const NumericalClassProperty<float64>* >(prop);
         if (n64prop != NULL) {
            resultVec.push_back((FloatType)(n64prop->getValue(target)));
         } else {
            const ArrayClassProperty<float32> *a32prop = dynamic_cast< const ArrayClassProperty<float32>* >(prop);
            if (a32prop != NULL) {
               const card32 kArr = a32prop->getNumEntries();
               for (card32 k=0; k<nArr; k++) {
                  resultVec.push_back((FloatType)(a32prop->getValue(target, k)));
               }
            } else {
               const ArrayClassProperty<float64> *a64prop = dynamic_cast< const ArrayClassProperty<float64>* >(prop);
               if (a64prop != NULL) {
                  const card32 kArr = a64prop->getNumEntries();
                  for (card32 k=0; k<nArr; k++) {
                     resultVec.push_back((FloatType)(a64prop->getValue(target, k)));
                  }
               }
            }
         }
      }
   }
   const nr = resultVec.size();
   DynamicVector<FloatType> result = DynamicVector<FloatType>(nr);
   for (card32 i=0; i<nr; i++) {
      result[i] = resultVec[i];
   }
   return result;
}

template <typename FloatType>
void CPPFloatParameterAccess::setParameterVector(const DynamicVector<FloatType> &params) {
   const card32 n = access.size();
   const card32 np = params.getDim();
   card32 pPos = 0;
   for (card32 i=0; (i<n && pPos <np); i++) {
      const ClassProperty *prop = access[i].prop;
      Object *target = access[i].target;

      const NumericalClassProperty<float32> *n32prop = dynamic_cast< const NumericalClassProperty<float32>* >(prop);
      if (n32prop != NULL) {
         n32prop->setValue(target, (float32)params[pPos++]);
      } else {
         const NumericalClassProperty<float64> *n64prop = dynamic_cast< const NumericalClassProperty<float64>* >(prop);
         if (n64prop != NULL) {
            n64prop->setValue(target, (float64)params[pPos++]);
         } else {
            const ArrayClassProperty<float32> *a32prop = dynamic_cast< const ArrayClassProperty<float32>* >(prop);
            if (a32prop != NULL) {
               const card32 nArr = a32prop->getNumEntries(target);
               for (card32 k=0; k<nArr; k++) {
                  a32prop->setValue(target, k, (float32)params[pPos++]);
               }
            } else {
               const ArrayClassProperty<float64> *a64prop = dynamic_cast< const ArrayClassProperty<float64>* >(prop);
               if (a64prop != NULL) {
                  const card32 nArr = a64prop->getNumEntries(target);
                  for (card32 k=0; k<nArr; k++) {
                     a64prop->setValue(target, k, (float64)params[pPos++]);
                  }
               }
            }
         }
      }
   }
   pAssert(pPos == np);
}


}





#endif
 