//---------------------------------------------------------------------------
#ifndef ObjectDictionaryH
#define ObjectDictionaryH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PTypes.h"
#include "MetaClass.h"
#include "Object.h"
#include "Persistent.h"
#include <vector>
#include <string>
//---------------------------------------------------------------------------

namespace geoxl {

class FOUNDATION_API ObjectDictionary : public Persistent {

  private:
    static MetaClass *ObjectDictionaryMetaClass;
    static Object* newInstance();

    typedef struct {
      Object* obj;
      std::string name;
      bool owned;
    } ObjListElem;
    typedef std::vector<ObjListElem> ObjVecType;
    ObjVecType objVector;

    /// Returns the object number for the object name given (internal use)
    card32 objectByName(std::string name) const;

    const MetaClass* rootClass;

  public:
    virtual const MetaClass *getInstanceClass() const;
    static const MetaClass *getClass();
    static void init(const MetaClass *parentClass);
    static void shutdown();
    void assign(const Object* obj, CopyContext *context = NULL);

    ObjectDictionary();
    ~ObjectDictionary();
    void setRootClass(const MetaClass* rootClass) { this->rootClass = rootClass; };

    /// Creates a new instance of the class given by the metaClassName with the name given
    std::string newObject(std::string metaClassName, std::string name = "", bool owned = true);
    /// Creates a new instance of the class given in metaClass
    std::string newObject(const MetaClass* metaClass, std::string name = "", bool owned = true);
    /// Deletes the instance of the object and removes it from the dictionary
    bool deleteObject(std::string name);
    /// Deletes the instance of the object and removes it from the dictionary
    bool deleteObject(Object* obj);
    /// Adds an existing object to the dictionary
    std::string addObject(Object* obj, std::string name = "", bool owned = true);
    /// Removes the object with the name given from the object dictionary
    void removeObject(std::string name);
    /// Renames the object given to the new name
    bool renameObject(std::string oldName, std::string newName);
    /// Clears the whole object dictionary
    void clear();

    /// Replaces the oldObject given with the newObject.
    void replaceObject(Object *oldObject, Object *newObject, std::string newName = "");

    /// Returns a pointer to the object with the name given in obj
    bool getObjectSafe(std::string name, Object* &obj) const;
    /// Returns a pointer to the object with the name given
    Object* getObject(std::string name) const;

    /// Returns true if the given name already exists and false otherwise
    bool nameExists(std::string name) const;
    /// Generates a new unique name for a new instance of the metaClass.
    std::string generateName(const MetaClass* metaClass);

    /// Returns whether the object given is owned by the dictionary.
    bool isOwned(std::string name) const;

    virtual void write(OutputObjectStream *s) const;
    virtual void read(InputObjectStream *s);

	 /// Returns number of objects
	 card32 getNumObjects() const;

	 /// Returns object
	 Object * getObject( card32 i ) const;

	 /// Returns object-name
	 std::string getObjectName( card32 i ) const;
	 
    /// Moves the object to another position within the object dictionary
	 void moveObject(card32 fromPos, card32 toPos);

};

class FOUNDATION_API EObjDictionaryException : public PException {
 public:
   EObjDictionaryException(const string msg) : PException(msg) {};
   virtual string getTypeString() {return "EObjDictionaryException";}
};

}

#endif