//---------------------------------------------------------------------------
#ifndef PropertyParserH
#define PropertyParserH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <string>
#include "Persistent.h"
#include "ExpressionParser.h"
//---------------------------------------------------------------------------

namespace geoxl {

class NumericalClassPropertyBase;
class BooleanClassProperty;
//class GeneralArrayClassProperty;
class StringClassProperty;
class FixedArrayClassProperty;
class VariableArrayClassProperty;


class FOUNDATION_API PropertyParser {
  public:
    /// Returns the value of the number property given
    bool getValue(Object* obj, std::string property, ExpressionNum& value);
    /// Sets the value of the number property given to the value
    bool setValue(Object* obj, std::string property, ExpressionNum value);

    /// Returns the value of the string property given
    bool getValue(Object* obj, std::string property, std::string& value);
    /// Sets the value of the string property given to the value
    bool setValue(Object* obj, std::string property, std::string value);

  private:
    typedef struct {
      const NumericalClassPropertyBase* numeric;
      const BooleanClassProperty* boolean;
      const StringClassProperty* stringProp;
      const FixedArrayClassProperty *fixedArray;
      const VariableArrayClassProperty *varArray;
      pointer_diff arrayOffset;
    } PropertyTypes;

    card32 arrayIndex;
    bool isArrayProperty;

    /// Recursive method that descends all the way to the last propertie given
    bool descendProperties(Object* &obj, std::string& property);
    /// Returns the property of the object obj with the name given.
    PropertyTypes getProperty(Object* &obj, std::string propertyName);
    /// Returns the respective array element from the class property
    const ClassProperty* getArrayOffset(Object* &obj, const ClassProperty* cp,
                                        string arrayIndexStr, pointer_diff &arrayOffset);
};

class FOUNDATION_API EPropertyParserException : public PException {
 public:
   EPropertyParserException(const string msg) : PException(msg) {};
   virtual string getTypeString() {return "EPropertyParserException";}
};


}

#endif
