#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "SimpleAttachment.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( SimpleAttachment ,0)
{
	BEGIN_CLASS_INIT( SimpleAttachment );
	ADD_VAR_ARRAY_PROP( Byte, 0, new Int8ClassProperty("element", MetaClass::FIRST_VERSION,NULL) );
	SET_EDIT_READONLY(true);
}

SimpleAttachment::SimpleAttachment(void)
{
}

SimpleAttachment::~SimpleAttachment(void)
{
}

void SimpleAttachment::assign(const Object* obj, CopyContext *context)
{
	const SimpleAttachment * o = dynamic_cast<const SimpleAttachment*>(obj);
	pAssert( o != NULL );

	copyObjectProperties(obj,this);
}


bool geoxl::SimpleAttachment::exists( AttachedDataContainer * att, const string &name )
{
	return dynamic_cast<SimpleAttachment*>(att->getData(name));
}

void * geoxl::SimpleAttachment::get( AttachedDataContainer * att, const string &name )
{
	SimpleAttachment * s = dynamic_cast<SimpleAttachment*>(att->getData(name));
	if( s == NULL )
		throw PException(string("SimpleAttachment::get() - Attachment ") + name  + " doesn't exists!");
	return &s->m_Data[0];
}

void geoxl::SimpleAttachment::set( AttachedDataContainer * att, const string &name, const void * ptr, card32 numBytes, bool persistent )
{
	SimpleAttachment * s = dynamic_cast<SimpleAttachment*>(att->getData(name));
	if( s != NULL )
	{
		if( s->m_Data.size() != numBytes )
		{
			throw PException(string("SimpleAttachment::set() - Attachment ") + name  + " already exists, but with a different number of bytes!");
		}
	}
	else
	{
		s = new SimpleAttachment;
		s->setup(name, persistent ? AttachedData::ADF_PERSISTENT:0,NULL );
		s->m_Data.resize(numBytes);
		att->attachData(s);
	}
	memcpy(&s->m_Data[0],ptr,numBytes);
}

template<class T>
T simpleAttachment_templated_get(AttachedDataContainer * att, const string &name )
{
	T * ret = (T*)SimpleAttachment::get(att,name);
	return *ret;
}

template<class T>
void simpleAttachment_templated_set(AttachedDataContainer * att, const string &name,const T &val,bool persistent )
{
	SimpleAttachment::set(att,name, &val, sizeof(T), persistent);
}


geoxl::float32 geoxl::SimpleAttachment::get1f( AttachedDataContainer * att, const string &name )
{
	return simpleAttachment_templated_get<float32>(att,name);
}

void geoxl::SimpleAttachment::set1f( AttachedDataContainer * att, const string &name, const float32 &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);
}

geoxl::Vector2f geoxl::SimpleAttachment::get2f( AttachedDataContainer * att, const string &name )
{
	return simpleAttachment_templated_get<Vector2f>(att,name);
}

void geoxl::SimpleAttachment::set2f( AttachedDataContainer * att, const string &name, const Vector2f &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);
}

geoxl::Vector3f geoxl::SimpleAttachment::get3f( AttachedDataContainer * att, const string &name )
{
	return simpleAttachment_templated_get<Vector3f>(att,name);

}

void geoxl::SimpleAttachment::set3f( AttachedDataContainer * att, const string &name, const Vector3f &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);
}

geoxl::Vector4f geoxl::SimpleAttachment::get4f( AttachedDataContainer * att, const string &name )
{	
	return simpleAttachment_templated_get<Vector4f>(att,name);
}

void geoxl::SimpleAttachment::set4f( AttachedDataContainer * att, const string &name, const Vector4f &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);
}

geoxl::Matrix3f geoxl::SimpleAttachment::getMatrix3f( AttachedDataContainer * att, const string &name )
{
	return simpleAttachment_templated_get<Matrix3f>(att,name);

}

void geoxl::SimpleAttachment::setMatrix3f( AttachedDataContainer * att, const string &name, const Matrix3f &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);

}

geoxl::Matrix4f geoxl::SimpleAttachment::getMatrix4f( AttachedDataContainer * att, const string &name )
{
	return simpleAttachment_templated_get<Matrix4f>(att,name);
}

void geoxl::SimpleAttachment::setMatrix4f( AttachedDataContainer * att, const string &name, const Matrix4f &v,bool persistent/*=true*/ )
{
	simpleAttachment_templated_set(att,name,v,persistent);

}