#ifndef SimpleAttachmentH
#define SimpleAttachmentH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "AttachedData.h"
//---------------------------------------------------------------------------

namespace geoxl
{

/**
	simple fixed-size memory attachment
*/
class FOUNDATION_API SimpleAttachment : public AttachedData
{
	GEOXL_CLASS( SimpleAttachment )

private:

	vector<char> m_Data;

	void setNumBytes(card32 size) {m_Data.resize(size);}
	void deleteByte( card32 index ) {m_Data.erase( m_Data.begin() + index);}
	char * getByte( card32 index ) {return &m_Data[index];}
	mpcard getNumBytes() {return m_Data.size();}
	string getByteName(card32 index ) {return "";}

public:
	SimpleAttachment(void);
	~SimpleAttachment(void);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

	// basic access methods
	static bool exists( AttachedDataContainer * att, const string &name);
	static void * get( AttachedDataContainer * att, const string &name);
	static void set( AttachedDataContainer * att, const string &name, const void * ptr, card32 numBytes,bool persistent=true);

	// specialized attachments
	static float32 get1f(AttachedDataContainer * att, const string &name);
	static void set1f(AttachedDataContainer * att, const string &name, const float32 &v,bool persistent=true);
	static Vector2f get2f(AttachedDataContainer * att, const string &name);
	static void set2f(AttachedDataContainer * att, const string &name, const Vector2f &v,bool persistent=true);
	static Vector3f get3f(AttachedDataContainer * att, const string &name);
	static void set3f(AttachedDataContainer * att, const string &name, const Vector3f &v,bool persistent=true);
	static Vector4f get4f(AttachedDataContainer * att, const string &name);
	static void set4f(AttachedDataContainer * att, const string &name, const Vector4f &v,bool persistent=true);

	static Matrix3f getMatrix3f(AttachedDataContainer * att, const string &name);
	static void setMatrix3f(AttachedDataContainer * att, const string &name, const Matrix3f &v,bool persistent=true);
	static Matrix4f getMatrix4f(AttachedDataContainer * att, const string &name);
	static void setMatrix4f(AttachedDataContainer * att, const string &name, const Matrix4f &v,bool persistent=true);

};



}

#endif