//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIDGPExperiment.h"
#include "PointCloudTools.h"
#include "ParametricInCorePointCloud.h"
#include "UnstructuredInCorePointCloud.h"
#include "PCCResampler.h"
#include "PointSetKNNQuery.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_GEOXL_CLASS( PCIDGPExperiment, 1 ) {
   BEGIN_CLASS_INIT( PCIDGPExperiment );

}

PCIDGPExperiment::PCIDGPExperiment() {}

PCIDGPExperiment::~PCIDGPExperiment() {}

/// returns only simple point cloud types
PointSet * geoxl::PCIDGPExperiment::getPointCloud( string name )
{
   PointCloud *pc = ::getPointCloud(getScene(), name);
   if (dynamic_cast<ParametricInCorePointCloud*>(pc))   return dynamic_cast<ParametricInCorePointCloud*>(pc)->getPointSet();
   if (dynamic_cast<UnstructuredInCorePointCloud*>(pc)) return dynamic_cast<UnstructuredInCorePointCloud*>(pc)->getPointSet();
   return NULL;
}

void geoxl::PCIDGPExperiment::replacePointCloud( string name, PointSet* newPS )
{
   ::replacePointCloud(getScene(), name, packagePointSet(newPS));
}

void geoxl::PCIDGPExperiment::addOrReplacePointCloud( string name, PointSet* newPS )
{
   ::addOrReplacePointCloud(getScene(), name, packagePointSet(newPS));
}

void geoxl::PCIDGPExperiment::addPointCloud( PointSet* newPS, string name )
{
   ::addPointCloud(getScene(), packagePointSet(newPS), name);
}

void geoxl::PCIDGPExperiment::updatePointCloud( string name )
{
	PointCloud *pc = ::getPointCloud(getScene(), name);
	UnstructuredInCorePointCloud *upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
	ParametricInCorePointCloud *ppc = dynamic_cast<ParametricInCorePointCloud*>(pc);
	if (upc) upc->deleteDelOnWriteAttachments(AttachedData::CT_EVERYTHING_CHANGED);
	if (ppc) {
		ppc->getAttachments()->deleteDelOnWriteData();
		SceneObjectIterator *it = ppc->createIterator(SOIT::CAP_ATTACH_SECONDARY);
		SecondaryDataStructureIterator *sit = dynamic_cast<SecondaryDataStructureIterator*>(it);
		sit->lockAttachments()->deleteDelOnWriteData();
		sit->unlockAttachments();
		delete it;
		ppc->getPointSet()->invalidateLastNumFlagged();
	}
}

void geoxl::PCIDGPExperiment::checkAttribute( string name, string attrib, card8 numComponents, card8 dataType )
{
   ::checkAttribute(getScene(), name, attrib, numComponents, dataType);
}

float geoxl::PCIDGPExperiment::getAveragePointDistance( PointSet *ps, card32 numDirectNeighbours /*= 5*/ )
{
   PointCloud *tmp = packagePointSet((PointSet*)ps->copy());
   float d = ::getAveragePointDistance(tmp, numDirectNeighbours);
   delete tmp;
   return d;
}

float geoxl::PCIDGPExperiment::getMedianPointDistance( PointSet *ps, card32 numDirectNeighbours /*= 5*/, card32 numSamples /*= 1000*/ )
{
   PointCloud *tmp = packagePointSet((PointSet*)ps->copy());
   float d = ::getMedianPointDistance(tmp, numDirectNeighbours, numSamples);
   delete tmp;
   return d;
}

bool geoxl::PCIDGPExperiment::deleteSceneGraphNode( string name )
{
   return ::deleteSceneGraphNode(getScene(), name);
}

PointCloud * geoxl::PCIDGPExperiment::packagePointSet( PointSet* ps )
{
   if (ps->getWidth() > 1) {
      ParametricInCorePointCloud *pc = new ParametricInCorePointCloud;
      pc->setPointSet(ps);
      return pc;
   } else {
      UnstructuredInCorePointCloud *pc = new UnstructuredInCorePointCloud;
      pc->setPointSet(ps);
      return pc;   
   }
}

void geoxl::PCIDGPExperiment::resamplePointCloud( PointSet *ps, float32 sampleDist )
{
   UnstructuredInCorePointCloud *tmp = new UnstructuredInCorePointCloud();
   tmp->setPointSet(createUnstructuredCopy(ps));
   PCCResampler::resamplePC(tmp, sampleDist);
   ps->assign(tmp->getPointSet());
   delete tmp;   
}

PointSet * geoxl::PCIDGPExperiment::createUnstructuredCopy( PointSet* ps )
{
   mpcard numPts = ps->getNumEntries();
   PointSet *newps = new PointSet();
   newps->clearAndSetup(0, numPts, ps->getDescr());
   for (mpcard i=0; i<numPts; i++) newps->setVertex(i, ps->getDataPointer(i));
   return newps;
}

void geoxl::PCIDGPExperiment::createPointSetKnnQuery( PointSet *ps, PSKnnQuery &query )
{
   query.octree = PointSetKNNQuery::createOctree(ps);
   query.knn = new PointSetKNNQuery(ps, query.octree, NULL_VECTOR3F, ps->getAAT("position", 3, VAD::DATA_FORMAT_FLOAT32));
}

void geoxl::PCIDGPExperiment::deletePointSetKnnQuery( PSKnnQuery &query )
{
   delete query.knn;
   delete query.octree;
}

void geoxl::PCIDGPExperiment::setKnnTarget( PSKnnQuery &query, const Vector3f &target )
{
   query.knn->setTargetPoint(target);
}

void geoxl::PCIDGPExperiment::getKnnNextNearestPointIndex( PSKnnQuery &query, card32 &nextIndex, bool &found )
{
   query.knn->getNextNearestPointIndex(nextIndex, found);
}
