//---------------------------------------------------------------------------
#ifndef PCIDGPExperimentH
#define PCIDGPExperimentH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PCInteractionTool.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PointSetKNNQuery;
class AttachedIndexedOctree;



/// this class is a base class for interactive experiments in the DGP lecture
/// derive a subclass to define your own experiment
///
class GEOXLKINECT_API PCIDGPExperiment : public PCInteractionTool
{
	GEOXL_CLASS( PCIDGPExperiment )
 private:
   PointCloud *packagePointSet(PointSet* ps);
   PointSet *createUnstructuredCopy(PointSet* ps);
 
 public:
	PCIDGPExperiment();


   // ---- defining default methods so that the class is always instantiatable
   //
   
   virtual void mouseDown(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {}
   virtual void mouseMoved(int32 x, int32 y) {}
   virtual void mouseUp(int32 x, int32 y, MouseButtons buttonsState, Modifiers modifiersState) {}
   virtual void areaResize(card32 width, card32 height) {}
   virtual void mouseWheelRotated(int32 rotatedDelta, Modifiers modifiersState) {}
   virtual void keyDown(GeneralKey key) {}
   virtual void keyUp(GeneralKey key) {}


   // ---- helpers for organizing the scene
   //

   /// Find point cloud in scene
   PointSet *getPointCloud(string name);

   /// Replace point cloud with scene graph path 'name' with 'newPC'
   void replacePointCloud(string name, PointSet* newPS );

   /// Replace point cloud 'pcID' with 'newPC' or add if it doesn't exist
   void  addOrReplacePointCloud(string name, PointSet* newPS);

   /// Add point cloud 'pc' as a child of the root node; set node name to 'name'.
   void  addPointCloud(PointSet* newPS, string name);

	/// update the point cloud (required for shader materials after changes)
	void updatePointCloud(string name);

   /// Check for attribute: Search attribute 'attrib' in point cloud 'id' in scene 'pcscn'. If it does not exist, create attribute as (numComponents, dataType)
   void  checkAttribute(string name, string attrib, card8 numComponents, card8 dataType ) ;

	/// Return the average distance between two points of a point cloud
	float getAveragePointDistance(PointSet *ps, card32 numDirectNeighbours = 5);

	/// returns median distance between two points (numSamples == 0 -> all points)
   float getMedianPointDistance(PointSet *ps, card32 numDirectNeighbours = 5, card32 numSamples = 1000);

	/// Delete node with name and path 'name' from scene. Returns true on success.
   bool  deleteSceneGraphNode(string name);

   /// creates a subsampled version of the original point set
   void resamplePointCloud(PointSet *ps, float32 sampleDist);

   /// helper data structure for efficient nearest-neighbor queries
   struct PSKnnQuery {
      PointSetKNNQuery *knn;
      AttachedIndexedOctree *octree;
   };

   /// this method creates a seach data structure that can find the k-nearest neighbors efficiently
   void createPointSetKnnQuery(PointSet *ps, PSKnnQuery &query);
   /// sets the target from where the nearest neighbors should be enumerated
   /// resets the nearest point to the first
   void setKnnTarget(PSKnnQuery &query, const Vector3f &target);
   /// searches for the next nearest point
   void getKnnNextNearestPointIndex(PSKnnQuery &query, card32 &nextIndex, bool &found);
   /// this method deletes the data structure
   void deletePointSetKnnQuery(PSKnnQuery &query);

	virtual ~PCIDGPExperiment();
};



}

#endif