//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "PCIKinectCaptureEditorWidget.h"
//---------------------------------------------------------------------------
#include "PCIKinectCapture.h"
#include "PCIKinectCaptureEditorWidget_moc.cpp"
#include "ExceptionHandling.h"
#include "PointCloudTools.h"
#include "SGListNode.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
#include "QtCore/qtimer.h"
#include "SGRelativeTimeAnimationNode.h"
#include "SceneGraphTools.h"
#include "ParametricInCorePointCloud.h"
#include "SGObjectNode.h"
#include "KinectCapture.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



IMPLEMENT_CLASS_EDITOR( PCIKinectCapture )


PCIKinectCaptureEditorWidget::PCIKinectCaptureEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
   recList = NULL;
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	edObj = NULL;
	timer = new QTimer(this);
	timer->setInterval(20);
	timer->stop();
   QObject::connect(timer, SIGNAL(timeout()), this, SLOT(on_timer()));
}

PCIKinectCaptureEditorWidget::~PCIKinectCaptureEditorWidget(void)
{
}

void PCIKinectCaptureEditorWidget::setup( PCIKinectCapture * obj )
{
   edObj = obj;
}


void PCIKinectCaptureEditorWidget::updateEditor()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}

void PCIKinectCaptureEditorWidget::on_pbStartDev_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
   edObj->startDevice();
   timer->start();
   ui.pbStopDev->setEnabled(true);
   ui.pbRec->setEnabled(true);
   debugOutput << "device started\n";
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}

void PCIKinectCaptureEditorWidget::on_pbStopDev_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
   timer->stop();
   //edObj->stopDevice();
   recList = NULL;
   ui.pbStopDev->setEnabled(false);
   ui.pbRec->setEnabled(false);
   ui.pbStopRec->setEnabled(false);
   debugOutput << "device stopped\n";
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}

void PCIKinectCaptureEditorWidget::on_pbRec_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
   ui.pbStopRec->setEnabled(true);
   string targetFolder = qString2STLString(ui.leTarget->text());
   SGListNode *list = dynamic_cast<SGListNode*>(getSceneGraphNodeByName(edObj->getScene(),targetFolder));
   if (!list) {
      list = new SGRelativeTimeAnimationNode();
      list->setName(removePathFromFilename(targetFolder));
      addSceneGraphNode(edObj->getScene(), removeFilenameFromPath(targetFolder), list);
   }
   recList = list;
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}

void PCIKinectCaptureEditorWidget::on_pbStopRec_clicked()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
   recList = NULL;
   ui.pbStopRec->setEnabled(false);
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}

void PCIKinectCaptureEditorWidget::on_timer()
{
	STD_GLOBAL_EXCEPTION_HANDLER_START
	KinectCapture::ChangeType change;
	const PointSet *ps = (PointSet*)edObj->fetchFrame(change);
   
   if (ps && ps->getWidth() == 640 && ps->getHeight() == 480) {
      AAT positionAAT = ps->getAAT("position");
      AAT colorAAT = ps->getAAT("color");
      QImage colImg(320,240,QImage::Format_RGB32);
      QImage depthImg(320,240,QImage::Format_RGB32);
      for (int y=0; y<480; y+=2) {
         for (int x=0; x<640; x+=2) {
            Vector3f pos = ps->get3f(x,y, positionAAT)/50;
            Vector3f col = ps->get3f(x,y, colorAAT)*255;
            colImg.setPixel(x/2,y/2,qRgb(col[0],col[1],col[2]));
            depthImg.setPixel(x/2,y/2,qRgb(pos[2],pos[2],pos[2]));
         }
      }
      QPixmap colPix = QPixmap::fromImage(colImg);
      QPixmap depthPix = QPixmap::fromImage(depthImg);
      ui.lbColor->setPixmap(colPix);
      ui.lbDepth->setPixmap(depthPix);
      if (recList) {
         if (change == KinectCapture::NewDepth || !ui.cbNewDepth->isChecked()) {
            ParametricInCorePointCloud *newPC = new ParametricInCorePointCloud;
            newPC->clearAndSetup(640, 480, ps->getDescr());
            newPC->setPointSet((PointSet*)ps->copy());
            SGObjectNode *sgo = new SGObjectNode();
            sgo->clearAndSetup(newPC);
            static int index = 1;
            sgo->setName("kinect"+fillNumberStringWithZeros(intToStr(index++),5));
            recList->addChildNode(sgo);
         }
      }
   }
   
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}
