//---------------------------------------------------------------------------
#ifndef GLContextH
#define GLContextH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <list>
#include "cg_include.h"
#include "Persistent.h"
#include "PTypes.h"
#include <string>
#include "StringHelper.h"
//---------------------------------------------------------------------------
#ifndef WIN32
	#include <GL/glx.h>
#endif

class QGLWidget;

namespace geoxl {
   using namespace std;

class GLObject;
class GLRenderTarget;

#define DebugGL( context ) (context)->debug( removePathFromFilename( __FILE__ ) + string(" Line[") + numToStr(__LINE__) + "]" )

class GLTOOLS_API GLContext
{
public:
	/// must have valid dc + rc
	/// parentContext means: this context shares objects with parentContext
	/// there is exactly ONE parent-context(root-context), childs cannot
	/// have childcontexts themselves
#ifdef WIN32
	GLContext(HDC dc, HGLRC rc, GLContext *parentContext=NULL, QGLWidget *glWidget=NULL);
#else
	GLContext(GLXDrawable dc, GLXContext rc, GLContext *parentContext=NULL, QGLWidget *glWidget=NULL);
#endif

	/// if this context is not attached to a parentContext,
	/// then all GL-Objects will be deleted ( in GL! for example glDeleteTextures... )
	~GLContext();

	/// returns CG context
	/// required for CG-shaders
	CGcontext getCGContext() { if(m_ParentContext) return m_ParentContext->getCGContext();
                               else return m_CGContext; }

	GLContext * getRootContext();

	/// checks if the context shares with this context
	bool isCompatible( GLContext * context );

	/// makes context avaible for rendering
	/// removes all deleted gl-objects ( which were deleted while context wasn't active )
	bool     activate();
	void     deactivate();
	bool     isGlobalActive() const;

	void     deleteTexture( GLuint texID );
	void     deleteBuffer( GLuint bufferID );
	void     deleteShader( GLuint shaderID );

	/// reads glGetError() and reports to debugWindow
   	void     debug( string msg );

	bool getUseVertexBufferObjects() const {return m_UseVertexBufferObjects;}
	void setUseVertexBufferObjects(bool v) {m_UseVertexBufferObjects=v;}

	/// returns QGLWidget (return NULL for rendertextures etc.)
	QGLWidget * getQGLWidget() const {return m_qglWidget;}

protected:

   void     setGlobalActive( bool active );

   static card64 globalContextNumber;
   card64 myContextNumber;

   void onDeleteContext();

   /// attach object to the root-context
   /// internal use only!
   list<GLObject *>::iterator addObject( GLObject * );

   /// removes object from root-context's object-list
   void removeObject( list<GLObject *>::iterator );

   /// removes child context from root-context
   void removeChildContext( GLContext * context );

   /// add context to list of child contexts
   void addChildContext(GLContext *context);

#ifdef WIN32
   HDC         		m_DC;
   HGLRC      		m_RC;
#else
   GLXDrawable		m_DC;
   GLXContext		m_RC;
#endif
   static CGcontext	m_CGContext;
   bool        		m_IsGlobalActive;

   /// "one-level-hierarchy"
   list<GLContext *> m_ChildContexts;
   GLContext *m_ParentContext;

	/// hint for using vbo or not (GLShaderMaterial)
	bool m_UseVertexBufferObjects;
	
	QGLWidget *m_qglWidget;


   /// objectlist (empty if this isn't the root-context)
   list<GLObject *> m_Objects;

   /// deletes unused objects
   void delayedDelete();

   /// delete lists
   vector<GLuint>    m_DelayedDeleteTextures,
                     m_DelayedDeleteBuffers,
                     m_DelayedDeleteShaders;
public:
   /// get a unique ID of the context valid during the whole application runtime (also after destruction of contextes)
   card64 getContextNumber() {return myContextNumber;}

#ifdef WIN32
   HDC      getDC() {return m_DC;}
   HGLRC    getRC() {return m_RC;}
#endif


   friend class GLObject;
   friend class GLRenderTarget;
};

};

#endif
