//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
// define prototypes w. "PACKAGE" decl.
#include <glh/glh_genext.h>
#undef GLH_GENEXT_H
#undef GLH_EXTERN
#undef GLH_INITIALIZER

// include code...
#define GLH_EXT_SINGLE_FILE
//#pragma option push -w-pia // avoid warning in header
#ifndef WIN32
	#include <glh/glh_genext.h>
#endif
#include <glh/glh_extensions.h>
//#pragma option pop

#include "GLExtensions.h"
#include "Exceptions.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


bool geoxl::glExtensionsInitialized = false;

void initFBO();
void initAdditionalFunctions();

void geoxl::initGLExtensions() {
   if (!glExtensionsInitialized) {
      glExtensionsInitialized = true;

      glh_init_extensions("GL_NV_vertex_program");
      glh_init_extensions("GL_EXT_vertex_shader");
      glh_init_extensions("GL_HP_occlusion_test");
      glh_init_extensions("GL_EXT_compiled_vertex_array");
      glh_init_extensions("GL_NV_vertex_array_range");
      glh_init_extensions("GL_EXT_point_parameters");
      glh_init_extensions("GL_EXT_texture3D");
      glh_init_extensions("GL_ARB_vertex_program");
      glh_init_extensions("GL_ARB_fragment_program");
      glh_init_extensions("GL_ARB_vertex_buffer_object");
      glh_init_extensions("GL_ARB_multitexture");
      glh_init_extensions("GL_ARB_texture_cube_map");
      glh_init_extensions("WGL_ARB_extension_string");
      glh_init_extensions("WGL_ARB_pixel_format");
      glh_init_extensions("WGL_ARB_pbuffer");

	  // Commented by pj - causing crash
      //glh_init_extensions("WGL_ARB_make_current_read");
      //glh_init_extensions("GL_ATI_draw_buffers");
      initFBO();
		initAdditionalFunctions();
   }
}

#ifdef _WIN32
#define geoxlGetProcAddress(PROC_TYPE, PROC_NAME, PROC_STR) PROC_NAME = (PROC_TYPE)wglGetProcAddress(PROC_STR)
#elif defined(UNIX)
#define geoxlGetProcAddress(PROC_TYPE, PROC_NAME, PROC_STR) PROC_NAME = (PROC_TYPE)glXGetProcAddressARB( (const GLubyte *) PROC_STR)
#endif

PFNGLISRENDERBUFFEREXTPROC glIsRenderbufferEXT = NULL;
PFNGLBINDRENDERBUFFEREXTPROC glBindRenderbufferEXT = NULL;
PFNGLDELETERENDERBUFFERSEXTPROC glDeleteRenderbuffersEXT = NULL;
PFNGLGENRENDERBUFFERSEXTPROC glGenRenderbuffersEXT = NULL;
PFNGLRENDERBUFFERSTORAGEEXTPROC glRenderbufferStorageEXT = NULL;
PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC glGetRenderbufferParameterivEXT = NULL;
PFNGLISFRAMEBUFFEREXTPROC glIsFramebufferEXT = NULL;
PFNGLBINDFRAMEBUFFEREXTPROC glBindFramebufferEXT = NULL;
PFNGLDELETEFRAMEBUFFERSEXTPROC glDeleteFramebuffersEXT = NULL;
PFNGLGENFRAMEBUFFERSEXTPROC glGenFramebuffersEXT = NULL;
PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC glCheckFramebufferStatusEXT = NULL;
PFNGLFRAMEBUFFERTEXTURE1DEXTPROC glFramebufferTexture1DEXT = NULL;
PFNGLFRAMEBUFFERTEXTURE2DEXTPROC glFramebufferTexture2DEXT = NULL;
PFNGLFRAMEBUFFERTEXTURE3DEXTPROC glFramebufferTexture3DEXT = NULL;
PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC glFramebufferRenderbufferEXT = NULL;
PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC glGetFramebufferAttachmentParameterivEXT = NULL;
PFNGLGENERATEMIPMAPEXTPROC glGenerateMipmapEXT = NULL;

//PFNGLVERTEXATTRIB1FARBPROC glVertexAttrib1fARB = NULL;


void initFBO()
{
   geoxlGetProcAddress( PFNGLISRENDERBUFFEREXTPROC, glIsRenderbufferEXT,"glIsRenderbufferEXT" );
   geoxlGetProcAddress( PFNGLBINDRENDERBUFFEREXTPROC, glBindRenderbufferEXT,"glBindRenderbufferEXT" );
   geoxlGetProcAddress( PFNGLDELETERENDERBUFFERSEXTPROC, glDeleteRenderbuffersEXT,"glDeleteRenderbuffersEXT" );
   geoxlGetProcAddress( PFNGLGENRENDERBUFFERSEXTPROC, glGenRenderbuffersEXT,"glGenRenderbuffersEXT" );
   geoxlGetProcAddress( PFNGLRENDERBUFFERSTORAGEEXTPROC, glRenderbufferStorageEXT,"glRenderbufferStorageEXT" );
   geoxlGetProcAddress( PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC, glGetRenderbufferParameterivEXT,"glGetRenderbufferParameterivEXT" );
   geoxlGetProcAddress( PFNGLISFRAMEBUFFEREXTPROC, glIsFramebufferEXT,"glIsFramebufferEXT" );
   geoxlGetProcAddress( PFNGLBINDFRAMEBUFFEREXTPROC, glBindFramebufferEXT,"glBindFramebufferEXT" );
   geoxlGetProcAddress( PFNGLDELETEFRAMEBUFFERSEXTPROC, glDeleteFramebuffersEXT,"glDeleteFramebuffersEXT" );
   geoxlGetProcAddress( PFNGLGENFRAMEBUFFERSEXTPROC, glGenFramebuffersEXT,"glGenFramebuffersEXT" );
   geoxlGetProcAddress( PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC, glCheckFramebufferStatusEXT,"glCheckFramebufferStatusEXT" );
   geoxlGetProcAddress( PFNGLFRAMEBUFFERTEXTURE1DEXTPROC, glFramebufferTexture1DEXT,"glFramebufferTexture1DEXT" );
   geoxlGetProcAddress( PFNGLFRAMEBUFFERTEXTURE2DEXTPROC, glFramebufferTexture2DEXT,"glFramebufferTexture2DEXT" );
   geoxlGetProcAddress( PFNGLFRAMEBUFFERTEXTURE3DEXTPROC, glFramebufferTexture3DEXT,"glFramebufferTexture3DEXT" );
   geoxlGetProcAddress( PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC, glFramebufferRenderbufferEXT,"glFramebufferRenderbufferEXT" );
   geoxlGetProcAddress( PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC, glGetFramebufferAttachmentParameterivEXT,"glGetFramebufferAttachmentParameterivEXT" );
   geoxlGetProcAddress( PFNGLGENERATEMIPMAPEXTPROC, glGenerateMipmapEXT,"glGenerateMipmapEXT" );
}

void initAdditionalFunctions()
{
	void * tmp = (void*)&glVertexAttrib3fvARB;
	if( tmp )
		tmp = NULL;
}

