//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "GLHelper.h"
//---------------------------------------------------------------------------
#include "VertexDescriptor.h"
#include <glh/glh_extensions.h>
#include <QtOpenGL/QGLWidget>
#include "GLContext.h"
//---------------------------------------------------------------------------
using namespace geoxl;


void GLHelper::pushMVP()
{
   glMatrixMode( GL_PROJECTION );
   glPushMatrix();
   glMatrixMode( GL_MODELVIEW );
   glPushMatrix();
}

void GLHelper::popMVP()
{
   glMatrixMode( GL_MODELVIEW );
   glPopMatrix();
   glMatrixMode( GL_PROJECTION );
   glPopMatrix();
   glMatrixMode( GL_MODELVIEW );
}

void GLHelper::begin2D( float width, float height )
{
   pushMVP();

   glMatrixMode( GL_PROJECTION );
   glLoadIdentity();
   gluOrtho2D( 0.0f, width, 0.0f, height );
   glMatrixMode( GL_MODELVIEW );
   glLoadIdentity();
//   glTranslatef( -1.0f, -1.0f, 0 );
//   glScalef( 2.0f / (float)width, 2.0f / (float)height, 1.0f );

}

void GLHelper::end2D()
{
   popMVP();
}

static const float32 coordsAlpha = 0.1f;

void GLHelper::drawCoordsSendVerticesNoXYZNoArrows() {
	glColor4f(1.0f, 0.5f, 0.5f, coordsAlpha);
	glVertex3f(0,0,0);
	glVertex3f(1,0,0);

	glColor4f(0.5f, 1.0f, 0.5f, coordsAlpha);
	glVertex3f(0,0,0);
	glVertex3f(0,1,0);

	glColor4f(0.5f, 0.5f,  1.0f, coordsAlpha);
	glVertex3f(     0, 0,     0);
	glVertex3f(     0, 0,     1);
}

void GLHelper::drawCoordsSendVerticesNoXYZ() {
	glColor4f(1.0f, 0.5f, 0.5f, coordsAlpha);
	glVertex3f(0,0,0);
	glVertex3f(1,0,0);
	glVertex3f(0.95f,0.05f,0);
	glVertex3f(1,0,0);
	glVertex3f(0.95f,-0.05f,0);
	glVertex3f(1,0,0);

	glColor4f(0.5f, 1.0f, 0.5f, coordsAlpha);
	glVertex3f(0,0,0);
	glVertex3f(0,1,0);
	glVertex3f(0.05f,0.95f,0);
	glVertex3f(0,1,0);
	glVertex3f(-0.05f,0.95f,0);
	glVertex3f(0,1,0);

	glColor4f(0.5f, 0.5f,  1.0f, coordsAlpha);
	glVertex3f(     0, 0,     0);
	glVertex3f(     0, 0,     1);
	glVertex3f( 0.05f, 0, 0.95f);
	glVertex3f(     0, 0,     1);
	glVertex3f(-0.05f, 0, 0.95f);
	glVertex3f(     0, 0,     1);
}

void GLHelper::drawCoordsSendVertices() {
	glColor4f(1.0f, 0.5f, 0.5f, coordsAlpha);
	glVertex3f(0,0,0);
	glVertex3f(1,0,0);
	glVertex3f(0.95f,0.05f,0);
	glVertex3f(1,0,0);
	glVertex3f(0.95f,-0.05f,0);
	glVertex3f(1,0,0);

	glVertex3f(0.94f, -0.1f, 0);
	glVertex3f(1.00f, -0.2f, 0);
	glVertex3f(1.00f, -0.1f, 0);
	glVertex3f(0.94f, -0.2f, 0);

	glColor4f(0.5f, 1.0f, 0.5f, coordsAlpha);
	glVertex3f(0,0,0);
	glVertex3f(0,1,0);
	glVertex3f(0.05f,0.95f,0);
	glVertex3f(0,1,0);
	glVertex3f(-0.05f,0.95f,0);
	glVertex3f(0,1,0);

	glVertex3f(-0.15f, 0.97f, 0);
	glVertex3f(-0.1f,  1.00f, 0);
	glVertex3f(-0.1f,  0.94f, 0);
	glVertex3f(-0.15f, 0.97f, 0);
	glVertex3f(-0.15f, 0.97f, 0);
	glVertex3f(-0.2f,  0.97f, 0);

	glColor4f(0.5f, 0.5f,  1.0f, coordsAlpha);
	glVertex3f(     0, 0,     0);
	glVertex3f(     0, 0,     1);
	glVertex3f( 0.05f, 0, 0.95f);
	glVertex3f(     0, 0,     1);
	glVertex3f(-0.05f, 0, 0.95f);
	glVertex3f(     0, 0,     1);

	glVertex3f( -0.2f, 0, 1.00f);
	glVertex3f( -0.2f, 0, 0.94f);
	glVertex3f( -0.2f, 0, 0.94f);
	glVertex3f( -0.1f, 0, 1.00f);
	glVertex3f( -0.1f, 0, 1.00f);
	glVertex3f( -0.1f, 0, 0.94f);
}

void GLHelper::drawCoords() {
		glPushAttrib(GL_ALL_ATTRIB_BITS);
		glDisable(GL_LIGHTING);
		
		glBegin(GL_LINES);
		drawCoordsSendVertices();
		glEnd();
				
		glPopAttrib();
}

void GLHelper::drawBuffer( const char * buffer, card32 bufferSize, 
			const VertexDescriptor * buffer_desc,	const VertexDescriptor * draw_desc, GLenum mode ) 
{
	card32 numElements = bufferSize / buffer_desc->getSize();

	bindSoftwareVertexBuffer( buffer, buffer_desc, draw_desc );
	glDrawArrays( mode, 0, numElements );
	unbindSoftwareVertexBuffer( buffer_desc, draw_desc );

}

void GLHelper::bindSoftwareVertexBuffer( const char * buffer, const VertexDescriptor * buffer_desc,	const VertexDescriptor * draw_desc ) 
{
	for( card32 i=0;i<draw_desc->getNumAttributes();i++ )
	{
		VAD vad = draw_desc->getAttribute(i);
		if( vad.dataFormat != VAD::DATA_FORMAT_FLOAT32 )
			continue;	// ignore non-float attributes

		if( buffer_desc->providesAttribute( vad.name ) )
		{
			const char * ptr = buffer + buffer_desc->getAttributeOffset( buffer_desc->getIndex( vad.name ) );
			glEnableVertexAttribArrayARB( i );
			glVertexAttribPointerARB( i, vad.numComponents, GL_FLOAT, 
				false, // don't normalize
				buffer_desc->getSize(), // stride
				ptr );
		}
	}

}

void GLHelper::unbindSoftwareVertexBuffer(const VertexDescriptor * buffer_desc,	const VertexDescriptor * draw_desc) 
{
	for( card32 i=0;i<draw_desc->getNumAttributes();i++ )
	{
		glDisableVertexAttribArrayARB( i );
	}
}

void geoxl::GLHelper::drawText( GLContext * context, const Vector3f &pos, const string &text )
{
	context->getQGLWidget()->renderText( pos[0],pos[1],pos[2],QString( text.c_str()));
}