//---------------------------------------------------------------------------
#ifndef GLHelperH
#define GLHelperH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------

namespace geoxl
{

/**
     provides some helpfull tools for gl
*/
class VertexDescriptor;
class GLContext;

class GLTOOLS_API GLHelper
{

 public:

	/// pushes modelview & projection matrix
	static void pushMVP();

	/// pops modelview & projection matrix
	static void popMVP();


	/// pushes mvp and sets up a orthogonal projection
	/// doesn't changes current viewport !
	static void begin2D( float width, float height );

	/// restores mvp before begin2D call
	static void end2D();

	/// draw an rgb-colored coordinate system
	static void drawCoordsSendVerticesNoXYZNoArrows();

	/// draw an rgb-colored coordinate system
	static void drawCoordsSendVerticesNoXYZ();

	/// draw an rgb-colored coordinate system
	static void drawCoordsSendVertices();

	/// draw an rgb-colored coordinate system
	static void drawCoords();

	/// render text at position pos (works for QGLWidgets-only)
	static void drawText(GLContext * context, const Vector3f &pos, const string &text);

	/// draws a buffer without using VBO
	/// buffer_desc describes buffer vertex format, draw_desc describes the
	/// vertex format defined in a shader
	static void drawBuffer( const char * buffer, card32 bufferSize, 
		const VertexDescriptor * buffer_desc,	const VertexDescriptor * draw_desc, GLenum mode );

	/// binds a software vertex buffers
	static void bindSoftwareVertexBuffer( const char * buffer, const VertexDescriptor * buffer_desc,	const VertexDescriptor * draw_desc );

	/// unbinds a software vertex buffers
	static void unbindSoftwareVertexBuffer(const VertexDescriptor * buffer_desc,	const VertexDescriptor * draw_desc);

};



}

#endif
