//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "GLImage.h"
//---------------------------------------------------------------------------
#include "GLRenderTextureFBO.h"
#include "GLRenderTextureCache.h"
#include "RGBAImage.h"
#include "GLHelper.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;


GLImage::GLImage()
{
   m_RTCache = NULL;
   m_RenderTexture = NULL;
}

GLImage::~GLImage()
{
   deleteRenderTexture();
}

void GLImage::setup( GLRenderTextureCache * cache,card32 width, card32 height,
      card32 numComponents, bool isRect, bool isFloat , bool isFloat32 ,
      bool hasDepth , bool hasStencil )
{
   deleteRenderTexture();
   m_RTCache = cache;
   m_RenderTexture = cache->getRenderTexture( width, height, numComponents,
      isRect, isFloat, isFloat32, hasDepth, hasStencil );
	m_IsOwner = true;
}

void GLImage::setup( GLRenderTextureFBO * renderTexture, bool isOwner )
{
   deleteRenderTexture();
   m_RenderTexture = renderTexture;
	m_IsOwner = isOwner;
}

void GLImage::setup( GLRenderTextureCache * cache, RGBAImage * image )
{
   setup( cache, image->getSizeX(), image->getSizeY(), 4, true, false, false, false, false );
   m_RenderTexture->bind(0);
   glTexSubImage2D( m_RenderTexture->getTextureTarget(),
      0, 0,0, image->getSizeX(), image->getSizeY(), GL_RGBA, GL_UNSIGNED_BYTE,
      image->getPixels() );
   glTexParameteri( m_RenderTexture->getTextureTarget(), GL_TEXTURE_MIN_FILTER, GL_NEAREST );
   glTexParameteri( m_RenderTexture->getTextureTarget(), GL_TEXTURE_MAG_FILTER, GL_NEAREST );
   m_RenderTexture->unbind();
	m_IsOwner = true;
}

void GLImage::setup( GLRenderTextureCache * cache, RGBAFloat32Image * image, bool isFloat32 )
{
   setup( cache, image->getSizeX(), image->getSizeY(), 4, true, true, isFloat32, false, false );
   m_RenderTexture->bind(0);
   glTexSubImage2D( m_RenderTexture->getTextureTarget(),
      0, 0,0, image->getSizeX(), image->getSizeY(), GL_RGBA, GL_FLOAT,
      image->getPixels() );
   glTexParameteri( m_RenderTexture->getTextureTarget(), GL_TEXTURE_MIN_FILTER, GL_NEAREST );
   glTexParameteri( m_RenderTexture->getTextureTarget(), GL_TEXTURE_MAG_FILTER, GL_NEAREST );
   m_RenderTexture->unbind();
	m_IsOwner = true;
}

void GLImage::setup( GLRenderTextureCache * cache, GLImage * refImage )
{
   setup( cache, refImage->getWidth(), refImage->getHeight(), refImage->getNumComponents(),
      refImage->isRectangle(), refImage->isFloat(), refImage->isFloat32(),
      refImage->hasDepth(), refImage->hasStencil() );
	m_IsOwner = true;
}


void GLImage::deleteRenderTexture()
{
	if( m_IsOwner )
	{
		if( m_RenderTexture != NULL)
		{
			if( m_RTCache )
				m_RTCache->freeRenderTexture( m_RenderTexture );
			else
				delete m_RenderTexture;
		}
	}
   m_RenderTexture = NULL;
   m_RTCache = NULL;
}

RGBAImage * GLImage::getRGBAImage()
{
   RGBAImage * ret = new RGBAImage;
   ret->resize( getWidth(), getHeight() );
   m_RenderTexture->bind(0);
   glGetTexImage( m_RenderTexture->getTextureTarget(),0, GL_RGBA, GL_UNSIGNED_BYTE,
      ret->getPixels() );
   m_RenderTexture->unbind();
   return ret;
}

RGBAFloat32Image * GLImage::getRGBAFloat32Image()
{
   RGBAFloat32Image * ret = new RGBAFloat32Image;
   ret->resize( getWidth(), getHeight() );
   m_RenderTexture->bind(0);
   glGetTexImage( m_RenderTexture->getTextureTarget(),0, GL_RGBA, GL_FLOAT,
      ret->getPixels() );
   m_RenderTexture->unbind();
   return ret;
}


void GLImage::copySubImage( GLImage * sourceImage, card32 srcX, card32 srcY, card32 width, card32 height,
        card32 destX, card32 destY )
{
   pAssert( !m_RenderTexture->isBound() );
   pAssert( !m_RenderTexture->isActive() );
   pAssert( !sourceImage->m_RenderTexture->isBound() );
   pAssert( !sourceImage->m_RenderTexture->isActive() );

   activate();

   sourceImage->bind(0);

   pAssert( false );
   // TODO

   sourceImage->unbind();

   deactivate();
}


void GLImage::activate()
{
   pAssert( m_RenderTexture != NULL );
   m_RenderTexture->activate();
}

void GLImage::deactivate()
{
   pAssert( m_RenderTexture != NULL );
   m_RenderTexture->deactivate();
}


void GLImage::bind(card32 stage)
{
   pAssert( m_RenderTexture != NULL );
   m_RenderTexture->bind(stage);
}

void GLImage::unbind()
{
   pAssert( m_RenderTexture != NULL );
   m_RenderTexture->unbind();
}



card32 GLImage::getWidth()
{
   return m_RenderTexture->getWidth();
}

card32 GLImage::getHeight()
{
   return m_RenderTexture->getHeight();
}

card32 GLImage::getNumComponents()
{
   return m_RenderTexture->getNumComponents();
}

bool GLImage::isRectangle()
{
   return m_RenderTexture->isRectangle();
}

bool GLImage::isFloat()   // float16 || float32
{
   return m_RenderTexture->isFloat16() || m_RenderTexture->isFloat32();
}

bool GLImage::isFloat16()
{
   return m_RenderTexture->isFloat16();
}

bool GLImage::isFloat32()
{
   return m_RenderTexture->isFloat32();
}

bool GLImage::hasDepth()
{
   return m_RenderTexture->hasDepth();
}

bool GLImage::hasStencil()
{
   return m_RenderTexture->hasDepth();
}


void GLImage::drawToScreen()
{
   int vp[4];
   glGetIntegerv( GL_VIEWPORT, vp );
   card32 width = vp[2]-vp[0];
   card32 height = vp[3]-vp[1];

   glPushAttrib( GL_ALL_ATTRIB_BITS );
   GLHelper::begin2D(width,height);

   bind(0);
   glEnable( getRenderTexture()->getTextureTarget() );
   glDisable( GL_DEPTH_TEST );

   glBegin( GL_QUADS );
   glColor3f(1,1,1);
   glTexCoord2f( 0,0 );
   glVertex2f(0,0);
   glTexCoord2f( getWidth(),0 );
   glVertex2f(width,0);
   glTexCoord2f( getWidth(),getHeight() );
   glVertex2f(width,height);
   glTexCoord2f( 0,getHeight() );
   glVertex2f(0,height);
   glEnd();


   unbind();
   GLHelper::end2D();
   glPopAttrib();
}

bool GLImage::isPowerOfTwo()
{
   card32 w = 1;
   while( w < m_RenderTexture->getWidth() )
      w *= 2;
   if( w != m_RenderTexture->getWidth() )
      return false;

   card32 h = 1;
   while( h < m_RenderTexture->getHeight() )
      h *= 2;
   if( h != m_RenderTexture->getHeight() )
      return false;

   return true;
}

bool GLImage::isSquare()
{
   return (m_RenderTexture->getWidth() ==  m_RenderTexture->getHeight());
}

