//---------------------------------------------------------------------------
#ifndef GLImageH
#define GLImageH
//---------------------------------------------------------------------------

namespace geoxl
{

class GLRenderTextureFBO;
class GLRenderTextureCache;
class RGBAImage;
class RGBAFloat32Image;

class GLTOOLS_API GLImage
{
private:

   void deleteRenderTexture();


   GLRenderTextureCache    *  m_RTCache;
   GLRenderTextureFBO      *  m_RenderTexture;
	bool								m_IsOwner;

public:

   GLImage();
   ~GLImage();

   /// setup rendertexture from cache with specified attrbiutes
   void setup( GLRenderTextureCache * cache,card32 width, card32 height,
      card32 numComponents, bool isRect = true,bool isFloat = false,
      bool isFloat32 = false, bool hasDepth = false, bool hasStencil = false );


   /// setups image from existing render texture, choose this method if you
   /// want to use a un-cached render texture
   /// GLImage becomes owner of renderTexture if owner is true
   void setup( GLRenderTextureFBO * renderTexture, bool isOwner );

   /// creates image from RGBAImage, no depth or stencil
   void setup( GLRenderTextureCache * cache, RGBAImage * source );

   /// creates image from RGBAFloat32Image, no depth or stencil
   void setup( GLRenderTextureCache * cache, RGBAFloat32Image * source, bool isFloat32 = false );

   /// creates image with refImage's parameters ( width, height, numComp, ... )
   void setup( GLRenderTextureCache * cache, GLImage * refImage );

   /// returns rendertexture for advanced access
   GLRenderTextureFBO * getRenderTexture() {return m_RenderTexture;}

   /// reads rendertexture's contents into a RGBA-image
   RGBAImage * getRGBAImage();

   /// reads rendertexture's contents into a RGBA-Float32-image
   RGBAFloat32Image * getRGBAFloat32Image();

   /// draws image to current gl window
   void drawToScreen();

   /// copy rect from image into this image unfinished -> TODO!
   void copySubImage( GLImage * sourceImage, card32 srcX, card32 srcY, card32 width, card32 height,
        card32 destX, card32 destY );


   void activate();
   void deactivate();

   void bind(card32 stage);
   void unbind();

   card32 getWidth();
   card32 getHeight();
   card32 getNumComponents();
   bool isRectangle();
   bool isFloat();   // float16 || float32
   bool isFloat16();
   bool isFloat32();
   bool hasDepth();
   bool hasStencil();

   bool isPowerOfTwo();
   bool isSquare();


};


}

#endif
 