//---------------------------------------------------------------------------
#ifndef GLRenderTextureCacheH
#define GLRenderTextureCacheH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <list>
#include <map>
//---------------------------------------------------------------------------

namespace geoxl
{
   using namespace std;

class GLContext;
class GLRenderTextureFBO;

class GLTOOLS_API GLRenderTextureCache
{
private:

   struct TRenderTexture
   {
      GLRenderTextureFBO   *  rt;
      card32   desc;    // 'hash'
   };

   card32 getDesc( card32 width, card32 height, card32 numComponents,
       bool isRect , bool isFloat , bool isFloat32 ,
       bool hasDepth , bool hasStencil );

   typedef list<TRenderTexture>              RTList;
   typedef list<TRenderTexture>::iterator    RTListIterator;


   GLContext   *  m_MyContext;

   RTList         m_Used, m_Unused;
   map<GLRenderTextureFBO*,RTListIterator>   m_IndexMap;

public:

    GLRenderTextureCache( GLContext * context );
    ~GLRenderTextureCache();


    /// returns a rendertexture with specified attributes
    GLRenderTextureFBO * getRenderTexture( card32 width, card32 height, card32 numComponents,
       bool isRect = true, bool isFloat = false, bool isFloat32 = false,
       bool hasDepth = false, bool hasStencil = false );

    /// marks render texture as not used
    void freeRenderTexture( GLRenderTextureFBO * rt );

    /// deletes every unused render texture
    void clearUnused();

    /// deletes every texture
    void clearAll();


    GLContext * getContext() {return m_MyContext;}


};


}

#endif
