//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "GLRenderTextureFBO.h"
//---------------------------------------------------------------------------
#include "GLExtensions.h"
#include "GLTexture.h"
#include "GLContext.h"
//---------------------------------------------------------------------------
 

using namespace geoxl;

GLRenderTextureFBO::GLRenderTextureFBO( GLContext * context )
 : GLObject( context )
{
   m_Texture = NULL;
   m_FrameBufferID = 0;
   m_DepthRenderBufferID = 0;
   m_StencilRenderBufferID = 0;
   m_Status = None;
	m_PureDepthTexture = false;
}

GLRenderTextureFBO::~GLRenderTextureFBO()
{
   delete m_Texture;

   if( isValid() )
      if( m_FrameBufferID )
         glDeleteFramebuffersEXT( 1, &m_FrameBufferID );
}

bool GLRenderTextureFBO::create( card32 width, card32 height, card32 numComponents, card32 flags )
{
   // check everything
   pAssert( m_Texture == NULL );
   pAssert( m_FrameBufferID == 0 );
   if( (flags & FLAG_RECTANGLE) == 0 )
   {
      card32 testw = width;
      while( testw > 1 )
      {
         pAssert( (testw & 1) == 0 );
         testw /= 2;
      }
      card32 testh = height;
      while( testh > 1 )
      {
         pAssert( (testh & 1) == 0 );
         testh /= 2;
      }
   }

   pAssert( (flags & FLAG_FLOAT32) == 0 );

   m_Width = width;
   m_Height = height;
   m_NumComponents = numComponents;
   m_Flags = flags;


   // generate texture
   if( flags & FLAG_RECTANGLE )
   {
      m_Texture = new GLTextureRect( getContext() );
      if( (flags & FLAG_FLOAT16) || (flags & FLAG_FLOAT32))
         ((GLTextureRect*)m_Texture)->createFloatTextureFromMemory( width, height, numComponents, NULL,
            (flags & FLAG_FLOAT32 ) ? true : false );
      else
         ((GLTextureRect*)m_Texture)->createFromMemory( width, height, numComponents, NULL );
      glTexParameteri( m_Texture->getTextureTarget(), GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
   }
   else
   {
      m_Texture = new GLTexture2D( getContext() );
      pAssert( (flags & FLAG_FLOAT16) == 0 );  // TODO
      pAssert( (flags & FLAG_FLOAT32) == 0 );  // TODO
      ((GLTexture2D*)m_Texture)->createFromMemory( width, height, numComponents, NULL, false );
   }


   glGenFramebuffersEXT( 1, &m_FrameBufferID );
   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, m_FrameBufferID );

   // attach texture to buffer
   glFramebufferTexture2DEXT(    GL_FRAMEBUFFER_EXT,
                                 GL_COLOR_ATTACHMENT0_EXT,
                                 m_Texture->getTextureTarget(),
                                 m_Texture->getTextureID(), 0);

   // attach depth buffer
   if( flags & FLAG_DEPTH )
   {
      glGenRenderbuffersEXT( 1, &m_DepthRenderBufferID );
      glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, m_DepthRenderBufferID);
      glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT,
                               GL_DEPTH_COMPONENT24, width, height);
      glFramebufferRenderbufferEXT( GL_FRAMEBUFFER_EXT,
                                    GL_DEPTH_ATTACHMENT_EXT,
                                    GL_RENDERBUFFER_EXT,
                                    m_DepthRenderBufferID);

   }

   // attach stencil buffer
   if( flags & FLAG_STENCIL )
   {
      glGenRenderbuffersEXT( 1, &m_StencilRenderBufferID );
      glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, m_StencilRenderBufferID);
      glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT,
                               GL_STENCIL_INDEX, width, height);
      glFramebufferRenderbufferEXT( GL_FRAMEBUFFER_EXT,
                                    GL_STENCIL_ATTACHMENT_EXT,
                                    GL_RENDERBUFFER_EXT,
                                    m_StencilRenderBufferID);

   }

   if( !checkStatus() )
      return false;

   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

   return true;
}


bool GLRenderTextureFBO::createDepthTexture( card32 width, card32 height ) 
{
	m_Flags = 0;
	m_PureDepthTexture = true;
   card32 testw = width;
   while( testw > 1 )
   {
      if( (testw & 1) != 0 )
			return false;
      testw /= 2;
   }
   card32 testh = height;
   while( testh > 1 )
   {
      if( (testh & 1) != 0 )
			return false;
      testh /= 2;
   }

   m_Width = width;
   m_Height = height;
   m_NumComponents = 0;
   m_Flags = FLAG_DEPTH;

	m_Texture = new GLTexture2D( getContext() );
	m_Texture->bind();
	glTexImage2D( m_Texture->getTextureTarget(), 0, GL_DEPTH_COMPONENT, width, height, 0, GL_DEPTH_COMPONENT,
		GL_FLOAT, NULL );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
	m_Texture->unbind();

	glGenFramebuffersEXT( 1, &m_FrameBufferID );
   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, m_FrameBufferID );

   // attach texture to buffer
   glFramebufferTexture2DEXT(    GL_FRAMEBUFFER_EXT,
                                 GL_DEPTH_ATTACHMENT_EXT,
                                 m_Texture->getTextureTarget(),
                                 m_Texture->getTextureID(), 0);
	glDrawBuffer(GL_NONE);
	glReadBuffer(GL_NONE);

   if( !checkStatus() )
      return false;

   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

   return true;
}

bool GLRenderTextureFBO::checkStatus()
{
    GLenum status;
    status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
    if( status ==  GL_FRAMEBUFFER_COMPLETE_EXT )
         return true;

    return false;
}

void GLRenderTextureFBO::activate()
{
   pAssert( m_Status == None );
   pAssert( m_Texture != NULL );

   glGetIntegerv( GL_VIEWPORT, m_OldViewport );

   m_Status = Active;
   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, m_FrameBufferID );

   glViewport(0,0, m_Width, m_Height);

}

void GLRenderTextureFBO::deactivate()
{
   pAssert( m_Status == Active );
   glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
   m_Status = None;
   glViewport(m_OldViewport[0], m_OldViewport[1], m_OldViewport[2], m_OldViewport[3]);
}

void GLRenderTextureFBO::bind( card32 stage )
{
   pAssert( m_Status == None );
   pAssert( m_Texture != NULL );

   m_LastStage = stage;
   m_Texture->bind( stage );
   m_Status = Bound;
}

void GLRenderTextureFBO::unbind()
{
   pAssert( m_Status == Bound );
   m_Texture->unbind( m_LastStage );
   m_Status = None;
}

GLenum GLRenderTextureFBO::getTextureTarget()
{
   pAssert( m_Texture != NULL );
   return m_Texture->getTextureTarget();
}

RGBAImage * GLRenderTextureFBO::getRGBAImage()
{
	bool wasActive = isActive();
	bool wasBound = isBound();

	if( isActive() )
		deactivate();

	RGBAImage * ret = new RGBAImage;
	ret->resize( getWidth(), getHeight() );
	if( !isBound())
		bind(0);
	glGetTexImage( getTextureTarget(),0, GL_RGBA, GL_UNSIGNED_BYTE,
		ret->getPixels() );
	if( !wasBound )
		unbind();

	if( wasActive )
		activate();
	
	return ret;
}

RGBAFloat32Image * geoxl::GLRenderTextureFBO::getRGBAFloat32Image()
{
	bool wasActive = isActive();
	bool wasBound = isBound();

	if( isActive() )
		deactivate();

	RGBAFloat32Image * ret = new RGBAFloat32Image;
	ret->resize( getWidth(), getHeight() );
	if( !isBound())
		bind(0);
	glGetTexImage( getTextureTarget(),0, GL_RGBA, GL_FLOAT,
		ret->getPixels() );
	if( !wasBound )
		unbind();

	if( wasActive )
		activate();

	return ret;
}