//---------------------------------------------------------------------------
#ifndef GLRenderTextureFBOH
#define GLRenderTextureFBOH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "GLObject.h"
//---------------------------------------------------------------------------


namespace geoxl
{

class GLContext;
class GLTexture;
class RGBAImage;
class RGBAFloat32Image;

class GLTOOLS_API GLRenderTextureFBO : public GLObject
{
private:

   bool checkStatus();

   GLTexture   *  m_Texture;
   GLuint         m_FrameBufferID;
   GLuint         m_DepthRenderBufferID;
   GLuint         m_StencilRenderBufferID;
   card32         m_LastStage;

   card32         m_Width, m_Height;
   card32         m_NumComponents;
   card32         m_Flags;

   GLint          m_OldViewport[4];

   enum EStatus
   {
      None,
      Active,
      Bound
   };

   EStatus        m_Status;
	bool				m_PureDepthTexture;

public:

   GLRenderTextureFBO( GLContext * context );
   ~GLRenderTextureFBO();


   enum Flags {   FLAG_DEPTH = 1,
                  FLAG_STENCIL = 2,
                  FLAG_FLOAT16 = 4,
                  FLAG_FLOAT32 = 8,
                  FLAG_RECTANGLE = 16
                   };


   /// create render texture
   bool create( card32 width, card32 height, card32 numComponents, card32 flags = 0 );

	/// create depth texture (power of two only!)
	bool createDepthTexture( card32 width, card32 height );


   /// activate rendering into texture, exception if texture is bound
   void activate();
   /// deactivate rendering into texture
   void deactivate();



   /// bind texture, exception if active for rendering
   void bind( card32 stage = 0 );

   /// unbind texture
   void unbind();

   enum EFormat {
      F_RGBA,
      F_Depth,
      F_Stencil };

   /// copy current bufferdata rect to ptr
   /// must NOT be in render-into-texture mode or texture-bound mode
   /// this is very slow, don't expect to much speed
   //void copyToBuffer( char * ptr, EFormat copymode = RGBA, unsigned int x=0, unsigned int y=0, unsigned int width=0, unsigned int height=0);



   GLenum getTextureTarget();
   inline card32 getWidth() const { return m_Width; }
   inline card32 getHeight() const { return m_Height; }
   inline card32 getNumComponents() const { return m_NumComponents; }
   inline bool   hasDepth() const { return (m_Flags & FLAG_DEPTH) ? true : false; }
   inline bool   hasStencil() const { return (m_Flags & FLAG_STENCIL) ? true : false; }
   inline bool   isFloat16() const { return (m_Flags & FLAG_FLOAT16) ? true : false; }
   inline bool   isFloat32() const { return (m_Flags & FLAG_FLOAT32) ? true : false; }
   inline bool   isRectangle() const { return (m_Flags & FLAG_RECTANGLE) ? true : false; }
	inline bool   isPureDepthTexture() const {return m_PureDepthTexture;}

   inline bool    isActive() const {return m_Status == Active;}
   inline bool    isBound() const {return m_Status == Bound;}


	RGBAImage * getRGBAImage();
	RGBAFloat32Image * getRGBAFloat32Image();

};


}

#endif
