//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "GLSimpleObjects.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra_templCode.h"
#include "BoundingBox_templCode.h"
#include "Polygon_templCode.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------

using namespace std;
using namespace geoxl;

GLSimpleObjectContainer::~GLSimpleObjectContainer()
{
   clear(0);
}

void GLSimpleObjectContainer::drawObjects( GLContext * context, card32 classID )
{
   for( card32 i=0;i<m_Objects.size();i++ )
      if( m_Objects[i]->getClassID() == classID || classID == 0 )
         m_Objects[i]->draw( context, this );
}

void GLSimpleObjectContainer::clear(card32 classID)
{
   vector<GLSimpleObject*> newObjects;
   for( card32 i=0;i<m_Objects.size();i++ )
   {
      if( m_Objects[i]->getClassID() == classID || classID == 0 )
      {
         delete m_Objects[i];
      }
      else
         newObjects.push_back( m_Objects[i] );
   }
   m_Objects = newObjects;
}

void GLSimpleObjectContainer::addBox( const BoundingBox3f &bb, card32 id )
{
   GLSimpleBox * box = new GLSimpleBox;
   box->setClassID( id );
   box->setup( bb );
   m_Objects.push_back( box );
}

void GLSimpleObjectContainer::addTriangle( const Vector3f &v1,const Vector3f &v2,const Vector3f &v3, card32 id,bool useNormalAsColor )
{
   GLSimpleMesh * mesh = new GLSimpleMesh;
   mesh->setClassID( id );
   mesh->addTriangle( v1,v2,v3 );
   mesh->setUseNormalAsColor( useNormalAsColor );
   m_Objects.push_back( mesh );
}

void GLSimpleObjectContainer::addLine( const Vector3f &v1, const Vector3f &v2, card32 id )
{
   GLSimpleLines * lines = new GLSimpleLines;
   lines->addLine( v1,v2);
   lines->setClassID( id );
   m_Objects.push_back( lines );
}



void GLSimpleBox::setup( const Vector3f &lowerCorner, const Vector3f &upperCorner )
{
   m_Box.lowerCorner = lowerCorner;
   m_Box.upperCorner = upperCorner;
}

void GLSimpleBox::setup( const BoundingBox3f &bb )
{
   m_Box = bb;
}

void GLSimpleBox::draw( GLContext * context, GLSimpleObjectContainer * container )
{
   glBegin( GL_QUADS );
      glNormal3f( -1, 0, 0 );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.lowerCorner[1],   m_Box.lowerCorner[2] );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.upperCorner[1],   m_Box.lowerCorner[2] );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.upperCorner[1],   m_Box.upperCorner[2] );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.lowerCorner[1],   m_Box.upperCorner[2] );

      glNormal3f( 1, 0, 0 );
      glVertex3f( m_Box.upperCorner[0],   m_Box.lowerCorner[1],   m_Box.lowerCorner[2] );
      glVertex3f( m_Box.upperCorner[0],   m_Box.lowerCorner[1],   m_Box.upperCorner[2] );
      glVertex3f( m_Box.upperCorner[0],   m_Box.upperCorner[1],   m_Box.upperCorner[2] );
      glVertex3f( m_Box.upperCorner[0],   m_Box.upperCorner[1],   m_Box.lowerCorner[2] );

      glNormal3f( 0, -1, 0 );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.lowerCorner[1],   m_Box.lowerCorner[2] );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.lowerCorner[1],   m_Box.upperCorner[2] );
      glVertex3f( m_Box.upperCorner[0],   m_Box.lowerCorner[1],   m_Box.upperCorner[2] );
      glVertex3f( m_Box.upperCorner[0],   m_Box.lowerCorner[1],   m_Box.lowerCorner[2] );

      glNormal3f( 0, 1, 0 );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.upperCorner[1],   m_Box.lowerCorner[2] );
      glVertex3f( m_Box.upperCorner[0],   m_Box.upperCorner[1],   m_Box.lowerCorner[2] );
      glVertex3f( m_Box.upperCorner[0],   m_Box.upperCorner[1],   m_Box.upperCorner[2] );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.upperCorner[1],   m_Box.upperCorner[2] );

      glNormal3f( 0, 0, -1 );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.lowerCorner[1],   m_Box.lowerCorner[2] );
      glVertex3f( m_Box.upperCorner[0],   m_Box.lowerCorner[1],   m_Box.lowerCorner[2] );
      glVertex3f( m_Box.upperCorner[0],   m_Box.upperCorner[1],   m_Box.lowerCorner[2] );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.upperCorner[1],   m_Box.lowerCorner[2] );

      glNormal3f( 0, 0,  1 );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.lowerCorner[1],   m_Box.upperCorner[2] );
      glVertex3f( m_Box.lowerCorner[0],   m_Box.upperCorner[1],   m_Box.upperCorner[2] );
      glVertex3f( m_Box.upperCorner[0],   m_Box.upperCorner[1],   m_Box.upperCorner[2] );
      glVertex3f( m_Box.upperCorner[0],   m_Box.lowerCorner[1],   m_Box.upperCorner[2] );
   glEnd();

   forget(context);
   forget(container);
}

void GLSimpleLines::addLine( const Vector3f v1,const Vector3f v2 )
{
   m_Points.push_back( v1 );
   m_Points.push_back( v2 );
}

void GLSimpleLines::draw( GLContext * context, GLSimpleObjectContainer * container )
{
   /*glEnableClientState( GL_VERTEX_ARRAY );
   glVertexPointer( 3, GL_FLOAT, 0, &m_Points[0] );
   glDrawArrays( GL_LINES, 0, m_Points.size() );
   glDisableClientState( GL_VERTEX_ARRAY );*/
   glBegin( GL_LINES );
      for( card32 i=0;i<m_Points.size();i++ )
         glVertex3fv( &m_Points[i][0] );
   glEnd();

   forget(context);
   forget(container);
}



void GLSimpleMesh::draw( GLContext * context, GLSimpleObjectContainer * container )
{
   if( !m_PolygonMode )
   {
      glBegin( GL_TRIANGLES );
      if( !m_UseNormalAsColor )
      {
         for( card32 i=0;i<m_Mesh.size();i++ )
         {
            for( card32 j=0;j<m_Mesh[i].getNumVertices()-2;j++ )
            {
               glVertex3fv( (float *)&m_Mesh[i][j] );
               glVertex3fv( (float *)&m_Mesh[i][j+1] );
               glVertex3fv( (float *)&m_Mesh[i][j+2] );
            }

         }
      }
      else
      {
         for( card32 i=0;i<m_Mesh.size();i++ )
         {
            bool valid = false;
            Vector3f normal = m_Mesh[i].calculateNormal(valid);
            normal *= 0.5f;
            normal += makeVector3f(0.5f,0.5f,0.5f);
            glColor3fv( (float *) &normal[0] );
            for( card32 j=0;j<m_Mesh[i].getNumVertices()-2;j++ )
            {
               glVertex3fv( (float *)&m_Mesh[i][j] );
               glVertex3fv( (float *)&m_Mesh[i][j+1] );
               glVertex3fv( (float *)&m_Mesh[i][j+2] );
            }

         }
      }
      glEnd();
   }
   else
   {
      for( card32 i=0;i<m_Mesh.size();i++ )
      {
         glBegin( GL_POLYGON );
         for( card32 j=0;j<m_Mesh[i].getNumVertices();j++ )
            glVertex3fv( (float *)&m_Mesh[i][j] );
         glEnd();
      }
   }
   forget(context);
   forget(container);
}

void GLSimpleMesh::addTriangle( const Vector3f &v1,const Vector3f &v2,const Vector3f &v3 )
{
   SimplePolygon p;
   p.addVertex( PolyVector3f::create( v1 ) );
   p.addVertex( PolyVector3f::create( v2 ) );
   p.addVertex( PolyVector3f::create( v3 ) );
   m_Mesh.push_back(p);
}

void GLSimpleMesh::addQuad( const Vector3f &v1,const Vector3f &v2,const Vector3f &v3,const Vector3f &v4 )
{
   SimplePolygon p;
   p.addVertex( PolyVector3f::create( v1 ) );
   p.addVertex( PolyVector3f::create( v2 ) );
   p.addVertex( PolyVector3f::create( v3 ) );
   p.addVertex( PolyVector3f::create( v4 ) );
   m_Mesh.push_back(p);
}

void GLSimpleMesh::beginPoly()
{
   m_ActPoly.clear();
}

void GLSimpleMesh::addPolyVertex( const Vector3f &v )
{
   m_ActPoly.addVertex( PolyVector3f::create( v ) );
}

void GLSimpleMesh::endPoly()
{
   m_Mesh.push_back( m_ActPoly );
   m_ActPoly.clear();
}

