//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "GLTexture.h"
//---------------------------------------------------------------------------
#include <GL/gl.h>
#include <glh/glh_extensions.h>
#include <gl/glext.h>
#include "GLContext.h"
#include "GLObject.h"
#include "RGBAImage.h"
#include "Cubemap.h"
#include "NumericalClassProperty.h"
#include "NumericalClassProperty_templCode.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

#ifndef GL_ARB_texture_float
#define GL_TEXTURE_RED_TYPE_ARB           0x8C10
#define GL_TEXTURE_GREEN_TYPE_ARB         0x8C11
#define GL_TEXTURE_BLUE_TYPE_ARB          0x8C12
#define GL_TEXTURE_ALPHA_TYPE_ARB         0x8C13
#define GL_TEXTURE_LUMINANCE_TYPE_ARB     0x8C14
#define GL_TEXTURE_INTENSITY_TYPE_ARB     0x8C15
#define GL_TEXTURE_DEPTH_TYPE_ARB         0x8C16
#define GL_UNSIGNED_NORMALIZED_ARB        0x8C17
#define GL_RGBA32F_ARB                    0x8814
#define GL_RGB32F_ARB                     0x8815
#define GL_ALPHA32F_ARB                   0x8816
#define GL_INTENSITY32F_ARB               0x8817
#define GL_LUMINANCE32F_ARB               0x8818
#define GL_LUMINANCE_ALPHA32F_ARB         0x8819
#define GL_RGBA16F_ARB                    0x881A
#define GL_RGB16F_ARB                     0x881B
#define GL_ALPHA16F_ARB                   0x881C
#define GL_INTENSITY16F_ARB               0x881D
#define GL_LUMINANCE16F_ARB               0x881E
#define GL_LUMINANCE_ALPHA16F_ARB         0x881F
#endif

MetaClass* GLTextureSettings::GLTextureSettingsMetaClass = NULL;

const MetaClass* GLTextureSettings::getInstanceClass() const {
   return getClass();
}

Object* GLTextureSettings::newInstance() {
   return new GLTextureSettings;
}

const MetaClass* GLTextureSettings::getClass() {
   if (GLTextureSettingsMetaClass==NULL) {
      GLTextureSettingsMetaClass = new MetaClass("GLTextureSettings", "no description.",
                                  newInstance, sizeof(GLTextureSettings));
   }
   return GLTextureSettingsMetaClass;
}

string desc_filter( card32 i )
{
   switch(i)
   {
   case GLTextureSettings::FILTER_NEAREST:
      return "nearest";
   case GLTextureSettings::FILTER_LINEAR:
      return "linear";
   }

   return "unkown";
}

string desc_wrap( card32 i )
{
   switch(i)
   {
   case GLTextureSettings::WRAP_REPEAT:
      return "repeat";
   case GLTextureSettings::WRAP_CLAMP:
      return "clamp";
   case GLTextureSettings::WRAP_CLAMP_TO_EDGE:
      return "clamp to edge";
   case GLTextureSettings::WRAP_CLAMP_TO_BORDER:
      return "clamp to border";
   }

   return "unkown";
}



void GLTextureSettings::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   GLTextureSettingsMetaClass->copyParentProperties();

	BEGIN_CLASS_INIT( GLTextureSettings );
	ADD_CARD32_PROP( m_MinFilter, 0 );
	ADD_CARD32_PROP( m_MagFilter, 0 );
	ADD_CARD32_PROP( m_MipmapFilter, 0 );
	ADD_CARD32_PROP( m_WrapS, 0 );
	ADD_CARD32_PROP( m_WrapT, 0 );
	ADD_CARD32_PROP( m_WrapR, 0 );
}

void GLTextureSettings::shutdown() {
   delete GLTextureSettingsMetaClass;
}


// ----


GLTextureSettings::GLTextureSettings()
{
   // defaultsettings
   m_MinFilter = FILTER_LINEAR;
   m_MagFilter = FILTER_LINEAR;
   m_MipmapFilter = FILTER_LINEAR;

   m_WrapS = WRAP_REPEAT;
   m_WrapT = WRAP_REPEAT;
   m_WrapR = WRAP_REPEAT;
}

GLint getGLWrap( card32 myWrap )
{
   switch( myWrap )
   {
   case GLTextureSettings::WRAP_REPEAT:
      return GL_REPEAT;
   case GLTextureSettings::WRAP_CLAMP:
      return GL_CLAMP;
   case GLTextureSettings::WRAP_CLAMP_TO_EDGE:
      return GL_CLAMP_TO_EDGE;
   case GLTextureSettings::WRAP_CLAMP_TO_BORDER:
      return GL_CLAMP_TO_BORDER;
   }

   return GL_REPEAT; // default
}

void GLTextureSettings::set(GLenum target, bool mipmap) const
{
   glTexParameteri( target, GL_TEXTURE_MAG_FILTER,
      m_MinFilter==FILTER_LINEAR? GL_LINEAR : GL_NEAREST);

   GLint minfilter = GL_NEAREST;
   if( mipmap )
   {
      if( m_MipmapFilter == FILTER_LINEAR )
      {
         minfilter = (m_MinFilter==FILTER_LINEAR)?
            GL_LINEAR_MIPMAP_LINEAR : GL_NEAREST_MIPMAP_LINEAR;
      }
      else
      {
         minfilter = (m_MinFilter==FILTER_LINEAR)?
            GL_LINEAR_MIPMAP_NEAREST : GL_NEAREST_MIPMAP_NEAREST;
      }
   }
   else
      minfilter =  (m_MinFilter==FILTER_LINEAR)? GL_LINEAR : GL_NEAREST;

   glTexParameteri( target, GL_TEXTURE_MIN_FILTER, minfilter );

   glTexParameteri( target, GL_TEXTURE_WRAP_S, getGLWrap( m_WrapS ) );
   glTexParameteri( target, GL_TEXTURE_WRAP_T, getGLWrap( m_WrapT ) );
//   glTexParameteri( target, GL_TEXTURE_WRAP_R, m_WrapR );

}

void GLTextureSettings::setDefaults(GLenum target, bool mipmap)
{
   glTexParameteri( target, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

   if( mipmap )
      glTexParameteri( target, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
   else
      glTexParameteri( target, GL_TEXTURE_MIN_FILTER, GL_LINEAR );

   GLenum wrapMode = GL_REPEAT;
   if( target == GL_TEXTURE_RECTANGLE_NV )
      wrapMode = GL_CLAMP_TO_EDGE;
   glTexParameteri( target, GL_TEXTURE_WRAP_S, wrapMode );
   glTexParameteri( target, GL_TEXTURE_WRAP_T, wrapMode );
}

void GLTextureSettings::setCubeDefaults()
{
	glTexParameteri( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
}

void GLTextureSettings::setFloatDefaults(GLenum target)
{
   glTexParameteri( target, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
   glTexParameteri( target, GL_TEXTURE_MIN_FILTER, GL_NEAREST );

   glTexParameteri( target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
   glTexParameteri( target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
   glTexParameteri( target, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE );
}


GLTexture::GLTexture( GLContext * context )
:GLObject( context )
{
   glGenTextures( 1, &m_TextureID );
   m_IsMipmapTexture = false;
   m_TextureSize = 0;
   m_NumComponents = 0;
}

GLTexture::~GLTexture()
{
   if( isValid() )
      //glDeleteTextures(1, &m_TextureID );
      getContext()->deleteTexture( m_TextureID );
}

void GLTexture::bind(GLuint stage)
{
   setStage(stage);
   glBindTexture( m_TextureTarget, m_TextureID);
}

void GLTexture::bind( const GLTextureSettings &settings, GLuint stage)
{
   bind(stage);
   settings.set( m_TextureTarget, m_IsMipmapTexture );
}

void GLTexture::unbind(GLuint stage)
{
   setStage(stage);
   glBindTexture( m_TextureTarget, 0 );
}


void GLTexture::setStage( int stage )
{
   glActiveTextureARB(GL_TEXTURE0_ARB + stage);
}

void GLTexture::onDeleteContext()
{
   if( isValid() )
      //glDeleteTextures(1, &m_TextureID );
      getContext()->deleteTexture( m_TextureID );

}

GLTexture2D::GLTexture2D( GLContext * context )
:GLTexture( context )
{
   m_TextureTarget = GL_TEXTURE_2D;
   m_Width = 0;
   m_Height = 0;
   m_BytesPerTexel = 0;
   m_HasMipmaps = true;
}


void GLTexture2D::createFromImage( RGBAImage &img, bool mipmap )
{
   m_NumComponents = 4;
   m_Format = GL_RGBA;
   m_IsMipmapTexture = mipmap;

   bind();
   GLTextureSettings::setDefaults( m_TextureTarget, mipmap);

   setDimensions( img.getSizeX(), img.getSizeY(), 4, mipmap );

   vector<card32> tmpData( img.getSizeX() * img.getSizeY() );
   card32 * pixels = (card32 *)img.getPixels();
   for( card32 i=0;i<tmpData.size();i++ )
   {
      tmpData[i] = pixels[i];
      tmpData[i] &= 0xff00ff00;
      tmpData[i] |= (pixels[i] >> 16) & 0xff;
      tmpData[i] |= (pixels[i] & 0xff) << 16;
   }

   if( mipmap )
   {
      gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGBA, img.getSizeX(), img.getSizeY(),
         GL_RGBA, GL_UNSIGNED_BYTE, &tmpData[0] );
   }
   else
   {
      glTexImage2D(  GL_TEXTURE_2D, 0, GL_RGBA,img.getSizeX(), img.getSizeY(),
         0, GL_RGBA, GL_UNSIGNED_BYTE, &tmpData[0] );
   }
}

void GLTexture2D::createFromMemory(  GLuint width, GLuint height, GLuint numComponents,
      char * dataPtr, bool mipmap)
{
   pAssert( (numComponents>=1) && (numComponents<=4) );
   m_NumComponents = numComponents;
   switch( numComponents )
   {
   case 1:
      m_Format = GL_LUMINANCE;
   break;
   case 2:
      m_Format = GL_LUMINANCE_ALPHA;
   break;
   case 3:
      m_Format = GL_RGB;
   break;
   case 4:
      m_Format = GL_RGBA;
   break;
   }

   bind();
   GLTextureSettings::setDefaults( m_TextureTarget, mipmap);

   setDimensions( width, height, numComponents, mipmap );

   if( mipmap )
   {
      gluBuild2DMipmaps( GL_TEXTURE_2D, m_Format, width, height, m_Format, GL_UNSIGNED_BYTE, dataPtr );
   }
   else
      glTexImage2D( GL_TEXTURE_2D, 0, m_Format, width, height,0, m_Format, GL_UNSIGNED_BYTE, dataPtr );
}

void GLTexture2D::calcSize()
{
   m_TextureSize = 0;
   if( m_HasMipmaps )
   {
      GLuint w = m_Width;
      GLuint h = m_Height;
      while( w * h > 1 )
      {
         m_TextureSize += m_BytesPerTexel * w * h;
         if( w > 1 ) w /= 2;
         if( h > 1 ) h /= 2;
      }
   }
   else
      m_TextureSize = m_Width * m_Height * m_BytesPerTexel;
}

void GLTexture2D::setDimensions( GLuint width, GLuint height,
      GLuint bytesPerTexel, bool mipmap )
{
   m_Width = width;
   m_Height = height;
   m_BytesPerTexel = bytesPerTexel;
   m_HasMipmaps = mipmap;
   calcSize();
}

GLTextureRect::GLTextureRect( GLContext * context )
:GLTexture( context )
{
   m_TextureTarget = GL_TEXTURE_RECTANGLE_NV;
   //m_TextureTarget = GL_TEXTURE_RECTANGLE_ARB;
}

void GLTextureRect::createFromImage( RGBAImage &img )
{
   m_NumComponents = 4;
   m_Format = GL_RGBA;

   bind();
   GLTextureSettings::setDefaults( m_TextureTarget, false);

   setDimensions( img.getSizeX(), img.getSizeY(), 4 );
   glTexImage2D(  m_TextureTarget, 0, m_Format,img.getSizeX(), img.getSizeY(),
      0, m_Format, GL_UNSIGNED_BYTE, img.getPixels() );
}

void GLTextureRect::createFromMemory( GLuint width, GLuint height, GLuint numComponents,
      char * dataPtr )
{
   pAssert( (numComponents>=1) && (numComponents<=4) );
   m_NumComponents = numComponents;
   switch( numComponents )
   {
   case 1:
      m_Format = GL_LUMINANCE;
   break;
   case 2:
      m_Format = GL_LUMINANCE_ALPHA;
   break;
   case 3:
      m_Format = GL_RGB;
   break;
   case 4:
      m_Format = GL_RGBA;
   break;
   }

   bind();
   GLTextureSettings::setDefaults( m_TextureTarget, false);

   setDimensions( width, height, numComponents );
   glTexImage2D( m_TextureTarget, 0, m_Format, width, height,0, m_Format, GL_UNSIGNED_BYTE, dataPtr );
}

void GLTextureRect::createFloatTextureFromMemory( GLuint width, GLuint height, GLuint numComponents,
      float * dataPtr, bool isFloat32 )
{
   pAssert( (numComponents>=1) && (numComponents<=4) );
   m_NumComponents = numComponents;
   GLenum dataFormat = GL_RGBA;
   switch( numComponents )
   {
   case 1:
      m_Format = isFloat32 ? GL_FLOAT_R32_NV : GL_FLOAT_R16_NV;
      dataFormat = GL_LUMINANCE;
   break;
   case 2:
      m_Format = isFloat32 ? GL_FLOAT_RG32_NV : GL_FLOAT_RG16_NV;
      dataFormat = GL_LUMINANCE_ALPHA;
   break;
   case 3:
      m_Format = isFloat32 ? GL_FLOAT_RGB32_NV : GL_FLOAT_RGB16_NV;
      dataFormat = GL_RGB;
   break;
   case 4:
      m_Format = isFloat32 ? GL_FLOAT_RGBA32_NV : GL_FLOAT_RGBA16_NV;
      dataFormat = GL_RGBA;
   break;
   }

   bind();
   GLTextureSettings::setFloatDefaults( m_TextureTarget);

   setDimensions( width, height, numComponents );
   glTexImage2D( m_TextureTarget, 0, m_Format, width, height,0, dataFormat, GL_FLOAT, dataPtr );
}


void GLTextureRect::calcSize()
{
   m_TextureSize = m_Width * m_Height * m_BytesPerTexel;
}

void GLTextureRect::setDimensions( GLuint width, GLuint height,
      GLuint bytesPerTexel )
{
   m_Width = width;
   m_Height = height;
   m_BytesPerTexel = bytesPerTexel;
   calcSize();
}


GLTextureCube::GLTextureCube(GLContext *context)
:GLTexture( context )
{
	m_Size = 0;
	m_NumComponents = 0;
	m_TextureSize = 0;

	m_TextureTarget = GL_TEXTURE_CUBE_MAP_ARB;
}

void GLTextureCube::createFromImage( RGBACubeMap &cubemap )
{
   m_NumComponents = 4;
   m_Format = GL_RGBA;


   m_Size = cubemap.getSide( CubeMap<card8>::CMS_POSITIVE_X )->getSizeX();
   bind();
	GLTextureSettings::setCubeDefaults();

   setPositiveX( (char*)cubemap.getSide( CubeMap<card8>::CMS_POSITIVE_X )->getPixels() );
   setPositiveY( (char*)cubemap.getSide( CubeMap<card8>::CMS_POSITIVE_Y )->getPixels() );
   setPositiveZ( (char*)cubemap.getSide( CubeMap<card8>::CMS_POSITIVE_Z )->getPixels() );
   setNegativeX( (char*)cubemap.getSide( CubeMap<card8>::CMS_NEGATIVE_X )->getPixels() );
   setNegativeY( (char*)cubemap.getSide( CubeMap<card8>::CMS_NEGATIVE_Y )->getPixels() );
   setNegativeZ( (char*)cubemap.getSide( CubeMap<card8>::CMS_NEGATIVE_Z )->getPixels() );

   calcSize();
}

void GLTextureCube::create( GLuint size, GLuint numComponents )
{
	m_Size = size;
	m_NumComponents = numComponents;
	switch( numComponents )
	{
	case 1:
		m_Format = GL_LUMINANCE;
		break;
	case 2:
		m_Format = GL_LUMINANCE_ALPHA;
		break;
	case 3:
		m_Format = GL_RGB;
		break;
	default:
		m_Format = GL_RGBA;
		break;
	}

	bind();
	GLTextureSettings::setCubeDefaults();

	glTexImage2D( GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB, 0, m_Format, m_Size, m_Size, 0,
		m_Format, GL_UNSIGNED_BYTE, NULL );
	glTexImage2D( GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB, 0, m_Format, m_Size, m_Size, 0,
		m_Format, GL_UNSIGNED_BYTE, NULL );
	glTexImage2D( GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB, 0, m_Format, m_Size, m_Size, 0,
		m_Format, GL_UNSIGNED_BYTE, NULL );
	glTexImage2D( GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB, 0, m_Format, m_Size, m_Size, 0,
		m_Format, GL_UNSIGNED_BYTE, NULL );
	glTexImage2D( GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB, 0, m_Format, m_Size, m_Size, 0,
		m_Format, GL_UNSIGNED_BYTE, NULL );
	glTexImage2D( GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB, 0, m_Format, m_Size, m_Size, 0,
		m_Format, GL_UNSIGNED_BYTE, NULL );

	calcSize();
}

void GLTextureCube::setPositiveX( char * dataPtr, bool BGRA )
{
	pAssert( m_Size > 0);
   pAssert( !BGRA || (m_NumComponents == 4 ) );
   if( !BGRA )
   	glTexImage2D( GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB, 0, m_Format, m_Size, m_Size, 0,
	   	m_Format, GL_UNSIGNED_BYTE, dataPtr );
   else
   	glTexImage2D( GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB, 0, m_Format, m_Size, m_Size, 0,
	   	GL_BGRA, GL_UNSIGNED_BYTE, dataPtr );
}

void GLTextureCube::setPositiveY( char * dataPtr, bool BGRA )
{
	pAssert( m_Size > 0);
   pAssert( !BGRA || (m_NumComponents == 4 ) );
	if( !BGRA )
      glTexImage2D( GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB, 0, m_Format, m_Size, m_Size, 0,
		   m_Format, GL_UNSIGNED_BYTE, dataPtr );
   else
      glTexImage2D( GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB, 0, m_Format, m_Size, m_Size, 0,
		   GL_BGRA, GL_UNSIGNED_BYTE, dataPtr );
}

void GLTextureCube::setPositiveZ( char * dataPtr, bool BGRA )
{
	pAssert( m_Size > 0);
   pAssert( !BGRA || (m_NumComponents == 4 ) );
	if( !BGRA )
      	glTexImage2D( GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB, 0, m_Format, m_Size, m_Size, 0,
		   m_Format, GL_UNSIGNED_BYTE, dataPtr );
   else
      	glTexImage2D( GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB, 0, m_Format, m_Size, m_Size, 0,
		   GL_BGRA, GL_UNSIGNED_BYTE, dataPtr );
}

void GLTextureCube::setNegativeX( char * dataPtr, bool BGRA )
{
	pAssert( m_Size > 0);
   pAssert( !BGRA || (m_NumComponents == 4 ) );
	if( !BGRA )
      	glTexImage2D( GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB, 0, m_Format, m_Size, m_Size, 0,
		   m_Format, GL_UNSIGNED_BYTE, dataPtr );
   else
      	glTexImage2D( GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB, 0, m_Format, m_Size, m_Size, 0,
		   GL_BGRA, GL_UNSIGNED_BYTE, dataPtr );
}

void GLTextureCube::setNegativeY( char * dataPtr, bool BGRA )
{
	pAssert( m_Size > 0);
   pAssert( !BGRA || (m_NumComponents == 4 ) );
	if( !BGRA )
      	glTexImage2D( GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB, 0, m_Format, m_Size, m_Size, 0,
		   m_Format, GL_UNSIGNED_BYTE, dataPtr );
   else
      	glTexImage2D( GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB, 0, m_Format, m_Size, m_Size, 0,
		   GL_BGRA, GL_UNSIGNED_BYTE, dataPtr );
}

void GLTextureCube::setNegativeZ( char * dataPtr, bool BGRA )
{
	pAssert( m_Size > 0);
   pAssert( !BGRA || (m_NumComponents == 4 ) );
	if( !BGRA )
      	glTexImage2D( GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB, 0, m_Format, m_Size, m_Size, 0,
		   m_Format, GL_UNSIGNED_BYTE, dataPtr );
   else
      	glTexImage2D( GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB, 0, m_Format, m_Size, m_Size, 0,
		   GL_BGRA, GL_UNSIGNED_BYTE, dataPtr );
}

void GLTextureCube::calcSize()
{
	m_TextureSize = 6 * m_Size * m_Size * m_NumComponents;
}

/*void GLTextureCube::createNormalizationCubemap(GLuint size)
{
	 create( size, 3 );

	 glTexParameteri( m_TextureTarget, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	 glTexParameteri( m_TextureTarget, GL_TEXTURE_MAG_FILTER, GL_NEAREST );

	 vector<unsigned char>	 tmp(size*size*m_NumComponents);
	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				CVector3f norm = CVector3f(0.5f,  -yf, -xf );
				norm.Normalize();
				tmp[(y*size+x)*m_NumComponents + 0] = norm.x[0] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 1] = norm.x[1] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 2] = norm.x[2] * 127.0f + 128.0f;
		  }
	 setPositiveX( (char *)&tmp[0] );

	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				CVector3f norm = CVector3f(xf, -yf,0.5f );
				norm.Normalize();
				tmp[(y*size+x)*m_NumComponents + 0] = norm.x[0] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 1] = norm.x[1] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 2] = norm.x[2] * 127.0f + 128.0f;
		  }
	 setPositiveZ( (char *)&tmp[0] );

	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				CVector3f norm = CVector3f(-0.5f, -yf,xf );
				norm.Normalize();
				tmp[(y*size+x)*m_NumComponents + 0] = norm.x[0] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 1] = norm.x[1] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 2] = norm.x[2] * 127.0f + 128.0f;
		  }
	 setNegativeX( (char *)&tmp[0] );

	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				CVector3f norm = CVector3f(-xf, -yf,-0.5f );
				norm.Normalize();
				tmp[(y*size+x)*m_NumComponents + 0] = norm.x[0] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 1] = norm.x[1] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 2] = norm.x[2] * 127.0f + 128.0f;
		  }
	 setNegativeZ( (char *)&tmp[0] );

	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				CVector3f norm = CVector3f(xf, -0.5f,-yf );
				norm.Normalize();
				tmp[(y*size+x)*m_NumComponents + 0] = norm.x[0] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 1] = norm.x[1] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 2] = norm.x[2] * 127.0f + 128.0f;
		  }
	 setNegativeY( (char *)&tmp[0] );

	 for( int x=0;x<size;x++ )
		  for( int y=0;y<size;y++ )
		  {
				float xf = ((float)x+0.5f) / (float)size - 0.5f;
				float yf = ((float)y+0.5f) / (float)size - 0.5f;
				CVector3f norm = CVector3f(xf, 0.5f,yf );
				norm.Normalize();
				tmp[(y*size+x)*m_NumComponents + 0] = norm.x[0] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 1] = norm.x[1] * 127.0f + 128.0f;
				tmp[(y*size+x)*m_NumComponents + 2] = norm.x[2] * 127.0f + 128.0f;
		  }
	 setPositiveY( (char *)&tmp[0] );

} */


GLTexture3D::GLTexture3D(GLContext * context)
:GLTexture( context )
{
   m_TextureTarget = GL_TEXTURE_3D;
   m_Width = m_Height = m_Depth = 0;
   m_BytesPerTexel = 0;
   m_NumComponents = 0;
}

void GLTexture3D::createFromMemory( GLuint width, GLuint height, GLuint depth, GLuint numComponents,
      char * dataPtr, bool mipmap, GLenum sourceDataFormat )
{
   m_Width = width;
   m_Height = height;
   m_Depth = depth;
   m_NumComponents = numComponents;
   m_BytesPerTexel = numComponents;
   m_HasMipmaps = mipmap;

   switch( numComponents )
   {
   case 1:
      m_Format = GL_LUMINANCE;
   break;
   case 2:
      m_Format = GL_LUMINANCE_ALPHA;
   break;
   case 3:
      m_Format = GL_RGB;
   break;
   case 4:
      m_Format = GL_RGBA;
   break;
   }

   bind();
   glTexParameteri( m_TextureTarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
   glTexParameteri( m_TextureTarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR );

   glTexParameteri( m_TextureTarget, GL_TEXTURE_WRAP_S, GL_CLAMP );
   glTexParameteri( m_TextureTarget, GL_TEXTURE_WRAP_T, GL_CLAMP );
   glTexParameteri( m_TextureTarget, GL_TEXTURE_WRAP_R, GL_CLAMP );

   m_TextureSize = 0;
   if( m_HasMipmaps )
   {
      GLuint w = m_Width;
      GLuint h = m_Height;
      GLuint d = m_Depth;
      while( w * h *d > 1 )
      {
         m_TextureSize += m_BytesPerTexel * w * h * d;
         if( w > 1 ) w /= 2;
         if( h > 1 ) h /= 2;
         if( d > 1 ) d /= 2;
      }
   }
   else
      m_TextureSize = m_Width * m_Height * m_Depth * m_BytesPerTexel;


   if( mipmap )
   {
//      gluBuild3DMipmaps( m_TextureTarget, m_Format, width, height, depth, m_Format, sourceDataFormat, dataPtr );
   }
   else
      glTexImage3DEXT( m_TextureTarget, 0, m_Format, width, height, depth,0, m_Format, sourceDataFormat, dataPtr );

}

void GLTexture3D::createPerlinNoise( GLuint size, GLuint octaves, float persistance )
{
   m_Width = size;
   m_Height = size;
   m_Depth = size;
   m_NumComponents = 1;
   m_BytesPerTexel = 1;
   m_HasMipmaps = false;
   m_Format = GL_LUMINANCE;
   m_TextureSize = m_Width * m_Height * m_Depth * m_BytesPerTexel;

   bind();
   GLTextureSettings::setDefaults( m_TextureTarget, false );

   vector<card8> buffer(size*size*size);

   for( card32 x=0;x<size;x++ )
      for( card32 y=0;y<size;y++ )
         for( card32 z=0;z<size;z++ )
         {
            card32 offset = z*size*size + y*size + x;
            buffer[offset] = (card8)(y+x+z);
         }

   glTexImage3DEXT( m_TextureTarget, 0, m_Format, size, size, size,0, m_Format, GL_UNSIGNED_BYTE, &buffer[0] );

   forget(octaves);
   forget(persistance);
}


bool GLTexture::isFloatTexture() const 
{
	return ( m_Format == GL_LUMINANCE16F_ARB || 
			m_Format == GL_LUMINANCE32F_ARB ||
			m_Format == GL_LUMINANCE_ALPHA16F_ARB ||
			m_Format == GL_LUMINANCE_ALPHA32F_ARB ||
			m_Format == GL_RGB16F_ARB ||
			m_Format == GL_RGB32F_ARB ||
			m_Format == GL_RGBA16F_ARB ||
			m_Format == GL_RGBA32F_ARB);
}


void GLTexture3D::createFloatTextureFromMemory( GLuint width, GLuint height, GLuint depth, GLuint numComponents, char * dataPtr, bool use16bit, bool mipmap  ) 
{
	m_Width = width;
	m_Height = height;
	m_Depth = depth;
	m_NumComponents = numComponents;
	m_BytesPerTexel = numComponents;
	m_HasMipmaps = mipmap;

	switch( numComponents )
	{
	case 1:
		m_Format = use16bit ? GL_LUMINANCE16F_ARB : GL_LUMINANCE32F_ARB;
		break;
	case 2:
		m_Format = use16bit ? GL_LUMINANCE_ALPHA16F_ARB : GL_LUMINANCE_ALPHA32F_ARB;
		break;
	case 3:
		m_Format = use16bit ? GL_RGB16F_ARB : GL_RGB32F_ARB;
		break;
	case 4:
		m_Format = use16bit ? GL_RGBA16F_ARB : GL_RGBA32F_ARB;
		break;
	}

	bind();
//	GLTextureSettings::setDefaults( m_TextureTarget, mipmap);
	glTexParameteri( m_TextureTarget, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( m_TextureTarget, GL_TEXTURE_MIN_FILTER, GL_NEAREST );

	glTexParameteri( m_TextureTarget, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
	glTexParameteri( m_TextureTarget, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
	glTexParameteri( m_TextureTarget, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE );

	m_TextureSize = 0;
	if( m_HasMipmaps )
	{
		GLuint w = m_Width;
		GLuint h = m_Height;
		GLuint d = m_Depth;
		while( w * h *d > 1 )
		{
			m_TextureSize += m_BytesPerTexel * w * h * d;
			if( w > 1 ) w /= 2;
			if( h > 1 ) h /= 2;
			if( d > 1 ) d /= 2;
		}
	}
	else
		m_TextureSize = m_Width * m_Height * m_Depth * m_BytesPerTexel;


	getContext()->debug("before teximage3d");
	if( mipmap )
	{
		throw "Not implemented yet! (Will be done only if it's really necessary)";
		//      gluBuild3DMipmaps( m_TextureTarget, m_Format, width, height, depth, m_Format, GL_UNSIGNED_BYTE, dataPtr );
	}
	else
		glTexImage3DEXT( m_TextureTarget, 0, m_Format, width, height, depth,0, m_Format, GL_FLOAT, dataPtr );
}
