//---------------------------------------------------------------------------
#ifndef Tex2dVolumeRendererH
#define Tex2dVolumeRendererH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------
#include <GL/gl.h>
//---------------------------------------------------------------------------


namespace geoxl {

class RegularVolume;


/**
	renders RegularVolumes
*/
class GLTOOLS_API Tex2dVolumeRenderer : public Persistent {
	GEOXL_CLASS( Tex2dVolumeRenderer )

 public:
	typedef Vector4ub (*TransferFunction)(void*, const StaticVector<float32, 1> &);

 private:
	vector< vector<GLuint> > channelTextures;
	vector< vector<mpcard> > texOffsets;

	void *mappingContextTmp;
	TransferFunction mappingTmp;
	
   static Vector4ub mapping2(void* ctxt, const StaticVector<float32, 2> &in);
   static Vector4ub mapping3(void* ctxt, const StaticVector<float32, 3> &in);
   static Vector4ub mapping4(void* ctxt, const StaticVector<float32, 4> &in);

 protected:
	virtual void prepareTextures(RegularVolume *vol, Vector3f viewDirection, 
	                             const vector<string> &channels, TransferFunction mapping, void *mappingContext);

 public:
	Tex2dVolumeRenderer();
	
	void computeOrder(const Vector3f &viewDirection, card32 &dir, int32 &sign);
	
	/// renders float channels with dimension 1..4 
	/// a transfer function can be applied to compute color (1D case) or to map attributes 
	/// (evaluates alpha channel of transfer function only)
	/// in 2-4D, data is just copied and alpha set to 16/255 (i.e., you might need to write a shader for the mapping)
	virtual void drawVolume(RegularVolume *vol, Vector3f viewDirection, 
                           const vector<string> &channels, TransferFunction mapping, void *mappingContext);

   /// alias function for compatibility (single channel)
	virtual void drawVolume(RegularVolume *vol, Vector3f viewDirection, 
                           string channel, TransferFunction mapping, void *mappingContext);

	virtual void disposeTextures();
	
	~Tex2dVolumeRenderer();
};


}

#endif