//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "GLEventDisplayerWidget.h"
//---------------------------------------------------------------------------
#include "GLEventDisplayer.h"
#include "ExceptionHandling.h"
#include "GLContext.h"
//---------------------------------------------------------------------------
#include <GL/gl.h>
#include <QtGui/QApplication>
#include <QtGui/QDesktopWidget>
#include <QtGui/QCursor>
#include <QtGui/QMouseEvent>
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

static const int MAX_SIZE_INDEX = 12;

static const int SIZES[MAX_SIZE_INDEX+1] =
	{16, 22, 32, 45, 64, 90, 128, 181, 256, 362, 512, 724, 1024};


GLEventDisplayerWidget::GLEventDisplayerWidget(QWidget* parent, Qt::WFlags f) 
: QWidget(parent, f)
{
	ui.setupUi(this);
	configPopup = NULL;
	glWidget = new MyGLWidget(ui.mainFrame);
	sizeIndex =	5;
	glWidget->setMinimumHeight(SIZES[sizeIndex]);
	glWidget->setMaximumHeight(SIZES[sizeIndex]);
	glWidget->resize(glWidget->width(), SIZES[sizeIndex]);
	ui.mainFrame->layout()->addWidget(glWidget);
	ui.mainFrame->layout()->setSizeConstraint(QLayout::SetMinAndMaxSize);
}

GLEventDisplayerWidget::MyGLWidget::MyGLWidget(QWidget *parent) 
: QGLWidget(parent)
{
	glEventObject = NULL;
	camCtrl.bindCamera(&cam);
	camCtrl.areaResize(width(), height());
	setFocusPolicy(Qt::StrongFocus);
	glContext = NULL;
}

GLEventDisplayerWidget::MyGLWidget::~MyGLWidget() 
{
	delete glEventObject;
	delete glContext;
}

void GLEventDisplayerWidget::MyGLWidget::paintGL() {
   STD_GLOBAL_EXCEPTION_HANDLER_START
   if (glContext == NULL) {
	    makeCurrent();
#ifdef WIN32
	    pAssert(wglGetCurrentDC()); 
	    pAssert(wglGetCurrentContext());
	    glContext = new GLContext(wglGetCurrentDC(), wglGetCurrentContext());
#else
	    pAssert(glXGetCurrentDrawable()); 
	    pAssert(glXGetCurrentContext());
	    glContext = new GLContext(glXGetCurrentDrawable(), glXGetCurrentContext());
#endif
   }
	glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	int w = width();
	int h = height();
	if (w <=0 || h <=0) return;
	if (glEventObject) {
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glViewport(0,0, w, h);
		gluPerspective(45.0, (double)w/(double)h, 1E-2, 1E3);
		glEventObject->drawGL(this, cam);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		camCtrl.areaResize(width(), height());
		camCtrl.apply();
		Vector3f pos = cam.getPosition();
		Vector3f lookAt = cam.getLookAt();
		Vector3f up = cam.getUp();
		gluLookAt(pos[0], pos[1], pos[2], lookAt[0], lookAt[1], lookAt[2], up[0], up[1], up[2]);
	}
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void GLEventDisplayerWidget::MyGLWidget::mouseMoveEvent(QMouseEvent *event){
	STD_GLOBAL_EXCEPTION_HANDLER_START
		camCtrl.mouseMoved(event->x(),event->y());
		repaint();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void GLEventDisplayerWidget::MyGLWidget::mousePressEvent(QMouseEvent *event){
	STD_GLOBAL_EXCEPTION_HANDLER_START
		camCtrl.mouseDown(
			event->x(),event->y(), 
			MouseButtons(event->button()==Qt::LeftButton,event->button()==Qt::RightButton, 
			event->button()==Qt::MidButton), 
			Modifiers(event->modifiers())
		);
		repaint();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void GLEventDisplayerWidget::MyGLWidget::mouseReleaseEvent(QMouseEvent *event) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
		camCtrl.mouseUp(
			event->x(),event->y(), 
			MouseButtons(event->button()==Qt::LeftButton, event->button()==Qt::RightButton, event->button()==Qt::MidButton),
			Modifiers(event->modifiers())
		); 
		repaint();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}


void GLEventDisplayerWidget::MyGLWidget::keyPressEvent(QKeyEvent *event) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	   camCtrl.keyDown(GeneralKey(event->key(),Modifiers(event->modifiers())));
		repaint();
	STD_GLOBAL_EXCEPTION_HANDLER_END
}

void GLEventDisplayerWidget::MyGLWidget::keyReleaseEvent(QKeyEvent *event) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
	   camCtrl.keyUp(GeneralKey(event->key(),Modifiers(event->modifiers())));
		repaint();
	STD_GLOBAL_EXCEPTION_HANDLER_END	
}

void GLEventDisplayerWidget::MyGLWidget::resizeEvent(QResizeEvent *event) {
	STD_GLOBAL_EXCEPTION_HANDLER_START
		camCtrl.areaResize(width(), height());
	STD_GLOBAL_EXCEPTION_HANDLER_END
}


void GLEventDisplayerWidget::on_pbIncreaseSize_clicked() {
	if (sizeIndex == MAX_SIZE_INDEX) return;
	sizeIndex++;
	glWidget->setMinimumHeight(SIZES[sizeIndex]);
	glWidget->setMaximumHeight(SIZES[sizeIndex]);
	glWidget->resize(glWidget->width(), SIZES[sizeIndex]);
	QPoint old = QCursor::pos();
	QPoint newPos = QPoint(old.x(), old.y() - SIZES[sizeIndex-1] + SIZES[sizeIndex]);
	if (newPos.y() < qApp->desktop()->screen(qApp->desktop()->screenNumber(old))->height()) {
		QCursor::setPos(newPos);
	}
}

void GLEventDisplayerWidget::on_pbDecreaseSize_clicked() {
	if (sizeIndex == 0) return;
	sizeIndex--;
	glWidget->setMinimumHeight(SIZES[sizeIndex]);
	glWidget->setMaximumHeight(SIZES[sizeIndex]);
	glWidget->resize(glWidget->width(), SIZES[sizeIndex]);
	QPoint old = QCursor::pos();
	QPoint newPos = QPoint(old.x(), old.y() - SIZES[sizeIndex+1] + SIZES[sizeIndex]);
	if (newPos.y() >= 0) {
		QCursor::setPos(newPos);
	}
}

void GLEventDisplayerWidget::on_pbConfigure_clicked() {
	if (configPopup) {
		configPopup->popup(ui.pbConfigure);
	}
}


void GLEventDisplayerWidget::setGLEventObject(GLEventObject *glevo, bool updateDataOnly) {
	if (glevo == NULL) return;
	if (glWidget->glEventObject == NULL) {
		updateDataOnly = false;
	} else if (glWidget->glEventObject->getInstanceClass() != glevo->getInstanceClass()) {
		updateDataOnly = false;
	}
	if (updateDataOnly) {
		bool resetBB = false;
		if (   glWidget->glEventObject->getBoundingBox().lowerCorner != glevo->getBoundingBox().lowerCorner
		    || glWidget->glEventObject->getBoundingBox().upperCorner != glevo->getBoundingBox().upperCorner) {
			resetBB = true;
		}
		glWidget->glEventObject->updateData(glevo);
		delete glevo;
		if (resetBB) {
			BoundingBox3f bb = glWidget->glEventObject->getBoundingBox();
			Camera* camera = glWidget->camCtrl.getCamera();
			Vector3f dir = normalize( camera->getLookAt() - camera->getPosition() );
			Vector3f newPos = camera->getLookAt() + dir * 2.0f*bb.getMaxSideLength();
			glWidget->camCtrl.setup( newPos, bb.getCenter(), camera->getUp() );
		}
		changed();
	} else {
		delete glWidget->glEventObject;
		glWidget->glEventObject = glevo;

		delete configPopup;
		configPopup = NULL;
		if (glevo != NULL) {
			configPopup = glevo->createConfigWindow(this);
			ui.pbConfigure->setEnabled(configPopup != NULL);
			if (configPopup) {
				QObject::connect(configPopup, SIGNAL(changed()), this, SLOT(changed()));
			}
		}

		BoundingBox3f bb = glevo->getBoundingBox();
		Camera* camera = glWidget->camCtrl.getCamera();
		Vector3f dir = normalize( camera->getLookAt() - camera->getPosition() );
		Vector3f newPos = camera->getLookAt() + dir * 2.0f*bb.getMaxSideLength();
		glWidget->camCtrl.setup( newPos, bb.getCenter(), camera->getUp() );
		changed();
	}
}

void GLEventDisplayerWidget::changed() {
	glWidget->update();
}

GLEventDisplayerWidget::~GLEventDisplayerWidget() {}




#include "GLEventDisplayerWidget_moc.cpp"
