//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "GLGraphDisplayer.h"
//---------------------------------------------------------------------------
#include "GLEventDisplayerWidget.h"
#include "DebugOutput.h"
#include "PointSet.h"
#include "VertexDescriptor.h"
#include "GLHelper.h"
#include "DebugOutput.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS(GLGraphDisplayer, 0) 
{
}

// ----


GLGraphDisplayer::GLGraphDisplayer() {
   showAxis = false;
   showZAxis = false;
}

GLGraphDisplayer::GLGraphDisplayer(
   const vector< vector<Vector3f> > &graphs, 
   const vector< Vector4ub > &colors) 
{
   showAxis = false;
   showZAxis = false;
   for (unsigned i=0; i<min(graphs.size(), colors.size()); i++) {
      addGraph(graphs[i], colors[i]);
   }
}

GLGraphDisplayer::GLGraphDisplayer(
   const vector< vector<Vector2f> > &graphs, 
   const vector< Vector4ub > &colors)
{
   showAxis = false;
   showZAxis = false;
   for (unsigned i=0; i<min(graphs.size(), colors.size()); i++) {
      addGraph(graphs[i], colors[i]);
   }
}

GLGraphDisplayer::GLGraphDisplayer(
   const vector< vector<float32> > &graphs, 
   const vector< Vector4ub > &colors)
{
   showAxis = false;
   showZAxis = false;
   for (unsigned i=0; i<min(graphs.size(), colors.size()); i++) {
      addGraph(graphs[i], colors[i]);
   }
}

void GLGraphDisplayer::addGraph(const vector<float32> &graph, const Vector4ub &color) {
   vector<Vector3f> g(graph.size());
   for (unsigned i=0; i<graph.size(); i++) {
      g[i] = makeVector3f((float32)i/(float32)graph.size(), graph[i], 0);
   }
   addGraph(g, color);
}

void GLGraphDisplayer::addGraph(const vector<Vector2f> &graph, const Vector4ub &color) {
   vector<Vector3f> g(graph.size());
   for (unsigned i=0; i<graph.size(); i++) {
      g[i] = makeVector3f(graph[i][0], graph[i][1], 0);
   }
   addGraph(g, color);
}

void GLGraphDisplayer::addGraph(const vector<Vector3f> &graph, const Vector4ub &color) {
   graphs.push_back(graph);
   colors.push_back(color);
}

BoundingBox3f GLGraphDisplayer::getBoundingBox() {
   bool noPoint = true;
   BoundingBox3f bb;
   for (unsigned i=0; i<graphs.size(); i++) {
      for (unsigned p=0; p<graphs[i].size(); p++) {
         if (noPoint) {
            bb.lowerCorner =graphs[i][p];
            noPoint = false;
         }
         bb.addPoint(graphs[i][p]);
      }
   }
	if (noPoint) {
	   return BoundingBox3f(makeVector3f(0,0,0), makeVector3f(1,1,1));
	} else {
	   return bb;
	}
}

void GLGraphDisplayer::drawGL(QGLWidget *widget, Camera &cam) {
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	for (unsigned i=0; i<graphs.size(); i++) {
	   glColor4ubv(colors[i].data());
	   glBegin(GL_LINE_STRIP);
	   for (card32 p=0; p<graphs[i].size(); p++) {
		   glVertex3fv(graphs[i][p].data());
	   }
	   glEnd();
	}
	if (showAxis) {
		BoundingBox3f bb = getBoundingBox();
		glTranslatef(-bb.lowerCorner[0]-bb.getSideLength(0)*0.1, -bb.lowerCorner[1]-bb.getSideLength(1)*0.1, -bb.lowerCorner[2]-bb.getSideLength(2)*0.1);
		glScalef(bb.getSideLength(0), bb.getSideLength(1), bb.getSideLength(2));
		GLHelper::drawCoords();
	}
   glPopAttrib();
}

GLEventConfiguration *GLGraphDisplayer::createConfigWindow(QWidget *parent) {
	return new GLGraphDisplayerConfig(parent, this);
}

void GLGraphDisplayer::updateData(GLEventObject *other) {
	GLGraphDisplayer *cOther = dynamic_cast<GLGraphDisplayer*>(other);
	if (cOther != NULL) {
      graphs = cOther->graphs;
      colors = cOther->colors;
	}
}


GLGraphDisplayer::~GLGraphDisplayer() {}

// --


GLGraphDisplayerConfig::GLGraphDisplayerConfig(QWidget *parent, GLEventObject *obj) 
: GLEventConfiguration(parent, obj)
{
	ui.setupUi(this);
}	

void GLGraphDisplayerConfig::on_cbShowCoords_stateChanged(int) {
	GLGraphDisplayer *pcobj = dynamic_cast<GLGraphDisplayer*>(obj);
	if (pcobj != NULL) pcobj->showAxis = ui.cbShowCoords->isChecked();
	emit changed();
}

void geoxl::debugOutputGraphs(string name, const vector< vector<float32> > &graphs, const vector<Vector4ub> &colors) {
	GLGraphDisplayer *disp = new GLGraphDisplayer(graphs, colors);
	GLEventDisplayer::SetNewGLEventObject ev(disp);
	debugOutput.displayEvent(GLEventDisplayer::getClass(), name, &ev);
}

void geoxl::debugOutputGraphs(string name, const vector< vector<Vector3f> > &graphs, const vector<Vector4ub> &colors) {
	GLGraphDisplayer *disp = new GLGraphDisplayer(graphs, colors);
	GLEventDisplayer::SetNewGLEventObject ev(disp);
	debugOutput.displayEvent(GLEventDisplayer::getClass(), name, &ev);
}

void geoxl::debugOutputGraphs(string name, const vector< vector<Vector2f> > &graphs, const vector<Vector4ub> &colors) {
	GLGraphDisplayer *disp = new GLGraphDisplayer(graphs, colors);
	GLEventDisplayer::SetNewGLEventObject ev(disp);
	debugOutput.displayEvent(GLEventDisplayer::getClass(), name, &ev);
}

void geoxl::debugOutputGraph(string name, const vector<float32> &graph, const Vector4ub color) {
	GLGraphDisplayer *disp = new GLGraphDisplayer();
	disp->addGraph(graph, color);
	GLEventDisplayer::SetNewGLEventObject ev(disp);
	debugOutput.displayEvent(GLEventDisplayer::getClass(), name, &ev);
   
}
void geoxl::debugOutputGraph(string name, const vector<Vector2f> &graph, const Vector4ub color) {
	GLGraphDisplayer *disp = new GLGraphDisplayer();
	disp->addGraph(graph, color);
	GLEventDisplayer::SetNewGLEventObject ev(disp);
	debugOutput.displayEvent(GLEventDisplayer::getClass(), name, &ev);
}
void geoxl::debugOutputGraph(string name, const vector<Vector3f> &graph, const Vector4ub color) {
	GLGraphDisplayer *disp = new GLGraphDisplayer();
	disp->addGraph(graph, color);
	GLEventDisplayer::SetNewGLEventObject ev(disp);
	debugOutput.displayEvent(GLEventDisplayer::getClass(), name, &ev);
}


#include "GLGraphDisplayer_moc.cpp"
