//---------------------------------------------------------------------------
#ifndef GLGraphDisplayerH
#define GLGraphDisplayerH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "GLEventDisplayer.h"
#include "Object.h"
//---------------------------------------------------------------------------
#include "GLGraphDisplayerConfig_ui.h"
//---------------------------------------------------------------------------


namespace geoxl {

class GLGraphDisplayerConfig;


class GLTOOLS_API GLGraphDisplayer : public GLEventObject {
	GEOXL_CLASS(GLGraphDisplayer)
 private:
   vector< vector<Vector3f> > graphs;
   vector< Vector4ub > colors;
   bool showZAxis;
   bool showAxis;
	
 public:
	GLGraphDisplayer();
	GLGraphDisplayer(const vector< vector<Vector3f> > &graphs, const vector< Vector4ub > &colors);
	GLGraphDisplayer(const vector< vector<Vector2f> > &graphs, const vector< Vector4ub > &colors);
	GLGraphDisplayer(const vector< vector<float32> > &graphs, const vector< Vector4ub > &colors);

   void addGraph(const vector<float32> &graph, const Vector4ub &color);
   void addGraph(const vector<Vector2f> &graph, const Vector4ub &color);
   void addGraph(const vector<Vector3f> &graph, const Vector4ub &color);
	
	virtual BoundingBox3f getBoundingBox();
	virtual void drawGL(QGLWidget *widget, Camera &cam);
	virtual GLEventConfiguration *createConfigWindow(QWidget *parent);
	virtual void updateData(GLEventObject *other);

	virtual ~GLGraphDisplayer();
	friend class GLGraphDisplayerConfig;
};

class GLTOOLS_API GLGraphDisplayerConfig : public GLEventConfiguration {
	Q_OBJECT
 private:
	Ui_GLGraphDisplayerConfig ui;
 private slots:
   void on_cbShowCoords_stateChanged(int);
 public:
	GLGraphDisplayerConfig(QWidget *parent, GLEventObject *obj);
};

void GLTOOLS_API debugOutputGraphs(string name, const vector< vector<float32> > &graphs, const vector<Vector4ub> &colors);
void GLTOOLS_API debugOutputGraphs(string name, const vector< vector<Vector2f> > &graphs, const vector<Vector4ub> &colors);
void GLTOOLS_API debugOutputGraphs(string name, const vector< vector<Vector3f> > &graphs, const vector<Vector4ub> &colors);
void GLTOOLS_API debugOutputGraph(string name, const vector<float32> &graph, const Vector4ub color);
void GLTOOLS_API debugOutputGraph(string name, const vector<Vector2f> &graph, const Vector4ub color);
void GLTOOLS_API debugOutputGraph(string name, const vector<Vector3f> &graph, const Vector4ub color);

}

#endif
