//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "GLPointCloudDisplayer.h"
//---------------------------------------------------------------------------
#include "GLEventDisplayerWidget.h"
#include "DebugOutput.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "VertexDescriptor.h"
#include "GLHelper.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS(GLPointCloudDisplayer, 0) 
{
}

// ----


GLPointCloudDisplayer::GLPointCloudDisplayer(PointSet *ps ) {
	this->ps = ps;
	showCoords = true;
}

BoundingBox3f GLPointCloudDisplayer::getBoundingBox() {
	if (ps == NULL) return BoundingBox3f(makeVector3f(0,0,0), makeVector3f(1,1,1));
	return ps->getBoundingBox();
}

void GLPointCloudDisplayer::drawGL(QGLWidget *widget, Camera &cam) {
	glEnable(GL_DEPTH_TEST);
	if (ps == NULL) return;
	if (!ps->getDescr()->providesAttribute("position", 3, VAD::DATA_FORMAT_FLOAT32)) {
		warning("GLPointCloudDisplayer::drawGL() - position3f channel not found.");
		return;
	}
	AAT POS = ps->getAAT("position");
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glDisable(GL_LIGHTING);
	glPointSize(1);
	glColor3f(1,1,1);
	glBegin(GL_POINTS);
	for (card32 i=0; i<ps->getNumEntries(); i++) {
		Vector3f pos = ps->get3f(i, POS);
		glVertex3fv(pos.data());		
	}
	glEnd();
	glPopAttrib();
	if (showCoords) {
		BoundingBox3f bb = getBoundingBox();
		glTranslatef(-bb.lowerCorner[0]-bb.getSideLength(0)*0.1, -bb.lowerCorner[1]-bb.getSideLength(1)*0.1, -bb.lowerCorner[2]-bb.getSideLength(2)*0.1);
      float32 minBB = bb.getMaxSideLength()/100;
		glScalef(
		   max(bb.getSideLength(0), minBB), 
		   max(bb.getSideLength(1), minBB), 
		   max(bb.getSideLength(2), minBB)
		);
		GLHelper::drawCoords();
	}
}

GLEventConfiguration *GLPointCloudDisplayer::createConfigWindow(QWidget *parent) {
	return new GLPointCloudDisplayerConfig(parent, this);
}

void GLPointCloudDisplayer::updateData(GLEventObject *other) {
	GLPointCloudDisplayer *cOther = dynamic_cast<GLPointCloudDisplayer*>(other);
	if (cOther != NULL) {
		delete ps;
		ps = cOther->ps;
		cOther->ps = NULL;
	}
}


GLPointCloudDisplayer::~GLPointCloudDisplayer() {
	delete ps;
}

// --


GLPointCloudDisplayerConfig::GLPointCloudDisplayerConfig(QWidget *parent, GLEventObject *obj) 
: GLEventConfiguration(parent, obj)
{
	ui.setupUi(this);
}	

void GLPointCloudDisplayerConfig::on_cbShowCoords_stateChanged(int) {
	GLPointCloudDisplayer *pcobj = dynamic_cast<GLPointCloudDisplayer*>(obj);
	if (pcobj != NULL) pcobj->showCoords = ui.cbShowCoords->isChecked();
	emit changed();
}

#include "GLPointCloudDisplayer_moc.cpp"
