//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "HierarchicalInCorePointCloud.h"
#include "MHInCorePointSetOctreeIterator.h"
#include "MHUtilities.h"
#include "MHUtilities_templCode.h"
#include "ObjectClassProperty.h"
#include "MHInstances_templCode.h"
#include "MHHierarchy.h"
#include "MHHierarchy_templCode.h"
#include "MHAllocators.h"
#include "MHAllocators_templCode.h"
#include "MHPointSetOctreeIterator_Templ.h"
#include "MHPointSetOctreeIterator_Templ_templCode.h"
#include "MHNodeData.h"
#include "MHNodeData_templCode.h"
#include "MHNodes.h"
#include "MHNodes_templCode.h"
#include "MHObjects.h"
#include "MHObjects_templCode.h"
#include "MHAllPointsPointSetOctreeIterator_Templ.h"
#include "MHAllPointsPointSetOctreeIterator_Templ_templCode.h"
#include "MHInstances_templCode.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


MetaClass* HierarchicalInCorePointCloud::HierarchicalInCorePointCloudMetaClass = NULL;

const MetaClass* HierarchicalInCorePointCloud::getInstanceClass() const {
   return getClass();
}

Object* HierarchicalInCorePointCloud::newInstance() {
   return new HierarchicalInCorePointCloud;
}

const MetaClass* HierarchicalInCorePointCloud::getClass() {
   if (HierarchicalInCorePointCloudMetaClass==NULL) {
      HierarchicalInCorePointCloudMetaClass = new MetaClass("HierarchicalInCorePointCloud", "no description.",
                                  newInstance, sizeof(HierarchicalInCorePointCloud));
   }
   return HierarchicalInCorePointCloudMetaClass;
}

void HierarchicalInCorePointCloud::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 2);
   HierarchicalInCorePointCloudMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT(HierarchicalInCorePointCloud)
	ADD_OBJECT_PROP(octree, 0,MHInCorePointSetOctree::getClass(), true); 
   SET_STREAM_PROPERTY(false);
}

void HierarchicalInCorePointCloud::shutdown() {
   delete HierarchicalInCorePointCloudMetaClass;
}


// ----

HierarchicalInCorePointCloud::HierarchicalInCorePointCloud() {
   octree = NULL;
   nAlloc = NULL;
}

void HierarchicalInCorePointCloud::clearAndSetup(const VertexDescriptor *vertDescr) {
   delete octree;
   delete nAlloc;
   nAlloc = new MHInCorePointSetOctree::NodeAllocator(MHInCorePointSetOctreeNode::getClass());
   octree = new MHInCorePointSetOctree();
   octree->clearAndSetup(nAlloc, vertDescr);
}


void HierarchicalInCorePointCloud::assign(const Object* obj, CopyContext *context) {
   PointCloud::assign(obj, context);
   const HierarchicalInCorePointCloud *other = dynamic_cast<const HierarchicalInCorePointCloud*>(obj);
   pAssert(other != NULL);
   delete nAlloc;
   delete octree;
   this->octree = (MHInCorePointSetOctree*)(other->octree->copy(context));
   nAlloc = octree->getAlloc();
}

void HierarchicalInCorePointCloud::write(OutputObjectStream *s) const {
   PointCloud::write(s);
   s->writeObject(octree);
}

void HierarchicalInCorePointCloud::read(InputObjectStream *s) {
   // legacy scene import
   if (s->getClassVersion(getClass()) < MetaClass::FIRST_VERSION + 2) {

      string fname;
      s->readString(fname);
      forget(fname);

      card32 fflags;
      s->read(fflags);
      forget(fflags);

      card32 fmaterialIndex;
      s->read(fmaterialIndex);
      setMaterialIndex(fmaterialIndex);

      Matrix4f ftransformation;
      s->read(ftransformation);
      setTransformation(ftransformation);

   } else {

      PointCloud::read(s);

   }

   delete octree;
   delete nAlloc;

   Persistent *octreeObj;
   s->readObject(octreeObj);
   octree = (MHInCorePointSetOctree*)octreeObj;
   if (octree != NULL) {
      nAlloc = octree->getAlloc();
   } else {
      nAlloc = NULL;
   }
}


const VertexDescriptor *HierarchicalInCorePointCloud::getDescr() const {
   return octree->getDescr();
}

PointCloud *HierarchicalInCorePointCloud::appendFields(const VertexDescriptor *newAttribs) const {
   HierarchicalInCorePointCloud *result = (HierarchicalInCorePointCloud*)copy();
   MHInCorePointSetOctree::NodePointer root;
   result->octree->getRoot(root);
   appendFieldsToPointSetNodes<MHInCorePointSetOctreeNodeBase>(root, newAttribs);
   result->octree->getDescr()->mergeAttributes(newAttribs);
   return result;
}

SceneObjectIterator *HierarchicalInCorePointCloud::createIterator(SceneObjectIteratorType type) {
   if (!supportsIteratorType(type)) {
      return NULL;
   }

   if (type.hierarchical()) {
      return new MHInCorePointSetOctreeIterator(octree);
   } else {
      MHInCorePointSetOctreeIterator *t = new MHInCorePointSetOctreeIterator(octree);
      PointCloudIterator *result =new MHAllPointsInCorePointSetOctreeIterator(t);
      delete t;
      return result;
   }
}

bool HierarchicalInCorePointCloud::supportsIteratorType(SceneObjectIteratorType cap) {
   SOIT possibleCaps(  SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_DYNAMIC_PC
                      | SOIT::CAP_INDEXED_PC | SOIT::CAP_RANDOM_ACCESS_PC
                      | SOIT::CAP_HIERARCHICAL | SOIT::CAP_ATTACH_SECONDARY);

	// indexing is only supported for hierarchial iterators
   if (cap.providesCapsFor( SOIT(SOIT::CAP_INDEXED_PC | SOIT::CAP_RANDOM_ACCESS_PC) ) ) {
		return possibleCaps.providesCapsFor(cap) && cap.providesCapsFor(SOIT::CAP_HIERARCHICAL);
   } else {
		return (possibleCaps.providesCapsFor(cap));
	}
}

card64 HierarchicalInCorePointCloud::getNumPoints() {
   return octree->getNumObjects();
}

void HierarchicalInCorePointCloud::clear() {
   octree->clear();
}

void HierarchicalInCorePointCloud::invalidateNodes() {
   if (octree != NULL) {
      octree->invalidateNodes();
   }
}


HierarchicalInCorePointCloud::~HierarchicalInCorePointCloud() {
   delete octree;
   delete nAlloc;
}






// former debug code...
//
//void recursiveTest(MHInCorePointSetOctreeNode::NodePointer np, const MHStandardSettings &settings) {
//	if (np == NULL) return;
//   if (np->isLeaf() && np->nodeShouldBeSplit(settings)) {
//		pAssert(true);
//   }
//   if (np->subtreeShouldBeMerged(settings)) {
//		pAssert(true);
//   }
//   if (np->objsInSubtree == 0) {
//		pAssert(true);
//   }
//   int n = np->getMaxNumChildren();
//   for (int i=0; i<n; i++) {
//		MHInCorePointSetOctreeNode::NodePointer c;
//		np->getChild(i, c);
//		recursiveTest(c, settings);
//   }
//}
//
//void HierarchicalInCorePointCloud::checkUpdateTest() 
//{
//	MHInCorePointSetOctreeNode::NodePointer root;
//	octree->getRoot(root);
//	MHStandardSettings settings;
//	octree->getSettings(settings);
//	recursiveTest(root, settings);
//}
