//---------------------------------------------------------------------------
#ifndef HierarchicalInCorePointCloudH
#define HierarchicalInCorePointCloudH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
#include "MHInstances.h"
#include "MHInCorePointSetOctree.h"
//---------------------------------------------------------------------------

namespace geoxl {
	
class HIERARCHIES_API HierarchicalInCorePointCloud : public PointCloud {
 private:
   static MetaClass *HierarchicalInCorePointCloudMetaClass;
   static Object* newInstance();

   MHInCorePointSetOctreeNode::NodeAllocator *nAlloc;
   MHInCorePointSetOctree *octree;

 public:
   HierarchicalInCorePointCloud();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);

   virtual const VertexDescriptor *getDescr() const;
   virtual PointCloud             *appendFields(const VertexDescriptor *newAttribs) const;
   virtual SceneObjectIterator    *createIterator(SceneObjectIteratorType type);
   virtual bool                    supportsIteratorType(SceneObjectIteratorType cap);
   virtual card64                  getNumPoints();
   virtual void                    clear();

   // new methods ----
   /// initialize instance
   virtual void clearAndSetup(const VertexDescriptor *vertDescr);
   /// set modified flag for all nodes (force reorganization)
   virtual void invalidateNodes();

   MHInCorePointSetOctree *getOctree() {return octree;}

   ~HierarchicalInCorePointCloud();
};


}




#endif
