//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "HierarchicalOutOfCorePointCloud.h"
//---------------------------------------------------------------------------
#include "MHAllocators_templCode.h"
#include "MHOutOfCorePointSetOctreeIterator.h"
#include "MHUtilities.h"
#include "ObjectClassProperty.h"
#include "MHInstances_templCode.h"
#include "MultiObjectStream.h"
#include "MHHierarchy_templCode.h"
#include "MHUtilities_templCode.h"
#include "MHPointSetOctreeIterator_Templ_templCode.h"
#include "MHNodeData_templCode.h"
#include "MHNodes_templCode.h"
#include "MHObjects_templCode.h"
#include "MHAllPointsPointSetOctreeIterator_Templ_templCode.h"
#include "MHInstances_templCode.h"
#include "PointCloudIterators.h"
#include "VertexDescriptor.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


vector<HierarchicalOutOfCorePointCloud*> HierarchicalOutOfCorePointCloud::outOfCoreClouds;


MetaClass* HierarchicalOutOfCorePointCloud::HierarchicalOutOfCorePointCloudMetaClass = NULL;

const MetaClass* HierarchicalOutOfCorePointCloud::getInstanceClass() const {
   return getClass();
}

Object* HierarchicalOutOfCorePointCloud::newInstance() {
   return new HierarchicalOutOfCorePointCloud;
}

const MetaClass* HierarchicalOutOfCorePointCloud::getClass() {
   if (HierarchicalOutOfCorePointCloudMetaClass==NULL) {
      HierarchicalOutOfCorePointCloudMetaClass = new MetaClass("HierarchicalOutOfCorePointCloud", "no description.",
                                  newInstance, sizeof(HierarchicalOutOfCorePointCloud));
   }
   return HierarchicalOutOfCorePointCloudMetaClass;
}

void HierarchicalOutOfCorePointCloud::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 2);
   HierarchicalOutOfCorePointCloudMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT(HierarchicalOutOfCorePointCloud)
	ADD_OBJECT_PROP(octree, 0,MHOutOfCorePointSetOctree::getClass(), true); 
   SET_STREAM_PROPERTY(false);
	ADD_OBJECT_PROP(nAlloc, 0, MHOutOfCorePointSetOctreeNode::getClass(), true); 
   SET_STREAM_PROPERTY(false);
}

void HierarchicalOutOfCorePointCloud::shutdown() {
   delete HierarchicalOutOfCorePointCloudMetaClass;
}


// ----


HierarchicalOutOfCorePointCloud::HierarchicalOutOfCorePointCloud() {
   octree = NULL;
   nAlloc = NULL;
   outOfCoreClouds.push_back(this);
}

void HierarchicalOutOfCorePointCloud::clearAndSetup(
   const VertexDescriptor *vertDescr, MHOutOfCorePointSetOctree::NodeAllocator *nAlloc)
{
   delete octree;
   this->nAlloc = nAlloc;
   octree = new MHOutOfCorePointSetOctree();
   octree->clearAndSetup(nAlloc, vertDescr);
}


void HierarchicalOutOfCorePointCloud::assign(const Object* obj, CopyContext *context) {
	throw PException("HierarchicalOutOfCorePointCloud::assign() - assign is not implemented for out-of-core clouds. This methods should not have been called.");

	/*
   PointCloud::assign(obj, context);
   const HierarchicalOutOfCorePointCloud *other = dynamic_cast<const HierarchicalOutOfCorePointCloud*>(obj);
   pAssert(other != NULL);
   delete octree;
   pAssert(nAlloc == other->nAlloc || nAlloc == NULL);
   nAlloc = other->nAlloc;
   tis->octree = (MHOutOfCorePointSetOctree*)(other->octree->copy(context));*/
}

SceneObject *HierarchicalOutOfCorePointCloud::createOOCAwareCopy(string idPrefix, bool insertOriginal, string idSuffix) {
	HierarchicalOutOfCorePointCloud *result = new HierarchicalOutOfCorePointCloud();
	MultiObjectStream *multiObj = nAlloc->getMultiObjectStream()->createOOCAwareCopy(idPrefix, insertOriginal, idSuffix);
	MHOutOfCorePointSetOctree::NodeAllocator *newAlloc = new MHOutOfCorePointSetOctree::NodeAllocator(
		nAlloc->getDynamicNodeType(), multiObj, nAlloc->getMaxNumNodesInMemory()
	);
	result->clearAndSetup(getDescr(), newAlloc);
	result->octree->assign(this->octree);
	return result;
}

void HierarchicalOutOfCorePointCloud::write(OutputObjectStream *s) const {
   PointCloud::write(s);
   s->writeObject(nAlloc);

   s->writeObject(octree);
}

void HierarchicalOutOfCorePointCloud::read(InputObjectStream *s) {
   // legacy scene import
   if (s->getClassVersion(getClass()) < MetaClass::FIRST_VERSION + 2) {

      string fname;
      s->readString(fname);
      forget(fname);

      card32 fflags;
      s->read(fflags);
      forget(fflags);

      card32 fmaterialIndex;
      s->read(fmaterialIndex);
      setMaterialIndex(fmaterialIndex);

      Matrix4f ftransformation;
      s->read(ftransformation);
      setTransformation(ftransformation);

   } else {

      PointCloud::read(s);

   }

   Persistent *nAllocObj;
   s->readObject(nAllocObj);
   nAlloc = (MHOutOfCorePointSetOctreeNode::NodeAllocator*)nAllocObj;

   delete octree;

   Persistent *octreeObj;
   s->readObject(octreeObj);
   octree = (MHOutOfCorePointSetOctree*)octreeObj;
   if (octree != NULL) {
      pAssert(octree->getAlloc() == nAlloc);
   }
}


const VertexDescriptor *HierarchicalOutOfCorePointCloud::getDescr() const {
   return octree->getDescr();
}

PointCloud *HierarchicalOutOfCorePointCloud::appendFields(const VertexDescriptor *newAttribs) const {
   VertexDescriptor *newDescr = (VertexDescriptor*)(this->getDescr()->copy());
   newDescr->mergeAttributes(newAttribs);
	HierarchicalOutOfCorePointCloud *copyCloud = new HierarchicalOutOfCorePointCloud();
	copyCloud->PointCloud::assign(this);
	MHOutOfCorePointSetOctree::NodeAllocator *newAlloc = 
		new MHOutOfCorePointSetOctree::NodeAllocator(
			nAlloc->getDynamicNodeType(),
			nAlloc->getMultiObjectStream(),
			nAlloc->getMaxNumNodesInMemory()
		);
   // mw - 10/18/07 - moved in constructor - nAlloc->getMultiObjectStream()->addGlobalObjectPointer(
   //   nAlloc, 
   //   nAlloc->getMultiObjectStream()->getNumGlobalObjects()+1
   //);
	copyCloud->clearAndSetup(newDescr, newAlloc);
	delete newDescr;
	MHStandardSettings settings;
	this->octree->getSettings(settings);
	copyCloud->octree->setSettings(settings);

	SceneObjectIterator *this_it = const_cast<HierarchicalOutOfCorePointCloud*>(this)->createIterator(SOIT::CAP_BASIC_PC);	   
	SceneObjectIterator *other_it = copyCloud->createIterator(SOIT::CAP_DYNAMIC_PC);
	BasicPointCloudIterator *this_basicIt = dynamic_cast<BasicPointCloudIterator*>(this_it);
	DynamicPointCloudIterator *other_dynamicIt = dynamic_cast<DynamicPointCloudIterator*>(other_it);
	
	const VertexDescriptor *thisDescr = this->getDescr();
	const VertexDescriptor *otherDescr = copyCloud->getDescr();
	
	card8 *thisBuffer = new card8[thisDescr->getSize()];
	card8 *otherBuffer = new card8[otherDescr->getSize()];
	
	while (!this_basicIt->atEnd()) {
		this_basicIt->getVertex((char*)thisBuffer);
		VertexDescriptor::appendAttribs(thisBuffer, thisDescr, otherBuffer, otherDescr);
		other_dynamicIt->add((const char*)otherBuffer); 
		this_basicIt->next();
	}
	
	delete this_it;
	delete other_it;

   return copyCloud;
}

SceneObjectIterator *HierarchicalOutOfCorePointCloud::createIterator(SceneObjectIteratorType type) {
   /// currently shutdown (disconnected from file)
   if (nAlloc == NULL) return NULL;
   if (nAlloc->getMultiObjectStream()->getStatus() == MultiObjectStream::statusDisconnected) return NULL;
   if (!supportsIteratorType(type)) {
      return NULL;
   }

   if (type.hierarchical()) {
      return new MHOutOfCorePointSetOctreeIterator(octree);
   } else {
      MHOutOfCorePointSetOctreeIterator *t = new MHOutOfCorePointSetOctreeIterator(octree);
      SceneObjectIterator *result = new MHAllPointsOutOfCorePointSetOctreeIterator(t);
      delete t;
      return result;
   }
}

bool HierarchicalOutOfCorePointCloud::supportsIteratorType(SceneObjectIteratorType cap) {
   SOIT possibleCaps(  SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_DYNAMIC_PC
                      | SOIT::CAP_INDEXED_PC | SOIT::CAP_RANDOM_ACCESS_PC
                      | SOIT::CAP_HIERARCHICAL | SOIT::CAP_ATTACH_SECONDARY);

	// indexing is only supported for hierarchial iterators
   if (cap.providesCapsFor( SOIT(SOIT::CAP_INDEXED_PC | SOIT::CAP_RANDOM_ACCESS_PC) ) ) {
		return possibleCaps.providesCapsFor(cap) && cap.providesCapsFor(SOIT::CAP_HIERARCHICAL);
   } else {
		return (possibleCaps.providesCapsFor(cap));
	}
}

card64 HierarchicalOutOfCorePointCloud::getNumPoints() {
   return octree->getNumObjects();
}

void HierarchicalOutOfCorePointCloud::clear() {
   octree->clear();
}

void HierarchicalOutOfCorePointCloud::invalidateNodes() {
   if (octree != NULL) {
      octree->invalidateNodes();
   }
}

void HierarchicalOutOfCorePointCloud::shutdownAllocator() {
   if (nAlloc != NULL) {
      MultiObjectStream *mObj = nAlloc->getMultiObjectStream();
      nAlloc->shutdownAllocator();
      delete nAlloc;
      mObj->removeGlobalObjectPointer(nAlloc);
      nAlloc = NULL;
      if (octree != NULL) octree->setNullAllocator();
   }
}

bool HierarchicalOutOfCorePointCloud::isConnected() {
   if (nAlloc == NULL) return false;
   if (nAlloc->getMultiObjectStream()->getStatus() == MultiObjectStream::statusDisconnected) return false;
   return true;
}


HierarchicalOutOfCorePointCloud::~HierarchicalOutOfCorePointCloud() {
   if (nAlloc != NULL && nAlloc->getMultiObjectStream()->getStatus() == MultiObjectStream::statusConnected) {
      shutdownAllocator();
      delete octree;
   }
   delete nAlloc;
   
	vector<HierarchicalOutOfCorePointCloud*>::iterator pos = outOfCoreClouds.begin();
	vector<HierarchicalOutOfCorePointCloud*>::iterator end = outOfCoreClouds.end();
	while (pos != end) {
		if (*pos == this) {
			outOfCoreClouds.erase(pos);
			break;
		}
		++pos;
	}
   
}



