//---------------------------------------------------------------------------
#ifndef HierarchicalOutOfCorePointCloudH
#define HierarchicalOutOfCorePointCloudH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
#include "MHInstances.h"
#include "MHOutOfCorePointSetOctree.h"
//---------------------------------------------------------------------------

namespace geoxl {


class HIERARCHIES_API HierarchicalOutOfCorePointCloud : public PointCloud {
 private:
   static MetaClass *HierarchicalOutOfCorePointCloudMetaClass;
   static Object* newInstance();

   MHOutOfCorePointSetOctreeNode::NodeAllocator *nAlloc;
   MHOutOfCorePointSetOctree *octree;

	static vector<HierarchicalOutOfCorePointCloud*> outOfCoreClouds;

 public:
   HierarchicalOutOfCorePointCloud();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);
   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);
   virtual SceneObject *createOOCAwareCopy(string idPrefix = "", bool insertOriginal = true, string idSuffix = "");

   virtual const VertexDescriptor *getDescr() const;
   virtual PointCloud             *appendFields(const VertexDescriptor *newAttribs) const;
   virtual SceneObjectIterator    *createIterator(SceneObjectIteratorType type);
   virtual bool                    supportsIteratorType(SceneObjectIteratorType cap);
   virtual card64                  getNumPoints();
   virtual void                    clear();

   // new methods ----
   /// initialize instance
   /// vertDescr --> point format
   /// nAlloc --> allocator being global to this point cloud; only a reference is copied!
   ///            nAlloc must exist as long as this point cloud exists (unless a new one is specied by clearAndSetup).
   virtual void clearAndSetup(const VertexDescriptor *vertDescr,
                              MHOutOfCorePointSetOctree::NodeAllocator *nAlloc);
   /// set modified flag for all nodes (force reorganization)
   virtual void invalidateNodes();

   /// syncs & deletes allocator; no iterator will be created afterwards (call before deleting this object);
   virtual void shutdownAllocator();

   /// returns true if beeing properly connected to the swapfile, false if not (e.g. if the allocator has been shutdown)
   virtual bool isConnected();

   MHOutOfCorePointSetOctree *getOctree() {return octree;}

   ~HierarchicalOutOfCorePointCloud();
   
   /// helper function for cleanup at program exit
   static vector<HierarchicalOutOfCorePointCloud*>	getActiveOOCPCs() {return outOfCoreClouds;}
};



}

#endif
