//---------------------------------------------------------------------------
#ifndef MHAllPointsPointSetOctreeIterator_TemplH
#define MHAllPointsPointSetOctreeIterator_TemplH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PointCloudIterators.h"
#include "MHPointSetOctreeIterator_Templ.h"
#include "Tools.h"
//---------------------------------------------------------------------------

namespace geoxl {


template <class OctreeType, class NodeType>
class HIERARCHIES_API MHAllPointsPointSetOctreeIterator_Templ : public PointCloudIterator,
                                                        public BasicPointCloudIterator,
                                                        public ModifyablePointCloudIterator,
                                                        public DynamicPointCloudIterator {
 private:
   /// go to next leaf node in tree, returns true if an additional node is found
   bool searchNextNode();
   bool beyondLastLeafNode;
   vector<card32> nextChildIndexStack;

   MHPointSetOctreeIterator_Templ<OctreeType, NodeType> *baseIterator;

 protected:
   MHAllPointsPointSetOctreeIterator_Templ();
   MHAllPointsPointSetOctreeIterator_Templ(const MHPointSetOctreeIterator_Templ<OctreeType, NodeType> *baseIterator);

 public:
   virtual void assign(const Object* obj, CopyContext *context = NULL);
   virtual bool equal(const Object* obj) const;

   virtual void reset();
   virtual void next();
   virtual bool atEnd();

   virtual card8 get1ub(const AAT &token) const;
   virtual Vector2ub get2ub(const AAT &token) const;
   virtual Vector3ub get3ub(const AAT &token) const;
   virtual Vector4ub get4ub(const AAT &token) const;

   virtual int32 get1i(const AAT &token) const;
   virtual Vector2i get2i(const AAT &token) const;
   virtual Vector3i get3i(const AAT &token) const;
   virtual Vector4i get4i(const AAT &token) const;

   virtual float32 get1f(const AAT &token) const;
   virtual Vector2f get2f(const AAT &token) const;
   virtual Vector3f get3f(const AAT &token) const;
   virtual Vector4f get4f(const AAT &token) const;

   virtual void getVertex(char *buffer) const;
   virtual bool nextBlock();
   virtual const PointSet * lockBlockRead();
   virtual void unlock();

   virtual void set1ub(const AAT &token, const card8 &x)     const;
   virtual void set2ub(const AAT &token, const Vector2ub &v) const;
   virtual void set3ub(const AAT &token, const Vector3ub &v) const;
   virtual void set4ub(const AAT &token, const Vector4ub &v) const;

   virtual void set1i(const AAT &token, const int32 &x)    const;
   virtual void set2i(const AAT &token, const Vector2i &v) const;
   virtual void set3i(const AAT &token, const Vector3i &v) const;
   virtual void set4i(const AAT &token, const Vector4i &v) const;

   virtual void set1f(const AAT &token, const float32 &x)  const;
   virtual void set2f(const AAT &token, const Vector2f &v) const;
   virtual void set3f(const AAT &token, const Vector3f &v) const;
   virtual void set4f(const AAT &token, const Vector4f &v) const;

   virtual void setVertex(const char *data) const;
   virtual PointSet *lockBlockReadWrite(LockRWHint hint = CHANGE_EVERYTHING);

   virtual void add(const char *newPoint);
   virtual void add(const PointSet *newPoints);
   virtual void deleteCurrent();

   virtual ~MHAllPointsPointSetOctreeIterator_Templ();
};



}


#endif
