//---------------------------------------------------------------------------
#ifndef MHAllPointsPointSetOctreeIterator_Templ_templCodeH
#define MHAllPointsPointSetOctreeIterator_Templ_templCodeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MHAllPointsPointSetOctreeIterator_Templ.h"
//---------------------------------------------------------------------------
#include "MHPointSetOctreeIterator_Templ_templCode.h"
//---------------------------------------------------------------------------

namespace geoxl {



template <class OctreeType, class NodeType>
MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------------------
MHAllPointsPointSetOctreeIterator_Templ()
//---------------------------------------
{
   reset();
}



template <class OctreeType, class NodeType>
MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------------------------------------------------------------------------------------------
MHAllPointsPointSetOctreeIterator_Templ(const MHPointSetOctreeIterator_Templ<OctreeType, NodeType> *baseIterator)
//---------------------------------------------------------------------------------------------------------------
{
   this->baseIterator = (MHPointSetOctreeIterator_Templ<OctreeType, NodeType>*)(baseIterator->copy());
   reset();
}



template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------------------------
assign(const Object* obj, CopyContext *context)
//---------------------------------------------
{
   const MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType> *other = dynamic_cast<const MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>*>(obj);
   pAssert(other != NULL);
   delete baseIterator;

   if (other->baseIterator == NULL) {
      baseIterator = NULL;
   } else {
      baseIterator = (MHPointSetOctreeIterator_Templ<OctreeType, NodeType>*)(other->baseIterator->copy(context));
   }

   nextChildIndexStack = other->nextChildIndexStack;
   beyondLastLeafNode = other->beyondLastLeafNode;
}


template <class OctreeType, class NodeType>
bool MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------
equal(const Object* obj) const
//----------------------------
{
   forget(obj);
   pAssert(false);
   return false;
}


template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//-----
reset()
//-----
{
   baseIterator->reset();
   bool atRoot;
   do {
      atRoot = !baseIterator->goToParent();
   } while (!atRoot);
   
   nextChildIndexStack.clear();
   if (baseIterator != NULL) {
      nextChildIndexStack.push_back(0);
      searchNextNode();
   }
   beyondLastLeafNode = false;
}


template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----
next()
//----
{
   if (!baseIterator->atEnd()) {
      baseIterator->next();
   } else {
      searchNextNode();
   }

}


template <class OctreeType, class NodeType>
bool MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//-----
atEnd()
//-----
{
   if (beyondLastLeafNode) {
      return true;
   } else {
      if (!baseIterator->atEnd()) {
         return false;
      } else {
         if (searchNextNode()) {
            return false;
         } else {
            return true;
         }
      }
   }
}



template <class OctreeType, class NodeType>
card8 MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------
get1ub(const AAT &token) const
//----------------------------
{
   return baseIterator->get1ub(token);
}
template <class OctreeType, class NodeType>
Vector2ub MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------
get2ub(const AAT &token) const
//----------------------------
{
   return baseIterator->get2ub(token);
}
template <class OctreeType, class NodeType>
Vector3ub MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------
get3ub(const AAT &token) const
//----------------------------
{
   return baseIterator->get3ub(token);
}
template <class OctreeType, class NodeType>
Vector4ub MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------
get4ub(const AAT &token) const
//----------------------------
{
   return baseIterator->get4ub(token);
}

template <class OctreeType, class NodeType>
int32 MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------
get1i(const AAT &token) const
//---------------------------
{
   return baseIterator->get1i(token);
}
template <class OctreeType, class NodeType>
Vector2i MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------
get2i(const AAT &token) const
//---------------------------
{
   return baseIterator->get2i(token);
}
template <class OctreeType, class NodeType>
Vector3i MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------
get3i(const AAT &token) const
//---------------------------
{
   return baseIterator->get3i(token);
}
template <class OctreeType, class NodeType>
Vector4i MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------
get4i(const AAT &token) const
//---------------------------
{
   return baseIterator->get4i(token);
}

template <class OctreeType, class NodeType>
float32 MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------
get1f(const AAT &token) const
//---------------------------
{
   return baseIterator->get1f(token);
}
template <class OctreeType, class NodeType>
Vector2f MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------
get2f(const AAT &token) const
//---------------------------
{
   return baseIterator->get2f(token);
}
template <class OctreeType, class NodeType>
Vector3f MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------
get3f(const AAT &token) const
//---------------------------
{
   return baseIterator->get3f(token);
}
template <class OctreeType, class NodeType>
Vector4f MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------
get4f(const AAT &token) const
//---------------------------
{
   return baseIterator->get4f(token);
}



template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------------------------
getVertex(char *buffer) const
//---------------------------
{
   baseIterator->getVertex(buffer);
}



template <class OctreeType, class NodeType>
bool MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//---------
nextBlock()
//---------
{
   if (searchNextNode()) {
      return true;
   } else {
      return false;
   }
}



template <class OctreeType, class NodeType>
const PointSet *MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//-------------
lockBlockRead()
//-------------
{
   return baseIterator->lockBlockRead();
}



template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//------
unlock()
//------
{
   baseIterator->unlock();
}



template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//--------------------------------------------
set1ub(const AAT &token, const card8 &x) const
//--------------------------------------------
{
   baseIterator->set1ub(token, x);
}
template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//------------------------------------------------
set2ub(const AAT &token, const Vector2ub &v) const
//------------------------------------------------
{
   baseIterator->set2ub(token, v);
}
template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//------------------------------------------------
set3ub(const AAT &token, const Vector3ub &v) const
//------------------------------------------------
{
   baseIterator->set3ub(token, v);
}
template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//------------------------------------------------
set4ub(const AAT &token, const Vector4ub &v) const
//------------------------------------------------
{
   baseIterator->set4ub(token, v);
}

template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//-------------------------------------------
set1i(const AAT &token, const int32 &x) const
//-------------------------------------------
{
   baseIterator->set1i(token, x);
}
template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------------------------
set2i(const AAT &token, const Vector2i &v) const
//----------------------------------------------
{
   baseIterator->set2i(token, v);
}
template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------------------------
set3i(const AAT &token, const Vector3i &v) const
//----------------------------------------------
{
   baseIterator->set3i(token, v);
}
template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------------------------
set4i(const AAT &token, const Vector4i &v) const
//----------------------------------------------
{
   baseIterator->set4i(token, v);
}

template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------------------------
set1f(const AAT &token, const float32 &x)  const
//----------------------------------------------
{
   baseIterator->set1f(token, x);
}
template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------------------------
set2f(const AAT &token, const Vector2f &v) const
//----------------------------------------------
{
   baseIterator->set2f(token, v);
}
template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------------------------
set3f(const AAT &token, const Vector3f &v) const
//----------------------------------------------
{
   baseIterator->set3f(token, v);
}
template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------------------------
set4f(const AAT &token, const Vector4f &v) const
//----------------------------------------------
{
   baseIterator->set4f(token, v);
}



template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//-------------------------------
setVertex(const char *data) const
//-------------------------------
{
   baseIterator->setVertex(data);
}



template <class OctreeType, class NodeType>
PointSet *MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//------------------
lockBlockReadWrite(LockRWHint hint)
//------------------
{
   return baseIterator->lockBlockReadWrite(hint);
}


template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//-----------------------
add(const char *newPoint)
//-----------------------
{
   baseIterator->add(newPoint);
}



template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------
add(const PointSet *newPoints)
//----------------------------
{
   baseIterator->add(newPoints);
}



template <class OctreeType, class NodeType>
void MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//------------
deleteCurrent()
//------------
{
   baseIterator->deleteCurrent();
}



template <class OctreeType, class NodeType>
bool MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//--------------
searchNextNode()
//--------------
{
   do {
      do {
         const card32 numChildren = baseIterator->getNumChildren();
         card32 nextNodeIndexToVisit = nextChildIndexStack.back();
         if (nextNodeIndexToVisit >= numChildren) {
            if (!baseIterator->goToParent()) {
               beyondLastLeafNode = true;
               return false;
            } else {
               nextChildIndexStack.pop_back();
            }
         } else {
            nextChildIndexStack.back()++;
            if (baseIterator->goToChild(nextNodeIndexToVisit)) {
               nextChildIndexStack.push_back(0);
               break;
            }
         }
      } while (true);
   } while (!baseIterator->isLeaf());
   return true;
}



template <class OctreeType, class NodeType>
MHAllPointsPointSetOctreeIterator_Templ<OctreeType, NodeType>::
//----------------------------------------
~MHAllPointsPointSetOctreeIterator_Templ()
//----------------------------------------
{
   delete baseIterator;
}


}


#endif
