//---------------------------------------------------------------------------
#ifndef MHAllocatorsH
#define MHAllocatorsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PAssert.h"
#include "PTypes.h"
#include "MetaClass.h"
#include "Object.h"
#include "Persistent.h"
#include "MultiPartPersistent.h"
#include "ObjectStreams.h"
//---------------------------------------------------------------------------
#include <vector>
#include <deque>
//---------------------------------------------------------------------------

#ifdef _DEBUG
 // activate compare of non-dirty data
 // very expensive!
 // #define DIRTY_FLAG_DEBUGGING
#endif

/*
 | members/parameters expected by MH template in NodeAllocators
 |
 | template <class AllocatorBaseClass, class NodeType = some dummy type>
 | class MHNodeAllocator {
 |  public:
 |   template<typename RebindNodeType>  // rebind mechanism to resolve cyclic dependencies in template paramter list
 |   class Rebind_NodeType {            // sets static node type
 |    public:
 |      typedef MHNodeAllocator<AllocatorBaseClass, RebindNodeType> ReboundType;
 |   };
 |
 |   class NodePointer { // embedded class or typename named "NodePointer" expected
 |    public:
 |       NodePointer();
 |       NodePointer(const NodePointer &p);
 |       NodePointer(const int &i); // for setting to NULL
 |       NodeType *operator*();
 |       NodeType *operator->(); // for dereferencing
 |       bool operator==(NodePointer);
 |       bool operator!=(NodePointer);
 |       bool operator==(int);
 |       bool operator!=(int); // for testing for equality and testing for NULL pointer (only)
 |       void operator=(int); // for setting to NULL
 |   };
 |
 |   MHNodeAllocator(const MetaClass *dynamicNodeType); // set dynamic node type that is actually instantiated
 |
 |   NodePointer newNode();
 |   static void deleteNode(NodePointer);
 |   NodeType *getPhysicalPtr(NodePointer);
 |   void lruTouch(const NodePointer &p);
 |
 | };
*/


namespace geoxl {
   using namespace std;


class MultiObjectStream;
class InputObjectStream;
class OutputObjectStream;

// ------------------------------ MH_HeapNodeAllocator_Templ ------------------------------



class MH_DummyNodeType : public MultiPartPersistent { 
 public:
	MH_DummyNodeType() {pAssert(false);} 
	bool getDirty() {pAssert(false); return false;}
	void setDirty(bool b) {pAssert(false);}
   void incLockCounter() {pAssert(false);}
   void decLockCounter() {pAssert(false);}
   card32 getLockCounter() {pAssert(false); return 0;}
};


template <class AllocatorBaseClass, class NodeType = MH_DummyNodeType>
class HIERARCHIES_API MH_HeapNodeAllocator_Templ : public AllocatorBaseClass {
 private:

	  const MetaClass *dynamicNodeType;
 public:
   MH_HeapNodeAllocator_Templ(const MetaClass *dynamicNodeType) {this->dynamicNodeType = dynamicNodeType;}

   template<typename RebindNodeType>
   class Rebind_NodeType {
    public:
      typedef MH_HeapNodeAllocator_Templ<AllocatorBaseClass, RebindNodeType> ReboundType;
   };

   typedef NodeType *NodePointer;

   inline NodePointer newNode();
   static void deleteNode(NodePointer p);
   inline NodeType *getPhysicalPtr(NodePointer p);
   inline void lruTouch(const NodePointer &p) {}
   inline void checkLRUSwapOut() {}
   inline bool isInMem(const NodePointer &p) const {return true;}
   inline void prefetch(const NodePointer &p) {}

};



// -------------------------------------- MH_MultiObjectStreamNodeAllocator_Templ --------------------------------------



/** Page-File based allocator. */
template <class AllocatorBaseClass, class NodeType = MH_DummyNodeType>
class HIERARCHIES_API MH_MultiObjectStreamNodeAllocator_Templ : public AllocatorBaseClass {
 private:
   static MetaClass *MH_MultiObjectStreamNodeAllocator_TemplMetaClass;
   static Object* newInstance();

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

 private:
   struct KeyComp {
      static bool equals(card64 a, card64 b) { return a == b; }
   };

   struct MHTEntry {
      NodeType *node;
      list<card64>::iterator lruPos;
      #ifdef DIRTY_FLAG_DEBUGGING
      vector<card8> debugCheckData;
      #endif
   };

   class MyHashTable : public HashTable<card64, MHTEntry, KeyComp> {
    protected:
      virtual max_platform_card hashFunction(card64 key) const {
         return (max_platform_card)(((card32)(key & 0xFFFFFFFFU)) ^ ((card32)(key >> 32)));
      };
   };

   const MetaClass *dynamicNodeType;
   MultiObjectStream *multiObj;
   MyHashTable objectIDToMemIndex;
   list<card64> lruQueue;
   max_platform_card maxNumNodesInMemory;
   //bool noDeepDelete;
   
   void debugCheck(MHTEntry &e);
   void prepareDebugCheck(MHTEntry &e);

 public:
   template<typename RebindNodeType>
   class Rebind_NodeType {
    public:
      typedef MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, RebindNodeType> ReboundType;
   };

   /** Pointer to a block in MH_MultiObjectStreamNodeAllocator_Templ */
   class NodePointer {
    private:
      card64 objectID;
      MH_MultiObjectStreamNodeAllocator_Templ *alloc;

    public:
      NodePointer();
      NodePointer(const NodePointer &p) {*this = p;}
      NodePointer(const int &i) {pAssert(i==0); objectID = 0; alloc = NULL;}
      NodePointer(card64 objectID, MH_MultiObjectStreamNodeAllocator_Templ *alloc);

      inline NodeType *operator*();
      inline const NodeType *operator*() const;
      inline NodeType *operator->();
      inline const NodeType *operator->() const;

      inline bool operator==(const NodePointer &other) const;
      inline bool operator!=(const NodePointer &other) const;

      inline bool operator==(int x) const;
      inline bool operator!=(int x) const;

      inline void operator=(int x);

      inline NodePointer deepCopy(CopyContext *context) const;

      void read(InputObjectStream *s);
      void write(OutputObjectStream *s) const;

      inline MH_MultiObjectStreamNodeAllocator_Templ *getAlloc() {return alloc;}

      friend class MH_MultiObjectStreamNodeAllocator_Templ;
   };

   MH_MultiObjectStreamNodeAllocator_Templ();
   MH_MultiObjectStreamNodeAllocator_Templ(const MetaClass *dynamicNodeType,
                                           MultiObjectStream *multiObj,
                                           max_platform_card maxNumNodesInMemory);

   MultiObjectStream *getMultiObjectStream() {return multiObj;}
   const MetaClass *getDynamicNodeType() {return dynamicNodeType;}
   max_platform_card getMaxNumNodesInMemory() {return maxNumNodesInMemory;}

   inline NodePointer newNode();
   static inline void deleteNode(NodePointer p);
   inline NodeType *getPhysicalPtr(NodePointer p);

   NodeType &getBlock(const NodePointer &p);
   bool isInMem(const NodePointer &p) const;
   void prefetch(const NodePointer &p);
	/// tell the system that the node is beeing used
   void lruTouch(const NodePointer &p);
	/// if too much memory is beeing used, this method swaps out nodes 
	/// according to an LRU scheme; locked nodes are kept in memory
   void checkLRUSwapOut();

   void sync();
   void shutdownAllocator();

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);
   
   void reportStatistics();

   ~MH_MultiObjectStreamNodeAllocator_Templ();
};



// IMPLEMENTATION of some methods here to avoid mysterious linker errors


// ----------------------- MH_MultiObjectStreamNodeAllocator_Templ::NodePointer ------------------------



template <class AllocatorBaseClass, class NodeType>
MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//--------------------------
::NodePointer::NodePointer()
//--------------------------
{
   objectID = 0;
   alloc = NULL;
};


template <class AllocatorBaseClass, class NodeType>
MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//---------------------------------------------------------------------------------------------------------------
::NodePointer::NodePointer(card64 objectID, MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType> *alloc)
//---------------------------------------------------------------------------------------------------------------
{
   this->objectID = objectID;
   this->alloc = alloc;
};


template <class AllocatorBaseClass, class NodeType>
inline NodeType *MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//------------------------
::NodePointer::operator*()
//------------------------
{
   pAssert(objectID != 0);
   return &(alloc->getBlock(*this));
};

template <class AllocatorBaseClass, class NodeType>
inline const NodeType *MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//------------------------
::NodePointer::operator*() const
//------------------------
{
   pAssert(objectID != 0);
   return &(alloc->getBlock(*this));
};


template <class AllocatorBaseClass, class NodeType>
inline NodeType *MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//-------------------------
::NodePointer::operator->()  {
//-------------------------
   pAssert(objectID != 0);
   return &(alloc->getBlock(*this));
};


template <class AllocatorBaseClass, class NodeType>
inline const NodeType *MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//-------------------------------
::NodePointer::operator->() const {
//-------------------------------
   pAssert(objectID != 0);
   return &(alloc->getBlock(*this));
};


template <class AllocatorBaseClass, class NodeType>
inline bool MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//-------------------------------------------------------
::NodePointer::operator==(const NodePointer &other) const
//-------------------------------------------------------
{
   return objectID == other.objectID && alloc == other.alloc;
};


template <class AllocatorBaseClass, class NodeType>
inline bool MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//-------------------------------------------------------
::NodePointer::operator!=(const NodePointer &other) const
//-------------------------------------------------------
{
   return objectID != other.objectID || alloc != other.alloc;
};


template <class AllocatorBaseClass, class NodeType>
inline bool MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//------------------------------------
::NodePointer::operator==(int x) const
//------------------------------------
{
   pAssert(x == NULL);
   return objectID == 0;
};


template <class AllocatorBaseClass, class NodeType>
inline bool MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//------------------------------------
::NodePointer::operator!=(int x) const
//------------------------------------
{
   pAssert(x == NULL);
   return objectID != 0;
};


template <class AllocatorBaseClass, class NodeType>
inline void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//-----------------------------
::NodePointer::operator=(int x)
//-----------------------------
{
   pAssert(x == NULL);
   objectID = 0;
};

              
template <class AllocatorBaseClass, class NodeType>
inline typename MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::NodePointer
MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::NodePointer
//------------------------------------
::deepCopy(CopyContext *context) const  
//------------------------------------
{
   NodePointer result;
   if (objectID == 0) {
      result.objectID = 0;
      result.alloc = alloc;
   } else {
      result = alloc->newNode();
      pAssert(result != NULL);
      alloc->getBlock(result).assign(&(alloc->getBlock(*this)), context);
   }
   return result;
};           



// ---- MH_MultiObjectStreamNodeAllocator_Templ 




template <class AllocatorBaseClass, class NodeType>
void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::NodePointer::
// -------------------------
read(InputObjectStream *s)
// -------------------------
{
   s->read(objectID);
   if (objectID != 0) {
      s->readAnyObject(alloc);
   } else {
      alloc = NULL;
   }
}


template <class AllocatorBaseClass, class NodeType>
void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::NodePointer::
// ---------------------------------
write(OutputObjectStream *s) const
// ---------------------------------
{
   s->write(objectID);
   if (objectID != 0) {
      s->writeObject((Persistent*)alloc);
   }
}




}

#endif
