//---------------------------------------------------------------------------
#ifndef MHAllocators_templCodeH
#define MHAllocators_templCodeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MHAllocators.h"
//---------------------------------------------------------------------------
#include "HashTables.h"
#include "HashTables_templCode.h"
#include "NumericalClassProperty.h"
#include "numericalclassproperty_templcode.h"
#include "ObjectStreams.h"
#include "MultiObjectStream.h"
#include "ObjectClassProperty.h"
//#ifdef _DEBUG
   #define SHOW_STATISTICS
	#ifdef _DEBUG
		//#define SHOW_DETAILED_STATISTICS
	#endif
   #include "EventCounterDisplay.h"
//#endif
#include "DebugOutput.h"
#ifdef DIRTY_FLAG_DEBUGGING
	#include "MemoryObjectStreams.h"
#endif
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;


// ---------------------------------------------------------------------------------------------
// -------------------------------- MH_HeapNodeAllocator_Templ ---------------------------------
// ---------------------------------------------------------------------------------------------


// ---- MH_MultiObjectStreamNodeAllocator_Templ


template <class AllocatorBaseClass, class NodeType>
MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// --------------------------------------
MH_MultiObjectStreamNodeAllocator_Templ()
// --------------------------------------
{
   dynamicNodeType = NULL;
   multiObj = NULL;
   //noDeepDelete = false;
}


template <class AllocatorBaseClass, class NodeType>
MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// -------------------------------------------------------------------
MH_MultiObjectStreamNodeAllocator_Templ(
   const MetaClass *dynamicNodeType,
   MultiObjectStream *multiObj, max_platform_card maxNumNodesInMemory)
// -------------------------------------------------------------------
{
   pAssert(dynamicNodeType != NULL);
   pAssert(multiObj != NULL);
   this->dynamicNodeType = dynamicNodeType;
   this->multiObj = multiObj;
   this->maxNumNodesInMemory = maxNumNodesInMemory;
   pAssert(!multiObj->hasGlobalObjectPointer(this));
   multiObj->addGlobalObjectPointer(
      this, 
      multiObj->getNumGlobalObjects()+1
   );
   //noDeepDelete = false;
}


template <class AllocatorBaseClass, class NodeType>
inline typename MH_HeapNodeAllocator_Templ<AllocatorBaseClass, NodeType>::NodePointer MH_HeapNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
//---------
newNode()
//---------
{
   return (NodePointer)(dynamicNodeType->newInstance());
}

template <class AllocatorBaseClass, class NodeType>
void MH_HeapNodeAllocator_Templ<AllocatorBaseClass, NodeType>
//-------------------------
::deleteNode(NodePointer p)
//-------------------------
{
   delete p;
}

template <class AllocatorBaseClass, class NodeType>
inline NodeType*
MH_HeapNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
//-----------------------------
getPhysicalPtr(NodePointer p) 
//-----------------------------
{
	return p;
}




// -----------------------------------------------------------------------------------------------------
// ------------------------------ MH_MultiObjectStreamNodeAllocator_Templ ------------------------------
// -----------------------------------------------------------------------------------------------------



template <class AllocatorBaseClass, class NodeType>
inline typename MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::NodePointer
MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// ------
newNode()
// ------
{

   NodeType *newNode = (NodeType*)(dynamicNodeType->newInstance());
   #ifdef SHOW_STATISTICS
   {
		#ifdef SHOW_DETAILED_STATISTICS
      EventCounterDisplay::Increment e;
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("nodes in memory"), &e);
      EventCounterDisplay::Display d;
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("nodes in memory"), &d);
      #endif

      EventCounterDisplay::Increment e2;
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("#nodes"), &e2);
      EventCounterDisplay::Display d2;
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("#nodes"), &d2);
   }
   #endif

   card64 newBlock = multiObj->createObjectEntry();

   EventCounterDisplay::Increment evd;
   debugOutput.displayEvent(EventCounterDisplay::getClass(), string("create nodes"), &evd);		

   lruQueue.push_front(newBlock);

   MHTEntry e;
   e.node = newNode;
   e.lruPos = lruQueue.begin();
   newNode->setDirty(true);
   objectIDToMemIndex.addElement(newBlock, e);

   pAssert(objectIDToMemIndex.count() > 0 && lruQueue.begin() == e.lruPos);

   NodePointer result = NodePointer(newBlock, this);

   return result;
}


template <class AllocatorBaseClass, class NodeType>
inline void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// ----------------------
deleteNode(NodePointer p)
// ----------------------
{
   if (p.objectID == 0) return;
   //if (p.alloc->noDeepDelete) return;

   pAssert(p.alloc != NULL);

   p.alloc->multiObj->deleteObject(p.objectID);
   EventCounterDisplay::Increment evd;
   debugOutput.displayEvent(EventCounterDisplay::getClass(), string("delete nodes"), &evd);

   MHTEntry e;

   bool inMem = p.alloc->objectIDToMemIndex.getData(p.objectID, e);

   if (inMem) {
      p.alloc->lruQueue.erase(e.lruPos);
      p.alloc->objectIDToMemIndex.deleteElement(p.objectID);
      delete e.node;
      #ifdef SHOW_STATISTICS
      {
			#ifdef SHOW_DETAILED_STATISTICS
         EventCounterDisplay::Decrement e;
         debugOutput.displayEvent(EventCounterDisplay::getClass(), string("nodes in memory"), &e);
         EventCounterDisplay::Display d;
         debugOutput.displayEvent(EventCounterDisplay::getClass(), string("nodes in memory"), &d);
         #endif
         
         EventCounterDisplay::Decrement e2;
         debugOutput.displayEvent(EventCounterDisplay::getClass(), string("#nodes"), &e2);
         EventCounterDisplay::Display d2;
         debugOutput.displayEvent(EventCounterDisplay::getClass(), string("#nodes"), &d2);
      }
      #endif
   }
}


template <class AllocatorBaseClass, class NodeType>
inline NodeType*
MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// --------------------------
getPhysicalPtr(NodePointer p)
// --------------------------
{
	if (p.objectID == 0) {
		return NULL;
	} else {
		return &getBlock(p);
	}
}


template <class AllocatorBaseClass, class NodeType>
void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// ---
sync()
// ---
{
   for (list<card64>::iterator pos = lruQueue.begin(); pos != lruQueue.end(); ++pos) {
      MHTEntry e;
      bool inMem = objectIDToMemIndex.getData(*pos, e);
      pAssert(inMem);
      if (e.node->getDirty()) {
         multiObj->updateObject(*pos, e.node);
         e.node->setDirty(false);
         objectIDToMemIndex.addElement(*pos, e);
         prepareDebugCheck(e);
		   EventCounterDisplay::Increment evd;
	      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("write nodes"), &evd);
      } else {
			debugCheck(e);
      }
   }
   multiObj->sync();
}

template <class AllocatorBaseClass, class NodeType>
void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// ----------------
shutdownAllocator()
// ----------------
{
   sync();
   multiObj->sync();
   multiObj = NULL;
}




template <class AllocatorBaseClass, class NodeType>
inline NodeType &MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// ------------------------------------------
getBlock(const NodePointer &p)
// ------------------------------------------
{
   // OutputDebugString((AnsiString("access: ") + IntToStr((int)p.objectID)).c_str());

   MHTEntry e;
   bool success = objectIDToMemIndex.getData(p.objectID, e);
   if (!success) {
      lruQueue.push_front(p.objectID);
      e.node = (NodeType*)(multiObj->getObject(p.objectID));
      e.node->setDirty(false);
      prepareDebugCheck(e);
		#ifdef SHOW_DETAILED_STATISTICS
      {
         EventCounterDisplay::Increment e;
         debugOutput.displayEvent(EventCounterDisplay::getClass(), string("nodes in memory"), &e);
         EventCounterDisplay::Display d;
         debugOutput.displayEvent(EventCounterDisplay::getClass(), string("nodes in memory"), &d);
      }
      #endif
      EventCounterDisplay::Increment evd;
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("fetch nodes"), &evd);
      e.lruPos = const_cast<MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>*>(this)->lruQueue.begin();
      pAssert(e.node != NULL);
      objectIDToMemIndex.addElement(p.objectID, e);
      pAssert(e.lruPos == lruQueue.begin());
   } 
   return *(e.node);
}

template <class AllocatorBaseClass, class NodeType>
inline void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// ------------------------------------------
lruTouch(const NodePointer &p)
// ------------------------------------------
{
	if (p.objectID == 0) return;
   MHTEntry e;
   bool success = objectIDToMemIndex.getData(p.objectID, e);
   if (success) {
      if (lruQueue.front() != *(e.lruPos)) {
         lruQueue.erase(e.lruPos);
         lruQueue.push_front(p.objectID);
         e.lruPos = const_cast<MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>*>(this)->lruQueue.begin();
         objectIDToMemIndex.addElement(p.objectID, e);
      }
	}
}


template <class AllocatorBaseClass, class NodeType>
inline bool MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// --------------------------------
isInMem(const NodePointer &p) const
// --------------------------------
{
   MHTEntry e;
   if (objectIDToMemIndex.getData(p.objectID, e)) {
		return true;
	} else {
		return multiObj->objectAvailable(p.objectID);
	}
}


template <class AllocatorBaseClass, class NodeType>
void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// ---------------------------
prefetch(const NodePointer &p)
// ---------------------------
{
	if (!isInMem(p)) {
		multiObj->prefetchObject(p.objectID);
      EventCounterDisplay::Increment evd;
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("prefetch nodes"), &evd);
	}
}




template <class AllocatorBaseClass, class NodeType>
void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>
// ----------------
::checkLRUSwapOut()
// ----------------
{
   max_platform_card count = objectIDToMemIndex.count();
	list<card64>::iterator pos = lruQueue.end();
	list<card64>::iterator firstLRUEntry = lruQueue.begin();
	
	// queue empty?
	if (firstLRUEntry == pos) return;
	
	// go to last element in queue
	pos--;
	bool atEnd = false;
   
   while (count > maxNumNodesInMemory && !atEnd) {
      card64 nextDel = *pos;

      MHTEntry e;
      bool success = objectIDToMemIndex.getData(nextDel, e);
      pAssert(success);

		if (e.node->getLockCounter() == 0) {

			if (e.node->getDirty()) {
				e.node->setDirty(false);
				multiObj->updateObject(nextDel, e.node);
				EventCounterDisplay::Increment evd;
		      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("write nodes"), &evd);
			} else {
				debugCheck(e);
			}

			//noDeepDelete = true;
			//delete e.node;
			if (e.node) {  // changed (noDeepDelete should not be necessary anymore)
				e.node->shallowDelete();
			}
			//noDeepDelete = false;

			objectIDToMemIndex.deleteElement(nextDel);

			if (pos == firstLRUEntry) {
				lruQueue.erase(pos);
				atEnd = true;
			} else {
				list<card64>::iterator nextPos = pos;
				nextPos--;
				lruQueue.erase(pos);				
				pos = nextPos;
			}

			count--;
		
		}  else {
			if (pos == firstLRUEntry) {
				atEnd = true;
			} else {
				pos--;
			}
		}
   }
	#ifdef SHOW_DETAILED_STATISTICS
   {
      EventCounterDisplay::Display d;
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("nodes in memory"), &d);
   }
   #endif
}


 
template <class AllocatorBaseClass, class NodeType>
MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::
// ---------------------------------------
~MH_MultiObjectStreamNodeAllocator_Templ()
// ---------------------------------------
{
   max_platform_card count = objectIDToMemIndex.count();
   while (count > 0) {
      card64 nextDel = lruQueue.front(); // delete front-to-back to reduce thrashing
      lruQueue.pop_front();

      MHTEntry e;
      bool success = objectIDToMemIndex.getData(nextDel, e);
      pAssert(success);

		if (e.node->getDirty()) {
			e.node->setDirty(false);
         multiObj->updateObject(nextDel, e.node);
      } else {
			debugCheck(e);
      }

      //noDeepDelete = true;
      //delete e.node;
		if (e.node) {  // changed (noDeepDelete should not be necessary anymore)
			e.node->shallowDelete();
		}
      //noDeepDelete = false;
	   EventCounterDisplay::Increment evd;
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("write nodes"), &evd);
		#ifdef SHOW_DETAILED_STATISTICS
      {
         EventCounterDisplay::Decrement e;
         debugOutput.displayEvent(EventCounterDisplay::getClass(), string("nodes in memory"), &e);
      }
      #endif

      objectIDToMemIndex.deleteElement(nextDel);

      count--;
   }
	#ifdef SHOW_DETAILED_STATISTICS
   {
      EventCounterDisplay::Display d;
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("nodes in memory"), &d);
   }
   #endif
   if (multiObj) multiObj->removeGlobalObjectPointer(this);
}


// --- MH_MultiObjectStreamNodeAllocator_Templ registration

template <class AllocatorBaseClass, class NodeType>
MetaClass* MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::MH_MultiObjectStreamNodeAllocator_TemplMetaClass = NULL;

template <class AllocatorBaseClass, class NodeType>
const MetaClass* MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::getInstanceClass() const {
   return getClass();
}

template <class AllocatorBaseClass, class NodeType>
Object* MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::newInstance() {
   return new MH_MultiObjectStreamNodeAllocator_Templ;
}

template <class AllocatorBaseClass, class NodeType>
const MetaClass* MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::getClass() {
   if (MH_MultiObjectStreamNodeAllocator_TemplMetaClass==NULL) {
      MH_MultiObjectStreamNodeAllocator_TemplMetaClass = new MetaClass("MH_MultiObjectStreamNodeAllocator_Templ", "no description.",
                                  newInstance, sizeof(MH_MultiObjectStreamNodeAllocator_Templ));
   }
   return MH_MultiObjectStreamNodeAllocator_TemplMetaClass;
}

template <class AllocatorBaseClass, class NodeType>
void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   MH_MultiObjectStreamNodeAllocator_TemplMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(MH_MultiObjectStreamNodeAllocator_Templ)
   ADD_MAX_PLATFORM_CARD_PROP(maxNumNodesInMemory, 0)
   ADD_OBJECT_PROP(multiObj, 0, MultiObjectStream::getClass(), false)
}

template <class AllocatorBaseClass, class NodeType>
void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::shutdown() {
   delete MH_MultiObjectStreamNodeAllocator_TemplMetaClass;
}


// ----


template <class AllocatorBaseClass, class NodeType>
void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::write(OutputObjectStream *s) const {
   s->writeMetaClass(dynamicNodeType);
   s->writeObject(multiObj);
   s->writeMaxPlatformCard(maxNumNodesInMemory);
}

template <class AllocatorBaseClass, class NodeType>
void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::read(InputObjectStream *s) {
   lruQueue.clear();
   objectIDToMemIndex.clear();
   s->readMetaClass(dynamicNodeType);
   Persistent *multiObjPersistent;
   s->readObject(multiObjPersistent);
   multiObj = (MultiObjectStream*)multiObjPersistent;
   s->readMaxPlatformCard(maxNumNodesInMemory);
}

#ifdef DIRTY_FLAG_DEBUGGING
	template <class AllocatorBaseClass, class NodeType>
	void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::debugCheck(MHTEntry &e) 
	{
		MemoryOutputObjectStream out;
		out.writeObject(e.node);
		const vector<card8> *newVal = out.closeAndGetMemoryBuffer();
		if (*newVal != e.debugCheckData) {
			error("non dirty data has been modified.");
			pAssert(false);
		}
	}
	
	template <class AllocatorBaseClass, class NodeType>
	void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::prepareDebugCheck(MHTEntry &e) 
	{
		MemoryOutputObjectStream out;
		out.writeObject(e.node);
		const vector<card8> *newVal = out.closeAndGetMemoryBuffer();
		e.debugCheckData = *newVal;
	}
#else
	template <class AllocatorBaseClass, class NodeType>
	void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::debugCheck(MHTEntry &e) {}
	template <class AllocatorBaseClass, class NodeType>
	void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::prepareDebugCheck(MHTEntry &e) {}
#endif


template <class AllocatorBaseClass, class NodeType>
void MH_MultiObjectStreamNodeAllocator_Templ<AllocatorBaseClass, NodeType>::reportStatistics() {
      EventCounterDisplay::NewValue v;
      v.value = (int32)lruQueue.size();
      if (objectIDToMemIndex.getNumElements() != lruQueue.size()) {
			throw PException("objectIDToMemIndex.getNumElements() != lruQueue.size()");
      }
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("nodes in mem"), &v);
}






}

#endif
