//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "MHInCorePointSetOctree.h"
//---------------------------------------------------------------------------
#include "MathIOTools.h"
#include "ObjectClassProperty.h"
#include "EmbeddedObjectClassProperty.h"
#include "EmbeddedObjectClassProperty_templCode.h"
#include "FixedArrayClassProperty.h"
#include "NumericalClassProperty.h"
#include "numericalclassproperty_templcode.h"
#include "MHNodes_templCode.h"
#include "MHHierarchy_templCode.h"
#include "SparseGrid_templCode.h"
#include "MHInstances_templCode.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


/// explicit instantiation
template class MH_DDimQuadtreeNode_Templ < float32, 3, MHPointSetPointObject, MH_HeapNodeAllocator_Templ<NULL_CLASS>,
                                           MHInCorePointSetNodeData, MHSparseGridMultiRes, MHStandardSettings, card32, Persistent >;
template class MHPointHierachy_TemplBase < float32, 3, MHPointSetPointObject, MHInCorePointSetOctreeNode, MHStandardSettings, Persistent >;


MetaClass* MHInCorePointSetOctreeNode::MHInCorePointSetOctreeNodeMetaClass = NULL;

const MetaClass* MHInCorePointSetOctreeNode::getInstanceClass() const {
   return getClass();
}

Object* MHInCorePointSetOctreeNode::newInstance() {
   return new MHInCorePointSetOctreeNode;
}

const MetaClass* MHInCorePointSetOctreeNode::getClass() {
   if (MHInCorePointSetOctreeNodeMetaClass==NULL) {
      MHInCorePointSetOctreeNodeMetaClass = new MetaClass("MHInCorePointSetOctreeNode", "no description.",
                                  newInstance, sizeof(MHInCorePointSetOctreeNode));
   }
   return MHInCorePointSetOctreeNodeMetaClass;
}

void MHInCorePointSetOctreeNode::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   MHInCorePointSetOctreeNodeMetaClass->copyParentProperties();
   
   BEGIN_CLASS_INIT(MHInCorePointSetOctreeNode)
   ADD_CARD32_PROP( flags, 0 )
   ADD_OBJECT_FIXED_ARRAY_PROP( children, 0, 8, MHInCorePointSetOctreeNode::getClass(), true )
   ADD_EMBEDDED_OBJECT_PROP( nodeData, 0, MHInCorePointSetNodeData::getClass() )
   ADD_CARD32_PROP( objsInSubtree, 0 )
}

void MHInCorePointSetOctreeNode::shutdown() {
   delete MHInCorePointSetOctreeNodeMetaClass;
}


// ----


MetaClass* MHInCorePointSetOctree::MHInCorePointSetOctreeMetaClass = NULL;

const MetaClass* MHInCorePointSetOctree::getInstanceClass() const {
   return getClass();
}

Object* MHInCorePointSetOctree::newInstance() {
   return new MHInCorePointSetOctree;
}

const MetaClass* MHInCorePointSetOctree::getClass() {
   if (MHInCorePointSetOctreeMetaClass==NULL) {
      MHInCorePointSetOctreeMetaClass = new MetaClass("MHInCorePointSetOctree", "no description.",
                                  newInstance, sizeof(MHInCorePointSetOctree));
   }
   return MHInCorePointSetOctreeMetaClass;
}

void MHInCorePointSetOctree::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   MHInCorePointSetOctreeMetaClass->copyParentProperties();

   BEGIN_CLASS_INIT(MHInCorePointSetOctree)
   ADD_EMBEDDED_OBJECT_PROP(settings, 0, MHStandardSettings::getClass())
   ADD_OBJECT_PROP(descr, 0, VertexDescriptor::getClass(), true);
   ADD_OBJECT_PROP(root, 0, MHInCorePointSetOctree::getClass(), true);
}

void MHInCorePointSetOctree::shutdown() {
   delete MHInCorePointSetOctreeMetaClass;
}


// ----


// -- MHInCorePointSetOctreeNode


void MHInCorePointSetOctreeNode::write(OutputObjectStream *s) const {
   s->write(flags);
   for (card32 i=0; i<MAX_NUM_CHILDREN; i++) {
      s->writeObject(children[i]);
   }
   s->includeClass(MHInCorePointSetNodeData::getClass());
   multiRes.write(s);
   nodeData.write(s);
   s->write(objsInSubtree);
   s->writeObject(parent);
   writeBoundingBox3f(s, boundingBox);
}

void MHInCorePointSetOctreeNode::read(InputObjectStream *s) {
   s->read(flags);
   for (card32 i=0; i<MAX_NUM_CHILDREN; i++) {
      s->readAnyObject(children[i]);
   }
   multiRes.read(s);
   nodeData.read(s);
   s->read(objsInSubtree);
   s->readAnyObject(parent);
   readBoundingBox3f(s, boundingBox);
}

void MHInCorePointSetOctreeNode::assign(const Object* obj, CopyContext *context) {
   const MHInCorePointSetOctreeNode *other = dynamic_cast<const MHInCorePointSetOctreeNode*>(obj);
   pAssert(other != NULL);
   flags = other->flags;
   boundingBox = other->boundingBox;
   for (card32 i=0; i<MAX_NUM_CHILDREN; i++) {
      delete children[i];
      if (other->children[i] == NULL) {
         children[i] = NULL;
      } else {
         children[i] = (MHInCorePointSetOctreeNode*)(other->children[i]->copy(context));
         children[i]->setParent(this);
      }
   }
   nodeData.assign(&other->nodeData, context);
   multiRes.assign(&other->multiRes, context);
   objsInSubtree = other->objsInSubtree;
}



// -- MHInCorePointSetOctree


void MHInCorePointSetOctree::write(OutputObjectStream *s) const {
   s->includeClass(settings.getInstanceClass());
   settings.write(s);
   s->writeObject(descr);
   s->writeObject(root);
   writeBoundingBox3f(s, rootBB);
   s->write(numObjects);
}


void MHInCorePointSetOctree::read(InputObjectStream *s) {
   settings.read(s);
   delete descr;
   s->readAnyObject(descr);
   if (nAlloc == NULL) nAlloc = new MHInCorePointSetOctreeNode::NodeAllocator(MHInCorePointSetOctreeNode::getClass());
   delete root;
   s->readAnyObject(root);
   readBoundingBox3f(s, rootBB);
   if (s->getClassVersion(MHInCorePointSetOctree::getClass()) >= MetaClass::FIRST_VERSION + 1) {
      s->read(numObjects);
   }
}


void MHInCorePointSetOctree::assign(const Object* obj, CopyContext *context) {
   const MHInCorePointSetOctree *other = dynamic_cast<const MHInCorePointSetOctree*>(obj);
   pAssert(other != NULL);

   setSettings(other->settings);
   delete descr;
   descr = (VertexDescriptor*)(other->descr->copy(context));
   if (nAlloc == NULL) nAlloc = new MHInCorePointSetOctreeNode::NodeAllocator(MHInCorePointSetOctreeNode::getClass());
   delete root;
   root = (MHInCorePointSetOctreeNode*)(other->root->copy(context));
   rootBB = other->rootBB;
   numObjects = other->numObjects;
}

