//---------------------------------------------------------------------------
#ifndef MHInCorePointSetOctreeH
#define MHInCorePointSetOctreeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MHInstances.h"
#include "Persistent.h"
#include "MHNodes.h"
#include "MHHierarchy.h"
//---------------------------------------------------------------------------

namespace geoxl {


// ---- the octree nodes


typedef MH_DDimQuadtreeNode_Templ
 <
   float32,                                 // typename FloatType
   3,                                       // int Dimension
   MHPointSetPointObject,                   // typename ObjectType
   MH_HeapNodeAllocator_Templ<NULL_CLASS>,  // typename NodeAllocator
   MHInCorePointSetNodeData,                // typename NodeDataType
   MHSparseGridMultiRes,                    // typename MultiRes
   MHStandardSettings,                      // typename SettingsType
   card32,                                  // typename SubtreeCounterType
   Persistent                               // class NodeBaseClass
 >
// -------------------------------
   MHInCorePointSetOctreeNodeBase;
// -------------------------------


class HIERARCHIES_API MHInCorePointSetOctreeNode : public MHInCorePointSetOctreeNodeBase {
 private:
   static MetaClass *MHInCorePointSetOctreeNodeMetaClass;
   static Object* newInstance();

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);
   virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual inline void incLockCounter() {}
   virtual inline void decLockCounter() {}
   virtual inline card32 getLockCounter() const {return 0;}
};


// ---- the octree itself


typedef MHPointHierachy_TemplBase
 <
   float32,                       // typename FloatType
   3,                             // int Dimension
   MHPointSetPointObject,         // typename ObjectType
   MHInCorePointSetOctreeNode,    // typename NodeType
   MHStandardSettings,            // typename SettingsType
   Persistent                     // class HierarchyBaseClass
 >
// -------------------------
 MHInCorePointSetOctreeBase;
// -------------------------


class HIERARCHIES_API MHInCorePointSetOctree : public MHInCorePointSetOctreeBase {
 private:
   static MetaClass *MHInCorePointSetOctreeMetaClass;
   static Object* newInstance();

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);
   virtual void assign(const Object* obj, CopyContext *context = NULL);

   NodeAllocator *getAlloc() {return nAlloc;}

   void outputDebugStatisticsShort() const {}
   
   /// disable debug conditions
   static bool noLockCounting() {return true;}
};



}

#endif
