//---------------------------------------------------------------------------
#ifndef MHMultiResH
#define MHMultiResH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "Persistent.h"
#include "SparseGrid.h"
#include "BoundingBox.h"
#include "BoundingBox_templCode.h"
#include "VertexDescriptor.h"
//---------------------------------------------------------------------------

namespace geoxl {


class MHStandardSettings;
class VertexDescriptor;
class MHPointSetPointObject;
class PointSet;

/*
 | members/parameters expected by MH template in MHMultiRes
 |
 | template <typename FloatType, int Dimension, typename ObjectType, typename SettingsType>
 | class MHMultiRes {
 |  public:
 |    void clear();
 |    // a new object has been added
 |    void topDownAddObject(const ObjectType &obj, const BoundingBox<FloatType, Dimension> &bb, SettingsType &settings);
 |    // an object has been removed
 |    void topDownRemoveObject(const ObjectType &obj, const BoundingBox<FloatType, Dimension> &bb, SettingsType &settings);
 |    // the attributes of an object have been changed (not the position)
 |    void topDownChangeObject(const ObjectType &obj, const BoundingBox<FloatType, Dimension> &bb, SettingsType &settings);
 |    // insert a whole subtree (necessary for extending the root node)
 |    void topDownAddMultiRes(const MHSparseGridMultiRes &other);
 |    // void buttomUpRebuildFromChildren(const NodeType &node);
 |    max_platform_card getNumObjects();
 |    ObjectType getObject(max_platform_card index);
 | };
 |
*/




template <typename FloatType, int Dimension, typename ObjectType, typename SettingsType>
class MHNullMultiResolution {
 public:
   inline void clear() {}
   inline void topDownAddObject(const ObjectType &obj,
                                const BoundingBox<FloatType, Dimension> &bb,
                                SettingsType &settings)
   {
      forget(obj); forget(settings);
   }
   inline void topDownRemoveObject(const ObjectType &obj,
                                   const BoundingBox<FloatType, Dimension> &bb,
                                   SettingsType &settings)
   {
      forget(obj); forget(settings);
   }
   inline void topDownChangeObject(const ObjectType &obj,
                                   const BoundingBox<FloatType, Dimension> &bb,
                                   SettingsType &settings)
   {
      forget(obj); forget(settings);
   }

   // inline void buttomUpRebuildFromChildren(const NodeType &node) {forget(node);}
   inline max_platform_card getNumObjects() {return 0;}
   inline ObjectType getObject(max_platform_card index) {throw PException("not available.");}
};




class HIERARCHIES_API MHSparseGridMultiRes : public Persistent {
 private:
   struct GridEntryType {
      card32 index;
      card64 weight;
   };
   SparseGrid<3, GridEntryType, Persistent> grid;
   PointSet *gridPoints;
   AAT aatPosition;
   AAT aatFlags;

 public:
   MHSparseGridMultiRes();

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);
   virtual void assign(const Object* obj, CopyContext *context = NULL);

   void clear();
   
   void topDownAddObject(const MHPointSetPointObject &obj,
                         const BoundingBox3f &bb,
                         MHStandardSettings &settings);
   void topDownRemoveObject(const MHPointSetPointObject &obj,
                            const BoundingBox3f &bb,
                            MHStandardSettings &settings);
   void topDownChangeObject(const MHPointSetPointObject &obj,
                            const BoundingBox3f &bb,
                            MHStandardSettings &settings);
   void topDownAddMultiRes(const MHSparseGridMultiRes &other,
                           const BoundingBox3f &bb,
                           MHStandardSettings &settings);

   max_platform_card getNumObjects() const;
   MHPointSetPointObject getObject(max_platform_card index);

   inline const PointSet *getPointSet() const {return gridPoints;}
   inline       PointSet *getPointSet()       {return gridPoints;}

   void appendFields(const VertexDescriptor *newAttribs);

   virtual ~MHSparseGridMultiRes();

};





}

#endif
