//---------------------------------------------------------------------------
#ifndef MHNodeDataH
#define MHNodeDataH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------
#include "BoundingBox.h"
#include "PTypes.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "MHObjects.h"
#include "AttachedData.h"
//---------------------------------------------------------------------------

/*
 | members/parameters expected by MH template in NodeData
 |
 | template <typename ObjectType, typename SubtreeObjCounterType>
 | class MHNodeData {
 |  public:
 |    max_platform_card getNumObjects();
 |    void addObject(const ObjectType &obj);
 |    ObjectType getObject(max_platform_card index);
 |    void setObject(max_plattform_card index, const ObjectType &obj);
 |    void deleteObject(max_platform_card index);
 |    void clear(bool clearFlagsCounters = true);
 |    void incSubtreeObjCounter();
 |    void decSubtreeObjCounter();
 |    void incFlagsCounters(card32 flags);
 |    void decFlagsCounters(card32 flags);
 |    void copyFlagsCounters(const MHNodeData &other);
 |    void decFlagsCounters(const MHNodeData &other);
 |    SubtreeObjCounterType getFlagsCounter(card32 flag);
 | };
*/


namespace geoxl {
   using namespace std;

class OutputObjectStream;
class InputObjectStream;
class PointSet;
class VertexDescriptor;

// ------------------------------ MHNodeDataObjectVector ------------------------------



template <typename ObjectType, typename FloatType, int Dimension>
class HIERARCHIES_API MHNodeDataObjectVector {
 private:
   vector<ObjectType> objs;
 public:
   inline max_platform_card getNumObjects() const;
   inline void addObject(const ObjectType &obj, const BoundingBox<FloatType, Dimension> &nodeBB);
   inline ObjectType getObject(max_platform_card index);
   inline void setObject(max_platform_card index, const ObjectType &obj);
   inline void deleteObject(max_platform_card index);
   inline void clear(bool clearFlagsCounters = true);
};



// -------------------------------- MHPointSetNodeData --------------------------------



template <typename SubtreeObjCounterType, int numCountedFlags>
class HIERARCHIES_API MHPointSetNodeData : public Persistent {
 protected:
   PointSet *ps;
   AttachedDataContainer *attachedData;
   AAT aatPosition;
   AAT aatFlags;
   SubtreeObjCounterType numFlaggedInSubtree[numCountedFlags];

 public:
   inline MHPointSetNodeData();

   inline max_platform_card getNumObjects() const;
   inline void addObject(const MHPointSetPointObject &obj, const BoundingBox3f &nodeBB);
   inline MHPointSetPointObject getObject(max_platform_card index);
   inline void setObject(max_platform_card index, const MHPointSetPointObject &obj);
   inline void deleteObject(max_platform_card index);
   inline void clear(bool clearFlagsCounters = true);

   inline PointSet *getPointSet() {return ps;}
   inline const PointSet *getPointSet() const {return ps;}
   void appendFields(const VertexDescriptor *newAttribs);

   AttachedDataContainer *getAttachments();

   inline void touchData(AttachedData::ChangeType whatChanged);

   void incFlagsCounters(card32 flags);
   void decFlagsCounters(card32 flags);
   void copyFlagsCounters(const MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags> &other);
   void decFlagsCounters(const MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags> &other);
   SubtreeObjCounterType getFlagsCounter(card32 flag);

   virtual void write(OutputObjectStream *s) const;

   template <typename CurrentClassType>
   void doReadTempl(InputObjectStream *s);

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   ~MHPointSetNodeData();
};


class HIERARCHIES_API MHInCorePointSetNodeData : public MHPointSetNodeData<max_platform_card, MAX_POINT_FLAGS_TRACKED_IN_HIERARCHY> {
 private:
   static MetaClass *MHInCorePointSetNodeDataMetaClass;
   static Object* newInstance();

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void read(InputObjectStream *s) {
      doReadTempl<MHInCorePointSetNodeData>(s);
   }
};

class HIERARCHIES_API MHOutOfCorePointSetNodeData : public MHPointSetNodeData<card64, MAX_POINT_FLAGS_TRACKED_IN_HIERARCHY> {
 private:
   static MetaClass *MHOutOfCorePointSetNodeDataMetaClass;
   static Object* newInstance();

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void read(InputObjectStream *s) {
      doReadTempl<MHOutOfCorePointSetNodeData>(s);
   }
};


}


#endif
