//---------------------------------------------------------------------------
#ifndef MHNodeData_templCodeH
#define MHNodeData_templCodeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MHNodeData.h"
//---------------------------------------------------------------------------
#include "BoundingBox_templCode.h"
#include "ObjectStreams.h"
#include "Persistent.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "MHObjects.h"
#include "MHObjects_templCode.h"
#include "AttachedData.h"
#include "VertexDescriptor.h"
#include "VertexDescriptor.h"
#include "MathTools.h"
#include "MathTools_templCode.h"
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;



// ------------------------------ MHNodeDataObjectVector ------------------------------



template <typename ObjectType, typename FloatType, int Dimension>
inline max_platform_card MHNodeDataObjectVector<ObjectType, FloatType, Dimension>
//---------------
::getNumObjects() const
//---------------
{
   return objs.size();
}

template <typename ObjectType, typename FloatType, int Dimension>
inline void MHNodeDataObjectVector<ObjectType, FloatType, Dimension>
//---------------------------------------------------------------------------------
::addObject(const ObjectType &obj, const BoundingBox<FloatType, Dimension> &nodeBB)
//---------------------------------------------------------------------------------
{
   objs.push_back(obj);
   forget(nodeBB);
}


template <typename ObjectType, typename FloatType, int Dimension>
inline ObjectType MHNodeDataObjectVector<ObjectType, FloatType, Dimension>
//----------------------------------
::getObject(max_platform_card index)
//----------------------------------
{
   return objs[index];
}


template <typename ObjectType, typename FloatType, int Dimension>
inline void MHNodeDataObjectVector<ObjectType, FloatType, Dimension>
//---------------------------------------------------------
::setObject(max_platform_card index, const ObjectType &obj)
//---------------------------------------------------------
{
   objs[index] = obj;
}


template <typename ObjectType, typename FloatType, int Dimension>
inline void MHNodeDataObjectVector<ObjectType, FloatType, Dimension>
//-------------------------------------
::deleteObject(max_platform_card index)
//-------------------------------------
{
   typename vector<ObjectType>::iterator pos = objs.begin();
   pos += index;
   objs.erase(pos);
}


template <typename ObjectType, typename FloatType, int Dimension>
inline void MHNodeDataObjectVector<ObjectType, FloatType, Dimension>
//-------
::clear(bool clearFlagsCounters)
//-------
{
   objs.clear();
   forget(clearFlagsCounters);
}



// -------------------------------- MHPointSetNodeData --------------------------------



template <typename SubtreeObjCounterType, int numCountedFlags>
inline MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::MHPointSetNodeData() {
   ps = NULL;
   attachedData = NULL;
   for (card32 i=0; i<numCountedFlags; i++) {
      numFlaggedInSubtree[i] = 0;
   }
}

template <typename SubtreeObjCounterType, int numCountedFlags>
inline max_platform_card MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::getNumObjects() const {
   if (ps == NULL) {
      return 0;
   } else {
      return ps->getNumEntries();
   }
}

template <typename SubtreeObjCounterType, int numCountedFlags>
inline void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::addObject(const MHPointSetPointObject &obj, const BoundingBox3f &nodeBB) {
   if (ps == NULL) {
      ps = new PointSet();
      ps->clearAndSetup(1, 1, obj.getVertexDescriptor());
      aatPosition = obj.getAATPosition();
      aatFlags = obj.getAATFlags();
      ps->setVertex(0, obj.getData());
   } else {
      ps->addPoint(obj.getData());
   }
   touchData(AttachedData::CT_ADD_DELETE);
   pAssert( obj.getVertexDescriptor()->equal(ps->getDescr()) );
}

template <typename SubtreeObjCounterType, int numCountedFlags>
inline MHPointSetPointObject MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::getObject(max_platform_card index) {
   MHPointSetPointObject result(
      ps->getDescr(), // vertex descriptor
      aatPosition,
      aatFlags,
      ps->getDataPointer(index) // data
   );
   return result;
}

template <typename SubtreeObjCounterType, int numCountedFlags>
inline void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::setObject(max_platform_card index, const MHPointSetPointObject &obj) {
   pAssert(ps != NULL);
   pAssert(ps->getNumEntries() > index);
   pAssert(obj.getAATPosition() == aatPosition);
   pAssert(obj.getAATFlags() == aatFlags);
   pAssert(obj.getVertexDescriptor()->equal(ps->getDescr()));
   ps->setVertex(index, obj.getData());
   if (ps->get<float32, 3>(index, aatPosition) == *(Vector3f*)(((card8*)obj.getData()) + aatPosition.getOffset()) ) {
      touchData(AttachedData::CT_ATTRIBUTE_CHANGED);
   } else {
      touchData(AttachedData::CT_POSITION_CHANGED);
   }
}

template <typename SubtreeObjCounterType, int numCountedFlags>
inline void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::deleteObject(max_platform_card index) {
   ps->deletePoint(index);
   touchData(AttachedData::CT_ADD_DELETE);
}

template <typename SubtreeObjCounterType, int numCountedFlags>
inline void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::clear(bool clearFlagsCounters) {
   delete ps;
   ps = NULL;
   if (clearFlagsCounters) {
      for (card32 i=0; i<numCountedFlags; i++) {
         numFlaggedInSubtree[i] = 0;
      }
   }
   touchData(AttachedData::CT_EVERYTHING_CHANGED);
}

template <typename SubtreeObjCounterType, int numCountedFlags>
void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::write(OutputObjectStream *s) const {
   s->writeObject(ps);
   s->writeObject(attachedData);
   s->write<card8>((card8)numCountedFlags);
   for (card32 i=0; i<numCountedFlags; i++) {
      s->write<SubtreeObjCounterType>(numFlaggedInSubtree[i]);
   }
}

template <typename SubtreeObjCounterType, int numCountedFlags>
template <typename CurrentClassType>
void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::doReadTempl(InputObjectStream *s) {
   Persistent *psObj;
   s->readObject(psObj);
   ps = (PointSet*)psObj;
   Persistent *attachedDataObj;
   s->readObject(attachedDataObj);
   attachedData = (AttachedDataContainer*)attachedDataObj;
   if (ps != NULL) {
      aatPosition = ps->getAAT("position");
      aatFlags = NULL_AAT;
      if (ps->providesAttribute("flags")) {
         aatFlags = ps->getAAT("flags");
      }
   }
   if (s->getClassVersion(CurrentClassType::getClass()) >= MetaClass::FIRST_VERSION + 1) {
      card8 numFlagsInFile;
      s->read<card8>(numFlagsInFile);
      SubtreeObjCounterType *temp = new SubtreeObjCounterType[numFlagsInFile];
      for (card8 i=0; i<numFlagsInFile; i++) {
         s->read<SubtreeObjCounterType>(temp[i]);
      }
      for (card8 i=0; i<numCountedFlags; i++) {
         if (i<numFlagsInFile) {
            numFlaggedInSubtree[i] = temp[i];
         } else {
            numFlaggedInSubtree[i] = 0;
         }
      }
      delete[] temp;
   }
   touchData(AttachedData::CT_EVERYTHING_CHANGED);
}

template <typename SubtreeObjCounterType, int numCountedFlags>
void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::assign(const Object* obj, CopyContext *context) {
   const MHPointSetNodeData *other = dynamic_cast<const MHPointSetNodeData*>(obj);
   pAssert(other != NULL);

   delete ps;
   ps = NULL;
   if (other->ps != NULL) {
      ps = (PointSet*)(other->ps->copy(context));
   }

   delete attachedData;
   attachedData = NULL;
   if (other->attachedData != NULL) {
      attachedData = (AttachedDataContainer*)(other->attachedData->copy(context));
   }
   for (int i=0; i<numCountedFlags; i++) {
      numFlaggedInSubtree[i] = other->numFlaggedInSubtree[i];
   }
   this->aatPosition = other->aatPosition;
   this->aatFlags = other->aatFlags;
   touchData(AttachedData::CT_EVERYTHING_CHANGED);
}

template <typename SubtreeObjCounterType, int numCountedFlags>
void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::appendFields(const VertexDescriptor *newAttribs) {
   PointSet *newps = ps->appendFields(newAttribs);
   delete ps;
   ps = newps;
   touchData(AttachedData::CT_EVERYTHING_CHANGED);
}


template <typename SubtreeObjCounterType, int numCountedFlags>
AttachedDataContainer *MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::getAttachments() {
   if (attachedData == NULL) {
      attachedData = new AttachedDataContainer();
   }
   return attachedData;
}

template <typename SubtreeObjCounterType, int numCountedFlags>
inline void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::touchData(AttachedData::ChangeType whatChanged) {
   if (attachedData != NULL) {
      attachedData->deleteDelOnWriteData(whatChanged);
   }
}


template <typename SubtreeObjCounterType, int numCountedFlags>
void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::incFlagsCounters(card32 flags) {
   for (card32 i=0; i<numCountedFlags; i++) {
      if ((flags & (1<<i)) != 0) {
         numFlaggedInSubtree[i]++;
      }
   }
}

template <typename SubtreeObjCounterType, int numCountedFlags>
void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::decFlagsCounters(card32 flags) {
   for (card32 i=0; i<numCountedFlags; i++) {
      if ((flags & (1<<i)) != 0) {
         pAssert(numFlaggedInSubtree[i] > 0);
         numFlaggedInSubtree[i]--;
      }
   }
}

template <typename SubtreeObjCounterType, int numCountedFlags>
void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::copyFlagsCounters(const MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags> &other) {
   for (card32 i=0; i<numCountedFlags; i++) {
      numFlaggedInSubtree[i] = other.numFlaggedInSubtree[i];
   }
}

template <typename SubtreeObjCounterType, int numCountedFlags>
void MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::decFlagsCounters(const MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags> &other) {
   for (card32 i=0; i<numCountedFlags; i++) {
      numFlaggedInSubtree[i] -= other.numFlaggedInSubtree[i];
   }
}


template <typename SubtreeObjCounterType, int numCountedFlags>
SubtreeObjCounterType MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::getFlagsCounter(card32 flag) {
   int32 index = simpleLog2(flag);
   pAssert(index >= 0 && index <numCountedFlags);
   return numFlaggedInSubtree[index];
}


template <typename SubtreeObjCounterType, int numCountedFlags>
MHPointSetNodeData<SubtreeObjCounterType, numCountedFlags>::~MHPointSetNodeData() {
   delete ps;
   delete attachedData;
}




}


#endif
