//---------------------------------------------------------------------------
#ifndef MHNodesH
#define MHNodesH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MHHierarchy.h"
#include "Tools.h"
//---------------------------------------------------------------------------

/*
 | members/parameters expected by MH template in Nodes:
 |
 | template <typename FloatType, int Dimension, typename ObjectType,
 |           typename NodeAllocator_RebindDummy, typename NodeDataType,
 |           typename SettingsType, typename SubtreeObjCounterType,
 |           class NodeBaseClass>
 | class MHNode {
 |  public:
 |    NodeData nodeData;
 |    MHMultiRes multiRes;
 |    SubtreeObjCounterType objsInSubtree;
 |
 |    int getMaxNumChildren() const;
 |    int getNumNonNullChildren() const;
 |    BoundingBox<FloatType, Dimension> getBoundingBox();
 |    bool isLeaf() const;
 |    void getChild(int index, NodePointer &child);
 |    void getParent(NodePointer &parent);
 |    void setParent(NodePointer parent);
 |    void createChild(NodeAllocator *alloc, int index, NodePointer thisNodePointer);
 |    void deleteChild(NodeAllocator *alloc, int index);
 |    void removeChild(int index);
 |    void enlargeRoot(NodeAllocator *alloc,
 |                     const ObjectType &obj,
 |                     NodePointer thisNodePointer,
 |                     NodePointer &newRoot);
 |
 |    static int getChildIndex(const StaticVector<FloatType, Dimension> &pos,
 |                             const BoundingBox<FloatType, Dimension> &boundingBox);
 |    static int getChildIndex(const ObjectType &obj,
 |                             const BoundingBox<FloatType, Dimension> &boundingBox);
 |    static BoundingBox<FloatType, Dimension> getChildBoundingBox(
 |                             const BoundingBox<FloatType, Dimension> nodeBB, int index);
 |    static void createRoot(NodeAllocator *alloc,
 |                           const ObjectType &obj,
 |                           const SettingsType &settings,
 |                           NodePointer &newRoot);
 |
 |    bool nodeShouldBeSplit(const SettingsType &settings);
 |    bool subtreeShouldBeMerged(const SettingsType &settings);
 |
 |    void setFlags(card32 flags);
 |    card32 getFlags();
 |
 |    bool modified() {return (flags & MH_NODE_FLAG_MODIFIED) != 0;}
 |    bool getDirty() {return (flags & MH_NODE_FLAG_DIRTY) != 0;}
 |    bool setDirty(bool d) {flags = d?(flags|MH_NODE_FLAG_DIRTY):(flags & (0xFFFFFFFF - MH_NODE_FLAG_DIRTY));}
 |
 |    virtual void incLockCounter();
 |    virtual void decLockCounter();
 |    virtual card32 getLockCounter();
 | };
*/





namespace geoxl {


/// indicates that the subtree needs an update
const card32 MH_NODE_FLAG_MODIFIED = 1;
/// indicates that the node has been modified since it has been constructed (i.e. loaded from disc in ooc case)
/// not reset during regular updates (only by write backs to disc)
const card32 MH_NODE_FLAG_DIRTY = 2;


// ------------------------------ MH_DDimQuadtreeNode_Templ ------------------------------



template <typename FloatType, int Dimension, typename ObjectType,
          typename NodeAllocator_RebindDummy, typename NodeDataType,
          typename MHMultiRes, typename SettingsType,
          typename SubtreeObjCounterType, class NodeBaseClass>
class HIERARCHIES_API MH_DDimQuadtreeNode_Templ : public NodeBaseClass {
 public:
   static const int MAX_NUM_CHILDREN = 1 << Dimension;

   typedef MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType,
                                     NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType,
                                     SubtreeObjCounterType, NodeBaseClass> OwnType;
   typedef typename NodeAllocator_RebindDummy::template Rebind_NodeType<OwnType>::ReboundType NodeAllocator;
   typedef typename NodeAllocator::NodePointer NodePointer;

 protected:
   card32 flags;
   BoundingBox<FloatType, Dimension> boundingBox;
   NodePointer children[MAX_NUM_CHILDREN];
   NodePointer parent;

 public:
   NodeDataType nodeData;
   MHMultiRes multiRes;
   SubtreeObjCounterType objsInSubtree;

   MH_DDimQuadtreeNode_Templ();

   inline int getMaxNumChildren() const;
   inline int getNumNonNullChildren() const;
   inline BoundingBox<FloatType, Dimension> getBoundingBox() const;
   inline bool isLeaf() const;
   inline void getChild(int index, NodePointer &child);
   inline void getParent(NodePointer &parent);
   inline void setParent(NodePointer parent);
   void createChild(NodeAllocator *alloc, int index, NodePointer thisNodePointer);
   void deleteChild(NodeAllocator *alloc, int index);
   void removeChild(int index);
   void enlargeRoot(NodeAllocator *alloc,
                    const ObjectType &obj,
                    NodePointer thisNodePointer,
                    NodePointer &newRoot);

   static int getChildIndex(const StaticVector<FloatType, Dimension> &pos,
                            const BoundingBox<FloatType, Dimension> &boundingBox);
   static inline int getChildIndex(const ObjectType &obj,
                            const BoundingBox<FloatType, Dimension> &boundingBox);
   static BoundingBox<FloatType, Dimension> getChildBoundingBox(
                             const BoundingBox<FloatType, Dimension> nodeBB, int index);
   static void createRoot(NodeAllocator *alloc,
                          const ObjectType &obj,
                          const SettingsType &settings,
                          NodePointer &newRoot);

   inline bool nodeShouldBeSplit(const SettingsType &settings);
   inline bool subtreeShouldBeMerged(const SettingsType &settings);

   inline void setFlags(card32 flags);
   inline card32 getFlags();

   inline bool modified() {return (flags & MH_NODE_FLAG_MODIFIED) != 0;}
	inline bool getDirty() {return (flags & MH_NODE_FLAG_DIRTY) != 0;}
	inline void setDirty(bool d) {flags = d?(flags|MH_NODE_FLAG_DIRTY):(flags & (0xFFFFFFFF - MH_NODE_FLAG_DIRTY));}

   virtual void incLockCounter() = 0;
   virtual void decLockCounter() = 0;
   virtual card32 getLockCounter() const = 0;

   ~MH_DDimQuadtreeNode_Templ();
};


}

#endif
