//---------------------------------------------------------------------------
#ifndef MHNodes_templCodeH
#define MHNodes_templCodeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MHNodes.h"
//---------------------------------------------------------------------------
#include "MHHierarchy_templCode.h"
//---------------------------------------------------------------------------

namespace geoxl {


// ------------------------------ MH_DDimQuadtreeNode_Templ ------------------------------


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//---------------------------
::MH_DDimQuadtreeNode_Templ()
//---------------------------
{
   objsInSubtree = 0;
   flags = MH_NODE_FLAG_DIRTY;
   for (int i=0; i<MAX_NUM_CHILDREN; i++) {
      children[i] = NULL;
   }
   parent = NULL;
}


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
int MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//-------------------------
::getMaxNumChildren() const
//-------------------------
{
   return MAX_NUM_CHILDREN;
}


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
int MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//-----------------------------
::getNumNonNullChildren() const
//-----------------------------
{
   int value = 0;
   for (int i=0; i<MAX_NUM_CHILDREN; i++) {
      if (children[i] != NULL) value++;
   }
   return value;
}



template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
BoundingBox<FloatType, Dimension> MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//----------------------
::getBoundingBox() const
//----------------------
{
   return boundingBox;
}



template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
bool MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//--------------
::isLeaf() const
//--------------
{
   return getNumNonNullChildren() == 0;
}


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
int MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//-------------------------------------------------------------------
::getChildIndex(const StaticVector<FloatType, Dimension> &pos,
                const BoundingBox<FloatType, Dimension> &boundingBox)
//-------------------------------------------------------------------
{
   int result = 0;
   int nextBit = 1;
   StaticVector<FloatType, Dimension> center = boundingBox.getCenter();
   for (int d=0; d<Dimension; d++) {
      if (pos[d] > center[d]) {
         result |= nextBit;
      }
      nextBit <<= 1;
   }
   return result;
}


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
inline int MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//-------------------------------------------------------------------
::getChildIndex(const ObjectType &obj,
                const BoundingBox<FloatType, Dimension> &boundingBox)
//-------------------------------------------------------------------
{
   return getChildIndex(obj.getCenter(), boundingBox);
}



template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
BoundingBox<FloatType, Dimension> MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//------------------------------------------------------------------------------
::getChildBoundingBox(const BoundingBox<FloatType, Dimension> nodeBB, int index)
//------------------------------------------------------------------------------
{
   BoundingBox<FloatType, Dimension> result = nodeBB;
   int nextBit = 1;
   StaticVector<FloatType, Dimension> center = nodeBB.getCenter();
   for (int d=0; d<Dimension; d++) {
      if ((index & nextBit) != 0) {
         result.lowerCorner[d] = center[d];
      } else {
         result.upperCorner[d] = center[d];
      }
      nextBit <<= 1;
   }
   return result;
}



template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
void MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes,
                               SettingsType, SubtreeObjCounterType, NodeBaseClass>
//---------------------------------------
::getChild(int index, NodePointer &child)
//---------------------------------------
{
   child = children[index];
}



template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
void MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes,
                               SettingsType, SubtreeObjCounterType, NodeBaseClass>
//------------------------------
::getParent(NodePointer &parent)
//------------------------------
{
   parent = this->parent;
}



template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
void MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes,
                               SettingsType, SubtreeObjCounterType, NodeBaseClass>
//------------------------------
::setParent(NodePointer parent)
//------------------------------
{
   this->parent = parent;
   setDirty(true);
}



template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
void MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//-------------------------------------------------------------------------
::createChild(NodeAllocator *alloc, int index, NodePointer thisNodePointer)
//-------------------------------------------------------------------------
{
   pAssert(index < MAX_NUM_CHILDREN);
   pAssert(children[index] == NULL);
   children[index] = alloc->newNode();
   setDirty(true);
   pAssert(children[index] != NULL);
   children[index]->parent = thisNodePointer;
   children[index]->boundingBox = getChildBoundingBox(boundingBox, index);
   children[index]->setDirty(true);
}


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
void MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//--------------------------------------------
::deleteChild(NodeAllocator *alloc, int index)
//--------------------------------------------
{
   pAssert(index < MAX_NUM_CHILDREN);
   alloc->deleteNode(children[index]);
   children[index] = NULL;
   setDirty(true);
}


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
void MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//----------------------
::removeChild(int index)
//----------------------
{
   pAssert(index < MAX_NUM_CHILDREN);
   children[index] = NULL;
   setDirty(true);
}


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
void MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//----------------------------------------
::enlargeRoot(NodeAllocator *alloc,
              const ObjectType &obj,
              NodePointer thisNodePointer,
              NodePointer &newRoot)
//----------------------------------------
{
   pAssert(parent == NULL);
   StaticVector<FloatType, Dimension> diag = boundingBox.upperCorner - boundingBox.lowerCorner;
   StaticVector<FloatType, Dimension> pos = obj.getCenter();
   BoundingBox<FloatType, Dimension> newRootBB = boundingBox;
   for (int d=0; d<Dimension; d++) {
      if (pos[d] > boundingBox.upperCorner[d]) {
         newRootBB.upperCorner[d] += diag[d];
      } else {
         newRootBB.lowerCorner[d] -= diag[d];
      }
   }
   newRoot = alloc->newNode();
   pAssert(newRoot != NULL);
   newRoot->boundingBox = newRootBB;
   int index = getChildIndex(boundingBox.getCenter(), newRootBB);
   newRoot->children[index] = thisNodePointer;
   thisNodePointer->parent = newRoot;
   setDirty(true);
}



inline long double log2(long double val) {return logl(val) / logl(2.0);}

template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
void MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//----------------------------------------
::createRoot(NodeAllocator *alloc,
             const ObjectType &obj,
             const SettingsType &settings,
             NodePointer &newRoot)
//----------------------------------------
{
   FloatType posmag = norm(obj.getCenter());
   FloatType magnitude;
   if ( posmag <= 1E-30) {
      magnitude = (FloatType)1.0;
   } else {
      magnitude = powl(2, floor(log2(posmag)));
   }
   BoundingBox<FloatType, Dimension> newRootBB = BoundingBox<FloatType, Dimension>(obj.getCenter());
   for (card32 d=0; d<Dimension; d++) {
      newRootBB.lowerCorner[d] = floor(newRootBB.lowerCorner[d] / magnitude) * magnitude;
      newRootBB.upperCorner[d] = floor(newRootBB.lowerCorner[d] / magnitude + (FloatType)1.0) * magnitude;
   }
   newRoot = alloc->newNode();
   pAssert(newRoot != NULL);
   newRoot->boundingBox = newRootBB;
	newRoot->setDirty(true);
   forget(settings);
}


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//----------------------------
::~MH_DDimQuadtreeNode_Templ()
//----------------------------
{
   for (int i=0; i<MAX_NUM_CHILDREN; i++) {
      NodeAllocator::deleteNode(children[i]);
   }
}


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
inline bool MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//-----------------------------------------------
::nodeShouldBeSplit(const SettingsType &settings)
//-----------------------------------------------
{
   pAssert(objsInSubtree == nodeData.getNumObjects());
   max_platform_card n = nodeData.getNumObjects();
   if (n == 0) return false;
   if (n <= settings.maxNumObjectsPerLeaf) return false;

   StaticVector<FloatType, Dimension> refPos = nodeData.getObject(0).getCenter();
   for (max_platform_card i=1; i<n; i++) {
      if (nodeData.getObject(i).getCenter() != refPos) {
         return true;
      }
   }
   return false;
}


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
inline bool MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//---------------------------------------------------
::subtreeShouldBeMerged(const SettingsType &settings)
//---------------------------------------------------
{
   return (objsInSubtree <= settings.maxNumObjectsPerLeaf) && (getNumNonNullChildren() > 0);
}


template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
inline void MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//----------------------
::setFlags(card32 flags)
//----------------------
{
   this->flags = flags;
}

template <typename FloatType, int Dimension, typename ObjectType, typename NodeAllocator_RebindDummy,
          typename NodeDataType, typename MHMultiRes, typename SettingsType, typename SubtreeObjCounterType, class NodeBaseClass>
inline card32 MH_DDimQuadtreeNode_Templ<FloatType, Dimension, ObjectType, NodeAllocator_RebindDummy, NodeDataType, MHMultiRes, SettingsType, SubtreeObjCounterType, NodeBaseClass>
//----------
::getFlags()
//----------
{
   return flags;
}




}

#endif
