//---------------------------------------------------------------------------
#ifndef MHObjectsH
#define MHObjectsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "PTypes.h"
#include "BoundingBox.h"
#include "BoundingBox_templCode.h"
//---------------------------------------------------------------------------
#include "VertexDescriptor.h"
//---------------------------------------------------------------------------

/*
 | members expected by MHNode template:
 |
 | class MHObject {
 |  public:
 |    bool inBoundingBox(const BoundingBox<FloatType, Dimension> &bb);
 |    StaticVector<FloatType, Dimension> getCenter() const;
 |    int32 getFlags(); // returns 0 if no flags are defined
 | };
*/


namespace geoxl {



template <typename FloatType, int Dimension>
class HIERARCHIES_API MHPointWAttribObject_TemplBase {
 public:
   StaticVector<FloatType, Dimension> pos;
   char *data;
   inline bool inBoundingBox(const BoundingBox<FloatType, Dimension> &bb) const;
   inline StaticVector<FloatType, Dimension> getCenter() const;
   // xxx flags
};


template <typename FloatType, int Dimension>
class HIERARCHIES_API MHIndexedPoint_TemplBase {
 public:
   StaticVector<FloatType, Dimension> pos;
   max_platform_card index;
   inline bool inBoundingBox(const BoundingBox<FloatType, Dimension> &bb) const;
   inline StaticVector<FloatType, Dimension> getCenter() const;
   // xxx flags
};


class HIERARCHIES_API MHPointSetPointObject {
 private:
   const VertexDescriptor *vd;
   AAT aatPosition;
   AAT aatFlags;
   char *data;

 public:
   inline MHPointSetPointObject(const VertexDescriptor *vd, AAT aatPosition, AAT aatFlags, char *data);

   inline bool inBoundingBox(const BoundingBox3f &bb) const;
   inline Vector3f getCenter() const;
   inline int32 getFlags() const;

   inline const VertexDescriptor *getVertexDescriptor() const;
   inline AAT getAATPosition() const;
   inline AAT getAATFlags() const;
   inline char *getData() const;

};




}

#endif
