//---------------------------------------------------------------------------
#ifndef MHObjects_templCodeH
#define MHObjects_templCodeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MHObjects.h"
//---------------------------------------------------------------------------
#include "linearalgebra_templcode.h"
#include "BoundingBox_templCode.h"
//---------------------------------------------------------------------------
#include "VertexDescriptor.h"
//---------------------------------------------------------------------------


namespace geoxl {


// ---- MHPointWAttribObject_TemplBase


template <typename FloatType, int Dimension>
inline bool MHPointWAttribObject_TemplBase<FloatType, Dimension>::inBoundingBox(const BoundingBox<FloatType, Dimension> &bb) const {
   return bb.inBoundingBox(pos);
}

template <typename FloatType, int Dimension>
inline StaticVector<FloatType, Dimension> MHPointWAttribObject_TemplBase<FloatType, Dimension>::getCenter() const {
   return pos;
}


// ---- MHIndexedPoint_TemplBase


template <typename FloatType, int Dimension>
inline bool MHIndexedPoint_TemplBase<FloatType, Dimension>::inBoundingBox(const BoundingBox<FloatType, Dimension> &bb) const {
   return bb.inBoundingBox(pos);
}

template <typename FloatType, int Dimension>
inline StaticVector<FloatType, Dimension> MHIndexedPoint_TemplBase<FloatType, Dimension>::getCenter() const {
   return pos;
}


// ---- MHPointSetPointObject


inline MHPointSetPointObject::MHPointSetPointObject(const VertexDescriptor *vd, AAT aatPosition, AAT aatFlags, char *data) {
   pAssert(aatPosition != NULL_AAT);
   this->vd = vd; this->aatPosition = aatPosition; this->aatFlags = aatFlags; this->data = data;
}

inline bool MHPointSetPointObject::inBoundingBox(const BoundingBox3f &bb) const {
   Vector3f *pos = (Vector3f*)(data+aatPosition.getOffset());
   return bb.inBoundingBox(*pos);
}

inline Vector3f MHPointSetPointObject::getCenter() const {
   Vector3f *pos = (Vector3f*)(data+aatPosition.getOffset());
   return *pos;
}

inline int32 MHPointSetPointObject::getFlags() const {
   if (aatFlags != NULL_AAT) {
      int32 *pos = (int32*)(data+aatFlags.getOffset());
      return (int32)*pos;
   } else {
      return 0;
   }
}

inline const VertexDescriptor *MHPointSetPointObject::getVertexDescriptor() const {
   return vd;
}

inline AAT MHPointSetPointObject::getAATPosition() const {
   return aatPosition;
}

inline AAT MHPointSetPointObject::getAATFlags() const {
   return aatFlags;
}

inline char *MHPointSetPointObject::getData() const {
   return data;
}




}

#endif
