//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "MHOutOfCorePointSetOctree.h"
//---------------------------------------------------------------------------
#include "MHAllocators_templCode.h"
#include "MathIOTools.h"
#include "ObjectClassProperty.h"
#include "EmbeddedObjectClassProperty.h"
#include "EmbeddedObjectClassProperty_templCode.h"
#include "NumericalClassProperty.h"
#include "numericalclassproperty_templcode.h"
#include "ClassMethods.h"
#include "MemoryObjectStreams.h"
#include "DebugOutput.h"
#include "MathIOTools.h"
#include "MHNodes_templCode.h"
#include "MHHierarchy_templCode.h"
#include "SparseGrid_templCode.h"
#include "PointSet.h"
#ifndef _DEBUG
#include "PointSet_templCode.h"
#endif
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

/*
// explicit instantiation
template class MH_DDimQuadtreeNode_Templ
 <
   float32,
   3,
   MHPointSetPointObject,
   MH_MultiObjectStreamNodeAllocator_Templ<Persistent>,
   MHOutOfCorePointSetNodeData,
   MHSparseGridMultiRes,
   MHStandardSettings,
   card64,
   Persistent
 >;

template class MHPointHierachy_TemplBase
 <
   float32,
   3,
   MHPointSetPointObject,
   MHOutOfCorePointSetOctreeNode,
   MHStandardSettings,
   Persistent
 >;    */

 

MetaClass* MHOutOfCorePointSetOctreeNode::MHOutOfCorePointSetOctreeNodeMetaClass = NULL;

const MetaClass* MHOutOfCorePointSetOctreeNode::getInstanceClass() const {
   return getClass();
}

Object* MHOutOfCorePointSetOctreeNode::newInstance() {
   return new MHOutOfCorePointSetOctreeNode;
}

const MetaClass* MHOutOfCorePointSetOctreeNode::getClass() {
   if (MHOutOfCorePointSetOctreeNodeMetaClass==NULL) {
      MHOutOfCorePointSetOctreeNodeMetaClass = new MetaClass("MHOutOfCorePointSetOctreeNode", "no description.",
                                  newInstance, sizeof(MHOutOfCorePointSetOctreeNode));
   }
   return MHOutOfCorePointSetOctreeNodeMetaClass;
}

void MHOutOfCorePointSetOctreeNode::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   MHOutOfCorePointSetOctreeNodeMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(MHOutOfCorePointSetOctreeNode)
   ADD_CARD32_PROP(flags, 0)
   ADD_EMBEDDED_OBJECT_PROP(nodeData, 0, MHOutOfCorePointSetNodeData::getClass())
   ADD_CARD64_PROP(objsInSubtree, 0)
}

void MHOutOfCorePointSetOctreeNode::shutdown() {
   delete MHOutOfCorePointSetOctreeNodeMetaClass;
}


// ----


MetaClass* MHOutOfCorePointSetOctree::MHOutOfCorePointSetOctreeMetaClass = NULL;

const MetaClass* MHOutOfCorePointSetOctree::getInstanceClass() const {
   return getClass();
}

Object* MHOutOfCorePointSetOctree::newInstance() {
   return new MHOutOfCorePointSetOctree;
}

const MetaClass* MHOutOfCorePointSetOctree::getClass() {
   if (MHOutOfCorePointSetOctreeMetaClass==NULL) {
      MHOutOfCorePointSetOctreeMetaClass = new MetaClass("MHOutOfCorePointSetOctree", "no description.",
                                  newInstance, sizeof(MHOutOfCorePointSetOctree));
   }
   return MHOutOfCorePointSetOctreeMetaClass;
}

void MHOutOfCorePointSetOctree::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   MHOutOfCorePointSetOctreeMetaClass->copyParentProperties();
	
	BEGIN_CLASS_INIT(MHOutOfCorePointSetOctree)
   ADD_EMBEDDED_OBJECT_PROP(settings, 0, MHStandardSettings::getClass())
   ADD_OBJECT_PROP(descr, 0, VertexDescriptor::getClass(), true)


	ADD_NOARGS_METHOD(MHOutOfCorePointSetOctree::outputDebugStatistics);
	ADD_NOARGS_METHOD(MHOutOfCorePointSetOctree::outputBenchmarkStatistics);
}

void MHOutOfCorePointSetOctree::shutdown() {
   delete MHOutOfCorePointSetOctreeMetaClass;
}


// ----


// -- MHOutOfCorePointSetOctreeNode


void MHOutOfCorePointSetOctreeNode::write(OutputObjectStream *s) const {
   s->write(flags);
   for (card32 i=0; i<MAX_NUM_CHILDREN; i++) {
      children[i].write(s);
   }
   s->includeClass(MHOutOfCorePointSetNodeData::getClass());
   multiRes.write(s);
   nodeData.write(s);
   s->write(objsInSubtree);
   parent.write(s);
   writeBoundingBox3f(s, boundingBox);
}

void MHOutOfCorePointSetOctreeNode::read(InputObjectStream *s) {
   s->read(flags);
   for (card32 i=0; i<MAX_NUM_CHILDREN; i++) {
      children[i].read(s);
   }
   multiRes.read(s);
   nodeData.read(s);
   s->read(objsInSubtree);
   parent.read(s);
   readBoundingBox3f(s, boundingBox);
}

struct MyOOCCopyContext : public Object {
   vector<MHOutOfCorePointSetOctreeNode::NodePointer> thisPointersStack;
};

void MHOutOfCorePointSetOctreeNode::assign(const Object* obj, CopyContext *context) {
	throw PException("MHOutOfCorePointSetOctreeNode::assign() - assign is not implemented for out-of-core nodes. This methods should not have been called.");
/* 
	const MHOutOfCorePointSetOctreeNode *other = dynamic_cast<const MHOutOfCorePointSetOctreeNode*>(obj);
   pAssert(other != NULL);
   flags = other->flags;
   // parent = other->parent;
   pAssert(false);
   boundingBox = other->boundingBox;

   pAssert(context != NULL);
   MyOOCCopyContext *stack = (MyOOCCopyContext*)context->getData("oocTPS");
   pAssert(stack != NULL);
   NodePointer thisInstancePointer = stack->thisPointersStack.back();

   for (card32 i=0; i<MAX_NUM_CHILDREN; i++) {
      if (children[i] != NULL) {
         children[i].getAlloc()->deleteNode(children[i]);
      }
      if (other->children[i] == NULL) {
         children[i] = NULL;
      } else {
         if (other->children[i] == NULL) {
            children[i] = NULL;
         } else {
            children[i] = ((NodePointer)(other->children[i])).getAlloc()->newNode();
            stack->thisPointersStack.push_back(children[i]);
            children[i]->assign(*(other->children[i]));
            stack->thisPointersStack.pop_back();
            children[i]->setParent(thisInstancePointer);
         }
      }
   }
   nodeData.assign(&other->nodeData, context);
   multiRes.assign(&other->multiRes, context);
   objsInSubtree = other->objsInSubtree;	  */
}



// -- MHOutOfCorePointSetOctree


MHOutOfCorePointSetOctreeNode::MHOutOfCorePointSetOctreeNode() {
	lockCounter = 0;
}

void MHOutOfCorePointSetOctreeNode::shallowDelete() {
	for (card32 i=0; i<8; i++) {
		children[i] = NULL;
	}
	MHOutOfCorePointSetOctreeNodeBase::shallowDelete();
}



void MHOutOfCorePointSetOctree::write(OutputObjectStream *s) const {
   s->writeObject(nAlloc);
   s->includeClass(MHStandardSettings::getClass());
   settings.write(s);
   s->writeObject(descr);
   root.write(s);
   writeBoundingBox3f(s, rootBB);
   s->write(numObjects);
}


void MHOutOfCorePointSetOctree::read(InputObjectStream *s) {
   s->readAnyObject(nAlloc);
   settings.read(s);
   delete descr;
   s->readAnyObject(descr);
   root.read(s);
   readBoundingBox3f(s, rootBB);
   s->read(numObjects);
}


void MHOutOfCorePointSetOctree::assign(const Object* obj, CopyContext *context) {
   const MHOutOfCorePointSetOctree *other = dynamic_cast<const MHOutOfCorePointSetOctree*>(obj);
   pAssert(other != NULL);

   nAlloc = other->nAlloc; // no copy! the page file must be accessed by one allocator only.
   setSettings(other->settings);
   delete descr;
   descr = (VertexDescriptor*)(other->descr->copy(context));

   bool delContext = false;

   if (context == NULL) {
      context = new CopyContext();
      delContext = true;
   }

   MyOOCCopyContext stack;
   stack.thisPointersStack.push_back(root);
   context->pushData("oocTPS", &stack);

   if (other->root == NULL) {
      root = NULL;
   } else {
      root = other->root.deepCopy(context);
   }

   if (delContext) delete context;

   rootBB = other->rootBB;

   numObjects = other->numObjects;
}

void MHOutOfCorePointSetOctree::outputDebugStatistics() {
   debugOutput << "type\tstream size\tdata size";
   debugOutput.cr();
   debugOutput << "---------------------------------";
   debugOutput.cr();
   doDBOutput(root);
}

void MHOutOfCorePointSetOctree::doDBOutput(NodePointer node) {
   if (node == NULL) {
      return;
   } else {
      MemoryOutputObjectStream testOut;
      MHOutOfCorePointSetOctreeNodeBase *n = *node;
      testOut.writeObject(n);
      const vector<card8> *buffer = testOut.closeAndGetMemoryBuffer();
      if (node->isLeaf()) {
         debugOutput << "Leaf\t";
      } else {
         debugOutput << "Inner\t";
      }
      debugOutput << (card32)buffer->size() << "\t";
      card32 s = descr->getSize();
      card32 minSize = 0;
      if (node->multiRes.getPointSet() != NULL) {
         minSize += node->multiRes.getPointSet()->getNumEntries() * s;
      }
      if (node->nodeData.getPointSet() != NULL) {
         minSize += node->nodeData.getPointSet()->getNumEntries() * s;
      }
      debugOutput << minSize;
      debugOutput.cr();
      this->getAlloc()->checkLRUSwapOut();
      for (card8 i=0; i<8; i++) {
         NodePointer child;
         node->getChild(i, child);
         doDBOutput(child);
      }
   }
}

void geoxl::MHOutOfCorePointSetOctree::outputBenchmarkStatistics()
{
	VertexDescriptor* vd=this->getDescr();
	card32 bytesPerVertex= vd->getSize();
	
	debugOutput << "--------------OOC PC Statistics--------------------\n";
	debugOutput << "Bytes per point:   " << bytesPerVertex << "\n";
	card32 numNodes=0;
	card32 numPoints= leafNodeNumObjects(root, numNodes);
	float32 pointsPerLeaf=numPoints/(float)numNodes;
	debugOutput << "All leafs point count:  " << numPoints << "     leafNodes count:  " <<  numNodes <<"\n";
	debugOutput << "=> mean points per leaf: " << pointsPerLeaf  <<"\n";
	card32 memLeaf=(bytesPerVertex* numPoints);
	debugOutput << "Leaf data (bpp*leafPointCnt) =   " << memLeaf/(float)1024  << " kB   (" << (int)(memLeaf/(float)1048576)+1 << " MB)\n\n";
	numNodes=0;
	numPoints= innerNodeNumObjects(root, numNodes);
	pointsPerLeaf=numPoints/(float)numNodes;
	card32 memInner=(bytesPerVertex* numPoints);
	debugOutput << "All inner nodes point count:  " << numPoints << "     innerNodes count:  " <<  numNodes <<"\n";
	debugOutput << "=> mean points per innerNode:   " << pointsPerLeaf  <<"\n";
	debugOutput << "Inner MR point data (bpp*innerPointCnt)   " << memInner/(float)1024  << " kB   (" << (int)(memInner/(float)1048576)+1 << " MB)\n--------------\n";
	memInner+=memLeaf;
	debugOutput << "Together point data (bpp*allPointCnt)    " << memInner/(float)1024  << " kB   (" << (int)(memInner/(float)1048576)+1 << " MB)\n\n";
	debugOutput.cr();
}

geoxl::card32 geoxl::MHOutOfCorePointSetOctree::innerNodeNumObjects( NodePointer node, card32& numInnerNodes )
{
	numInnerNodes=0;
	if (node == NULL) {
		return 0;
	} else {

		if (node->isLeaf()) {
			pAssert(node->nodeData.getPointSet() != NULL);
			return 0;
		} else {
			//Inner node
			numInnerNodes=1;
			pAssert(node->multiRes.getPointSet() != NULL);

			//Own mr data
			card32 result= node->multiRes.getPointSet()->getNumEntries();

			//Plus children mr data
			for (card8 i=0; i<8; i++) {
				NodePointer child;
				node->getChild(i, child);
				if (child!=NULL)
				{
					card32 childrensInnernodes=0;
					result+=innerNodeNumObjects(child, childrensInnernodes);
					numInnerNodes+=childrensInnernodes;
				}
			}
			return result;
		}
	}
}

card32 MHOutOfCorePointSetOctree::leafNodeNumObjects( NodePointer node, card32& numLeafNodes )
{
	numLeafNodes=0;
	if (node == NULL) {
		return 0;
	} else {

		if (node->isLeaf()) {
			pAssert(node->nodeData.getPointSet() != NULL);
			numLeafNodes=1;
			return node->nodeData.getPointSet()->getNumEntries();
		} else {
			//Inner node
			card32 result=0;
			for (card8 i=0; i<8; i++) {
				NodePointer child;
				node->getChild(i, child);
				if (child!=NULL)
				{
					card32 childrensLeafnodes=0;
					result+=leafNodeNumObjects(child, childrensLeafnodes);
					numLeafNodes+=childrensLeafnodes;
				}
			}
			return result;
		}
	}
}

void MHOutOfCorePointSetOctree::outputDebugStatisticsShort() {
   debugOutput.cr();
   debugOutput << "short in-mem ooc-octree statistics";
   debugOutput.cr();
   debugOutput << "----------------------------------";
   debugOutput.cr();
   card32 mins = 1000000000;
   card32 maxs = 0;
   card32 sum = 0;
   card32 inMem = 0;
   doDBOutputShort(root, mins, maxs, sum, inMem);
   debugOutput << "minNodeSize " << mins;
   debugOutput.cr();
   debugOutput << "maxNodeSize " << maxs;
   debugOutput.cr();
   debugOutput << "average " << (float)sum/(float)inMem;
   debugOutput.cr();
   debugOutput << "inMem" << inMem;
   debugOutput.cr();
   debugOutput.cr();
}

void MHOutOfCorePointSetOctree::doDBOutputShort(NodePointer node, card32 &minSize, card32 &maxSize, card32 &sizeSum, card32 &inMem) {
   if (node == NULL) {
      return;
   } else {
      card32 s = descr->getSize();
      card32 ns = 0;
      if (node->multiRes.getPointSet() != NULL) {
         ns += node->multiRes.getPointSet()->getNumEntries() * s;
      }
      if (node->nodeData.getPointSet() != NULL) {
         ns += node->nodeData.getPointSet()->getNumEntries() * s;
      }

      if (minSize > ns) minSize =ns;
      if (maxSize < ns) maxSize =ns;
      sizeSum += ns;
      inMem++;

      for (card8 i=0; i<8; i++) {
         NodePointer child;
         node->getChild(i, child);
         if (nAlloc->isInMem(child)) {
            doDBOutputShort(child, minSize, maxSize, sizeSum, inMem);
         }
      }
   }
}

void MHOutOfCorePointSetOctree::instantiateforlinking_donotcall() {
   NodeAllocator na(MHOutOfCorePointSetOctreeNode::getClass(), NULL, 10);
}



