//---------------------------------------------------------------------------
#ifndef MHOutOfCorePointSetOctreeH
#define MHOutOfCorePointSetOctreeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MHInstances.h"
#include "Persistent.h"
#include "MultiPartPersistent.h"
#ifdef _DEBUG
   #include "EventCounterDisplay.h"
   #include "DebugOutput.h"
#endif
//---------------------------------------------------------------------------


namespace geoxl {

// ---- the octree nodes


typedef MH_DDimQuadtreeNode_Templ
 <
   float32,                      // typename FloatType
   3,                            // int Dimension
   MHPointSetPointObject,        // typename ObjectType
   MH_MultiObjectStreamNodeAllocator_Templ<Persistent>, // typename NodeAllocator
   MHOutOfCorePointSetNodeData,  // typename NodeDataType
   MHSparseGridMultiRes,         // typename MultiRes
   MHStandardSettings,           // typename SettingsType
   card64,                       // typename SubtreeCounterType
   MultiPartPersistent           // class NodeBaseClass
 >
// ----------------------------------
   MHOutOfCorePointSetOctreeNodeBase;
// ----------------------------------


class HIERARCHIES_API MHOutOfCorePointSetOctreeNode : public MHOutOfCorePointSetOctreeNodeBase {
 private:
   static MetaClass *MHOutOfCorePointSetOctreeNodeMetaClass;
   static Object* newInstance();

	/// counter used to avoid swapping nodes that are still referenced
	card32 lockCounter;
	
 public:
   MHOutOfCorePointSetOctreeNode();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);
   /// the reference to the allocator is copied (the allocator is not owned by this object)
   /// copies are only possible within the same allocator space
   virtual void assign(const Object* obj, CopyContext *context = NULL);
   
   virtual inline void incLockCounter() {
		lockCounter++; 
		//#ifdef _DEBUG
		//	EventCounterDisplay::NewValue e;	e.value = lockCounter;
		//	debugOutput.displayEvent(EventCounterDisplay::getClass(), string("locks"), &e);
		//#endif
	}
   virtual inline void decLockCounter() {
		pAssert(lockCounter > 0); lockCounter--; 
		//#ifdef _DEBUG
		//	EventCounterDisplay::NewValue e; e.value = lockCounter;
		//	debugOutput.displayEvent(EventCounterDisplay::getClass(), string("locks"), &e);
		//#endif
	}
   virtual inline card32 getLockCounter() const {return lockCounter;}
   
   /// do not delete children
   virtual void shallowDelete();
   
   ~MHOutOfCorePointSetOctreeNode() {pAssert(lockCounter == 0);}
};


// ---- the octree itself


typedef MHPointHierachy_TemplBase
 <
   float32,                       // typename FloatType
   3,                             // int Dimension
   MHPointSetPointObject,         // typename ObjectType
   MHOutOfCorePointSetOctreeNode, // typename NodeType
   MHStandardSettings,            // typename SettingsType
   Persistent                     // class HierarchyBaseClass
 >
// ----------------------------
 MHOutOfCorePointSetOctreeBase;
// ----------------------------


class HIERARCHIES_API MHOutOfCorePointSetOctree : public MHOutOfCorePointSetOctreeBase {
 private:
   static MetaClass *MHOutOfCorePointSetOctreeMetaClass;
   static Object* newInstance();

   void doDBOutput(NodePointer node);
   void doDBOutputShort(NodePointer node, card32 &minSize, card32 &maxSize, card32 &sizeSum, card32 &inMem);

	card32 leafNodeNumObjects( NodePointer node, card32& numNodes );
	card32 innerNodeNumObjects( NodePointer node, card32& numNodes );

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void write(OutputObjectStream *s) const;
   virtual void read(InputObjectStream *s);
   virtual void assign(const Object* obj, CopyContext *context = NULL);

   NodeAllocator *getAlloc() {return nAlloc;}
   bool connectedToAllocator() {return nAlloc != NULL;}

   void outputDebugStatisticsShort();
   void outputDebugStatistics();
	void outputBenchmarkStatistics();

   /// do not use - just there to make the linker create all necessary methods
   virtual void instantiateforlinking_donotcall();

   /// enable debug conditions
   static bool noLockCounting() {return false;}
};



}

#endif
