//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "MHOutOfCorePointSetOctreeIterator.h"
//---------------------------------------------------------------------------
#include "LargeBinaryFile.h"
#include "BlockedMultiObjectStream.h"
#include "MHPointSetOctreeIterator_Templ_templCode.h"
#include "MHAllPointsPointSetOctreeIterator_Templ_templCode.h"
#include "MHHierarchy_templCode.h"
#include "MHNodeData_templCode.h"
#include "MHAllocators_templCode.h"
#include "MHNodes_templCode.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

/// maximum number of points which are kept in memory during add operations until swapping out to a temporary file
const card32 MAX_ADDED_POINTS_IN_MEMORY = 100000;

MetaClass* MHOutOfCorePointSetOctreeIterator::MHOutOfCorePointSetOctreeIteratorMetaClass = NULL;

const MetaClass* MHOutOfCorePointSetOctreeIterator::getInstanceClass() const {
   return getClass();
}

Object* MHOutOfCorePointSetOctreeIterator::newInstance() {
   return new MHOutOfCorePointSetOctreeIterator;
}

const MetaClass* MHOutOfCorePointSetOctreeIterator::getClass() {
   if (MHOutOfCorePointSetOctreeIteratorMetaClass==NULL) {
      MHOutOfCorePointSetOctreeIteratorMetaClass = new MetaClass("MHOutOfCorePointSetOctreeIterator", "no description.",
                                  newInstance, sizeof(MHOutOfCorePointSetOctreeIterator));
   }
   return MHOutOfCorePointSetOctreeIteratorMetaClass;
}

void MHOutOfCorePointSetOctreeIterator::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   MHOutOfCorePointSetOctreeIteratorMetaClass->copyParentProperties();
}

void MHOutOfCorePointSetOctreeIterator::shutdown() {
   delete MHOutOfCorePointSetOctreeIteratorMetaClass;
}


// ----



MetaClass* MHAllPointsOutOfCorePointSetOctreeIterator::MHAllPointsOutOfCorePointSetOctreeIteratorMetaClass = NULL;

const MetaClass* MHAllPointsOutOfCorePointSetOctreeIterator::getInstanceClass() const {
   return getClass();
}

Object* MHAllPointsOutOfCorePointSetOctreeIterator::newInstance() {
   return new MHAllPointsOutOfCorePointSetOctreeIterator;
}

const MetaClass* MHAllPointsOutOfCorePointSetOctreeIterator::getClass() {
   if (MHAllPointsOutOfCorePointSetOctreeIteratorMetaClass==NULL) {
      MHAllPointsOutOfCorePointSetOctreeIteratorMetaClass = new MetaClass("MHAllPointsOutOfCorePointSetOctreeIterator", "no description.",
                                  newInstance, sizeof(MHAllPointsOutOfCorePointSetOctreeIterator));
   }
   return MHAllPointsOutOfCorePointSetOctreeIteratorMetaClass;
}

void MHAllPointsOutOfCorePointSetOctreeIterator::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   MHAllPointsOutOfCorePointSetOctreeIteratorMetaClass->copyParentProperties();
}

void MHAllPointsOutOfCorePointSetOctreeIterator::shutdown() {
   delete MHAllPointsOutOfCorePointSetOctreeIteratorMetaClass;
}


// ----


MHOutOfCorePointSetOctreeIterator::OutOfCoreTemporaryPointsBuffer::
OutOfCoreTemporaryPointsBuffer(const VertexDescriptor *descr, string swapFileName)
{
   ps = new PointSet();
   ps->clearAndSetup(1, 0, descr);
   file = NULL;
   this->swapFileName = swapFileName;
}

void MHOutOfCorePointSetOctreeIterator::OutOfCoreTemporaryPointsBuffer::add(const char *point) {
   if (ps->getNumEntries() < MAX_ADDED_POINTS_IN_MEMORY) {
      ps->addPoint(point);
   } else {
      PointSet *newps = new PointSet();
      /// create new buffer
      newps->clearAndSetup(1, 1, ps->getDescr());
      /// copy last element (for "atModifiedPoint")
      newps->setVertex(0, ps->getDataPointer(ps->getNumEntries()-1) );
      /// write old elements
      LargeBinaryFile *temp = checkCreateFile();
      temp->writeBuffer( ps->getDataPointer(0), (ps->getNumEntries() - 1) * ps->getDescr()->getSize() );
      delete ps;
      ps = newps;
   }
}

void MHOutOfCorePointSetOctreeIterator::OutOfCoreTemporaryPointsBuffer::getLastModified(PointSet *&ps) {
   ps = this->ps;
}


LargeBinaryFile *MHOutOfCorePointSetOctreeIterator::OutOfCoreTemporaryPointsBuffer::checkCreateFile() {
   if (file == NULL) {          
      string path = removeFilenameFromPath(swapFileName);
      if (path == "") {
         path = string(".\\");
      } else {
         if (path[path.size()-1] != '\\') {
            path += "\\";
         }
      }
      char tempName[10000];
#ifdef TARGET_PLATFORM_MSVC_2005FF
      unsigned result = GetTempFileNameA(path.c_str(), "ADD", 0, tempName);
      pAssert(result != 0);
#else 
      strcpy(tempName, tempnam(path.c_str(), "ADD"));
#endif
      file = new LargeBinaryFile(tempName);
      tempFileName = string(tempName);
   }
   return file;
}

void MHOutOfCorePointSetOctreeIterator::OutOfCoreTemporaryPointsBuffer::deleteFile() {
   pAssert(file != NULL);
   delete file;
   file = NULL;
#ifdef TARGET_PLATFORM_MSVC_2005FF
   bool success = DeleteFileA(tempFileName.c_str());
#else
   bool success = remove(tempFileName.c_str()) == 0;
#endif
   pAssert(success);
   tempFileName = "";
}


void MHOutOfCorePointSetOctreeIterator::OutOfCoreTemporaryPointsBuffer::addPointsToHierarchyAndClear(MHOutOfCorePointSetOctree *dest) {
   const card32 n = ps->getNumEntries();
   AAT FLAGS = NULL_AAT;
   if (ps->providesAttribute("flags")) FLAGS = ps->getAAT("flags");
   const AAT POSITION = ps->getAAT("position");
   for (card32 i=0; i<n; i++) {
      if (FLAGS != NULL_AAT && (((ps->get<int32, 1>(i, FLAGS)[0]) & PF_FLAG_INVALID)) != 0) continue;
      MHPointSetPointObject pobj(dest->getDescr(), POSITION, FLAGS, ps->getDataPointer(i));
      dest->addObject(pobj);
   }
   dest->checkUpdate();
   dest->getAlloc()->checkLRUSwapOut();
   ps->clear();
   LargeBinaryFile *temp = file;
   if (temp != NULL) {
      const card32 size = ps->getDescr()->getSize();
      const card64 n = temp->getFileSize() / (card64)size;
      pAssert( (temp->getFileSize() % (card64)size) == 0);
      const card64 numBlocks = n / (card64)(MAX_ADDED_POINTS_IN_MEMORY-1);
      pAssert( (n % (card64)(MAX_ADDED_POINTS_IN_MEMORY-1)) == 0);
      char *buffer = new char[(MAX_ADDED_POINTS_IN_MEMORY-1) * size];
      temp->seek(0);
      for (card64 i=0; i<numBlocks; i++) {
         temp->readBuffer(buffer, (MAX_ADDED_POINTS_IN_MEMORY-1) * size);
         for (card32 j=0; j<(MAX_ADDED_POINTS_IN_MEMORY-1); j++) {
            MHPointSetPointObject pobj(dest->getDescr(), POSITION, FLAGS, &(buffer[j*size]));
            dest->addObject(pobj);
            if (j % 1000 == 999) {
               dest->checkUpdate();
					dest->getAlloc()->checkLRUSwapOut();
            }
         }
         dest->checkUpdate();
		   dest->getAlloc()->checkLRUSwapOut();         
      }

      deleteFile();
   }
}

void MHOutOfCorePointSetOctreeIterator::OutOfCoreTemporaryPointsBuffer::deleteLastModified() {
   card32 h = ps->getHeight();
   ps->changeHeight(h-1);
}

MHOutOfCorePointSetOctreeIterator::OutOfCoreTemporaryPointsBuffer::~OutOfCoreTemporaryPointsBuffer() {
   pAssert(ps->getNumEntries() == 0);
   delete ps;
}

MHOutOfCorePointSetOctreeIterator::TemporaryPointsBuffer*
   MHOutOfCorePointSetOctreeIterator::createTempBuffer(MHOutOfCorePointSetOctree *hierarchy)
{
   MHOutOfCorePointSetOctree::NodeAllocator *nAlloc;
   hierarchy->getAllocator(nAlloc);

   MultiObjectStream *mobj = nAlloc->getMultiObjectStream();
   string swapFileName = "";
   if (dynamic_cast<FileBasedMultiObjectStream*>(mobj) != NULL) {
      swapFileName = dynamic_cast<FileBasedMultiObjectStream*>(mobj)->getFilename();
   }

   return new OutOfCoreTemporaryPointsBuffer(hierarchy->getDescr(), swapFileName);
}


// ---


const PointSet *MHOutOfCorePointSetOctreeIterator::lockBlockRead() {
   delete lockBlockReadCopy;
   const PointSet *base = MHPointSetOctreeIterator_Templ<MHOutOfCorePointSetOctree, MHOutOfCorePointSetOctreeNode>::lockBlockRead();
   if (base != NULL) {
      lockBlockReadCopy = (PointSet*)(base->copy());
   } else {
      lockBlockReadCopy = NULL;
   }
   return lockBlockReadCopy;
}

void MHOutOfCorePointSetOctreeIterator::unlock() {
   delete lockBlockReadCopy;
   lockBlockReadCopy = NULL;
   MHPointSetOctreeIterator_Templ<MHOutOfCorePointSetOctree, MHOutOfCorePointSetOctreeNode>::unlock();
}

MHOutOfCorePointSetOctreeIterator::~MHOutOfCorePointSetOctreeIterator() {
   delete lockBlockReadCopy;
	// immediately write back all data (safer...)
	if (needSync && getHierarchy() != NULL) {
		if (getHierarchy()->getAlloc()) {
		   getHierarchy()->getAlloc()->sync();
		}
	}	
	getHierarchy()->getAlloc()->reportStatistics();
}

