//---------------------------------------------------------------------------
#ifndef MHOutOfCorePointSetOctreeIteratorH
#define MHOutOfCorePointSetOctreeIteratorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MHPointSetOctreeIterator_Templ.h"
#include "MHAllPointsPointSetOctreeIterator_Templ.h"
//---------------------------------------------------------------------------

namespace geoxl {

class HierarchicalOutOfCorePointCloud;
class LargeBinaryFile;


class MHOutOfCorePointSetOctreeIterator
   : public MHPointSetOctreeIterator_Templ<MHOutOfCorePointSetOctree, MHOutOfCorePointSetOctreeNode> {
 private:
   static MetaClass *MHOutOfCorePointSetOctreeIteratorMetaClass;
   static Object* newInstance();

   /// store a copy for locked read-only blocks, too, in order to prevent access violation on swapping.
   PointSet *lockBlockReadCopy;

 protected:

   MHOutOfCorePointSetOctreeIterator()
      : MHPointSetOctreeIterator_Templ<MHOutOfCorePointSetOctree, MHOutOfCorePointSetOctreeNode>() {lockBlockReadCopy = NULL;}
   MHOutOfCorePointSetOctreeIterator(MHOutOfCorePointSetOctree *hierarchy)
      : MHPointSetOctreeIterator_Templ<MHOutOfCorePointSetOctree, MHOutOfCorePointSetOctreeNode>(hierarchy) {lockBlockReadCopy = NULL;}

   friend class HierarchicalOutOfCorePointCloud;

   class OutOfCoreTemporaryPointsBuffer  : public TemporaryPointsBuffer {
    private:
      PointSet *ps;
      LargeBinaryFile *file;
      string swapFileName;
      string tempFileName;

      LargeBinaryFile *checkCreateFile();
      void deleteFile();

    public:
      OutOfCoreTemporaryPointsBuffer(const VertexDescriptor *descr, string swapFileName);
      virtual void add(const char *point);
      virtual void getLastModified(PointSet *&ps);
      virtual void addPointsToHierarchyAndClear(MHOutOfCorePointSetOctree *dest);
      virtual void deleteLastModified();
      virtual ~OutOfCoreTemporaryPointsBuffer();
   };

   virtual TemporaryPointsBuffer *createTempBuffer(MHOutOfCorePointSetOctree *hierarchy);

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual const PointSet * lockBlockRead();
   virtual void unlock();

   virtual ~MHOutOfCorePointSetOctreeIterator();
};


class MHAllPointsOutOfCorePointSetOctreeIterator : public
   MHAllPointsPointSetOctreeIterator_Templ<MHOutOfCorePointSetOctree, MHOutOfCorePointSetOctreeNode> {
 private:
   static MetaClass *MHAllPointsOutOfCorePointSetOctreeIteratorMetaClass;
   static Object* newInstance();

 protected:
   MHAllPointsOutOfCorePointSetOctreeIterator()
      : MHAllPointsPointSetOctreeIterator_Templ<MHOutOfCorePointSetOctree, MHOutOfCorePointSetOctreeNode>() {}
   MHAllPointsOutOfCorePointSetOctreeIterator(MHOutOfCorePointSetOctreeIterator *baseIterator)
      : MHAllPointsPointSetOctreeIterator_Templ<MHOutOfCorePointSetOctree, MHOutOfCorePointSetOctreeNode>(baseIterator) {}

   friend class HierarchicalOutOfCorePointCloud;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();
};


}

#endif
