//---------------------------------------------------------------------------
#ifndef MHPointSetOctreeIterator_Templ_templCodeH
#define MHPointSetOctreeIterator_Templ_templCodeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MHPointSetOctreeIterator_Templ.h"
//---------------------------------------------------------------------------
#include "MHIterators.h"
#include "MHObjects_templCode.h"
//---------------------------------------------------------------------------

namespace geoxl {


template <class OctreeType, class NodeType>
MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// -----------------------------
MHPointSetOctreeIterator_Templ()
// -----------------------------
{
   hierarchy = NULL;
   currentNode = NULL;
   currentNodeDirectPointer = NULL;
   currentIndex = 0;
   modifiedPoints = NULL;
   rwPointSet = NULL;
   atModifiedPoint = false;
   pointSetLocked = false;
   needSync = false;
}



template <class OctreeType, class NodeType>
MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// --------------------------------------------------
MHPointSetOctreeIterator_Templ(OctreeType *hierarchy)
// --------------------------------------------------
{
   pAssert(hierarchy != NULL);
   this->hierarchy = hierarchy;
   hierarchy->getRoot(currentNode);
	currentNodeDirectPointer = (NodeType*)hierarchy->getAllocator()->getPhysicalPtr(currentNode);
	if (currentNode != NULL) {
		((NodeType*)currentNodeDirectPointer)->incLockCounter();
		hierarchy->getAllocator()->lruTouch(currentNode);
	}
	
   currentIndex = 0;
   modifiedPoints = NULL;
   rwPointSet = NULL;
   atModifiedPoint = false;
   aatPosition = hierarchy->getDescr()->getToken("position");
   if (hierarchy->getDescr()->providesAttribute("flags")) {
      aatFlags = hierarchy->getDescr()->getToken("flags");
   } else {
      aatFlags = NULL_AAT;
   }
   pointSetLocked = false;
   hierarchy->getAlloc()->checkLRUSwapOut();
   needSync = false;
}



template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ----------------
addModifiedPoints() // must only be called in destructor!
// ----------------
{
   if (modifiedPoints == NULL) return;
   pAssert(hierarchy != NULL);
   modifiedPoints->addPointsToHierarchyAndClear(hierarchy);

   hierarchy->checkUpdate();

   atModifiedPoint = false;
   hierarchy->getAlloc()->checkLRUSwapOut();
	needSync = true;
}



template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ---------------------------
checkCreateModifiedPtsBuffer()
// ---------------------------
{
   if (modifiedPoints == NULL) {
      modifiedPoints = createTempBuffer(hierarchy);
   }
}



template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// --------------------------------------------
assign(const Object* obj, CopyContext *context)
// --------------------------------------------
{
   pAssert(obj != NULL);
	if (currentNode != NULL) {
		hierarchy->getAllocator()->lruTouch(currentNode);
		pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
		((NodeType*)currentNodeDirectPointer)->decLockCounter();
	}
   MHPointSetOctreeIterator_Templ<OctreeType, NodeType> *other = (MHPointSetOctreeIterator_Templ<OctreeType, NodeType>*)(obj);
	if (pointSetLocked || other->pointSetLocked) {
		throw PException("MHPointSetOctreeIterator_Templ<>::assign() - operation not permitted while a point set is locked.");
	}
   hierarchy = other->hierarchy;
   currentNode = other->currentNode;
   currentNodeDirectPointer = other->currentNodeDirectPointer;
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
	if (currentNodeDirectPointer != NULL) {
		currentNodeDirectPointer->incLockCounter();
	}
   currentIndex = other->currentIndex;
   aatPosition = other->aatPosition;
   aatFlags = other->aatFlags;
   delete modifiedPoints;
   modifiedPoints = NULL;
   forget(context);
	needSync = true;
}



template <class OctreeType, class NodeType>
bool MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ---------------------------
equal(const Object* obj) const
// ---------------------------
{
   if (obj == NULL) return false;
   MHPointSetOctreeIterator_Templ<OctreeType, NodeType> *other = (MHPointSetOctreeIterator_Templ<OctreeType, NodeType>*)(obj);

   return hierarchy == other->hierarchy
       && currentNode == other->currentNode
       && currentIndex == other->currentIndex
       && modifiedPoints == other->modifiedPoints  // i.e. == NULL
       && rwPointSet == other->rwPointSet // i.e. == NULL
       && atModifiedPoint == other->atModifiedPoint // (should be false at this point)
       && aatPosition == other->aatPosition
       && aatFlags == other->aatFlags
       && pointSetLocked == other->pointSetLocked;
}



template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ----
reset()
// ----
{
   currentIndex = 0;
   atModifiedPoint = false;
}



template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ---
next()
// ---
{
   if (atModifiedPoint) {
      atModifiedPoint = false;
   } else {
      if (!atEnd()) {
         currentIndex++;
      }
   }
}




template <class OctreeType, class NodeType>
card64 MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ------------------------
getNumPointsInBlock() const 
// ------------------------
{
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
	return currentNodeDirectPointer->nodeData.getPointSet()->getNumEntries();
}


template <class OctreeType, class NodeType>
card64 MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// -------------------------
getCurrentPointIndex() const
// -------------------------
{
	return currentIndex;
}


template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ----------------
goToIndex(card64 i)
// ----------------
{
	pAssert(i < getNumPointsInBlock());
	if ((max_platform_card)i != currentIndex) {
		atModifiedPoint = false;
	}
	currentIndex = (max_platform_card)i;
}



template <class OctreeType, class NodeType>
bool MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ----
atEnd()
// ----
{
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));

   if (currentNodeDirectPointer == NULL) return true;
   if (currentNodeDirectPointer->isLeaf()) {
      return currentIndex >= currentNodeDirectPointer->nodeData.getNumObjects();
   } else {
      return currentIndex >= currentNodeDirectPointer->multiRes.getNumObjects();
   }
}



template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ----------------------
add(const char *newPoint)
// ----------------------
{
   checkCreateModifiedPtsBuffer();
   modifiedPoints->add(newPoint);
	needSync = true;
}


template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ---------------------------
add(const PointSet *newPoints)
// ---------------------------
{
   checkCreateModifiedPtsBuffer();
   modifiedPoints->addPointSet(newPoints);
	needSync = true;
}



template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ------------
deleteCurrent()
// ------------
{
   pAssert(hierarchy != NULL);
   pAssert(currentNode != NULL);
   if (atModifiedPoint) {
	   pAssert(modifiedPoints != NULL);
	   modifiedPoints->deleteLastModified();
	   atModifiedPoint = false;
   } else {
	   hierarchy->deleteObject(currentNode, currentIndex);
	   hierarchy->getAlloc()->checkLRUSwapOut();
   }
	needSync = true;
}



template <class OctreeType, class NodeType>
card32 MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// -------------
getNumChildren()
// -------------
{
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));

   if (currentNode == NULL) {
      return 0;
   } else {
      return currentNodeDirectPointer->getMaxNumChildren();
   }
}



template <class OctreeType, class NodeType>
bool MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// --------------------
goToChild(card32 index)
// --------------------
{
	if (pointSetLocked) {
		throw PException("MHPointSetOctreeIterator_Templ<>::goToChild() - operation not permitted while a point set is locked.");
	}
   reset();
   if (currentNode != NULL) {		
      NodePointer child;
      currentNode->getChild(index, child);
      if (child != NULL) {
			hierarchy->getAllocator()->lruTouch(currentNode);
			pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
			((NodeType*)currentNodeDirectPointer)->decLockCounter();

         currentNode = child;

			currentNodeDirectPointer = (NodeType*)hierarchy->getAllocator()->getPhysicalPtr(currentNode);
			((NodeType*)currentNodeDirectPointer)->incLockCounter();
			hierarchy->getAllocator()->lruTouch(currentNode);
		   hierarchy->getAlloc()->checkLRUSwapOut();

         return true;
      } else {
         return false;
      }
   } else {
      return false;
   }
}



template <class OctreeType, class NodeType>
bool MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ---------
goToParent()
// ---------
{
	if (pointSetLocked) {
		throw PException("MHPointSetOctreeIterator_Templ<>::goToParent() - operation not permitted while a point set is locked.");
	}
   reset();
   if (currentNode != NULL) {
      NodePointer parent;
      currentNode->getParent(parent);
      if (parent != NULL) {
			pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
			((NodeType*)currentNodeDirectPointer)->decLockCounter();

         currentNode = parent;

			currentNodeDirectPointer = (NodeType*)hierarchy->getAllocator()->getPhysicalPtr(currentNode);
			((NodeType*)currentNodeDirectPointer)->incLockCounter();
			hierarchy->getAllocator()->lruTouch(currentNode);
		   hierarchy->getAlloc()->checkLRUSwapOut();

         return true;
      } else {
         return false;
      }
   } else {
      return false;
   }
}



template <class OctreeType, class NodeType>
BoundingBox3f MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// -------------
getBoundingBox()
// -------------
{
   if (currentNodeDirectPointer != NULL) {
		pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
      return currentNodeDirectPointer->getBoundingBox();
   } else {
      return BoundingBox3f(NULL_VECTOR3F);
   }
}



template <class OctreeType, class NodeType>
bool MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// -----------
isLeaf() const
// -----------
{
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
   if (currentNodeDirectPointer != NULL) {
      return currentNodeDirectPointer->isLeaf();
   } else {
      return true;
   }
}



template <class OctreeType, class NodeType>
bool MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// -------------------------------------------------------------------
getNodeInfo(NodeInfoType whichInfo, const void *parameter, void *data)
// -------------------------------------------------------------------
{
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
   if (currentNodeDirectPointer == NULL) return false;

   switch (whichInfo) {

    case niPointSpacing: {
      if (currentNodeDirectPointer->isLeaf()) {
         *((float32*)data) = 0;
      } else {
         static const float32 SQRT3 = sqrt(3.0f);
         MHStandardSettings settings;
         hierarchy->getSettings(settings);
         *((float32*)data) = currentNodeDirectPointer->getBoundingBox().getSideLength(0) / settings.multiResGridSize * SQRT3;
      }
      return true;
    }

    case niFlaggedInSubtree: {
      *((card64*)data) = currentNodeDirectPointer->nodeData.getFlagsCounter(*((card32*)parameter));
      return true;
    }

    case niNumPointsInNode: {
      if (currentNodeDirectPointer->isLeaf()) {
         *((card64*)data) = currentNodeDirectPointer->nodeData.getNumObjects();
      } else {
         *((card64*)data) = currentNodeDirectPointer->multiRes.getNumObjects();
      }
      return true;
    }

    case niNumPointsInSubtree: {
      *((card64*)data) = currentNodeDirectPointer->objsInSubtree;
      return true;
    }
    
    case niChildAvailable:	{
		card32 childMask = *((card32*)parameter);
		bool result = true;
		if (childMask >= 8) {
			for (card32 i=0; i<8; i++) {
				NodePointer child;
				currentNodeDirectPointer->getChild(i, child);
				if (child != NULL) {
					result = result && hierarchy->getAlloc()->isInMem(child);
				}
			}
		} else {
			NodePointer child;
			currentNodeDirectPointer->getChild(childMask, child);
			if (child != NULL) {
				result = hierarchy->getAlloc()->isInMem(child);
			} else {
				result = true;
			}
		}		
		*((bool*)data) = result;
		return true;
    }
    case niPrefetchChild: {
		card32 childMask = *((card32*)parameter);
		if (childMask >= 8) {
			for (card32 i=0; i<8; i++) {
				NodePointer child;
				currentNodeDirectPointer->getChild(i, child);
				if (child != NULL) {
					hierarchy->getAlloc()->prefetch(child);	
				}
			}
		} else {
			NodePointer child;
			currentNodeDirectPointer->getChild(childMask, child);
			hierarchy->getAlloc()->prefetch(child);	
		}		
		return true;
    }
    

    default:
    return false;
   };

}


// --- attachments


template <class OctreeType, class NodeType>
AttachedDataContainer *MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// --------------
lockAttachments()
// --------------
{
   pAssert(currentNode != NULL);
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
	((NodeType*)currentNodeDirectPointer)->incLockCounter();
   return currentNodeDirectPointer->nodeData.getAttachments();
}



template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ----------------
unlockAttachments()
// ----------------
{
   pAssert(currentNode != NULL);
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
	((NodeType*)currentNodeDirectPointer)->decLockCounter();
	pAssert(currentNodeDirectPointer->getLockCounter() > 0 || OctreeType::noLockCounting());
}



// --- generic set/get



template <class OctreeType, class NodeType>
template <class Type, int dim>
inline StaticVector<Type, dim> MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::genericGet(AAT token) const {
   #ifdef _DEBUG
   pAssert(sizeof(StaticVector<Type, dim>) == getAATSize(token));
   #endif
   pAssert(currentNode != NULL);
   pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));

   if (!atModifiedPoint) {
      if (isLeaf()) {
         pAssert(currentIndex < currentNodeDirectPointer->nodeData.getNumObjects());
         PointSet *ps = currentNodeDirectPointer->nodeData.getPointSet();
         return ps->get<Type, dim>(currentIndex, token);
      } else {
         pAssert(currentIndex < currentNodeDirectPointer->multiRes.getNumObjects());
         PointSet *ps = currentNodeDirectPointer->multiRes.getPointSet();
         return ps->get<Type, dim>(currentIndex, token);
      }
   } else {
      PointSet *lastModPts;
      pAssert(modifiedPoints != NULL);
      modifiedPoints->getLastModified(lastModPts);
      return lastModPts->get<Type, dim>(lastModPts->getNumEntries()-1, token);
   }
}




template <class OctreeType, class NodeType>
template <class Type>
inline Type MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::genericGetS(AAT token) const {
   #ifdef _DEBUG
   pAssert(sizeof(Type) == getAATSize(token));
   #endif
   pAssert(currentNode != NULL);
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));

   if (!atModifiedPoint) {
      if (currentNodeDirectPointer->isLeaf()) {
         pAssert(currentIndex < currentNodeDirectPointer->nodeData.getNumObjects());
         PointSet *ps = currentNodeDirectPointer->nodeData.getPointSet();
         const StaticVector<Type, 1> v = ps->get<Type, 1>(currentIndex, token);
         return v[0];
      } else {
         pAssert(currentIndex < currentNodeDirectPointer->multiRes.getNumObjects());
         PointSet *ps = currentNodeDirectPointer->multiRes.getPointSet();
         const StaticVector<Type, 1> v = ps->get<Type, 1>(currentIndex, token);
         return v[0];
      }
   } else {
      PointSet *lastModPts;
      pAssert(modifiedPoints != NULL);
      modifiedPoints->getLastModified(lastModPts);
      const StaticVector<Type, 1> v = lastModPts->get<Type, 1>(lastModPts->getNumEntries()-1, token);
      return v[0];
   }
}



template <class OctreeType, class NodeType>
template <class Type>
inline void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::genericSetS(AAT token, Type x) const {
   pAssert(isLeaf() && currentNode != NULL);
   pAssert(token != aatPosition);
   #ifdef _DEBUG
   pAssert(sizeof(Type) == getAATSize(token));
   #endif
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
	currentNodeDirectPointer->setDirty(true);
	needSync = true;

   if (!atModifiedPoint) {

      pAssert(currentNodeDirectPointer != NULL && currentIndex < currentNodeDirectPointer->nodeData.getNumObjects());

      char *data = new char[hierarchy->getDescr()->getSize()];
      currentNodeDirectPointer->nodeData.getPointSet()->getVertex(currentIndex, data);
      StaticVector<Type, 1> v1; v1[0] = x;
      token.setAttrib<Type, 1>(data, v1);
      MHPointSetPointObject temp(hierarchy->getDescr(), aatPosition, aatFlags, data);
      hierarchy->changeObjectAttributes(temp, currentNode, currentIndex);
      delete[] data;

   } else {

      PointSet *lastModPts;
      pAssert(modifiedPoints != NULL);
      modifiedPoints->getLastModified(lastModPts);
      StaticVector<Type, 1> v1; v1[0] = x;
      lastModPts->set<Type, 1>(lastModPts->getNumEntries()-1, token, v1);

   }
}



template <class OctreeType, class NodeType>
template <class Type, int dim>
inline void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::genericSet(AAT token, StaticVector<Type, dim> v) const {
   pAssert(isLeaf() && currentNode != NULL);
   //pAssert(aatFlags == NULL_AAT || token.getOffset() != aatFlags.getOffset());
   pAssert(token != aatFlags);
   #ifdef _DEBUG
   pAssert(sizeof(StaticVector<Type, dim>) == getAATSize(token));
   #endif

	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
	currentNodeDirectPointer->setDirty(true);
	needSync = true;

   if (!atModifiedPoint) {

      pAssert(currentNode != NULL && currentIndex < currentNode->nodeData.getNumObjects());

      if (token == aatPosition) {

         const char *vertex = currentNodeDirectPointer->nodeData.getPointSet()->getDataPointer(currentIndex);
         const card32 n = hierarchy->getDescr()->getSize();
         char *buffer = new char[n];
         for (card32 i=0; i<n; i++) {
            buffer[i] = vertex[i];
         }
         const_cast<MHPointSetOctreeIterator_Templ<OctreeType, NodeType>*>(this)->deleteCurrent();
         atModifiedPoint = true;
         token.setAttrib<Type, dim>(buffer,v);
         const_cast<MHPointSetOctreeIterator_Templ<OctreeType, NodeType>*>(this)->add(buffer);
         delete[] buffer;

      } else {

         char *data = new char[hierarchy->getDescr()->getSize()];
         currentNodeDirectPointer->nodeData.getPointSet()->getVertex(currentIndex, data);
         token.setAttrib<Type, dim>(data, v);
         MHPointSetPointObject temp(hierarchy->getDescr(), aatPosition, aatFlags, data);
         hierarchy->changeObjectAttributes(temp, currentNode, currentIndex);
         delete[] data;

      }

   } else {

      PointSet *lastModPts;
      pAssert(modifiedPoints != NULL);
      modifiedPoints->getLastModified(lastModPts);
      lastModPts->set<Type, dim>(lastModPts->getNumEntries()-1, token, v);

   }
}



// ---- locking



template <class OctreeType, class NodeType>
const PointSet *MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::lockBlockRead() {
   pAssert(rwPointSet == NULL);
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));

   if(currentNode != NULL){
      if (isLeaf()) {
			pointSetLocked = true;
         return currentNodeDirectPointer->nodeData.getPointSet();
      } else {
			pointSetLocked = true;
         return currentNodeDirectPointer->multiRes.getPointSet();
      }
   } else {
      return NULL;
   }
}


template <class OctreeType, class NodeType>
PointSet *MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::lockBlockReadWrite(LockRWHint hint) {
   pAssert(rwPointSet == NULL);
   pAssert(isLeaf() && currentNode != NULL);
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));

	if(hint==ModifyablePointCloudIterator::CHANGE_NOFLAG_ATTRIBS_ONLY)
		return (PointSet*)(currentNodeDirectPointer->nodeData.getPointSet());
	else{
		currentNodeDirectPointer->setDirty(true);
		needSync = true;
		pointSetLocked = true;
		rwPointSet = (PointSet*)(currentNodeDirectPointer->nodeData.getPointSet()->copy());

		currentIndex = 0;
		atModifiedPoint = false;
		const max_platform_card n = currentNodeDirectPointer->nodeData.getNumObjects();
		for (max_platform_card i=0; i<n; i++) {
			deleteCurrent();
			hierarchy->getAlloc()->checkLRUSwapOut();
		}

		return rwPointSet;
	}
}




template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::unlock() {
	pointSetLocked = false;
   if (rwPointSet != NULL) {
      add(rwPointSet);
      delete rwPointSet;
      rwPointSet = NULL;
   }
   hierarchy->getAlloc()->checkLRUSwapOut();
}



// --- set/get operations



template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::getVertex(char *buffer) const {
   pAssert(currentNode != NULL);
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));

   if (!atModifiedPoint) {
      if (isLeaf()) {
         pAssert(currentNode != NULL && currentIndex < currentNode->nodeData.getNumObjects());
         currentNodeDirectPointer->nodeData.getPointSet()->getVertex(currentIndex, buffer);
      } else {
         pAssert(currentNode != NULL && currentIndex < currentNode->multiRes.getNumObjects());
         currentNodeDirectPointer->multiRes.getPointSet()->getVertex(currentIndex, buffer);
      }
   } else {
      PointSet *lastModPts;
      pAssert(modifiedPoints != NULL);
      modifiedPoints->getLastModified(lastModPts);
      lastModPts->getVertex(lastModPts->getNumEntries()-1, buffer);
   }
}



template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::setVertex(const char *buffer) const {
   pAssert(isLeaf() && currentNode != NULL);
	pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));

	currentNodeDirectPointer->setDirty(true);
	needSync = true;
   if (!atModifiedPoint) {

      pAssert(currentNode != NULL && currentIndex < currentNode->nodeData.getNumObjects());

      PointSet *ps = currentNodeDirectPointer->nodeData.getPointSet();
      Vector3f pos = ps->get<float32, 3>(currentIndex, aatPosition);
      Vector3f *newPos = (Vector3f*)(buffer + aatPosition.getOffset());

      if (pos == *newPos) {

         MHPointSetPointObject temp(hierarchy->getDescr(), aatPosition, aatFlags, (char*)buffer);
         hierarchy->changeObjectAttributes(temp, currentNode, currentIndex);

      } else {  // position is modified ---> delete and create new point

         const_cast<MHPointSetOctreeIterator_Templ<OctreeType, NodeType>*>(this)->deleteCurrent();
         const_cast<MHPointSetOctreeIterator_Templ<OctreeType, NodeType>*>(this)->add(buffer);
         const_cast<MHPointSetOctreeIterator_Templ<OctreeType, NodeType>*>(this)->atModifiedPoint = true;
		   hierarchy->getAlloc()->checkLRUSwapOut();

      }

   } else {  // currently at modified point

      PointSet *lastModPts;
      pAssert(modifiedPoints != NULL);
      modifiedPoints->getLastModified(lastModPts);
      lastModPts->setVertex(lastModPts->getNumEntries()-1, buffer);

   }
}



template <class OctreeType, class NodeType>
card8 MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get1ub(const AAT &token) const {
   return genericGetS<card8>(token);
}
template <class OctreeType, class NodeType>
Vector2ub MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get2ub(const AAT &token) const {
   return genericGet<card8, 2>(token);
}
template <class OctreeType, class NodeType>
Vector3ub MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get3ub(const AAT &token) const {
   return genericGet<card8, 3>(token);
}
template <class OctreeType, class NodeType>
Vector4ub MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get4ub(const AAT &token) const {
   return genericGet<card8, 4>(token);
}
template <class OctreeType, class NodeType>
int32 MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get1i(const AAT &token) const {
   return genericGetS<int32>(token);
}
template <class OctreeType, class NodeType>
Vector2i MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get2i(const AAT &token) const {
   return genericGet<int32, 2>(token);
}
template <class OctreeType, class NodeType>
Vector3i MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get3i(const AAT &token) const {
   return genericGet<int32, 3>(token);
}
template <class OctreeType, class NodeType>
Vector4i MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get4i(const AAT &token) const {
   return genericGet<int32, 4>(token);
}
template <class OctreeType, class NodeType>
float32 MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get1f(const AAT &token) const {
   return genericGetS<float32>(token);
}
template <class OctreeType, class NodeType>
Vector2f MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get2f(const AAT &token) const {
   return genericGet<float32, 2>(token);
}
template <class OctreeType, class NodeType>
Vector3f MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get3f(const AAT &token) const {
   return genericGet<float32, 3>(token);
}
template <class OctreeType, class NodeType>
Vector4f MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::get4f(const AAT &token) const {
   return genericGet<float32, 4>(token);
}



template <class OctreeType, class NodeType>
bool MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::nextBlock() {
   return false;
}



template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set1ub(const AAT &token, const card8 &x)     const {
   genericSetS<card8>(token, x);
}
template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set2ub(const AAT &token, const Vector2ub &v) const {
   genericSet<card8, 2>(token, v);
}
template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set3ub(const AAT &token, const Vector3ub &v) const {
   genericSet<card8, 3>(token, v);
}
template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set4ub(const AAT &token, const Vector4ub &v) const {
   genericSet<card8, 4>(token, v);
}
template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set1i(const AAT &token, const int32 &x)    const {
   genericSetS<int32>(token, x);
}
template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set2i(const AAT &token, const Vector2i &v) const {
   genericSet<int32, 2>(token, v);
}
template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set3i(const AAT &token, const Vector3i &v) const {
   genericSet<int32, 3>(token, v);
}
template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set4i(const AAT &token, const Vector4i &v) const {
   genericSet<int32, 4>(token, v);
}
template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set1f(const AAT &token, const float32 &x)  const {
   genericSetS<float32>(token, x);
}
template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set2f(const AAT &token, const Vector2f &v) const {
   genericSet<float32, 2>(token, v);
}
template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set3f(const AAT &token, const Vector3f &v) const {
   genericSet<float32, 3>(token, v);
}
template <class OctreeType, class NodeType>
void MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::set4f(const AAT &token, const Vector4f &v) const {
   genericSet<float32, 4>(token, v);
}



// --- destructor


template <class OctreeType, class NodeType>
MHPointSetOctreeIterator_Templ<OctreeType, NodeType>::
// ------------------------------
~MHPointSetOctreeIterator_Templ()
// ------------------------------
{
	// release current node lock (because nodes can be destroyed in checkUpdate() [which is also called in addModifiedPoints()])
   if (currentNode != NULL) {
		pAssert(currentNodeDirectPointer == hierarchy->getAlloc()->getPhysicalPtr(currentNode));
		((NodeType*)currentNodeDirectPointer)->decLockCounter();
   }
   currentNode = NULL;
   currentNodeDirectPointer = NULL;   
   
   addModifiedPoints();
   delete modifiedPoints;
   
   if (hierarchy != NULL) hierarchy->checkUpdate();
   pAssert(rwPointSet == NULL);
   hierarchy->getAlloc()->checkLRUSwapOut();
}


}


#endif

