//---------------------------------------------------------------------------
#ifndef SparseGridH
#define SparseGridH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "HashTables.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
#include "PAssert.h"
//---------------------------------------------------------------------------

namespace geoxl {

class OutputObjectStream;
class InputObjectStream;


template <int Dimension, typename ObjectType, class BaseClass>
class SparseGrid : public BaseClass {
 private:
   typedef StaticVector<int32, Dimension> KeyType;

   struct KeyComp {
      static bool equals(KeyType pos1, KeyType pos2) { return pos1 == pos2; }
   };

   class SparseGridHashTable : public HashTable<KeyType, ObjectType, KeyComp> {
    protected:
      virtual max_platform_card hashFunction(KeyType key) const {
         return (key[0]+521) + 1031*((key[1]+521) + 1031*(key[2]+521));
      };
    public:
   };

   SparseGridHashTable grid;

 public:
   void clear();
   inline bool getObject(StaticVector<int32, Dimension> index, ObjectType &obj);
   inline void setObject(StaticVector<int32, Dimension> index, ObjectType obj);
   inline void deleteObject(StaticVector<int32, Dimension> index);

   template <typename FloatType>
   inline static StaticVector<int32, Dimension> calcIndex(StaticVector<FloatType, Dimension> pos,
                                                        StaticVector<FloatType, Dimension> origin,
                                                        FloatType gridSize);
   void operator=(const SparseGrid<Dimension, ObjectType, BaseClass> &other);
   void write(OutputObjectStream *s) const;
   void read(InputObjectStream *s);

   inline max_platform_card getNumElements()           const {return grid.count();}
   inline ObjectType getEntry(max_platform_card index) const {return grid.getData(index);}
   inline KeyType getKey(max_platform_card index)      const {return grid.getKey(index);}
};


}

#endif
