//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "InitPointClouds.h"
//---------------------------------------------------------------------------

/*
	Please separate between system code and interface code ( editors ), in 
	order to build a non-interface commandline version of geoxl.

File structure:
	includes
		..
		..
	interface includes
		..
		..
	initPointClouds()
	{
		init geoxl classes
		..
		..
		init geoxl class editors
		..
		..
	}
	shutdownPointClouds()
	{
		shutdown geoxl classes
		..
		..
		shutdown geoxl class editors
		..
		..
				
	}
*/
//---------------------------------------------------------------------------

#include "ParametricInCorePointCloud.h"
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "RegularVolume.h"
#include "PointSetIterator.h"
#include "UnstructuredInCorePointCloud.h"
#include "VertexDescriptor.h"
#include "SceneObject.h"
#include "UnstructuredInCoreTriangleMesh.h"
#include "VolumeObject.h"
#include "MultiResolutionInCoreVolume.h"
#include "InCoreVolume.h"
#include "RegularInCoreVolume.h"
#include "PointCloudTopologyGraph.h"
#include "InCorePCTopologyGraph.h"
#include "TopologyAttachment.h"
#include "AttachedIndexedOctree.h"
#include "ScatteredDataInterpolation.h"
#include "SplatPullPushInterpolation.h"
#include "GenericSwapAllOutOfCorePointCloud.h"
// ---------------------------------------------------------------------------------
//					Interface Includes
// ---------------------------------------------------------------------------------
#ifdef GEOXL_GUI
   #include "VertexDescriptorEditorWidget.h"
   #include "pointClouds_resources_rcc.h"
#endif	

//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


void geoxl::initPointClouds()
{
// ---------------------------------------------------------------------------------
//					Class Registration ( no Editors ! )
// ---------------------------------------------------------------------------------
	PointCloudIterator::init(Object::getClass());
      PointSetIterator::init(PointCloudIterator::getClass());
      BasicParametricInCorePointCloudIterator::init(PointSetIterator::getClass());

   PointSet::init(Persistent::getClass());
   VertexDescriptor::init(Persistent::getClass());
   
   RegularVolume::init(Persistent::getClass());
   
   SceneObject::init(Persistent::getClass());
      PointCloud::init(SceneObject::getClass());
         ParametricInCorePointCloud::init(PointCloud::getClass());
         UnstructuredInCorePointCloud::init(PointCloud::getClass());
            UnstructuredInCoreTriangleMesh::init(UnstructuredInCorePointCloud::getClass());
         VolumeObject::init(PointCloud::getClass());
            InCoreVolume::init(VolumeObject::getClass());
               RegularInCoreVolume::init(InCoreVolume::getClass());
               MultiResolutionInCoreVolume::init(InCoreVolume::getClass());
			GenericSwapAllOutOfCorePointCloud::init(PointCloud::getClass());

	PointCloudTopologyGraph::init(Persistent::getClass());
		InCorePCTopologyGraph::init(PointCloudTopologyGraph::getClass());
	TopologyAttachment::init(AttachedData::getClass());
	AttachedIndexedOctree::init(AttachedData::getClass());
	ScatteredDataInterpolation::init( Persistent::getClass() );
	SplatPullPushInterpolation::init(ScatteredDataInterpolation::getClass());


// ---------------------------------------------------------------------------------
//					Editor Class Registration 
// ---------------------------------------------------------------------------------
#ifdef GEOXL_GUI

	VertexDescriptorEditor::init(ClassEditor::getClass());
		((GUIDescriptor * )VertexDescriptor::getClass()->getGUIDescriptor())->addClassEditor(
			VertexDescriptorEditor::getClass(), true);

   qInitResources();

#endif	// end of #ifdef GEOXL_GUI

}

void geoxl::shutdownPointClouds()
{
// ---------------------------------------------------------------------------------
//					Class Shutdown
// ---------------------------------------------------------------------------------
	PointCloudIterator::shutdown();
   PointSetIterator::shutdown();
   BasicParametricInCorePointCloudIterator::shutdown();

   PointSet::shutdown();
   VertexDescriptor::shutdown();
   
   RegularVolume::shutdown();
   
   SceneObject::shutdown();
   PointCloud::shutdown();
   ParametricInCorePointCloud::shutdown();
   UnstructuredInCorePointCloud::shutdown();
   UnstructuredInCoreTriangleMesh::shutdown();
   VolumeObject::shutdown();
   RegularInCoreVolume::shutdown();
   MultiResolutionInCoreVolume::shutdown();
   GenericSwapAllOutOfCorePointCloud::shutdown();

	PointCloudTopologyGraph::shutdown();
	InCorePCTopologyGraph::shutdown();
	TopologyAttachment::shutdown();
	AttachedIndexedOctree::shutdown();
	ScatteredDataInterpolation::shutdown();
	SplatPullPushInterpolation::shutdown();


// ---------------------------------------------------------------------------------
//					Editor Class Shutdown
// ---------------------------------------------------------------------------------
#ifdef GEOXL_GUI

   VertexDescriptorEditor::shutdown();

#endif	// end of #ifdef GEOXL_GUI

}
