//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "AttachedIndexedOctreeGeometricIterator.h"
//---------------------------------------------------------------------------
#include "DebugOutput.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


void AttachedIndexedOctreeGeometricIterator::setup(SceneObjectIterator* pcIt, const card32& subTreeSize,
                 MemoryManager *attachedOctreeMemManager,GeometricRange *range){


   pntSet=NULL;
   m_range=range;

   ModifyablePointCloudIterator* mod= dynamic_cast<ModifyablePointCloudIterator*>(pcIt);
   pntSet= mod->lockBlockReadWrite();

   POSITION = pntSet->getDescr()->getToken("position");

   //get the Octree
   #ifdef _DEBUG
   HierarchicalIterator* m_hpi=dynamic_cast<HierarchicalIterator*>(pcIt);
   pAssert(m_hpi->isLeaf());
   #endif

   SecondaryDataStructureIterator* m_sds = dynamic_cast<SecondaryDataStructureIterator*>(pcIt);
   pAssert(m_sds);
	AttachedDataContainer *attachments = m_sds->lockAttachments();
   AttachedIndexedOctree* m_octree = dynamic_cast<AttachedIndexedOctree*>(attachments->getData("SimpleOctree"));
   if(m_octree==NULL){
            m_octree=new AttachedIndexedOctree();
            m_octree->setup("SimpleOctree", AttachedData::ADF_QUERY_DELETE_ON_UPDATE,  attachedOctreeMemManager); // attachedOctreeMemManager !!
            m_octree->octreesetup( pntSet, subTreeSize );
            attachments->attachData(m_octree);
   }
   rootNode= m_octree->getOctree();
   reset();
   m_sds->unlockAttachments();
}

bool AttachedIndexedOctreeGeometricIterator::getNextPointIndex(card32 &nextPoint ){

   //if one more point in currentBox, get it!
   if(currentBox!=NULL && currentBoxPosition<currentBox->size()){
      nextPoint=(*currentBox)[currentBoxPosition];
      currentBoxPosition++;
      return true;
   }

   //no more point in currentBox -> open next box
   bool moreBoxesOnStack=true;
   while ((currentBox==NULL || currentBoxPosition>=currentBox->size()) && moreBoxesOnStack)
         moreBoxesOnStack=pullBoxFromStack();

   //if now there is one more point in currentBox, get it!
   if(currentBox!=NULL && currentBoxPosition<currentBox->size()){
      nextPoint=(*currentBox)[currentBoxPosition];
      currentBoxPosition++;
      return true;
   } else
   //no more Box and no more Point
   {
      return false;
   }

}

void AttachedIndexedOctreeGeometricIterator::reset(){
   currentBox=NULL;
   nodeStack.clear();

   if (m_range->intersects(rootNode->getContent().m_bb)){
        nodeStack.push_back(rootNode);
        noMorePoints=false;
        next();
   }else{
        noMorePoints=true;
   }

}

void AttachedIndexedOctreeGeometricIterator::next(){

   noMorePoints=!getNextPointIndex(atPoint);     //write new index to atPoint if found
   if (!noMorePoints && !m_range->intersects(get3f(POSITION))) next();
}

bool AttachedIndexedOctreeGeometricIterator::atEnd(){

   return noMorePoints;
}

card64 AttachedIndexedOctreeGeometricIterator::getNumPointsInBlock() const {
	return pntSet->getNumEntries();
}

card64 AttachedIndexedOctreeGeometricIterator::getCurrentPointIndex() const {
	return atPoint;
}



bool AttachedIndexedOctreeGeometricIterator::pullBoxFromStack(){
   if (nodeStack.empty())
      return false;
   else{
      AIOctreeNode* currentNode=nodeStack.back();
      nodeStack.pop_back();
      if (currentNode->isLeaf()){
         currentBox=&(currentNode->getContent().m_index);
         currentBoxPosition=0;

      }else{
         for (card32 i=0;i<8;i++){
            AIOctreeNode* child= currentNode->getChild(i);
            if (m_range->intersects(child->getContent().m_bb))
               nodeStack.push_back(child); //if child is in range put on stack, else dicard
         }
      }
   }
   return true;

}



card8 AttachedIndexedOctreeGeometricIterator::get1ub(const AAT &token) const {
   return genericGetS<card8>(token);
}
Vector2ub AttachedIndexedOctreeGeometricIterator::get2ub(const AAT &token) const {
   return genericGet<card8, 2>(token);
}
Vector3ub AttachedIndexedOctreeGeometricIterator::get3ub(const AAT &token) const {
   return genericGet<card8, 3>(token);
}
Vector4ub AttachedIndexedOctreeGeometricIterator::get4ub(const AAT &token) const {
   return genericGet<card8, 4>(token);
}
int32 AttachedIndexedOctreeGeometricIterator::get1i(const AAT &token) const {
   return genericGetS<int32>(token);
}
Vector2i AttachedIndexedOctreeGeometricIterator::get2i(const AAT &token) const {
   return genericGet<int32, 2>(token);
}
Vector3i AttachedIndexedOctreeGeometricIterator::get3i(const AAT &token) const {
   return genericGet<int32, 3>(token);
}
Vector4i AttachedIndexedOctreeGeometricIterator::get4i(const AAT &token) const {
   return genericGet<int32, 4>(token);
}
float32 AttachedIndexedOctreeGeometricIterator::get1f(const AAT &token) const {
   return genericGetS<float32>(token);
}
Vector2f AttachedIndexedOctreeGeometricIterator::get2f(const AAT &token) const {
   return genericGet<float32, 2>(token);
}
Vector3f AttachedIndexedOctreeGeometricIterator::get3f(const AAT &token) const {
   return genericGet<float32, 3>(token);
}
Vector4f AttachedIndexedOctreeGeometricIterator::get4f(const AAT &token) const {
   return genericGet<float32, 4>(token);
}

void AttachedIndexedOctreeGeometricIterator::getVertex(char *buffer) const {
   forget(buffer);
   pAssert(false);
}



void AttachedIndexedOctreeGeometricIterator::set1ub(const AAT &token, const card8 &x)     const {
   genericSetS<card8>(token, x);
}
void AttachedIndexedOctreeGeometricIterator::set2ub(const AAT &token, const Vector2ub &v) const {
   genericSet<card8, 2>(token, v);
}
void AttachedIndexedOctreeGeometricIterator::set3ub(const AAT &token, const Vector3ub &v) const {
   genericSet<card8, 3>(token, v);
}
void AttachedIndexedOctreeGeometricIterator::set4ub(const AAT &token, const Vector4ub &v) const {
   genericSet<card8, 4>(token, v);
}
void AttachedIndexedOctreeGeometricIterator::set1i(const AAT &token, const int32 &x)    const {
   genericSetS<int32>(token, x);
}
void AttachedIndexedOctreeGeometricIterator::set2i(const AAT &token, const Vector2i &v) const {
   genericSet<int32, 2>(token, v);
}
void AttachedIndexedOctreeGeometricIterator::set3i(const AAT &token, const Vector3i &v) const {
   genericSet<int32, 3>(token, v);
}
void AttachedIndexedOctreeGeometricIterator::set4i(const AAT &token, const Vector4i &v) const {
   genericSet<int32, 4>(token, v);
}
void AttachedIndexedOctreeGeometricIterator::set1f(const AAT &token, const float32 &x)  const {
   genericSetS<float32>(token, x);
}
void AttachedIndexedOctreeGeometricIterator::set2f(const AAT &token, const Vector2f &v) const {
   genericSet<float32, 2>(token, v);
}
void AttachedIndexedOctreeGeometricIterator::set3f(const AAT &token, const Vector3f &v) const {
   genericSet<float32, 3>(token, v);
}
void AttachedIndexedOctreeGeometricIterator::set4f(const AAT &token, const Vector4f &v) const {
   genericSet<float32, 4>(token, v);
}

void AttachedIndexedOctreeGeometricIterator::setVertex(const char *buffer) const {
   forget(buffer);
   pAssert(false);
}




