#ifndef CachedRangeQuerryH
#define CachedRangeQuerryH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------

namespace geoxl
{

class PointCloud;
class HierarchicalKNNIterator;
class PointSetKNNQuery;
class AttachedIndexedOctree;

/**
	cached range querry - uses a cache to avoid unnecessary querries.
	returns full cache! don't mix different querries!!
*/
class POINTCLOUDS_API CachedRangeQuerry
{

public:
	CachedRangeQuerry(PointCloud * pc);
	CachedRangeQuerry(PointSet * ps);
	~CachedRangeQuerry(void);

	/// returns all points in ball (p,radius) and some points in the overlapping zone.
	/// cacheradius = radius + absolute_overlap
	PointSet * getSpherePoints( const Vector3f &p, float radius, float absolute_overlap, int minNumberOfPoints=10 );

	/// returns all positions with |p-x| <= radius + absolute_overlap
	/// 
	vector<Vector3f> * getSpherePositions(const Vector3f &p, float radius, float absolute_overlap, int minNumberOfPoints=10,
		unsigned ** indices = NULL);

	/// returns center point of current cache
	Vector3f getCacheCenter() const {return m_SpherePoint;}

private:

	void setSeekPoint( const Vector3f &point);
	unsigned getCurrentIndex();
	Vector3f getPosition();
	void getVertex( char * buffer );
	void nextPoint();
	bool atEnd();


	PointCloud *	m_PointCloud;
	PointSet		*	m_PointSet;
	HierarchicalKNNIterator * m_hIt;
	AAT				m_PosAAT;

	PointSet *		m_Cache;
	enum ECacheType
	{
		CT_None,
		CT_Sphere,
		CT_SpherePosOnly
	};
	ECacheType		m_CacheType;
	Vector3f			m_SpherePoint;
	float				m_SphereRadius;

	vector<Vector3f> m_PosCache;
	vector<unsigned> m_IndexCache;

	PointSetKNNQuery * m_PSKNNQuerry;
	AttachedIndexedOctree * m_AIOctree;
	card32				m_CurrentIndex;
	bool					m_AtEnd;

};

}

#endif