#ifndef FastSphereQueryH
#define FastSphereQueryH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl
{

class AttachedIndexedOctree;
class PointCloud;
class SecondaryDataStructureIterator;
class SceneObjectIterator;
class BasicPointCloudIterator;

/**
	fast sphere querry - operates on AttachedIndexedOctree
	no ordering is performed
*/
class POINTCLOUDS_API FastSphereQuery
{

private:

	vector<mpcard> m_IndexCache;
	mpcard			m_NumPoints;
	
	AttachedIndexedOctree	* m_Octree;
	SecondaryDataStructureIterator	* m_SDS;
	SceneObjectIterator * m_SceneIter;
	BasicPointCloudIterator * m_BasicIt;

public:
	FastSphereQuery(PointCloud * pc);
	FastSphereQuery(AttachedIndexedOctree * octree);
	~FastSphereQuery(void);

	typedef void (Object::* ProcessPointMethod)( const Vector3f &, float, mpcard );

	/// querries all points in a sphere
	/// indices are valid until next call or destruction of this object
	void querry( const Vector3f &pos, const float radius, mpcard ** indices, mpcard &numPoints );

	/// processes all points in a sphere
	void process( const Vector3f &pos, const float radius, Object * obj, ProcessPointMethod method);

};

}

#endif