//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "HierarchicalIteratorAdaptor.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "PointSetIterator.h"
#include "DebugOutput.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


///Abstract HierarchicalIteratorAdaptor Base Class
//==================================================================================

HierarchicalIteratorAdaptor::HierarchicalIteratorAdaptor
        ( PointCloudIterator* native, const VertexDescriptor *vd)
         :IteratorAdaptor(native, vd)
{

// _________________________________________________

   m_hr = dynamic_cast<HierarchicalIterator*>(native);
   pAssert(  m_hr!=NULL);
   nextChildIndexStack.clear();
   nextChildIndexStack.push_back(0);
   allNodesFinished=false;
// _________________________________________________


}

HierarchicalIteratorAdaptor::~HierarchicalIteratorAdaptor(){
//   debugOutput << "HierarchicalIteratorAdaptor: Destruktor" ;
//   debugOutput.cr();
}


/// go to first point
void HierarchicalIteratorAdaptor::reset(){
   allNodesFinished=false;
   m_base->reset();

   nextChildIndexStack.clear();
   nextChildIndexStack.push_back(0);
   
   if ((m_hr->getNumChildren()>0) && !m_hr->isLeaf())  //m_hr->getNumChildren()==0 means no hr
   {
      if (!nodeMeetsCond()) {
         allNodesFinished = true;
      }else{
         allNodesFinished = !searchNextNode();    // >0 :hr - step to first leaf
      }
   }

   //step to first point
   allNodesFinished = this->atEnd();

}


bool HierarchicalIteratorAdaptor::atEnd() {


   if (allNodesFinished) {
      return true;
   }

   do{
      while (!m_base->atEnd() && !pointMeetsCond() )
          {
             m_base->next();
          }
          if (!m_base->atEnd()) {
             return false;
          }
    }while (searchNextNode());

    // no valid point found
    allNodesFinished = true;
    return true;

}


/// go to next point
void HierarchicalIteratorAdaptor::next() {
   m_base->next();
   this->atEnd();
}


bool HierarchicalIteratorAdaptor::searchNextNode() {

   do {                                      // find next leaf
      do {
         const card32 numChildren = m_hr->getNumChildren();          // how many children
         card32 nextNodeIndexToVisit = nextChildIndexStack.back();   // get next node
         if (nextNodeIndexToVisit >= numChildren) {
            if (!m_hr->goToParent()) {                               //no more child for this node
               return false;
            } else {
               nextChildIndexStack.pop_back();
            }
         } else {
            nextChildIndexStack.back()++;
            if (m_hr->goToChild(nextNodeIndexToVisit)) {  // if child exists

               if (nodeMeetsCond()){                      // and child meets cond.
                  nextChildIndexStack.push_back(0);       // yes, go to children of child
                  break;
               }else{
                  bool noError = m_hr->goToParent();      // if child doesn't meet condition, go back
                  pAssert(noError);
               }

            }
         }
      } while (true);
   } while (!m_hr->isLeaf());    //while current node is not a leaf that MeetsCond()
   pAssert(m_hr->isLeaf());
   return true;
}




// --- blockwise access (faster)
//==================================================================

/// step foreward to next data block, false if end-of-set (affects the same position pointer as "next()")
bool     HierarchicalIteratorAdaptor::nextBlock(){
   //TODO
   pAssert(false); //To be coded
        return next_block;
}

  /// access raw data of current data block{  } call unlock to free{  }
/// one data block is guaranteed to easily fit into main memory

const PointSet*  HierarchicalIteratorAdaptor::lockBlockRead(){
   pAssert(!locked); // first unlock() last block
   locked=true;

   //TODO
   pAssert(false); //To be coded

   tmp_ps= new PointSet();// ( w, h        , descr)


   return tmp_ps;

 }



/// free locked PointSet again
void HierarchicalIteratorAdaptor::unlock(){

   //TODO
   pAssert(false); //To be coded

   delete tmp_ps;
   tmp_ps=NULL;
   locked=false;


}

/// access raw data of current data block for read-write access
/// call unlock to free locked PointSet
PointSet* HierarchicalIteratorAdaptor::lockBlockReadWrite(LockRWHint hint){

   //TODO
   pAssert(false); //To be coded

   write=true;
   pAssert(!locked); // first unlock() last block
   locked=true;
   tmp_ps= new PointSet();

   return tmp_ps;


}




