//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "InCorePCTopologyGraph.h"
#include "VertexDescriptor.h"
#include "PointSetIterator.h"
//---------------------------------------------------------------------------
#include "PointSet.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

IMPLEMENT_GEOXL_CLASS( InCorePCTopologyGraph, 1 ) 
{
	BEGIN_CLASS_INIT( InCorePCTopologyGraph );
}

          
// ---
          
          
InCorePCTopologyGraph::InCorePCTopologyGraph() {
   edges = new PointSet();
	aatVIndex = NULL_AAT;
	tmpBuffer = NULL;
	tmpBuffer2 = NULL;
	tmpBufferSize = 0;
}

void InCorePCTopologyGraph::assign(const Object* obj, CopyContext *context) {
	const InCorePCTopologyGraph *other = dynamic_cast<const InCorePCTopologyGraph*>(obj);
	pAssert(other);
	delete edges;
	edges = (PointSet*)(other->edges->copy());
	edgeTable = other->edgeTable;
	aatVIndex = other->aatVIndex;	
	tmpBufferSize = other->tmpBufferSize;
	delete[] tmpBuffer;
	delete[] tmpBuffer2;
	tmpBuffer = new char[tmpBufferSize];
	tmpBuffer2 = new char[tmpBufferSize];
}

void InCorePCTopologyGraph::write(OutputObjectStream *s) const {
	s->writeObject(edges);
	mpcard numEdges = edgeTable.size();
	s->writeMaxPlatformCard(numEdges);
	for (mpcard i=0; i<numEdges; i++) {
		mpcard numNeighbors = edgeTable[i].size();
		s->writeMaxPlatformCard(numNeighbors);
		for (mpcard j=0; j<numNeighbors; j++) {
			s->write(edgeTable[i][j]);
		}
	}
}

void InCorePCTopologyGraph::read(InputObjectStream *s) {
	if (s->getClassVersion(getClass()) >= MetaClass::FIRST_VERSION + 1) {
		s->readAnyObject(edges);
		mpcard numEdges;
		if (edges != NULL) {
			aatVIndex = edges->getDescr()->getToken("vindex");	
			tmpBufferSize = edges->getDescr()->getSize();
			delete[] tmpBuffer;
			delete[] tmpBuffer2;
			tmpBuffer = new char[tmpBufferSize];
			tmpBuffer2 = new char[tmpBufferSize];
		}
		s->readMaxPlatformCard(numEdges);
		edgeTable.resize(numEdges);
		for (mpcard i=0; i<numEdges; i++) {
			mpcard numNeighbors;
			s->readMaxPlatformCard(numNeighbors);
			 edgeTable[i].resize(numNeighbors);
			for (mpcard j=0; j<numNeighbors; j++) {
				s->read(edgeTable[i][j]);
			}
		}
	}
}


void InCorePCTopologyGraph::clearAndSetup(const VertexDescriptor *vd) {
	if (!vd->providesAttribute("vindex", 2, VAD::DATA_FORMAT_INT32)) {
		throw PException("InCorePCTopologyGraph::clearAndSetup - VertexDescriptor must contain attribute int32 vindex[2].");
	}
	aatVIndex = vd->getToken("vindex");
	edges->clearAndSetup(1,0,vd);
	edgeTable.clear();
	tmpBufferSize = vd->getSize();
	delete[] tmpBuffer;
	delete[] tmpBuffer2;
	tmpBuffer = new char[tmpBufferSize];
	tmpBuffer2 = new char[tmpBufferSize];
}

void InCorePCTopologyGraph::addEdge(VertexIndex v1, VertexIndex v2) 
{
	/*
	const VertexDescriptor *vd = edges->getDescr();
	pAssert(vd->providesAttribute("vindex", 2, VAD::DATA_FORMAT_INT32));
	char *data = new char[vd->getSize()];
	vd->zero(data);
	AAT AAT_VINDEX = vd->getToken("vindex");
	Vector2i indices = makeVector2i(v1, v2);
	AAT_VINDEX.setAttrib<int32, 2>(data, indices);
	addEdge(data);
	delete[] data;
	*/

	  memset(tmpBuffer, 0, tmpBufferSize);
	  Vector2i indices = makeVector2i(v1, v2);
	  aatVIndex.setAttrib<int32, 2>(tmpBuffer, indices);
	  addEdge(tmpBuffer);
}

/**
 * Remove connection 'index' from the vertex in the graph
 */
void InCorePCTopologyGraph::deleteEdge( VertexIndex v, const card32 index )
{
	if ( ( v >= edgeTable.size() ) || ( index >= edgeTable[v].size() ) ) {
		debugOutput << "deleteEdge: invalid parameters.\n"; debugOutput.flush();
	} else {
		edgeTable[v].erase( edgeTable[v].begin() + index );
	}
}

/**
 * Remove all connection for a vertex in the graph
 */
void InCorePCTopologyGraph::clearVertex( VertexIndex v )
{
	if ( v >= edgeTable.size() ) {
		debugOutput << "clearEdge: invalid vertex index.\n"; debugOutput.flush();
	} else {
		edgeTable[v].clear();
	}
}

void InCorePCTopologyGraph::addEdge(const char *data) {
   Vector2i indices = aatVIndex.getAttrib<int32,2>(data);
   //Vector2i indices = *((Vector2i*)data+aatVIndex.getOffset());

   if ((int32)edgeTable.size() <= indices[0]) edgeTable.resize(indices[0]+1);
   if ((int32)edgeTable.size() <= indices[1]) edgeTable.resize(indices[1]+1);

	VertexIndex n = (VertexIndex)(edgeTable[indices[0]].size());

	for (VertexIndex i=0; i<n; i++) {
		//edges->getVertex(edgeTable[indices[0]][i], tmpBuffer2);
      //Vector2i edgeIndices = aatVIndex.getAttrib<int32,2>(tmpBuffer2);	
		const char *ptr = edges->getDataPointer(edgeTable[indices[0]][i]);
		Vector2i edgeIndices = *((Vector2i*)ptr+aatVIndex.getOffset());	
		if (   (edgeIndices[0] == indices[0] && edgeIndices[1] == indices[1])
		    || (edgeIndices[1] == indices[0] && edgeIndices[0] == indices[1])) {
			// first case: edge already present - update data
			edges->setVertex(edgeTable[indices[0]][i], data);
			return;
		}
	}
	
	// second case: edge not found - create new one
	edges->addPoint(data);
	EdgeIndex e = edges->getNumEntries() - 1;
	edgeTable[indices[0]].push_back(e);
	edgeTable[indices[1]].push_back(e);
	return;
}

/** 
* Create copy of topology graph
*/
InCorePCTopologyGraph* InCorePCTopologyGraph::copy() const
{
	InCorePCTopologyGraph* tg = new InCorePCTopologyGraph();
	delete tg->edges;
	tg->edges = (PointSet*)(edges->copy());
	tg->edgeTable = edgeTable;
	tg->aatVIndex = aatVIndex;

	// Added by pj
	tg->tmpBufferSize = tmpBufferSize;
	tg->tmpBuffer = new char[tmpBufferSize];
	tg->tmpBuffer2 = new char[tmpBufferSize];
	
	return tg;
}


vector<InCorePCTopologyGraph::EdgeIndex> InCorePCTopologyGraph::getEdgeIndices(VertexIndex v) const {
	if ( v >= edgeTable.size() ) {
		return vector<EdgeIndex>();
	} else {
		return edgeTable[v];
	}
}

InCorePCTopologyGraph::EdgeIndex InCorePCTopologyGraph::getEdgeIndex(VertexIndex v, VertexIndex i) const {
	pAssert(edgeTable.size() > v);
	pAssert(edgeTable[v].size() > i);
	return edgeTable[v][i];
}

InCorePCTopologyGraph::EdgeIndex InCorePCTopologyGraph::getNumEdges() const {
	return edges->getNumEntries();
}


InCorePCTopologyGraph::VertexIndex InCorePCTopologyGraph::getNumAdjacentVertices(VertexIndex v) const {
	if (v >= edgeTable.size()) {
		return 0;
	} else {
		return (VertexIndex)(edgeTable[v].size());
	}
}

InCorePCTopologyGraph::VertexIndex InCorePCTopologyGraph::getAdjacentVertex(VertexIndex v, VertexIndex i) const {
	pAssert(v < edgeTable.size());
	pAssert(i < edgeTable[v].size());
	EdgeIndex e = edgeTable[v][i];
	Vector2i vindex = edges->get<int32, 2>(e, aatVIndex);
	if (vindex[0] == v) return vindex[1]; else return vindex[0];
}

	
PointCloudIterator *InCorePCTopologyGraph::createIterator(PointSet *ps) {
   InCorePCTopologyGraphPointSetIterator *result = new InCorePCTopologyGraphPointSetIterator(
      ps, NULL, this );

   return result;
}

void InCorePCTopologyGraph::appendFields(const VertexDescriptor *newAttribs) {
	PointSet *newPs = edges->appendFields(newAttribs);
	if (newPs != NULL) {
		delete edges;
		edges = newPs;
      aatVIndex = edges->getDescr()->getToken("vindex");	
	}
}



InCorePCTopologyGraph::~InCorePCTopologyGraph() {
   delete edges;
   delete[] tmpBuffer;
   delete[] tmpBuffer2;
}


// --



InCorePCTopologyGraphPointSetIterator::InCorePCTopologyGraphPointSetIterator(PointSet *ps, AttachedDataContainer *attData, 
	InCorePCTopologyGraph * graph)
	: AttachedDataPointSetIterator(ps, attData)
{
	m_Graph = graph;
}

InCorePCTopologyGraphPointSetIterator::~InCorePCTopologyGraphPointSetIterator()
{
}

card32 InCorePCTopologyGraphPointSetIterator::getNumNeighbours()
{
	return m_Graph->getNumAdjacentVertices( getCurrentPointIndex() );
}

/// returns an index of a neighbour point
PointIndex InCorePCTopologyGraphPointSetIterator::getNeighbour( card32 nIndex )
{
	return m_Graph->getAdjacentVertex( getCurrentPointIndex(), nIndex );
}
