//---------------------------------------------------------------------------
#ifndef MultiResolutionInCoreVolumeH
#define MultiResolutionInCoreVolumeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "InCoreVolume.h"
#include "PointSetIterator.h"
//---------------------------------------------------------------------------


namespace geoxl {

class RegularVolume;
class AttachedDataContainer;


///
///  multi-resolution volume object (regulary sampled);
///  provides pyramid of downsampled volume grids
///
class POINTCLOUDS_API MultiResolutionInCoreVolume : public InCoreVolume {
	GEOXL_CLASS( MultiResolutionInCoreVolume )
 private:

   /// list of resolution levels; decreasing resolution
   vector<RegularVolume*> voxels;
   vector<AttachedDataContainer*> hierarchyAttachments;

	IMPLEMENT_VAR_ARRAY_METHODS_STL(voxels)
	IMPLEMENT_VAR_ARRAY_METHODS_STL(hierarchyAttachments)

 public:
	MultiResolutionInCoreVolume();

   virtual void assign(const Object *obj, CopyContext *context = NULL);

   virtual SceneObjectIterator *createIterator(SceneObjectIteratorType type);
   virtual bool                 supportsIteratorType(SceneObjectIteratorType cap);

   virtual const VertexDescriptor *getDescr() const;
   virtual PointCloud             *appendFields(const VertexDescriptor *newAttribs) const;
   virtual card64                  getNumPoints();
   virtual void                    clear();

   virtual void clearAndSetup(card32 width, card32 height, card32 depth, const VertexDescriptor *descr);
   virtual void clearAndSetup(card32 width, card32 height, card32 depth, const VertexDescriptor *descr, const BoundingBox3f &bb);
   virtual const RegularVolume *getVoxels() const;
   virtual void setVoxels(RegularVolume *v);
   virtual RegularVolume *getVoxels();
   
   card32 getNumMRLevels();
   RegularVolume *getDownsampled(card32 i);
   void recreateMRLevels();
   

	~MultiResolutionInCoreVolume();
};

class POINTCLOUDS_API MultiResolutionInCoreVolumeIterator : public AttachedDataPointSetIterator,
                                                            public VolumetricIterator {
   GEOXL_CLASS( MultiResolutionInCoreVolumeIterator )
   
 private:
   vector<RegularVolume*> *voxels;
   vector<AttachedDataContainer*> *attData;
   card32 currentLevel;
   bool lockedReadWrite;

   MultiResolutionInCoreVolumeIterator() {voxels = NULL; attData = NULL; currentLevel = 0; lockedReadWrite = false;}

 public:
   MultiResolutionInCoreVolumeIterator(vector<RegularVolume*> *voxels, vector<AttachedDataContainer*> *attData);

   /// SecondaryDataStructureIterator
   virtual AttachedDataContainer *lockAttachments();
	virtual void unlockAttachments();

   /// HierarchicalIterator
   virtual card32 getNumChildren();
   virtual bool goToChild(card32 index);
   virtual card32 getCurrentLevel(void);
   virtual bool goToParent();
   virtual BoundingBox3f getBoundingBox();
   virtual bool isLeaf() const;
   virtual bool getNodeInfo(NodeInfoType whichInfo, const void *parameter, void *data);

   /// RegularVolumeIterator
   virtual BoundingBox3f getVolumeRegion();
	virtual void goTo(card32 x, card32 y, card32 z);
	virtual Vector3i getVolumeSize();
	virtual const RegularVolume *lockVolumeBlockRead();
	virtual RegularVolume *lockVolumeBlockReadWrite();
	virtual void unlockVolume();

	virtual ~MultiResolutionInCoreVolumeIterator();
};


}

#endif