//---------------------------------------------------------------------------
#ifndef PCTopologyNeighborhoodIteratorH
#define PCTopologyNeighborhoodIteratorH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PointCloudIterators.h"
//---------------------------------------------------------------------------

namespace geoxl {

class InCorePCTopologyGraph;
class PointSet;


/// iterator that iterates through vertices adjacent to some other vertex through topology graph edges
class POINTCLOUDS_API PCTopologyNeighborhoodIterator : public PointCloudIterator, 
																		 public BasicPointCloudIterator,
																		 public IndexedPointCloudIterator,
																		 public RandomAccessPointCloudIterator 
{
	GEOXL_CLASS(PCTopologyNeighborhoodIterator)
	
 private:
	InCorePCTopologyGraph *tg;
	PointSet *ps;
	
	card32 currentPoint;
	card32 currentEdge;
	card32 numPoints;
	
	PCTopologyNeighborhoodIterator();

 public:
	/// construct iterator from a neighborhood graph and a point set storing the vertices
	PCTopologyNeighborhoodIterator(InCorePCTopologyGraph *tg, PointSet *ps);
	
	/// go to a vertex
	void goToVertex(card32 v);
	/// get current vertex index
	card32 getCurrentVertex() const;
	/// get vertex index adjacent to current vertex
	card32 getCurrentAdjacentVertex() const;
	/// go to the next vertex in the point set
	void nextVertex();
	/// true if behind last vertex
	bool atEndOfVertices() const;
	
	/// go to first adjacent edge of current vertex
   virtual void reset();
   /// go to next adjacent vertex
   virtual void next();
   /// no more adjacent vertices?
   virtual bool atEnd();
   
   // -- adjacent vertices attribute access
   virtual card8     get1ub(const AAT &token) const;
   virtual Vector2ub get2ub(const AAT &token) const;
   virtual Vector3ub get3ub(const AAT &token) const;
   virtual Vector4ub get4ub(const AAT &token) const;

   virtual int32    get1i(const AAT &token) const;
   virtual Vector2i get2i(const AAT &token) const;
   virtual Vector3i get3i(const AAT &token) const;
   virtual Vector4i get4i(const AAT &token) const;

   virtual float32  get1f(const AAT &token) const;
   virtual Vector2f get2f(const AAT &token) const;
   virtual Vector3f get3f(const AAT &token) const;
   virtual Vector4f get4f(const AAT &token) const;

   virtual void     getVertex(char *buffer) const;

	/// no effect, returns always false
   virtual bool nextBlock();
   /// locks the point set
   virtual const PointSet * lockBlockRead();
   /// currently no effect
   virtual void unlock();

	/// get number of points in point set (not number of edges);
   virtual card64 getNumPointsInBlock() const; 
   /// get index of current point, same result as "getCurrentVertex()"
   virtual card64 getCurrentPointIndex() const;
   /// go to a certain vertex index (same as goToVertex())
   virtual void goToIndex(card64 i);
	
};


}

#endif