//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "ParametricInCorePointCloud.h"
//---------------------------------------------------------------------------
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "PointSetIterator.h"
#include "ObjectClassProperty.h"
#include "AttachedData.h"
#include "VertexArray.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* ParametricInCorePointCloud::ParametricInCorePointCloudMetaClass = NULL;

const MetaClass* ParametricInCorePointCloud::getInstanceClass() const {
   return getClass();
}

Object* ParametricInCorePointCloud::newInstance() {
   return new ParametricInCorePointCloud;
}

const MetaClass* ParametricInCorePointCloud::getClass() {
   if (ParametricInCorePointCloudMetaClass==NULL) {
      ParametricInCorePointCloudMetaClass = new MetaClass("ParametricInCorePointCloud", "no description.",
                                  newInstance, sizeof(ParametricInCorePointCloud));
   }
   return ParametricInCorePointCloudMetaClass;
}

void ParametricInCorePointCloud::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   ParametricInCorePointCloudMetaClass->copyParentProperties();
/*   ParametricInCorePointCloudMetaClass->addProperty(new ObjectClassProperty("ps", 1, (ObjectElementPointer)&ps, PointSet::getClass(), true));
   ObjectClassProperty *dsprop = new ObjectClassProperty("hierarchyAttachedDSs", MetaClass::FIRST_VERSION + 1,
      (ObjectElementPointer)&hierarchyAttachedDSs, AttachedDataContainer::getClass(), true);
   ParametricInCorePointCloudMetaClass->addProperty(dsprop);*/

	BEGIN_CLASS_INIT( ParametricInCorePointCloud );
ADD_OBJECT_PROP( ps, 0, PointSet::getClass(), true );
ADD_OBJECT_PROP( hierarchyAttachedDSs, 0, AttachedDataContainer::getClass(), true );
}

void ParametricInCorePointCloud::shutdown() {
   delete ParametricInCorePointCloudMetaClass;
}


// ----


MetaClass* BasicParametricInCorePointCloudIterator::BasicParametricInCorePointCloudIteratorMetaClass = NULL;

const MetaClass* BasicParametricInCorePointCloudIterator::getInstanceClass() const {
   return getClass();
}

Object* BasicParametricInCorePointCloudIterator::newInstance() {
   return new BasicParametricInCorePointCloudIterator;
}

const MetaClass* BasicParametricInCorePointCloudIterator::getClass() {
   if (BasicParametricInCorePointCloudIteratorMetaClass==NULL) {
      BasicParametricInCorePointCloudIteratorMetaClass = new MetaClass("BasicParametricInCorePointCloudIterator", "no description.",
                                                                       newInstance, sizeof(BasicParametricInCorePointCloudIterator));
   }
   return BasicParametricInCorePointCloudIteratorMetaClass;
}

void BasicParametricInCorePointCloudIterator::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   BasicParametricInCorePointCloudIteratorMetaClass->copyParentProperties();
}

void BasicParametricInCorePointCloudIterator::shutdown() {
   delete BasicParametricInCorePointCloudIteratorMetaClass;
}


// ----



ParametricInCorePointCloud::ParametricInCorePointCloud() {
   ps = new PointSet();
   hierarchyAttachedDSs = new AttachedDataContainer();
}

void ParametricInCorePointCloud::assign(const Object* obj, CopyContext *context) {
   PointCloud::assign(obj, context);
   const ParametricInCorePointCloud *pobj = dynamic_cast<const ParametricInCorePointCloud*>(obj);
   pAssert(pobj != NULL);
   if (pobj->ps == NULL) {
      delete ps;
      ps = NULL;
   } else if (ps != pobj->ps) {
      delete ps;
      ps = (PointSet*)(pobj->ps)->copy(context);
   }
   this->hierarchyAttachedDSs = (AttachedDataContainer*)(pobj->hierarchyAttachedDSs->copy(context));
}

const VertexDescriptor *ParametricInCorePointCloud::getDescr() const {
   return ps->getDescr();
}

PointCloud *ParametricInCorePointCloud::appendFields(const VertexDescriptor *newAttribs) const {
   return createCopyWithNewPointSet(ps->appendFields(newAttribs));
}

PointCloudIterator *ParametricInCorePointCloud::createIterator(SceneObjectIteratorType type) {
   if (!supportsIteratorType(type)) {
      return NULL;
   } else {
      return new BasicParametricInCorePointCloudIterator(ps, hierarchyAttachedDSs);
   }
}

bool ParametricInCorePointCloud::supportsIteratorType(SceneObjectIteratorType cap) {
   SOIT possibleCaps(  SOIT::CAP_BASIC_PC | SOIT::CAP_INDEXED_PC | SOIT::CAP_RANDOM_ACCESS_PC | SOIT::CAP_MODIFYABLE_PC
                     | SOIT::CAP_HIERARCHICAL | SOIT::CAP_ATTACH_SECONDARY
                     | SOIT::CAP_PARAMETRIC_PC | SOIT::CAP_DYNAMIC_PARAMETRIC_PC);
   return (possibleCaps.providesCapsFor(cap));
}

card64 ParametricInCorePointCloud::getNumPoints() {
   if (ps == NULL) return 0;
   return ps->getNumEntries();
}

void ParametricInCorePointCloud::clear() {
   ps->clear();
}


void ParametricInCorePointCloud::clearAndSetup(card32 width, card32 height, const VertexDescriptor *vertDescr) {
   ps->clearAndSetup(width, height, vertDescr);
}


ParametricInCorePointCloud *ParametricInCorePointCloud::createCopyWithNewPointSet(PointSet *ps) const {
   PointSet *pscopy = this->ps;
   ((ParametricInCorePointCloud*)this)->ps = NULL;
   ParametricInCorePointCloud *result = (ParametricInCorePointCloud*)copy();
   ((ParametricInCorePointCloud *)this)->ps = pscopy;
   result->ps = ps;
   return result;
}

void ParametricInCorePointCloud::clearAttachments() {
   PointCloud::clearAttachments();
   hierarchyAttachedDSs->clear();
}
                    
ParametricInCorePointCloud::~ParametricInCorePointCloud() {
   delete ps;
   delete hierarchyAttachedDSs;
}


// --

BasicParametricInCorePointCloudIterator::BasicParametricInCorePointCloudIterator(PointSet *ps, AttachedDataContainer *attData)
 : AttachedDataPointSetIterator(ps, attData)
{}




void ParametricInCorePointCloud::invalidateSelectedPoints()
{
		VertexArray vertArray( ps );
		card32 num= vertArray.getNumElements();
		for (card32 i=0; i<num; i++ )
		{
			int32 flags=vertArray.getFlags(i);
			if (flags & PF_FLAG_SELECTED)
			{
				int32 newFlags= flags & (0xFFFFFFFF ^ PF_FLAG_SELECTED); //deselect
				newFlags= newFlags | PF_FLAG_INVALID;  //set invalid
				vertArray.setPosition3f(i,makeVector3f(0,0,0)); //set position to 0
				vertArray.setFlags(i,newFlags);
			}
		}

		ps->invalidateLastNumFlagged();
		clearAttachments();
}