//---------------------------------------------------------------------------
#ifndef PointCloudH
#define PointCloudH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "VertexDescriptor.h"
#include "SceneObject.h"
#include "PointCloudIterators.h"
//---------------------------------------------------------------------------


namespace geoxl {
   using namespace std;



/// A collection of points with attributes in 3-space.
/// Data access via iterators (createIterator).
class POINTCLOUDS_API PointCloud : public SceneObject {
 private:
   static MetaClass *PointCloudMetaClass;
   static Object* newInstance();

 public:
   PointCloud();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   // new methods (abstract)
   /// retrieve the per-point attribute description
   virtual const VertexDescriptor *getDescr() const = 0;
   /// append additional attributes to the point - attention, this method creates a _copy_.
   virtual PointCloud             *appendFields(const VertexDescriptor *newAttribs) const = 0;
   /// get number of points in cloud
   virtual card64                  getNumPoints() = 0;
   /// throw away all points, keep settings
   virtual void                    clear() = 0;

   // macros
   /// macro - get attribute access token by name
   inline AttributeAccessToken getAAT(string name) const {
      return getDescr()->getToken(name);
   }
   /// macro - check for attribute by name
   inline bool providesAttribute(string name) const {
      return getDescr()->providesAttribute(name);
   }
   /// macro - are there any points in here?
   inline virtual bool isEmpty() {return getNumPoints() == 0;}

   virtual ~PointCloud();
};


}

#endif
 