//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "PointSetIterator.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "AttachedData.h"
#include "Tools.h"
#include "LinearAlgebra_templCode.h"
#include "NumericalClassProperty.h"
#include "HierarchicalKNNIterator.h"
#include "ProgressWindow.h"
//---------------------------------------------------------------------------
#include "DebugOutput.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* PointSetIterator::PointSetIteratorMetaClass = NULL;

const MetaClass* PointSetIterator::getInstanceClass() const {
   return getClass();
}

Object* PointSetIterator::newInstance() {
   return new PointSetIterator;
}

const MetaClass* PointSetIterator::getClass() {
   if (PointSetIteratorMetaClass==NULL) {
      PointSetIteratorMetaClass = new MetaClass("PointSetIterator", "no description.",
                                  newInstance, sizeof(PointSetIterator));
   }
   return PointSetIteratorMetaClass;
}

void PointSetIterator::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   PointSetIteratorMetaClass->copyParentProperties();
}

void PointSetIterator::shutdown() {
   delete PointSetIteratorMetaClass;
}

IMPLEMENT_GEOXL_CLASS(FixedKNNTopologyAttachment, 0)
{
	BEGIN_CLASS_INIT(FixedKNNTopologyAttachment);

	ADD_CARD32_VAR_ARRAY_PROP( m_Graph,0);
}

// ----

PointSetIterator::PointSetIterator(PointSet *ps) {
   initSetup(ps);
}

void PointSetIterator::initSetup(PointSet *ps) {
   this->ps = ps;
   x = 0;
   y = 0;
   width = ps->getWidth();
   height = ps->getHeight();
   attachedDataLargeChange = false;
   attachedDataAttribChange = false;
   aatPosition = NULL_AAT;
   if (ps->getDescr()->providesAttribute("position")) {
      VAD vad = ps->getDescr()->getAttribute("position");
      if (vad.numComponents == 3 && vad.dataFormat == VAD::DATA_FORMAT_FLOAT32) {
         aatPosition = ps->getAAT("position");
      }
   }
}

void PointSetIterator::retarget(PointSet *ps) {
   initSetup(ps);
}

void PointSetIterator::reset() {
   x = 0; y = 0;
}

void PointSetIterator::next() {
   x++;
   if (x >= width) {
      x = 0;
      y++;
   }
}
														
card64 PointSetIterator::getNumPointsInBlock() const {
	return width * height;	
}

card64 PointSetIterator::getCurrentPointIndex() const {
	return x+width*y;
}

void PointSetIterator::goToIndex(card64 i) {
	pAssert(width > 0);
	if (width <= 1) {
		y = (card32)i;
	} else {
		y = ((card32)i) / width;
		x = ((card32)i) % width;
	}
	if (x >= width) x = width - 1;
	if (y >= height) y = height - 1;
}													 

bool PointSetIterator::atEnd() {

//   if (x >= stopw && y >= stoph) {

   if (y >= height) {  //ab - fixed "last point" bug
      return true;
   }
   return false;
}



template <class Type, int dim>
inline StaticVector<Type, dim> PointSetIterator::genericGet(AAT token) const {
   #ifdef _DEBUG
   pAssert(sizeof(StaticVector<Type, dim>) == getAATSize(token));
   #endif
   return ps->get<Type, dim>(x, y, token);
}

template <class Type>
inline Type PointSetIterator::genericGetS(AAT token) const {
   #ifdef _DEBUG
   pAssert(sizeof(Type) == getAATSize(token));
   #endif
   const StaticVector<Type, 1> v = ps->get<Type, 1>(x, y, token);
   return v[0];
}

template <class Type, int dim>
inline void PointSetIterator::genericSet(AAT token, StaticVector<Type, dim> v) const {
   #ifdef _DEBUG
   pAssert(sizeof(StaticVector<Type, dim>) == getAATSize(token));
   #endif
   ps->set<Type, dim>(x, y, token, v);
   if (token == aatPosition) {
      attachedDataLargeChange = true;
   } else {
      attachedDataAttribChange = true;
   }
   ps->invalidateLastNumFlagged();
}

template <class Type>
inline void PointSetIterator::genericSetS(AAT token, Type v) const {
   #ifdef _DEBUG
   pAssert(sizeof(Type) == getAATSize(token));
   #endif
   StaticVector<Type, 1> v1; v1[0] = v;
   ps->set<Type, 1>(x, y, token, v1);
   if (token == aatPosition) {
      attachedDataLargeChange = true;
   } else {
      attachedDataAttribChange = true;
   }
   ps->invalidateLastNumFlagged();
}

card8 PointSetIterator::get1ub(const AAT &token) const {
   return genericGetS<card8>(token);
}
Vector2ub PointSetIterator::get2ub(const AAT &token) const {
   return genericGet<card8, 2>(token);
}
Vector3ub PointSetIterator::get3ub(const AAT &token) const {
   return genericGet<card8, 3>(token);
}
Vector4ub PointSetIterator::get4ub(const AAT &token) const {
   return genericGet<card8, 4>(token);
}
int32 PointSetIterator::get1i(const AAT &token) const {
   return genericGetS<int32>(token);
}
Vector2i PointSetIterator::get2i(const AAT &token) const {
   return genericGet<int32, 2>(token);
}
Vector3i PointSetIterator::get3i(const AAT &token) const {
   return genericGet<int32, 3>(token);
}
Vector4i PointSetIterator::get4i(const AAT &token) const {
   return genericGet<int32, 4>(token);
}
float32 PointSetIterator::get1f(const AAT &token) const {
   return genericGetS<float32>(token);
}
Vector2f PointSetIterator::get2f(const AAT &token) const {
   return genericGet<float32, 2>(token);
}
Vector3f PointSetIterator::get3f(const AAT &token) const {
   return genericGet<float32, 3>(token);
}
Vector4f PointSetIterator::get4f(const AAT &token) const {
   return genericGet<float32, 4>(token);
}

void PointSetIterator::getVertex(char *buffer) const {
   ps->getVertex(x, y, buffer);
}

bool PointSetIterator::nextBlock() {
   return false;
}
const PointSet *PointSetIterator::lockBlockRead() {
   return ps;
}
void PointSetIterator::unlock() {}

void PointSetIterator::set1ub(const AAT &token, const card8 &x)     const {
   genericSetS<card8>(token, x);
}
void PointSetIterator::set2ub(const AAT &token, const Vector2ub &v) const {
   genericSet<card8, 2>(token, v);
}
void PointSetIterator::set3ub(const AAT &token, const Vector3ub &v) const {
   genericSet<card8, 3>(token, v);
}
void PointSetIterator::set4ub(const AAT &token, const Vector4ub &v) const {
   genericSet<card8, 4>(token, v);
}
void PointSetIterator::set1i(const AAT &token, const int32 &x)    const {
   genericSetS<int32>(token, x);
}
void PointSetIterator::set2i(const AAT &token, const Vector2i &v) const {
   genericSet<int32, 2>(token, v);
}
void PointSetIterator::set3i(const AAT &token, const Vector3i &v) const {
   genericSet<int32, 3>(token, v);
}
void PointSetIterator::set4i(const AAT &token, const Vector4i &v) const {
   genericSet<int32, 4>(token, v);
}
void PointSetIterator::set1f(const AAT &token, const float32 &x)  const {
   genericSetS<float32>(token, x);
}
void PointSetIterator::set2f(const AAT &token, const Vector2f &v) const {
   genericSet<float32, 2>(token, v);
}
void PointSetIterator::set3f(const AAT &token, const Vector3f &v) const {
   genericSet<float32, 3>(token, v);
}
void PointSetIterator::set4f(const AAT &token, const Vector4f &v) const {
   genericSet<float32, 4>(token, v);
}

void PointSetIterator::setVertex(const char *buffer) const {
   ps->setVertex(x, y, buffer);
}

PointSet *PointSetIterator::lockBlockReadWrite(LockRWHint hint) {
   attachedDataLargeChange = true;
   attachedDataAttribChange = true;
   ps->invalidateLastNumFlagged();
   return ps;
}


//----------


void PointSetIterator::setSize(card32 width, card32 height) {
   ps->clearAndSetup(width, height,ps->getDescr());
   attachedDataLargeChange = true;
   attachedDataAttribChange = true;
   ps->invalidateLastNumFlagged();
   this->width = width;
   this->height = height;
   this->x = 0;
   this->y = 0;
}


//----------


void PointSetIterator::goTo(int32 x, int32 y) {
   if (x<0) x = 0; if (x>=(int32)width) x=width-1;
   if (y<0) y = 0; if (y>=(int32)height) y=height-1;
   this->x = x;
   this->y = y;
}

card32 PointSetIterator::getWidth() {
   return width;
}

card32 PointSetIterator::getHeight() {
   return height;
}

card32 PointSetIterator::getNumChildren() {
   return 0;
}
bool PointSetIterator::goToChild(card32 index) {
   forget(index);
   return false;
}
bool PointSetIterator::goToParent() {
   return false;
}

BoundingBox3f PointSetIterator::getBoundingBox() {
   return ps->getBoundingBox();
}

bool PointSetIterator::isLeaf() const {
   return true;
}

bool PointSetIterator::getNodeInfo(NodeInfoType whichInfo, const void *parameter, void *data) {
   switch (whichInfo) {
    case niPointSpacing:
      *((float32*)data) = 0.0f;
    return true;

    case niFlaggedInSubtree:
      *((card64*)data) = ps->getNumFlagged(*((card32*)parameter));
    return true;

    case niNumPointsInNode:
      *((card64*)data) = ps->getNumEntries();
    return true;

    case niNumPointsInSubtree:
      *((card64*)data) = ps->getNumEntries();
    return true;

    default:
    return false;
   };
}


//---------------------


AttachedDataPointSetIterator::AttachedDataPointSetIterator(PointSet *ps, AttachedDataContainer *attData)
 : PointSetIterator(ps)
{
   this->attData = attData;
   attachedDataLargeChange = false;
   attachedDataAttribChange = false;
}

void AttachedDataPointSetIterator::retarget(PointSet *ps, AttachedDataContainer *attData) {
   if (attachedDataLargeChange || attachedDataAttribChange) {
      this->attData->deleteDelOnWriteData(attachedDataLargeChange ? AttachedData::CT_EVERYTHING_CHANGED : AttachedData::CT_ATTRIBUTE_CHANGED);
   }
   PointSetIterator::retarget(ps);
   this->attData = attData;
}


AttachedDataContainer *AttachedDataPointSetIterator::lockAttachments() {
   if (attachedDataLargeChange || attachedDataAttribChange) {
      attData->deleteDelOnWriteData(attachedDataLargeChange ? AttachedData::CT_EVERYTHING_CHANGED : AttachedData::CT_ATTRIBUTE_CHANGED);
   }
   attachedDataLargeChange = false;
   attachedDataAttribChange = false;
	return attData;
}

void AttachedDataPointSetIterator::unlockAttachments() {
   if (attachedDataLargeChange || attachedDataAttribChange) {
      attData->deleteDelOnWriteData(attachedDataLargeChange ? AttachedData::CT_EVERYTHING_CHANGED : AttachedData::CT_ATTRIBUTE_CHANGED);
   }
   attachedDataLargeChange = false;
   attachedDataAttribChange = false;
}

AttachedDataPointSetIterator::~AttachedDataPointSetIterator() {
   if (attachedDataLargeChange || attachedDataAttribChange) {
      attData->deleteDelOnWriteData(attachedDataLargeChange ? AttachedData::CT_EVERYTHING_CHANGED : AttachedData::CT_ATTRIBUTE_CHANGED);
   }
   attachedDataLargeChange = false;
   attachedDataAttribChange = false;
}



FixedKNNTopologyIterator::FixedKNNTopologyIterator(PointSet *ps, AttachedDataContainer *attData,
																	 PointCloud * pc, card32 numNeighbours)
:AttachedDataPointSetIterator(ps, attData)
{
	m_NumNeighbours = numNeighbours;
	m_PointCloud = pc;
	m_LockedAttachments = lockAttachments();
	calculate();
}

FixedKNNTopologyIterator::~FixedKNNTopologyIterator()
{
	unlockAttachments();
}

void FixedKNNTopologyIterator::calculate()
{
	m_Graph = dynamic_cast<FixedKNNTopologyAttachment*>( m_LockedAttachments->getData( "FixedKNNTopology" ) );
	if( m_Graph == NULL )
	{
		m_Graph = new FixedKNNTopologyAttachment;
		m_Graph->setup( "FixedKNNTopology", AttachedData::ADF_DELETEONUPDATE );
		m_LockedAttachments->attachData( m_Graph );
	
		m_Graph->resize(m_NumNeighbours * (card32)this->getNumPointsInBlock());

		HierarchicalKNNIterator * hIt = new HierarchicalKNNIterator( m_PointCloud, 32, NULL );
		AAT posAAT = m_PointCloud->getAAT( "position" );

		progressWindow->pushStep( true, "topology" );
		reset();

		for( mpcard i=0;i<m_PointCloud->getNumPoints();i++ )
		{
			Vector3f pos = get3f( posAAT );
			hIt->setSeekPointAndReset( pos );
			
			card32 pointIndex = i;
			for( card32 j=0;j<m_NumNeighbours; )
			{
				hIt->next();
				card32 neighbourIndex = hIt->getPointSetPointNumber();
				if( neighbourIndex == pointIndex )
					continue;

				// force minimum distance
				Vector3f nPos = hIt->get3f( posAAT );
				if( (pos - nPos).getSqrNorm() < 1e-30f )
					continue;

				(*m_Graph)[ m_NumNeighbours * pointIndex + j] = neighbourIndex;

				j++;

			}
			
			next();
			progressWindow->progress( (card32)((float)i / (float)m_PointCloud->getNumPoints() * 100.0f) );
		}

		bool aborted;
		progressWindow->popSteps(aborted);
	}

	reset();


}

card32 FixedKNNTopologyIterator::getNumNeighbours() 
{
	return m_NumNeighbours;
}

PointIndex FixedKNNTopologyIterator::getNeighbour( card32 nIndex ) 
{
	return (*m_Graph)[(card32)getCurrentPointIndex()*m_NumNeighbours+nIndex];
}

