//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "SceneObject.h"
//---------------------------------------------------------------------------
#include "ObjectClassProperty.h"
#include "FixedArrayClassProperty.h"
#include "NumericalClassProperty.h"
#include "NumericalClassProperty_templCode.h"
#include "PointCloud.h"
#include "LinearAlgebra_templCode.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* SceneObject::SceneObjectMetaClass = NULL;

const MetaClass* SceneObject::getInstanceClass() const {
   return getClass();
}

const MetaClass* SceneObject::getClass() {
   if (SceneObjectMetaClass==NULL) {
      SceneObjectMetaClass = new MetaClass("SceneObject", "no description.",
                                           NULL, sizeof(SceneObject));
   }
   return SceneObjectMetaClass;
}

void SceneObject::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   SceneObjectMetaClass->copyParentProperties();

   BEGIN_CLASS_INIT(SceneObject);
   ADD_CARD32_PROP( materialIndex, 0);
   ADD_MATRIX4F_PROP( transformation,0 );
   ADD_OBJECT_PROP( attachments,0,AttachedDataContainer::getClass(), true );
}


void SceneObject::shutdown() {
   delete SceneObjectMetaClass;
}


// ----

SceneObject::SceneObject() {
   materialIndex  = 0;
   transformation = IDENTITY4F;
   attachments = new AttachedDataContainer;
}

void SceneObject::assignSettings(const SceneObject *assignFrom) {
   pAssert(assignFrom != NULL);
   this->materialIndex = assignFrom->materialIndex;
   this->transformation = assignFrom->transformation;
   attachments->assign(assignFrom->attachments);
   attachments->deleteDelOnWriteData(AttachedData::CT_EVERYTHING_CHANGED);
}

void SceneObject::assign(const Object* obj, CopyContext *context) {
   const SceneObject *so = dynamic_cast<const SceneObject*>(obj);
   assignSettings(so);
   forget(context);
}

SceneObject *SceneObject::createOOCAwareCopy(string idPrefix, bool insertOriginal, string idSuffix) {
	forget(idPrefix);
	forget(insertOriginal);
	forget(idSuffix);
	return (SceneObject*)copy();
}


void SceneObject::read(InputObjectStream *s) {
   bool oldStyle = s->getClassVersion(PointCloud::getClass()) < MetaClass::FIRST_VERSION + 2;
   if (oldStyle) {
      /// legacy parameter
      string name;
      s->readString(name);
      forget(name);

      card32 flags;
      s->read(flags);
      forget(flags);
   }
   Persistent::read(s);
}

void SceneObject::clearAttachments() {
   attachments->clear();
}

void SceneObject::deleteAttachment(const string &name){
	attachments->deleteData(name);
}

SceneObject::~SceneObject() {
   delete attachments;
}
