//---------------------------------------------------------------------------
#ifndef StandardChannelsH
#define StandardChannelsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "VertexDescriptor.h"
//---------------------------------------------------------------------------

namespace geoxl {

//
// --- this module defines a set of standard attribute channels used in most of the rest of the system
//


// --- vertex attributes
const	string VERTEX_CHANNEL_NAME_POSITION              = "position";  // (3f)

const string VERTEX_CHANNEL_NAME_FLAGS                 = "flags";		 // (1i)
const string VERTEX_CHANNEL_NAME_NORMAL                = "normal";	 // (3f)
const string VERTEX_CHANNEL_NAME_COLOR                 = "color";     // (3f)
const string VERTEX_CHANNEL_NAME_INTENSITY             = "intensity"; // (1f)

const string VERTEX_CHANNEL_NAME_TANGENT_U             = "tangent_u"; // (3f)
const string VERTEX_CHANNEL_NAME_TANGENT_V             = "tangent_v"; // (3f)

const string VERTEX_CHANNEL_NAME_SAMPLESPACING         = "sampleSpacing"; // (1f)
const string VERTEX_CHANNEL_NAME_COVARIANCE            = "covariance";    // (6f)
const string VERTEX_CHANNEL_NAME_LOCALMODELUNCERTAINTY = "localModelUncertainty"; // (1f)

const string VERTEX_CHANNEL_NAME_REFERENCE_POSITION	 = "ref_position"; // (3f)

extern POINTCLOUDS_API const VAD VERTEX_CHANNEL_VAD_POSITION;
extern POINTCLOUDS_API const VAD VERTEX_CHANNEL_VAD_FLAGS;
extern POINTCLOUDS_API const VAD VERTEX_CHANNEL_VAD_NORMAL;
extern POINTCLOUDS_API const VAD VERTEX_CHANNEL_VAD_COLOR;
extern POINTCLOUDS_API const VAD VERTEX_CHANNEL_VAD_INTENSITY;
extern POINTCLOUDS_API const VAD VERTEX_CHANNEL_VAD_TANGENT_U;
extern POINTCLOUDS_API const VAD VERTEX_CHANNEL_VAD_TANGENT_V;
extern POINTCLOUDS_API const VAD VERTEX_CHANNEL_VAD_SAMPLESPACING;
extern POINTCLOUDS_API const VAD VERTEX_CHANNEL_VAD_COVARIANCE;
extern POINTCLOUDS_API const VAD VERTEX_CHANNEL_VAD_LOCALMODELUNCERTAINTY;
extern POINTCLOUDS_API const VAD VERTEX_CHANNEL_VAD_REFERENCE_POSITION;


// --- triangle face attributes
const string TRIANGLE_CHANNEL_NAME_INDEX = "index";   // (3i)

extern POINTCLOUDS_API const VAD TRIANGLE_CHANNEL_VAD_INDEX;


// --- topology graph edge attributes
const string TOPGRAPH_CHANNEL_NAME_VINDEX = "vindex"; // (2i)
const string TOPGRAPH_CHANNEL_NAME_COLOR  = "color";  // (4f)

extern POINTCLOUDS_API const VAD TOPGRAPH_CHANNEL_VAD_VINDEX;
extern POINTCLOUDS_API const VAD TOPGRAPH_CHANNEL_VAD_COLOR;


// --- some simple templates
extern POINTCLOUDS_API const VertexDescriptor POSITION_FLAGS_VDESCR;
extern POINTCLOUDS_API const VertexDescriptor POSITION_NORMAL_FLAGS_VDESCR;
extern POINTCLOUDS_API const VertexDescriptor POSITION_NORMAL_COLOR_FLAGS_VDESCR;

extern POINTCLOUDS_API const VertexDescriptor INDEX_TRI_DESCR;

extern POINTCLOUDS_API const VertexDescriptor VINDEX_TOPEDGE_DESCR;
extern POINTCLOUDS_API const VertexDescriptor VINDEX_COLOR_TOPEDGE_DESCR;


}

#endif