//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "TopologyRangeSearch.h"
//---------------------------------------------------------------------------
#include "InCorePCTopologyGraph.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



TopNeighborhoodRangeSearch::TopNeighborhoodRangeSearch() {
   tg = NULL;
   ps = NULL;
}
void TopNeighborhoodRangeSearch::setup(const InCorePCTopologyGraph *tg, const PointSet *ps, card32 startIndex) {
   this->tg = tg;
   this->ps = ps;
   
   this->indexStack.clear();
   this->visited.clear();
   
   if (inRange(ps, startIndex)) {
      indexStack.push_back(startIndex);
      visited.addElement(startIndex, 1);
   }
}

bool TopNeighborhoodRangeSearch::atEnd() {
   return indexStack.empty();
}

void TopNeighborhoodRangeSearch::next() {
   pAssert(!indexStack.empty());
   card32 index = indexStack.back();
   indexStack.pop_back();
   const card32 n = tg->getNumAdjacentVertices(index);
   for (card32 i=0; i<n; i++) {
      card32 j = tg->getAdjacentVertex(index, i);
      card32 forgetThis;
      if (!visited.getElement(j, forgetThis)) {
         if (inRange(ps, j)) {
            indexStack.push_back(j);
            visited.addElement(j, 1);
         }
      }
   }
}

card32 TopNeighborhoodRangeSearch::getCurrentIndex() {
   pAssert(!indexStack.empty());
   return indexStack.back();
}


// ---


void TopNeighborhoodSphereSearch::setup(
	const InCorePCTopologyGraph *tg, const PointSet *ps, card32 startIndex,
	const Vector3f &center, const float32 dist, const AAT &posChannel)
{
	this->center = center;
	this->dist = dist;
	this->posChannel = posChannel;


	TopNeighborhoodRangeSearch::setup(tg, ps, startIndex);	
}

bool TopNeighborhoodSphereSearch::inRange(const PointSet *ps, card32 index) {
	Vector3f pos = ps->get<float32, 3>(index, posChannel);
	return norm(pos-center) <= dist;
}


// ---


TopNeighborhoodKNNSearch::TopNeighborhoodKNNSearch(const PointSet *ps, const InCorePCTopologyGraph *tg, card32 startIndex, const AAT &posChannel) {
   this->tg = tg;
   this->ps = ps;
   this->posChannel = posChannel;
   reset(startIndex);
}

void TopNeighborhoodKNNSearch::reset(card32 startIndex) {
   pos = ps->get3f(startIndex, posChannel);
	neighbors = priority_queue<QueueEntry>();
	visited.clear();
   QueueEntry q(0, startIndex);
	neighbors.push(q);
   visited.insert(startIndex);
}

bool TopNeighborhoodKNNSearch::atEnd() {
   return neighbors.empty();
}

void TopNeighborhoodKNNSearch::next() {
   pAssert(!neighbors.empty());
   card32 index = neighbors.top().index;
   neighbors.pop();
   const card32 n = tg->getNumAdjacentVertices(index);
   for (card32 i=0; i<n; i++) {
      card32 j = tg->getAdjacentVertex(index, i);
      if (visited.find(j) == visited.end()) {
		   Vector3f npos = ps->get<float32, 3>(j, posChannel);
		   QueueEntry q(norm(pos-npos), j);
         neighbors.push(q);
         visited.insert(j);
      }
   }
}

card32 TopNeighborhoodKNNSearch::getCurrentIndex() {
   pAssert(!neighbors.empty());
   return neighbors.top().index;
}







// ---


TopNeighborhoodKNNSearch_Fast::TopNeighborhoodKNNSearch_Fast(const PointSet *ps, const InCorePCTopologyGraph *tg, card32 startIndex, const AAT &posChannel)
: visited(ps->getNumEntries(), 0)
 {
   this->tg = tg;
   this->ps = ps;
   this->posChannel = posChannel;
   reset(startIndex);
}

void TopNeighborhoodKNNSearch_Fast::reset(card32 startIndex) {
   visited.clear();
   visited.resize(ps->getNumEntries(), 0);
   pos = ps->get3f(startIndex, posChannel);
	neighbors = priority_queue<QueueEntry>();
   QueueEntry q(0, startIndex);
	neighbors.push(q);
   visited[startIndex] = 1;
}

bool TopNeighborhoodKNNSearch_Fast::atEnd() {
   return neighbors.empty();
}

void TopNeighborhoodKNNSearch_Fast::next() {
   pAssert(!neighbors.empty());
   card32 index = neighbors.top().index;
   neighbors.pop();
   const card32 n = tg->getNumAdjacentVertices(index);
   for (card32 i=0; i<n; i++) {
      card32 j = tg->getAdjacentVertex(index, i);
      if (visited[j] == 0) {
		   Vector3f npos = ps->get<float32, 3>(j, posChannel);
		   QueueEntry q(norm(pos-npos), j);
         neighbors.push(q);
         visited[j] = 1;
      }
   }
}

card32 TopNeighborhoodKNNSearch_Fast::getCurrentIndex() {
   pAssert(!neighbors.empty());
   return neighbors.top().index;
}
