//---------------------------------------------------------------------------
#ifndef TriangleMeshPointCloudH
#define TriangleMeshPointCloudH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PTypes.h"
//---------------------------------------------------------------------------


namespace geoxl {

class VertexDescriptor;
class PointCloud;


/// Additional interface that can be implemented by a point cloud to provide a triangle mesh representation of the points.
/// Access to the data is provided via iterators.
/// The triangle mesh is accessed as a point cloud that provides a attribute channel int32 index[3] to refer to the triangle vertices.
/// Access to the data is via Iterators, but include CAP_TRIANGLE_MESH to access the triangles rather than the vertices.
class TriangleMeshPointCloud { /* INTERFACE CLASS */
 public:
   /// query attribute descriptor for faces. it always provides int32 index[3] for refering to the triangle vertices
   virtual const VertexDescriptor *getTriangleDescr() const = 0;
   /// append additional per-triangle attribute channels
   virtual PointCloud             *appendTriangleFields(const VertexDescriptor *newAttribs) const = 0;
   /// clear connectivity and setup new triangle set with new attributes; points (vertices) remain unaffected.
   virtual void                    clearAndSetupTriangles(const VertexDescriptor *triangleDescr) = 0;
   /// get number of triangles.
   virtual card64                  getNumTriangles() = 0;
};


}

#endif
 