//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "UnstructuredInCorePointCloud.h"
//---------------------------------------------------------------------------
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "PointSetIterator.h"
#include "ObjectClassProperty.h"
#include "AttachedData.h"
#include "Tools.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* UnstructuredInCorePointCloud::UnstructuredInCorePointCloudMetaClass = NULL;

const MetaClass* UnstructuredInCorePointCloud::getInstanceClass() const {
   return getClass();
}

Object* UnstructuredInCorePointCloud::newInstance() {
   return new UnstructuredInCorePointCloud;
}

const MetaClass* UnstructuredInCorePointCloud::getClass() {
   if (UnstructuredInCorePointCloudMetaClass==NULL) {
      UnstructuredInCorePointCloudMetaClass = new MetaClass("UnstructuredInCorePointCloud", "no description.",
                                  newInstance, sizeof(UnstructuredInCorePointCloud));
   }
   return UnstructuredInCorePointCloudMetaClass;
}

void UnstructuredInCorePointCloud::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   UnstructuredInCorePointCloudMetaClass->copyParentProperties();
//   UnstructuredInCorePointCloudMetaClass->addProperty(new ObjectClassProperty("ps", 1, (ObjectElementPointer)&ps, PointSet::getClass(), true));
//   ObjectClassProperty *dsprop = new ObjectClassProperty("hierarchyAttachedDSs", MetaClass::FIRST_VERSION + 1,
//      (ObjectElementPointer)&hierarchyAttachedDSs, AttachedDataContainer::getClass(), true);
//   UnstructuredInCorePointCloudMetaClass->addProperty(dsprop);
	BEGIN_CLASS_INIT( UnstructuredInCorePointCloud );
	ADD_OBJECT_PROP( ps, 0, PointSet::getClass(), true );
	ADD_OBJECT_PROP( hierarchyAttachedDSs, 0, AttachedDataContainer::getClass(), true );
}

void UnstructuredInCorePointCloud::shutdown() {
   delete UnstructuredInCorePointCloudMetaClass;
}


// ----


MetaClass* BasicUnstructuredInCorePointCloudIterator::BasicUnstructuredInCorePointCloudIteratorMetaClass = NULL;

const MetaClass* BasicUnstructuredInCorePointCloudIterator::getInstanceClass() const {
   return getClass();
}

Object* BasicUnstructuredInCorePointCloudIterator::newInstance() {
   return new BasicUnstructuredInCorePointCloudIterator;
}

const MetaClass* BasicUnstructuredInCorePointCloudIterator::getClass() {
   if (BasicUnstructuredInCorePointCloudIteratorMetaClass==NULL) {
      BasicUnstructuredInCorePointCloudIteratorMetaClass = new MetaClass("BasicUnstructuredInCorePointCloudIterator", "no description.",
                                                                       newInstance, sizeof(BasicUnstructuredInCorePointCloudIterator));
   }
   return BasicUnstructuredInCorePointCloudIteratorMetaClass;
}

void BasicUnstructuredInCorePointCloudIterator::init(const MetaClass *parentClass, card32 uniqueIdentifier) {
   ((MetaClass*)getClass())->init(uniqueIdentifier, parentClass);
   BasicUnstructuredInCorePointCloudIteratorMetaClass->copyParentProperties();
}

void BasicUnstructuredInCorePointCloudIterator::shutdown() {
   delete BasicUnstructuredInCorePointCloudIteratorMetaClass;
}


// ----



UnstructuredInCorePointCloud::UnstructuredInCorePointCloud() {
   ps = new PointSet();
   hierarchyAttachedDSs = new AttachedDataContainer();
}

void UnstructuredInCorePointCloud::assign(const Object* obj, CopyContext *context) {
   PointCloud::assign(obj, context);
   const UnstructuredInCorePointCloud *pobj = dynamic_cast<const UnstructuredInCorePointCloud*>(obj);
   pAssert(pobj != NULL);
   if (pobj->ps == NULL) {
      delete ps;
      ps = NULL;
   } else if (ps != pobj->ps) {
      delete ps;
      ps = (PointSet*)(pobj->ps)->copy(context);
   }
   delete this->hierarchyAttachedDSs;
   this->hierarchyAttachedDSs = (AttachedDataContainer*)(pobj->hierarchyAttachedDSs->copy(context));
}

const VertexDescriptor *UnstructuredInCorePointCloud::getDescr() const {
   return ps->getDescr();
}

PointCloud *UnstructuredInCorePointCloud::appendFields(const VertexDescriptor *newAttribs) const {
   return createCopyWithNewPointSet(ps->appendFields(newAttribs));
}

PointCloudIterator *UnstructuredInCorePointCloud::createIterator(SceneObjectIteratorType  type) {
   if (!supportsIteratorType(type)) {
      return NULL;
   } else {
      return new BasicUnstructuredInCorePointCloudIterator(ps, hierarchyAttachedDSs);
   }
}

bool UnstructuredInCorePointCloud::supportsIteratorType(SceneObjectIteratorType cap) {
   SOIT possibleCaps(  SOIT::CAP_BASIC_PC 
							| SOIT::CAP_INDEXED_PC 
							| SOIT::CAP_RANDOM_ACCESS_PC 
							| SOIT::CAP_MODIFYABLE_PC
                     | SOIT::CAP_DYNAMIC_PC 
							| SOIT::CAP_HIERARCHICAL  
							| SOIT::CAP_ATTACH_SECONDARY );
   if( !possibleCaps.providesCapsFor(cap))
		return false;
	else
		return true;
}

card64 UnstructuredInCorePointCloud::getNumPoints() {
   if (ps == NULL) return 0;
   return ps->getNumEntries();
}

void UnstructuredInCorePointCloud::clear() {
   ps->clear();
}


void UnstructuredInCorePointCloud::clearAndSetup(const VertexDescriptor *vertDescr, card32 height) {
   ps->clearAndSetup(1, height, vertDescr);
}

UnstructuredInCorePointCloud *UnstructuredInCorePointCloud::createCopyWithNewPointSet(PointSet *ps) const {
   PointSet *pscopy = this->ps;
   ((UnstructuredInCorePointCloud*)this)->ps = NULL;
   UnstructuredInCorePointCloud *result = (UnstructuredInCorePointCloud*)copy();
   ((UnstructuredInCorePointCloud*)this)->ps = pscopy;
   result->ps = ps;
   return result;
}

void UnstructuredInCorePointCloud::clearAttachments() {
   PointCloud::clearAttachments();
   hierarchyAttachedDSs->clear();
}

void UnstructuredInCorePointCloud::deleteDelOnWriteAttachments(AttachedData::ChangeType change) {
   getAttachments()->deleteDelOnWriteData(change);
   hierarchyAttachedDSs->deleteDelOnWriteData(change);
   ps->invalidateLastNumFlagged();
}

void UnstructuredInCorePointCloud::deleteAttachment(const string &name){
	hierarchyAttachedDSs->deleteData(name);
}


UnstructuredInCorePointCloud::~UnstructuredInCorePointCloud() {
   delete ps;
   delete hierarchyAttachedDSs;
}



// ---------------------------------------



BasicUnstructuredInCorePointCloudIterator::BasicUnstructuredInCorePointCloudIterator(PointSet *ps, AttachedDataContainer *attData)
 : AttachedDataPointSetIterator(ps, attData)
{}


//----------


void BasicUnstructuredInCorePointCloudIterator::add(const char *newPoint) {
   ps->addPoint(newPoint);
   width = ps->getWidth();
   height = ps->getHeight();
   attachedDataLargeChange = true;
   attachedDataAttribChange = true;
   invalidateLastNumFlagged();
}


void BasicUnstructuredInCorePointCloudIterator::add(const PointSet *newPoints) {

   // to-do: not tested and slow!!

   char* buffer = new char[ps->getDescr()->getSize()];
   for (card32 i=0; i<=newPoints->getNumEntries(); i++)
   {
      newPoints->getVertex(i,buffer);
      ps->addPoint(buffer);
   };
   delete[] buffer;

   width = ps->getWidth();
   height = ps->getHeight();
   attachedDataLargeChange = true;
   attachedDataAttribChange = true;
   invalidateLastNumFlagged();

   forget(newPoints);
}


void BasicUnstructuredInCorePointCloudIterator::deleteCurrent() {
   char *tmpbuf = new char[ps->getDescr()->getSize()];
   ps->getVertex(width-1,height-1,tmpbuf);
   ps->setVertex(x,y,tmpbuf);
   ps->changeHeight(height-1);
   width = ps->getWidth();
   height = ps->getHeight();

   // one back, cause the new copyed has to be tested too
/*   if (x == 0 ) {
      x = width;
      y--;
   }
   else x--;

 */

   attachedDataLargeChange = true;
   attachedDataAttribChange = true;
   invalidateLastNumFlagged();
   delete tmpbuf;
}



