//---------------------------------------------------------------------------
#ifndef UnstructuredInCorePointCloudH
#define UnstructuredInCorePointCloudH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PointCloud.h"
#include "PointCloudIterators.h"
#include "PointSetIterator.h"
#include "PointSet.h"
//#include "PointSetOctree.h"
//---------------------------------------------------------------------------

namespace geoxl {

class AttachedData;
class AttachedDataContainer;


class BasicUnstructeredInCorePointCloudIterator;

class POINTCLOUDS_API UnstructuredInCorePointCloud : public PointCloud {
 private:
   static MetaClass *UnstructuredInCorePointCloudMetaClass;
   static Object* newInstance();

 protected:
   PointSet *ps;
   AttachedDataContainer *hierarchyAttachedDSs;

 protected:
   AttachedDataContainer *getHierarchyAttachedDSs() {return hierarchyAttachedDSs;}

 public:
   UnstructuredInCorePointCloud();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual const VertexDescriptor *getDescr() const;
   virtual PointCloud             *appendFields(const VertexDescriptor *newAttribs) const;
   virtual PointCloudIterator     *createIterator(SceneObjectIteratorType type);
   virtual bool                    supportsIteratorType(SceneObjectIteratorType cap);
   virtual card64                  getNumPoints();
   virtual void                    clear();

   virtual void clearAttachments();
   virtual void deleteDelOnWriteAttachments(AttachedData::ChangeType change = AttachedData::CT_EVERYTHING_CHANGED);
   virtual void deleteAttachment(const string &name);

   // --- new methods
   virtual void clearAndSetup(const VertexDescriptor *vertDescr, card32 height);
   virtual const PointSet *getPointSet() const {return ps;}
   virtual void setPointSet(PointSet *ps) {delete this->ps; this->ps = ps;}
   virtual PointSet *getPointSet() {return ps;}
   virtual UnstructuredInCorePointCloud *createCopyWithNewPointSet(PointSet *ps) const; // ownership of ps will be transfered to this

   // ---

   ~UnstructuredInCorePointCloud();
   friend class BasicUnstructuredInCorePointCloudIterator;
};

class POINTCLOUDS_API BasicUnstructuredInCorePointCloudIterator : public AttachedDataPointSetIterator,
                                                          public DynamicPointCloudIterator {
 private:
   static MetaClass *BasicUnstructuredInCorePointCloudIteratorMetaClass;
   static Object* newInstance();

   BasicUnstructuredInCorePointCloudIterator() {}

 public:
   BasicUnstructuredInCorePointCloudIterator(PointSet *ps, AttachedDataContainer *attData);

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass, card32 uniqueIdentifier);
   static void shutdown();

   // DynamicPointIterator
   virtual void add(const char *newPoint);
   virtual void add(const PointSet *newPoints);
   virtual void deleteCurrent();

};


}

#endif
