//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "UnstructuredInCoreTriangleMesh.h"
//---------------------------------------------------------------------------
#include "HashTables_templCode.h"
#include "LinearAlgebra_templCode.h"
#include "PointSet_templCode.h"
#include "HierarchicalKNNIterator.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------
#include "VertexArray.h"


MetaClass* UnstructuredInCoreTriangleMesh::UnstructuredInCoreTriangleMeshMetaClass = NULL;

const MetaClass* UnstructuredInCoreTriangleMesh::getInstanceClass() const {
   return getClass();
}

Object* UnstructuredInCoreTriangleMesh::newInstance() {
   return new UnstructuredInCoreTriangleMesh;
}

const MetaClass* UnstructuredInCoreTriangleMesh::getClass() {
   if (UnstructuredInCoreTriangleMeshMetaClass==NULL) {
      UnstructuredInCoreTriangleMeshMetaClass = new MetaClass("UnstructuredInCoreTriangleMesh", "no description.",
                                  newInstance, sizeof(UnstructuredInCoreTriangleMesh));
   }
   return UnstructuredInCoreTriangleMeshMetaClass;
}

void UnstructuredInCoreTriangleMesh::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   UnstructuredInCoreTriangleMeshMetaClass->copyParentProperties();
   //UnstructuredInCoreTriangleMeshMetaClass->addProperty(new ObjectClassProperty("triangles", 1, (ObjectElementPointer)&triangles, PointSet::getClass(), true));
	BEGIN_CLASS_INIT( UnstructuredInCoreTriangleMesh );
	ADD_OBJECT_PROP( triangles, 0, PointSet::getClass(), true );
}

void UnstructuredInCoreTriangleMesh::shutdown() {
   delete UnstructuredInCoreTriangleMeshMetaClass;
}


// ----


UnstructuredInCoreTriangleMesh::UnstructuredInCoreTriangleMesh()
{
   triangles = new PointSet();
   VertexDescriptor vd;
   vd.pushAttrib(mVAD("position", 3, VAD::DATA_FORMAT_FLOAT32));
   vd.pushAttrib(mVAD("normal", 3, VAD::DATA_FORMAT_FLOAT32));
   clearAndSetup( &vd, 0 );
   //setMaterialIndex(3);
}

void UnstructuredInCoreTriangleMesh::clearAndSetup(const VertexDescriptor *vertDescr, card32 numPoints) {
   UnstructuredInCorePointCloud::clearAndSetup(vertDescr, numPoints);
   VertexDescriptor vd;
   vd.pushAttrib(mVAD("index", 3, VAD::DATA_FORMAT_INT32));
   triangles->clearAndSetup(1, 0, &vd);
}

void UnstructuredInCoreTriangleMesh::clearAndSetup(const VertexDescriptor *vertDescr, mpcard numPoints,
                                                   const VertexDescriptor *triangleDescr, mpcard numTriangles)
{
   UnstructuredInCorePointCloud::clearAndSetup(vertDescr, numPoints);
   triangles->clearAndSetup(1, numTriangles, triangleDescr);

}                                            

void UnstructuredInCoreTriangleMesh::assign(const Object* obj, CopyContext *context) {
   UnstructuredInCorePointCloud::assign(obj, context);
   const UnstructuredInCoreTriangleMesh *other = dynamic_cast<const UnstructuredInCoreTriangleMesh*>(obj);
   pAssert(other);
   if (other != this) {
      triangles->assign(other->triangles);
   }
}

PointCloudIterator *UnstructuredInCoreTriangleMesh::createIterator(SceneObjectIteratorType type) {
   if (!supportsIteratorType(type)) {
      return NULL;
   }
   if (type.providesCapsFor(SOIT::CAP_TRIANGLE_MESH)) {
      return new ConnectivityPointSetIterator(triangles, getHierarchyAttachedDSs(), getPointSet()->getBoundingBox());
   } else {
      return UnstructuredInCorePointCloud::createIterator(type);
   }
}

bool UnstructuredInCoreTriangleMesh::supportsIteratorType(SceneObjectIteratorType cap) {
   if (!cap.providesCapsFor(SOIT::CAP_TRIANGLE_MESH)) {
      return UnstructuredInCorePointCloud::supportsIteratorType(cap);
   } else {
      SOIT possibleCaps(  SOIT::CAP_BASIC_PC | SOIT::CAP_MODIFYABLE_PC | SOIT::CAP_DYNAMIC_PC
                        | SOIT::CAP_HIERARCHICAL  | SOIT::CAP_ATTACH_SECONDARY | SOIT::CAP_TRIANGLE_MESH );
      return (possibleCaps.providesCapsFor(cap));
   }
}

void UnstructuredInCoreTriangleMesh::clear() {
   UnstructuredInCorePointCloud::clear();
   triangles->clear();
}

const PointSet *UnstructuredInCoreTriangleMesh::getTriangles() const {
   return triangles;
}

void UnstructuredInCoreTriangleMesh::setTriangles(PointSet *psTriangles) {
   if (triangles != psTriangles) {
      delete triangles;
      triangles = psTriangles;
   }
}

PointSet *UnstructuredInCoreTriangleMesh::getTriangles() {
   return triangles;
}

UnstructuredInCoreTriangleMesh *UnstructuredInCoreTriangleMesh::createCopyWithNewTriangles(PointSet *psTriangles) const {
   PointSet *tcopy = this->triangles;
   ((UnstructuredInCoreTriangleMesh*)this)->triangles = NULL;
   UnstructuredInCoreTriangleMesh *result = (UnstructuredInCoreTriangleMesh*)copy();
   ((UnstructuredInCoreTriangleMesh*)this)->triangles = tcopy;
   result->triangles = psTriangles;
   return result;
}

UnstructuredInCoreTriangleMesh * geoxl::UnstructuredInCoreTriangleMesh::createCopyOfGivenTriangles( const vector<int32> &indexes )
{

	UnstructuredInCoreTriangleMesh *result = new UnstructuredInCoreTriangleMesh();
	result->setPointSet((PointSet*)this->getPointSet()->copy());
	PointSet* givenTriangles=new PointSet();
	givenTriangles->clearAndSetup(1, indexes.size(), this->getTriangles()->getDescr());
	AAT INDEX=this->getTriangles()->getAAT("index");
	for (card32 i=0;i<indexes.size();i++)
	{
		Vector3i tri=this->getTriangles()->get3i(indexes[i],INDEX);
		givenTriangles->set3i(i,INDEX,tri);
	}

	result->setTriangles(givenTriangles);
	result->clearGarbage();
	return result;
}

const VertexDescriptor *UnstructuredInCoreTriangleMesh::getTriangleDescr() const {
   return triangles->getDescr();
}

PointCloud *UnstructuredInCoreTriangleMesh::appendTriangleFields(const VertexDescriptor *newAttribs) const {
   return createCopyWithNewTriangles(triangles->appendFields(newAttribs));
}

void UnstructuredInCoreTriangleMesh::clearAndSetupTriangles(const VertexDescriptor *triangleDescr) {
   triangles->clearAndSetup(1, 0, triangleDescr);
}

card64 UnstructuredInCoreTriangleMesh::getNumTriangles() {
   return triangles->getNumEntries();
}

UnstructuredInCoreTriangleMesh::~UnstructuredInCoreTriangleMesh() {
   delete triangles;
}



string UnstructuredInCoreTriangleMesh::calcHashString( const Vector3f &vec )
{
   string ret;
   card8 * tmp = (card8 *)&vec[0];
   for( card32 i=0;i<3*4;i++ )
      ret += tmp[i];
   return ret;
}

void UnstructuredInCoreTriangleMesh::closeMesh()
{
   MyHashTable hashTable;

   // new vertex index = indexTable[ old vertex index ]
   vector<card32> indexTable(getPointSet()->getNumEntries());
   card32 numVertices = 0;

   VertexArray array(getPointSet());

   for( card32 i=0;i<getPointSet()->getNumEntries();i++ )
   {
      string hashID = calcHashString( array.getPosition3f(i) );
      card32 index = 0;
      if( hashTable.getData( hashID, index ) )
      {
         // vertex exists, take index from existing vertex
         indexTable[i] = index;
      }
      else
      {
         // new vertex
         indexTable[i] = numVertices;
         array.copyVertex( i, numVertices );
         numVertices++;
         hashTable.addElement( hashID, indexTable[i] );
      }
   }

   // shrink ps
   if( numVertices != getPointSet()->getNumEntries() )
   {
      PointSet * newPS = new PointSet;
      newPS->clearAndSetup( 1, numVertices, getPointSet()->getDescr() );

      vector<char> tmpBuffer( getPointSet()->getDescr()->getSize() );

      for( card32 i=0;i<numVertices;i++ )
      {
         getPointSet()->getVertex(i, &tmpBuffer[0]);
         newPS->setVertex(i, &tmpBuffer[0] );
      }
      setPointSet(newPS);
   }

   // adapt triangles
   VertexArray triangleArray(triangles);
   for( card32 i=0;i<triangles->getNumEntries();i++ )
   {
      Vector3i index = triangleArray.getIndex3i(i);
      for( card32 c=0;c<3;c++ )
         index[c] = indexTable[index[c]];
      triangleArray.setIndex3i(i,index);
   }

}

void UnstructuredInCoreTriangleMesh::calculateVertexNormals()
{
   pAssert( getPointSet()->providesAttribute( "normal" ) );

   VertexArray vertArray( getPointSet() );
   VertexArray triArray( triangles );

	// init normals to 0
	for( card32 i=0;i<vertArray.getNumElements();i++ )
	{
		vertArray.setNormal3f( i, makeVector3f(0,0,0));
	}

   // calculate triangle normal and add to vertex normals
	for( card32 i=0;i<triArray.getNumElements();i++ )
   {
      Vector3i tri = triArray.getIndex3i(i);
      Vector3f p1 = vertArray.getPosition3f( tri[0] );
      Vector3f p2 = vertArray.getPosition3f( tri[1] );
      Vector3f p3 = vertArray.getPosition3f( tri[2] );
      Vector3f normal = (p2-p1).crossProduct(p2-p3);
      //normal.normalize();
      vertArray.setNormal3f(tri[0], vertArray.getNormal3f(tri[0]) + normal );
      vertArray.setNormal3f(tri[1], vertArray.getNormal3f(tri[1]) + normal );
      vertArray.setNormal3f(tri[2], vertArray.getNormal3f(tri[2]) + normal );
   }

   // normalize normals
   for( card32 i=0;i<vertArray.getNumElements();i++ )
   {
      Vector3f normal = vertArray.getNormal3f(i);
      normal.normalize();
		normal= -normal;
      vertArray.setNormal3f(i,normal);
   }

}

void UnstructuredInCoreTriangleMesh::clearGarbage()
{
   VertexArray triArray( triangles );

   PointSet * tmpPS = new PointSet;
   tmpPS->clearAndSetup( 1, getPointSet()->getNumEntries(), getPointSet()->getDescr() );

   vector<int32> moveTo( getPointSet()->getNumEntries() );
   for( card32 i=0;i<moveTo.size();i++ )
      moveTo[i] = -1;

   int32 numVertices = 0;
   vector<char> tmpBuffer( getPointSet()->getDescr()->getSize() );
   for( card32 i=0;i<triArray.getNumElements();i++ )
   {
      Vector3i tri = triArray.getIndex3i(i);

      if( tri[0] == tri[1]
         || tri[1] == tri[2]
         || tri[0] == tri[2] )
      {
         triArray.setIndex3i( i, makeVector3i(0,0,0) );
         continue;
      }

      for( card32 j=0;j<3;j++ )
      {
         if( tri[j] < 0 )
            continue;
         if( tri[j] >= (int32)getPointSet()->getNumEntries() )
            continue;
         if( moveTo[ tri[j] ] == -1 )
         {
            getPointSet()->getVertex(tri[j], &tmpBuffer[0]);
            //getPointSet()->getVertex(getPointSet()->getNumEntries()-1, &tmpBuffer[0] );
            tmpPS->setVertex(numVertices, &tmpBuffer[0] );
            moveTo[ tri[j] ] = numVertices;
            tri[j] = numVertices;
            numVertices++;
         }
         else
         {
            tri[j] = moveTo[ tri[j] ];
         }

      }

      triArray.setIndex3i( i, tri );

   }
   // shrink ps
   PointSet * newPS = new PointSet;
   newPS->clearAndSetup( 1, numVertices, getPointSet()->getDescr() );

   for( card32 i=0;i<newPS->getNumEntries();i++ )
   {
      tmpPS->getVertex(i, &tmpBuffer[0] );
      newPS->setVertex(i, &tmpBuffer[0] );
   }
   setPointSet(newPS);
   delete tmpPS;

}


void UnstructuredInCoreTriangleMesh::snapMesh( float max_tolerance )
{
   vector<int32> moveTo( getPointSet()->getNumEntries() );
   for( card32 i=0;i<moveTo.size();i++ )
      moveTo[i] = i;

   clearAttachments();
   HierarchicalKNNIterator * iter = new HierarchicalKNNIterator( this, 32, NULL );

   VertexArray vertices( getPointSet() );
   VertexArray triArray( triangles );

   for( card32 i=0;i<moveTo.size();i++ )
   {
      Vector3f pos = vertices.getPosition3f(i);
      iter->setSeekPointAndReset( pos );

      while(true)
      {
         int32 tmpI = iter->getPointSetPointNumber();
         float32 dist = norm( pos - vertices.getPosition3f( moveTo[ tmpI ] ) );
         if( tmpI != (int32)i && dist <= max_tolerance )
         {
            moveTo[ i ] = moveTo[ tmpI ];
            break;
         }
         if( dist > max_tolerance )
            break;

         iter->next();
      }

   }

   for( card32 i=0;i<triArray.getNumElements();i++ )
   {
      Vector3i tri = triArray.getIndex3i( i );
      tri[0] = moveTo[ tri[0] ];
      tri[1] = moveTo[ tri[1] ];
      tri[2] = moveTo[ tri[2] ];
      triArray.setIndex3i( i, tri );
   }

   delete iter;

   clearGarbage();
}

void UnstructuredInCoreTriangleMesh::deleteMarkedTriangles()
{
	VertexArray vertArray( getPointSet() );
	VertexArray triArray( triangles );


	int32 lastTriIndex=triArray.getNumElements()-1;

	// calculate triangle normal and add to vertex normals
	for( int32 i=0;i<=lastTriIndex;i++ )
	{
		Vector3i triIndexes = triArray.getIndex3i(i);
		bool p1 = (vertArray.getFlags( triIndexes[0])& PF_FLAG_SELECTED);
		bool p2 = (vertArray.getFlags( triIndexes[1] )& PF_FLAG_SELECTED);
		bool p3 = (vertArray.getFlags( triIndexes[2] )& PF_FLAG_SELECTED);

		if (p1 || p2 || p3)  //delete this tri
		{
			triArray.copyVertex(lastTriIndex, i);
			lastTriIndex--;
			i--;
		}
	}
	triangles->changeHeight(lastTriIndex+1);
	clearGarbage();
	clearAttachments();

	
}

// --


ConnectivityPointSetIterator::ConnectivityPointSetIterator(PointSet *meshPS, AttachedDataContainer *attData, BoundingBox3f bb)
: AttachedDataPointSetIterator(meshPS, attData) {
   this->bb = bb;
}

BoundingBox3f ConnectivityPointSetIterator::getBoundingBox() {
   return bb;
}


