//---------------------------------------------------------------------------
#ifndef UnstructuredInCoreTriangleMeshH
#define UnstructuredInCoreTriangleMeshH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "UnstructuredInCorePointCloud.h"
#include "TriangleMeshPointCloud.h"
#include "HashTables.h"
//---------------------------------------------------------------------------

namespace geoxl {


class POINTCLOUDS_API UnstructuredInCoreTriangleMesh : public UnstructuredInCorePointCloud,
                                               public TriangleMeshPointCloud {
 private:
   static MetaClass *UnstructuredInCoreTriangleMeshMetaClass;
   static Object* newInstance();

   PointSet *triangles;

 public:
   UnstructuredInCoreTriangleMesh();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context = NULL);


   // --- UnstructuredInCorePointCloud

   /// creates empty mesh with standard triangle face descriptor (index only)
   virtual void clearAndSetup(const VertexDescriptor *vertDescr, card32 numPoints = 0);
   virtual void clearAndSetup(const VertexDescriptor *vertDescr, mpcard numPoints,
                              const VertexDescriptor *triangleDescr, mpcard numTriangles);
   virtual PointCloudIterator *createIterator(SceneObjectIteratorType type);
   virtual bool supportsIteratorType(SceneObjectIteratorType cap);
   virtual void clear();


   /// --- additional methods, similar to UnstructuredInCorePointCloud point processing methods

   virtual const PointSet *getTriangles() const;
   virtual void setTriangles(PointSet *psTriangles);
   virtual PointSet *getTriangles();
   /// ownership of ps will be transfered to this
   virtual UnstructuredInCoreTriangleMesh *createCopyWithNewTriangles(PointSet *psTriangles) const;
			  UnstructuredInCoreTriangleMesh *createCopyOfGivenTriangles(const vector<int32> &indexes);


   // ---  TriangleMeshPointCloud
   virtual const VertexDescriptor *getTriangleDescr() const;
   virtual PointCloud             *appendTriangleFields(const VertexDescriptor *newAttribs) const;
   virtual void                    clearAndSetupTriangles(const VertexDescriptor *triangleDescr);
   virtual card64                  getNumTriangles();


   ~UnstructuredInCoreTriangleMesh();

   // --- standard mesh methods ---

   /// searches for double vertices with exactly equal positions and merges them
   void closeMesh();

   /// calculate vertex normals
   void calculateVertexNormals();

	///remove all triangles, connected to selected points
	void deleteMarkedTriangles();

   /// removes any unconnected vertices from mesh
   void clearGarbage();

   /// snap vertices to closest neighbour 
   void snapMesh( float max_tolerance );

private:
   class POINTCLOUDS_API StringComp {
     public:
       static bool equals(string obj1, string obj2) { return obj1 == obj2; }
   };

   class MyHashTable : public HashTable<string, card32, StringComp> {
    protected:
      virtual max_platform_card hashFunction(string key) const {
         card32 hk = 0;
         const card32 n = (const card32 )key.size();
         for (card32 i=0; i<n; i++) {
            const card8 c = key[i];
            hk <<= 1;
            hk ^=c;
         }
         return (max_platform_card)hk;
      };
    public:
      MyHashTable(card32 fillFactor = 1, card32 startSizeIndex = 8) : HashTable<string, card32, StringComp>(fillFactor, startSizeIndex) {}

   };

   string calcHashString( const Vector3f &vec );

};



class ConnectivityPointSetIterator : public AttachedDataPointSetIterator {
 private:
   BoundingBox3f bb;

 public:
   ConnectivityPointSetIterator(PointSet *meshPS, AttachedDataContainer *attData, BoundingBox3f bb);
   /// must be redefined because triangles themselfs do not contain positions (but indices to positions)
   /// --> the bb must be provided externally
   virtual BoundingBox3f getBoundingBox();

};


};


#endif
