//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "Vertexarray.h"
//---------------------------------------------------------------------------
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "LinearAlgebra_templCode.h"
#include "PointCloud.h"
#include "UnstructuredInCorePointCloud.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


VertexArray::VertexArray( card8 * buffer, card32 numElements, const VertexDescriptor * desc)
{
	m_Buffer = buffer;
	m_NumElements = numElements;
   initVD( desc );
   m_Iter = NULL;
}

VertexArray::VertexArray( PointSet * ps )
{
	m_Buffer = (card8*)ps->getDataPtr();
	m_NumElements = ps->getNumEntries();
   initVD( ps->getDescr() );
   m_Iter = NULL;
}


VertexArray::VertexArray( PointCloud * pc) 
{
	/*m_Iter = pc->createIterator( SceneObjectIteratorType::CAP_BASIC_PC | SceneObjectIteratorType::CAP_MODIFYABLE_PC) ;
	ModifyablePointCloudIterator * mIt = dynamic_cast<ModifyablePointCloudIterator*>(m_Iter);
	m_Buffer = (card8*)mIt->lockBlockReadWrite()->getDataPtr();
	m_NumElements = pc->getNumPoints();*/
	UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
	m_Buffer = (card8*)upc->getPointSet()->getDataPtr();
	m_NumElements = pc->getNumPoints();
	m_Iter = NULL;
	initVD( pc->getDescr() );
}

VertexArray::~VertexArray(void)
{
	delete m_VD;
	if( m_Iter )
	{
		BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(m_Iter);
		bIt->unlock();
	}
	delete m_Iter;
}

card32 VertexArray::getBufferSize() const
{
   return m_NumElements * m_VD->getSize();
}


void VertexArray::copyVertex( card32 sourceIndex, card32 destIndex )
{
   card8 * destPtr = &m_Buffer[destIndex*m_VD->getSize()];
   card8 * sourcePtr = &m_Buffer[sourceIndex*m_VD->getSize()];
   for( card32 i=0;i<m_VD->getSize();i++ )
      destPtr[i] = sourcePtr[i];
}

void VertexArray::copyVertices( const VertexArray &source)
{
   pAssert( m_Buffer != NULL );
   pAssert( m_VD != NULL );
   pAssert( *m_VD == source.m_VD );
   pAssert( m_NumElements == source.m_NumElements );

   for( card32 i=0;i<m_NumElements * m_VD->getSize();i++ )
   {
      m_Buffer[i] = source.m_Buffer[i];
   }
}

void VertexArray::copyVertex( const VertexArray &sourceArray, card32 sourceIndex,
   card32 destIndex )
{
   pAssert( *m_VD == sourceArray.m_VD );
   pAssert( sourceIndex < sourceArray.m_NumElements );
   pAssert( destIndex < m_NumElements );

   card8 * destPtr = &m_Buffer[destIndex * m_VD->getSize()];
   card8 * sourcePtr = &sourceArray.m_Buffer[sourceIndex * m_VD->getSize()];
   for( card32 i=0;i<m_VD->getSize();i++ )
      destPtr[i] = sourcePtr[i];
}



void VertexArray::initVD( const VertexDescriptor * desc )
{
	m_VD = new VertexDescriptor;
	m_VD->assign( desc );

   m_HasPosition3f = initAAT( POSITION, "position", VAD::DATA_FORMAT_FLOAT32, 3 );
   m_HasColor3f = initAAT( COLOR, "color", VAD::DATA_FORMAT_FLOAT32, 3 );
   m_HasNormal3f = initAAT( NORMAL, "normal", VAD::DATA_FORMAT_FLOAT32, 3 );
   m_HasFlags = initAAT( FLAGS, "flags", VAD::DATA_FORMAT_INT32, 1 );
   m_HasIndex3i = initAAT( INDEX, "index", VAD::DATA_FORMAT_INT32, 3 );
   m_HasTypeID = initAAT( TYPEID, "typeID", VAD::DATA_FORMAT_INT32, 1 );
   m_HasTypeFlag = initAAT( TYPEFLAG, "typeFlag", VAD::DATA_FORMAT_INT32, 1 );
}

bool VertexArray::initAAT( AAT &aat, string name, card32 format, card32 numComponents )
{
    if( !m_VD->providesAttribute( name ) )
      return false;

    VAD vad = m_VD->getAttribute( name );
    aat = m_VD->getToken( name );
    return (format == vad.dataFormat) && (numComponents == vad.numComponents );
}

