//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "VolumeTools.h"
//---------------------------------------------------------------------------
#include "CopyObjectProperties.h"
#include "RegularVolume.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

// a 2x2x2 voxel cube collapses to one voxel weighted equally
// die funktion kann man spter verbessern auch hinsichtlich verchiedenener auflsungen in den dimensionen
// und eventuell einer echten gauss pyramide nicht nur interpolation
// clearandsetup copiert den vertex deskriptor
// den teil spaeter in volume mathtools uebernehmen und diese datei(en) loeschen

RegularVolume *geoxl::downsampleVolumeByTwo(const RegularVolume *vol) 
{
   pAssert(vol);
   card32 sourceWidth = vol->getWidth();
   card32 sourceHeight = vol->getHeight();
   card32 sourceDepth = vol->getDepth();
   card32 destWidth = sourceWidth / 2;
	card32 destHeight = sourceHeight / 2;
	card32 destDepth = sourceDepth / 2;
		
   if (!(destWidth && destHeight && destDepth)) {
      throw PException("xrt::downsampleVolumeByTwo() - volume too small (need at least side lengths of 2)");
   }
   RegularVolume *result = new RegularVolume;
   const VertexDescriptor *descr = vol->getDescr();
   result->clearAndSetup(destWidth, destHeight, destDepth, descr, vol->getBoundingBox());
   const PointSet *sourcePS = vol->getVoxelData();
   PointSet *destPS = result->getVoxelData();
   char *buffer = new char[descr->getSize()];
   const char *ptrs[8];
   float factors[8] = {0.125f, 0.125f, 0.125f, 0.125f, 0.125f, 0.125f, 0.125f, 0.125f};
   for (card32 z=0; z<destDepth; z++) {
      for (card32 y=0; y<destHeight; y++) {
         for (card32 x=0; x<destWidth; x++) {
            card32 xs = x*2;
            card32 ys = y*2;
            card32 zs = z*2;
            ptrs[0] = sourcePS->getDataPointer((xs  ) + (ys  )*sourceWidth + (zs  )*sourceWidth*sourceHeight);
            ptrs[1] = sourcePS->getDataPointer((xs+1) + (ys  )*sourceWidth + (zs  )*sourceWidth*sourceHeight);
            ptrs[2] = sourcePS->getDataPointer((xs  ) + (ys+1)*sourceWidth + (zs  )*sourceWidth*sourceHeight);
            ptrs[3] = sourcePS->getDataPointer((xs+1) + (ys+1)*sourceWidth + (zs  )*sourceWidth*sourceHeight);
            ptrs[4] = sourcePS->getDataPointer((xs  ) + (ys  )*sourceWidth + (zs+1)*sourceWidth*sourceHeight);
            ptrs[5] = sourcePS->getDataPointer((xs+1) + (ys  )*sourceWidth + (zs+1)*sourceWidth*sourceHeight);
            ptrs[6] = sourcePS->getDataPointer((xs  ) + (ys+1)*sourceWidth + (zs+1)*sourceWidth*sourceHeight);
            ptrs[7] = sourcePS->getDataPointer((xs+1) + (ys+1)*sourceWidth + (zs+1)*sourceWidth*sourceHeight);
            char *destPtr = destPS->getDataPointer(x + y*destWidth +z*destWidth*destHeight);
            descr->interpolate(ptrs, factors, 8, destPtr);
         }
      }
   }
   return result;
}
