//---------------------------------------------------------------------------
#ifndef ImageDisplayerWidgetH
#define ImageDisplayerWidgetH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "PopupWindow.h"
#include "RegularData.h"
//---------------------------------------------------------------------------
#include <QtGui/QWidget.h>
//---------------------------------------------------------------------------
#include "ImageDisplayerWidget_ui.h"
#include "IDWImagePanel_ui.h"
#include "IDWMappingPanel_ui.h"
#include "IDWFileIOPanel_ui.h"
//---------------------------------------------------------------------------

namespace geoxl {

class PointSet;
class RGBAFloat32Image;
					
class POINTCLOUDS_API ImagePopup : public PopupWindow {
	Q_OBJECT
 private slots:
   void on_spinBoxMaxHeight_valueChanged(int i);
   void on_doubleSpinBoxZoom_valueChanged(double);
 public:
	Ui_IDWImagePanel ui;
	ImagePopup(QWidget *parent);
 signals:
	void changed();
	void newMaxHeight(int);
};

// --

class POINTCLOUDS_API MappingPopup : public PopupWindow {
	Q_OBJECT
 private slots:
	void on_dsbScale_valueChanged(double);
	void on_dsbOffset_valueChanged(double);
	void on_dsbGamma_valueChanged(double);
	void on_cbAuto_stateChanged(int);

 public:
	Ui_IDWMappingPanel ui;
	bool noSignals;

	MappingPopup(QWidget *parent);

 signals:
	void changed();
};

// --

class POINTCLOUDS_API FileIOPopup : public PopupWindow {
	Q_OBJECT

 private slots:
	void on_pbSaveNow_clicked();
	void on_pbCopyToConsole_clicked();
	
 public:
	Ui_IDWFileIOPanel ui;
	FileIOPopup(QWidget *parent);

 signals:
	void saveNow();
	void copyToConsole();
};
										  

// --


class POINTCLOUDS_API ImageDisplayerWidget : public QWidget {
	Q_OBJECT
 
 private:
	Ui_ImageDisplayerWidget ui;
	ImagePopup *imgPopup;
	MappingPopup *mapPopup;
	FileIOPopup *filePopup;
	
	PointSet *psImage;
	string currentChannel;
	int maxHeight;
	bool deactivateChannelChoice;
	 
	Vector3i mapColor(Vector3f col, float32 scale, float32 offset, float32 gamma);
	bool createImage(const PointSet *ps, QImage &img, double &minv, double &maxv);
	void rethinkLayout();
	 
 private slots:
	void on_pushButtonImage_clicked();
	void on_pushButtonMapping_clicked();
	void on_pushButtonFileIO_clicked();
	void changed();
	void newMaxHeight(int i);
	void on_cbChannel_currentIndexChanged(int);
	void on_cbChannel_editTextChanged(const QString &text);
	void saveNow();
	void copyToConsole();
	void on_pbZoomDouble_clicked();
	void on_pbZoomHalf_clicked();
	
 public:
	ImageDisplayerWidget(QWidget *parent=NULL, Qt::WFlags f=0);
	
	void displayPointSet(const PointSet *ps, string channel = "");
	void displayQImage(const QImage *img);
	void display2DRegularData(const RegularDataF *img);
	void displayRGBAFloat32Image(const RGBAFloat32Image *img);
	
	static const card32 PNG = 0;
	static const card32 JPG = 1;
	card32 getLoggingFormat();
	QPixmap getCurrentImage();
	
	void changeZoom(double factor);
	void changeMapping(double offset, double scale, double gamma);
	
	~ImageDisplayerWidget();
};

}

#endif