#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "VertexDescriptorEditorWidget.h"
//---------------------------------------------------------------------------
#include "VertexDescriptor.h"
#include "StringHelper.h"
#include <QtCore/QStringList.h>
#include <QtGui/QHeaderView.h>
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( VertexDescriptor )

VertexDescriptorEditorWidget::VertexDescriptorEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	// used Qt auto connect feature for slots, see public slots in header file

	// init combo boxes
	ui.ComboDim->addItems(QStringList() << "1" << "2" << "3" << "4" << "5" << "6" << "7" << "8" << "9" << "10");
	ui.ComboDataType->addItems(QStringList() << "Float32" << "Card8" << "Int32");
	ui.ComboDim->setCurrentIndex(2);
	ui.ComboDataType->setCurrentIndex(0);

	// set header for vd list
	ui.AttributeList->setColumnCount(5);
	ui.AttributeList->setHeaderLabels(QStringList() << "ATTR #" << "Identifier" << "Dim" << "Datatype" << "");
	ui.AttributeList->resizeColumnToContents(0);
	ui.AttributeList->resizeColumnToContents(1);
	ui.AttributeList->resizeColumnToContents(2);
	ui.AttributeList->resizeColumnToContents(3);

	// init the rest
   m_VertexDescriptor = NULL;

}

VertexDescriptorEditorWidget::~VertexDescriptorEditorWidget(void)
{
}

void VertexDescriptorEditorWidget::setup( VertexDescriptor * obj )
{
   m_VertexDescriptor = obj;
   update();
}

void VertexDescriptorEditorWidget::updateEditor()
{
   if( m_VertexDescriptor )
   {
      m_InternalVD.clear();
		ui.AttributeList->clear();

		for( card32 i=0;i<m_VertexDescriptor->getNumAttribs();i++ )
      {
         VAD tmp = m_VertexDescriptor->getAttribute(i);
         card8 dataFormat=0xff;
         switch( tmp.dataFormat )
         {
         case VAD::DATA_FORMAT_FLOAT32:
            dataFormat = FORMAT_FLOAT32;
         break;
         case VAD::DATA_FORMAT_CARD8:
            dataFormat = FORMAT_CARD8;
         break;
         case VAD::DATA_FORMAT_INT32:
            dataFormat = FORMAT_INT32;
         break;
         }
         internalAdd( tmp.name, (card8)tmp.numComponents, (card8)dataFormat );
         addListItem( i, tmp.name, (card8)tmp.numComponents, (card8)dataFormat );
      }
   }
}



// ----------- user functions ----------

void VertexDescriptorEditorWidget::addListItem( card32 index, const string &name, card8 numComp, card8 dataFormat )
{
   string attrNr;
   if( index >= 10 )
      attrNr += (char)((index / 10) % 10 + '0');
   attrNr += (char)(index % 10 + '0');

   // add to listview
	QStringList s;
	s << attrNr.c_str();
	s << name.c_str();
	if ((numComp<1) || (numComp>10)) s << "unknown";
		else s << intToStr(numComp).c_str();

   QString formatStr = "unknown";
   switch( dataFormat )
   {
		case FORMAT_FLOAT32: formatStr = "Float32"; break;
		case FORMAT_CARD8: formatStr = "Card8"; break;
		case FORMAT_INT32: formatStr = "Int32"; break;
   }
	s << formatStr;

	QTreeWidgetItem *li = new QTreeWidgetItem(ui.AttributeList, s);
}


void VertexDescriptorEditorWidget::internalApply()
{
   if( m_VertexDescriptor )
   {
      m_VertexDescriptor->clear();
		ui.AttributeList->clear();

		for( card32 i=0;i<m_InternalVD.size();i++ )
      {
         VAttrib a = m_InternalVD[i];
         card32 dataFormat=0xfffff;
         switch( a.dataFormat )
         {
            case FORMAT_FLOAT32:
               dataFormat = VAD::DATA_FORMAT_FLOAT32;
            break;
            case FORMAT_CARD8:
               dataFormat = VAD::DATA_FORMAT_CARD8;
            break;
            case FORMAT_INT32:
               dataFormat = VAD::DATA_FORMAT_INT32;
            break;
         }
         m_VertexDescriptor->pushAttrib( VAD(a.name, (card8)a.numComp, (card8)dataFormat) );
         addListItem( i, a.name, a.numComp, a.dataFormat );
      }
   }
}


void VertexDescriptorEditorWidget::internalAdd(const string &name, card8 numComp, card8 dataFormat )
{
   // add to internal
   VAttrib a;
   a.name = name;
   a.numComp = numComp;
   a.dataFormat = dataFormat;
   m_InternalVD.push_back(a);
}


void VertexDescriptorEditorWidget::internalDelete(card32 index)
{
   if( m_InternalVD.size() > index )
   {
      card32 i=0;
      for( vector<VAttrib>::iterator iter = m_InternalVD.begin();
         iter != m_InternalVD.end(); ++iter )
            if( i==index )
            {
               m_InternalVD.erase(iter);
               break;
            }
            else
               i++;
   }
}



// ----------- slots ----------

void VertexDescriptorEditorWidget::on_BTAdd_clicked() {
   if( !m_VertexDescriptor )
      return;

	string name = qPrintable(ui.LEAttribName->text());
   if( m_VertexDescriptor->providesAttribute( name ) )
      return;

	card32 numComp = ui.ComboDim->currentIndex()+1;
	card32 dataFormat = ui.ComboDataType->currentIndex();
   internalAdd( name, (card8)numComp, (card8)dataFormat );
   internalApply();
};


void VertexDescriptorEditorWidget::on_BTDelete_clicked() {
	if (ui.AttributeList->currentIndex().isValid())
   {
		internalDelete( ui.AttributeList->currentIndex().row() );
      internalApply();
   }
};


void VertexDescriptorEditorWidget::on_BTUp_clicked() {
	int32 currentRow = ui.AttributeList->currentIndex().row();
	if (ui.AttributeList->currentIndex().isValid() && (currentRow>0))  // item selected & ! first item ?
   {
      VAttrib a = m_InternalVD[currentRow];
      m_InternalVD[currentRow] = m_InternalVD[currentRow-1];
      m_InternalVD[currentRow-1] = a;
      internalApply();
   }
	QList<QTreeWidgetItem *> foundItems = ui.AttributeList->findItems(QString(intToStr(currentRow-1).c_str()), Qt::MatchExactly|Qt::MatchWrap, 0);
	if (!foundItems.empty()) {
		ui.AttributeList->setCurrentItem(foundItems.first());
	};
};


void VertexDescriptorEditorWidget::on_BTDown_clicked() {
	int32 currentRow = ui.AttributeList->currentIndex().row();
	if (ui.AttributeList->currentIndex().isValid() && (currentRow<ui.AttributeList->topLevelItemCount()-1))  // item selected & ! last item ?
   {
      VAttrib a = m_InternalVD[currentRow];
      m_InternalVD[currentRow] = m_InternalVD[currentRow+1];
      m_InternalVD[currentRow+1] = a;
      internalApply();
   }
	QList<QTreeWidgetItem *> foundItems = ui.AttributeList->findItems(QString(intToStr(currentRow+1).c_str()), Qt::MatchExactly|Qt::MatchWrap, 0);
	if (!foundItems.empty()) {
		ui.AttributeList->setCurrentItem(foundItems.first());
	};
};


#include "VertexDescriptorEditorWidget_moc.cpp"



