//---------------------------------------------------------------------------
#include "stdafx.h"
 //---------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------
#include "InitRendering.h"
//---------------------------------------------------------------------------

/*
	Please separate between system code and interface code ( editors ), in 
	order to build a non-interface commandline version of geoxl.

File structure:
	includes
		..
		..
	interface includes
		..
		..
	initRendering()
	{
		init geoxl classes
		..
		..
		init geoxl class editors
		..
		..
	}
	shutdownRendering()
	{
		shutdown geoxl classes
		..
		..
		shutdown geoxl class editors
		..
		..
				
	}
*/

//---------------------------------------------------------------------------
#include "CubeTextureAttachment.h"
#include "LCTraversal.h"
#include "MaterialAttachment.h"
#include "MaterialParameter.h"
#include "SimplePCMaterial.h"
#include "GLShaderMaterial.h"
#include "GLSimpleVolumeMaterial.h"
#include "GLVolumeMaterialStandard.h"
#include "GLVolumeMaterialGeneral.h"
#include "GLTexture.h"
#include "Renderer.h"
#include "Shader.h"
#include "RGBATextureAttachment.h"
#include "LCHierarchyRenderer.h"
#include "MPContainer.h"
#include "TwoPassBlendingRenderer.h"
#include "InterpFunctionTextureAttachment.h"
#include "SceneCreator.h"
#include "Scene.h"
#include "LegacyClassNameTranslation.h"
#include "SimpleGLTriMeshMaterial.h"
#include "GLVolumeDisplayer.h"
#include "GLVolumeRenderer2DSlices.h"
#include "UnitTests.h"
#include "RenderingTool.h"
#include "InterpFunctionTextureAttachmentRGB.h"
#include "PointsAsSpheresMaterial.h"
#include "DirectionsMaterial.h"
// ---------------------------------------------------------------------------------
//					Interface Includes
// ---------------------------------------------------------------------------------
#ifdef GEOXL_GUI

#include "CubeTextureAttachmentEditorWidget.h"
#include "RGBATextureAttachmentEditorWidget.h"
#include "ShaderEditorWidget.h"
#include "GLShaderMaterialEditorWidget.h"
#include "InterpFunctionTextureAttachmentEditorWidget.h"
#include "InterpFunctionTextureAttachmentRGBEditorWidget.h"

#endif	// end of #ifdef GEOXL_GUI

#include "DebugRenderer.h"

//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


void geoxl::initRendering()
{

// ---------------------------------------------------------------------------------
//					Class Registration ( no Editors ! )
// ---------------------------------------------------------------------------------

   GLTextureSettings::init(Persistent::getClass());

   Renderer::init(Persistent::getClass());
      LCHierarchyRenderer::init(Renderer::getClass());
      TwoPassBlendingRenderer::init(Renderer::getClass() );

   Shader::init(Persistent::getClass());

	// GLMaterial (InitSceneGraph) ---
		GLShaderMaterial::init(GLMaterial::getClass());
		SimplePCMaterial::init(GLMaterial::getClass());
		SimpleGLTriMeshMaterial::init(GLMaterial::getClass());
		GLSimpleVolumeMaterial::init(GLMaterial::getClass());
		PointsAsSpheresMaterial::init(GLMaterial::getClass());
		DirectionsMaterial::init(GLMaterial::getClass());
		GLVolumeMaterialStandard::init(GLMaterial::getClass());
		GLVolumeMaterialGeneral::init(GLMaterial::getClass());
		
   GLRenderObject::init(Object::getClass());
	   GLImModePtsRenderObject::init(GLRenderObject::getClass());

   MPContainer::init( Persistent::getClass() );
   MaterialParameter::init( Persistent::getClass() );
      MPMatrix4x4::init( MaterialParameter::getClass() );
      MPSingleFloat::init( MaterialParameter::getClass() );
      MPTexture::init( MaterialParameter::getClass() );
      MPVector4::init( MaterialParameter::getClass() );

   // AttachedData (InitFoundation)---
      Matrix4fAttachment::init( AttachedData::getClass() );
      TextureAttachment::init( AttachedData::getClass() );
         CubeTextureAttachment::init( TextureAttachment::getClass() );
         CubeTextureFunctionAttachment::init( TextureAttachment::getClass() );
         RGBATextureAttachment::init( TextureAttachment::getClass() );
         InterpFunctionTextureAttachment::init( TextureAttachment::getClass() );
		 InterpFunctionTextureAttachmentRGB::init( TextureAttachment::getClass());
      Vector4fAttachment::init( AttachedData::getClass() );
      TextureStackAttachment::init(AttachedData::getClass());
      
   CubeFunction3toN::init(Persistent::getClass());
      CubeFunctionIdentity::init(CubeFunction3toN::getClass());

   LCState::init(Persistent::getClass());
   LCTraversal::init(Persistent::getClass());


   SceneCreator::init(ClassCreator::getClass());

	//   SceneCreatorEditor::init(ClassEditor::getClass());
	const_cast<MetaClass*>(Scene::getClass())->getGUIDescriptor()->addClassCreator(SceneCreator::getClass());

	GLVolumeDisplayer::init(GLEventObject::getClass());
   GLVolumeRenderer2DSlices::init(Persistent::getClass());


	LegacyClassNameTranslation::registerTranslation("PCShaderMaterial", GLShaderMaterial::getClass());


	UnitTests::addTest(glEventDisplayer_unitTest, "glEventDisplayer_unitTest");
	RenderingTool::init( Persistent::getClass() );


// ---------------------------------------------------------------------------------
//					Editor Class Registration 
// ---------------------------------------------------------------------------------
#ifdef GEOXL_GUI
	
	CubeTextureAttachmentEditor::init(ClassEditor::getClass());
		((GUIDescriptor * )CubeTextureAttachment::getClass()->getGUIDescriptor())->addClassEditor(
		CubeTextureAttachmentEditor::getClass(), true);

	RGBATextureAttachmentEditor::init(ClassEditor::getClass());
		((GUIDescriptor * )RGBATextureAttachment::getClass()->getGUIDescriptor())->addClassEditor(
		RGBATextureAttachmentEditor::getClass(), true);

	ShaderEditor::init(ClassEditor::getClass());
    ((GUIDescriptor * )Shader::getClass()->getGUIDescriptor())->addClassEditor(
    ShaderEditor::getClass(), true);

	GLShaderMaterialEditor::init(ClassEditor::getClass());
    ((GUIDescriptor * )GLShaderMaterial::getClass()->getGUIDescriptor())->addClassEditor(
    GLShaderMaterialEditor::getClass(), true);

	InterpFunctionTextureAttachmentEditor::init( ClassEditor::getClass());
	((GUIDescriptor * )InterpFunctionTextureAttachment::getClass()->getGUIDescriptor())->addClassEditor(
		InterpFunctionTextureAttachmentEditor::getClass(), true);
	
	InterpFunctionTextureAttachmentRGBEditor::init( ClassEditor::getClass());
	((GUIDescriptor * )InterpFunctionTextureAttachmentRGB::getClass()->getGUIDescriptor())->addClassEditor(
		InterpFunctionTextureAttachmentRGBEditor::getClass(), true);

	

 
#endif	// end of #ifdef GEOXL_GUI

}


void geoxl::shutdownRendering()
{
// ---------------------------------------------------------------------------------
//					Class Shutdown
// ---------------------------------------------------------------------------------

   GLTextureSettings::shutdown();

   Renderer::shutdown();
   LCHierarchyRenderer::shutdown();
   TwoPassBlendingRenderer::shutdown();

   Shader::shutdown();

	GLShaderMaterial::shutdown();
	SimplePCMaterial::shutdown();
	SimpleGLTriMeshMaterial::shutdown();
	GLSimpleVolumeMaterial::shutdown();
	PointsAsSpheresMaterial::shutdown();
	DirectionsMaterial::shutdown();
	GLVolumeMaterialStandard::shutdown();
	GLVolumeMaterialGeneral::shutdown();

   GLRenderObject::shutdown();
   GLImModePtsRenderObject::shutdown();

   MPContainer::shutdown();
   MaterialParameter::shutdown();
   MPMatrix4x4::shutdown();
   MPSingleFloat::shutdown();
   MPTexture::shutdown();
   MPVector4::shutdown();

   Matrix4fAttachment::shutdown();
   TextureAttachment::shutdown();
   CubeTextureAttachment::shutdown();
   CubeTextureFunctionAttachment::shutdown();
   RGBATextureAttachment::shutdown();
   InterpFunctionTextureAttachment::shutdown();
   InterpFunctionTextureAttachmentRGB::shutdown();
   Vector4fAttachment::shutdown();
   TextureStackAttachment::shutdown();

   CubeFunction3toN::shutdown();
   CubeFunctionIdentity::shutdown();

   LCState::shutdown();
   LCTraversal::shutdown();



   SceneCreator::shutdown();
	//   SceneCreatorEditor::::shutdown();

	GLVolumeDisplayer::shutdown();
	GLVolumeRenderer2DSlices::shutdown();
	RenderingTool::shutdown();


// ---------------------------------------------------------------------------------
//					Editor Class Shutdown
// ---------------------------------------------------------------------------------
#ifdef GEOXL_GUI
	
	CubeTextureAttachmentEditor::shutdown();
	RGBATextureAttachmentEditor::shutdown();
	ShaderEditor::shutdown();
	GLShaderMaterialEditor::shutdown();
	InterpFunctionTextureAttachmentEditor::shutdown();
	InterpFunctionTextureAttachmentRGBEditor::shutdown();
 
#endif	// end of #ifdef GEOXL_GUI

   delete debugRenderer;
   debugRenderer = NULL;

}
