//---------------------------------------------------------------------------
#ifndef CubeTextureAttachmentH
#define CubeTextureAttachmentH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MaterialAttachment.h"
#include "Persistent.h"
//---------------------------------------------------------------------------
#include <vector>
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;

class RGBACubeMap;


class RENDERING_API CubeTextureAttachment : public TextureAttachment
{
private:
   static MetaClass *CubeTextureAttachmentMetaClass;
   static Object* newInstance();

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign( const Object * obj, CopyContext * context );

private:

   // properties
   RGBACubeMap  *    m_CubeMap;
   bool              m_MagFilterLinear;
   bool              m_MinFilterLinear;



public:

   CubeTextureAttachment();
   virtual ~CubeTextureAttachment();

   void setMinFilterLinear( bool b ) {m_MinFilterLinear=b;}
   void setMagFilterLinear( bool b ) {m_MagFilterLinear=b;}
   bool getMinFilterLinear() const {return m_MinFilterLinear;}
   bool getMagFilterLinear() const {return m_MagFilterLinear;}

   virtual card32 getTextureType() const {return TYPE_CUBE; }
   virtual GLTexture * createGLTexture(GLContext * context);
//   virtual bool needsUpdate();

   RGBACubeMap * getCubeMap() {return m_CubeMap;}

//   void setNeedsUpdate() {m_NeedsUpdate = true;}

};


class CubeFunction3toN;

class RENDERING_API CubeTextureFunctionAttachment : public TextureAttachment
{
private:
   static MetaClass *CubeTextureFunctionAttachmentMetaClass;
   static Object* newInstance();

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign( const Object * obj, CopyContext * context );

private:

   void calcCubemap();

   static const int FACE_POSX = 0;
   static const int FACE_POSY = 1;
   static const int FACE_POSZ = 2;
   static const int FACE_NEGX = 3;
   static const int FACE_NEGY = 4;
   static const int FACE_NEGZ = 5;
   vector<card8>        m_CubemapFace[6];   // temporal data
   card32               m_ActSize;
   card32               m_ActFunctionVersion;

   card32               m_Size;
   CubeFunction3toN  *  m_Function;


public:

   CubeTextureFunctionAttachment();
   virtual ~CubeTextureFunctionAttachment();

   virtual card32 getTextureType() const {return TYPE_CUBE; }
   virtual GLTexture * createGLTexture(GLContext * context);
   virtual bool needsUpdate();

//   virtual void write(OutputObjectStream *s) const;
//   virtual void read(InputObjectStream *s);
};


class RENDERING_API CubeFunction3toN : public Persistent
{
private:
   static MetaClass *CubeFunction3toNMetaClass;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

protected:
   card32   m_ImageDimension;
   card32   m_Version;     /// parameter change --> m_Version++

public:

   card32   getImageDimension() const {return m_ImageDimension;}
   virtual void calculate( float x, float y, float z, float * out ) = 0;
   card32 getVersion() const {return m_Version;}

};

/// -> Normalization CubeMap
class RENDERING_API CubeFunctionIdentity : public CubeFunction3toN
{
private:
   static MetaClass *CubeFunctionIdentityMetaClass;
   static Object* newInstance();

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   CubeFunctionIdentity()
   {
      m_ImageDimension = 3;
      m_Version = 1;
   }

   virtual void calculate( float x, float y, float z, float * out )
   {
      out[0] = x;
      out[1] = y;
      out[2] = z;
   }

};


}

#endif
