//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "GLAttachments.h"
//---------------------------------------------------------------------------
#include "GLContext.h"
#include "GLBuffer.h"
#include "GLTexture.h"
#include "DebugOutput.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------


SharedVertexBufferAttachment::SharedVertexBufferAttachment()
:MemoryManagedAttachment()
{
   m_VertexBuffer = NULL;
}

SharedVertexBufferAttachment::~SharedVertexBufferAttachment()
{
   delete m_VertexBuffer;
}

card32 SharedVertexBufferAttachment::getManagedSize()
{
   if( m_VertexBuffer )
      return m_VertexBuffer->getSizeInBytes();
   else
   {
      pAssert(false);
      return 0;
   }
}

void SharedVertexBufferAttachment::setupMMA( const string &name, MemoryManager * mm, bool deleteOnUpdate )
{
   MemoryManagedAttachment::setupMMA( name, mm, deleteOnUpdate );
}

void SharedVertexBufferAttachment::setupVB(GLVertexBuffer * vbuffer, const string &addCustomID,
	MemoryManager * mm, bool deleteOnUpdate, const int32 includeFlags, const int32 excludeFlags, 
   Vector3f lastDirection, Matrix4f lastCTM)
{
   m_VertexBuffer = vbuffer;
   this->lastDirection = lastDirection;
   this->lastCTM = lastCTM;
   setupMMA( generateID( vbuffer->getVertexDescriptor(), vbuffer->getRootContext(), addCustomID, includeFlags, excludeFlags ), mm, deleteOnUpdate );
}

string SharedVertexBufferAttachment::generateID( const VertexDescriptor * vdesc, GLContext * context,
      const string &addCustomID, const int32 includeFlags, const int32 excludeFlags )
{
   char tmpbuf[64];
   _i64toa( context->getContextNumber(), tmpbuf, 10 );
   char incl[64];
   itoa(includeFlags, incl, 10);
   char excl[64];
   itoa(excludeFlags, excl, 10);
   string result = vdesc->getDescription() + string("_C") + string( tmpbuf ) + addCustomID + string("_I") + string(incl) + string("_E") + string(excl);
   return result;
}

GLTextureAttachment::GLTextureAttachment()
   :MemoryManagedAttachment()
{
   m_Texture = NULL;
}

GLTextureAttachment::~GLTextureAttachment()
{
   delete m_Texture;
}

void GLTextureAttachment::setupTA( GLTexture * texture, const string &addCustomID,
   MemoryManager * memManager, bool deleteOnUpdate )
{
   m_Texture = texture;
   setupMMA( generateID( texture->getRootContext(), addCustomID ), memManager, deleteOnUpdate );
}

string GLTextureAttachment::generateID( GLContext * context, const string &addCustomID )
{
   char tmpbuf[64];
   for( card32 i=0;i<64;i++ )
      tmpbuf[i] = 0;
   itoa( (int)context, tmpbuf, 10 );
   tmpbuf[63] = 0;

   return string( tmpbuf ) + addCustomID;
}

card32 GLTextureAttachment::getManagedSize()
{
   if( m_Texture )
      return m_Texture->getSizeInBytes();
   else
   {
      pAssert(false);
      return 0;
   }
}
