//---------------------------------------------------------------------------
#ifndef GLAttachmentsH
#define GLAttachmentsH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MemoryManagedAttachment.h"
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;

class GLVertexBuffer;
class GLTexture;


class SharedVertexBufferAttachment : public MemoryManagedAttachment
{
private:

   GLVertexBuffer *  m_VertexBuffer;

protected:
   virtual card32 getManagedSize();

   // force using setupVB
   virtual void setupMMA( const string &name, MemoryManager * mm, bool deleteOnUpdate=true);

public:

	/// hints for sorting
	Vector3f lastDirection;
	/// hints for sorting
	Matrix4f lastCTM;

   SharedVertexBufferAttachment();
   virtual ~SharedVertexBufferAttachment();

   GLVertexBuffer * getVertexBuffer() const {return m_VertexBuffer;}

   /// setup
   /// SharedVertexBuffer becomes owner of vbuffer!
   /// uses generateID to determine attachment - name
   void setupVB( GLVertexBuffer * vbuffer, const string &addCustomID, MemoryManager * mm,
      bool deleteOnUpdate, const int32 includeFlags, const int32 excludeFlags, 
      Vector3f lastDirection, Matrix4f lastCTM );

   /// generates ID-String with
   /// vdesc->getDescription() + intToStr( (int)context ) + addCustomID
   static string generateID( const VertexDescriptor * vdesc, GLContext * context,
      const string &addCustomID, const int32 includeFlags, const int32 excludeFlags );

};


class GLTextureAttachment : public MemoryManagedAttachment
{
private:
   GLTexture   *  m_Texture;

protected:

   virtual card32 getManagedSize();

public:

   GLTextureAttachment();
   ~GLTextureAttachment();

   /// init attachment
   /// name = generateID( texture->gerRootContext(), addCustomID )
   void setupTA( GLTexture * texture, const string &addCustomID, MemoryManager * memManager, bool deleteOnUpdate=true );

   /// generates ID-String with
   /// intToStr( (int)context ) + addCustomID
   static string generateID( GLContext * context, const string &addCustomID = "" );

   GLTexture   *  getTexture() {return m_Texture;}



};



}

#endif
