//---------------------------------------------------------------------------
#ifndef GLVolumeRenderer2DSlicesH
#define GLVolumeRenderer2DSlicesH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "VariableArrayClassProperty.h"
//---------------------------------------------------------------------------

namespace geoxl {

class Tex2dVolumeRenderer;
class Camera;
class Shader;
class RegularVolume;

///
///   simple volume renderer that uses 2d slices and provides some setup options (gamma, transfer functions etc.)
///
class RENDERING_API GLVolumeRenderer2DSlices : public Persistent {
	GEOXL_CLASS( GLVolumeRenderer2DSlices )

 public:
	void invalidate();

 private:

	bool showCoords;
	bool needRebuild;
	card32 automatic;

	vector<string> channels;
	
	int transferFunction;
		
	float32 scale;
	float32 offset;
	float32 gamma;
	
   Shader *vertexShader;
   Shader *pixelShader;

	inline int32 mapValue(float32 v);
	static Vector4ub whiteOpacityMapping(void *context, const StaticVector<float32, 1> &in);
	static Vector4ub intensityOpacityMapping(void *context, const StaticVector<float32, 1> &in);
	static Vector4ub blueRedOpacityMapping(void *context, const StaticVector<float32, 1> &in);

   IMPLEMENT_VAR_STRING_ARRAY_METHODS_STL_UPDATE(channels, invalidate)

 public:
	GLVolumeRenderer2DSlices();

	virtual void assign(const Object* obj, CopyContext *context = NULL);

   /// draw volume, needs a texture stack to perform drawing (might be updated in the process of rendering)
   /// the textureStack could e.g. be cached in an attachment of a volume object
   void drawGL(RegularVolume *volume, Tex2dVolumeRenderer *&textureStack, Camera &cam, float32 minValue, float32 maxValue);

	bool getShowCoords() const {return showCoords;}
	void setShowCoords(bool v) {showCoords=v;}
	bool getAutomatic() const {return automatic;}
	void setAutomatic(bool v) {automatic=v; invalidate();}
	string getChannel() const;
	void setChannel(string v);
	int getTransferFunction() const {return transferFunction;}
	void setTransferFunction(int v) {transferFunction=v; invalidate();}
	float32 getScale() const {return scale;}
	void setScale(float32 v) {scale=v; invalidate();}
	float32 getOffset() const {return offset;}
	void setOffset(float32 v) {offset=v; invalidate();}
	float32 getGamma() const {return gamma;}
	void setGamma(float32 v) {gamma=v; invalidate();}

   ~GLVolumeRenderer2DSlices();
};

}

#endif