//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "InterpFunctionTextureAttachment.h"
//---------------------------------------------------------------------------
#include "GLContext.h"
#include "GLTexture.h"
#include <glh/glh_extensions.h>
//#include "InterpFunctionTextureAttachmentEditor.h"
#include "DebugOutput.h"
#include "LinearAlgebra_templCode.h"
#include "ObjectStreams.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------

// ################################ PERSISTENT REG ###############################################
// ################################ PERSISTENT REG ###############################################
// ################################ PERSISTENT REG ###############################################

MetaClass* InterpFunctionTextureAttachment::InterpFunctionTextureAttachmentMetaClass = NULL;

const MetaClass* InterpFunctionTextureAttachment::getInstanceClass() const {
   return getClass();
}

Object* InterpFunctionTextureAttachment::newInstance() {
   return new InterpFunctionTextureAttachment;
}

const MetaClass* InterpFunctionTextureAttachment::getClass() {
   if (InterpFunctionTextureAttachmentMetaClass==NULL) {
      InterpFunctionTextureAttachmentMetaClass = new MetaClass("InterpFunctionTextureAttachment", "no description.",
                                  newInstance, sizeof(InterpFunctionTextureAttachment));
   }
   return InterpFunctionTextureAttachmentMetaClass;
}

void InterpFunctionTextureAttachment::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass , MetaClass::FIRST_VERSION + 0);
   InterpFunctionTextureAttachmentMetaClass->copyParentProperties();
//   InterpFunctionTextureAttachmentMetaClass->getGUIDescriptor()->addClassEditor(InterpFunctionTextureAttachmentEditor::getClass(), true);
}

void InterpFunctionTextureAttachment::shutdown() {
   delete InterpFunctionTextureAttachmentMetaClass;
}

// -------------------------------- PERSISTENT REG -----------------------------------------------
// -------------------------------- PERSISTENT REG -----------------------------------------------
// -------------------------------- PERSISTENT REG -----------------------------------------------

float minDist = 0.01f;

InterpFunctionTextureAttachment::InterpFunctionTextureAttachment()
{
   TPoint p1,p2;
   p1.p = makeVector2f(0,0);
   p2.p = makeVector2f(1,1);
   m_Points.push_back( p1 );
   m_Points.push_back( p2 );
   m_TextureResolution = 128;
}

InterpFunctionTextureAttachment::~InterpFunctionTextureAttachment()
{
}

void InterpFunctionTextureAttachment::assign( const Object * obj, CopyContext * context )
{
   TextureAttachment::assign( obj, context );
   InterpFunctionTextureAttachment * t =
      (InterpFunctionTextureAttachment *)obj;
   m_Points = t->m_Points;
   m_TextureResolution = t->m_TextureResolution;
}

void InterpFunctionTextureAttachment::write(OutputObjectStream *s) const
{
   TextureAttachment::write(s);
   s->write( m_TextureResolution );
   card32 numP = (card32)m_Points.size();
   s->write(numP);
   for( card32 i=0;i<numP;i++ )
      s->write( m_Points[i] );
}

void InterpFunctionTextureAttachment::read(InputObjectStream *s)
{
   TextureAttachment::read(s);

   s->read( m_TextureResolution );
   card32 numP;
   s->read( numP );
   m_Points.clear();
   for( card32 i=0;i<numP;i++ )
   {
      TPoint p;
      s->read(p);
      m_Points.push_back( p );
   }
}


GLTexture * InterpFunctionTextureAttachment::createGLTexture( GLContext * context )
{
   card32 yDim = 16;
   vector<card32> tmpData = createImage(m_TextureResolution,yDim);

   GLTexture2D * tex2D = new GLTexture2D( context );
   tex2D->createFromMemory( m_TextureResolution, yDim, 4, (char*)&tmpData[0], false );
   tex2D->bind();
   glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_MIN_FILTER, GL_LINEAR );
   glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_MAG_FILTER, GL_LINEAR );
   glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
   glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );

   return tex2D;
}

void InterpFunctionTextureAttachment::addPoint( TPoint point )
{
	point.p[0] = min(1.0f,max(0.0f,point.p[0]));
	point.p[1] = min(1.0f,max(0.0f,point.p[1]));
   card32 n = findPoint( point.p[0] );

   if( m_Points[n].p[0] > point.p[0] )
      if( n == 0 )
         return;
      else
         n--;

   if( fabs( m_Points[n].p[0] - point.p[0] ) > minDist )   // don't add points too near to each other
   {
      m_Points.insert( m_Points.begin() + n + 1, point );
   }

}

void InterpFunctionTextureAttachment::deletePoint( float x )
{
   if( m_Points.size() == 2 )
      return;

   card32 n = findPoint( x );
   if( n == 0 ) n++;
   if( n == m_Points.size()-1 ) n--;


   m_Points.erase( m_Points.begin() + n  );
}

card32 InterpFunctionTextureAttachment::getPointIndex( float x, float y )
{
   Vector2f vec = makeVector2f( x,y );
   float dist = norm( vec - m_Points[0].p );
   card32 n = 0;
   for( card32 i=1;i<m_Points.size();i++ )
   {
      float tmpDist = norm( vec - m_Points[i].p );
      if( dist > tmpDist )
      {
         dist = tmpDist;
         n = i;
      }
   }
   return n;
}

void InterpFunctionTextureAttachment::movePoint( card32 n, Vector2f dir )
{
   if( (n == 0) || (n == m_Points.size()-1) )
   {
      dir[0] = 0;
   }
   else
   {
      if( dir[0] > 0 )
      {
         if( m_Points[n].p[0] + dir[0] > m_Points[n+1].p[0] - minDist )
            dir = makeVector2f(0,0);
      }
      else
      {
         if( m_Points[n].p[0] + dir[0] < m_Points[n-1].p[0] + minDist )
            dir = makeVector2f(0,0);
      }
   }

   m_Points[n].p += dir;
   if( m_Points[n].p[1] > 1 )
      m_Points[n].p[1] = 1;
   if( m_Points[n].p[1] < 0 )
      m_Points[n].p[1] = 0;

}

card32 InterpFunctionTextureAttachment::findPoint( float x )
{
   float dist = fabs( x - m_Points[0].p[0] );
   card32 n = 0;
   for( card32 i=1;i<m_Points.size();i++ )
   {
      float tmpDist = fabs( x - m_Points[i].p[0] );
      if( dist > tmpDist )
      {
         dist = tmpDist;
         n = i;
      }
   }
   return n;
}

void InterpFunctionTextureAttachment::setupEditorHistogram( const vector<float> &histogram ) 
{
	m_Histogram = histogram;
}

vector<card32> InterpFunctionTextureAttachment::createImage( card32 dimX, card32 dimY ) 
{
	vector<card32> tmpData( dimX * dimY);
	card32 a=0,b=1;
	for( int32 i=0;i<(int32)dimX;i++ )
	{
		float s = (float)i / (float)(dimX-1);

		while( (s > m_Points[b].p[0]) && (b < m_Points.size()) )
		{
			a++; b++;
		}

		float y = (m_Points[a].p[1] + (m_Points[b].p[1] - m_Points[a].p[1])
			/ (m_Points[b].p[0] - m_Points[a].p[0])*(s - m_Points[a].p[0])) * 255.0f;

		card32 yi = y;
		for( card32 yd = 0;yd<dimY; yd++ )
		{
			tmpData[i + dimX * yd] = yi + (yi << 8) + (yi << 16);
		}
	}
	return tmpData;
}
