//---------------------------------------------------------------------------
#ifndef InterpFunctionTextureAttachmentH
#define InterpFunctionTextureAttachmentH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "MaterialAttachment.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------


namespace geoxl
{
class InterpFunctionTextureAttachmentEditorWidget;

class RENDERING_API InterpFunctionTextureAttachment : public TextureAttachment
{
private:

   static MetaClass *InterpFunctionTextureAttachmentMetaClass;
   static Object* newInstance();

   friend InterpFunctionTextureAttachmentEditorWidget;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   void write( OutputObjectStream * s ) const;
   void read( InputObjectStream * s );

   virtual card32 getTextureType() const  {return TYPE_2D; }
   virtual GLTexture * createGLTexture(GLContext * context);

   virtual void assign( const Object * obj,  CopyContext *context = NULL );

   InterpFunctionTextureAttachment();
   ~InterpFunctionTextureAttachment();

   struct TPoint
   {
      Vector2f p;
      float weight;
   };

   mpcard getNumPoints() const { return m_Points.size(); }
   TPoint getPoint( card32 i ) {return m_Points[i]; }
	
   void addPoint( TPoint point );
   void deletePoint( float x );
   void movePoint( card32 index, Vector2f dir );
   card32 getPointIndex( float x, float y );

   void setupEditorHistogram( const vector<float> &histogram );
   vector<float> & getHistogram() {return m_Histogram;}

   vector<card32> createImage( card32 dimX, card32 dimY );

private:

   card32 findPoint( float x );

   vector<TPoint>    m_Points;
   card32            m_TextureResolution;    // default 128
	
   vector<float>	m_Histogram;

};




}

#endif
 