#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "InterpFunctionTextureAttachmentRGB.h"
//---------------------------------------------------------------------------
#include <glh/glh_extensions.h>
#include "InterpFunctionTextureAttachment.h"
#include "CopyObjectProperties.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_GEOXL_CLASS( InterpFunctionTextureAttachmentRGB ,0)
{
	BEGIN_CLASS_INIT( InterpFunctionTextureAttachmentRGB );
	ADD_OBJECT_PROP( m_Functions[0],0,InterpFunctionTextureAttachment::getClass(),true );
	ADD_OBJECT_PROP( m_Functions[1],0,InterpFunctionTextureAttachment::getClass(),true );
	ADD_OBJECT_PROP( m_Functions[2],0,InterpFunctionTextureAttachment::getClass(),true );
	ADD_CARD32_PROP( m_TextureResolution,0);
}

InterpFunctionTextureAttachmentRGB::InterpFunctionTextureAttachmentRGB(void)
{
	m_TextureResolution = 128;
	for( card32 i=0;i<3;i++)
		m_Functions[i] = new InterpFunctionTextureAttachment;
}

InterpFunctionTextureAttachmentRGB::~InterpFunctionTextureAttachmentRGB(void)
{
	for( card32 i=0;i<3;i++)
		delete m_Functions[i];
}

void InterpFunctionTextureAttachmentRGB::assign(const Object* obj, CopyContext *context)
{
	TextureAttachment::assign(obj,context);
	const InterpFunctionTextureAttachmentRGB * o = dynamic_cast<const InterpFunctionTextureAttachmentRGB*>(obj);
	pAssert( o != NULL );

	copyObjectProperties(obj,this);
}

GLTexture *InterpFunctionTextureAttachmentRGB::createGLTexture(GLContext * context) 
{
	card32 dimY = 16;
	vector<card32> tmpData = createImage(m_TextureResolution, dimY);

	GLTexture2D * tex2D = new GLTexture2D( context );
	tex2D->createFromMemory( m_TextureResolution, dimY, 4, (char*)&tmpData[0], false );
	tex2D->bind();
	glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
	glTexParameteri( tex2D->getTextureTarget(), GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );

	return tex2D;

}

vector<card32> InterpFunctionTextureAttachmentRGB::createImage( card32 dimX, card32 dimY ) 
{
	vector<card32> ret( dimX * dimY,0 );
	for(card32 i=0;i<3;i++ )
	{
		vector<card32> componentData = m_Functions[i]->createImage(dimX,dimY);
		for( card32 j=0;j<ret.size();j++)
		{
			ret[j] |= componentData[j] & ((0xff) << (i*8));
		}
	}
	return ret;
}

void InterpFunctionTextureAttachmentRGB::setHistogram( const vector<float> &histogram) 
{
	for( card32 i=0;i<3;i++ )
		m_Functions[i]->setupEditorHistogram(histogram);
}
