//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "LCFilters.h"
//---------------------------------------------------------------------------
#include "SceneObjectIterator.h"
#include "Tools.h"
#include "BoundingBox.h"
#include "BoundingBox_templCode.h"
#include "MathTools.h"
#include "MathTools_templCode.h"
#include "Scene.h"
#include "SceneObject.h"
#include "GLMaterial.h"
#include "GLShaderMaterial.h"
#include "PointFlags.h"
#include "AttachedIndexedOctree.h"
#include "SimplePCMaterial.h"
#include "EventCounterDisplay.h"
#include "DebugOutput.h"
#include "SceneGraphState.h"
#include "ViewFrustum.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



void drawBoundingBoxGL(const BoundingBox3f &box) {
   glBegin(GL_LINE_LOOP);
   glVertex3f(box.lowerCorner[0], box.lowerCorner[1], box.upperCorner[2]);
   glVertex3f(box.upperCorner[0], box.lowerCorner[1], box.upperCorner[2]);
   glVertex3f(box.upperCorner[0], box.upperCorner[1], box.upperCorner[2]);
   glVertex3f(box.lowerCorner[0], box.upperCorner[1], box.upperCorner[2]);
   glEnd();
   glBegin(GL_LINE_LOOP);
   glVertex3f(box.lowerCorner[0], box.lowerCorner[1], box.lowerCorner[2]);
   glVertex3f(box.upperCorner[0], box.lowerCorner[1], box.lowerCorner[2]);
   glVertex3f(box.upperCorner[0], box.upperCorner[1], box.lowerCorner[2]);
   glVertex3f(box.lowerCorner[0], box.upperCorner[1], box.lowerCorner[2]);
   glEnd();
   glBegin(GL_LINES);
   glVertex3f(box.lowerCorner[0], box.lowerCorner[1], box.lowerCorner[2]);
   glVertex3f(box.lowerCorner[0], box.lowerCorner[1], box.upperCorner[2]);
   glVertex3f(box.upperCorner[0], box.lowerCorner[1], box.lowerCorner[2]);
   glVertex3f(box.upperCorner[0], box.lowerCorner[1], box.upperCorner[2]);
   glVertex3f(box.upperCorner[0], box.upperCorner[1], box.lowerCorner[2]);
   glVertex3f(box.upperCorner[0], box.upperCorner[1], box.upperCorner[2]);
   glVertex3f(box.lowerCorner[0], box.upperCorner[1], box.lowerCorner[2]);
   glVertex3f(box.lowerCorner[0], box.upperCorner[1], box.upperCorner[2]);
   glEnd();
}


void LCProjectiveClassification::filterNode(card32 &currentClassification, LCState *state, SceneObjectIterator *node)
{
   HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(node);

   if (state->PARAM_useExactRecursionDepth) {
      if (state->hierarchyLevel >= state->PARAM_exactRecursionDepth) {
         currentClassification = LCTraversal::NFR_PROCESS;
      } else {
         currentClassification = LCTraversal::NFR_DESCEND;
      }
   } else {
      float32 zMin = calcMinProjection(state->camera.getPosition(),
                                       state->camera.getOrthoNormViewDirection(),
                                       hit->getBoundingBox());

      float32 pointSpacing;

      if (!hit->getNodeInfo(HierarchicalIterator::niPointSpacing, NULL, &pointSpacing)) {
         pointSpacing = HierarchicalIterator::MAX_POINT_SPACING;
      }

      if (zMin * state->DERIVED_PARAM_tanOfHalfVerticalViewAngle
          >= state->DERIVED_PARAM_halfVerticalSamplingResolution * pointSpacing) {
         currentClassification = LCTraversal::NFR_PROCESS;
      } else {
         currentClassification = LCTraversal::NFR_DESCEND;
      }
   }
}


void LCViewFrustumCulling::filterNode(card32 &currentClassification, LCState *state, SceneObjectIterator *node) {
   HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(node);
   BVIntersection inF;
   card8 viewFrustumTestBits;
   inF = state->viewFrustumPlanes.firstCheckInsideFrustum(hit->getBoundingBox().getBoundingSphere(), viewFrustumTestBits);
   if (inF == BVQueryDisjoint) {
      currentClassification = LCTraversal::NFR_DROP;
   }
}

void LCNonPresentNodesCulling::filterNode(card32 &currentClassification, LCState *state, SceneObjectIterator *node) {
	if (!state->PARAM_lazySwapping) return;
	if (currentClassification == LCTraversal::NFR_DROP) return;
   HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(node);
   bool available;
   card32 whichChild = 0xFFFFFFFF; // all children
   bool infoAvail = hit->getNodeInfo(HierarchicalIterator::niChildAvailable, &whichChild, &available);

	// no prefetching info - filter has no effect
	if (!infoAvail) return;
	// all children present - filter has no effect
	if (available) return;
	
	// prefetch children
	if (currentClassification == LCTraversal::NFR_DESCEND) {
		if (state->thisFrameFetches < state->PARAM_maxPrefetch) {
		   hit->getNodeInfo(HierarchicalIterator::niPrefetchChild, &whichChild, NULL);
		   state->thisFrameFetches++;
		}
		
	}
	// child nodes not available - stop recursion
	currentClassification = LCTraversal::NFR_PROCESS;      
}



void LCDrawBoxesNodeProcessor::processNode(LCState *state, SceneObjectIterator *node) {
   HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(node);
   if (state->PARAM_drawHierarchy) {
      glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT);
      glDisable(GL_LIGHTING);
      //glEnable(GL_BLEND);
      //glEnable(GL_LINE_SMOOTH);
      //glBlendFunc(GL_SRC_ALPHA, GL_ONE);
      glColor4f(1.0f, 1.0f, 1.0f, 0.3f);
      drawBoundingBoxGL(hit->getBoundingBox());
      glPopAttrib();
   }
}

void LCDrawInnerBoxesNodeProcessor::processNode(LCState *state, SceneObjectIterator *node) {
   HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(node);
   if (state->PARAM_drawHierarchyInnerNodes) {
      glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT);
      glDisable(GL_LIGHTING);
      //glEnable(GL_BLEND);
      //glEnable(GL_LINE_SMOOTH);
      //glBlendFunc(GL_SRC_ALPHA, GL_ONE);
      glColor4f(0.3f, 0.3f, 1.0f, 0.3f);
      drawBoundingBoxGL(hit->getBoundingBox());
      glPopAttrib();
   }
}

/*
void LCSimpleDrawPointsNodeProcessor::processNode(LCState *state, SceneObjectIterator *node) {
   glColor3f(0.8f, 0.8f, 0.8f);
   glBegin(GL_POINTS);
   AAT aatPosition = state->vertexDescriptor->getToken("position");
   BasicPointCloudIterator *bit = dynamic_cast<BasicPointCloudIterator*>(node);
   while (!bit->atEnd()) {
      Vector3f pos = bit->get3f(aatPosition);
      glVertex3fv(pos.data());
      bit->next();
   }
   glEnd();
}
*/


void LCDrawPointsMaterial::processNode(LCState *state, SceneObjectIterator *node) {
   if (state->PARAM_drawPoints) {
      glPushAttrib(GL_ALL_ATTRIB_BITS);
      glEnable(GL_DEPTH_TEST);
      glDepthMask(true);
      card32 matInd = state->sceneObject->getMaterialIndex();

      HierarchicalIterator *hi = dynamic_cast<HierarchicalIterator*>(node);

      if (state->scene->getNumGLMaterials() > matInd) {
         GLMaterial *material = state->scene->getGLMaterial(matInd);

		   GLShaderMaterial *shMat = dynamic_cast<GLShaderMaterial*>(material);
		   if (shMat != NULL) {
			   float32 minPointSpacing;
			   hi->getNodeInfo(HierarchicalIterator::niPointSpacing, NULL, &minPointSpacing);
			   shMat->setParameter1f(state->glContext, "minPointSpacing", minPointSpacing);
			   shMat->setParameter1f(state->glContext, "vertViewAngle", state->sceneGraphState->staticState->viewFrustum.getVerticalFieldOfView());
			   shMat->setParameter1f(state->glContext, "tanOfHalfVerticalViewAngle", state->DERIVED_PARAM_tanOfHalfVerticalViewAngle);
			   shMat->setParameter1f(state->glContext, "vertResolution", state->viewportHeight);
			   if (state->PARAM_passStateVariablesToShaders) {
			      vector< pair<string, Vector4f> > stateVars4f = state->sceneGraphState->dynamicState->getStateVariables4F();
			      for (size_t i=0; i<stateVars4f.size(); i++) {
			         string name = stateVars4f[i].first;
			         Vector4f vect = stateVars4f[i].second;
   			      shMat->setParameter4fv(state->glContext, name, vect.data());
   			   }
			   }
		   }

         if (material) 
		 {
			material->draw(state->scene, state->sceneGraphState, state->sceneObject, node, state->glContext, state->memManager );
	         card64 numPts = 0;

	         if (hi->getNodeInfo(HierarchicalIterator::niNumPointsInNode, NULL, &numPts)) 
			 {
		        state->numPoints += (card32)numPts;
			 }
         }
      }
	  glPopAttrib();
   }
}

void LCDrawPointsMaterialNoStateChange::processNode(LCState *state, SceneObjectIterator *node)
{
   if (state->PARAM_drawPoints) {
      card32 matInd = state->sceneObject->getMaterialIndex();
      GLMaterial *material = state->scene->getGLMaterial(matInd);
      if (material == NULL) return;

		GLShaderMaterial *shMat = dynamic_cast<GLShaderMaterial*>(material);
		if (shMat != NULL) {
			HierarchicalIterator *hi = dynamic_cast<HierarchicalIterator*>(node);
			float32 minPointSpacing;
			hi->getNodeInfo(HierarchicalIterator::niPointSpacing, NULL, &minPointSpacing);
			shMat->setParameter1f(state->glContext, "minPointSpacing", minPointSpacing);
		}
      material->draw(state->scene, state->sceneGraphState, state->sceneObject, node, state->glContext, state->memManager );
   }
}


void LCDrawCloudSelection::processNode(LCState *state, SceneObjectIterator *node) {
	if(state->currentCloudIsSelected && state->PARAM_drawCloudSelection )
	{
		if( !state->useBBSelection) 
		{
			glPushAttrib(GL_ALL_ATTRIB_BITS);
			glEnable(GL_DEPTH_TEST);
			glDepthMask(false);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			state->cloudSelectionMaterial->draw(state->scene, state->sceneGraphState, state->sceneObject, node, state->glContext, state->memManager);
			glPopAttrib();
		}
		else
		{
			HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(node);
			glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT);
			glDisable(GL_LIGHTING);
			//glEnable(GL_BLEND);
			//glEnable(GL_LINE_SMOOTH);
			//glBlendFunc(GL_SRC_ALPHA, GL_ONE);
			glColor4f(0.1f, 1.0f, 0.1f, 1.0f);
			drawBoundingBoxGL(hit->getBoundingBox());
			glPopAttrib();
		}
   }
}

void LCDrawPointSelection::processNode(LCState *state, SceneObjectIterator *node) {
   HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(node);
   bool drawPointSelection = state->PARAM_drawPointSelection;
   card64 numSelected = 0;
   if (drawPointSelection) {
      drawPointSelection &= hit->getNodeInfo(HierarchicalIterator::niFlaggedInSubtree, &PF_FLAG_SELECTED, &numSelected);
   }
   if (drawPointSelection) {
      drawPointSelection &= (numSelected > 0);
   }
   if (drawPointSelection) {
      glPushAttrib(GL_ALL_ATTRIB_BITS);
      glEnable(GL_DEPTH_TEST);
      glDepthMask(false);
      glEnable(GL_BLEND);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      state->pointSelectionMaterial->draw(state->scene, state->sceneGraphState, state->sceneObject, node, state->glContext, state->memManager, PF_FLAG_SELECTED, PF_FLAG_INVALID);
      glPopAttrib();
   }
}

void LCStatisticsOutput::processState(LCState *state) {
	if (state->PARAM_outputStatistics) {
      EventCounterDisplay::NewValueStepOnChange v;
      v.value = state->numPoints;
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("num points rendered"), &v);
      v.value = state->numBoxes;
      debugOutput.displayEvent(EventCounterDisplay::getClass(), string("num boxes visited"), &v);
	}
}
