#include "stdafx.h"
//---------------------------------------------------------------------------
#include "LCTraversal.h"
//---------------------------------------------------------------------------
#include "NumericalClassProperty.h"
#include "NumericalClassProperty_templCode.h"
#include "BooleanClassProperty.h"
#include "SceneObjectIterator.h"
#include "SceneGraphState.h"
#include "ViewFrustumRoutines.h"
#include "StandardMaterials.h"
#include "GLShaderMaterial.h"
#include "AttachedIndexedOctree.h"
#include "ObjectViewsTable.h"
#include "ObjectClassProperty.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* LCTraversal::LCTraversalMetaClass = NULL;

const MetaClass* LCTraversal::getInstanceClass() const {
   return getClass();
}

Object* LCTraversal::newInstance() {
   return new LCTraversal;
}

const MetaClass* LCTraversal::getClass() {
   if (LCTraversalMetaClass==NULL) {
      LCTraversalMetaClass = new MetaClass("LCTraversal", "universal renderer for spatial hierarchies.",
                                  newInstance, sizeof(LCTraversal));
   }
   return LCTraversalMetaClass;
}

void LCTraversal::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   LCTraversalMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT(LCTraversal)
	ADD_OBJECT_PROP(traversalState,0,LCState::getClass(),false)
}

void LCTraversal::shutdown() {
   delete LCTraversalMetaClass;
}


// ----


const card32 LCState::DRAW_ORDER_ANY = 0;
const card32 LCState::DRAW_ORDER_FRONT_TO_BACK = 1;
const card32 LCState::DRAW_ORDER_BACK_TO_FRONT = 2;

MetaClass* LCState::LCStateMetaClass = NULL;

const MetaClass* LCState::getInstanceClass() const {
   return getClass();
}

Object* LCState::newInstance() {
   return new LCState;
}

const MetaClass* LCState::getClass() {
   if (LCStateMetaClass==NULL) {
      LCStateMetaClass = new MetaClass("LCState", "no description.",
                                  newInstance, sizeof(LCState));
   }
   return LCStateMetaClass;
}

static string describeDrawOrder(card32 c) {
   if (c == 0) {
      return "DRAW_ORDER_ANY";
   } else if (c == 1) {
      return "DRAW_ORDER_FRONT_TO_BACK";
   } else if (c == 2) {
      return "DRAW_ORDER_BACK_TO_FRONT";
   } else {
      return "UNDEFINED";
   }
}

void LCState::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 7);
   LCStateMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT(LCState)
   ADD_CARD32_PROP_UPDATE_DESCR(PARAM_drawOrder, 0, NULL, &describeDrawOrder, 3)
   ADD_BOOLEAN_PROP(PARAM_lockCamera,0)
	ADD_CARD32_PROP(PARAM_maxHierarchyLevel,0)
   ADD_BOOLEAN_PROP(PARAM_drawPoints,0)
   ADD_BOOLEAN_PROP(PARAM_drawHierarchy ,0)
   ADD_BOOLEAN_PROP(PARAM_drawHierarchyInnerNodes ,0)
   ADD_BOOLEAN_PROP(PARAM_drawCloudSelection ,0)
   ADD_BOOLEAN_PROP(PARAM_drawPointSelection ,0)
   ADD_BOOLEAN_PROP(PARAM_drawNodeGLRenderings, 4)
   ADD_BOOLEAN_PROP(PARAM_drawObjectGLRenderings, 4)
   ADD_BOOLEAN_PROP(PARAM_drawAttachmentGLRenderings, 4)
   ADD_BOOLEAN_PROP(PARAM_passStateVariablesToShaders, 7)
   ADD_BOOLEAN_PROP(PARAM_lazySwapping, 5);
   ADD_CARD32_PROP(PARAM_maxPrefetch, 6);
   

   ADD_FLOAT32_PROP(PARAM_maxPointSampleSpacing ,0)
   ADD_BOOLEAN_PROP(PARAM_useExactRecursionDepth ,0)
   ADD_CARD32_PROP(PARAM_exactRecursionDepth ,0)
	ADD_FLOAT32_PROP(PARAM_viewFrustumScaleFactor ,0)
   ADD_OBJECT_PROP(cloudSelectionMaterial, 0, GLShaderMaterial::getClass(), true) 
   ADD_OBJECT_PROP(pointSelectionMaterial, 0, GLShaderMaterial::getClass(), true)

   //ADD_CARD32_PROP(numPoints ,0)
	//SET_STREAM_PROPERTY( false )
	//SET_EDIT_READONLY( true )
	  
   //ADD_CARD32_PROP(numBoxes ,0)
	//SET_STREAM_PROPERTY( false )
	//SET_EDIT_READONLY( true )
   ADD_BOOLEAN_PROP(PARAM_outputStatistics, 6)


 /*  LCStateMetaClass->addProperty(
      new Card32ClassProperty("drawOrder", 1,(Card32ElementPointer)&PARAM_drawOrder, NULL,
                              (Card32ClassProperty::GetEnumDescriptionMethod)describeDrawOrder)
   );

   LCStateMetaClass->addProperty(
      new BooleanClassProperty("PARAM_lockCamera", 1, (BooleanElementPointer)&PARAM_lockCamera)
   );

   LCStateMetaClass->addProperty(
      new Card32ClassProperty("PARAM_maxHierarchyLevel", 1,(Card32ElementPointer)&PARAM_maxHierarchyLevel)
   );
   LCStateMetaClass->addProperty(
      new BooleanClassProperty("PARAM_drawPoints",1,(BooleanElementPointer)&PARAM_drawPoints)
   );

   LCStateMetaClass->addProperty(
      new BooleanClassProperty("PARAM_drawHierarchy",1,(BooleanElementPointer)&PARAM_drawHierarchy)
   );
   LCStateMetaClass->addProperty(
      new BooleanClassProperty("PARAM_drawHierarchyInnerNodes",1,(BooleanElementPointer)&PARAM_drawHierarchyInnerNodes)
   );
   LCStateMetaClass->addProperty(
      new BooleanClassProperty("PARAM_drawCloudSelection", MetaClass::FIRST_VERSION + 1,(BooleanElementPointer)&PARAM_drawCloudSelection)
   );
   LCStateMetaClass->addProperty(
      new BooleanClassProperty("PARAM_drawPointSelection", MetaClass::FIRST_VERSION + 1,(BooleanElementPointer)&PARAM_drawPointSelection)
   );

   LCStateMetaClass->addProperty(
      new Float32ClassProperty("PARAM_maxPointSampleSpacing",1,(Float32ElementPointer)&PARAM_maxPointSampleSpacing)
   );
   LCStateMetaClass->addProperty(
      new BooleanClassProperty("PARAM_useExactRecursionDepth",1,(BooleanElementPointer)&PARAM_useExactRecursionDepth)
   );
   LCStateMetaClass->addProperty(
      new Card32ClassProperty("PARAM_exactRecursionDepth",1,(Card32ElementPointer)&PARAM_exactRecursionDepth)
   );
   LCStateMetaClass->addProperty(
      new Float32ClassProperty("PARAM_viewFrustumScaleFactor",1,(Float32ElementPointer)&PARAM_viewFrustumScaleFactor)
   );

   LCStateMetaClass->addProperty(new ObjectClassProperty("cloudSelectionMaterial", MetaClass::FIRST_VERSION + 2,
      (ObjectElementPointer)&cloudSelectionMaterial, GLShaderMaterial::getClass(), true));
   LCStateMetaClass->addProperty(new ObjectClassProperty("pointSelectionMaterial", MetaClass::FIRST_VERSION + 2,
      (ObjectElementPointer)&pointSelectionMaterial, GLShaderMaterial::getClass(), true));

   Card32ClassProperty *np = new Card32ClassProperty("numPoints", MetaClass::FIRST_VERSION + 3, (Card32ElementPointer)&numPoints);
   np->setStreamProperty(false);
   np->setEditReadonly(true);
   LCStateMetaClass->addProperty(np);
   Card32ClassProperty *nb = new Card32ClassProperty("numBoxes", MetaClass::FIRST_VERSION + 3, (Card32ElementPointer)&numBoxes);
   nb->setStreamProperty(false);
   nb->setEditReadonly(true);
   LCStateMetaClass->addProperty(nb);
*/
}

void LCState::shutdown() {
   delete LCStateMetaClass;
}


// ----


LCState::LCState() {
   PARAM_drawOrder = DRAW_ORDER_ANY;

   PARAM_lockCamera = false;

   PARAM_maxHierarchyLevel = 3;
   PARAM_drawPoints = true;
   PARAM_drawHierarchy = false;
   PARAM_drawHierarchyInnerNodes = false;
   PARAM_drawCloudSelection = true;
   PARAM_drawPointSelection = true;
   PARAM_drawNodeGLRenderings = true;
   PARAM_drawObjectGLRenderings = true;
   PARAM_drawAttachmentGLRenderings = true;
   PARAM_passStateVariablesToShaders = true;
   PARAM_lazySwapping = true;
   PARAM_maxPrefetch = 3;
   //PARAM_maxPointSampleSpacing = 4.0f;
   PARAM_maxPointSampleSpacing = 9.0f;
   PARAM_useExactRecursionDepth = false;
   PARAM_exactRecursionDepth = 3;
   PARAM_viewFrustumScaleFactor = 1.0f;
   PARAM_outputStatistics = false;

   cloudSelectionMaterial = makeDefaultCloudSelectionMaterial();
   pointSelectionMaterial = makeDefaultPointSelectionMaterial();
}

void LCState::resetState(SceneGraphState *sceneGraphState, Renderer *renderer, card32 viewportWidth, card32 viewportHeight,
                         Scene *scene, SceneObject *sceneObject, GLContext *glContext, MemoryManager *memManager) {
   hierarchyLevel = 0;
   numPoints = 0;
   numBoxes = 0;
   thisFrameFetches = 0;
   this->sceneGraphState = sceneGraphState;
   this->renderer = renderer;

   if (!PARAM_lockCamera) {
      camera = sceneGraphState->staticState->camera;
      saveCamera = camera;
   } else {
      camera = saveCamera;
   }

   Matrix4f transform = sceneGraphState->dynamicState->ctm;
   Matrix4f invTransform = invertMatrix(transform);
   camera.transformCamera(invTransform);

   viewFrustum = sceneGraphState->staticState->viewFrustum;

   viewFrustumPlanes = setViewingFrustum(camera, viewFrustum, PARAM_viewFrustumScaleFactor);

   this->viewportWidth = viewportWidth;
   this->viewportHeight = viewportHeight;

   DERIVED_PARAM_tanOfHalfVerticalViewAngle = tan(viewFrustum.getVerticalFieldOfView() / 180.0f * M_PI / 2.0f);
   DERIVED_PARAM_halfVerticalSamplingResolution = (float32)viewportHeight / 2.0f / PARAM_maxPointSampleSpacing;

   this->scene = scene;
   this->sceneObject = sceneObject;
   this->glContext = glContext;
   this->memManager = memManager;
}

LCState::~LCState() {   
   delete cloudSelectionMaterial;
   delete pointSelectionMaterial;
}


// --


const card32 LCTraversal::NFR_PROCESS = 0; const card32 NFR_PROCESS_C = 0;
const card32 LCTraversal::NFR_DESCEND = 1; const card32 NFR_DESCEND_C = 1;
const card32 LCTraversal::NFR_DROP = 2;    const card32 NFR_DROP_C = 2;

LCTraversal::LCTraversal() {
   traversalState = NULL;
   processAnimations = false;
}

LCTraversal::LCTraversal(LCState *state) {
   traversalState = state;
   processAnimations = false;
}

void LCTraversal::setTraversalState(LCState *shtst) {
   if (traversalState != shtst) {
      delete traversalState;
      traversalState = shtst;
   }
}

void LCTraversal::addNodeFilters(LCNodeFilter *nf) {
   nodeFilters.push_back(nf);
}
void LCTraversal::addNodeProcessors(LCNodeProcessor *np) {
   nodeProcessors.push_back(np);
}
void LCTraversal::addSubdividedNodeProcessors(LCNodeProcessor *sp) {
   subdividedNodesProcessors.push_back(sp);
}
void LCTraversal::addUnSubdividableNodesProcessors(LCNodeProcessor *usnp) {
   unSubdividableNodesProcessors.push_back(usnp);
}
void LCTraversal::addStatePreProcessors(LCStateProcessor *prep) {
   statePreProcessors.push_back(prep);
}
void LCTraversal::addStatePostProcessors(LCStateProcessor *postp) {
   statePostProcessors.push_back(postp);
}
void LCTraversal::addRootNodePreProcessors(LCNodeProcessor *prep) {
   rootNodePreProcessors.push_back(prep);
}

void LCTraversal::addRootNodePostProcessors(LCNodeProcessor *postp) {
   rootNodePostProcessors.push_back(postp);
}

void LCTraversal::doStatePreProcessing() {
   vector<LCStateProcessor*>::iterator pos = statePreProcessors.begin();
   vector<LCStateProcessor*>::iterator end = statePreProcessors.end();
   while (pos != end) {
      (*pos)->processState(traversalState);
      ++pos;
   }
}

void LCTraversal::doStatePostProcessing() {
   vector<LCStateProcessor*>::iterator pos = statePostProcessors.begin();
   vector<LCStateProcessor*>::iterator end = statePostProcessors.end();
   while (pos != end) {
      (*pos)->processState(traversalState);
      ++pos;
   }
}

void LCTraversal::doRootNodePreProcessing(SceneObjectIterator *rootNode) {
   vector<LCNodeProcessor*>::iterator pos = rootNodePreProcessors.begin();
   vector<LCNodeProcessor*>::iterator end = rootNodePreProcessors.end();
   while (pos != end) {
      (*pos)->processNode(traversalState, rootNode);
      ++pos;
   }
}

void LCTraversal::doRootNodePostProcessing(SceneObjectIterator *rootNode) {
   vector<LCNodeProcessor*>::iterator pos = rootNodePostProcessors.begin();
   vector<LCNodeProcessor*>::iterator end = rootNodePostProcessors.end();
   while (pos != end) {
      (*pos)->processNode(traversalState, rootNode);
      ++pos;
   }
}

void LCTraversal::doProcess(SceneObjectIterator *node) {
   vector<LCNodeProcessor*>::iterator pos = nodeProcessors.begin();
   vector<LCNodeProcessor*>::iterator end = nodeProcessors.end();
   while (pos != end) {
      (*pos)->processNode(traversalState, node);
      ++pos;
   }
}

void LCTraversal::doProcessSubdivided(SceneObjectIterator *node) {
   vector<LCNodeProcessor*>::iterator pos = subdividedNodesProcessors.begin();
   vector<LCNodeProcessor*>::iterator end = subdividedNodesProcessors.end();
   while (pos != end) {
      (*pos)->processNode(traversalState, node);
      ++pos;
   }
}

void LCTraversal::doProcessUnSubdividable(SceneObjectIterator *node) {
   vector<LCNodeProcessor*>::iterator pos = unSubdividableNodesProcessors.begin();
   vector<LCNodeProcessor*>::iterator end = unSubdividableNodesProcessors.end();
   while (pos != end) {
      (*pos)->processNode(traversalState, node);
      ++pos;
   }
}

class ChildNodeEntry {
 public:
   card32 index;
   float32 distance;
   SceneObjectIterator *instancePtr;
};
inline bool operator<(const ChildNodeEntry &e1, const ChildNodeEntry &e2) {
   return e1.distance < e2.distance;
}

void LCTraversal::doRekTraversal(SceneObjectIterator *node) {
   HierarchicalIterator *hit = dynamic_cast<HierarchicalIterator*>(node);

   pAssert(hit);
   traversalState->numBoxes++;
   card32 nfr = NFR_DESCEND;
   vector<LCNodeFilter*>::iterator pos = nodeFilters.begin();
   vector<LCNodeFilter*>::iterator end = nodeFilters.end();
   while (nfr != NFR_DROP && pos != end) {
      (*pos)->filterNode(nfr, traversalState, node);
      ++pos;
   }
   switch (nfr) {
      case NFR_PROCESS_C: {
         doProcess(node);
         break;
      }
      case NFR_DESCEND_C: {

         bool childFound = false;

         if (traversalState->PARAM_drawOrder == LCState::DRAW_ORDER_ANY) {

            card32 n = hit->getNumChildren();
            for (card32 i=0; i<n; i++) {
               if (hit->goToChild(i)) {
                  traversalState->hierarchyLevel++;
                  childFound = true;
                  doRekTraversal(node); 
                  hit->goToParent();
                  traversalState->hierarchyLevel--;
               }
            }

         } /*xxx else { // DRAW_ORDER_FRONT_TO_BACK or DRAW_ORDER_BACK_TO_FRONT
            LCProjectiveClassificationState *prjState = dynamic_cast<LCProjectiveClassificationState*>(traversalState);
            if (prjState == NULL) return;
            vector<ChildNodeEntry> children;

            card32 n = node->getNumChildren();

            children.resize(n);
            card32 numEntries = 0;

            for (card32 i=0; i<n; i++) {
               if (node->goToChild(i)) {
                  childFound = true;
                  ChildNodeEntry cne;

                  cne.index = i;
                  cne.distance = (node->getBoundingBox().getCenter() - prjState->camera.getPosition()) * prjState->camera.getOrthoNormViewDirection();
                  cne.instancePtr = NULL;
                  cne.stateDiffs = NULL;

                  children[numEntries] = cne;
                  node->goToParent();
                  numEntries++;
               }
            }

            // assume DRAW_ORDER_FRONT_TO_BACK
            children.resize(numEntries);
            sort(children.begin(), children.end());
            if (traversalState->PARAM_drawOrder == LCState::DRAW_ORDER_BACK_TO_FRONT) {
               reverse(children.begin(), children.end());
            }

            for (card32 i=0; i<numEntries; i++) {
               if (children[i].instancePtr == NULL) {
                  node->goToChild(children[i].index);
                  traversalState->hierarchyLevel++;
                  doRekTraversal(node);
                  node->goToParent();
                  traversalState->hierarchyLevel--;
               }
            }
         }*/

         if (!childFound) {
            doProcessUnSubdividable(node);
         } else {
            doProcessSubdivided(node);
         }
         break;
      }
      case NFR_DROP_C: {
         return;
      }
   }
}

void LCTraversal::performTraversal(SceneObjectIterator *node, SceneGraphState *state, Renderer *renderer,
                                   card32 viewportWidth, card32 viewportHeight,
                                   Scene *scene, SceneObject *sceneObject, GLContext *glContext, MemoryManager *memManager)
{
   traversalState->resetState(state, renderer, viewportWidth, viewportHeight, scene, sceneObject, glContext, memManager);

   beforeTraversal();

   doStatePreProcessing();
   doRootNodePreProcessing(node);

   doRekTraversal(node);

   doRootNodePostProcessing(node);
   doStatePostProcessing();

   afterTraversal();

   ObjectViewsTable::update(state);
}


LCTraversal::~LCTraversal() {
   delete traversalState;
   {
      size_t n = nodeFilters.size();
      for (card32 i=0; i<n; i++) {
         delete nodeFilters[i];
      }
   }{
      size_t n = nodeProcessors.size();
      for (card32 i=0; i<n; i++) {
         delete nodeProcessors[i];
      }
   }{
      size_t n = subdividedNodesProcessors.size();
      for (card32 i=0; i<n; i++) {
         delete subdividedNodesProcessors[i];
      }
   }{
      size_t n = unSubdividableNodesProcessors.size();
      for (card32 i=0; i<n; i++) {
         delete unSubdividableNodesProcessors[i];
      }
   }{
      size_t n = statePreProcessors.size();
      for (card32 i=0; i<n; i++) {
         delete statePreProcessors[i];
      }
   }{
      size_t n = statePostProcessors.size();
      for (card32 i=0; i<n; i++) {
         delete statePostProcessors[i];
      }
   }{
      size_t n = rootNodePreProcessors.size();
      for (card32 i=0; i<n; i++) {
         delete rootNodePreProcessors[i];
      }
   }{
      size_t n = rootNodePostProcessors.size();
      for (card32 i=0; i<n; i++) {
         delete rootNodePostProcessors[i];
      }
   }
}

