//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "MaterialAttachment.h"
//---------------------------------------------------------------------------
#include "FixedArrayClassProperty.h"
#include "GLTexture.h"
#include "LinearAlgebra_templCode.h"
#include "GLAttachments.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* Matrix4fAttachment::Matrix4fAttachmentMetaClass = NULL;

const MetaClass* Matrix4fAttachment::getInstanceClass() const {
   return getClass();
}

Object* Matrix4fAttachment::newInstance() {
   return new Matrix4fAttachment;
}

const MetaClass* Matrix4fAttachment::getClass() {
   if (Matrix4fAttachmentMetaClass==NULL) {
      Matrix4fAttachmentMetaClass = new MetaClass("Matrix4fAttachment", "no description.",
                                  newInstance, sizeof(Matrix4fAttachment));
   }
   return Matrix4fAttachmentMetaClass;
}

void Matrix4fAttachment::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   Matrix4fAttachmentMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT(Matrix4fAttachment)
	ADD_FLOAT32_FIXED_ARRAY_PROP(m_Matrix, 0, 16, 4)

}

void Matrix4fAttachment::shutdown() {
   delete Matrix4fAttachmentMetaClass;
}


// ----

MetaClass* Vector4fAttachment::Vector4fAttachmentMetaClass = NULL;

const MetaClass* Vector4fAttachment::getInstanceClass() const {
   return getClass();
}

Object* Vector4fAttachment::newInstance() {
   return new Vector4fAttachment;
}

const MetaClass* Vector4fAttachment::getClass() {
   if (Vector4fAttachmentMetaClass==NULL) {
      Vector4fAttachmentMetaClass = new MetaClass("Vector4fAttachment", "no description.",
                                  newInstance, sizeof(Vector4fAttachment));
   }
   return Vector4fAttachmentMetaClass;
}

void Vector4fAttachment::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   Vector4fAttachmentMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT(Vector4fAttachment)
	ADD_FLOAT32_FIXED_ARRAY_PROP(m_Vector, 0, 4, 4)
	ADD_CARD32_PROP( m_NumComponentsUsed, 1 );

}

void Vector4fAttachment::shutdown() {
   delete Vector4fAttachmentMetaClass;
}


// ----

MetaClass* TextureAttachment::TextureAttachmentMetaClass = NULL;

const MetaClass* TextureAttachment::getInstanceClass() const {
   return getClass();
}

const MetaClass* TextureAttachment::getClass() {
   if (TextureAttachmentMetaClass==NULL) {
      TextureAttachmentMetaClass = new MetaClass("TextureAttachment", "no description.",
                                  NULL, sizeof(TextureAttachment));
   }
   return TextureAttachmentMetaClass;
}

void TextureAttachment::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 0);
   TextureAttachmentMetaClass->copyParentProperties();
}

void TextureAttachment::shutdown() {
   delete TextureAttachmentMetaClass;
}


// ----

void Matrix4fAttachment::assign(const Object* obj, CopyContext *context) {
   AttachedData::assign( obj, context );
   const Matrix4fAttachment *other = dynamic_cast<const Matrix4fAttachment*>(obj);
   pAssert(other);
   this->m_Matrix = other->m_Matrix;
}

void Vector4fAttachment::assign(const Object* obj, CopyContext *context) {
   AttachedData::assign( obj, context );
   const Vector4fAttachment *other = dynamic_cast<const Vector4fAttachment*>(obj);
   pAssert(other);
   this->m_Vector = other->m_Vector;
}


void TextureAttachment::assign( const Object * obj,  CopyContext *context )
{
   AttachedData::assign( obj, context );
   TextureAttachment * ta = (TextureAttachment*)obj;
   m_Container->assign( ta->m_Container );
}

GLTexture * TestTextureAttachment::createGLTexture( GLContext * context )
{
   int size = 64;

   vector<card8> lightTab;
   lightTab.push_back(20);
   lightTab.push_back(60);
   lightTab.push_back(70);
   lightTab.push_back(80);
   lightTab.push_back(140);
   lightTab.push_back(200);
   lightTab.push_back(210);
   lightTab.push_back(255);

   vector<card8> tmp(size*size*4);
   for( int i=0;i<size;i++ )
   {
      float f = (float)i/(float)size;

      card8 col = lightTab[ f * lightTab.size() ];

      for( int j=0;j<size;j++ )
      {
         tmp[(j*size+i)*4+0] = col;
         tmp[(j*size+i)*4+1] = col;
         tmp[(j*size+i)*4+2] = col;
         tmp[(j*size+i)*4+3] = col;
      }
   }

   GLTexture2D * ret = new GLTexture2D( context );
   ret->createFromMemory( size, size, 4, (char*)&tmp[0], true );
   return ret;
}



GLTexture * geoxl::TextureAttachment::getCachedTexture(GLContext * glContext)
{
	string texID = GLTextureAttachment::generateID( glContext->getRootContext(), "" );
	GLTextureAttachment * glTexAtt = dynamic_cast<GLTextureAttachment*>(fetchData(texID));
	if( !glTexAtt )
	{
		glTexAtt = new GLTextureAttachment;

		GLTexture * texture = createGLTexture( glContext );
		glTexAtt->setupTA( texture, "", NULL, true );
		attachData( glTexAtt );
	}

	return glTexAtt->getTexture();
}