//---------------------------------------------------------------------------
#ifndef MaterialAttachmentH
#define MaterialAttachmentH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "AttachedData.h"
#include "LinearAlgebra.h"
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;

class GLTexture;
class GLContext;


class RENDERING_API Matrix4fAttachment : public AttachedData
{
private:
   static MetaClass *Matrix4fAttachmentMetaClass;
   static Object* newInstance();

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

private:

   Matrix4f m_Matrix;

public:

   Matrix4fAttachment() : AttachedData()
   {
      m_Matrix = IDENTITY4F;
   }

   void setMatrix( const Matrix4f &mat ) {m_Matrix = mat; }
   Matrix4f getMatrix() const {return m_Matrix; }
   virtual void assign(const Object* obj, CopyContext *context = NULL);
};

class RENDERING_API Vector4fAttachment : public AttachedData
{
private:
   static MetaClass *Vector4fAttachmentMetaClass;
   static Object* newInstance();

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

private:

   Vector4f m_Vector;
	card32	m_NumComponentsUsed;	// out of [1,4]

public:

   Vector4fAttachment() : AttachedData()
   {
      m_Vector = makeVector4f(0,0,0,0);
		m_NumComponentsUsed = 4;
   }

   void setVector( const Vector4f &vec ) {m_Vector = vec; }
   Vector4f getVector() const {return m_Vector;}
	void setNumComponentsUsed( card32 n ) {m_NumComponentsUsed = n;}
	card32 getNumComponentsUsed() {return m_NumComponentsUsed;}

   virtual void assign(const Object* obj, CopyContext *context = NULL);

};

class RENDERING_API TextureAttachment : public AttachedData
{
private:
   static MetaClass *TextureAttachmentMetaClass;

public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   TextureAttachment() : AttachedData()
   {
      m_Container = new AttachedDataContainer;
   }

   ~TextureAttachment()
   {
      delete m_Container;
   }

   static const card32 TYPE_2D = 1;
   static const card32 TYPE_3D = 2;
   static const card32 TYPE_CUBE = 3;
   static const card32 TYPE_RECT_FLOAT = 4;

   virtual card32 getTextureType() const = 0;
   virtual GLTexture * createGLTexture(GLContext * context) = 0;

   virtual void assign( const Object * obj,  CopyContext *context = NULL );

   inline AttachedData * fetchData( const string &name)
   {
      return m_Container->getData( name );
   }

   inline void deleteData( const string &name )
   {
      m_Container->deleteData(name);
   }

   inline void attachData( AttachedData * data )
   {
      m_Container->attachData(data);
   }

   inline void clearAttachments()
   {
      m_Container->clear();
   }

	GLTexture * getCachedTexture(GLContext * glContext);


private:

   AttachedDataContainer   *  m_Container;

};

class RENDERING_API TestTextureAttachment : public TextureAttachment
{
public:
   virtual card32 getTextureType() const {return TYPE_2D; }
   virtual GLTexture * createGLTexture(GLContext * context);
   virtual bool needsUpdate() {return false;}

};

}

#endif
