//---------------------------------------------------------------------------
#ifndef PointsAsSpheresMaterialH
#define PointsAsSpheresMaterialH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "GLMaterial.h"
//---------------------------------------------------------------------------

namespace geoxl {

///
///   A simple immediate mode material that draws points in point clouds as spheres
///
class RENDERING_API PointsAsSpheresMaterial : public GLMaterial {
	GEOXL_CLASS( PointsAsSpheresMaterial )
 private:
	string positionChannel;
	bool useRadiusChannel;
	string radiusChannel;
	float32 defaultRadius;
	float32 radiusScale;
	bool useColorChannel;
	string colorChannel;
	Vector3f defaultColor;
	bool useAxesChannel;
	string axesChannel;
	Matrix3f defaultAxes;//to remove
	string axesScaleChannel;
	Vector3f axesScale;
	
 public:
	PointsAsSpheresMaterial();

   virtual void assign(const Object* obj, CopyContext *context = NULL);

   virtual void draw(Scene *scene, SceneGraphState *state, SceneObject *obj,
      SceneObjectIterator *iter, GLContext *context, MemoryManager * mm = NULL,
      const int32 includeFlags = 0, const int32 excludeFlags = PF_FLAG_INVALID );

   string getPositionChannel() const {return positionChannel;}
   void setPositionChannel(string v) {positionChannel=v;}

   bool getUseRadiusChannel() const {return useRadiusChannel;}
   void setUseRadiusChannel(bool v) {useRadiusChannel=v;}

   string getRadiusChannel() const {return radiusChannel;}
   void setRadiusChannel(string v) {radiusChannel=v;}

   float32 getDefaultRadius() const {return defaultRadius;}
   void setDefaultRadius(float32 v) {defaultRadius=v;}

   float32 getRadiusScale() const {return radiusScale;}
   void setRadiusScale(float32 v) {radiusScale=v;}

   bool getUseColorChannel() const {return useColorChannel;}
   void setUseColorChannel(bool v) {useColorChannel=v;}

   string getColorChannel() const {return colorChannel;}
   void setColorChannel(string v) {colorChannel=v;}

   Vector3f getDefaultColor() const {return defaultColor;}
   void setDefaultColor(Vector3f v) {defaultColor=v;}

   ~PointsAsSpheresMaterial();
};

}

#endif