//---------------------------------------------------------------------------
#ifndef RendererH
#define RendererH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "GLContext.h"
//---------------------------------------------------------------------------


namespace geoxl {
   using namespace std;

class Scene;

class RENDERING_API Renderer : public Persistent {
 private:
   static MetaClass *RendererMetaClass;

   Scene* scene;
   GLContext* m_glContext;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   Renderer();

   /// usage: call once before rendering to setup renderer
   /// in derived class: call inherited method first
   /// context must be active
   virtual void connect(Scene *scene, GLContext* ctxt);

   /// render with settings according to connect
   /// context must be active
   virtual void render(card32 viewportWidth, card32 viewportHeight) = 0;

   /// call once when renderer is no longer used (call connect again to reconnect)
   /// frees temporary render objects (etc...)
   /// context must be active
   virtual void disconnect(GLContext* ctxt) {}

   inline Scene* getScene() const {return scene;}
   inline GLContext* getGLContext(void){ return(m_glContext); };

	/// set selection render mode (a bit ugly...)
	virtual void setUseBBSelection(bool b){}

	/// returns true if renderer allows debugrenderer
	virtual bool debugRendererAllowed() {return true;}

   virtual ~Renderer();
};

}

#endif
