//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "SceneCreator.h"
//---------------------------------------------------------------------------
#include "Scene.h"
#include "Tools.h"
#include "StandardMaterials.h"
#include "GLShaderMaterial.h"
#include "GLMaterial.h"
#include "SimplePCMaterial.h"
#include "SimpleGLTriMeshMaterial.h"
#include "PointsAsSpheresMaterial.h"
#include "DebugOutput.h"
#include "BinaryObjectStreams.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* SceneCreator::SceneCreatorMetaClass = NULL;

const MetaClass* SceneCreator::getInstanceClass() const {
   return getClass();
}

Object* SceneCreator::newInstance() {
   return new SceneCreator;
}

const MetaClass* SceneCreator::getClass() {
   if (SceneCreatorMetaClass==NULL) {
      SceneCreatorMetaClass = new MetaClass("SceneCreator", "no description.",
                                  newInstance, sizeof(SceneCreator));
   }
   return SceneCreatorMetaClass;
}

void SceneCreator::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   SceneCreatorMetaClass->copyParentProperties();
//   SceneCreatorMetaClass->getGUIDescriptor()->addClassEditor(SceneCreatorEditor::getClass(), true);
}

void SceneCreator::shutdown() {
   delete SceneCreatorMetaClass;
}


// ----


static void tryAddMaterial(Scene *scene, string path) {
	string materialPath = getConfigurationString( "BIN_PATH" ) + path;
	debugOutput << string("trying to load ")+materialPath+string(".\n");
	BinaryInputObjectStream in(materialPath.c_str());
	Persistent *p;
	GLMaterial *mat = NULL;
	try {
		in.readObject(p);
		mat = dynamic_cast<GLMaterial*>(p);
	} catch (...) {
		debugOutput << "could not load this material - io error.\n";
		return;
	}
	if (!mat) {
		delete p;
		debugOutput << "wrong type - not loaded\n";
	} else {
		scene->addGLMaterial(mat);
		debugOutput << "material loaded successfully.\n";
	}	
}

SceneCreator::SceneCreator() {
	createStdMaterials = true;
}

Object *SceneCreator::makeObject() {
   Scene *scene = new Scene();
   scene->addGLMaterial(new SimplePCMaterial());
   if (createStdMaterials) {
      scene->addGLMaterial(makeWhitePointsMaterial());
      scene->addGLMaterial(makeColoredPointsMaterial());
      scene->addGLMaterial(new SimpleGLTriMeshMaterial);
      scene->addGLMaterial(new PointsAsSpheresMaterial);
		tryAddMaterial(scene, "/shaders/blinn_phong.material.object");
		tryAddMaterial(scene, "/shaders/blinn_phong_colored.material.object");
   }
   return scene;
}


