//---------------------------------------------------------------------------
#ifndef ShaderH
#define ShaderH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------

namespace geoxl {
   using namespace std;

class GLShader;
class GLContext;


class RENDERING_API Shader : public Persistent
{
 private:
   static MetaClass *ShaderMetaClass;
   static Object* newInstance();

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   Shader();
   virtual void assign(const Object* obj, CopyContext *context=NULL );

    GLShader * createShader( GLContext *context );
   bool compile( GLContext *testcontext);
   string getError() {return m_LastError;}

   bool isValid();

   static const card32 INVALID = 0xffff;
   static const card32 VERTEX = 1;
   static const card32 FRAGMENT = 2;
   static const card32 CLASS_ARB = 1;
   static const card32 CLASS_CG = 2;

   card32 getShaderType() const {return m_ShaderType;}
   void setShaderType(card32 stype );
   card32 getShaderClass() const {return m_ShaderClass;}
   void setShaderClass(card32 stype );

   string getSource() const;
   void setSource(string);

   bool wasChanged() const {return m_Changed;}
   void setChanged() {m_Changed = true; }

 protected:

   GLShader * createShaderInstance( GLContext * context );

   bool     m_Changed;

   // persistent members:
   string   m_ShaderSource;
   card32   m_ShaderType;
   card32   m_ShaderClass;

    // nonpersistent members:
    string  m_LastError;
 //   GLContext * m_TestContext;
};

};
#endif
