//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "StandardMaterials.h"
//---------------------------------------------------------------------------
#include "GLShaderMaterial.h"
#include "Shader.h"
#include "VertexDescriptor.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


void geoxl::setDefaultCloudSelectionVertexShader(Shader *vShader) {
   static const string DEFAULT_SELECTION_MATERIAL_VSHADER =
      string("struct TIn\n") +
      string("{\n") +
      string("	float4 pos : ATTR0;\n") +
      string("};\n") +
      string("\n") +
      string("struct TOut\n") +
      string("{\n") +
      string("	float4 pos : POSITION;\n") +
      string("};\n") +
      string("\n") +
      string("TOut main( TIn IN, uniform float4x4 matMVP)\n") +
      string("{\n") +
      string("	TOut OUT;\n") +
      string("	OUT.pos = mul( matMVP, IN.pos );\n") +
      string("	OUT.pos.z -= 0.01;\n") +
      string("	return OUT;\n") +
      string("}\n");

   vShader->setShaderType(Shader::VERTEX);
   vShader->setShaderClass(Shader::CLASS_CG);
   vShader->setSource(DEFAULT_SELECTION_MATERIAL_VSHADER);
}

void geoxl::setDefaultPointSelectionVertexShader(Shader *vShader) {
   static const string DEFAULT_SELECTION_MATERIAL_VSHADER =
      string("struct TIn\n") +
      string("{\n") +
      string("	float4 pos : ATTR0;\n") +
      string("};\n") +
      string("\n") +
      string("struct TOut\n") +
      string("{\n") +
      string("	float4 pos : POSITION;\n") +
      string("};\n") +
      string("\n") +
      string("TOut main( TIn IN, uniform float4x4 matMVP)\n") +
      string("{\n") +
      string("	TOut OUT;\n") +
      string("	OUT.pos = mul( matMVP, IN.pos );\n") +
      string("	OUT.pos.z -= 0.02;\n") +
      string("	return OUT;\n") +
      string("}\n");

   vShader->setShaderType(Shader::VERTEX);
   vShader->setShaderClass(Shader::CLASS_CG);
   vShader->setSource(DEFAULT_SELECTION_MATERIAL_VSHADER);
}

void geoxl::setDefaultCloudSelectionPixelShader(Shader *pShader) {
   static const string DEFAULT_SELECTION_MATERIAL_PSHADER =
      string("float4 main() : COLOR\n") +
      string("{\n") +
      string("	return float4(0.3, 0.8, 0.3, 0.3);\n") +
      string("}\n");

   pShader->setShaderType(Shader::FRAGMENT);
   pShader->setShaderClass(Shader::CLASS_CG);
   pShader->setSource(DEFAULT_SELECTION_MATERIAL_PSHADER);
}

void geoxl::setDefaultPointSelectionPixelShader(Shader *pShader) {
   static const string DEFAULT_SELECTION_MATERIAL_PSHADER =
      string("float4 main() : COLOR\n") +
      string("{\n") +
      string("	return float4(1.0, 0.5, 0.5, 0.4);\n") +
      string("}\n");

   pShader->setShaderType(Shader::FRAGMENT);
   pShader->setShaderClass(Shader::CLASS_CG);
   pShader->setSource(DEFAULT_SELECTION_MATERIAL_PSHADER);
}


GLShaderMaterial *geoxl::makeDefaultCloudSelectionMaterial() {

   GLShaderMaterial *selectionMaterial = new GLShaderMaterial;
//   selectionMaterial->setRenderMode(GLShaderMaterial::RENDERMODE_QUADS);

   Shader *vsh = selectionMaterial->getVertexShader();
   setDefaultCloudSelectionVertexShader(vsh);

   Shader *psh = selectionMaterial->getFragmentShader();
   setDefaultCloudSelectionPixelShader(psh);

   selectionMaterial->setPointSize(3);
   selectionMaterial->setBlending(true);
   selectionMaterial->setDepthWrite(true);
   selectionMaterial->setDestBlendFunc(GLShaderMaterial::BM_ONE_MINUS_SOURCE_ALPHA);
   selectionMaterial->setSourceBlendFunc(GLShaderMaterial::BM_SOURCE_ALPHA);

   return selectionMaterial;
}


GLShaderMaterial *geoxl::makeDefaultPointSelectionMaterial() {

   GLShaderMaterial *selectionMaterial = new GLShaderMaterial;
//   selectionMaterial->setRenderMode(GLShaderMaterial::RENDERMODE_QUADS);

   Shader *vsh = selectionMaterial->getVertexShader();
   setDefaultPointSelectionVertexShader(vsh);

   Shader *psh = selectionMaterial->getFragmentShader();
   setDefaultPointSelectionPixelShader(psh);

   selectionMaterial->setPointSize(4);
   selectionMaterial->setBlending(true);
   selectionMaterial->setDepthWrite(true);
   selectionMaterial->setDestBlendFunc(GLShaderMaterial::BM_ONE_MINUS_SOURCE_ALPHA);
   selectionMaterial->setSourceBlendFunc(GLShaderMaterial::BM_SOURCE_ALPHA);

   return selectionMaterial;
}


GLShaderMaterial *geoxl::makeWhitePointsMaterial() {
   static const string VSHADER =
      string("struct TIn\n") +
      string("{\n") +
      string("	float4 pos : ATTR0;\n") +
      string("};\n") +
      string("\n") +
      string("struct TOut\n") +
      string("{\n") +
      string("	float4 pos : POSITION;\n") +
      string("};\n") +
      string("\n") +
      string("TOut main( TIn IN, uniform float4x4 matMVP)\n") +
      string("{\n") +
      string("	TOut OUT;\n") +
      string("	OUT.pos = mul( matMVP, IN.pos );\n") +
      string("	return OUT;\n") +
      string("}\n");

   static const string PSHADER =
      string("float4 main() : COLOR\n") +
      string("{\n") +
      string("	return float4(1.0, 1.0, 1.0, 1.0);\n") +
      string("}\n");

   GLShaderMaterial *result = new GLShaderMaterial;

   Shader *vShader = result->getVertexShader();
   vShader->setShaderType(Shader::VERTEX);
   vShader->setShaderClass(Shader::CLASS_CG);
   vShader->setSource(VSHADER);

   Shader *pShader = result->getFragmentShader();
   pShader->setShaderType(Shader::FRAGMENT);
   pShader->setShaderClass(Shader::CLASS_CG);
   pShader->setSource(PSHADER);

   result->setName("WhitePointsMaterial");

   return result;
}


GLShaderMaterial *geoxl::makeColoredPointsMaterial() {
   static const string VSHADER =
      string("struct TIn\n") +
      string("{\n") +
      string("	float4 pos : ATTR0;\n") +
      string("	float3 color : ATTR1;\n") +
      string("};\n") +
      string("\n") +
      string("struct TOut\n") +
      string("{\n") +
      string("	float4 pos : POSITION;\n") +
      string("	float3 color : COLOR;\n") +
      string("};\n") +
      string("\n") +
      string("TOut main( TIn IN, uniform float4x4 matMVP)\n") +
      string("{\n") +
      string("	TOut OUT;\n") +
      string("	OUT.pos = mul( matMVP, IN.pos );\n") +
      string("	OUT.color = IN.color;\n") +
      string("	return OUT;\n") +
      string("}\n");

   static const string PSHADER =
      string("struct TIn\n") +
      string("{\n") +
      string("	float4 color : COLOR;\n") +
      string("};\n") +
      string("\n") +
      string("float4 main(TIn IN) : COLOR\n") +
      string("{\n") +
      string("	return IN.color;\n") +
      string("}\n");

   GLShaderMaterial *result = new GLShaderMaterial;

   VertexDescriptor *vd = result->getVertexDescriptor();
   vd->pushAttrib(mVAD("color", 3, VAD::DATA_FORMAT_FLOAT32));

   Shader *vShader = result->getVertexShader();
   vShader->setShaderType(Shader::VERTEX);
   vShader->setShaderClass(Shader::CLASS_CG);
   vShader->setSource(VSHADER);

   Shader *pShader = result->getFragmentShader();
   pShader->setShaderType(Shader::FRAGMENT);
   pShader->setShaderClass(Shader::CLASS_CG);
   pShader->setSource(PSHADER);

   result->setName("ColoredPointsMaterial");

   return result;
}


