//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "TwoPassBlendingRenderer.h"
//---------------------------------------------------------------------------
#include <string>
#include "GLRenderTextureFBO.h"
#include "GLContext.h"
#include "GLTexture.h"
#include "LCHierarchyRenderer.h"
#include "GLShader.h"
#include "DebugOutput.h"
#include "MaterialAttachment.h"
#include "Scene.h"
#include "BooleanClassProperty.h"
#include "GLRenderTextureCache.h"
#include "PointCloud.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------



/*MetaClass* TwoPassBlendingRenderer::TwoPassBlendingRendererMetaClass = NULL;

const MetaClass* TwoPassBlendingRenderer::getInstanceClass() const {
   return getClass();
}

Object* TwoPassBlendingRenderer::newInstance() {
   return new TwoPassBlendingRenderer;
}

const MetaClass* TwoPassBlendingRenderer::getClass() {
   if (TwoPassBlendingRendererMetaClass==NULL) {
      TwoPassBlendingRendererMetaClass = new MetaClass("TwoPassBlendingRenderer", "no description.",
                                  newInstance, sizeof(TwoPassBlendingRenderer));
   }
   return TwoPassBlendingRendererMetaClass;
}

void TwoPassBlendingRenderer::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   TwoPassBlendingRendererMetaClass->copyParentProperties();
}

void TwoPassBlendingRenderer::shutdown() {
   delete TwoPassBlendingRendererMetaClass;
}*/

IMPLEMENT_GEOXL_CLASS( TwoPassBlendingRenderer, 0)
{
	BEGIN_CLASS_INIT( TwoPassBlendingRenderer );

	ADD_BOOLEAN_PROP(m_MultiPassSplattingMode, 0);
	ADD_OBJECT_PROP( m_CombineMaterial, 0, GLShaderMaterial::getClass(), true );
	ADD_OBJECT_PROP( m_DrawTextureMaterial, 0, GLShaderMaterial::getClass(), true );
	
	ADD_CARD32_VAR_ARRAY_PROP( m_AttributeMaterial, 0);
}


// ----


string vsource =
"struct VIn { float4 pos : POSITION; float2 tex : TEXCOORD0; };\n"
"struct VOut { float4 pos : POSITION; float2 tex : TEXCOORD0; };\n"
"VOut main( VIn IN, uniform float4x4 matMVP)\n"
"{ VOut OUT; OUT.pos = mul(matMVP,IN.pos); OUT.tex = IN.tex; return OUT; }";

string psource = "struct VIn { float2 tex : TEXCOORD0; };"
"float3 main( VIn IN, uniform samplerRECT texture ) : COLOR"
"{ float4 col = texRECT( texture, IN.tex);"
//" col.w = col.w == 0 ? 1 : col.w;"
"  return (col.w == 0) ? 1 : 1 - col.xyz / col.w;"
"}";

string psource2 = "struct VIn { float2 tex : TEXCOORD0; };"
"float3 main( VIn IN, uniform samplerRECT texture ) : COLOR"
"{ float4 col = texRECT( texture, IN.tex); return col;"
"}";

TwoPassBlendingRenderer::TwoPassBlendingRenderer()
{
   m_RenderTarget = NULL;
   m_Context = NULL;
   // could be replaced by one renderer
   imagePassRenderer = new LCHierarchyRenderer(LCHierarchyRenderer::CREATE_NOGLSTATECHANGE);
   depthPassRenderer = new LCHierarchyRenderer(LCHierarchyRenderer::CREATE_NOGLSTATECHANGE);
   m_ShaderV = m_ShaderF = NULL;
   m_Scene = NULL;
	m_MultiPassSplattingMode = false;
	m_RTCache = NULL;
	m_CombineMaterial = new GLShaderMaterial;
	m_CombineMaterial->getVertexShader()->setSource(vsource);
	m_CombineMaterial->getFragmentShader()->setSource(psource);
	m_CombineMaterial->getVertexShader()->setShaderClass( Shader::CLASS_CG);
	m_CombineMaterial->getFragmentShader()->setShaderClass( Shader::CLASS_CG);
	
	m_DrawTextureMaterial = new GLShaderMaterial;
	m_DrawTextureMaterial->getVertexShader()->setSource(vsource);
	m_DrawTextureMaterial->getFragmentShader()->setSource(psource2);
	m_DrawTextureMaterial->getVertexShader()->setShaderClass( Shader::CLASS_CG);
	m_DrawTextureMaterial->getFragmentShader()->setShaderClass( Shader::CLASS_CG);
}

void TwoPassBlendingRenderer::connect(Scene* scene, GLContext* ctxt)
{
   m_Context = ctxt;
   m_Scene = scene;
   imagePassRenderer->connect( scene, ctxt );
   depthPassRenderer->connect( scene, ctxt );
	m_RTCache = new GLRenderTextureCache(ctxt);
}

void TwoPassBlendingRenderer::render(card32 viewportWidth, card32 viewportHeight)
{
	if( m_MultiPassSplattingMode )
	{
		renderMultiPass(viewportWidth,viewportHeight);
		return;
	}
   float modelview[16], projection[16];
   glGetFloatv( GL_MODELVIEW_MATRIX, modelview );
   glGetFloatv( GL_PROJECTION_MATRIX, projection );


   if( m_ShaderV == NULL )
   {
     m_ShaderV = new GLShaderCG(m_Context, GLShaderCG::getLatestVSProfile() );
     if( !m_ShaderV->compile( vsource ) )
     {
        debugOutput << "TwoPassBlending ShaderV:\n" << m_ShaderV->getLastError();
        debugOutput.cr();
     }

     m_ShaderF = new GLShaderCG( m_Context, GLShaderCG::getLatestPSProfile() );
     if( !m_ShaderF->compile( psource ) )
     {
         debugOutput << "TwoPassBlending ShaderF:\n"<< m_ShaderF->getLastError() << "\n";
     }
   }

   if( m_RenderTarget != NULL )
   {
      if( m_RenderTarget->getWidth() != viewportWidth ||
         m_RenderTarget->getHeight() != viewportHeight )
         {
            delete m_RenderTarget;
            m_RenderTarget = NULL;
         }
   }

   if( m_RenderTarget == NULL )
   {
      m_RenderTarget = new GLRenderTextureFBO( m_Context );
		bool sucess = m_RenderTarget->create(viewportWidth, viewportHeight, 4, GLRenderTextureFBO::FLAG_RECTANGLE |
			GLRenderTextureFBO::FLAG_DEPTH |	GLRenderTextureFBO::FLAG_FLOAT16);
         pAssert( sucess );
   }

   m_RenderTarget->activate();
   glMatrixMode( GL_PROJECTION );
   glLoadIdentity();
   glMultMatrixf( projection );
   glMatrixMode( GL_MODELVIEW );
   glLoadIdentity();
   glMultMatrixf( modelview );
   glClearColor(0,0,0,0);

   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
   glPushAttrib( GL_ALL_ATTRIB_BITS );

   glEnable( GL_DEPTH_TEST );
   glColorMask(false,false,false,false);
   glDepthMask(true);

   Vector4fAttachment * att = (Vector4fAttachment *)m_Scene->getAttachments()->getData( "render_z_offset" );
   if( att == NULL )
   {
      att = new Vector4fAttachment;
      att->setup( "render_z_offset", 0 );
      m_Scene->getAttachments()->attachData( att );
   }

   att->setVector( makeVector4f( 2.0f, 4.0f, 0, 0 ) );   // scale , offset , ,

//   glPolygonOffset( 4.0, 16.0f );
//   glEnable( GL_POLYGON_OFFSET_FILL );
//   glDepthFunc( GL_LEQUAL );
   depthPassRenderer->render( viewportWidth, viewportHeight );

   glColorMask(true,true,true,true);
   glDepthMask(false);

   glEnable( GL_BLEND );
   glBlendFunc( GL_ONE, GL_ONE );

   att->setVector( makeVector4f( 1.0f, 0.0f, 0,0 ) );
//   glDisable( GL_POLYGON_OFFSET_FILL );
   imagePassRenderer->render(viewportWidth,viewportHeight);

   glPopAttrib();

   m_RenderTarget->deactivate();

   glClearColor(1,0,0,1);
   //glClear( GL_COLOR_BUFFER_BIT );

   glPushAttrib( GL_ENABLE_BIT );
   glDisable( GL_DEPTH_TEST );
   glDisable( GL_CULL_FACE );

   glMatrixMode( GL_PROJECTION );
   glPushMatrix();
   glLoadIdentity();

   glMatrixMode( GL_MODELVIEW );
   glPushMatrix();
   glLoadIdentity();

	m_RenderTarget->bind();
   m_ShaderV->bind();
   m_ShaderF->bind();

   glBegin( GL_QUADS );
      glTexCoord2f(0,0);
      glVertex3f(-1,-1,1);
      glTexCoord2f(viewportWidth,0);
      glVertex3f(1,-1,1);
      glTexCoord2f(viewportWidth,viewportHeight);
      glVertex3f(1,1,1);
      glTexCoord2f(0,viewportHeight);
      glVertex3f(-1,1,1);
   glEnd();

   m_ShaderV->unbind();
   m_ShaderF->unbind();
	m_RenderTarget->unbind();


   glMatrixMode( GL_MODELVIEW );
   glPopMatrix();
   glMatrixMode( GL_PROJECTION );
   glPopMatrix();

   glPopAttrib();

}

void TwoPassBlendingRenderer::disconnect(GLContext* ctxt)
{
   m_Context = NULL;
   delete m_RenderTarget;
   delete m_ShaderV;
   delete m_ShaderF;
   m_RenderTarget = NULL;
   m_ShaderV = NULL;
   m_ShaderF = NULL;
   imagePassRenderer->disconnect( ctxt );
   depthPassRenderer->disconnect( ctxt );
	delete m_RTCache;
	m_RTCache = NULL;
}

TwoPassBlendingRenderer::~TwoPassBlendingRenderer()
{
   delete imagePassRenderer;
   delete depthPassRenderer;
	delete m_CombineMaterial;
	delete m_DrawTextureMaterial;
}

void geoxl::TwoPassBlendingRenderer::renderMultiPass( card32 viewportWidth, card32 viewportHeight )
{
	vector<PointCloud*> clouds;
	StandardSceneGraphIterator * sceneIt = new StandardSceneGraphIterator();
	sceneIt->clearAndSetup( m_Scene->getRootNode(), m_Scene->getRootState() );
	while( !sceneIt->atEnd() )
	{
		SGObjectNode * objNode = dynamic_cast<SGObjectNode*>(sceneIt->getNode());
		if( objNode )
		{
			PointCloud * pc = dynamic_cast<PointCloud*>(objNode->getSceneObject());
			if( pc)
				clouds.push_back(pc);
		}
		sceneIt->next();
	}


	float modelview[16], projection[16];
	glGetFloatv( GL_MODELVIEW_MATRIX, modelview );
	glGetFloatv( GL_PROJECTION_MATRIX, projection );

	// init render textures
	vector<GLRenderTextureFBO*> textures;
	for( unsigned i=0;i<m_AttributeMaterial.size();i++ )
	{
		//GLImage *tex = new GLImage();
		//tex->setup( m_RTCache, viewportWidth, viewportHeight, 4, true, true, false, true, false);
		GLRenderTextureFBO * tex = m_RTCache->getRenderTexture(viewportWidth,viewportHeight,4,true,true,false,true,false);
		textures.push_back(tex);
	}

	// render passes

	for( unsigned i=0;i<m_AttributeMaterial.size();i++ )
	{
		textures[i]->activate();
		for( unsigned j=0;j<clouds.size();j++ )
			clouds[j]->setMaterialIndex(m_AttributeMaterial[i]);
		GLShaderMaterial * mat = dynamic_cast<GLShaderMaterial*>(m_Scene->getGLMaterial(m_AttributeMaterial[i]));


		glMatrixMode( GL_PROJECTION );
		glLoadIdentity();
		glMultMatrixf( projection );
		glMatrixMode( GL_MODELVIEW );
		glLoadIdentity();
		glMultMatrixf( modelview );
		glClearColor(i == 0? 1 : 0,0,0,0);

		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
		glPushAttrib( GL_ALL_ATTRIB_BITS );

		glEnable( GL_DEPTH_TEST );
		glColorMask(false,false,false,false);
		glDepthMask(true);

		Vector4fAttachment * att = (Vector4fAttachment *)m_Scene->getAttachments()->getData( "render_z_offset" );
		if( att == NULL )
		{
			att = new Vector4fAttachment;
			att->setup( "render_z_offset", 0 );
			m_Scene->getAttachments()->attachData( att );
		}

		att->setVector( makeVector4f( 2.0f, 4.0f, 0, 0 ) );   // scale , offset , ,

		//   glPolygonOffset( 4.0, 16.0f );
		//   glEnable( GL_POLYGON_OFFSET_FILL );
		//   glDepthFunc( GL_LEQUAL );
		depthPassRenderer->render( viewportWidth, viewportHeight );

		glColorMask(true,true,true,true);
		glDepthMask(false);

		glEnable( GL_BLEND );
		glBlendFunc( GL_ONE, GL_ONE );

		att->setVector( makeVector4f( 1.0f, 0.0f, 0,0 ) );
		//   glDisable( GL_POLYGON_OFFSET_FILL );
		imagePassRenderer->render(viewportWidth,viewportHeight);

		glPopAttrib();

		textures[i]->deactivate();
	}

	if( m_RenderTarget != NULL )
	{
		if( m_RenderTarget->getWidth() != viewportWidth ||
			m_RenderTarget->getHeight() != viewportHeight )
		{
			delete m_RenderTarget;
			m_RenderTarget = NULL;
		}
	}

	if( m_RenderTarget == NULL )
	{
		m_RenderTarget = new GLRenderTextureFBO( m_Context );
		bool sucess = m_RenderTarget->create(viewportWidth, viewportHeight, 4, GLRenderTextureFBO::FLAG_RECTANGLE);
		pAssert( sucess );
	}

	m_RenderTarget->activate();

	glClearColor(1,0,0,1);
	//glClear( GL_COLOR_BUFFER_BIT );

	glPushAttrib( GL_ENABLE_BIT );
	glDisable( GL_DEPTH_TEST );
	glDisable( GL_CULL_FACE );

	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();

	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();

	m_CombineMaterial->bindShaders(m_Context);
	for( unsigned i=0;i<textures.size();i++ )
		textures[i]->bind(i);
	//if( textures.size() )
	//textures.back()->bind(0);

	glBegin( GL_QUADS );
	glTexCoord2f(0,0);
	glVertex3f(-1,-1,1);
	glTexCoord2f(viewportWidth,0);
	glVertex3f(1,-1,1);
	glTexCoord2f(viewportWidth,viewportHeight);
	glVertex3f(1,1,1);
	glTexCoord2f(0,viewportHeight);
	glVertex3f(-1,1,1);
	glEnd();

//	if( textures.size() )
//		textures.back()->unbind();
	for( unsigned i=0;i<textures.size();i++ )
		textures[i]->unbind();
	m_CombineMaterial->unbindShaders(m_Context);


	glMatrixMode( GL_MODELVIEW );
	glPopMatrix();
	glMatrixMode( GL_PROJECTION );
	glPopMatrix();

	glPopAttrib();

	m_RenderTarget->deactivate();

	m_RenderTarget->bind(0);
	glClearColor(1,0,0,1);
	//glClear( GL_COLOR_BUFFER_BIT );

	glPushAttrib( GL_ENABLE_BIT );
	glDisable( GL_DEPTH_TEST );
	glDisable( GL_CULL_FACE );

	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();

	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();

	m_DrawTextureMaterial->bindShaders(m_Context);
	//for( unsigned i=0;i<textures.size();i++ )
	//	textures[i]->bind(i);
	//if( textures.size() )
	//	textures.back()->bind(0);

	glBegin( GL_QUADS );
	glTexCoord2f(0,0);
	glVertex3f(-1,-1,1);
	glTexCoord2f(viewportWidth,0);
	glVertex3f(1,-1,1);
	glTexCoord2f(viewportWidth,viewportHeight);
	glVertex3f(1,1,1);
	glTexCoord2f(0,viewportHeight);
	glVertex3f(-1,1,1);
	glEnd();

	//if( textures.size() )
	//	textures.back()->unbind();
	//	for( unsigned i=0;i<textures.size();i++ )
	//		textures[i]->unbind();
	m_DrawTextureMaterial->unbindShaders(m_Context);


	glMatrixMode( GL_MODELVIEW );
	glPopMatrix();
	glMatrixMode( GL_PROJECTION );
	glPopMatrix();

	glPopAttrib();
	m_RenderTarget->unbind();

	for( unsigned i=0;i<textures.size();i++ )
		m_RTCache->freeRenderTexture(textures[textures.size()-i-1]);
}