//---------------------------------------------------------------------------
#include "stdafx.h"
 
//---------------------------------------------------------------------------
#include "ViewFrustumRoutines.h"
//---------------------------------------------------------------------------
#include "LinearAlgebra_templCode.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



BoundedFrustum3f geoxl::setViewingFrustum(const Camera &camera, const ViewFrustum &vFrustum,
                                   float32 scalefactor) {
   BoundedFrustum3f f;
   float32 tanHalfVert = tan(vFrustum.getVerticalFieldOfView() / 180.0f * M_PI / 2.0f) * scalefactor;
   float32 tanHalfHoriz = tanHalfVert * vFrustum.getXToYAspectRatio();
   Vector3f upperLeft  = camera.getOrthoNormViewDirection()
                       - camera.getOrthoNormRightDirection()*tanHalfHoriz
                       + camera.getOrthoNormUpDirection()*tanHalfVert;

   Vector3f upperRight = camera.getOrthoNormViewDirection()
                       + camera.getOrthoNormRightDirection()*tanHalfHoriz
                       + camera.getOrthoNormUpDirection()*tanHalfVert;

   Vector3f lowerLeft  = camera.getOrthoNormViewDirection()
                       - camera.getOrthoNormRightDirection()*tanHalfHoriz
                       - camera.getOrthoNormUpDirection()*tanHalfVert;

   Vector3f lowerRight = camera.getOrthoNormViewDirection()
                       + camera.getOrthoNormRightDirection()*tanHalfHoriz
                       - camera.getOrthoNormUpDirection()*tanHalfVert;

   Vector3f normal_lx = upperLeft.crossProduct(lowerLeft);
   Vector3f normal_ly = lowerLeft.crossProduct(lowerRight);
   Vector3f normal_ux = lowerRight.crossProduct(upperRight);
   Vector3f normal_uy = upperRight.crossProduct(upperLeft);

   f.lowerPlane[0] = HalfSpace3f(normal_lx, camera.getPosition());
   f.lowerPlane[1] = HalfSpace3f(normal_ly, camera.getPosition());
   f.upperPlane[0] = HalfSpace3f(normal_ux, camera.getPosition());
   f.upperPlane[1] = HalfSpace3f(normal_uy, camera.getPosition());

   f.frontPlane = HalfSpace3f(camera.getOrthoNormViewDirection(),
                              camera.getOrthoNormViewDirection() * vFrustum.getZNearClip() + camera.getPosition());
   f.backPlane = HalfSpace3f(-camera.getOrthoNormViewDirection(),
                             camera.getOrthoNormViewDirection() * vFrustum.getZFarClip() + camera.getPosition());

   return f;
}

BoundedFrustum3f geoxl::setViewingFrustumTransformed(const Camera &camera, const ViewFrustum &vFrustum,
                                   const Matrix4f &absTrans, const Matrix4f &normalTrans,
                                   float32 scalefactor) {
   BoundedFrustum3f f = geoxl::setViewingFrustum(camera, vFrustum, scalefactor);
   transformHalfSpace(f.lowerPlane[0], absTrans, normalTrans);
   transformHalfSpace(f.lowerPlane[1], absTrans, normalTrans);
   transformHalfSpace(f.upperPlane[0], absTrans, normalTrans);
   transformHalfSpace(f.upperPlane[1], absTrans, normalTrans);
   transformHalfSpace(f.frontPlane, absTrans, normalTrans);
   transformHalfSpace(f.backPlane, absTrans, normalTrans);

   return f;
}


