#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "CubeTextureAttachmentEditorWidget.h"
//---------------------------------------------------------------------------
#include "CubeTextureAttachment.h"
#include "CubeMap.h"
#include "CubeMap_templCode.h"
#include "ClassEditor.h"
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( CubeTextureAttachment )


CubeTextureAttachmentEditorWidget::CubeTextureAttachmentEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	//Example: easy access to QtDesigner buttons, etc. by ui object
	QWidget::connect(ui.BTUpdate, SIGNAL(clicked()), this, SLOT(sltBTUpdateClicked()));
	QWidget::connect(ui.CBMagFilterLinear, SIGNAL(clicked()), this, SLOT(sltCBMagFilterLinearClicked()));
	QWidget::connect(ui.CBMinFilterLinear, SIGNAL(clicked()), this, SLOT(sltCBMinFilterLinearClicked()));

	// make layout for adding property editors
	QVBoxLayout *layout = new QVBoxLayout;
	layout->setMargin(0);
	layout->setSpacing(0);
	ui.CubeEditFrame->setLayout(layout);

	// init the rest
	m_CubeAtt = NULL;
   m_Editor = NULL;
   m_EditorWidget = NULL;
}

CubeTextureAttachmentEditorWidget::~CubeTextureAttachmentEditorWidget(void)
{
}

void CubeTextureAttachmentEditorWidget::setup( CubeTextureAttachment * obj )
{
	m_CubeAtt = obj;

   RGBACubeMap * cubemap = m_CubeAtt->getCubeMap();
   const MetaClass *newEditor = cubemap->getInstanceClass()->getGUIDescriptor()->getDefaultClassEditor();

   if (newEditor->isAbstract()) return;

		m_Editor = (ClassEditor*)(newEditor->newInstance());
      m_EditorWidget = m_Editor->makeEditor(ui.CubeEditFrame, cubemap);
		ui.CubeEditFrame->layout()->addWidget(m_EditorWidget);
		m_EditorWidget->show();
      m_Editor->update();

}

void CubeTextureAttachmentEditorWidget::updateEditor()
{
   pAssert( m_CubeAtt != NULL );

   if( m_Editor )
      m_Editor->update();

	ui.CBMagFilterLinear->setChecked(m_CubeAtt->getMagFilterLinear());
	ui.CBMinFilterLinear->setChecked(m_CubeAtt->getMinFilterLinear());
}


// ---------- slots ----------

void CubeTextureAttachmentEditorWidget::sltBTUpdateClicked() {
	m_CubeAtt->clearAttachments();
}

void CubeTextureAttachmentEditorWidget::sltCBMagFilterLinearClicked() {
	m_CubeAtt->setMagFilterLinear( ui.CBMagFilterLinear->isChecked() );
}

void CubeTextureAttachmentEditorWidget::sltCBMinFilterLinearClicked() {
   m_CubeAtt->setMinFilterLinear( ui.CBMinFilterLinear->isChecked() );
}

#include "CubeTextureAttachmentEditorWidget_moc.cpp"


