#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "GLShaderMaterialEditorWidget.h"
//---------------------------------------------------------------------------
#include "GLShaderMaterial.h"
#include "GLMaterial.h"
#include "Shader.h"
#include "VertexDescriptor.h"
#include "VertexDescriptorEditorWidget.h"
#include "ClassEditor.h"
#include "MPContainer.h"
#include "Tools.h"
#include "Configuration.h"
#include "QtGui/QButtonGroup"
#include "QtGui/QFileDialog"
#include "QtGui/QMessageBox"
#include "QtGui/QColorDialog"
#include "BinaryObjectStreams.h"

//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;

IMPLEMENT_CLASS_EDITOR( GLShaderMaterial )

GLShaderMaterialEditorWidget::GLShaderMaterialEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	m_ShaderMaterial = NULL;
    m_TestContext = NULL;
    m_ShaderEditorVS =   m_ShaderEditorPS = NULL;
    m_ShaderEditorWidgetVS =   m_ShaderEditorWidgetPS = NULL;
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);

	ui.RenderMode->addItem("Points");
	ui.RenderMode->addItem("Quads");
	ui.RenderMode->addItem("Normals");
	ui.RenderMode->addItem("Splatting");
	ui.RenderMode->addItem("SlowSplatting");


	BG_source=new QButtonGroup(this);
	BG_dest=new QButtonGroup(this);
	BG_source->addButton(ui.radioButton,0);
	BG_source->addButton(ui.radioButton_2,1);
	BG_source->addButton(ui.radioButton_3,2);
	BG_source->addButton(ui.radioButton_4,3);
	BG_dest->addButton(ui.radioButton_5,0);
	BG_dest->addButton(ui.radioButton_6,1);
	BG_dest->addButton(ui.radioButton_7,2);
	BG_dest->addButton(ui.radioButton_8,3);


	ui.ColorLabel->setPalette(QColor(0,0,0));

	//Example: easy access to QtDesigner buttons, etc. by ui object
	//QWidget::connect(ui.loadButton,SIGNAL(clicked()),this,SLOT(sltLoadFile()));
	QWidget::connect(ui.MatName,SIGNAL(editingFinished()),this,SLOT(MatNameChange()));
	QWidget::connect(ui.RenderMode,SIGNAL(activated(int)),this,SLOT(RenderModeChange()));
	QWidget::connect(ui.Button_RebuildObjects,SIGNAL(clicked()),this,SLOT(Button_RebuildObjectsClick()));
	QWidget::connect(ui.ButtonLoadVS,SIGNAL(clicked()),this,SLOT(ButtonLoadVSClick()));
	QWidget::connect(ui.ButtonSaveVS,SIGNAL(clicked()),this,SLOT(ButtonSaveVSClick()));
	QWidget::connect(ui.ButtonLoadPS,SIGNAL(clicked()),this,SLOT(ButtonLoadPSClick()));
	QWidget::connect(ui.ButtonSavePS,SIGNAL(clicked()),this,SLOT(ButtonSavePSClick()));
	QWidget::connect(ui.cb_blending,SIGNAL(clicked()),this,SLOT(CB_blendingClick()));
	QWidget::connect(ui.cbVertexShaderPointSizeControl, SIGNAL(clicked()), this, SLOT(cb_vspointsizeClick()));
	QWidget::connect(ui.cb_DepthTest,SIGNAL(clicked()),this,SLOT(CB_depthTestClick()));
	QWidget::connect(ui.cb_DepthWrite,SIGNAL(clicked()),this,SLOT(CB_depthWriteClick()));
	QWidget::connect(ui.Scale_Edit,SIGNAL(valueChanged(double)),this,SLOT(Scale_EditValueChange()));
	QWidget::connect(ui.sbPointSize,SIGNAL(valueChanged(double)),this,SLOT(FE_pointSizeValueChange()));
	QWidget::connect(ui.ImportShaders,SIGNAL(clicked()),this,SLOT(ImportShadersClick()));
	QWidget::connect(ui.ForceTriangleNormal,SIGNAL(clicked()),this,SLOT(ForceTriangleNormalClick()));
	QWidget::connect(ui.DrawWireMesh,SIGNAL(clicked()),this,SLOT(DrawWireMeshClick()));
	QWidget::connect(ui.ColorButton,SIGNAL(clicked()),this,SLOT(ColorButtonClick()));
	QWidget::connect(BG_source,SIGNAL(buttonClicked(int)),this,SLOT(RG_sourceBlendClick(int)));
	QWidget::connect(BG_dest,SIGNAL(buttonClicked(int)),this,SLOT(RG_destBlendClick(int)));
	QWidget::connect(ui.cbSortPoints,SIGNAL(clicked()),this,SLOT(cbSortPoints_clicked()));
	QWidget::connect(ui.cbFrontToBack,SIGNAL(clicked()),this,SLOT(cbFrontToBack_clicked()));
	

	QVBoxLayout* tl=new QVBoxLayout;
	tl->setMargin(0);
	tl->setSpacing(0);
	ui.FrameVS->setLayout(tl);
	QVBoxLayout* cl=new QVBoxLayout;
	cl->setMargin(0);
	cl->setSpacing(0);
	ui.FramePS->setLayout(cl);
	

}

GLShaderMaterialEditorWidget::~GLShaderMaterialEditorWidget(void)
{
	freeVSEditor();
    freePSEditor();
	delete BG_source;
	delete BG_dest;
}

void GLShaderMaterialEditorWidget::setup( GLShaderMaterial * obj )
{
	m_ShaderMaterial = obj;
	
   createPSEditor( m_ShaderMaterial->m_PixelShader );
   createVSEditor( m_ShaderMaterial->m_VertexShader );

   m_VertexDescriptorEditor = new VertexDescriptorEditor;
   m_VertexDescriptorEditorWidget = (VertexDescriptorEditorWidget *)m_VertexDescriptorEditor->makeEditor( this,
         m_ShaderMaterial->m_VertexDescriptor );
   m_VertexDescriptorEditorWidget->setParent(ui.Page_Vdesc);
   

   m_ParameterEditor = (ClassEditor*)m_ShaderMaterial->m_Parameters->getClass()->getGUIDescriptor()->getDefaultClassEditor()->newInstance();
   m_ParameterEditorWidget = m_ParameterEditor->makeEditor( this, m_ShaderMaterial->m_Parameters );
   m_ParameterEditorWidget->setParent(ui.Page_Par);
  

   updateEditor();
}

void GLShaderMaterialEditorWidget::updateEditor()
{
   ui.MatName->setText(QString(m_ShaderMaterial->getName().c_str()));
   switch( m_ShaderMaterial->getRenderMode() )
   {
      case GLShaderMaterial::RENDERMODE_POINTS:
         ui.RenderMode->setCurrentIndex(0);
      break;
      case GLShaderMaterial::RENDERMODE_QUADS:
         ui.RenderMode->setCurrentIndex(1);
      break;
      case GLShaderMaterial::RENDERMODE_NORMALS:
         ui.RenderMode->setCurrentIndex(2);
      break;
      case GLShaderMaterial::RENDERMODE_SPLATTING:
         ui.RenderMode->setCurrentIndex(3);
      break;
		case GLShaderMaterial::RENDERMODE_SLOWSPLATTING:
			ui.RenderMode->setCurrentIndex(4);
			break;
   }
   m_ShaderEditorVS->update();
   m_ShaderEditorPS->update();
   //m_VertexDescriptorEditor->update();

   ui.Scale_Edit->setValue(m_ShaderMaterial->getScaleValue());
   ui.sbPointSize->setValue(m_ShaderMaterial->getPointSize());

	ui.bDontChangeState->setChecked( m_ShaderMaterial->getNoStateChanges() );
	ui.cb_DepthTest->setChecked(m_ShaderMaterial->getDepthTest());
   ui.cb_DepthWrite->setChecked(m_ShaderMaterial->getDepthWrite());
   ui.cb_blending->setChecked(m_ShaderMaterial->getBlending());
   ui.cbVertexShaderPointSizeControl->setChecked(m_ShaderMaterial->getVSPointSizeControl());
   ui.bPointSpriteMode->setChecked( m_ShaderMaterial->getPointSpriteMode()  );

   switch(m_ShaderMaterial->getSourceBlendFunc())
   {
	case 0: ui.radioButton->setChecked(true);break;
	case 1: ui.radioButton_2->setChecked(true);break;
	case 2: ui.radioButton_3->setChecked(true);break;
	case 3: ui.radioButton_4->setChecked(true);break;
	default: break;

   }

   
   switch(m_ShaderMaterial->getDestBlendFunc())
   {
	case 0: ui.radioButton_5->setChecked(true);break;
	case 1: ui.radioButton_6->setChecked(true);break;
	case 2: ui.radioButton_7->setChecked(true);break;
	case 3: ui.radioButton_8->setChecked(true);break;
	default: break;

   }

    ui.RG_sourceblend->setEnabled(ui.cb_blending->checkState());
    ui.RG_destBlend->setEnabled(ui.cb_blending->checkState());
    
 
   ui.ForceTriangleNormal->setChecked( m_ShaderMaterial->getForceTriangleNormal());
   ui.DrawWireMesh->setChecked(m_ShaderMaterial->getDrawWireMesh());

    card32 c=m_ShaderMaterial->getWireMeshColor();
    QColor color=QColor((QRgb)c);
	QPalette palette;
	palette.setColor(QPalette::Window,color);
	ui.ColorLabel->setPalette(palette) ;
	ui.cbSortPoints->setChecked(m_ShaderMaterial->getSortPoints());
	ui.cbFrontToBack->setChecked(m_ShaderMaterial->getFrontToBack());
}


void GLShaderMaterialEditorWidget::createVSEditor( Shader * shader) 
{
	m_ShaderEditorVS = new ShaderEditor;
	m_ShaderEditorWidgetVS = (ShaderEditorWidget *)m_ShaderEditorVS->makeEditor( ui.FrameVS, shader );
	m_ShaderEditorWidgetVS->setObjectName("VS_Memo");
	m_ShaderEditorWidgetVS->setTestContext( m_TestContext );
	m_ShaderEditorWidgetVS->resize( ui.FrameVS->size() );
	ui.FrameVS->layout()->addWidget(m_ShaderEditorWidgetVS);

};
void GLShaderMaterialEditorWidget::freeVSEditor(){
   delete m_ShaderEditorWidgetVS;
   m_ShaderEditorWidgetVS = NULL;
   delete m_ShaderEditorVS;
   m_ShaderEditorVS = NULL;};

void GLShaderMaterialEditorWidget::createPSEditor( Shader * shader){
	m_ShaderEditorPS = new ShaderEditor;
	m_ShaderEditorWidgetPS = (ShaderEditorWidget *)m_ShaderEditorPS->makeEditor( ui.FramePS, shader );
	m_ShaderEditorWidgetPS->setObjectName("PS_Memo");
	m_ShaderEditorWidgetPS->setTestContext( m_TestContext );
	ui.FramePS->layout()->addWidget(m_ShaderEditorWidgetPS);
};

void GLShaderMaterialEditorWidget::freePSEditor(){
   delete m_ShaderEditorWidgetPS;
   m_ShaderEditorWidgetPS = NULL;
   delete m_ShaderEditorPS;
   m_ShaderEditorPS = NULL;
};

void GLShaderMaterialEditorWidget::setTestContext(GLContext * context )
{
	m_TestContext = context;
	m_ShaderEditorWidgetVS->setTestContext( m_TestContext );
	m_ShaderEditorWidgetPS->setTestContext( m_TestContext );
}


void GLShaderMaterialEditorWidget::resizeEvent( QResizeEvent *e)
{
	if( m_ShaderEditorWidgetVS )
		m_ShaderEditorWidgetVS->resize( ui.FrameVS->size() );
	if( m_ShaderEditorWidgetPS )
		m_ShaderEditorWidgetPS->resize( ui.FramePS->size() );
}


void GLShaderMaterialEditorWidget::MatNameChange() {

	m_ShaderMaterial->setName( ui.MatName->text().toLatin1().constData() );
};
void GLShaderMaterialEditorWidget::RenderModeChange() {

	switch( ui.RenderMode->currentIndex() )
   {
   case 0:
      m_ShaderMaterial->setRenderMode( GLShaderMaterial::RENDERMODE_POINTS);
      break;
   case 1:
      m_ShaderMaterial->setRenderMode( GLShaderMaterial::RENDERMODE_QUADS);
      break;
   case 2:
      m_ShaderMaterial->setRenderMode( GLShaderMaterial::RENDERMODE_NORMALS);
      break;
   case 3:
      m_ShaderMaterial->setRenderMode( GLShaderMaterial::RENDERMODE_SPLATTING);
      break;
	case 4:
		m_ShaderMaterial->setRenderMode( GLShaderMaterial::RENDERMODE_SLOWSPLATTING);
		break;
   }
   
};
void GLShaderMaterialEditorWidget::Button_RebuildObjectsClick() {

	m_ShaderMaterial->increaseVersion();
};
void GLShaderMaterialEditorWidget::ButtonLoadVSClick( ) {

   
   QString s = QFileDialog::getOpenFileName(
                    this,
                    "Choose a vertexshader",
                    "",
                    " (*.vs.object)");

	if( !s.isEmpty() )
   {
      BinaryInputObjectStream in(s.toLatin1().constData());
      Persistent * testObj;
      in.readObject(testObj);
      if( dynamic_cast<Shader*>(testObj) == NULL )
          QMessageBox::information(this, "GEOXL",
			"The selected file is not a shader.");
      else
      {
         freeVSEditor();
         delete m_ShaderMaterial->m_VertexShader;
         m_ShaderMaterial->m_VertexShader = (Shader *)testObj;
         createVSEditor( m_ShaderMaterial->m_VertexShader);
		 m_ShaderEditorWidgetVS->show();
      }

      update();
   }
};
void GLShaderMaterialEditorWidget::ButtonSaveVSClick( ) {

   QString s = QFileDialog::getSaveFileName(this, "Save File...", "", "VertexShaders (*.vs.object);;All Files (*.*)");

   if( !s.isEmpty() )
   {
      BinaryOutputObjectStream out(s.toLatin1().constData());
      out.writeObject( m_ShaderMaterial->m_VertexShader );
   }
};
void GLShaderMaterialEditorWidget::ButtonLoadPSClick( ) {

	 
	 QString s = QFileDialog::getOpenFileName(
                    this,
                    "Choose a pixelshader",
                    "",
                    " (*.ps.object)");


	 if( !s.isEmpty() )
   {
	   BinaryInputObjectStream in(s.toLatin1().constData());
      Persistent * testObj;
      in.readObject(testObj);
      if( dynamic_cast<Shader*>(testObj) == NULL )
          QMessageBox::information(this, "GEOXL",
			"The selected file is not a shader.");
      else
      {
         freePSEditor();
         delete m_ShaderMaterial->m_PixelShader;
         m_ShaderMaterial->m_PixelShader = (Shader *)testObj;
         createPSEditor( m_ShaderMaterial->m_PixelShader);
		 m_ShaderEditorWidgetPS->show();
      }

      update();
   }
};
void GLShaderMaterialEditorWidget::ButtonSavePSClick( ) {

	QString s = QFileDialog::getSaveFileName(this, "Save File...", "", "PixelShaders (*.ps.object);;All Files (*.*)");
	
	if( !s.isEmpty() )
   {
	   BinaryOutputObjectStream out(s.toLatin1().constData());
      out.writeObject( m_ShaderMaterial->m_PixelShader );
   }
};

void GLShaderMaterialEditorWidget::Scale_EditValueChange( ) {

	m_ShaderMaterial->setScaleValue( ui.Scale_Edit->value() );
};
void GLShaderMaterialEditorWidget::FE_pointSizeValueChange( ) {

	if (ui.sbPointSize->value() <= 0) ui.sbPointSize->setValue(1);
	m_ShaderMaterial->setPointSize(ui.sbPointSize->value());
};
void GLShaderMaterialEditorWidget::CB_blendingClick( ) {

	ui.RG_sourceblend->setEnabled(ui.cb_blending->checkState());
    ui.RG_destBlend->setEnabled(ui.cb_blending->checkState());
    m_ShaderMaterial->setBlending(ui.cb_blending->checkState());
};

void GLShaderMaterialEditorWidget::cb_vspointsizeClick( ) {
    m_ShaderMaterial->setVSPointSizeControl(ui.cbVertexShaderPointSizeControl->checkState());
};

void GLShaderMaterialEditorWidget::CB_depthTestClick( ) {
	
	m_ShaderMaterial->setDepthTest(ui.cb_DepthTest->checkState());
};
void GLShaderMaterialEditorWidget::CB_depthWriteClick( ) {
	
	m_ShaderMaterial->setDepthWrite(ui.cb_DepthWrite->checkState());
};
void GLShaderMaterialEditorWidget::RG_sourceBlendClick(int id) {
	
	m_ShaderMaterial->setSourceBlendFunc(id);
};
void GLShaderMaterialEditorWidget::RG_destBlendClick(int id ) {
	
	m_ShaderMaterial->setDestBlendFunc(id);
};
void GLShaderMaterialEditorWidget::ImportShadersClick( ) {

    

   string binPath = getConfigurationString( "BIN_PATH" );
   if( binPath.length() )
   {
      binPath += "/shaders";
   }
   else binPath="";
   
   QString s = QFileDialog::getOpenFileName(
                    this,
                    "Choose a textfile",
					binPath.c_str(),
                    " (*.*)");
   

   if( !s.isEmpty() )
   {
	   if( m_ShaderMaterial->importFromTextFile( s.toLatin1().constData() ) )
      {
         updateEditor();
      }
   }

	
};
void GLShaderMaterialEditorWidget::ForceTriangleNormalClick( ) {

	m_ShaderMaterial->setForceTriangleNormal( ui.ForceTriangleNormal->checkState() );
};
void GLShaderMaterialEditorWidget::DrawWireMeshClick( ) {

	m_ShaderMaterial->setDrawWireMesh( ui.DrawWireMesh->checkState() );
};
void GLShaderMaterialEditorWidget::ColorButtonClick( ) {

	
	QColor color=QColorDialog::getColor();
	QPalette palette;
	palette.setColor(QPalette::Window,color);
	
	ui.ColorLabel->setPalette(palette) ;
	m_ShaderMaterial->m_WireMeshColor =  (card32)color.rgb() ;

};

#include "GLShaderMaterialEditorWidget_moc.cpp"
void GLShaderMaterialEditorWidget::on_bDontChangeState_clicked() 
{
	m_ShaderMaterial->setNoStateChanges( ui.bDontChangeState->isChecked() );
}

void GLShaderMaterialEditorWidget::on_bPointSpriteMode_clicked() 
{
	m_ShaderMaterial->setPointSpriteMode( ui.bPointSpriteMode->isChecked() );
}

void GLShaderMaterialEditorWidget::cbSortPoints_clicked() 
{
	m_ShaderMaterial->setSortPoints(ui.cbSortPoints->isChecked());
}

void GLShaderMaterialEditorWidget::cbFrontToBack_clicked() 
{
	m_ShaderMaterial->setFrontToBack(ui.cbFrontToBack->isChecked());
}
