#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ShaderEditorWidget.h"
//---------------------------------------------------------------------------
#include "Shader.h"
#include "QTGui/QMessageBox.h"
#include "ShaderSyntaxHighlighter.h"
#include <qtGui/qtextcursor>
//---------------------------------------------------------------------------
using namespace geoxl;

IMPLEMENT_CLASS_EDITOR( Shader )

 GLContext	*	ShaderEditorWidget::m_DefaultGLContext = NULL;

ShaderEditorWidget::ShaderEditorWidget(QWidget * parent, Qt::WFlags f)
:QWidget(parent,f)
{
	//call ui.setupUi(this) to set up the layout from QtDesigner
	ui.setupUi(this);
	m_Shader = NULL;
    m_TestContext = NULL;
	ui.ShaderClass->addItem(QString("select"));
	ui.ShaderClass->addItem(QString("ARB"));
	ui.ShaderClass->addItem(QString("CG"));
	ui.ShaderType->addItem(QString("select"));
	ui.ShaderType->addItem(QString("Vertex"));
	ui.ShaderType->addItem(QString("Fragment"));
	//Example: easy access to QtDesigner buttons, etc. by ui object
	QWidget::connect(ui.ButtonCompile,SIGNAL(clicked()),this,SLOT(sltCompile()));
	QWidget::connect(ui.Button_Save,SIGNAL(clicked()),this,SLOT(sltSave()));
	QWidget::connect(ui.Button_Load,SIGNAL(clicked()),this,SLOT(sltLoad()));
	QWidget::connect(ui.ShaderClass,SIGNAL(activated(int)),this,SLOT(sltShaderClassChange(int)));
	QWidget::connect(ui.ShaderType,SIGNAL(activated(int)),this,SLOT(sltShaderTypeChange(int)));
	QWidget::connect(ui.Edit,SIGNAL(textChanged()),this,SLOT(sltEditChange()));
	
	ui.Edit->setTabStopWidth( 30 );
	ShaderSyntaxHighlighter * highlighter = new ShaderSyntaxHighlighter( ui.Edit );
	connect(ui.Edit, SIGNAL(cursorPositionChanged()), SLOT(sltCursorPosChanged()));
	sltCursorPosChanged();
}

ShaderEditorWidget::~ShaderEditorWidget(void)
{
}

void ShaderEditorWidget::setup( Shader * shader )
{
	m_Shader = shader;
	updateEditor();
}



void ShaderEditorWidget::updateEditor()
{
	ui.Output->clear();
	string tmp = replaceChar( removeChar(m_Shader->getSource(),'\r'),'\n', "\r\n" );
	ui.Edit->setPlainText(QString(tmp.c_str()));

	if( m_Shader->getShaderClass() != Shader::INVALID )
		ui.ShaderClass->setCurrentIndex(m_Shader->getShaderClass());
	else
	  ui.ShaderClass->setCurrentIndex(0);

	if( m_Shader->getShaderType() != Shader::INVALID )
	  ui.ShaderType->setCurrentIndex(m_Shader->getShaderType());
	else
	  ui.ShaderType->setCurrentIndex(0);
}

Shader* ShaderEditorWidget::getObj(){

	return m_Shader;

};

string ShaderEditorWidget::replaceChar( const string &str, char c, const string &replaceStr)
{
   string ret;
   for( card32 i=0;i<str.length();i++ )
      if( str[i] == c )
         ret += replaceStr;
      else
         ret += str[i];

   return ret;
}

string ShaderEditorWidget::removeChar( const string &str, char c)
{
   string ret;
   for( card32 i=0;i<str.length();i++ )
      if( str[i] != c )
         ret += str[i];

   return ret;
}

void ShaderEditorWidget::sltCompile(){
	
   m_Shader->setChanged();
	if( !m_TestContext )
		m_TestContext = m_DefaultGLContext;

   if( !m_TestContext)
   {
      ui.Output->setPlainText(QString("Testcontext is NULL!. (required for compilation)"));
      return;
   }
   if( (m_Shader->getShaderType() == Shader::INVALID) || (m_Shader->getShaderClass() == Shader::INVALID) )
   {
	   QMessageBox::information(this,"Error", "Chose Shadertype AND Shaderclass before compiling","Error");
      return;
   }
   if( m_Shader->compile(m_TestContext) )
   {
	   ui.Output->setPlainText(QString("shader compiled"));
   }
   else
   {
      string tmp = m_Shader->getError();
	  ui.Output->setPlainText(QString(replaceChar( tmp, '\n', "\r\n").c_str()));
   }

	
}

void ShaderEditorWidget::sltSave(){
//To beImplemented in GLMaterialEditor

}
void ShaderEditorWidget::sltLoad(){
//To beImplemented in GLMaterialEditor
}
void ShaderEditorWidget::sltShaderClassChange(int n){
 if( ui.ShaderClass->currentIndex() == 0 )
   {
      if( m_Shader->getShaderClass() != Shader::INVALID )
         ui.ShaderClass->setCurrentIndex(m_Shader->getShaderClass());
   }
   else
   {
	   m_Shader->setShaderClass( ui.ShaderClass->currentIndex() );
   }
}
void ShaderEditorWidget::sltShaderTypeChange(int n){

if( ui.ShaderType->currentIndex() == 0 )
   {
      if( m_Shader->getShaderClass() != Shader::INVALID )
         ui.ShaderType->setCurrentIndex(m_Shader->getShaderType());
   }
   else
   {
      m_Shader->setShaderType( ui.ShaderType->currentIndex() );
   }
 
}
void ShaderEditorWidget::sltEditChange(){
m_Shader->setSource( removeChar(ui.Edit->toPlainText().toLatin1().constData(),'\r' ) );
}

void ShaderEditorWidget::sltCursorPosChanged()
{
	QTextCursor cursor = ui.Edit->textCursor();
	int line = 0;
	QString text = ui.Edit->toPlainText();
	for( int i=0;i<cursor.position();i++ )
		if( text[i] == '\n' )
			line++;
		
	ui.LineLabel->setText( QString("Line: ") + QString::number( line ) );
}

#include "ShaderEditorWidget_moc.cpp"
