//---------------------------------------------------------------------------
#include "StdAfx.h"
//---------------------------------------------------------------------------
#include "ShaderSyntaxHighlighter.h"
//---------------------------------------------------------------------------
#include <map>
//---------------------------------------------------------------------------
using namespace geoxl;

void ShaderSyntaxHighlighter::highlightBlock ( const QString & text )
{
	QTextCharFormat multiLineCommentFormat;
	multiLineCommentFormat.setForeground(Qt::darkGreen);

	QTextCharFormat keywordFormat;
	keywordFormat.setForeground( Qt::darkRed );
	keywordFormat.setFontWeight(QFont::Bold);

	QTextCharFormat constFormat;
	constFormat.setForeground( Qt::darkBlue );
	constFormat.setFontWeight(QFont::Bold);


	QRegExp startExpression("/\\*");
	QRegExp endExpression("\\*/");

	QRegExp keywords("\\b("
		"(main|uniform|sampler2D|samplerRECT|samplerCUBE|"
		"sampler1D|sampler3D|struct|return|POSITION|COLOR|NORMAL|float|half|bool|int|fixed)"
		"|((ATTR)[0-9])"
		"|((TEXCOORD)[0-7])"
		"|((float|half|bool|int|fixed)[1-4])"
		"|((float|half|bool|int|fixed)[1-4]x[1-4])"
		")\\b");

	QRegExp constExp("\\b(matMV|matMVP|matMVinv|matMVinvtrans)\\b");

	setCurrentBlockState(0);

	int startIndex = 0;
	if (previousBlockState() != 1)
	  startIndex = text.indexOf(startExpression);

	while (startIndex >= 0) 
	{
		int endIndex = text.indexOf(endExpression, startIndex);
		int commentLength;
		if (endIndex == -1) 
		{
			setCurrentBlockState(1);
			commentLength = text.length() - startIndex;
		}
		else 
		{
			commentLength = endIndex - startIndex + endExpression.matchedLength();
		}
		setFormat(startIndex, commentLength, multiLineCommentFormat);
		startIndex = text.indexOf(startExpression,startIndex + commentLength);
	}


	string stlText = text.toLatin1().constData();
	int begin = (int)stlText.find("//");
	while( begin >= 0 )
	{
		int end = (int)stlText.find( "\n", begin );
		if( end == -1 )
		{
			setFormat( begin, (int)stlText.length() - begin, multiLineCommentFormat );
			begin = -1;
		}
		else
		{
			setFormat( begin, end - begin, multiLineCommentFormat );
			begin = (int)stlText.find( "//",end);
		}
	}

	int index = text.indexOf(keywords);
	while (index >= 0) 
	{
		int length = keywords.matchedLength();
		if( format( index ) != multiLineCommentFormat )
			setFormat(index, length, keywordFormat);
		index = text.indexOf(keywords, index + length);
	}

	index = text.indexOf(constExp);
	while (index >= 0) 
	{
		int length = constExp.matchedLength();
		if( format( index ) != multiLineCommentFormat )
			setFormat(index, length, constFormat);
		index = text.indexOf(constExp, index + length);
	}

}



#include "ShaderSyntaxHighlighter_moc.cpp"