//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "initSceneGraph.h"
//---------------------------------------------------------------------------

/*
	Please separate between system code and interface code ( editors ), in 
	order to build a non-interface commandline version of geoxl.

File structure:
	includes
		..
		..
	interface includes
		..
		..
	initSceneGraph()
	{
		init geoxl classes
		..
		..
		init geoxl class editors
		..
		..
	}
	shutdownSceneGraph()
	{
		shutdown geoxl classes
		..
		..
		shutdown geoxl class editors
		..
		..
				
	}
*/

//---------------------------------------------------------------------------
#include "AttachedDataTypes.h"
#include "Scene.h"
#include "SceneGraphIterators.h"
#include "SceneGraphNode.h"
#include "SceneGraphState.h"
#include "SGIndexedAndByNameSceneGraphIteratorAdaptor.h"
#include "SGListNode.h"
#include "SGNodeTypeFilterAdaptor.h"
#include "SGObjectNode.h"
#include "SGSceneObjectFilterAdaptor.h"
#include "SGAnimationNode.h"
#include "SGSimpleAnimationNode.h"
#include "SGRelativeTimeAnimationNode.h"
#include "SGStateChangeNode.h"
#include "SGStateChanger.h"
#include "SGTransformationNode.h"
#include "SGTransformationStateChanger.h"
#include "SGSetVariable4FNode.h"
#include "SGVariable4FStateChanger.h"
#include "StandardSceneGraphIterator.h"
#include "VisitSGNodesOnceIterator.h"
#include "AttachedData.h"
#include "GLMaterial.h"
#include "LegacyClassNameTranslation.h"
#include "ProcessPointsCmdObj.h"
#include "SGAllocatorList.h"
#include "SGCameraNode.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


void geoxl::initSceneGraph()
{
   AttachedRenderObject::init(AttachedData::getClass());
   AttachedDictionaryObject::init(AttachedData::getClass());

   SceneGraphIterator::init(Object::getClass());
      StandardSceneGraphIterator::init(SceneGraphIterator::getClass());
      SGNodeTypeFilterAdaptor::init(SceneGraphIterator::getClass());
         SGSceneObjectFilterAdaptor::init(SGNodeTypeFilterAdaptor::getClass());
      SGIndexedAndByNameSceneGraphIteratorAdaptor::init(SceneGraphIterator::getClass());
      VisitSGNodesOnceIterator::init(SceneGraphIterator::getClass());

   SceneGraphNode::init(Persistent::getClass());
      SGListNode::init(SceneGraphNode::getClass());
         SGAnimationNode::init(SGListNode::getClass());
         SGSimpleAnimationNode::init(SGAnimationNode::getClass());
         SGRelativeTimeAnimationNode::init(SGAnimationNode::getClass());
         
      SGStateChangeNode::init(SceneGraphNode::getClass());
         SGTransformationNode::init(SGStateChangeNode::getClass());
         SGSetVariable4FNode::init(SGStateChangeNode::getClass());
      SGObjectNode::init(SceneGraphNode::getClass());
		SGCameraNode::init(SceneGraphNode::getClass());

   SGStaticState::init(Persistent::getClass());
   SGDynamicState::init(Persistent::getClass());
   SceneGraphState::init(Persistent::getClass());
   SGStateChanger::init(Persistent::getClass());
      SGTransformationStateChanger::init(SGStateChanger::getClass());
      SGVariable4FStateChanger::init(SGStateChanger::getClass());
   
   Scene::init(Persistent::getClass());

   GLMaterial::init(Persistent::getClass());

	ProcessPointsCmdObj::init(CommandObject::getClass());

	SGAllocatorList::init(Persistent::getClass());

	LegacyClassNameTranslation::registerTranslation("PCMaterial", GLMaterial::getClass());
  // LegacyClassNameTranslation::registerTranslation("PointCloudScene", Scene::getClass());
  // LegacyClassNameTranslation::registerTranslation("PCRenderState", SceneGraphState::getClass());
}



void geoxl::shutdownSceneGraph()
{
   AttachedRenderObject::shutdown();
   AttachedDictionaryObject::shutdown();
   SceneGraphIterator::shutdown();
   StandardSceneGraphIterator::shutdown();
   SGNodeTypeFilterAdaptor::shutdown();
   SGSceneObjectFilterAdaptor::shutdown();
   SGIndexedAndByNameSceneGraphIteratorAdaptor::shutdown();
   VisitSGNodesOnceIterator::shutdown();
   SceneGraphNode::shutdown();
   SGListNode::shutdown();
	SGAnimationNode::shutdown();
	SGSimpleAnimationNode::shutdown();
	SGRelativeTimeAnimationNode::shutdown();
   SGStateChangeNode::shutdown();
   SGObjectNode::shutdown();
	SGCameraNode::shutdown();
   SGTransformationNode::shutdown();
   SGStaticState::shutdown();
   SGDynamicState::shutdown();
   SceneGraphState::shutdown();
   SGStateChanger::shutdown();
   SGTransformationStateChanger::shutdown();
   Scene::shutdown();
   GLMaterial::shutdown();
	ProcessPointsCmdObj::shutdown();
	SGAllocatorList::shutdown();
}

