//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "GLMaterial.h"
//---------------------------------------------------------------------------
#include "DebugOutput.h"
#include "FixedArrayClassProperty.h"
#include "StringClassProperty.h"
#include "BooleanClassProperty.h"
#include "NumericalClassProperty.h"
#include "NumericalClassProperty_templCode.h"
#include "PAssert.h"
#include "PointCloudIterators.h"
#include "PointCloud.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "GLBuffer.h"
#include "GLContext.h"
#include "LinearAlgebra.h"
#include "LinearAlgebra_templCode.h"
#include "VertexDescriptor.h"
#include "Tools.h"
//---------------------------------------------------------------------------

using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------



MetaClass* GLMaterial::GLMaterialMetaClass = NULL;

const MetaClass* GLMaterial::getInstanceClass() const {
   return getClass();
}


const MetaClass* GLMaterial::getClass() {
   if (GLMaterialMetaClass==NULL) {
      GLMaterialMetaClass = new MetaClass("GLMaterial", "no description.",
                                          NULL, sizeof(GLMaterial));
   }
   return GLMaterialMetaClass;
}

void GLMaterial::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   GLMaterialMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT( GLMaterial );
   ADD_STRING_PROP_METHOD( name, 0, &getName, &setName );
   SET_USER_EDIT( false );

}

void GLMaterial::shutdown() {
   delete GLMaterialMetaClass;
}


// ----


MetaClass* GLRenderObject::GLRenderObjectMetaClass = NULL;

const MetaClass* GLRenderObject::getInstanceClass() const {
   return getClass();
}

const MetaClass* GLRenderObject::getClass() {
   if (GLRenderObjectMetaClass == NULL) {
      GLRenderObjectMetaClass = new MetaClass("GLRenderObject", "no description.",
                                              NULL, sizeof(GLRenderObject));
   }
   return GLRenderObjectMetaClass;
}

void GLRenderObject::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   GLRenderObjectMetaClass->copyParentProperties();
}

void GLRenderObject::shutdown() {
   delete GLRenderObjectMetaClass;
}


// ----


MetaClass* BufferedRenderObject::BufferedRenderObjectMetaClass = NULL;

const MetaClass* BufferedRenderObject::getInstanceClass() const {
   return getClass();
}

const MetaClass* BufferedRenderObject::getClass() {
   if (BufferedRenderObjectMetaClass==NULL) {
      BufferedRenderObjectMetaClass = new MetaClass("BufferedRenderObject", "no description.",
                                  NULL, sizeof(BufferedRenderObject));
   }
   return BufferedRenderObjectMetaClass;
}

void BufferedRenderObject::init(const MetaClass *parentClass, card32 uniqueIdentifier) {
   ((MetaClass*)getClass())->init(uniqueIdentifier, parentClass);
   BufferedRenderObjectMetaClass->copyParentProperties();
}

void BufferedRenderObject::shutdown() {
   delete BufferedRenderObjectMetaClass;
}


// ----

GLMaterial::GLMaterial() {
   name = "unnamed material";
//   m_Container = new AttachedDataContainer;
}

GLMaterial::~GLMaterial()
{
//   delete m_Container;
}


void GLMaterial::assign(const Object* obj, CopyContext *context )
{
   GLMaterial * mat = (GLMaterial*)obj;
   name = mat->name;
   forget(context);
}


BufferedRenderObject::BufferedRenderObject()
{
   m_VertexBuffer = NULL;
   m_RenderMode = RENDERMODE_POINTS;
}

BufferedRenderObject::~BufferedRenderObject()
{
   if( m_VertexBuffer )
      delete m_VertexBuffer;
}

void BufferedRenderObject::setup( GLContext * context, const VertexDescriptor &vdesc)
{
   if( m_VertexBuffer )
      throw PException("BufferedRenderObject::setup - vertex buffer already exists! multiple use of setup is not allowed!");

   GLRenderObject::setup(context);
   m_VertexBuffer = new GLVertexBuffer( context, vdesc );
}

void BufferedRenderObject::setBufferData( void * data, card32 offset, card32 numVertices )
{
   if( m_VertexBuffer == NULL )
      throw PException("BufferedRenderObject::setBufferData - vertex buffer doesn't exists! use setup before using setBufferData");

   m_VertexBuffer->init( numVertices );
   m_VertexBuffer->writeToBuffer( data, offset, numVertices );
}

void BufferedRenderObject::setMappingFlag( bool f )
{
   if( m_VertexBuffer )
      m_VertexBuffer->setMappingFlag(f);
}




