//---------------------------------------------------------------------------
#ifndef GLMaterialH
#define GLMaterialH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
#include "PointFlags.h"
//---------------------------------------------------------------------------
#include <map>
//---------------------------------------------------------------------------

namespace geoxl {

class PointSet;
class Shader;
class VertexDescriptor;
class Scene;
class SceneObject;
class SceneObjectIterator;
class MemoryManager;
class GLContext;
class GLVertexBuffer;
class GLShader;
class SceneGraphState;


class SCENEGRAPH_API GLMaterial : public Persistent {
 private:
   static MetaClass *GLMaterialMetaClass;
   string name;

 public:
   GLMaterial();

   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   virtual void assign(const Object* obj, CopyContext *context=NULL );

   // new methods
   string getName() const {return name;}
   void setName(string n) {name = n;}

   /// Draw point cloud defined by "iter", beeing part of "cloud", which itself beeing a part of "scene".
   /// GL buffer objects are created automatically and attached to the input parameters.
   /// A memory manager can be specified for limiting memory usage (as well as for final cleanup).
   /// Flagged point clouds: points not including "includeFlags" or including "excludeFlags" are ignored.
   virtual void draw(Scene *scene, SceneGraphState *state, SceneObject *obj,
      SceneObjectIterator *iter, GLContext *context, MemoryManager * mm = NULL,
      const int32 includeFlags = 0, const int32 excludeFlags = PF_FLAG_INVALID ) = 0;

   virtual void connect( Scene * scene, GLContext * context ) {}

   virtual ~GLMaterial();
};



class SCENEGRAPH_API GLRenderObject : public Object {
 private:
   static MetaClass *GLRenderObjectMetaClass;

   GLContext *ctxt;

 public:
   virtual ~GLRenderObject(){};
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   void setup(GLContext *ctxt) {this->ctxt = ctxt;}
   virtual bool isCompatible(GLContext *ctxt) = 0;
};



class SCENEGRAPH_API BufferedRenderObject : public GLRenderObject
{
 private:
   static MetaClass *BufferedRenderObjectMetaClass;
   static Object* newInstance();

   GLVertexBuffer * m_VertexBuffer;
   card32               m_RenderMode;
   card64               m_Version;

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass, card32 uniqueIdentifier);
   static void shutdown();

   BufferedRenderObject();
   virtual ~BufferedRenderObject();

   // rendermode
   static const card32 RENDERMODE_POINTS = 1;
   static const card32 RENDERMODE_QUADS = 2;
   static const card32 RENDERMODE_LINES = 3;
   void setRenderMode( card32 rm ) {m_RenderMode = rm;}
   card32 getRenderMode() const {return m_RenderMode;}

   // version
   card64 getVersion() const {return m_Version;}

   // creation
   void setVersion( card64 version ) {m_Version = version;}
   void setup( GLContext * context, const VertexDescriptor &vdesc);
   void setBufferData( void * data, card32 offset, card32 numVertices );
   void setMappingFlag( bool f);

   // draw
   // for special use
   GLVertexBuffer * getGLVertexBuffer() const {return m_VertexBuffer;}


};




}

#endif

