//---------------------------------------------------------------------------
#include "stdafx.h"
#include "GlobalMacros.h"
//---------------------------------------------------------------------------
#include "PointCloudTools.h"
#include "ParametricInCorePointCloud.h"

#include "Random.h"
#include "DebugOutput.h"

#include "SGIndexedAndByNameSceneGraphIteratorAdaptor.h"
#include "StandardSceneGraphIterator.h"
#include "SGObjectNode.h"
#include "SGListNode.h"
#include "StringHelper.h"
#include "PointSet.h"
#include "PointSet_templCode.h"
#include "NormalDistribution.h"
#include "VertexArray.h"
#include "ProgressWindow.h"
#include <float.h>
#include "SimpleAttachment.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//--------------------------------------------------------------------------

/**
* Find point cloud in scene
*/
PointCloud* geoxl::getPointCloud( Scene* scene, string pcID )
{
	if ( scene == NULL ) return NULL;
	PointCloud* pc = NULL;
	// Search for point cloud
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	while ( !it->atEnd() ){
		string name = it->getNameWithPath();
		if ( name == pcID ){
			SGObjectNode* sgo = dynamic_cast<SGObjectNode*>( it->getNode() );
			if (sgo != NULL) {
				pc = dynamic_cast<PointCloud*>( sgo->getSceneObject() );
				break;
			}
		}
		it->next();
	}
	delete it;
	return pc;
}

/**
* Find any scene object
*/
SceneObject* geoxl::getSceneObject( Scene* scene, string pcID ) {
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	while ( !it->atEnd() ){
		if ( it->getNameWithPath() == pcID ){
			SGObjectNode* sgo = dynamic_cast<SGObjectNode*>( it->getNode() );
			if (sgo != NULL) {
				delete it;
				return sgo->getSceneObject();
			}
		}
		it->next();
	}
	delete it;
	return NULL;
}

/**
* Replace point cloud 'pcID' with 'newPC'
*/
void geoxl::replacePointCloud( Scene* scene, string pcID, PointCloud* newPC )
{
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	while ( !it->atEnd() ){
		if ( it->getNameWithPath() == pcID ){
			SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( it->getNode() );
			sgon->clearAndSetup( newPC );
			return;
		}
		it->next();
	}
	delete it;
}

SCENEGRAPH_API void geoxl::addOrReplacePointCloud( Scene* scene, string pcID, PointCloud* newPC )
{
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	while ( !it->atEnd() ){
		if ( it->getNameWithPath() == pcID ){
			SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( it->getNode() );
			sgon->clearAndSetup( newPC );
			return;
		}
		it->next();
	}
	delete it;

	addSceneObject(scene, newPC, pcID);
}

SceneGraphNode* geoxl::addSceneObject( Scene * scene, SceneObject *so, string name )
{
	SGObjectNode *n = new SGObjectNode();
	n->clearAndSetup( so );
	if( ((int)name.find( "/" ))<0  )
	{
		n->setName( name );
		SGListNode *rootNode = dynamic_cast<SGListNode*>(scene->getRootNode());
		rootNode->addNode( n );
	}
	else
	{
		vector<string> words = splitString( name, "/" );
		SGListNode *node = dynamic_cast<SGListNode*>(scene->getRootNode());
		if (node->getName() != words[0]) {
			// root name missing
			words.insert(words.begin(), node->getName());
		}
		pAssert( node->getName() == words[0] );
		for( card32 i=1;i<words.size()-1;i++ )
		{
			bool found = false;
			for( card32 j=0;j<node->getNumChildNodes( NULL );j++ )
			{
				SGListNode * tmpNode = dynamic_cast<SGListNode*>(node->getChildNode( NULL, j ));
				if( tmpNode )
					if( tmpNode->getName() == words[i] )
					{
						found = true;
						node = tmpNode;
						break;
					}
			}

			pAssert( found ); // maybe create listnode here ?
		}
		n->setName( words.back() );
		node->addNode( n );
	}
	return n;
}


/**
* Add point cloud 'pc' as a child of the root node; set node name to 'name'.
*/
void geoxl::addPointCloud( Scene* scene, PointCloud* pc, string name )
{
	addSceneObject(scene,pc,name);
}


/**
* Add point cloud 'pc' as a child of the root node; set node name to 'name'; returns really set name because this can differ from the requested name. always use this returned name afterwards!
*/
string geoxl::addPointCloudAndReturnNodeName( Scene* scene, PointCloud* pc, string name )
{
	SGObjectNode *n = new SGObjectNode();
	n->clearAndSetup( pc );
	n->setName( name );
	SGListNode *rootNode = dynamic_cast<SGListNode*>(scene->getRootNode());
	rootNode->addNode( n );
	return n->getName();
}


// Check if PointCloud has attribute (attrib, numComponents, dataType), if not add it
void geoxl::checkAttribute( Scene* scene, string id, string attrib, card8 numComponents, card8 dataType )
{
	// Find node in sg;
	PointCloud *pc = NULL;
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	while ( !it->atEnd() ){
		//debugOutput << it->getNameWithPath() << " vs. " << id << "\n";
		//debugOutput.flush();
		if ( it->getNameWithPath() == id ){
			SGObjectNode* sgon = dynamic_cast<SGObjectNode*>( it->getNode() );
			pc = dynamic_cast<PointCloud*>( sgon->getSceneObject() );
			if ( pc == NULL ){
				debugOutput << "geoxl::checkAttribute: invalid point cloud: " << id << "!\n";
				debugOutput.flush();
				return;
			}
			// Create new node with attribute
			if ( !pc->providesAttribute( attrib ) ){
					checkAttribute(pc, attrib, numComponents, dataType);
// 				VertexDescriptor *newDescr = (VertexDescriptor*)(pc->getDescr()->copy());
// 				newDescr->pushAttrib( mVAD( attrib, numComponents, dataType ) );
// 				// Replace old node
// 				newPC = pc->appendFields( newDescr );
// 				sgon->clearAndSetup( newPC );

			} else {
				//debugOutput << "Point cloud " << id << " already has attribute " << attrib << "!\n";
				//debugOutput.flush();
			}
			return;
		}
		it->next();
	}

	debugOutput << "geoxl::checkAttribute: can find point cloud: " << id << "!\n";
	debugOutput.flush();
}

void geoxl::checkAttribute(PointCloud * pc, string attrib, card8 numComponents, card8 dataType)
{
	if( !pc->providesAttribute( attrib ) )
	{
		VertexDescriptor vd;
		vd.pushAttrib( VAD( attrib, numComponents, dataType ) );
		UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
		if( upc )
		{
			PointSet * psNew = upc->getPointSet()->appendFields(&vd);
			upc->setPointSet(psNew);
		}
		else
		{
			PointCloud * pcCopy = pc->appendFields( &vd );
			pc->assign( pcCopy );
			delete pcCopy;
		}
	}
}

/**
* CheckAttribute: Check for (position, normal, flags, color)
*/
void geoxl::checkSceneAllAttributes(Scene* scene){
	SGLegacyPCSceneAdaptor sceneLA(scene);
	for ( card32 i=0; i<sceneLA.getNumPointClouds(); i++ ){
		string pcName = sceneLA.getNameWithPath( i );
		checkAttribute( scene, pcName, "color", 3, VAD::DATA_FORMAT_FLOAT32 );
		checkAttribute( scene, pcName, "flags", 1, VAD::DATA_FORMAT_INT32 );
		checkAttribute( scene, pcName, "normal", 3, VAD::DATA_FORMAT_FLOAT32 );
		checkAttribute( scene, pcName, "position", 3, VAD::DATA_FORMAT_FLOAT32 );
	}
}

/**
* Converts input PointCloud to UnstructuredInCorePointCloud. Returns NULL on Error.
* Input PointCloud is not deleted!!!
*/
UnstructuredInCorePointCloud* geoxl::convertToUnstructuredInCorePointCloud(PointCloud* inPC) {
	// input
	SceneObjectIterator *inIt = inPC->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	if (inIt == NULL) return NULL;
	BasicPointCloudIterator *inBasicIt = dynamic_cast<BasicPointCloudIterator*>(inIt);
	if (inBasicIt == NULL) {delete inIt; return NULL;}

	//output
	UnstructuredInCorePointCloud* outPC = new UnstructuredInCorePointCloud();
	outPC->clearAndSetup(inPC->getDescr(), 0);

	outPC->assignSettings(inPC);

	PointCloudIterator *outIt = outPC->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	if (outIt == NULL) { delete inIt; delete outPC; return NULL; };
	DynamicPointCloudIterator *outDynIt = dynamic_cast<DynamicPointCloudIterator*>(outIt);
	if (outDynIt==NULL) {delete inIt; delete outIt; delete outPC; return NULL; }

	char *buffer = new char[inPC->getDescr()->getSize()];

	while(!inBasicIt->atEnd()) {
		inBasicIt->getVertex(buffer);
		outDynIt->add(buffer);
		inBasicIt->next();
	};

	delete inIt;
	delete outIt;

	return outPC;
};

/// Return the bounding box of a point cloud
BoundingBox3f  geoxl::getPCBBox( PointCloud* pc )
{
	BoundingBox3f bb;
	if ( !pc->providesAttribute( "position" ) ) return bb;
	SceneObjectIterator *newIt = pc->createIterator( SOIT( SOIT::CAP_BASIC_PC ) );
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
	AAT POSITION = pc->getAAT( "position" );
	Matrix4f transformation = pc->getTransformation();
	basicIt->reset();
	bool bbIsEmpty = true;
	while( !basicIt->atEnd() ){
		if ( bbIsEmpty ){
			bb = BoundingBox3f( transformVector3f(transformation, basicIt->get3f( POSITION )) );
			bbIsEmpty = false;
		} else
			bb.addPoint( transformVector3f(transformation, basicIt->get3f( POSITION ) ));

		basicIt->next();
	}
	delete newIt;
	return bb;
}

/// Return the bounding box of all scene point clouds
BoundingBox3f geoxl::getScenePCBBox( Scene* scene )
{
	BoundingBox3f bb;
	PointCloud* pc = NULL;
	// Search for point cloud
	StandardSceneGraphIterator* it = new StandardSceneGraphIterator();
	it->clearAndSetup( scene->getRootNode(), scene->getRootState() );
	bool containsPointClouds = false;
	while ( !it->atEnd() ){
		SGObjectNode* sgo = dynamic_cast<SGObjectNode*>( it->getNode() );
		if ( sgo != NULL ){
			pc = dynamic_cast<PointCloud*>( sgo->getSceneObject() );
			if ( pc != NULL && pc->providesAttribute("position")){
				if ( !containsPointClouds )
					bb = getPCBBox( pc );
				else
					bb.addBB( getPCBBox( pc ) );
				containsPointClouds = true;
			}
		}
		it->next();
	}
	delete it;
	if ( !containsPointClouds )
		return BoundingBox3f( makeVector3f( -1, -1, -1 ), makeVector3f( 1,1,1 ) );
	else 
		return bb;
}

/**
* Return transformation that automatically aligns the point cloud
*/
Matrix4f geoxl::getAutoAlignMatrix(PointCloud* pc)
{
	// calc bb
	BoundingBox3f bb;
	bool posset = false;
	AAT POSITION = pc->getAAT("position");
	SceneObjectIterator *newIt = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(newIt);
	while( !basicIt->atEnd() ) {
		Vector3f pos = basicIt->get3f(POSITION);
		if (posset) {
			bb.addPoint(pos);
		} else {
			bb = BoundingBox3f(pos);
			posset = true;
		}
		basicIt->next();
	}
	delete newIt;
	// compute transform
	Matrix4f transform = IDENTITY4F;
	Vector3f mean = bb.getCenter();
	transform = makeTranslation4f(-mean);
	float32 diameter = bb.getMaxSideLength();
	if (diameter > 1E-10f) {
		float32 s = 2.0f/diameter;
		transform = makeScale4f(s,s,s) * transform;
	}
	return transform;
}

/**
* Returns resampled point cloud
*/
PointCloud* geoxl::createResampled( PointCloud *input, card32 x, card32 y ) {

	if (0 == x  || 0 == y ){
		warning("PointCloudTools::createResampled()\nParameters may not be zero!");
		return input;
	}

	// Input Iterator
	SceneObjectIterator *readIt = input->createIterator(SOIT(SOIT::CAP_BASIC_PC | SOIT::CAP_PARAMETRIC_PC));
	BasicPointCloudIterator *basicIt = dynamic_cast<BasicPointCloudIterator*>(readIt);
	ParametricPointCloudIterator *paramIt = dynamic_cast<ParametricPointCloudIterator*>(readIt);
	if (basicIt == NULL || paramIt == NULL){
		warning("PointCloudTools::createResampled()\nInput not parametric!");
		delete readIt;
		return input;
	}

	//Input Dimensions
	card32 w=paramIt->getWidth();
	card32 h=paramIt->getHeight();
	if (w < x  || h< y ){
		warning("PointCloudTools::createResampled()\nInput not parametric, wrong input Dimensions!");
		delete readIt;
		return input;
	}

	// Create Output
	PointCloud *output = NULL;

	if (dynamic_cast<ParametricInCorePointCloud*>(input) != NULL) {
		output = (ParametricInCorePointCloud*)input->copy();
		((ParametricInCorePointCloud*)output)->clearAndSetup(w/x, h/y, input->getDescr());
	} else if (dynamic_cast<UnstructuredInCorePointCloud*>(input) != NULL) {
		output = (UnstructuredInCorePointCloud*)input->copy();
		((UnstructuredInCorePointCloud*)output)->clearAndSetup(input->getDescr(), (w/x)*(h/y));
	} else {
		warning("PointCloudTools::createResampled()\nUnrecognized input type!");
		delete readIt;
		return input;
	}

	// Output Iterator
	SceneObjectIterator *writeIt = output->createIterator(SOIT::CAP_BASIC_PC | SOIT::CAP_PARAMETRIC_PC | SOIT::CAP_MODIFYABLE_PC);
	ModifyablePointCloudIterator *mody2It = dynamic_cast<ModifyablePointCloudIterator*>(writeIt);
	ParametricPointCloudIterator *param2It = dynamic_cast<ParametricPointCloudIterator*>(writeIt);
	if (mody2It == NULL || param2It == NULL){
		warning("PointCloudTools::createResampled()\nOutput Iterator could not be created!");
		delete readIt;
		delete output;
		delete writeIt;
		return input;
	}

	// Copy every xth, yth Point from Input to Output
	char *buffer=new char[input->getDescr()->getSize()];
	for (card32 i=0; i<=w-x; i+=x)
		for(card32 j=0; j<=h-y; j+=y)
		{
			paramIt->goTo(i,j);
			basicIt->getVertex(buffer);
			param2It->goTo(i/x,j/y);
			mody2It->setVertex(buffer);
		}

		// clean up
		delete basicIt;
		delete writeIt;
		delete[] buffer;
		return output;
};

/**
* Return node from name
*/
SceneGraphNode *geoxl::getSceneGraphNodeByName(Scene *scene, string name) {
	StandardSceneGraphIterator sceneIterator;
	sceneIterator.clearAndSetup(scene->getRootNode(), scene->getRootState());
	SGIndexedAndByNameSceneGraphIteratorAdaptor indexingAdaptor;
	indexingAdaptor.clearAndSetup(&sceneIterator);
	if (!indexingAdaptor.goToName(name)) {
		return NULL;
	} else {
		return indexingAdaptor.getNode();
	}
}

/**
* Return point cloud from scene graph node
*/
PointCloud *geoxl::getPointCloud(SceneGraphNode *node) {
	SGObjectNode *objn = dynamic_cast<SGObjectNode*>(node);
	if (objn == NULL) return NULL;
	return dynamic_cast<PointCloud*>(objn->getSceneObject());
}

/**
* Permute points in set
*/
void geoxl::randomlyPermutePointSet(PointSet *ps) {
	size_t size = ps->getNumEntries();
	for (mpcard i=0; i<size; i++) {
		swapPSPoints(ps, i, i+random64bit(size-i-1)+1);
	}
}

/**
* Swap two points in point set
*/
void geoxl::swapPSPoints(PointSet *ps, mpcard i1, mpcard i2) {
	mpcard n = ps->getElementSize();
	char *p1 = ps->getDataPointer(i1);
	char *p2 = ps->getDataPointer(i2);
	for (card32 i=0; i<n; i++) {
		swap(*p1, *p2);
		++p1;
		++p2;
	}
}

bool geoxl::deletePointCloud( Scene *scene, string pcID )
{
   return deleteSceneGraphNode(scene, pcID);
}


/**
* Delete point cloud 'id' from scene. Returns true on success.
*/
bool geoxl::deleteSceneGraphNode( Scene* scene, string id) {
	StandardSceneGraphIterator it;
	it.clearAndSetup(scene->getRootNode(), scene->getRootState());
	SGIndexedAndByNameSceneGraphIteratorAdaptor byName;
	byName.clearAndSetup(&it);
	if (byName.goToName(id)) {
		if (byName.getParent() != NULL) {
			byName.getParent()->removeChildNode(byName.getNode()->getName());
			return true;
		}
	}
	return false;
}


/**
* Compute the regression plane from closest points
*/
void geoxl::getLocalPlane( RegressionPlane& plane, Vector3f pos, PointCloud* pc, HierarchicalKNNIterator* pcNNIt )
{
	AttributeAccessToken POSITION = pc->getAAT( "position" );
	// Extract region of closest neighbor
	plane.clear();
	pcNNIt->setSeekPointAndReset( pos );
	for ( unsigned int n = 0; n < NUM_NEIGHBORS_LOCAL_CONTROL_PLANE; n++ ){
		plane.addPoint( pcNNIt->get3f( POSITION ) );
		pcNNIt->next();
	}
	plane.computePlane();
}


/**
* Compute the regression plane from closest points
*/
void geoxl::getLocalPlane( RegressionPlane& plane, Vector3f pos, float32 lfs, PointCloud* pc, HierarchicalKNNIterator* pcNNIt )
{
	AttributeAccessToken POSITION = pc->getAAT( "position" );
	Vector3f nPos;
	// Extract region of closest neighbor
	plane.clear();
	pcNNIt->setSeekPointAndReset( pos );
	while( !pcNNIt->atEnd() ){
		nPos = pcNNIt->get3f( POSITION );
		if ( fabs( ( nPos - pos ) * ( nPos - pos ) ) > lfs ){
			break;
		}
		plane.addPoint( nPos );
		pcNNIt->next();
	}
	plane.computePlane();
}

Matrix4f geoxl::getTransformation(Scene *scene, string node) {
	StandardSceneGraphIterator sceneIterator;
	sceneIterator.clearAndSetup(scene->getRootNode(), scene->getRootState());
	SGIndexedAndByNameSceneGraphIteratorAdaptor indexingAdaptor;
	indexingAdaptor.clearAndSetup(&sceneIterator);
	bool found = indexingAdaptor.goToName(node);	
	pAssert(found);
	if (found) {
		return indexingAdaptor.getState()->dynamicState->ctm;
	} else {
		return IDENTITY4F;
	} 
}

float geoxl::getAveragePointDistance( PointCloud *pc, card32 numDirectNeighbours ) 
{
	SceneObjectIterator *iter = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	HierarchicalKNNIterator * hIt = new HierarchicalKNNIterator(pc, 32, NULL);
	AAT posAAT = pc->getAAT( "position" );

	BoundingBox3f bb = getPCBBox(pc);

	float averageDistance = 0;
	card32 numSamples = 0;

	while( !bIt->atEnd() )
	{
		Vector3f samplePos = bIt->get3f(posAAT);
		hIt->setSeekPointAndReset( samplePos );

		// calc average distance to direct neighbours
		float averageSampleDistance = 0.0f;
		for( card32 i=0;i<numDirectNeighbours;i++ )
		{
			hIt->next();
			float dist = norm( samplePos - hIt->get3f( posAAT ) );
			averageSampleDistance += dist;
		}

		averageSampleDistance /= (float)numDirectNeighbours;
		averageDistance += averageSampleDistance;

		bIt->next();
		numSamples++;
	}

	delete hIt;
	delete iter;

	return averageDistance / (float)numSamples;
}

float geoxl::getMedianPointDistance( PointCloud *pc, card32 numDirectNeighbours, card32 numSamples )
{
	if( pc->getNumPoints() < 2)
	{
		throw PException(string("getMedianPointDistance() - Pointcloud has ") + intToStr(pc->getNumPoints()) + " points!");
	}
	SceneObjectIterator *iter = pc->createIterator(SOIT(SOIT::CAP_BASIC_PC));
	BasicPointCloudIterator *bIt = dynamic_cast<BasicPointCloudIterator*>(iter);
	RandomAccessPointCloudIterator *rIt = dynamic_cast<RandomAccessPointCloudIterator*>(iter);
	HierarchicalKNNIterator * hIt = new HierarchicalKNNIterator(pc, 32, NULL);
	AAT posAAT = pc->getAAT( "position" );
	static vector<float> distances;
	if( numSamples == 0)
		distances.resize((card32)pc->getNumPoints() * numDirectNeighbours );
	else
		distances.resize( min( (card32)pc->getNumPoints(), numSamples) * numDirectNeighbours );

	card32 index = 0;
	if( numSamples == 0 || numSamples >= pc->getNumPoints() )
	{
		while( !bIt->atEnd() )
		{
			Vector3f samplePos = bIt->get3f(posAAT);
			hIt->setSeekPointAndReset( samplePos );

			for( card32 i=0;i<numDirectNeighbours;i++ )
			{
				distances[index++] = norm( samplePos - hIt->get3f( posAAT ) );
				hIt->next();
			}

			bIt->next();
		}
	}
	else
	{
		for( card32 i=0;i<numSamples;i++ )
		{
			rIt->goToIndex( random64bit(pc->getNumPoints()) );
			Vector3f samplePos = bIt->get3f(posAAT);
			hIt->setSeekPointAndReset( samplePos );

			for( card32 i=0;i<numDirectNeighbours;i++ )
			{
				distances[index++] = norm( samplePos - hIt->get3f( posAAT ) );
				hIt->next();
			}
		}
	
	}
	sort( distances.begin(), distances.end() );

	return distances[ distances.size() / 2];
}

float geoxl::getCachedMedianPointDistance( PointCloud * pc, card32 numDirectNeighbours, card32 numSamples )
{
	const string attName = "medianPointDistance";
	AttachedDataContainer * att = pc->getAttachments();
	if( SimpleAttachment::exists(att,attName) )
		return SimpleAttachment::get1f(att,attName);
	else
	{
		float32 medPointDistance = getMedianPointDistance(pc,numDirectNeighbours,numSamples);
		SimpleAttachment::set1f(att,attName,medPointDistance);
		return medPointDistance;
	}
}

/// smoothes attribute "inputChannel" and stores the (normalized?) result in "outputChannel"
/// uses a gaussian filter kernel exp( -distance^2 / h^2), distance will be scaled by 1/averagePointDistance
/// averagePointDistance will be estimated when not specified 
/// this function doesn't perform smoothing in a two step way, therefore: inputChannel != outputChannel
/// (otherwise there will be different results)
template<typename FloatType,int dim> void smoothAttribute( PointCloud * pc, string inputChannel, 
														  string outputChannel, float h,bool normalize=false,
														  float averagePointDistance = 0.0f )
{
	if( !pc->providesAttribute( inputChannel ) || !pc->providesAttribute( outputChannel ) )
		throw "Error! Missing channel!\n";

	if( averagePointDistance == 0.0f )
	{
		averagePointDistance = getAveragePointDistance( pc );
	}
	
	float scaleKernel = 1.0f / pow(h * averagePointDistance,2.0f);

	UnstructuredInCorePointCloud * upc = dynamic_cast<UnstructuredInCorePointCloud*>(pc);
	pAssert( upc != NULL );
	VertexArray va( upc->getPointSet() );

	HierarchicalKNNIterator * hIt = new HierarchicalKNNIterator(pc, 32, NULL );

	AAT posAAT = pc->getAAT( "position" );
	AAT inputAAT = pc->getAAT( inputChannel );
	AAT outputAAT = pc->getAAT( outputChannel );

	
	progressWindow->pushStep( true, "smoothing" );
	for( card32 i=0;i<va.getNumElements();i++ )
	{
		Vector3f pos = va.getPosition3f(i);
		hIt->setSeekPointAndReset( pos );

		float summed_weight = 1.0f;
		StaticVector<FloatType,dim> attribute = hIt->genericGet<FloatType,dim>(inputAAT);
		hIt->next();

		while(!hIt->atEnd() )
		{
			Vector3f nPos = hIt->get3f( posAAT );
			float weight = exp( - normQuad( pos - nPos ) * scaleKernel );
			attribute += hIt->genericGet<FloatType,dim>(inputAAT)*weight;
			summed_weight += weight;

			if( weight < 0.01f )
				break;

			hIt->next();
		}

		attribute /= summed_weight;
		if( normalize )
			attribute.normalize();
		va.set<FloatType,dim>(i, outputAAT, attribute);
		if( !progressWindow->progress((card32)((float)i/(float)va.getNumElements() * 100.0f )) )
			break;
	}

	bool aborted;
	progressWindow->popSteps( aborted );
	

}


void geoxl::smoothAttribute1f( PointCloud * pc, string inputChannel, string outputChannel, float h,
							  bool normalize, float averagePointDistance)
{
	smoothAttribute<float,1>( pc, inputChannel, outputChannel, h, normalize, averagePointDistance );
}

void geoxl::smoothAttribute2f( PointCloud * pc, string inputChannel, string outputChannel, float h,
							  bool normalize, float averagePointDistance)
{
	smoothAttribute<float,2>( pc, inputChannel, outputChannel, h, normalize, averagePointDistance );
}

void geoxl::smoothAttribute3f( PointCloud * pc, string inputChannel, string outputChannel, float h,
							  bool normalize, float averagePointDistance)
{
	smoothAttribute<float,3>( pc, inputChannel, outputChannel, h, normalize, averagePointDistance );
}

void geoxl::smoothAttribute4f( PointCloud * pc, string inputChannel, string outputChannel, float h,
							  bool normalize, float averagePointDistance)
{
	smoothAttribute<float,4>( pc, inputChannel, outputChannel, h, normalize, averagePointDistance );
}

void geoxl::getSGNodeTransformation(Scene *scene, string nodeID, Matrix4f &transformation, SceneGraphNode *&node) 
{
	node = NULL;
	if ( scene == NULL )	return;
	StandardSceneGraphIterator it;
	it.clearAndSetup( scene->getRootNode(), scene->getRootState() );
	SGIndexedAndByNameSceneGraphIteratorAdaptor byName;
	byName.clearAndSetup(&it);
	if (byName.goToName(nodeID)) {
		node = byName.getNode();
		transformation = byName.getState()->dynamicState->ctm;
	}
}

VertexDescriptor * geoxl::createDefaultVertexDescriptor( bool position, bool color, bool normal, bool flags/*=true*/, bool tangents/*=false */ )
{
	VertexDescriptor * vd = new VertexDescriptor;
	if( position )
		vd->pushAttrib(VAD("position",3,VAD::DATA_FORMAT_FLOAT32));
	if( color )
		vd->pushAttrib(VAD("color",3,VAD::DATA_FORMAT_FLOAT32));
	if( normal )
		vd->pushAttrib(VAD("normal",3,VAD::DATA_FORMAT_FLOAT32));
	if( flags )
		vd->pushAttrib(VAD("flags",1,VAD::DATA_FORMAT_INT32));
	if( tangents )
	{
		vd->pushAttrib(VAD("tangent_u",3,VAD::DATA_FORMAT_FLOAT32));
		vd->pushAttrib(VAD("tangent_v",3,VAD::DATA_FORMAT_FLOAT32));
	}

	return vd;
}

SCENEGRAPH_API void geoxl::merge( PointCloud * pc1, const PointCloud * pc2 )
{
	if( *(pc1->getDescr()) != (VertexDescriptor*)(pc2->getDescr()))
		throw "merge(point clouds) - descriptors don't match!";

	SceneObjectIterator * iter = pc1->createIterator( SOIT::CAP_DYNAMIC_PC );
	DynamicPointCloudIterator * dIt = dynamic_cast<DynamicPointCloudIterator*>(iter);
	SceneObjectIterator * iter2 = ((PointCloud*)pc2)->createIterator( SOIT::CAP_BASIC_PC );
	BasicPointCloudIterator * bIt = dynamic_cast<BasicPointCloudIterator*>(iter2);
	vector<char> buffer(pc1->getDescr()->getSize());

	Vector3f * pos = NULL;
	if( pc1->providesAttribute("position"))
	{
		AAT posAAT = pc1->getAAT("position");
		pos = (Vector3f*)(&buffer[0] + posAAT.getOffset());
	}
	Vector3f * normal = NULL;
	if( pc1->providesAttribute("normal"))
	{
		AAT normalAAT = pc1->getAAT("normal");
		normal = (Vector3f*)(&buffer[0] + normalAAT.getOffset());
	}

	Matrix4f mat_2to1 = invertFrame(pc1->getTransformation()) * pc2->getTransformation();
	Matrix3f mat_2to1_3f = shrink4To3(mat_2to1);

	while( !bIt->atEnd())
	{
		bIt->getVertex(&buffer[0]);

		if( pos )
			*pos = transformVector3f(mat_2to1, *pos);
		if( normal )
			*normal = mat_2to1_3f* *normal;

		dIt->add(&buffer[0]);
		bIt->next();
	}


	delete iter;
	delete iter2;
}

