//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "SGListNode.h"
//---------------------------------------------------------------------------
#include "SceneGraphState.h"
#include "Tools.h"
#include "GLHelper.h"
//---------------------------------------------------------------------------
#include <GL/gl.h>
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* SGListNode::SGListNodeMetaClass = NULL;

const MetaClass* SGListNode::getInstanceClass() const {
   return getClass();
}

Object* SGListNode::newInstance() {
   return new SGListNode;
}

const MetaClass* SGListNode::getClass() {
   if (SGListNodeMetaClass==NULL) {
      SGListNodeMetaClass = new MetaClass("SGListNode", "no description.",
                                  newInstance, sizeof(SGListNode));
   }
   return SGListNodeMetaClass;
}

void SGListNode::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION+1);
   SGListNodeMetaClass->copyParentProperties();
}

void SGListNode::shutdown() {
   delete SGListNodeMetaClass;
}


// ----


void SGListNode::write(OutputObjectStream *s) const {
   SceneGraphNode::write(s);
   max_platform_card num = nodes.size();
   s->writeMaxPlatformCard(num);
   for (max_platform_card i=0; i<num; i++) {
      s->writeObject(nodes[i]);
   }
}

void SGListNode::read(InputObjectStream *s) {
   SceneGraphNode::read(s);
   clear();
   max_platform_card num;
   if (s->getClassVersion(SGListNode::getClass()) >= MetaClass::FIRST_VERSION+1) {
      s->readMaxPlatformCard(num);
   } else {
      card32 convert;
      s->read<card32>(convert);  // x64 bug in original version
      num = convert;
   }
   nodes.resize(num);
   for (max_platform_card i=0; i<num; i++) {
      s->readObject((Persistent*&)(nodes[i]));
   }
}

max_platform_card SGListNode::getNumChildNodes(const SceneGraphState *state) const{
   forget(state);
   return nodes.size();
}

SceneGraphNode *SGListNode::getChildNode(const SceneGraphState *state, max_platform_card i) {
   forget(state);
   pAssert(i < nodes.size());
   if (i < nodes.size()) return nodes[i]; else return NULL;
}

void SGListNode::removeChildNode(string name) {
   vector<SceneGraphNode*>::iterator pos = nodes.begin();
   vector<SceneGraphNode*>::iterator end = nodes.end();

   for (; pos!=end; ++pos) {
      if ((*pos)->getName() == name) {
         (*pos)->releaseReference(this);
         nodes.erase(pos);
         return;
      }
   }
   pAssert(false);
}

bool SGListNode::canAddChildNode() {
   return true;
}

bool SGListNode::addChildNode(SceneGraphNode *node) {
   addNode(node);
   return true;
}

max_platform_int SGListNode::getChildIndex(string name) {
   max_platform_card n = nodes.size();
   for (max_platform_card i=0; i<n; i++) {
      if (nodes[i] != NULL && nodes[i]->getName() == name) {
         return i;
      }
   }
   return -1;
}

void SGListNode::swapChildren(mpcard index1, mpcard index2) {
   bool check = index1 < nodes.size() && index2 < nodes.size();
   pAssert(check);
   if (!check) return;
   swap(nodes[index1], nodes[index2]);
}



void SGListNode::addNode(SceneGraphNode *n) {
   nodes.push_back(n);
   n->addReference(this);
}

void SGListNode::removeNode(card32 index) {
   pAssert(index < nodes.size());
   vector<SceneGraphNode*>::iterator pos = nodes.begin();
   if (index < nodes.size()) {
      pos += index;
      (*pos)->releaseReference(this);
      nodes.erase(pos);
   }
}

void SGListNode::clear() {
   const size_t n = nodes.size();
   for (card32 i=0; i<n; i++) {
      nodes[i]->releaseReference(this);
   }
   nodes.clear();
}

void SGListNode::renderNodeGL(const SceneGraphState *state) {
	if (state->staticState->rfShowLocalCoords) {
		GLHelper::drawCoords();
	}
}

SGListNode::~SGListNode() {
   clear();
}

