//---------------------------------------------------------------------------
#ifndef SGRelativeTimeAnimationNodeH
#define SGRelativeTimeAnimationNodeH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "SGAnimationNode.h"
//---------------------------------------------------------------------------

namespace geoxl {

///
///   In contrast to the simple animation node, this node supports a separate time
///   interval for each child node. In addition, the nodes can be cascaded hierarchically;
///   the animation node changes the dynamic time in the scene graph state to
///   time - (start of the current frame)
///
class SCENEGRAPH_API SGRelativeTimeAnimationNode : public SGAnimationNode {
	GEOXL_CLASS( SGRelativeTimeAnimationNode )
 private:
	vector<float32> timePerFrame;
	vector<float32> startTimes;
	bool startTimesValid;

	void computeStartTimes();
	
	void setNumtimePerFrames(card32 size);
	void deletetimePerFrame( card32 index );
	char *gettimePerFrame( card32 index );
	mpcard getNumtimePerFrames();
	string gettimePerFrameName(card32 index );
	
 public:
	SGRelativeTimeAnimationNode();

	/// set duration of a frame - standard value is one time unit
   void setDuration(card32 frameIndex, float32 frameTime);
   float32 getFrameTime(card32 frameIndex);

   void write(OutputObjectStream *s) const;
   void read(InputObjectStream *s);

   virtual max_platform_card getNumChildNodes(const SceneGraphState *state) const;
   virtual SceneGraphNode *getChildNode(const SceneGraphState *state, max_platform_card i);
   virtual SGStateChanger *getChildNodeEntryStateChange(const SceneGraphState *state, max_platform_card i);

   virtual void removeChildNode(string name);
   virtual bool addChildNode(SceneGraphNode *node);
   virtual void swapChildren(card32 index1, card32 index2);

   virtual void addNode(SceneGraphNode *n);
   virtual void removeNode(card32 index);
   virtual void clear();

   ~SGRelativeTimeAnimationNode();
};

}

#endif