//---------------------------------------------------------------------------
#ifndef SGStateChangerH
#define SGStateChangerH
//---------------------------------------------------------------------------
#include "GlobalDefinitions.h"
//---------------------------------------------------------------------------
#include "Persistent.h"
//---------------------------------------------------------------------------


namespace geoxl {


class SceneGraphState;



/// This class encapsulates a change to the scene graph state.
/// Objects of this class are created during traversal an kept on a stack
/// to track changes to the scene graph state.
class SCENEGRAPH_API SGStateChanger : public Persistent {
 private:
   static MetaClass *SGStateChangerMetaClass;
   static Object* newInstance();

 public:
   virtual const MetaClass *getInstanceClass() const;
   static const MetaClass *getClass();
   static void init(const MetaClass *parentClass);
   static void shutdown();

   /// performs the change to the current state, stores infos to restore original state later
   virtual void doChange(SceneGraphState *state) = 0;
   /// restores previous state: performs the inverse state transformtion of doChange
   /// (call only after doChange has been applied, state must be the same as directly after doChange;
   ///  i.e. doChange/undoChange should be used in a stack-like fashion)
   virtual void undoChange(SceneGraphState *state) = 0;
};

}


#endif
