//---------------------------------------------------------------------------
#include "stdafx.h"

//---------------------------------------------------------------------------
#include "SceneGraphNode.h"
//---------------------------------------------------------------------------
#include "StringClassProperty.h"
#include "NumericalClassProperty.h"
#include "NumericalClassProperty_templCode.h"
#include "StringHelper.h"
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------



MetaClass* SceneGraphNode::SceneGraphNodeMetaClass = NULL;

const MetaClass* SceneGraphNode::getInstanceClass() const {
   return getClass();
}

const MetaClass* SceneGraphNode::getClass() {
   if (SceneGraphNodeMetaClass==NULL) {
      SceneGraphNodeMetaClass = new MetaClass("SceneGraphNode", "no description.",
                                              NULL, sizeof(SceneGraphNode));
   }
   return SceneGraphNodeMetaClass;
}

void SceneGraphNode::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION+1);
   SceneGraphNodeMetaClass->copyParentProperties();

   BEGIN_CLASS_INIT(SceneGraphNode);
   ADD_STRING_PROP( name, 0 );
   SceneGraphNodeMetaClass->actProperty_setStreamProperty(false); 

   ADD_CARD32_PROP(flags, 0);
   SceneGraphNodeMetaClass->actProperty_setStreamProperty(false); 

}

void SceneGraphNode::shutdown() {
   delete SceneGraphNodeMetaClass;
}


// ----


SceneGraphNode::SceneGraphNode() {
   flags = FLAG_VISIBLE;
   name = "unnamed";
}

void SceneGraphNode::assign(const Object* obj, CopyContext *context) {
   forget(obj);
   forget(context);
   notImplemented();
}

void SceneGraphNode::write(OutputObjectStream *s) const {
   Persistent::write(s);
   s->writeString(name);
   s->write(flags);
   max_platform_card n = parents.size();
   s->writeMaxPlatformCard(n);
   for (max_platform_card i=0; i<n; i++) {
      s->writeObject(parents[i]);
   }
}

void SceneGraphNode::read(InputObjectStream *s) {
   Persistent::read(s);
   s->readString(name);
   s->read(flags);
   max_platform_card n;
   if (s->getClassVersion(SceneGraphNode::getClass()) >= MetaClass::FIRST_VERSION+1) {
      s->readMaxPlatformCard(n);
   } else {
      card32 conv;
      s->read(conv);
      n = conv;
   }
   parents.resize(n);
   for (max_platform_card i=0; i<n; i++) {
      s->readObject((Persistent*&)parents[i]);
   }
}

bool SceneGraphNode::findNameInParents(const string name) const {
   const size_t np = parents.size();
   for (card32 p=0; p<np; p++) {
      if (parents[p] == NULL) continue;
      const mpcard n = parents[p]->getNumChildNodes(NULL);
      for (card32 i=0; i<n; i++) {
         if (parents[p]->getChildNode(NULL, i) != this && parents[p]->getChildNode(NULL, i)->getName() == name) {
            return true;
         }
      }
   }
   return false;
}

static string postfixIncrement(string s) {
   string number;
   int32 n = (int32)s.size();
   int32 beginText = -1;
   for (int32 i=n-1; i>=0; i--) {
      if (s[i] >= '0' && s[i] <= '9') {
         string n = "0"; n[0] = s[i];
         number = n + number;
      } else {
         beginText = i;
         break;
      }
   }
   if (number == "") {
      number = "0";
   }

	int numberInt;
	bool ok= strToInt(number, numberInt);
	pAssert(ok);


   numberInt++;
   string baseName;
   if (beginText >= 0) {
      baseName = s.substr(0, beginText+1);
   }

   return baseName + intToStr(numberInt);
   
}

string SceneGraphNode::makeUniqueName(string name) const {
   while (findNameInParents(name)) {
      name = postfixIncrement(name);
   }
   return name;
}

void SceneGraphNode::addReference(SceneGraphNode *parent) {
   parents.push_back(parent);
   name = makeUniqueName(name);
}

void SceneGraphNode::releaseReference(SceneGraphNode *parent) {
   vector<SceneGraphNode*>::iterator pos = parents.begin();
   vector<SceneGraphNode*>::iterator end = parents.end();
   while (pos != end) {
      if (*pos == parent) {
         parents.erase(pos);
         if (parents.size() == 0) {
            delete this;
         }
         return;
      }
      ++pos;
   }
   // parent not found
   pAssert(false);
}

max_platform_card SceneGraphNode::getNumReferences() {
   return parents.size();
}

string SceneGraphNode::getName() const {
   return name;
}

void SceneGraphNode::setName(const string name) {
	string newName = name;
	if (newName == "") newName = "unnamed";
   this->name = makeUniqueName(newName);
}

void SceneGraphNode::setVisible(bool v) {
   if (v) {
      flags |= FLAG_VISIBLE;
   } else {
      flags &= 0xFFFFFFFF - FLAG_VISIBLE;
   }
}

bool SceneGraphNode::nodeReachable(SceneGraphNode *node, SceneGraphNode *searchFrom) {
   if (node == searchFrom) return true;
   mpcard n = searchFrom->getNumChildNodes(NULL);
   for (mpcard i=0; i<n; i++) {
      if (nodeReachable(node, searchFrom->getChildNode(NULL, i))) return true;
   }
   return false;
}

