//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "SceneGraphState.h"
//---------------------------------------------------------------------------
#include "ObjectStreams.h"
#include "Tools.h"
#include "ObjectClassProperty.h"
#include "NumericalClassProperty.h"
#include "NumericalClassProperty_templCode.h"
#include "FixedArrayClassProperty.h"
#include "EmbeddedObjectClassProperty.h"
#include "StringClassProperty.h"
#include "LinearAlgebra_templCode.h"
#include "BooleanClassProperty.h"
#include "SGAllocatorList.h"
#include "EmbeddedObjectClassPropertyEditor.h"
//---------------------------------------------------------------------------
using namespace geoxl;
//---------------------------------------------------------------------------


Vector4f geoxl::sgStateStandardBackgroundColor = makeVector4f(30/255.0f, 40/255.0f, 70/255.0f, 0.5f);


MetaClass* SGStaticState::SGStaticStateMetaClass = NULL;

const MetaClass* SGStaticState::getInstanceClass() const {
   return getClass();
}

Object* SGStaticState::newInstance() {
   return new SGStaticState;
}

const MetaClass* SGStaticState::getClass() {
   if (SGStaticStateMetaClass==NULL) {
      SGStaticStateMetaClass = new MetaClass("SGStaticState", "no description.",
                                  newInstance, sizeof(SGStaticState));
   }
   return SGStaticStateMetaClass;
}

static string getTraversalModeDescr(card32 i) {
	if (i == SGStaticState::tmAllNodes) return "tmAllNodes";
	if (i == SGStaticState::tmRendering) return "tmRendering";
	return "unknown";
}

void SGStaticState::init(const MetaClass *parentClass) {
	((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 6);
	SGStaticStateMetaClass->copyParentProperties();
	BEGIN_CLASS_INIT( SGStaticState );
	ADD_EMBEDDED_OBJECT_PROP( camera, 1, Camera::getClass() )
	ADD_EMBEDDED_OBJECT_PROP( viewFrustum, 1, ViewFrustum::getClass() )
   SET_EDITOR(EmbeddedObjectClassPropertyInlineEditor::getClass());
	ADD_FLOAT32_FIXED_ARRAY_PROP(backgroundColor,0,4,4) 
	ADD_STRING_PROP_METHOD( selection, 0, &getSelectionString, NULL )
	SET_EDIT_READONLY( true )
	SET_STREAM_PROPERTY( false )
	ADD_BOOLEAN_PROP( rfShowLocalCoords, 2 )
	ADD_OBJECT_PROP( allocators, 3, SGAllocatorList::getClass(), true )
	ADD_CARD32_PROP_UPDATE_DESCR( defaultTraversalMode, 4, NULL, getTraversalModeDescr, 2 );
}

void SGStaticState::shutdown() {
   delete SGStaticStateMetaClass;
}


// --


MetaClass* SGDynamicState::SGDynamicStateMetaClass = NULL;

const MetaClass* SGDynamicState::getInstanceClass() const {
   return getClass();
}

Object* SGDynamicState::newInstance() {
   return new SGDynamicState;
}

const MetaClass* SGDynamicState::getClass() {
   if (SGDynamicStateMetaClass==NULL) {
      SGDynamicStateMetaClass = new MetaClass("SGDynamicState", "no description.",
                                  newInstance, sizeof(SGDynamicState));
   }
   return SGDynamicStateMetaClass;
}

void SGDynamicState::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass);
   SGDynamicStateMetaClass->copyParentProperties();
   BEGIN_CLASS_INIT( SGDynamicState );
   ADD_FLOAT32_FIXED_ARRAY_PROP(ctm,0,15,4)   
   ADD_FLOAT32_PROP(time, 0);

}

void SGDynamicState::shutdown() {
   delete SGDynamicStateMetaClass;
}


// --


MetaClass* SceneGraphState::SceneGraphStateMetaClass = NULL;

const MetaClass* SceneGraphState::getInstanceClass() const {
   return getClass();
}

Object* SceneGraphState::newInstance() {
   return new SceneGraphState;
}

const MetaClass* SceneGraphState::getClass() {
   if (SceneGraphStateMetaClass==NULL) {
      SceneGraphStateMetaClass = new MetaClass("SceneGraphState", "no description.",
                                  newInstance, sizeof(SceneGraphState));
   }
   return SceneGraphStateMetaClass;
}

void SceneGraphState::init(const MetaClass *parentClass) {
   ((MetaClass*)getClass())->init(parentClass, MetaClass::FIRST_VERSION + 1);
   SceneGraphStateMetaClass->copyParentProperties();

   BEGIN_CLASS_INIT(SceneGraphState)
   ADD_OBJECT_PROP( staticState, 0, SGStaticState::getClass(), true )
   ADD_OBJECT_PROP( dynamicState, 0, SGDynamicState::getClass(), true )
}

void SceneGraphState::shutdown() {
   delete SceneGraphStateMetaClass;
}


// ----


SGStaticState::SGStaticState() {                                                    
   camera = Camera();
   viewFrustum = ViewFrustum();
   backgroundColor = sgStateStandardBackgroundColor;
   rfShowLocalCoords = true;
   allocators = new SGAllocatorList();
	defaultTraversalMode = tmAllNodes;
}

SGStaticState::~SGStaticState() {
	delete allocators;
}

string SGStaticState::getSelectionString() {
   string result;
   set<string>::iterator pos = selectedNodes.begin();
   set<string>::iterator end = selectedNodes.end();
   while (pos != end) {
      result += *pos;
      ++pos;
      if (pos != end) {
         result += "; ";
      }
   }
   return result;
}

/**
 * Return vector of names of selected nodes
 */
vector<string> SGStaticState::getSelectedNodes() {
   vector<string> result;
   result.clear();
   set<string>::iterator pos = selectedNodes.begin();
   set<string>::iterator end = selectedNodes.end();
   for (; pos != end; ++pos) {
      string name = *pos;
      result.push_back( name );
   }
   return result;
}

void SGStaticState::assign(const Object* obj, CopyContext *context) {
   const SGStaticState *s = dynamic_cast<const SGStaticState*>(obj);
   pAssert(s != NULL);
   this->camera = s->camera;
   this->viewFrustum = s->viewFrustum;
   this->backgroundColor = s->backgroundColor;
   this->selectedNodes = s->selectedNodes;
   this->rfShowLocalCoords = s->rfShowLocalCoords;
	this->defaultTraversalMode = s->defaultTraversalMode;
	delete this->allocators;
	this->allocators = (SGAllocatorList*)s->allocators->copy(context);
}

bool SGStaticState::equal(const Object* obj) const {
   const SGStaticState *s = dynamic_cast<const SGStaticState*>(obj);
   pAssert(s != NULL);
   return camera == s->camera &&
          viewFrustum == s->viewFrustum &&
          backgroundColor == s->backgroundColor &&
          selectedNodes == s->selectedNodes &&
          rfShowLocalCoords == s->rfShowLocalCoords &&
			 defaultTraversalMode == s->defaultTraversalMode &&
			 allocators->equal(s->allocators);
}

void SGStaticState::write(OutputObjectStream *s) const {
   camera.write(s);
   viewFrustum.write(s);
   s->write<Vector4f>(backgroundColor);

   max_platform_card nsel = selectedNodes.size();
   s->writeMaxPlatformCard(nsel);

   set<string>::const_iterator pos = selectedNodes.begin();
   for (max_platform_card i=0; i<nsel; i++) {
      s->writeString(*pos);
      ++pos;
   }
   s->writeBool(rfShowLocalCoords);

   s->write<card32>(defaultTraversalMode);
   
   s->writeObject(allocators);
}

void SGStaticState::read(InputObjectStream *s) {
   camera.read(s);
   viewFrustum.read(s);
   s->read<Vector4f>(backgroundColor);

   selectedNodes.clear();
   max_platform_card nsel;
   if (s->getClassVersion(SGStaticState::getClass()) > MetaClass::FIRST_VERSION+5) {
      s->readMaxPlatformCard(nsel);
   } else {
      card32 convert;
      s->read<card32>(convert);
      nsel = convert;
   }
   for (max_platform_card i=0; i<nsel; i++) {
      string item;
      s->readString(item);
      selectedNodes.insert(item);
   }
   // drop traversal state
   if (s->getClassVersion(SGStaticState::getClass()) < MetaClass::FIRST_VERSION + 1) {
      card32 c;
      s->read<card32>(c);
   }
   if (s->getClassVersion(SGStaticState::getClass()) >= MetaClass::FIRST_VERSION + 2) {
		s->readBool(rfShowLocalCoords);
	}

   if (s->getClassVersion(SGStaticState::getClass()) >= MetaClass::FIRST_VERSION + 4) {
		card32 c;
		s->read<card32>(c);
	   defaultTraversalMode = (TraversalModeType)c;
	}
   if (s->getClassVersion(SGStaticState::getClass()) >= MetaClass::FIRST_VERSION + 5) {
		s->readAnyObject(allocators);
	}
}



// --



SGDynamicState::SGDynamicState() {
   ctm = IDENTITY4F;
   time = 0.0f;
   traversalMode = tmAllNodes;
}

void SGDynamicState::assign(const Object *obj, CopyContext *context) {
   const SGDynamicState *d = dynamic_cast<const SGDynamicState*>(obj);
   pAssert(d != NULL);

   this->ctm = d->ctm;
   this->time = d->time;
   this->traversalMode = d->traversalMode;
   forget(context);
}

bool SGDynamicState::equal(const Object *obj) const {
   const SGDynamicState *d = dynamic_cast<const SGDynamicState*>(obj);
   pAssert(d != NULL);
   return this->ctm == d->ctm && this->time == d->time && traversalMode == d->traversalMode;
}

void SGDynamicState::write(OutputObjectStream *s) const {
   s->write<Matrix4f>(ctm);
   s->write<float32>(time);
   s->write<card32>(traversalMode);
}

void SGDynamicState::read(InputObjectStream *s) {
   s->read<Matrix4f>(ctm);
   s->read<float32>(time);
   card32 c;
   s->read<card32>(c);
   traversalMode = (TraversalModeType)c;
}


void SGDynamicState::setStateVariable(string name, Vector4f vectValue) 
{
   pair<string, Vector4f> val(name, vectValue);
   StrHashTable::iterator end = stateVariables4F.end();
   StrHashTable::iterator pos = stateVariables4F.find(name);
   if (pos != end) {
      pos->second = vectValue;
   } else {
      stateVariables4F.insert(val);
   }
}

bool SGDynamicState::getStateVariable(string name, Vector4f &vectValue) 
{
   StrHashTable::const_iterator end = stateVariables4F.end();
   StrHashTable::const_iterator pos = stateVariables4F.find(name);
   if (pos != end) {
      vectValue = pos->second;
      return true;
   } else {
      return false;
   }
}

vector< pair<string, Vector4f> > SGDynamicState::getStateVariables4F() {
   vector< pair<string, Vector4f> > result(stateVariables4F.size());
   StrHashTable::const_iterator end = stateVariables4F.end();
   StrHashTable::const_iterator pos = stateVariables4F.begin();
   size_t i = 0;
   while (pos != end) {
      result[i++] = *pos;
      ++pos;
   }
   return result;
}

void SGDynamicState::deleteStateVariable(string name)
{
   StrHashTable::const_iterator end = stateVariables4F.end();
   StrHashTable::const_iterator pos = stateVariables4F.find(name);
   if (pos != end) {
      stateVariables4F.erase(pos);
   } else {
      debugOutput << "SGSGDynamicState::deleteStateVariable - var does not exist\n";
   }
}


// ----


SceneGraphState::SceneGraphState() {
   traversalCopy = false;
   staticState = new SGStaticState;
   dynamicState = new SGDynamicState;
}

void SceneGraphState::assign(const Object* obj, CopyContext *context) {
   const SceneGraphState *other = dynamic_cast<const SceneGraphState*>(obj);
   pAssert(other != NULL);
   pAssert(other->staticState != NULL);
   pAssert(other->dynamicState != NULL);
   delete staticState;
   staticState = (SGStaticState*)other->staticState->copy(context);
   delete dynamicState;
   dynamicState = (SGDynamicState*)other->dynamicState->copy(context);
   traversalCopy = false;
}

SceneGraphState *SceneGraphState::createTraversalState() {
   SceneGraphState *result = new SceneGraphState;
   delete result->staticState;
   result->staticState = staticState;
   delete result->dynamicState;
   result->dynamicState = (SGDynamicState*)dynamicState->copy();
   result->traversalCopy = true;
   return result;
}


bool SceneGraphState::equal(const Object* obj) const {
   const SceneGraphState *other = dynamic_cast<const SceneGraphState*>(obj);
   pAssert(other != NULL);
   return staticState->equal(other->staticState) && dynamicState->equal(other->dynamicState);
}

void SceneGraphState::write(OutputObjectStream *s) const {
   s->writeObject(staticState);
   s->writeObject(dynamicState);
}

void SceneGraphState::read(InputObjectStream *s) {
   if (s->getClassVersion(getClass()) >= MetaClass::FIRST_VERSION +1) {

      delete staticState; staticState = NULL;
      delete dynamicState; dynamicState = NULL;

      // standard behaviour
      s->readObject((Persistent*&)staticState);
      s->readObject((Persistent*&)dynamicState);

   } else {

      // import legacy "PCRenderState" format
      staticState->camera.read(s);
      staticState->viewFrustum.read(s);
      card32 arraySize;
      s->read(arraySize);
      forget(arraySize);
      s->read(dynamicState->ctm);
      s->read(arraySize);
      forget(arraySize);
      s->read(staticState->backgroundColor);
      s->read(dynamicState->time);

   }
}


SceneGraphState::~SceneGraphState() {
   delete dynamicState;
   if (!traversalCopy) {
      delete staticState;
   }
}

