//---------------------------------------------------------------------------
#include "stdafx.h"
//---------------------------------------------------------------------------
#include "SceneGraphTools.h"
//---------------------------------------------------------------------------
#include "Scene.h"
#include "SceneGraphNode.h"
//---------------------------------------------------------------------------
using namespace geoxl;
using namespace std;
//---------------------------------------------------------------------------

string geoxl::extractSGPath(const string &nodeWPath) {
   const int n = (int)nodeWPath.size();
   int result = -1;
   for (int i=n-1; i>=0; i--) {
      if (nodeWPath[i] == '/') {
         result = i;
         break;
      }
   }
   if (result != -1) {
      return nodeWPath.substr(0, result+1);
   } else {
      return nodeWPath;
   }
}

string geoxl::extractSGNode(const string &nodeWPath) {
   const int n = (int)nodeWPath.size();
   int result = -1;
   for (int i=n-1; i>=0; i--) {
      if (nodeWPath[i] == '/') {
         result = i;
         break;
      }
   }
   if (result != -1) {
      return nodeWPath.substr(result+1, n-result-1);
   } else {
      return string("");
   }
}

vector<string> geoxl::parseSGPath(const string &path) {
   vector<string> result;
   string currentString;
   const size_t n=path.size();
   for (size_t i=0; i<n; i++) {
      if (path[i] == '/') {
         result.push_back(currentString);
         currentString = "";
      } else {
         currentString.push_back(path[i]);
      }
   }
   if (currentString.size() > 0) {
      result.push_back(currentString);
   }
   return result;
}

string geoxl::endPathWithSlash(const string &str) {
	string result = str;
	if (result.empty()) {
      pAssert(false);
		return "";
	}
	if (result[result.size()-1] != '/') {
		result.append(string("/"));
	}
   return result;
}

SceneGraphNode *geoxl::getSceneGraphNode(Scene *scene, string path) 
{
   vector<string> pathParsed = parseSGPath(path);
   return getSceneGraphNode(scene, pathParsed);
}

SceneGraphNode *geoxl::getSceneGraphNode(Scene *scene, vector<string> pathComponents) 
{
   SceneGraphNode *currentNode = scene->getRootNode();
   if (currentNode->getName() != pathComponents[0]) return NULL;
   for (size_t i=1; i<pathComponents.size(); i++) {
      mpcard n = currentNode->getNumChildNodes(NULL);
      SceneGraphNode *nextNode = NULL;
      for (mpcard j=0; j<n; j++) {
         if (currentNode->getChildNode(NULL, j)->getName() == pathComponents[i]) {
            nextNode = currentNode->getChildNode(NULL, j);
            break;
         }
      }
      if (nextNode == NULL) {
         return NULL;
      } else {
         currentNode = nextNode;
      }
   }
   return currentNode;
}

void geoxl::addSceneGraphNode(Scene *scene, string path, SceneGraphNode *addThis) 
{
   SceneGraphNode *addWhere = getSceneGraphNode(scene, path);
   if (!addWhere) {
      throw EInvalidState("SceneGraphTools::addSceneGraphNode() - path not found.");
   }
   addWhere->addChildNode(addThis);
}
